Package: lme4
Version: 2.0-6
Title: Linear Mixed-Effects Models using 'Eigen' and S4
Authors@R: 
    c(person("Douglas", "Bates", role = "aut",
             comment = c(ORCID = "0000-0001-8316-9503")),
      person("Martin", "Maechler", role = "aut",
             comment = c(ORCID = "0000-0002-8685-9910")),
      person("Ben", "Bolker", role = c("cre", "aut"),
             email = "bbolker+lme4@gmail.com",
             comment = c(ORCID = "0000-0002-2127-0443")),
      person("Steven", "Walker", role = "aut",
             comment = c(ORCID = "0000-0002-4394-9078")),
      person("Anna", "Ly", role = "aut",
             comment = c(ORCID = "0000-0002-0210-0342")),
      person("Mikael", "Jagan", role = "aut",
             comment = c(ORCID = "0000-0002-3542-2938")),
      person("Rune Haubo Bojesen", "Christensen", role = "ctb",
             comment = c(ORCID = "0000-0002-4494-3399")),
      person("Henrik", "Singmann", role = "ctb",
             comment = c(ORCID = "0000-0002-4842-3657")),
      person("Bin", "Dai", role = "ctb"),
      person("Fabian", "Scheipl", role = "ctb",
             comment = c(ORCID = "0000-0001-8172-3603")),
      person("Gabor", "Grothendieck", role = "ctb"),
      person("Peter", "Green", role = "ctb",
             comment = c(ORCID = "0000-0002-0238-9852")),
      person("John", "Fox", role = "ctb"),
      person("Alexander", "Bauer", role = "ctb"),
      person("Pavel N.", "Krivitsky", role = c("ctb", "cph"),
             comment = c(ORCID = "0000-0002-9101-3362",
                         "shared copyright on simulate.formula")),
      person("Emi", "Tanaka", role = "ctb",
             comment = c(ORCID = "0000-0002-1455-259X")),
      person("Ross D.", "Boylan", role = "ctb",
             comment = c(ORCID = "0009-0003-4123-8090")),
      person("Henrik", "Bengtsson", role = "ctb",
             comment = c(ORCID = "0000-0002-9280-4430")))
Description: 
    Fit linear and generalized linear mixed-effects models.  The
    models and their components are represented using S4 classes and
    methods.  The core computational algorithms are implemented using
    the 'Eigen' C++ library for numerical linear algebra and
    'RcppEigen' "glue".
Depends: R (>= 3.6), Matrix, methods, stats
LinkingTo: Matrix (>= 1.5-0), Rcpp (>= 1.1.1-1.1), RcppEigen (>=
        0.3.3.9.4)
Imports: MASS, Rdpack, boot, graphics, grid, lattice, minqa (>=
        1.1.15), nlme (>= 3.1-123), nloptr (>= 1.0.4), parallel,
        reformulas (>= 0.4.3.1), splines, utils
Suggests: DHARMa, HSAUR3, MEMSS, car, dfoptim, future.apply, gamm4,
        ggplot2, glmmTMB, gridExtra, knitr, merDeriv, mgcv, mlmRev,
        numDeriv, optimx (>= 2013.8.6), pbkrtest, performance,
        rmarkdown, rr2, see, semEff, statmod, testthat (>= 0.8.1),
        tibble
RdMacros: Rdpack
VignetteBuilder: knitr
LazyData: yes
License: GPL (>= 2)
URL: https://github.com/lme4/lme4/
BugReports: https://github.com/lme4/lme4/issues
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2026-07-15 14:34:40 UTC; bolker
Author: Douglas Bates [aut] (ORCID: <https://orcid.org/0000-0001-8316-9503>),
  Martin Maechler [aut] (ORCID: <https://orcid.org/0000-0002-8685-9910>),
  Ben Bolker [cre, aut] (ORCID: <https://orcid.org/0000-0002-2127-0443>),
  Steven Walker [aut] (ORCID: <https://orcid.org/0000-0002-4394-9078>),
  Anna Ly [aut] (ORCID: <https://orcid.org/0000-0002-0210-0342>),
  Mikael Jagan [aut] (ORCID: <https://orcid.org/0000-0002-3542-2938>),
  Rune Haubo Bojesen Christensen [ctb] (ORCID:
    <https://orcid.org/0000-0002-4494-3399>),
  Henrik Singmann [ctb] (ORCID: <https://orcid.org/0000-0002-4842-3657>),
  Bin Dai [ctb],
  Fabian Scheipl [ctb] (ORCID: <https://orcid.org/0000-0001-8172-3603>),
  Gabor Grothendieck [ctb],
  Peter Green [ctb] (ORCID: <https://orcid.org/0000-0002-0238-9852>),
  John Fox [ctb],
  Alexander Bauer [ctb],
  Pavel N. Krivitsky [ctb, cph] (ORCID:
    <https://orcid.org/0000-0002-9101-3362>, shared copyright on
    simulate.formula),
  Emi Tanaka [ctb] (ORCID: <https://orcid.org/0000-0002-1455-259X>),
  Ross D. Boylan [ctb] (ORCID: <https://orcid.org/0009-0003-4123-8090>),
  Henrik Bengtsson [ctb] (ORCID: <https://orcid.org/0000-0002-9280-4430>)
Maintainer: Ben Bolker <bbolker+lme4@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-16 12:30:08 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-07-16 13:07:46 UTC; unix
Archs: lme4.so.dSYM

useDynLib(lme4, .registration=TRUE)

## base packages
importFrom("graphics", par, plot)
importFrom("grid", gpar, viewport)
importFrom("splines", backSpline, interpSpline, periodicSpline)
importFrom("methods", as, callNextMethod, cbind2, getClass, getDataPart,
           extends, initialize, is, new, rbind2, setClass, setGeneric,
           setMethod, setRefClass, show, slot, slotNames)
importFrom("stats",
           AIC, BIC, anova, approx, approxfun,
           as.formula, asOneSidedFormula, ave,
           coef, confint,
           contrasts, "contrasts<-",
           cooks.distance,
           delete.response,
           deviance, dfbeta, dfbetas,
           df.residual, dnorm, drop.scope, drop1, extractAIC,
           family, fitted, formula, gaussian, getCall, glm,
           hatvalues, influence, lm, logLik, model.extract, model.frame,
           model.matrix, model.offset, model.response, model.weights,
           na.exclude, na.omit, na.pass, napredict, naresid, nlminb,
           nobs, optim, optimize, pchisq, pnorm, poisson, ppoints,
           predict, printCoefmat, profile, pt, qchisq, qnorm,
           qqnorm, qt, quantile, rbinom, reformulate, reorder,
           resid, residuals, rgamma, rlnorm, rnbinom, rnorm, rpois,
           rstandard, rstudent,
           runif, sd, setNames, simulate, symnum, terms, terms.formula,
           update, update.formula, var, vcov, weights, na.action)
importFrom("utils", flush.console, packageVersion, sessionInfo, stack, str, capture.output)
if(getRversion() >= "3.5.0") {
    importFrom("utils", warnErrList)
} else {
    importFrom("utils", capture.output) # for "our" warnErrList()
}


## Recommended packages
importFrom("lattice",
           bwplot,
           current.panel.limits,
           densityplot, dotplot,
           histogram,
           llines,lpoints,lsegments,ltext,
           panel.abline,panel.axis,panel.bwplot,panel.grid,panel.histogram,
           panel.lines, panel.number, panel.points,
           panel.segments, panel.xyplot,
           prepanel.qqmathline, panel.qqmathline, panel.qqmath, qqmath,
           splom, strip.custom, strip.default, trellis.par.get,
           xyplot)
importFrom("MASS", negative.binomial, theta.ml)
importFrom("boot", boot.ci)
importFrom(Rdpack, reprompt)

## generics we provide methods for and re-export:
importFrom("nlme", fixef,ranef, VarCorr, getGroups, getData)

importFrom("Matrix", drop0, rankMatrix, sparseMatrix, sparse.model.matrix,
           forceSymmetric, fac2sparse, KhatriRao, Diagonal, .bdiag,
           as.matrix, bdiag, Matrix)
importClassesFrom("Matrix", corMatrix, dgCMatrix, dpoMatrix, dCHMsimpl)
## methods incl. S4 generics:
importMethodsFrom("Matrix", coerce, cov2cor, "%*%", crossprod,tcrossprod,
                  t, diag, chol2inv, solve, colSums,rowSums)
## other CRAN packages:
importFrom("minqa", bobyqa)
importFrom("nloptr", nloptr)

## Re-Exports :
export(negative.binomial)
## currently needed for some eval()ing methods for glmer.nb() objects

## Our Exports:
export(allFit)
export(bootMer)
export(checkConv)
export(devcomp)
export(devfun2)
export(dummy)
export(factorize)  ## needed for qqmath???
export(fixef)
## export(fortify)
export(getL)
export(getME)
export(GHrule)
export(glFormula)
export(glmer.nb)
export(glmer)
export(glmerLaplaceHandle)
export(glmFamily)
export(glmResp)
export(golden)
export(GQdk)
export(GQN)
export(isLMM, isGLMM, isNLMM)
export(isREML)
export(lFormula)
export(lme4_testlevel)
export(lmer)
export(lmerControl, glmerControl, nlmerControl,
       .makeCC)
export(lmerResp)
export(lmList)
export(lmResp)
export(logProf)
export(varianceProf)
export(mlist2vec)
export(vec2mlist)
export(vec2STlist)
export(sdcor2cov)
export(cov2sdcor)
export(Vv_to_Cv, Sv_to_Cv, Cv_to_Vv, Cv_to_Sv)
export(merPredD)
export(mkGlmerDevfun)
export(mkLmerDevfun)
export(mkMerMod)
export(mkRespMod)
export(mkNewReTrms)
export(mkVarCorr)
export(mkParsTemplate)
export(mkDataTemplate)
export(Nelder_Mead)
export(NelderMead)
export(ngrps)
export(nlformula)
export(nlmer)
export(nloptwrap, nlminbwrap)# export both (1st has been there "forever")
export(nlsResp)
export(optimizeGlmer)
export(optimizeLmer)
export(ranef)
export(refit)
export(refitML)
export(rePos)
export(REMLcrit)
if(getRversion() >= "3.3.0") {
    importFrom("stats", sigma)
} else {
    export(sigma)
}
export(.simulateFun)
export(simulate.formula)
export(updateGlmerDevfun)
export(VarCorr)
export(getData)
export(varianceProf)

## print() and print.summary() utilities -- other [NG]LMM packages can import
export(.prt.VC,		.prt.aictab,	.prt.call,	.prt.family,
       .prt.grps,	.prt.methTit,	.prt.resids,	.prt.warn,
       formatVC,
       llikAIC,	methTitle)
## export(tnames)

##------ Our S4 Classes ------------------------
exportClasses(glmerMod, lmerMod, nlmerMod, merMod)
exportClasses(lmList4)
exportClasses(Covariance, Covariance.us, Covariance.diag, Covariance.cs, 
              Covariance.ar1)

##------ Our S4 Generics / Methods --------------
exportMethods(getL)
exportMethods(show)

##------ Our S3 Methods -------------------------
S3method(anova,merMod)
S3method(as.data.frame,bootMer)
S3method(as.data.frame,thpr)
S3method(as.data.frame,ranef.mer)
S3method(as.data.frame,VarCorr.merMod)
S3method(as.function,merMod)
S3method(coef,lmList4)
S3method(coef,merMod)
S3method(confint,bootMer)
S3method(confint,lmList4)
S3method(confint,merMod)# but don't hide:
export(confint.merMod)
S3method(confint,thpr)
S3method(cooks.distance,merMod)
S3method(cooks.distance,influence.merMod)
S3method(densityplot,thpr)
S3method(deviance,merMod)
S3method(df.residual,merMod)
## S3method(dim,merMod) ???
S3method(dotplot,coef.mer)
S3method(dotplot,ranef.mer)
## helpful to allow direct access for glmmTMB
export(dotplot.ranef.mer)
S3method(drop1,merMod)
S3method(extractAIC,merMod)
S3method(family,glmResp)
S3method(family,lmResp)
S3method(family,merMod)
S3method(family,nlsResp)
S3method(fitted,merMod)
S3method(fixef,merMod)
S3method(formula,lmList4)
S3method(formula,merMod)
S3method(getData,merMod)
## S3method(fortify,merMod)# but don't hide:
export(fortify.merMod)
S3method(getME, merMod)
S3method(hatvalues,merMod)
S3method(influence,merMod)
S3method(isGLMM,merMod)
S3method(isLMM,merMod)
S3method(isNLMM,merMod)
S3method(isREML,merMod)
S3method(log,thpr)
S3method(logLik,merMod)
S3method(model.frame,merMod)
S3method(model.matrix,merMod)
S3method(ngrps, default)
S3method(ngrps,factor)
S3method(ngrps,merMod)
S3method(nobs,merMod)
S3method(plot,coef.mer)
S3method(plot,lmList4.confint)
S3method(plot,merMod)
S3method(plot,ranef.mer)
S3method(predict,merMod)
S3method(print,allFit)
S3method(print,bootMer)
S3method(print,merMod)
S3method(print,ranef.mer)
S3method(print,summary.merMod)
S3method(print,VarCorr.merMod)
S3method(profile,merMod)
S3method(qqmath,merMod)
S3method(qqmath,ranef.mer)
S3method(ranef,merMod)
S3method(refit,merMod)
S3method(refitML,merMod)
S3method(residuals,glmResp)
S3method(residuals,lmResp)
S3method(residuals,merMod)
S3method(rstudent,merMod)
S3method(rstudent,glmerMod)
S3method(rstandard,glmerMod)
S3method(sigma,merMod)
S3method(simulate,merMod)
S3method(simulate,formula)
S3method(simulate,formula_lhs_)
S3method(simulate,formula_lhs_matrix)
S3method(simulate,formula_lhs_numeric)
S3method(simulate,formula_lhs_factor)
S3method(simulate,formula_lhs_integer)
S3method(simulate,formula_lhs_logical)
S3method(simulate,formula_lhs)
S3method(splom,thpr)
S3method(summary,allFit)
S3method(summary,merMod)
S3method(summary,summary.merMod)
S3method(terms,merMod)
S3method(update,lmList4)
S3method(update,merMod)
S3method(VarCorr,merMod)
S3method(vcov,merMod)# but do not hide:
export(vcov.merMod)
S3method(vcov,summary.merMod)
S3method(weights,merMod)
S3method(xyplot,thpr)

S3method(sigma,lmList4)
S3method(dfbeta,influence.merMod)
S3method(dfbetas,influence.merMod)
## Re-using S3 methods from nlme (for 'lmList') as methods for our 'lmList4':
S3method(fitted,lmList4)
S3method(fixef,lmList4)
S3method(logLik,lmList4)
S3method(pairs,lmList4)
S3method(plot,lmList4)
S3method(predict,lmList4)
S3method(qqnorm,lmList4)
S3method(ranef,lmList4)
S3method(residuals,lmList4)
S3method(summary,lmList4)
## Auxiliaries:
S3method(getGroups,lmList4)

## RePsychLing stuff
S3method(rePCA,merMod)
S3method(summary,prcomplist)
export(rePCA)

importFrom(reformulas,"RHSForm<-")
importFrom(reformulas,RHSForm)
importFrom(reformulas,findbars_x)
importFrom(reformulas,noSpecials)

## don't import, avoid masking warning
## clean up once reformulas 0.4.5 is on CRAN
## importFrom(reformulas,reOnly)
export(reOnly)
importFrom(reformulas,sub_specials)
importFrom(reformulas,subbars)
export(subbars)
importFrom(reformulas,nobars)
export(nobars)
importFrom(reformulas,findbars)
export(findbars)
importFrom(reformulas,mkReTrms)
export(mkReTrms)
importFrom(reformulas,isNested)
export(isNested)
importFrom(reformulas, expandDoubleVerts)
export(expandDoubleVerts)
importFrom(reformulas,formatVC,get_sd,format_corr)
export(formatVC)
## 
export(namedList)
importFrom(rlang, warn)
##
exportMethods(isSingular)
export(getSingTol)
export(getDoublevertDefault)
export(isNewMerMod, forceNewMerMod, anyStructured)
export(getReCovs)
##
S3method(na.action,merMod)

Catalog of currently-failing examples (commented out, testsx, etc.):

glmmExt.R: "fail for MM" on Gaussian/inverse examples -- seems fine for me
lmer-0.R: sstudy9 example.  Should *not* work; is a meaningful error message possible?
prLogistic.R: Thailand/clustered-data example from ?prLogisticDelta example in prLogistic package
           Presumably the problem is that 100/411 random-effect levels have only zeros -- but should this mess things up?
           glmmML and lme4.0 give nearly identical answers

profile.R: fails on CBPP profiling

from testsx:
  testcolonizer: definite case where complete separation occurs, GLM does not really give a fit
  testcrabs: ?? not sure ??

#### Saved fits for lme4 testing
####  ----------------------------------
fn <- system.file("testdata", (fn0 <- "lme-tst-fits.rda"),
                  package="lme4", mustWork=TRUE)

run_Pix_prof <- FALSE

if(FALSE) ### "Load" these by  load(fn)
    ## or "better"
    attach(fn)

library(lme4)
str(packageDescription("lme4")[c("Version", "Packaged", "Built")])

## intercept only in both fixed and random effects
fit_sleepstudy_0 <- lmer(Reaction ~ 1 + (1|Subject), sleepstudy)
## fixed slope, intercept-only RE
fit_sleepstudy_1 <- lmer(Reaction ~ Days + (1|Subject), sleepstudy)
## fixed slope, intercept & slope RE
fit_sleepstudy_2 <- lmer(Reaction ~ Days + (Days|Subject), sleepstudy)
## fixed slope, independent intercept & slope RE
fit_sleepstudy_3 <- lmer(Reaction ~ Days + (1|Subject)+ (0+Days|Subject), sleepstudy)

cbpp$obs <- factor(seq(nrow(cbpp)))
## intercept-only fixed effect
fit_cbpp_0 <- glmer(cbind(incidence, size-incidence) ~ 1 + (1|herd),
                    cbpp, family=binomial)
## include fixed effect of period
fit_cbpp_1 <- update(fit_cbpp_0, . ~ . + period)
## include observation-level RE
fit_cbpp_2 <- update(fit_cbpp_1, . ~ . + (1|obs))
## specify formula by proportion/weights instead
fit_cbpp_3 <- update(fit_cbpp_1, incidence/size ~ period + (1 | herd), weights = size)

fit_penicillin_1 <- lmer(diameter ~ (1|plate) + (1|sample), Penicillin)
fit_cake_1 <- lmer(angle ~ temp + recipe + (1 | replicate), data=cake)

## an example with >20 fixed effects (for testing print.summary.merMod)
if (require(agridat)) {
    ## Define main plot and subplot
    d.apple.agridat <- transform(archbold.apple, rep=factor(rep),
                                 spacing=factor(spacing), trt=factor(trt),
                                 mp = factor(paste0(row,spacing)),
                                 sp = factor(paste0(row,spacing,stock)))
    fit_agridat_archbold <- lmer(yield ~ -1 + trt + (1|rep/mp/sp), d.apple.agridat)
    to.save <- "d.apple.agridat"
} else
    to.save <- character()

##
data("Pixel", package="nlme")
fit_Pix.full <- lmer(pixel ~ day + I(day^2) + (day | Dog) + (1 | Side/Dog),
                     data = Pixel)
fit_Pix.1Dog <- lmer(pixel ~ day + I(day^2) +   (1 | Dog) + (1 | Side/Dog),
                     data = Pixel)
fit_Pix.noD  <- update(fit_Pix.1Dog, .~. - (1 | Dog))
anova(fit_Pix.full,
      fit_Pix.1Dog,
      fit_Pix.noD)

if (run_Pix_prof) {
    ## Warnings about non-monotonic profile (and more):
    options(warn=1) # print as they happen {interspersed in verbose profile() msgs}:
    system.time(prof.fit_Pix.f <- profile(fit_Pix.full, verbose=1))
    ## ~ 90 sec on nb-mm4 [i7-5600U, 2015]

    signif(confint(prof.fit_Pix.f), digits=3)
    ## Results in Nov.2014: -- now .sig03 now shows [-1, 1]
    ##              2.5 % 97.5 %
    ## .sig01      10.449 28.909
    ## .sig02      12.951 48.203
    ## .sig03          NA     NA <<
    ## .sig04       1.073  3.066
    ## .sig05       0.000 27.794
    ## .sigma       7.651 10.592
    ## (Intercept)     NA     NA <<
    ## day             NA     NA <<
    ## I(day^2)    -0.434 -0.298
    
    try( ## FIXME --> ../../R/profile.R  [FIXME: show plots for the *valid* parts!]
        lattice::xyplot(prof.fit_Pix.f)
    )
    ## FIXME: Error is ok, but error *message* is unhelpful
    ## Error in approx(bspl$x, bspl$y, xout = zeta) :
    ##   need at least two non-NA values to interpolate
}

save(list=c(to.save, ls(pattern="fit_")), file=fn0)

# This code is used to create the testdata called
# test-covariance_structures_data.rds

library(glmmTMB)
library(nlme)
library(lme4)

## First set of tests: lmer
fm1.glmmTMB <- glmmTMB(Reaction ~ Days + (Days | Subject), sleepstudy, 
                       REML = FALSE)
fm2.glmmTMB <- glmmTMB(Reaction ~ Days + us(Days | Subject), sleepstudy, 
                       REML = FALSE)
fm3.glmmTMB <- glmmTMB(Reaction ~ Days + cs(Days | Subject), sleepstudy, 
                       REML = FALSE)
fm4.glmmTMB <- glmmTMB(Reaction ~ Days + diag(Days | Subject), sleepstudy, 
                       REML = FALSE)
sleepstudy$Daysf <- factor(sleepstudy$Days, ordered = TRUE)
fm5.glmmTMB <- glmmTMB(Reaction ~ Daysf + ar1(0 + Daysf | Subject), 
                       sleepstudy, REML = FALSE)

fm1.nlme <- lme(fixed = Reaction ~ Days, random = ~ Days | Subject,
  data = sleepstudy, method = "ML")

fm1.nlme.REML <- lme(fixed = Reaction ~ Days, random = ~ Days | Subject,
                     data = sleepstudy)

fm2.nlme <- lme(fixed = Reaction ~ Days, random = ~ Days | Subject,
                correlation = corCompSymm(form = ~ Days | Subject),
                data = sleepstudy, method = "ML")

fm2.nlme.REML <- lme(fixed = Reaction ~ Days, random = ~ Days | Subject,
                correlation = corCompSymm(form = ~ Days | Subject),
                data = sleepstudy)

## Second set of tests: glmer
data(Contraception, package = "mlmRev")

gm.glmmTMB.us <- glmmTMB(use ~ age + urban + us(1 + urban | district),
                         data = Contraception,
                         family = binomial)
gm.glmmTMB.cs <- glmmTMB(use ~ age + urban + cs(1 + urban | district),
                         data = Contraception,
                         family = binomial)
gm.glmmTMB.diag <- glmmTMB(use ~ age + urban + diag(1 + urban | district),
                           data = Contraception,
                           family = binomial)

startvec <- c(Asym = 200, xmid = 725, scal = 350)
nm.nlme <- nlme(
  circumference ~ SSlogis(age, Asym, xmid, scal),
  data = Orange,
  fixed = Asym + xmid + scal ~ 1,   
  random = Asym ~ 1 | Tree, 
  start = startvec)

nm.nlme.cs <- nlme(
  circumference ~ SSlogis(age, Asym, xmid, scal),
  data = Orange,
  fixed = Asym + xmid + scal ~ 1,   
  random = Asym ~ 1 | Tree, 
  correlation = corCompSymm(form = Asym ~ 1 | Tree),
  start = startvec)

# tests regarding stdev and corr of glmms

simfun_gamma <- function(ngrp = 50, nrep = 50, shape_gam = 2, intercept = 1, 
                         theta_val = 2, seed = NULL) {
  if (!is.null(seed)) set.seed(seed)
  dd <- expand.grid(group = 1:ngrp, rep = 1:nrep)
  dd$y <- simulate(~ 1 + (1 | group), newdata = dd, 
                   family = Gamma(link = "log"), 
                   newparams = list(
                     theta = theta_val, beta = 1, 
                     sigma = 1/sqrt(shape_gam)))[[1]]
  dd
}

simfun_pois <- function(ngrp = 50, nrep = 50, intercept = 1,
                        theta_val = 1, seed = NULL) {
  if (!is.null(seed)) set.seed(seed)
  
  dd <- expand.grid(group = 1:ngrp, rep = 1:nrep)
  
  dd$y <- simulate(~ 1 + (1 | group), newdata = dd,
                   family = poisson(link = "log"), 
                   newparams = list(theta = theta_val, beta = intercept))[[1]]
  dd
}

dd2 <- simfun_gamma(seed = 101)

dd3 <- simfun_pois(seed = 101)

TMB1 <- glmmTMB(y ~ 1 + (1|group), family = Gamma(link = "log"), data = dd2)
TMB_v1 <- VarCorr(TMB1)$cond$group

TMB2 <- glmmTMB(y ~ 1 + (1|group), family = poisson(link = "log"), data = dd3)
TMB_v2 <- VarCorr(TMB2)$cond$group

cov_lmer_test <- list(
  ## First: set of tests for lmer
  ## sigma
  fm1.glmmTMB_sigma = sigma(fm1.glmmTMB),
  fm1.glmmTMB.cs_sigma = sigma(fm3.glmmTMB),
  fm1.glmmTMB.diag_sigma = sigma(fm4.glmmTMB),
  fm1.glmmTMB.ar1_sigma = sigma(fm5.glmmTMB),
  fm1.nlme_sigma = sigma(fm1.nlme),
  fm1.nlme.cs_sigma = sigma(fm2.nlme),
  fm1.nlme.REML_sigma = sigma(fm1.nlme.REML),
  fm1.nlme.cs.REML_sigma = sigma(fm2.nlme.REML),
  ## log likelihoods
  fm1.glmmTMB_logLik = logLik(fm1.glmmTMB),
  fm1.glmmTMB.cs_logLik = logLik(fm3.glmmTMB),
  fm1.glmmTMB.diag_logLik = logLik(fm4.glmmTMB),
  fm1.glmmTMB.ar1_logLik = logLik(fm5.glmmTMB),
  fm1.nlme_logLik = logLik(fm1.nlme),
  fm1.nlme.cs_logLik = logLik(fm2.nlme),
  fm1.nlme.REML_logLik = logLik(fm1.nlme.REML),
  fm1.nlme.cs.REML_logLik = logLik(fm2.nlme.REML),
  ## vcov
  fm1.glmmTMB_vcov = vcov(fm1.glmmTMB)$cond,
  fm1.glmmTMB.cs_vcov = vcov(fm3.glmmTMB)$cond,
  fm1.glmmTMB.diag_vcov = vcov(fm4.glmmTMB)$cond,
  fm1.glmmTMB.ar1_vcov = vcov(fm5.glmmTMB)$cond,
  fm1.nlme_vcov = vcov(fm1.nlme),
  fm1.nlme.cs_vcov = vcov(fm2.nlme),
  fm1.nlme.REML_vcov = vcov(fm1.nlme.REML),
  fm1.nlme.cs.REML_vcov = vcov(fm2.nlme.REML),
  ## VarCorr
  fm1.glmmTMB_var = VarCorr(fm1.glmmTMB)$cond[[1]],
  fm1.glmmTMB.us_var = VarCorr(fm2.glmmTMB)$cond[[1]],
  fm1.glmmTMB.cs_var = VarCorr(fm3.glmmTMB)$cond[[1]],
  fm1.glmmTMB.diag_var = VarCorr(fm4.glmmTMB)$cond[[1]],
  fm1.glmmTMB.ar1_var = VarCorr(fm5.glmmTMB)$cond[[1]],
  fm1.nlme_var = getVarCov(fm1.nlme),
  fm1.nlme.REML_var = getVarCov(fm1.nlme.REML),
  fm1.nlme.cs_var = getVarCov(fm2.nlme),
  fm1.nlme.cs.REML_var = getVarCov(fm2.nlme.REML),
  ## ranef
  fm1.glmmTMB_ranef = ranef(fm1.glmmTMB)$cond$Subject,
  fm1.glmmTMB.cs_ranef = ranef(fm3.glmmTMB)$cond$Subject,
  fm1.glmmTMB.diag_ranef = ranef(fm4.glmmTMB)$cond$Subject,
  fm1.glmmTMB.ar1_ranef = ranef(fm5.glmmTMB)$cond$Subject,
  fm1.nlme_ranef = ranef(fm1.nlme),
  fm1.nlme.REML_ranef = ranef(fm1.nlme.REML),
  fm1.nlme.cs_ranef = ranef(fm2.nlme),
  fm1.nlme.cs.REML_ranef = ranef(fm2.nlme.REML),
  ## predict
  fm1.glmmTMB_predict = predict(fm1.glmmTMB),
  fm1.glmmTMB.cs_predict = predict(fm3.glmmTMB),
  fm1.glmmTMB.diag_predict = predict(fm4.glmmTMB),
  fm1.glmmTMB.ar1_predict = predict(fm5.glmmTMB),
  fm1.nlme_predict = predict(fm1.nlme),
  fm1.nlme.REML_predict = predict(fm1.nlme.REML),
  fm1.nlme.cs_predict = predict(fm2.nlme),
  fm1.nlme.cs.REML_predict = predict(fm2.nlme.REML),
  ## Second: tests for glmer
  ## sigma
  gm.glmmTMB_sigma = sigma(gm.glmmTMB.us),
  gm.glmmTMB.cs_sigma = sigma(gm.glmmTMB.cs),
  gm.glmmTMB.diag_sigma = sigma(gm.glmmTMB.diag),
  ## log likelihoods
  gm.glmmTMB_logLik = logLik(gm.glmmTMB.us),
  gm.glmmTMB.cs_logLik = logLik(gm.glmmTMB.cs),
  gm.glmmTMB.diag_logLik = logLik(gm.glmmTMB.diag),
  ## vcov
  gm.glmmTMB_vcov = vcov(gm.glmmTMB.us)$cond,
  gm.glmmTMB.cs_vcov = vcov(gm.glmmTMB.cs)$cond,
  gm.glmmTMB.diag_vcov = vcov(gm.glmmTMB.diag)$cond,
  ## VarCorr
  gm.glmmTMB_var = VarCorr(gm.glmmTMB.us)$cond[[1]],
  gm.glmmTMB.cs_var = VarCorr(gm.glmmTMB.cs)$cond[[1]],
  gm.glmmTMB.diag_var = VarCorr(gm.glmmTMB.diag)$cond[[1]],
  ## ranef
  gm.glmmTMB_ranef = ranef(gm.glmmTMB.us)$cond$Subject,
  gm.glmmTMB.cs_ranef = ranef(gm.glmmTMB.cs)$cond$Subject,
  gm.glmmTMB.diag_ranef = ranef(gm.glmmTMB.diag)$cond$Subject,
  ## predict
  gm.glmmTMB_predict = predict(gm.glmmTMB.us),
  gm.glmmTMB.cs_predict = predict(gm.glmmTMB.cs),
  gm.glmmTMB.diag_predict = predict(gm.glmmTMB.diag),
  ## Third: tests for nlmer
  ## sigma
  nm.nlme_sigma = sigma(nm.nlme),
  nm.nlme.cs_sigma = sigma(nm.nlme.cs),
  ## log likelihoods
  nm.nlme_logLik = logLik(nm.nlme),
  nm.nlme.cs_logLik = logLik(nm.nlme.cs),
  ## vcov
  nm.nlme_vcov = vcov(nm.nlme),
  nm.nlme.cs_vcov = vcov(nm.nlme.cs),
  ## ranef
  nm.nlme_ranef = ranef(nm.nlme),
  nm.nlme.cs_ranef = ranef(nm.nlme.cs),
  ## predict
  nm.nlme_predict = predict(nm.nlme),
  nm.nlme.cs_predict = predict(nm.nlme.cs),
  ## glmm stdevs
  TMB_v1 = TMB_v1,
  TMB_v2 = TMB_v2
)

saveRDS(cov_lmer_test, "inst/testdata/test-covariance_structures_data.rds")
saveRDS(mlmRev::Contraception, "inst/testdata/Contraception.rds")

## code for reducing prof

dropform <- function(z) {
    lapply(z, function(x) {
        environment(attr(x,"formula")) <- NULL
        x
    })
}
rpt <- function(p, hdr) {
    cat(hdr, capture.output(pryr::object_size(p)), "\n")
}

strip_profile <- function(p, quietly=FALSE) {
    
    if (!quietly) {
        rpt(p,"initial : ")
    }
    attr(p,"forward") <- dropform(attr(p,"forward"))
    attr(p,"backward") <- dropform(attr(p,"backward"))
    if (!quietly) {
        rpt(p,"final : ")
    }
    return(p)
}

## ----opts, echo = FALSE, message = FALSE--------------------------------------
library("knitr")
knitr::opts_chunk$set(
  )
(load(system.file("testdata", "lmerperf.rda", package="lme4")))# 'ss' 'fitlist'

## ----loadpkg,message=FALSE----------------------------------------------------
library("lme4")

## ----noderivs, eval = FALSE---------------------------------------------------
# m0 <- lmer(y ~ service * dept + (1|s) + (1|d), InstEval,
#      control = lmerControl(calc.derivs = FALSE))

## ----calcs, echo = FALSE------------------------------------------------------
## based on loaded lmerperf file
t1 <- fitlist$basic$times[["elapsed"]]
t2 <- fitlist$noderivs$times[["elapsed"]]
pct <- round(100*(t1-t2)/t1)
e1 <- fitlist$basic$optinfo$feval

## ----glmeropt, echo=FALSE-----------------------------------------------------
gg <- glmerControl()$optimizer

## ----times, as.is=TRUE, echo=FALSE--------------------------------------------
tt <- sort(ss$times[,"elapsed"])
tt2 <- data.frame(optimizer = names(tt), elapsed = tt)
rownames(tt2) <- NULL
knitr::kable(tt2)

## ----default------------------------------------------------------------------
environment(nloptwrap)$defaultControl

## ----calcs2, echo = FALSE-----------------------------------------------------
## based on loaded lmerperf file
t1 <- fitlist$basic$times[["elapsed"]]
t2 <- fitlist$noderivs$times[["elapsed"]]
t3 <- fitlist$nlopt_sloppy$times[["elapsed"]]
pct <- round(100*(t1-t2)/t1)
e1 <- fitlist$basic$optinfo$feval

####  Utility Functions for lme4 Testing
####  ----------------------------------
if(FALSE) ### "Load" these by
    source(system.file("testdata", "lme-tst-funs.R", package="lme4", mustWork=TRUE))
## e.g. from ../../tests/glmmWeights.R

##' example originally from Gabor Grothendieck
##' https://stat.ethz.ch/pipermail/r-sig-mixed-models/2010q2/003726.html
##'
##' @title Simple LMM with 1 ran.eff., 1 fixed eff.
rSim.11 <- function(n, k, sigma=1, beta=2) {
    stopifnot(length(n) == 1, length(k) == 1, n > k, k >= 1, k == round(k))
    x <- 1:n
    fac <- gl(k, 1, n)
    fac.eff <- rnorm(k, 0, 4)[fac]
    e <- rnorm(n, sd=sigma)
    data.frame(x, fac,
               y = 1 + beta * x + fac.eff + e)
}

##' General GLMM simulation  for the model   y ~ x + (1|block)
##' i.e., in "2-way notation",
##'   g( E[Y_{ij}|*] ) =: \eta_{ij} = \beta_1 + \beta_2 x_{ij} + B_j,
##'                                   for  j = 1..nblk,  i = 1..nperblk.
##' @param nblk integer number of "blocks" (sometimes named \eqn{m}).
##' @param nperblk integer number of observations per block. ((may be as small as 1,
##'                e.g., for the binomial or poisson family
##' @param sigma_B standard deviation of random effect  B  (default = 1).
##' @param beta length-2 (intercept, slope) fixed eff. parameter  = c(4,3),
##' @param x  numeric vector of length \eqn{n}   (default = runif(n) (with sd = sqrt(1/12) = 0.2887)
##'          allow also x=c(0,1)   [length ?? FIXME !!]
##' @param shape numeric parameter for Gamma shape  (default = 2)
##' @param nbinom integer N (\code{size}) for binomial trials      (default = 10)
##' @param sd standard deviation ('sigma') for gaussian() family   (default = 1 )
##' @param dInitial an "initial" data frame alternatively to the previous arguments; must contain
##'                 valid columns
##' @param family a \code{\link{family}} object such as \code{Gamma()}, \code{binomial()},
##'               or \code{gaussian(link = log)}.
##'
##' @author Martin Maechler
gSim <- function(nblk   =  26,
                 nperblk= 100,
                 sigma_B = 1,
                 beta = c(4,3),
                 x = runif(n),
                 shape  = 2,  ## shape parameter for Gamma
                 nbinom = 10, ## N for binomial trials
                 sd     = 1,  ## standard deviation ('sigma') for gaussian family
                 dInitial = NULL,
                 family = Gamma())
{
  if(is.null(dInitial)) {
    stopifnot(1 <= nblk, nblk <= 1e7, 1 <= nperblk,
              nblk * nperblk <= 5e7) # some sanity ..
    ## ch.set := a large enough set of "letters", as level-labels for 'block':
    nc <- length(ch.set <- c(LETTERS, letters,
                             paste0(LETTERS,LETTERS), paste0(LETTERS,letters)))
    while(nblk > nc) nc <-
	length(ch.set <- c(paste0(ch.set, LETTERS),
			   paste0(ch.set, letters)))
    stopifnot(nblk <= length(ch.set))
    d <- expand.grid(block = ch.set[1:nblk],
                     rep= 1:nperblk, KEEP.OUT.ATTRS=FALSE)
    stopifnot(nblk == length(levels(d$block)),
              (n <- nrow(d)) == nblk * nperblk, length(x) == n, length(beta) == 2)
    d$ x <- x
    reff_f <- rnorm(nblk, sd = sigma_B)
    ## need intercept large enough to avoid negative values
    d$ eta0 <- beta[1] + beta[2]*x  ## fixed effects only
    d$ eta  <- d$eta0 + reff_f[d$block]
  } else { # use 'dInitial'
    stopifnot(is.data.frame(dInitial),
              is.character(nm.d <- c("block", "x", "eta0", "eta")), # "rep" not really required
              nm.d %in% names(dInitial),
              is.factor(dInitial[["block"]]),
              vapply(nm.d[-1], function(nm) is.numeric(dInitial[[nm]]), NA))
    d <- dInitial
    n <- nrow(d)
  }
    ## Only now  "family-specific" : ----------------------
    d$ mu   <- family$linkinv(d$eta)
    d$ y <- switch(family$family,
                   "Gamma"    = rgamma(n, scale = d$mu/shape, shape=shape),
                   "gaussian" = rnorm(n, d$mu, sd = sd),
                   "poisson"  = rpois(n, d$mu),
		   "binomial" = {
		       z <- rbinom(n, prob=d$mu, size=nbinom)
		       if (nbinom==1)
                           factor(z)
                       else
                           cbind(succ = z, fail = nbinom - z)
		   },
                   stop("Family ", family$family, " not supported here"))
    d
}

### For Model comparisons

## a version of getME() that can be used for objects, both from lme4.0 or lme4
gimME <- function(mod, nm, is.mer = is(mod,"mer"))
    if(is.mer) lme4.0::getME(mod,nm) else lme4::getME(mod,nm)

##' @title All Coefficients of fitted  (G)LMM model
##' @param mod a fitted (G)LMM model from pkg lme4.0 or lme4
##' @param incl.t logical, indicating if 'beta' should include std.errors and t-values
##' @return named vector of coefficients [ beta | theta | sigma ]
##' @author Martin Maechler
allcoefs <- function(mod, incl.t = FALSE) {
    iMer <- is(mod, "mer")
    sigmaF <- {
	if (iMer) lme4.0::sigma else if(getRversion() >= "3.3.0")
	    stats::sigma else lme4::sigma
    }
    ## incl.t: rather also the std.err., t-values:
    c(beta = if(incl.t) coef(summary(mod)) else gimME(mod, "beta", iMer),
      gimME(mod,"theta", iMer),# have their own names
      sigma = sigmaF(mod))
}

##' S3 method but also works for lme4.0:
all.equal.merMod <- function(target, current, incl.t=FALSE, ...) {
    all.equal(allcoefs(target,  incl.t),
              allcoefs(current, incl.t), ...)
}

##' some optimizer info
isOptimized <- function(mod) mod@optinfo[["conv"]][["opt"]] == 0
baseOpti <- function(mod) mod@optinfo[c("optimizer", "conv","feval")]

## use case:  all.equal(emptyCall(mod1), emptyCall(mod2))
emptyCall <- function(mod, arg = quote(`.....`)) {
    cl <- mod@call[1:2]
    names(cl) <- NULL
    cl[[2]] <- arg
    mod@call <- cl
    mod
}

## Matrix >= 1.4.1  keeps names in  diag(.)
Mge141 <- packageVersion("Matrix") >= "1.4.1"
uc <- if(Mge141) c else function(...) c(..., use.names = FALSE)
## uc(a = 1.234, b = 4...)  has names for Matrix >= 1.4.1
unn <- if(Mge141) identity else unname
## unn(vec)  # keeps names for Matrix >= 1.4.1

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----cs-fit-------------------------------------------------------------------
library(lme4)
fm1.cs <- lmer(Reaction ~ Days + cs(1 + Days | Subject), sleepstudy)

## ----getReCovs----------------------------------------------------------------
print(fm1.cs_cov <- getReCovs(fm1.cs))

## ----getInfo------------------------------------------------------------------
getME(fm1.cs, "par")
getME(fm1.cs, "theta")

## ----lambda-mat---------------------------------------------------------------
library(Matrix)
image(getME(fm1.cs, "Lambda"))

## ----varcorr------------------------------------------------------------------
vc_mat <- VarCorr(fm1.cs)
vc_mat$Subject

## ----comparison-setup---------------------------------------------------------
if (!requireNamespace("glmmTMB", quietly = TRUE)) {
  knitr::opts_chunk$set(eval = FALSE)
} else {
  library(glmmTMB)
}

## Often want to ignore attributes and class.
## Set a fairly large tolerance for convenience.
all.equal.nocheck <- function(x, y, tolerance = 3e-5, ..., check.attributes = FALSE, check.class = FALSE) {
  require("Matrix", quietly = TRUE)
  ## working around mode-matching headaches
  if (is(x, "Matrix")) x <- matrix(x)
  if (is(y, "Matrix")) y <- matrix(y)
  all.equal(x, y, ..., tolerance = tolerance, check.attributes = check.attributes, check.class = check.class)
}

get.cor1 <- function(x) {
  v <- VarCorr(x)
  vv <- if (inherits(x, "merMod")) v$group else v$cond$group
  attr(vv, "correlation")[1,2]
}

## ----unstruc-sim--------------------------------------------------------------
n_groups <- 20
n_per_group <- 20
n <- n_groups * n_per_group

set.seed(1)
dat.us <- data.frame(
  group = rep(1:n_groups, each = n_per_group),
  x1 = rnorm(n),
  x2 = rnorm(n)
)
## Constructing a similar dataset for the other examples
gdat.us <- dat.diag <- gdat.diag <- dat.us

form <- y ~ 1 + x1 * x2 + us(1 + x1|group)
dat.us$y <- simulate(form[-2], 
                    newdata = dat.us,
                    family = gaussian,
                    newparams = list(beta = c(-7, 5, -100, 20),
                                     theta = c(2.5, 1.4, 6.3),
                                     sigma = 2))[[1]]

form2 <- y ~ 1 + x1 + us(1 + x1|group)
gdat.us$y <- simulate(
  form2[-2],
  newdata = gdat.us,
  family = binomial,
  newparams = list(
    beta  = c(-1.5, 0.5),     
    theta = c(0.3, 0.1, 0.3)
  ))[[1]]

## ----compare-lmm--------------------------------------------------------------
lme4.us <- lmer(form, data = dat.us, REML = "FALSE")
glmmTMB.us <- glmmTMB(form, dat = dat.us)

## Fixed effects
fixef(lme4.us); fixef(glmmTMB.us)$cond
all.equal.nocheck(fixef(lme4.us), fixef(glmmTMB.us)$cond)

## Sigma
sigma(lme4.us); sigma(glmmTMB.us)
all.equal.nocheck(sigma(lme4.us), sigma(glmmTMB.us))

## Log likelihoods
logLik(lme4.us); logLik(glmmTMB.us)
all.equal.nocheck(logLik(lme4.us), logLik(glmmTMB.us))

## ----compare-cov--------------------------------------------------------------
## Variance-Covariance Matrix
vcov(lme4.us); vcov(glmmTMB.us)$cond
all.equal.nocheck(vcov(lme4.us), vcov(glmmTMB.us)$cond)

## Variance and Covariance Components
all.equal.nocheck(VarCorr(lme4.us)$group,
          VarCorr(glmmTMB.us)$cond$group)

## Conditional Modes of the Random Effects
all.equal.nocheck(ranef(lme4.us)$group, ranef(glmmTMB.us)$cond$group)

## ----compare-glmm-------------------------------------------------------------
glme4.us <- glmer(form2, data = gdat.us, family = binomial)
gglmmTMB.us <- glmmTMB(form2, dat = gdat.us, family = binomial)

## Fixed effects
fixef(glme4.us); fixef(gglmmTMB.us)$cond
all.equal.nocheck(fixef(glme4.us), fixef(gglmmTMB.us)$cond)

## Sigma
all.equal.nocheck(sigma(glme4.us), sigma(gglmmTMB.us))

## Log likelihoods
logLik(glme4.us); logLik(gglmmTMB.us)
all.equal.nocheck(logLik(glme4.us), logLik(gglmmTMB.us))

## ----compare-glmm-cov---------------------------------------------------------
## Variance-Covariance Matrix
vcov(glme4.us); vcov(gglmmTMB.us)$cond
all.equal.nocheck(vcov(glme4.us), vcov(gglmmTMB.us)$cond)

## Variance and Covariance Components
all.equal.nocheck(VarCorr(glme4.us)$group,
          VarCorr(gglmmTMB.us)$cond$group)

## Conditional Modes of the Random Effects
all.equal.nocheck(ranef(glme4.us)$group, ranef(gglmmTMB.us)$cond$group)

## ----fit-homog-diag, eval = FALSE---------------------------------------------
# lme4.us <- lmer(Reaction ~ Days + diag(Days | Subject, hom = TRUE), sleepstudy)
# glmmTMB.us <- glmmTMB(Reaction ~ Days + homdiag(Days | Subject), sleepstudy)

## ----sim-het-diag-------------------------------------------------------------
form <- y ~ 1 + x1 * x2 + diag(1|group)
dat.diag$y <- simulate(form[-2], 
                       newdata = dat.diag,
                       family = gaussian,
                       newparams = list(beta = c(-7, 5, -100, 20),
                                        theta = c(2.5),
                                        sigma = 2))[[1]]

## ----fit-het-diag-------------------------------------------------------------
lme4.diag <- lmer(form, data = dat.diag, REML = "FALSE")
glmmTMB.diag <- glmmTMB(form, dat = dat.diag)

## Fixed effects
fixef(lme4.diag); fixef(glmmTMB.diag)$cond
all.equal.nocheck(fixef(lme4.diag), fixef(glmmTMB.diag)$cond)

## Sigma
sigma(lme4.diag); sigma(glmmTMB.diag)
all.equal.nocheck(sigma(lme4.diag), sigma(glmmTMB.diag))

## Log likelihoods
logLik(lme4.diag); logLik(glmmTMB.diag)
all.equal.nocheck(logLik(lme4.diag), logLik(glmmTMB.diag))

## Variance-Covariance Matrix
vcov(lme4.diag); vcov(glmmTMB.diag)$cond
all.equal.nocheck(vcov(lme4.diag), vcov(glmmTMB.diag)$cond)

## Variance and Covariance Components
all.equal.nocheck(VarCorr(lme4.diag)[[1]], 
          VarCorr(glmmTMB.diag)$cond$group)

## Conditional Modes of the Random Effects
all.equal.nocheck(ranef(lme4.diag)$group, ranef(glmmTMB.diag)$cond$group)

## ----cs-fit-2, eval = FALSE---------------------------------------------------
# lme4.us <- lmer(Reaction ~ Days + cs(Days | Subject, hom = TRUE), sleepstudy)
# glmmTMB.us <- glmmTMB(Reaction ~ Days + cs(Days | Subject), sleepstudy)

## ----simgroup-----------------------------------------------------------------
simGroup <- function(g, n=6, phi=0.6) {
  x <- MASS::mvrnorm(mu = rep(0,n),
                     Sigma = phi^as.matrix(dist(1:n)) )  
  y <- x + rnorm(n)                              
  times <- factor(1:n)
  group <- factor(rep(g,n))
  data.frame(y, times, group)
}

set.seed(1)
dat.cs <- do.call("rbind", lapply(1:2000, simGroup))

## ----more-comp, cache=TRUE----------------------------------------------------
lme4.cs <- lmer(y ~ times + cs(0 + times | group, hom = TRUE), data = dat.cs, REML = FALSE)
glmmTMB.cs <- glmmTMB(y ~ times + homcs(0 + times | group), data = dat.cs)

## Fixed effects
fixef(lme4.cs); fixef(glmmTMB.cs)$cond
all.equal.nocheck(fixef(lme4.cs), fixef(glmmTMB.cs)$cond)

## Sigma
sigma(lme4.cs); sigma(glmmTMB.cs)
all.equal.nocheck(sigma(lme4.cs), sigma(glmmTMB.cs))

## Log likelihoods
logLik(lme4.cs); logLik(glmmTMB.cs)
all.equal.nocheck(logLik(lme4.cs), logLik(glmmTMB.cs))

## Variance-Covariance Matrix
all.equal.nocheck(vcov(lme4.cs), vcov(glmmTMB.cs)$cond)

## Variance and Covariance Components
all.equal.nocheck(VarCorr(lme4.cs)[[1]], 
          VarCorr(glmmTMB.cs)$cond$group)

## Conditional Modes of the Random Effects
all.equal.nocheck(ranef(lme4.cs)$group, ranef(glmmTMB.cs)$cond$group)

## Comparing against the predicted rho value
lme4.rho <- get.cor1(lme4.cs)
glmmTMB.rho <- get.cor1(glmmTMB.cs)
lme4.rho; glmmTMB.rho
all.equal.nocheck(lme4.rho, glmmTMB.rho)

## ----ar1-sim------------------------------------------------------------------
set.seed(1)
dat.ar1 <- do.call("rbind", lapply(1:2000, function(g) simGroup(g, phi = 0.7)))

## ----het-ar1-fit, eval = FALSE------------------------------------------------
# lme4.ar1 <- lmer(y ~ times + ar1(0 + times | group), data = dat.ar1, REML = FALSE)
# glmmTMB.ar1 <- glmmTMB(y ~ times + hetar1(0 + times | group), data = dat.ar1)

## ----ar1-fit, cache=TRUE------------------------------------------------------
lme4.ar1 <- lmer(y ~ times + ar1(0 + times | group, hom = TRUE), data = dat.ar1, REML = FALSE)
glmmTMB.ar1 <- glmmTMB(y ~ times + ar1(0 + times | group), data = dat.ar1)

## Fixed effects
fixef(lme4.ar1); fixef(glmmTMB.ar1)$cond
all.equal.nocheck(fixef(lme4.ar1), fixef(glmmTMB.ar1)$cond)

## Sigma
sigma(lme4.ar1); sigma(glmmTMB.ar1)
all.equal.nocheck(sigma(lme4.ar1), sigma(glmmTMB.ar1))

## Log likelihoods
logLik(lme4.ar1); logLik(glmmTMB.ar1)
all.equal.nocheck(logLik(lme4.ar1), logLik(glmmTMB.ar1))

## Variance-Covariance Matrix
all.equal.nocheck(vcov(lme4.ar1), vcov(glmmTMB.ar1)$cond)

## Variance and Covariance Components
all.equal.nocheck(VarCorr(lme4.ar1)$group, 
                  VarCorr(glmmTMB.ar1)$cond$group)

## Conditional Modes of the Random Effects
all.equal.nocheck(ranef(lme4.ar1)$group, ranef(glmmTMB.ar1)$cond$group)

## Comparing against the predicted rho value
lme4.ar1.rho <- get.cor1(lme4.ar1)
glmmTMB.ar1.rho <- get.cor1(glmmTMB.ar1)
lme4.ar1.rho; glmmTMB.ar1.rho
all.equal.nocheck(lme4.ar1.rho, glmmTMB.ar1.rho)

######################################################################
# Below is code frequently used for testing
# see: test-covariance_structures.R and test-covariance_nlmer.R

all.equal.nocheck <- function(x, y, ..., check.attributes = FALSE, check.class = FALSE) {
  ## working around mode-matching headaches
  if (is(x, "Matrix")) x <- matrix(x)
  if (is(y, "Matrix")) y <- matrix(y)
  all.equal(x, y, ..., check.attributes = check.attributes, check.class = check.class)
}

## set default tolerance to 5e-5 since we mostly use that
## 'tolerance' must be written out in full since it comes after ...
expect_equal_nocheck <- function(...,  tolerance = 5e-5) {
  expect_true(isTRUE(all.equal.nocheck(..., tolerance = tolerance)))
}

## Getting all equal as a number (in the all.equal examples documentation;
## don't know why they didn't make an argument instead!?)
all.eqNum <- function(...) {
  an <- all.equal.nocheck(...)
  if (isTRUE(an)) return(0)
  ## if check is less than tolerance all.equal returns TRUE, so sub() coerces to "TRUE"
  ##  and as.numeric() returns NA ...
  as.numeric(sub(".*:", '', an))
}

hsb <- read.csv('https://raw.githubusercontent.com/rnorouzian/e/master/hsb.csv')
source("strip_env.R")
library(lme4)
library(lattice)

m31 <- lmer(math ~ ses*meanses + (ses | sch.id), data = hsb)
p <-  strip_profile(profile(m31))
densityplot(p)
saveRDS(p, file="harmel_profile.rds", version=2)

### How to generate the  trees513.RData, needed in ../../tests/HSAURtrees.R:

data("trees513", package = "multcomp")
trees513A <- droplevels(subset(trees513, !species %in%
			       c("fir", "ash/maple/elm/lime", "softwood (other)")))
levels(trees513A$species)[nlevels(trees513A$species)] <- "hardwood"
trees513B <- droplevels(subset(trees513,
			       !species %in% c("fir", "softwood (other)")))
save("trees513A", "trees513B", file="trees513.RData")

## ----preliminaries,include=FALSE,cache=FALSE,message=FALSE----------
options(width=70, show.signif.stars=FALSE,
        str=strOptions(strict.width="cut"),
        ## prefer empty continuation for reader's cut'n'paste:
        continue = "   ", #JSS: prompt = "R> ", continue = "+  ",
        useFancyQuotes = FALSE)
library("knitr")
library("lme4")
library("ggplot2")# Keeping default theme, nicer "on line":
#JSS theme_set(theme_bw())
library("grid")
zmargin <- theme(panel.spacing=unit(0,"lines"))
library("lattice")
library("minqa")
library("reformulas")
opts_chunk$set(engine='R',dev='pdf', fig.width=9, fig.height=5.5,
               prompt=TRUE, cache=TRUE, tidy=FALSE, comment=NA, error = FALSE)
knitr::render_sweave()

## ----sleep----------------------------------------------------------
str(sleepstudy)

## ----sleepPlot, echo=FALSE, fig.scap="Reaction time versus days by subject", fig.cap="Average reaction time versus days of sleep deprivation by subject.  Subjects ordered (from left to right starting on the top row) by increasing slope of subject-specific linear regressions.", fig.align='center', fig.pos='tb'----
## BMB: seemed more pleasing to arrange by increasing slope rather than
## intercept ...
xyplot(Reaction ~ Days | Subject, sleepstudy, aspect = "xy",
       layout = c(9, 2), type = c("g", "p", "r"),
       index.cond = function(x, y) coef(lm(y ~ x))[2],
       xlab = "Days of sleep deprivation",
       ylab = "Average reaction time (ms)",
       as.table = TRUE)

## ----outputExample,results="hide"-----------------------------------
fm1 <- lmer(Reaction ~ Days + (Days | Subject), sleepstudy)

## ----modularExampleFormula, tidy=FALSE, eval=FALSE------------------
# parsedFormula <- lFormula(formula = Reaction ~ Days + (Days | Subject),
#                              data = sleepstudy)

## ----modularExampleObjective, tidy=FALSE, eval=FALSE----------------
# devianceFunction <- do.call(mkLmerDevfun, parsedFormula)

## ----modularExampleOptimization, tidy=FALSE, eval=FALSE-------------
# optimizerOutput <- optimizeLmer(devianceFunction)

## ----modularExampleOutput, tidy=FALSE, eval=FALSE-------------------
# mkMerMod(   rho = environment(devianceFunction),
#             opt = optimizerOutput,
#          reTrms = parsedFormula$reTrms,
#              fr = parsedFormula$fr)

## ----MMformula, eval=FALSE------------------------------------------
# resp ~ FEexpr + (REexpr1 | factor1) + (REexpr2 | factor2) + ...

## ----uncorrelatedModel,results="hide"-------------------------------
fm2 <- lmer(Reaction ~ Days + (Days || Subject), sleepstudy)

## ----setSeed, echo=FALSE--------------------------------------------
set.seed(2)

## ----factorToSparseMatrix-------------------------------------------
(f <- gl(3, 2))
(Ji <- t(as(f, Class = "sparseMatrix")))

## ----rawModelMatrix-------------------------------------------------
(Xi <- cbind(1, rep.int(c(-1, 1), 3L)))

## ----KhatriRao------------------------------------------------------
(Zi <- t(KhatriRao(t(Ji), t(Xi))))

## ----sanityCheck, include=FALSE-------------------------------------
## alternative formulation of Zi (eq:Zi)
rbind(
  Ji[1,] %x% Xi[1,],
  Ji[2,] %x% Xi[2,],
  Ji[3,] %x% Xi[3,],
  Ji[4,] %x% Xi[4,],
  Ji[5,] %x% Xi[5,],
  Ji[6,] %x% Xi[6,])

## ----nc, echo=FALSE-------------------------------------------------
nc <- 3

## ----template-------------------------------------------------------
(rowIndices <- rep(1:nc, 1:nc))
(colIndices <- sequence(1:nc))
(template <- sparseMatrix(rowIndices, colIndices,
                          x = 1 * (rowIndices == colIndices)))

## ----thetaFromTemplate----------------------------------------------
(theta <- template@x)

## ----thetaSanityCheck, include=FALSE--------------------------------
lFormula(Reaction ~ (Days + I(Days^2) | Subject), sleepstudy)$reTrms$theta

## ----nl, echo=FALSE-------------------------------------------------
nl <- 2

## ----relativeCovarianceBlock----------------------------------------
(Lambdati <- .bdiag(rep(list(t(template)), nl)))

## ----relativeCovarianceBlockIndices---------------------------------
LindTemplate <- rowIndices + nc * (colIndices - 1) - choose(colIndices, 2)
(Lind <- rep(LindTemplate, nl))

## ----newTheta-------------------------------------------------------
thetanew <- round(runif(length(theta)), 1)

## ----relativeCovarianceBlockUpdate----------------------------------
Lambdati@x <- thetanew[Lind]

## ----PLSupdateTheta, eval = FALSE-----------------------------------
# Lambdat@x[] <- mapping(theta)

## ----PLSupdateThetaWithLind-----------------------------------------
mapping <- function(theta) theta[Lind]

## ----exampleNewTheta------------------------------------------------
thetanew <- c(1, -0.1, 2, 0.1, -0.2, 3)

## ----exampleLindUpdate----------------------------------------------
Lambdati@x[] <- mapping(thetanew)
Lambdati

## ----updateL, eval=FALSE--------------------------------------------
# L <- update(L, Lambdat %*% ZtW, mult = 1)

## ----PLSsolve, eval = FALSE-----------------------------------------
# cu[] <- as.vector(solve(L, solve(L, Lambdat %*% ZtWy,
#                                  system = "P"), system = "L"))
# RZX[] <- as.vector(solve(L, solve(L, Lambdat %*% ZtWX,
#                                   system = "P"), system = "L"))
# RXtRX <- as(XtWX - crossprod(RZX), "dpoMatrix")
# beta[] <- as.vector(solve(RXtRX, XtWy - crossprod(RZX, cu)))
# u[] <- as.vector(solve(L, solve(L, cu - RZX %*% beta,
#                                 system = "Lt"), system = "Pt"))

## ----PLSupdateResp, eval = FALSE------------------------------------
# b[] <- as.vector(crossprod(Lambdat, u))
# mu[] <- as.vector(crossprod(Zt, b) + X %*% beta + offset)
# wtres <- sqrtW * (y - mu)

## ----PLScalculateObjective, eval = FALSE----------------------------
# pwrss <- sum(wtres^2) + sum(u^2)
# logDet <- 2*determinant(L, logarithm = TRUE)$modulus
# if (REML) logDet <- logDet + determinant(RXtRX,
#                                          logarithm = TRUE)$modulus
# attributes(logDet) <- NULL
# profDev <- logDet + degFree * (1 + log(2 * pi * pwrss) - log(degFree))

## ----vcovByHand-----------------------------------------------------
RX <- getME(fm1, "RX")
sigma2 <- sigma(fm1)^2
sigma2 * chol2inv(RX)

## ----condVarExample, echo=FALSE, eval=FALSE-------------------------
# s2 <- sigma(fm1)^2
# Lambdat <- getME(fm1, "Lambdat")
# Lambda <- getME(fm1, "Lambda")
# Zt <- getME(fm1, "Zt")
# Z <- getME(fm1, "Z")
# y <- getME(fm1, "y")
# X <- getME(fm1, "X")
# beta <- getME(fm1, "beta")
# L <- getME(fm1, "L")
# (V <- crossprod(solve(L, system = "L")))[1:2, 1:2]
# (s2 * Lambda %*% V %*% Lambdat)[1:2, 1:2]
# attr(ranef(fm1, condVar = TRUE)$Subject, "postVar")[ , , 1] # should be same as Lam %*% V %*% Lamt
# (U <- as.vector((V %*% Lambdat %*% Zt %*%  (y - X %*% beta))))
# getME(fm1, "u")

## ----updateModel----------------------------------------------------
fm3 <- update(fm1, . ~ . - (Days | Subject) + (1 | Subject))
formula(fm3)

## ----summary1,echo=FALSE--------------------------------------------
ss <- summary(fm1)
cc <- capture.output(print(ss))
reRow <- grep("^Random effects", cc)
cat(cc[1:(reRow - 2)], sep = "\n")

## ----summary1reproduced, eval=FALSE---------------------------------
# formula(fm1)
# REMLcrit(fm1)
# quantile(residuals(fm1, "pearson", scaled = TRUE))

## ----summary2, echo=FALSE-------------------------------------------
feRow <- grep("^Fixed effects", cc)
cat(cc[reRow:(feRow - 2)], sep = "\n")

## ----summary2reproduced, eval=FALSE---------------------------------
# print(vc <- VarCorr(fm1), comp = c("Variance", "Std.Dev."))
# nobs(fm1)
# ngrps(fm1)
# sigma(fm1)

## ----VarCorr--------------------------------------------------------
as.data.frame(VarCorr(fm1))

## ----summary3, echo=FALSE-------------------------------------------
corRow <- grep("^Correlation", cc)
cat(cc[feRow:(corRow - 2)], sep = "\n")

## ----summary4, echo=FALSE-------------------------------------------
cat(cc[corRow:length(cc)], sep = "\n")

## ----diagplot1,fig.keep="none"--------------------------------------
plot(fm1, type = c("p", "smooth"))

## ----diagplot2,fig.keep="none"--------------------------------------
plot(fm1, sqrt(abs(resid(.))) ~ fitted(.),
     type = c("p", "smooth"))

## ----diagplot3,fig.keep="none"--------------------------------------
qqmath(fm1, id = 0.05)

## ----ppsim,results="hide"-------------------------------------------
iqrvec <- sapply(simulate(fm1, 1000), IQR)
obsval <- IQR(sleepstudy$Reaction)
post.pred.p <- mean(obsval >= c(obsval, iqrvec))

## ----anovaQuadraticModel--------------------------------------------
fm4 <- lmer(Reaction ~ polyDays[ , 1] + polyDays[ , 2] +
            (polyDays[ , 1] + polyDays[ , 2] | Subject),
            within(sleepstudy, polyDays <- poly(Days, 2)))
anova(fm4)

## ----anovaSanityCheck, include=FALSE--------------------------------
(getME(fm4, "RX")[2, ] %*% getME(fm4, "fixef"))^2

## ----anovaManyModels------------------------------------------------
anova(fm1, fm2, fm3)

## ----anovaRes,echo=FALSE--------------------------------------------
fm3ML <- refitML(fm3)
fm2ML <- refitML(fm2)
fm1ML <- refitML(fm1)
ddiff <- deviance(fm3ML) - deviance(fm2ML)
dp <- pchisq(ddiff, 1, lower.tail = FALSE)
ddiff2 <- deviance(fm2ML) - deviance(fm1ML)

## ----pvaluesHelp, eval=FALSE----------------------------------------
# help("pvalues")

## ----compareCI,echo=FALSE,cache=TRUE,message=FALSE,warning=FALSE----
ccw <- confint(fm1, method = "Wald")
ccp <- confint(fm1, method = "profile", signames = FALSE)
ccb <- confint(fm1, method = "boot")

## ----CIqcomp,echo=FALSE,eval=TRUE-----------------------------------
rtol <- function(x,y) {
    abs((x - y) / ((x + y) / 2))
}
bw <- apply(ccb, 1, diff)
pw <- apply(ccp, 1, diff)
mdiffpct <- round(100 * max(rtol(bw, pw)))

## ----CIplot,echo=FALSE,eval=FALSE-----------------------------------
# ## obsolete
# ## ,fig.cap="Comparison of confidence intervals",fig.scap="CI comparison"
# tf <- function(x, method) data.frame(method = method,
#                par = rownames(x),
#                setNames(as.data.frame(x), c("lwr", "upr")))
# cc.all <- do.call(rbind, mapply(tf, list(ccw, ccp, ccb),
#                                c("Wald", "profile", "boot"),
#                      SIMPLIFY = FALSE))
# ggplot(cc.all, aes(x = 1, ymin = lwr, ymax = upr, colour = method)) +
#     geom_linerange(position = position_dodge(width = 0.5)) +
#     facet_wrap( ~ par, scale = "free") +
#     theme(axis.text.x = element_blank()) +
#     labs(x = "")

## ----profile_calc,echo=FALSE,cache=TRUE-----------------------------
pf <- profile(fm1)

## ----profile_zeta_plot,fig.cap="Profile zeta plot: \\code{xyplot(prof.obj)}",fig.scap="Profile zeta plot",echo=FALSE,fig.align='center',fig.pos='tb'----
xyplot(pf)

## ----profile_density_plot,fig.cap="Profile density plot: \\code{densityplot(prof.obj)}",echo=FALSE,fig.align='center',fig.pos='tb'----
densityplot(pf)

## ----profile_pairs_plot,fig.cap="Profile pairs plot: \\code{splom(prof.obj)}",echo=FALSE,fig.height=8,fig.width=8,fig.align='center',fig.pos='htb',out.height='5.5in'----
splom(pf)

## ----modularSimulationFormula---------------------------------------
form <- respVar ~ 1 + (explVar1 + explVar2 | groupFac)

## ----dataTemplate---------------------------------------------------
set.seed(1)
dat <- mkDataTemplate(form,
  nGrps = 500,
  nPerGrp = 20,
  rfunc = rnorm)
head(dat)

## ----parsTemplate---------------------------------------------------
(pars <- mkParsTemplate(form, dat))

## ----simCorr--------------------------------------------------------
vc <- matrix(c(1.0, 0.5, 0.5,
  0.5, 1.0, 0.0,
  0.5, 0.0, 1.0), 3, 3)

## ----vcTheta--------------------------------------------------------
pars$theta[] <- Vv_to_Cv(mlist2vec(vc))

## ----varCorrStructure-----------------------------------------------
dat$respVar <- simulate(form,
  newdata = dat,
  newparams = pars,
  family = "gaussian")[[1]]

## ----graphSims------------------------------------------------------
formLm <- form
formLm[[3]] <- reformulas::findbars(form)[[1]]
print(formLm)
cor(t(sapply(lmList(formLm, dat), coef)))

## ----phiToTheta, cache = FALSE--------------------------------------
phiToTheta <- function(phi) {
    theta5 <- -(phi[2]*phi[3])/phi[4]
    c(phi[1:4], theta5, phi[5])
}

## ----compute deviance function modular, cache = FALSE---------------
lf <- lFormula(formula = form, data = dat, REML = TRUE)
devf <- do.call(mkLmerDevfun, lf)

## ----wrapper modular, cache = FALSE---------------------------------
devfWrap <- function(phi) devf(phiToTheta(phi))

## ----opt modular, cache = FALSE-------------------------------------
opt <- bobyqa(par = lf$reTrms$theta[-5],
  fn = devfWrap,
  lower = lf$reTrms$lower[-5])

## ----varCorr modular, cache = FALSE---------------------------------
vcEst <- vec2mlist(Cv_to_Vv(phiToTheta(opt$par)))[[1]]
dimnames(vcEst) <- rep(lf$reTrms$cnms, 2)
round(vcEst, 2)
vc

## ----simulateSplineData,message=FALSE-------------------------------
library("gamm4")
library("splines")
set.seed(1)
n <- 100
pSimulation <- 3
pStatistical <- 8
x <- rnorm(n)
Bsimulation <- ns(x, pSimulation)
Bstatistical <- ns(x, pStatistical)
beta <- rnorm(pSimulation)
y <- as.numeric(Bsimulation %*% beta + rnorm(n, sd = 0.3))

## ----splineExampleDataPlot, fig.width=4, fig.height=3,fig.align="center"----
par(mar = c(4, 4, 1, 1), las = 1, bty = "l")
plot(x, y, las = 1)
lines(x[order(x)], (Bsimulation %*% beta)[order(x)])

## ----splineExampleApproximateFormula--------------------------------
pseudoGroups <- as.factor(rep(1:pStatistical, length = n))
parsedFormula <- lFormula(y ~ x + (1 | pseudoGroups))

## ----splineExampleModifyZt------------------------------------------
parsedFormula$reTrms <- within(parsedFormula$reTrms, {
    Bt <- t(as.matrix(Bstatistical))[]
    cnms$pseudoGroups <- "spline"
    Zt <- as(Bt, class(Zt))
})

## ----splineExampleRemainingModularSteps-----------------------------
devianceFunction <- do.call(mkLmerDevfun, parsedFormula)
optimizerOutput <- optimizeLmer(devianceFunction)
mSpline <- mkMerMod(   rho = environment(devianceFunction),
                       opt = optimizerOutput,
                    reTrms = parsedFormula$reTrms,
                        fr = parsedFormula$fr)
mSpline

## ----splineExampleFittedModelPlot, fig.width=4, fig.height=3, fig.align="center"----
xNew <- seq(min(x), max(x), length = 100)
newBstatistical <- predict(Bstatistical, xNew)
yHat <-   cbind(1, xNew) %*% getME(mSpline, "fixef") +
        newBstatistical %*% getME(mSpline, "u")
par(mar = c(4, 4, 1, 1), las = 1, bty = "l")
plot(x, y)
lines(xNew, yHat)
lines(x[order(x)], (Bsimulation %*% beta)[order(x)],lty = 2)
legend("topright", bty = "n", c("fitted", "generating"), lty = 1:2,col = 1)

####  Utility Functions for lme4 Testing
####  ----------------------------------
if(FALSE) ### "Load" these by
    source(system.file("testdata", "lme-tst-funs.R", package="lme4", mustWork=TRUE))
## e.g. from ../../tests/glmmWeights.R

##' example originally from Gabor Grothendieck
##' https://stat.ethz.ch/pipermail/r-sig-mixed-models/2010q2/003726.html
##'
##' @title Simple LMM with 1 ran.eff., 1 fixed eff.
rSim.11 <- function(n, k, sigma=1, beta=2) {
    stopifnot(length(n) == 1, length(k) == 1, n > k, k >= 1, k == round(k))
    x <- 1:n
    fac <- gl(k, 1, n)
    fac.eff <- rnorm(k, 0, 4)[fac]
    e <- rnorm(n, sd=sigma)
    data.frame(x, fac,
               y = 1 + beta * x + fac.eff + e)
}

##' General GLMM simulation  for the model   y ~ x + (1|block)
##' i.e., in "2-way notation",
##'   g( E[Y_{ij}|*] ) =: \eta_{ij} = \beta_1 + \beta_2 x_{ij} + B_j,
##'                                   for  j = 1..nblk,  i = 1..nperblk.
##' @param nblk integer number of "blocks" (sometimes named \eqn{m}).
##' @param nperblk integer number of observations per block. ((may be as small as 1,
##'                e.g., for the binomial or poisson family
##' @param sigma_B standard deviation of random effect  B  (default = 1).
##' @param beta length-2 (intercept, slope) fixed eff. parameter  = c(4,3),
##' @param x  numeric vector of length \eqn{n}   (default = runif(n) (with sd = sqrt(1/12) = 0.2887)
##'          allow also x=c(0,1)   [length ?? FIXME !!]
##' @param shape numeric parameter for Gamma shape  (default = 2)
##' @param nbinom integer N (\code{size}) for binomial trials      (default = 10)
##' @param sd standard deviation ('sigma') for gaussian() family   (default = 1 )
##' @param dInitial an "initial" data frame alternatively to the previous arguments; must contain
##'                 valid columns
##' @param family a \code{\link{family}} object such as \code{Gamma()}, \code{binomial()},
##'               or \code{gaussian(link = log)}.
##'
##' @author Martin Maechler
gSim <- function(nblk   =  26,
                 nperblk= 100,
                 sigma_B = 1,
                 beta = c(4,3),
                 x = runif(n),
                 shape  = 2,  ## shape parameter for Gamma
                 nbinom = 10, ## N for binomial trials
                 sd     = 1,  ## standard deviation ('sigma') for gaussian family
                 dInitial = NULL,
                 family = Gamma())
{
  if(is.null(dInitial)) {
    stopifnot(1 <= nblk, nblk <= 1e7, 1 <= nperblk,
              nblk * nperblk <= 5e7) # some sanity ..
    ## ch.set := a large enough set of "letters", as level-labels for 'block':
    nc <- length(ch.set <- c(LETTERS, letters,
                             paste0(LETTERS,LETTERS), paste0(LETTERS,letters)))
    while(nblk > nc) nc <-
	length(ch.set <- c(paste0(ch.set, LETTERS),
			   paste0(ch.set, letters)))
    stopifnot(nblk <= length(ch.set))
    d <- expand.grid(block = ch.set[1:nblk],
                     rep= 1:nperblk, KEEP.OUT.ATTRS=FALSE)
    stopifnot(nblk == length(levels(d$block)),
              (n <- nrow(d)) == nblk * nperblk, length(x) == n, length(beta) == 2)
    d$ x <- x
    reff_f <- rnorm(nblk, sd = sigma_B)
    ## need intercept large enough to avoid negative values
    d$ eta0 <- beta[1] + beta[2]*x  ## fixed effects only
    d$ eta  <- d$eta0 + reff_f[d$block]
  } else { # use 'dInitial'
    stopifnot(is.data.frame(dInitial),
              is.character(nm.d <- c("block", "x", "eta0", "eta")), # "rep" not really required
              nm.d %in% names(dInitial),
              is.factor(dInitial[["block"]]),
              vapply(nm.d[-1], function(nm) is.numeric(dInitial[[nm]]), NA))
    d <- dInitial
    n <- nrow(d)
  }
    ## Only now  "family-specific" : ----------------------
    d$ mu   <- family$linkinv(d$eta)
    d$ y <- switch(family$family,
                   "Gamma"    = rgamma(n, scale = d$mu/shape, shape=shape),
                   "gaussian" = rnorm(n, d$mu, sd = sd),
                   "poisson"  = rpois(n, d$mu),
		   "binomial" = {
		       z <- rbinom(n, prob=d$mu, size=nbinom)
		       if (nbinom==1)
                           factor(z)
                       else
                           cbind(succ = z, fail = nbinom - z)
		   },
                   stop("Family ", family$family, " not supported here"))
    d
}

### For Model comparisons

## a version of getME() that can be used for objects, both from lme4.0 or lme4
gimME <- function(mod, nm, is.mer = is(mod,"mer"))
    if(is.mer) lme4.0::getME(mod,nm) else lme4::getME(mod,nm)

##' @title All Coefficients of fitted  (G)LMM model
##' @param mod a fitted (G)LMM model from pkg lme4.0 or lme4
##' @param incl.t logical, indicating if 'beta' should include std.errors and t-values
##' @return named vector of coefficients [ beta | theta | sigma ]
##' @author Martin Maechler
allcoefs <- function(mod, incl.t = FALSE) {
    iMer <- is(mod, "mer")
    sigmaF <- {
	if (iMer) lme4.0::sigma else if(getRversion() >= "3.3.0")
	    stats::sigma else lme4::sigma
    }
    ## incl.t: rather also the std.err., t-values:
    c(beta = if(incl.t) coef(summary(mod)) else gimME(mod, "beta", iMer),
      gimME(mod,"theta", iMer),# have their own names
      sigma = sigmaF(mod))
}

##' S3 method but also works for lme4.0:
all.equal.merMod <- function(target, current, incl.t=FALSE, ...) {
    all.equal(allcoefs(target,  incl.t),
              allcoefs(current, incl.t), ...)
}

##' some optimizer info
isOptimized <- function(mod) mod@optinfo[["conv"]][["opt"]] == 0
baseOpti <- function(mod) mod@optinfo[c("optimizer", "conv","feval")]

## use case:  all.equal(emptyCall(mod1), emptyCall(mod2))
emptyCall <- function(mod, arg = quote(`.....`)) {
    cl <- mod@call[1:2]
    names(cl) <- NULL
    cl[[2]] <- arg
    mod@call <- cl
    mod
}

### suppressPackageStartupMessages(...)  as we have an *.Rout.save to Rdiff against
stopifnot(suppressPackageStartupMessages(require(lme4)))
options(show.signif.stars = FALSE, useFancyQuotes=FALSE)

source(system.file("test-tools-1.R", package = "Matrix"))# identical3() etc
all.EQ <- function(u,v, ...) all.equal.X(u, v, except = c("call", "frame"), ...)
S4_2list <- function(obj) {   # no longer used
   sn <- slotNames(obj)
   structure(lapply(sn, slot, object = obj), .Names = sn)
}

if (lme4:::testLevel() <= 1)
    quit("no")
## otherwise *print* normally:

oldOpts <- options(digits=2)
(fm1 <- lmer(Reaction ~ Days + (Days|Subject), sleepstudy))
(fm1a <- lmer(Reaction ~ Days + (Days|Subject), sleepstudy, REML = FALSE))
(fm2 <- lmer(Reaction ~ Days + (1|Subject) + (0+Days|Subject), sleepstudy))
anova(fm1, fm2)

## Now works for glmer
fm1. <- suppressWarnings(glmer(Reaction ~ Days + (Days|Subject), sleepstudy))
## default family=gaussian/identity link -> automatically calls  lmer()  (but with a warning)
## hack call -- comes out unimportantly different
fm1.@call[[1]] <- quote(lmer)
stopifnot(all.equal(fm1, fm1.))
## Test against previous version in lmer1 (using bobyqa for consistency)
#(fm1. <- lmer1(Reaction ~ Days + (Days|Subject), sleepstudy, opt = "bobyqa"))
#stopifnot(all.equal(fm1@devcomp$cmp['REML'], fm1.@devcomp$cmp['REML']),
#          all.equal(fixef(fm1), fixef(fm1.)),
#          all.equal(fm1@re@theta, fm1.@theta, tolerance = 1.e-7),
#          all.equal(ranef(fm1), ranef(fm1.)))

## compDev = FALSE no longer applies to lmer
## Test 'compDev = FALSE' (vs TRUE)
## fm1. <- lmer(Reaction ~ Days + (Days|Subject), sleepstudy,
##              compDev = FALSE)#--> use R code (not C++) for deviance computation
## stopifnot(all.equal(fm1@devcomp$cmp['REML'], fm1.@devcomp$cmp['REML']),
##           all.equal(fixef(fm1), fixef(fm1.)),
##           all.equal(fm1@re@theta, fm1.@re@theta, tolerance = 1.e-7),
##           all.equal(ranef(fm1), ranef(fm1.), tolerance = 1.e-7))

vv <- vcov(fm1)
cc <- Matrix::cov2cor(vv)
dimnames(cc) <- dimnames(vv) ## work around Matrix 1.5.2 buglet

stopifnot(
    all.equal(fixef(fm1), fixef(fm2), tolerance = 1.e-13)
   ,
    all.equal(unname(fixef(fm1)),
              c(251.405104848485, 10.467285959595), tolerance = 1e-13)
   ,
    all.equal(cc["(Intercept)", "Days"],
              -0.1375, tolerance = 4e-4)
)

fm1ML <- refitML(fm1)
fm2ML <- refitML(fm2)
(cbind(AIC= c(m1= AIC(fm1ML), m2= AIC(fm2ML)),
       BIC= c(    BIC(fm1ML),     BIC(fm2ML))) -> ICm)
stopifnot(all.equal(c(ICm), c(1763.94, 1762, 1783.1, 1777.97),
                    tolerance = 1e-5))# see 1.2e-6

(fm3 <- lmer(Yield ~ 1|Batch, Dyestuff2))
stopifnot(all.equal(coef(summary(fm3)),
		    array(c(5.6656, 0.67838803150, 8.3515624346),
			  c(1,3), dimnames = list("(Intercept)",
				  c("Estimate", "Std. Error", "t value")))))
showProc.time() #

### {from ../man/lmer.Rd } --- compare lmer & lmer1 ---------------
(fmX1 <- lmer(Reaction ~ Days + (Days|Subject), sleepstudy))
(fm.1 <- lmer(Reaction ~ Days + (1|Subject) + (0+Days|Subject), sleepstudy))

#(fmX2 <- lmer2(Reaction ~ Days + (Days|Subject), sleepstudy))
#(fm.2 <- lmer2(Reaction ~ Days + (1|Subject) + (0+Days|Subject), sleepstudy))
## check update(<mer>, <formula>):
fm.3 <- update(fmX1, . ~ Days + (1|Subject) + (0+Days|Subject))
stopifnot(all.equal(fm.1, fm.3))

fmX1s <- lmer(Reaction ~ Days + (Days|Subject), sleepstudy )# no longer:, sparseX=TRUE)
#fmX2s <- lmer2(Reaction ~ Days + (Days|Subject), sleepstudy, sparseX=TRUE)
options(oldOpts)  ## restore digits

showProc.time() #

for(nm in c("coef", "fixef", "ranef", "sigma",
	     "model.matrix", "model.frame" , "terms")) {
    cat(sprintf("%15s : ", nm))
    FUN <- get(nm)
    F.fmX1s <- FUN(fmX1s)
#    F.fmX2s <- FUN(fmX2s)
#    if(nm == "model.matrix") {
#        F.fmX1s <- as(F.fmX1s, "denseMatrix")
#        F.fmX2s <- as(F.fmX2s, "denseMatrix")
#	FF <- function(.) {r <- FUN(.); row.names(r) <- NULL
#			   as(r, "generalMatrix") }
#    } # else
    FF <- FUN
    stopifnot(
	      all.equal( FF(fmX1), F.fmX1s, tolerance =  1e-6)
#	      ,
#	      all.equal( FF(fmX2), F.fmX2s, tolerance = 1e-5)
#              ,
#	      all.equal( FF(fm.1), F.fmX2s, tolerance = 9e-6) ## these are different models
#              ,
#              all.equal(F.fmX2s,   F.fmX1s, tolerance = 6e-6)
#              ,
#              all.equal(FUN(fm.1), FUN(fm.2), tolerance = 6e-6)
              ,
              TRUE)
    cat("[Ok]\n")
}


## transformed vars should work[even if non-sensical as here;failed in 0.995-1]
fm2l <- lmer(log(Reaction) ~ log(Days+1) + (log(Days+1)|Subject),
             data = sleepstudy, REML = FALSE)
## no need for an expand method now : xfm2 <- expand(fm2)

stopifnot(dim(ranef(fm2l)[[1]]) == c(18, 2),
          is((c3 <- coef(fm3)), "coef.mer"),
          all(fixef(fm3) == c3$Batch),## <-- IFF  \hat{\sigma^2} == 0
          TRUE)



## Simple example by Andrew Gelman (2006-01-10) ----
n.groups <- 10 ; n.reps <- 2
n <- length(group.id <- gl(n.groups, n.reps))
## simulate the varying parameters and the data:
set.seed(0)
a.group <- rnorm(n.groups, 1, 2)
y <- rnorm (n, a.group[group.id], 1)
## fit and summarize the model
fit.1 <- lmer (y ~ 1 + (1 | group.id))
oldOpts <- options(digits=3)
coef (fit.1)
options(oldOpts)
## check show( <"summary.mer"> ):
(sf1 <- summary(fit.1)) # --> now looks as for fit.1

stopifnot(all.equal(fixef(fit.1), c("(Intercept)" = 1.571312129)),
	  all.equal(unname(ranef(fit.1, drop=TRUE)[["group.id"]]), structure(
		   c(1.8046888, -1.8097665, 1.6146451, 1.5408268, -0.1331995,
                     -3.3306655, -1.8259277, -0.8735145, -0.3591311,  3.3720441),
                   postVar = rep.int(0.311091076, 10)),
		    tolerance = 1e-5)
	  )


## ranef and coef
rr <- ranef(fm1)
stopifnot(is.list(rr), length(rr) == 1, is.data.frame(rr[[1]]))
print(plot(rr))
stopifnot(is(cc <- coef(fm1), "coef.mer"),
	  is.list(cc), length(cc) == 1, is.data.frame(cc[[1]]))
print(plot(cc))
rr <- ranef(fm2)
stopifnot(is.list(rr), length(rr) == 1, is.data.frame(rr[[1]]))
print(plot(rr))
stopifnot(is(cc <- coef(fm2), "coef.mer"),
	  is.list(cc), length(cc) == 1, is.data.frame(cc[[1]]))
print(plot(cc))

showProc.time() #

## Invalid factor specification -- used to seg.fault:
set.seed(1)
dat <- within(data.frame(lagoon = factor(rep(1:4,each = 25)),
                         habitat = factor(rep(1:20, each = 5))),
          {
              y <- round(10*rnorm(100, m = 10*as.numeric(lagoon)))
          })

tt <- suppressWarnings(try(reg <- lmer(y ~ habitat + (1|habitat*lagoon), data = dat)
                                        )
                                         # did seg.fault)
    ) # now gives error                 ^- should be ":"
## suppress warning that uses different quoting conventions on
## R-release vs. R-devel

## ignore singular fits as well as hess/grad problems
## (Windows gets singular fits, other platforms don't ...)
ctrl0 <- lmerControl(
    check.conv.singular="ignore",
    check.conv.hess="ignore",
    check.conv.grad="ignore")
r1  <- lmer(y ~ 0+habitat + (1|habitat:lagoon), data = dat,
            control=ctrl0) # ok, but senseless
r1b <- lmer(y ~ 0+habitat + (1|habitat), data = dat,
            control=ctrl0) # same model, clearly unidentifiable
## "TODO" :  summary(r1)  should ideally warn the user
stopifnot(all.equal(fixef(r1), fixef(r1b), tolerance= 1e-15),
          all.equal(ranef(r1), ranef(r1b), tolerance= 1e-15, check.attributes=FALSE))

## Use a more sensible model:
r2.0 <- lmer(y ~ 0+lagoon + (1|habitat:lagoon), data = dat) # ok
r2   <- lmer(y ~ 0+lagoon + (1|habitat), data = dat) # ok, and more clear
stopifnot(all.equal(fixef(r2), fixef(r2.0), tolerance= 1e-15),
          all.equal(ranef(r2), ranef(r2.0), tolerance= 1e-15, check.attributes=FALSE))
V2 <- vcov(r2)
assert.EQ.mat(V2, diag(x = 9.9833/3, nr = 4))
stopifnot(all.equal(unname(fixef(r2)) - (1:4)*100,
		    c(1.72, 0.28, 1.76, 0.8), tolerance = 1e-13))

## sparseX version should give same numbers:
## (only gives a warning now -- sparseX disregarded)
if(FALSE) { ## no longer
r2.  <- lmer(y ~ 0+lagoon + (1|habitat), data = dat,
             sparseX = TRUE)

## the summary() components we do want to compare 'dense X' vs 'sparse X':
nmsSumm <- c("methTitle", "devcomp", "logLik", "ngrps", "coefficients",
             "sigma", "REmat", "AICtab")
sr2  <- summary(r2)
sr2. <- summary(r2.)
sr2.$devcomp$dims['spFe'] <- 0L       # to allow for comparisons below
stopifnot(all.equal(sr2[nmsSumm], sr2.[nmsSumm], tolerance= 1e-14)
          , all.equal(ranef(r2), ranef(r2.), tolerance= 1e-14)
          , Matrix:::isDiagonal(vcov(r2.)) # ok
          , all.equal(Matrix::diag(vcov(r2.)), rep.int(V2[1,1], 4), tolerance= 1e-13)
#          , all(vcov(r2.)@factors$correlation == diag(4))  # not sure why this fails
          , TRUE)
r2.
}

### mcmcsamp() :
## From: Andrew Gelman <gelman@stat.columbia.edu>
## Date: Wed, 18 Jan 2006 22:00:53 -0500

if (FALSE) {  # mcmcsamp still needs work
    ## NB: Need to restore coda to the Suggests: field of DESCRIPTION
    ## file if this code block is reinstated.
    ## has.coda <- require(coda)
    ## if(!has.coda)
    ##     cat("'coda' package not available; some outputs will look suboptimal\n")

    ## Very simple example
    y <- 1:10
    group <- gl(2,5)
    (M1 <- lmer (y ~ 1 + (1 | group))) # works fine
    (r1 <- mcmcsamp (M1))              # dito
    r2 <- mcmcsamp (M1, saveb = TRUE)  # gave error in 0.99-* and 0.995-[12]
    (r10 <- mcmcsamp (M1, n = 10, saveb = TRUE))

    ## another one, still simple
    y <- (1:20)*pi
    x <- (1:20)^2
    group <- gl(2,10)
    M1 <- lmer (y ~ 1 | group)
    mcmcsamp (M1, n = 2, saveb=TRUE) # fine

    M2 <- lmer (y ~ 1 + x + (1 + x | group)) # false convergence
    ## should be identical (and is)
    M2 <- lmer (y ~ x + ( x | group))#  false convergence -> simulation doesn't work:
    if(FALSE) ## try(..) fails here (in R CMD check) [[why ??]]
        mcmcsamp (M2, saveb=TRUE)
    ## Error: inconsistent degrees of freedom and dimension ...

    ## mcmc for glmer:
    rG1k <- mcmcsamp(m1, n = 1000)
    summary(rG1k)
    rG2 <- mcmcsamp(m1, n = 3, verbose = TRUE)
}

## Spencer Graves' example (from a post to S-news, 2006-08-03) ----------------
## it should give an error, rather than silent non-sense:
tstDF <- data.frame(group = letters[1:5], y = 1:5)
assertError(## Now throws an error, as desired :
            lmer(y ~ 1 + (1|group), data = tstDF)
            )

showProc.time() #

## Wrong formula gave a seg.fault at times:
set.seed(2)# !
D <-  data.frame(y= rnorm(12,10), ff = gl(3,2,12),
                 x1=round(rnorm(12,3),1), x2=round(rnorm(12,7),1))
## NB: The first two are the same, having a length-3 R.E. with 3 x 3 vcov-matrix:
## --> do need CPU
## suppressWarnings() for warning about too-few random effects levels
tmpf <- function(form) lmer(form, data = D , control=lmerControl(check.conv.singular="ignore",
                                                                 check.nobs.vs.nRE="ignore",
                                                                 calc.derivs=FALSE))
m0 <- tmpf(y ~ (x1 + x2)|ff)
m1 <- tmpf(y ~ x1 + x2|ff)
m2 <- tmpf(y ~ x1 + (x2|ff))
m3 <- tmpf(y ~ (x2|ff) + x1)
suppressWarnings(stopifnot(all.equal(ranef(m0), ranef(m1), tolerance = 1e-5),
          all.equal(ranef(m2), ranef(m3), tolerance = 1e-5),
          inherits(tryCatch(lmer(y ~ x2|ff + x1, data = D), error = function(e)e),
                   "error")))

showProc.time() #

## Reordering of grouping factors should not change the internal structure
#Pm1  <- lmer1(strength ~ (1|batch) + (1|sample), Pastes, doFit = FALSE)
#Pm2  <- lmer1(strength ~ (1|sample) + (1|batch), Pastes, doFit = FALSE)
#P2.1 <- lmer (strength ~ (1|batch) + (1|sample), Pastes, devFunOnly = TRUE)
#P2.2 <- lmer (strength ~ (1|sample) + (1|batch), Pastes, devFunOnly = TRUE)

## The environments of Pm1 and Pm2 should be identical except for
## "call" and "frame":
#stopifnot(## all.EQ(env(Pm1), env(Pm2)),
#	  all.EQ(S4_2list(P2.1),
#		 S4_2list(P2.2)))


## example from Kevin Thorpe: synthesized equivalent
## http://thread.gmane.org/gmane.comp.lang.r.lme4.devel/9835

## NA issue: simpler example
d <- data.frame(y=1:60,f=factor(rep(1:6,each=10)))
d$y[2] <- NA
d$f[3:4] <- NA
lmer(y~(1|f),data=d)
glmer(y~(1|f),data=d,family=poisson)

## we originally thought that these examples should be
## estimating non-zero variances, but they shouldn't ...
## number of levels with each level of replication
levs <- c(800,300,150,100,50,50,50,20,20,5,2,2,2,2)
n <- seq_along(levs)
flevels <- seq(sum(levs))
set.seed(101)
fakedat <- data.frame(DA = factor(rep(flevels,rep(n,levs))),
                      zbmi=rnorm(sum(n*levs)))
## add NA values
fakedat[sample(nrow(fakedat),100),"zbmi"] <- NA
fakedat[sample(nrow(fakedat),100),"DA"] <- NA

m5 <- lmer(zbmi ~ (1|DA) , data = fakedat,
	   control=lmerControl(check.nobs.vs.rankZ="ignore"))
m6 <- update(m5, data=na.omit(fakedat))
stopifnot(VarCorr(m5)[["DA"]] == 0,
	  VarCorr(m6)[["DA"]] == 0)

showProc.time()

if (.Platform$OS.type != "windows") {
    library(lme4)
    fm1 <- lmer(Reaction ~ Days + (Days|Subject), sleepstudy)
    stopifnot(isREML(fm1),
              isLMM(fm1),
              !isGLMM(fm1),
              !isNLMM(fm1))

    fm1ML <- refitML(fm1)
    stopifnot(!isREML(fm1ML),
              isLMM(fm1ML),
              !isGLMM(fm1ML),
              !isNLMM(fm1ML))

    gm1 <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
                 data = cbpp, family = binomial)
    stopifnot(!isREML(gm1),
              !isLMM(gm1),
              isGLMM(gm1),
              !isNLMM(gm1))

    nm1 <- nlmer(circumference ~ SSlogis(age, Asym, xmid, scal) ~ Asym|Tree,
                 Orange, start = c(Asym = 200, xmid = 725, scal = 350))
    stopifnot(!isREML(nm1),
              !isLMM(nm1),
              !isGLMM(nm1),
              isNLMM(nm1))


} ## skip on windows (for speed)

library("lme4")
L <- load(system.file("testdata","crabs_randdata2.Rda",package="lme4"))
## randdata0: simulated data, in form suitable for plotting
## randdata: simulated data, in form suitable for analysis

## fr  ## alive/dead formula
## fr2 ## proportion alive formula (use with weights=initial.snail.density)

## FIXME: there are still bigger differences than I'd like between the approaches
## (mostly in the random-effects correlation).  It's not clear who's right;
## lme4 thinks its parameters are better, but ??  Could be explored further.

if (FALSE) {
## library(ggplot2)  ## commented to avoid triggering Suggests: requirement
library(grid)
zmargin <- theme(panel.margin=unit(0,"lines"))
theme_set(theme_bw())
g1 <- ggplot(randdata0,aes(x=snail.size,y=surv,colour=snail.size,fill=snail.size))+
    geom_hline(yintercept=1,colour="black")+
    stat_sum(aes(size=factor(..n..)),alpha=0.6)+
    facet_grid(.~ttt)+zmargin+
    geom_boxplot(fill=NA,outlier.colour=NULL,outlier.shape=3)+  ## set outliers to same colour as points
    ## (hard to see which are outliers, but it doesn't really matter in this case)
    scale_size_discrete("# obs",range=c(2,5))
}

if (.Platform$OS.type != "windows") {
t1 <- system.time(glmer1 <- glmer(fr2,weights=initial.snail.density,
                                  family ="binomial", data=randdata))
t1B <- system.time(glmer1B <- glmer(fr,family ="binomial", data=randdata))

res1 <- c(fixef(glmer1),c(VarCorr(glmer1)$plot))
res1B <- c(fixef(glmer1B),c(VarCorr(glmer1B)$plot))
p1 <- unlist(getME(glmer1,c("theta","beta")))
stopifnot(all.equal(res1,res1B))

dfun <- update(glmer1,devFunOnly=TRUE)
stopifnot(all.equal(dfun(p1),c(-2*logLik(glmer1))))
##
## library(lme4.0)  ## version 0.999999.2 results
## t1_lme4.0 <- system.time(glmer1X <-
##                           glmer(fr2,weights=initial.snail.density,
##                                 family ="binomial", data=randdata))
## dput(c(fixef(glmer1X),c(VarCorr(glmer1X)$plot)))
## p1X <- c(getME(glmer1X,"theta"),getME(glmer1X,"beta"))
p1X <- c(0.681301656652347, -1.14775239687404, 0.436143018123226, 
2.77730476938968, 0.609023583738824, -1.60055813739844, 2.0324468778545, 
0.624173873057839, -1.7908793509579, -2.44540201631615, -1.42365990002708, 
-2.26780929006268, 0.700928084600075, -1.26220238391029, 0.369024582097804, 
3.44325347343035, 2.26400391093108)
stopifnot(all.equal(unname(p1),p1X,tolerance=0.03))
dfun(p1X)
dfun(p1)
## ~ 1.8 seconds elapsed time
lme4.0_res <- 
    structure(c(2.77730476938968, 0.609023583738824, -1.60055813739844, 
                2.0324468778545, 0.624173873057839, -1.7908793509579, -2.44540201631615, 
                -1.42365990002708, -2.26780929006268, 0.700928084600075, -1.26220238391029, 
                0.369024582097804, 3.44325347343035, 2.26400391093108, 0.464171947357232, 
                -0.532754465140956, -0.532754465140956, 0.801690946568518),
          .Names = c("(Intercept)", 
          "crab.speciesS", "crab.speciesW", "crab.sizeS", "crab.sizeM", 
          "snail.sizeS", "crab.speciesS:crab.sizeS", "crab.speciesS:crab.sizeM", 
          "crab.speciesS:snail.sizeS", "crab.speciesW:snail.sizeS",
          "crab.sizeS:snail.sizeS", "crab.sizeM:snail.sizeS",
          "crab.speciesS:crab.sizeS:snail.sizeS", 
          "crab.speciesS:crab.sizeM:snail.sizeS", "", "", "", ""))

stopifnot(all.equal(res1,lme4.0_res,tolerance=0.015))


## library("glmmADMB")
## prop/weights formulation: ~ 7 seconds
## t1_glmmadmb <- system.time(glmer1B <- glmmadmb(fr,family ="binomial",
##                                               corStruct="full",data=randdata))
## dput(c(fixef(glmer1B),c(VarCorr(glmer1B)$plot)))
glmmADMB_res <- structure(c(2.7773101267224, 0.609026276823218, -1.60055704634712, 
2.03244174458562, 0.624171008585953, -1.79088398816641, -2.44540300134182, 
-1.42366043619683, -2.26780858382505, 0.700927141726545, -1.26219964572264, 
0.369029052442189, 3.44326297908383, 2.26403738918967, 0.46417, 
-0.53253, -0.53253, 0.80169), .Names = c("(Intercept)", "crab.speciesS", 
"crab.speciesW", "crab.sizeS", "crab.sizeM", "snail.sizeS", "crab.speciesS:crab.sizeS", 
"crab.speciesS:crab.sizeM", "crab.speciesS:snail.sizeS", "crab.speciesW:snail.sizeS", 
"crab.sizeS:snail.sizeS", "crab.sizeM:snail.sizeS", "crab.speciesS:crab.sizeS:snail.sizeS", 
"crab.speciesS:crab.sizeM:snail.sizeS", "", "", "", ""))

stopifnot(all.equal(res1B,glmmADMB_res,tolerance=0.015))
} ## skip on windows (for speed)

## test for models containing data-defined bases

## ?makepredictcall
## ?model.frame
## ????

data(sleepstudy,package="lme4")
library(splines)

## lm0 <- lm(Reaction~ns(Days,2),sleepstudy)
## attr(terms(lm0),"predvars")
## library(nlme)
## lme1 <- lme(Reaction~ns(Days,2),random=~1|Subject,sleepstudy)
## attr(terms(lme1),"predvars")  ## no!
## attr(lme1$terms,"predvars")   ## yes
## detach("package:nlme")

library(lme4)
fm1 <- lmer(Reaction ~ ns(Days,2) + (1|Subject), sleepstudy)
fm2 <- lmer(Reaction ~ poly(Days,2) + (1|Subject), sleepstudy)
fm3 <- lmer(Reaction ~ poly(Days,2,raw=TRUE) + (1|Subject), sleepstudy)

newdat0 <- data.frame(Days = unique(sleepstudy$Days))
newdat  <- data.frame(Days = 5:12)
tmpf <- function(fit) {
    with(sleepstudy, {
        plot  (Reaction~Days, xlim=c(0,12))
        points(Days, predict(fit), col=2)
    })
    lines(newdat0$ Days, predict(fit,re.form=NA,newdata=newdat0), col=4)
    lines(newdat $ Days, predict(fit,re.form=NA,newdata=newdat ), col=5)
}

stopifnot(all.equal(predict(fm2,newdat,re.form=NA),
                    predict(fm3,newdat,re.form=NA)))

## pictures
tmpf(fm1)
tmpf(fm2)
tmpf(fm3)

## test for GLMMs
set.seed(101)
d <- data.frame(y=rbinom(10,size=1,prob=0.5),
                x=1:10,
                f=factor(rep(1:5,each=2)))
gm1 <- glmer(y ~ poly(x,2) + (1|f), d, family=binomial)
gm2 <- glmer(y ~ poly(x,2,raw=TRUE) + (1|f), d, family=binomial)

newdat <- data.frame(x=c(1,4,6))
stopifnot(all.equal(predict(gm1,newdat,re.form=NA),
                    predict(gm2,newdat,re.form=NA),tolerance=3e-6))

meth.tab.0 <- cbind(optimizer=
                      rep(c("bobyqa",
                            "Nelder_Mead",
                            "nlminbwrap",
                            "nmkbw",
                            "optimx",
                            "nloptwrap" ),
                          c(rep(1,5),2)),
                    method= c(rep("",4), "L-BFGS-B",
                            "NLOPT_LN_NELDERMEAD", "NLOPT_LN_BOBYQA"))

## ugh: hardcoded list (incomplete?) of allowable *control* options by optimizer
## could make more of an effort to match max-iterations/evals,
## (x|f)*(abs|rel) tolerance, ...
opt.ctrls <- list(bobyqa=c("npt","rhobeg","rhoend","iprint","maxfun"),
                  Nelder_Mead=c("iprint","maxfun","FtolAbs",
                                "FtolRel","XtolRel","MinfMax",
                                "xst","xt","verbose","warnOnly"),
                  nlminbwrap=c("eval.max","iter.max","trace","abs.tol",
                               "rel.tol","x.tol","xf.tol","step.min",
                               "step.max","sing.tol","scale.init",
                               "diff.g"),
                  optimx=c("trace","fnscale","parscale","ndeps",
                          "maxit","abstol","reltol","method"),
                  nloptwrap=c("minf_max","ftol_rel","ftol_abs",
                              "xtol_rel", "xtol_abs", "maxeval", "maxtime",
                              "algorithm"),
                  nmkbw=c("tol","maxfeval","regsimp","maximize",
                          "restarts.max","trace","maxfun"))

## name of 'max function evaluations' for each optimizer
maxfun_arg <- c(bobyqa = "maxfun",
                Nelder_Mead = "maxfun",
                nlminbwrap = "eval.max",
                optimx = "maxit",
                nloptwrap = "maxeval",
                nmkbw = "maxfeval")
                           
                
nmkbw <- function(fn, par, lower, upper, control) {
    if (length(par)==1) {
        res <- optim(fn=fn,par=par,lower=lower,upper=100*par,
                     method="Brent")
    } else {
        res <- dfoptim::nmkb(fn=fn,par=par,
                             lower=lower,upper=upper,control=control)
    }
    res$fval <- res$value
    res
}


##' Attempt to re-fit a [g]lmer model with a range of optimizers.
##' The default is to use all known optimizers for R that satisfy the
##' requirements (do not require explicit gradients, allow
##' box constraints), in three categories; (i) built-in
##' (minqa::bobyqa, lme4::Nelder_Mead, nlminbwrap), (ii) wrapped via optimx
##' (most of optimx's optimizers that allow box constraints require
##' an explicit gradient function to be specified; the two provided
##' here are really base R functions that can be accessed via optimx,
##' (iii) wrapped via nloptr; (iv)
##'
##' @param object a fitted model
##' @param meth.tab a matrix (or data.frame) with columns
##' - method  the name of a specific optimization method to pass to the optimizer
##'           (leave blank for built-in optimizers)
##' - optimizer  the \code{optimizer} function to use
##' @param verbose print progress messages?
##' @param catch.err catch errors?
##' @param start_from_mle (logical) initialize the refitting process with starting values from the fitted model?
##' @return a list of fitted \code{merMod} objects
##' @seealso slice, slice2D in the bbmle package
##' @examples
##' library(lme4)
##' gm1 <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
##'                  data = cbpp, family = binomial)
##' gm_all <- allFit(gm1, parallel=TRUE)
##' ss <- summary(gm_all)
##' ss$fixef               ## extract fixed effects
##' ss$llik                ## log-likelihoods
##' ss$sdcor               ## SDs and correlations
##' ss$theta               ## Cholesky factors
##' ss$which.OK            ## which fits worked

allFit <- function(object, meth.tab = NULL,
                   data=NULL,
                   verbose=TRUE,
                   show.meth.tab = FALSE,
                   maxfun = 1e5,
                   parallel = c("no", "multicore", "snow"),
                   ncpus = getOption("allFit.ncpus", 1L),
                   cl = NULL,
                   catch.errs = TRUE,
                   start_from_mle = TRUE) {

    if (is.null(meth.tab)) {
        meth.tab <- meth.tab.0
    }
    optvec <- meth.tab.0[,"optimizer"]
    if (!requireNamespace("dfoptim", quietly = TRUE)) {
        optvec <- setdiff(optvec, "nmkbw")
        meth.tab <- meth.tab[meth.tab[,"optimizer"] %in% optvec, ]
    }
    if (!requireNamespace("optimx", quietly = TRUE)) {
        optvec <- setdiff(optvec, "optimx")
        meth.tab <- meth.tab[meth.tab[,"optimizer"] %in% optvec,]
    }
    if (show.meth.tab) {
        return(meth.tab)
    }
    stopifnot(length(dm <- dim(meth.tab)) == 2, dm[1] >= 1, dm[2] >= 2,
	      is.character(optimizer <- meth.tab[,"optimizer"]),
	      is.character(method    <- meth.tab[,"method"]))

    parallel <- match.arg(parallel)

    # (parallel, ncpus, cl) -> (parallel)
    eval(initialize.parallel)

    fit.names <- gsub("\\.$", "", paste(optimizer, method, sep="."))
    ffun <- local({
        ## required local vars
        object
        verbose
        fit.names
        optimizer
        method
        maxfun
        function(..i) {
            if (verbose) cat(fit.names[..i],": ")
            ctrl <- getCall(object)$control
            ## NB:  'ctrl' must become a correct *argument* list for g?lmerControl()
            if(is.null(ctrl)) {
                ctrl <- list(optimizer=optimizer[..i])
            } else {
                if(is.call(ctrl)) # typically true
                    ctrl <- lapply(as.list(ctrl)[-1], eval)
                ctrl$optimizer <- optimizer[..i]
            }
            ## add method/algorithm to optCtrl as necessary
            mkOptCtrl <- function(...) {
                x <- list(...)
                cc <- ctrl$optCtrl
                if (is.null(cc)) cc <- list()
                cc[[maxfun_arg[[optimizer[[..i]]]]]] <- maxfun
                for (n in names(x)) { ## replace existing values
                    cc[[n]] <- x[[n]]
                }
                cc
            }
            sanitize <- function(x,okvals) {
                if (is.null(x)) return(NULL)
                if (is.null(okvals)) return(x)
                x <- x[intersect(names(x),okvals)]
                ## ?? having names(control) be character(0)
                ##  screws up nmkbw ... ??
                if (length(names(x))==0) names(x) <- NULL
                x
            }
            ctrl$optCtrl <- switch(optimizer[..i],
                                   optimx    = mkOptCtrl(method   = method[..i]),
                                   nloptwrap = mkOptCtrl(algorithm= method[..i]),
                                   mkOptCtrl())
                                       
            ctrl$optCtrl <- sanitize(ctrl$optCtrl,
                                     opt.ctrls[[optimizer[..i]]])
            ctrl <- do.call(if(isGLMM(object)) glmerControl else lmerControl, ctrl)
            ## need to stick ctrl in formula env so it can be found ...
            form <- formula(object)
            env <- environment(form)
            tmp_env <- new.env(parent = env)
            # temporarily changing the environment
            environment(form) <- tmp_env
            assign("ctrl", ctrl, envir = tmp_env, inherits = FALSE)
            # Using the MLE as a starting point
            if (start_from_mle) {
              if (isGLMM(object)) {
                pars <- getME(object, c("theta", "fixef"))
              } else {
                pars <- getME(object, "theta")
                if(isNLMM(object)){
                  warning("results are not guaranteed when using nlmer")
                }
              }
              assign("pars", pars, envir = tmp_env, inherits = FALSE)
            }
            
            fit_call <- if (start_from_mle) {
              quote(update(object, start = pars, control = ctrl))
            } else {
              quote(update(object, control = ctrl))
            }
            
            tt <- system.time(
              rr <- if (catch.errs) {
                      tryCatch(eval(fit_call), error = function(e) e)
                    } else {
                      eval(fit_call)
                    }
            )

            attr(rr, "optCtrl") <- ctrl$optCtrl # contains crucial info here
            attr(rr, "time") <- tt  # store timing info
            
            ## restore original values to environment of the object
            on.exit({ environment(form) <- env }, add = TRUE)
            
            if (verbose) {
              if (inherits(rr,"error")) {
                cat("[failed]\n")
              } else {
                cat("[OK]\n")
              }
            }
            return(rr)
        }
    })

    seq_fit <- seq_along(fit.names)
    res <- if (parallel == "multicore") {
         parallel::mclapply(seq_fit, ffun, mc.cores = ncpus)
    } else if(parallel == "snow") {
        if (is.null(cl)) {
            cl <- parallel::makeCluster(ncpus)
            on.exit(parallel::stopCluster(cl))
            ## consider exporting data/package ?
            ## parallel::clusterEvalQ(cl,library("lme4"))
            ## try to get data and export it?
            ## parallel::clusterExport(cl,??)
        }
        parallel::parLapply(cl, seq_fit, ffun)
    } else
        lapply(seq_fit, ffun)

    names(res) <- fit.names
    structure(res, class = "allFit", fit = object, sessionInfo =  sessionInfo(),
              data = data # is dropped if NULL
              )
}

print.allFit <- function(x, width=80, ...) {
    cat("original model:\n")
    f <- attr(x,"fit")
    ss <- function(x) {
        if (nchar(x)>width) {
            strwrap(paste0(substr(x,1,width-3),"..."))
        } else x
    }
    ff <- ss(deparse1(formula(f)))
    cat(ff,"\n")
    cat("data: ",deparse(getCall(f)$data),"\n")
    cat("optimizers (",length(x),"): ",
        ss(paste(names(x),collapse=", ")),"\n",
        sep="")
    which.bad <- vapply(x,FUN=is,"error",FUN.VALUE=logical(1))
    if ((nbad <- sum(which.bad))>0) {
        cat(nbad,"optimizer(s) failed\n")
    }
    cat("differences in negative log-likelihoods:\n")
    nllvec <- -vapply(x[!which.bad],logLik,numeric(1))
    cat("max=",signif(max(nllvec-min(nllvec)),3),
        "; std dev=",signif(sd(nllvec),3), "\n")
    ## FIXME: print magnitudes of parameter diffs
    ## cat("differences in parameters:\n")
    ## ss <- summary(x)
    ## allpars <- cbind(ss$fixef, ss$sdcor)
    ## par_max <-
    invisible(x)
}

summary.allFit <- function(object, ...) {
    afun <- function(x, FUN, ...) {
        f1 <- FUN(x[[1]], ...)
        nm <- names(f1)
        n <- length(f1)
        res <- vapply(x, FUN, numeric(n), ...)
        if (!is.null(dim(res))) {
            res <- t(res)
        } else {
            res <- as.matrix(res)
            colnames(res) <- nm
        }
        res
    }
    which.OK <- !vapply(object, is, "error", FUN.VALUE=logical(1))
    objOK <- object[which.OK]
    msgs <- lapply(objOK, function(x) x@optinfo$conv$lme4$messages)
    fixef <- afun(objOK, fixef)
    llik <- vapply(objOK, logLik, numeric(1))
    times <- afun(objOK, attr, "time")
    feval <- vapply(objOK, function(x) x@optinfo$feval, numeric(1))
    vfun <- function(x) as.data.frame(VarCorr(x))[["sdcor"]]
    sdcor <- afun(objOK, vfun)
    theta <- afun(objOK, getME, name="theta")
    vcnms <- getVCNames(objOK[[1]])
    cnm <- unlist(rbind(vcnms$vcomp, vcnms$ccomp), FALSE, FALSE)
    if (sigma(objOK[[1]])!=1) cnm <- c(cnm,"sigma")
    colnames(sdcor) <- unname(cnm)
    sdcor <- as.data.frame(sdcor)
    res <- namedList(which.OK, msgs, fixef, llik, sdcor, theta, times, feval)
    class(res) <- "summary.allFit"
    res
}

## should add a print method for summary:
##  * fixed effects, random effects: summary of differences?

## not yet ...
plot.allFit <- function(x, abbr=16, ...) {
    values <- opt <- NULL ## R code check/non-standard evaluation
     if (! (requireNamespace("ggplot2"))) {
         stop("ggplot2 package must be installed to plot allFit objects")
     }
     aes <- NULL ## code check
     ss <- summary(x)
     ff <- stack(as.data.frame(ss$fixef))
     ff$opt <- rep(rownames(ss$fixef),length.out=nrow(ff))
     if (!is.null(abbr)) ff$opt <- abbreviate(ff$opt, minlength=abbr)
     (ggplot2::ggplot(ff, aes(values, opt, colour=opt))
         + ggplot2::geom_point()
         + ggplot2::facet_wrap(~ind,scale="free")
         + ggplot2::theme(legend.position="none")
     )
}

testLevel <- if (nzchar(s <- Sys.getenv("LME4_TEST_LEVEL"))) as.numeric(s) else 1
if (testLevel>1) {

  L <- load(system.file("testdata", "lme-tst-fits.rda",
                        package="lme4", mustWork=TRUE))
  
  had_pars <- exists("pars", envir = globalenv(), inherits = FALSE)
  had_ctrl <- exists("ctrl", envir = globalenv(), inherits = FALSE)
  
  gm_all <- allFit(fit_cbpp_1, verbose=FALSE)
  gm_all_nostart <- allFit(fit_cbpp_1, verbose=FALSE, start_from_mle = FALSE)

  test_that("pars and ctrl did not leak into the environment", {
    expect_equal(exists("pars", envir = globalenv(), inherits = FALSE), 
                 had_pars)
    expect_equal(exists("ctrl", envir = globalenv(), inherits = FALSE), 
                 had_ctrl)
  })
  
  summary(gm_all)$times[,"elapsed"]
  summary(gm_all_nostart)$times[,"elapsed"]

  sapply(gm_all, function(x) x@optinfo$feval)
  sapply(gm_all_nostart, function(x) x@optinfo$feval)

  ## library(microbenchmark)
  ##  mb1 <- microbenchmark(
  ##   start = allFit(fit_cbpp_1, verbose=FALSE),
  ##   nostart = allFit(fit_cbpp_1, verbose=FALSE, start_from_mle = FALSE)
  ##  )

    test_that("allFit print/summary is fine", {
        expect_is(gm_all, "allFit")
        expect_is(summary(gm_all), "summary.allFit")
    })

    test_that("nloptwrap switches optimizer correctly", {
        expect_equal(attr(gm_all[["nloptwrap.NLOPT_LN_BOBYQA"]],"optCtrl"),
                     list(maxeval = 1e5, algorithm = "NLOPT_LN_BOBYQA"))
        expect_equal(attr(gm_all[["nloptwrap.NLOPT_LN_NELDERMEAD"]],"optCtrl"),
                     list(maxeval = 1e5, algorithm = "NLOPT_LN_NELDERMEAD"))

    })

    test_that("lmerControl() arg works too", {
        fm0 <- lmer(Reaction ~ Days + (Days | Subject), sleepstudy)
        fm  <- update(fm0,
                      control = lmerControl(optCtrl = list(xtol_rel = 1e-8,
                                                           ftol_rel = 1e-8),
                                            calc.derivs=FALSE))
        afm0 <- allFit(fm0,verbose=FALSE)
        afm  <- allFit(fm,verbose=FALSE) # used to fail
        drop_ <- function(x) {
            x[setdiff(names(x), c("times","feval"))]
        }
        ## should be approximately the same
        expect_equal(drop_(summary(afm0)),
                     drop_(summary(afm)), tolerance = 1e-2)
        ## should NOT be the same!
        expect_false(isTRUE(all.equal(drop_(summary(afm0)),
                                      drop_(summary(afm)), tolerance=1e-10)))

    })

    test_that("glmerControl() arg + optimizer", {
        ## GH #523?
        fit_cbpp_1u <- update(fit_cbpp_1,
                              control=glmerControl(optimizer="nloptwrap",
                                                   optCtrl=list(xtol_abs=1e-10, ftol_abs=1e-10)))
        af2 <- allFit(fit_cbpp_1u, verbose=FALSE)
        expect_equal(class(af2),"allFit")
    })

    test_that("control passed as a variable does not break allFit (GH #985)", {
        ## previously: getCall(object)$control was a bare symbol (not a
        ## call), so allFit() failed with
        ## "object of type 'symbol' is not subsettable"
        cc <- lmerControl(optimizer="bobyqa", optCtrl = list(maxfun = 1e5))
        fm <- lmer(Reaction ~ Days + (1|Subject), control = cc, data = sleepstudy)
        aa <- expect_no_error(allFit(fm, verbose=FALSE))
        expect_true(all(summary(aa)$which.OK))
    })

    test_that("allFit does not corrupt nmkbw's optCtrl (GH #985)", {
        ## previously: a stale 'maxfun' entry (valid for bobyqa but not
        ## for nmkbw) survived sanitize() alongside the correct
        ## 'maxfeval' entry, causing dfoptim::nmkb() to warn
        ## "number of items to replace is not a multiple of replacement length"
        gm <- glmer(round(Reaction) ~ Days + (1|Subject),
                   control = glmerControl(optimizer="bobyqa",
                                          optCtrl = list(maxfun = 1e5)),
                   data = sleepstudy, family = poisson)
        warnings_seen <- character(0)
        aa <- withCallingHandlers(
            allFit(gm, verbose=FALSE),
            warning = function(w) {
                warnings_seen <<- c(warnings_seen, conditionMessage(w))
                invokeRestart("muffleWarning")
            }
        )
        expect_true(all(summary(aa)$which.OK))
        expect_false(any(grepl("number of items to replace",
                               warnings_seen, fixed=TRUE)))
    })

    test_that("i in model call is OK", {
        ## GH #538
        ## ugh, testthat scoping is insane ...
        ## if d and i are
        ## assigned normally with <- outside expect_true(), test fails
        ## BUT global assignment of 'd' breaks downstream tests in
        ##  'data= argument and formula evaluation' (test-formulaEval.R)
        ## ddd breaks similar test in 'fitting lmer models' (test-lmer.R)
        ##  (where 'd' is supposed to be nonexistent)
        ## if we do global assignment with <<-
        ##   can't figure out how to remove d (or ddd) after it's created to leave
        ##   the environment clean ...
        ## tried to encapsulate all the necessary assignments
        ## within expect_true({ ... }) but that fails in other ways

        nr <- nrow(sleepstudy)
        ..dd <<- list(sleepstudy[1:nr,], sleepstudy[-(1:nr)])
        i <<- 1
        fm0 <- lmer(Reaction ~ Days + (1 | Subject), data=..dd[[i]])
        aa <- allFit(fm0, verbose=FALSE)
        expect_true(
            all(summary(aa)$which.OK)
        )
    })

    test_that("allFit/update scoping", {
        ## GH #601
        fit_func <- function(dataset) {
            gm1 <- glmer(
                cbind(incidence, size - incidence) ~ period + (1 | herd),
                data = dataset, family = binomial
            )
            gm1@call$data <- dataset
            allFit(gm1, catch.errs=FALSE)
        }

        cc <- capture.output(ff <- fit_func(cbpp))
        expect_true(all(summary(ff)$which.OK))
    })

    test_that("maxfun works", {
        gm_it10 <- suppressWarnings(allFit(fit_cbpp_1, verbose=FALSE, maxfun = 10))
        v <- vapply(gm_it10, function(x) as.integer(x@optinfo$feval), FUN.VALUE=1L)
        ## function values are sometimes off a bit (due to initialization or Hessian calculation)
        ##  but close enough ...
        expect_true(all(is.na(v) | v < 12))
    })

    test_that("parallel = 'future' works", {
        if (require("future.apply", quietly = TRUE)) {
            plan(sequential)
            gm_all_future_seq <- allFit(fit_cbpp_1, verbose = FALSE, parallel = "future")
            expect_is(gm_all_future_seq, "allFit")
            expect_equal(names(gm_all_future_seq), names(gm_all))
            plan(multisession)
            gm_all_future_multi <- allFit(fit_cbpp_1, verbose = FALSE, parallel = "future")
            expect_is(gm_all_future_multi, "allFit")
            ## Comparing results is tricky due to timing/environment differences
            ## but the set of optimizers that worked should be the same
            expect_equal(names(gm_all_future_multi), names(gm_all))
            plan(sequential)
        }
    })

}  ## testLevel

##' This function computes the 1st and 2nd derivative (gradient and
##' Hessian) of a function
##' with vector-valued input and scalar output using a central finite
##' difference method.
##'
##' The function has to return a single scalar numerical value and take
##' as argument a numeric vector of the same length as \code{x}. The function also
##' has to be evaluable in a neighborhood around \code{x}, i.e. at
##' \code{x - delta} and at \code{x + delta} for all elements in \code{x}.
##'
##' @title Compute 1st and 2nd derivative
##' @param fun function for which to compute the gradient.
##' @param x numeric vector of values at which to compute the
##' gradient.
##' @param delta the amount to subtract and add to \code{x} when computing the
##' central difference.
##' @param fx optional value of \code{fun(x, ...)}.
##' @param ... additional arguments to \code{fun}.
##'
##' @keywords utilities
##' @export
##' @family derivatives
##'
##' @return a list with components
##' \item{gradient}{the first derivative vector}
##' \item{Hessian}{the second derivative matrix}
##' @author Rune Haubo Bojesen Christensen
##' 
deriv12 <- function(fun, x, delta=1e-4, fx=NULL,
                    lower=rep(NA_real_,length(x)), upper=rep(NA_real_,length(x)), ...) {
### Compute gradient and Hessian simultaneously (to save computing time)
    nx <- length(x)
    if(is.null(fx)) fx <- fun(x, ...)
    stopifnot(length(fx) == 1, nx >= 1)
    H <- array(NA_real_, dim=c(nx, nx))
    g <- numeric(nx)
    xadd <- x + delta
    hasUB <- !missing(upper) && any(active <- !is.na(upper))
    if(hasUB) { # only then may have non-NA
        if((hasUB <- any(active <- active & xadd > upper))) {
            udelta <- ifelse(active, upper-x, delta)
            xadd[active] <- upper[active]
        }
    }
    xsub <- x - delta
    hasLB <- !missing(lower) && any(active <- !is.na(lower))
    if(hasLB) { # only then may have non-NA
        if((hasLB <- any(active <- active & xsub < lower))) {
            ldelta <- ifelse(active, x-lower, delta)
            xsub[active] <- lower[active]
        }
    }
    ## substitute elements of 'mod' vectors into position(s) 'pos'
    ## in base
    spos <- function(base,mod,pos) {
        if (is.list(mod)) {
            for (i in seq_along(mod)) {
                base <- spos(base,mod[[i]],pos[i])
            }
            base
        } else {
            base[pos] <- mod[pos]
            base
        }
    }
    for(j in 1:nx) {
        ## Diagonal elements:
        fadd <- fun(spos(x,xadd,j), ...)
        fsub <- fun(spos(x,xsub,j), ...)
        udj <- if(hasUB) udelta[j] else delta
        ldj <- if(hasLB) ldelta[j] else delta
        H[j, j] <- fadd/udj^2 - 2 * fx/(udj*ldj) + fsub/ldj^2
        g[j] <- (fadd - fsub) / (udj+ldj) #  "total delta" (udelta+ldelta)[j]
        ## Off diagonal elements:
        for(i in seq_len(j - 1L)) { #  1 <= i < j
            ## Compute upper triangular elements (and mirror to lower tri.):
            ud.i <- if(hasUB) udelta[i] else delta
            ld.i <- if(hasLB) ldelta[i] else delta
            xaa <- spos(x,list(xadd,xadd),c(i,j))
            xas <- spos(x,list(xadd,xsub),c(i,j))
            xsa <- spos(x,list(xsub,xadd),c(i,j))
            xss <- spos(x,list(xsub,xsub),c(i,j))
            H[i, j] <- H[j, i] <-
                fun(xaa, ...)/(ud.i+udj)^2 -
                    fun(xas, ...)/(ud.i+ldj)^2 -
                        fun(xsa, ...)/(ld.i+udj)^2 +
                            fun(xss, ...)/(ld.i+ldj)^2
        }
    }
    list(gradient = g, Hessian = H)
}

#### --> ../man/modular.Rd
####     ==================

### Small utilities to be used in lFormula() and glFormula()

doCheck <- function(x) {
    is.character(x) && !any(x == "ignore")
}

RHSForm <- function(form,as.form=FALSE) {
    rhsf <- form[[length(form)]]
    if (as.form) reformulate(deparse(rhsf)) else rhsf
}

`RHSForm<-` <- function(formula,value) {
    formula[[length(formula)]] <- value
    formula
}

##' Original formula, minus response ( = '~ <RHS>') :
noLHSform <- function(formula) {
    if (length(formula)==2) formula else formula[-2]
}

##' @param cstr name of control being set
##' @param val value of control being set
checkCtrlLevels <- function(cstr, val, smallOK=FALSE) {
    bvals <- c("message","warning","stop","ignore")
    if (smallOK) bvals <- outer(bvals, c("","Small"), paste0)
    if (!is.null(val) && !val %in% bvals)
        stop("invalid control level ",sQuote(val)," in ",cstr,": valid options are {",
             paste(sapply(bvals,sQuote),collapse=","),"}")
    invisible(NULL)
}

## general identifiability checker, used both in checkZdim and checkZrank
wmsg <- function(n, cmp.val, allow.n, msg1="", msg2="", msg3="") {
    if (allow.n) { ## allow  n == cmp.val
        unident <- n < cmp.val
        cmp <- "<"
        rstr <- ""
    } else {
        unident <- n <= cmp.val
        cmp <- "<="
        rstr <- " and the residual variance (or scale parameter)"
    }
    ## %s without spaces intentional (don't want an extra space if the
    ## message component is empty)
    wstr <- sprintf("%s (=%d) %s %s (=%d)%s; the random-effects parameters%s are probably unidentifiable",
                    msg1, n, cmp,msg2, cmp.val,msg3,                    rstr)
    list(unident=unident, wstr=wstr)
}

##' For each r.e. term, test if Z has more columns than rows to detect
##' unidentifiability:
##' @title
##' @param Ztlist list of Zt matrices - one for each r.e. term
##' @param n no. observations
##' @param ctrl
##' @param allow.n allow as many random-effects as there are observations
##' for each term?
##' @return possibly empty character string with warning messages
checkZdims <- function(Ztlist, n, ctrl, allow.n=FALSE) {
    stopifnot(is.list(Ztlist), is.numeric(n))
    cstr <- "check.nobs.vs.nRE"
    checkCtrlLevels(cstr, cc <- ctrl[[cstr]])
    term.names <- names(Ztlist)
    rows <- vapply(Ztlist, nrow, 1L)
    cols <- vapply(Ztlist, ncol, 1L)
    stopifnot(all(cols == n))
    if (doCheck(cc)) {
        unique(unlist(lapply(seq_along(Ztlist), function(i) {
            ww <- wmsg(cols[i], rows[i], allow.n, "number of observations",
                       "number of random effects",
                       sprintf(" for term (%s)", term.names[i]))
            if(ww$unident) {
                switch(cc,
                       "warning" = warning(ww$wstr, call.=FALSE),
                       "stop"    = stop   (ww$wstr, call.=FALSE),
                       stop(gettextf("unknown check level for '%s'", cstr), domain=NA))
                ww$wstr
            } else character()
        }))) ## -> possibly empty vector of error messages
    } else character()
}

##' @importFrom Matrix rankMatrix
checkZrank <- function(Zt, n, ctrl, nonSmall = 1e6, allow.n=FALSE)
{
    stopifnot(is.list(ctrl), is.numeric(n), is.numeric(nonSmall))
    cstr <- "check.nobs.vs.rankZ"
    if (doCheck(cc <- ctrl[[cstr]])) { ## not NULL or "ignore"
        checkCtrlLevels(cstr, cc, smallOK=TRUE)
        d <- dim(Zt)
        doTr <- d[1L] < d[2L] # Zt is "wide" => qr needs transpose(Zt)
        if(!(grepl("Small",cc) && prod(d) > nonSmall)) {
            rankZ <- rankMatrix(if(doTr) t(Zt) else Zt, method="qr")
            ww <- wmsg(n, rankZ, allow.n, "number of observations", "rank(Z)")
            if(is.na(rankZ)) {
                cc <- "stop"
                ww <-
                    list(unident = TRUE,
                         wstr = sub("^.*;",
                                    "rank(Z) is NA: invalid random effect factors?",
                                    ww$wstr))
            }
            if (ww$unident) {
                switch(cc,
                       "warningSmall" =, "warning" = warning(ww$wstr,call.=FALSE),
                       "stopSmall" =, "stop" = stop(ww$wstr,call.=FALSE),
                       stop(gettextf("unknown check level for '%s'", cstr),
                            domain=NA))
                ww$wstr
            } else character()
        } else character()
    } else character()
}

## check scale of non-dummy columns of X, both
## against each other and against 1 (implicit scale of theta parameters)?
## (shouldn't matter for lmer models?)
## TODO: check for badly centred models?
## TODO: check scale of Z columns?
## What should the rules be?  try to find problematic columns
##   and rescale them?  scale+center?  Just scale or scale+center
##   all numeric columns?
##
checkScaleX <- function(X,  kind="warning", tol=1e3) {
    ## cstr <- "check.scaleX"
    kinds <- eval(formals(lmerControl)[["check.scaleX"]])
    if (!kind %in% kinds) stop(gettextf("unknown check-scale option: %s",kind))
    if (is.null(kind) || kind == "ignore") return(X)
    ## else :
    cont.cols <- apply(X,2,function(z) !all(z %in% c(0,1)))
    col.sd <- apply(X[,cont.cols, drop=FALSE], 2L, sd)
    sdcomp <- outer(col.sd,col.sd,"/")
    logcomp <- abs(log(sdcomp[lower.tri(sdcomp)]))
    logsd <- abs(log(col.sd))
    if (any(c(logcomp,logsd) > log(tol))) {
        wmsg <- "Some predictor variables are on very different scales:"
        if (kind %in% c("warning","stop")) {
            msg2 <- "\nYou may also use (g)lmerControl(autoscale = TRUE) to improve numerical stability."
            wmsg <- paste(wmsg, "consider rescaling.", msg2)
            switch(kind,
                   "warning" = warning(wmsg, call.=FALSE),
                   "stop" = stop(wmsg, call.=FALSE))
        } else {
            wmsg <- paste(wmsg, "auto-rescaled (results NOT adjusted)")
            ## mimic scale() because we don't want to make a copy in
            ##  order to retrieve the center/scale
            X[,cont.cols] <- sweep(X[,cont.cols,drop=FALSE],2,col.sd,"/")
            attr(X,"scaled:scale") <- setNames(col.sd,colnames(X)[cont.cols])
            if (kind == "warn+rescale") warning(wmsg, call.=FALSE)
        }
    } else
        wmsg <- character()
    structure(X, msgScaleX = wmsg)
}


##' @title Check that grouping factors have at least 2 and </<= nobs(.) levels
##' @param flist mkReTrms(.)$flist
##' @param n
##' @param ctrl
##' @param allow.n
##' @return
checkNlevels <- function(flist, n, ctrl, allow.n=FALSE)
{
    stopifnot(is.list(ctrl), is.numeric(n))
    nlevelVec <- vapply(flist, function(x) nlevels(factor(x, exclude=NA)), 1)
    ## Part 1 ----------------
    cstr <- "check.nlev.gtr.1"
    checkCtrlLevels(cstr, cc <- ctrl[[cstr]])
    if (doCheck(cc) && any(nlevelVec < 2)) {
        wstr <- "grouping factors must have > 1 sampled level"
        switch(cc,
               "warning" = warning(wstr,call.=FALSE),
               "stop"    =    stop(wstr,call.=FALSE),
               ## FIXME: should never get here since we have checkCtrLevels test above?
               stop(gettextf("unknown check level for '%s'", cstr), domain=NA))
    } else wstr <- character()
    ## Part 2 ----------------
    cstr <- "check.nobs.vs.nlev"
    checkCtrlLevels(cstr, cc <- ctrl[[cstr]])
    if (doCheck(cc) && any(if(allow.n) nlevelVec > n else nlevelVec >= n)) {
        ## figure out which factors are the problem?
        w <- if (allow.n) which(nlevelVec>n) else which(nlevelVec>=n)
        bad_facs <- names(nlevelVec)[w]
        wst2 <- gettextf(
            "number of levels of each grouping factor must be %s number of observations",
            if(allow.n) "<=" else "<")
        wst2 <- paste0(wst2," (problems: ",paste(bad_facs,collapse=", "),")")
        switch(cc,
               "warning" = warning(wst2, call.=FALSE),
               "stop"    =    stop(wst2, call.=FALSE)
               ## shouldn't reach here
               )
    } else wst2 <- character()

    ## Part 3 ----------------
    cstr <- "check.nlev.gtreq.5"
    checkCtrlLevels(cstr, cc <- ctrl[[cstr]])
    if (doCheck(cc) && any(nlevelVec < 5)) {
        wst3 <- "grouping factors with < 5 sampled levels may give unreliable estimates"
        switch(cc,
               "warning" = warning(wst3, call.=FALSE),
               "stop"    = stop   (wst3, call.=FALSE),
               stop(gettextf("unknown check level for '%s'", cstr), domain=NA))
    } else wst3 <- character()
    ## return:
    c(wstr, wst2, wst3) ## possibly == character(0)
}

##' Coefficients (columns) are dropped from a design matrix to
##' ensure that it has full rank.
##'
##'  Redundant columns of the design matrix are identified with the
##'  LINPACK implementation of the \code{\link{qr}} decomposition and
##'  removed. The returned design matrix will have \code{qr(X)$rank}
##'  columns.
##'
##' (Note: I have lifted this function from the ordinal (soon Rufus)
##' package and modified it slightly./rhbc)
##'
##' @title Ensure full rank design matrix
##' @param X a design matrix, e.g., the result of
##' \code{\link{model.matrix}} possibly of less than full column rank,
##' i.e., with redundant parameters.
##' @param silent should a message not be issued if X is column rank
##' deficient?
##' @return The design matrix \code{X} without redundant columns.
##' @seealso \code{\link{qr}} and \code{\link{lm}}
##' @importFrom Matrix rankMatrix
##' @author Rune Haubo Bojesen Christensen (drop.coef()); Martin Maechler
chkRank.drop.cols <- function(X, kind, tol = 1e-7, method = "qr") {
    ## Test and match arguments:
    stopifnot(is.matrix(X)) # i.e., *not* sparse
    kinds <- eval(formals(lmerControl)[["check.rankX"]])
    if (!kind %in% kinds) stop(gettextf("undefined option for 'kind': %s", kind))
    ## c("message+drop.cols", "ignore",
    ##   "silent.drop.cols", "warn+drop.cols", "stop.deficient"),

    if(kind == "ignore") return(X)
    ## else :
    p <- ncol(X)
    if (kind == "stop.deficient") {
        if ((rX <- rankMatrix(X, tol=tol, method=method)) < p)
            stop(gettextf(sub("\n +", "\n",
                  "the fixed-effects model matrix is column rank deficient (rank(X) = %d < %d = p);
                   the fixed effects will be jointly unidentifiable"),
                          rX, p), call. = FALSE)
    } else {
        ## kind is one of "message+drop.cols", "silent.drop.cols", "warn+drop.cols"
        ## --> consider to drop extraneous columns: "drop.cols":

        ## Perform the qr-decomposition of X using LINPACK method,
        ## as we need the "good" pivots (and the same as lm()):
        ## this rankMatrix(X, method="qrLINPACK"):  FIXME?  rankMatrix(X, method= "qr.R")
        qr.X <- qr(X, tol = tol, LAPACK = FALSE)
        rnkX <- qr.X$rank
        if (rnkX == p)
            return(X) ## return X if X has full column rank
        ## else:

        ## message about no. dropped columns:
        msg <- sprintf(ngettext(p - rnkX,
                "fixed-effect model matrix is rank deficient so dropping %d column / coefficient",
                "fixed-effect model matrix is rank deficient so dropping %d columns / coefficients"),
                       p - rnkX)
        if (kind != "silent.drop.cols")
            (if(kind == "warn+drop.cols") warning else message)(msg, domain = NA)
        ## Save properties of X
        contr <- attr(X, "contrasts")
        asgn <- attr(X, "assign")

        ## Return the columns correponding to the first qr.x$rank pivot
        ## elements of X:
        keep <- qr.X$pivot[seq_len(rnkX)]
        dropped.names <- colnames(X[,-keep,drop=FALSE])
        X <- X[, keep, drop = FALSE]
        if (rankMatrix(X, tol=tol, method=method) < ncol(X))
            stop(gettextf("Dropping columns failed to produce full column rank design matrix"),
                 call. = FALSE)

        ## Re-assign relevant attributes:
        if(!is.null(contr)) attr(X, "contrasts") <- contr
        if(!is.null(asgn))  attr(X, "assign")    <- asgn[keep]
        attr(X, "msgRankdrop") <- msg
        attr(X, "col.dropped") <- setNames(qr.X$pivot[(rnkX+1L):p],
                                           dropped.names)
    }
    X
}

# check that response is not constant
checkResponse <- function(y, ctrl) {
  stopifnot(is.list(ctrl))
  cstr <- "check.response.not.const"
  checkCtrlLevels(cstr, cc <- ctrl[[cstr]])
  if (doCheck(cc) && length(unique(y)) < 2L) {
    wstr <- "Response is constant"
    switch(cc,
           "warning" = warning(wstr, call.=FALSE),
           "stop"   =    stop(wstr, call.=FALSE),
           stop(gettextf("unknown check level for '%s'", cstr), domain=NA))
  } else character()
}

##' Extract all warning msgs from a merMod object
.merMod.msgs <- function(x) {
    ## currently only those found with 'X' :
    aX <- attributes(x@pp$X)
    wmsgs <- grep("^msg", names(aX))
    if(any(has.msg <- nchar(Xwmsgs <- unlist(aX[wmsgs])) > 0))
        Xwmsgs[has.msg]
    else
        character()
}


## NA predict and restore rownames of original data if necessary
## napredictx <- function(x,...) {
##     res <- napredict(x)
## }

##' @rdname modular
##' @param control a list giving (for \code{[g]lFormula}) all options (see \code{\link{lmerControl}} for running the model;
##' (for \code{mkLmerDevfun,mkGlmerDevfun}) options for inner optimization step;
##' (for \code{optimizeLmer} and \code{optimize[Glmer}) control parameters for nonlinear optimizer (typically inherited from the \dots argument to \code{lmerControl})
##' @return \bold{lFormula, glFormula}: A list containing components,
##' \item{fr}{model frame}
##' \item{X}{fixed-effect design matrix}
##' \item{reTrms}{list containing information on random effects structure: result of \code{\link{mkReTrms}}}
##' \item{REML}{(lFormula only): logical flag: use restricted maximum likelihood? (Copy of argument.)}
##' @export
lFormula <- function(formula, data=NULL, REML = TRUE,
                     subset, weights, na.action, offset, contrasts = NULL,
                     control=lmerControl(), ...)
{
    control <- control$checkControl ## this is all we really need
    mf <- mc <- match.call()

    dontChk <- c("start", "verbose", "devFunOnly")
    dots <- list(...)
    do.call(checkArgs, c(list("lmer"), dots[!names(dots) %in% dontChk]))
    if (!is.null(dots[["family"]])) {
        ## lmer(...,family=...); warning issued within checkArgs
        mc[[1]] <- quote(lme4::glFormula)
        if (missing(control)) mc[["control"]] <- glmerControl()
        return(eval(mc, parent.frame()))
    }

    cstr <- "check.formula.LHS"
    checkCtrlLevels(cstr,control[[cstr]])
    denv <- checkFormulaData(formula, data,
                             checkLHS = control$check.formula.LHS == "stop")
    formula <- as.formula(formula, env=denv)
    if (getDoublevertDefault() == "split") {
      RHSForm(formula) <- reformulas::expandDoubleVerts(RHSForm(formula))
    }
    ## as.formula ONLY sets environment if not already explicitly set.
    ## ?? environment(formula) <- denv

    ## (DRY! copied from glFormula)
    m <- match(c("data", "subset", "weights", "na.action", "offset"),
               names(mf), 0L)
    mf <- mf[c(1L, m)]
    mf$drop.unused.levels <- TRUE
    mf[[1L]] <- quote(stats::model.frame)

    ## substitute  special(x | f)  with  (x | f)
    fr.form. <- noSpecials(formula, specials = lme4_specials, delete = FALSE)
    ## substitute  (x | f)  and  (x || f)  with  (x + f)
    fr.form <- sub_specials(fr.form., specials = c("|", "||"),
                            keep_args = c(2L, 2L))
    environment(fr.form.) <- environment(fr.form) <-
        environment(formula)
    ## model.frame.default looks for these objects in the environment
    ## of the *formula* (see 'extras', which is anything passed in '...'),
    ## so they have to be put there:
    for (i in c("weights", "offset")) {
        if (!eval(bquote(missing(x=.(i)))))
            assign(i,get(i,parent.frame()),environment(fr.form))
    }
    mf$formula <- fr.form
    fr <- eval(mf, parent.frame())
    if (nrow(fr) == 0L) stop("0 (non-NA) cases")
    ## convert character vectors to factor (defensive)
    fr <- factorize(fr.form, fr, char.only=TRUE)
    ## store full, original formula & offset
    attr(fr,"formula") <- formula
    attr(fr,"offset") <- mf$offset
    n <- nrow(fr)
    ## random effects and terms modules
    ## get list of calls whose first argument is a call to '|'
    ##                x | f  ->      us(x | f)
    ##     nonspecial(x | f) ->      us(x | f)
    ##        special(x | f) -> special(x | f)
    bb1 <- findbars_x(formula, specials = lme4_specials,
                      default.special = "us", target = "|",
                      expand_doublevert_method = getDoublevertDefault())
    bb0 <- lapply(bb1, `[[`, 2L)
    reTrms <- reformulas::mkReTrms(bb0, fr, calc.lambdat = FALSE)
    reTrms <- upReTrms(reTrms, bb1) # local calc.lambdat=TRUE step
    ## If there is a covariance structure; ignore the check nobs.vs.nRE
    if (.anyStructured(reTrms$reCovs))
        control$check.nobs.vs.nRE <- "ignore"
    wmsgNlev <- checkNlevels(reTrms$flist, n=n, control)
    wmsgZdims <- checkZdims(reTrms$Ztlist, n=n, control, allow.n=FALSE)
    if (anyNA(reTrms$Zt)) {
        stop("NA in Z (random-effects model matrix): ",
             "please use ",
             shQuote("na.action='na.omit'"),
             " or ",
             shQuote("na.action='na.exclude'"))
    }
    wmsgZrank <- checkZrank(reTrms$Zt, n=n, control, nonSmall = 1e6)

    ## fixed-effects model matrix X - remove random effect parts from formula:
    fixedform <- fr.form.
    RHSForm(fixedform) <- reformulas::nobars(RHSForm(fixedform))
    mf$formula <- fixedform
    ## re-evaluate model frame to extract predvars component
    fixedfr <- eval(mf, parent.frame())
    attr(attr(fr,"terms"), "predvars.fixed") <-
        attr(attr(fixedfr,"terms"), "predvars")
    ## so we don't have to fart around retrieving which vars we need
    ##  in model.frame(.,fixed.only=TRUE)
    attr(attr(fr,"terms"), "varnames.fixed") <- names(fixedfr)

    ## ran-effects model frame (for predvars)
    ## important to COPY formula (and its environment)?
    ranform <- fr.form.
    RHSForm(ranform) <- reformulas::subbars(
        RHSForm(noSpecials(lme4_reOnly(ranform), delete = FALSE, specials = lme4_specials))
    )
    mf$formula <- ranform
    ranfr <- eval(mf, parent.frame())
    attr(attr(fr,"terms"), "predvars.random") <-
        attr(terms(ranfr), "predvars")

    ## FIXME: shouldn't we have this already in the full-frame predvars?
    X <- model.matrix(fixedform, fr, contrasts)#, sparse = FALSE, row.names = FALSE) ## sparseX not yet
    
    ## Scaling (if autoscale is on...)
    if (!is.null(control$autoscale) && control$autoscale) {
      if("(Intercept)" %in% colnames(X)){
        X_scaled <- scale(X[, -1])
        X[,-1] <- X_scaled
      } else {
        X_scaled <- scale(X)
        X <- X_scaled
      }
      attr(X, "scaled:center") <- attr(X_scaled, "scaled:center")
      attr(X, "scaled:scale") <- attr(X_scaled, "scaled:scale")
    }
    
    ## backward compatibility (keep no longer than ~2015):
    if(is.null(rankX.chk <- control[["check.rankX"]]))
        rankX.chk <- eval(formals(lmerControl)[["check.rankX"]])[[1]]
    X <- chkRank.drop.cols(X, kind=rankX.chk, tol = 1e-7)
    if(is.null(scaleX.chk <- control[["check.scaleX"]]))
        scaleX.chk <- eval(formals(lmerControl)[["check.scaleX"]])[[1]]
    X <- checkScaleX(X, kind=scaleX.chk)

    list(fr = fr, X = X, reTrms = reTrms, REML = REML, formula = formula,
         wmsgs = c(Nlev = wmsgNlev, Zdims = wmsgZdims, Zrank = wmsgZrank))
}

## utility f'n for checking starting values
getStart <- function(start, rho, nAGQ) {
    ## default values
    par <- par0 <- (rho$mkPar %||% identity)(rho$pp$theta)
    fixef <- fixef0 <- rho$pp$delb
    if (is.null(start))
        NULL # do nothing
    else if (is.numeric(start))
        par <- start
    else if (is.list(start)) {
        if (length(start) > 0L && is.null(names(start)))
            stop(gettextf("'%s' does not have names",
                          "start"),
                 domain = NA)
        valid <- c(c("par", "theta"), if (nAGQ > 0L) c("fixef", "beta"))
        invalid <- setdiff(names(start), valid)
        if (length(invalid) > 0L)
            stop(gettextf("'%s' has invalid names %s",
                          "start", deparse(invalid)),
                 domain = NA)
        if (!all(vapply(start, is.numeric, FALSE)))
            stop(gettextf("'%s' has non-numeric components",
                          "start"),
                 domain = NA)
        npar <- is.null(start$par)
        ntheta <- is.null(start$theta)
        nfixef <- is.null(start$fixef)
        nbeta <- is.null(start$beta)
        if (!(npar && ntheta)) {
            if (!(npar || ntheta))
                message(gettextf("ignoring %s", "start$theta"),
                        domain = NA)
            par <- if (npar) start$theta else start$par
        }
        if (!(nfixef && nbeta)) {
            if (!(nfixef || nbeta))
                message(gettextf("ignoring %s", "start$beta"),
                        domain = NA)
            fixef <-
            if (nfixef)
                start$beta
            else if (length(start$fixef) > length(fixef) &&
                     !is.null(nms <- names(start$fixef)) &&
                     ncol(rho$pp$X) == length(fixef) && # ever FALSE?
                     !is.null(nms. <- colnames(rho$pp$X)) && # ever FALSE?
                     all(m <- match(nms., nms, 0L)))
                start$fixef[m]
            else start$fixef
        }
    }
    else stop(gettextf("'%s' is not NULL, a numeric vector, or a list",
                       "start"),
              domain = NA)
    if (length(par) != length(par0))
        stop(gettextf("starting value of '%s' has length not equal to %.0f",
                      "par", length(par0)),
             domain = NA)
    if (length(fixef) != length(fixef0))
        stop(gettextf("starting value of '%s' has length not equal to %.0f",
                      "fixef", length(fixef0)),
             domain = NA)
    if (nAGQ > 0L) c(par, fixef) else par
}

updateStart <- function(start, par) {
    if (is.null(start))
        start
    else if (is.numeric(start))
        par
    else if (is.list(start)) {
        if (!is.null(start[["par"]]))
            start[["par"]] <- par
        if (!is.null(start[["theta"]]))
            start[["theta"]] <- par
        start
    }
    else stop(gettextf("'%s' is not NULL, a numeric vector, or a list",
                       "start"),
              domain = NA)
}

##' @rdname modular
##' @param fr A model frame containing the variables needed to create an
##'   \code{\link{lmerResp}} or \code{\link{glmResp}} instance
##' @param X fixed-effects design matrix
##' @param reTrms information on random effects structure (see \code{\link{mkReTrms}})
##' @param REML (logical) fit restricted maximum likelihood model?
##' @param start starting values
##' @param verbose print output?
##' @return \bold{mkLmerDevfun, mkGlmerDevfun}: A function to calculate deviance
##' (or restricted deviance) as a function of the theta (random-effect) parameters
##' (for GlmerDevfun, of beta (fixed-effect) parameters as well).  These deviance
##' functions have an environment containing objects required for their evaluation.
##' CAUTION: The output object of \code{mk(Gl|L)merDevfun} is an \code{\link{environment}}
##' containing reference class objects (see \code{\link{ReferenceClasses}}, \code{\link{merPredD-class}},
##' \code{\link{lmResp-class}}), which behave in ways that may surprise many users. For example, if the
##' output of \code{mk(Gl|L)merDevfun} is naively copied, then modifications to the original will
##' also appear in the copy (and vice versa). To avoid this behavior one must make a deep copy
##' (see \code{\link{ReferenceClasses}} for details).
##' \cr
##' \cr
##' @export
mkLmerDevfun <- function(fr, X, reTrms, REML = TRUE, start = NULL,
                         verbose = 0, control = lmerControl(), ...)
{
    ## FIXME: make sure verbose gets handled properly
    #if (missing(fr)) {
    ## reconstitute frame
    #}
    ## pull necessary arguments for making the model frame out of ...
    p <- ncol(X) # maybe also do rank check on X here??
    rho <- new.env(parent=parent.env(environment()))
    rho$pp <- do.call(merPredD$new, c(reTrms[c("Zt","theta","Lambdat","Lind")],
                                      n=nrow(X), list(X=X)))
    REMLpass <- if(REML) p else 0L
    rho$resp <-
        if(missing(fr))
             mkRespMod(    REML = REMLpass, ...)
        else mkRespMod(fr, REML = REMLpass)
    ## FIXME / note: REML does double duty as rank of X and a flag for using
    ## REML maybe this should be mentioned in the help file for
    ## mkRespMod??  currently that help file says REML is logical.  a
    ## consequence of this double duty is that it is impossible to fit
    ## a model with no fixed effects using REML (MM: ==> FIXME)
    ## devfun <- mkdevfun(rho, 0L, verbose=verbose, control=control)

    ## prevent R CMD check false pos. warnings (in this function only):
    pp <- resp <- mkTheta <- NULL
    rho$lmer_Deviance <- lmer_Deviance
    rho$mkPar <- mkMkPar(reTrms$reCovs)
    rho$mkTheta <- mkMkTheta(reTrms$reCovs)
    devfun <- function(par)
        .Call(lmer_Deviance, pp$ptr(), resp$ptr(), mkTheta(as.double(par)))
    environment(devfun) <- rho

    # if all random effects are of the form 1|f and starting values not
    # otherwise provided (and response variable is present, i.e. not doing
    # a simulation) then compute starting values
    if (is.null(start) &&
        all(reTrms$cnms == "(Intercept)") &&
        length(reTrms$flist) == length(reTrms$lower) &&
        !is.null(y <- model.response(fr))) {
        v <- sapply(reTrms$flist, function(f) var(ave(y, f)))
        v.e <- var(y) - sum(v)
        if (!is.na(v.e) && v.e > 0) {
            v.rel <- v / v.e
            if (all(v.rel >= reTrms$lower^2)) rho$pp$setTheta(sqrt(v.rel))
        }
    }

    ## theta <- getStart(start, rho$pp)
    ## ^^^^^ unused / obfuscation? should the above be rho$pp$setTheta(.) ?
    ## MM: commenting it did not break any of our checks
    if (length(rho$resp$y) > 0)  ## only if non-trivial y
        devfun(rho$mkPar(rho$pp$theta)) # one evaluation to ensure all values are set
    rho$lower <- reTrms$lower # to be more consistent with mkGlmerDevfun
    rho$upper <- reTrms$upper %||% rep(Inf, length(reTrms$lower))
    devfun # this should pass the rho environment implicitly
}


##' @param devfun a deviance function, as generated by \code{\link{mkLmerDevfun}}
##' @return \bold{optimizeLmer}: Results of an optimization.
optimizeLmer <- function(devfun,
                         optimizer=    formals(lmerControl)$optimizer,
                         restart_edge= formals(lmerControl)$restart_edge,
                         boundary.tol = formals(lmerControl)$boundary.tol,
                         start = NULL,
                         verbose = 0L,
                         control = list(),
                         ...) {
    verbose <- as.integer(verbose)
    rho <- environment(devfun)
    start <- getStart(start, rho, 0L)
    lower <- rho$lower
    upper <- rho$upper %||% rep(Inf, length(rho$lower))
    opt <- optwrap(optimizer,
                   devfun,
                   start,
                   lower=lower,
                   upper=upper,
                   control=control,
                   adj=FALSE, verbose=verbose,
                   ...)


    if (restart_edge) {
        ## FIXME: should we be looking at rho$pp$theta or opt$par
        ##  at this point???  in koller example (for getData(13)) we have
        ##   rho$pp$theta=0, opt$par=0.08
        par0 <- rho$mkPar(rho$pp$theta)
        if (length(wl <- which(par0 == lower)) > 0L |
            length(wu <- which(par0 == upper)) > 0L) {
            ## *don't* use numDeriv -- cruder but fewer dependencies, no worries
            ##  about keeping to the interior of the allowed space

            ## <MJ>
            ## Removing the following line (hence *not* replacing 'par0'
            ## with a copy) breaks several tests in ../tests/boundary.R
            ## and ../tests/lmer-1.R.  OMG ...
            par0 <- par0 + 0
            ## [ same is seen in 'check.boundary' ]
            ## </MJ>

            d0 <- devfun(par0)
            btol <- 1e-5  ## FIXME: make user-settable?
            bgrad <- mapply(function(i, bval, btol) {
                                par <- par0
                                par[i] <- bval + btol
                                (devfun(par) - d0)/btol
                            },
                            i = c(wl, wu),
                            bval = c(lower[wl], upper[wu]),
                            btol = rep(c(btol, -btol), c(length(wl), length(wu))))
            ## what do I need to do to reset rho$pp$theta to original value???
            devfun(par0) ## reset rho$pp$theta after tests
            ## FIXME: allow user to specify ALWAYS restart if on boundary?
            if (any(is.na(bgrad))) {
                warning("some gradient components are NA near boundaries, skipping boundary check")
                return(opt)
            } else {
                if (any(bgrad < 0)) {
                    if (verbose) message("some theta parameters on the boundary, restarting")
                    opt <- optwrap(optimizer,
                                   devfun,
                                   opt$par,
                                   lower=lower,
                                   upper=upper,
                                   control=control,
                                   adj=FALSE, verbose=verbose,
                                   ...)
                }
            } ## bgrad not NA
        }
    } ## if restart.edge
    if (boundary.tol > 0)
        check.boundary(rho, opt, devfun, boundary.tol)
    else
        opt
}

## TODO: remove any arguments that aren't actually used by glFormula (same for lFormula)
## TODO(?): lFormula() and glFormula()  are very similar: merge or use common baseFun()
##' @rdname modular
##' @inheritParams glmer
##' @export
glFormula <- function(formula, data=NULL, family = gaussian,
                      subset, weights, na.action, offset,
                      contrasts = NULL, start, mustart, etastart,
                      control = glmerControl(), ...) {
    ## FIXME: does start= do anything? test & fix

    control <- control$checkControl ## this is all we really need
    mf <- mc <- match.call()
    ## extract family, call lmer for gaussian
    if (is.character(family))
        family <- get(family, mode = "function", envir = parent.frame(2))
    if( is.function(family)) family <- family()
    if (isTRUE(all.equal(family, gaussian()))) {
        mc[[1]] <- quote(lme4::lFormula)
        mc["family"] <- NULL            # to avoid an infinite loop
        return(eval(mc, parent.frame()))
    }
    if (family$family %in% c("quasibinomial", "quasipoisson", "quasi"))
        stop('"quasi" families cannot be used in glmer')

    dontChk <- c("verbose", "devFunOnly", "optimizer", "nAGQ")
    dots <- list(...)
    do.call(checkArgs, c(list("glmer"), dots[!names(dots) %in% dontChk]))

    cstr <- "check.formula.LHS"
    checkCtrlLevels(cstr, control[[cstr]])

    denv <- checkFormulaData(formula, data,
                             checkLHS = control$check.formula.LHS == "stop")
    formula <- as.formula(formula, env = denv) # substitute evaluated version
    if (getDoublevertDefault() == "split") {
      RHSForm(formula) <- reformulas::expandDoubleVerts(RHSForm(formula))
    }

    ## DRY ...
    m <- match(c("data", "subset", "weights", "na.action", "offset",
                 "mustart", "etastart"), names(mf), 0L)
    mf <- mf[c(1L, m)]
    mf$drop.unused.levels <- TRUE
    mf[[1L]] <- quote(stats::model.frame)

    ## substitute  special(x | f)  with  (x | f)
    fr.form. <- noSpecials(formula, specials = lme4_specials, delete = FALSE)
    ## substitute  (x | f)  and  (x || f)  with  (x + f)
    fr.form <- sub_specials(fr.form., specials = c("|", "||"),
                            keep_args = c(2L, 2L))
    environment(fr.form.) <- environment(fr.form) <-
        environment(formula)
    ## model.frame.default looks for these objects in the environment
    ## of the *formula* (see 'extras', which is anything passed in '...'),
    ## so they have to be put there:
    for (i in c("weights", "offset")) {
        if (!eval(bquote(missing(x=.(i)))))
            assign(i, get(i, parent.frame()), environment(fr.form))
    }
    mf$formula <- fr.form
    fr <- eval(mf, parent.frame())
    ## convert character vectors to factor (defensive)
    fr <- factorize(fr.form, fr, char.only = TRUE)
    ## store full, original formula & offset
    attr(fr,"formula") <- formula
    attr(fr,"offset") <- mf$offset
    ## attach starting coefficients to model frame so we can
    ##  pass them through to mkRespMod -> family()$initialize ...
    if (!missing(start) && is.list(start)) {
        fixef <- start$fixef %||% start$beta
        attr(fr,"start") <- fixef
    }
    n <- nrow(fr)
    ## random effects and terms modules
    ## get list of calls whose first argument is a call to '|'
    ##                x | f  ->      us(x | f)
    ##     nonspecial(x | f) ->      us(x | f)
    ##        special(x | f) -> special(x | f)
    bb1 <- findbars_x(formula, specials = lme4_specials,
                      default.special = "us", target = "|",
                      expand_doublevert_method = getDoublevertDefault())
    bb0 <- lapply(bb1, `[[`, 2L)
    reTrms <- reformulas::mkReTrms(bb0, fr, calc.lambdat = FALSE)
    reTrms <- upReTrms(reTrms, bb1) # local calc.lambdat=TRUE step
    ## If there is a covariance structure; ignore the check nobs.vs.nRE
    if (.anyStructured(reTrms$reCovs))
        control$check.nobs.vs.nRE <- "ignore"
    ## TODO: allow.n = !useSc {see FIXME below}
    wmsgNlev <- checkNlevels(reTrms$ flist, n = n, control, allow.n = TRUE)
    wmsgZdims <- checkZdims(reTrms$Ztlist, n = n, control, allow.n = TRUE)
    wmsgZrank <- checkZrank(reTrms$ Zt, n = n, control, nonSmall = 1e6, allow.n = TRUE)

    ## FIXME: adjust test for families with estimated scale parameter:
    ##   useSc is not defined yet/not defined properly?
    ##  if (useSc && maxlevels == n)
    ##          stop("number of levels of each grouping factor must be",
    ##                "greater than number of obs")

    ## fixed-effects model matrix X - remove random effect parts from formula:
    fixedform <- fr.form.
    RHSForm(fixedform) <- reformulas::nobars(RHSForm(fixedform))
    mf$formula <- fixedform
    ## re-evaluate model frame to extract predvars component
    fixedfr <- eval(mf, parent.frame())
    attr(attr(fr,"terms"),"predvars.fixed") <-
        attr(attr(fixedfr,"terms"),"predvars")

    ## ran-effects model frame (for predvars)
    ## important to COPY formula (and its environment)?
    ranform <- fr.form.
    RHSForm(ranform) <- reformulas::subbars(
        RHSForm(noSpecials(lme4_reOnly(ranform), specials = lme4_specials, delete = FALSE))
    )
    mf$formula <- ranform
    ranfr <- eval(mf, parent.frame())
    attr(attr(fr,"terms"), "predvars.random") <-
        attr(terms(ranfr), "predvars")

    X <- model.matrix(fixedform, fr, contrasts)#, sparse = FALSE, row.names = FALSE) ## sparseX not yet
    
    ## Scaling (if autoscale is on...)
    if (!is.null(control$autoscale) && control$autoscale) {
      if("(Intercept)" %in% colnames(X)){
        X_scaled <- scale(X[, -1])
        X[,-1] <- X_scaled
      } else {
        X_scaled <- scale(X)
        X <- X_scaled
      }
      attr(X, "scaled:center") <- attr(X_scaled, "scaled:center")
      attr(X, "scaled:scale") <- attr(X_scaled, "scaled:scale")
    }
    
    ## backward compatibility (keep no longer than ~2015):
    if(is.null(rankX.chk <- control[["check.rankX"]]))
        rankX.chk <- eval(formals(lmerControl)[["check.rankX"]])[[1]]
    X <- chkRank.drop.cols(X, kind=rankX.chk, tol = 1e-7)
    if(is.null(scaleX.chk <- control[["check.scaleX"]]))
        scaleX.chk <- eval(formals(lmerControl)[["check.scaleX"]])[[1]]
    X <- checkScaleX(X, kind=scaleX.chk)

    list(fr = fr, X = X, reTrms = reTrms, family = family, formula = formula,
         wmsgs = c(Nlev = wmsgNlev, Zdims = wmsgZdims, Zrank = wmsgZrank))
}

##' @rdname modular
##' @export
mkGlmerDevfun <- function(fr, X, reTrms, family,
                          nAGQ = if (control$nAGQ0initStep) 0L else 1L,
                          verbose = 0L,
                          maxit = 100L, control = glmerControl(), ...) {
    stopifnot(length(nAGQ <- as.integer(nAGQ)) == 1L,
              0L <= nAGQ, nAGQ <= 100L)
    verbose <- as.integer(verbose)
    maxit   <- as.integer(maxit)
    rho <- list2env(list(verbose=verbose, maxit=maxit,
                         tolPwrss= control$tolPwrss,
                         compDev = control$compDev),
                    parent = parent.frame())
    rho$pp <- do.call(merPredD$new,
                      c(reTrms[c("Zt","theta","Lambdat","Lind")],
                        n=nrow(X), list(X=X)))
    rho$resp <- if (missing(fr))
        mkRespMod(family=family, ...)
    else
        mkRespMod(fr, family=family)
    rho$mkPar <- mkMkPar(reTrms$reCovs)
    rho$mkTheta <- mkMkTheta(reTrms$reCovs)
    ## allow trivial y
    if (length(y <- rho$resp$y) > 0) {
        checkResponse(y, control$checkControl)
        rho$verbose <- as.integer(verbose)

        ## initialize (from mustart)
        .Call(glmerLaplace, rho$pp$ptr(), rho$resp$ptr(), nAGQ > 0L,
              control$tolPwrss, maxit, verbose)
        rho$lp0         <- rho$pp$linPred(1) # each pwrss opt begins at this eta
        rho$pwrssUpdate <- glmerPwrssUpdate
    }
    devfun <-
    mkdevfun(rho, 0L, maxit=maxit, verbose=verbose, control=control)
    if (nAGQ > 0L)
        updateGlmerDevfun(devfun, reTrms, nAGQ = nAGQ)
    else {
        rho$nAGQ  <- 0L
        rho$lower <- reTrms$lower
        rho$upper <- reTrms$upper %||% rep(Inf, length(reTrms$lower))
        devfun
    }
}



##' @rdname modular
##' @param nAGQ number of Gauss-Hermite quadrature points
##' @param stage optimization stage (1: nAGQ=0, optimize over theta only; 2: nAGQ possibly >0, optimize over theta and beta)
##' @export
optimizeGlmer <- function(devfun,
                          optimizer = if (nAGQ > 0L) "Nelder_Mead" else "bobyqa",
                          restart_edge=FALSE,
                          boundary.tol = formals(glmerControl)$boundary.tol,
                          verbose = 0L,
                          control = list(),
                          nAGQ = if (missing(stage) || stage == 1L) 0L else 1L,
                          stage,
                          start = NULL,
                          ...) {
    ## FIXME: deprecate 'stage' ... ?
    if (!missing(stage) && (nAGQ > 0L) != (stage != 1L))
        stop(gettextf("incompatible '%s' and '%s'", "nAGQ", "stage"),
             domain = NA)
    verbose <- as.integer(verbose)
    rho <- environment(devfun)
    start <- getStart(start, rho, nAGQ)
    lower <- rho$lower
    upper <- rho$upper %||% rep(Inf, length(rho$lower))
    opt <- optwrap(optimizer, devfun, start, lower=lower, upper=upper,
                   control=control, adj=nAGQ > 0L, verbose=verbose,
                   ...)
    if (nAGQ > 0L)
        rho$resp$setOffset(rho$baseOffset)
    if (restart_edge) ## FIXME: implement this ...
        stop("restart_edge not implemented for optimizeGlmer yet")
    if (boundary.tol > 0) {
        opt <- check.boundary(rho, opt, devfun, boundary.tol)
        if (nAGQ > 0L)
            rho$resp$setOffset(rho$baseOffset)
    }
    opt
}

check.boundary <- function(rho,opt,devfun,boundary.tol) {
    par0 <- opt$par
    lower <- rho$lower
    upper <- rho$upper %||% rep(Inf, length(rho$lower))
    dl <- par0 - lower
    du <- upper - par0
    if (!is.null(rho$dpars)) {
        dl <- dl[rho$dpars]
        du <- du[rho$dpars]
    }
    if (length(wl <- which(0 < dl & dl < boundary.tol)) > 0L |
        length(wu <- which(0 < du & du < boundary.tol)) > 0L) {
        ## try sucessive "close-to-edge parameters" to see
        ## if we can improve by setting them equal to the boundary
        for (i in wl) {
            par <- par0
            par[i] <- lower[i]
            if (devfun(par) < opt$fval) par0[i] <- par[i]
        }
        for (i in wu) {
            par <- par0
            par[i] <- upper[i]
            if (devfun(par) < opt$fval) par0[i] <- par[i]
        }
        opt$par <- par0
        opt$fval <- devfun(par0)
        ## re-run to reset (whether successful or not)
        ## FIXME: derivatives, Hessian etc. (and any other
        ## opt messages) *not* recomputed
    }
    return(opt)
}

## only do this function if nAGQ > 0L
##' @rdname modular
##' @export
updateGlmerDevfun <- function(devfun, reTrms, nAGQ = 1L){
    stopifnot(length(nAGQ <- as.integer(nAGQ)) == 1L,
              0L <= nAGQ, nAGQ <= 100L)
    if (nAGQ > 1L) {
        if (length(reTrms$flist) != 1L || length(reTrms$cnms[[1]]) != 1L)
            stop("nAGQ > 1 is only available for models with a single, scalar random-effects term")
    }
    rho <- environment(devfun)
    rho$nAGQ       <- nAGQ
    rho$lower      <- c(reTrms$lower,
                        rep(-Inf, length(rho$pp$beta0)))
    rho$upper      <- c(reTrms$upper %||% rep(Inf, length(reTrms$lower)),
                        rep( Inf, length(rho$pp$beta0)))
    rho$lp0        <- rho$pp$linPred(1)
    rho$dpars      <- seq_along(reTrms$lower)
    rho$baseOffset <- forceCopy(rho$resp$offset) # forcing a copy (!)
    rho$GQmat      <- GHrule(nAGQ)
    rho$fac        <- reTrms$flist[[1]]
    mkdevfun(rho, nAGQ)  # does this attach rho to devfun??
}

if (.Platform$OS.type != "windows") {
    library(lme4)
    library(testthat)

    .get.checkingOpts <- lme4:::.get.checkingOpts
    stopifnot(identical(
        .get.checkingOpts(
            c("CheckMe", "check.foo", "check.conv.1", "check.rankZ", "check.rankX"))
      , c("check.foo", "check.rankZ")))

    lmod <- lFormula(Reaction ~ Days + (Days|Subject), sleepstudy)
    devfun <- do.call(mkLmerDevfun, lmod)
    opt <- optimizeLmer(devfun)
    cc <- lme4:::checkConv(attr(opt,"derivs"), opt$par, ctrl = lmerControl()$checkConv,
                           lbound=environment(devfun)$lower)
    fm1 <- mkMerMod(environment(devfun), opt, lmod$reTrms, fr = lmod$fr,
                    lme4conv=cc)
    fm2 <- lmer(Reaction ~ Days + (Days|Subject), sleepstudy)

    ## basic equivalence
    fm1C <- fm1
    fm1C@call <- fm2@call
    expect_equal(fm2,fm1C)
    expect_equal(range(residuals(fm1)), c(-101.18, 132.547), tolerance = 1e-5) # these are "outliers"!
    expect_is(model.frame(fm1),"data.frame")
    ## formulae
    mfm1 <- model.frame(fm1)
    expect_equal(formula(fm1),         Reaction ~ Days + (Days | Subject))
    expect_equal(formula(terms(mfm1)), Reaction ~ Days + (Days + Subject))
    new_form_modframe <- (getRversion() >= "3.6.0" &&
                          as.numeric(version[["svn rev"]]) >= 75891)
    expect_equal(formula(mfm1),
                 if(new_form_modframe) {
                     Reaction ~ Days + (Days + Subject)
                 } else
                     Reaction ~ Days + Subject
                 )
    ## predictions
    expect_equal(predict(fm1,newdata=sleepstudy[1:10,],re.form=NULL),
                 predict(fm2,newdata=sleepstudy[1:10,],re.form=NULL))
    expect_equal(predict(fm1,newdata=sleepstudy),
                 predict(fm1))

    lmodOff <- lFormula(Reaction ~ Days + (Days|Subject) + offset(0.5*Days),
                        sleepstudy)
    devfunOff <- do.call(mkLmerDevfun, lmodOff)
    opt <- optimizeLmer(devfunOff)
    fm1Off <- mkMerMod(environment(devfunOff), opt, lmodOff$reTrms, fr = lmodOff$fr)
    fm2Off <- lmer(Reaction ~ Days + (Days|Subject) + offset(0.5*Days), sleepstudy)
    expect_equal(predict(fm1Off,newdata=sleepstudy[1:10,],re.form=NULL),
                 predict(fm2Off,newdata=sleepstudy[1:10,],re.form=NULL))

    ## FIXME: need more torture tests with offset specified, in different environments ...

    ## FIXME: drop1(.) doesn't work with modular objects ... hard to see how it
    ##  could, though ...
    ## drop1(fm1Off)
    drop1(fm2Off)

} ## skip on windows (for speed)

#context("testing '||' notation for independent ranefs")

form1 <- Reaction ~ Days + (Days || Subject)
form2 <- Reaction ~ Days + (1 | Subject) + (0 + Days | Subject)
m1 <- lmer(form1, sleepstudy)
m2 <- lmer(form2, sleepstudy)

test_that("basic intercept + slope '||' works", {
  if (!getOption("lme4.doublevert.default", "split") == "split") skip()
  
  expect_equivalent(lFormula(form1, sleepstudy)$reTrms,
                    lFormula(form2, sleepstudy)$reTrms)
  expect_equivalent(fitted(m1),
                    fitted(m2))


  if (FALSE) {
    ## library(glmmTMB)
    m3 <- glmmTMB(form1, sleepstudy, REML = TRUE,
                  start = list(theta = log(c(25, 6))))
    dput(head(fitted(m3), 4L))
    }
    expect_equivalent(head(fitted(m2), 4L),
                      ## fitted values from glmmTMB:
                      c(252.917801422647, 272.708576602026,
                        292.499351781404, 312.290126960783),
                      tolerance = 5e-5)
})

test_that("factors", {

    set.seed(101)
    d <- data.frame(x=runif(1000),y=runif(1000),f1=rep(1:10,each=100),f2=rep(1:10,100))
    d2 <- transform(d,f1=factor(f1),f2=factor(f2))
    expect_that(lm1 <- lmer(y~x+(1|f1/f2),data=d), is_a("lmerMod"))
    expect_that(lm2 <- lmer(y~x+(1|f1/f2),data=d2),is_a("lmerMod"))
    expect_equivalent(lm1,lm2)

})

## this will fail/take a long time unless we handle interactions carefully
test_that("savvy interactions", {
    dd <- data.frame(y = 1:10000, f1 = factor(1:10000), f2 = factor(1:10000))
    F1 <- lFormula(y ~ 1 + (1|f1/f2), data =dd,
             control = lmerControl(check.nobs.vs.nlev = "ignore",
                                   check.nobs.vs.nRE = "ignore"))
    expect_equal(dim(F1$reTrms$Zt), c(20000, 10000))
})

test_that("savvy factor level ordering", {

    check_f <- function(n = 200, frac = 0.7, fix_order = TRUE, check_order = TRUE) {
        dd <- expand.grid(f1 = seq(n), f2 = seq(n))
        dd <- within(dd, {
            f1 <- factor(f1, levels = sample(unique(f1)))
            f2 <- factor(f2, levels = sample(unique(f2)))
        })
        dd <- dd[sample(nrow(dd), size = round(frac*nrow(dd)), replace = FALSE), ]
        dd <- within(dd, {
            f12 <- f1:f2
            f12d <- droplevels(f12)
        })
        new_levels <- with(dd, levels(`%i%`(f1,f2, fix.order = fix_order)))
        ## don't want to pay the cost of checking if unneeded {for benchmarking}
        if (fix_order && check_order) { stopifnot(identical(levels(dd$f12d), new_levels)) }
        return(TRUE)
    }

    ## should fail within check_f() if levels don't match
    expect_true(check_f(), "'savvy' factor levels match brute-force version")
    
    ## library(microbenchmark)
    ## set.seed(101)
    ## m1 <- microbenchmark(check_f(fix_order = TRUE, check_order = FALSE),
    ## check_f(fix_order = FALSE))
})

## List of linear models according to a grouping factor

## Extract the model formula
modelFormula <- function(form)
{
    if (!inherits(form, "formula") || length(form) != 3)
        stop("formula must be a two-sided formula object")
    rhs <- form[[3]]
    if (!inherits(rhs, "call") || rhs[[1]] != as.symbol('|'))
        stop("rhs of formula must be a conditioning expression")
    form[[3]] <- rhs[[2]]
    list(model = dropOffset(form), groups = rhs[[3]])
}

dropOffset <- function(form) {
    ## atomic
    if (is.symbol(form) || is.numeric(form)) return(form)
    ## binary
    if (identical(form[[1]],quote(offset))) {
        NULL
    } else {
        ## unary operator
        if (length(form)==2) {
            form[[2]] <- dropOffset(form[[2]])
            return(form)
        }
        nb2 <- dropOffset(form[[2]])
        nb3 <- dropOffset(form[[3]])
        if (is.null(nb2))
            nb3
        else if (is.null(nb3))
            nb2
        else {
            form[[2]] <- nb2
            form[[3]] <- nb3
            return(form)
        }
    }
}

## dropOffset(y~x)
## dropOffset(y~x+offset(stuff))
## dropOffset(y~-x+offset(stuff))
## dropOffset(~-x+offset(stuff))

if(getRversion() < "3.5.0") {
##' Utility for lmList(), ...: Collect errors from a list \code{x},
##' produce a "summary warning" and keep that message as "warningMsg" attribute
warnErrList <- function(x, warn = TRUE) {
  errs <- vapply(x, inherits, NA, what = "error")
  if (any(errs)) {
    v.err <- x[errs]
    e.call <- paste(deparse(conditionCall(v.err[[1]])), collapse = "\n")
    tt <- table(vapply(v.err, conditionMessage, ""))
    msg <-
      if(length(tt) == 1)
        sprintf(ngettext(tt[[1]],
                         "%d error caught in %s: %s",
                         "%d times caught the same error in %s: %s"),
                tt[[1]], e.call, names(tt)[[1]])
      else ## at least two different errors caught
        paste(gettextf(
          "%d errors caught in %s.  The error messages and their frequencies are",
          sum(tt), e.call),
          paste(capture.output(sort(tt)), collapse="\n"), sep="\n")

    if(warn)
        warning(msg, call. = FALSE, domain = NA)
    x[errs] <- list(NULL)
    attr(x, "warningMsg") <- msg
  }
  x
}
}# R <= 3.4.x



##' @title List of lm Objects with a Common Model
##' @param formula a linear formula object of the form
##'     \code{y ~ x1+...+xn | g}. In the formula object, \code{y} represents
##'     the response, \code{x1,...,xn} the covariates, and \code{g} the
##'     grouping factor specifying the partitioning of the data according to
##'     which different \code{lm} fits should be performed.
##' @inheritParams lmer
##' @param family an optional family specification for a generalized
##'     linear model.
##' @param pool logical scalar, should the variance estimate pool the
##'     residual sums of squares
##' @param ... additional, optional arguments to be passed to the
##'     model function or family evaluation.
##' @export
lmList <- function(formula, data, family, subset, weights,
                   na.action, offset,
                   pool = !isGLM || .hasScale(family2char(family)),
                   warn = TRUE, ...) {
    stopifnot(inherits(formula, "formula"))

    ## model.frame(groupedData) was problematic ... but not as we
    ## are currently using it.

    mCall <- mf <- match.call()
    ## MM: I had this (instead of below  (inherited from nlme?)):
    ## if(!missing(subset))
    ##     data <- data[eval(asOneSidedFormula(mf[["subset"]])[[2]], data),, drop = FALSE]

    ## in contrast to the usual R model-fitting idiom, we do **not**
    ## want to evaluate the model frame here; it will mess up any derived
    ## variables we have when we go to fit the sub-models.  We were previously
    ## using model.frame() on the entire data set, but that does not
    ## exclude unused columns ... and hence screws us up when there are
    ## NA values in unused columns.  All we need the model frame for
    ## is evaluating the groups.

    # TODO: import reformulas::get_grpvars once reformulas has updated
    ff <- get_grpvars(formula)
    if (length(ff) > 1) 
      stop("lme4::lmList only works with simple grouping formulas (not involving * or /); 
            use interaction(...) to construct an appropriate grouping variable first.")
    
    ## keep weights and offsets in case we have NAs there??
    m <- match(c("formula", "data", "subset", "na.action",
                 "weights", "offset"), names(mf), 0)
    mf <- mf[c(1, m)]
    ## substitute `+' for `|' in the formula
    mf$formula <- reformulas::subbars(formula)
    mf$drop.unused.levels <- TRUE
    ## pass NAs for now -- want *all* groups, weights, offsets recovered
    mf$na.action <- na.pass
    mf[[1L]] <- quote(stats::model.frame)
    frm <- eval.parent(mf) ## <- including "..."
    data[["(weights)"]] <- model.weights(frm)
    data[["(offset)"]] <- model.offset(frm)
    mform <- modelFormula(formula)
    isGLM <- !missing(family) ## TODO in future, consider isNLM / isNLS
    groups <- eval(mform$groups, frm)
    if (!is.factor(groups)) groups <- factor(groups)
    fit <- if (isGLM) glm else lm
    mf2 <- if (missing(family)) NULL else list(family=family)
    fitfun <- function(data, formula) {
        tryCatch({
          do.call(fit, c(list(formula, data,
                              ## don't use model.offset()/model.weights from stats() - warning with tibbles
                               weights = data[["(weights)"]],
                               offset = data[["(offset)"]], ...),
                          mf2))
        }, error = function(x) x)
    }
    ## split *original data*, not frm (derived model frame), on groups
    ## we have to do this because we need raw, not derived variables
    ## when evaluating linear regression.

    ## need to apply subset first ((or even much earlier ??))
    ## (hope there aren't tricky interactions with NAs in subset ... ??)
    if (!missing(subset)) {
        data <- eval(substitute(data[subset,]), list2env(data))
    }

    frm.split <- split(data, groups)
    ## NB:  levels() is only  OK if grouping variable is a factor
    nms <- names(frm.split)
    val <- ## mapply(fitfun,
        lapply(frm.split, fitfun, formula = as.formula(mform$model))
    ## use warnErrList(), but expand msg for back compatibility and user-friendliness:
    val <- warnErrList(val, warn = FALSE)
    ## Contrary to nlme, we keep the erronous ones as well (with a warning):
    if(warn && length(wMsg <- attr(val,"warningMsg"))) {
        if(grepl("contrasts.* factors? .* 2 ", wMsg)){ # try to match translated msg, too
            warning("Fitting failed for ", sum(vapply(val, is.null, NA)),
                    " group(s), probably because a factor only had one level",
                    sub(".*:", ":\n ", wMsg), domain=NA)
        } else
            warning(wMsg, domain=NA)
    }
    val <- new("lmList4", 
        setNames(val, nms),
        call = mCall, pool = pool,
        groups = ordered(groups),
        origOrder = match(unique(as.character(groups)), nms)
        )
    attr(val, "groupsForm") <- reformulate(ff)
    val
}

## (currently hidden) auxiliaries
isGLMlist <- function(object, ...) {
    D <- getDataPart(object)
    length(D) >= 1 && inherits(D[[1]], "glm")
}

## does a glm family have a "scale" [from stats:::logLik.glm() ] :
.hasScale <- function(family)
    family %in% c("gaussian", "Gamma", "inverse.gaussian")
family2char <- function(fam) {
    if(is.function(fam)) fam()$family else if(!is.character(fam)) fam$family else fam
}

##' Does a lmList4 object have a "scale" / sigma / useScale ?
hasScale <- function(object)
    !isGLMlist(object) || .hasScale(family(object[[1]])$family)


##' @importFrom stats coef
##' @S3method coef lmList4
## Extract the coefficients and form a  data.frame if possible
## FIXME: commented out nlme stuff (augFrame etc.).  Restore, or delete for good
## FIXME: modified so that non-estimated values will be NA rather than set to
##        coefs of first non-null estimate.  Is that OK??
coef.lmList4 <- function(object,
                        ## augFrame = FALSE, data = NULL,
                        ##which = NULL, FUN = mean, omitGroupingFactor = TRUE,
                        ...) {
    coefs <- lapply(object, coef)
    non.null <- !vapply(coefs, is.null, logical(1))
    if (any(non.null)) {
        template <- coefs[non.null][[1]]
        ## different parameter sets may be estimated for different subsets of data ...
        allnames <- Reduce(union, lapply(coefs[non.null], names))
        if (is.numeric(template)) {
            co <- matrix(NA_real_,
                         ncol = length(allnames),
                         nrow = length(coefs),
                         dimnames = list(names(object), allnames))
            for (i in names(object)) {
                co[i,names(coefs[[i]])] <- coefs[[i]]
            }
            coefs <- as.data.frame(co)
            effectNames <- names(coefs)
            ## if(augFrame) {
            ##     if (is.null(data)) {
            ##         data <- getData(object)
            ##     }
            ##     data <- as.data.frame(data)
            ##     if (is.null(which)) {
            ##         which <- 1:ncol(data)
            ##     }
            ##     data <- data[, which, drop = FALSE]
            ##     ## eliminating columns with same names as effects
            ##     data <- data[, is.na(match(names(data), effectNames)), drop = FALSE]
            ##     data <- gsummary(data, FUN = FUN, groups = getGroups(object))
            ##     if (omitGroupingFactor) {
            ##         data <- data[, is.na(match(names(data),
            ##                                    names(getGroupsFormula(object,
            ##                                                           asList = TRUE)))),
            ##                      drop = FALSE]
            ##     }
            ##     if (length(data) > 0) {
            ##         coefs <- cbind(coefs, data[row.names(coefs),,drop = FALSE])
            ##     }
            ## }
            attr(coefs, "level") <- attr(object, "level")
            attr(coefs, "label") <- "Coefficients"
            attr(coefs, "effectNames") <- effectNames
            attr(coefs, "standardized") <- FALSE
        } ## is.numeric(template)
    }
    coefs
}

### FIXME?:  nlme *does* export this -- we export sigma()  [instead ?]
pooledSD <- function(x, allow.0.df = TRUE)
{
    stopifnot(is(x, "lmList4"))
    if(!hasScale(x)) {
        if(allow.0.df)
            return(structure(1, df = NA)) ## scale := 1  if(!useScale)
        ## else
        stop("no scale, hence no pooled SD for this object")
    }

    sumsqr <- rowSums(sapply(x,
                             function(el) {
                                 if (is.null(el)) {
                                     c(0,0)
                                 } else {
                                     res <- resid(el)
                                     c(sum(res^2), df.residual(el))
                                 }
                             }))
    if (sumsqr[2] == 0) { ## FIXME? rather return NA with a warning ??
        stop("No degrees of freedom for estimating std. dev.")
    }
    val <- sqrt(sumsqr[1]/sumsqr[2])
    attr(val, "df") <- sumsqr[2]
    val
}

sigma.lmList4 <- function(object, ...)
    if(hasScale(object)) as.vector(pooledSD(object)) else 1
## 1 for GLM  <==>  1 when useScale is FALSE for [G]LMMs


##' @importFrom methods show
##' @exportMethod show
setMethod("show", "lmList4", function(object)
{
    mCall <- object@call
    cat("Call:", deparse(mCall), "\n")
    cat("Coefficients:\n")
    print(coef(object))
    if (object@pool) {
        cat("\n")
        poolSD <- pooledSD(object)
        dfRes <- attr(poolSD, "df")
        RSE <- c(poolSD)
        cat("Degrees of freedom: ", length(unlist(lapply(object, fitted))),
            " total; ", dfRes, " residual\n", sep = "")
        cat("Residual standard error:", format(RSE))
        cat("\n")
    }
})

##' @S3method confint lmList4
confint.lmList4 <- function(object, parm, level = 0.95, ...)
{
    mCall <- match.call()
    if (length(object) < 1)
        return(new("lmList4.confint", array(numeric(0), c(0,0,0))))
    mCall$object <- object[[1]]
    ## the old recursive strategy doesn't work with S3 objects --
    ##  calls "confint.lmList4" again instead of calling "confint"
    mCall[[1]] <- quote(confint)
    template <- eval(mCall)
    if(is.null(d <- dim(template))) ## MASS:::confint.profile.glm() uses drop(), giving vector
        d <- dim(template <- rbind("(Intercept)" = template))
    template[] <- NA_real_
    val <- array(template, c(d, length(object)),
                 c(dimnames(template), list(names(object))))
    pool <- list(...)$pool
    if (is.null(pool)) pool <- object$pool
    if (length(pool) > 0 && pool[1]) { ## do our own
        sd <- pooledSD(object)
        a <- (1 - level)/2
        fac <- sd * qt(c(a, 1 - a), attr(sd, "df"))
        parm <- dimnames(template)[[1]]
        for (i in seq_along(object))
            if(!is.null(ob.i <- object[[i]]))
                val[ , , i] <- coef(ob.i)[parm] +
                    sqrt(diag(summary(object[[i]], corr = FALSE)$cov.unscaled
                              )[parm]) %o% fac
    } else { ## build on confint() method for "glm" / "lm" :
        for (i in seq_along(object))
            if(!is.null(mCall$object <- object[[i]])) {
                ci <- eval(mCall)
                if(is.null(dim(ci))) ## MASS:::confint.profile.glm() ...
                    ci <- rbind("(Intercept)" = ci)
                if(identical(dim(ci), d))
                    val[ , , i] <- ci
                else ## some coefficients were not estimable
                    val[rownames(ci), , i] <- ci
            }
    }
    new("lmList4.confint", aperm(val, 3:1))
}

##' @importFrom graphics plot
##' @importFrom lattice .......
##' @S3method plot lmList4.confint
plot.lmList4.confint <- function(x, y, order, ...)
{
##    stopifnot(require("lattice"))
    arr <- as(x, "array")
    dd <- dim(arr)
    dn <- dimnames(arr)
    levs <- dn[[1]]
    if (!missing(order) &&
        (ord <- round(order[1])) %in% seq(dd[3]))
        levs <- levs[order(rowSums(arr[ , , ord]))]
    ll <- length(arr)
    df <- data.frame(group =
                     ordered(rep(dn[[1]], dd[2] * dd[3]),
                             levels = levs),
                     intervals = as.vector(arr),
                     what = gl(dd[3], dd[1] * dd[2], length = ll, labels = dn[[3]]),
                     end = gl(dd[2], dd[1], length = ll))
    panelfun <- function(x, y, pch = dot.symbol$pch,
            col = dot.symbol$col, cex = dot.symbol$cex,
            font = dot.symbol$font, ...)
        {
            x <- as.numeric(x)
            y <- as.numeric(y)
            ok <- !is.na(x) & !is.na(y)
            yy <- y[ok]
            xx <- x[ok]
            dot.symbol <- trellis.par.get("dot.symbol")
            dot.line <- trellis.par.get("dot.line")
            panel.abline(h = yy, lwd = dot.line$lwd, lty = dot.line$lty, col =
                         dot.line$col)
            lpoints(xx, yy, pch = "|", col = col, cex = cex, font = font, ...)
            lower <- tapply(xx, yy, min)
            upper <- tapply(xx, yy, max)
            nams <- as.numeric(names(lower))
            lsegments(lower, nams, upper, nams, col = col, lty = 1, lwd =
                      if (dot.line$lwd) {
                          dot.line$lwd
                      } else {
                          2
                      })
        }
    dotplot(group ~ intervals | what,
            data = df,
            scales = list(x="free"),
            panel=panelfun, ...)
}

##' @importFrom stats update
##' @S3method update lmList4
update.lmList4 <- function(object, formula., ..., evaluate = TRUE) {
    call <- object@call
    if (is.null(call))
        stop("need an object with call slot")
    extras <- match.call(expand.dots = FALSE)$...
    if (!missing(formula.))
        call$formula <- update.formula(formula(object), formula.)
    if (length(extras) > 0) {
        existing <- !is.na(match(names(extras), names(call)))
        for (a in names(extras)[existing]) call[[a]] <- extras[[a]]
        if (any(!existing)) {
            call <- c(as.list(call), extras[!existing])
            call <- as.call(call)
        }
    }
    if (evaluate)
        eval(call, parent.frame())
    else call
}

##' @importFrom stats formula
##' @S3method formula lmList4
##' @return of class "formula" ==> as.formula() rather than just [["formula"]]
formula.lmList4 <- function(x, ...) structure(x@call[["formula"]], class = "formula")

##' Get the grouping factor of an "lmList4" object
##' Important as auxiliary method for many of the nlme-imported methods:
getGroups.lmList4 <- function(object, ...) object@groups


### All the other "lmList4" S3 methods are imported from  nmle :
##
.ns.nlme <- asNamespace("nlme")
.ns.lme4 <- environment() ## == asNamespace("lme4") during build/load
##
## To do this, we need to make them use *our* namespace, e.g. to use our  pooledSD()
## However, then we get from codetools :
##
## fitted.lmList4: no visible global function definition for 'getGroups'
## pairs.lmList4: no visible global function definition for 'gsummary'
## pairs.lmList4: no visible global function definition for 'getGroups'
## plot.lmList4: no visible global function definition for 'c_deparse'
## plot.lmList4: no visible global function definition for 'getGroups'
## predict.lmList4: no visible global function definition for 'getGroups'
## print.lmList4: no visible global function definition for 'c_deparse'
## qqnorm.lmList4: no visible global function definition for 'getGroups'
## qqnorm.lmList4: no visible global function definition for 'gsummary'
## residuals.lmList4: no visible global function definition for 'getGroups'
##
## which we avoid via
for(fn in c("gsummary", "c_deparse")) {
  if (exists(fn, envir = .ns.nlme)) {
    assign(fn, get(fn, envir = .ns.nlme, inherits=FALSE))
  }
}

for(fn in c("fitted", "fixef", "logLik", "pairs", "plot", "predict",
            ## "print", <- have our own show()
           "qqnorm", "ranef", "residuals", "summary")) {
    meth <- get(paste(fn, "lmList",  sep="."), envir = .ns.nlme, inherits=FALSE)
    environment(meth) <- .ns.lme4 # e.g. in order to use *our* pooledSD()
    assign(paste(fn, "lmList4", sep="."), meth)
}
rm(fn)

## use old (<=3.5.2) sample() algorithm if necessary
if ("sample.kind" %in% names(formals(RNGkind))) {
    suppressWarnings(RNGkind("Mersenne-Twister", "Inversion", "Rounding"))
}

data("cbpp", package = "lme4")

#context("lmList")
test_that("basic lmList", {
    set.seed(17)
    fm1. <- lmList(Reaction ~ Days | Subject, sleepstudy, pool=FALSE)
    fm1  <- lmList(Reaction ~ Days | Subject, sleepstudy)
    cf.fm1 <- data.frame(
        `(Intercept)` =
        c(244.19267, 205.05495, 203.48423, 289.68509, 285.73897, 264.25161,
          275.01911, 240.16291, 263.03469, 290.10413, 215.11177, 225.8346,
          261.14701, 276.37207, 254.96815, 210.44909, 253.63604, 267.0448),
    Days =
        c(21.764702, 2.2617855, 6.1148988, 3.0080727, 5.2660188, 9.5667679,
          9.1420455, 12.253141, -2.8810339, 19.025974, 13.493933, 19.504017,
          6.4334976, 13.566549, 11.348109, 18.056151, 9.1884448, 11.298073))
    expect_equal(signif(coef(fm1), 8), cf.fm1,
		    tolerance = 1e-7, check.attributes=FALSE)
    expect_equal(coef(fm1.), coef(fm1))
    expect_true(inherits(formula(fm1), "formula")) ## <- had been wrong till 2015-04-09

    sm1. <- summary(fm1.)
    sm1 <- summary(fm1)
    expect_equal(sm1$RSE, 25.5918156267, tolerance = 1e-10)
    cf1 <- confint(fm1)


    ## Calling the plot.lmList4.confint() method :
    expect_true(inherits(pcf1 <- plot(cf1), "trellis"))

})

test_that("orthodont", {
    data(Orthodont, package="nlme")
    fm2 <- lmList(distance ~ age | Subject, Orthodont)
    fe2 <- fixef(fm2)
    expect_equal(fe2, c("(Intercept)" = 16.7611111111111,
                               age = 0.660185185185185))
    expect_true(inherits(pairs(fm2), "trellis"))
})

test_that("simulated", {
    set.seed(12)
    d <- data.frame(
        g = sample(c("A","B","C","D","E"), 250, replace=TRUE),
        y1 = runif(250, max=100),
        y2 = sample(c(0,1), 250, replace=TRUE)
    )

    fm3.1 <- lmList(y1 ~ 1 | g, data=d)
    expect_equal(coef(fm3.1),
    structure(list(`(Intercept)` = c(45.8945525606396, 50.1127995110841,
                 49.5320538515225, 52.4286874305165, 48.7716343882989)),
              .Names = "(Intercept)",
              row.names = c("A",
                            "B", "C", "D", "E"), class = "data.frame",
              label = "Coefficients", effectNames = "(Intercept)",
              standardized = FALSE))

    cf31 <- confint(fm3.1)
    expect_true(inherits(plot(cf31), "trellis"))

    fm3.2 <- lmList(y2 ~ 1 | g, data=d, family=binomial)
    ##                                ^^^^^^^^ "glmList"
    cf32 <- suppressMessages(confint(fm3.2,quiet=TRUE))
    expect_identical(dim(cf32), c(5L,2:1))
    expect_true(inherits(plot(cf32), "trellis"))
    expect_equal(unname(getDataPart(signif(drop(cf32), 6))),
	    cbind(c(-0.400041, -0.311489, -1.07774, -0.841075, -0.273828),
	  c( 0.743188,  0.768538, 0.0723138, 0.274392,  0.890795)))


})

test_that("cbpp", {
    ## "glmList" (2) -- here,  herd == 8 has only one observation => not estimable
    expect_warning(fm4 <- lmList(cbind(incidence, size - incidence) ~ period | herd,
                                 family=binomial, data=cbpp),
                   "Fitting failed for ")

    cf4 <- coef(fm4) # with some 5 NA's
    ## match NA locations
    expect_equal(dim(cf4),c(15,4))
    expect_identical(which(is.na(cf4)),
                     sort(as.integer(c(8+15*(0:3), 47))))

    expect_warning(fm4B <- lme4::lmList(incidence ~ period | herd,
                                        data=cbpp),
                   "Fitting failed")



if(FALSE) {
        ## FIXME: this is actually an nlme bug ...
        ## https://bugs.r-project.org/bugzilla/show_bug.cgi?id=16542
        try(summary(fm4))
        ## Error in `[<-`(`*tmp*`, use, use, ii, value = lst[[ii]]) :
        ##   subscript out of bounds
        library(nlme)
        data("cbpp",package="lme4")
        fm6 <- nlme::lmList(incidence ~ period | herd, data=cbpp)
        try(coef(fm6))  ## coef does *not* work here
        try(summary(fm6))

        ## this is a slightly odd example because the residual df from
        ##  these fits are in fact zero ...  so pooled.SD fails, as it should

    }
})

test_that("NA,weights,offsets", {

    ## from GH #320
    set.seed(101)
    x <- 1:8
    y <- c(2,2,5,4,3,1,2,1)
    g <- c(1,1,1,2,2,3,3,3)
    dat <- data.frame(x=x, y=y, g=g)
    m1 <- lmList(y ~ x | g, data=dat)
    expect_false(any(is.na(coef(m1))))
    w <- runif(nrow(sleepstudy))
    m2 <- lmList(Reaction ~ Days | Subject,
                 weights=w, sleepstudy)
    ss <- subset(sleepstudy,Subject==levels(Subject)[1])
    m2X <- lm(Reaction ~ Days, ss, weights=w[1:nrow(ss)])
    expect_equal(coef(m2X),as.matrix(coef(m2))[1,])
    m3 <- lmList(Reaction ~ Days | Subject, sleepstudy)
    m4 <- lmList(Reaction ~ Days | Subject,
                 offset=w, sleepstudy)
    m4X <- lm(Reaction ~ Days, ss, offset=w[1:nrow(ss)])
    expect_equal(coef(m4X),as.matrix(coef(m4))[1,])
    expect_false(identical(m2,m3))
    expect_false(identical(m4,m3))
    m5 <- lmList(Reaction ~ Days + offset(w) | Subject, sleepstudy)
    expect_equal(coef(m5),coef(m4))

    ## more from GH 320

    dat2 <- data.frame(dat,xx=c(NA,NA,NA,1:4,NA))
    m5 <- lmList(y ~ x | g, data=dat2)
    expect_equal(unlist(coef(m5)[1,]),
                 coef(lm(y~x,subset=(g==1))))
    expect_equal(unlist(coef(m5)[3,]),
                 coef(lm(y~x,subset=(g==3))))
})

test_that("pooled",
{
    ## GH #26
    fm_lme4 <- lme4:::lmList(Reaction ~ Days | Subject, sleepstudy)
    fm_nlme <- nlme:::lmList(Reaction ~ Days | Subject, sleepstudy)
    fm_nlme_nopool <- nlme:::lmList(Reaction ~ Days | Subject, sleepstudy, pool=FALSE)
    ci_lme4_pooled <- confint(fm_lme4,pool=TRUE) #get low and high CI estimates and pooled sd
    ci_nlme_pooled <- nlme:::intervals(fm_nlme,pool=TRUE)
    expect_equal(unname(ci_lme4_pooled[,,1]),unname(ci_nlme_pooled[,c(1,3),1]))
    ci_lme4_nopool1 <- confint(fm_lme4,pool=FALSE)
    ci_lme4_nopool2 <- confint(fm_lme4)
    expect_identical(ci_lme4_nopool1,ci_lme4_nopool2)
    ## BUG in nlme::intervals ... ? can't get CIs on unpooled fits
    ## nlme::intervals(fm_nlme,pool=FALSE)
    ## nlme::intervals(fm_nlme_nopool)
    expect_equal(ci_lme4_nopool1[1:3,,1],
                 structure(c(179.433862895996, 193.026448122379, 186.785722998616,
                             308.951475285822, 217.083442786712, 220.182727910474),
                           .Dim = c(3L, 2L), .Dimnames = list(c("308", "309", "310"),
                                                              c("2.5 %", "97.5 %"))))
})

test_that("derived variables",
          {
              fm_lme4 <- lme4:::lmList(log(Reaction) ~ Days | Subject, sleepstudy)
              fm_nlme <- nlme:::lmList(log(Reaction) ~ Days | Subject, sleepstudy)
              expect_equal(c(coef(fm_lme4)),c(coef(fm_nlme)),tolerance=1e-5)
          })

test_that("subset", {
    data(MathAchieve, package="nlme")
    data(MathAchSchool, package="nlme")
    RB <- merge(MathAchieve, MathAchSchool[, c("School", "Sector")],
                by="School")
    names(RB) <- tolower(names(RB))
    RB$cses <- with(RB, ses - meanses)
    cat.list.nlme <- nlme::lmList(mathach ~ cses | school,
                  subset = sector=="Catholic",
                  data=RB)
    cat.list.lme4 <- lme4::lmList(mathach ~ cses | school,
                                  subset = sector=="Catholic", data=RB)
    expect_equal(c(coef(cat.list.lme4)),
                 c(coef(cat.list.nlme)),tolerance=1e-5)
})

if (requireNamespace("tibble")) {
  test_that("avoid tibble warnings", {
    ## GH 645
    op <- options(warn = 2)
    m1 <- lmList(Reaction ~ Days | Subject, data = sleepstudy)
    m2 <- lmList(Reaction ~ Days | Subject, data = tibble::as_tibble(sleepstudy))
    expect_identical(coef(m1), coef(m2))
    options(op)
  })
}

## Generate sparse multidimensional Gaussian quadrature grids --->  ../man/GQdk.Rd
## Unused currently; rather GHrule() --> ./GHrule.R
GQdk <- function(d=1L, k=1L) {
    stopifnot(0L < (d <- as.integer(d)[1]),
              d <= 20L,
              0L < (k <- as.integer(k)[1]),
	      k <= length(GQNd <- GQN[[d]]))## -> GQN, stored in ./sysdata.rda
    tmat <- t(GQNd[[k]])
    ##rperms<- combinat::permn(seq_len(d) + 1L, function(v) c(1L,v))
    ## rperms <- lapply(.Call(allPerm_int, seq_len(d) + 1L), function(v) c(1L, v))
    perms <- tryCatch (
                       .Call(allPerm_int, seq_len(d) + 1L, as.integer(factorial(d))),
                       warning = function (w) w,
                       error = function (e) e)
    if (methods::is(perms, "error") | methods::is(perms, "warning"))
        stop("Can not allocate a vector that large")
    rperms <- lapply(perms, function(v) c(1L, v))

    dd <- unname(as.matrix(do.call(expand.grid, c(rep.int(list(c(-1,1)), d), KEEP.OUT.ATTRS=FALSE))))
    #unname(unique(t(do.call(cbind,
    #                        lapply(as.data.frame(t(cbind(1, dd))),
    #                               "*", e2=do.call(cbind, lapply(rperms, function(ind) tmat[ind,])))))))
    e2 <- do.call(cbind, lapply(rperms, function(ind) tmat[ind,]))
    ddf <- as.data.frame(t(cbind(1,dd)))
    res <- NULL
    for (i in 1:ncol(ddf))
        res <- unique(rbind(res, t(ddf[, i] * e2)))
    return(res)
}

## require(agridat)
## dat <- gotway.hessianfly

if (.Platform$OS.type != "windows") {
## don't actually use gotway_hessianfly_fit or gotway_hessianfly_prof,
## so we should be OK even with R< 3.0.1
load(system.file("testdata","gotway_hessianfly.rda",package="lme4"))
# Block random.  See Glimmix manual, output 1.18.
# Note: (Different parameterization)

## require("lme4.0")
## fit2 <- glmer(cbind(y, n-y) ~ gen + (1|block), data=dat, family=binomial)
## params <- list(fixef=fixef(fit2),theta=getME(fit2,"theta"))
## detach("package:lme4.0")
lme4.0fit <- structure(list(fixef = structure(c(1.50345713031203, -0.193853259383803, 
-0.540808391060274, -1.43419379979154, -0.203701042949808, -0.978322555343941, 
-0.604078624475678, -1.67742449813309, -1.39842466673692, -0.681709344788684, 
-1.46295367186169, -1.45908310198959, -3.55285756517073, -2.50731975980307, 
-2.08716296677356, -2.96974270029992), names = c("(Intercept)", 
"genG02", "genG03", "genG04", "genG05", "genG06", "genG07", "genG08", 
"genG09", "genG10", "genG11", "genG12", "genG13", "genG14", "genG15", 
"genG16")), theta = structure(0.0319087494293615, names = "block.(Intercept)")), names = c("fixef", 
"theta"))

## start doesn't work because we don't get there
library(lme4)
m1 <- glmer(cbind(y, n-y) ~ gen + (1|block), data=gotway.hessianfly,
            family=binomial)
m1B <- update(m1,control=glmerControl(optimizer="bobyqa"))
max(abs(m1@optinfo$derivs$gradient))  ## 0.0012
max(abs(m1B@optinfo$derivs$gradient)) ## 2.03e-5
abs(m1@optinfo$derivs$gradient)/abs(m1B@optinfo$derivs$gradient)
## bobyqa gets gradients *at least* 1.64* lower

lme4fit <- list(fixef=fixef(m1),theta=getME(m1,"theta"))

## hack around slight naming differences
lme4fit$theta <- unname(lme4fit$theta)
lme4.0fit$theta <- unname(lme4.0fit$theta)
## difference in theta on x86_64-w64-mingw32 (64-bit) with r-devel is  0.000469576
stopifnot(all.equal(lme4fit, lme4.0fit, tolerance = 5e-4))

## Fun stuff: visualize and alternative model

## library(ggplot2)
## dat$prop <- dat$y/dat$n
## theme_set(theme_bw())
## ggplot(dat,aes(x=gen,y=prop,colour=block))+geom_point(aes(size=n))+
##     geom_line(aes(group=block,colour=block))+
##     geom_smooth(family=binomial,aes(weight=n,colour=block,group=block),method="glm",
##                 alpha=0.1)

## dat$obs <- factor(seq(nrow(dat)))
## m2 <- glmer(cbind(y, n-y) ~ block+ (1|gen) + (1|obs), data=dat, family=binomial)

}  ## not on windows/CRAN

if (.Platform$OS.type!="windows") {
    library(lme4)

    ## ----------------------------------------------------------------------
    ## test that deviance(REMLfit, REML = FALSE) gives the same answer as
    ## the ML objective function at the REML fit
    ## ----------------------------------------------------------------------
    set.seed(1)
    w <- runif(nrow(sleepstudy))
    fm <- lmer(Reaction ~ Days + (Days | Subject),
               sleepstudy, weights = w)
    dfun <- update(fm, devFunOnly = TRUE, REML = FALSE)
    stopifnot(all.equal(deviance(fm, REML = FALSE),
                        dfun(getME(fm, "theta"))))

    ## ----------------------------------------------------------------------
    ## TODO: test the opposite case that deviance(MLfit, REML = TRUE)
    ## gives the same answer as the REML objective function at the ML fit
    ## ----------------------------------------------------------------------
}

library(lme4)

## raw residuals for LMMs
fm1 <- lmer(Reaction ~ Days + (Days|Subject), sleepstudy)
stopifnot(all.equal(residuals(fm1),sleepstudy$Reaction-fitted(fm1)))

r1 <- residuals(fm1,type="pearson")

## deviance/Pearson residuals for GLMMs
gm1 <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
             family = binomial, data = cbpp)
p <- fitted(gm1)
n <- cbpp$size
v <- n*p*(1-p)
obs_p <- cbpp$incidence/cbpp$size
rp <- residuals(gm1,"pearson")
rp1 <- (obs_p-p)/sqrt(p*(1-p))
rp2 <- rp1*n
## FIXME:: restore this test
## stopifnot(all.equal(rp,rp2))

r2 <- residuals(gm1,type="deviance")

profnames <- function(object, signames=TRUE,
                      useSc=isLMM(object), prefix=c("sd","cor")) {
  res <- if (signames) {
    sprintf(".sig%02d", seq_len(getParLength(object)))
  } else {
    vv <- getVCNames(object, prf=prefix, old=FALSE)
    ## result is stored as {vcomp, ccomp}; 'zip' it
    unlist(Map(c, vv$vcomp, vv$ccomp))
  }
  if (useSc) {
    sdname <- if (signames) ".sigma" else "sigma"
    res <- c(res, sdname)
  }
  res
}

##' @importFrom splines backSpline interpSpline periodicSpline
##' @importFrom stats profile
##' @method profile merMod
##' @export
profile.merMod <- function(fitted,
                           which=NULL,
                           alphamax = 0.01,
                           maxpts = 100,
                           delta = NULL,
                           delta.cutoff = 1/8,
                           verbose=0,
                           devtol=1e-9,
                           devmatchtol=1e-5,
                           maxmult = 10,
                           startmethod = "prev",
                           optimizer = NULL,
                           control = NULL,
                           signames = TRUE,
                           parallel = c("no", "multicore", "snow"),
                           ncpus = getOption("profile.ncpus", 1L),
                           cl = NULL,
                           prof.scale = c("sdcor","varcov"),
                           ...)
{

    ## FIXME: allow choice of nextstep/nextstart algorithm?
    ## FIXME: allow selection of individual variables to profile by name?
    ## FIXME: allow for failure of bounds (non-pos-definite correlation matrices) when >1 cor parameter

    prof.scale <- match.arg(prof.scale)

    parallel <- match.arg(parallel)

    # (parallel, ncpus, cl) -> (parallel)
    eval(initialize.parallel)

    if (is.null(optimizer)) optimizer <- fitted@optinfo$optimizer
    ## hack: doesn't work to set bobyqa parameters to *ending* values stored
    ## in @optinfo$control
    ignore.pars <- c("xst", "xt")
    control.internal <- fitted@optinfo$control
    if (length(ign <- which(names(control.internal) %in% ignore.pars)) > 0)
        control.internal <- control.internal[-ign]
    if (!is.null(control)) {
        i <- names(control)
        control.internal[[i]] <- control[[i]]
    }
    control <- control.internal
    useSc <- isLMM(fitted) || isNLMM(fitted)
    prof.prefix <-
        switch(prof.scale,
               "sdcor" = c("sd", "cor"),
               "varcov" = c("var", "cov"),
               stop("internal error, prof.scale=", prof.scale))
    dd <- devfun2(fitted, useSc=useSc, signames=signames,
                  prefix=prof.prefix, scale = prof.scale, ...)
    ## FIXME: figure out to what do here ...
    if (isGLMM(fitted) && fitted@devcomp$dims[["useSc"]])
        stop("can't (yet) profile GLMMs with non-fixed scale parameters")
    stopifnot(devtol >= 0)
    base <- attr(dd, "basedev")
    ## protect against accidental tampering by later devfun calls
    paropt <- forceCopy(attr(dd, "paropt"))
    stderr <- attr(dd, "stderr")
    pp <- environment(dd)$pp
    X.orig <- pp$X
    n <- environment(dd)$n
    p <- length(pp$beta0)

    opt <- attr(dd, "optimum")
    nptot <- length(opt)
    nvp <- nptot - p    # number of variance-covariance pars
    wi.vp <- seq_len(nvp)
    if(nvp > 0) fe.orig <- opt[- wi.vp]

    which <- get.which(which, nvp, nptot, names(opt), verbose)

    res <- c(.zeta = 0, opt)
    res <- matrix(res, nrow = maxpts, ncol = length(res),
                  dimnames = list(NULL, names(res)), byrow = TRUE)

    ## FIXME: why is cutoff based on nptot
    ## (i.e. boundary of simultaneous LRT conf region for nptot values)
    ##  when we are computing (at most) 2-parameter profiles here?
    cutoff <- sqrt(qchisq(1 - alphamax, nptot))

    if (is.null(delta))
        delta <- cutoff*delta.cutoff

    ## helper functions

    ## nextpar calculates the next value of the parameter being
    ## profiled based on the desired step in the profile zeta
    ## (absstep) and the values of zeta and column cc for rows
    ## r-1 and r.  The parameter may not be below lower (or above upper)
    nextpar <- function(mat, cc, r, absstep,
                        lower = -Inf, upper = Inf, minstep=1e-6) {
        rows <- r - (1:0)         # previous two row numbers
        pvals <- mat[rows, cc]
        zeta <- mat[rows, ".zeta"]
        num <- diff(pvals)
        if (is.na(denom <- diff(zeta)) || denom==0) {
            warning("Last two rows have identical or NA .zeta values: using minstep")
            step <- minstep
        } else {
            step <- absstep*num/denom
            if (step<0) {
                warning("unexpected decrease in profile: using minstep")
                step <- minstep
            } else {
                if (r>1) {
                    if (abs(step) > (maxstep <- abs(maxmult*num))) {
                        maxstep <- sign(step)*maxstep
                        if (verbose) cat(sprintf("capped step at %1.2f (multiplier=%1.2f > %1.2f)\n",
                                                 maxstep,abs(step/num),maxmult))
                        step <- maxstep
                    }
                }
            }
        }
        min(upper, max(lower, pvals[2] + sign(num) * step))
    }

    nextstart <- function(mat, pind, r, method) {
        ## FIXME: indexing may need to be checked (e.g. for fixed-effect parameters)
        switch(method,
               global= opt[seqpar1][-pind],  ## address opt, no zeta column
               prev  = mat[r,1+seqpar1][-pind],
               extrap = stop("not yet implemented"),## do something with mat[r-(1:0),1+seqnvp])[-pind]
               stop("invalid nextstart method"))
    }

    ## mkpar generates the parameter vector of theta and
    ## sigma from the values being profiled in position w
    mkpar <- function(np, w, pw, pmw) {
        par <- numeric(np)
        par[w] <- pw
        par[-w] <- pmw
        par
    }

    ## fillmat fills the third and subsequent rows of the matrix
    ## using nextpar and zeta
    ## FIXME:  add code to evaluate more rows near the minimum if that
    ##        constraint was active.
    fillmat <- function(mat, lowcut, upcut, zetafun, cc) {
        nr <- nrow(mat)
        i <- 2L
        ## REF: is.na(curzeta) behaviour last modified in commit b1a28914e72be845e
        while (i < nr && mat[i, cc] > lowcut && mat[i,cc] < upcut &&
               (i > 1 && !is.na(curzeta <- abs(mat[i, ".zeta"]))) && curzeta <= cutoff) {
                np <- nextpar(mat, cc, i, delta, lowcut, upcut)
                ns <- nextstart(mat, pind = cc-1, r=i, method=startmethod)
                mat[i + 1L, ] <- zetafun(np,ns)
                if (verbose>0) {
                    cat(i,cc,mat[i+1L,],"\n")
                }
                i <- i + 1L
            }
        if (mat[i-1,cc]==lowcut) {
            ## fill in more values near the minimum
        }
        if (mat[i-1,cc]==upcut) {
            ## fill in more values near the maximum
        }
        mat <- mat[!is.na(mat[,".zeta"]),]
        mat
    }

    ## bounds on Cholesky (== fitted@lower): [0,Inf) for diag, (-Inf,Inf) for off-diag
    ## bounds on sd-corr:  [0,Inf) for diag, (-1.0,1.0) for off-diag
  ## bounds on var-corr: [0,Inf) for diag, (-Inf,Inf) for off-diag
    covs <- getReCovs(fitted)
    lower <- unlist(lapply(covs, getProfLower, prof.scale))
    upper <- unlist(lapply(covs, getProfUpper, prof.scale))
    if (useSc) { # bounds for sigma
        lower <- c(lower,0)
        upper <- c(upper,Inf)
    }
    ## bounds on fixed parameters (TODO: allow user-specified bounds, e.g. for NLMMs)
    lower <- c(lower,rep.int(-Inf, p))
    upper <- c(upper, rep.int(Inf, p))
    npar1 <- if (isLMM(fitted)) nvp else nptot
    ## check that devfun2() computation for the base parameters is (approx.) the
    ##  same as the original devfun() computation
    basecheck <- all.equal(unname(dd(opt[seq(npar1)])), base, tolerance=devmatchtol)
    if(!isTRUE(basecheck)) {
        basediff <- abs(dd(opt[seq(npar1)])/base - 1)
        stop(sprintf(paste0("Profiling over both the residual variance and\n",
                            "fixed effects is not numerically consistent with\n",
                            "profiling over the fixed effects only (relative difference: %1.4g);\n",
                            "consider adjusting devmatchtol"),
                     basediff))
    }

    ## sequence of variance parameters to profile
    seqnvp <- intersect(seq_len(npar1), which)
    ## sequence of 'all' parameters
    seqpar1 <- seq_len(npar1)
    lowvp <- lower[seqpar1]
    upvp <- upper[seqpar1]
    form <- .zeta ~ foo # pattern for interpSpline formula

    ## as in bootMer.R, define FUN as a
    ##    closure containing the referenced variables
    ##    in its scope to avoid explicit clusterExport statement
    ##    in the PSOCKcluster case
    FUN <- local({
        function(w) {
        if (verbose) cat(if(isLMM(fitted)) "var-cov " else "",
                         "parameter ",w,":\n",sep="")
        wp1 <- w + 1L
        pw <- opt[w]
        lowcut <- lower[w]
        upcut <- upper[w]
        zeta <- function(xx,start) {
            ores <- tryCatch(optwrap(optimizer, par=start,
                                     fn=function(x) dd(mkpar(npar1, w, xx, x)),
                                     lower = lowvp[-w],
                                     upper = upvp [-w],
                                     control = control),
                             error=function(e) NULL)
            if (is.null(ores)) {
                devdiff <- NA
                pars <- NA
            } else {
                devdiff <- ores$fval - base
                pars <- ores$par
            }
            if (is.na(devdiff)) {
                warning("NAs detected in profiling")
            } else {
                if(verbose && devdiff < 0)
                    cat("old deviance ",base,",\n",
                        "new deviance ",ores$fval,",\n",
                        "new params ",
                        paste(mkpar(npar1,w,xx,ores$par),
                              collapse=","),"\n")
                
                if (devdiff < (-devtol))
                    stop("profiling detected new, lower deviance ",
                         sprintf("(deviance diff = %1.3g, tolerance = %1.3g)",
                                 abs(devdiff), devtol))
                             
                if(devdiff < 0)
                    warning(gettextf("slightly lower deviances (diff=%g) detected",
                                     devdiff), domain=NA)
            }
            devdiff <- max(0,devdiff)
            zz <- sign(xx - pw) * sqrt(devdiff)
            r <- c(zz, mkpar(npar1, w, xx, pars))
            if (isLMM(fitted)) c(r, pp$beta(1)) else r
        }## {zeta}

        ## intermediate storage for pos. and neg. increments
        pres <- nres <- res
        ## assign one row, determined by inc. sign, from a small shift
        ## FIXME:: do something if pw==0 ???
        shiftpar <- if (pw==0) 1e-3 else pw*1.01
        ## Since both the pos- and neg-increment matrices are already
        ## filled with the opt. par. results, this sets the first
        ## two rows of the positive-increment matrix
        ## to (opt. par, shiftpar) and the first two rows of
        ## the negative-increment matrix to (shiftpar, opt. par),
        ## which sets up two points going in the right direction
        ## for each matrix (since the profiling algorithm uses increments
        ## between rows to choose the next parameter increment)
        nres[1, ] <- pres[2, ] <- zeta(shiftpar, start=opt[seqpar1][-w])
        ## fill in the rest of the arrays and collapse them
        upperf <- fillmat(pres, lowcut, upcut, zeta, wp1)
        lowerf <-
            if (pw > lowcut)
                fillmat(nres, lowcut, upcut, zeta, wp1)
            else ## don't bother to fill in 'nres' matrix
                nres
        ## this will throw away the extra 'opt. par' and 'shiftpar'
        ## rows introduced above:
        bres <- as.data.frame(unique(rbind2(upperf,lowerf)))
        pname <- names(opt)[w]
        bres$.par <- pname
        bres <- bres[order(bres[, wp1]), ]

        ## FIXME: test for bad things here??
        form[[3]] <- as.name(pname)
        forspl <- NULL # (in case of error)
        ## bakspl
        bakspl <-
            tryCatch(backSpline(
                forspl <- interpSpline(form, bres, na.action=na.omit)),
                     error=function(e)e)
        if (inherits(bakspl, "error"))
            warning("non-monotonic profile for ",pname)
        ## return:
        namedList(bres,bakspl,forspl) # namedList() -> lmerControl.R

    }}) ## FUN()

    ## copied from bootMer: DRY!
    L <- if (parallel == "multicore") {
        parallel::mclapply(seqnvp, FUN, mc.cores = ncpus)
    } else if (parallel == "snow") {
        if (is.null(cl)) {
            cl <- parallel::makeCluster(ncpus)
            on.exit(parallel::stopCluster(cl))
            ## explicit export of the lme4 namespace since most FUNs will probably
            ## use some of them
            parallel::clusterExport(cl, varlist=getNamespaceExports("lme4"))
            if(RNGkind()[1L] == "L'Ecuyer-CMRG")
                parallel::clusterSetRNGStream(cl)
        }
        parallel::parLapply(cl, seqnvp, FUN)
    } else lapply(seqnvp, FUN)
    nn <- names(opt[seqnvp])
    ans <-    setNames(lapply(L, `[[`,  "bres"), nn)
    bakspl <- setNames(lapply(L, `[[`,"bakspl"), nn)
    forspl <- setNames(lapply(L, `[[`,"forspl"), nn)

    ## profile fixed effects separately (for LMMs)
    if (isLMM(fitted)) {
        reCovs <- getReCovs(fitted)
        mkPar <- mkMkPar(reCovs)
        mkTheta <- mkMkTheta(reCovs)
        offset.orig <- fitted@resp$offset
        fp <- seq_len(p)
        fp <- fp[(fp+nvp) %in% which]
        ## FIXME: parallelize this too ...
        for (j in fp) {
            if (verbose) cat("fixed-effect parameter ",j,":\n",sep="")
            pres <-            # intermediate results for pos. incr.
                nres <- res    # and negative increments
            est <- opt[nvp + j]
            std <- stderr[j]
            Xw <- X.orig[, j, drop=TRUE]
            Xdrop <- .modelMatrixDrop(X.orig, j)
            pp1 <- new(class(pp),
                       X = Xdrop,
                       Zt = pp$Zt,
                       Lambdat = pp$Lambdat,
                       Lind = pp$Lind,
                       theta = pp$theta,
                       n = nrow(Xdrop))
### FIXME Change this to use the deep copy and setWeights, setOffset, etc.
            rr <- new(Class=class(fitted@resp), y=fitted@resp$y)
            rr$setWeights(fitted@resp$weights)
            fe.zeta <- function(fw, start) {
                ## (start parameter ignored)
                rr$setOffset(Xw * fw + offset.orig)
                rho <- list2env(list(pp=pp1, resp=rr,
                                     mkPar=mkPar, mkTheta=mkTheta),
                                parent = parent.frame())
                ## optimization on `par` scale (not `theta`)
                ores <- optwrap(optimizer, par = paropt, fn = mkdevfun(rho, 0L),
                                ## FIXME: do we need to extend this for GLMMs?
                                lower = fitted@lower)
                ## this optimization is done on the ORIGINAL
                ##   `par` scale (i.e. not the sigma/corr scale)
                fv <- ores$fval
                sig <- sqrt((rr$wrss() + pp1$sqrL(1))/n)
                ## translate from ores$par (`par` scale) back to `profPar` scale
                ppars <- convParToProfPar(ores$par, fitted, profscale = prof.scale, sc = sig)
                c(sign(fw - est) * sqrt(fv - base),
                  ppars,
                  mkpar(p, j, fw, pp1$beta(1)))
            }
            nres[1, ] <- pres[2, ] <- fe.zeta(est + delta * std)
            poff <- nvp + 1L + j
            ## Workaround R bug [rbind2() is S4 generic; cannot catch warnings in its arg]
            ## see lme4 GH issue #304
            upperf <- fillmat(pres, -Inf, Inf, fe.zeta, poff)
            lowerf <- fillmat(nres, -Inf, Inf, fe.zeta, poff)
            bres <- as.data.frame(unique(rbind2(upperf, lowerf)))
            bres$.par <- n.j <- names(fe.orig)[j]
            ans[[n.j]] <- bres[order(bres[, poff]), ]
            form[[3]] <- as.name(n.j)
            bakspl[[n.j]] <-
                tryCatch(backSpline(forspl[[n.j]] <- interpSpline(form, bres)),
                         error=function(e)e)
            if (inherits(bakspl[[n.j]], "error"))
                warning("non-monotonic profile for ", n.j)
        } ## for(j in 1..p)
    } ## if isLMM

    ans <- do.call(rbind, ans)
    row.names(ans) <- NULL ## TODO: rbind(*, make.row.names=FALSE)
    ans$.par <- factor(ans$.par)
    structure(ans,
              forward = forspl,
              backward = bakspl,
              lower = lower[seqnvp],
              upper = upper[seqnvp],
              class = c("thpr", "data.frame"))# 'thpr': see ../man/profile-methods.Rd
} ## profile.merMod

##' Transform 'which' \in {parnames | integer | "beta_" | "theta_"}
##' into integer indices
##' @param which numeric or character vector
##' @param nvp number of variance-covariance parameters
##' @param nptot total number of parameters
##' @param parnames vector of parameter names
##' @param verbose print messages?
##' @examples
##' fm1 <- lmer(Reaction ~ Days + (Days | Subject), sleepstudy)
##' tn <- names(getME(fm1,"theta"))
##' nn <- c(tn,names(fixef(fm1)))
##' get.which("theta_",length(tn),length(nn),nn, verbose=TRUE)
##'
get.which <- function(which, nvp, nptot, parnames, verbose=FALSE) {
    if (is.null(which))
        seq_len(nptot)
    else if (is.character(which)) {
        wi <- integer(); wh <- which
        if(any(j <- wh == "theta_")) {
            wi <- seq_len(nvp); wh <- wh[!j] }
        if(any(j <- wh == "beta_") && nptot > nvp) {
            wi <- c(wi, seq(nvp+1, nptot)); wh <- wh[!j] }
        if(any(j <- parnames %in% wh)) { ## which containing param.names
            wi <- sort(unique(c(wi, seq_len(nptot)[j])))
        }
        if(verbose) message(gettextf("From original which = %s: new which <- %s",
                                     deparse(which, nlines=1), deparse(wi, nlines=1)),
                            domain=NA)
        if(length(wi) == 0)
            warning(gettextf("Nothing selected by 'which=%s'", deparse(which)),
                    domain=NA)
        wi
    } else #  stopifnot( .. numeric ..)
        which
}

## This is a hack.  The preferred approach is to write a
## subset method for the ddenseModelMatrix and dsparseModelMatrix
## classes
.modelMatrixDrop <- function(mm, w) {
    if (isS4(mm)) {
        nX <- slotNames(X <- mm[, -w, drop = FALSE])
        do.call(new,
                c(list(Class = class(mm),
                       assign = attr(mm,"assign")[-w],
                       contrasts = NULL
                       ## FIXME: where did the contrasts information go??
                       ##      mm@contrasts
                       ),
                  lapply(structure(nX, .Names=nX),
                         function(nm) slot(X, nm))))
    } else {
        structure(mm[, -w, drop=FALSE],
                  assign = attr(mm, "assign")[-w])
    }
}

## The deviance is profiled with respect to the fixed-effects
## parameters but not with respect to sigma. The other parameters
## are on the standard deviation scale, not the theta scale.
##
## @title Return a function for evaluation of the deviance.
## @param fm a fitted model of class merMod
## @param useSc (logical) whether a scale parameter is used
## @param \dots arguments passed to profnames (\code{signames=TRUE}
## to use old-style .sigxx names, FALSE uses (sd_cor|xx);
## also \code{prefix=c("sd","cor")})
## @return a function for evaluating the deviance in the extended
##     parameterization.  This is profiled with respect to the
##     variance-covariance parameters (fixed-effects done separately).
devfun2 <- function(fm,
                    useSc = if(isLMM(fm)) TRUE else NA,
                    scale = c("sdcor", "varcov"),
                    ...)
{

    scale <- match.arg(scale)
    if (scale == "varcov" && anyStructured(fm))
        stop("haven't thought about varcov scale for structured cov matrices")
    ## TODO: change to work with 'par' instead of 'theta'

    ## FIXME: have to distinguish between
    ## 'useSc' (GLMM: report profiled scale parameter) and
    ## 'useSc' (NLMM/LMM: scale theta by sigma)
    ## hasSc := GLMMuseSc <- fm@devcomp$dims[["useSc"]]
    stopifnot(is(fm, "merMod"))
    fm <- refitML(fm)
    mkTheta <- mkMkTheta(getReCovs(fm))
    basedev <- -2*c(logLik(fm))  ## no longer deviance()
    vlist <- lengths(fm@cnms)
    ## "has scale" := isLMM  or  GLMM with scale parameter
    hasSc <- as.logical(fm@devcomp$dims[["useSc"]])
    stdErr <- unname(coef(summary(fm))[,2])
    pp <- fm@pp$copy()
    sig <- if (useSc) sigma(fm) else NULL
    opt <- getProfPar(fm, profscale = scale, sc = sig)
    names(opt) <- profnames(fm, useSc=useSc, ...)
    opt <- c(opt, fixef(fm))
    resp <- fm@resp$copy()
    np <- length(pp$theta)
    nf <- length(fixef(fm))
    if(hasSc) np <- np + 1L # was  if(!isGLMM(fm)) np <- np + 1L
    n <- nrow(pp$V) # use V, not X so it works with nlmer
    if (isLMM(fm)) { # ==> hasSc
        ans <- function(pars)
        {
          pp <- fm@pp$copy() ## fresh copy of predictor object
          ## convert 'profPar' -> 'pars'
          sig <- pars[length(pars)]
          thpars <- try(
              convProfParToPar(pars, fm, profscale = scale, sc = sig),
              silent = TRUE)
          if (inherits(thpars, "try-error")) return(NA)
          .Call(lmer_Deviance, pp$ptr(), resp$ptr(), mkTheta(thpars))
          sigsq <- if (scale=="varcov") sig else sig^2
          pp$ldL2() - ldW + (resp$wrss() + pp$sqrL(1))/sigsq + n * log(2 * pi * sigsq)
        }
        ldW <- sum(log(environment(ans)$resp$weights))
        assign("ldW", ldW, envir = environment(ans))
    } else { # GLMM *and* NLMMs
        d0 <- getME(fm, "devfun")
        ## from glmer:
        ## rho <- new.env(parent=parent.env(environment()))
        ## rho$pp <- do.call(merPredD$new, c(reTrms[c("Zt","theta","Lambdat","Lind")], n=nrow(X), list(X=X)))
        ## rho$resp <- mkRespMod(fr, if(REML) p else 0L)
        ans <- function(pars)
        {
            stopifnot(is.numeric(pars), length(pars) == np+nf)
            thpars <- try(
                convProfParToPar(pars, fm, profscale = scale, sc = NULL),
                silent = TRUE)
            if (inherits(thpars, "try-error")) return(NA)
            fixpars <- pars[-seq(np)]
            d0(c(thpars, fixpars))
        }
    }
    attr(ans, "optimum") <- opt         # w/ names()
    attr(ans, "basedev") <- basedev
    ## FIXME: should make this GLMM-aware
    attr(ans, "paropt") <- getPar(fm)
    attr(ans, "stderr") <- stdErr
    class(ans) <- "devfun"
    ans
}

## extract only the y component from a prediction
predy <- function(sp, vv) {
    if(inherits(sp,"error")) rep(NA_real_, length(vv))
    else predict(sp, vv)$y
}

stripExpr <- function(ll, nms) {
    stopifnot(is.list(ll), is.character(nms))
    fLevs <- as.expression(nms) # variable names; now replacing log(sigma[.]) etc:
    fLevs[nms ==  ".sigma"] <- expression(sigma)
    fLevs[nms == ".lsigma"] <- expression(log(sigma))
    fLevs[nms == ".sigmasq"] <- expression(sigma^2)
    sigNms  <- grep("^\\.sig[0-9]+", nms)
    lsigNms <- grep("^\\.lsig[0-9]+", nms)
    sig2Nms <- grep("^\\.sigsq[0-9]+", nms)
    ## the <n> in ".sig0<n>" and then in ".lsig0<n>":
    sigsub  <- as.integer(substring(nms[ sigNms], 5))
    lsigsub <- as.integer(substring(nms[lsigNms], 6))
    sig2sub <- as.integer(substring(nms[sig2Nms], 7))
    fLevs[ sigNms] <- lapply( sigsub, function(i) bquote(    sigma[.(i)]))
    fLevs[lsigNms] <- lapply(lsigsub, function(i) bquote(log(sigma[.(i)])))
    fLevs[sig2Nms] <- lapply(sig2sub, function(i) bquote(   {sigma[.(i)]}^2))
    ## result of using { .. }^2  is easier to understand    ==          ==
    levsExpr <- substitute(strip.custom(factor.levels=foo), list(foo=fLevs))
    snames <- c("strip", "strip.left")
    if(!any(.in <- snames %in% names(ll))) {
        ll$strip <- levsExpr
    } else {
        for(nm in snames[.in])
            if(is.logical(v <- ll[[nm]]) && v)
                ll[[nm]] <- levsExpr
    }
    ll
}

panel.thpr <- function(x, y, spl, absVal, ...)
{
    panel.grid(h = -1, v = -1)
    myspl <- spl[[panel.number()]]
    lsegments(x, y, x, 0, ...)
    if (absVal) {
        y[y == 0] <- NA
        lsegments(x, y, rev(x), y)
    } else {
        panel.abline(h = 0, ...)
    }
    if (!is(myspl,"spline")) {
        ## 'difficult' data
        if (absVal) myspl$y <- abs(myspl$y)
        panel.lines (myspl$x, myspl$y)
        panel.points(myspl$x, myspl$y, pch="+")
        warning(gettextf("bad profile for variable %d: using linear interpolation",
                         panel.number()), domain=NA)
    } else {
        lims <- current.panel.limits()$xlim
        krange <- range(myspl$knots)
        pr <- predict(myspl,
                      seq(max(lims[1], krange[1]),
                          min(lims[2], krange[2]), len = 101))
        if (absVal) pr$y <- abs(pr$y)
        panel.lines(pr$x, pr$y)
    }
}

## A lattice-based plot method for profile objects
##' @importFrom lattice xyplot
##' @S3method xyplot thpr
xyplot.thpr <-
    function (x, data = NULL,
              levels = sqrt(qchisq(pmax.int(0, pmin.int(1, conf)), df = 1)),
              conf = c(50, 80, 90, 95, 99)/100,
              absVal = FALSE, scales = NULL,
              which = 1:nptot, ...)
{
    if(any(!is.finite(conf) | conf <= 0 | conf >= 1))
        stop("values of 'conf' must be strictly between 0 and 1")
    stopifnot(1 <= (nptot <- length(nms <- levels(x[[".par"]]))))
    ## FIXME: is this sufficiently reliable?
    ## (include "sigma" in 'theta' parameters)
    nvp <- length(grep("^(\\.sig[0-9]+|.sigma|sd_|cor_)", nms))
    which <- get.which(which, nvp, nptot, nms)
    levels <- sort(levels[is.finite(levels) & levels > 0])
    spl  <- attr(x, "forward") [which]
    bspl <- attr(x, "backward")[which]
    ## for parameters for which spline couldn't be computed,
    ## replace the 'spl' element with the raw profile data
    if(any(badSpl <- vapply(spl, is.null, NA))) {
        spl[badSpl] <- lapply(which(badSpl), function(i) {
            n <- names(badSpl)[i]
            r <- x[x[[".par"]] == n, ]
            data.frame(y = r[[".zeta"]], x = r[[n]])
        })
        bspl[badSpl] <- lapply(spl[badSpl], function(d) data.frame(x=d$y,y=d$x))
        ## FIXME: more efficient, not yet ok ?
        ## ibad <- which(badSpl)
        ## spl[ibad] <- lapply(names(ibad), function(n) {
        ##     r <- x[x[[".par"]]==n,]
        ##     data.frame(y = r[[".zeta"]], x = r[[n]])
        ## })
        ## bspl[ibad] <- lapply(spl[ibad], function(d) data.frame(x=d$y,y=d$x))
    }
    zeta <- c(-rev(levels), 0, levels)
    mypred <- function(bs, zeta) { ## use linear approximation if backspline doesn't work
        if (inherits(bs,"spline"))
            predy(bs, zeta)
        else if(is.numeric(x <- bs$x) && is.numeric(y <- bs$y) && length(x) == length(y))
            approx(x, y, xout = zeta)$y
        else
            rep_len(NA, length(zeta))
    }
    fr <- data.frame(zeta = rep.int(zeta, length(spl)),
                     pval = unlist(lapply(bspl, mypred, zeta)),
                     pnm = gl(length(spl), length(zeta), labels = names(spl)))
    if (length(ind <- which(is.na(fr$pval)))) {
        fr[ind, "zeta"] <- 0
        for (i in ind)
### FIXME: Should check which bound has been violated, although it
### will almost always be the minimum.
            if (inherits(curspl <-  spl[[fr[i, "pnm"] ]], "spline")) {
                fr[i, "pval"] <- min(curspl$knots)
            }
    }
    ylab <- if (absVal) {
        fr$zeta <- abs(fr$zeta)
        expression("|" * zeta * "|")
    } else
        expression(zeta)
    intscales <- list(x = list(relation = 'free'))
    ## FIXME: is there something less clunky we can do here
    ##   that allows for all possible user inputs
    ##   (may want to (1) override x$relation (2) add elements to $x
    ##    (3) add elements to scales)
    if (!is.null(scales)) {
        if (!is.null(scales[["x"]])) {
            if (!is.null(scales[["x"]]$relation)) {
                intscales[["x"]]$relation <- scales[["x"]]$relation
                scales[["x"]]$relation <- NULL
            }
            intscales[["x"]] <- c(intscales[["x"]],scales[["x"]])
            scales[["x"]] <- NULL
        }
        intscales <- c(intscales,scales)
    }
    ll <- c(list(...),
            list(x = zeta ~ pval | pnm, data=fr,
                 scales = intscales,
                 ylab = ylab, xlab = NULL, panel=panel.thpr,
                 spl = spl, absVal = absVal))

    do.call(xyplot, stripExpr(ll, names(spl)))
}

## copy of stats:::format.perc (not exported, and ":::" being forbidden nowadays):
format.perc <- function (x, digits, ...) {
    paste(format(100 * x, trim = TRUE,
                 scientific = FALSE, digits = digits),
          "%")
}

##' confint() method for  our profile() results 'thpr'
##' @importFrom stats confint
confint.thpr <- function(object, parm, level = 0.95, zeta,
                         ## tolerance for non-monotonic profiles
                         ## (raw values, not splines)
                         non.mono.tol=1e-2,
                         ...)
{
    bak <- attr(object, "backward")
    ## fallback strategy for old profiles that don't have a lower/upper
    ##  attribute saved ...
    if (is.null(lower <- attr(object,"lower")))
        lower <- rep(NA,length(parm))
    if (is.null(upper <- attr(object,"upper")))
        upper <- rep(NA,length(parm))
    ## FIXME: work a little harder to add -Inf/Inf for fixed effect
    ##  parameters?  (Should only matter for really messed-up profiles)
    bnms <- names(bak)
    parm <- if (missing(parm))
                bnms
            else if(is.numeric(parm)) # e.g., when called from confint.merMod()
                bnms[parm]
            else if (length(parm <- as.character(parm)) == 1) {
                if (parm == "theta_")
                    grep("^(sd_|cor_|.sig|sigma$)", bnms, value=TRUE)
                else if (parm == "beta_")
                    grep("^(sd_|cor_|.sig|sigma$)", bnms, value=TRUE, invert=TRUE)
                else if(parm %in% bnms) # just that one
                    parm
                ## else NULL : will return 0-row matrix
            } else
                intersect(parm, bnms)
    cn <-
        if (missing(zeta)) {
            a <- (1 - level)/2
            a <- c(a, 1 - a)
            zeta <- qnorm(a)
            format.perc(a, 3)
        } ## else NULL
    ci <- matrix(NA_real_, nrow=length(parm), ncol=2L, dimnames = list(parm,cn))
    for (i in seq_along(parm)) {
        ## would like to build this machinery into predy, but
        ## predy is used in many places and it's much harder to
        ## tell in general whether an NA indicates a lower or an
        ## upper bound ...
        badprof <- FALSE
        p <- rep(NA,2)
        if (!inherits(b <- bak[[parm[i]]], "error")) {
            p <- predy(b, zeta)
        } else {
            obj1 <- object[object$.par==parm[[i]],c(parm[[i]],".zeta")]
            if (all(is.na(obj1[,2]))) {
                badprof <- TRUE
                warning("bad profile for ",parm[i])
            } else if (min(diff(obj1[,2])<(-non.mono.tol),na.rm=TRUE)) {
                badprof <- TRUE
                warning("non-monotonic profile for ",parm[i])
            } else {
                warning("bad spline fit for ",parm[i],": falling back to linear interpolation")
                p <- approxfun(obj1[,2],obj1[,1])(zeta)
            }
        }
        if (!badprof) {
            if (is.na(p[1])) p[1] <- lower[i]
            if (is.na(p[2])) p[2] <- upper[i]
        }
        ci[i,] <- p
    }
    ci
}

## FIXME: make bootMer more robust; make profiling more robust;
## more warnings; documentation ...

##' Compute confidence intervals on the parameters of an lme4 fit
##' @param object a fitted [ng]lmer model
##' @param parm parameters (specified by integer position)
##' @param level confidence level
##' @param method for computing confidence intervals
##' @param zeta likelihood cutoff
##' (if not specified, computed from \code{level}: "profile" only)
##' @param nsim number of simulations for parametric bootstrap intervals
##' @param boot.type bootstrap confidence interval type
##' @param quiet (logical) suppress messages about computationally intensive profiling?
##' @param signames (logical) use old-style names for \code{method="profile"}? (See \code{signames} argument to \code{\link{profile}}
##' @param oldNames (logical) deprecated; has the same purpose as the \code{signames} argument.
##' @param \dots additional parameters to be passed to  \code{\link{profile.merMod}} or \code{\link{bootMer}}
##' @return a numeric table of confidence intervals
confint.merMod <- function(object, parm, level = 0.95,
                           method = c("profile", "Wald", "boot"),
                           zeta, nsim=500, boot.type = c("perc", "basic", "norm"),
                           FUN = NULL, quiet=FALSE, oldNames, signames = TRUE,
                           boot.scale = c("sdcor", "vcov"), ...)
{
    method <- tolower(match.arg(method))
    boot.type <- match.arg(boot.type)
    boot.scale <- match.arg(boot.scale)
    if (!missing(oldNames)) {
      warning("'oldNames' is deprecated. Please use 'signames' instead.", call. = FALSE)
      signames <- oldNames
    }
    ## 'parm' corresponds to 'which' in other contexts
    if (method=="boot" && !is.null(FUN)) {
        ## custom boot function, don't expand parameter names
    } else {
        ## "use scale" = GLMM with scale parameter *or* LMM ..
        useSc <- as.logical(object@devcomp$dims[["useSc"]])
        vn <- profnames(object, signames, useSc=useSc)
        an <- c(vn, names(fixef(object)))
        parm <- if(missing(parm)) seq_along(an)
                else
                    get.which(parm, nvp=length(vn), nptot=length(an), parnames=an)
        if (!quiet && method %in% c("profile","boot")) {
            mtype <- switch(method, profile="profile", boot="bootstrap")
            message("Computing ",mtype," confidence intervals ...")
            flush.console()
        }
    }
    switch(method,
           "profile" = {
               pp <- profile(object, which=parm, signames=signames, ...)
               ## confint.thpr() with missing(parm) using all names:
               confint(pp, level=level, zeta=zeta)
           },
           "wald" = {
               a <- (1 - level)/2
               a <- c(a, 1 - a)
               ci.vcov <- array(NA,dim = c(length(vn), 2L),
                                dimnames = list(vn, format.perc(a,3)))
               ## copied with small changes from confint.default
               cf <- fixef(object)   ## coef() -> fixef()
               pnames <- names(cf)
               ## N.B. can't use sqrt(...)[parm] (diag() loses names)
               ses <- sqrt(diag(vcov(object)))
               ci.fixed <- array(cf + ses %o% qnorm(a),
                                 dim = c(length(pnames), 2L),
                                 dimnames = list(pnames, format.perc(a, 3)))
               ci.all <- rbind(ci.vcov,ci.fixed)
               ci.all[parm,,drop=FALSE]
           },
           "boot" = {
             bootFun <- function(x) {
               ## need to get profile-scale pars from fitted model
               useSc <- !(isGLMM(x) && hasNoScale(family(x)))
               sc <- if (useSc) sigma(x) else NULL
               ss <- convParToProfPar(x@optinfo$val, x, profscale = boot.scale, sc = sc)
               c(setNames(ss, vn), fixef(x))
             }
             if (is.null(FUN)) FUN <- bootFun
             bb <- bootMer(object, FUN=FUN, nsim=nsim,...)
             if (all(is.na(bb$t))) stop("*all* bootstrap runs failed!")
             print.bootWarnings(bb, verbose=FALSE)
             citab <- confint(bb,level=level,type=boot.type)
             if (missing(parm)) {
               ## only happens if we have custom boot method
               if (is.null(parm <- rownames(citab))) {
                 parm <- seq(nrow(citab))
               }
             }
             citab[parm, , drop=FALSE]
           },
           ## should never get here ...
           stop("unknown confidence interval method"))
}

##' Convert x-cosine and y-cosine to average and difference.
##'
##' Convert the x-cosine and the y-cosine to an average and difference
##' ensuring that the difference is positive by flipping signs if
##' necessary
##' @param xc x-cosine
##' @param yc y-cosine
ad <- function(xc, yc)
{
    a <- (xc + yc)/2
    d <- (xc - yc)
    cbind(sign(d)* a, abs(d))
}

##' convert d versus a (as an xyVector) and level to a matrix of taui and tauj
##' @param xy an xyVector
##' @param lev the level of the contour
tauij <- function(xy, lev) lev * cos(xy$x + outer(xy$y/2, c(-1, 1)))

##' @title safe arc-cosine
##' @param x numeric vector argument
##' @return acos(x) being careful of boundary conditions
sacos <- function(x) acos(pmax.int(-0.999, pmin.int(0.999, x)))

##' Generate a contour
##'
##' @title Generate a contour
##' @param sij the arc-cosines of i on j
##' @param sji the arc-cosines of j on i
##' @param levels numeric vector of levels at which to interpolate
##' @param nseg number of segments in the interpolated contour
##' @return a list with components
##' \item{tki}{the tau-scale predictions of i on j at the contour levels}
##' \item{tkj}{the tau-scale predictions of j on i at the contour levels}
##' \item{pts}{an array of dimension (length(levels), nseg, 2) containing the points on the contours}
cont <- function(sij, sji, levels, nseg = 101)
{
    ada <- array(0, c(length(levels), 2L, 4L))
    ada[, , 1] <- ad(0,  sacos(predy(sij,  levels)/levels))
    ada[, , 2] <- ad(    sacos(predy(sji,  levels)/levels), 0)
    ada[, , 3] <- ad(pi, sacos(predy(sij, -levels)/levels))
    ada[, , 4] <- ad(sacos(predy(sji, -levels)/levels), pi)
    pts <- array(0, c(length(levels), nseg + 1, 2))
    for (i in seq_along(levels))
        pts[i, ,] <- tauij(predict(periodicSpline(ada[i, 1, ], ada[i, 2, ]),
                                   nseg = nseg), levels[i])
    levs <- c(-rev(levels), 0, levels)
    list(tki = predict(sij, levs), tkj = predict(sji, levs), pts = pts)
}

## copied from lattice:::chooseFace
chooseFace <- function (fontface = NULL, font = 1)
{
    if (is.null(fontface))
        font
    else fontface
}


##' Draws profile pairs plots.  Contours are for the marginal
##' two-dimensional regions (i.e. using df = 2).
##'
##' @title Profile pairs plot
##' @param x the result of \code{\link{profile}} (or very similar structure)
##' @param data unused - only for compatibility with generic
##' @param levels the contour levels to be shown; usually derived from \code{conf}
##' @param conf numeric vector of confidence levels to be shown as contours
##' @param ... further arguments passed to \code{\link{splom}}
##' @importFrom grid gpar viewport
##' @importFrom lattice splom
##' @method splom thpr
##' @export
splom.thpr <- function (x, data,
                        levels = sqrt(qchisq(pmax.int(0, pmin.int(1, conf)), 2)),
                        conf = c(50, 80, 90, 95, 99)/100, which = 1:nptot,
                        draw.lower = TRUE, draw.upper = TRUE, ...)
{
    x <- na.omit(x)
    stopifnot(1 <= (nptot <- length(nms <- names(attr(x, "forward")))))
    singfit <- FALSE
    for (i in grep("^(\\.sig[0-9]+|sd_)", names(x)))
        singfit <- singfit || any(x[,".zeta"] == 0  &  x[,i] == 0)
    if (singfit) warning("splom is unreliable for singular fits")

    nvp <- length(grep("^(\\.sig[0-9]+|.sigma|sd_|cor_)", nms))
    which <- get.which(which, nvp, nptot, nms)
    if (length(which) == 1)
        stop("can't draw a scatterplot matrix for a single variable")

    mlev <- max(levels)
    spl <- attr(x, "forward")[which]
    frange <- sapply(spl, function(x) range(x$knots))
    bsp <- attr(x, "backward")[which]
    x <- x[x[[".par"]] %in% nms[which],c(".zeta",nms[which],".par")]
    ## brange <- sapply(bsp, function(x) range(x$knots))
    pfr <- do.call(cbind, lapply(bsp, predy, c(-mlev, mlev)))
    pfr[1, ] <- pmax.int(pfr[1, ], frange[1, ], na.rm = TRUE)
    pfr[2, ] <- pmin.int(pfr[2, ], frange[2, ], na.rm = TRUE)
    nms <- names(spl)
    ## Create data frame fr of par. vals in zeta coordinates
    fr <- x[, -1]
    for (nm in nms) fr[[nm]] <- predy(spl[[nm]], na.omit(fr[[nm]]))
    fr1 <- fr[1, nms]
    ## create a list of lists with the names of the parameters
    traces <- lapply(fr1, function(el) lapply(fr1, function(el1) list()))
    .par <- NULL  ## =>  no R CMD check warning
    for (j in seq_along(nms)[-1]) {
        frj <- subset(fr, .par == nms[j])
        for (i in seq_len(j - 1L)) {
            fri <- subset(fr, .par == nms[i])
            sij <- interpSpline(fri[ , i], fri[ , j])
            sji <- interpSpline(frj[ , j], frj[ , i])
            ll <- cont(sij, sji, levels)
            traces[[j]][[i]] <- list(sij = sij, sji = sji, ll = ll)
        }
    }
    if(draw.lower) ## panel function for lower triangle
    lp <- function(x, y, groups, subscripts, i, j, ...) {
        tr <- traces[[j]][[i]]
        grid::pushViewport(viewport(xscale = c(-1.07, 1.07) * mlev,
                                    yscale = c(-1.07, 1.07) * mlev))
        dd <- sapply(current.panel.limits(), diff)/50
        psij <- predict(tr$sij)
        ll <- tr$ll
        ## now do the actual plotting
        panel.grid(h = -1, v = -1)
        llines(psij$y, psij$x, ...)
        llines(predict(tr$sji), ...)
        with(ll$tki, lsegments(y - dd[1], x, y + dd[1], x, ...))
        with(ll$tkj, lsegments(x, y - dd[2], x, y + dd[2], ...))
        for (k in seq_along(levels)) llines(ll$pts[k, , ], ...)
        grid::popViewport(1)
    }
    if(draw.upper) ## panel function for upper triangle
    up <- function(x, y, groups, subscripts, i, j, ...) {
        ## panels are transposed so reverse i and j
        jj <- i
        ii <- j
        tr <- traces[[jj]][[ii]]
        ll <- tr$ll
        pts <- ll$pts
        ## limits <- current.panel.limits()
        psij <- predict(tr$sij)
        psji <- predict(tr$sji)
        ## do the actual plotting
        panel.grid(h = -1, v = -1)
        llines(predy(bsp[[ii]], psij$x), predy(bsp[[jj]], psij$y), ...)
        llines(predy(bsp[[ii]], psji$y), predy(bsp[[jj]], psji$x), ...)
        for (k in seq_along(levels))
            llines(predy(bsp[[ii]], pts[k, , 2]),
                   predy(bsp[[jj]], pts[k, , 1]), ...)
    }
    dp <- function(x = NULL,            # diagonal panel
                   varname = NULL, limits, at = NULL, lab = NULL,
                   draw = TRUE,

                   varname.col = add.text$col,
                   varname.cex = add.text$cex,
                   varname.lineheight = add.text$lineheight,
                   varname.font = add.text$font,
                   varname.fontfamily = add.text$fontfamily,
                   varname.fontface = add.text$fontface,

                   axis.text.col = axis.text$col,
                   axis.text.alpha = axis.text$alpha,
                   axis.text.cex = axis.text$cex,
                   axis.text.font = axis.text$font,
                   axis.text.fontfamily = axis.text$fontfamily,
                   axis.text.fontface = axis.text$fontface,

                   axis.line.col = axis.line$col,
                   axis.line.alpha = axis.line$alpha,
                   axis.line.lty = axis.line$lty,
                   axis.line.lwd = axis.line$lwd,
                   i, j,
                   ...)
    {
        n.var <- eval.parent(expression(n.var))
        add.text <- trellis.par.get("add.text")
        axis.line <- trellis.par.get("axis.line")
        axis.text <- trellis.par.get("axis.text")
        if (!is.null(varname))
            grid::grid.text(varname,
                            gp =
                                gpar(col = varname.col,
                                     cex = varname.cex,
                                     lineheight = varname.lineheight,
                                     fontface = chooseFace(varname.fontface, varname.font),
                                     fontfamily = varname.fontfamily))
        if (draw)
        {
            at <- pretty(limits)
            sides <- c("left", "top")
            if (j == 1) sides <- "top"
            if (j == n.var) sides <- "left"
            for (side in sides)
                panel.axis(side = side,
                           at = at,
                           labels = format(at, trim = TRUE),
                           ticks = TRUE,
                           check.overlap = TRUE,
                           half = side == "top" && j > 1,

                           tck = 1, rot = 0,

                           text.col = axis.text.col,
                           text.alpha = axis.text.alpha,
                           text.cex = axis.text.cex,
                           text.font = axis.text.font,
                           text.fontfamily = axis.text.fontfamily,
                           text.fontface = axis.text.fontface,

                           line.col = axis.line.col,
                           line.alpha = axis.line.alpha,
                           line.lty = axis.line.lty,
                           line.lwd = axis.line.lwd)
            lims <- c(-1.07, 1.07) * mlev
            grid::pushViewport(viewport(xscale = lims, yscale = lims))
            side <- if(j == 1) "right" else "bottom"
            which.half <- if(j == 1) "lower" else "upper"
            at <- pretty(lims)
            panel.axis(side = side, at = at, labels = format(at, trim = TRUE),
                       ticks = TRUE, half = TRUE, which.half = which.half,
                       tck = 1, rot = 0,

                       text.col = axis.text.col,
                       text.alpha = axis.text.alpha,
                       text.cex = axis.text.cex,
                       text.font = axis.text.font,
                       text.fontfamily = axis.text.fontfamily,
                       text.fontface = axis.text.fontface,

                       line.col = axis.line.col,
                       line.alpha = axis.line.alpha,
                       line.lty = axis.line.lty,
                       line.lwd = axis.line.lwd)
            grid::popViewport(1)
        }
    }

    panel.blank <- function(...) {}
    splom(~ pfr,
          lower.panel = if(draw.lower) lp else panel.blank,
          upper.panel = if(draw.upper) up else panel.blank,
          diag.panel = dp, ...)
}

## return an lmer profile like x with all the .sigNN parameters
## replaced by .lsigNN.  The forward and backward splines for
## these parameters are recalculated.  -> ../man/profile-methods.Rd
logProf <- function (x, base = exp(1), ranef=TRUE,
                     sigIni = if(ranef) "sig" else "sigma")
{
    stopifnot(inherits(x, "thpr"))
    cn <- colnames(x)
    sigP <- paste0("^\\.", sigIni)
    if (length(sigs <- grep(sigP, cn))) {
        repP <- sub("sig", ".lsig", sigIni)
        colnames(x) <- cn <- sub(sigP, repP, cn)
        levels(x[[".par"]]) <- sub(sigP, repP, levels(x[[".par"]]))
        names(attr(x, "backward")) <-
            names(attr(x, "forward")) <-
                sub(sigP, repP, names(attr(x, "forward")))
        for (nm in cn[sigs]) {
            x[[nm]] <- log(x[[nm]], base = base)
            fr <- x[x[[".par"]] == nm & is.finite(x[[nm]]), TRUE, drop=FALSE]
            form <- eval(substitute(.zeta ~ nm, list(nm = as.name(nm))))
            attr(x,  "forward")[[nm]] <- isp <- interpSpline(form, fr)
            attr(x, "backward")[[nm]] <- backSpline(isp)
        }
        ## eliminate rows that produced non-finite logs
        x <- x[apply(is.finite(as.matrix(x[, sigs])), 1, all), , drop=FALSE]
    }
    x
}
## the log() method must have (x, base); no other arguments
log.thpr <- function (x, base = exp(1)) logProf(x, base=base)




##' Create an approximating density from a profile object  -- called only from densityplot.thpr()
##'
##' @title Approximate densities from profiles
##' @param pr a profile object
##' @param npts number of points at which to evaluate the density
##' @param upper upper bound on cumulative for a cutoff
##' @return a data frame
dens <- function(pr, npts=201, upper=0.999) {
    npts <- as.integer(npts)
    spl  <- attr(pr, "forward")
    bspl <- attr(pr, "backward")
    stopifnot(inherits(pr, "thpr"), npts > 0,
              is.numeric(upper), 0.5 < upper, upper < 1,
              identical((vNms <- names(spl)), names(bspl)))
    bad_vars <- character(0)
    zeta <- c(-1,1) * qnorm(upper)
    rng <- vector(mode="list", length=length(bspl))
    names(rng) <- vNms
    dd <- rng # empty named list to be filled
    for (i in seq_along(bspl)) {
        if (inherits(bspl[[i]], "error")) {
            rng[[i]] <- rep(NA,npts)
            bad_vars <- c(bad_vars, vNms[i])
            next
        }
        rng0 <- predy(bspl[[i]], zeta)
        if (is.na(rng0[1])) rng0[1] <- 0
        if (is.na(rng0[2])) {
            ## try harder to pick an upper bound
            for(upp in 1 - 10^seq(-4,-1, length=21)) { # <<-- TODO: should be related to 'upper'
                if(!is.na(rng0[2L] <- predy(bspl[[i]], qnorm(upp))))
                    break
            }
            if (is.na(rng0[2])) {
                warning("can't find an upper bound for the profile")
                bad_vars <- c(bad_vars, vNms[i])
                next
            }
        }
        rng[[i]] <- seq(rng0[1], rng0[2], len=npts)
    }
    fr <- data.frame(pval = unlist(rng),
                     pnm = gl(length(rng), npts, labels=vNms))
    for (nm in vNms) {
        dd[[nm]] <-
            if (!inherits(spl[[nm]], "spline")) {
                rep(NA_real_, npts)
            } else {
                zz <- predy(spl[[nm]], rng[[nm]])
                dnorm(zz) * predict(spl[[nm]], rng[[nm]], deriv = 1L)$y
            }
    }
    fr$density <- unlist(dd)
    if (length(bad_vars))
        warning("unreliable profiles for some variables skipped: ",
                paste(bad_vars, collapse=", "))
    fr
}

##' Densityplot method for a mixed-effects model profile
##
##' @title densityplot from a mixed-effects profile
##' @param x a mixed-effects profile
##' @param data not used - for compatibility with generic
##' @param ... optional arguments to \code{\link[lattice]{densityplot}()}
##' from package \pkg{lattice}.
##' @return a density plot
##' @examples ## see   example("profile.merMod")
##' @importFrom lattice densityplot
##' @method densityplot thpr
##' @export
densityplot.thpr <- function(x, data, npts=201, upper=0.999, ...) {
    dd <- dens(x, npts=npts, upper=upper)
    ll <- c(list(...),
            list(x=density ~ pval|pnm,
                 data=dd,
                 type=c("l","g"),
                 scales=list(relation="free"),
                 xlab=NULL))
    do.call(xyplot, stripExpr(ll, names(attr(x, "forward"))))
}

##' Transform a mixed-effects profile to the variance scale
varianceProf <- function(x, ranef=TRUE) {
    ## "parallel" to logProf()
    stopifnot(inherits(x, "thpr"))
    cn <- colnames(x)
    if(length(sigs <- grep(paste0("^\\.", if(ranef)"sig" else "sigma"), cn))) {
        ## s/sigma/sigmasq/ ;  s/sig01/sig01sq/  etc
        sigP <- paste0("^(\\.sig", if(ranef) "(ma)?" else "ma", ")")
        repP <- "\\1sq"
        colnames(x) <- cn <- sub(sigP, repP, cn)
        levels(x[[".par"]]) <- sub(sigP, repP, levels(x[[".par"]]))
        names(attr(x, "backward")) <-
            names(attr(x, "forward")) <-
                sub(sigP, repP, names(attr(x, "forward")))
        for (nm in cn[sigs]) {
            x[[nm]] <- x[[nm]]^2
            ## select rows (and currently drop extra attributes)
            fr <- x[x[[".par"]] == nm, TRUE, drop=FALSE]
            form <- eval(substitute(.zeta ~ nm, list(nm = as.name(nm))))
            attr(x, "forward")[[nm]] <- isp <- interpSpline(form, fr)
            attr(x, "backward")[[nm]] <- backSpline(isp)
        }
    }
    x
}

## convert profile to data frame, adding a .focal parameter to simplify lattice/ggplot plotting
##' @method as.data.frame thpr
##' @param x the result of \code{\link{profile}} (or very similar structure)
##' @export
##' @rdname profile-methods
as.data.frame.thpr <- function(x,...) {
    class(x) <- "data.frame"
    m <- as.matrix(x[,seq(ncol(x))-1]) ## omit .par
    x.p <- x[[".par"]]
    x[[".focal"]] <- m[cbind(seq(nrow(x)),match(x.p,names(x)))]
    x[[".par"]] <- factor(x.p, levels=unique(as.character(x.p))) ## restore order
    x
}

library("lme4")

m1 <- glmer(incidence/size ~ period + (1|herd), weights = size, cbpp, family = binomial)
c1 <- confint(m1, method = "boot", nsim = 10)
c2 <- confint(m1, method = "boot", nsim = 10, parallel = "multicore")

cbpp2 <- read.csv("cbpp2.csv")
source("glmer_batch_funs.R")

cbpp2 <- transform(
  cbpp2,
  period = factor(period),
  treatment = factor(
    treatment,
    levels = c("Partial/null", "Complete", "Unknown")
  ),
  obs = factor(seq(nrow(cbpp2))),
  avg_size = avg_size / sd(avg_size, na.rm = TRUE)
)

cbpp_df_name <- data.frame(
  mnames = c("herd", "herdobs", "obs"),
  mdesc = c("herd", "herdobs", "obs"))

mforms <- list()
mforms$herd <- incidence / size ~ period + treatment + avg_size + (1 | herd)
mforms$herdobs <- update(mforms$herd, . ~ . + (1 | obs))
mforms$obs <- update(mforms$herd, . ~ . - (1|herd) + (1 | obs))

## don't use lapply because of parent.frame() eval nonsense -- check back once GH#961 is merged?
## this fails:
##   mod_list <- lapply(mforms, glmer, data = cbpp2, family = binomial, weights = size)
## when evaluating 'weights'

cc0 <- glmerControl(optimizer = "bobyqa",
                    ## avoid instability/problems with nAGQ=0 step ...
                    nAGQ0initStep = FALSE,
                    optCtrl = list(maxfun = 1e3),
                    calc.derivs = FALSE)
cc1 <- cc0
cc1$nAGQ0initStep <- TRUE

## sapply(cbpp_mod_list, \(x) -c(logLik(x)))
## sapply(cbpp_mod_list2, \(x) -c(logLik(x)))

cbpp_mod_list <- cbpp_mod_list2 <- list()
for (i in names(mforms)) {
  cbpp_mod_list[[i]] <- glmer(mforms[[i]],
                              data = cbpp2,
                              family = binomial,
                              weights = size,
                              control = cc0),
  cbpp_mod_list2[[i]] <- glmer(mforms[[i]],
                              data = cbpp2,
                              family = binomial,
                              weights = size,
                              control = cc1)

}

if (lme4:::testLevel() > 1 || .Platform$OS.type!="windows") {
    ## example posted by Stéphane Laurent
    ## exercises bug where Nelder-Mead min objective function value was >0
    set.seed(666)
    sims <- function(I, J, sigmab0, sigmaw0){
        Mu <- rnorm(I, mean=0, sd=sigmab0)
        y <- c(sapply(Mu, function(mu) rnorm(J, mu, sigmaw0)))
        data.frame(y=y, group=gl(I,J))
    }

    I <- 3  # number of groups
    J <- 8  # number of repeats per group
    sigmab0 <- 0.15  # between standard deviation
    sigmaw0 <- 0.15  # within standard deviation

    dat <- sims(I, J, sigmab0, sigmaw0)

    library(lme4)
    isOldTol <- environment(nloptwrap)$defaultControl$xtol_abs == 1e-6

    fm3 <- lmer(y ~ (1|group), data=dat)
    stopifnot(all.equal(unname(unlist(VarCorr(fm3))),
                        switch(fm3@optinfo$optimizer,
                               "Nelder_Mead" = 0.029662844,
                               "bobyqa"      = 0.029662698,
                               "nloptwrap"   =
                                   if (isOldTol) 0.029679755 else 0.029662699,
                               stop("need new case here: value is ",unname(unlist(VarCorr(fm3))))
                               ),
                        tolerance = 1e-7))
} ## skip on windows (for speed)

.onLoad <- function(libname, pkgname) {
  ## don't do this in production; also flags problems in downstream packages
  ## options(Matrix.warnDeprecatedCoerce = 3)
  options(lme4.summary.cor.max = 12)

  Ons <- parent.env(environment()) # our own namespace
  if((Rv <- getRversion()) < "4.4.0") {
    assign('%||%', envir = Ons, inherits = FALSE,
           function(x, y) if(is.null(x)) y else x)
    if(Rv < "4.1.0") {
      ## https://stackoverflow.com/questions/49056642/how-to-make-variable-available-to-namespace-at-loading-time/67664852#67664852
      ## not quite equivalent; this *forces* ... entries whereas true ...names()  doesn't
      assign('...names', envir = Ons,
             function() eval(quote(names(list(...))), sys.frame(-1L)))
      if(Rv < "4.0.0") {
        ## NB: R >= 4.0.0's deparse1() is a generalization of our previous safeDeparse()
        assign('deparse1', envir = Ons,
               function (expr, collapse = " ", width.cutoff = 500L, ...)
                   paste(deparse(expr, width.cutoff, ...), collapse = collapse)
               )
        ## not equivalent ...
        assign('...length', envir = Ons,
               function() eval(quote(length(list(...))), sys.frame(-1L))
               )
        assign("sequence.default", envir = Ons,
               function(nvec, from = 1L, by = 1L, ...) {
                   if(length(nvec) == 0L)
                       return(integer(0L))
                   else if(length(from) == 0L || length(by) == 0L)
                       stop(gettextf("'%s' has length 0 but '%s' does not",
                                     if(length(from) == 0L) "from" else "by", "nvec"),
                            domain = NA)
                   unlist(.mapply(seq.int,
                                  list(from = as.integer(from),
                                       by = as.integer(by),
                                       length.out = as.integer(nvec)),
                                  NULL),
                          recursive = FALSE, use.names = FALSE)
               })
        if (Rv < "3.6.0") {
          assign('reformulate', envir = Ons,
                 function(..., env = parent.env) {
                     f <- stats::reformulate(...)
                     environment(f) <- env
                     f
                 })
        if (Rv < "3.2.1") {
        assign('lengths', envir = Ons,
               function (x, use.names = TRUE) vapply(x, length, 1L, USE.NAMES = use.names)
               )
        if(Rv < "3.1.0") {
          assign('anyNA', envir = Ons, function(x) any(is.na(x)) )
          if(Rv < "3.0.0") {
            assign('rep_len', envir = Ons, function(x, length.out) rep(x, length.out=length.out) )
            if(Rv < "2.15") {
              assign('paste0', envir = Ons, function(...) paste(..., sep = '') )
            } ## R < 2.15
          } ## R < 3.0.0
        } ## R < 3.1.0
        } ## R < 3.2.1
      } ## R < 3.6.0
      } ## R < 4.0.0
    } ## R < 4.1.0
  } ## R < 4.4.0
  rm(Rv, Ons)
  ## check Matrix ABI version
  check_dep_version()
}

## https://github.com/lme4/lme4/issues/768
## https://github.com/kaskr/adcomp/issues/387
get_abi_version <- function() {
    if (utils::packageVersion("Matrix") < "1.6-2") return(numeric_version("0"))
    Matrix::Matrix.Version()[["abi"]]
}

.Matrix.abi.build.version <- get_abi_version()

## simplified version of glmmTMB package checking
##' @param this_pkg downstream package being tested
##' @param dep_pkg upstream package on which \code{this_pkg} depends
##' @param dep_type "ABI" or "package"
##' @param built_version a \code{numeric_version} object indicating what version of \code{dep_pkg} was used to  build \code{this_pkg}
##' @param warn (logical) warn if condition not met?
##' @noRd
check_dep_version <- function(this_pkg = "lme4",  dep_pkg = "Matrix", dep_type = "ABI",
                              built_version = .Matrix.abi.build.version,
                              warn = TRUE) {
    cur_version <- get_abi_version()
    result_ok <- cur_version == built_version
    if (!result_ok) {
        warning(
            sprintf("%s version mismatch: \n", dep_type),
            sprintf("%s was built with %s %s version %s\n",
                    this_pkg, dep_pkg, dep_type, built_version),
            sprintf("Current %s %s version is %s\n",
                    dep_pkg, dep_type, cur_version),
            sprintf("Please re-install %s from source ", this_pkg),
            "or restore original ",
            sQuote(dep_pkg), " package"
        )
    }
    return(result_ok)
}

.onUnload <- function(libpath) {
  gc()
  if (is.loaded("lmer_Deviance", PACKAGE="lme4")) {
    library.dynam.unload("lme4", libpath)
  }
}

namedList <- function(...) {
    L <- list(...)
    snm <- sapply(substitute(list(...)), deparse)[-1]
    if (is.null(nm <- names(L))) nm <- snm
    if (any(nonames <- nm == "")) nm[nonames] <- snm[nonames]
    setNames(L,nm)
}

##' By extracting "checking options" from \code{nms}, this function
##' implicitly defines what "checking options" are.
##'
##' @title Extract "checking options" or "checking arguments" (-> ../man/lmerControl.Rd)
##' those starting with "check." but *not* the "check.conv.." nor "check.rankX.." ones:
##' @param nms character vector
##' @return those elements of \code{nms} which are "checking options"
##' @author Martin Maechler
.get.checkingOpts <- function(nms)
    nms[grepl("^check\\.(?!conv|rankX|scaleX)", nms, perl=TRUE)]


##' Check check.conv.*() options and produce good error message
chk.convOpt <- function(opt) {
    cnm <- deparse(nm <- substitute(opt))[[1]]
    if(!is.list(opt)) stop("check.conv* option ", cnm, " must be a list")
    if(!is.character(opt$action))
        stop("check.conv* option ", cnm, " has no 'action' string")
    if(!is.numeric(tol <- opt$tol))
        stop("check.conv* option ", cnm, " must have a numeric 'tol' component")
    if(length(tol) != 1 || tol < 0)
        stop("check.conv* option ", cnm, "$tol must be number >= 0")
    if(!is.null(relTol <- opt$relTol))
        stopifnot(is.numeric(relTol), length(relTol) == 1, relTol >= 0)
    invisible()
}

##' Exported constructor for the user calling  *lmerControl():
.makeCC <- function(action, tol, relTol, ...) {
    stopifnot(is.character(action), length(action) == 1)
    if(!is.numeric(tol))
        stop("must have a numeric 'tol' component")
    if(length(tol) != 1 || tol < 0)
        stop("'tol' must be number >= 0")
    mis.rt <- missing(relTol)
    if(!mis.rt && !is.null(relTol))
        stopifnot(is.numeric(relTol), length(relTol) == 1, relTol >= 0)
    ## and return the list,  the  "..." just being appended unchecked
    c(list(action = action, tol = tol),
      if(!mis.rt) list(relTol = relTol),
      list(...))
}

##' Internal utility :  Allow check.conv.* to be a string
chk.cconv <- function(copt, callingFn) {
    cnm <- deparse(substitute(copt))
    if(is.character(copt)) {
        def <- eval(formals(callingFn)[[cnm]])
        def$action <- copt
        assign(cnm, def, envir=sys.frame(sys.parent()))
    } else chk.convOpt(copt)
}

## work around code check since we adjust formals() later
check.response.not.const <- compDev <- nAGQ0initStep <- tolPwrss <- NULL

merControl <-
    function(optimizer="nloptwrap", # originally Nelder_Mead, then bobyqa ...
             ## glmer: c("bobyqa","Nelder_Mead")
             restart_edge=TRUE,  ## glmer: FALSE
             ## don't call any of these arguments "check.*" -- will fail
             ## automatic check-option-checking in
             ## inst/tests/test-lmer.R
             boundary.tol=1e-5,
             calc.derivs=NULL,
             use.last.params=FALSE,
             sparseX=FALSE,
             standardize.X=FALSE,
             autoscale=NULL,
             ## input checking options:
             check.nobs.vs.rankZ="ignore", ## "warningSmall",
             check.nobs.vs.nlev="stop",
             check.nlev.gtreq.5="ignore",
             check.nlev.gtr.1="stop",
             check.nobs.vs.nRE="stop",
             check.rankX = c("message+drop.cols",
                             "silent.drop.cols", "warn+drop.cols",
                             "stop.deficient", "ignore"),
             check.scaleX = c("warning","stop","silent.rescale",
                              "message+rescale","warn+rescale","ignore"),
             check.formula.LHS = "stop",
             ## convergence options
             check.conv.nobsmax = 1e4,
             check.conv.nparmax = 10,
             check.conv.grad     = .makeCC("warning", tol = 2e-3, relTol = NULL),
             check.conv.singular = .makeCC(action = "message", tol = getSingTol()),
             check.conv.hess     = .makeCC(action = "warning", tol = 1e-6),
             optCtrl = list(),
             mod.type="lmer"   ## glmer: "glmer"
             ## glmer: tolPwrss=1e-7
             ##        compDev = TRUE
             ##        nAGQ0initStep = TRUE
             ##        check.response.not.const="stop"
             ) {
        ## FIXME: is there a better idiom?  match.call() ?
        ## fill in values from options, but **only if not specified explicitly in arguments**
        ##  (ugh ... is there a better way to do this?  mapply() is clunky:
        ##  http://stackoverflow.com/questions/16276667/using-apply-with-assign-in-r

        stopifnot(is.list(optCtrl))
        if (mod.type=="glmer" && length(optimizer)==1) {
            ## replicate() works whether 'optimizer' is a function or a string
            optimizer <- replicate(2,optimizer)
        }
        c.opts <- paste0(mod.type,"Control")
        merOpts <- getOption(c.opts)
        if (!is.null(merOpts)) {
            nn <- names(merOpts)
            nn.ok <- .get.checkingOpts(names(merOpts))
            if (length(nn.ignored <- setdiff(nn,nn.ok))>0) {
                warning("some options in ",shQuote(sprintf("getOption('%s')",c.opts)),
                        " ignored : ",paste(nn.ignored,collapse=", "))
            }
            for (arg in nn.ok) {
                if (do.call(missing,list(arg))) ## only if missing from explicit arguments
                    assign(arg,merOpts[[arg]])
            }
        }
        check.rankX <- match.arg(check.rankX)
        check.scaleX <- match.arg(check.scaleX)
        ## compatibility and convenience, caller can specify action string only:
        me <- sys.function()
        chk.cconv(check.conv.grad,     me)
        chk.cconv(check.conv.singular, me)
        chk.cconv(check.conv.hess    , me)

        if (mod.type=="glmer" && use.last.params && calc.derivs) {
            warning("using ",shQuote("use.last.params"),"=TRUE and ",
                    shQuote("calc.derivs"),"=TRUE with ",shQuote("glmer"),
                    " will not give backward-compatible results")
        }

        ret <- namedList(optimizer,
                         restart_edge,
                         boundary.tol,
                         calc.derivs,
                         use.last.params,
                         checkControl =
                             namedList(autoscale,
                                       check.nobs.vs.rankZ,
                                       check.nobs.vs.nlev,
                                       check.nlev.gtreq.5,
                                       check.nlev.gtr.1,
                                       check.nobs.vs.nRE,
                                       check.rankX,
                                       check.scaleX,
                                       check.formula.LHS),
                         checkConv=
                             namedList(check.conv.nobsmax,
                                       check.conv.nparmax,
                                       check.conv.grad,
                                       check.conv.singular,
                                       check.conv.hess),
                         optCtrl=optCtrl)
        if (mod.type=="glmer") {
            ret <- c(ret, namedList(tolPwrss,
                                    compDev,
                                    nAGQ0initStep))
            ret$checkControl <- c(ret$checkControl,
                                 namedList(check.response.not.const))
        }
        class(ret) <- c(c.opts, "merControl")
        ret
    }

lmerControl <- merControl
glmerControl <- merControl
formals(glmerControl)[["check.conv.nparmax"]] <- 20
formals(glmerControl)[["optimizer"]] <- c("bobyqa","Nelder_Mead")
formals(glmerControl)[["mod.type"]] <- "glmer"
formals(glmerControl)[["restart_edge"]] <- FALSE
formals(glmerControl) <- c(formals(glmerControl),
                           list(tolPwrss=1e-7,
                                compDev = TRUE,
                                nAGQ0initStep = TRUE,
                                check.response.not.const="stop")
                           )

##' @rdname lmerControl
##' @export
nlmerControl <- function(optimizer="Nelder_Mead",
                         tolPwrss = 1e-10,
                         optCtrl = list())
{
    stopifnot(is.list(optCtrl))
    if (length(optimizer)==1) {
        optimizer <- replicate(2,optimizer)
    }
    structure(namedList(optimizer,
                        tolPwrss,
                        optCtrl=optCtrl),
              class = c("nlmerControl", "merControl"))
}

testLevel <- if (nzchar(s <- Sys.getenv("LME4_TEST_LEVEL"))) as.numeric(s) else 1

#context("lower/upper bounds on nlmer models")
test_that("nlmer", {
    startvec <- c(Asym = 200, xmid = 725, scal = 350)
    nm1 <- nlmer(circumference ~ SSlogis(age, Asym, xmid, scal) ~ Asym|Tree,
                 Orange, start = startvec,
                 control=nlmerControl(optCtrl=list(lower=c(0,200,-Inf,-Inf))))
    expect_equal(unname(fixef(nm1)[1]),200)
})

if (.Platform$OS.type != "windows") withAutoprint({
    library("lme4")

    ## example from HSAUR2 package; data from 'multcomp'; see ../inst/testdata/trees513.R
    load(system.file("testdata","trees513.RData",package="lme4"))

    ## model formula:
    modForm <- damage ~ species - 1 + (1 | lattice / plot)

    dfun <- glmer(modForm, data = trees513B, family = binomial,
                  devFunOnly = TRUE)
    ls.str(environment(dfun))# "for your information"

    .not.call <- function(x) x[names(x) != "call"]

    if(lme4:::testLevel() < 2) q("no")
    ## else  (testLevel >= 2) : --------------------------------------------------

    ## Generate oldres:
    ## ----------------
    ## library(lme4.0)
    ## system.time(mmod0 <- glmer(damage ~ species - 1 + (1 | lattice / plot),
    ##               data = trees513, family = binomial()))
    ## ## 4 seconds
    ## oldres <- c(fixef(mmod0),getME(mmod0,"theta"))
    ## detach("package:lme4.0")
    ## dput(oldres)
    oldres <- structure(c(5.23645064474105, 4.73568475545248, 2.65289926317093,
                          1.29043984816924, 1.59329381563025,
                          0.532663142106669, 1.16703186884403
                          ), names = c("speciesspruce", "speciespine",
                                        "speciesbeech",
                                        "speciesoak", "specieshardwood",
                                        "plot:lattice.(Intercept)",
                                        "lattice.(Intercept)"))
    system.time(mmodA <- glmer(modForm, data = trees513A, family = binomial()))
    ## 7 seconds
    newres <- c(fixef(mmodA), getME(mmodA,"theta"))
    stopifnot(all.equal(oldres, newres, tolerance=1.5e-3))
    system.time(mmodB <- glmer(modForm, data = trees513B, family = binomial()))
    ## 10.4 seconds
    ##
    if(FALSE) { ## defuncted in 2019-05 [been deprecated since 2013-06]
        ## lmer( + family) -> diverts to glmer() with a warning [TODO: use assertWarning(.) eventually]
        system.time(lmodB <-
                        lmer(modForm, data = trees513B, family = binomial()))
        stopifnot(all.equal(.not.call(summary(mmodB)),
                            .not.call(summary(lmodB))))
        newresB <- c(fixef(mmodB),getME(mmodB,"theta"))
        stopifnot(length(newresB) == length(oldres) + 1)# extra: species[ash/maple/elm/lime]
    }


}) ## skip on windows (for speed)

library(lme4)

allEQ <- function(x,y, tolerance = 4e-4, ...)
    all.equal.numeric(x,y, tolerance=tolerance, ...)

(nm1 <- nlmer(circumference ~ SSlogis(age, Asym, xmid, scal) ~ (Asym|Tree),
              Orange, start = c(Asym = 200, xmid = 725, scal = 350)))
fixef(nm1)

if (lme4:::testLevel() > 2) {
    ## 'Theoph' Data modeling
    Th.start <- c(lKe = -2.5, lKa = 0.5, lCl = -3)

system.time(nm2 <- nlmer(conc ~ SSfol(Dose, Time,lKe, lKa, lCl) ~
                         (lKe+lKa+lCl|Subject),
                         Theoph, start = Th.start,
                         control=nlmerControl(tolPwrss=1e-8)))
print(nm2, corr=FALSE)

system.time(nm3 <- nlmer(conc ~ SSfol(Dose, Time,lKe, lKa, lCl) ~
                         (lKe|Subject) + (lKa|Subject) + (lCl|Subject),
                         Theoph, start = Th.start))
print(nm3, corr=FALSE)

## dropping   lKe  from random effects:
system.time(nm4 <- nlmer(conc ~ SSfol(Dose, Time,lKe, lKa, lCl) ~ (lKa+lCl|Subject),
                         Theoph, start = Th.start,
                         control=nlmerControl(tolPwrss=1e-8)))
print(nm4, corr=FALSE)

system.time(nm5 <- nlmer(conc ~ SSfol(Dose, Time,lKe, lKa, lCl) ~
                         (lKa|Subject) + (lCl|Subject),
                         Theoph,
                         start = Th.start,
                         control=nlmerControl(tolPwrss=1e-8)))
    print(nm5, corr=FALSE)

    ## this has not worked in a *long* time anyway, and PKPDmodels is currently archived, so ...
    
    ## if (require("PKPDmodels")) {
    ##     oral1cptSdlkalVlCl <-
    ##         PKmod("oral", "sd", list(ka ~ exp(lka), Cl ~ exp(lCl), V ~ exp(lV)))
    ##     if (FALSE) {
    ##         ## FIXME: Error in get(nm, envir = nlenv) : object 'k' not found
    ##         ##  probably with environments/call stack etc.?
    ##         ## 'pnames' is c("lV","lka","k") -- not ("lV","lka","lCl")
    ##         ##  nlmer -> nlformula -> MkRespMod
    ##         ##  pnames are OK in nlformula, but in MkRespMod we try to recover
    ##         ##   them from the column names of the gradient attribute of the
    ##         ##   model evaluated in nlenv -- which are wrong.
    ##         system.time(nm2a <- nlmer(conc ~ oral1cptSdlkalVlCl(Dose, Time, lV, lka, lCl) ~
    ##                                   (lV+lka+lCl|Subject),
    ##                                   Theoph, start = c(lV=-1, lka=-0.5, lCl=-3), tolPwrss=1e-8))
    ##         print(nm2a, corr=FALSE)
    ##     }
    ## }
    
}  ## testLevel > 2

library(lme4)
n <- nrow(sleepstudy)
op <- options(warn = 1, # show as they happen ("false" convergence warnings)
              useFancyQuotes = FALSE)

if (.Platform$OS.type != "windows") {
##' remove all attributes but names
dropA <- function(x) `attributes<-`(x, list(names = names(x)))
##' transform result of "numeric" all.equal.list() to a named vector
all.eqL <- function(x1, x2, ...) {
    r <- sub("^Component ", '', all.equal(x1, x2, tolerance = 0, ...))
    r <- strsplit(sub(": Mean relative difference:", "&&", r),
                  split="&&", fixed=TRUE)
    setNames(as.numeric(vapply(r, `[`, "1.234", 2L)),
             ## drop surrounding "..."
             nm = sub('"$', '', substring(vapply(r, `[`, "nam",   1L), first=2)))
}
seedF <- function(s) {
    if(s %in% c(6, 39, 52, 57, 63, 74, 76, 86))
        switch(as.character(s)
               , "52"=, "63"=, "74" = 2
               , "6"=, "39" = 3
               , "86" =  8 # needs  4 on Lnx-64b
               , "76" = 70 # needs 42 on Lnx-64b
               , "57" = 90 # needs 52 on Lnx-64b
               )
    else if(s %in% c(1, 12, 15, 34, 36, 41, 42, 43, 49, 55, 59, 67, 80, 85)) ## seeds 41,59, .. 15
        1.0
    else ## seeds 22, 20, and better
        0.25
}
## be fast, running only 10 seeds by default:
sMax <- if(lme4:::testLevel() > 1) 99L else 9L
mySeeds <- 0L:sMax

lapply(setNames(,mySeeds), function(seed) {
    cat("\n------ random seed =", seed, "---------\n")
    set.seed(seed)
    v <- rpois(n,1) + 1
    w <- 1/v
    cat("weights w:\n")
    fm1    <- lmer(Reaction ~ Days + (Days | Subject), sleepstudy, REML=FALSE, weights = w); cat("..2:\n")
    fm2    <- lmer(Reaction ~ Days + (1    | Subject), sleepstudy, REML=FALSE, weights = w)
    cat("weights w*10:\n")
    fm1.10 <- lmer(Reaction ~ Days + (Days | Subject), sleepstudy, REML=FALSE, weights = w*10);cat("..2:\n")
    fm2.10 <- lmer(Reaction ~ Days + (1    | Subject), sleepstudy, REML=FALSE, weights = w*10)
    ##
    ano12... <- dropA(anova(fm1,    fm2   ))
    ano12.10 <- dropA(anova(fm1.10, fm2.10))
    print(aEQ <- all.eqL(ano12..., ano12.10)) # showing differences
    if(!exists("notChisq"))
	notChisq <<-
	    local({ n <- names(ano12...)
		grep("Chisq", n, value=TRUE, fixed=TRUE, invert=TRUE) })
    stopifnot(
        all.equal(ano12...$Chisq,
                  ano12.10$Chisq, tol = 1e-6 * seedF(seed))
       ,
        all.equal(ano12...[notChisq],
                  ano12.10[notChisq], tol= 1.5e-8 * seedF(seed))
    )
    aEQ
}) -> rallEQ

cat("=====================================\n")

rallEQ <- t(simplify2array(rallEQ))
notChisq <- intersect(notChisq, colnames(rallEQ))
## sort according to "severity":
srallEQ <- rallEQ[with(as.data.frame(rallEQ), order(AIC, Chisq)), ]
round(log10(srallEQ), 2)
saveRDS(srallEQ, "priorWeightsMod_relerr.rds")

if(!dev.interactive(orNone=TRUE)) pdf("priorWeightsMod_relerr.pdf")

matplot(mySeeds, log10(srallEQ), type="l", xlab=NA) ; grid()
legend("topleft", ncol=3, bty="n",
       paste(1:6, colnames(srallEQ), sep = ": "), col=1:6, lty=1:6)
tolD <- sqrt(.Machine$double.eps) # sqrt(eps_C)
abline(h = log10(tolD), col = "forest green", lty=3)
axis(4, at=log10(tolD), label=quote(sqrt(epsilon[c])), las=1)
LRG <- which(srallEQ[,"AIC"] > tolD)
if (length(LRG)>0) {
    text(LRG, log10(srallEQ[LRG, "AIC"]), names(LRG), cex = .8)
}

## how close are we ..
str(tF <- sapply(mySeeds, seedF))
round(sort(      rallEQ[, "Chisq"] / (tF * 1e-6  ),          decreasing=TRUE), 1)
round(sort(apply(rallEQ[,notChisq] / (tF * 1.5e-8), 1, max), decreasing=TRUE), 1)
} ## skip on windows (for speed)
options(op)

##' test for no-random-effect specification: TRUE if NA or ~0, other
##' possibilities are NULL or a non-trivial formula
isRE <- function(re.form) {
    isForm <- inherits(re.form, "formula")
    (is.null(re.form) || isForm || !is.na(re.form)) &&
    (!isForm || length(re.form) != 2L || !identical(re.form[[2L]], 0))
}

if(FALSE) {
isRE(NA)   ## FALSE
isRE(~0)   ##  "
isRE(~y+x) ## TRUE
isRE(NULL) ##  "
isRE(~0+x) ##  "
}

## for now, copied from reformulas:
##  'specials'/'default.special' need to be passed through,
##  can't be easily hacked
safe_length <- function(x) length(unclass(x))

my_reOnly <- function(f, response=FALSE, bracket=TRUE, doublevert_split = TRUE, specials=character(0), default.special = NULL) {
    ee <- environment(f)
    flen <- safe_length(f)
    f2 <- f[[2]]
    if (bracket) {
        xdv <- if (doublevert_split) "split" else "diag_special"
        fb <- findbars_x(f, expand_doublevert_method = xdv,
                         specials = specials, default.special = default.special)
        f <- lapply(fb, reformulas::makeOp, quote(`(`)) ## bracket-protect terms
    }
    f <- reformulas::sumTerms(f)
    if (response && flen==3) {
        form <- reformulas::makeOp(f2, f, quote(`~`))
    } else {
        form <- reformulas::makeOp(f, quote(`~`))
    }
    ## form may be a 'language' object by now ...
    form <- as.formula(form)
    environment(form) <- ee
    return(form)
}
  
##' Random Effects formula only
##' wrapper for reformulas (next version should handle environment protection properly ...
##'
lme4_reOnly <- function(form, ...) {
  my_reOnly(form, ..., specials = lme4_specials, doublevert_split = (getDoublevertDefault()=="split"))
}

## '...' may contain fixed.only=TRUE, random.only=TRUE, ..
get.orig.levs <- function(object, FUN=levels, newdata=NULL, sparse = FALSE, ...) {
    Terms <- terms(object, data = newdata, ...)
    mf <- model.frame(object, ...)
    isFac <- vapply(mf, is.factor, FUN.VALUE=TRUE)
    ## ignore response variable
    isFac[attr(Terms,"response")] <- FALSE
    mf <- mf[isFac]
    hasSparse <- any(grepl("sparse", names(formals(FUN))))   # check if FUN has sparse argument
    orig_levs <- if (any(isFac) && hasSparse) lapply(mf, FUN, sparse = sparse) else if(any(isFac) && !hasSparse) lapply(mf, FUN) # else NULL
    ## if necessary (allow.new.levels ...) add in new levels
    if (!is.null(newdata)) {
        for (n in names(mf)) {
            orig_levs[[n]] <- c(orig_levs[[n]],
                                setdiff(unique(as.character(newdata[[n]])),orig_levs[[n]]))
        }
    }
    ## more clues about factor-ness of terms
    if(!is.null(orig_levs)) attr(orig_levs,"isFac") <- isFac
    orig_levs
}


##' Force new parameters into a merMod object
##' (should this be an S3 method?):   params(object) <- newvalue
##' @param object
##' @param params a list of the form specified by \code{start} in
##' \code{\link{lmer}} or \code{\link{glmer}} (i.e. a list containing
##' theta and/or beta; maybe eventually further parameters ...
##' What updating do we have to do in order to make the resulting object
##' consistent/safe?
##' TODO: What kind of checking of input do we have to do?  (Abstract from
##' lmer/glmer code ...)
##' TODO: make sure this gets updated when the parameter structure changes
##' from (theta, beta) to alpha=(theta, beta, phi)
##' @param inplace logical specifying if object should be modified in place; not yet
##' @param subset logical; needs to be true, if only parts of params are to be reset
##' @examples
##' fm1 <- lmer(Reaction ~ Days + (Days|Subject), sleepstudy)
##' fm1M <- setParams(fm1,list(theta=rep(1,3)))
##' getME(fm1M,"theta")
##' getME(fm1,"theta") ## check that original didn't get messed up
##' ## check that @resp and @pp are the only reference class slots ...
##' sapply(slotNames(fm1),function(x) class(slot(fm1,x)))
setParams <- function(object, params, inplace=FALSE, subset=FALSE) {
    pNames <- c("beta", "theta", "par")
    if (object@devcomp$dims[["useSc"]]) {
      pNames <- c(pNames, "sigma")
    }
    if (!is.list(params) || length(setdiff(names(params), pNames)) > 0)
        stop("params should be specifed as a list with elements from ",
             "{",paste(shQuote(pNames),collapse=", "),"}")
    if (!subset && (is.null(params$beta) || (is.null(params$theta) && is.null(params$par)))) {
        warning("some parameters not specified in setParams()")
    }
    nbeta <- length(object@pp$beta(1))
    ntheta <- length(object@pp$theta)
    npar <- getParLength(object)
    
    lengthTestFun <- function(nm, expected_len) {
      x <- params[[nm]]
      if (!is.null(x) && length(x) != expected_len) {
        stop(sprintf("length mismatch in %s (%d != %d)",
                     nm, length(x), expected_len))
      }
      return(x)
    }

    beta <- lengthTestFun("beta", nbeta)
    if (!is.null(params$theta) && !is.null(params$par)) {
      stop("should specify exactly one of 'theta' and 'par'")
    }
    theta <- lengthTestFun("theta", ntheta)
    par <- lengthTestFun("par", npar)

    matchNames <- function(x,tn,vecname="theta") {
        if (!is.null(pn <- names(x))) {
            if (!setequal(pn,tn)) {
                ## pn.not.tn <- setdiff(pn,tn)
                ## tn.not.pn <- setdiff(tn,pn)
                ## TO DO: more detail?
                stop("mismatch between ",shQuote(vecname)," parameter vector names and internal names (",
                     paste(tn,collapse=","),")")
            }
            x <- x[tn]  ## reorder
        }
        ## no longer issue warning if unnamed ...
        return(x)
    }
    theta <- matchNames(theta, getThetaNames(object), "theta")
    beta <- matchNames(beta,colnames(getME(object,"X")),"beta")
    sigma <- params$sigma
    if(inplace) {
        stop("modification in place (copy=FALSE) not yet implemented")
    } else { ## copy :
        newObj <- object
        ## make a copy of the reference class slots to
        ##  decouple them from the original object
        newObj@pp <- newObj@pp$copy()
        newObj@resp <- newObj@resp$copy()
        if (!is.null(beta)) {
            newObj@pp$setBeta0(beta)
            newObj@beta <- beta
        }
        if (!is.null(par)) {
          ## replace NULL theta with computed values
          theta <- mkMkTheta(getReCovs(object))(par)
        }
        if (!is.null(theta)) {
            ## where does theta live and how do I set it?
            ## (1) in .@theta
            ## (2) in .@pp$theta
            newObj@theta <- theta
            newObj@pp$setTheta(theta)
        }
        if (!is.null(sigma)) {
            snm <- if (object@devcomp$dims[["REML"]]) "sigmaREML" else "sigmaML"
            newObj@devcomp[["cmp"]][snm] <- sigma
        }
        return(newObj)
    }
}


##' Make new random effect terms from specified object and new data,
##' possibly omitting some random effect terms
##' @param object fitted model object
##' @param newdata (optional) data frame containing new data
##' @param re.form formula specifying random effect terms to include (NULL=all, ~0)
##' @param na.action
##'
##' @note Hidden; _only_ used (twice) in this file
mkNewReTrms <- function(object, newdata,
                        re.form=NULL,
                        na.action=na.pass,
                        allow.new.levels=FALSE,
                        sparse = max(lengths(orig.random.levs)) > 100)
{
    ## construct (fixed) model frame in order to find out whether there are
    ## missing data/what to do about them
    ## need rfd to inherit appropriate na.action; need grouping
    ## variables as well as any covariates that are included
    ## in RE terms
    ## FIXME: mfnew is new data frame, rfd is processed new data
    ##        why do we need both/what is each doing/how do they differ?
    ##        rfd is *only* used in mkReTrms
    ##        mfnew is *only* used for its na.action attribute (!) [fixed only]
    ##        using model.frame would mess up matrix-valued predictors (GH #201)
    fixed.na.action <- NULL
    re.form <- re.form %||% lme4_reOnly(formula(object))
    if (is.null(newdata)) {
        rfd <- mfnew <- model.frame(object)
        fixed.na.action <- attr(mfnew,"na.action")
    } else {
        if (!identical(na.action,na.pass)) {
            ## only need to re-evaluate for NAs if na.action != na.pass
            mfnew <- model.frame(delete.response(terms(object, fixed.only=TRUE)),
                                 newdata, na.action=na.action)
            fixed.na.action <- attr(mfnew,"na.action")
        }
        ## make sure we pass na.action with new data
        ## it would be nice to do something more principled like
        ## rfd <- model.frame(~.,newdata,na.action=na.action)
        ## but this adds complexities (stored terms, formula, etc.)
        ## that mess things up later on ...
        ## rfd <- na.action(get_all_vars(delete.response(terms(object,fixed.only=FALSE)), newdata))
        newdata.NA <- newdata
        if (!is.null(fixed.na.action)) {
            newdata.NA <- newdata.NA[-fixed.na.action,]
        }
        tt <- delete.response(terms(object, random.only=TRUE))
        orig.random.levs <- get.orig.levs(object, random.only=TRUE, newdata=newdata.NA)
        orig.random.cntr <- get.orig.levs(object, random.only=TRUE,
                                          FUN=contrasts, sparse=sparse)
        ## need to let NAs in RE components go through -- they're handled downstream
        if (inherits(re.form,"formula")) {
            ## We use the RE terms *from the original model fit* to construct
            ##  the model frame. This is good for preserving predvars information,
            ##  getting interactions constructed correctly, etc etc etc,
            ##  but can fail if a partial RE specification is used and some of the variables
            ##  in the original RE form are missing from 'newdata' ...
            ##  Fill them in as necessary. Filling in NA is OK - these vars won't actually
            ##  be used later ...
            pv <- attr(tt,"predvars")
            for (i in 2:(length(pv))) {
                missvars <- setdiff(all.vars(pv[[i]]), all.vars(re.form))
                for (mv in missvars) {
                    newdata.NA[[mv]] <- NA
                }
            }
        }

        ## see comments about why suppressWarnings() is needed below ...
        rfd <- suppressWarnings(
            model.frame(tt, newdata.NA, na.action=na.pass, xlev=orig.random.levs))
        ## restore contrasts (why???)
        ## find *factor* variables involved in terms (left-hand side of RE formula): reset their contrasts
        ## only interested in components in re.form, not al REs
        ff <- re.form  ## was: formula(object,random.only=TRUE)
        termvars <- unique(unlist(lapply(reformulas::findbars(ff), function(x) all.vars(x[[2]]))))
        for (fn in Reduce(intersect, list(
                              names(orig.random.cntr), termvars, names(rfd)))) {
            ## a non-factor grouping variable *may* sneak in here via simulate(...)
            if (!is.factor(rfd[[fn]])) rfd[[fn]] <- factor(rfd[[fn]])
            contrasts(rfd[[fn]]) <- orig.random.cntr[[fn]]
        }

        if (!is.null(fixed.na.action))
            attr(rfd,"na.action") <- fixed.na.action
        ##
        ## ## need terms to preserve info about spline/orthog polynomial bases
        ## attr(rfd,"terms") <- terms(object)
        ## ## ... but variables list messes things up; can we fix it?
        ## vlist <- lapply(all.vars(terms(object)), as.name)
        ## attr(attr(rfd,"terms"),"variables") <-  as.call(c(quote(list), vlist))
        ##
        ## take out variables that appear *only* in fixed effects
        ## all.v <- all.vars(delete.response(terms(object,fixed.only=FALSE)))
        ## ran.v <- vapply(findbars(formula(object)),all.vars,"")
        ## fix.v <- all.vars(delete.response(terms(object,fixed.only=TRUE)))
        ## rfd <- model.frame(delete.response(terms(object,fixed.only=FALSE)),
        ## newdata,na.action=na.action)
    }
    if (inherits(re.form, "formula")) {
        ## DROP values with NAs in fixed effects
        if (length(fixed.na.action) > 0) {
            newdata <- newdata[-fixed.na.action,]
        }
        ## note: mkReTrms automatically *drops* unused levels
        bb1 <- findbars_x(re.form, specials = lme4_specials,
                      default.special = "us", target = "|",
                      expand_doublevert_method = getDoublevertDefault())
        bb0 <- lapply(bb1, `[[`, 2L)
        reTrms <- reformulas::mkReTrms(bb0, rfd, calc.lambdat = FALSE)
        reTrms <- upReTrms(reTrms, bb1) # local calc.lambdat=TRUE step

        ## ReTrms <- reformulas::mkReTrms(reformulas::findbars(re.form[[2]]), rfd)
        ## update Lambdat (ugh, better way to do this?)
        reTrms <- within(reTrms,Lambdat@x <- unname(getME(object,"theta")[Lind]))
        if (!allow.new.levels && any(vapply(reTrms$flist, anyNA, NA)))
            stop("NAs are not allowed in prediction data",
                 " for grouping variables unless allow.new.levels is TRUE")
        ns.re <- names(re <- ranef(object, condVar = FALSE))
        nRnms <- names(Rcnms <- reTrms$cnms)
        if (!all(nRnms %in% ns.re))
            stop("grouping factors specified in re.form that were not present in original model")
        new_levels <- lapply(reTrms$flist, function(x) levels(factor(x)))
        ## fill in/delete levels as appropriate
        re_x <- Map(function(r,n) levelfun(r,n,
                                           allow.new.levels=allow.new.levels),
                    re[names(new_levels)],
                    new_levels)
        ## pick out random effects values that correspond to
        ##  random effects incorporated in re.form ...
        ## NB: Need integer indexing, as nRnms can be duplicated: (age|Subj) + (sex|Subj) :
        hacked_names <- FALSE
        get_re <- function(rname, cnms) {
            nms <- names(re[[rname]])
            if (identical(cnms,"(Intercept)") && length(nms)==1 && grepl("^s(.*)$",nms)) {
                ## HACK to allow gamm4 prediction
                hacked_names <<- TRUE
                cnms <- nms
            }
            miss_names <- setdiff(cnms, nms)
            if (length(miss_names)>0) {
                stop("random effects specified in re.form that were not present in original model ",
                     paste(miss_names, collapse=", "))
            }
            t(re_x[[rname]][,cnms]) ## transpose to make sure unlisting works
        }
        re_new <- unlist(Map(get_re, nRnms, Rcnms))
        ## only issue warning once per prediction ...
        if (hacked_names) warning("modified RE names for gamm4 prediction")
    }
    Zt <- reTrms$Zt
    attr(Zt, "na.action") <- attr(re_new, "na.action") <- fixed.na.action
    list(Zt=Zt, b=re_new, Lambdat = reTrms$Lambdat, flist = reTrms$flist)
}

##' @param x a random effect (i.e., data frame with rows equal to levels, columns equal to terms
##' @param n vector of new levels
levelfun <- function(x, nl.n, allow.new.levels=FALSE) {
    ## 1. find and deal with new levels

    new.levels <- setdiff(nl.n, rownames(x))
    if (length(new.levels)>0) {
        if (!allow.new.levels) {
            max.err.len <- 60
            err.str <- paste(new.levels, collapse = ", ")
            if (nchar(err.str) > max.err.len) {
                err.str <- substr(err.str, 1, max.err.len)
                err.str <- gsub(",[^,]*$", ", ...", err.str)
            }
            stop("new levels detected in newdata: ", err.str)
        }
        ## create an all-zero data frame corresponding to the new set of levels ...
        nl.n.comb <- union(nl.n, rownames(x))
        newx <- as.data.frame(matrix(0, nrow=length(nl.n.comb), ncol=ncol(x),
                                     dimnames=list(nl.n.comb, names(x))))
        ## then paste in the matching RE values from the original fit/set of levels
        newx[rownames(x),] <- x
        x <- newx
    }
    ## 2. find and deal with missing old levels
    ## ... these should have been dropped when making the Z matrices
    ##     etc. in mkReTrms, so we'd better drop them here to match ...
    if (!all(r.inn <- rownames(x) %in% nl.n)) {
        x <- x[r.inn,,drop=FALSE]
    }
    return(x)
}

##'
##' \code{\link{predict}} method for \code{\linkS4class{merMod}} objects
##'
##' @title Predictions from a model at new data values
##' @param object a fitted model object
##' @param newdata data frame for which to evaluate predictions
##' @param newparams new parameters to use in evaluating predictions
##' @param re.form formula for random effects to condition on.  If \code{NULL},
##' include all random effects; if \code{NA} or \code{~0},
##' include no random effects
##' @param terms a \code{\link{terms}} object - not used at present
##' @param type character string - either \code{"link"}, the default,
##'    or \code{"response"} indicating the type of prediction object returned
##' @param allow.new.levels (logical) if FALSE (default), then any new levels
##'    (or NA values) detected in \code{newdata} will trigger an error; if TRUE, then
##'    the prediction will use the unconditional (population-level)
##'    values for data with previously unobserved levels (or \code{NA}s)
##' @param na.action function determining what should be done with missing values for fixed effects in \code{newdata}. The default is to predict \code{NA}: see \code{\link{na.pass}}.
##' @param se.fit A logical value indicating whether the standard errors should be included or not. Default is FALSE.
##' @param ... optional additional parameters.  None are used at present.
##' @return a numeric vector of predicted values, unless \code{se.fit=TRUE} (in which case a list with elements \code{fit} (predicted values) and \code{se.fit} is returned)
##' @note There is no option for computing standard errors of predictions because it is difficult to define an efficient method that incorporates uncertainty in the variance parameters; we recommend \code{\link{bootMer}} for this task.
##' @examples
##' (gm1 <- glmer(cbind(incidence, size - incidence) ~ period + (1 |herd), cbpp, binomial))
##' str(p0 <- predict(gm1))            # fitted values
##' str(p1 <- predict(gm1,re.form=NA))  # fitted values, unconditional (level-0)
##' newdata <- with(cbpp, expand.grid(period=unique(period), herd=unique(herd)))
##' str(p2 <- predict(gm1,newdata))    # new data, all RE
##' str(p3 <- predict(gm1,newdata,re.form=NA)) # new data, level-0
##' str(p4 <- predict(gm1,newdata,re.form=~(1|herd))) # explicitly specify RE
##' @method predict merMod
##' @export
predict.merMod <- function(object, newdata=NULL, newparams=NULL,
                           re.form=NULL,
                           random.only=FALSE,
                           terms=NULL, type=c("link","response"),
                           allow.new.levels=FALSE, na.action=na.pass,
                           se.fit = FALSE, ...) {
    ## FIXME: appropriate names for result vector?
    ## FIXME: make sure behaviour is entirely well-defined for NA in grouping factors

    ## Dealing with NAs:
    ## we might need to distinguish among
    ##  (i) NAs in original data and in new data
    ##  (ii) na.action possibilities (exclude, fail, omit, pass)
    ##  (iii) na.action setting in original fit and in predict()
    ##  (iii) NAs in (fixed effect) predictors vs RE grouping variables
    ##  (iv) setting of allow.new.level

    ## NAs in original data (in the fixed or random effects)
    ## may lead to a model frame within the
    ## fitted object that is missing rows; if na.exclude was used,
    ## these will need to be reconstituted in the prediction.
    ##
    ## For the most part, 'na.action's used at the predict stage
    ## (i.e. for newdata) will work on NAs *in the fixed effects*
    ## without further intervention; 'na.pass' will automatically
    ## produce NA values in the output, so 'na.exclude' is not really
    ## necessary (but might get specified anyway)
    ##
    ## In the random effects, NAs in newdata will give a population-level
    ## prediction if allow.new.levels is TRUE; if it's FALSE they give
    ## an error (although it could be argued that in that case they
    ## should follow 'na.action' instead ...)

    if (any(...names() %in% c("ReForm", "REForm", "REform")))
        stop("synonyms 'ReForm', 'REForm', 'REform' are deprecated: please use 're.form' instead")

    if (...length() > 0) warning("unused arguments ignored")

    type <- match.arg(type)
    if (!is.null(terms))
        stop("terms functionality for predict not yet implemented")
    if (!is.null(newparams))
        object <- setParams(object,newparams)

    if (is.null(newdata) && is.null(re.form) &&
        is.null(newparams) && !random.only) {
        ## raw predict() call, just return fitted values
        ##   (inverse-link if appropriate)
        if (isLMM(object) || isNLMM(object)) {
            ## make sure we do *NOT* have NAs in fitted object
            pred <- na.omit(fitted(object))
        } else { ## inverse-link
            pred <-  switch(type,response=object@resp$mu, ## == fitted(object),
                            link=object@resp$eta)
            eta <- object@resp$eta
            if (is.null(nm <- rownames(model.frame(object))))
                nm <- seq_along(pred)
            names(pred) <- nm
        }
        fit.na.action <- NULL
        ## flow jumps to end for na.predict
    } else { ## newdata and/or re.form and/or newparams and/or random.only specified
        fit.na.action <- attr(object@frame,"na.action")  ## original NA action

        nobs <- if (is.null(newdata)) nrow(object@frame) else nrow(newdata)
        pred <- rep(0,nobs)

        if (!random.only) {
            X <- getME(object, "X")
            X.col.dropped <- attr(X, "col.dropped")
            ## modified from predict.glm ...
            if (is.null(newdata)) {
                ## Use original model 'X' matrix and offset
                ## orig. offset: will be zero if there are no matches ...
                offset <- model.offset(model.frame(object))
                if (is.null(offset)) offset <- 0
            } else {  ## new data specified
                ## evaluate new fixed effect
                RHS <- formula(substitute(~R,
                         list(R=RHSForm(formula(object,fixed.only=TRUE)))))
                ## https://github.com/lme4/lme4/issues/414
                ## contrasts are not relevant in random effects;
                ##  model.frame.default warns about dropping contrasts
                ##  if (1) xlev is specified and (2) any factors in
                ##  original data frame had contrasts set

                ## alternative solution: drop contrasts manually
                ## (could assign to a new variable newdata2 for safety,
                ## but I don't think newdata
                ##  is used downstream in this function?)

                ## isFacND <- which(vapply(newdata, is.factor, FUN.VALUE = TRUE))
                ## for (j in isFacND) {
                ##    attr(newdata[[j]], "contrasts") <- NULL
                ## }

              orig.fixed.levs <- get.orig.levs(object, fixed.only=TRUE,
                                               newdata = newdata)
                mfnew <- suppressWarnings(
                    model.frame(delete.response(terms(object, fixed.only=TRUE,
                                                      data = newdata)),
                                newdata,
                                na.action = na.action, xlev = orig.fixed.levs))

                X <- model.matrix(RHS, data=mfnew,
                                  contrasts.arg=attr(X,"contrasts"))
                ## hack to remove unused interaction levels?
                ## X <- X[,colnames(X0)]

                offset <- 0 # rep(0, nrow(X))
                tt <- terms(object, data  = newdata)
                if (!is.null(off.num <- attr(tt, "offset"))) {
                    for (i in off.num)
                        offset <- offset + eval(attr(tt,"variables")[[i + 1]], newdata)
                }
                ## FIXME?: simplify(no need for 'mfnew'): can this be different from 'na.action'?
                fit.na.action <- attr(mfnew,"na.action")
                ## only need to drop if new data specified ...
                if(is.numeric(X.col.dropped) && length(X.col.dropped) > 0)
                    X <- X[, -X.col.dropped, drop=FALSE]
            }
            pred <- drop(X %*% fixef(object))

            ## FIXME:: need to unname()  ?
            ## FIXME: is this redundant??
            ## if (!is.null(frOffset <- attr(object@frame,"offset")))
            ##     offset <- offset + eval(frOffset, newdata)
            pred <- pred+offset

        } ## end !(random.only)

        if (isRE(re.form)) {
            if (is.null(re.form))
                re.form <- lme4_reOnly(formula(object)) # RE formula only
            rfd <- if (is.null(newdata)) {
                       ## try to retrieve original data ... fall back to model frame if necessary
                       ## FIXME: this doesn't solve the problem if columns of model frame and data
                       ## diverge (e.g. transformed objects [log(x)], offsets [offset(x)] ... will
                       ## fail farther along
                       tryCatch(getData(object),
                                error = function(e) object@frame)
                   } else newdata
            newRE <- mkNewReTrms(object, rfd, re.form, na.action=na.action,
                                 allow.new.levels=allow.new.levels)
            REvals <- base::drop(as(newRE$b %*% newRE$Zt, "matrix"))
            ## only needed if called as simulation? NAs sometimes excluded within mkNewReTrms ...
            if (length(pred) != length(REvals)) {
                if (!class(fit.na.action) %in% c("omit", "exclude") && length(fit.na.action)>0) {
                    stop("fixed/RE pred length mismatch")
                }
                 REvals <- REvals[-fit.na.action]
            }
            pred <- pred + REvals
            if (random.only) {
                fit.na.action <- attr(newRE$Zt,"na.action")
            }
        }
        if (isGLMM(object) && type=="response") {
            eta <- pred
            pred <- object@resp$family$linkinv(pred)
        }
    }  ## newdata/newparams/re.form

    ## fill in NAs as appropriate:
    ##   if NAs were detected in original model fit, OR in updated model frame construction
    ## but DON'T double-NA if raw prediction in the first place
    if (is.null(newdata)) {
        fit.na.action <- attr(model.frame(object),"na.action")
        if (!missing(na.action)) {
            ## hack to override action where explicitly specified
            if (!is.null(fit.na.action))
                class(fit.na.action) <- class(attr(na.action(NA),"na.action"))
        }
    }
    pred <- napredict(fit.na.action, pred)

    if (!se.fit) return(pred)

    if (!isLMM(object)) warning("se.fit computation uses an approximation to estimate the sampling distribution of the parameters")

    Cmat <- vcov_full(object)

    n_u <- length(getME(object, "u"))
    n_beta <- length(getME(object, "beta"))
    ## FIXME: these need to be fixed (????)
    if(is.null(newdata)) {
        X <- getME(object, "X")
        if(is.null(re.form)) {
            Z <- getME(object, "Z")
        } else {
            if(isRE(re.form)) {
                ## FIXME: newRE is not computed here
                Z <- t(newRE$Zt)
            } else {
                Z <- Matrix(0, nrow = nrow(X), ncol = n_u)
            }
        }
    } else {
        if(isRE(re.form)) {
            Z <- t(newRE$Zt)
        } else {
            ## this is inefficient and we could just calculate
            ## X %*% Cmat[X part only] t(X) instead
            Z <- Matrix(0, nrow = nrow(X), ncol = n_u)
        }
    }

    if(random.only) X <- Matrix(0, nrow = nrow(Z), ncol = n_beta)

    ZX <- cbind(Z, X)
    
    ## Subsetting Cmat
    if (ncol(ZX) != nrow(Cmat)) {
      Cmat_names <- rownames(Cmat)
      ## Subsetting appears to occur in the case we use newRE;
      Z_factors <- newRE$flist
      C_factors <- object@flist
      cnms <- object@cnms

      fix_nms <- colnames(object@pp$X)
      
      ## Cmat is padded with zeros for new levels
      if(allow.new.levels){
        Cmat_mod <- pad_Cmat(Cmat, C_factors, Z_factors, Cmat_names, cnms)
        Cmat <- Cmat_mod$Cmat
        Cmat_names <- Cmat_mod$Cmat_names
        C_factors <- Cmat_mod$C_factors
      }
    
      is_group_term <- !Cmat_names %in% fix_nms

      ## looking to compute the groups (factor levels) that are actually 
      ## included in the Z matrix
      keep_idx <- !is_group_term
      
      mask <- unlist(lapply(
        intersect(names(C_factors), names(Z_factors)),
        function(grp) {
          level_mask <- levels(C_factors[[grp]]) %in% levels(Z_factors[[grp]])
          rep(level_mask, each = length(cnms[[grp]]))
        }
      ))
      
      keep_idx[seq_along(mask)] <- is_group_term[seq_along(mask)] & mask
      
      # For now, don't worry about keeping it as dpoMatrix?
      Cmat <- Cmat[keep_idx, keep_idx]
      #Cmat <- as(Cmat[keep_idx, keep_idx], "dpoMatrix")
    }
    
    res <- list(fit = pred,
         se.fit = sqrt(quad.tdiag(Cmat, ZX))
         )
    if (isGLMM(object) && type=="response") {
      ## pred0 (linear predictor) will have been stored previously in this case ...  
      res$se.fit <- res$se.fit*abs(family(object)$mu.eta(eta))
    }
    res

} # end {predict.merMod}


## all possible LHS evaluated values ...
simulate.formula_lhs_matrix <- simulate.formula_lhs_numeric <-
    simulate.formula_lhs_integer <- simulate.formula_lhs_factor <-
        simulate.formula_lhs_logical <-
        simulate.formula_lhs_ <-
        function(object, nsim = 1, seed = NULL,
                 newdata,
                 ...) {
            ## N.B. *must* name all arguments so that 'object' is missing in .simulateFun()
            .simulateFun(formula=object, nsim=nsim, seed=seed, newdata=newdata, ...)
        }

simulate.merMod <- function(object, nsim = 1, seed = NULL, use.u = FALSE,
                            re.form=NA,
                            newdata=NULL, newparams=NULL,
                            family=NULL,
                            cluster.rand=rnorm,
                            allow.new.levels=FALSE, na.action=na.pass, ...) {

    ## FIXME: is there a reason this can't be a copy of .simulateFun ... ?
    mc <- match.call()
    mc[[1]] <- quote(lme4::.simulateFun)
    eval(mc, parent.frame(1L))
}

.simulateFun <- function(object, nsim = 1, seed = NULL, use.u = FALSE,
                         re.form=NA,
                         newdata=NULL, newparams=NULL,
                         formula=NULL,family=NULL,
                         cluster.rand=rnorm,
                         weights=NULL,
                         offset=NULL,
                         allow.new.levels=FALSE,
                         na.action=na.pass,
                         cond.sim=TRUE,
                         ...) {

    if (...length() > 0) warning("unused arguments ignored")

    if (missing(object) && (is.null(formula) || is.null(newdata) || is.null(newparams))) {
        stop("if ",sQuote("object")," is missing, must specify all of ",
             sQuote("formula"),", ",sQuote("newdata"),", and ",
             sQuote("newparams"))
    }

    nullWts <- FALSE
    if (is.null(weights)) {
        if (is.null(newdata)) {
            weights <- weights(object)
        } else {
            nullWts <- TRUE # this flags that 'weights' wasn't supplied by the user
            weights <- rep(1,nrow(newdata))
        }
    }

    if (missing(object)) {

        ## construct fake-fitted object from data, params
        ## copied from glm(): DRY; this all stems from the
        ## original sin of handling family=gaussian as a special
        ## case
        if (is.character(family))
            family <- get(family, mode = "function", envir = parent.frame())
        if (is.function(family))
            family <- family()
        if (is.null(family) ||
            (family$family=="gaussian" && family$link=="identity")) {
            lmod <- lFormula(formula,newdata,
                             weights=weights,
                             offset=offset,
                             control=lmerControl(check.formula.LHS="ignore"))
            devfun <- do.call(mkLmerDevfun, lmod)
            object <- mkMerMod(environment(devfun),
                               ## (real parameters will be filled in later)
                               opt = list(par=NA,fval=NA,conv=NA),
                               lmod$reTrms, fr = lmod$fr)
        } else {

            glmod <- glFormula(formula,newdata,family=family,
                               weights=weights,
                               offset=offset,
                               control=glmerControl(check.formula.LHS="ignore"))
            devfun <- do.call(mkGlmerDevfun, glmod)
            object <- mkMerMod(environment(devfun),
                               ## (real parameters will be filled in later)
                               opt = list(par=NA,fval=NA,conv=NA),
                               glmod$reTrms, fr = glmod$fr)
        }
        ## would like to do this:
        ## so predict() -> fitted() -> set default names will work
        ## instead we have a special case in fitted()
        ## object@resp$mu <- rep(NA_real_,nrow(model.frame(object)))
    }

    stopifnot((nsim <- as.integer(nsim[1])) > 0,
              is(object, "merMod"))
    if (!is.null(newparams)) {
        object <- setParams(object, newparams)
    }

    if (!missing(use.u)) {
        if (!missing(re.form)) {
            stop("should specify only one of ",sQuote("use.u"),
                 " and ",sQuote("re.form"))
        }
        re.form <- if (use.u) NULL else ~0
    }

    if (is.null(re.form)) { # formula w/o response
        re.form <- lme4_reOnly(formula(object))
    }

    if(!is.null(seed)) set.seed(seed)
    if(!exists(".Random.seed", envir = .GlobalEnv))
        runif(1) # initialize the RNG if necessary
    RNGstate <- .Random.seed

    sigma <- sigma(object)
    ## OBSOLETE: no longer use X?
    ## n <- nrow(X <- getME(object, "X"))
    ## link <- if (isGLMM(object)) "response"

    ## predictions, conditioned as specified, on link scale
    ## previously: do **NOT** use na.action as specified here (inherit
    ##     from object instead, for consistency)
    ## now: use na.omit, because we have to match up
    ##    with whatever is done in mkNewReTrms
    etapred <- predict(object, newdata=newdata, re.form=re.form,
                       type="link", na.action=na.omit,
                       allow.new.levels=allow.new.levels)
    n <- length(etapred)

    ## now add random components:
    ##  only the ones we did *not* condition on

    ## compre.form <- noLHSform(formula(object))
    ## construct RE formula ONLY: leave out fixed terms,
    ##   which might have loose terms like offsets in them ...

    ##' combine unary or binary operator + arguments (sugar for 'substitute')
    makeOp <- function(x,y,op=NULL) {
        if (is.null(op)) {  ## unary
            substitute(OP(X),list(X=x,OP=y))
        } else substitute(OP(X,Y), list(X=x,OP=op,Y=y))
    }

    compReForm <- lme4_reOnly(formula(object))
    if (isRE(re.form)) {
        rr <- lme4_reOnly(re.form)[[2]] ## expand RE and strip ~
        ftemplate <- substitute(.~.-XX, list(XX=rr))
        compReForm <- update.formula(compReForm,ftemplate)[-2]
        ## update, then delete LHS
    }

    ## (1) random effect(s)
    sim.reff <- if (!is.null(reformulas::findbars(compReForm))) {
        newRE <- mkNewReTrms(object, newdata, compReForm,
                             na.action=na.action,
                             allow.new.levels=allow.new.levels)
        ## this *can* justifiably happen, if we are using mkNewReTrms
        ## in the context of predicting/simulating with a non-trivial
        ## re.form ...
        ## <obsolete> paranoia ...
        ## <obsolete> stopifnot(!is.null(newdata) ||
        ##       isTRUE(all.equal(newRE$Lambdat,getME(object,"Lambdat"))))
        U <- t(newRE$Lambdat %*% newRE$Zt) # == Z Lambda
        u <- cluster.rand(ncol(U)*nsim)
        ## UNSCALED random-effects contribution:
        as(U %*% matrix(u, ncol = nsim), "matrix")
    } else 0

    val <- if (isLMM(object)) {
          ## result will be matrix  n x nsim :
          etapred + sigma * (sim.reff +
                               ## residual contribution:
                               if (cond.sim)
                                   # always rnorm regardless of cluster.rand
                                   matrix(rnorm(n * nsim), ncol = nsim)
                               else 0)
    } else if (isGLMM(object)) {
        ## GLMM
        ## n.b. DON'T scale random-effects (???)
        etasim <- etapred+sim.reff
        family <- normalizeFamilyName(object@resp$family)
        musim <- family$linkinv(etasim) #-> family$family == "negative.binomial" if(NB)
        ## ntot <- length(musim) ## FIXME: or could be dims[["n"]]?
        ##

        if (family$family=="binomial" && is.matrix(r <- model.response(object@frame))) {

            # unless the user passed in new weights, take them from the response matrix
            # e.g. cbind(incidence, size-incidence) ~ ...
            if(nullWts) weights <- rowSums(r)
        }

        if (is.null(sfun <- simfunList[[family$family]])) {
            ## family$simulate just won't work ...
            ## sim funs must be hard-coded, see below
            stop("simulation not implemented for family ",
                 sQuote(family$family))
        }

        ## don't rely on automatic recycling
        if (cond.sim) {
             val <- sfun(object, nsim=1, ftd = rep_len(musim, n*nsim),
                         wts = weights)
        } else {
             val  <- rep_len(musim, n*nsim)
        }
        ## split results into nsims: need special case for binomial matrix/factor responses
        if (family$family=="binomial" && is.matrix(r <- model.response(object@frame))) {
            lapply(split(val[[1]], gl(nsim, n, 2 * nsim * n)), matrix,
                          ncol = 2, dimnames = list(NULL, colnames(r)))
        } else if (family$family=="binomial" && is.factor(val[[1]])) {
            split(val[[1]], gl(nsim,n))
        } else split(val, gl(nsim,n))
    } else
        stop("simulate method for NLMMs not yet implemented")

    ## from src/library/stats/R/lm.R
    if(!is.list(val)) {
        dim(val) <- c(n, nsim)
        val <- as.data.frame(val)
    } else class(val) <- "data.frame"
    names(val) <- paste("sim", seq_len(nsim), sep="_")
    ## have not yet filled in NAs, so need to use names of fitted
    ## object NOT including values with NAs
    f <- fitted(object)
    nm <- names(f)[!is.na(f)]
    ## unnamed input, *or* simulation from new data ...
    if (length(nm) == 0) {
        nm <- as.character(seq(n))
    } else if (!is.null(newdata)) {
        nm <- rownames(newdata)
    }
    row.names(val) <- nm

    fit.na.action <- attr(model.frame(object), "na.action")
    if (!missing(na.action) &&  !is.null(fit.na.action)) {
        ## retrieve name of na.action type ("omit", "exclude", "pass")
        class.na.action <- class(attr(na.action(NA), "na.action"))
        if (!identical(class.na.action, class(fit.na.action))) {
            ## hack to override action where explicitly specified
            class(fit.na.action) <- class.na.action
        }
    }

    nafun <- function(x) { x[] <- apply(x,
                                        2L,
                                        napredict,
                                        omit = fit.na.action); x }
    val <- if (is.matrix(val[[1]])) {
        ## have to handle binomial response matrices differently --
        ## fill in NAs as appropriate in *both* columns
        structure(lapply(val, nafun),
                  ## have to put this back into a (weird) data frame again,
                  ## carefully (should do the napredict stuff
                  ## earlier, so we don't have to redo this transformation!)
                  class = "data.frame")
    } else {
        as.data.frame(lapply(val, napredict, omit=fit.na.action))
    }

    ## reconstruct names: first get rid of NAs, then refill them
    ## as appropriate based on fit.na.action (which may be different
    ## from the original model's na.action spec)
    nm2 <-
        if (is.null(newdata))
            names(napredict(na.omit(f), omit=fit.na.action))
        else
            rownames(napredict(newdata, omit=fit.na.action))
    if (length(nm2) > 0)
        row.names(val) <- nm2

    structure(val,
              ## as.data.frame(lapply(...)) blows away na.action attribute,
              ##  so we have to re-assign here
              na.action = fit.na.action,
              seed = RNGstate)
}## .simulateFun()

########################
## modified from stats/family.R
## TODO: the $simulate methods included with R families by default
## are not sufficiently flexible to be re-used by lme4.
## these are modified by:
## (1) adding a 'ftd' argument for the fitted values
##     that defaults to fitted(object), to allow more flexibility
##     e.g. in conditioning on or marginalizing over random effects
##     (fitted(object) can be produced from predict.merMod() with
##     alternative parameters rather than being extracted directly
##     from the fitted objects -- this allows simulation with new
##     parameters or new predictor variables
## (2) modifying wts from object$prior.weights to weights(object)
## (3) adding wts as an argument
##
## these can be incorporated by overwriting the simulate()
## components, or calling them
##
gaussian_simfun <- function(object, nsim, ftd=fitted(object),
                            wts=weights(object)) {

    if (any(wts != 1)) warning("ignoring prior weights")
    rnorm(nsim*length(ftd), ftd, sd=sigma(object))
}

binomial_simfun <- function(object, nsim, ftd=fitted(object),
                            wts=weights(object)) {
    n <- length(ftd)
    ntot <- n*nsim
    if (any(wts %% 1 != 0))
        stop("cannot simulate from non-integer prior.weights")
    ## Try to figure out if the original data were
    ## proportions, a factor or a two-column matrix
    if (!is.null(m <- model.frame(object))) {
        y <- model.response(m)
        if(is.factor(y)) {
            ## ignore weights
            yy <- factor(levels(y)[1 + rbinom(ntot, size = 1, prob = ftd)],
                         levels = levels(y))
            split(yy, rep(seq_len(nsim), each = n))
        } else if(is.matrix(y) && ncol(y) == 2) {
            yy <- vector("list", nsim)
            for (i in seq_len(nsim)) {
                Y <- rbinom(n, size = wts, prob = ftd)
                YY <- cbind(Y, wts - Y)
                colnames(YY) <- colnames(y)
                yy[[i]] <- YY
            }
            yy
        } else
            rbinom(ntot, size = wts, prob = ftd)/wts
    } else rbinom(ntot, size = wts, prob = ftd)/wts
}

poisson_simfun <- function(object, nsim, ftd=fitted(object),
                           wts=weights(object)) {
        ## A Poisson GLM has dispersion fixed at 1, so prior weights
        ## do not have a simple unambiguous interpretation:
        ## they might be frequency weights or indicate averages.
        wts <- weights(object)
        if (any(wts != 1)) warning("ignoring prior weights")
        rpois(nsim*length(ftd), ftd)
    }


##' FIXME: need a gamma.shape.merMod method in order for this to work.
##'        (see initial shot at gamma.shape.merMod below)
Gamma_simfun <- function(object, nsim, ftd=fitted(object),
                         wts=weights(object)) {
    if (any(wts != 1)) message("using weights to scale shape parameter")
    ## used to use gamma.shape(), but sigma() is more general
    ## (wouldn't work *outside* of the merMod context though)
    shape <- 1/sigma(object)^2*wts
    rgamma(nsim*length(ftd), shape = shape, rate = shape/ftd)
}

gamma.shape.merMod <- function(object, ...) {
    if(family(object)$family != "Gamma")
        stop("Can not fit gamma shape parameter because Gamma family not used")

    y <- getME(object, "y")
    mu <- getME(object, "mu")
    w <- weights(object)
                                        # Sec 8.3.2 (MN)
    L <- w*(log(y/mu)-((y-mu)/mu))
    dev <- -2*sum(L)
                                        # Eqs. between 8.2 & 8.3 (MN)
    Dbar <- dev/length(y)
    structure(list(alpha = (6+2*Dbar)/(Dbar*(6+Dbar)),
                   SE = NA), # FIXME: obtain standard error
              class = "gamma.shape")
}

inverse.gaussian_simfun <- function(object, nsim, ftd=fitted(object),
                                    wts = weights(object)) {
    if (any(wts != 1)) message("using weights as inverse variances")
    if (!requireNamespace("statmod")) {
      stop("The ",sQuote("statmod")," package must be installed ",
           " in order to simulate inverse-Gaussian distributions")
    }
    statmod::rinvgauss(nsim * length(ftd), mean = ftd,
                       shape= wts/sigma(object))
}

## in the original MASS version, .Theta is assigned into the environment
## (triggers a NOTE in R CMD check)
## modified from @aosmith16 GH contribution

negative.binomial_simfun <- function (object, nsim,
                                      ftd = fitted(object),
                                      wts=weights(object))
{

    if (any(wts != 1))
        warning("ignoring prior weights")
    theta <- getNBdisp(object)
    rnbinom(nsim * length(ftd), mu = ftd, size = theta)
}

simfunList <- list(gaussian = gaussian_simfun,
                   binomial = binomial_simfun,
                   poisson  = poisson_simfun,
                   Gamma    = Gamma_simfun,
                   negative.binomial = negative.binomial_simfun,
                   inverse.gaussian = inverse.gaussian_simfun)

#' Catch errors and warnings and store them for subsequent evaluation
#'
#' Factory modified from a version written by Martin Morgan on Stack Overflow (see below).  
#' Factory generates a function which is appropriately wrapped by error handlers.  
#' If there are no errors and no warnings, the result is provided.  
#' If there are warnings but no errors, the result is provided with a warn attribute set.
#' If there are errors, the result retutrns is a list with the elements of warn and err.
#' This is a nice way to recover from a problems that may have occurred during loop evaluation or during cluster usage.
#' Check the references for additional related functions.
#' I have not included the other factory functions included in the original Stack Overflow answer because they did not play well with the return item as an S4 object.
#' @export
#' @param fun The function to be turned into a factory
#' @param debug print debugging statements?
#' @param errval the value to be returned from the function if an error is thrown
#' @param types which types to catch?
#' @return The result of the function given to turn into a factory.  If this function was in error "An error as occurred" as a character element.  factory-error and factory-warning attributes may also be set as appropriate.
#' @references
#' \url{http://stackoverflow.com/questions/4948361/how-do-i-save-warnings-and-errors-as-output-from-a-function}
#' @author Martin Morgan; Modified by Russell S. Pierce and Ben Bolker
#' @examples 
#' f.log <- factory(log)
#' f.log("a")
#' f.log.NA <- factory(log,errval=NA)
#' f.log.NA("a")
#' f.as.numeric <- factory(as.numeric)
#' f.as.numeric(c("a","b",1))
factory <- function (fun, debug=FALSE,
                     errval="An error occurred in the factory function",
                     types=c("message","warning","error")) {
    function(...) {
    errorOccurred <- FALSE
    warn <- err <- msg <- NULL
    res <- withCallingHandlers(tryCatch(fun(...),
                                        error = function(e) {
        if (debug) cat("error: ",conditionMessage(e),"\n")
        err <<- conditionMessage(e)
        errorOccurred <<- TRUE
        NULL
    }), warning = function(w) {
        if (!"warning" %in% types) {
            warning(conditionMessage(w))
        } else {
            warn <<- append(warn, conditionMessage(w))
            invokeRestart("muffleWarning")
        }
    },
    message = function(m) {
        if (debug) cat("message: ",conditionMessage(m),"\n")
        if (!"message" %in% types) {
            message(conditionMessage(m))
        } else {
            msg <<- append(msg, conditionMessage(m))
            invokeRestart("muffleMessage")
        }
    })
    if (errorOccurred) {
        if (!"error" %in% types) stop(err)
        res <- errval
    }

    setattr <- function(x, attrib, value) {
        attr(x,attrib) <- value
        x
    }

    attr_fun <- function(x,str,msg) {
        setattr(x,paste0("factory-",str), if(is.character(msg)) msg else NULL)
    }

    res <- attr_fun(res, "message", msg)
    res <- attr_fun(res, "warning", warn)
    res <- attr_fun(res, "error", err)

    return(res)
  }
}


    

quietly <- TRUE

## factory for making methods
mk_method <- function(class, print_dims=FALSE) {
    method <- sprintf("simulate.formula_lhs_%s",class)
    sim_generic <- function(object, nsim=1, seed=NULL, ...) {
        if (!quietly) message(sprintf("%s called",method))
        if (!quietly) cat(".Basis from attributes:\n")
        if (!quietly) print(attr(object,".Basis")) # NULL
        if (print_dims) {
            if (!quietly) print(dim(attr(object,".Basis")))
        }
        return(attr(object,".Basis"))
    }
    assign(method,sim_generic,.GlobalEnv)
    invisible(NULL)
}

## (**) these methods should (??) _mask_ package versions ...
## works in source(), not in devtools::test() ...
mk_method("NULL")
mk_method("numeric")
mk_method("array",print_dims=TRUE)
mk_method("")

test_that("simple numerics", {
    ## expect_equal(simulate(1~.),1)  ## FIXME re-enable if we resolve (**) above
    ## One-sided formula is not the same as an LHS that evaluates to NULL:
    expect_equal(simulate(NULL~.),NULL)
})

test_that("raw formulas", {
    expect_error(suppressWarnings(simulate(x~.)), "Error evaluating")
})

simulate.formula_lhs_character <- function(object, nsim=1, seed=NULL, ...) {
    if (!quietly) message("simulate.formula_lhs_character() called.")
    if (!quietly) print(ls(all.names=TRUE))
    NextMethod() # Calls simulate.formula(), resulting in an infinite recursion.
}

test_that("prevent recursion", {
     expect_error(simulate("a"~.), "No applicable method")
})

dd <- expand.grid(A=factor(1:3),B=factor(1:10),rep=1:10)
test_that("two-sided formula warning", {
    expect_error(suppressMessages(simulate(.~1 + (A|B),
                                             newdata=dd,
                                             newparams=list(beta=1,theta=rep(1,6),
                                                            sigma=1),
                                             family=gaussian,
                                             seed=101))[[1]],
                   "object '.' not found")
})

## cleanup: mk_method() assigns into .GlobalEnv, so remove explicitly from there
suppressWarnings(try(rm(list = c("simulate.formula_lhs_", "simulate.formula_lhs_numeric",
                                 "simulate.formula_lhs_array"),
                        envir = .GlobalEnv), silent = TRUE))

## a one-sided formula with neither 'family' nor 'newparams' used to fall
## through to the generic (and unhelpful) "No applicable method for LHS of
## type 'NULL'" error; it should instead report the missing arguments.
test_that("one-sided formula without family/newparams gives an informative error", {
    expect_error(simulate(~1 + (1|Days), newdata=sleepstudy),
                 "must specify all of.*formula.*newdata.*newparams")
})

## GH#948: simulate with re.form=NULL should work when a predictor is a 1-col
## matrix (e.g. from scale()) used inside poly() -- poly() with pre-specified
## coefs rejects matrix input, so mkNewReTrms must drop such columns first
test_that("simulate re.form=NULL works with poly(scale(x)) predictor (GH#948)", {
  ss_sc <- transform(sleepstudy, Days = scale(Days))
  fit1  <- lmer(Reaction ~ 1 + poly(Days, 2) + (1|Subject), data = sleepstudy)
  fit_sc <- update(fit1, data = ss_sc)
  s1 <- simulate(fit_sc, seed = 101, re.form = NULL)
  s2 <- simulate(fit_sc, seed = 101, re.form = NULL)
  expect_equal(s1, s2)
})

mkVarCorr <-
function(sc, cnms, nc = lengths(cnms, use.names = FALSE),
         theta, nms = names(cnms), reCovs = NULL, relReCovs = TRUE) {
    if (!missing(nc)) {
        nc <- as.integer(nc)
        if (!missing(cnms) && !identical(nc, lengths(cnms, use.names = FALSE)))
            stop("'nc' and lengths(cnms) are inconsistent")
    }
    if (!missing(nms)) {
        if (!missing(cnms) && !identical(nms, names(cnms)))
            stop("'nms' and names(cnms) are inconsistent")
    }
    if (is.null(reCovs))
        reCovs <- .mapply(new,
                          list(nc = nc,
                               par = split(theta, rep(seq_along(nc), (nc * (nc + 1L)) %/% 2L))),
                          list(Class = "Covariance.us"))
    ans <- lapply(seq_along(reCovs), function(i) {
        ## Li := \Lambda_i, the i-th diagonal block of \Lambda(\theta)
        ## Si := \sigma^2 \Lambda_i \Lambda_i'
        object <- reCovs[[i]]
        nci <- nc[i]
        cnmsi <- cnms[[i]]
        Li <- getLambda(object)
        jj <- seq.int(from = 1L, by = nci + 1L, length.out = nci)
        Si <- if (relReCovs) sc * sc * tcrossprod(Li) else tcrossprod(Li)
        Si.sd <- sqrt(Si[jj])
        Si.cor <- Si/Si.sd/rep(Si.sd, each = nci)
        Si.cor[jj] <- 1
        typ0 <- typ1 <- sub("^Covariance[.]", "", class(object))
        switch(typ0, # respecting the conventions of, e.g., glmmTMB:
               "cs"  = if ( object@hom) typ1 <- "homcs",
               "ar1" = if (!object@hom) typ1 <- "hetar1")
        class(Si) <- c(paste0("vcmat_", typ1), "matrix", "array")
        dimnames(Si) <- dimnames(Si.cor) <- list(cnmsi, cnmsi)
        names(Si.sd) <- cnmsi
        attr(Si, "stddev") <- Si.sd
        attr(Si, "correlation") <- Si.cor
        Si
    })
    if (!is.null(nms)) {
        ## FIXME:
        ## Do we want this?  Maybe not.  'nms' are not necessarily
        ## unique, e.g., 'fm2' from example("lmer") has *two* Subject
        ## terms, so the names are "Subject", "Subject".  The 'print'
        ## method for 'VarCorr.merMod' handles this just fine, but
        ## it's a little awkward if we want to dig out components of
        ## the list ...
        if (anyDuplicated(nms))
            nms <- make.names(nms, unique = TRUE)
        names(ans) <- nms
    }
 ## if (relReCovs) # rev. dep. (e.g., 'gamm4') use it unconditionally
        attr(ans, "sc") <- sc
    ans
}

## MJ:
## Maintain this list manually as prefixing convention is inconsistent.
for (.nm in c("us", "diag", "cs", "homcs", "ar1", "hetar1"))
    setOldClass(c(paste0("vcmat_", .nm), "matrix", "array"))
rm(.nm)
                             
VarCorr.merMod <-
function(x, sigma = 1, ...) {
    ## TODO? add argument type=c("varcov", "sdcor", "logs")
    useSc <- !isGLMM(x)
    sc <- if (missing(sigma)) sigma(x) else sigma
    cnms <- x@cnms
    nc <- lengths(cnms, use.names = FALSE)
    theta <- x@theta
    nms <- names(x@flist)[attr(x@flist, "assign")]
    reCovs <- getReCovs(x)
    ans <- mkVarCorr(sc = sc, cnms = cnms, nc = nc,
                     theta = theta, nms = nms,
                     reCovs = reCovs, relReCovs = useSc)
    class(ans) <- "VarCorr.merMod"
    ## FIXME:
    ## Here we have
    ##     'useSc' is TRUE  <=>  'sc' is a residual standard deviation
    ## but that does *not* match the interpretation of
    ##     as.logical(x@devcomp$dims[["useSc"]])
    ## ...
    attr(ans, "useSc") <- useSc # used by reformulas::formatVC
    ans
}

as.data.frame.VarCorr.merMod <-
function(x, row.names = NULL, optional = FALSE,
         order = c("cov.last", "lower.tri"), ...) {
    order <- match.arg(order)
    dlist <- lapply(seq_along(x), function(i) {
        Si <- x[[i]]
        Si.sd <- attr(Si, "stddev")
        Si.cor <- attr(Si, "correlation")
        nci <- ncol(Si)
        cnmsi <- colnames(Si)
        jj <- seq.int(from = 1L, by = nci + 1L, length.out = nci)
        ij1 <- sequence.default(from = jj, nvec = nci:1L)
        ij0 <- sequence.default(from = jj + 1L, nvec = (nci - 1L):0L)
        if (order == "cov.last")
            data.frame(grp = names(x)[i],
                       var1 = cnmsi[c(1L:nci,
                                      1L + (ij0 - 1L) %/% nci)],
                       var2 = cnmsi[c(rep(NA_integer_, nci),
                                      1L + (ij0 - 1L) %% nci)],
                       vcov = Si[c(jj, ij0)],
                       sdcor = c(Si.sd, Si.cor[ij0]),
                       row.names = NULL,
                       stringsAsFactors = FALSE)
        else {
            jj. <- cumsum(c(1L, if (nci > 1L) nci:2L))
            data.frame(grp = names(x)[i],
                       var1 = cnmsi[1L + (ij1 - 1L) %/% nci],
                       var2 = `[<-`(cnmsi[1L + (ij1 - 1L) %% nci],
                                    jj., NA_character_),
                       vcov = Si[ij1],
                       sdcor = `[<-`(Si.cor[ij1], jj., Si.sd),
                       row.names = NULL,
                       stringsAsFactors = FALSE)
        }
    })
    if (attr(x, "useSc")) {
        sc <- attr(x, "sc")
        r <- data.frame(grp = "Residual",
                        var1 = NA_character_,
                        var2 = NA_character_,
                        vcov = sc * sc,
                        sdcor = sc,
                        row.names = NULL,
                        stringsAsFactors = FALSE)
        dlist <- c(dlist, list(r))
    }
    do.call(rbind, dlist)
}

print.VarCorr.merMod <-
function(x, digits = max(3L, getOption("digits") - 2L),
         comp = "Std.Dev.", formatter = format, ...) {
    y <- formatVC(x, digits = digits, comp = comp, formatter = formatter)
    print(y, quote = FALSE)
    invisible(x)
}

if (.Platform$OS.type != "windows") withAutoprint({

    library(lme4)
    source(system.file("testdata", "lme-tst-funs.R", package="lme4", mustWork=TRUE))# -> unn()

    m <- lmer(Reaction ~ Days + (Days | Subject), sleepstudy)
    bruteForceHat <- function(object) {
        with(getME(object, c("Lambdat", "Lambda", "Zt", "Z", "q", "X")), {
            ## cp:= the cross product block matrix in (17) and (18):
            W <- Diagonal(x = weights(object))
            I <- Diagonal(q)
            A.21 <- t(X) %*% W %*% Z %*% Lambda
            cp <- rbind(cbind(Lambdat %*% Zt %*% W %*% Z %*% Lambda + I, t(A.21)),
                        cbind(A.21, t(X) %*% W %*% X))
            mm <- cbind(Z %*% Lambda, X)
            ## a bit efficient: both cp and mm are typically quite sparse
            ## mm %*% solve(as.matrix(cp)) %*% t(mm)
            mm %*% solve(cp, t(mm), sparse=FALSE)
        })
    }


    str(H <- bruteForceHat(m))

    set.seed(7)
    ii <- sample(nrow(sleepstudy), 500, replace=TRUE)
    m2 <- lmer(Reaction ~ Days + (Days | Subject), sleepstudy[ii, ])

    
    stopifnot(all.equal(diag(H),
                        unn(hatvalues(m)),  tol= 1e-14),
              all.equal(diag(bruteForceHat(m2)),
                        unn(hatvalues(m2)), tol= 1e-14)
              )
}) ## skip on windows (for speed)

library(lme4)
## This example takes  long : only for testLevel >= 3 :
d.ok <- isTRUE(try(data(star, package = 'mlmRev')) == 'star')
if(!interactive() && (lme4:::testLevel() < 3 || !d.ok))
    q("no")

## This worked in an *older* version of lme4.0 
## fm1 <- lme4:::carryOver(math ~ gr+sx*eth+cltype+(yrs|id)+(1|tch)+(yrs|sch),
##                         star, yrs ~ tch/id,
##                         control = list(msV = 1, nit = 0, grad = 0))

system.time(
    fm1 <- lmer(math ~ gr + sx*eth + cltype + schtype + hdeg + clad + exp + trace +
                (yrs | id) + (1 | tch) + (yrs | sch), data = star, verbose = TRUE)
)
##   user  system elapsed 
## 34.991   0.037  35.132 -- lme4.0
## 36.599   0.031  36.745 -- lme4 {bobyqa; 2014-01-09 @ lynne}

sm1 <- summary(fm1)
print(sm1, corr=TRUE, symbolic.cor=TRUE)# now message *and* gives the correlation

## simple examples with offsets, to exercise methods etc.

library(lme4)

if (.Platform$OS.type != "windows") {
## generate a basic Gamma/random effects sim
set.seed(101)
d <- expand.grid(block=LETTERS[1:26],rep=1:100)
d$x <- runif(nrow(d))  ## sd=1
reff_f <- rnorm(length(levels(d$block)),sd=1)
## need intercept large enough to avoid negative values
d$eta0 <- 4+3*d$x  ## version without random effects
d$eta <- d$eta0+reff_f[d$block]

## lmer() test:
d$mu <- d$eta
d$y <- rnorm(nrow(d),mean=d$mu,sd=1)

fm1    <- lmer(y~x+(1|block),            data=d)
fm1off <- lmer(y~x+(1|block)+offset(3*x),data=d)

## check equality
stopifnot(all.equal(fixef(fm1)[2]-3,fixef(fm1off)[2]))

p0 <- predict(fm1)
p1 <- predict(fm1,newdata=d)
p2 <- predict(fm1off,newdata=d)
stopifnot(all.equal(p0,p1),
          all.equal(p1,p2))


## glmer() test:
d$mu <- exp(d$eta)
d$y <- rpois(nrow(d),d$mu)

gm1    <- glmer(y~x+(1|block),            data=d,family=poisson,
                control=glmerControl(check.conv.grad="ignore"))
gm1off <- glmer(y~x+(1|block)+offset(3*x),data=d,family=poisson,
                control=glmerControl(check.conv.grad="ignore"))

## check equality
stopifnot(all.equal(fixef(gm1)[2]-3,fixef(gm1off)[2],tolerance=3e-4))

p0 <- predict(gm1)
p1 <- predict(gm1,newdata=d)
p2 <- predict(gm1off,newdata=d)
stopifnot(all.equal(p0,p1),
          all.equal(p1,p2))

## FIXME: should also test simulations
} ## skip on windows (for speed)

## utilities, these *exported*:
##' @export getL
setGeneric("getL", function(x) standardGeneric("getL"))

fixed.effects <- function(object, ...) {
    ## fixed.effects was an alternative name for fixef
    .Deprecated("fixef")
    mCall = match.call()
    mCall[[1]] = as.name("fixef")
    eval(mCall, parent.frame())
}

random.effects <- function(object, ...) {
    ## random.effects was an alternative name for ranef
    .Deprecated("ranef")
    mCall = match.call()
    mCall[[1]] = as.name("ranef")
    eval(mCall, parent.frame())
}

## Create a Markov chain Monte Carlo sample from the posterior
## distribution of the parameters
##
##
## @title Create an MCMC sample
## @param object a fitted model object
## @param n number of samples to generate.  Defaults to 1; for real use values of 200-1000 are more typical
## @param verbose should verbose output be given?
## @param ... additional, optional arguments (not used)
## @return a Markov chain Monte Carlo sample as a matrix
mcmcsamp <- function(object, n = 1L, verbose = FALSE, ...) UseMethod("mcmcsamp")

if(getRversion() < "3.3") {
    sigma <- function(object, ...) UseMethod("sigma")
}

isREML <- function(x, ...) UseMethod("isREML")

isLMM <- function(x, ...) UseMethod("isLMM")
isGLMM <- function(x, ...) UseMethod("isGLMM")
isNLMM <- function(x, ...) UseMethod("isNLMM")


##' Refit a model using the maximum likelihood criterion
##'
##' This function is primarily used to get a maximum likelihood fit of
##' a linear mixed-effects model for an \code{\link{anova}} comparison.
##'
##' @title Refit a model by maximum likelihood criterion
##' @param x a fitted model, usually of class \code{"\linkS4class{lmerMod}"},
##'     to be refit according to the maximum likelihood criterion
##' @param ... optional additional parameters.  None are used at present.
##' @return an object like \code{x} but fit by maximum likelihood
##' @export
refitML <- function(x, ...) UseMethod("refitML")

refit <- function(object, newresp, ...) UseMethod("refit")

if (FALSE) {
setGeneric("HPDinterval",
           function(object, prob = 0.95, ...) standardGeneric("HPDinterval"))
}

stopifnot(require("testthat"), require("lme4"))

#context("NA (and Inf) handling")

## Modified sleepstudy data :
sleepst.a <- sleepstudy
rownames(sleepst.a) <- paste0("a", rownames(sleepstudy))
sleepstudyNA  <- within(sleepst.a,  Reaction[1:3] <- NA)
sleepstudyNA2 <- within(sleepst.a,  Days[1:3] <- NA)
sleepInf      <- within(sleepstudy, Reaction[Reaction > 400] <- Inf)

## Modified  cake  data :
cakeNA <- rbind(cake, tail(cake,1))
cakeNA[nrow(cakeNA), "angle"] <- NA
## Create new data frame with some NAs in fixed effect
cakeNA.X <- within(cake, temp[1:5] <- NA)
## NA values in random effects -- should get treated
cakeNA.Z <- within(cake, replicate[1:5] <- NA)

test_that("naming", {
    ## baseline model
    fm1 <- lmer(Reaction~Days+(Days|Subject), sleepst.a)
    ## default: na.omit
    fm2 <- update(fm1, data=sleepstudyNA,
                  control=lmerControl(check.conv.grad="ignore"))
    expect_equal(head(names(fitted(fm1))), paste0("a",1:6))
    expect_equal(head(names(fitted(fm2))), paste0("a",4:9))
    expect_equal(names(predict(fm2)), names(fitted(fm2)))
    expect_equal(length(p1 <- predict(fm2)), 177)
    ## predict with na.exclude -> has 3 NA's, but otherwise identical:
    expect_equal(length(p2 <- predict(fm2, na.action=na.exclude)), 180)
    expect_identical(p1, p2[!is.na(p2)])
    expect_equal(length((s1 <- simulate(fm1,1))[[1]]),180)
    expect_equal(length((s2 <- simulate(fm2,1))[[1]]),177)
    expect_equal(head(rownames(s1)),paste0("a",1:6))
    expect_equal(head(rownames(s2)),paste0("a",4:9))

    ## test simulation
    expect_is(attr(simulate(fm2),"na.action"),"omit")
    expect_is(refit(fm2,simulate(fm2)),"merMod")
    expect_equal(fixef(fm2),
                 fixef(refit(fm2, sleepstudyNA$Reaction)), tolerance = 1e-5)
    fm2ex <- update(fm2, na.action=na.exclude)
    expect_equal(nrow(ss2 <- simulate(fm2ex)),180)
    expect_is(refit(fm2,ss2[[1]]),"merMod")
    ## issue #197, 18 new subjects; some with NA in y
    d2 <- sleepstudyNA[c(1:180, 1:180),]
    d2[,"Subject"] <- factor(rep(1:36, each=10))
    d2[d2$Subject == 19, "Reaction"] <- NA
    expect_equal(dim( simulate(fm1, newdata=d2, allow.new.levels=TRUE) ), c(360,1))

    ## na.pass (pretty messed up)
    expect_error(update(fm1,data=sleepstudyNA,
                  control=lmerControl(check.conv.grad="ignore"),
                  na.action=na.pass),
                 "NA/NaN/Inf in 'y'")
    sleepstudyNA2 <- within(sleepst.a, Days[1:3] <- NA)
    expect_error(fm4 <- update(fm1, data = sleepstudyNA2,
                               control=lmerControl(check.conv.grad="ignore"),
                               na.action=na.pass),"NA in Z")
    expect_is(suppressWarnings(confint(fm2,method="boot",nsim=3,
                                       quiet=TRUE)),"matrix")
    expect_error(update(fm1, data = sleepstudyNA2,
                        control = lmerControl(check.conv.grad="ignore"),
                        na.action = na.pass),
                 "NA in Z")
    expect_is(suppressWarnings(
                  ci2 <- confint(fm2, method="boot", nsim=3, quiet=TRUE)), "matrix")
})

test_that("other_NA", {
    expect_error(lmer(Reaction ~ Days + (Days | Subject), sleepInf), "\\<Inf\\>")

    fm0 <- lmer(angle ~ recipe * temperature + (1|recipe:replicate), cake)
    ## NA's in response :
    fm1 <- update(fm0, data = cakeNA)
    expect_true(all.equal(  fixef(fm0),  fixef(fm1)))
    expect_true(all.equal(VarCorr(fm0),VarCorr(fm1)))
    expect_true(all.equal(  ranef(fm0),  ranef(fm1)))

    fm1_omit <-  update(fm1, na.action = na.omit)
    fm1_excl <-  update(fm1, na.action = na.exclude)
    expect_error(update(fm1, na.action = na.pass), "NA/NaN")
    expect_error(update(fm1, na.action = na.fail), "missing values in object")
    fm1_omit@call <- fm1@call ## <- just for comparing:
    expect_equal(fm1, fm1_omit)
    expect_equal(length(fitted(fm1_omit)), 270)
    expect_equal(length(fitted(fm1_excl)), 271)
    expect_true(is.na(tail(predict(fm1_excl),1)))

    ## test predict.lm
    d <- data.frame(x = 1:10, y = c(rnorm(9),NA))
    lm1 <- lm(y~x, data=d, na.action=na.exclude)
    expect_is(predict(lm1), "numeric")
    expect_equal(1, sum(is.na(predict(lm1, newdata = data.frame(x=c(1:4,NA))))))

    ## Triq examples ...
    m.lmer <- lmer (angle ~ temp + (1 | recipe) + (1 | replicate), data=cake)
    ## NAs in fixed effect
    p1_pass <- predict(m.lmer, newdata=cakeNA.X, re.form=NA,
                       na.action=na.pass)
    expect_true(length(p1_pass)==nrow(cakeNA.X))
    expect_true(all(is.na(p1_pass[1:5])))
    p1_omit <- predict(m.lmer, newdata=cakeNA.X, re.form=NA,
                       na.action=na.omit)
    p1_exclude <- predict(m.lmer, newdata=cakeNA.X, re.form=NA,
                          na.action=na.exclude)
    expect_true(length(p1_omit)==nrow(na.omit(cakeNA.X)))
    expect_true(length(p1_exclude)==nrow(cakeNA.X))
    expect_true(all.equal(c(na.omit(p1_exclude)),p1_omit))
    expect_error(predict(m.lmer, newdata=cakeNA.X, re.form=NA, na.action=na.fail),
                 "missing values in object")

    ## now try it with re.form==NULL
    p2_pass <- predict(m.lmer, newdata=cakeNA.X, re.form=NULL,
                       na.action=na.pass)
    expect_true(length(p2_pass)==nrow(cakeNA.X))
    expect_true(all(is.na(p2_pass[1:5])))
    p2_omit <- predict(m.lmer, newdata=cakeNA.X, re.form=NULL,
                       na.action=na.omit)
    p2_exclude <- predict(m.lmer, newdata=cakeNA.X, re.form=NULL,
                          na.action=na.exclude)
    expect_true(length(p2_omit)==nrow(na.omit(cakeNA.X)))
    expect_true(all.equal(c(na.omit(p2_exclude)),p2_omit))
    expect_error(predict(m.lmer, newdata=cakeNA.X, re.form=NULL, na.action=na.fail),
                 "missing values in object")

    ## experiment with NA values in random effects -- should get treated
    expect_error(predict(m.lmer, newdata=cakeNA.Z, re.form=NULL),
                "NAs are not allowed in prediction data")
    p4 <- predict(m.lmer, newdata=cakeNA.Z, re.form=NULL,
                  allow.new.levels=TRUE)
    p4B <- predict(m.lmer, newdata=cakeNA.Z, re.form=~1|recipe,
                   allow.new.levels=TRUE)
    expect_true(all.equal(p4[1:5],p4B[1:5]))
    p4C <- predict(m.lmer, newdata=cakeNA.Z, re.form=NA)

    d <- data.frame(x=runif(100),f=factor(rep(1:10,10)))
    set.seed(101)
    u <- rnorm(10)
    d <- transform(d,y=rnorm(100,1+2*x+u[f],0.2))
    d0 <- d
    d[c(3,5,7),"x"] <- NA

    ## 'omit' and 'exclude' are the only choices under which
    ##  we will see NA values in the results
    fm0 <- lmer(y~x+(1|f), data=d0)
    ## no 'na.action' attribute because no NAs in this data set
    expect_equal(attr(model.frame(fm0),"na.action"),NULL)
    fm1 <- update(fm0, data=d)
    ## no NAs in predict or residuals because na.omit
    expect_false(any(is.na(predict(fm1))))
    expect_false(any(is.na(residuals(fm1))))
    fm2 <- update(fm1,na.action="na.exclude")
    ## no NAs in predict or residuals because na.omit
    nNA <- sum(is.na(d$x))
    expect_equal(sum(is.na(predict(fm2))),nNA)
    expect_equal(sum(is.na(residuals(fm2))),nNA)
    expect_error(fm3 <- lmer(y~x+(1|f), data=d, na.action="na.pass"),
                 "(Error in qr.default|NA/NaN/Inf in foreign function call)")
    expect_is(refit(fm0),"merMod")
    expect_is(refit(fm1),"merMod")
    expect_is(refit(fm2),"merMod")

    ## GH 420: NAs in training data should *not* get
    ##  carried over into predictions!
    fm4 <- lmer(Reaction~Days+(1|Subject),sleepstudyNA2)
    pp4 <- predict(fm4,newdata=sleepstudy)
    expect_equal(length(pp4),nrow(sleepstudy))
    expect_equal(sum(is.na(pp4)),0)
})

test_that("NAs in fitting data ignored in newdata with random.only=TRUE",
          {
              set.seed(101)
              dd <- data.frame(x=c(rnorm(199),NA),y=rnorm(200),
                               f=factor(rep(1:10,each=20)),
                               g=factor(rep(1:20,each=10)))
              m1 <- lmer(y~x+(1|f)+(1|g),data=dd,na.action=na.exclude)
              expect_equal(length(predict(m1,newdata=dd[1:5,],random.only=TRUE)),5)
              nd.NA <- dd[1:5,]
              nd.NA$x[5] <- NA
              ## ?? not *quite* sure what should happen here ...
              predict(m1,newdata=nd.NA,random.only=TRUE)
          })

## There are lots of problems with nlmer.
## If it eventually gets fixed, comparing the covariance structures between
## lme4::nlmer and the nlme::nlme may be worthwhile. 
## The following tests are skipped since (most) of them fail.

other_mod <- readRDS(
  system.file("testdata", "test-covariance_structures_data.rds", package = "lme4")
)

## lme4 nonlinear mixed effects model
startvec <- c(Asym = 200, xmid = 725, scal = 350)
nm <-  nlmer(circumference ~ SSlogis(age, Asym, xmid, scal) ~ Asym|Tree,
             Orange, start = startvec)

nm.us <- nlmer(circumference ~ SSlogis(age, Asym, xmid, scal) ~ us(Asym|Tree),
               Orange, start = startvec)

nm.cs <- nlmer(circumference ~ SSlogis(age, Asym, xmid, scal) ~ cs(Asym|Tree),
               Orange, start = startvec)

test_that("nlmer tests", {
  skip()
  ## Ensuring unstructured covariance results are the same as default
  expect_equal(coef(nm), coef(nm.us))
  expect_equal(fixef(nm), fixef(nm.us))
  ## Ensuring unstructured covariance results are the same as default
  expect_equal(fixef(nm), fixef(nm.us))
  
  # integration tests for sigma
  expect_equal(sigma(nm), sigma(nm.us))
  expect_equal_nocheck(sigma(nm), other_mod$nm.nlme_sigma, tolerance = 5e-4)
  expect_equal_nocheck(sigma(nm.cs), other_mod$nm.nlme.cs_sigma)

  ## Log likelihood tests 
  expect_equal(logLik(nm), logLik(nm.us))
  expect_equal_nocheck(logLik(nm.us), other_mod$nm.nlme_logLik)
  expect_equal_nocheck(logLik(nm.cs), other_mod$nm.nlme.cs_logLik)
  
  ## Integration tests for vcov
  nm_vcov <- suppressWarnings(vcov(nm))
  nm.us_vcov <- suppressWarnings(vcov(nm.us))
  expect_equal(nm_vcov, nm.us_vcov)
  expect_equal_nocheck(nm_vcov, other_mod$nm.nlme_vcov)
  nm.cs_vcov <- suppressWarnings(vcov(nm.cs))
  expect_equal_nocheck(nm.cs_vcov, other_mod$nm.nlme.cs_vcov)
  
  ## Integration tests for ranef
  expect_equal(ranef(nm), ranef(nm.us))
  expect_equal_nocheck(as.matrix(other_mod$nm.nlme_ranef), 
                         as.matrix(ranef(nm)$Tree))
  expect_equal_nocheck(as.matrix(other_mod$nm.nlme.cs_ranef), 
                         as.matrix(ranef(nm.cs)$Tree))
  
  ## Integration tests for predict
  expect_equal(predict(nm), predict(nm.us))
  expect_equal_nocheck(other_mod$nm.nlme_predict, predict(nm))
  expect_equal_nocheck(other_mod$nm.nlme.cs_predict, predict(nm.cs))
})

### Plots for the ranef.mer class ----------------------------------------

##' @importFrom lattice dotplot
##' @S3method  dotplot ranef.mer
dotplot.ranef.mer <- function(x, data, main = TRUE, transf=I, level = 0.95, ...)
{
    rng <- qnorm((1+level)/2)
    prepanel.ci <- function(x, y, se, subscripts, ...) {
        if (is.null(se)) return(list())
        x <- as.numeric(x)
        hw <- rng * as.numeric(se[subscripts])
        list(xlim = range(transf(x - hw), transf(x + hw), finite = TRUE))
    }
    panel.ci <- function(x, y, se, subscripts, pch = 16,
                         horizontal = TRUE, col = dot.symbol$col, 
                         lty.h = dot.line$lty, lty.v = dot.line$lty, 
                         lwd.h = dot.line$lwd, lwd.v = dot.line$lwd, 
                         col.line.h = dot.line$col, col.line.v = dot.line$col, 
                         levels.fos = unique(y),
                         groups = NULL, ...)
    {
        x <- as.numeric(x)
        y <- as.numeric(y)
        dot.line <- trellis.par.get("dot.line")
        dot.symbol <- trellis.par.get("dot.symbol")
        sup.symbol <- trellis.par.get("superpose.symbol")
        panel.abline(h = levels.fos, col = col.line.h, lty = lty.h, lwd = lwd.h)
        panel.abline(v = 0, col = col.line.v, lty = lty.v, lwd = lwd.v)
        if (!is.null(se)) {
            se <- as.numeric(se[subscripts])
            panel.segments( transf(x - rng * se), y,
                            transf(x + rng * se), y, col = 'black')
        }
        panel.xyplot(transf(x), y, pch = pch, col = col, ...)
    }
    f <- function(nx, ...) {
        ss <- asDf0(x,nx)
        mtit <- if(main) nx 
        dotplot(.nn ~ values | ind, ss, se = ss$se,
                prepanel = prepanel.ci, panel = panel.ci,
                xlab = NULL, main = mtit, ...)
    }
    setNames(lapply(names(x), f, ...), names(x))
}

##' @importFrom graphics plot
##' @S3method plot ranef.mer
plot.ranef.mer <- function(x, y, ...)
{
    lapply(x, function(x) {
        cn <- lapply(colnames(x), as.name)
        switch(min(ncol(x), 3),
               qqmath(eval(substitute(~ x, list(x = cn[[1]]))), x, ...),
               xyplot(eval(substitute(y ~ x,
                                      list(y = cn[[1]],
                                           x = cn[[2]]))), x, ...),
               splom(~ x, ...))
    })
}

##' @importFrom lattice qqmath
##' @S3method qqmath ranef.mer
qqmath.ranef.mer <- function(x, data, main = TRUE, level = 0.95, ...)
{
    rng <- qnorm((1+level)/2)
    prepanel.ci <- function(x, y, se, subscripts, ...) {
        x <- as.numeric(x)
        se <- as.numeric(se[subscripts])
        hw <- rng * se
        list(xlim = range(x - hw, x + hw, finite = TRUE))
    }
    panel.ci <- function(x, y, se, subscripts, pch = 16, ...)  {
        panel.grid(h = -1,v = -1)
        panel.abline(v = 0)
        x <- as.numeric(x)
        y <- as.numeric(y)
        se <- as.numeric(se[subscripts])
        panel.segments(x - rng * se, y, x + rng * se, y, col = 'black')
        panel.xyplot(x, y, pch = pch, ...)
    }
    f <- function(nx) {
        xt <- x[[nx]]
        mtit <- if(main) nx # else NULL
        if (!is.null(pv <- attr(xt, "postVar")))
        {
            d <- dim(pv)
            se <- vapply(seq_len(d[1]), function(i) sqrt(pv[i, i, ]), numeric(d[3]))
            nr <- nrow(xt)
            nc <- ncol(xt)
            ord <- unlist(lapply(xt, order)) + rep((0:(nc - 1)) * nr, each = nr)
            rr <- 1:nr
            ind <- gl(nc, nr, labels = names(xt))
            xyplot(rep(qnorm((rr - 0.5)/nr), nc) ~ unlist(xt)[ord] | ind[ord],
                   se = se[ord], prepanel = prepanel.ci, panel = panel.ci,
                   scales = list(x = list(relation = "free")),
                   ylab = "Standard normal quantiles",
                   xlab = NULL, main = mtit, ...)
        } else {
            qqmath(~values|ind,
                   data = stack(xt),
                   scales = list(y = list(relation = "free")),
                   xlab = "Standard normal quantiles",
                   ylab = NULL, main = mtit, ...)
        }
    }
    sapply(names(x), f, simplify = FALSE)
}

##' @importFrom graphics plot
##' @S3method plot coef.mer
plot.coef.mer <- function(x, y, ...)
{
    ## remove non-varying columns from frames
    reduced <- lapply(x, function(el)
                      el[, !vapply(el, function(cc) all(cc == cc[1L]), NA)])
    plot.ranef.mer(reduced, ...)
}

##' @importFrom lattice dotplot
##' @S3method dotplot coef.mer
dotplot.coef.mer <- function(x, data, ...) {
    mc <- match.call()
    mc[[1]] <- as.name("dotplot.ranef.mer")
    eval(mc)
}

## data set and formula extracted from ?prLogisticDelta example
##   (Thailand, clustered-data) in prLogistic package
load(system.file("testdata","prLogistic.RData",package="lme4"))
library(lme4)

(testLevel <- lme4:::testLevel())
if (testLevel > 2) {
    print(system.time(
    lme4_est <- glmer(rgi ~ sex + pped + (1|schoolid),
                      data = dataset, family=binomial)
    ))
    lme4_results <- list(sigma= sqrt(unname(unlist(VarCorr(lme4_est)))),
                         beta = fixef(lme4_est))

    ## stored results from other pkgs
    glmmML_est <- list(sigma = 1.25365353546143,
                       beta = c("(Intercept)" = -2.19478801858317,
                           "sex" = 0.548884468743364, "pped"= -0.623835613907385))
    lme4.0_est <- list(sigma = 1.25369539060849,
                       beta = c("(Intercept)" = -2.19474529099587,
                           "sex" = 0.548900267825802, "pped"= -0.623934772981894))

    source(system.file("test-tools-1.R", package = "Matrix"))#-> assert.EQ() etc
    assert.EQ.(lme4_results, glmmML_est, tol=3e-3)
    assert.EQ.(lme4_results, lme4.0_est, tol=3e-3)
    print(lme4_est)
}

## transient patches for moving formula manipulation machinery to lme4 without breaking downstream packages

## tracks which one-time deprecation warnings have already fired this session
.reformulas_warned <- new.env(parent = emptyenv())

warn_once <- function(msg, id) {
  if (!exists(id, envir = .reformulas_warned, inherits = FALSE)) {
    assign(id, TRUE, envir = .reformulas_warned)
    warning(msg, call. = FALSE)
  }
}

mkWarnFun <- function(FUN) {
  fn <- function(...) {
    msg <- sprintf("the %s function has moved to the reformulas package. Please update your imports, or ask an upstream package maintainer to do so.", sQuote(FUN))
    warn_once(msg, FUN)
    reformulas_fun <- getExportedValue("reformulas", FUN)
    reformulas_fun(...)
  }
  assign(FUN, fn, envir = parent.frame())
}

for (f in c("findbars","subbars", "nobars",
            "mkReTrms", "expandDoubleVerts", "isNested")) {
  mkWarnFun(f)
}

## handle reOnly separately, it needs to be wrapped to return a formula properly
## reformulas 0.4.5 will fix this (can then move back to the deprecation block above)
reOnly <- function(...) {
  res <- lme4_reOnly(...)
}

## these tests may fail if run a second time in the same session, because of rlang only-report-once-per-session setting
f <- ~ 1 + (1|f)
test_that("deprecation warnings from lme4 formula processing", {
  expect_warning(subbars(f), "has moved")
  expect_warning(nobars(f), "has moved")
  expect_warning(findbars(f), "has moved")
  expect_warning(mkReTrms(findbars(f), fr = data.frame(f = factor(1:10))), "has moved")
  expect_warning(expandDoubleVerts(f), "has moved")
})

test_that("reOnly shims", {
  reo <- reOnly(Reaction ~ 1 + (1|Subject))
  expect_equal(reo, ~(1 | Subject))
})

## In both of these cases boundary fit (i.e. estimate of zero RE
## variance) is *incorrect*. (Nelder_Mead, restart_edge=FALSE) is the
## only case where we get stuck; either optimizer=bobyqa or
## restart_edge=TRUE (default) works
if (.Platform$OS.type != "windows") {

    library(lme4)
    library(testthat)

    if(!dev.interactive(orNone=TRUE)) pdf("boundary_plots.pdf")

    ## Stephane Laurent:
    dat <- read.csv(system.file("testdata","dat20101314.csv", package="lme4"))

    fit   <- lmer(y ~ (1|Operator)+(1|Part)+(1|Part:Operator), data=dat,
                  control= lmerControl(optimizer="Nelder_Mead"))
    fit_b <- lmer(y ~ (1|Operator)+(1|Part)+(1|Part:Operator), data=dat,
                  control= lmerControl(optimizer="bobyqa", restart_edge=FALSE))
    fit_c <- lmer(y ~ (1|Operator)+(1|Part)+(1|Part:Operator), data=dat,
                  control= lmerControl(optimizer="Nelder_Mead", restart_edge=FALSE,
                                       check.conv.hess="ignore"))
    ## final fit gives degenerate-Hessian warning
    ## FIXME: use fit_c with expect_warning() as a check on convergence tests
    ## tolerance=1e-5 seems OK in interactive use but not in R CMD check ... ??
    stopifnot(all.equal(getME(fit,  "theta") -> th.f,
                        getME(fit_b,"theta"), tolerance=5e-5),
              all(th.f > 0))

    ## Manuel Koller

    source(system.file("testdata", "koller-data.R", package="lme4"))
    ldata <- getData(13)
    ## old (backward compatible/buggy)
    fm4  <- lmer(y ~ (1|Var2), ldata, control=lmerControl(optimizer="Nelder_Mead",
                                                          use.last.params=TRUE),
                 start=list(theta=1))

    fm4b <- lmer(y ~ (1|Var2), ldata,
                 control = lmerControl(optimizer="Nelder_Mead", use.last.params=TRUE,
                                       restart_edge = FALSE,
                                       check.conv.hess="ignore", check.conv.grad="ignore"),
                 start = list(theta=1))
    ## FIXME: use as convergence test check
    stopifnot(getME(fm4b,"theta") == 0)
    fm4c <- lmer(y ~ (1|Var2), ldata, control=lmerControl(optimizer="bobyqa",
                                                          use.last.params=TRUE),
                 start=list(theta=1))
    stopifnot(all.equal(getME(fm4, "theta") -> th4,
                        getME(fm4c,"theta"), tolerance=1e-4),
              th4 > 0)


    ## new: doesn't get stuck at edge any more,  but gets stuck somewhere else ...
    fm5 <- lmer(y ~ (1|Var2), ldata, control=lmerControl(optimizer="Nelder_Mead",
                                                         check.conv.hess="ignore",
                                                         check.conv.grad="ignore"),
                start=list(theta=1))
    fm5b <- lmer(y ~ (1|Var2), ldata, control=lmerControl(optimizer="Nelder_Mead",
                                                          restart_edge=FALSE,
                                                          check.conv.hess="ignore",
                                                          check.conv.grad="ignore"),
                 start = list(theta = 1))
    fm5c <- lmer(y ~ (1|Var2), ldata, control=lmerControl(optimizer="bobyqa"),
                 start = list(theta = 1))
    stopifnot(all.equal(unname(getME(fm5c,"theta")), 0.21067645, tolerance = 1e-7))
					#	 0.21067644264 [64-bit, lynne]

    if (require("optimx")) {
    ## additional stuff for diagnosing Nelder-Mead problems.

    fm5d <- update(fm5,control=lmerControl(optimizer="optimx",
                                           optCtrl=list(method="L-BFGS-B")))

    fm5e <- update(fm5, control=lmerControl(optimizer="nloptwrap"))

    mList <- setNames(list(fm4,fm4b,fm4c,fm5,fm5b,fm5c,fm5d,fm5e),
                      c("NM/uselast","NM/uselast/norestart","bobyqa/uselast",
                        "NM","NM/norestart","bobyqa","LBFGSB","nloptr/bobyqa"))
    pp <- profile(fm5c,which=1)
    dd <- as.data.frame(pp)
    par(las=1,bty="l")
    v <- sapply(mList,
                function(x) sqrt(VarCorr(x)[[1]]))
    plot(.zeta^2~.sig01, data=dd, type="b")
    abline(v=v)

    res <- cbind(VCorr  = sapply(mList, function(x) sqrt(VarCorr(x)[[1]])),
                 theta  = sapply(mList, getME,"theta"),
                 loglik = sapply(mList, logLik))
    res
    print(sessionInfo(), locale=FALSE)
    }

######################
    library(lattice)
    ## testing boundary and near-boundary cases

    tmpf <- function(i,...) {
        set.seed(i)
        d <- data.frame(x=rnorm(60),f=factor(rep(1:6,each=10)))
        d$y <- simulate(~x+(1|f),family=gaussian,newdata=d,
                        newparams=list(theta=0.01,beta=c(1,1),sigma=5))[[1]]
        lmer(y~x+(1|f),data=d,...)
    }
    sumf <- function(m) {
        unlist(VarCorr(m))[1]
    }
    if (FALSE) {
        ## figuring out which seeds will give boundary and
        ## near-boundary solutions
        mList <- lapply(1:201,tmpf) # [FIXME tons of messages "theta parameters vector not named"]
        ss <- sapply(mList,sumf)+1e-50
        par(las=1,bty="l")
        hist(log(ss),col="gray",breaks=50)
        ## values lying on boundary
        which(log(ss)<(-40))   ## 5, 7-13, 15, 21, ...
        ## values close to boundary (if check.edge not set)
        which(log(ss)>(-40) & log(ss) <(-20))  ## 16, 44, 80, 86, 116, ...
    }
    ## diagnostic plot
    tmpplot <- function(i, FUN=tmpf) {
        dd <- FUN(i, devFunOnly=TRUE)
        x <- 10^seq(-10,-6.5,length=201)
        dvec <- sapply(x,dd)
        op <- par(las=1,bty="l"); on.exit(par(op))
        plot(x,dvec-min(dvec)+1e-16, log="xy", type="b")
        r <- FUN(i)
        abline(v = getME(r,"theta"), col=2)
        invisible(r)
    }

    ## Case #1: boundary estimate with or without boundary.tol
    m5  <- tmpf(5)
    m5B <- tmpf(5,control=lmerControl(boundary.tol=0))
    stopifnot(getME(m5, "theta")==0,
              getME(m5B,"theta")==0)
    p5 <- profile(m5)  ## bobyqa warnings but results look reasonable
    xyplot(p5)
    ## reveals slight glitch (bottom row of plots doesn't look right)
    expect_warning(splom(p5),"unreliable for singular fits")
    p5B <- profile(m5, signames=FALSE) # -> bobyqa convergence warning (code 3)
    expect_warning(splom(p5B), "unreliable for singular fits")

    if(lme4:::testLevel() >= 2) { ## avoid failure to warn
        ## Case #2: near-boundary estimate, but boundary.tol can't fix it
        m16 <- tmpplot(16)
        ## sometimes[2014-11-11] fails (??) :
        p16 <- profile(m16)  ## warning message*s* (non-monotonic profile and more)
        plotOb <- xyplot(p16)
        ## NB: It's the print()ing of 'plotOb' which warns ==> need to do this explicitly:
        expect_warning(print(plotOb), ## warns about linear interpolation in profile for variable 1
                       "using linear interpolation")
        d16 <- as.data.frame(p16)
        xyplot(.zeta ~ .focal|.par, data=d16, type=c("p","l"),
               scales = list(x=list(relation="free")))
        try(splom(p16))  ## breaks when calling predict(.)
    }

    ## bottom line:
    ##  * xyplot.thpr could still be improved
    ##  * most of the near-boundary cases are noisy and can't easily be
    ##    fixed

    tmpf2 <- function(i,...) {
        set.seed(i)
        d <- data.frame(x=rnorm(60),f=factor(rep(1:6,each=10)),
                        w=rep(10,60))
        d$y <- simulate(~x+(1|f),family=binomial,
                        weights=d$w,newdata=d,
                        newparams=list(theta=0.01,beta=c(1,1)))[[1]]
        glmer(y~x+(1|f),data=d,family=binomial,weights=w,...)
    }

    if (FALSE) {
        ## figuring out which seeds will give boundary and
        ## near-boundary solutions
        mList <- lapply(1:201,tmpf2)
        ss <- sapply(mList,sumf)+1e-50
        par(las=1,bty="l")
        hist(log(ss),col="gray",breaks=50)
        ## values lying on boundary
        head(which(log(ss)<(-50)))   ## 1-5, 7 ...
        ## values close to boundary (if check.edge not set)
        which(log(ss)>(-50) & log(ss) <(-20))  ## 44, 46, 52, ...
    }

    ##  m1 <- tmpf2(1)

    ## FIXME: doesn't work if we generate m1 via tmpf2(1) --
    ## some environment lookup problem ...

    set.seed(1)
    d <- data.frame(x=rnorm(60),f=factor(rep(1:6,each=10)),
                    w=rep(10,60))
    d$y <- simulate(~x+(1|f),family=binomial,
                    weights=d$w,newdata=d,
                    newparams=list(theta=0.01,beta=c(1,1)))[[1]]
    m1 <- glmer(y~x+(1|f),data=d,family=binomial,weights=w)

    p1 <- profile(m1)
    xyplot(p1)
    expect_warning(splom(p1),"splom is unreliable")

} ## skip on windows (for speed)

library(lme4)

 ## setup
 ## library(polytomous)
 ##   data(think)
 ##   think.polytomous.lmer1 <-  polytomous(Lexeme ~ Agent + Patient + (1|Register),
 ##                                         data=think, heuristic="poisson.reformulation")
 ##   save("formula.poisson","data.poisson",file="polytomous_test.RData")

load(system.file("testdata","polytomous_test.RData",package="lme4"))

if (FALSE) {
    ## infinite loop
    glmer(formula.poisson,data=data.poisson,family=poisson,verbose=10)
    ## Cholmod not positive definite -> infinite loop
    glmer(formula.poisson,data=data.poisson,family=poisson,
          verbose=10,control=glmerControl(optimizer="bobyqa"))
    ## caught warning: maxfun < 10 * length(par)^2 is not recommended. -> infinite loop
}
## works but sloooow ....
if (FALSE) {
    try(g1 <- glmer(formula.poisson,data=data.poisson,family=poisson,
                    control=glmerControl(compDev=FALSE),verbose=1))
    ## runs for 2880 steps until:
    ## Error in pp$updateDecomp() : Downdated VtV is not positive definite
}

(testLevel <- lme4:::testLevel())
if (testLevel > 2) {
    glmer(formula.poisson,data=data.poisson,family=poisson,
          control=glmerControl(compDev=FALSE,optimizer="bobyqa"))
    ## caught warning: maxfun < 10 * length(par)^2 is not recommended.
    ## but runs to completion
}

## original code was designed to detect segfaults/hangs from error handling

library(lme4)
set.seed(101)
d <- expand.grid(block = LETTERS[1:26],
                 rep = 1:100)
d$x <- runif(nrow(d))
reff_f <- rnorm(length(levels(d$block)),sd=1)
## need intercept large enough to avoid negative values
d$eta0 <- 4+3*d$x  ## version without random effects
d$eta <- d$eta0+reff_f[d$block]
## inverse link
d$mu <- 1/d$eta
d$y <- rgamma(nrow(d), scale=d$mu/2, shape=2)

if (.Platform$OS.type != "windows") {
gm0     <- glmer(y ~      1|block,  d, Gamma)
gm0.A25 <- glmer(y ~      1|block,  d, Gamma, nAGQ=25L)
gm1     <- glmer(y ~ x + (1|block), d, Gamma)
gm1.A25 <- glmer(y ~ x + (1|block), d, Gamma, nAGQ=25L)

## strange things happening for logLik  ==> AIC, etc for nAGQ ???
anova(gm0, gm1)
anova(gm0, gm0.A25)
anova(gm1, gm1.A25)

summary(gm1) # "fine"
summary(gm1.A25) # Inf logLik etc ?

}

require(lme4)
# sorry for fitting yet another sleepstudy model in the tests
m <- lmer(Reaction ~ Days + (Days | Subject), sleepstudy)
ST <- getME(m, "ST")$Subject

# copied from vince dorie's simmer.R in arm:
dimension <- nrow(ST)
T <- ST
diag(T) <- rep(1, dimension)
S <- diag(diag(ST), dimension)

vc0 <- getME(m, 'sigma')^2*tcrossprod(T %*% S)
vc1 <- VarCorr(m)$Subject[,]
dimnames(vc0) <- dimnames(vc1)

all.equal(vc0, vc1, tolerance = 1e-6)

influence.merMod <- function(model, groups, data, maxfun=1000, do.coef = TRUE,
                             start=NULL,
                             parallel = c("no", "multicore", "snow"),
                             ncpus = getOption("influence.ncpus", 1L),
                             cl = NULL,
                             ncores,
                             ...) {

    if (!missing(ncores)) {
        .Deprecated(msg = "argument 'ncores' is deprecated; please use 'parallel', 'ncpus', and 'cl' instead")
        ncpus <- ncores
        parallel <- "snow"
    }

    parallel <- match.arg(parallel)

    # (parallel, ncpus, cl) -> (parallel)
    eval(initialize.parallel)

    .groups <- NULL  ## avoid false-positive code checks
    .vcov <- function(x) Matrix::as.matrix(vcov(x))

    if (...length()>0) warning("disregarded additional arguments")
    if (!do.coef) {
        ## simple/quick/trivial results
        result <- list(hat=hatvalues(model))
        class(result) <- "influence.merMod"
        return(result)
    }
    if (missing(data)) {
        data <- getCall(model)$data
        data <- if (!is.null(data)) eval(data, parent.frame())
                else stop("model did not use the data argument")
    }
    data <- as.data.frame(data) ## prevent tibble trouble
    if (missing(groups)) {
        groups <- ".case"
        data$.case <- rownames(data)
    }
    else if (length(groups) > 1){
        del.var <- paste0(groups, collapse=".")
        data[, del.var] <- apply(data, 1, paste0, collapse=".")
        groups <- del.var
    }
    unique.del <- unique(data[, groups])
    data[[".groups"]] <- data[, groups]
    if (!is.null(start)) {
      par <- start
    } else {
      par <- list(theta=getME(model, "theta"))
      if (inherits(model, "glmerMod") &&
          getME(model, "devcomp")$dims[["nAGQ"]]>0) {
        par$fixef <- fixef(model)
      }
    }
    fixed <- fixef(model)
    fixed.1 <- matrix(0, length(unique.del), length(fixed))
    rownames(fixed.1) <- unique.del
    colnames(fixed.1) <- names(fixed)
    Vs <- VarCorr(model)
    nms <- names(Vs)
    sep <- ":"
    if (length(nms) == 1) {
        nms <- ""
        sep <- ""
    }
    vc <- getME(model, "sigma")^2
    names(vc) <- "sigma^2"
    for (i in 1:length(Vs)){
        V <- Vs[[i]]
        c.names <- colnames(V)
        e.names <- outer(c.names, c.names, function(a, b) paste0("C[", a, ",", b, "]"))
        diag(e.names) <- paste0("v[", c.names, "]")
        v <- V[lower.tri(V, diag=TRUE)]
        names(v) <- paste0(nms[i], sep, e.names[lower.tri(e.names, diag=TRUE)])
        vc <- c(vc, v)
    }
    vc.1 <- matrix(0, length(unique.del), length(vc))
    rownames(vc.1) <- unique.del
    colnames(vc.1) <- names(vc)
    feval <- numeric(length(unique.del))
    converged <- logical(length(unique.del))
    vcov.1 <- vector(length(unique.del), mode="list")
    names(vcov.1) <- names(feval) <- names(converged) <- unique.del
    # control <- if (one.step){
    #     if (inherits(model, "lmerMod")) lmerControl(optimizer="optimx", optCtrl=list(method="L-BFGS-B", maxit=1))
    #     else if (inherits(model, "glmerMod")) glmerControl(optimizer="optimx", optCtrl=list(method="L-BFGS-B", maxit=1))
    # } else {
    control <- if (inherits(model, "lmerMod")) lmerControl(optCtrl=list(maxfun=maxfun))
        else if (inherits(model, "glmerMod")) glmerControl(optCtrl=list(maxfun=maxfun))
                                        # }
    deleteGroup <- function(del) {
        data$del <- del
        mod.1 <- suppressWarnings(update(model, data=data,
                                         subset=(.groups != del),
                                         start=par,
                                         control=control))

        opt <- mod.1@optinfo
        feval <- opt$feval
        converged <- opt$conv$opt == 0 && length(opt$warnings) == 0
        fixed.1 <- fixef(mod.1)
        Vs.1 <- VarCorr(mod.1)
        vc.0 <- getME(mod.1, "sigma")^2
        for (V in Vs.1){
            vc.0 <- c(vc.0, V[lower.tri(V, diag=TRUE)])
        }
        vc.1 <- vc.0
        vcov.1 <<- .vcov(mod.1)
        namedList(fixed.1, vc.1, vcov.1, converged, feval)
    }
    result <- if (parallel == "multicore") {
        parallel::mclapply(unique.del, deleteGroup, mc.cores = ncpus)
    } else if (parallel == "snow") {
        if (is.null(cl)) {
            cl <- parallel::makeCluster(ncpus)
            on.exit(parallel::stopCluster(cl))
            parallel::clusterEvalQ(cl, require("lme4"))
        }
        parallel::parLapply(cl, unique.del, deleteGroup)
    } else {
        lapply(unique.del, deleteGroup)
    }
    result <- combineLists(result)
    fixed.1 <- result$fixed.1
    rownames(fixed.1) <- unique.del
    colnames(fixed.1) <- names(fixed)
    vc.1 <- result$vc.1
    rownames(vc.1) <- unique.del
    colnames(vc.1) <- names(vc)
    feval <- as.vector(result$feval)
    converged <- as.vector(result$converged)
    vcov.1 <- result$vcov.1
    names(vcov.1) <- names(feval) <- names(converged) <- unique.del
    left <- "[-"
    right <- "]"
    if (groups == ".case") {
        groups <- "case"
    }
    nms <- c("fixed.effects", paste0("fixed.effects", left, groups, right),
             "var.cov.comps", paste0("var.cov.comps", left, groups, right),
             "vcov", paste0("vcov", left, groups, right),
             "groups", "deleted", "converged", "function.evals")
    result <- list(fixed, fixed.1, vc, vc.1,
                   .vcov(model),
                   vcov.1, groups, unique.del, converged, feval)
    names(result) <- nms
    class(result) <- "influence.merMod"
    result
}

## lookup table for influence.merMod elements
ipos <- c("fixed"=1, "fixed.sub"=2,
          "var.cov"=3, "var.cov.sub"=4,
          "vcov"=5, "vcov.sub"=6)

dfbeta.influence.merMod <- function(model, which=c("fixed", "var.cov"), ...){
    which <- match.arg(which)
    b <- model[[ipos[sprintf("%s.sub",which)]]]
    b0 <- model[[ipos[which]]]
    b - matrix(b0, nrow=nrow(b), ncol=ncol(b), byrow=TRUE)
}

dfbetas.influence.merMod <- function(model, ...){
    vList <- model[[ipos["vcov.sub"]]]
    n <- nrow(vList[[1]])
    vmat <- t(vapply(vList, function(x) sqrt(diag(x)),
                     numeric(n)))
    dfbeta(model)/vmat
}

cooks.distance.merMod <- function(model, ...) {
    p <- Matrix::rankMatrix(getME(model,"X"))
    hat <- hatvalues(model)
    dispersion <- sigma(model)^2
    res <- residuals(model,type="pearson")
    res <-  (res/(1 - hat))^2 * hat/(dispersion * p)
    res[is.infinite(res)] <- NaN
    res
}

cooks.distance.influence.merMod <- function(model, ...) {
    db <- dfbeta(model)
    n <- nrow(db)
    p <- ncol(db)
    d <- numeric(n)
    names(d) <- names(model[["vcov[-case]"]])
    vcov.inv <- (n - p)/(n*p)*solve(model$vcov)
    for (i in 1:n) {
        d[i] <- db[i, ] %*% vcov.inv %*% db[i, ]
    }
    d
}

## from ?lm.influence:
##  wt.res: a vector of _weighted_ (or for class 'glm' rather _deviance_)
##          residuals.
##
## residuals.lm gives r*sqrt(object$weights) (if non-NULL weights)
##   for type %in% c("deviance","pearson")
##
## residuals.glm gives  (y - mu) * sqrt(wts)/sqrt(object$family$variance(mu))
##
rstudent.merMod <- function (model, ...) {
    r <- residuals(model, type="deviance")
    hat <- hatvalues(model)
    pr <- residuals(model, type="pearson")
    r <- sign(r) * sqrt(r^2 + (hat * pr^2)/(1 - hat))
    r[is.infinite(r)] <- NaN
    r/sigma(model)
}

rstandard.glmerMod <- function (model,
                              infl = influence(model, do.coef = FALSE),
                              type = c("pearson", "deviance"), ...)
{
    type <- match.arg(type)
    res <- residuals(model, type = type)
    res <- res/(sigma(model) * sqrt(1 - hatvalues(model)))
    res[is.infinite(res)] <- NaN
    res
}

rstudent.glmerMod <- function(model, infl = influence(model, do.coef=FALSE), ...) {
  useSc <- model@devcomp$dims[["useSc"]]
  if (useSc) return(rstandard(model, infl, type="pearson"))
  residuals(model, type = "pearson")/
    (sigma(model)*sqrt(1 - hatvalues(model)))
}


##' @S3method sigma merMod
sigma.merMod <- function(object, ...) {
    dc <- object@devcomp
    dd <- dc$dims
    if(dd[["useSc"]])
        dc$cmp[[if(dd[["REML"]]) "sigmaREML" else "sigmaML"]] else 1.
}

##' @importFrom stats terms
##' @S3method terms merMod
terms.merMod <- function(x, fixed.only=TRUE, random.only=FALSE, ...) {
    if (missing(fixed.only) && random.only) fixed.only <- FALSE
    if (fixed.only && random.only) stop("can't specify 'only fixed' and 'only random' terms")
    tt <- attr(x@frame,"terms")
    if (fixed.only) {
        ## ... may contain 'data' (needed when formula contains .)
        tt <- terms.formula(formula(x,fixed.only=TRUE), ...)
        attr(tt,"predvars") <- attr(terms(x@frame),"predvars.fixed")
    }
    if (random.only) {
        ff <- reformulas::subbars(formula(x, random.only=TRUE))
        ff <- noSpecials(ff, specials = lme4_specials, delete = FALSE)
        tt <- terms.formula(ff)
        attr(tt,"predvars") <- attr(terms(x@frame),"predvars.random")
    }
    tt
}

##' @importFrom stats update
##' @S3method update merMod
update.merMod <- function(object, formula., ..., evaluate = TRUE) {
    if (is.null(call <- getCall(object)))
        stop("object should contain a 'call' component")
    extras <- match.call(expand.dots = FALSE)$...
    if (!missing(formula.))
        call$formula <- update.formula(formula(object), formula.)
    if (length(extras) > 0) {
        existing <- !is.na(match(names(extras), names(call)))
        for (a in names(extras)[existing]) call[[a]] <- extras[[a]]
        if (any(!existing)) {
            call <- c(as.list(call), extras[!existing])
            call <- as.call(call)
        }
    }
    if (!evaluate) return(call)
    ## should be able to find model components somewhere in (1) formula env; (2) calling env;
    ##  (3) parent frame [plus its parent frames]
    ## see discusion at https://stackoverflow.com/questions/64268994/evaluate-call-when-components-may-be-scattered-among-environments
    ## FIXME: suppressWarnings(update(model)) will give
    ## Error in as.list.environment(X[[i]], ...) :
    ## promise already under evaluation: recursive default argument reference or earlier problems?

    ff <- environment(formula(object))
    pf <- parent.frame()
    sf <- sys.frames()[[1]]
    tryCatch(eval(call,  envir = ff),  ## try formula environment
             error = function(e) {
               tryCatch(eval(call, envir = sf),  ## try stack frame
                        error = function(e) {
                          eval(call, envir=pf) ## try parent frame
                        })
             })

    ##
    ## combf <- tryCatch(
    ##     do.call("c", lapply(list(ff, sf), as.list)),
    ##     error=function(e) as.list(ff)
    ## )
    ## eval(call,combf, enclos=pf)
}

if (.Platform$OS.type != "windows") {
    ## test of false zero problem reported by Vince Dorie
    ## (no longer occurs with current development lme4)
    ## https://github.com/lme4/lme4/issues/17
    library(lme4)

    sigma.eps <- 2
    sigma.the <- 0.75
    mu <- 2

    n <- 5
    J <- 10
    g <- gl(J, n)

    set.seed(1)

    theta <- rnorm(J, 0, sigma.eps * sigma.the)
    y <- rnorm(n * J, mu + theta[g], sigma.eps)
    lmerFit <- lmer(y ~ 1 + (1 | g), REML = FALSE, verbose=TRUE)

    y.bar <- mean(y)
    y.bar.j <- sapply(1:J, function(j) mean(y[g == j]))
    S.w <- sum((y - y.bar.j[g])^2)
    S.b <- n * sum((y.bar.j - y.bar)^2)
    R <- S.b / S.w

    sigma.the.hat <- sqrt(max((n - 1) * R / n - 1 / n, 0))
    stopifnot(all.equal(sigma.the.hat,lme4Sigma <- unname(getME(lmerFit,"theta")),
                        tolerance=2e-5))
} ## skip on windows (for speed)

get_est <- function(mod, model_name) {
  v <- as.data.frame(VarCorr(mod))
  vnames <- with(v,
                 ifelse(is.na(var2),
                        paste0("sd_", var1, "|", grp),
                        paste0("cor_", var1, ".", var2, "|", grp)))
  est <- c(fixef(mod), setNames(v$sdcor, vnames))
  data.frame(model=model_name, var=names(est), est=est, row.names=NULL)
}

ncores <- min(10, parallel::detectCores() - 1)

b_cifun <- function(x, nsim = 501) {
  cat(deparse(getCall(x)$formula), "\n")
  confint(x, method = "boot", seed = 101, nsim = nsim, signames = FALSE,
          parallel = "multicore", ncpus = ncores)
}

p_fun <- function(x) {
  cat(deparse(getCall(x)$formula), "\n")
  profile(x, signames = FALSE,
          devtol = 1e-2,
          maxpts = 250,
          parallel = "multicore", ncpus = ncores)
}


combfun1 <- function(x, model, model_id) {
   df <- data.frame(var = rownames(x), x, check.names = FALSE)
   names(df)[names(df) == "2.5 %"]  <- "lwr"
   names(df)[names(df) == "97.5 %"] <- "upr"
   rownames(df) <- NULL
   df$model <- model
   df$model_id <- model_id
   df
}

combfun2 <- function(x, type, df_name, df_est) {
  df <- do.call("rbind", Map(combfun1, x, df_name$mdesc, df_name$mnames))
  ## if necessary, hack correlation names (ugh); different parts of the pipeline
  ## put the term names in a different order
  ## this assumes that this is the **ONLY** model we are considering with a correlation ...
  corvals <- grep("cor", df$var)
  if (length(corvals) > 0 && !any(grepl("urban", df$var[corvals]))) stop("unexpected cor name pattern in combfun2")
  if (length(corvals) > 0) {
    df$var[corvals] <- "cor_(Intercept).urban1|district"
  }
  df <- merge(df, df_est, by.x = c("model_id", "var"), by.y = c("model", "var"))
  df$model <- factor(df$model, levels = df_name$mdesc)
  df$type <- type
  df <- df[order(df$model),]
  df
}


re_wald_ci_hack <- function(x, q = 0.95) {
  lev <- qnorm((q+1)/2)
  th <- unname(getME(x, "theta"))
  nth <- length(th)
  H <- x@optinfo$derivs$Hessian
  v <- try(solve(H)[1:nth, 1:nth, drop = FALSE], silent = TRUE)
  if (inherits(v, "try-error")) return (data.frame(lwr = rep(NA_real_, nth), upr = rep(NA_real_, nth)))
  ## in this special case, this means one or two
  ##  scalar REs, each of which correspond to a SD
  if (nth <= 2) {
    vals <- th
    s <- sqrt(diag(v))
  } else {
    ## delta method hack.
    phi <- th[2]^2 + th[3]^2
    vals <- c(sd1 = th[1], sd2 = sqrt(phi), cor = th[2]/sqrt(phi))
    s <- rep(NA_real_, 3)
    s[1] <- sqrt(v[1,1])
    cvec2 <- c(0, th[2]/sqrt(phi), th[3]/sqrt(phi))
    s[2] <- sqrt(drop(t(cvec2) %*% (v %*% cvec2)))
    cvec3 <- c(0, (1-th[2]^2/phi)/sqrt(phi), -th[2]*th[3]*phi^(-3/2))
    s[3] <- sqrt(drop(t(cvec3) %*% (v %*% cvec3)))
  }
  return(data.frame(lwr = vals - lev*s, upr = vals + lev*s))
}

wald_cifun <- function(x) {
  cc <- confint(x, method = "Wald", signames = FALSE)
  nth <- length(getME(x, "theta"))
  cc[1:nth, ] <- as.matrix(re_wald_ci_hack(x))
  cc
}

##' @importFrom MASS negative.binomial
##' @importFrom MASS theta.ml

## ==> user should use  getME(object, "glmer.nb.theta")
getNBdisp <- function(object) environment(object@resp$family$aic)[[".Theta"]]

## Hidden, originally used at least once, well tested (!), but
if(FALSE) # not needed anymore currently
getNBdisp.fam <- function(familyString)
    as.numeric(sub(".*([-+]?\\d+(\\.\\d*)?([Ee][-+]?\\d+)?){1}.*",
                   "\\1", familyString))

## Package "constants" {only on depending the glmResp definition in ./AllClass.R}:
glmResp.f.nms <- names(glmResp$fields())
glmNB.to.change <- setdiff(glmResp.f.nms, c("Ptr", "family"))

##' setNBdisp(object,theta) :=
##' NB-object with changed [DISP]ersion parameter 'theta' (and all that entails)
setNBdisp <- function(object,theta) {
  ## assign(".Theta",theta,envir=environment(object@resp$family$aic))
  rr <- object@resp
  newresp <- do.call(glmResp$new,
                     c(lapply(setNames(nm=glmNB.to.change), rr$field),
                       list(family = MASS::negative.binomial(theta=theta))))
  newresp$setOffset(rr$offset)
  newresp$updateMu(rr$eta - rr$offset)
  object@resp <- newresp
  object
}

refitNB <- function(object, theta, control = NULL) {
    refit(setNBdisp(object, theta), control = control)
}

##' @title Optimize over the Negative Binomial Parameter Theta
##' @param object a "glmerMod" object, updated from poisson to negative.binomial()
##' @param interval and
##' @param tol      are both passed to \code{\link{optimize}()}.
##' @param verbose  ## show our own progress
##' @param control passed to \code{\link{refit}()}
##' @return the last fit, an object like 'object'
optTheta <- function(object,
                     interval = c(-5,5),
                     tol = .Machine$double.eps^0.25,
                     verbose = FALSE,
                     control = NULL)
{
  lastfit <- object
  it <- 0L
  NBfun <- function(t) {
      ## Kluge to retain last value and evaluation count {good enough for ..}
      f_refitNB <- factory(refitNB,types=c("message","warning"))
      lastfit <<- f_refitNB(lastfit, theta = exp(t),
                            control = control)
      dev <- -2*logLik(lastfit)
      it <<- it+1L
      if (verbose)
          cat(sprintf("%2d: th=%#15.10g, dev=%#14.8f, beta[1]=%#14.8f\n",
                      it, exp(t), dev, lastfit@beta[1]))
      dev
  }
  optval <- optimize(NBfun, interval=interval, tol=tol)
  stopifnot(all.equal(optval$minimum, log(getNBdisp(lastfit))))
  ## FIXME: return eval count info somewhere else? MM: new slot there, why not?
  attr(lastfit,"nevals") <- it
  ## fix up the 'th' expression, replacing it by the real number,
  ## so effects:::mer.to.glm() can eval() it:
  lastfit@call$family[["theta"]] <- exp(optval$minimum)
  ## output warnings/messages from last fit
  for (m in c("warning","message")) {
      if (!is.null(x <- attr(lastfit,paste0("factory-",m)))) {
          for (i in x) {
              get(m,pos="package:base")(i)
          }
      }
  }

  lastfit
}

## use MASS machinery to estimate theta from residuals
est_theta <- function(object, limit = 20,
                      eps = .Machine$double.eps^0.25, trace = 0)
{
  Y <- model.response(model.frame(object))
  ## may have NA values if na.exclude was used ...
  mu <- na.omit(fitted(object))
  theta.ml(Y, mu, weights = object@resp$weights,
           limit = limit, eps = eps, trace = trace)
}

## -------> ../man/glmer.Rd
##' glmer() for Negative Binomial
##' @param ... formula, data, etc: the arguments for
##' \code{\link{glmer}(..)} (apart from \code{family}!).
glmer.nb <- function(..., interval = log(th) + c(-3,3),
                     tol = 5e-5, verbose = FALSE, nb.control = NULL,
                     initCtrl = list(limit = 20, eps = 2*tol, trace = verbose,
                                     theta = NULL))
{
    ## ?? E() is a placeholder; [-1] removes it
    ## left with a 'headless' list of elements in ...
    dotE <- as.list(substitute(E(...))[-1])
    ## nE <- names(dotE <- as.list(substitute(E(...))[-1]))
    ## i <- match(c("formula",""), nE)
    ## i.frml <- i[!is.na(i)][[1]] # the index of "formula" in '...'
    ## dots <- list(...)

    mc <- match.call()

    if (is.null(th <- initCtrl$theta)) {
        mc[[1]] <- quote(lme4::glmer)
        mc$family <- quote(stats::poisson)
        mc$verbose <- (verbose>=2)
        mc$nb.control <- NULL
        ## ** FIXME: specifically add check.conv.singular="ignore"?
        ##  suppress other warnings unless explicitly specified?
        g0 <- suppressMessages(
            eval(mc, parent.frame(1L))
        )

        th <- est_theta(g0, limit = initCtrl$limit,
                        eps = initCtrl$eps, trace = initCtrl$trace)

        ## using format() on purpose, influenced by options(digits = *) :
        if(verbose) cat("th := est_theta(glmer(..)) =", format(th))
    }

    mc$initCtrl <- NULL ## clear to prevent infinite recursion
                        ##  in initCtrl$theta reference above ...
    mc$family <- bquote(MASS::negative.binomial(theta=.(th)))
    ## ** see FIXME above
    g1 <- suppressMessages(
        eval(mc, parent.frame(1L))
    )

    if(verbose) cat(" --> dev.= -2*logLik(.) =", format(-2*logLik(g1)),"\n")
    ## fix the 'data' part (only now!)
    if("data" %in% names(g1@call)) {
        if (!is.null(dotE[["data"]])) {
            g1@call[["data"]] <- dotE[["data"]]
        }
    } else
        warning("no 'data = *' in glmer.nb() call ... Not much is guaranteed")
    other.args <- c("verbose","control")
    for (a in other.args) {
        if (a %in% names(g1@call)) {
            g1@call[[a]] <- dotE[[a]]
        }
    }

    ## FIXME: optTheta should also work by modifying mc directly,
    ##  then re-evaluating, not via refit() ...

    control <- eval.parent(g1@call$control)
    res <- optTheta(g1, interval=interval, tol=tol, verbose=verbose,
                    control = c(control, nb.control))
    res
}

## do we want to facilitate profiling on theta??
## save evaluations used in optimize() fit?
## ('memoise'?)
## Again, I think that a reference class object would be a better approach.

## DON'T load lme4; test is to see if glmer.nb works when
## lme4 is not loaded
## this does *not* work properly in a devtools::test environment
##  (lme4 is not really detached)
## see tests/test-glmernbref.R for working test ...

test_that("glmer.nb ref to glmer", {
  set.seed(101)
  dd <- data.frame(x=runif(200), f= rep(1:20, each=10))
  b <- rnorm(20)
  dd <- transform(dd, y = rnbinom(200, mu  = exp(1 + 2*x + b[f]), size = 2))
  ## lme4 may not be attached if running tests via devtools::test()
  if ("package:lme4" %in% search()) detach("package:lme4")
  g <- lme4::glmer.nb(y~x + (1|f), data = dd)
  expect_is(g, "glmerMod")
})

testLevel <- if (nzchar(s <- Sys.getenv("LME4_TEST_LEVEL"))) as.numeric(s) else 1

set.seed(101)
dd <- expand.grid(f1 = factor(1:3),
                  f2 = LETTERS[1:2], g=1:9, rep=1:15,
          KEEP.OUT.ATTRS=FALSE)
mu <- 5*(-4 + with(dd, as.integer(f1) + 4*as.numeric(f2)))
dd$y <- rnbinom(nrow(dd), mu = mu, size = 0.5)

## mimic glmer.nb protocol
if (testLevel>1) {
test_that("most messages suppressed", {
    expect_message(glmer.nb(y ~ f1 + (1|g), data=dd[1:10,]),
                   "singular")
})

test_that("ok with negative.binomial masking", {
    negative.binomial <- function() {}
    ## use shortened version of data for speed ...
    m.base <- glmer.nb(y ~ f1 + (1|g), data=dd[1:200,])
    expect_is(m.base,"merMod")
})

test_that("ok with Poisson masking", {
    poisson <- NA
    ## use shortened version of data for speed ...
    m.base <- glmer.nb(y ~ f1 + (1|g), data=dd[1:200,])
    expect_is(m.base,"merMod")
    rm(poisson)
})

if (testLevel>2) {
#context("testing glmer refit")

test_that("glmer refit", {
            ## basic Poisson fit
            m.base <- glmer(y ~ f1*f2 + (1|g), data=dd, family=poisson)
            expect_equal(m.base@beta,(m.base.r <- refit(m.base))@beta,
                         tolerance = 1e-5)

            th <- lme4:::est_theta(m.base,limit=20,eps=1e-4,trace=FALSE)
            th0 <- structure(0.482681268108477, SE = 0.0244825021248148)
            th1 <- structure(0.482681277470945)
            th2 <- 0.482681268108477
            th3 <- 0.4826813
            ## NB update with raw number
            m.numth1 <- update(m.base,family=MASS::negative.binomial(theta=0.4826813))
            expect_equal(m.numth1@beta,(m.numth1.r <- refit(m.numth1))@beta)

            ## strip NB value
            m.symth4 <- update(m.base,family=MASS::negative.binomial(theta=c(th)))
            expect_equal(m.symth4@beta,(m.symth4.r <- refit(m.symth4))@beta)

            ## IDENTICAL numeric value to case #1 above
            m.symth6 <- update(m.base,family=MASS::negative.binomial(theta=th3))
            expect_equal(m.symth6@beta,(m.symth6.r <- refit(m.symth6))@beta)

            ## standard NB update with computed theta from est_theta (incl SE attribute)
            m.symth <- update(m.base,family=MASS::negative.binomial(theta=th))
            expect_equal(m.symth@beta,(m.symth.r <- refit(m.symth))@beta)

            ## NB update with equivalent value
            m.symth2 <- update(m.base,family=MASS::negative.binomial(theta=th0))
            expect_equal(m.symth2@beta,(m.symth2.r <- refit(m.symth2))@beta) 

            ## NB update with theta value (stored as variable, no SE) only
            m.symth3 <- update(m.base,family=MASS::negative.binomial(theta=th1))
            expect_equal(m.symth3@beta,(m.symth3.r <- refit(m.symth3))@beta)

            ## strip NB value (off by 5e-16)
            m.symth5 <- update(m.base,family=MASS::negative.binomial(theta=th2))
            expect_equal(m.symth5@beta,(m.symth5.r <- refit(m.symth5))@beta)
        })

## GH #399
test_that("na_exclude", {
    dd1 <- dd[1:200,]
    dd1$f1[1:5] <- NA
    expect_error(glmer.nb(y ~ f1 + (1|g), data=dd1, na.action=na.fail),
                 "missing values in object")
    m1 <- glmer.nb(y ~ f1 + (1|g), data=dd1, na.action=na.omit)
    m2 <- glmer.nb(y ~ f1 + (1|g), data=dd1, na.action=na.exclude)
    expect_equal(fixef(m1),fixef(m1))
    expect_equal(length(predict(m2))-length(predict(m1)),5)
})

## GH 423
test_that("start_vals", {
    dd1 <- dd[1:200,]
    g1 <- glmer.nb(y ~ f1 + (1|g), data=dd1)
    g2 <- glmer.nb(y ~ f1 + (1|g), data=dd1,
                   initCtrl=list(theta=getME(g1,"glmer.nb.theta")))
    expect_equal(fixef(g1),fixef(g2),tol=1e-5)
})

test_that("control arguments", {
    dd1 <- dd[1:200,]
    g1 <- glmer.nb(y ~ f1 + (1|g), data=dd1, initCtrl=list(theta=10))
    expect_is(g1,"merMod")  ## dumb test - just checking for run w/o error
    suppressWarnings(g1 <- glmer.nb(y ~ f1 + (1|g), data=dd1,
                                    nb.control=glmerControl(optimizer="bobyqa")))
    expect_equal(g1@optinfo$optimizer, "bobyqa")
    suppressWarnings(g1 <- glmer.nb(y ~ f1 + (1|g), data=dd1,
                                    nb.control=glmerControl(optCtrl=list(maxfun=2))))
    expect_equal(g1@optinfo$feval,3)
})
} ## testLevel>2
} ## testLevel>1

## ----preliminaries,include=FALSE,cache=FALSE,message=FALSE----------
options(width=70, show.signif.stars=FALSE,
        str=strOptions(strict.width="cut"),
        ## prefer empty continuation for reader's cut'n'paste:
        continue = "   ", #JSS: prompt = "R> ", continue = "+  ",
        useFancyQuotes = FALSE)
library("knitr")
library("lme4")
library("ggplot2")# Keeping default theme, nicer "on line":
#JSS theme_set(theme_bw())
library("grid")
zmargin <- theme(panel.spacing=unit(0,"lines"))
library("lattice")
library("minqa")
library("reformulas")
opts_chunk$set(engine='R',dev='pdf', fig.width=9, fig.height=5.5,
               prompt=TRUE, cache=TRUE, tidy=FALSE, comment=NA, error = FALSE)
knitr::render_sweave()

## ----sleep----------------------------------------------------------
str(sleepstudy)

## ----sleepPlot, echo=FALSE, fig.scap="Reaction time versus days by subject", fig.cap="Average reaction time versus days of sleep deprivation by subject.  Subjects ordered (from left to right starting on the top row) by increasing slope of subject-specific linear regressions.", fig.align='center', fig.pos='tb'----
## BMB: seemed more pleasing to arrange by increasing slope rather than
## intercept ...
xyplot(Reaction ~ Days | Subject, sleepstudy, aspect = "xy",
       layout = c(9, 2), type = c("g", "p", "r"),
       index.cond = function(x, y) coef(lm(y ~ x))[2],
       xlab = "Days of sleep deprivation",
       ylab = "Average reaction time (ms)",
       as.table = TRUE)

## ----outputExample,results="hide"-----------------------------------
fm1 <- lmer(Reaction ~ Days + (Days | Subject), sleepstudy)

## ----modularExampleFormula, tidy=FALSE, eval=FALSE------------------
# parsedFormula <- lFormula(formula = Reaction ~ Days + (Days | Subject),
#                              data = sleepstudy)

## ----modularExampleObjective, tidy=FALSE, eval=FALSE----------------
# devianceFunction <- do.call(mkLmerDevfun, parsedFormula)

## ----modularExampleOptimization, tidy=FALSE, eval=FALSE-------------
# optimizerOutput <- optimizeLmer(devianceFunction)

## ----modularExampleOutput, tidy=FALSE, eval=FALSE-------------------
# mkMerMod(   rho = environment(devianceFunction),
#             opt = optimizerOutput,
#          reTrms = parsedFormula$reTrms,
#              fr = parsedFormula$fr)

## ----MMformula, eval=FALSE------------------------------------------
# resp ~ FEexpr + (REexpr1 | factor1) + (REexpr2 | factor2) + ...

## ----uncorrelatedModel,results="hide"-------------------------------
fm2 <- lmer(Reaction ~ Days + (Days || Subject), sleepstudy)

## ----setSeed, echo=FALSE--------------------------------------------
set.seed(2)

## ----factorToSparseMatrix-------------------------------------------
(f <- gl(3, 2))
(Ji <- t(as(f, Class = "sparseMatrix")))

## ----rawModelMatrix-------------------------------------------------
(Xi <- cbind(1, rep.int(c(-1, 1), 3L)))

## ----KhatriRao------------------------------------------------------
(Zi <- t(KhatriRao(t(Ji), t(Xi))))

## ----sanityCheck, include=FALSE-------------------------------------
## alternative formulation of Zi (eq:Zi)
rbind(
  Ji[1,] %x% Xi[1,],
  Ji[2,] %x% Xi[2,],
  Ji[3,] %x% Xi[3,],
  Ji[4,] %x% Xi[4,],
  Ji[5,] %x% Xi[5,],
  Ji[6,] %x% Xi[6,])

## ----nc, echo=FALSE-------------------------------------------------
nc <- 3

## ----template-------------------------------------------------------
(rowIndices <- rep(1:nc, 1:nc))
(colIndices <- sequence(1:nc))
(template <- sparseMatrix(rowIndices, colIndices,
                          x = 1 * (rowIndices == colIndices)))

## ----thetaFromTemplate----------------------------------------------
(theta <- template@x)

## ----thetaSanityCheck, include=FALSE--------------------------------
lFormula(Reaction ~ (Days + I(Days^2) | Subject), sleepstudy)$reTrms$theta

## ----nl, echo=FALSE-------------------------------------------------
nl <- 2

## ----relativeCovarianceBlock----------------------------------------
(Lambdati <- .bdiag(rep(list(t(template)), nl)))

## ----relativeCovarianceBlockIndices---------------------------------
LindTemplate <- rowIndices + nc * (colIndices - 1) - choose(colIndices, 2)
(Lind <- rep(LindTemplate, nl))

## ----newTheta-------------------------------------------------------
thetanew <- round(runif(length(theta)), 1)

## ----relativeCovarianceBlockUpdate----------------------------------
Lambdati@x <- thetanew[Lind]

## ----PLSupdateTheta, eval = FALSE-----------------------------------
# Lambdat@x[] <- mapping(theta)

## ----PLSupdateThetaWithLind-----------------------------------------
mapping <- function(theta) theta[Lind]

## ----exampleNewTheta------------------------------------------------
thetanew <- c(1, -0.1, 2, 0.1, -0.2, 3)

## ----exampleLindUpdate----------------------------------------------
Lambdati@x[] <- mapping(thetanew)
Lambdati

## ----updateL, eval=FALSE--------------------------------------------
# L <- update(L, Lambdat %*% ZtW, mult = 1)

## ----PLSsolve, eval = FALSE-----------------------------------------
# cu[] <- as.vector(solve(L, solve(L, Lambdat %*% ZtWy,
#                                  system = "P"), system = "L"))
# RZX[] <- as.vector(solve(L, solve(L, Lambdat %*% ZtWX,
#                                   system = "P"), system = "L"))
# RXtRX <- as(XtWX - crossprod(RZX), "dpoMatrix")
# beta[] <- as.vector(solve(RXtRX, XtWy - crossprod(RZX, cu)))
# u[] <- as.vector(solve(L, solve(L, cu - RZX %*% beta,
#                                 system = "Lt"), system = "Pt"))

## ----PLSupdateResp, eval = FALSE------------------------------------
# b[] <- as.vector(crossprod(Lambdat, u))
# mu[] <- as.vector(crossprod(Zt, b) + X %*% beta + offset)
# wtres <- sqrtW * (y - mu)

## ----PLScalculateObjective, eval = FALSE----------------------------
# pwrss <- sum(wtres^2) + sum(u^2)
# logDet <- 2*determinant(L, logarithm = TRUE)$modulus
# if (REML) logDet <- logDet + determinant(RXtRX,
#                                          logarithm = TRUE)$modulus
# attributes(logDet) <- NULL
# profDev <- logDet + degFree * (1 + log(2 * pi * pwrss) - log(degFree))

## ----vcovByHand-----------------------------------------------------
RX <- getME(fm1, "RX")
sigma2 <- sigma(fm1)^2
sigma2 * chol2inv(RX)

## ----condVarExample, echo=FALSE, eval=FALSE-------------------------
# s2 <- sigma(fm1)^2
# Lambdat <- getME(fm1, "Lambdat")
# Lambda <- getME(fm1, "Lambda")
# Zt <- getME(fm1, "Zt")
# Z <- getME(fm1, "Z")
# y <- getME(fm1, "y")
# X <- getME(fm1, "X")
# beta <- getME(fm1, "beta")
# L <- getME(fm1, "L")
# (V <- crossprod(solve(L, system = "L")))[1:2, 1:2]
# (s2 * Lambda %*% V %*% Lambdat)[1:2, 1:2]
# attr(ranef(fm1, condVar = TRUE)$Subject, "postVar")[ , , 1] # should be same as Lam %*% V %*% Lamt
# (U <- as.vector((V %*% Lambdat %*% Zt %*%  (y - X %*% beta))))
# getME(fm1, "u")

## ----updateModel----------------------------------------------------
fm3 <- update(fm1, . ~ . - (Days | Subject) + (1 | Subject))
formula(fm3)

## ----summary1,echo=FALSE--------------------------------------------
ss <- summary(fm1)
cc <- capture.output(print(ss))
reRow <- grep("^Random effects", cc)
cat(cc[1:(reRow - 2)], sep = "\n")

## ----summary1reproduced, eval=FALSE---------------------------------
# formula(fm1)
# REMLcrit(fm1)
# quantile(residuals(fm1, "pearson", scaled = TRUE))

## ----summary2, echo=FALSE-------------------------------------------
feRow <- grep("^Fixed effects", cc)
cat(cc[reRow:(feRow - 2)], sep = "\n")

## ----summary2reproduced, eval=FALSE---------------------------------
# print(vc <- VarCorr(fm1), comp = c("Variance", "Std.Dev."))
# nobs(fm1)
# ngrps(fm1)
# sigma(fm1)

## ----VarCorr--------------------------------------------------------
as.data.frame(VarCorr(fm1))

## ----summary3, echo=FALSE-------------------------------------------
corRow <- grep("^Correlation", cc)
cat(cc[feRow:(corRow - 2)], sep = "\n")

## ----summary4, echo=FALSE-------------------------------------------
cat(cc[corRow:length(cc)], sep = "\n")

## ----diagplot1,fig.keep="none"--------------------------------------
plot(fm1, type = c("p", "smooth"))

## ----diagplot2,fig.keep="none"--------------------------------------
plot(fm1, sqrt(abs(resid(.))) ~ fitted(.),
     type = c("p", "smooth"))

## ----diagplot3,fig.keep="none"--------------------------------------
qqmath(fm1, id = 0.05)

## ----ppsim,results="hide"-------------------------------------------
iqrvec <- sapply(simulate(fm1, 1000), IQR)
obsval <- IQR(sleepstudy$Reaction)
post.pred.p <- mean(obsval >= c(obsval, iqrvec))

## ----anovaQuadraticModel--------------------------------------------
fm4 <- lmer(Reaction ~ polyDays[ , 1] + polyDays[ , 2] +
            (polyDays[ , 1] + polyDays[ , 2] | Subject),
            within(sleepstudy, polyDays <- poly(Days, 2)))
anova(fm4)

## ----anovaSanityCheck, include=FALSE--------------------------------
(getME(fm4, "RX")[2, ] %*% getME(fm4, "fixef"))^2

## ----anovaManyModels------------------------------------------------
anova(fm1, fm2, fm3)

## ----anovaRes,echo=FALSE--------------------------------------------
fm3ML <- refitML(fm3)
fm2ML <- refitML(fm2)
fm1ML <- refitML(fm1)
ddiff <- deviance(fm3ML) - deviance(fm2ML)
dp <- pchisq(ddiff, 1, lower.tail = FALSE)
ddiff2 <- deviance(fm2ML) - deviance(fm1ML)

## ----pvaluesHelp, eval=FALSE----------------------------------------
# help("pvalues")

## ----compareCI,echo=FALSE,cache=TRUE,message=FALSE,warning=FALSE----
ccw <- confint(fm1, method = "Wald")
## compute the profile once here and re-use it below rather than
## profiling fm1 a second time; profiling is expensive and
## makes us exceed CRAN time limits
pf <- profile(fm1)
ccp <- confint(pf)
ccb <- confint(fm1, method = "boot")

## ----CIqcomp,echo=FALSE,eval=TRUE-----------------------------------
rtol <- function(x,y) {
    abs((x - y) / ((x + y) / 2))
}
bw <- apply(ccb, 1, diff)
pw <- apply(ccp, 1, diff)
mdiffpct <- round(100 * max(rtol(bw, pw)))

## ----CIplot,echo=FALSE,eval=FALSE-----------------------------------
# ## obsolete
# ## ,fig.cap="Comparison of confidence intervals",fig.scap="CI comparison"
# tf <- function(x, method) data.frame(method = method,
#                par = rownames(x),
#                setNames(as.data.frame(x), c("lwr", "upr")))
# cc.all <- do.call(rbind, mapply(tf, list(ccw, ccp, ccb),
#                                c("Wald", "profile", "boot"),
#                      SIMPLIFY = FALSE))
# ggplot(cc.all, aes(x = 1, ymin = lwr, ymax = upr, colour = method)) +
#     geom_linerange(position = position_dodge(width = 0.5)) +
#     facet_wrap( ~ par, scale = "free") +
#     theme(axis.text.x = element_blank()) +
#     labs(x = "")

## ----profile_calc,echo=FALSE,cache=TRUE-----------------------------
## pf was computed above, in the confidence interval
## comparison; re-used here to save time by not profiling
## fm1 twice

## ----profile_zeta_plot,fig.cap="Profile zeta plot: \\code{xyplot(prof.obj)}",fig.scap="Profile zeta plot",echo=FALSE,fig.align='center',fig.pos='tb'----
xyplot(pf)

## ----profile_density_plot,fig.cap="Profile density plot: \\code{densityplot(prof.obj)}",echo=FALSE,fig.align='center',fig.pos='tb'----
densityplot(pf)

## ----profile_pairs_plot,fig.cap="Profile pairs plot: \\code{splom(prof.obj)}",echo=FALSE,fig.height=8,fig.width=8,fig.align='center',fig.pos='htb',out.height='5.5in'----
splom(pf)

## ----modularSimulationFormula---------------------------------------
form <- respVar ~ 1 + (explVar1 + explVar2 | groupFac)

## ----dataTemplate---------------------------------------------------
set.seed(1)
dat <- mkDataTemplate(form,
  nGrps = 500,
  nPerGrp = 20,
  rfunc = rnorm)
head(dat)

## ----parsTemplate---------------------------------------------------
(pars <- mkParsTemplate(form, dat))

## ----simCorr--------------------------------------------------------
vc <- matrix(c(1.0, 0.5, 0.5,
  0.5, 1.0, 0.0,
  0.5, 0.0, 1.0), 3, 3)

## ----vcTheta--------------------------------------------------------
pars$theta[] <- Vv_to_Cv(mlist2vec(vc))

## ----varCorrStructure-----------------------------------------------
dat$respVar <- simulate(form,
  newdata = dat,
  newparams = pars,
  family = "gaussian")[[1]]

## ----graphSims------------------------------------------------------
formLm <- form
formLm[[3]] <- reformulas::findbars(form)[[1]]
print(formLm)
cor(t(sapply(lmList(formLm, dat), coef)))

## ----phiToTheta, cache = FALSE--------------------------------------
phiToTheta <- function(phi) {
    theta5 <- -(phi[2]*phi[3])/phi[4]
    c(phi[1:4], theta5, phi[5])
}

## ----compute deviance function modular, cache = FALSE---------------
lf <- lFormula(formula = form, data = dat, REML = TRUE)
devf <- do.call(mkLmerDevfun, lf)

## ----wrapper modular, cache = FALSE---------------------------------
devfWrap <- function(phi) devf(phiToTheta(phi))

## ----opt modular, cache = FALSE-------------------------------------
opt <- bobyqa(par = lf$reTrms$theta[-5],
  fn = devfWrap,
  lower = lf$reTrms$lower[-5])

## ----varCorr modular, cache = FALSE---------------------------------
vcEst <- vec2mlist(Cv_to_Vv(phiToTheta(opt$par)))[[1]]
dimnames(vcEst) <- rep(lf$reTrms$cnms, 2)
round(vcEst, 2)
vc

## ----simulateSplineData,message=FALSE-------------------------------
library("gamm4")
library("splines")
set.seed(1)
n <- 100
pSimulation <- 3
pStatistical <- 8
x <- rnorm(n)
Bsimulation <- ns(x, pSimulation)
Bstatistical <- ns(x, pStatistical)
beta <- rnorm(pSimulation)
y <- as.numeric(Bsimulation %*% beta + rnorm(n, sd = 0.3))

## ----splineExampleDataPlot, fig.width=4, fig.height=3,fig.align="center"----
par(mar = c(4, 4, 1, 1), las = 1, bty = "l")
plot(x, y, las = 1)
lines(x[order(x)], (Bsimulation %*% beta)[order(x)])

## ----splineExampleApproximateFormula--------------------------------
pseudoGroups <- as.factor(rep(1:pStatistical, length = n))
parsedFormula <- lFormula(y ~ x + (1 | pseudoGroups))

## ----splineExampleModifyZt------------------------------------------
parsedFormula$reTrms <- within(parsedFormula$reTrms, {
    Bt <- t(as.matrix(Bstatistical))[]
    cnms$pseudoGroups <- "spline"
    Zt <- as(Bt, class(Zt))
})

## ----splineExampleRemainingModularSteps-----------------------------
devianceFunction <- do.call(mkLmerDevfun, parsedFormula)
optimizerOutput <- optimizeLmer(devianceFunction)
mSpline <- mkMerMod(   rho = environment(devianceFunction),
                       opt = optimizerOutput,
                    reTrms = parsedFormula$reTrms,
                        fr = parsedFormula$fr)
mSpline

## ----splineExampleFittedModelPlot, fig.width=4, fig.height=3, fig.align="center"----
xNew <- seq(min(x), max(x), length = 100)
newBstatistical <- predict(Bstatistical, xNew)
yHat <-   cbind(1, xNew) %*% getME(mSpline, "fixef") +
        newBstatistical %*% getME(mSpline, "u")
par(mar = c(4, 4, 1, 1), las = 1, bty = "l")
plot(x, y)
lines(xNew, yHat)
lines(x[order(x)], (Bsimulation %*% beta)[order(x)],lty = 2)
legend("topright", bty = "n", c("fitted", "generating"), lty = 1:2,col = 1)

## ----opts, echo = FALSE, message = FALSE--------------------------------------
library("knitr")
knitr::opts_chunk$set(
  eval = FALSE
)

## ----loadpkg,message=FALSE----------------------------------------------------
#  library("lme4")

## ----noderivs,eval=FALSE------------------------------------------------------
#  lmer(y ~ service * dept + (1|s) + (1|d), InstEval,
#       control = lmerControl(calc.derivs = FALSE))

## ----nlminbfit,eval=FALSE-----------------------------------------------------
#  library("optimx")
#  lmer(y ~ service * dept + (1|s) + (1|d), InstEval,
#       control = lmerControl(optimizer = "optimx", calc.derivs = FALSE,
#       optCtrl = list(method = "nlminb", starttests = FALSE, kkt = FALSE)))

## ----nlopt,eval=FALSE---------------------------------------------------------
#  nlopt <- function(par, fn, lower, upper, control) {
#      .nloptr <<- res <- nloptr(par, fn, lb = lower, ub = upper,
#          opts = list(algorithm = "NLOPT_LN_BOBYQA", print_level = 1,
#          maxeval = 1000, xtol_abs = 1e-6, ftol_abs = 1e-6))
#      list(par = res$solution,
#           fval = res$objective,
#           conv = if (res$status > 0) 0 else res$status,
#           message = res$message
#      )
#  }
#  lmer(y ~ service * dept + (1|s) + (1|d), InstEval,
#      control = lmerControl(optimizer = "nloptwrap", calc.derivs = FALSE))

## Data originally from Davis 1991 Stat. Med., as packaged in geepack
## and transformed (center, id -> factor, idctr created, levels labeled)
library(lme4)

if (.Platform$OS.type != "windows") {
load(system.file("testdata","respiratory.RData",package="lme4"))
m_glmer_4.L <- glmer(outcome~center+treat+sex+age+baseline+(1|idctr),
                     family=binomial,data=respiratory)

m_glmer_4.GHQ5 <- glmer(outcome~center+treat+sex+age+baseline+(1|idctr),
                        family=binomial,data=respiratory,nAGQ=5)

m_glmer_4.GHQ8 <- glmer(outcome~center+treat+sex+age+baseline+(1|idctr),
                        family=binomial,data=respiratory,nAGQ=8)

m_glmer_4.GHQ16 <- glmer(outcome~center+treat+sex+age+baseline+(1|idctr),
                        family=binomial,data=respiratory,nAGQ=16)
} ## skip on windows (for speed)

library(lme4)
source(system.file("test-tools-1.R", package = "Matrix"), keep.source = FALSE)
## N.B. is.all.equal4() and assert.EQ() use 'tol', not 'tolerance' 


## should be able to run any example with any bounds-constrained optimizer ...
##  Nelder_Mead, bobyqa built in; optimx/nlminb, optimx/L-BFGS-B
##  optimx/Rcgmin will require a bit more wrapping/interface work (requires gradient)

if (.Platform$OS.type != "windows") {
fm1 <- lmer(Reaction ~ Days + (Days|Subject), sleepstudy)  ## Nelder_Mead
fm1B <- lmer(Reaction ~ Days + (Days|Subject), sleepstudy,
             control=lmerControl(optimizer="bobyqa"))
stopifnot(all.equal(fixef(fm1),fixef(fm1B)))
require(optimx)
lmerCtrl.optx <- function(method, ...)
    lmerControl(optimizer="optimx", ..., optCtrl=list(method=method))
glmerCtrl.optx <- function(method, ...)
    glmerControl(optimizer="optimx", ..., optCtrl=list(method=method))

(testLevel <- lme4:::testLevel())

## FAILS on Windows (on r-forge only, not win-builder)... 'function is infeasible at initial parameters'
## (can we test whether we are on r-forge??)
if (.Platform$OS.type != "windows") {
    fm1C <- lmer(Reaction ~ Days + (Days|Subject), sleepstudy,
                 control=lmerCtrl.optx(method="nlminb"))
    fm1D <- lmer(Reaction ~ Days + (Days|Subject), sleepstudy,
                 control=lmerCtrl.optx(method="L-BFGS-B"))
    stopifnot(is.all.equal4(fixef(fm1),fixef(fm1B),fixef(fm1C),fixef(fm1D)))

    fm1E <- update(fm1,control=lmerCtrl.optx(method=c("nlminb","L-BFGS-B")))
    ## hack equivalence of call and optinfo
    fm1E@call <- fm1C@call
    fm1E@optinfo <- fm1C@optinfo
    assert.EQ(fm1C,fm1E, tol=1e-5, giveRE=TRUE)# prints unless tolerance=0--equality
}

gm1 <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
             data = cbpp, family = binomial,
             control=glmerControl(tolPwrss=1e-13))
gm1B <- update(gm1, control=glmerControl  (tolPwrss=1e-13, optimizer="bobyqa"))
gm1C <- update(gm1, control=glmerCtrl.optx(tolPwrss=1e-13, method="nlminb"))
gm1D <- update(gm1, control=glmerCtrl.optx(tolPwrss=1e-13, method="L-BFGS-B"))
stopifnot(is.all.equal4(fixef(gm1),fixef(gm1B),fixef(gm1C),fixef(gm1D),
                        tol=1e-5))

if (testLevel > 1) {
    gm1E <- update(gm1, control=
		   glmerCtrl.optx(tolPwrss=1e-13, method=c("nlminb","L-BFGS-B")))
    ## hack equivalence of call and optinfo
    gm1E@call <- gm1C@call
    gm1E@optinfo <- gm1C@optinfo
    assert.EQ(gm1E,gm1C, tol=1e-5, giveRE=TRUE)# prints unless tol=0--equality
}
} ## skip on windows (for speed)

## .... CLASSES ........................................................

##
## Objects inheriting from virtual class "Covariance" represent
## 'nc'-by-'nc' covariance matrices 'S' using 'par', a vector storing
## nc*(nc+1)/2 or fewer parameters.
##
## Subclasses must define a mapping
##
##     par [double]  ---->  ( theta [double] ,  thetaIndex [integer] )
##
## such that theta[thetaIndex] stores in column-major order the
## structurally nonzero entries of Lambdat := chol(S).  By convention,
## subclasses will define 'theta' as storing in *row-major* order only
## the *independent* structurally nonzero entries of 'Lambdat', keeping
## the length of 'theta' to a minimum.
##
## MJ:
## Why do we change the storage order, when doing so greatly complicates
## determination of 'thetaIndex'?
##

setClass("Covariance",
         contains = "VIRTUAL",
         slots = c(nc = "integer", par = "numeric"),
         prototype = list(nc = 0L),
         validity =
         function (object) {
             if (length(nc <- object@nc) != 1L || is.na(nc) || nc < 0L)
                 gettextf("'%s' is not a non-negative integer",
                          "nc")
             else if (!is.double(object@par))
                 gettextf("type of '%s' is not \"%s\"",
                          "par", "double")
             else TRUE
         })

setClass("Covariance.us",
         contains = "Covariance",
         validity =
         function (object) {
             nc <- object@nc
             if (length(object@par) != (nc * (nc - 1L)) %/% 2L + nc)
                 gettextf("length of '%s' is not %s",
                          "par", "nc*(nc+1)/2")
             else TRUE
         })

setClass("Covariance.diag",
         contains = "Covariance",
         slots = c(hom = "logical"),
         prototype = list(hom = FALSE),
         validity =
         function (object) {
             nc <- object@nc
             if (length(hom <- object@hom) != 1L || is.na(hom))
                 gettextf("'%s' is not %s or %s",
                          "hom", "TRUE", "FALSE")
             else if (length(par <- object@par) !=
                      if (hom) nc > 0L else nc) {
                 if (hom)
                     gettextf("length of '%s' is not %d",
                              "par", nc > 0L)
                 else gettextf("length of '%s' is not '%s'",
                               "par", "nc")
             }
             else TRUE
         })

setClass("Covariance.cs",
         contains = "Covariance",
         slots = c(hom = "logical"),
         prototype = list(hom = FALSE),
         validity =
         .fn <-
         function (object) {
             nc <- object@nc
             if (length(hom <- object@hom) != 1L || is.na(hom))
                 gettextf("'%s' is not %s or %s",
                          "hom", "TRUE", "FALSE")
             else if (length(par <- object@par) !=
                      (if (hom) nc > 0L else nc) + (nc > 1L))
                 gettextf("length of '%s' is not %s",
                          "par", if (hom) "(nc>0)+(nc>1)" else "nc+(nc>1)")
             else TRUE
         })

setClass("Covariance.ar1",
         contains = "Covariance",
         slots = c(hom = "logical"),
         prototype = list(hom = TRUE),
         validity = .fn)

rm(.fn)


## .... GENERIC FUNCTIONS ..............................................

setGeneric("getPar",
           function (object)
               standardGeneric("getPar"))

setGeneric("getParLength",
           function (object)
               standardGeneric("getParLength"))

setGeneric("getParNames",
           function (object, cnm, gnm, prf = NULL, old = TRUE)
               standardGeneric("getParNames"),
           signature = c("object", "cnm", "gnm"))

setGeneric("setPar",
           function (object, value, pos = 0L)
               standardGeneric("setPar"))

setGeneric("getTheta",
           function (object)
               standardGeneric("getTheta"))

setGeneric("getThetaLength",
           function (object)
               standardGeneric("getThetaLength"))

setGeneric("getThetaNames",
           function (object, cnm, gnm, prf = NULL, old = TRUE)
               standardGeneric("getThetaNames"),
           signature = c("object", "cnm", "gnm"))

setGeneric("getThetaIndex",
           function (object)
               standardGeneric("getThetaIndex"))

setGeneric("getThetaIndexLength",
           function (object)
               standardGeneric("getThetaIndexLength"))

setGeneric("setTheta",
           function (object, value, pos = 0L)
               standardGeneric("setTheta"),
           signature = c("object", "value"))

setGeneric("getLower",
           function (object)
               standardGeneric("getLower"))

setGeneric("getUpper",
           function (object)
               standardGeneric("getUpper"))

setGeneric("getLambda",
           function (object)
               standardGeneric("getLambda"))

setGeneric("getLambdat.dp",
           function (object)
               standardGeneric("getLambdat.dp"))

setGeneric("getLambdat.i",
           function (object)
               standardGeneric("getLambdat.i"))

setGeneric("getVC",
           function (object)
               standardGeneric("getVC"))

setGeneric("getVCNames",
           function (object, cnm, gnm, prf = NULL, old = TRUE)
               standardGeneric("getVCNames"),
           signature = c("object", "cnm", "gnm"))

setGeneric("setVC",
           function (object, vcomp, ccomp)
               standardGeneric("setVC"))

setGeneric("getProfPar",
           function (object, profscale, sc = NULL) {
               profscale <- validProfScale(profscale)
               standardGeneric("getProfPar")
           },
           signature = "object")

setGeneric("setProfPar",
           function (object, profscale, sc = NULL, value, pos = 0L) {
               profscale <- validProfScale(profscale)
               standardGeneric("setProfPar")
           },
           signature = c("object", "value"))

setGeneric("getProfLower",
           function (object, profscale, sc = NULL) {
               profscale <- validProfScale(profscale)
               standardGeneric("getProfLower")
           },
           signature = "object")

setGeneric("getProfUpper",
           function (object, profscale, sc = NULL) {
               profscale <- validProfScale(profscale)
               standardGeneric("getProfUpper")
           },
           signature = "object")

setGeneric("isSingular",
           ## S4 default method dispatching S3 methods:
           function (object, tol = getSingTol())
               UseMethod("isSingular"),
           signature = "object")


## .... METHODS ........................................................

setMethod("initialize",
          c(.Object = "Covariance.us"),
          function (.Object, nc = .Object@nc,
                    par, simulate = FALSE, ...) {
              if (missing(par) &&
                  is.integer(nc) && length(nc) == 1L && !is.na(nc) &&
                  nc >= 0L) {
                  .Object@nc <- nc
                  .Object@par <-
                  `[<-`((if (simulate) rnorm else double)(
                            (nc * (nc - 1L)) %/% 2L + nc),
                        cumsum(c(if (nc > 0L) 1L, if (nc > 1L) nc:2L)),
                        if (simulate) rlnorm(nc) else 1)
                  .Object
              }
              else callNextMethod()
          })

setMethod("initialize",
          c(.Object = "Covariance.diag"),
          function (.Object, nc = .Object@nc, hom = .Object@hom,
                    par, simulate = FALSE, ...) {
              if (missing(par) &&
                  is.integer(nc) && length(nc) == 1L && !is.na(nc) &&
                  nc >= 0L &&
                  is.logical(hom) && length(hom) == 1L && !is.na(hom)) {
                  .Object@nc <- nc
                  .Object@hom <- hom
                  .Object@par <-
                  (if (simulate) rlnorm else function (n) rep(1, n))(
                      if (hom) 1L * (nc > 0L) else nc)
                  .Object
              }
              else callNextMethod()
          })

setMethod("initialize",
          c(.Object = "Covariance.cs"),
          function (.Object, nc = .Object@nc, hom = .Object@hom,
                    par, simulate = FALSE, ...) {
              if (missing(par) &&
                  is.integer(nc) && length(nc) == 1L && !is.na(nc) &&
                  nc >= 0L &&
                  is.logical(hom) && length(hom) == 1L && !is.na(hom)) {
                  .Object@nc <- nc
                  .Object@hom <- hom
                  .Object@par <-
                  c((if (simulate) rlnorm else function (n) rep(1, n))(
                        if (hom) 1L * (nc > 0L) else nc),
                    if (nc > 1L) { if (simulate) runif(1L, -1/(nc - 1), 1) else 0 })
                  .Object
              }
              else callNextMethod()
          })

setMethod("initialize",
          c(.Object = "Covariance.ar1"),
          function (.Object, nc = .Object@nc, hom = .Object@hom,
                    par, simulate = FALSE, ...) {
              if (missing(par) &&
                  is.integer(nc) && length(nc) == 1L && !is.na(nc) &&
                  nc >= 0L &&
                  is.logical(hom) && length(hom) == 1L && !is.na(hom)) {
                  .Object@nc <- nc
                  .Object@hom <- hom
                  .Object@par <-
                  c((if (simulate) rlnorm else function (n) rep(1, n))(
                        if (hom) 1L * (nc > 0L) else nc),
                    if (nc > 1L) { if (simulate) runif(1L, -1, 1) else 0 })
                  .Object
              }
              else callNextMethod()
          })

setMethod("getPar",
          c(object = "Covariance"),
          function (object)
              object@par)

setMethod("getParLength",
          c(object = "Covariance"),
          function (object)
              length(object@par))

## cnm: column names (i.e., effect names)
## gnm: group name
## prf: prefix
setMethod("getParNames",
          c(object = "Covariance.us", cnm = "character", gnm = "character"),
          getParNames.us <-
          function (object, cnm, gnm, prf = NULL, old = TRUE) {
              nc <- object@nc
              stopifnot(length(cnm) == nc, length(gnm) == 1L)
              inc <- seq_len(nc)
              dec <- seq.int(from = nc, by = -1L, length.out = nc)
              i <- sequence.default(from = inc, nvec = dec)
              j <- rep(inc, dec)
              ans <-
                  if (old)
                      paste0(gnm, ".", cnm[i], ".", cnm[j])
                  else if (is.null(prf))
                      paste0(cnm[i], ".", cnm[j], "|", gnm)
                  else paste0(prf[[2L]], "_", cnm[i], ".", cnm[j], "|", gnm)
              ans[cumsum(c(if (nc > 0L) 1L, if (nc > 1L) nc:2L))] <-
                  if (old)
                      paste0(gnm, ".", cnm)
                  else if (is.null(prf))
                      paste0(cnm, "|", gnm)
                  else paste0(prf[[1L]], "_", cnm, "|", gnm)
              ans
          })

setMethod("getParNames",
          c(object = "Covariance.diag", cnm = "character", gnm = "character"),
          getParNames.diag <-
          function (object, cnm, gnm, prf = NULL, old = TRUE) {
              nc <- object@nc
              stopifnot(length(cnm) == nc, length(gnm) == 1L)
              if (object@hom && nc > 0L)
                  cnm <- "*"
              if (old)
                  paste0(gnm, ".", cnm)
              else if (is.null(prf))
                  paste0(cnm, "|", gnm)
              else paste0(prf[[1L]], "_", cnm, "|", gnm)
          })

setMethod("getParNames",
          c(object = "Covariance.cs", cnm = "character", gnm = "character"),
          .fn <-
          function (object, cnm, gnm, prf = NULL, old = TRUE) {
              nc <- object@nc
              stopifnot(length(cnm) == nc, length(gnm) == 1L)
              if (object@hom && nc > 0L)
                  cnm <- "*"
              if (old)
                  paste0(gnm, ".", c(cnm, if (nc > 1L) "rho"))
              else if (is.null(prf))
                  paste0(c(cnm, if (nc > 1L) "rho"), "|", gnm)
              else c(paste0(prf[[1L]], "_", cnm, "|", gnm),
                     paste0(prf[[2L]], "_", "rho", "|", gnm))
          })

setMethod("getParNames",
          c(object = "Covariance.ar1", cnm = "character", gnm = "character"),
          .fn)

rm(.fn)

setMethod("setPar",
          c(object = "Covariance", value = "numeric"),
          function (object, value, pos = 0L) {
              np <- length(object@par)
              validValuePos(np, value, pos)
              object@par <-
              if (np == length(value))
                  value
              else
                  value[seq.int(from = pos + 1L, length.out = np)]
              object
          })

setMethod("getTheta",
          c(object = "Covariance.us"),
          .fn <-
          function (object)
              object@par)

## L[i, j]/sigma[i] =
##     if (i == j)
##         1
##     else 0
setMethod("getTheta",
          c(object = "Covariance.diag"),
          .fn)

rm(.fn)

## L[i, j]/sigma[i] =
##     if (i == j)
##         sqrt(1 - rho^2 * a[j])
##     else (rho - rho^2 * a[j])/sqrt(1 - rho^2 * a[j])
## where
##     a[    1] := 0
##     a[j + 1] := a[j] + (1 - rho * a[j])^2/(1 - rho^2 * a[j])
setMethod("getTheta",
          c(object = "Covariance.cs"),
          function (object) {
              nc <- object@nc
              par <- object@par
              if (nc <= 1L)
                  return(par)
              rho2 <- (rho <- par[length(par)])^2
              a <- double(nc); a. <- 0
              for (j in 1L:nc) {
                  a[j] <- a.
                  a. <- a. + (1 - rho * a.)^2/(1 - rho2 * a.)
              }
              ## some optimizers (bobyqa?) will violate bounds, try abs(rho)>1
              ## suppress warnings (NaN values will be handled downstream)
              v. <- suppressWarnings(sqrt(1 - rho2 * a))
              v <- rbind(v., (rho - rho2 * a)/v.)
              if (object@hom)
                  par[1L] * v[1L:(2L * nc - 1L)]
              else
                  par[sequence.default(from = 1L:nc, nvec = nc:1L)] *
                      rep(v, rbind(1L, (nc - 1L):0L))
          })

## L[i, j]/sigma[i] =
##     if (j == 1)
##         rho^(i - 1)
##     else rho^(i - j) * sqrt(1 - rho^2)
setMethod("getTheta",
          c(object = "Covariance.ar1"),
          function (object) {
              nc <- object@nc
              par <- object@par
              if (nc <= 1L)
                  return(par)
              rho2 <- (rho <- par[length(par)])^2
              v1 <- rho^(0L:(nc - 1L))
              v2 <- rho^(0L:(nc - 2L)) * sqrt(1 - rho2)
              if (object@hom)
                  par[1L] * c(v1, v2)
              else
                  par[sequence.default(from = 1L:nc, nvec = nc:1L)] *
                      c(v1, v2[sequence.default(from = 1L, nvec = (nc - 1L):1L)])
          })

setMethod("getThetaLength",
          c(object = "Covariance.us"),
          .fn <-
          function (object)
              length(object@par))

setMethod("getThetaLength",
          c(object = "Covariance.diag"),
          .fn)

rm(.fn)

setMethod("getThetaLength",
          c(object = "Covariance.cs"),
          .fn <-
          function (object) {
              nc <- object@nc
              if (object@hom)
                  2L * nc - (nc > 0L)
              else (nc * (nc - 1L)) %/% 2L + nc
          })

setMethod("getThetaLength",
          c(object = "Covariance.ar1"),
          .fn)

rm(.fn)

setMethod("getThetaNames",
          c(object = "Covariance.us", cnm = "character", gnm = "character"),
          function (object, cnm, gnm, prf = NULL, old = TRUE)
              getParNames.us(object, cnm, gnm, prf, old))

setMethod("getThetaNames",
          c(object = "Covariance.diag", cnm = "character", gnm = "character"),
          function (object, cnm, gnm, prf = NULL, old = TRUE)
              getParNames.diag(object, cnm, gnm, prf, old))

setMethod("getThetaNames",
          c(object = "Covariance.cs", cnm = "character", gnm = "character"),
          function (object, cnm, gnm, prf = NULL, old = TRUE) {
              if (object@hom) {
                  nc <- object@nc
                  rbind(if (old)
                            paste0(gnm, ".", cnm)
                        else if (is.null(prf))
                            paste0(cnm, "|", gnm)
                        else paste0(prf[[1L]], "_", cnm, "|", gnm),
                        if (old)
                            paste0(gnm, ".", "*", ".", cnm)
                        else if (is.null(prf))
                            paste0("*", ".", cnm, "|", gnm)
                        else paste0(prf[[2L]], "_", "*", ".", cnm, "|", gnm))[seq_len(2L * nc - 1L)]
              }
              else getParNames.us(object, cnm, gnm, prf, old)
          })

setMethod("getThetaNames",
          c(object = "Covariance.ar1", cnm = "character", gnm = "character"),
          function (object, cnm, gnm, prf = NULL, old = TRUE) {
              if (object@hom) {
                  nc <- object@nc
                  i <- if (nc > 1L) c(1L:nc, 2L:nc) else nc
                  j <- if (nc > 1L) rep(1L:2L, nc:(nc - 1L)) else nc
                  k <- seq_len(min(2L, nc))
                  ans <-
                      if (old)
                          paste0(gnm, ".", cnm[i], ".", cnm[j])
                      else if (is.null(prf))
                          paste0(cnm[i], ".", cnm[j], "|", gnm)
                      else paste0(prf[[2L]], "_", cnm[i], ".", cnm[j], "|", gnm)
                  ans[c(1L, nc + 1L)[k]] <-
                      if (old)
                          paste0(gnm, ".", cnm[k])
                      else if (is.null(prf))
                          paste0(cnm[k], "|", gnm)
                      else paste0(prf[[2L]], "_", cnm[k], "|", gnm)
                  ans
              }
              else getParNames.us(object, cnm, gnm, prf, old)
          })

setMethod("getThetaIndex",
          c(object = "Covariance.us"),
          getThetaIndex.us <-
          function (object) {
              snc <- seq_len(nc <- object@nc)
              rep(seq.int(from = 1L - nc, length.out = nc),
                  snc) +
              cumsum(seq.int(from = nc, by = -1L, length.out = nc))[
                  sequence.default(from = 1L, nvec = snc)]
          })

setMethod("getThetaIndex",
          c(object = "Covariance.diag"),
          function (object) {
              nc <- object@nc
              if (object@hom) rep(1L, nc) else seq_len(nc)
          })

setMethod("getThetaIndex",
          c(object = "Covariance.cs"),
          function (object) {
              if (object@hom) {
                  snc <- seq_len(nc <- object@nc)
                  `[<-`(sequence.default(from = 2L, by = 2L, nvec = snc),
                        cumsum(snc),
                        seq.int(from = 1L, by = 2L, length.out = nc))
              }
              else getThetaIndex.us(object)
          })

setMethod("getThetaIndex",
          c(object = "Covariance.ar1"),
          function (object) {
              if (object@hom) {
                  snc <- seq_len(nc <- object@nc)
                  `[<-`(sequence.default(from = seq.int(from = nc + 1L, length.out = nc),
                                         by = -1L,
                                         nvec = snc),
                        1L + cumsum(seq.int(from = 0L, length.out = nc)),
                        snc)
              }
              else getThetaIndex.us(object)
          })

setMethod("getThetaIndexLength",
          c(object = "Covariance.us"),
          function (object) {
              nc <- object@nc
              (nc * (nc - 1L)) %/% 2L + nc
          })

setMethod("getThetaIndexLength",
          c(object = "Covariance.diag"),
          function (object)
              object@nc)

setMethod("getThetaIndexLength",
          c(object = "Covariance.cs"),
          function (object) {
              nc <- object@nc
              (nc * (nc - 1L)) %/% 2L + nc
          })

setMethod("getThetaIndexLength",
          c(object = "Covariance.ar1"),
          function (object) {
              nc <- object@nc
              (nc * (nc - 1L)) %/% 2L + nc
          })

setMethod("setTheta",
          c(object = "Covariance.us", value = "numeric"),
          .fn <-
          function (object, value, pos = 0L) {
              nt <- length(object@par)
              validValuePos(nt, value, pos)
              object@par <-
              if (nt == length(value))
                  value
              else
                  value[seq.int(from = pos + 1L, length.out = nt)]
              object
          })

setMethod("setTheta",
          c(object = "Covariance.diag", value = "numeric"),
          .fn)

rm(.fn)

setMethod("setTheta",
          c(object = "Covariance.cs", value = "numeric"),
          function (object, value, pos = 0L) {
              nc <- object@nc
              hom <- object@hom
              nt <- if (hom) 2L * nc - (nc > 0L) else (nc * (nc - 1L)) %/% 2L + nc
              validValuePos(nt, value, pos)
              object@par <-
              if (nc <= 1L)
                  value[if (nc > 0L) 1L]
              else {
                  ## L[2, 1] = sigma[2] * rho
                  l21 <- value[pos + 1L + 1L]
                  ## L[2, 2] = sigma[2] * sqrt(1 - rho^2)
                  l22 <- value[pos + 1L + if (hom) 2L else nc]
                  rho <-
                  if (l21 != 0 || l22 != 0)
                      sign(l21) * 1/sqrt(1 + (l22/l21)^2)
                  else {
                      ## Fall back to constructing the Cholesky factor
                      i <- sequence.default(from = seq.int(from = 1L, by = nc + 1L, length.out = nc),
                                            nvec = nc:1L)
                      j <-
                      if (hom)
                          rep((pos + 1L):(pos + 2L * nc - 1L),
                              rbind(1L, (nc - 1L):0L))
                      else (pos + 1L):(pos + (nc * (nc - 1L)) %/% 2L + nc)
                      L <- matrix(0, nc, nc)
                      L[i] <- value[j]
                      d <- sqrt(rowSums(L * L))
                      h <- order(d, decreasing = TRUE)[1L:2L]
                      d12 <- prod(d[h])
                      if (d12 == 0)
                          stop(gettextf("'%s' is not identifiable as there is no pair of nonzero standard deviations",
                                        "rho"),
                               domain = NA)
                      sum(L[h[1L], ] * L[h[2L], ])/d12
                  }
                  sigma <-
                  if (hom)
                      value[pos + 1L]
                  else if (rho == 0)
                      value[cumsum(c(pos + 1L, nc:2L))]
                  else
                      value[seq.int(from = pos + 1L, length.out = nc)]/
                          rep(c(1, rho), c(1L, nc - 1L))
                  c(sigma, rho)
              }
              object
          })

setMethod("setTheta",
          c(object = "Covariance.ar1", value = "numeric"),
          function (object, value, pos = 0L) {
              nc <- object@nc
              hom <- object@hom
              nt <- if (hom) 2L * nc - (nc > 0L) else (nc * (nc - 1L)) %/% 2L + nc
              validValuePos(nt, value, pos)
              object@par <-
              if (nc <= 1L)
                  value[if (nc > 0L) 1L]
              else {
                  ## L[2, 1] = sigma[2] * rho
                  l21 <- value[pos + 1L + 1L]
                  ## L[2, 2] = sigma[2] * sqrt(1 - rho^2)
                  l22 <- value[pos + 1L + nc]
                  rho <-
                  if (l21 != 0 || l22 != 0)
                      sign(l21) * 1/sqrt(1 + (l22/l21)^2)
                  else {
                      ## Fall back to constructing the Cholesky factor
                      i <- sequence.default(from = seq.int(from = 1L, by = nc + 1L, length.out = nc),
                                            nvec = nc:1L)
                      j <-
                      if (hom)
                          sequence.default(from = rep(pos + 1L + c(0L, nc), c(1L, nc - 1L)),
                                           nvec = nc:1L)
                      else (pos + 1L):(pos + (nc * (nc - 1L)) %/% 2L + nc)
                      L <- matrix(0, nc, nc)
                      L[i] <- value[j]
                      d <- sqrt(rowSums(L * L))
                      h <- order(d, decreasing = TRUE)[1L:2L]
                      d12 <- prod(d[h])
                      if (d12 == 0)
                          stop(gettextf("'%s' is not identifiable as there is no pair of nonzero standard deviations",
                                        "rho"),
                               domain = NA)
                      (sum(L[h[1L], ] * L[h[2L], ])/d12)^(1/abs(h[1L] - h[2L]))
                  }
                  sigma <-
                  if (hom)
                      value[pos + 1L]
                  else if (rho == 0)
                      value[cumsum(c(pos + 1L, nc:2L))]
                  else # avoid underflow of powers of abs(rho)
                      exp(log(abs(value[seq.int(from = pos + 1L, length.out = nc)])) - (0L:(nc - 1L)) * log(abs(rho)))
                  c(sigma, rho)
              }
              object
          })

setMethod("getLower",
          c(object = "Covariance.us"),
          function (object) {
              nc <- object@nc
              ## FIXME: make more scrutable!
              `[<-`(rep(-Inf, length(object@par)),
                    cumsum(c(if (nc > 0L) 1L, if (nc > 1L) nc:2L)),
                    0)
          })

setMethod("getLower",
          c(object = "Covariance.diag"),
          function (object)
              double(length(object@par)))

setMethod("getLower",
          c(object = "Covariance.cs"),
          function (object) {
              nc <- object@nc
              c(double(if (object@hom) 1L * (nc > 0L) else nc),
                if (nc > 1L) -1/(nc - 1L))
          })

setMethod("getLower",
          c(object = "Covariance.ar1"),
          function (object) {
              nc <- object@nc
              c(double(if (object@hom) 1L * (nc > 0L) else nc),
                if (nc > 1L) -1)
          })

setMethod("getUpper",
          c(object = "Covariance.us"),
          function (object)
              rep(Inf, length(object@par)))

setMethod("getUpper",
          c(object = "Covariance.diag"),
          function (object)
              rep(Inf, length(object@par)))

setMethod("getUpper",
          c(object = "Covariance.cs"),
          function (object) {
              nc <- object@nc
              c(rep(Inf, if (object@hom) 1L * (nc > 0L) else nc),
                if (nc > 1L) 1)
          })

setMethod("getUpper",
          c(object = "Covariance.ar1"),
          function (object) {
              nc <- object@nc
              c(rep(Inf, if (object@hom) 1L * (nc > 0L) else nc),
                if (nc > 1L) 1)
          })

setMethod("getLambda",
          c(object = "Covariance.us"),
          function (object) {
              nc <- object@nc
              ans <- matrix(0, nc, nc)
              if (nc > 0L)
              ans[lower.tri(ans, diag = TRUE)] <- object@par
              ans
          })

setMethod("getLambda",
          c(object = "Covariance.diag"),
          function (object) {
              nc <- object@nc
              diag(object@par, nc, nc)
          })

setMethod("getLambda",
          c(object = "Covariance.cs"),
          function (object) {
              nc <- object@nc
              ans <- matrix(0, nc, nc)
              if (nc > 0L)
              ans[lower.tri(ans, diag = TRUE)] <-
                  if (object@hom)
                      rep(getTheta(object), rbind(1L, (nc - 1L):0L)[seq_len(2L * nc - 1L)])
                  else getTheta(object)
              ans
          })

setMethod("getLambda",
          c(object = "Covariance.ar1"),
          function (object) {
              nc <- object@nc
              ans <- matrix(0, nc, nc)
              if (nc > 0L)
              ans[lower.tri(ans, diag = TRUE)] <-
                  if (object@hom)
                      getTheta(object)[sequence.default(from = rep(c(1L, nc + 1L), c(1L, nc - 1L)), nvec = nc:1L)]
                  else getTheta(object)
              ans
          })

setMethod("getLambdat.dp",
          c(object = "Covariance.us"),
          function (object)
              seq_len(object@nc))

setMethod("getLambdat.dp",
          c(object = "Covariance.diag"),
          function (object)
              rep(1L, object@nc))

setMethod("getLambdat.dp",
          c(object = "Covariance.cs"),
          function (object)
              seq_len(object@nc))

setMethod("getLambdat.dp",
          c(object = "Covariance.ar1"),
          function (object)
              seq_len(object@nc))

setMethod("getLambdat.i",
          c(object = "Covariance.us"),
          function (object)
              sequence.default(from = 0L, nvec = seq_len(object@nc)))

setMethod("getLambdat.i",
          c(object = "Covariance.diag"),
          function (object)
              seq.int(from = 0L, length.out = object@nc))

setMethod("getLambdat.i",
          c(object = "Covariance.cs"),
          function (object)
              sequence.default(from = 0L, nvec = seq_len(object@nc)))

setMethod("getLambdat.i",
          c(object = "Covariance.ar1"),
          function (object)
              sequence.default(from = 0L, nvec = seq_len(object@nc)))

setMethod("getVC",
          c(object = "Covariance.us"),
          function (object) {
              nc <- object@nc
              if (nc <= 1L) {
                  vcomp <- object@par
                  ccomp <- double(0L)
              } else {
                  ii <- seq.int(from = 1L, by = nc + 1L, length.out = nc)
                  i0 <- sequence.default(from = seq.int(from = 2L, by = nc + 1L, length.out = nc - 1L),
                                         nvec = (nc - 1L):1L)
                  i1 <- sequence.default(from = ii,
                                         nvec = nc:1L)
                  L <- matrix(0, nc, nc)
                  L[i1] <- object@par
                  S <- tcrossprod(L)
                  vcomp <- sqrt(S[ii])
                  ccomp <- (S/vcomp/rep(vcomp, each = nc))[i0]
              }
              list(vcomp = vcomp, ccomp = ccomp)
          })

setMethod("getVC",
          c(object = "Covariance.diag"),
          function (object)
              list(vcomp = object@par, ccomp = double(0L)))

setMethod("getVC",
          c(object = "Covariance.cs"),
          .fn <-
          function (object) {
              nc <- object@nc
              par <- object@par
              if (nc <= 1L)
                  list(vcomp = par, ccomp = double(0L))
              else list(vcomp = par[seq_len(length(par) - 1L)], ccomp = par[length(par)])
          })

setMethod("getVC",
          c(object = "Covariance.ar1"),
          .fn)

rm(.fn)

setMethod("getVCNames",
          c(object = "Covariance.us", cnm = "character", gnm = "character"),
          function (object, cnm, gnm, prf = NULL, old = TRUE) {
              nms <- getParNames(object, cnm, gnm, prf, old)
              nc <- object@nc
              ii <- cumsum(c(if (nc > 0L) 1L, if (nc > 1L) nc:2L))
              list(vcomp = nms[ii], ccomp = nms[-ii])
          })

setMethod("getVCNames",
          c(object = "Covariance.diag", cnm = "character", gnm = "character"),
          function (object, cnm, gnm, prf = NULL, old = TRUE) {
              nms <- getParNames(object, cnm, gnm, prf, old)
              list(vcomp = nms, ccomp = character(0L))
          })

setMethod("getVCNames",
          c(object = "Covariance.cs", cnm = "character", gnm = "character"),
          .fn <-
          function (object, cnm, gnm, prf = NULL, old = TRUE) {
              nms <- getParNames(object, cnm, gnm, prf, old)
              nc <- object@nc
              if (nc <= 1L)
                  list(vcomp = nms, ccomp = character(0L))
              else list(vcomp = nms[seq_len(nc)], ccomp = nms[nc + 1L])
          })

setMethod("getVCNames",
          c(object = "Covariance.ar1", cnm = "character", gnm = "character"),
          .fn)

rm(.fn)

setMethod("setVC",
          c(object = "Covariance.us", vcomp = "numeric", ccomp = "numeric"),
          function (object, vcomp, ccomp) {
              nc <- object@nc
              stopifnot(is.double(vcomp),
                        length(vcomp) == nc,
                        is.double(ccomp),
                        length(ccomp) == length(object@par) - nc)
              object@par <-
              if (nc <= 1L)
                  vcomp
              else {
                  i0 <- sequence.default(from = seq.int(from = nc + 1L, by = nc + 1L, length.out = nc - 1L),
                                         by = nc,
                                         nvec = (nc - 1L):1L)
                  i1 <- sequence.default(from = seq.int(from = 1L, by = nc + 1L, length.out = nc),
                                         by = nc,
                                         nvec = nc:1L)
                  S <- diag(1, nc, nc)
                  S[i0] <- ccomp
                  (semichol(S) * rep(vcomp, each = nc))[i1]
              }
              object
          })

setMethod("setVC",
          c(object = "Covariance.diag", vcomp = "numeric", ccomp = "numeric"),
          function (object, vcomp, ccomp) {
              stopifnot(is.double(vcomp),
                        length(vcomp) == length(object@par),
                        is.double(ccomp),
                        length(ccomp) == 0L)
              object@par <- vcomp
              object
          })

setMethod("setVC",
          c(object = "Covariance.cs", vcomp = "numeric", ccomp = "numeric"),
          .fn <-
          function (object, vcomp, ccomp) {
              nc <- object@nc
              stopifnot(is.double(vcomp),
                        length(vcomp) == length(object@par) - (nc > 1L),
                        is.double(ccomp),
                        length(ccomp) == (nc > 1L))
              object@par <- c(vcomp, ccomp)
              object
          })

setMethod("setVC",
          c(object = "Covariance.ar1", vcomp = "numeric", ccomp = "numeric"),
          .fn)

rm(.fn)

setMethod("getProfPar",
          c(object = "Covariance.us"),
          function (object, profscale, sc = NULL) {
              vc <- getVC(object)
              vcomp <- vc$vcomp
              ccomp <- vc$ccomp
              if (!is.null(sc))
                  vcomp <- vcomp * sc
              if (profscale == "varcov") {
                  nc <- object@nc
                  if (nc > 1L)
                      ccomp <- ccomp *
                          vcomp[sequence.default(from = 2L:nc, nvec = (nc - 1L):1L)] *
                          vcomp[rep(1L:(nc - 1L), (nc - 1L):1L)]
                  vcomp <- vcomp * vcomp
              }
              c(vcomp, ccomp)
          })

setMethod("getProfPar",
          c(object = "Covariance.diag"),
          function (object, profscale, sc = NULL) {
              vcomp <- object@par
              if (!is.null(sc))
                  vcomp <- vcomp * sc
              if (profscale == "varcov")
                  vcomp <- vcomp * vcomp
              vcomp
          })

setMethod("getProfPar",
          c(object = "Covariance.cs"),
          .fn <-
          function (object, profscale, sc = NULL) {
              if (profscale == "varcov")
                  stop(gettextf("%s=\"%s\" not implemented for class \"%s\"",
                                "profscale", "varcov", class(object)),
                       domain = NA)
              vc <- getVC(object)
              vcomp <- vc$vcomp
              ccomp <- vc$ccomp
              if (!is.null(sc))
                  vcomp <- vcomp * sc
              c(vcomp, ccomp)
          })

setMethod("getProfPar",
          c(object = "Covariance.ar1"),
          .fn)

rm(.fn)

setMethod("setProfPar",
          c(object = "Covariance.us", value = "numeric"),
          function (object, profscale, sc = NULL, value, pos = 0L) {
              np <- length(object@par)
              validValuePos(np, value, pos)
              nc <- object@nc
              vcomp <- value[seq.int(from = pos + 1L     , length.out =      nc)]
              ccomp <- value[seq.int(from = pos + 1L + nc, length.out = np - nc)]
              if (profscale == "varcov") {
                  vcomp <- sqrt(vcomp)
                  if (nc > 1L)
                      ccomp <- ccomp/
                          vcomp[sequence.default(from = 2L:nc, nvec = (nc - 1L):1L)]/
                          vcomp[rep(1L:(nc - 1L), (nc - 1L):1L)]
              }
              if (!is.null(sc))
                  vcomp <- vcomp/
                      (if (profscale == "varcov") sqrt(sc) else sc)
              setVC(object, vcomp, ccomp)
          })

setMethod("setProfPar",
          c(object = "Covariance.diag", value = "numeric"),
          function (object, profscale, sc = NULL, value, pos = 0L) {
              np <- length(object@par)
              validValuePos(np, value, pos)
              vcomp <-
              if (np == length(value))
                  value
              else
                  value[seq.int(from = pos + 1L, length.out = np)]
              if (profscale == "varcov")
                  vcomp <- sqrt(vcomp)
              if (!is.null(sc))
                  vcomp <- vcomp/
                      (if (profscale == "varcov") sqrt(sc) else sc)
              setPar(object, vcomp)
          })

setMethod("setProfPar",
          c(object = "Covariance.cs", value = "numeric"),
          .fn <-
          function (object, profscale, sc = NULL, value, pos = 0L) {
              if (profscale == "varcov")
                  stop(gettextf("%s=\"%s\" not implemented for class \"%s\"",
                                "profscale", "varcov", class(object)),
                       domain = NA)
              np <- length(object@par)
              validValuePos(np, value, pos)
              nc <- object@nc
              nc. <- if (object@hom) 1L * (nc > 0L) else nc
              vcomp <- value[seq.int(from = pos + 1L      , length.out =      nc.)]
              ccomp <- value[seq.int(from = pos + 1L + nc., length.out = np - nc.)]
              if (!is.null(sc))
                  vcomp <- vcomp/sc
              setVC(object, vcomp, ccomp)
          })

setMethod("setProfPar",
          c(object = "Covariance.ar1", value = "numeric"),
          .fn)

rm(.fn)

setMethod("getProfLower",
          c(object = "Covariance.us"),
          function (object, profscale, sc = NULL) {
              nc <- object@nc
              rep(c(0, if (profscale == "varcov") -Inf else -1),
                  c(nc, (nc * (nc - 1L)) %/% 2L))
          })

setMethod("getProfLower",
          c(object = "Covariance.diag"),
          function (object, profscale, sc = NULL)
              getLower(object))

setMethod("getProfLower",
          c(object = "Covariance.cs"),
          .fn <-
          function (object, profscale, sc = NULL) {
              if (profscale == "varcov")
                  stop(gettextf("%s=\"%s\" not implemented for class \"%s\"",
                                "profscale", "varcov", class(object)),
                       domain = NA)
              getLower(object)
          })

setMethod("getProfLower",
          c(object = "Covariance.ar1"),
          .fn)

rm(.fn)

setMethod("getProfUpper",
          c(object = "Covariance.us"),
          function (object, profscale, sc = NULL) {
              nc <- object@nc
              rep(c(Inf, if (profscale == "varcov") Inf else 1),
                  c(nc, (nc * (nc - 1L)) %/% 2L))
          })

setMethod("getProfUpper",
          c(object = "Covariance.diag"),
          function (object, profscale, sc = NULL)
              getUpper(object))

setMethod("getProfUpper",
          c(object = "Covariance.cs"),
          .fn <-
          function (object, profscale, sc = NULL) {
              if (profscale == "varcov")
                  stop(gettextf("%s=\"%s\" not implemented for class \"%s\"",
                                "profscale", "varcov", class(object)),
                       domain = NA)
              getUpper(object)
          })

setMethod("getProfUpper",
          c(object = "Covariance.ar1"),
          .fn)

rm(.fn)

setMethod("isSingular",
          c(object = "Covariance.us"),
          function (object, tol = getSingTol()) {
              nc <- object@nc
              if (nc > 0L) {
                  ii <- cumsum(c(1L, if (nc > 1L) nc:2L))
                  min(object@par[ii]) < tol
              }
              else FALSE
          })

setMethod("isSingular",
          c(object = "Covariance.diag"),
          function (object, tol = getSingTol()) {
              nc <- object@nc
              if (nc > 0L)
                  min(object@par) < tol
              else FALSE
          })

setMethod("isSingular",
          c(object = "Covariance.cs"),
          function (object, tol = getSingTol()) {
              nc <- object@nc
              if (nc > 0L) {
                  vc <- getVC(object)
                  vcomp <- vc$vcomp
                  ccomp <- vc$ccomp # rho
                  min(vcomp) < tol || (nc > 1L && min(ccomp - -1/(nc - 1L), 1 - ccomp) < tol)
              }
              else FALSE
          })

setMethod("isSingular",
          c(object = "Covariance.ar1"),
          function (object, tol = getSingTol()) {
              nc <- object@nc
              if (nc > 0L) {
                  vc <- getVC(object)
                  vcomp <- vc$vcomp
                  ccomp <- vc$ccomp # rho
                  min(vcomp) < tol || (nc > 1L && 1 - abs(ccomp) < tol)
              }
              else FALSE
          })


## .... HELPERS ........................................................

## used in methods for set* which have formal arguments 'value', 'pos'
validValuePos <-
function (n, value, pos) {
    if (!is.double(value))
        stop(gettextf("type of '%s' is not \"%s\"",
                      "value", "double"),
             domain = NA)
    if (n > length(value) - pos)
        stop(gettextf("attempt to read past end of '%s'",
                      "value"),
             domain = NA)
    NULL
}

## used in functions with formal argument 'profscale'
validProfScale <-
function (profscale) {
    if (missing(profscale))
        "sdcor"
    else match.arg(profscale, c("sdcor", "varcov"))
}
if (FALSE) {
## This version is not equivalent due to lazy evaluation.  It assumes
## that 'profscale' can be evaluated in the calling frame when nargs()
## is 1, but that assumption can be false in current usage.
validProfScale <-
function (profscale = c("sdcor", "varcov"))
    match.arg(profscale)
}

## return a function that maps
##     c(theta_1, ..., theta_k) -> c(par_1, ..., par_k)
mkMkPar <-
function (reCovs) {
    if (all(vapply(reCovs, is, FALSE, "Covariance.us")))
        return(function (theta) theta)
    nt <- vapply(reCovs, getThetaLength, 0L, USE.NAMES = FALSE)
    np <- vapply(reCovs, getParLength, 0L, USE.NAMES = FALSE)
    snt <- sum(nt)
    snp <- sum(np)
    jt <- split(seq_len(snt), rep(seq_along(nt), nt))
    jp <- split(seq_len(snp), rep(seq_along(np), np))
    par <- double(snp)
    ii <- seq_along(reCovs)
    function (theta) {
        for (i in ii)
            par[jp[[i]]] <<-
                getPar(setTheta(reCovs[[i]], theta[jt[[i]]]))
        par
    }
}

## return a function that maps
##     c(par_1, ..., par_k) -> c(theta_1, ..., theta_k)
mkMkTheta <-
function (reCovs) {
    if (all(vapply(reCovs, is, FALSE, "Covariance.us")))
        return(function (par) par)
    nt <- vapply(reCovs, getThetaLength, 0L, USE.NAMES = FALSE)
    np <- vapply(reCovs, getParLength, 0L, USE.NAMES = FALSE)
    snt <- sum(nt)
    snp <- sum(np)
    jt <- split(seq_len(snt), rep(seq_along(nt), nt))
    jp <- split(seq_len(snp), rep(seq_along(np), np))
    theta <- double(snt)
    ii <- seq_along(reCovs)
    function (par) {
        for (i in ii)
            theta[jt[[i]]] <<-
                getTheta(setPar(reCovs[[i]], par[jp[[i]]]))
        theta
    }
}

## update mkReTrms(..., calc.lambdat = FALSE) so that it has components
## 'Lambdat', 'Lind', 'theta', 'par', 'lower', 'upper', 'reCovs' where
##
##     length(theta) >= length(par) == length(lower) == length(upper)
##
## note that previously there were no 'par', 'upper', and 'reCovs' as
## previously 'par' and 'theta' were identical by construction
upReTrms <-
function (reTrms, spCalls) {
    getHom <- function (spCall) {
        ## FIXME?  not evaluating 'hom' in environment of formula
        hom <- spCall$hom
        if (is.null(hom) ||
            (is.logical(hom) && length(hom) == 1L && !is.na(hom)))
            hom
        else stop(gettextf("'%s' is not %s or %s in special call",
                           "hom", "TRUE", "FALSE"),
                  domain = NA)
    }
    spNames <- as.character(lapply(spCalls, `[[`, 1L))
    nc <- lengths(reTrms$cnms, use.names = FALSE)
    hom <- lapply(spCalls, getHom)
    reCovs <- .mapply(function (Class, nc, hom) {
                          if (is.null(hom))
                              new(Class, nc = nc)
                          else new(Class, nc = nc, hom = hom)
                      },
                      list(Class = paste0("Covariance.", spNames),
                           nc = nc,
                           hom = hom),
                      NULL)
    theta <- lapply(reCovs, getTheta)
    thetaIndex <- lapply(reCovs, getThetaIndex)
    nt <- lengths(theta)
    nti <- lengths(thetaIndex)
    nl <- reTrms$nl
    nc.nl <- rep(nc, nl)
    nti.nl <- rep(nti, nl)
    R.dp <- unlist(rep(lapply(reCovs, getLambdat.dp), nl), FALSE, FALSE)
    R.p <- cumsum(c(0L, R.dp))
    R.i <- rep(cumsum(c(0L, nc.nl)[seq_along(nc.nl)]), nti.nl) +
        unlist(rep(lapply(reCovs, getLambdat.i), nl), FALSE, FALSE)
    R.x <- rep(cumsum(c(0L, nt)[seq_along(nt)]), nti * nl) +
        unlist(rep(thetaIndex, nl), FALSE, FALSE)
    reTrms$Lambdat <- new("dgCMatrix", Dim = rep(length(R.dp), 2L),
                          p = R.p, i = R.i, x = as.double(R.x))
    reTrms$Lind    <- R.x
    reTrms$theta   <- unlist(                   theta, FALSE, FALSE)
    reTrms$par     <- unlist(lapply(reCovs, getPar  ), FALSE, FALSE)
    reTrms$lower   <- unlist(lapply(reCovs, getLower), FALSE, FALSE)
    reTrms$upper   <- unlist(lapply(reCovs, getUpper), FALSE, FALSE)
    reTrms$reCovs  <- reCovs
    reTrms
}

## use c(theta_1, ..., theta_k) to update par_1, ..., par_k
upReCovs <-
function (reCovs, theta) {
    ## FIXME?  function (reCovs, par) instead?
    pos <- 0L
    for (i in seq_along(reCovs)) {
        elt <- reCovs[[i]]
        reCovs[[i]] <- setTheta(elt, theta, pos)
        pos <- pos + getThetaLength(elt)
    }
    reCovs
}

## return 'reCovs' given 'object' of class "merMod", w/ or w/o attribute
getReCovs <-
function (object) {
    ## stopifnot(is(object, "merMod"))
    if (!is.null(ans <- attr(object, "reCovs")))
        return(ans)
    ## 'object' was obtained using older 'lme4' => all 'us' or 'diag'.
    ## It seems that distinguishing these two cases requires some
    ## gymnastics.  We nonetheless try to be fast in the all-'us' case.
    nc <- lengths(object@cnms, use.names = FALSE)
    form <- formula(object)
    doublebars <- reformulas::findbars_x(form, target = "||",
                                         default.special = NULL)
    reCovs <-
    if (length(doublebars) > 0L) {
        nc.. <- vapply(mmList(form, model.frame(object)), ncol, 0L,
                       USE.NAMES = FALSE)
        dptr <- diff(ptr <- c(0L, match(cumsum(nc), cumsum(nc..))))
        Class <- rep("Covariance.us", length(nc))
        Class[dptr > 1L] <- "Covariance.diag"
        .mapply(new, list(Class = Class, nc = nc), NULL)
    }
    else .mapply(new, list(nc = nc), list(Class = "Covariance.us"))
    upReCovs(reCovs, object@theta)
}

convParToProfPar <-
function (value, object, profscale, sc = NULL) {
    profscale <- validProfScale(profscale)
    reCovs <- getReCovs(object)
    np <- vapply(reCovs, getParLength, 0L, USE.NAMES = FALSE)
    pos <- cumsum(c(0L, np))[seq_along(np)]
    L <- .mapply(function (object, pos)
                     getProfPar(setPar(object, value, pos), profscale, sc),
                 list(object = reCovs, pos = pos),
                 NULL)
    c(L, if (!is.null(sc) && profscale == "varcov") sc^2 else sc,
      recursive = TRUE, use.names = FALSE)
}

convProfParToPar <-
function (value, object, profscale, sc = NULL) {
    profscale <- validProfScale(profscale)
    reCovs <- getReCovs(object)
    np <- vapply(reCovs, getParLength, 0L, USE.NAMES = FALSE)
    pos <- cumsum(c(0L, np))[seq_along(np)]
    ## FIXME?  it could make sense to here do something like:
    ## if (is.null(sc) && length(value) > (snp <- sum(np)))
    ##     sc <- value[snp + 1L]
    L <- .mapply(function (object, pos)
                     getPar(setProfPar(object, profscale, sc, value, pos)),
                 list(object = reCovs, pos = pos),
                 NULL)
    c(L, recursive = TRUE, use.names = FALSE)
}

isNewMerMod <-
function (object)
    !is.null(attr(object, "upper")) && !is.null(attr(object, "reCovs"))

forceNewMerMod <-
function (object, reference = object) {
    if (is.null(attr(object, "upper")))
        attr(object, "upper") <- getUpper(reference)
    if (is.null(attr(object, "reCovs")))
        attr(object, "reCovs") <- getReCovs(reference)
    object
}

.anyStructured <-
function (reCovs)
    !all(vapply(reCovs, is, FALSE, "Covariance.us"))

anyStructured <-
function (object)
    !is.null(reCovs <- attr(object, "reCovs")) && .anyStructured(reCovs)

semichol <-
if (FALSE) {
function (x, tol = -1, etol = 256 * .Machine$double.neg.eps, type = "O") {
    R <- tryCatch(chol(x), error = function (e) NULL)
    if (!is.null(R))
        return(R)
    R <- suppressWarnings(chol(x, pivot = TRUE, tol = tol))
    r <- attr(R, "rank")
    p <- order(attr(R, "pivot"))
    n <- ncol(x)
    if (r < n) {
        j <- (r + 1L):n
        R[j, j] <- 0
    }
    RP <- R[, p, drop = FALSE]
    if (n > 1L) { # symmetrize 'x' (put upper part into lower part)
        j <- 1L:(n - 1L)
        x[sequence.default(from = 2L:n, by = n, nvec = j)] <-
            x[sequence.default(from = j * n + 1L, nvec = j)]
    }
    e <- norm(x - crossprod(RP), type = type)
    if (is.na(e) || e > etol * norm(x, type = type))
        stop(gettextf("'%s' is not positive semidefinite", "x"),
             domain = NA)
    RP # upper triangular if is.unsorted(p) is FALSE
}
} else {
function (x, tol = 256 * .Machine$double.neg.eps) {
    R <- tryCatch(chol(x), error = function (e) NULL)
    if (!is.null(R))
        return(R)
    e <- eigen(t(x), symmetric = TRUE)
    r <- range(e$values)
    if (r[2L] < 0 || r[1L] < -tol * r[2L])
        stop(gettextf("'%s' is not positive semidefinite", "x"),
             domain = NA)
    if (r[1L] < 0)
        e$values[e$values < 0] <- 0
    qr.R(qr(sqrt(e$values) * t(e$vectors))) # upper triangular
}
}


## .... METHODS [for class "merMod"] ...................................

setMethod("getPar",
          c(object = "merMod"),
          function (object)
              `length<-`(object@optinfo[["val"]],
                         length(object@lower)))

setMethod("getParLength",
          c(object = "merMod"),
          function (object)
              length(object@lower))

setMethod("getParNames",
          c(object = "merMod", cnm = "missing", gnm = "missing"),
          function (object, cnm, gnm, prf = NULL, old = TRUE) {
              reCovs <- getReCovs(object)
              gnm <- names(cnm <- object@cnms)
              unlist(.mapply(getParNames, list(object = reCovs, cnm = cnm, gnm = gnm), list(prf = prf, old = old)), FALSE, FALSE)
          })

setMethod("getTheta",
          c(object = "merMod"),
          function (object)
              object@theta)

setMethod("getThetaLength",
          c(object = "merMod"),
          function (object)
              length(object@theta))

setMethod("getThetaNames",
          c(object = "merMod", cnm = "missing", gnm = "missing"),
          function (object, cnm, gnm, prf = NULL, old = TRUE) {
              reCovs <- getReCovs(object)
              gnm <- names(cnm <- object@cnms)
              unlist(.mapply(getThetaNames, list(object = reCovs, cnm = cnm, gnm = gnm), list(prf = prf, old = old)), FALSE, FALSE)
          })

setMethod("getLower",
          c(object = "merMod"),
          function (object)
              object@lower)

setMethod("getUpper",
          c(object = "merMod"),
          function (object)
              attr(object, "upper") %||% rep(Inf, length(object@lower)))

setMethod("getLambda",
          c(object = "merMod"),
          function (object) {
              reCovs <- getReCovs(object)
              nc <- vapply(reCovs, slot, 0L, "nc")
           ## theta <- lapply(reCovs, getTheta)
           ## nt <- lengths(theta)
              nt <- vapply(reCovs, getThetaLength, 0L)
              thetaIndex <- lapply(reCovs, getThetaIndex)
              nti <- lengths(thetaIndex)
           ## nti <- vapply(reCovs, getThetaIndexLength, 0L)
              nl <- vapply(object@flist[names(object@cnms)], nlevels, 0L)
              ## <copy-paste from upReTrms>
              nc.nl <- rep(nc, nl)
              nti.nl <- rep(nti, nl)
              R.dp <- unlist(rep(lapply(reCovs, getLambdat.dp), nl), FALSE, FALSE)
              R.p <- cumsum(c(0L, R.dp))
              R.i <- rep(cumsum(c(0L, nc.nl)[seq_along(nc.nl)]), nti.nl) +
                  unlist(rep(lapply(reCovs, getLambdat.i), nl), FALSE, FALSE)
              R.x <- rep(cumsum(c(0L, nt)[seq_along(nt)]), nti * nl) +
                  unlist(rep(thetaIndex, nl), FALSE, FALSE)
              ## </copy-paste from upReTrms>
              t(new("dgCMatrix", Dim = rep(length(R.dp), 2L),
                    p = R.p, i = R.i, x = object@theta[R.x]))
          })

setMethod("getVCNames",
          c(object = "merMod", cnm = "missing", gnm = "missing"),
          function (object, cnm, gnm, prf = NULL, old = TRUE) {
              reCovs <- getReCovs(object)
              gnm <- names(cnm <- object@cnms)
              L <- .mapply(getVCNames, list(object = reCovs, cnm = cnm, gnm = gnm), list(prf = prf, old = old))
              list(vcomp = lapply(L, `[[`, 1L),
                   ccomp = lapply(L, `[[`, 2L))
          })

setMethod("getProfPar",
          c(object = "merMod"),
          function (object, profscale, sc = NULL) {
              reCovs <- getReCovs(object)
              L <- lapply(reCovs, getProfPar, profscale = profscale, sc = sc)
              c(L, if (!is.null(sc) && profscale == "varcov") sc^2 else sc,
                recursive = TRUE, use.names = FALSE)
          })

setMethod("getProfLower",
          c(object = "merMod"),
          function (object, profscale, sc = NULL) {
              reCovs <- getReCovs(object)
              L <- lapply(reCovs, getProfLower, profscale = profscale)
              c(L, if (!is.null(sc)) 0,
                recursive = TRUE, use.names = FALSE)
          })

setMethod("getProfUpper",
          c(object = "merMod"),
          function (object, profscale, sc = NULL) {
              reCovs <- getReCovs(object)
              L <- lapply(reCovs, getProfUpper, profscale = profscale)
              c(L, if (!is.null(sc)) Inf,
                recursive = TRUE, use.names = FALSE)
          })

setMethod("isSingular",
          c(object = "merMod"),
          function (object, tol = getSingTol()) {
              reCovs <- getReCovs(object)
              for (object in reCovs)
                  if (isSingular(object, tol = tol))
                      return(TRUE)
              FALSE
          })

library(lme4)

if (.Platform$OS.type != "windows") {
data(Orthodont, package="nlme")
fm1 <- lmer(distance ~ age + (age|Subject), data = Orthodont)
VarCorr(fm1)

fm2ML <- lmer(diameter ~ 1 + (1|plate) + (1|sample), Penicillin, REML=0)
VarCorr(fm2ML)

gm1 <- glmer(cbind(incidence,size-incidence) ~ period + (1|herd),data=cbpp,
             family=binomial)
VarCorr(gm1)

cbpp$obs <- factor(seq(nrow(cbpp)))
gm2 <- update(gm1,.~.+(1|obs))
VarCorr(gm2)

if (FALSE) {
  ## testing lme4/lme4 incompatibility
##  library(lme4)
  VarCorr(fm1)
  lme4:::VarCorr.merMod(fm1) ## OK
}
} ## skip on windows (for speed)

if (.Platform$OS.type != "windows") withAutoprint({
    library(lme4)

    fm1 <- lmer(Reaction ~ Days + (Days|Subject), sleepstudy)

    ## slightly weird model but plausible --- not that
    ##   one would want to try drop1() on this model ...
    fm2 <- lmer(Reaction ~ 1+ (Days|Subject), sleepstudy)
    drop1(fm2)  ## empty
    update(fm1, . ~ . - Days)
    anova(fm2) ## empty

    terms(fm1)
    terms(fm1,fixed.only=FALSE)

    extractAIC(fm1)

    drop1(fm1)
    drop1(fm1, test="Chisq")

    gm1 <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
                 family = binomial, data = cbpp, nAGQ=25L)

    drop1(gm1, test="Chisq")

}) ## skip on windows (for speed)

## compare range, average, etc. of simulations to
## conditional and unconditional prediction
library(lme4)
do.plot <- FALSE

if (.Platform$OS.type != "windows") {
## use old (<=3.5.2) sample() algorithm if necessary
if ("sample.kind" %in% names(formals(RNGkind))) {
    suppressWarnings(RNGkind("Mersenne-Twister", "Inversion", "Rounding"))
}

fm1 <- lmer(Reaction~Days+(1|Subject),sleepstudy)
set.seed(101)
pp <- predict(fm1)
rr <- range(usim2 <- simulate(fm1,1,use.u=TRUE)[[1]])
stopifnot(all.equal(rr,c(159.3896,439.1616),tolerance=1e-6))
if (do.plot) {
    plot(pp,ylim=rr)
    lines(sleepstudy$Reaction)
    points(simulate(fm1,1)[[1]],col=4)
    points(usim2,col=2)
}

set.seed(101)

## conditional prediction
ss <- simulate(fm1,1000,use.u=TRUE)
ss_sum <- t(apply(ss,1,quantile,c(0.025,0.5,0.975)))
plot(pp)
matlines(ss_sum,col=c(1,2,1),lty=c(2,1,2))
stopifnot(all.equal(ss_sum[,2],pp,tolerance=5e-3))

## population-level prediction
pp2 <- predict(fm1, re.form=NA)
ss2 <- simulate(fm1,1000,use.u=FALSE)
ss_sum2 <- t(apply(ss2,1,quantile,c(0.025,0.5,0.975)))

if (do.plot) {
    plot(pp2,ylim=c(200,400))
    matlines(ss_sum2,col=c(1,2,1),lty=c(2,1,2))
}

stopifnot(all.equal(ss_sum2[,2],pp2,tolerance=8e-3))

## predict(...,newdata=...) on models with derived variables in the random effects
## e.g. (f:g, f/g)
set.seed(101)
d <- expand.grid(f=factor(letters[1:10]),g=factor(letters[1:10]),
                 rep=1:10)
d$y <- rnorm(nrow(d))
m1 <- lmer(y~(1|f:g),d)
p1A <- predict(m1)
p1B <- predict(m1,newdata=d)
stopifnot(all.equal(p1A,p1B))
m2 <- lmer(y~(1|f/g),d)
p2A <- predict(m2)
p2B <- predict(m2,newdata=d)
stopifnot(all.equal(p2A,p2B))

## with numeric grouping variables
dn <- transform(d,f=as.numeric(f),g=as.numeric(g))
m1N <- update(m1,data=dn)
p1NA <- predict(m1N)
p1NB <- predict(m1N,newdata=dn)
stopifnot(all.equal(p1NA,p1NB))

## simulate with modified parameters
set.seed(1)
s1 <- simulate(fm1)
set.seed(1)
s2 <- simulate(fm1,newdata=model.frame(fm1),
               newparams=getME(fm1,c("theta","beta","sigma")))
all.equal(s1,s2)

fm0 <- update(fm1,.~.-Days)
##
## sim() -> simulate() -> refit() -> deviance
##

## predictions and simulations with offsets

set.seed(101)
d <- data.frame(y=rpois(100,5),x=rlnorm(100,1,1),
                f=factor(sample(10,size=100,replace=TRUE)))
gm1 <- glmer(y~offset(log(x))+(1|f),data=d,
             family=poisson)
s1 <- simulate(gm1)
} ## skip on windows (for speed)

if (lme4:::testLevel() > 1 || .Platform$OS.type!="windows") withAutoprint({

    ## generalized linear mixed model
    stopifnot(suppressPackageStartupMessages(require(lme4)))
    options(show.signif.stars = FALSE)

    source(system.file("test-tools-1.R", package = "Matrix"), keep.source = FALSE)
    ##
    ##' Check that coefficient +- "2" * SD  contains true value
    ##'
    ##' @title Check that confidence interval for coefficients contains true value
    ##' @param fm fitted model, e.g., from  lm(), lmer(), glmer(), ..
    ##' @param true.coef numeric vector of true (fixed effect) coefficients
    ##' @param conf.level confidence level for confidence interval
    ##' @param sd.factor the "2", i.e. default 1.96 factor for the confidence interval
    ##' @return TRUE or a string of "error"
    ##' @author Martin Maechler
    chkFixed <- function(fm, true.coef, conf.level = 0.95,
                         sd.factor = qnorm((1+conf.level)/2))
    {
        stopifnot(is.matrix(cf <- coefficients(summary(fm))), ncol(cf) >= 2)
        cc <- cf[,1]
        sd <- cf[,2]
        if(any(out1 <- true.coef < cc - sd.factor*sd))
            return(sprintf("true coefficient[j], j=%s, is smaller than lower confidence limit",
                           paste(which(out1), collapse=", ")))
        if(any(out2 <- true.coef > cc + sd.factor*sd))
            return(sprintf("true coefficient[j], j=%s, is larger than upper confidence limit",
                           paste(which(out2), collapse=", ")))
        ## else, return
        TRUE
    }


    ## TODO: (1) move these to ./glmer-ex.R [DONE]
    ## ----  (2) "rationalize" with ../man/cbpp.Rd
                                        #m1e <- glmer1(cbind(incidence, size - incidence) ~ period + (1 | herd),
                                        #              family = binomial, data = cbpp, doFit = FALSE)
    ## now
                                        #bobyqa(m1e, control = list(iprint = 2L))

    m1 <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
                family = binomial, data = cbpp)
    m1. <- update(m1, start = getME(m1, c("theta", "fixef")))
    dm1 <- drop1(m1)
    stopifnot(all.equal(drop1(m1.), dm1, tol = 1e-10))# Lnx(F28) 64b: 4e-12
    ## response as a vector of probabilities and usage of argument "weights"
    m1p <- glmer(incidence / size ~ period + (1 | herd), weights = size,
                 family = binomial, data = cbpp)
    ## Confirm that these are equivalent:
    stopifnot(all.equal(fixef(m1), fixef(m1p)),
              all.equal(ranef(m1), ranef(m1p)),
              TRUE)
    ## for(m in c(m1, m1p)) {
    ##     cat("-------\\n\\nCall: ",
    ##         paste(format(getCall(m)), collapse="\\n"), "\\n")
    ##     print(logLik(m)); cat("AIC:", AIC(m), "\\n") ; cat("BIC:", BIC(m),"\\n")
    ## }
    stopifnot(all.equal(logLik(m1), logLik(m1p)),
              all.equal(AIC(m1),    AIC(m1p)),
              all.equal(BIC(m1),    BIC(m1p)))


    ## changed tolPwrss to 1e-7 to match other default
    m1b <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
                 family = binomial, data = cbpp, verbose = 2L,
                 control =
                     glmerControl(optimizer="bobyqa", tolPwrss=1e-7,
                                  optCtrl=list(rhobeg=0.2, rhoend=2e-7)))

    ## using nAGQ=9L provides a better evaluation of the deviance
    m.9 <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
                 family = binomial, data = cbpp, nAGQ = 9)

    ## check with nAGQ = 25
    m2 <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
                family = binomial, data = cbpp, nAGQ = 25)

    ## loosened tolerance on parameters
    stopifnot(is((cm2 <- coef(m2)), "coef.mer"),
              dim(cm2$herd) == c(15,4),
              all.equal(fixef(m2),
### lme4a [from an Ubuntu 11.10 amd64 system]
                        c(-1.39922533406847, -0.991407294757321,
                          -1.12782184600404, -1.57946627431248),
                        ##c(-1.3766013, -1.0058773,
                        ##  -1.1430128, -1.5922817),
                        tolerance = 5.e-4,
                        check.attributes=FALSE),
              all.equal(c(-2*logLik(m2)), 100.010030538022, tolerance=1e-9),
              all.equal(deviance(m2), 73.373, tolerance=1e-5)
              ## with bobyqa first (AGQ=0), then
              ##all.equal(deviance(m2), 101.119749563, tolerance=1e-9)
              )

    ## 32-bit Ubuntu 10.04:
    coef_m1_lme4.0 <- structure(c(-1.39853505102576,
                                  -0.992334712470269, -1.12867541092127,
                                  -1.58037389566025),
                                names = c("(Intercept)", "period2", "period3",
                                           "period4"))

    ## library(glmmADMB)
    ## mg <- glmmadmb(cbind(incidence, size - incidence) ~ period + (1 | herd),
    ##                family = "binomial", data = cbpp)
    coef_m1_glmmadmb <- structure(c(-1.39853810064827, -0.99233330126975, -1.12867317840779,
                                    -1.58031150854503), names = c("(Intercept)", "period2", "period3",
                                                                   "period4"))

    ## library(glmmML)
    ## mm <- glmmML(cbind(incidence, size - incidence) ~ period,
    ##              cluster=herd,
    ##             family = "binomial", data = cbpp)
    coef_m1_glmmML <- structure(c(-1.39853234657711, -0.992336901732793, -1.12867036466201,
                                  -1.58030977686564), names = c("(Intercept)", "period2", "period3",
                                                                 "period4"))

    ## lme4[r 1636], 64-bit ubuntu 11.10:
    ## c(-1.3788385, -1.0589543,
    ##                      -1.1936382, -1.6306271),

    stopifnot(is((cm1 <- coef(m1b)), "coef.mer"),
              dim(cm1$herd) == c(15,4),
              all.equal(fixef(m1b),fixef(m1),tolerance=4e-5),
              is.all.equal4(fixef(m1b),
                            coef_m1_glmmadmb,
                            coef_m1_lme4.0,
                            coef_m1_glmmML,
                            tol = 5e-4)
              )


    ## Deviance for the new algorithm is lower, eventually we should change the previous test
    ##stopifnot(deviance(m1) <= deviance(m1e))

    showProc.time() #

    if (require('MASS', quietly = TRUE)) {
        bacteria$wk2 <- bacteria$week > 2
        contrasts(bacteria$trt) <-
            structure(contr.sdif(3),
                      dimnames = list(NULL, c("diag", "encourage")))
        print(fm5 <- glmer(y ~ trt + wk2 + (1|ID),
                           data=bacteria, family=binomial))
        showProc.time() #

        stopifnot(
            all.equal(logLik(fm5),
                      ## was	  -96.127838
                      structure(-96.13069, nobs = 220L, nall = 220L,
                                df = 5L, REML = FALSE,
                                class = "logLik"),
                      tolerance = 5e-4, check.attributes = FALSE)
           ,
            all.equal(fixef(fm5),
                      ## was		 2.834218798		 -1.367099481
                      c("(Intercept)"= 2.831609490, "trtdiag"= -1.366722631,
                        ## now	 0.5842291915,		 -1.599148773
                        "trtencourage"=0.5840147802, "wk2TRUE"=-1.598591346),
                      tolerance = 1e-4 )
        )
    }

    ## Failure to specify a random effects term - used to give an obscure message
    ## Ensure *NON*-translated message; works on Linux,... :
    if(.Platform$OS.type == "unix") {
        Sys.setlocale("LC_MESSAGES", "C")
        tc <- tryCatch(
            m2 <- glmer(incidence / size ~ period, weights = size,
                        family = binomial, data = cbpp)
          , error = function(.) .)
        stopifnot(inherits(tc, "error"),
                  identical(tc$message,
                            "No random effects terms specified in formula"))
    }


    ## glmer - Modeling overdispersion as "mixture" aka
    ## ----- - *ONE* random effect *PER OBSERVATION" -- example inspired by Ben Bolker:

    ##' <description>
    ##'
    ##' <details>
    ##' @title
    ##' @param ng number of groups
    ##' @param nr number of "runs", i.e., observations per groups
    ##' @param sd standard deviations of group and "Individual" random effects,
    ##'    (\sigma_f, \sigma_I)
    ##' @param b  true beta (fixed effects)
    ##' @return a data frame (to be used in glmer()) with columns
    ##'    (x, f, obs, eta0, eta, mu, y), where y ~ Pois(lambda(x)),
    ##'                                   log(lambda(x_i)) = b_1 + b_2 * x + G_{f(i)} + I_i
    ##'    and G_k ~ N(0, \sigma_f);  I_i ~ N(0, \sigma_I)
    ##' @author Ben Bolker and Martin Maechler
    rPoisGLMMi <- function(ng, nr, sd=c(f = 1, ind = 0.5), b=c(1,2))
    {
        stopifnot(nr >= 1, ng >= 1,
                  is.numeric(sd), names(sd) %in% c("f","ind"), sd >= 0)
        ntot <- nr*ng
        b.reff <- rnorm(ng,  sd= sd[["f"]])
        b.rind <- rnorm(ntot,sd= sd[["ind"]])
        x <- runif(ntot)
        within(data.frame(x,
                          f = factor(rep(LETTERS[1:ng], each=nr)),
                          obs = 1:ntot,
                          eta0 = cbind(1, x) %*% b),
        {
            eta <- eta0 + b.reff[f] + b.rind[obs]
            mu <- exp(eta)
            y <- rpois(ntot, lambda=mu)
        })
    }

    set.seed(1)
    dd <- rPoisGLMMi(12, 20)
    m0  <- glmer(y~x + (1|f),           family="poisson", data=dd)
    m1 <- glmer(y~x + (1|f) + (1|obs), family="poisson", data=dd)
    stopifnot(isTRUE(chkFixed(m0, true.coef = c(1,2))),
              isTRUE(chkFixed(m1, true.coef = c(1,2))))
    (a01 <- anova(m0, m1))

    stopifnot(all.equal(a01$Chisq[2], 554.334056, tolerance=1e-5),
              all.equal(a01$logLik, c(-1073.77193, -796.604902), tolerance=1e-6),
              a01$ npar == 3:4,
              na.omit(a01$ Df) == 1)

    if(lme4:::testLevel() > 1) {
        nsim <- 10
        set.seed(2)
        system.time(
            simR <- lapply(1:nsim,  function(i) {
                cat(i,"", if(i %% 20 == 0)"\n")
                dd <- rPoisGLMMi(10 + rpois(1, lambda=3),
                                 16 + rpois(1, lambda=5))
                m0 <- glmer(y~x + (1|f),           family="poisson", data=dd)
                m1 <- glmer(y~x + (1|f) + (1|obs), family="poisson", data=dd)
                a01 <- anova(m0, m1)
                stopifnot(a01$ npar == 3:4,
                          na.omit(a01$ Df) == 1)
                list(chk0 = chkFixed(m0, true.coef = c(1,2)),
                     chk1 = chkFixed(m1, true.coef = c(1,2)),
                     chisq= a01$Chisq[2],
                     lLik = a01$logLik)
            }))

        ## m0 is the wrong model, so we don't expect much here:
        table(unlist(lapply(simR, `[[`, "chk0")))


        ## If the fixed effect estimates were unbiased and the standard errors correct,
        ## and N(0,sigma^2) instead of t_{nu} good enough for the fixed effects,
        ## the confidence interval should contain the true coef in ~95 out of 100:
        table(unlist(lapply(simR, `[[`, "chk1")))

        ## The tests are all highly significantly in favor of  m1 :
        summary(chi2s <- sapply(simR, `[[`, "chisq"))
        ##  Min. 1st Qu.  Median    Mean 3rd Qu.    Max.
        ## 158.9   439.0   611.4   698.2   864.3  2268.0
        stopifnot(chi2s > qchisq(0.9999, df = 1))
    }

    showProc.time()
})  ## skip if windows and testLevel<1

## NB:  doc in ../man/*.Rd  ***not*** auto generated
## FIXME: need to document S3 methods better (can we pull from r-forge version?)

##' Fit a linear mixed model (LMM)
lmer <- function(formula, data=NULL, REML = TRUE,
                 control = lmerControl(), start = NULL
               , verbose = 0L
               , subset, weights, na.action, offset
               , contrasts = NULL
               , devFunOnly=FALSE
                )
    ## , ...)
{
    mc <- mcout <- match.call()
    missCtrl <- missing(control)
    ## see functions in modular.R for the body ..
    if (!missCtrl && !inherits(control, "lmerControl")) {
        if(!is.list(control)) stop("'control' is not a list; use lmerControl()")
        ## back-compatibility kluge
        warning("passing control as list is deprecated: please use lmerControl() instead",
                immediate.=TRUE)
        control <- do.call(lmerControl, control)
    }
    ## if (!is.null(list(...)[["family"]])) {
    ##    warning("calling lmer with 'family' is deprecated; please use glmer() instead")
    ##    mc[[1]] <- quote(lme4::glmer)
    ##    if(missCtrl) mc$control <- glmerControl()
    ##    return(eval(mc, parent.frame(1L)))
    ## }
    mc$control <- control ## update for  back-compatibility kluge

    ## https://github.com/lme4/lme4/issues/50
    ## parse data and formula
    mc[[1]] <- quote(lme4::lFormula)
    lmod <- eval(mc, parent.frame(1L))
    mcout$formula <- lmod$formula
    lmod$formula <- NULL

    if (is.matrix(y <- model.response(lmod$fr)) && ncol(y) > 1) {
        stop("can't handle matrix-valued responses: consider using refit()")
    }

    ## create deviance function for covariance parameters
    lmod.. <- list(start=start, verbose=verbose, control=control)
    devfun <- do.call(mkLmerDevfun, c(lmod, lmod..))
    if (devFunOnly) return(devfun)
    rho <- environment(devfun)

    nobs <- nrow(lmod$fr)
    npar <- length(rho$lower)
    calc.derivs <- control$calc.derivs %||%
        (nobs < control$checkConv$check.conv.nobsmax &&
         npar < control$checkConv$check.conv.nparmax)
    
    if (identical(control$optimizer, "none"))
        stop("deprecated use of optimizer==\"none\"; use NULL instead")

    ## optimize deviance function over covariance parameters
    opt <- if (length(control$optimizer)==0) {
               s <- getStart(start, rho, 0L)
               list(par=s,fval=devfun(s),
                    conv=1000,message="no optimization")
           }  else {
               optimizeLmer(devfun, optimizer = control$optimizer,
                     restart_edge = control$restart_edge,
                     boundary.tol = control$boundary.tol,
                     control = control$optCtrl,
                     verbose=verbose,
                     start=start,
                     calc.derivs=calc.derivs,
                     force.calc.derivs=control$calc.derivs %||% FALSE,
                     use.last.params=control$use.last.params)
           }
    cc <- checkConv(attr(opt,"derivs"), opt$par,
                    ctrl = control$checkConv,
                    lbound = rho$lower,
                    ubound = rho$upper,
                    nobs = nobs,
                    ndim = npar)
    mkMerMod(rho, opt, lmod$reTrms, fr = lmod$fr,
             mc = mcout, lme4conv=cc) ## prepare output
}## { lmer }


##' Fit a generalized linear mixed model (GLMM)
glmer <- function(formula, data=NULL
                , family = gaussian
                , control = glmerControl()
                , start = NULL
                , verbose = 0L
                , nAGQ = 1L
                , subset, weights, na.action, offset, contrasts = NULL
                , mustart, etastart
                , devFunOnly = FALSE)
{
    if (!inherits(control, "glmerControl")) {
        if(!is.list(control)) stop("'control' is not a list; use glmerControl()")
        ## back-compatibility kluge
        if (class(control)[1]=="lmerControl") {
            warning("please use glmerControl() instead of lmerControl()",
                    immediate.=TRUE)
            control <-
                ## unpack sub-lists
                c(control[!names(control) %in% c("checkConv","checkControl")],
                  control$checkControl,control$checkConv)
            control["restart_edge"] <- NULL ## not implemented for glmer
        } else {
            msg <- "Use control=glmerControl(..) instead of passing a list"
            if(length(cl <- class(control))) {
                msg <- paste(msg, "of class", dQuote(cl[1]))
            }
            warning(msg, immediate.=TRUE)
        }

        control <- do.call(glmerControl, control)
    }
    mc <- mcout <- match.call()

    ## family-checking code duplicated here and in glFormula (for now) since
    ## we really need to redirect at this point; eventually deprecate formally
    ## and clean up
    if (is.character(family))
        family <- get(family, mode = "function", envir = parent.frame(2))
    if( is.function(family)) family <- family()
    if (isTRUE(all.equal(family, gaussian()))) {
        ## redirect to lmer (with warning)
        warning("calling glmer() with family=gaussian (identity link) as a shortcut to lmer() is deprecated;",
                " please call lmer() directly")
        mc[[1]] <- quote(lme4::lmer)
        mc["family"] <- NULL            # to avoid an infinite loop
        return(eval(mc, parent.frame()))
    }

    ## see https://github.com/lme4/lme4/issues/50
    ## parse the formula and data
    mc[[1]] <- quote(lme4::glFormula)
    glmod <- eval(mc, parent.frame(1L))
    mcout$formula <- glmod$formula
    glmod$formula <- NULL
    
    if (is.matrix(y <- model.response(glmod$fr))
        && ((family$family != "binomial" && ncol(y) > 1) ||
            (ncol(y) >2))) {
        stop("can't handle matrix-valued responses: consider using refit()")
    }

    ## create deviance function for covariance parameters
    nAGQinit <- if (control$nAGQ0initStep) 0L else nAGQ
    glmod.. <- list(nAGQ=nAGQinit, verbose=verbose, control=control)
    devfun <- do.call(mkGlmerDevfun, c(glmod, glmod..))
    if (devFunOnly && nAGQ == 0L) return(devfun)
    rho <- environment(devfun)

    if (nAGQinit == 0L) {
        start0 <-
        if (nAGQ > 0L && is.list(start) && !is.null(names(start)))
            start[!names(start) %in% c("fixef", "beta")]
        else start
        ## optimize deviance function over covariance parameters
        ## FIXME: allow calc.derivs, use.last.params, etc. if nAGQ=0
        opt <- optimizeGlmer(devfun,
                             optimizer = control$optimizer[[1]],
                             ## DON'T try fancy edge tricks unless nAGQ=0 explicitly set
                             restart_edge=if (nAGQ==0) control$restart_edge else FALSE,
                             boundary.tol=if (nAGQ==0) control$boundary.tol else 0,
                             control = control$optCtrl,
                             start = start0,
                             nAGQ = 0,
                             verbose=verbose,
                             calc.derivs=FALSE)

        if (nAGQ > 0L) {
        ## update deviance function to include fixed effects
        devfun <- updateGlmerDevfun(devfun, glmod$reTrms, nAGQ = nAGQ)
        if (devFunOnly) return(devfun)
        start <- updateStart(start, opt$par)
        }
    }

    nobs <- nrow(glmod$fr)
    npar <- length(rho$lower)
    calc.derivs <- control$calc.derivs %||%
        (nobs < control$checkConv$check.conv.nobsmax &&
         npar < control$checkConv$check.conv.nparmax)

    if (nAGQ > 0L)
        ## optimize deviance function over covariance parameters and
        ## fixed effects
        opt <- optimizeGlmer(devfun,
                             optimizer = control$optimizer[[2]],
                             restart_edge=control$restart_edge,
                             boundary.tol=control$boundary.tol,
                             control = control$optCtrl,
                             start=start,
                             nAGQ=nAGQ,
                             verbose = verbose,
                             calc.derivs=calc.derivs,
                             use.last.params=control$use.last.params)

    cc <- if (!calc.derivs) NULL else {
        if (verbose > 10) cat("checking convergence\n")
        checkConv(attr(opt,"derivs"),opt$par,
                  ctrl = control$checkConv,
                  lbound = rho$lower,
                  ubound = rho$upper,
                  nobs = nobs,
                  ndim = npar)
    }

    ## prepare output
    mkMerMod(rho, opt, glmod$reTrms, fr = glmod$fr,
             mc = mcout, lme4conv=cc)

}## {glmer}

##' Fit a nonlinear mixed-effects model
nlmer <- function(formula, data=NULL, control = nlmerControl(), start = NULL, verbose = 0L,
                  nAGQ = 1L, subset, weights, na.action, offset,
                  contrasts = NULL, devFunOnly = FALSE)
{

    vals <- nlformula(mc <- match.call())
    p <- ncol(X <- vals$X)
    if ((rankX <- rankMatrix(X)) < p)
        stop(gettextf("rank of X = %d < ncol(X) = %d", rankX, p))

    rho <- list2env(list(verbose=verbose,
                         tolPwrss=0.001, # this is reset to the tolPwrss argument's value later
                         resp=vals$respMod),
                    parent=parent.frame())
    rho$pp <- do.call(merPredD$new,
                      c(vals$reTrms[c("Zt","theta","Lambdat","Lind")],
                        list(X=X, n=length(vals$respMod$mu), Xwts=vals$respMod$sqrtXwt,
                             beta0=qr.coef(qr(X), unlist(lapply(vals$pnames, get,
                             envir = vals$respMod$nlenv))))))
    rho$mkPar <- mkMkPar(vals$reTrms$reCovs)
    rho$mkTheta <- mkMkTheta(vals$reTrms$reCovs)
    rho$u0 <- rho$pp$u0
    rho$beta0 <- rho$pp$beta0
    ## deviance as a function of theta only :
    devfun <- mkdevfun(rho, 0L, verbose=verbose, control=control)
    if (devFunOnly && !nAGQ) return(devfun)
    devfun(rho$mkPar(rho$pp$theta)) # initial coarse evaluation to get u0 and beta0
    rho$u0 <- rho$pp$u0
    rho$beta0 <- rho$pp$beta0
    rho$tolPwrss <- control$tolPwrss # Reset control parameter (the initial optimization is coarse)
    rho$nAGQ <- nAGQ
    
    ## set lower and upper bounds: if user-specified, select
    ##  only the ones corresponding to random effects
    lower <- vals$reTrms$lower
    if (!is.null(lwr <- control$optCtrl$lower)) {
        lower <- lwr[seq_along(lower)]
        control$optCtrl$lower <- NULL
    }
    upper <- vals$reTrms$upper
    if (!is.null(upr <- control$optCtrl$upper)) {
        upper <- upr[seq_along(upper)]
        control$optCtrl$upper <- NULL
    }

    opt <- optwrap(control$optimizer[[1]], devfun,
                   rho$mkPar(rho$pp$theta),
                   lower=lower,
                   upper=upper,
                   control=control$optCtrl,
                   adj=FALSE)

    if (nAGQ > 0L) {
        ## set lower/upper to values already harvested from control$optCtrl$upper
        lower <- lwr %||% c(lower, rep.int(-Inf, length(rho$pp$beta0)))
        upper <- upr %||% c(upper, rep.int( Inf, length(rho$pp$beta0)))
        rho$dpars <- seq_len(length(lower) - length(rho$pp$beta0))
        rho$u0    <- rho$pp$u0
        rho$beta0 <- pmin(upper[-rho$dpars],
                          pmax(rho$pp$beta0, lower[-rho$dpars]))
        if (nAGQ > 1L) {
            if (length(vals$reTrms$flist) != 1L || length(vals$reTrms$cnms[[1]]) != 1L)
                stop("nAGQ > 1 is only available for models with a single, scalar random-effects term")
            rho$fac <- vals$reTrms$flist[[1]]
        }
        devfun <- mkdevfun(rho, nAGQ, verbose=verbose, control=control)
        if (devFunOnly) return(devfun)

        opt <- optwrap(control$optimizer[[2]], devfun,
                       par = c(rho$mkPar(rho$pp$theta), rho$beta0),
                       lower = lower,
                       upper = upper,
                       control = control$optCtrl,
                       adj = TRUE, verbose=verbose)

    }
    mkMerMod(environment(devfun), opt, vals$reTrms, fr = vals$frame, mc = mc)
}## {nlmer}

## R 3.1.0 devel [2013-08-05]: This does not help yet
if(getRversion() >= "3.1.0") utils::suppressForeignCheck("nlmerAGQ")
if(getRversion() < "3.1.0") dontCheck <- identity

## *not* exported (had help page till early 2018)
## -> issue #92: -> also look at devfun2() in  ./profile.R (which returns class!)
##' Create a deviance evaluation function from a predictor and a response module
##' @param rho an `environment` already containing `verbose` and tolPwrss
##' @param nAGQ for glmer/nlmer: #{AGQ steps}; 0 <==> Laplace
##' @param maxit maximal number of PIRLS iterations
##' @param verbose integer specifying if outputs should be produced
##' @param control a list as from lmerControl() etc
mkdevfun <- function(rho, nAGQ=1L, maxit = if(extends(rho.cld, "nlsResp")) 300L else 100L,
                     verbose=0, control=list()) {
    ## FIXME: should nAGQ be automatically embedded in rho?
    stopifnot(is.environment(rho), ## class definition, compute and save :
              extends(rho.cld <- getClass(class(rho$resp)), "lmResp"))

    ## silence R CMD check warnings *locally* in this function
    ## (clearly preferred to using globalVariables() !]
    fac <- pp <- resp <- lp0 <- compDev <- dpars <- baseOffset <- tolPwrss <-
        pwrssUpdate <- ## <-- even though it's a function below
        GQmat <- nlmerAGQ <- mkTheta <- NULL

    ## The deviance function (to be returned, with 'rho' as its environment):
    ff <-
    if (extends(rho.cld, "lmerResp")) {
        rho$lmer_Deviance <- lmer_Deviance
        function(par)
            .Call(lmer_Deviance, pp$ptr(), resp$ptr(),
                  mkTheta(as.double(par)))
    } else if (extends(rho.cld, "glmResp")) {
        ## control values will override rho values *if present*
        if (!is.null(tp <- control$tolPwrss)) rho$tolPwrss <- tp
        if (!is.null(cd <- control$ compDev)) rho$compDev <- cd
        if (nAGQ == 0L)
            function(par) {
                resp$updateMu(lp0)
                pp$setTheta(mkTheta(as.double(par)))
                p <- pwrssUpdate(pp, resp, tol=tolPwrss, GQmat=GHrule(0L),
                                 compDev=compDev, maxit=maxit, verbose=verbose)
                resp$updateWts()
                p
            }
        else  ## nAGQ > 0
            function(pars) {
                ## pp$setDelu(rep(0, length(pp$delu)))
                resp$setOffset(baseOffset)
                resp$updateMu(lp0)
                pp$setTheta(mkTheta(as.double(pars[dpars])))
                spars <- as.double(pars[-dpars])
                offset <- if (length(spars)==0) baseOffset else baseOffset + pp$X %*% spars
                resp$setOffset(offset)
                p <- pwrssUpdate(pp, resp, tol=tolPwrss, GQmat=GQmat,
                                 compDev=compDev, grpFac=fac, maxit=maxit, verbose=verbose)
                resp$updateWts()
                p
            }
    } else if (extends(rho.cld, "nlsResp")) {
        if (nAGQ <= 1L) {
            rho$nlmerLaplace <- nlmerLaplace
            rho$tolPwrss <- control$tolPwrss
            rho$maxit <- maxit
            switch(nAGQ + 1L,
                   function(par)
                       .Call(nlmerLaplace, pp$ptr(), resp$ptr(),
                             as.double(par),
                             as.double(u0), as.double(beta0),
                             verbose, FALSE, tolPwrss, maxit),
                   function(pars)
                       .Call(nlmerLaplace, pp$ptr(), resp$ptr(),
                             as.double(pars[dpars]),
                             as.double(u0), as.double(pars[-dpars]),
                             verbose, TRUE, tolPwrss, maxit))
        } else {
            stop("nAGQ > 1  not yet implemented for nlmer models")
            rho$nlmerAGQ <- nlmerAGQ
            rho$GQmat    <- GHrule(nAGQ)
            ## function(pars) {
            ## .Call(nlmerAGQ, ## <- dontCheck(nlmerAGQ)  should work according to docs but does not
            ## pp$ptr(), resp$ptr(), fac, GQmat, pars[dpars],
            ## u0, pars[-dpars], tolPwrss)
            ##}
        }
    }
    else stop("code not yet written")
    environment(ff) <- rho
    ff
}

## Determine a step factor that will reduce the pwrss
##
## The penalized, weighted residual sum of squares (pwrss) is the sum
## of the weighted residual sum of squares from the resp module and
## the squared length of u from the predictor module.  The predictor module
## contains a base value and an increment for the coefficients.
## @title Determine a step factor
## @param pp predictor module
## @param resp response module
## @param verbose logical value determining verbose output
## @return NULL if successful
## @note Typically all this is done in the C++ code.
##     The R code is for debugging and comparisons of
##     results.
## stepFac <- function(pp, resp, verbose, maxSteps = 10) {
##     stopifnot(is.numeric(maxSteps), maxSteps >= 2)
##     pwrss0 <- resp$wrss() + pp$sqrL(0)
##     for (fac in 2^(-(0:maxSteps))) {
##      wrss <- resp$updateMu(pp$linPred(fac))
##      pwrss1 <- wrss + pp$sqrL(fac)
##      if (verbose > 3L)
##          cat(sprintf("pwrss0=%10g, diff=%10g, fac=%6.4f\n",
##                      pwrss0, pwrss0 - pwrss1, fac))
##      if (pwrss1 <= pwrss0) {
##          pp$installPars(fac)
##          return(NULL)
##      }
##     }
##     stop("step factor reduced below ",signif(2^(-maxSteps),2)," without reducing pwrss")
## }

RglmerWrkIter <- function(pp, resp, uOnly=FALSE) {
    pp$updateXwts(resp$sqrtWrkWt())
    pp$updateDecomp()
    pp$updateRes(resp$wtWrkResp())
    if (uOnly) pp$solveU() else pp$solve()
    resp$updateMu(pp$linPred(1))        # full increment
    resp$resDev() + pp$sqrL(1)
}

##' @param pp pred module
##' @param resp resp module
##' @param tol numeric tolerance
##' @param GQmat matrix of Gauss-Hermite quad info
##' @param compDev compute in C++ (as opposed to doing as much as possible in R)
##' @param grpFac grouping factor (normally found in environment ..)
##' @param verbose verbosity, of course
glmerPwrssUpdate <- function(pp, resp, tol, GQmat, compDev=TRUE, grpFac=NULL, maxit = 70L, verbose=0) {
    nAGQ <- nrow(GQmat)
    if (compDev) {
        if (nAGQ < 2L)
            return(.Call(glmerLaplace, pp$ptr(), resp$ptr(),
                         nAGQ, tol, as.integer(maxit),
                         verbose))
        return(.Call(glmerAGQ, pp$ptr(), resp$ptr(),
                     tol, as.integer(maxit),
                     GQmat, grpFac, verbose))
    }
 ### does this show anywhere ??? [i.e. is it ever used in our checks/examples/scripts/vignettes ?
 ### message("glmerPwrssUpdate(*, compDev=FALSE)  --> using more R, no direct .Call() to C.") # [DBG] only
    oldpdev <- .Machine$double.xmax
    uOnly   <- nAGQ != 0L
    i <- 0
    repeat {
        ## oldu <- pp$delu
        ## olddelb <- pp$delb
        pdev <- RglmerWrkIter(pp, resp, uOnly=uOnly)
        if (verbose > 2) cat(i,": ",pdev,"\n",sep="")
        ## check convergence first so small increases don't trigger errors
        if (is.na(pdev)) stop("encountered NA in PWRSS update")
        if (abs((oldpdev - pdev) / pdev) < tol)
            break
        ## if (pdev > oldpdev) {
        ##     ## try step-halving
        ##     k <- 0
        ##     while (k < 10 && pdev > oldpdev) {
        ##         pp$setDelu((oldu + pp$delu)/2.)
        ##         if (!uOnly) pp$setDelb((olddelb + pp$delb)/2.)
        ##         pdev <- RglmerWrkIter(pp, resp, uOnly=uOnly)
        ##         k <- k+1
        ##     }
        ## }
        if (pdev > oldpdev) stop("PIRLS update failed")
        oldpdev <- pdev
        i <- i+1
    }
    resp$Laplace(pp$ldL2(), 0., pp$sqrL(1))  ## FIXME: should 0. be pp$ldRX2 ?
}

## create a deviance evaluation function that uses the sigma parameters
## df2 <- function(dd) {
##     stopifnot(is.function(dd),
##            length(formals(dd)) == 1L,
##            is((rem <- (rho <- environment(dd))$rem), "Rcpp_reModule"),
##            is((fem <- rho$fem), "Rcpp_deFeMod"),
##            is((resp <- rho$resp), "Rcpp_lmerResp"),
##            all((lower <- rem$lower) == 0))
##     Lind <- rem$Lind
##     n <- length(resp$y)
##     function(pars) {
##      sigma <- pars[1]
##      sigsq <- sigma * sigma
##      sigmas <- pars[-1]
##      theta <- sigmas/sigma
##      rem$theta <- theta
##      resp$updateMu(numeric(n))
##      solveBetaU(rem, fem, resp$sqrtXwt, resp$wtres)
##      resp$updateMu(rem$linPred1(1) + fem$linPred1(1))
##      n * log(2*pi*sigsq) + (resp$wrss + rem$sqrLenU)/sigsq + rem$ldL2
##     }
## }

## bootMer() ---> now in ./bootMer.R


## Methods for the merMod class

## Anova for merMod objects
##
## @title anova() for merMod objects
## @param a merMod object
## @param ...   further such objects
## @param refit should objects be refitted with ML (if applicable)
## @return an "anova" data frame; the traditional (S3) result of anova()
anovaLmer <- function(object, ..., refit = TRUE, model.names=NULL) {
    mCall <- match.call(expand.dots = TRUE)
    dots <- list(...)
    .sapply <- function(L, FUN, ...) unlist(lapply(L, FUN, ...))
    modp <- (as.logical(vapply(dots, is, NA, "merMod")) |
             as.logical(vapply(dots, is, NA, "lm")))
    if (any(modp)) {   ## multiple models - form table
        ## opts <- dots[!modp]
        mods <- c(list(object), dots[modp])
        nobs.vec <- vapply(mods, nobs, 1L)
        if (var(nobs.vec) > 0)
            stop("models were not all fitted to the same size of dataset")

        ## model names
        if (is.null(mNms <- model.names))
            mNms <- vapply(as.list(mCall)[c(FALSE, TRUE, modp)], deparse1, "")

        ## HACK to try to identify model names in situations such as
        ## 'do.call(anova,list(model1,model2))' where the model names
        ## are lost in the call stack ... this doesn't quite work but might
        ## be useful for future attempts?
        ## maxdepth <- -2
        ## depth <- -1
        ## while (depth >= maxdepth &
        ##        all(grepl("S4 object of class structure",mNms))) {
        ##     xCall <- match.call(call=sys.call(depth))
        ##     mNms <- .sapply(as.list(xCall)[c(FALSE, TRUE, modp)], deparse)
        ##     depth <- depth-1
        ## }
        ## if (depth < maxdepth) {
        if (any(substr(mNms, 1,4) == "new(") ||
            any(duplicated(mNms)) || ## <- only if S4 objects are *not* properly deparsed
            max(nchar(mNms)) > 200) {
            warning("failed to find model names, assigning generic names")
            mNms <- paste0("MODEL",seq_along(mNms))
        }
        if (length(mNms) != length(mods))
            stop("model names vector and model list have different lengths")
        names(mods) <- sub("@env$", '', mNms) # <- hack
        models.reml <- vapply(mods, function(x) is(x,"merMod") && isREML(x), NA)
        models.GHQ <- vapply(mods, function(x) is(x,"glmerMod") && getME(x,"devcomp")$dims[["nAGQ"]]>1 , NA)
        if (any(models.GHQ) && any(vapply(mods, function(x) is(x,"glm"), NA)))
            stop("GLMMs with nAGQ>1 have log-likelihoods incommensurate with glm() objects")
        if (refit) {
            ## message only if at least one models is REML:
            if (any(models.reml)) message("refitting model(s) with ML (instead of REML)")
            mods[models.reml] <- lapply(mods[models.reml], refitML)
        } else { ## check that models are consistent (all REML or all ML)
            if(any(models.reml) && any(!models.reml))
                warning("some models fit with REML = TRUE, some not")
        }
        ## devs <- sapply(mods, deviance)
        llks <- lapply(mods, logLik)
        ## Order models by increasing degrees of freedom:
        ii <- order(npar <- vapply(llks, attr, FUN.VALUE=numeric(1), "df"))
        mods <- mods[ii]
        llks <- llks[ii]
        npar   <- npar  [ii]
        calls <- lapply(mods, getCall)
        data <- lapply(calls, `[[`, "data")
        if(!all(vapply(data, identical, NA, data[[1]])))
            stop("all models must be fit to the same data object")
        header <- paste("Data:", abbrDeparse(data[[1]]))
        subset <- lapply(calls, `[[`, "subset")
        if(!all(vapply(subset, identical, NA, subset[[1]])))
            stop("all models must use the same subset")
        if (!is.null(subset[[1]]))
            header <- c(header, paste("Subset:", abbrDeparse(subset[[1]])))
        llk <- unlist(llks)
        chisq <- 2 * pmax(0, c(NA_real_, diff(llk)))
        dfChisq <- c(NA_integer_, diff(npar))
        val <- data.frame(npar = npar,
                          ## afraid to swap in vapply here; wondering
                          ##   why .sapply was needed in the first place ...
                          AIC = .sapply(llks, AIC), # FIXME? vapply()
                          BIC = .sapply(llks, BIC), #  "       "
                          logLik = llk,
                          "-2*log(L)" = -2*llk,
                          Chisq = chisq,
                          Df = dfChisq,
                          "Pr(>Chisq)" = ifelse(dfChisq==0,NA_integer_,pchisq(chisq, dfChisq, lower.tail = FALSE)),
                          row.names = names(mods), check.names = FALSE)
        class(val) <- c("anova", class(val))
        forms <- lapply(lapply(calls, `[[`, "formula"), deparse1)
        structure(val,
                  heading = c(header, "Models:",
                              paste(rep.int(names(mods), lengths(forms)),
                                    unlist(forms), sep = ": ")))
    }
    else { ## ------ single model ---------------------
        if (length(dots)>0) {
            warnmsg <- "additional arguments ignored"
            nd <- names(dots)
            nd <- nd[nzchar(nd)]
            if (length(nd)>0) {
                warnmsg <- paste0(warnmsg,": ",
                                  paste(sQuote(nd),collapse=", "))
            }
            warning(warnmsg)
        }
        dc <- getME(object, "devcomp")
        X <- getME(object, "X")
        stopifnot(length(asgn <- attr(X, "assign")) == dc$dims[["p"]])
        ss <- as.vector(object@pp$RX() %*% object@beta)^2
        names(ss) <- colnames(X)
        terms <- terms(object)
        nmeffects <- attr(terms, "term.labels")[unique(asgn)]
        if ("(Intercept)" %in% names(ss))
            nmeffects <- c("(Intercept)", nmeffects)
        ss <- unlist(lapply(split(ss, asgn), sum))
        stopifnot(length(ss) == length(nmeffects))
        df <- lengths(split(asgn, asgn))
        ## dfr <- unlist(lapply(split(dfr, asgn), function(x) x[1]))
        ms <- ss/df
        f <- ms/(sigma(object)^2)
        ## No longer provide p-values, but still the F statistic (may not be F distributed):
        ##
        ## P <- pf(f, df, dfr, lower.tail = FALSE)
        ## table <- data.frame(df, ss, ms, dfr, f, P)
        table <- data.frame(df, ss, ms, f)
        dimnames(table) <-
            list(nmeffects,
                 ## c("npar", "Sum Sq", "Mean Sq", "Denom", "F value", "Pr(>F)"))
                 c("npar", "Sum Sq", "Mean Sq", "F value"))
        if ("(Intercept)" %in% nmeffects)
            table <- table[-match("(Intercept)", nmeffects), ]
        structure(table, heading = "Analysis of Variance Table",
                  class = c("anova", "data.frame"))
    }
}## {anovaLmer}

##' @importFrom stats anova
##' @S3method anova merMod
anova.merMod <- anovaLmer

##' @S3method as.function merMod
as.function.merMod <- function(x, ...) {
    reCovs <- getReCovs(x)
    rho <- list2env(list(resp  = x@resp$copy(),
                         pp    = x@pp$copy(),
                         lower = getLower(x),
                         upper = getUpper(x),
                         mkPar = mkMkPar(reCovs),
                         mkTheta = mkMkTheta(reCovs)),
                    parent=as.environment("package:lme4"))
    ## FIXME: extract verbose [, maxit] and control
    mkdevfun(rho, getME(x, "devcomp")$dims[["nAGQ"]], ...)
}

## coef() method for all kinds of "mer", "*merMod", ... objects
## ------  should work with fixef() + ranef()  alone
coefMer <- function(object, ...)
{
    if(...length())
        warning('arguments named ', paste(sQuote(...names()), collapse = ", "),
                ' ignored')
    fef <- data.frame(rbind(fixef(object)), check.names = FALSE)
    ref <- ranef(object, condVar = FALSE)
    ## check for variables in RE but missing from FE, fill in zeros in FE accordingly
    refnames <- unlist(lapply(ref,colnames))
    nmiss <- length(missnames <- setdiff(refnames,names(fef)))
    if (nmiss > 0) {
        fillvars <- setNames(data.frame(rbind(rep(0,nmiss))),missnames)
        fef <- cbind(fillvars,fef)
    }
    val <- lapply(ref, function(x)
                  fef[rep.int(1L, nrow(x)),,drop = FALSE])
    for (i in seq_along(val)) {
        refi <- ref[[i]]
        row.names(val[[i]]) <- row.names(refi)
        nmsi <- colnames(refi)
        if (!all(nmsi %in% names(fef)))
            stop("unable to align random and fixed effects")
        for (nm in nmsi) val[[i]][[nm]] <- val[[i]][[nm]] + refi[,nm]
    }
    class(val) <- "coef.mer"
    val
} ##  {coefMer}

##' @importFrom stats coef
##' @S3method coef merMod
coef.merMod <- coefMer

## FIXME: should these values (i.e. ML criterion for REML models
##  and vice versa) be computed and stored in the object in the first place?
##' @importFrom stats deviance
##' @S3method deviance merMod
deviance.merMod <- function(object, REML = NULL, ...) {
                            ## type = c("conditional", "unconditional", "penalized"),
                            ## relative = TRUE, ...) {
    if (isGLMM(object)) {
        return(sum(residuals(object,type="deviance")^2))
        ## ------------------------------------------------------------
        ## proposed change to deviance function for GLMMs
        ## ------------------------------------------------------------
        ## @param type Type of deviance (can be unconditional,
        ## penalized, conditional)
        ## @param relative Should deviance be shifted relative to a
        ## saturated model? (only available with type == penalized or
        ## conditional)
        ## ------------------------------------------------------------
        ## ans <- switch(type[1],
        ##               unconditional = {
        ##                   if (relative) {
        ##                       stop("unconditional and relative deviance is undefined")
        ##                   }
        ##                   c(-2 * logLik(object))
        ##               },
        ##               penalized = {
        ##                   sqrL <- object@pp$sqrL(1)
        ##                   if (relative) {
        ##                       object@resp$resDev() + sqrL
        ##                   } else {
        ##                       useSc <- unname(getME(gm1, "devcomp")$dims[["useSc"]])
        ##                       qLog2Pi <- unname(getME(object, "q")) * log(2 * pi)
        ##                       object@resp$aic() - (2 * useSc) + sqrL + qLog2Pi
        ##                   }
        ##               },
        ##               conditional = {
        ##                   if (relative) {
        ##                       object@resp$resDev()
        ##                   } else {
        ##                       useSc <- unname(getME(gm1, "devcomp")$dims[["useSc"]])
        ##                       object@resp$aic() - (2 * useSc)
        ##                   }
        ##               })
        ## return(ans)
    }
    if (isREML(object) && is.null(REML)) {
        warning("deviance() is deprecated for REML fits; use REMLcrit for the REML criterion or deviance(.,REML=FALSE) for deviance calculated at the REML fit")
        return(devCrit(object, REML=TRUE))
    }
    devCrit(object, REML=FALSE)
}

REMLcrit <- function(object) {
    devCrit(object, REML=TRUE)
}

## original deviance.merMod -- now wrapped by REMLcrit
## REML=NULL:
##    if REML fit return REML criterion
##    if ML fit, return deviance
## REML=TRUE:
##    if not LMM, stop.
##    if ML fit, compute and return REML criterion
##    if REML fit, return REML criterion
## REML=FALSE:
##    if ML fit, return deviance
##    if REML fit, compute and return deviance
devCrit <- function(object, REML = NULL) {
    ## cf. (1) lmerResp::Laplace in respModule.cpp
    ##     (2) section 5.6 of lMMwR, listing lines 34-42
    if (isTRUE(REML) && !isLMM(object))
        stop("can't compute REML deviance for a non-LMM")
    cmp <- object@devcomp$cmp
    if (is.null(REML) || is.na(REML[1]))
        REML <- isREML(object)
    if (REML) {
        if (isREML(object)) {
            cmp[["REML"]]
        } else {
            ## adjust ML results to REML
            lnum <- log(2*pi*cmp[["pwrss"]])
            n <- object@devcomp$dims[["n"]]
            nmp <- n - length(object@beta)
            ldW <- sum(log(weights(object, method = "prior")))
            - ldW + cmp[["ldL2"]] + cmp[["ldRX2"]] + nmp*(1 + lnum - log(nmp))
        }
    } else {
        if (!isREML(object)) {
            cmp[["dev"]]
        } else {
            ## adjust REML results to ML
            n <- object@devcomp$dims[["n"]]
            lnum <- log(2*pi*cmp[["pwrss"]])
            ldW <- sum(log(weights(object, method = "prior")))
            - ldW + cmp[["ldL2"]] + n*(1 + lnum - log(n))
        }
    }
}

## copied from stats:::safe_pchisq
safe_pchisq <- function (q, df, ...) {
    df[df <= 0] <- NA
    pchisq(q = q, df = df, ...)
}

##' @importFrom stats drop1
##' @S3method drop1 merMod
drop1.merMod <- function(object, scope, scale = 0, test = c("none", "Chisq", "user"),
                         k = 2, trace = FALSE,
                         sumFun=NULL, ...) {
    evalhack <- "formulaenv"
    test <- match.arg(test)
    if ((test=="user" && is.null(sumFun)) ||
        ((test!="user" && !is.null(sumFun))))
        stop(sQuote("sumFun"),' must be specified if (and only if) test=="user"')
    tl <- attr(terms(object), "term.labels")
    if(missing(scope)) scope <- drop.scope(object)
    else {
        if(!is.character(scope)) {
            scope <- attr(terms(getFixedFormula(update.formula(object, scope))),
                                "term.labels")
        }
        if(!all(match(scope, tl, 0L) > 0L))
            stop("scope is not a subset of term labels")
    }
    ns <- length(scope)
    if (is.null(sumFun)) {
        sumFun <- function(x,scale,k,...)
            setNames(extractAIC(x,scale,k,...),c("df","AIC"))
    }
    ss <- sumFun(object, scale=scale, k=k, ...)
    ans <- matrix(nrow = ns + 1L, ncol = length(ss),
                  dimnames =  list(c("<none>", scope), names(ss)))
    ans[1, ] <- ss
    n0 <- nobs(object, use.fallback = TRUE)
    env <- environment(formula(object)) # perhaps here is where trouble begins??
    for(i in seq_along(scope)) {  ## was seq(ns), failed on empty scope
        tt <- scope[i]
        if(trace > 1) {
            cat("trying -", tt, "\n", sep='')
            flush.console()
        }
        ## FIXME: make this more robust, somehow?
        ## three choices explored so far:
        ##  (1) evaluate nfit in parent frame: tests in inst/tests/test-formulaEval.R
        ##      will fail on lapply(m_data_List,drop1)
        ##      (formula environment contains r,x,y,z but not d)
        ##  (2) evaluate nfit in frame of formula: tests will fail when data specified and formula is character
        ##  (3) update with data=NULL: fails when ...
        ##
        if (evalhack %in% c("parent","formulaenv")) {
            nfit <- update(object,
                           as.formula(paste("~ . -", tt)),
                           evaluate = FALSE)
            ## nfit <- eval(nfit, envir = env) # was  eval.parent(nfit)
            if (evalhack=="parent") {
                nfit <- eval.parent(nfit)
            } else if (evalhack=="formulaenv") {
                nfit <- eval(nfit,envir=env)
            }
        } else {
            nfit <- update(object,
                           as.formula(paste("~ . -", tt)),data=NULL,
                           evaluate = FALSE)
            nfit <- eval(nfit,envir=env)
        }
        if (test=="user") {
            ans[i+1, ] <- sumFun(object, nfit, scale=scale, k=k, ...)
        } else {
            ans[i+1, ] <- sumFun(nfit, scale, k = k, ...)
        }
        nnew <- nobs(nfit, use.fallback = TRUE)
        if(all(is.finite(c(n0, nnew))) && nnew != n0)
            stop("number of rows in use has changed: remove missing values?")
    }
    if (test=="user") {
        aod <- as.data.frame(ans)
    } else {
        dfs <- ans[1L, 1L] - ans[, 1L]
        dfs[1L] <- NA
        aod <- data.frame(npar = dfs, AIC = ans[,2])
        if(test == "Chisq") {
            ## reconstruct deviance from AIC (ugh)
            dev <- ans[, 2L] - k*ans[, 1L]
            dev <- dev - dev[1L] ; dev[1L] <- NA
            nas <- !is.na(dev)
            P <- dev
            P[nas] <- safe_pchisq(dev[nas], dfs[nas], lower.tail = FALSE)
            aod[, c("LRT", "Pr(Chi)")] <- list(dev, P)
        } else if (test == "F") {
            ## FIXME: allow this if denominator df are specified externally?
            stop("F test STUB -- unfinished maybe forever")
            dev <- ans[, 2L] - k*ans[, 1L]
            dev <- dev - dev[1L] ; dev[1L] <- NA
            nas <- !is.na(dev)
            P <- dev
            P[nas] <- safe_pchisq(dev[nas], dfs[nas], lower.tail = FALSE)
            aod[, c("LRT", "Pr(F)")] <- list(dev, P)
        }
    }
    head <- c("Single term deletions", "\nModel:", deparse(formula(object)),
              if(scale > 0) paste("\nscale: ", format(scale), "\n"))
    if (!is.null(method <- attr(ss,"method"))) {
        head <- c(head,"Method: ",method,"\n")
    }
    structure(aod, heading = head, class = c("anova", "data.frame"))
}

##' @importFrom stats extractAIC
##' @S3method extractAIC merMod
extractAIC.merMod <- function(fit, scale = 0, k = 2, ...) {
    L <- logLik(refitML(fit))
    edf <- attr(L,"df")
    c(edf,-2*L + k*edf)
}

##' @importFrom stats family
##' @S3method family merMod
family.merMod <- function(object, ...) family(object@resp, ...)

##' @S3method family glmResp
family.glmResp <- function(object, ...) {
                                        # regenerate initialize
                                        # expression if necessary

    ## FIXME: may fail with user-specified/custom family?
    ## should be obsolete
    if(is.null(object$family$initialize))
        return(do.call(object$family$family,
                       list(link=object$family$link)))

    object$family
}

##' @S3method family lmResp
family.lmResp <- function(object, ...) gaussian()

##' @S3method family nlsResp
family.nlsResp <- function(object, ...) gaussian()

##' @importFrom stats fitted
##' @S3method fitted merMod
fitted.merMod <- function(object, ...) {
    xx <- object@resp$mu
    if (length(xx)==0) {
        ## handle 'fake' objects created by simulate()
        xx <- rep(NA_real_,nrow(model.frame(object)))
    }
    nm <- rownames(model.frame(object)) %||% seq_along(xx)
    names(xx) <- nm
    if (!is.null(fit.na.action <- attr(model.frame(object),"na.action")))
        napredict(fit.na.action, xx)
    else
        xx
}

##' Extract the fixed-effects estimates
##'
##' Extract the estimates of the fixed-effects parameters from a fitted model.
##' @name fixef
##' @title Extract fixed-effects estimates
##' @aliases fixef fixed.effects fixef.merMod
##' @docType methods
##' @param object any fitted model object from which fixed effects estimates can
##' be extracted.
##' @param noScale logical; if TRUE, returns the non-scaled parameters
##' @param \dots optional additional arguments. Currently none are used in any
##' methods.
##' @return a named, numeric vector of fixed-effects estimates.
##' @keywords models
##' @examples
##' fixef(lmer(Reaction ~ Days + (1|Subject) + (0+Days|Subject), sleepstudy))
##' @importFrom nlme fixef
##' @export fixef
##' @method fixef merMod
##' @export
fixef.merMod <- function(object, add.dropped = FALSE, noScale = NULL, ...) {
    X <- getME(object,"X")
    ff <- structure(object@beta, names = dimnames(X)[[2]])
    
    if(is.null(noScale) || (!is.null(noScale) && !noScale)){
      if (!is.null(sc <- attr(X, "scaled:scale"))) {
        ce <- attr(X, "scaled:center")
        ## modifying intercept
        if ("(Intercept)" %in% names(ff)) {
          intercept_shift <- sum((ff[names(sc)] * ce[names(sc)]) / sc[names(sc)])
          ff[["(Intercept)"]] <- ff[["(Intercept)"]] - intercept_shift
        }
        # modifying beta coefficients
        ff[names(sc)] <- ff[names(sc)] / sc[names(sc)]
      }
    }
    
    if (add.dropped) {
        if (!is.null(dd <- attr(X,"col.dropped"))) {
            ## restore positions dropped for rank deficiency
            vv <- numeric(length(ff)+length(dd))
            all.pos <- seq_along(vv)
            kept.pos <- all.pos[-dd]
            vv[kept.pos] <- ff
            names(vv)[kept.pos] <- names(ff)
            vv[dd] <- NA
            names(vv)[dd] <- names(dd)
            ff <- vv
        }
    }
    return(ff)
}


getFixedFormula <- function(form) {
    RHSForm(form) <- reformulas::nobars(RHSForm(form))
    form
}

##' @importFrom stats formula
##' @S3method formula merMod
formula.merMod <- function(x, fixed.only=FALSE, random.only=FALSE, ...) {
    if (missing(fixed.only) && random.only) fixed.only <- FALSE
    if (fixed.only && random.only) stop("can't specify 'only fixed' and 'only random' terms")
    if (is.null(form <- attr(x@frame,"formula"))) {
        if (!grepl("lmer$",deparse(getCall(x)[[1]])))
            stop("can't find formula stored in model frame or call")
        form <- as.formula(formula(getCall(x),...))
    }
    if (fixed.only) {
        form <- getFixedFormula(form)
    }
    if (random.only) {
        ## from predict.R
        form <- lme4_reOnly(form, response=TRUE)
    }
    form
}

##' @S3method isREML merMod
isREML.merMod <- function(x, ...) as.logical(x@devcomp$dims[["REML"]])

##' @S3method isGLMM merMod
isGLMM.merMod <- function(x,...) {
  as.logical(x@devcomp$dims[["GLMM"]])
  ## or: is(x@resp,"glmResp")
}

##' @S3method isNLMM merMod
isNLMM.merMod <- function(x,...) {
  as.logical(x@devcomp$dims[["NLMM"]])
  ## or: is(x@resp,"nlsResp")
}

##' @S3method isLMM merMod
isLMM.merMod <- function(x,...) {
  !isGLMM(x) && !isNLMM(x)
  ## or: is(x@resp,"lmerResp") ?
}

npar.merMod <- function(object) {
    n <- length(object@beta) + getParLength(object) +
      object@devcomp[["dims"]][["useSc"]]
    ## FIXME: this is a bit of a hack: a user *might* have specified
    ## negative binomial family with a known theta, in which case we
    ## shouldn't count it as extra.  Either glmer.nb needs to set a
    ## flag somewhere, or we need class 'nbglmerMod' to extend 'glmerMod' ...
    ## We do *not* want to use the 'useSc' slot (as above), because
    ## although theta is in some sense a scale parameter, it's not
    ## one in the formal sense (and isn't stored in the 'sigma' slot)
    if (grepl("Negative Binomial",family(object)$family)) {
        n <- n+1
    }
    return(n)
    ## TODO: how do we feel about counting the scale parameter ???
}

##' @importFrom stats logLik
##' @S3method logLik merMod
logLik.merMod <- function(object, REML = NULL, ...) {
    if (is.null(REML) || is.na(REML[1]))
        REML <- isREML(object)
    val <- -devCrit(object, REML = REML)/2
    ## dc <- object@devcomp
    nobs <- nobs.merMod(object)
    structure(val,
              nobs = nobs,
              nall = nobs,
              df = npar.merMod(object),
              ## length(object@beta) + length(object@theta) + dc$dims[["useSc"]],
              class = "logLik")
}

##' @importFrom stats df.residual
##' @S3method df.residual merMod
##  TODO: not clear whether the residual df should be based
##  on p=length(beta) or p=length(c(theta,beta)) ... but
##  this is just to allow things like aods3::gof to work ...
##
df.residual.merMod <- function(object, ...) {
    nobs(object)-npar.merMod(object)
}

##' @importFrom stats logLik
##' @S3method model.frame merMod
model.frame.merMod <- function(formula, fixed.only=FALSE, ...) {
    fr <- formula@frame
    if (fixed.only) {
        vars <- attr(terms(fr),"varnames.fixed")
        if (is.null(vars)) {
            ## back-compatibility: saved objects pre 1.1-15
            ff <- formula(formula,fixed.only=TRUE)
            ## thanks to Thomas Leeper and Roman Lustrik, Stack Overflow
            ## https://stackoverflow.com/questions/18017765/extract-variables-in-formula-from-a-data-frame
            vars <- rownames(attr(terms.formula(ff), "factors"))
        }
        vars <- gsub("`","",vars) ## weirdness in deparsing variable names with spaces
        fr <- fr[vars]
    }
    fr
}

##' @importFrom stats model.matrix
##' @S3method model.matrix merMod
##' @param noScale logical; if TRUE, returns the non-scaled parameters
model.matrix.merMod <-
  function(object, type = c("fixed", "random", "randomListRaw"), 
           noScale = NULL,...) {
    X <- object@pp$X
    # Re-scales back the model matrix on command
    if (!is.null(sc <- attr(object@pp$X, "scaled:scale"))){
      if((is.null(noScale)) || (!is.null(noScale) && !noScale)){
        unscale_cols <- setdiff(colnames(X), "(Intercept)")
        ce <- attr(object@pp$X, "scaled:center")
        X[, unscale_cols] <- sweep(X[, unscale_cols], 2, sc, `*`)
        X[, unscale_cols] <- sweep(X[, unscale_cols], 2, ce, `+`)
      }
    }
    
    switch(type[1],
           "fixed" = X,
           "random" = getME(object, "Z"),
           "randomListRaw" = mmList(object))
  }

##' Dummy variables (experimental)
##'
##' Largely a wrapper for \code{model.matrix} that
##' accepts a factor, \code{f}, and returns a dummy
##' matrix with \code{nlevels(f)-1} columns.
dummy <- function(f, levelsToKeep){
  f <- as.factor(f)
  if (all(is.na(f))) return(rep(NA_real_, length(f)))
  mm <- model.matrix(~ 0 + f)
  colnames(mm) <- levels(f)

                                        # sort out levels to keep
  missingLevels <- missing(levelsToKeep)
  if(missingLevels) levelsToKeep <- levels(f)[-1]
  if(!any(levels(f) %in% levelsToKeep))
      stop("at least some of the levels in f ",
           "must also be present in levelsToKeep")
  if(!all(levelsToKeep %in% levels(f)))
      stop("all of the levelsToKeep must be levels of f")
  mm <- mm[, levelsToKeep, drop=FALSE]

  ##                                       # communicate that some usages are unlikely
  ##                                       # to help with readibility, which is the
  ##                                       # whole purpose of dummy()
  ## if((!missingLevels)&&(ncol(mm) > 1))
  ##     message("note from dummy:  explicitly specifying more than one ",
  ##             "level to keep may do little to improve readibility")
  return(mm)
}


##' @importFrom stats nobs
##' @S3method nobs merMod
nobs.merMod <- function(object, ...) nrow(object@frame)

## used in  summary.merMod():
ngrps <- function(object, ...) UseMethod("ngrps")

ngrps.default <- function(object, ...) stop("Cannot extract the number of groups from this object")

ngrps.merMod <- function(object, ...) vapply(object@flist, nlevels, 1)

ngrps.factor <- function(object, ...) nlevels(object)

##' @importFrom nlme ranef
##' @export ranef
NULL

##' Extract the modes of the random effects
##'
##' A generic function to extract the conditional modes of the random effects
##' from a fitted model object.  For linear mixed models the conditional modes
##' of the random effects are also the conditional means.
##'
##' If grouping factor i has k levels and j random effects per level the ith
##' component of the list returned by \code{ranef} is a data frame with k rows
##' and j columns.  If \code{condVar} is \code{TRUE} the \code{"postVar"}
##' attribute is an array of dimension j by j by k.  The kth face of this array
##' is a positive definite symmetric j by j matrix.  If there is only one
##' grouping factor in the model the variance-covariance matrix for the entire
##' random effects vector, conditional on the estimates of the model parameters
##' and on the data will be block diagonal and this j by j matrix is the kth
##' diagonal block.  With multiple grouping factors the faces of the
##' \code{"postVar"} attributes are still the diagonal blocks of this
##' conditional variance-covariance matrix but the matrix itself is no longer
##' block diagonal.
##' @name ranef
##' @aliases ranef ranef.merMod
##' @param object an object of a class of fitted models with random effects,
##' typically an \code{"\linkS4class{merMod}"} object.
##' @param condVar an optional logical argument indicating if the conditional
##' variance-covariance matrices of the random effects should be added as an attribute.
##' @param postVar a (deprecated) synonym for \code{condVar}
##' @param drop an optional logical argument indicating components of the return
##' value that would be data frames with a single column, usually a column
##' called \sQuote{\code{(Intercept)}}, should be returned as named vectors.
##' @param whichel an optional character vector of names of grouping factors for
##' which the random effects should be returned.  Defaults to all the grouping
##' factors.
##' @param \dots some methods for this generic function require additional
##' arguments.
##' @return A list of data frames, one for each grouping factor for the random
##' effects.  The number of rows in the data frame is the number of levels of
##' the grouping factor.  The number of columns is the dimension of the random
##' effect associated with each level of the factor.
##'
##' If \code{condVar} is \code{TRUE} each of the data frames has an attribute
##' called \code{"postVar"} which is a three-dimensional array with symmetric
##' faces.
##'
##' When \code{drop} is \code{TRUE} any components that would be data frames of
##' a single column are converted to named numeric vectors.
##' @note To produce a \dQuote{caterpillar plot} of the random effects apply
##' \code{\link[lattice:xyplot]{dotplot}} to the result of a call to
##' \code{ranef} with \code{condVar = TRUE}.
##' @examples
##' fm1 <- lmer(Reaction ~ Days + (Days|Subject), sleepstudy)
##' fm2 <- lmer(Reaction ~ Days + (1|Subject) + (0+Days|Subject), sleepstudy)
##' fm3 <- lmer(diameter ~ (1|plate) + (1|sample), Penicillin)
##' ranef(fm1)
##' str(rr1 <- ranef(fm1, condVar = TRUE))
##' dotplot(rr1)  ## default
##' ## specify free scales in order to make Day effects more visible
##' dotplot(rr1,scales = list(x = list(relation = 'free')))[["Subject"]]
##' str(ranef(fm2, condVar = TRUE))
##' op <- options(digits = 4)
##' ranef(fm3, drop = TRUE)
##' options(op)
##' @keywords models methods
##' @method ranef merMod
##' @export
ranef.merMod <- function(object, condVar = TRUE, drop = FALSE,
                         whichel = names(ans), postVar = FALSE, ...)
{

    if (length(L <- list(...))>0) {
        warning(paste("additional arguments to ranef.merMod ignored:",
                      paste(names(L),collapse=", ")))
    }
    if (!missing(postVar) && missing(condVar)) {
        warning(sQuote("postVar")," is deprecated: please use ",
                sQuote("condVar")," instead")
        condVar <- postVar
    }
    ans <- object@pp$b(1) ## not always == c(matrix(unlist(getME(object,"b"))))
    if (!is.null(fl <- object@flist)) {
        ## evaluate the list of matrices
        levs <- lapply(fl, levels)
        asgn <- attr(fl, "assign")
        cnms <- object@cnms
        nc <- lengths(cnms) ## number of terms
        ## nb <- nc * lengths(levs)[asgn] ## number of cond modes per term
        nb <- diff(object@Gp)  ## differencing group index is more robust
        nbseq <- rep.int(seq_along(nb), nb)
        ml <- split(ans, nbseq)
        for (i in seq_along(ml))
            ml[[i]] <- matrix(ml[[i]], ncol = nc[i], byrow = TRUE,
                              dimnames = list(NULL, cnms[[i]]))
        ## create a list of data frames corresponding to factors
        ans <- lapply(seq_along(fl),
                      function(i) {
                                     m <- ml[asgn == i]
                                     b2 <- vapply(m,nrow,numeric(1))
                                     ub2 <- unique(b2)
                                     if (length(ub2)>1)
                                         stop("differing numbers of b per group")
                         ## if number of sets of modes != number of levels (e.g. Gaussian process/phyloglmm),
                         ##   generate numeric sequence for names

                         rnms <- if (ub2==length(levs[[i]])) levs[[i]] else seq(ub2)
                         data.frame(do.call(cbind, m),
                                    row.names = rnms,
                                    check.names = FALSE)
        })
        names(ans) <- names(fl)
                                        # process whichel
        stopifnot(is(whichel, "character"))
        whchL <- names(ans) %in% whichel
        ans <- ans[whchL]

        if (condVar) {
            sigsqr <- sigma(object)^2
            rp <- rePos$new(object)
            if(any(lengths(rp$terms) > 1L)) {
                ## use R machinery here ...
                vv <- arrange.condVar(object,condVar(object, scaled=TRUE))
            } else {
                vv <- .Call(merPredDcondVar, object@pp$ptr(), as.environment(rp))
                vv <- lapply(vv, "*", sigsqr)
            }
            for (i in names(ans)) {
                attr(ans[[i]], "postVar") <- vv[[i]]
            }
        }
        if (drop)
            ans <- lapply(ans, function(el)
                      {
                          if (ncol(el) > 1) return(el)
                          pv <- drop(attr(el, "postVar"))
                          el <- drop(as.matrix(el))
                          if (!is.null(pv))
                              attr(el, "postVar") <- pv
                          el
                      })
        class(ans) <- "ranef.mer"
    }
    ans
}## ranef.merMod

print.ranef.mer <- function(x, ...) {
    print(unclass(x), ...)
    if(any(has.pv <- vapply(x, function(el)
                            !is.null(attr(el, "postVar")), NA)))
        cat('with conditional variances for',
            paste(dQuote(names(x)[has.pv]), sep=", "), "\n")
    invisible(x)
}

## try to redo refit by calling modular structure ...
refit2.merMod <- function(object,
                          newresp=NULL) {
    ## the idea is to steal as much structure as we can from the
    ## previous fit, including
    ##  * starting parameter values
    ##  * random-effects structure
    ##  * fixed-effects structure
    ##  * model frame
    ## and jump into the modular structure at an appropriate place;
    ##  essentially, this should merge with a smart-as-possible
    ##  version of 'update' ...

}

## FIXME DRY: much of copy'n'paste from lmer() etc .. ==> become more modular (?)
refit.merMod <- function(object,
                         newresp = NULL,
                         newweights = NULL,
                         ## formula=NULL, weights=NULL,
                         rename.response = FALSE,
                         maxit = 100L, ...)
{

    if (isNLMM(object))
        stop(gettextf("'%s' does not yet support nonlinear mixed models; try '%s' instead",
                      "refit.merMod", "update.merMod"),
             domain = NA)
    haveGLMM <- isGLMM(object)
    l... <- list(...)

    ctrl.arg <- NULL
    if("control" %in% names(l...)) ctrl.arg <- l...$control

    if(!all(names(l...) %in% c("control", "verbose"))) {
        warning("additional arguments to refit.merMod ignored")
    }
    ## TODO: not clear whether we should reset the names
    ##       to the new response variable.  Maybe not.

    ## retrieve name before it gets mangled by operations on newresp
    newrespSub <- substitute(newresp)

    ## for backward compatibility/functioning of refit(fit,simulate(fit))
    if (is.list(newresp)) {
        if (length(newresp)==1) {
            na.action <- attr(newresp,"na.action")
            newresp <- newresp[[1]]
            attr(newresp,"na.action") <- na.action
        } else {
            stop("refit not implemented for 'newresp' lists of length > 1: ",
                 "consider ", sQuote("lapply(object,refit)"))
        }
    }

    ## oldresp <- object@resp$y # need to set this before deep copy,
    ##                          # otherwise it gets reset with the call
    ##                          # to setResp below

    ## somewhat repeated from profile.merMod, but sufficiently
    ##  different that refactoring is slightly non-trivial
    ## "three minutes' thought would suffice ..."
    control <-
        if (!is.null(ctrl.arg)) {
            if (length(ctrl.arg$optCtrl) == 0) { ## use object's version:
                obj.control <- object@optinfo$control
                ignore.pars <- c("xst", "xt")
                if (any(ign <- names(obj.control) %in% ignore.pars))
                    obj.control <- obj.control[!ign]
                ctrl.arg$optCtrl <- obj.control
            }
            ctrl.arg
        }
        else if (haveGLMM)
            glmerControl()
        else
            lmerControl()
    if (object@optinfo$optimizer == "optimx") {
        control$optCtrl <- object@optinfo$control
    }
    ## we need this stuff defined before we call .glmerLaplace below ...
    pp      <- object@pp$copy()
    dc      <- object@devcomp
    nAGQ    <- if (haveGLMM) dc$dims[["nAGQ"]]
 ## nth     <- dc$dims[["nth"]]
    verbose <- l...$verbose; if (is.null(verbose)) verbose <- 0L
    if (!is.null(newresp)) {
        ## update call and model frame with new response
        rcol <- attr(attr(model.frame(object), "terms"), "response")
        if (rename.response) {
            attr(object@frame,"formula")[[2]] <- object@call$formula[[2]] <-
                newrespSub
            names(object@frame)[rcol] <- deparse(newrespSub)
        }
        if (!is.null(na.act <- attr(object@frame,"na.action")) &&
            is.null(attr(newresp,"na.action"))) {
            ## will only get here if na.action is 'na.omit' or 'na.exclude'
            ## *and* newresp does not have an 'na.action' attribute
            ## indicating that NAs have already been filtered
            newresp <- if (is.matrix(newresp))
                newresp[-na.act, ]
            else newresp[-na.act]
        }
        object@frame[[rcol]] <- newresp
    }

    if (!is.null(newweights)) {
        ## DRY ...
        if (!is.null(na.act <- attr(object@frame,"na.action")) &&
            is.null(attr(newweights, "na.action"))) {
            newweights <- newweights[-na.act]
        }
        object@frame[["(weights)"]] <- newweights
        oc <- attr(attr(object@frame, "terms"), "dataClasses")
        attr(attr(object@frame, "terms"), "dataClasses") <- c(oc, `(weights)` = "numeric")

        object@call$weights <- substitute(newweights)

        ## try to make sure new weights are findable later
        assign(deparse(substitute(newweights)),
               newweights,
               environment(formula(object)))
    }

    rr <-
    if (!haveGLMM)
        mkRespMod(model.frame(object), REML = object@resp$REML)
    else
        mkRespMod(model.frame(object), family = family(object))

    if(!is.null(newresp)) {
        if(family(object)$family == "binomial") {
            ## re-do conversion of two-column matrix and factor
            ##  responses to proportion/weights format
            if (is.matrix(newresp) && ncol(newresp) == 2) {
                ntot <- rowSums(newresp)
                ## FIXME: test what happens for (0,0) rows
                newresp <- newresp[,1]/ntot
                rr$setWeights(ntot)
            }
            if (is.factor(newresp)) {
                ## FIXME: would be better to do this consistently with
                ## whatever machinery is used in glm/glm.fit/glmer  ??
                newresp <- as.numeric(newresp)-1
            }
        }
        ## if (haveGLMM && rr$family$family=="binomial") {

        ## }
        stopifnot(length(newresp <- as.numeric(as.vector(newresp))) ==
                  length(rr$y))

    }

    if (haveGLMM) {
        GQmat <- GHrule(nAGQ)

        if (nAGQ <= 1) {
            glmerPwrssUpdate(pp,rr, control$tolPwrss, GQmat, maxit=maxit)
        } else {
            glmerPwrssUpdate(pp,rr, control$tolPwrss, GQmat, maxit=maxit,
                             grpFac = object@flist[[1]])
        }
    }

    reCovs <- getReCovs(object)
    devlist <-
        c(list(pp = pp,
               resp = rr,
               mkPar = mkMkPar(reCovs),
               mkTheta = mkMkTheta(reCovs)),
        if (haveGLMM) {
            baseOffset <- forceCopy(object@resp$offset)

            list(tolPwrss= dc$cmp [["tolPwrss"]],
                 compDev = dc$dims[["compDev"]],
                 nAGQ = nAGQ,
                 lp0 = pp$linPred(1), ## object@resp$eta - baseOffset,
                 baseOffset = baseOffset,
                 pwrssUpdate = glmerPwrssUpdate,
                 ## save GQmat in the object and use that instead of nAGQ
                 GQmat = GHrule(nAGQ),
                 fac = object@flist[[1]],
                 verbose=verbose,
                 dpars=seq_len(getParLength(object)))
        }
        )
    rho <- list2env(devlist)
    ff <- mkdevfun(rho, nAGQ=nAGQ, maxit=maxit, verbose=verbose)
 ## xst       <- rep.int(0.1, nth)
    x0        <- rho$mkPar(rho$pp$theta)
    lower     <- getLower(object)
    upper     <- getUpper(object)
    if (haveGLMM && nAGQ > 0L) {
     ## xst   <- c(xst, sqrt(diag(pp$unsc())))
        n_beta <- length(fixef(object))
        x0    <- c(x0, unname(fixef(object)))
        lower <- c(lower, rep(-Inf, n_beta))
        upper <- c(upper, rep( Inf, n_beta))
    }
    ## control <- c(control,list(xst=0.2*xst, xt=xst*0.0001))
    ## FIX ME: allow use.last.params to be passed through
    calc.derivs <- !is.null(object@optinfo$derivs)
    ## if (haveGLMM) {
    ##     rho$resp$updateWts()
    ##     rho$pp$updateDecomp()
    ##     rho$lp0 <- rho$pp$linPred(1)
    ## }
    optimizer <- object@optinfo$optimizer
    if (!is.null(newopt <- ctrl.arg$optimizer)) {
        ## we might end up with a length-2 optimizer vector ...
        ##  use the *last* element
        optimizer <- newopt[length(newopt)]
    }
    opt <- optwrap(optimizer,
                   ff, x0, lower=lower, upper=upper,
                   control=control$optCtrl,
                   calc.derivs=calc.derivs)
    cc <- checkConv(attr(opt,"derivs"),opt$par,
                    ## FIXME: was there a reason that ctrl was passed
                    ## via the call slot?  it was causing problems
                    ## when optTheta called refit (github issue #173)
                    # ctrl = eval(object@call$control)$checkConv,
                    ctrl = control$checkConv,
                    lbound=lower, ubound=upper)
    if (haveGLMM) rr$setOffset(baseOffset)
    mkMerMod(environment(ff), opt,
             list(flist=object@flist, cnms=object@cnms,
                  Gp=object@Gp,
                  lower=getLower(object),
                  upper=getUpper(object),
                  reCovs=reCovs),
             object@frame, getCall(object), cc)
}

refitML.merMod <- function (x, optimizer="bobyqa", ...) {
    ## FIXME: optimizer is set to 'bobyqa' for back-compatibility, but that's not
    ##  consistent with lmer (default NM).  Should be based on internally stored 'optimizer' value
    if (!isREML(x)) return(x)
    stopifnot(is(rr <- x@resp, "lmerResp"))
    reCovs <- getReCovs(x)
    rho <- new.env(parent=parent.env(environment()))
    rho$resp <- new(class(rr), y=rr$y, offset=rr$offset, weights=rr$weights, REML=0L)
    xpp <- x@pp$copy()
    rho$pp <- new(class(xpp), X=xpp$X, Zt=xpp$Zt, Lambdat=xpp$Lambdat,
                  Lind=xpp$Lind, theta=xpp$theta, n=nrow(xpp$X))
    rho$mkPar <- mkMkPar(reCovs)
    rho$mkTheta <- mkMkTheta(reCovs)
    devfun <- mkdevfun(rho, 0L) # FIXME? also pass {verbose, maxit, control}
    opt <- ## "smart" calc.derivs rules
        if(optimizer == "bobyqa" && !any("calc.derivs" == ...names()))
            optwrap(optimizer, devfun, rho$mkPar(rho$pp$theta), lower=getLower(x), upper=getUpper(x), calc.derivs=TRUE, ...)
        else
            optwrap(optimizer, devfun, rho$mkPar(rho$pp$theta), lower=getLower(x), upper=getUpper(x), ...)
    ## FIXME: Should be able to call mkMerMod() here, and be done
    n <- length(rr$y)
    pp <- rho$pp
    p <- ncol(pp$X)
    dims <- c(N=n, n=n, p=p, nmp=n-p, q=nrow(pp$Zt), nth=length(pp$theta),
              useSc=1L, reTrms=length(x@cnms),
              spFe=0L, REML=0L, GLMM=0L, NLMM=0L)#, nAGQ=NA_integer_)
    wrss <- rho$resp$wrss()
    ussq <- pp$sqrL(1)
    pwrss <- wrss + ussq
    cmp <- c(ldL2=pp$ldL2(), ldRX2=pp$ldRX2(), wrss=wrss, ussq=ussq,
             pwrss=pwrss, drsum=NA_real_, dev=opt$fval, REML=NA,
             sigmaML=sqrt(pwrss/n), sigmaREML=sqrt(pwrss/(n-p)))
    ## modify the call  to have REML=FALSE. (without evaluating the call!)
    cl <- x@call
    cl[["REML"]] <- FALSE
    ans <-
    new("lmerMod", call = cl, frame=x@frame, flist=x@flist,
        cnms=x@cnms, theta=pp$theta, beta=pp$delb, u=pp$delu,
        optinfo = .optinfo(opt),
        lower=getLower(x), devcomp=list(cmp=cmp, dims=dims), pp=pp, resp=rho$resp,
        Gp=x@Gp)
    attr(ans, "upper") <- getUpper(x)
    attr(ans, "reCovs") <- upReCovs(reCovs, rho$pp$theta)
    ans
}

##' residuals of merMod objects                 --> ../man/residuals.merMod.Rd
##' @param object a fitted [g]lmer (\code{merMod}) object
##' @param type type of residuals
##' @param scaled scale residuals by residual standard deviation (=scale parameter)?
##' @param \dots additional arguments (ignored: for method compatibility)
residuals.merMod <- function(object,
                             type = if(isGLMM(object)) "deviance" else "response",
                             scaled = FALSE, ...) {
    r <- residuals(object@resp, type,...)
    fr <- model.frame(object)
    if (is.null(nm <- rownames(fr))) nm <- seq_along(r)
    names(r) <- nm
    if (scaled) r <- r/sigma(object)
    if (!is.null(na.action <- attr(fr, "na.action")))
        naresid(na.action, r)
    else
        r
}

##' @rdname residuals.merMod
##' @S3method residuals lmResp
##' @method residuals lmResp
residuals.lmResp <- function(object,
                             type = c("working", "response", "deviance",
                                      "pearson", "partial"), ...) {
    y <- object$y
    r <- object$wtres
    mu <- object$mu
    switch(match.arg(type),
           working =,
           response = y-mu,
           deviance =,
           pearson = r,
           partial = stop(gettextf("partial residuals are not implemented yet"),
                          call. = FALSE)
           )
}

##' @rdname residuals.merMod
##' @S3method residuals glmResp
##' @method residuals glmResp
residuals.glmResp <- function(object, type = c("deviance", "pearson",
                                      "working", "response", "partial"),
                              ...) {
    type <- match.arg(type)
    y <- object$y
    mu <- object$mu
    switch(type,
           deviance = {
               ## protect against slightly negative resids
               ## (GH 812)
               d.res <- sqrt(pmax(0,object$devResid()))
               ifelse(y > mu, d.res, -d.res)
           },
           pearson = object$wtres,
           working = object$wrkResids(),
           response = y - mu,
           partial = stop(gettextf("partial residuals are not implemented yet"),
                    call. = FALSE)
       )

}

hatvalues.merMod <- function(model, fullHatMatrix = FALSE, ...) {
    if(isGLMM(model)) warning("the hat matrix may not make sense for GLMMs")
    ## FIXME:  add restriction for NLMMs?

    ## prior weights, W ^ {1/2} :
    ## weights() will restore NA values if na.action = na.exclude,
    ##  need to drop them for now and restore at the end
    sqrtW <- Diagonal(x = na.omit(sqrt(weights(model, type = "prior"))))
    vList <- getME(model, c("L", "Lambdat", "Zt", "RX", "X", "RZX"))
    ## CL:= right factor of the random-effects component of the hat matrix  (64)
    CL <- with(vList,
               solve(L, solve(L,
                              Lambdat %*% Zt %*% sqrtW,
                              system = "P"), system = "L"))
    ## restore dimnames (needed since Matrix 1.5.2)
    dimnames(CL) <- dimnames(vList$Zt)
    ## CR:= right factor of the fixed-effects component of the hat matrix   (65)
    ##      {MM (FIXME Matrix):  t(.) %*% here faster than crossprod()}
    CR <- with(vList,
               solve(t(RX),
                     ## colScale(t(X), sqrtW) - crossprod(RZX, CL)
                     t(X) %*% sqrtW - crossprod(RZX, CL))
                     )
    res <- if(fullHatMatrix) { ## H = (C_L^T C_L + C_R^T C_R)             (63)
               crossprod(CL) + crossprod(CR)
           } else {
               ## diagonal of the hat matrix, diag(H) :
               colSums(CR^2) + colSums(CL^2)
           }
    napredict(attr(model.frame(model),"na.action"), res)
}


###----- Printing etc ----------------------------

## lme4.0, for GLMM had
## 'Generalized linear mixed model fit by the Laplace approximation'
## 'Generalized linear mixed model fit by the adaptive Gaussian Hermite approximation'
## so did *not* mention  "maximum likelihood" at all in the GLMM case


methTitle <- function(dims) { # dims == object@devcomp$dims
    GLMM <- dims[["GLMM"]]
    kind <- switch(1L + GLMM * 2L + dims[["NLMM"]],
                   "Linear", "Nonlinear",
                   "Generalized linear", "Generalized nonlinear")
    paste(kind, "mixed model fit by",
          if(dims[["REML"]]) "REML"
          else paste("maximum likelihood",
                     if(GLMM) {
                         ## TODO? Use shorter wording here, for (new) 'long = FALSE' argument
                         if((nAGQ <- dims[["nAGQ"]]) == 1)
                             "(Laplace Approximation)"
                         else
                             sprintf("(Adaptive Gauss-Hermite Quadrature, nAGQ = %d)",
                                     nAGQ)
                         }))
}


cat.f <- function(...) cat(..., fill = TRUE)

famlink <- function(object, resp = object@resp) {
    if(is(resp, "glmResp"))
        resp$family[c("family", "link")]
    else list(family = NULL, link = NULL)
}

##' @title print method title
##' @param mtit the result of methTitle(obj)
##' @param class typically class(obj)
.prt.methTit <- function(mtit, class) {
    if(nchar(mtit) + 5 + nchar(class) > (w <- getOption("width"))) {
        ## wrap around
        mtit <- strwrap(mtit, width = w - 2, exdent = 2)
        cat(mtit, " [",class,"]", sep = "", fill = TRUE)
    } else ## previous: simple one-liner
        cat(sprintf("%s ['%s']\n", mtit, class))
}

.prt.family <- function(famL) {
    if (!is.null(f <- famL$family)) {
        cat.f(" Family:", f,
              if(!is.null(ll <- famL$link)) paste(" (", ll, ")"))
    }
}

.prt.resids <- function(resids, digits, title = "Scaled residuals:", ...) {
    cat(title,"\n")
    ## FIXME: need testing code
    rq <- setNames(zapsmall(quantile(resids, na.rm=TRUE), digits + 1L),
                   c("Min", "1Q", "Median", "3Q", "Max"))
    print(rq, digits = digits, ...)
    cat("\n")
}

.prt.call <- function(call, long = TRUE) {
    if (!is.null(cc <- call$formula))
        cat.f("Formula:", deparse(cc))
    if (!is.null(cc <- call$data))
        cat.f("   Data:", deparse(cc))
    if (!is.null(cc <- call$weights))
        cat.f("Weights:", deparse(cc))
    if (!is.null(cc <- call$offset))
        cat.f(" Offset:", deparse(cc))
    if (long && length(cc <- call$control) &&
        !identical((dc <- deparse(cc)), "lmerControl()"))
        ## && !identical(eval(cc), lmerControl()))
        cat.f("Control:", dc)
    if (!is.null(cc <- call$subset))
        cat.f(" Subset:", deparse(cc))
}

##' @title Extract Log Likelihood, AIC, and related statics from a Fitted LMM
##' @param object a LMM model fit
##' @param devianceFUN the function to be used for computing the deviance; should not be changed for  \pkg{lme4} created objects.
##' @param chkREML optional logical indicating \code{object} maybe a REML fit.
##' @param devcomp
##' @return a list with components \item{logLik} and \code{AICtab} where the first is \code{\link{logLik(object)}} and \code{AICtab} is a "table" of AIC, BIC, logLik, deviance, df.residual() values.
llikAIC <- function(object, devianceFUN = devCrit, chkREML = TRUE, devcomp = object@devcomp) {
    llik <- logLik(object)   # returns NA for a REML fit - maybe change?
    AICstats <- {
        if(chkREML && devcomp$dims[["REML"]])
            devcomp$cmp["REML"] # *no* likelihood stats here
        else {
            c(AIC = AIC(llik), BIC = BIC(llik), logLik = c(llik),
              `-2*log(L)` = devianceFUN(object),
              df.resid = df.residual(object))
        }
    }
    list(logLik = llik, AICtab = AICstats)
}

.prt.aictab <- function(aictab, digits = 1) {
    t.4 <- round(aictab, digits)
    if (length(aictab) == 1 && names(aictab) == "REML")
        cat.f("REML criterion at convergence:", t.4)
    else {
        ## slight hack to get residual df formatted as an integer
        t.4F <- format(t.4)
        t.4F["df.resid"] <- format(t.4["df.resid"])
        print(t.4F, quote = FALSE)
    }
}

.prt.VC <- function(varcor, digits,
                    comp = "Std.Dev.",
                    formatter = format, ...) { # '...' *only* passed to print()
    cat("Random effects:\n")
    fVC <- formatVC(varcor, digits=digits, formatter=formatter, comp=comp)
    print(fVC, quote = FALSE, digits = digits, ...)
}

.prt.grps <- function(ngrps, nobs) {
    cat(sprintf("Number of obs: %d, groups: ", nobs),
        paste(paste(names(ngrps), ngrps, sep = ", "), collapse = "; "),
        fill = TRUE)
}

## FIXME: print header ("Warnings:\n") ?
##  change position in output? comes at the very end, could get lost ...
.prt.warn <- function(optinfo, summary=FALSE, ...) {
    if(length(optinfo) == 0) return() # currently, e.g., from refitML()
    ## check all warning slots: print numbers of warnings (if any)
    cc <- optinfo$conv$opt
    msgs <- unlist(optinfo$conv$lme4$messages)
    ## can't put nmsgs/nwarnings compactly into || expression
    ##   because of short-circuiting
    nmsgs <- length(msgs)
    warnings <- optinfo$warnings
    nwarnings <- length(warnings)
    if (cc > 0 || nmsgs > 0 || nwarnings > 0) {
        m <- if (cc==0) {
                 "(OK)"
             } else if (!is.null(optinfo$message)) {
                 sprintf("(%s)",optinfo$message)
             } else ""
        convmsg <- sprintf("optimizer (%s) convergence code: %d %s",
                           optinfo$optimizer, cc, m)
        if (summary) {
            cat(convmsg,sprintf("; %d optimizer warnings; %d lme4 warnings",
                nwarnings,nmsgs),"\n")
        } else {
            cat(convmsg,
                msgs,
                unlist(warnings),
                sep="\n")
            cat("\n")
        }
    }
}

##  options(lme4.summary.cor.max = 20)  --> ./hooks.R
##                                           ~~~~~~~~
## was      .summary.cor.max <- 20    a lme4-namespace hidden global variable

## This is modeled a bit after  print.summary.lm :
## Prints *both*  'mer' and 'merenv' - as it uses summary(x) mainly
##' @S3method print summary.merMod
print.summary.merMod <- function(x, digits = max(3, getOption("digits") - 3),
                                 correlation = NULL, symbolic.cor = FALSE,
                                 signif.stars = getOption("show.signif.stars"),
                                 ranef.comp = c("Variance", "Std.Dev."),
                                 ranef.corr = any(ranef.comp == "Std.Dev."),
                                 show.resids = TRUE, ...)
{
    .prt.methTit(x$methTitle, x$objClass)
    .prt.family(x)
    .prt.call(x$call); cat("\n")
    .prt.aictab(x$AICtab); cat("\n")
    if (show.resids)
        ## need residuals.merMod() rather than residuals():
        ##  summary.merMod has no residuals method
        .prt.resids(x$residuals, digits = digits)
    .prt.VC(x$varcor, digits = digits, useScale = x$useScale,
            comp = ranef.comp,
            corr = ranef.corr, ...)
    .prt.grps(x$ngrps, nobs = x$devcomp$dims[["n"]])

    p <- nrow(x$coefficients)
    if (p > 0) {
        cat("\nFixed effects:\n")
        printCoefmat(x$coefficients, # too radical: zap.ind = 3, #, tst.ind = 4
                     digits = digits, signif.stars = signif.stars)
        ## do not show correlation when   summary(*, correlation=FALSE)  was used:
        hasCor <- !is.null(VC <- x$vcov) && !is.null(VC@factors$correlation)
        ## FIXME: don't understand the logic here. We can easily
        ## defend against the problem of missing pre-computed correlation
        ## function by reconstituting it if necessary
        ## (e.g. if using merDeriv::vcov.lmerMod), as in commented code below.
        ## However, we currently have a test (using fit_agridat_archbold,
        ## see test-methods.R) that fails if we 'fix' this problem ...
        ##
        ## if (hasCor && is.null(VC@factors$correlation)) {
        ##     ## defend against merDeriv definition of vcov.lmerMod; reconstruct
        ##     cc <- cov2cor(VC)
        ##     dimnames(cc) <- dimnames(VC) ## Matrix 1.5.2 bug
        ##     VC@factors <- c(VC@factors, list(correlation = cc))
        ## }
        if(is.null(correlation)) { # default
            cor.max <- getOption("lme4.summary.cor.max")
            correlation <- hasCor && (isTRUE(x$corrSet) || p <= cor.max)
            if(!correlation && p > cor.max && is.na(x$corrSet)) {
                nam <- deparse(substitute(x))
                if(length(nam) > 1 || nchar(nam) >= 32) nam <- "...."
                message(sprintf(paste(
                    "\nCorrelation matrix not shown by default, as p = %d > %d.",
                    "Use print(%s, correlation=TRUE)  or",
                    "    vcov(%s)        if you need it\n", sep = "\n"),
                                p, cor.max, nam, nam))
            }
        }
        else if(!is.logical(correlation)) stop("'correlation' must be NULL or logical")
        if(correlation) {
            if(is.null(VC)) VC <- vcov(x, correlation = TRUE)
            corF <- VC@factors$correlation
            if (is.null(corF)) { # can this still happen?
                message("\nCorrelation of fixed effects could have been required in summary()")
                corF <- cov2cor(VC)
            }
            p <- ncol(corF)
            if (p > 1) {
                rn <- rownames(x$coefficients)
                rns <- abbreviate(rn, minlength = 11)
                cat("\nCorrelation of Fixed Effects:\n")
                if (is.logical(symbolic.cor) && symbolic.cor) {
                    corf <- as(corF, "matrix")
                    dimnames(corf) <- list(rns,
                                           abbreviate(rn, minlength = 1, strict = TRUE))
                    print(symnum(corf))
                } else {
                    corf <- matrix(format(round(corF@x, 3), nsmall = 3),
                                   ncol = p,
                                   dimnames = list(rns, abbreviate(rn, minlength = 6)))
                    corf[!lower.tri(corf)] <- ""
                    print(corf[-1, -p, drop = FALSE], quote = FALSE)
                } ## !symbolic.cor
            }  ## if (p > 1)
        } ## if (correlation)
    } ## if (p>0)

    if(length(x$fitMsgs) && any(nchar(x$fitMsgs) > 0)) {
        cat("fit warnings:\n"); writeLines(x$fitMsgs)
    }
    .prt.warn(x$optinfo,summary=FALSE)
    invisible(x)
}## print.summary.merMod


##' @S3method print merMod
print.merMod <- function(x, digits = max(3, getOption("digits") - 3),
                         correlation = NULL, symbolic.cor = FALSE,
                         signif.stars = getOption("show.signif.stars"),
                         ranef.comp = "Std.Dev.",
                         ranef.corr = any(ranef.comp == "Std.Dev."), ...)
{
    dims <- x@devcomp$dims
    .prt.methTit(methTitle(dims), class(x))
    .prt.family(famlink(x, resp = x@resp))
    .prt.call(x@call, long = FALSE)
    ## useScale <- as.logical(dims[["useSc"]])

    llAIC <- llikAIC(x)
    .prt.aictab(llAIC$AICtab, 4)
    varcor <- VarCorr(x)
    .prt.VC(varcor, digits = digits, comp = ranef.comp, corr = ranef.corr, ...)
    ngrps <- vapply(x@flist, nlevels, 0L)
    .prt.grps(ngrps, nobs = dims[["n"]])
    if(length(cf <- fixef(x)) > 0) {
        cat("Fixed Effects:\n")
        print.default(format(cf, digits = digits),
                      print.gap = 2L, quote = FALSE, ...)
    } else cat("No fixed effect coefficients\n")

    fitMsgs <- .merMod.msgs(x)
    if(any(nchar(fitMsgs) > 0)) {
        cat("fit warnings:\n"); writeLines(fitMsgs)
    }
    .prt.warn(x@optinfo,summary=TRUE)
    invisible(x)
}

##' @exportMethod show
setMethod("show",  "merMod", function(object) print.merMod(object))

##' Return the deviance component list
devcomp <- function(x) {
    .Deprecated("getME(., \"devcomp\")")
    stopifnot(is(x, "merMod"))
    x@devcomp
}

##' @exportMethod getL
setMethod("getL", "merMod", function(x) {
    .Deprecated("getME(., \"L\")")
    getME(x, "L")
})

##' used by tnames
mkPfun <- function(diag.only = FALSE, old = TRUE, prefix = NULL){
    local({
        function(g,e) {
            mm <- outer(e,e,paste,sep = ".")
            if(old) {
                diag(mm) <- e
            } else {
                mm[] <- paste(mm,g,sep = "|")
                if (!is.null(prefix)) mm[] <- paste(prefix[2],mm,sep = "_")
                diag(mm) <- paste(e,g,sep = "|")
                if (!is.null(prefix))  diag(mm) <- paste(prefix[1],diag(mm),sep = "_")
            }
            mm <- if (diag.only) diag(mm) else mm[lower.tri(mm,diag = TRUE)]
            if(old) paste(g,mm,sep = ".") else mm
        }
    })
}


## -> ../man/getME.Rd
getME <- function(object, name, ...) UseMethod("getME")


##' Extract or Get Generalized Components from a Fitted Mixed Effects Model
getME.merMod <- function(object,
                  name = c("X", "Z","Zt", "Ztlist", "mmList",
                  "y", "mu", "u", "b",
                  "Gp", "Tp",
                  "L", "Lambda", "Lambdat", "Lind", "Tlist", "A",
                  "RX", "RZX", "sigma",
                  "flist",
                  "fixef", "beta", "theta", "ST", "par",
                  "REML", "is_REML",
                  "n_rtrms", "n_rfacs",
                  "N", "n", "p", "q", "npar",
                  "p_i", "l_i", "q_i", "k", "m_i", "m",
                  "cnms",
                  "devcomp", "offset", "lower",
                  "devfun", "devarg",
                  "glmer.nb.theta"
                  ), ...)
{
    if(missing(name)) stop("'name' must not be missing")
    ## Deal with multiple names -- "FIXME" is inefficiently redoing things
    if (length(name <- as.character(name)) > 1) {
        names(name) <- name
        return(lapply(name, getME, object = object))
    }
    if(name == "ALL") ## recursively get all provided components
        return(sapply(eval(formals()$name),
                      getME.merMod, object=object, simplify=FALSE))

    stopifnot(is(object,"merMod"))
    name <- match.arg(name)
    rsp  <- object@resp
    PR   <- object@pp
    dc   <- object@devcomp
    th   <- object@theta
    cnms <- object@cnms
    dims <- dc $ dims
    Tpfun <- function(cnms) {
        ltsize <- function(n) n*(n+1)/2 # lower triangle size
        cLen <- cumsum(ltsize(lengths(cnms)))
        setNames(c(0, cLen),
                 c("beg__", names(cnms))) ## such that diff( Tp ) is well-named
    }
    ## mmList. <- mmList(object)
    if(any(name == c("p_i", "q_i", "m_i")))
        p_i <- vapply(mmList(object), ncol, 1L)
    if(any(name == c("l_i", "q_i")))
        l_i <- vapply(object@flist, nlevels, 1L)
    switch(name,
           "X" = PR$X, ## ok ? - check -- use model.matrix() method instead?
           "Z" = t(PR$Zt),
           "Zt" = PR$Zt,
           "Ztlist" =
           {
               getInds <- function(i) {
                   n <- diff(object@Gp)[i] ## number of elements in this block
                   nt <- length(cnms[[i]]) ## number of REs
                   inds <- lapply(seq(nt), seq, to = n, by = nt)  ## pull out individual RE indices
                   inds <- lapply(inds,function(x) x + object@Gp[i])  ## add group offset
               }
               inds <- do.call(c,lapply(seq_along(cnms),getInds))
               setNames(lapply(inds,function(i) PR$Zt[i,]),
                        unlist(getVCNames(object)$vcomp, FALSE, FALSE))
           },
           "mmList" = mmList.merMod(object),
           "y" = rsp$y,
           "mu" = rsp$mu,
           "u" = object@u,
           "b" = crossprod(PR$Lambdat, object@u), # == Lambda %*% u
           "L" = PR$ L(),
           "Lambda" = t(PR$ Lambdat),
           "Lambdat" = PR$ Lambdat,
           "A" = PR$Lambdat %*% PR$Zt,
           "Lind" = PR$ Lind,
           "RX" = structure(PR$RX(), dimnames = list(colnames(PR$X), colnames(PR$X))), ## maybe add names elsewhere?
           "RZX" = structure(PR$RZX, dimnames = list(NULL, colnames(PR$X))), ## maybe add names elsewhere?
           "sigma" = sigma(object),
           "Gp" = object@Gp,
           "Tp" = Tpfun(cnms), # "term-wise theta pointer"
           "flist" = object@flist,
           "fixef" = fixef(object),
           "beta"  = object@beta,
           "theta" = setNames(th, getThetaNames(object)),
           "par" = setNames(object@optinfo$val[seq_len(getParLength(object))],
                            getParNames(object)),
           "ST" = setNames(vec2STlist(object@theta, n = lengths(cnms)),
                           names(cnms)), ## FIXME: check for flexSigma?
           "Tlist" = setNames(lapply(getReCovs(object), getLambda), names(object@cnms)),
           "REML" = dims[["REML"]],
           "is_REML" = isREML(object),
           ## number of random-effects terms
           "n_rtrms" = length(cnms),
           ## number of random-effects grouping factors
           "n_rfacs" = length(object@flist),
           "N" = dims[["N"]],
           "n" = dims[["n"]],
           "p" = dims[["p"]],
           "q" = dims[["q"]],
           "npar" = as.list(dims)[["npar"]] %||% length(getLower(object)),
           "p_i" = p_i,
           "l_i" = l_i,
           "q_i" = p_i * l_i,
           "k" = length(cnms),
           "m_i" = choose(p_i + 1, 2),
           "m" = dims[["nth"]],
           "cnms" = cnms,
           "devcomp" = dc,
           "offset" = rsp$offset,
           "lower" = setNames(getLower(object), getParNames(object)),
           "devfun" = {
               verbose <- getCall(object)$verbose; if (is.null(verbose)) verbose <- 0L
               reCovs <- getReCovs(object)
               if (isGLMM(object)) {
                   reTrms <- getME(object,c("Zt","theta","Lambdat","Lind","flist","cnms"))
                   reTrms$lower <- getLower(object)
                   reTrms$upper <- getUpper(object)
                   reTrms$reCovs <- reCovs
                   mkGlmerDevfun(object@frame, X=getME(object, "X"),
                                 reTrms=reTrms, family=family(object),
                                 nAGQ=object@devcomp$dims[["nAGQ"]],
                                 verbose=verbose,
                                 control=glmerControl(
                                     compDev=as.logical(object@devcomp$dims[["compDev"]]),
                                     tolPwrss=object@devcomp$cmp[["tolPwrss"]]))
               } else {
                   ## copied from refit ... DRY ...
                   devlist <- list(pp=PR, resp=rsp,
                                   lower = getLower(object),
                                   upper = getUpper(object),
                                   mkPar = mkMkPar(reCovs),
                                   mkTheta = mkMkTheta(reCovs))
                   mkdevfun(rho=list2env(devlist),
                            ## FIXME: fragile ... // also pass 'maxit' ?
                            verbose=verbose, control=object@optinfo$control)
               }
           },
           "devarg" = {
             nm <- getParNames(object)
             if (!isLMM(object)) nm <- c(nm, names(fixef(object)))
             setNames(object@optinfo$val, nm)
           },
           ## FIXME: current version gives lower bounds for theta parameters only:
           ## -- these must be extended for [GN]LMMs -- give extended value including -Inf values for beta values?

           "glmer.nb.theta" = if(isGLMM(object) && isNBfamily(rsp$family$family))
               getNBdisp(object) else NA_real_,

           "..foo.." = # placeholder!
           stop(gettextf("'%s' is not implemented yet",
                         sprintf("getME(*, \"%s\")", name))),
           ## otherwise
           stop(sprintf("Mixed-Effects extraction of '%s' is not available for class \"%s\"",
                        name, class(object))))
}## {getME}

##' @importMethodsFrom Matrix t %*% crossprod diag tcrossprod
##' @importClassesFrom Matrix dgCMatrix dpoMatrix corMatrix
NULL

## Extract the conditional variance-covariance matrix of the fixed-effects
## parameters
vcov.merMod <- function(object, correlation = TRUE, sigm = sigma(object),
                        use.hessian = NULL, full = FALSE, noScale = NULL, ...)
{
    ## FIXME: warn/message if GLMM (RX-computation is approximate),
    ## if other vars are specified?
    if (full) return(vcov_full(object, sigm))

    hess.avail <-
         ## (1) numerical Hessian computed?
        (!is.null(h <- object@optinfo$derivs$Hessian) &&
         ## (2) does Hessian include fixed-effect parameters?
         nrow(h) > (ntheta <- length(object@theta)))
    if (is.null(use.hessian)) use.hessian <- hess.avail
    if (use.hessian && !hess.avail) stop(shQuote("use.hessian"),
                                         "=TRUE specified, ",
                                         "but Hessian is unavailable")
    calc.vcov.hess <- function(h) {
        ## invert 2*Hessian, catching errors and forcing symmetric result
        ## ~= forceSymmetric(solve(h/2)[i,i]) : solve(h/2) = 2*solve(h)
        h <- tryCatch(solve(h),
                      error=function(e) matrix(NA_real_,nrow=nrow(h),ncol=ncol(h)))
        i <- -seq_len(ntheta)  ## drop var-cov parameters
        h <- h[i,i]
        forceSymmetric(h + t(h))
    }

    ## alternately: calculate var-cov from implicit (RX) information
    ## provided by fit (always the case for lmerMods)
    V <- sigm^2 * object@pp$unsc()

    if (hess.avail) {
        V.hess <- calc.vcov.hess(h)
        bad.V.hess <- any(is.na(V.hess))
        if (!bad.V.hess) {
            ## another 'bad var-cov' check: positive definite?
            e.hess <- eigen(V.hess,symmetric = TRUE,only.values = TRUE)$values
            if (min(e.hess) <= 0) bad.V.hess <- TRUE
        }
    }
    if (!use.hessian && hess.avail) {
        ## if hessian is available, go ahead and check
        ## for similarity with the RX-based estimate
        var.hess.tol <- 1e-4 # FIXME: should var.hess.tol be user controlled?
        if (!bad.V.hess && any(abs(V-V.hess) > var.hess.tol * V.hess))
            warning("variance-covariance matrix computed ",
                    "from finite-difference Hessian\nand ",
                    "from RX differ by >",var.hess.tol,": ",
                    "consider ",shQuote("use.hessian=TRUE"))
    }
    if (use.hessian) {
        if (!bad.V.hess) {
            V <- V.hess
        } else {
            warning("variance-covariance matrix computed ",
                    "from finite-difference Hessian is\n",
                    "not positive definite or contains NA values: falling back to ",
                    "var-cov estimated from RX")
        }
    }

    ## FIXME: try to catch non-PD matrices
    rr <- tryCatch(as(V, "dpoMatrix"), error = function(e)e)
    if (inherits(rr, "error")) {
        warning(gettextf("Computed variance-covariance matrix problem: %s;\nreturning NA matrix",
                         rr$message), domain = NA)
        rr <- matrix(NA_real_,nrow(V),ncol(V))
    }

    nmsX <- colnames(object@pp$X)
    dimnames(rr) <- list(nmsX,nmsX)

    if(correlation)
        rr@factors$correlation <-
            if(!is.na(sigm)) as(rr, "corMatrix") else rr # (is NA anyway)
    
    ## If auto-scaling is enabled
    if(is.null(noScale) || (!is.null(noScale) && !noScale)){
      if (!is.null(sc <- attr(object@pp$X, "scaled:scale"))) {
        ce <- attr(object@pp$X, "scaled:center")
        
        rr <- scale_vcov(rr, sc, ce)
      }
    }
    rr
}

vcov_full <- function(object, s = sigma(object)) {
    L <- getME(object, "L")
    RX <- getME(object, "RX")
    RZX <- getME(object, "RZX")
    Lambdat <- getME(object, "Lambdat")

    RXtinv <- solve(t(RX))
    LinvLambdat <- solve(L, Lambdat, system = "L")
    Minv <- s * rbind(
                    cbind(LinvLambdat,
                          Matrix(0, nrow = nrow(L), ncol = ncol(RX))),
                    cbind(-RXtinv %*% t(RZX) %*% LinvLambdat, RXtinv)
                )
    Cmat <- crossprod(Minv)

    ## do we have machinery elsewhere for this (names for b-vector) ?
    ## should this be extracted into a utility f'n (e.g. for getME(., "b") ?
    fix_nms <- colnames(object@pp$X)
    rr <- ranef(object, condVar = FALSE)

    gnms <- function(x) c(outer(colnames(x), rownames(x), function(x,y) paste(y, x, sep = ".")))
    rnms <- lapply(rr, gnms)
    re_nms <- unlist(Map(function(n, r) paste(n, r, sep = "."), names(rr), rnms))

    all_nms <- unname(c(re_nms, fix_nms))
    dimnames(Cmat) <- list(all_nms, all_nms)
    return(Cmat)
}

##' @importFrom stats vcov
##' @S3method vcov summary.merMod
vcov.summary.merMod <- function(object, ...) {
    if(is.null(object$vcov)) stop("logic error in summary of merMod object")
    object$vcov
}

if(FALSE)## *NOWHERE* used _FIXME_ ??
## Compute standard errors of fixed effects from an merMod object
##
## @title Standard errors of fixed effects
## @param object "merMod" object,
## @param ... additional, optional arguments.  None are used at present.
## @return numeric vector of length length(fixef(.))
unscaledVar <- function(object, ...) {
    stopifnot(is(object, "merMod"))
    sigma(object) * diag(object@pp$unsc())
}


##' @S3method summary merMod
summary.merMod <- function(object,
                           correlation = (p <= getOption("lme4.summary.cor.max")),
                           use.hessian = NULL,
                           ...)
{
    if (...length() > 0) {
        ## FIXME: need testing code
        warning("additional arguments ignored")
    }
    ## se.calc:
    hess.avail <- (!is.null(h <- object@optinfo$derivs$Hessian) &&
                   nrow(h) > length(object@theta))
    if (is.null(use.hessian)) use.hessian <- hess.avail
    if (use.hessian && !hess.avail)
        stop("'use.hessian=TRUE' specified, but Hessian is unavailable")
    resp <- object@resp
    devC <- object@devcomp
    dd <- devC$dims
    ## cmp <- devC$cmp
    useSc <- as.logical(dd[["useSc"]])
    sig <- sigma(object)
    ## REML <- isREML(object)

    famL <- famlink(resp = resp)
    p <- length(coefs <- fixef(object))

    ## protect against merDeriv's vcov.glmerMod(), which only
    ## handles binomial and Poisson values
    if (is(object, "glmerMod") && !family(object)$family %in% c("binomial", "poisson")) {
        vcov <- vcov.merMod
    }
    vc <- vcov(object, use.hessian = use.hessian)
    stdError <- sqrt(diag(vc))
    coefs <- cbind("Estimate" = coefs,
                   "Std. Error" = stdError)
    if (p > 0) {
        coefs <- cbind(coefs, (cf3 <- coefs[,1]/coefs[,2]), deparse.level = 0)
        colnames(coefs)[3] <- paste(if(useSc) "t" else "z", "value")
        if (isGLMM(object)) # FIXME: if "t" above, cannot have "z" here
            coefs <- cbind(coefs, "Pr(>|z|)" =
                           2*pnorm(abs(cf3), lower.tail = FALSE))
    }

    llAIC <- llikAIC(object)
    ## FIXME: You can't count on object@re@flist,
    ##        nor compute VarCorr() unless is(re, "reTrms"):
    varcor <- VarCorr(object)
                                        # use S3 class for now
    structure(list(methTitle = methTitle(dd),
                   objClass = class(object),
                   devcomp = devC,
                   isLmer = is(resp, "lmerResp"),
                   useScale = useSc,
                   logLik = llAIC[["logLik"]],
                   family = famL$family,
                   link = famL$link,
                   ngrps = ngrps(object),
                   coefficients = coefs,
                   sigma = sig,
                   ## explicitly call method to avoid getting m
                   ## messed up by merDeriv's vcov method
                   ##  (which doesn't assign a VC attribute)
                   vcov = vcov.merMod(object, correlation = correlation, sigm = sig),
                   varcor = varcor, # and use formatVC(.) for printing.
                   AICtab = llAIC[["AICtab"]],
                   call = object@call,
                   residuals = residuals(object,"pearson",scaled = TRUE),
                   fitMsgs = .merMod.msgs(object),
                   optinfo = object@optinfo,
                   ## put corrSet **last** so we don't mess up people relying on numeric indexing
                   ##  of elements (!!)
                   corrSet = if(!missing(correlation)) correlation else NA # TRUE/FALSE (when set) / NA
                   ), class = "summary.merMod")
}

## TODO: refactor? Why the hell do we need a summary method for summary.merMod?
##' @S3method summary summary.merMod
summary.summary.merMod <- function(object, varcov = TRUE, ...) {
    if(varcov && is.null(object$vcov))
        object$vcov <- vcov.merMod(object, correlation = TRUE, sigm = object$sigma)
    object
}


##' @importFrom stats weights
##' @S3method weights merMod
weights.merMod <- function(object, type = c("prior","working"), ...) {
    type <- match.arg(type)
    res <- if (type == "prior") object@resp$weights else object@pp$Xwts^2
    res <- napredict(na.action(object), res)

    ## the working weights available through pp$Xwts should be
    ## equivalent to:
    ##     object@resp$weights*(object@resp$muEta()^2)/object@resp$variance()
    ## however, the unit tests in tests/glmmWeights.R suggest that this
    ## equivalence is approximate.  this may be fine, however, if the
    ## discrepancy is due to another instance of the general problem of
    ## reference class fields not being updated at the optimum, then this
    ## could cause real problems.  see for example:
    ## https://github.com/lme4/lme4/issues/166
    
    return(res)
}

## utility function: x is a ranef.mer object, nx is the name of an element
asDf0 <- function(x,nx,id=FALSE) {
    xt <- x[[nx]]
    ss <- stack(xt)
    ss$ind <- factor(as.character(ss$ind), levels = colnames(xt))
    ss$.nn <- rep.int(reorder(factor(rownames(xt)), xt[[1]],
                              FUN = mean,sort = sort), ncol(xt))
    ## allow 'postVar' *or* 'condVar' names
    pv <- attr(xt,"postVar")
    if (is.null(pv)) {
        pv <- attr(xt,"condVar")
    }
    if (!is.null(pv)) {
        tmpfun <- function(pvi) {
            unlist(lapply(1:nrow(pvi), function(i) sqrt(pvi[i, i, ])))
        }
        if (!is.list(pv)) {
            ss$se <- tmpfun(pv)
        } else {
            ## rely on ordering when unpacking!
            ss$se <- unlist(lapply(pv,tmpfun))
        }
    }
    if (id) ss$id <- nx
    return(ss)
}

## convert ranef object to a long-format data frame, e.g. suitable
##  for ggplot2 (or homemade lattice plots)
## FIXME: have some gymnastics to do if terms, levels are different
##  for different grouping variables - want to maintain ordering
##  but still allow rbind()ing
as.data.frame.ranef.mer <- function(x, ...) {
    xL <- lapply(names(x), asDf0, x=x, id=TRUE)
    ## combine
    xD <- do.call(rbind,xL)
    ## rename ...
    oldnames <- c("values", "ind", ".nn", "se",    "id")
    newnames <- c("condval","term","grp", "condsd","grpvar")
    names(xD) <- newnames[match(names(xD),oldnames)]
    ## reorder ...
    neworder <- c("grpvar","term","grp","condval")
    if ("condsd" %in% names(xD)) neworder <- c(neworder,"condsd")
    xD[neworder]
}

dim.merMod <- function(x) {
    getME(x, c("n", "p", "q", "p_i", "l_i", "q_i", "k", "m_i", "m"))
}

## Internal utility, only used in optwrap() :
##' @title Get the optimizer function and check it minimally
##' @param optimizer character string ( = function name) *or* function
getOptfun <- function(optimizer) {
    if (((is.character(optimizer) && optimizer == "optimx") ||
         deparse(substitute(optimizer)) == "optimx")) {
        if (!requireNamespace("optimx")) {
            stop(shQuote("optimx")," package must be installed order to ",
                 "use ",shQuote('optimizer="optimx"'))
        }
        optfun <- optimx::optimx
    } else if (is.character(optimizer)) {
        optfun <- tryCatch(get(optimizer), error = function(e) NULL)
    } else optfun <- optimizer
    if (is.null(optfun)) stop("couldn't find optimizer function ",optimizer)
    if (!is.function(optfun)) stop("non-function specified as optimizer")
    needArgs <- c("fn","par","lower","control")
    if (anyNA(match(needArgs, names(formals(optfun)))))
        stop("optimizer function must use (at least) formal parameters ",
             paste(sQuote(needArgs), collapse = ", "))
    optfun
}

optwrap <- function(optimizer, fn, par, lower = -Inf, upper = Inf,
                    control = list(), adj = FALSE, calc.derivs = TRUE,
                    force.calc.derivs = FALSE,
                    use.last.params = FALSE,
                    verbose = 0L)
{
    ## calc.derivs may be passed as NULL by some upstream pkgs ...
    calc.derivs <- calc.derivs %||% TRUE
    ## control must be specified if adj==TRUE;
    ##  otherwise this is a fairly simple wrapper
    optfun <- getOptfun(optimizer)
    optName <- if(is.character(optimizer)) optimizer
    else ## "good try":
        deparse(substitute(optimizer))[[1L]]

    lower <- rep_len(lower, length(par))
    upper <- rep_len(upper, length(par))

    if (adj)
        ## control parameter tweaks: only for second round in nlmer, glmer
        switch(optName,
               "bobyqa" = {
                   if(!is.numeric(control$rhobeg)) control$rhobeg <- 0.0002
                   if(!is.numeric(control$rhoend)) control$rhoend <- 2e-7
               },
               "Nelder_Mead" = {
                   if (is.null(control$xst))  {
                       betaSD <- sqrt(diag(environment(fn)$pp$unsc()))
                       xst0 <- rep(0.1, length(par) - length(betaSD))
                       xst1 <- pmin(betaSD, 10)
                       control$xst <- 0.2 * c(xst0, xst1)
                   }
                   if (is.null(control$xt)) control$xt <- control$xst*5e-4
               })
    switch(optName,
           "bobyqa" = {
               if(all(par == 0)) par[] <- 0.001  ## minor kludge
               if(!is.numeric(control$iprint)) control$iprint <- min(verbose, 3L)
           },
           "Nelder_Mead" = control$verbose <- verbose,
           "nloptwrap" = control$print_level <- min(as.numeric(verbose),3L),
           ## otherwise:
           if(verbose) warning(gettextf(
               "'verbose' not yet passed to optimizer '%s'; consider fixing optwrap()",
                                        optName), domain = NA)
           )
    arglist <- list(fn = fn, par = par, lower = lower, upper = upper, control = control)
    ## optimx: must pass method in control (?) because 'method' was previously
    ## used in lme4 to specify REML vs ML
    if (optName == "optimx") {
        if (is.null(method <- control$method))
            stop("must specify 'method' explicitly for optimx")
        arglist$control$method <- NULL
        arglist <- c(arglist, list(method = method))
    }
    ## FIXME: test!  effects of multiple warnings??
    ## may not need to catch warnings after all??
    curWarnings <- list()
    opt <- withCallingHandlers(do.call(optfun, arglist),
                               warning = function(w) {
                                   curWarnings <<- append(curWarnings,list(w$message))
                               })
    ## cat("***",unlist(tail(curWarnings,1)))
    ## FIXME: set code to warn on convergence !=0
    ## post-fit tweaking
    if (optName == "bobyqa") {
        opt$convergence <- opt$ierr
    } else if (optName == "Nelder_Mead") {
        ## fix-up: Nelder_Mead treats running out of iterations as "convergence" (!?)
        if (opt$NM.result==4) opt$convergence <- 4
    } else if (optName == "optimx") {
        opt <- list(par = coef(opt)[1,],
                    fvalues = opt$value[1],
                    method = method,
                    conv = opt$convcode[1],
                    feval = opt$fevals + opt$gevals,
                    message = attr(opt,"details")[,"message"][[1]])
    }
    if ((optconv <- getConv(opt)) != 0) {
        wmsg <- paste("convergence code",optconv,"from",optName)
        if (!is.null(getMsg(opt))) wmsg <- paste0(wmsg,": ",getMsg(opt))
        warning(wmsg)
        curWarnings <<- append(curWarnings,list(wmsg))
    }
    ## pp_before <- environment(fn)$pp
    ## save(pp_before,file="pp_before.RData")

    singular <- min(opt$par - lower, upper - opt$par) < getSingTol()
    if (force.calc.derivs || (calc.derivs && !singular)){
        if (use.last.params) {
            ## +0 tricks R into doing a deep copy ...
            ## otherwise element of ref class changes!
            ## FIXME:: clunky!!
            orig_pars <- opt$par
            orig_theta <- environment(fn)$pp$theta+0
            orig_pars[seq_along(orig_theta)] <- orig_theta
        }
        if (verbose > 10) cat("computing derivatives\n")
        derivs <- deriv12(fn, opt$par, fx = opt$value)
        if (use.last.params) {
            ## run one more evaluation of the function at the optimized
            ##  value, to reset the internal/environment variables in devfun ...
            fn(orig_pars)
        }
    } else derivs <- NULL

    if (!use.last.params) {
        ## run one more evaluation of the function at the optimized
        ##  value, to reset the internal/environment variables in devfun ...
        fn(opt$par)
    }
    structure(opt, ## store all auxiliary information
              optimizer = optimizer,
              control   = control,
              warnings  = curWarnings,
              derivs    = derivs)
}

(testLevel <- if (nzchar(s <- Sys.getenv("LME4_TEST_LEVEL"))) as.numeric(s) else 1)

if (testLevel > 1) {

  ## read system file 
  glmmTMB_glmer_ar1 <- readRDS(
    system.file("testdata", "glmmTMB_glmer_ar1_comparison.rds", package = "lme4")
  )
  Contraception <- readRDS(
    system.file("testdata", "Contraception.rds", package = "lme4")
  )

  test_that("glmer + ar1 does not return zero values for fixed effects coefficients", {
    ## Test 1; make sure the results match as expected
    form <- out_bin_1 ~ ftime + trt + ar1(ftime + 0 | id_cluster) +
      (1 | id_individual)
    dd <- glmmTMB_glmer_ar1$glmer_ar1_dd
    lme4_1 <- glmer(form, data = dd, family = binomial)
    
    thetas <- c(getReCovs(lme4_1)[[1]]@par, getReCovs(lme4_1)[[2]]@par)
    betas <- fixef(lme4_1)
    expect_equal_nocheck(c(thetas, betas), lme4_1@optinfo$val)
    
    ## Test 2; make sure we still have nonzero results
    form2 <- update(form, out_gauss ~ .)
    lme4_2 <- lmer(form2, data = dd, REML = FALSE)
    expect_true(!all(fixef(lme4_2) == 0.0))
    
    ## Test 3; make sure ncol(X) matches beta
    gm.ar1_0 <- glmer(use ~ age + urban + ar1(0 + urban | district),
                      data = Contraception,
                      family = binomial)
    expect_equal(length(colnames(getME(gm.ar1_0, "X"))), 
                 length(getME(gm.ar1_0, "beta")))
  })
}

test_that("npar test works for models with 0 FE parameters", {
  expect_is(glmer(round(Reaction) ~ 0 + (1|Subject), family = poisson,
                  data = sleepstudy),
            "merMod")
})

library(lme4)
library(testthat)

(testLevel <- lme4:::testLevel())
L <- load(system.file("testdata/lme-tst-fits.rda",
                      package="lme4", mustWork=TRUE))

if (testLevel>1) {
if (getRversion() > "3.0.0") {
    ## saved fits are not safe with old R versions

fm1 <- fit_sleepstudy_1


s1 <- simulate(fm1,seed=101)[[1]]
s2 <- simulate(fm1,seed=101,use.u=TRUE)
s3 <- simulate(fm1,seed=101,nsim=10)
s4 <- simulate(fm1,seed=101,use.u=TRUE,nsim=10)
stopifnot(length(s3)==10,all(sapply(s3,length)==180),
          length(s4)==10,all(sapply(s4,length)==180))

# test hook for cluster random effects
fakerand <- function(n) seq(from=1.0, by=0.4, length.out=n)
# In fact there are 10 observations/subject for each, but more robustly
ns <- tabulate(model.frame(fm1)$Subject)
s5 <- simulate(fm1, nsim=1, seed=12345, cluster.rand=fakerand)
su <- as.data.frame(VarCorr(fm1))[1, "sdcor"] # sd if cluster effects
y1 <- predict(fm1, re.form = ~0, se.fit=FALSE) # fixed effect
y2 <- su*rep(fakerand(length(ns)), times = ns) # cluster effect, per individual
set.seed(12345)
y3 <- sigma(fm1)*rnorm(length(y1)) # individual error terms
y <- y1+y2+y3
# testthat::expect_equal automatically incorporates the tolerance *if*
# it is in 3rd edition mode.  Currently, this package does not use that mode.
stopifnot(all(abs(s5-y)< testthat_tolerance()))


## binomial (2-column and prob/weights)
gm1 <- fit_cbpp_1
gm2 <- fit_cbpp_3

gm1_s1 <- simulate(gm1,seed=101)[[1]]
gm1_s2 <- simulate(gm2,seed=101)[[1]]
stopifnot(all.equal(gm1_s1[,1]/rowSums(gm1_s1),gm1_s2))
gm1_s3 <- simulate(gm1,seed=101,use.u=TRUE)
gm1_s4 <- simulate(gm1,seed=101,nsim=10)
gm1_s5 <- simulate(gm2,seed=101,nsim=10)
stopifnot(length(gm1_s4)==10,all(sapply(gm1_s4,ncol)==2),all(sapply(gm1_s4,nrow)==56))
stopifnot(length(gm1_s5)==10,all(sapply(gm1_s5,length)==56))

## binomial (factor): Kubovy bug report 1 Aug 2013
d <- data.frame(y=factor(rep(letters[1:2],each=100)),
                f=factor(rep(1:10,10)))
g1 <- glmer(y~(1|f),data=d,family=binomial)
s6 <- simulate(g1,nsim=10)
stopifnot(length(s6)==10,all(sapply(s6,length)==200))

## test explicitly stated link function
gm3 <- glmer(cbind(incidence, size - incidence) ~ period +
             (1 | herd), data = cbpp, family = binomial(link="logit"))
s4 <- simulate(gm3,seed=101)[[1]]
stopifnot(all.equal(gm1_s1,s4))

cbpp$obs <- factor(seq(nrow(cbpp)))
gm4 <- fit_cbpp_2
## glmer(cbind(incidence, size - incidence) ~ period +
##             (1 | herd) + (1|obs), data = cbpp, family = binomial)

s5 <- simulate(gm4,seed=101)[[1]]
s6 <- simulate(gm4,seed=101,use.u=TRUE)[[1]]

## Bernoulli
## works, but too slow
if (testLevel > 2) {
    if(require("mlmRev")) {
        data(guImmun, package="mlmRev")
        table(guImmun$immun)
        ##    N    Y
        ## 1195  964
        g1i <- glmer(immun ~ kid2p+mom25p+ord+ethn+momEd+husEd+momWork+rural+pcInd81+
                         (1|comm/mom), family="binomial", data=guImmun)
        ## In checkConv(attr(opt, "derivs"), opt$par, ctrl = control$checkConv,  :
        ##    Model failed to converge with max|grad| = 0.326795 (tol = 0.002, component 1)
        sg1 <- simulate(g1i)
        if(FALSE) { ## similar: not relevant here {comment out for 'R CMD check'}:
        ## if(require("glmmTMB")) {
            g2 <- glmmTMB(immun ~ kid2p+mom25p+ord+ethn+momEd+husEd+momWork+rural+pcInd81+
                          (1|comm/mom), family="binomial", data=guImmun)
            sg2 <- simulate(g2)
        ## }
        }
    }
}

set.seed(101)
d <- data.frame(f = factor(rep(LETTERS[1:10],each=10)))
d$x <- runif(nrow(d))
u <- rnorm(10)
d$eta <- with(d, 1 + 2*x + u[f])
d$y <- rbinom(nrow(d), size=1, prob = plogis(d$eta))

g1 <- glmer(y ~ x + (1|f), data=d, family="binomial")
## tolPwrss=1e-5: no longer necessary

if (testLevel > 2) { ## trying a set of  glmerControl(tolPwrss = 10^t) :
  allcoef <- function(x) c(dev = deviance(x), th = getME(x,"theta"), beta = getME(x,"beta"))
  tfun <- function(t) {
      gg <- try( ## << errors (too small tolPwrss) are still printed :
          glmer(y~x+(1|f),data=d,family="binomial",
                    control = glmerControl(tolPwrss = 10^t)))
    if (inherits(gg,"try-error")) rep(NA,4) else allcoef(gg)
  }
  tvec <- seq(-4,-16,by=-0.25)
  tres <- cbind(t = tvec, t(sapply(tvec, tfun)))
  print(tres)
}

gm_s5 <- simulate(g1, seed=102)[[1]]

d$y <- factor(c("N","Y")[d$y+1])
g1B <- glmer(y ~ x + (1|f), data=d, family="binomial") ## ,tolPwrss=1e-5)
s1B <- simulate(g1B, seed=102)[[1]]
stopifnot(all.equal(gm_s5,as.numeric(s1B)-1))

## another Bernoulli
if(requireNamespace("mlmRev")) {
    data(Contraception, package="mlmRev")
    gm5 <- glmer(use ~ urban+age+livch+(1|district), Contraception, binomial)
    s3 <- simulate(gm5)
}
d$y <- rpois(nrow(d),exp(d$eta))
gm6 <- glmer(y~x+(1|f),data=d,family="poisson")
s4 <- simulate(gm6)

## simulation 'from scratch' with formulas:
## binomial
## form <- formula(gm1)[-2]
form <- ~ (1|herd) + period
gm1_s4 <- simulate(form,newdata=model.frame(gm1),
               newparams=list(theta=getME(gm1,"theta"),
               beta=fixef(gm1)),
               family=binomial,
               weights=rowSums(model.frame(gm1)[[1]]),
               seed=101)[[1]]
stopifnot(all.equal(gm1_s2,gm1_s4))

gm1_s5 <- simulate(formula(gm1),newdata=cbpp,
               newparams=list(theta=getME(gm1,"theta"),
               beta=fixef(gm1)),
               family=binomial,
               seed=101)[[1]]
stopifnot(all.equal(gm1_s1,gm1_s5))

tt <- getME(gm1,"theta")
bb <- fixef(gm1)
expect_error(simulate(form,newdata=model.frame(gm1),
               newparams=list(theta=setNames(tt,"abc"),
               beta=fixef(gm1)),
               family=binomial,
               weights=rowSums(model.frame(gm1)[[1]]),
               seed=101),"mismatch between")
expect_error(simulate(form,newdata=model.frame(gm1),
               newparams=list(theta=tt,
               beta=setNames(bb,c("abc",names(bb)[-1]))),
               family=binomial,
               weights=rowSums(model.frame(gm1)[[1]]),
               seed=101),"mismatch between")

## Gaussian
form <- formula(fm1)[-2]
s7 <- simulate(form,newdata=model.frame(fm1),
               newparams=list(theta=getME(fm1,"theta"),
               beta=fixef(fm1),
               sigma=sigma(fm1)),
               family=gaussian,
               seed=101)[[1]]
stopifnot(all.equal(s7,s1))

## TO DO: wider range of tests, including offsets ...

}# R >= 3.0.0
} ## testLevel>1

if (.Platform$OS.type != "windows") {
    library(lme4)
    library(testthat)

    ## [glmer(*, gaussian) warns to rather use lmer()]
    m3 <- suppressWarnings(glmer(Reaction ~ Days + (Days|Subject), sleepstudy))
    m4 <- lmer(Reaction ~ Days + (Days|Subject), sleepstudy)
    m5 <- suppressWarnings(glmer(Reaction ~ Days + (Days|Subject), sleepstudy,
                                 family=gaussian))
    expect_equal(fixef(m3),fixef(m5))
    ## hack call -- comes out unimportantly different
    m4@call[[1]] <- quote(lme4::lmer)
    expect_equal(m3,m4)
    expect_equal(m3,m5)

    ## would like m3==m5 != m4 ??
    expect_equal(VarCorr(m4), VarCorr(m5), tolerance = 1e-14)
    print(th4 <- getME(m4,"theta"))
    expect_equal(th4, getME(m5,"theta"), tolerance = 1e-14)

    ## glmer() - poly() + interaction
    if (requireNamespace("mlmRev")) withAutoprint({
        data(Contraception, package="mlmRev")
        ## ch := with child
        Contraception <- within(Contraception, ch <- livch != "0")
        ## gmC1 <- glmer(use ~ poly(age,2) + ch + age:ch + urban + (1|district),
        ##               Contraception, binomial)
### not a 'warning' per se {cannot suppressWarnings(.)}:
###    fixed-effect model matrix is rank deficient so dropping 1 column / coefficient
### also printed with print(): labeled as  "fit warnings"

        ## ==> from ../R/modular.R  chkRank.drop.cols()
        ## --> Use   control = glmerControl(check.rankX = "ignore+drop.cols"))
        ## because further investigation shows "the problem" is really already
        ##     in model.matrix():
        set.seed(101)
        dd <- data.frame(ch = c("Y","N")[1+rbinom(12, 1, 0.7)], age = rlnorm(12, 16))
        colnames(mm1 <- model.matrix( ~ poly(age,2) + ch + age:ch, dd))
        ## "(Int.)" "poly(age, 2)1" "poly(age, 2)2" "chY" "chN:age" "chY:age"      
        ## If we make the poly() columns to regular variables, can interact:
        d2 <- within(dd, { p2 <- poly(age,2); ageL <- p2[,1]; ageQ <- p2[,2]; rm(p2)})
        ## then, we can easily get what want
        (mm2 <- model.matrix( ~ ageL+ageQ + ch + ageL:ch, d2))
        ## actually even more compactly now ("drawback": 'ageQ' at end):
        (mm2. <- model.matrix( ~ ageL*ch + ageQ, d2))
        cn2 <- colnames(mm2)
        stopifnot(identical(mm2[,cn2], mm2.[,cn2]))
    })
} ## skip on windows (for speed)

if (.Platform$OS.type != "windows") {
    library(lme4)
    library(testthat)
    fm1 <- lmer(Reaction ~ Days + (Days|Subject), sleepstudy)
    expect_equal(colnames(model.frame(fm1)),c("Reaction","Days","Subject"))
    expect_equal(colnames(model.frame(fm1,fixed.only=TRUE)),c("Reaction","Days"))
    expect_equal(formula(fm1),Reaction ~ Days + (Days | Subject))
    expect_equal(formula(fm1,fixed.only=TRUE),Reaction ~ Days)

    ## ugly example: model frame with compound elements
    fm2 <- lmer(log(Reaction) ~ splines::ns(Days,3) +
                    + I(1+Days^3) + (Days|Subject), sleepstudy)
    expect_equal(names(model.frame(fm2)),
                 c("log(Reaction)", "splines::ns(Days, 3)",
                   "I(1 + Days^3)", "Days", "Subject"))
    expect_equal(names(model.frame(fm2,fixed.only=TRUE)),
                 c("log(Reaction)", "splines::ns(Days, 3)",
                   "I(1 + Days^3)"))

} ## skip on windows (for speed)

#context("residuals")
test_that("lmer", {
    C1 <- lmerControl(optimizer="nloptwrap",
                      optCtrl=list(xtol_abs=1e-6, ftol_abs=1e-6))
    fm1 <- lmer(Reaction ~ Days + (Days|Subject),sleepstudy,
                control=C1)
    fm2 <- lmer(Reaction ~ Days + (Days|Subject),sleepstudy,
                control=lmerControl(calc.derivs=FALSE,
                                    optimizer="nloptwrap",
                                    optCtrl=list(xtol_abs=1e-6, ftol_abs=1e-6)))
    expect_equal(resid(fm1), resid(fm2))
    expect_equal(range(resid(fm1)),              c(-101.17996, 132.54664), tolerance=1e-6)
    expect_equal(range(resid(fm1, scaled=TRUE)), c(-3.9536067, 5.1792598), tolerance=1e-6)
    expect_equal(resid(fm1,"response"),resid(fm1))
    expect_equal(resid(fm1,"response"),resid(fm1,type="working"))
    expect_equal(resid(fm1,"deviance"),resid(fm1,type="pearson"))
    expect_equal(resid(fm1),resid(fm1,type="pearson"))  ## because no weights given
    expect_error(residuals(fm1,"partial"),
                 "partial residuals are not implemented yet")
    sleepstudyNA <- sleepstudy
    na_ind <- c(10,50)
    sleepstudyNA[na_ind,"Days"] <- NA
    fm1NA <- update(fm1,data=sleepstudyNA)
    fm1NA_exclude <- update(fm1,data=sleepstudyNA,na.action="na.exclude")
    expect_equal(length(resid(fm1)),length(resid(fm1NA_exclude)))
    expect_true(all(is.na(resid(fm1NA_exclude)[na_ind])))
    expect_true(!any(is.na(resid(fm1NA_exclude)[-na_ind])))
})

test_that("glmer", {
    gm1 <- glmer(incidence/size ~ period + (1|herd), cbpp,
                 family=binomial, weights=size)
    gm2 <- update(gm1,control=glmerControl(calc.derivs=FALSE))
    gm1.old <- update(gm1,control=glmerControl(calc.derivs=FALSE,
                          use.last.params=TRUE))
    expect_equal(resid(gm1),resid(gm2))
    ## y, wtres, mu change ??
    ## FIX ME:: why does turning on derivative calculation make these tests fail???
    expect_equal(range(resid(gm1.old)), c(-3.197512,2.356677), tolerance=1e-6)
    expect_equal(range(resid(gm1)), c(-3.1975034,2.35668826), tolerance=1e-6)
    expect_equal(range(resid(gm1.old, "response")), c(-0.1946736,0.3184579), tolerance=1e-6)
    expect_equal(range(resid(gm1,"response")),c(-0.194674747774946, 0.318458889275477))
    expect_equal(range(resid(gm1.old, "pearson")),  c(-2.381643,2.879069),tolerance=1e-5)
    expect_equal(range(resid(gm1,"pearson")), c(-2.38163599828335, 2.87908806084918))
    expect_equal(range(resid(gm1.old, "working")),  c(-1.241733,5.410587),tolerance=1e-5)
    expect_equal(range(resid(gm1, "working")),    c(-1.24173431447365, 5.41064465283686))
    expect_equal(resid(gm1),resid(gm1,scaled=TRUE))  ## since sigma==1

    expect_error(resid(gm1,"partial"),
                 "partial residuals are not implemented yet")

    cbppNA <- cbpp
    na_ind <- c(10,50)
    cbppNA[na_ind,"period"] <- NA
    gm1NA <- update(gm1,data=cbppNA)
    gm1NA_exclude <- update(gm1,data=cbppNA,na.action="na.exclude")
    expect_equal(length(resid(gm1)),length(resid(gm1NA_exclude)))
    expect_true(all(is.na(resid(gm1NA_exclude)[na_ind])))
    expect_true(!any(is.na(resid(gm1NA_exclude)[-na_ind])))
})

test_that("floating-point issues -> NaN dev resids", {
    dat <- data.frame(x=1:5, n=100, id=1:5)
    res <- glmer(cbind(x,n-x) ~ 1 + (1 | id), data=dat, family=binomial,
                 control = glmerControl(check.conv.singular = "ignore"))
    r <- residuals(res, type="deviance")
    expect_equal(unname(r[3]), 0)
})

test_that("weighted residuals", {
    if (as.package_version(R.Version()) <= "4.5.2") skip()
    ss <- sleepstudy
    ## make sure napredict() is exercised
    ss$Reaction[1] <- NA_real_
    fm1 <- lmer(Reaction ~ 1 + (1|Subject), ss, na.action = na.exclude)
    expect_equal(head(weighted.residuals(fm1), 3),
                 structure(c(NA, -86.4020327538219, -94.3061327538219), names = c(NA, "2", "3")))

    gm1 <- glmer(round(Reaction) ~ 1 + (1|Subject), ss, na.action = na.exclude,
                 family = poisson)
    expect_equal(head(weighted.residuals(gm1), 3),
                 structure(c(NA, -4.9271857060919, -5.35397661950064), names = c(NA, "2", "3")))


})

(testLevel <- if (nzchar(s <- Sys.getenv("LME4_TEST_LEVEL"))) as.numeric(s) else 1)

## use old (<=3.5.2) sample() algorithm if necessary
if ("sample.kind" %in% names(formals(RNGkind)))
  suppressWarnings(RNGkind("Mersenne-Twister", "Inversion", "Rounding"))

L <- load(system.file("testdata", "lme-tst-fits.rda",
                      package="lme4", mustWork=TRUE))

nosingcheck_lmer <- lmerControl(check.conv.singular = "ignore")
nosingcheck_glmer <- glmerControl(check.conv.singular = "ignore")
## FIXME: should test for old R versions, skip reloading test data in that
## case?
fm0 <- fit_sleepstudy_0
fm1 <- fit_sleepstudy_1
fm2 <- fit_sleepstudy_2
gm1 <- fit_cbpp_1
gm2 <- fit_cbpp_2
gm3 <- fit_cbpp_3
## More objects to use in all contexts :
set.seed(101)
dNA <- data.frame(
    xfac= sample(letters[1:10], 100, replace=TRUE),
    xcov= runif(100),
    resp= rnorm(100))
dNA[sample(1:100, 10), "xcov"] <- NA

CI.boot <- function(fm, nsim=10, seed=101, ...) {
  suppressMessages(
    suppressWarnings(confint(fm, method="boot", nsim=nsim,
                             quiet=TRUE, seed=seed, ...))
  )
}

##
rSimple <- function(rep = 2, m.u = 2, kinds = c('fun', 'boring', 'meh')) {
  stopifnot(is.numeric(rep), rep >= 1,
            is.numeric(m.u), m.u >= 1,
            is.character(kinds), (nk <- length(kinds)) >= 1)
  nobs <- rep * m.u * nk
  data.frame(kind= rep(kinds, each=rep*m.u),
             unit = gl(m.u, 1, nobs), y = round(50*rnorm(nobs)))
}
d12 <- rSimple()

data("Pixel", package="nlme")
nPix <- nrow(Pixel)
fmPix <- lmer(pixel ~ day + I(day^2) + (day | Dog) + (1 | Side/Dog), data = Pixel)


test_that("summary", {
  ## test for multiple-correlation-warning bug and other 'correlation = *' variants
  ## Have 2 summary() versions, each with 3 print(.) ==> 6 x capture.output(.)
  sf.aa <- summary(fit_agridat_archbold)
  msg1 <- "Correlation.* not shown by default"
  ## message => *not* part of capture.*(.)
  expect_message(c1 <- capture.output(sf.aa), msg1)
                                        # correlation = NULL - default
  cF <- capture.output(print(sf.aa, correlation=FALSE))
  ## TODO? ensure the above gives *no* message/warning/error
  expect_identical(c1, cF)
  expect_message(
      cT <- capture.output(print(sf.aa, correlation=TRUE))
    , "Correlation.* could have been required in summary()")
  expect_identical(cF, cT[seq_along(cF)])
  sfT.aa <- summary(fit_agridat_archbold, correlation=TRUE)
  ## no message any more
  ## expect_message(cT2 <- capture.output(sfT.aa), msg1)
  ## expect_identical(cF, cT2)
  cT3 <- capture.output(print(sfT.aa, correlation=TRUE))
  expect_identical(cT, cT3)
  cF2 <- capture.output(print(sfT.aa, correlation=FALSE))
  expect_identical(cF, cF2)
})

test_that("lmer anova", {
  aa <- suppressMessages(anova(fm0,fm1))
  expect_that(aa, is_a("anova"))
  expect_equal(names(aa),
               c("npar", "AIC", "BIC", "logLik", "-2*log(L)", "Chisq", "Df",
                 "Pr(>Chisq)"))
  expect_warning(suppressMessages(do.call(anova,list(fm0,fm1))), "assigning generic names")
  ##
  dat <- data.frame(y = 1:5,
                    u = c(rep("A",2), rep("B",3)),
                    t = c(rep("A",3), rep("B",2)))
  datfun <- function(x) dat
  aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa <-  dat
  expect_is(stats::anova(lmer(y ~ u + (1 | t),
                              dat = aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa,
                              REML=FALSE),
                         lmer(y ~ 1 + (1 | t),
                              dat = aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa,
                              REML=FALSE)), "anova")
  expect_equal(rownames(stats::anova(lmer(y ~ u + (1 | t),
                                          dat = aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa,
                                          REML=FALSE),
                                     lmer(y ~ 1 + (1 | t),
                                          dat = aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa,
                                          REML=FALSE),
                                     model.names=c("a","b"))),
               c("b","a"))
  ff <- function(form) {
    lmer(form, dat=dat, REML=FALSE, control=nosingcheck_lmer)
  }
  expect_error(rownames(stats::anova(ff(y ~ u + (1 | t)),
                                     ff(y ~ 1 + (1 | t)),
                                     model.names=c("a","b","c"))),
               "different lengths")
  z <- 1
  ## output not tested (but shouldn't fail)
  ss <- stats::anova(lmer(y ~ u + (1 | t), data = datfun(z), REML=FALSE),
                     lmer(y ~ 1 + (1 | t), data = datfun(z), REML=FALSE))
  ##
  ## from Roger Mundry via Roman Lustrik
  full <- lmer(resp ~ xcov + (1|xfac), data=dNA)
  null <- lmer(resp ~  1   + (1|xfac), data=dNA)
  expect_error(anova(null,full),
               "models were not all fitted to the same size of dataset")
})

if (requireNamespace("merDeriv")) {
    test_that("summary with merDeriv", {
        library(merDeriv)
        cc <- capture.output(print(summary(fm1)))
        expect_true(any(grepl("Correlation of Fixed Effects", cc)))
        ## WARNING, this will detach package but may not undo
        ##  method loading ...
        detach("package:merDeriv")
    })
}


## Github issue #256  from Jonas Lindeløv -- issue is *not* specific for this dataset
test_that("Two models with subset() within lmer()", {
    full3 <- lmer(y ~ kind + (1|unit), subset(d12, kind != 'boring'), REML=FALSE)
    null3 <- update(full3, .~. - kind)
    op <- options(warn = 2) # no warnings!
    ano3 <- anova(full3, null3)## issue #256: had warning in data != data[[1]] : ...
    o3 <- capture.output(ano3) # now prints with only one 'Data:'
    expect_equal(1, grep("^Data:", o3))
    d12sub <- subset(d12, kind != 'boring')
    expect_is(full3s <- lmer(y ~ kind + (1|unit), d12sub, REML=FALSE), "lmerMod")
    expect_is(null3s <- update(full3s, .~. - kind), "lmerMod")
    expect_is(ano3s <- anova(full3s, null3s), "anova")
    expect_equal(ano3, ano3s, check.attributes=FALSE)
    options(op)
})



test_that("anova() of glmer+glm models", {
  dat <<- data.frame(y = 1:5,
                    u = c(rep("A",2), rep("B",3)),
                    t = c(rep("A",3), rep("B",2)))
  cs <- glmerControl(check.conv.singular = "ignore") ## ignore singular fits
  gm1 <- glmer(y~(1|u), data=dat[1:4,], family=poisson, control = cs)
  gm0 <- glm(y~1, data=dat[1:4,], family=poisson)
  gm2 <- glmer(y~(1|u), data=dat[1:4,], family=poisson,nAGQ=2, control = cs)
  aa <- anova(gm1,gm0)
  expect_equal(aa[2,"Chisq"],0)
  expect_error(anova(gm2,gm0),"incommensurate")
})

  test_that("anova() of lmer+glm models", {
    dat2 <- dat
    set.seed(101)
    dat2$y <- rnorm(5)
    fm1 <- lmer(y~(1|u),data=dat2,REML=FALSE)
    fm0 <- lm(y~1,data=dat2)
    aa2 <- anova(fm1,fm0)
    expect_equal(aa2[2,"Chisq"],0)
    expect_warning(anova(fm1,type="III"),"additional arguments ignored")
  })

  test_that("set p-values to NA for equivalent models: #583", {
    fm0B <- fm0
    aa <- suppressMessages(anova(fm0B,fm0))
    expect_true(all(is.na(aa[["Pr(>Chisq)"]])))
  })

  test_that("long names", {
    ## GH
    names(sleepstudy) <- c("Reaction", "Days", "Subject_xxxxxxxxxxxxxxxxxxxxxxxxxxx")
    fm1 <- lmer(Reaction ~ Days + (Days | Subject_xxxxxxxxxxxxxxxxxxxxxxxxxxx), sleepstudy)
    fm2 <- lmer(Reaction ~ Days + (Days || Subject_xxxxxxxxxxxxxxxxxxxxxxxxxxx), sleepstudy)
    expect_equal(length(attributes(suppressMessages(anova(fm1,fm2)))$heading),4)
  })

if (testLevel>1) {
  #context("bootMer confint()")
  set.seed(47)
  test_that("bootMer", {
    ## testing bug-fix for ordering of sd/cor components in sd/cor matrix with >2 rows
    dd <- expand.grid(A=factor(1:3), B=factor(1:10), rep=1:20)
    dd$y <- suppressMessages(simulate(~1 + (A|B),
                                      newdata=dd,
                                      newparams=list(beta=1,theta=rep(1,6),
                                                     sigma=1),
                                      family=gaussian,
                                      seed=101))[[1]]
    m1 <- lmer(y ~ 1 + (A|B), data=dd, control=lmerControl(calc.deriv=FALSE))
    ci <- CI.boot(m1, seed=101, signames = FALSE)
    ci2 <- CI.boot(m1, seed = 101, signames = FALSE)
    expect_equal(ci, ci2)
    ci_50 <- CI.boot(m1,level=0.5,seed=101)
    expect_true(all(ci_50[,"25 %"]>ci[,"2.5 %"]))
    expect_true(all(ci_50[,"75 %"]<ci[,"97.5 %"]))
    corvals <- ci[grep("^cor_", rownames(ci)),]
    expect_true(all(abs(corvals) <= 1))
    ## test bootMer with GLMM, multiple RE
    ci1 <- CI.boot(gm2, nsim=5, seed = 101)
    ci2  <- CI.boot(gm2, nsim=5, parm=3:6, seed = 101)
    ci2B <- CI.boot(gm2, nsim=5, parm="beta_", seed = 101)
    ## previously tested with nsim=5 vs nsim=3
    expect_true(nrow(ci2) == 4)
    expect_equal(ci2, ci2B)
    expect_equal(ci1[3:6,], ci2) ## , tolerance = 0.4)# 0.361
    ## bootMer with NA values
    ddNA <- dd
    ddNA$A[1:3] <- NA
    m2 <- update(m1, data=ddNA)
    ci3 <- CI.boot(m2, signames = FALSE)
    expect_equal(ci, ci3, tolerance=0.2)
    sleepstudyNA <- sleepstudy
    sleepstudyNA$Days[1:3] <- NA
    m4 <- update(fm2, data = sleepstudyNA,
                 control=lmerControl(check.conv.grad = .makeCC("warning",tol=4e-3)))
    expect_true(nrow(ci4 <- CI.boot(m4)) == 6) # could check more
    ##
    ## semipar bootstrapping
    m5 <- lmer(Yield ~ 1|Batch, Dyestuff)
    set.seed(1)
    suppressPackageStartupMessages(require(boot))
    boo01.sp <- bootMer(m5, fixef, nsim = 100, use.u = TRUE,
                        type = "semiparametric")
    expect_equal(sd(boo01.sp$t), 8.215586, tolerance = 1e-4)
    ## passing FUN to confint: Torbjørn Håkan Ergon
    testFun <- function(fit) fixef(fit)[1]
    expect_equal(c(unclass(
        suppressWarnings(confint(fm1, method="boot", FUN=testFun, nsim=10,
                                 quiet=TRUE)))),
        c(243.7551,256.9104),tolerance=1e-3)

    ## passing re.form to bootMer
    ## FUN <- function(.) predict(., type="response")
    fm3 <- lmer(strength ~ (1|batch/cask), Pastes)
    expect_is(bootMer(fm3, predict, nsim=3),"boot")
    expect_is(bootMer(fm3, predict, re.form=NULL, nsim=3),"boot")
    expect_is(bootMer(fm3, predict, re.form=~(1|batch)+(1|cask:batch), nsim=3),
              "boot")
    expect_is(b3 <- bootMer(fm3, predict, re.form=~(1|batch), nsim=3),
              "boot")

    FUN_name <- function(.) getME(.,"theta")
    FUN_noname <- function(.) unname(getME(.,"theta"))

    c_name <- suppressWarnings(
        confint(fm3, method="boot", FUN=FUN_name, nsim=3, seed=101))

    c_noname <- suppressWarnings(
        confint(fm3, method="boot", FUN=FUN_noname, nsim=3, seed=101))

    expect_equal(unname(c_name),unname(c_noname))

    ## example from @Mark
    ## bootstrapping etc. on GLMMs with scale
    ## SO 37466771:
    ## https://stackoverflow.com/questions/37466771/using-profile-and-boot-method-within-confint-option-with-glmer-model
    df2 <- data.frame(
        prop1 = c(0.46, 0.471, 0.458, 0.764, 0.742, 0.746,
                  0.569, 0.45,    0.491,    0.467, 0.464,
                  0.556, 0.584, 0.478, 0.456, 0.46, 0.493, 0.704, 0.693, 0.651),
        prop2 = c(0.458, 0.438, 0.453, 0.731, 0.738, 0.722, 0.613,
                  0.498, 0.452, 0.451, 0.447,
                  0.48, 0.576, 0.484, 0.473, 0.467, 0.467, 0.722, 0.707, 0.709),
        site = factor(rep(1:8,c(2,1,3,5,3,2,1,3))))

    gaussmodel <- glmer(prop2 ~ prop1 + (1|site),
                        data=df2, family=gaussian(link="logit"))

    ## parametric bootstrap:
    ##  terrible, but not a computational bug I think?
    boot_res <- matrix(
      c(
        0.7876999074589053, 0.03674961560266387, -4.210250900535643,
        3.1483596407466985, 1.7682636983854518, 0.05647611260302035,
        -1.0059308984192392, 4.706443226891899
      ), nrow = 4L, ncol = 2L,
      dimnames = list(c(".sig01", ".sigma", "(Intercept)", "prop1"),
                      c("2.5 %", "97.5 %"))
    )

    set.seed(101)
    bci <- suppressMessages(suppressWarnings(confint(gaussmodel,method="boot",nsim=10,quiet=TRUE)))
    expect_equal(bci, boot_res, tolerance=1e-5)

  })
} ## testLevel>1

test_that("change in deviance name for anova", {
    cc <- suppressMessages(capture.output(anova(fm0,fm1)))
    expect_identical(sum(grepl("deviance", cc)), 0L)
    expect_identical(sum(grepl("-2*log(L)", cc, fixed = TRUE)), 1L)
})

test_that("confint", {
  load(system.file("testdata", "gotway_hessianfly.rda", package = "lme4"))
  ## generated via:
  ## gotway_hessianfly_fit <- glmer(cbind(y, n-y) ~ gen + (1|block),
  ##              data=gotway.hessianfly, family=binomial,
  ##              control=glmerControl(check.nlev.gtreq.5="ignore"))
  ## gotway_hessianfly_prof <- profile(gotway_hessianfly_fit,which=1)
  ## save(list=ls(pattern="gotway"),file="gotway_hessianfly.rda")
  expect_equal(confint(gotway_hessianfly_prof)[1,1],0)
  ## FIXME: should add tests for {-1,1} bounds on correlations as well
  expect_equal(c(confint(fm1,method="Wald",parm="beta_")),
               c(232.301892,8.891041,270.508318,12.043531),
               tolerance=1e-5)
  ## Wald gives NA for theta values
  expect_true(all(is.na(confint(fm1,method="Wald",parm="theta_"))))

  ## check names
  ci1.p <- suppressWarnings(confint(fm1,quiet=TRUE))
  ci1.w <- confint(fm1,method="Wald")
  ci1.b <- CI.boot(fm1, nsim=2)
  expect_equal(dimnames(ci1.p),
               list(c(".sig01", ".sigma", "(Intercept)", "Days"),
                    c("2.5 %", "97.5 %")))
  expect_equal(dimnames(ci1.p),dimnames(ci1.w))
  expect_equal(dimnames(ci1.p),dimnames(ci1.b))
  ci1.p.n <- suppressWarnings(confint(fm1, quiet=TRUE, signames=FALSE))
  ci1.w.n <- confint(fm1, method="Wald", signames=FALSE)
  ci1.b.n <- CI.boot(fm1, nsim=2, signames=FALSE)
  expect_equal(dimnames(ci1.p.n),
               list(c("sd_(Intercept)|Subject", "sigma", "(Intercept)", "Days"),
                    c("2.5 %", "97.5 %")))
  expect_equal(dimnames(ci1.p.n),dimnames(ci1.w.n))
  expect_equal(dimnames(ci1.p.n),dimnames(ci1.b.n))
})

test_that("monotonic profile but bad spline",
{
  ## doesn't produce warnings on Solaris, or win-builder, or M1mac ...
  skip_on_os("windows")
  skip_on_os("solaris")
  skip_on_os("mac", arch = "aarch64")
  ## test case of slightly wonky (spline fit fails) but monotonic profiles:
  ##
  simfun <- function(J,n_j,g00,g10,g01,g11,sig2_0,sig01,sig2_1){
    N <- sum(rep(n_j,J))
    x <- rnorm(N)
    z <- rnorm(J)
    mu <- c(0,0)
    sig <- matrix(c(sig2_0,sig01,sig01,sig2_1),ncol=2)
    u   <- MASS::mvrnorm(J,mu=mu,Sigma=sig)
    b_0j <- g00 + g01*z + u[,1]
    b_1j <- g10 + g11*z + u[,2]
    y <- rep(b_0j,each=n_j)+rep(b_1j,each=n_j)*x + rnorm(N,0,sqrt(0.5))
    sim_data <- data.frame(Y=y,X=x,Z=rep(z,each=n_j),
                           group=rep(1:J,each=n_j))
  }
  set.seed(102)
  dat <- simfun(10,5,1,.3,.3,.3,(1/18),0,(1/18))
  fit <- lmer(Y~X+Z+X:Z+(X||group),data=dat,
              control = lmerControl(check.conv.singular = "ignore"))
  expect_warning(pp <- profile(fit,"theta_"), "non-monotonic profile")
  expect_warning(cc <- confint(pp),"falling back to linear interpolation")
  ## very small/unstable problem, needs large tolerance
  expect_equal(unname(cc[2,]), c(0, 0.509), tolerance=0.09) # "bobyqa" had 0.54276
})

test_that("confint with bad profile", {
  badprof <- readRDS(system.file("testdata","badprof.rds",
                                 package="lme4"))
  expect_warning(cc <- confint(badprof), "falling back to linear")
  expect_equal(cc,
               array(c(0, -1, 2.50856219044636, 48.8305727797906, NA, NA,
                       33.1204478717389, 1, 7.33374326592662, 68.7254711217912, -6.90462047196017, NA),
                     dim = c(6L, 2L),
                     dimnames = list(c(".sig01", ".sig02", ".sig03", ".sigma", "(Intercept)", "cYear"),
                                     c("2.5 %", "97.5 %"))),
               tolerance=1e-3)
})

test_that("refit", {
  s1 <- simulate(fm1)
  expect_is(refit(fm1,s1), "merMod")
  s2 <- simulate(fm1,2)
  expect_error(refit(fm1,s2), "refit not implemented .* lists")
  data(Orthodont,package = "nlme")
  fmOrth <- fm <- lmer(distance ~ I(age - 11) + (I(age - 11) | Subject),
                       data = Orthodont)
  expect_equal(s1 <- simulate(fm,newdata = Orthodont,seed = 101),
               s2 <- simulate(fm,seed = 101))


  ## works *without* offset ...
  m5 <- glmer(round(Reaction) ~ Days + (1|Subject),
              data = sleepstudy, family=poisson,
              offset=rep(0,nrow(sleepstudy)))
  m5R <- refit(m5)
  ## lots of fussy details make expect_equal() on the whole object difficult
  expect_equal(coef(m5),coef(m5R),tolerance=3e-6)
  expect_equal(VarCorr(m5),VarCorr(m5R),tolerance=1e-6)
  expect_equal(logLik(m5),logLik(m5R))
})

if (testLevel>1) {
  #context("predict method")
  test_that("predict", {
    ## when running via source(), cbpp has been corrupted at this point
    ## (replaced by a single empty factor obs()
    d1 <- expand.grid(period = unique(cbpp$period), herd = unique(cbpp$herd))
    d2 <- data.frame(period = "1", herd = unique(cbpp$herd))
    d3 <- expand.grid(period = as.character(1:3),
                      herd = unique(cbpp$herd))
    p0 <- predict(gm1)
    p1 <- predict(gm1,d1)
    p2 <- predict(gm1,d2)
    p3 <- predict(gm1,d3)
    expect_equal(p0[1], p1[1])
    expect_equal(p0[1], p2[1])
    expect_equal(p0[1], p3[1])
    expect_error(predict(gm1, ReForm=NA))
    ## matrix-valued predictors: Github #201 from Fabian S.
    sleepstudy$X <- cbind(1, sleepstudy$Days)
    m <- lmer(Reaction ~ -1 + X  + (Days | Subject), sleepstudy)
    pm <- predict(m, newdata=sleepstudy)
    expect_is(pm, "numeric")
    expect_equal(quantile(pm, names = FALSE),
                 c(211.0108, 260.9496, 296.873, 328.6378, 458.1584), tol=1e-5)
    op <- options(warn = 2) # there should be no warnings!
    if (require("MEMSS",quietly=TRUE)) {
      ## test spurious warning with factor as response variable
      data("Orthodont", package = "MEMSS") # (differently "coded" from the 'default' "nlme" one)
      silly <- glmer(Sex ~ distance + (1|Subject),
                     data = Orthodont, family = binomial)
      sillypred <- data.frame(distance = c(20, 25))
      ps <- predict(silly, sillypred, re.form=NA, type = "response")
      expect_is(ps, "numeric")
      expect_equal(unname(ps), c(0.999989632, 0.999997201), tolerance=1e-6)
      detach("package:MEMSS")
    }
    ## a case with interactions (failed in one temporary version):
    expect_warning(fmPixS <<- update(fmPix, .~. + Side),
                   "nearly unidentifiable|unable to evaluate scaled gradient|failed to converge")
    ## (1|2|3); 2 and 3 seen (as Error??) on CRAN's Windows 32bit
    options(op)

    set.seed(1); ii <- sample(nrow(Pixel), 16)
    expect_equal(predict(fmPix,  newdata = Pixel[ii,]), fitted(fmPix )[ii])
    expect_equal(predict(fmPixS, newdata = Pixel[ii,]), fitted(fmPixS)[ii])

    set.seed(7); n <- 100; y <- rnorm(n)
    dd <- data.frame(id = factor(sample(10, n, replace = TRUE)),
                     x1 = 1, y = y, x2 = rnorm(n, mean = sign(y)))
    expect_message(m <- lmer(y ~ x1 + x2 + (1 | id), data = dd),
                   "fixed-effect model matrix is rank deficient")
    expect_is(summary(m),"summary.merMod")
    ii <- sample(n, 16)
    expect_equal(predict(m, newdata = dd[ii,]), fitted(m)[ii])
    ## predict(*, new..) gave Error in X %*% fixef(object) - now also drops col.

    ## predict(*, new..) with NA in data {and non-simple model}, issue #246:
    m1 <- lmer(Reaction ~ Days + (Days|Subject), sleepstudy)
    sleepst.NA <- sleepstudy ; sleepst.NA$Days[2] <- NA
    m2 <- update(fm1, data = sleepst.NA)
    ## maybe tricky for evaluation; fm1 was defined elsewhere, so data
    expect_equal(length(predict(m2, sleepst.NA[1:4,])),4)

    ## Wrong 'b' constructed in mkNewReTrms() -- issue #257
    data(Orthodont,package="nlme")
    Orthodont <- within(Orthodont, nsex <- as.numeric(Sex == "Male"))
    m3 <- lmer(distance ~ age + (age|Subject) + (0 + Sex |Subject),
               data=Orthodont,
               control=lmerControl(check.conv.hess="ignore",
                                   check.conv.grad="ignore"))
    m4 <- lmer(distance ~ age + (age|Subject) + (0 + nsex|Subject), data=Orthodont)
    expect_equal(p3 <- predict(m3, Orthodont), fitted(m3), tolerance=1e-14)
    expect_equal(p4 <- predict(m4, Orthodont), fitted(m4), tolerance=1e-14)

    ## related to GH #275 (*passes*),
    ss <- sleepstudy
    set.seed(1)
    ss$noiseChar <- ifelse(runif(nrow(sleepstudy)) > 0.8, "Yes", "No")
    ss$noiseFactor <- factor(ss$noiseChar)
    fm4 <- lmer(Reaction ~ Days + noiseChar + (Days | Subject), ss)
    expect_equal(predict(fm4, newdata = model.frame(fm4)[2:3, ])[2],
                 predict(fm4, newdata = model.frame(fm4)[3, ]))
    fm3 <- lmer(Reaction ~ Days + noiseFactor + (Days | Subject), ss)
    expect_equal(predict(fm3, newdata = model.frame(fm3)[2:3, ])[2],
                 predict(fm3, newdata = model.frame(fm3)[3, ]))

    ## complex-basis functions in RANDOM effect
    fm5 <- lmer(Reaction~Days+(poly(Days,2)|Subject),sleepstudy)
    expect_equal(predict(fm5,sleepstudy[1,]),fitted(fm5)[1])
    ## complex-basis functions in FIXED effect
    fm6 <- lmer(Reaction~poly(Days,2)+(1|Subject),sleepstudy)
    expect_equal(predict(fm6,sleepstudy[1,]),fitted(fm6)[1])

    ## GH #414: no warning about dropping contrasts on random effects
    op <- options(warn = 2) # there should be no warnings!
    set.seed(1)
    dat <- data.frame(
        fac = factor(rep(c("a", "b"), 100)),
        grp = rep(1:25, each = 4))
    dat$y <- 0
    contr <- 0.5 * contr.sum(2)
    rownames(contr) <- c("a", "b")
    colnames(contr) <- "a"
    contrasts(dat$fac) <- contr
    m1_contr <- lmer(y~fac+(fac|grp),dat)
    pp <- predict(m1_contr,newdata=dat)
    options(op)

  })

  ## testLevel>1
  test_that("simulate", {
    ## simulate() will look for data in environment of formula, find
    ##   unmodified version of cbpp -- need to re-add observation-level factor
    ee <- environment(formula(gm2))
    ee$cbpp$obs <- factor(seq(nrow(ee$cbpp)))
    expect_is(simulate(gm2), "data.frame")
    p1 <- simulate(gm2, re.form = NULL, seed = 101)
    p2 <- simulate(gm2, re.form = ~0, seed = 101)
    p3 <- simulate(gm2, re.form = NA, seed = 101)
    p4 <- simulate(gm2, re.form = NULL, seed = 101)
    ## p5 was: sim with ReForm
    p6 <- simulate(gm2, re.form = NA, seed = 101)
    ## p7 was: sim with ReForm
    p8 <- simulate(gm2, re.form = ~0, seed = 101)
    p9 <- simulate(gm2, re.form = NA, seed = 101)
    p10 <- simulate(gm2,use.u = FALSE, seed = 101)
    p11 <- simulate(gm2,use.u = TRUE, seed = 101)
    ## minimal check of content:
    expect_identical(colSums(p1[,1]), c(incidence =  95, 747))
    expect_identical(colSums(p2[,1]), c(incidence = 109, 733))
    ## equivalences:
    ## group ~0:
    expect_equal(p2,p3)
    expect_equal(p2,p6)
    expect_equal(p2,p8)
    expect_equal(p2,p9)
    expect_equal(p2,p10)
    ## group 1:
    expect_equal(p1,p4)
    expect_equal(p1,p11)
    expect_error(simulate(gm2,use.u = TRUE, re.form = NA), "should specify only one")
    ##
    ## hack: test with three REs
    p1 <- lmer(diameter ~ (1|plate) + (1|plate) + (1|sample), Penicillin,
               control = lmerControl(check.conv.hess = "ignore",
                                     check.conv.grad = "ignore"))
    expect_is(sp1 <- simulate(p1, seed=123), "data.frame")
    expect_identical(dim(sp1), c(nrow(Penicillin), 1L))
    expect_equal(fivenum(sp1[,1]),
                 c(20.864, 22.587, 23.616, 24.756, 28.599), tolerance=0.01)
    ## Pixel example

    expect_identical(dim(simulate(fmPixS)), c(nPix, 1L))
    expect_identical(dim(simulate(fmPix )), c(nPix, 1L))

    ## simulation with newdata smaller/larger different from original
    fm <- lmer(diameter ~ 1 + (1|plate) + (1|sample), Penicillin)
    expect_is(simulate(fm,newdata=Penicillin[1:10,],allow.new.levels=TRUE),"data.frame")
    expect_is(simulate(fm,newdata=do.call(rbind,replicate(4,Penicillin,simplify=FALSE))),"data.frame")

    ## negative binomial sims
    set.seed(101)
    dd <- data.frame(f=factor(rep(1:10,each=20)),
                     x=runif(200),
                     y=rnbinom(200,size=2,mu=2))
    g1 <- glmer.nb(y ~ x + (1|f), data=dd)
    th.g1 <- getME(g1, "glmer.nb.theta")
    ## changed to setting seed internally
    ts1 <- table(s1 <- simulate(g1,seed=101)[,1])
    ## ts1B <- table(s1 <- simulate(g1,seed=101)[,1])
    expect_equal(fixef(g1),
                 c("(Intercept)" = 0.630067, x = -0.0167248),
                 tolerance = 1e-4)
    ## ?? Travis is getting hung up here/ignoring tolerance spec??
    expect_equal(th.g1, 2.013, tolerance = 1e-4)
    expect_equal(th.g1, g1@call$family[["theta"]])# <- important for pkg{effects} eval(<call>)
    expect_identical(sum(s1), 413)
    expect_identical(as.vector(ts1[as.character(0:5)]),
                     ##                     c(51L, 54L, 36L, 21L, 14L, 9L))
                     c(49L,56L,32L,25L,11L,9L))

    ## de novo NB simulation ...
    s2 <- simulate(~x + (1|f),seed=101,
                   family=MASS::negative.binomial(theta=th.g1),
                   newparams=getME(g1,c("theta","beta")),
                   newdata=dd)[,1]
    expect_equal(s1,s2)

    ## Simulate with newdata with *new* RE levels:
    d <- sleepstudy[-1] # droping the response ("Reaction")
    ## d$Subject <- factor(rep(1:18, each=10))
    ## Add 18 new subjects:
    d <- rbind(d, d)
    d$Subject <- factor(rep(1:36, each=10))
    d$simulated <- simulate(fm1, seed=1, newdata = d,
                            re.form=NULL,
                            allow.new.levels = TRUE)[,1]
    expect_equal(mean(d$simulated), 299.9384608)

    ## Simulate with weights:

    newdata <- with(cbpp, expand.grid(period=unique(period),
                                      herd=unique(herd)))
    ss <- simulate(gm1, newdata=newdata[1:3,],
                   weights=20, seed=101)[[1]]
    expect_equal(ss,
                 matrix(c(4,2,0,16,18,20),nrow=3,
                        dimnames=list(NULL,c("incidence",""))))
    ss <- simulate(gm3, newdata=newdata[1:3,],
                   weights=20, seed=101)[[1]]
    expect_equal(ss,c(0.2,0.1,0.0))
    ss <- simulate(gm1, newdata=newdata[1,],
                   weights=20, seed=101)[[1]]
    expect_equal(unname(ss),matrix(c(4,16),nrow=1))

    ## simulate Gamma, from function and de novo
    set.seed(102)
    dd <- data.frame(x=rep(seq(-2,2,length=15),10),
                     f=factor(rep(1:10,each=15)))
    u <- rnorm(10)
    dd$y <- with(dd,
                 rgamma(nrow(dd),shape=2,
                        scale=exp(2+1*x+u[as.numeric(f)])/2))
    g1 <- glmer(y~x+(1|f),family=Gamma(link="log"),dd)
    s1 <- simulate(g1,seed=101)
    s2 <- suppressMessages(simulate(~x+(1|f), family=Gamma(link="log"),
                                    seed=101,
                                    newdata=dd,
                                    newparams=getME(g1,c("theta","beta","sigma"))))
    expect_equal(s1, s2)
    dd$y2 <- s2[[1]]
    g2 <- glmer(y2~x+(1|f), family=Gamma(link="log"),dd)
    expect_equal(fixef(g2), tolerance = 4e-7, # 32-bit windows showed 1.34e-7
                 c(`(Intercept)` = 2.90871404438183, x = 0.988265230798941))
##                 c("(Intercept)" = 2.81887136759369, x= 1.06543222163626))

    ## simulate with re.form = NULL and derived/offset components in formula
    fm7 <- lmer(Reaction ~ Days + offset(Days) + (1|Subject), sleepstudy)
    s7 <- simulate(fm7, seed = 101, re.form = NULL)
    ## thought this would break but it doesn't ???
    f_wrap <- function() { Reaction ~ Days + offset(Days) + (1|Subject) }
    fm8 <- lmer(f_wrap(), sleepstudy)
    s8 <- simulate(fm8, seed = 101, re.form = NULL)
    expect_identical(s7, s8)

    ## harder: insert NA values in the offset and see if it handles this OK??

  })

  #context("misc")
  test_that("misc", {
    expect_equal(df.residual(fm1),176)
    if (suppressWarnings(require(ggplot2))) {
      ## ggplot calls sample() [for silly start-up messages
      ## throws warning because we're using backward-compatible RNGkind
      expect_is(fortify.merMod(fm1), "data.frame")
      expect_is(fortify.merMod(gm1), "data.frame")
    }
    expect_is(as.data.frame(VarCorr(fm1)), "data.frame")
  })
} ## testLevel>1

#context("plot")
test_that("plot", {
  ## test getData() within plot function: reported by Dieter Menne
  doFit <- function(){
    data(Orthodont,package = "nlme")
    data1 <- Orthodont
    lmer(distance ~ age + (age|Subject), data = data1)
  }
  data(Orthodont, package = "nlme")
  fm0 <- lmer(distance ~ age + (age|Subject), data = Orthodont)
  expect_is(plot(fm0), "trellis")
  suppressWarnings(rm("Orthodont"))
  fm <- doFit()
  pp <- plot(fm, resid(., scaled = TRUE) ~ fitted(.) | Sex, abline = 0)
  expect_is(pp, "trellis")

  ## test qqmath/getIDLabels()
  expect_is(q1 <- lattice::qqmath(fm,id=0.05),"trellis")

  cake2 <- transform(cake,replicate=as.numeric(replicate),
                     recipe=as.numeric(recipe))
  fm2 <- lmer(angle ~ recipe + temp        +
                (1|recipe:replicate), cake2, REML= FALSE)
  expect_is(lattice::qqmath(fm2, id=0.05), "trellis")
  expect_is(lattice::qqmath(fm2, id=0.05, idLabels=~recipe), "trellis")
  expect_warning(lattice::qqmath(fm2, 0.05, ~recipe), "please specify")
  expect_warning(lattice::qqmath(fm2, 0.05),          "please specify")
})

#context("misc")
test_that("summary", {
  ## test that family() works when $family element is weird
  ## FIXME: is convergence warning here a false positive?
  gnb <- suppressWarnings(glmer(TICKS~1+(1|BROOD),
                                family=MASS::negative.binomial(theta=2),
                                data=grouseticks))
  expect_is(family(gnb),"family")
})


if (testLevel>1) {
  #context("profile")
  test_that("profile", {
    ## FIXME: can we deal with convergence warning messages here ... ?
    ## fit profile on default sd/cor scale ...
    p1 <- suppressWarnings(profile(fm1,which="theta_"))
    ## and now on var/cov scale ...
    p2 <- suppressWarnings(profile(fm1,which="theta_",
                                   prof.scale="varcov"))
    ## because there are no correlations, squaring the sd results
    ## gives the same result as profiling on the variance scale
    ## in the first place
    expect_equal(confint(p1)^2,confint(p2),
                 tolerance=1e-5)
    ## or via built-in varianceProf() function
    expect_equal(unname(confint(varianceProf(p1))),
                 unname(confint(p2)),
                 tolerance=1e-5)
    s_inds <- c(1,2,4) ## variance terms
    p3 <- profile(fm2, which=s_inds)
    p4 <- suppressWarnings(profile(fm2,
                                   which = "theta_",
                                   prof.scale="varcov",
                                   signames=FALSE))
    ## compare only for sd/var components, not corr component
    ## FAILS on r-patched-solaris-x86 2018-03-30 ???
    ##    2/6 mismatches (average diff: 4.62)
    ##    [1] 207 - 216 == -9.23697
    ##    [4] 1422 - 1422 == -0.00301

    if (Sys.info()["sysname"] != "SunOS") {
      expect_equal(unname(confint(p3)^2),
                   unname(confint(p4)[s_inds,]),
                   tolerance=5e-3)
    }

    ## check naming convention properly adjusted
    expect_equal(as.character(unique(p4$.par)),
                 c("var_(Intercept)|Subject", 
                   "var_Days|Subject",
                   "cov_Days.(Intercept)|Subject", "sigma"))
  })

  test_that("profile name/value order consistency",
             ## can use parallel comp since testLevel > 1, won't
             ##  be tried on CRAN
             ## use snow instead of multicore for Windows safety
  {
    ## GitHub actions complains about >2 cores ...
    pp <- profile(fm2, parallel = "snow", ncpus = 2,
                  signames = FALSE)
    ci_pp <- confint(pp)
    expect_all_true(abs(ci_pp["cor_Days.(Intercept)|Subject",]) < 1)

    ci_bb <- suppressWarnings(
      suppressMessages(
        confint(fm2, method = "boot", nsim = 10, signames = FALSE)
      ))
    expect_all_true(abs(ci_bb["cor_Days.(Intercept)|Subject",]) <= 1)
  })
    
  test_that("densityplot is robust", {
    p <- readRDS(system.file("testdata","harmel_profile.rds",
                             package="lme4"))
    expect_warning(lattice::densityplot(p),
                   "unreliable profiles for some variables")
  })

  

} ## testLevel>1

#context("model.frame")
test_that("model.frame", {
  ## non-syntactic names
  d <- sleepstudy
  names(d)[1] <- "Reaction Time"
  ee <- function(m,nm) {
    expect_equal(names(model.frame(m, fixed.only=TRUE)),nm)
  }
  m <- lmer(Reaction ~ 1 + (1 | Subject), sleepstudy)
  ee(m,"Reaction")
  m2 <- lmer(Reaction ~ Days + (1 | Subject), sleepstudy)
  ee(m2,c("Reaction","Days"))
  m3 <- lmer(`Reaction Time` ~ Days + (1 | Subject), d)
  ee(m3, c("Reaction Time","Days"))
  m4 <- lmer(Reaction ~ log(1+Days) + (1 | Subject), sleepstudy)
  ee(m4, c("Reaction","log(1 + Days)"))
})


#context("influence measures")

d <- as.data.frame(ChickWeight)
colnames(d) <- c("y", "x", "subj", "tx")
dNAs <- d
dNAs$y[c(1, 3, 5)] <- NA
fitNAs <- lmer(y ~ tx*x + (x | subj), data = dNAs,
               na.action=na.exclude)

test_that("influence/hatvalues works", {
  ifm1 <- influence(fm1, do.coef=FALSE)
  expect_equal(unname(head(ifm1$hat)),
               c(0.107483311203734, 0.102096105816528,
                 0.0980557017761242, 0.0953620990825215,
                 0.0940152977357202, 0.0940152977357202),
               tolerance=1e-6)
  expect_equal(nrow(dNAs),length(hatvalues(fitNAs)))
})

test_that("influence OK with tibbles", {
  if (requireNamespace("tibble")) {
    ## make small data set/example so influence() isn't too slow ...
    ss <- tibble::as_tibble(sleepstudy[1:60,])
    smallfit <- lmer(Reaction ~ 1 + (1 | Subject),
                     data = ss)
    i1 <- influence(smallfit, ncpus = 1)
    expect_equal(head(i1[["fixed.effects[-case]"]]),
                 structure(c(286.35044481665, 286.179896199062,
                             286.327301507498, 285.014692121823,
                             284.36060419176, 283.297551183126),
                           dim = c(6L, 1L),
                           dimnames = list(c("1", "2", "3", "4", "5", "6"),
                                           "(Intercept)")),
                 tolerance = 1e-6)
  }
})

test_that("rstudent", {
  rfm1 <- rstudent(fm1)
  expect_equal(unname(head(rfm1)),
               c(-1.45598270922089, -1.49664543508657, -2.11747425025103,
                 -0.0729690066951975, 0.772716397142335, 2.37859408861768),
               tolerance=1e-6)
  expect_equal(nrow(dNAs),length(rstudent(fitNAs)))
})

test_that("cooks distance", {
  expect_equal(
      unname(head(cooks.distance(fm1))),
      c(0.127645976734753, 0.127346548123793, 0.243724627125036, 0.000280638917214881,
        0.0309804642689636, 0.293554225380831),
      tolerance=1e-6)
  expect_equal(nrow(dNAs),length(cooks.distance(fitNAs)))
})

test_that("cooks distance on subject-level influence", {
  ifm1S <- influence(fm1, "Subject", ncpus=1)
  expect_equal(
      unname(head(cooks.distance(ifm1S),2)),
      c(0.33921460279262, 0.290309061006305),
      tolerance = 1e-6)
})

test_that("cooks distance on glmer models", {
  inf <- influence(gm1)
  inf.h <- influence(gm1, "herd", ncpus=1)
  cook <- cooks.distance(inf)
  expect_equal(unname(head(cook, 3)),
               c(0.0532998800033037, 0.0405931172763581, 0.252608337928438),
               tolerance = 1e-6)
  cook.h <- cooks.distance(inf.h)
  expect_equal(unname(head(cook.h, 3)),
               c(0.256630560723611, 0.00525856231971531, 0.103355658099396),
               tolerance = 1e-6)
})

## tweaked example so estimated var = 0
zerodat <- data.frame(x=seq(0,1,length.out=120),
                      f=rep(1:3,each=40))
zerodat$y1 <- simulate(~x+(1|f),
                       family=gaussian,
                       seed=102,
                       newparams=list(beta=c(1,1),
                                      theta=c(0.001),
                                      sigma=1),
                       newdata=zerodat)[[1]]
zerodat$y2 <- simulate(~x+(1|f),
                       family=poisson,
                       seed=102,
                       newparams=list(beta=c(1,1),
                                      theta=c(0.001)),
                       newdata=zerodat)[[1]]

test_that("rstudent matches for zero-var cases",
{

  skip_if(getRversion() < "4.6.0") ## methods have changed to match r-devel
  lmer_zero <- lmer(y1~x+(1|f), data=zerodat, control = nosingcheck_lmer)
  glmer_zero <- glmer(y2~x+(1|f),family=poisson, data=zerodat, control = nosingcheck_glmer)
  lm_zero <- lm(y1~x, data=zerodat)
  glm_zero <- glm(y2~x,family=poisson, data=zerodat)
  ## suppressWarnings() for "hatvalues may not be well defined" warning
  expect_equal(suppressWarnings(rstudent(glmer_zero)),
               rstudent(glm_zero),
               tolerance=0.01)
  expect_equal(suppressWarnings(rstudent(lmer_zero)),
               rstudent(lm_zero),tolerance=0.01)
  expect_equal(suppressWarnings(rstandard(glmer_zero)), rstandard(glm_zero),
          tolerance = 0.01)

})

if (testLevel>1) {
  ## n.b. influence() doesn't work under system.time();
  ##  weird evaluation stuff ?
  ## FIXME: work on timing some more
  i1 <- influence(fm1, ncpus=1)
  test_that("full version of influence", {
    expect_equal(c(head(i1[["fixed.effects[-case]"]],1)),
                 c(252.323536264131, 10.3222704729148))
  })
  cd <- cooks.distance(i1)
  expect_equal(unname(head(cd,2)),
               c(0.016503344184025, 0.0106634053477361))
  if (max(parallel::detectCores(), 1L, na.rm = TRUE) > 1) {
    test_that("parallel influence", {
        # ncores = 2 (deprecated)
        expect_warning(
            i2 <- influence(fm1, ncores=2),
            "argument 'ncores' is deprecated"
        )
        i3 <- influence(fm1, parallel = "snow", ncpus = 2)
        expect_equal(i1, i2)
        expect_equal(i1, i3)

        if (.Platform$OS.type != "windows") {
            i4 <- influence(fm1, parallel = "multicore", ncpus = 2)
            expect_equal(i1, i4)
        }

        # provide cl
        cl <- parallel::makeCluster(2)
        on.exit(parallel::stopCluster(cl))
        parallel::clusterEvalQ(cl, library(lme4))
        i5 <- influence(fm1, parallel = "snow", ncpus = length(cl), cl = cl)
        expect_equal(i1, i5)
    })
  }
}

## car method testing: influence timing with ncores > 1 ...
##  car version 3.0.10.
## L <- load(system.file("testdata", "lme-tst-fits.rda",
##                       package="lme4", mustWork=TRUE))
## data("sleepstudy", package="lme4")
## library(lme4)
## library(car) ## WANT warning about S3 method overwrite ...
## fm1 <- fit_sleepstudy_1
## library(pracma)  ## because system.time() is weird
## tic(); i1 <- influence(fm1); toc()  ## 2+ seconds
## tic(); i2 <- influence(fm1, ncores=8); toc() ## 3.4 seconds


test_that("influence with nAGQ=0", {
  gm1Q0 <- update(gm1, nAGQ=0)
  expect_is(influence(gm1Q0), "influence.merMod")
})

if (testLevel > 1) withAutoprint({
  test_that("cook's distance comparison", {
  ## generate data with zero variance
  set.seed(101)
  n <- 50
  dd <- data.frame(x = rnorm(n),
                   f = factor(rep(1:2, each = n/2)))
  suppressMessages(dd$y <- simulate(~ x + (1|f),
                                  newdata = dd,
                                  newparams = list(beta = c(2,2),
                                                   theta = 0,
                                                   sigma = 5),
                                  family = gaussian)[[1]])
  (fm2  <- lmer(y~x + (1|f), dd, REML = FALSE))
  (fm2L <- lm(y~x , dd))
  i2 <- influence(fm2)
  ## hatvalues version does **not** match exactly ...
  expect_equal(cooks.distance(i2), cooks.distance(fm2L))
  expect_equal(cooks.distance(fm2), cooks.distance(fm2L), tolerance = 1e-2)
  })
}) ## testLevel > 1

test_that("oldNames warning in confint", {
  fm1 <- lmer(Reaction ~ Days + (Days|Subject), sleepstudy)
  expect_warning(confint(fm1, oldNames = TRUE))
})

if (.Platform$OS.type != "windows") {

library(lme4)
library(testthat)

source(system.file("testdata/lme-tst-funs.R", package="lme4", mustWork=TRUE))
##-> gSim(), a general simulation function ...

## hand-coded Pearson residuals {for  sumFun() }
mypresid <- function(x) {
    mu <- fitted(x)
    (getME(x,"y") - mu) * sqrt(weights(x)) / sqrt(x@resp$family$variance(mu))
}

## should be equal (up to numerical error) to weights(.,type="working")
workingWeights <- function(mod) mod@resp$weights*(mod@resp$muEta()^2)/mod@resp$variance()

##' Sum of weighted residuals, 4 ways; the last three are identical
sumFun <- function(m) {
    wrss1 <- m@devcomp$cmp["wrss"]
    wrss2 <- sum(residuals(m,type="pearson")^2)
    wrss3 <- sum(m@resp$wtres^2)
    ## compare to hand-fitted Pearson resids ...
    wrss4 <- sum(mypresid(m)^2)
    c(wrss1,wrss2,wrss3,wrss4)
}
## The relative "error"/differences of the weights w[] entries
rel.diff <- function(w) abs(1 - w[-1]/w[1])

set.seed(101)

## GAMMA
g0 <-  glmer(y~x+(1|block),data=gSim(),family=Gamma)
expect_true(all(rel.diff(sumFun(g0)) < 1e-13))
expect_equal(weights(g0, type = "working"), workingWeights(g0),
             tolerance = 1e-4)  ## FIXME:  why is such a high tolerance required?

## BERNOULLI
g1 <-  glmer(y~x+(1|block),data=gSim(family=binomial(),nbinom=1),
             family=binomial)
expect_true(all(rel.diff(sumFun(g1)) < 1e-13))
expect_equal(weights(g1, type = "working"), workingWeights(g1),
             tolerance = 1e-5)  ## FIXME:  why is such a high tolerance required?


## POISSON
(n <- nrow(d.P <- gSim(family=poisson())))
g2 <-  glmer(y ~ x + (1|block), data = d.P, family=poisson)
g2W <- glmer(y ~ x + (1|block), data = d.P, family=poisson, weights = rep(2,n))
expect_true(all(rel.diff(sumFun(g2 )) < 1e-13))
expect_true(all(rel.diff(sumFun(g2W)) < 1e-13))
## correct
expect_equal(weights(g2, type = "working"), workingWeights(g2),
             tolerance = 1e-5)  ## FIXME:  why is such a high tolerance required?
expect_equal(weights(g2W, type = "working"), workingWeights(g2W),
             tolerance = 1e-5)  ## FIXME:  why is such a high tolerance required?


## non-Bernoulli BINOMIAL
g3 <- glmer(y ~ x + (1|block), data= gSim(family=binomial(), nbinom=10),
            family=binomial)
expect_true(all(rel.diff(sumFun(g3)) < 1e-13))
expect_equal(weights(g3, type = "working"), workingWeights(g3),
             tolerance = 1e-4)  ## FIXME:  why is such a high tolerance required?



d.b.2 <- gSim(nperblk = 2, family=binomial())
g.b.2 <- glmer(y ~ x + (1|block), data=d.b.2, family=binomial)

expect_true(all(rel.diff(sumFun(g.b.2 )) < 1e-13))


## Many blocks of only 2 observations each - (but nicely balanced)
## Want this "as" https://github.com/lme4/lme4/issues/47
## (but it "FAILS" survival already):
##
## n2 = n/2 :
n2 <- 2048
if(FALSE)
n2 <-  100 # for building/testing
set.seed(47)
dB2 <- gSim(n2, nperblk = 2, x= rep(0:1, each= n2), family=binomial())
##                       --  --     ---  --------
gB2 <- glmer(y ~ x + (1|block), data=dB2, family=binomial)
expect_true(all(rel.diff(sumFun(gB2)) < 1e-13))

## NB: Finite sample bias of  \hat\sigma_1 and  \hat\beta_1 ("Intercept")
##     tend to zero only slowly for  n2 -> Inf,  e.g., for
## n2 = 2048,  b1 ~= 4.3 (instead of 4);  s1 ~= 1.3 (instead of 1)

## FAILS -----
## library(survival)
## (gSurv.B2 <- clogit(y ~ x + strata(block), data=dB2))
## ## --> Error in Surv(rep(1, 200L), y) : Time and status are different lengths
## summary(gSurv.B2)
## (SE.surf <- sqrt(diag(vcov(gSurv.B2))))



g3 <-  glmer(y ~ x + (1|block),data=gSim(family=binomial(),nbinom=10),
             family=binomial)
expect_equal(var(sumFun(g3)),0)

## check dispersion parameter
## (lowered tolerance to pass checks on my machine -- SCW)
expect_equal(sigma(g0)^2, 0.4888248, tolerance=1e-4)

} ## skip on windows (for speed)

#context("specifying starting values")

##' Update 'mod', copying .@call and attr(.@frame, "start")  from 'from'
copysome <- function(mod, from) {
    stopifnot(all.equal(class(mod), class(from)), isS4(mod))
    mod@call <- from@call
    attr(mod@frame, "start") <- attr(from@frame, "start")
    mod
}

## is "Nelder_Mead" default optimizer?
isNM <- formals(lmerControl)$optimizer == "Nelder_Mead"

stMsg <- "NULL,.*numeric vector,.*list"
test_that("lmer", {
    frm <- Reaction ~ Days + (Days|Subject)
    ctrl <- lmerControl(optCtrl = list(maxfun= if(isNM) 50 else 100))
    x <- suppressWarnings(lmer(frm, data=sleepstudy, control=ctrl, REML=FALSE))
    x2 <- suppressWarnings(update(x,start=c(1,0,1)))
    x3 <- suppressWarnings(update(x,start=list(theta=c(1,0,1))))
    ff <- update(x,devFunOnly=TRUE)
    x2@call <- x3@call <- x@call  ## hack call component
    expect_equal(x,x2)
    expect_equal(x,x3)
    ## warning on deprecated list ...
    suppressWarnings(expect_error(update(x, start = "a"), stMsg))
    ## misspelled
    suppressWarnings(
        expect_error(update(x,start=list(Theta=c(1,0,1))), "invalid names")
    )
    th0 <- getME(x,"theta")
    y <- suppressWarnings(update(x,start=th0))
    if(isNM) {
        expect_equal(AIC(x), 1768.025, tolerance=1e-6)
        expect_equal(AIC(y), 1763.949, tolerance=1e-6)
    } else { ## only "bobyqa" tested:
        expect_equal(AIC(x), 1763.939344, tolerance=1e-6)
        expect_equal(AIC(x), AIC(y))
    }
    if(isNM)
        expect_equal(suppressWarnings(optimizeLmer(ff,control=list(maxfun=50),
                                                   start=c(1,0,1))$fval),
                 unname(deviance(x)))
    expect_equal(suppressWarnings(optimizeLmer(ff,control=list(maxfun=50),
                                               start=th0)$fval),
                 unname(deviance(y)))
})
test_that("glmer", {
    ctrl <- glmerControl(optCtrl=list(maxfun=50)) # -> non-convergence warnings
    x <- suppressWarnings(glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
                   data = cbpp, family = binomial, control=ctrl))
    ## theta only
    x2 <- suppressWarnings(update(x, start= 1))
    x3 <- suppressWarnings(update(x, start= list(theta = 1)))
    ff <- update(x,devFunOnly=TRUE)
    x2@call <- x3@call <- x@call  ## hack call component
    expect_equal(x,x2)
    expect_equal(x,x3)
    expect_error(update(x, start="a"), stMsg)
    expect_error(update(x, start=list(Theta=1)), "invalid names")
    ## test fixef/beta synonymy
    x3B <- suppressWarnings(update(x, start=list(beta = rep(1,4))))
    x3C <- suppressWarnings(update(x, start=list(fixef = rep(1,4))))
    ## hack to set calls equal
    x3B@call <- x3C@call
    expect_equal(x3B, x3C)
    ##
    th0 <- getME(x,"theta")
    y <- suppressWarnings(update(x, start=th0)) # expect_equal() fails: optinfo -> derivs -> Hessian

    ## theta and beta
    x0 <- update(x,nAGQ=0)
    x4 <- suppressWarnings(update(x, start = list(theta=1, fixef=fixef(x0))))
    expect_equal(x, copysome(x4, from=x))
    x5 <- suppressWarnings(update(x, start = list(theta=1, fixef=rep(0,4))))
    expect_equal(AIC(x5), 221.5823, tolerance=1e-6)
    x6 <- expect_error(suppressWarnings(update(x, start = list(theta=1, fixef=rep(0,5))),
                       "incorrect number of fixef components"))
    ## beta only
    x7 <- suppressWarnings(update(x,start=list(fixef=fixef(x0))))
    expect_equal(x, copysome(x7, from=x))
    x8 <- suppressWarnings(update(x,start=list(fixef=rep(0,4))))
    expect_equal(x5, copysome(x8, from=x5))
})

## Tests to prevent accidentally committing debugging statements.

lme4_find_src_dir <- function() {
  pkg_dir <- find.package("lme4", quiet = TRUE)
  src_dir <- file.path(pkg_dir, "src")
  ## C++ source files are only present in the source tree, not the installed package
  if (length(list.files(src_dir, pattern = "\\.cpp$")) > 0L) src_dir else NULL
}

lme4_find_r_dir <- function() {
  pkg_dir <- find.package("lme4", quiet = TRUE)
  r_dir <- file.path(pkg_dir, "R")
  if (length(list.files(r_dir, pattern = "\\.R$")) > 0L) r_dir else NULL
}

test_that("no active debug flags in C++ source files", {
  src_dir <- lme4_find_src_dir()
  skip_if(is.null(src_dir),
          "C++ source files not available (skipping in installed package)")
  
  cpp_files <- list.files(src_dir, pattern = "\\.(cpp|h|hpp)$",
                          full.names = TRUE)
  expect_gt(length(cpp_files), 0L)   ## sanity check: we found files
  
  bad_lines <- character(0L)
  for (f in cpp_files) {
    lines <- readLines(f, warn = FALSE)
    ## Flag any line where a 'debug' variable is set to a non-zero value.
    ## Catches:  int debug=1;   debug = 2;   debug=10;   etc.
    ## Ignores:  int debug=0;   (safe — debug disabled)
    ##           // debug=1;    (single-line comment)
    ##           /* debug=1; */ (block comment — line starts with '/', so
    ##                           the leading [^/]* anchor prevents a match)
    idx <- grep("^[^/]*\\bdebug\\s*=\\s*[1-9][0-9]*", lines)
    if (length(idx) > 0L) {
      bad_lines <- c(bad_lines,
                     sprintf("%s:%d: %s",
                             basename(f), idx, trimws(lines[idx])))
    }
  }
  
  expect_length(bad_lines, 0L)
})

test_that("lmer and glmer do not write to stdout", {
    out <- capture.output(
        suppressMessages(
            suppressWarnings({
                fit_lmer <- lmer(Reaction ~ Days + (Days | Subject), data = sleepstudy)
                fit_glmer <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
                                   data = cbpp, family = binomial)
                invisible(list(fit_lmer, fit_glmer))
            })
        )
    )
    expect_length(out, 0L)
})

test_that("no active debugging calls in R source files", {
    r_dir <- lme4_find_r_dir()
    skip_if(is.null(r_dir),
            "R source files not available (skipping in installed package)")

    r_files <- list.files(r_dir, pattern = "\\.R$", full.names = TRUE)
    expect_gt(length(r_files), 0L)   ## sanity check: we found files

    bad_lines <- character(0L)
    for (f in r_files) {
        lines <- readLines(f, warn = FALSE)
        ## Flag browser() calls that are not commented out.
        ## Catches:  browser()   browser(expr = ...)
        idx <- grep("\\bbrowser\\s*\\(", lines)
        if (length(idx) > 0L) {
            bad_lines <- c(bad_lines,
                           sprintf("%s:%d: %s",
                                   basename(f), idx, trimws(lines[idx])))
        }
    }

    expect_length(bad_lines, 0L)
})

load(system.file("testdata","crabs_randdata00.Rda",package="lme4"))

test_that('RZX is being calculated properly', {
  # this is a test for an old problem, documented here:
  # http://stevencarlislewalker.github.io/notebook/RZX_problems.html
  fr <- cbind(final.snail.density, snails.lost) ~ crab.speciesS + crab.sizeS + 
    crab.speciesS:crab.sizeS + (snail.size | plot)
  m <- glmer(fr, data = randdata00, family = binomial)
  expect_that(m, is_a("glmerMod"))
})

##  (as.function.merMod() assumes it)
data("Dyestuff", package = "lme4")
data("cbpp", package = "lme4")
library(lme4)  ## this *is* necessary, for as.function.merMod() ...

## use old (<=3.5.2) sample() algorithm if necessary
if ("sample.kind" %in% names(formals(RNGkind))) {
    suppressWarnings(RNGkind("Mersenne-Twister", "Inversion", "Rounding"))
}

#context("fitting lmer models")
## is "Nelder_Mead" default optimizer? -- no longer
(isNM <- formals(lmerControl)$optimizer == "Nelder_Mead")

test_that("lmer", {
    set.seed(101)
    d <- data.frame(z=rnorm(200),
                    f=factor(sample(1:10,200, replace=TRUE)))

    ## Using 'method=*' defunct in 2019-05 (after 6 years of deprecation)
    ## expect_warning(lmer(z~ 1|f, d, method="abc"),"Use the REML argument")
    ## expect_warning(lmer(z~ 1|f, d, method="Laplace"),"Use the REML argument")
    ##sp No '...' anymore
    ##sp expect_warning(lmer(z~ 1|f, d, sparseX=TRUE),"has no effect at present")
    expect_error(lmer(z~ 1|f, ddd), "bad 'data': object 'ddd' not found")
    expect_error(lmer(z~ 1|f), "object 'z' not found")
    expect_error(lmer(z~ 1|f, d[,1:1000]), "bad 'data': undefined columns selected")
    expect_is(fm1 <- lmer(Yield ~ 1|Batch, Dyestuff), "lmerMod")
    expect_is(fm1_noCD <- update(fm1,control=lmerControl(calc.derivs=FALSE)),
              "lmerMod")
    expect_equal(VarCorr(fm1),VarCorr(fm1_noCD))
    ## backward compatibility version {for optimizer="Nelder-Mead" only}:
    if(isNM) expect_is(fm1.old <- update(fm1,control=lmerControl(use.last.params=TRUE)),
                                "lmerMod")
    expect_is(fm1@resp,				"lmerResp")
    expect_is(fm1@pp, 				"merPredD")
    expect_that(fe1 <- fixef(fm1),                      is_equivalent_to(1527.5))
    expect_that(VarCorr(fm1)[[1]][1,1], ## "bobyqa" : 1764.050060
		equals(1764.0375195, tolerance = 1e-5))
    ## back-compatibility ...
    if(isNM) expect_that(VarCorr(fm1.old)[[1]][1,1], equals(1764.0726543))

    expect_that(isREML(fm1),                            equals(TRUE))
    expect_is(REMLfun <- as.function(fm1),	"function")
    expect_that(REMLfun(1),                             equals(319.792389042002))
    expect_that(REMLfun(0),                             equals(326.023232155879))
    expect_that(family(fm1),                            equals(gaussian()))
    expect_that(isREML(fm1ML <- refitML(fm1)),          equals(FALSE))
    expect_that(REMLcrit(fm1),                		equals(319.654276842342))
    expect_that(deviance(fm1ML),                        equals(327.327059881135))
    ##						"bobyqa":      49.51009984775
    expect_that(sigma(fm1),                             equals(49.5101272946856, tolerance=1e-6))
    if(isNM) expect_that(sigma(fm1.old),		equals(49.5100503990048))
    expect_that(sigma(fm1ML),                           equals(49.5100999308089))
    expect_that(extractAIC(fm1),                        equals(c(3, 333.327059881135)))
    expect_that(extractAIC(fm1ML),                      equals(c(3, 333.327059881135)))
    ##						"bobyqa":      375.71667627943
    expect_that(vcov(fm1)    [1,1],			equals(375.714676744, tolerance=1e-5))
    if(isNM) expect_that(vcov(fm1.old)[1,1],		equals(375.72027872986))
    expect_that(vcov(fm1ML)  [1,1],			equals(313.09721874266, tolerance=1e-7))
					#		   was 313.0972246957
    expect_is(fm2 <- refit(fm1, Dyestuff2$Yield), "lmerMod")
    expect_that(fixef(fm2),                             is_equivalent_to(5.6656))
    expect_that(VarCorr(fm2)[[1]][1,1],                 is_equivalent_to(0))
    expect_that(getME(fm2, "theta"),                    is_equivalent_to(0))
    expect_that(X  <- getME(fm1, "X"),                  is_equivalent_to(array(1, c(1, 30))))
    expect_is(Zt <- getME(fm1, "Zt"),		"dgCMatrix")
    expect_that(dim(Zt),                                equals(c(6L, 30L)))
    expect_that(Zt@x,                                   equals(rep.int(1, 30L)))
    expect_equal(dimnames(Zt),
                  list(levels(Dyestuff$Batch),
                       rownames(Dyestuff)))
    ##						"bobyqa":      0.8483237982
    expect_that(theta <- getME(fm1, "theta"),           equals(0.84832031, tolerance=6e-6, check.attributes=FALSE))
    if(isNM) expect_that(getME(fm1.old, "theta"),	is_equivalent_to(0.848330078))
    expect_is(Lambdat <- getME(fm1, "Lambdat"), "dgCMatrix")
    expect_that(as(Lambdat, "matrix"),                  is_equivalent_to(diag(theta, 6L, 6L)))
    expect_is(fm3 <- lmer(Reaction ~ Days + (1|Subject) + (0+Days|Subject), sleepstudy),
              					"lmerMod")
    expect_that(getME(fm3,"n_rtrms"),                   equals(2L))
    expect_that(getME(fm3,"n_rfacs"),                   equals(1L))

    expect_equal(getME(fm3, "lower"), c(`Subject.(Intercept)` = 0, Subject.Days = 0))

    expect_error(fm4 <- lmer(Reaction ~ Days + (1|Subject),
                            subset(sleepstudy,Subject==levels(Subject)[1])), "must have > 1")
    expect_warning(fm4 <- lFormula(Reaction ~ Days + (1|Subject),
                             subset(sleepstudy,Subject==levels(Subject)[1]),
                             control=lmerControl(check.nlev.gtr.1="warning")), "must have > 1")
    expect_warning(fm4 <- lmer(Reaction ~ Days + (1|Subject),
                            subset(sleepstudy,Subject %in% levels(Subject)[1:4]),
                               control=lmerControl(check.nlev.gtreq.5="warning")),
                   "< 5 sampled levels")
    sstudy9 <- subset(sleepstudy, Days == 1 | Days == 9)
    expect_error(lmer(Reaction ~ 1 + Days + (1 + Days | Subject),
                        data = sleepstudy, subset = (Days == 1 | Days == 9)),
                   "number of observations \\(=36\\) <= number of random effects \\(=36\\)")
    expect_error(lFormula(Reaction ~ 1 + Days + (1 + Days | Subject),
                           data = sleepstudy, subset = (Days == 1 | Days == 9)),
                 "number of observations \\(=36\\) <= number of random effects \\(=36\\)")
    ## with most recent Matrix (1.1-1), should *not* flag this
    ## for insufficient rank
    dat <- readRDS(system.file("testdata", "rankMatrix.rds", package="lme4"))
    expect_is(lFormula(y ~ (1|sample) + (1|day) + (1|day:sample) +
                           (1|operator) + (1|day:operator) + (1|sample:operator) +
                           (1|day:sample:operator),
                       data = dat,
                       control = lmerControl(check.nobs.vs.rankZ = "stop")),
                       "list")
    ## check scale
    ss <- within(sleepstudy, Days <- Days*1e6)
    expect_warning(lmer(Reaction ~ Days + (1|Subject), data=ss),
                 "predictor variables are on very different scales")

    ## Promote warning to error so that warnings or errors will stop the test:
    options(warn=2)
    expect_is(lmer(Yield ~ 1|Batch, Dyestuff, REML=TRUE), "lmerMod")
    expect_is(lmer(Yield ~ 1|Batch, Dyestuff, start=NULL), "lmerMod")
    expect_is(lmer(Yield ~ 1|Batch, Dyestuff, verbose=0L), "lmerMod")
    expect_is(lmer(Yield ~ 1|Batch, Dyestuff, subset=TRUE), "lmerMod")
    expect_is(lmer(Yield ~ 1|Batch, Dyestuff, weights=rep(1,nrow(Dyestuff))), "lmerMod")
    expect_is(lmer(Yield ~ 1|Batch, Dyestuff, na.action="na.exclude"), "lmerMod")
    expect_is(lmer(Yield ~ 1|Batch, Dyestuff, offset=rep(0,nrow(Dyestuff))), "lmerMod")
    expect_is(lmer(Yield ~ 1|Batch, Dyestuff, contrasts=NULL), "lmerMod")
    expect_is(lmer(Yield ~ 1|Batch, Dyestuff, devFunOnly=FALSE), "lmerMod")
    expect_is(lmer(Yield ~ 1|Batch, Dyestuff, control=lmerControl(optimizer="Nelder_Mead")), "lmerMod")
    expect_is(lmer(Yield ~ 1|Batch, Dyestuff, control=lmerControl()), "lmerMod")
    ## avoid _R_CHECK_LENGTH_1_LOGIC2_ errors ...
    if (getRversion() < "3.6.0" || (requireNamespace("optimx", quietly = TRUE) &&
                                    packageVersion("optimx") > "2018.7.10")) {
        expect_error(lmer(Yield ~ 1|Batch, Dyestuff, control=lmerControl(optimizer="optimx")),"must specify")
        expect_is(lmer(Yield ~ 1|Batch, Dyestuff,
                       control=lmerControl(optimizer="optimx",
                                           optCtrl=list(method="L-BFGS-B"))),
                  "lmerMod")
    }
    expect_error(lmer(Yield ~ 1|Batch, Dyestuff, control=lmerControl(optimizer="junk")),
                 "couldn't find optimizer function")
    ## disable test ... should be no warning
    expect_is(lmer(Reaction ~ 1 + Days + (1 + Days | Subject),
                   data = sleepstudy, subset = (Days == 1 | Days == 9),
                   control=lmerControl(check.nobs.vs.rankZ="ignore",
                   check.nobs.vs.nRE="ignore",
                   check.conv.hess="ignore",
                   ## need to ignore relative gradient check too;
                   ## surface is flat so *relative* gradient gets large
                   check.conv.grad="ignore")),
              "merMod")
    expect_is(lmer(Reaction ~ 1 + Days + (1|obs),
                   data = transform(sleepstudy,obs=seq(nrow(sleepstudy))),
                   control=lmerControl(check.nobs.vs.nlev="ignore",
                   check.nobs.vs.nRE="ignore",
                   check.nobs.vs.rankZ="ignore")),
              "merMod")
    expect_error(lmer(Reaction ~ 1 + Days + (1|obs),
                      data = transform(sleepstudy,obs=seq(nrow(sleepstudy))),
                      "number of levels of each grouping factor"))

    ## check for errors with illegal input checking options
    flags <- lme4:::.get.checkingOpts(names(formals(lmerControl)))
    .t <- lapply(flags, function(OPT) {
	## set each to invalid string:
	## cat(OPT,"\n")
	expect_error(lFormula(Reaction~1+Days+(1|Subject), data = sleepstudy,
			      control = do.call(lmerControl,
				  ## Deliberate: fake typo
				  ##		       vvv
				  setNames(list("warnign"), OPT))),
		     "invalid control level")
    })
    ## disable warning via options
    options(lmerControl=list(check.nobs.vs.rankZ="ignore",check.nobs.vs.nRE="ignore"))
    expect_is(fm4 <- lmer(Reaction ~ Days + (1|Subject),
			  subset(sleepstudy,Subject %in% levels(Subject)[1:4])), "merMod")
    expect_is(lmer(Reaction ~ 1 + Days + (1 + Days | Subject),
                   data = sleepstudy, subset = (Days == 1 | Days == 9),
                   control=lmerControl(check.conv.hess="ignore",
                   check.conv.grad="ignore")),
              "merMod")
    options(lmerControl=NULL)
    ## check for when ignored options are set
    options(lmerControl=list(junk=1,check.conv.grad="ignore"))
    expect_warning(lmer(Reaction ~ Days + (1|Subject),sleepstudy),
                   "some options")
    options(lmerControl=NULL)
    options(warn=0)
    expect_error(lmer(Yield ~ 1|Batch, Dyestuff, junkArg=TRUE), "unused argument")
    expect_warning(lmer(Yield ~ 1|Batch, Dyestuff, control=list()),
                    "passing control as list is deprecated")
    if(FALSE) ## Hadley broke this
        expect_warning(lmer(Yield ~ 1|Batch, Dyestuff, control=glmerControl()),
                       "passing control as list is deprecated")

    ss <- transform(sleepstudy,obs=factor(seq(nrow(sleepstudy))))
    expect_warning(lmer(Reaction ~ 1 + (1|obs), data=ss,
         control=lmerControl(check.nobs.vs.nlev="warning",
                             check.nobs.vs.nRE="ignore")),
                   "number of levels of each grouping factor")

    ## test deparsing of very long terms inside mkReTrms
    set.seed(101)
    longNames <- sapply(letters[1:25],
                        function(x) paste(rep(x,8),collapse=""))
    tstdat <- data.frame(Y=rnorm(10),
                         F=factor(1:10),
                         matrix(runif(250),ncol=25,
                                dimnames=list(NULL,
                                longNames)))
    expect_is(lFormula(Y~1+(aaaaaaaa+bbbbbbbb+cccccccc+dddddddd+
                        eeeeeeee+ffffffff+gggggggg+hhhhhhhh+
                        iiiiiiii+jjjjjjjj+kkkkkkkk+llllllll|F),
                   data=tstdat,
                   control=lmerControl(check.nobs.vs.nlev="ignore",
                   check.nobs.vs.nRE="ignore",
                   check.nobs.vs.rankZ="ignore")),"list")

    ## do.call(new,...) bug
    new <- "foo"
    expect_is(refit(fm1),"merMod")
    rm("new")

    ## test subset-with-( printing from summary
    fm1 <- lmer(z~1|f,d,subset=(z<1e9))
    expect_equal(sum(grepl("Subset: \\(",capture.output(summary(fm1)))),1)

    ## test messed-up Hessian
    ## UPDATE: modified this test since we now automatically ignore Hessian
    ## checks when there is a singular fit
    fm1 <- lmer(z~ as.numeric(f) + 1|f, d)
    
    expect_equal(summary(fm1)$optinfo$conv$lme4$messages,
                  "boundary (singular) fit: see help('isSingular')")
    expect_null(fm1@optinfo$derivs$Hessian)
    ## Previous:
    ##fm1@optinfo$derivs$Hessian[2,2] <- NA
    ##expect_warning(lme4:::checkConv(fm1@optinfo$derivs,
    ##                 coefs=c(1,1),
    ##                 ctrl=lmerControl()$checkConv,lbound=0),
    ##                 "Problem with Hessian check")

    ## test ordering of Ztlist names
    ## this is a silly model, just using it for a case
    ## where nlevs(RE term 1) < nlevs(RE term 2)x
    data(cbpp)
    cbpp <- transform(cbpp,obs=factor(1:nrow(cbpp)))
    fm0 <- lmer(incidence~1+(1|herd)+(1|obs),cbpp,
         control=lmerControl(check.nobs.vs.nlev="ignore",
                             check.nobs.vs.rankZ="ignore",
                             check.nobs.vs.nRE="ignore",
                             check.conv.grad="ignore",
                             check.conv.singular="ignore",
                             check.conv.hess="ignore"))
    fm0B <- update(fm0, .~1+(1|obs)+(1|herd))
    expect_equal(names(getME(fm0,"Ztlist")),
                 c("obs.(Intercept)", "herd.(Intercept)"))
    ## stable regardless of order in formula
    expect_equal(getME(fm0,"Ztlist"),getME(fm0B,"Ztlist"))
    ## no optimization  (GH #408)
    fm_noopt <- lmer(z~1|f,d,
                     control=lmerControl(optimizer=NULL))
    expect_equal(unname(unlist(getME(fm_noopt,c("theta","beta")))),
                 c(0.244179074357121, -0.0336616441209862))
    expect_error(lmer(z~1|f,d,
                     control=lmerControl(optimizer="none")),
                 "deprecated use")
    my_opt <- function(fn,par,lower,upper,control) {
        opt <- optim(fn=fn,par=par,lower=lower,
              upper=upper,control=control,,method="L-BFGS-B")
        return(list(par=opt$par,fval=opt$value,conv=opt$convergence))
    }
    expect_is(fm_noopt <- lmer(z~1|f,d,
                     control=lmerControl(optimizer=my_opt)),"merMod")

    ## test verbose option for nloptwrap
    cc <- capture.output(lmer(Reaction~1+(1|Subject),
         data=sleepstudy,
         control=lmerControl(optimizer="nloptwrap",
                 optCtrl=list(xtol_abs=1e-6, ftol_abs=1e-6)),
         verbose=5))
    expect_equal(sum(grepl("^iteration:",cc)),14)

}) ## test_that(..)

test_that("coef_lmer", {
    ## test coefficient extraction in the case where RE contain
    ## terms that are missing from the FE ...
    set.seed(101)
    d <- data.frame(resp=runif(100),
                    var1=factor(sample(1:5,size=100,replace=TRUE)),
                    var2=runif(100),
                    var3=factor(sample(1:5,size=100,replace=TRUE)))
    mix1 <- lmer(resp ~ 0 + var1 + var1:var2 + (1|var3), data=d)
    c1 <- coef(mix1)
    expect_is(c1, "coef.mer")
    cd1 <- c1$var3
    expect_is   (cd1, "data.frame")
    n1 <- paste0("var1", 1:5)
    nn <- c(n1, paste(n1, "var2", sep=":"))
    expect_identical(names(cd1), c("(Intercept)", nn))
    expect_equal(fixef(mix1),
                 setNames(c(0.2703951, 0.3832911, 0.451279, 0.6528842, 0.6109819,
                            0.4949802, 0.1222705, 0.08702069, -0.2856431, -0.01596725),
                          nn), tolerance= 6e-6)# 64-bit:  6.73e-9
})

test_that("getCall", {
    ## GH #535
    getClass <- function() "foo"
    expect_is(glmer(round(Reaction) ~ 1 + (1|Subject), sleepstudy,
        family=poisson), "glmerMod")
    rm(getClass)
})

test_that("better info about optimizer convergence",
{
    set.seed(14)
    cbpp$var <- rnorm(nrow(cbpp), 10, 10)

    suppressWarnings(gm2 <-
                         glmer(cbind(incidence, size - incidence) ~ period * var + (1 | herd),
                               data = cbpp, family = binomial,
                               control=glmerControl(optimizer=c("bobyqa","Nelder_Mead")))
                     )

    ## FIXME: with new update, suppressWarnings(update(gm2)) will give
    ## Error in as.list.environment(X[[i]], ...) :
    ## promise already under evaluation: recursive default argument reference or earlier problems?
    op <- options(warn=-1)
    gm3 <- update(gm2,
                  control=glmerControl(optimizer="bobyqa",
                                       optCtrl=list(maxfun=2)))
    options(op)

    cc <-capture.output(print(summary(gm2)))
    expect_equal(tail(cc,4)[1],
                 "optimizer (Nelder_Mead) convergence code: 0 (OK)")
})

#context("convergence warnings etc.")

fm1 <- lmer(Reaction~ Days + (Days|Subject), sleepstudy)
suppressMessages(fm0 <- lmer(Reaction~ Days + (Days|Subject), sleepstudy[1:20,]))

msg_in_output <- function(x, str) {
    cc <- capture.output(.prt.warn(x))
    any(grepl(str , cc))
}

test_that("convergence warnings from limited evals", {
    expect_warning(fm1B <- update(fm1, control=lmerControl(optCtrl=list(maxeval=3))),
                   "convergence code 5")
    expect_true(msg_in_output(fm1B@optinfo, "convergence code: 5"))
    expect_warning(fm1C <- update(fm1, control=lmerControl(optimizer="bobyqa",optCtrl=list(maxfun=3))),
                   "maximum number of function evaluations exceeded")
    expect_true(msg_in_output(fm1C@optinfo,
                              "maximum number of function evaluations exceeded"))
    ## one extra (spurious) warning here ...
    expect_warning(fm1D <- update(fm1, control=lmerControl(optimizer="Nelder_Mead",optCtrl=list(maxfun=3))),
                   "failure to converge in 3 evaluations")
    expect_true(msg_in_output(fm1D@optinfo,
                              "failure to converge in 3 evaluations"))
    expect_message(fm0D <- update(fm0, control=lmerControl(optimizer="Nelder_Mead",calc.derivs=FALSE)),
                   "boundary")
    expect_true(msg_in_output(fm0D@optinfo,
                              "(OK)"))
})

## GH 533
test_that("test for zero non-NA cases", {
    data_bad <- sleepstudy
    data_bad$Days <- NA_real_
    expect_error(lmer(Reaction ~ Days + (1| Subject), data_bad),
                 "0 \\(non-NA\\) cases")
})

##
test_that("catch matrix-valued responses in lmer/glmer but not in formulas", {
    dd <- data.frame(x = rnorm(1000), batch = factor(rep(1:20, each=50)))
    dd$y <- matrix(rnorm(1e4), ncol = 10)
    dd$y2 <- matrix(rpois(1e4, lambda = 1), ncol = 10)
    expect_error(lmer(y ~ x + (1|batch), dd), "matrix-valued")
    fr <- lFormula(y ~ x + (1|batch), dd)$fr
    expect_true(is.matrix(model.response(fr)))
    expect_error(glmer(y ~ x + (1|batch), dd, family = poisson), "matrix-valued")
    fr <- glFormula(y ~ x + (1|batch), dd, family = poisson)$fr
})

test_that("catch matrix-valued responses", {
    dd <- data.frame(x = rnorm(1000), batch = factor(rep(1:20, each=50)))
    dd$y <- matrix(rnorm(1e4), ncol = 10)
    expect_error(lmer(y ~ x + (1|batch), dd), "matrix-valued")
})

test_that("update works as expected", {
    m <- lmer(Reaction ~ Days + (Days || Subject), sleepstudy)
    m1 <- lmer(Reaction ~ Days + (1 | Subject), sleepstudy)
    m2 <- update(m, . ~ . - (0 + Days | Subject))
    if (getOption("lme4.doublevert.default", "split") == "split") {
      expect_equivalent(fitted(m1), fitted(m2))
    }
})

test_that("turn off conv checking for nobs > check.conv.nobsmax", {
  ## calc derivs and check convergence
  fm1 <- lmer(Reaction ~ 1 + (1|Days), sleepstudy)
  nn <- nrow(sleepstudy)-1
  ## neither derivs nor conv check
  fm2 <- update(fm1,
                control = lmerControl(check.conv.nobsmax = nn))
  ## no conv check, do calc derivs
  fm3 <- update(fm1, 
                control = lmerControl(check.conv.nobsmax = nn,
                                      calc.derivs = TRUE))
  expect_null(fm2@optinfo$derivs)
  expect_false(is.null(fm1@optinfo$derivs))
  expect_false(is.null(fm3@optinfo$derivs))
  expect_equal(fm1@optinfo$conv$lme4, list())
  expect_null(fm2@optinfo$conv$lme4)
  expect_null(fm3@optinfo$conv$lme4)
})

test_that("turn off conv checking for npara > check.conv.nparmax", {
  set.seed(1)
  
  n_groups <- 20
  n_per_group <- 20
  n <- n_groups * n_per_group
  
  dat <- data.frame(
    group = rep(1:n_groups, each = n_per_group),
    x1 = rnorm(n),
    x2 = rnorm(n)
  )
  
  set.seed(101)
  form <- y ~ 1 + x1 * x2 + (1 + x1|group)
  dat$y <- simulate(form[-2], ## one-sided formula
                    newdata = dat,
                    family = gaussian,
                    newparams = list(beta = c(-7, 5, -100, 20),
                                     theta = c(2.5, 1.4, 6.3),
                                     sigma = 2))[[1]]
  
  mod1 <- suppressWarnings(lmer(form, data = dat,
               control = lmerControl(
                 optimizer = "bobyqa",
                 optCtrl = list(maxfun = 10)
               )))
  
  mod2 <- suppressWarnings(update(mod1, 
                 control = lmerControl(
                   optimizer = "bobyqa",
                   optCtrl = list(maxfun = 10),
                   check.conv.nparmax = 3))
  )
  
  ## First should give a warning
  expect_false(is.null(mod1@optinfo$conv$lme4))
  ## Second shouldn't be evaluated
  expect_null(mod2@optinfo$conv$lme4)
})

test_that("gradient and Hessian checks are skipped when singular fit occurs",{
  set.seed(1)
  group <- factor(rep(1:3, each = 20))
  b <- rnorm(3, mean = 0, sd = 0.01)
  x <- rnorm(60)
  y <- x + b[group] + rnorm(60, sd = 1)
  dat <- data.frame(y, x, group)
  
  fm1 <- lmer(y ~ x + (1 | group), data = dat)
  
  expect_null(summary(fm1)$optinfo$derivs$gradient)
  expect_null(summary(fm1)$optinfo$derivs$Hessian)
  
  ## Always skipping derivative checks
  options(lme4.singular.tolerance = 1)
  fm2 <- lmer(Reaction ~ Days + (Days | Subject), sleepstudy)
  expect_null(summary(fm2)$optinfo$derivs$gradient)
  expect_null(summary(fm2)$optinfo$derivs$Hessian)
  ## Switching back (in case needed)
  options(lme4.singular.tolerance = 1e-4)
  
  ## force calculating the derivatives even if the fit is singular
  fm2 <- lmer(y ~ x + (1 | group), 
              control = lmerControl(calc.derivs = TRUE), data = dat)
  expect_false(is.null(summary(fm2)$optinfo$derivs))
})

test_that("doubleverts expanded in stored formula by default", {
  lf <- lFormula(Reaction ~ Days + (Days || Subject), sleepstudy)
  expect_equal(formula(lf),
               Reaction ~ Days + ((1 | Subject) + (0 + Days | Subject)))
})

stopifnot(require("testthat"), require("lme4"))

set.seed(101)
n <- 500
d <- data.frame(x=rnorm(n),
                f=factor(sample(1:10,n,replace=TRUE),
                         labels=LETTERS[1:10]),
                g=factor(sample(1:25,n,replace=TRUE),
                         labels=letters[1:25]))
d$y <- suppressMessages(simulate(~1+x+(1|f)+(x|g),family=binomial,
                                 newdata=d,
                                 newparams=list(beta=c(0,1),
                                                theta=c(1,1,2,1)))[[1]])
fm1 <- glmer(y~(1|f)+(x|g),family=binomial,data=d)

#context("ranef")
test_that("warn extra args", {
    expect_warning(ranef(fm1,transf=exp),"additional arguments")
})
test_that("dotplot_ranef", {
    rr <- ranef(fm1,condVar=TRUE)
    expect_is(lattice::dotplot(rr,scales=list(x = list(relation = 'free')))$g,
              "trellis")
    expect_is(lattice::dotplot(rr,transf=exp,
                     scales=list(x = list(relation = 'free')))$g,
              "trellis")
    expect_is(as.data.frame(rr),"data.frame")
    rr0 <- ranef(fm1)
    expect_is(as.data.frame(rr0),"data.frame")
})

test_that("Dyestuff consistent with lme4.0", {
    lme4.0condVarDyestuff <- c(362.3583, 362.3583, 362.3583, 362.3583, 362.3583, 362.3583)
    fm <- lmer(Yield ~ 1|Batch, Dyestuff, REML=FALSE)
    lme4condVarDyestuff <- drop(attr(ranef(fm,condVar=TRUE)$Batch,"postVar"))
    expect_equal(lme4.0condVarDyestuff, lme4condVarDyestuff, tolerance = 1e-3)
})


test_that("sleepstudy consistent with lme4.0", {
        lme4.0condVarsleepstudy <- matrix(c(145.71273, -21.440414,
                                            -21.44041,   5.310927), 2, 2)
        fm <- lmer(Reaction ~ Days + (Days | Subject), sleepstudy)
        lme4condVarsleepstudy <- attr(ranef(fm,condVar=TRUE)$Subject,"postVar")[,,1]
        expect_equal(lme4.0condVarsleepstudy, lme4condVarsleepstudy, tolerance = 2e-4)
})


test_that("cbpp consistent with lme4.0", {
    lme4.0condVarcbpp <- c(0.12128867, 0.13363275, 0.08839850, 0.17337928, 0.12277914, 0.14436663,
                           0.10658333, 0.10309812, 0.21289738, 0.13740279, 0.09555677, 0.19460241,
                           0.14808316, 0.12631006, 0.15816769)
    gm <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
                data = cbpp, family = binomial)
    lme4condVarcbpp <- as.numeric(attr(ranef(gm,condVar=TRUE)$herd,"postVar"))
    expect_equal(lme4.0condVarcbpp, lme4condVarcbpp, tolerance = 1e-3)
})

#context("multiple terms per factor")
test_that("multiple terms work", {
    fm <- lmer(Reaction ~ Days + (1|Subject)+ (0+Days | Subject), sleepstudy,
               control=lmerControl(optimizer="nloptwrap",
                                   optCtrl=list(xtol_abs=1e-6, ftol_abs=1e-6)))
    rr <- ranef(fm, condVar=TRUE)
    expect_equal(as.data.frame(rr)[c(1,19),],
                 structure(
                     list(grpvar = c("Subject", "Subject"),
                          term = structure(1:2, .Label = c("(Intercept)", "Days"), class = "factor"),
                          grp = structure(c(9L, 9L),
                                          .Label = c("309", "310", "370", "349", "350", "334", "335", "371", "308", "369",
                                                     "351", "332", "372", "333", "352", "331", "330", "337"), class = "factor"),
                          condval = c(1.5116973008, 9.32373076098),
                          condsd  = c(12.238845590, 2.33546851406)),
                     row.names = c(1L, 19L), class = "data.frame"),
                 tolerance = 1e-5)
    cv <- attr(rr$Subject, "postVar")
    expect_equal(lapply(cv, drop),
                 list(`(Intercept)` = rep(149.79166, 18),
                      Days          = rep(5.4543894, 18)), tolerance = 1e-4)
})

#context("testing fixed-effect design matrices for full rank")

test_that("lmerRank", {
    set.seed(101)
    n <- 20
    x <- y <- rnorm(n)
    d <- data.frame(x,y,
		    z = rnorm(n),
		    r = sample(1:5, size=n, replace=TRUE),
		    y2 = y + c(0.001, rep(0,n-1)))
    expect_message(fm <- lmer( z ~ x + y + (1|r), data=d),
		   "fixed-effect model matrix is .*rank deficient")
    ## test reconstitution of full parameter vector (with NAs)
    expect_equal(names(fixef(fm,add.dropped=TRUE)),
                 c("(Intercept)","x","y"))
    expect_equal(fixef(fm,add.dropped=TRUE)[1:2],
                 fixef(fm))

    expect_equal(nrow(anova(fm)), 1L)
    expect_error(lmer( z ~ x + y + (1|r), data=d,
		      control=lmerControl(check.rankX="stop")),
		 "rank deficient")
    expect_error(lmer( z ~ x + y + (1|r), data=d,
		      control=lmerControl(check.rankX="ignore")),
		 "not positive definite")
    ## should work:
    expect_is(lmer( z ~ x + y2 + (1|r), data=d), "lmerMod")

    d2 <- expand.grid(a=factor(1:4),b=factor(1:4),rep=1:10)
    n <- nrow(d2)
    d2 <- transform(d2,r=sample(1:5, size=n, replace=TRUE),
                     z=rnorm(n))
    d2 <- subset(d2,!(a=="4" & b=="4"))
    expect_error(lmer( z ~ a*b + (1|r), data=d2,
                      control=lmerControl(check.rankX="stop")),
                 "rank deficient")
    expect_message(fm <- lmer( z ~ a*b + (1|r), data=d2),
                   "fixed-effect model matrix is rank deficient")
    d2 <- transform(d2, ab=droplevels(interaction(a,b)))
    ## should work:
    expect_is(fm2 <- lmer( z ~ ab + (1|r), data=d2), "lmerMod")
    expect_equal(logLik(fm), logLik(fm2))
    expect_equal(sum(anova(fm)[, "npar"]), anova(fm2)[, "npar"])
    expect_equal(sum(anova(fm)[, "Sum Sq"]), anova(fm2)[, "Sum Sq"])
})

test_that("glmerRank", {
    set.seed(111)
    n <- 100
    x <- y <- rnorm(n)
    d <- data.frame(x, y,
		    z = rbinom(n,size=1,prob=0.5),
		    r = sample(1:5, size=n, replace=TRUE),
                    y2 = ## y + c(0.001,rep(0,n-1)), ## too small: get convergence failures
                    ## FIXME: figure out how small a difference will still fail?
                    rnorm(n))
    expect_message(fm <- glmer( z ~ x + y + (1|r), data=d, family=binomial),
                   "fixed-effect model matrix is rank deficient")
    expect_error(glmer( z ~ x + y + (1|r), data=d, family=binomial,
                       control=glmerControl(check.rankX="stop")),
                 "rank deficient.*rank.X.")
    expect_is(glmer( z ~ x + y2 + (1|r), data=d, family=binomial), "glmerMod")
})

test_that("nlmerRank", {
    set.seed(101)
    n <- 1000
    nblock <- 15
    x <- abs(rnorm(n))
    y <- rnorm(n)
    z <- rnorm(n,mean=x^y)
    r <- sample(1:nblock, size=n, replace=TRUE)
    d <- data.frame(x,y,z,r)
    ## save("d","nlmerRank.RData")  ## see what's going on with difference in contexts

    fModel <- function(a,b)  (exp(a)*x)^(b*y)
    fModf <- deriv(body(fModel), namevec = c("a","b"),
                   func = fModel)

    fModel2 <- function(a,b,c)  (exp(a+c)*x)^(b*y)
    fModf2 <- deriv(body(fModel2), namevec = c("a","b","c"),
                   func = fModel2)

    ## should be OK: fails in test mode?
    nlmer(y ~ fModf(a,b) ~ a|r, d, start = c(a=1,b=1))

    ## FIXME: this doesn't get caught where I expected
    expect_error(nlmer(y ~ fModf2(a,b,c) ~ a|r, d, start = c(a=1,b=1,c=1)),"Downdated VtV")

})

test_that("ranksim", {
    set.seed(101)
    x <- data.frame(id = factor(sample(10, 100, replace = TRUE)))
    x$y <- rnorm(nrow(x))
    x$x1 <- 1
    x$x2 <- ifelse(x$y<0, rnorm(nrow(x), mean=1), rnorm(nrow(x), mean=-1))
    m <- suppressMessages(lmer(y ~ x1 + x2 + (1 | id), data=x))
    expect_equal(simulate(m, nsim = 1, use.u = FALSE, newdata=x, seed=101),
                 simulate(m, nsim = 1, use.u = FALSE, seed=101))
})                 

testLevel <- if (nzchar(s <- Sys.getenv("LME4_TEST_LEVEL"))) as.numeric(s) else 1

if (testLevel>1) {
#context("glmer.nb")
test_that("basic", {
   set.seed(101)
   dd <- expand.grid(f1 = factor(1:3),
                     f2 = LETTERS[1:2], g=1:9, rep=1:15,
                     KEEP.OUT.ATTRS=FALSE)
   mu <- 5*(-4 + with(dd, as.integer(f1) + 4*as.numeric(f2)))
   dd$y <- rnbinom(nrow(dd), mu = mu, size = 0.5)
   require("MASS")
   m.glm <- glm.nb(y ~ f1*f2, data=dd)
   m.nb <- glmer.nb(y ~ f1*f2 + (1|g), data=dd)
   expect_equal(unname(fixef(m.nb)),
                c(1.65008, 0.76715, 1.01147, 1.51241, -0.61506, -0.6104),
                tol=1e-5)

   
   expect_is(m.nb,"glmerMod")
   ## 'family' properly quoted/not expanded in call?
   expect_true(grepl("negative\\.binomial\\(theta *= *[0-9]*\\.[0-9]+\\)",
                     deparse(m.nb@call$family)))
   expect_null(m.nb@call$verbose)  ## check: GH #321
   expect_equal(fixef(m.nb), coef (m.glm), tol=1e-5)    ## GH #319

   ## GH #285
   m.nb1 <- glmer(Reaction > 250 ~ Days + (1|Subject),
                  data = sleepstudy, family=poisson)
   

   ## previously failing on Travis-CI
   m.nb2 <- glmer.nb(y ~ f1*f2 + (1|g), data=dd,
                     subset = g!=8)
   expect_equal(unname(ngrps(m.nb2)),8)
   ## expect parameters, ngrps *not* to equal full model
   expect_equal(unname(fixef(m.nb2)),
                c(1.629240234, 0.76028840, 1.008629913, 1.6172507, 
                  -0.6814426, -0.66468330),tol=1e-5)
   

   ## control handling ... this should suppress warnings ...
   old.opts <- options(warning=2)
   m.nb2 <- glmer.nb(round(Reaction) ~ Days + (1|Subject),
                     data = sleepstudy, subset = Subject != 370,
                     control=glmerControl(check.conv.grad="ignore"))
   expect_is(m.nb2,"glmerMod")
   options(old.opts)

   m.nb3 <- glmer.nb(y~f1+(1|g),
                       data=dd,
                       contrasts=list(f1=contr.sum))
   ## make sure *different* fixed effects from previous fit ... 
   expect_equal(fixef(m.nb3),
                structure(c(2.93061, -0.29779, 0.02586),
                          .Names = c("(Intercept)", 
                          "f11", "f12")),tol=1e-5)
   
   ## make sure 'data' is in call even if unnamed
   m.nb4 <- glmer.nb(y~f1+(1|g), dd)
   expect_equal(names(m.nb4@call),c("","formula","data","family"))

   ## GH 322; allow offset
   m.nb2 <- glmer.nb(y~f1+(1|g),
                         data=dd,
                         offset=rep(0,nrow(dd)))
})

if (requireNamespace("merDeriv")) {
    test_that("summary not broken by merDeriv", {
        library(merDeriv)
        data(Arabidopsis)
        mod <- glmer.nb(total.fruits~status+nutrient+(1|gen), data=Arabidopsis)
        ## trivial test of existence/non-failure ...
        expect_is(summary(mod), "summary.merMod")
        ## may? expose other problems as S3 methods won't be unloaded properly ...
        detach("package:merDeriv")
    })
}

} ## testLevel > 1

data("sleepstudy", package = "lme4")

source(system.file("testdata/lme-tst-funs.R", package="lme4", mustWork=TRUE))
##-> gSim(), a general simulation function ...

set.seed(101)
dBc <- gSim(family=binomial(link="cloglog"), nbinom = 1) # {0,1} Binomial

## m1 <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
##             family = binomial, data = cbpp)
#context("Errors and warnings from glmer")
test_that("glmer", {
    expect_error(glmer(y ~ 1 + (1|block), data=dBc, family=binomial(link="cloglog")),
                 "Response is constant")
    expect_error(glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
                       family = binomial, data = cbpp, REML=TRUE),
                   "unused argument.*REML")
    expect_warning(glmer(Reaction ~ Days + (Days|Subject), sleepstudy),
                   "calling glmer.*family=gaussian.*deprecated")
    expect_warning(glmer(Reaction ~ Days + (Days|Subject), sleepstudy, family=gaussian),
                   "calling glmer.*family=gaussian.*deprecated")
    m3 <- suppressWarnings(glmer(Reaction ~ Days + (Days|Subject), sleepstudy))
    m4 <- lmer(Reaction ~ Days + (Days|Subject), sleepstudy)
    m5 <- suppressWarnings(glmer(Reaction ~ Days + (Days|Subject), sleepstudy, family=gaussian))
    expect_equal(fixef(m3),fixef(m5))
    m3@call[[1]] <- m5@call[[1]] <- quote(lmer)  ## hack call
    expect_equal(m3,m4)
    expect_equal(m3,m5)
})

source(system.file("testdata", "lme-tst-funs.R", package="lme4", mustWork=TRUE))# -> uc() [back-compatible c()]
testLevel <- lme4:::testLevel()

gives_error_or_warning <- function (regexp = NULL, all = FALSE, ...)
{
    function(expr) {
        res <- try(evaluate_promise(expr),silent=TRUE)
        no_error <- !inherits(res, "try-error")
        if (no_error) {
            warnings <- res$warnings

            if (!is.null(regexp) && length(warnings) > 0) {
                return(matches(regexp, all = FALSE, ...)(warnings))
            } else {
                return(expectation(length(warnings) > 0, "no warnings or errors given",
                            paste0(length(warnings), " warnings created")))
            }
        }
        if (!is.null(regexp)) {
            return(matches(regexp, ...)(res))
        }
        else {
            expectation(TRUE, "no error thrown", "threw an error")
        }
    }
}
    ## expect_that(stop("foo"),gives_error_or_warning("foo"))
    ## expect_that(warning("foo"),gives_error_or_warning("foo"))
    ## expect_that(TRUE,gives_error_or_warning("foo"))
    ## expect_that(stop("bar"),gives_error_or_warning("foo"))
    ## expect_that(warning("bar"),gives_error_or_warning("foo"))

if(testLevel > 1) {
#context("fitting glmer models")
test_that("glmer", {
    set.seed(101)
    d <- data.frame(z=rbinom(200,size=1,prob=0.5),
                    f=factor(sample(1:10,200,replace=TRUE)))
    ## Using 'method=*' defunct in 2019-05 (after 6 years of deprecation)
    ## expect_warning(glmer(z~ 1|f, d, family=binomial, method="abc"),"Use the nAGQ argument")
    ## expect_warning(glmer(z~ 1|f, d, family=binomial, method="Laplace"),"Use the nAGQ argument")
    ##sp expect_warning(glmer(z~ 1|f, d, sparseX=TRUE),"has no effect at present")
    expect_that(gm1 <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
                             data = cbpp, family = binomial), is_a("glmerMod"))
    expect_that(gm1@resp,                               is_a("glmResp"))
    expect_that(gm1@pp,                                 is_a("merPredD"))
    expect_equal(ge1 <- unname(fixef(gm1)),             c(-1.39854982537216, -0.992335519118859,
                                                          -1.12867532780426, -1.58030423764517),
                                                         tolerance=5e-4)
    expect_equal(c(VarCorr(gm1)[[1]]),                  0.41245527438386, tolerance=6e-4)
### expect_that(family(gm1),                            equals(binomial()))
### ?? binomial() has an 'initialize' component ... and the order is different
    expect_equal(deviance(gm1),                         73.47428, tolerance=1e-5)
    ## was -2L = 184.05267459802
    expect_equal(sigma(gm1),                            1)
    expect_equal(extractAIC(gm1),                       c(5, 194.052674598026), tolerance=1e-5)

    expect_equal(theta <- unname(getME(gm1, "theta")),  0.642226809144453, tolerance=6e-4)
    expect_that(X  <- getME(gm1, "X"),                  is_equivalent_to(
        model.matrix(model.frame(~ period, data=cbpp), cbpp)))
    expect_that(Zt <- getME(gm1, "Zt"),                 is_a("dgCMatrix"))
    expect_equal(dim(Zt),                               c(15L, 56L))
    expect_equal(Zt@x,                                  rep.int(1, 56L))
    expect_that(Lambdat <- getME(gm1, "Lambdat"),       is_a("dgCMatrix"))
    expect_equivalent(as(Lambdat, "matrix"),            diag(theta, 15L, 15L))

    expect_is(gm1_probit <- update(gm1,family=binomial(link="probit")),"merMod")
    expect_equal(family(gm1_probit)$link,"probit")

    ## FIXME: test user-specified/custom family?

    expect_error(glFormula(cbind(incidence, size - incidence) ~ period + (1 | herd),
                             data = subset(cbpp, herd==levels(herd)[1]), family = binomial),
                 "must have > 1")
    expect_warning(glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
                         data = subset(cbpp, herd %in% levels(herd)[1:4]),
                         family = binomial,
                         control=glmerControl(check.nlev.gtreq.5="warning")),
                   "< 5 sampled levels")
    expect_warning(fm1. <- glmer(Reaction ~ Days + (Days|Subject), sleepstudy),
                   regexp="calling .* with family=gaussian .* as a shortcut")
    options(warn=2)
    options(glmerControl=list(junk=1,check.conv.grad="ignore"))
    expect_warning(glmer(z~ 1|f, d, family=binomial),
                   "some options")
    options(glmerControl=NULL)
    cbppX <- transform(cbpp,prop=incidence/size)
    expect_is(glmer(prop ~ period + (1 | herd),
		      data = cbppX, family = binomial, weights=size),
	      "glmerMod")
    expect_is(glmer(prop ~ period + (1 | herd),
		      data = cbppX, family = binomial, weights=size, start=NULL),
	      "glmerMod")
    expect_is(glmer(prop ~ period + (1 | herd),
		      data = cbppX, family = binomial, weights=size, verbose=0L),
	      "glmerMod")
    expect_is(glmer(prop ~ period + (1 | herd),
		      data = cbppX, family = binomial, weights=size, subset=TRUE),
	      "glmerMod")
    expect_is(glmer(prop ~ period + (1 | herd),
		      data = cbppX, family = binomial, weights=size, na.action="na.exclude"),
	      "glmerMod")

    expect_is(glmer(prop ~ period + (1 | herd),
    		      data = cbppX, family = binomial, weights=size, offset=rep(0,nrow(cbppX))),
    "glmerMod")
    expect_is(glmer(prop ~ period + (1 | herd),
		      data = cbppX, family = binomial, weights=size, contrasts=NULL),
	      "glmerMod")
    expect_is(glmer(prop ~ period + (1 | herd),
		      data = cbppX, family = binomial, weights=size, devFunOnly=FALSE),
	      "glmerMod")
    expect_is(glmer(prop ~ period + (1 | herd),
		      data = cbppX, family = binomial, weights=size,
		    control=glmerControl(optimizer="Nelder_Mead")),
	      "glmerMod")
    expect_is(glmer(prop ~ period + (1 | herd),
		      data = cbppX, family = binomial, weights=size, control=glmerControl()),
	      "glmerMod")
    options(warn=0)
    expect_error(glmer(prop ~ period + (1 | herd),
                       data = cbppX, family = binomial, weights=size, junkArg=TRUE),
                 "unused argument")
if(FALSE) { ## Hadley broke this
    expect_warning(glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
                         data = cbpp, family = binomial,
                         control=list()),
                   "instead of passing a list of class")
    expect_warning(glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
                         data = cbpp, family = binomial,
                         control=lmerControl()),
                   "instead of passing a list of class")
}
    ##
    load(system.file("testdata","radinger_dat.RData",package="lme4"))
    mod <- glmer(presabs~predictor+(1|species),family=binomial,
                 radinger_dat)
    expect_is(mod,"merMod")
    ## tolerance: 32-bit Windows (CRAN) reported ave.diff of 5.33e-8
    ## 64-bit Win-builder r73242 now reports ave. diff of 1.31e-5 ...
    expect_equal(unname(fixef(mod)), c(0.5425528,6.4289962), tolerance = 1e-4)
    set.seed(101)
    ## complete separation case
    d <- data.frame(y=rbinom(1000,size=1,p=0.5),
                    x=runif(1000),
                    f=factor(rep(1:20,each=50)),
                    x2=rep(0:1,c(999,1)))
    expect_message(mod2 <- glmer(y~x+x2+(1|f),data=d,family=binomial,
                                 control=glmerControl(check.conv.hess="ignore",
                                                      check.conv.grad="ignore")),
                   "singular")
    expect_equal(unname(fixef(mod2))[1:2],
                 c(-0.10036244,0.03548523), tolerance=1e-4)
    expect_true(unname(fixef(mod2)[3] < -10))
    expect_message(mod3 <- update(mod2, family=binomial(link="probit")),
                   "singular")
    # singular Hessian warning
    expect_equal(unname(fixef(mod3))[1:2], c(-0.062889, 0.022241), tolerance=1e-4)
    expect_true(fixef(mod3)[3] < -4)
    expect_message(mod4 <- update(mod2, family=binomial(link="cauchit"),
                   control=glmerControl(check.conv.hess="ignore",
                                        check.conv.grad="ignore")))#--> singular Hessian warning

    ## on-the-fly creation of index variables
    if (FALSE) {
        ## FIXME: fails in testthat context -- 'd' is not found
        ##  in the parent environment of glmer() -- but works fine
        ## otherwise ...
        set.seed(101)
        d <- data.frame(y1=rpois(100,1),  x=rnorm(100), ID=1:100)
        fit1 <- glmer(y1 ~ x+(1|ID),data=d,family=poisson)
        fit2 <- update(fit1, .~ x+(1|rownames(d)))
        expect_equal(unname(unlist(VarCorr(fit1))),
                     unname(unlist(VarCorr(fit2))))
    }

    ##
    if(testLevel > 2) {
        load(system.file("testdata","mastitis.rda",package="lme4"))
        t1 <- system.time(g1 <-
                              suppressWarnings(glmer(NCM ~ birth + calvingYear + (1|sire) + (1|herd),
                                mastitis, poisson,
                                ## current (2014-04-24) default: --> Warning
                                control=glmerControl(  # max|grad| = 0.021 ..
                                    optimizer=c("bobyqa","Nelder_Mead")))))

        t2 <- system.time(g2 <- suppressWarnings(update(g1,
                         control=glmerControl(optimizer="bobyqa"))))
        ## rbind(t1,t2)[,"elapsed"]
        ## 20 (then 13.0) seconds N-M vs 8 (then 4.8) seconds bobyqa ...
        ## print(t1[3] / t2[3]) # 0.37; => 1.25 should be on the safe side
        expect_lte(t2[3], 1.25 * t1[3])
        ## problem is fairly ill-conditioned so parameters
        ##  are relatively far apart even though likelihoods are OK
        expect_equal(logLik(g1),logLik(g2),tolerance=2e-7)
    }
    ## test bootstrap/refit with nAGQ>1
    gm1AGQ <- update(gm1,nAGQ=2)
    s1 <- simulate(gm1AGQ)
    expect_equal(attr(bootMer(gm1AGQ,fixef),"bootFail"),0)

    ## do.call(new,...) bug
    new <- "foo"
    expect_that(gm1 <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
                             data = cbpp, family = binomial), is_a("glmerMod"))
    rm("new")

    ## test issue #47, from Wolfgang Viechtbauer
    ## create some data
    n <- 100
    ai <- rep(0:1, each = n/2)
    bi <- 1-ai
    ci <- c(rep(0,42), rep(1,8), rep(0,18), rep(1,32))
    di <- 1-ci
    event <- c(rbind(ai,ci))
    group <- rep(c(1,0), times=n)
    id    <- rep(1:n, each=2)
    gm3 <- glmer(event ~ group + (1 | id), family=binomial, nAGQ=21)
    sd3 <- sqrt(diag(vcov(gm3)))
    expect_equal(uc(`(Intercept)` = 0.42542855, group = 0.42492581), sd3, tolerance=1e-5) # 7e-9 {Lnx}
    ## unfortunately these answers aren't reliably "wrong" any more (2024-07-02 Pop!OS Linux)
    expect_warning(vcov(gm3, use.hessian=FALSE), "finite-difference Hessian")
    expect_equal(suppressWarnings(sqrt(diag(vcov(gm3,use.hessian=FALSE)))),
                 uc(`(Intercept)` = 0.3840921, group = 0.3768747), tolerance=1e-7) # 6.5e-8
    expect_equal(sd3, unn(coef(summary(gm3))[,"Std. Error"]))
    ## test non-pos-def finite-difference Hessian ...
    if(getRversion() > "3.0.0") {
        ## saved fits are not safe with old R versions
        L <- load(system.file("testdata","polytomous_vcov_ex.RData",
                              package="lme4", mustWork=TRUE))
        expect_warning(vcov(polytomous_vcov_ex),"falling back to var-cov")
    }

    ## damage Hessian to make it singular
    ## (example thanks to J. Dushoff)
    gm1H <- gm1
    gm1H@optinfo$derivs$Hessian[5,] <- 0
    expect_warning(vcov(gm1H),"falling back to var-cov")

    ## test convergence warnings
    L <- load(system.file("testdata","gopherdat2.RData",
                          package="lme4", mustWork=TRUE))
    g0 <- glmer(shells~prev + (1|Site)+offset(log(Area)),
                family=poisson, data=Gdat)
    ## fit year as factor: OK
    gc <- glmerControl(check.conv.grad="stop")
    expect_is(update(g0,.~.+factor(year), control=gc), "glmerMod")
    ## error/warning with year as numeric:
    ## don't have full knowledge of which platforms lead to which
    ##     results, and can't detect whether we're running on valgrind,
    ##     which changes the result on 32-bit linux ...
    ## SEGFAULT on MacOS? why?
    if (FALSE) {
       expect_that(update(g0,.~.+year),
         gives_error_or_warning("(failed to converge|pwrssUpdate did not converge)"))
    }
    ## ("(failed to converge|pwrssUpdate did not converge in)"))
    ## if (sessionInfo()$platform=="i686-pc-linux-gnu (32-bit)") {
    ##     expect_warning(update(g0, .~. +year), "failed to converge")
    ## } else {
    ##     ## MacOS x86_64-apple-darwin10.8.0 (64-bit)
    ##     ## MM's platform
    ##     ## "pwrssUpdate did not converge in (maxit) iterations"
    ##     expect_error(update(g0, .~. +year), "pwrssUpdate did not converge in")
    ## }
    ## OK if we scale & center it
    expect_is(update(g0,.~. + scale(year), control=gc), "glmerMod")
    ## not OK if we scale and don't center
    expect_warning(update(g0,.~. + scale(year,center=FALSE)),
                   "failed to converge with max|grad|")
    ## OK if center and don't scale
    expect_is(update(g0,.~. + scale(year,center=TRUE,scale=FALSE),
                     control=gc),
              "glmerMod")

    ## try higher-order AGQ
    expect_is   (update(gm1,nAGQ=90), "glmerMod")
    expect_error(update(gm1,nAGQ=101),"nAGQ <= 100")

    ## non-numeric response variables
    ss <- transform(sleepstudy, Reaction = as.character(Reaction))
    expect_error(glmer(Reaction~(1|Days), family="poisson", data=ss),
                 "response must be numeric")
    expect_error(glmer(Reaction~(1|Days), family="binomial", data=ss),
                 "response must be numeric or factor")
    ss2 <- transform(ss,rr=rep(c(TRUE,FALSE),length.out=nrow(ss)))
    ## should work OK with logical too
    expect_is(glmer(rr~(1|Days),family="binomial",data=ss2),"merMod")

    ## starting values with log(.) link -- thanks to Eric Weese @ Yale:
    grp <- rep(letters[1:5], 20); set.seed(1); x <- rnorm(100)
    expect_error(glmer(x ~ 1 + (1|grp), family=gaussian(link="log")),
		 "valid starting values")

    ## related to GH 231
    ## fails on some platforms, skip for now
    if (FALSE) {
        rr <- gm1@resp$copy()
        ff <- setdiff(ls(gm1@resp),c("copy","initialize","initialize#lmResp","ptr",
                                     "updateMu","updateWts","resDev","setOffset","wrss"))
        for (i in ff) {
            expect_equal(gm1@resp[[i]],rr[[i]])
        }
    }

    ## bad start case
    load(system.file("testdata","fakesim.RData",package="lme4"))
    rfit <- glmer(Inew/S ~ R0-1 + offset(log(I/N)) + (1|R0:trial)
        , family=binomial(link="cloglog")
	, data=dat
	, weight=S
	, control=glmerControl(optimizer="bobyqa",
                               nAGQ0initStep=FALSE)
	, start = list(fixef=c(0,0,0),theta=1))
    expect_equal(exp(fixef(rfit)),
                 c(R01 = 1.2735051, R02 = 2.0330635, R03 = 2.9764088),
                 tolerance=1e-5)

    ## gaussian with log link and zero values in response ...
    ##  fixed simulation code, passing mustart properly
    dd <- expand.grid(x = seq(-2,3,length.out=10),
                      f = factor(1:10))
    dd$y <- simulate(~x+(1|f),
                 family=gaussian(link="log"),
                 newdata=dd,
                 newparams=list(beta=c(0,1),theta=1,sigma=1),
                 seed=101)[[1]]
    dd$y <- pmax(dd$y,0)
    expect_error(glmer (y ~ x + (1|f),
                        family = gaussian(link="log"),
                        data=dd),"cannot find valid starting values")
    g1 <- glmer (y ~ x + (1|f),
                 family = gaussian(link="log"),
                 data=dd,
                 mustart=pmax(dd$y,0.1))
    
    msum <- c(fixef(g1),unlist(c(VarCorr(g1))),c(logLik(g1)))
    expect_equal(msum,
                 c(`(Intercept)` = 0.23389405, x = 1.0017436, f = 0.3187655, -156.7773),
                 tolerance=1e-5)

    ## GH 415
    expect_warning(glmer (round(Reaction) ~ Days + (1|Subject),
                          data=sleepstudy[1:100,],
                          family=poisson,
                          control=lmerControl()),
                   "please use glmerControl")

})
} ## testlevel>1

test_that("glmer with etastart",
{
    ## make sure etastart is passed through
    ## (fixed in commit b6fb1ac83885ff06 but never tested?)
    m1 <- glmer(incidence/size ~ period + (1|herd),
                weights = size,
                family = binomial,
                data = cbpp)
    m1E <- update(m1, etastart = rep(1, nrow(cbpp)))
    expect_true(!identical(fixef(m1), fixef(m1E)))
})


test_that("turn off conv checking for nobs > check.conv.nobsmax", {
  ## calc derivs and check convergence
  gm1 <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
               data = cbpp, family = binomial)
  nn <- nrow(cbpp)-1
  ## neither derivs nor conv check
  gm2 <- update(gm1,
                control = glmerControl(check.conv.nobsmax = nn))
  ## no conv check, do calc derivs
  gm3 <- update(gm1, 
                control = glmerControl(check.conv.nobsmax = nn,
                                      calc.derivs = TRUE))
  expect_null(gm2@optinfo$derivs)
  expect_false(is.null(gm1@optinfo$derivs))
  expect_false(is.null(gm3@optinfo$derivs))
  expect_equal(gm1@optinfo$conv$lme4, list())
  expect_null(gm2@optinfo$conv$lme4)
  expect_null(gm3@optinfo$conv$lme4)
})

test_that("turn off conv checking for npara > check.conv.nparmax", {
  ## Code suggestion from Claude ai:
  ## https://claude.ai/share/06aa5947-f447-4e08-b65b-92f36c4b19a9
  set.seed(1)
  
  n_groups <- 50
  n_per_group <- 20
  n <- n_groups * n_per_group
  
  dat <- data.frame(
    group = rep(1:n_groups, each = n_per_group),
    x1 = rnorm(n),
    x2 = rnorm(n)
  )

  set.seed(101)
  form <- y ~ 1 + x1 * x2 + (1|group)
  dat$y <- simulate(form[-2], ## one-sided formula
                  newdata = dat,
                  family = binomial,
                  newparams = list(beta = c(-3, 2.5, 3, 1.5),
                                   theta = 2.5))[[1]]

  # note: maxfun had to be artificially low for convergence warnings...
  mod1 <- suppressWarnings(
    glmer(form, data = dat, family = binomial,
    control = glmerControl(optCtrl = list(maxfun = 100)))
  )
  
  mod2 <- suppressWarnings(
    update(mod1, control = glmerControl(optCtrl = list(maxfun = 100),
                                        check.conv.nparmax = 2))
  )

  ## First should give a warning
  expect_false(is.null(mod1@optinfo$conv$lme4))
  ## Second shouldn't be evaluated
  expect_null(mod2@optinfo$conv$lme4)
})

load(system.file("testdata","survdat_reduced.Rda",package="lme4"))

test_that('Step-halving works properly', {
  # this example is known to require step-halving (or at least has in the past
  # required step-halving)
  form <- survprop~(1|nobs)
  m <- glmer(form,weights=eggs,data=survdat_reduced,family=binomial,nAGQ=1L)
  expect_that(m, is_a("glmerMod"))
})

## read system file 
other_mod <- readRDS(
  system.file("testdata", "test-covariance_structures_data.rds", package = "lme4")
)
Contraception <- readRDS(
  system.file("testdata", "Contraception.rds", package = "lme4")
)

######################################################################
# Below is code frequently used for testing
# see: test-covariance_structures.R and test-covariance_nlmer.R

all.equal.nocheck <- function(x, y, ..., check.attributes = FALSE, check.class = FALSE) {
  require("Matrix", quietly = TRUE)
  ## working around mode-matching headaches
  if (is(x, "Matrix")) x <- matrix(x)
  if (is(y, "Matrix")) y <- matrix(y)
  all.equal(x, y, ..., check.attributes = check.attributes, check.class = check.class)
}

## set default tolerance to 5e-5 since we mostly use that
## 'tolerance' must be written out in full since it comes after ...
expect_equal_nocheck <- function(...,  tolerance = 5e-5) {
    aa <- all.equal.nocheck(..., tolerance = tolerance)
    if (!isTRUE(aa)) cat("tolerance: ", tolerance, "\n", aa, "\n")
    expect_true(isTRUE(aa))
}

## Getting all equal as a number (in the all.equal examples documentation;
## don't know why they didn't make an argument instead!?)
all.eqNum <- function(...) {
  an <- all.equal.nocheck(...)
  if (isTRUE(an)) return(0)
  ## if check is less than tolerance all.equal returns TRUE, so sub() coerces to "TRUE"
  ##  and as.numeric() returns NA ...
  as.numeric(sub(".*:", '', an))
}

test_that("unit test for unstructured covariances", {
  
  for (nc in c(0:4, 16L, 64L, 256L)){
    
    x.us <- new("Covariance.us", nc = nc, simulate = TRUE)
    
    ## Basic sanity tests
    expect_s4_class(x.us, "Covariance.us")
    expect_true(validObject(x.us))
    
    ## par should be the same as theta
    expect_equal(getPar(x.us), getTheta(x.us))
    
    ## checking for specific par/theta lengths
    expect_equal(getParLength(x.us), (nc * (nc+1)/2))
    expect_equal(getParLength(x.us), getThetaLength(x.us))
    expect_equal(length(getThetaIndex(x.us)), 
                 getThetaIndexLength(x.us))
    expect_equal(getLambdat.dp(x.us), seq_len(nc))
    
    ## Testing getUpper() and getLower() in a different manner
    expect_equal(getUpper(x.us),
                 rep(Inf, getThetaIndexLength(x.us)))
    
    if(nc > 1L){
      test_low <- rep(-Inf, nc * (nc + 1) / 2)
      diag_positions <- c(1, cumsum(nc:2) + 1)
      test_low[diag_positions] <- 0
    } else if (nc == 1L){
      test_low <- 0
    } else if (nc == 0L){
      test_low <- numeric(0)
    }
    expect_equal(getLower(x.us), test_low)
    
    ## Testing getLambdat.i in a different manner
    expect_equal(getLambdat.i(x.us),
      (row(matrix(0, nc, nc)) - 1)[upper.tri(matrix(0, nc, nc), diag = TRUE)])
    
    ## Testing getLambda in a different manner
    mat <- matrix(0, nc, nc)
    mat[upper.tri(mat, diag = TRUE)] <- getTheta(x.us)[getThetaIndex(x.us)]
    expect_equal(getLambda(x.us), t(mat))
   
    ## This is quite similar to what is in setMethod("getVC", ...) but using
    ## getPar() to make sure everything is working as planned
    ## TODO: think of a more creative method?
    if (nc <= 1L) {
      vcomp_test <- getPar(x.us)
      ccomp_test <- double(0L)
    } else {
      ii <- seq.int(from = 1L, by = nc + 1L, length.out = nc)
      i0 <- sequence.default(from = seq.int(from = 2L, by = nc + 1L, length.out = nc - 1L),
                             by = 1L,
                             nvec = (nc - 1L):1L)
      i1 <- sequence.default(from = ii,
                             by = 1L,
                             nvec = nc:1L)
      L <- matrix(0, nc, nc)
      L[i1] <- getPar(x.us)
      S <- tcrossprod(L)
      vcomp_test <- sqrt(S[ii])
      ccomp_test <- (S/vcomp_test/rep(vcomp_test, each = nc))[i0]
    }
    vc <- getVC(x.us)
    expect_equal(vc$vcomp, vcomp_test)
    expect_equal(vc$ccomp, ccomp_test)
    
    # Testing getProfPar
    expect_equal(c(vcomp_test, ccomp_test), getProfPar(x.us))
    expect_equal(c(vcomp_test*2, ccomp_test), getProfPar(x.us, sc = 2))
    expect_equal(c(rep(0, length(vc$vcomp)), rep(-1, length(vc$ccomp))), 
                   getProfLower(x.us))
    expect_equal(c(rep(Inf, length(vc$vcomp)), rep(1, length(vc$ccomp))), 
                 getProfUpper(x.us))
  }
})

test_that("unit tests for diagonal covariances", {
  
  for (nc in c(0:4, 16L, 64L, 256L)){
    for(hom_test in c(TRUE, FALSE)){
      x.di <- new("Covariance.diag", nc = nc, hom = hom_test, simulate = TRUE)
      
      ## Basic sanity tests
      expect_s4_class(x.di, "Covariance.diag")
      expect_true(validObject(x.di))
      
      ## par should be the same as theta
      expect_equal(getPar(x.di), getTheta(x.di))
      
      ## checking for specific par/theta lengths
      if(hom_test){
        expect_equal(getParLength(x.di), if (nc > 0L) 1 else 0)
      } else {
        expect_equal(getParLength(x.di), if (nc > 0L) nc else 0)
      }
      
      expect_equal(getParLength(x.di), getThetaLength(x.di))
      expect_equal(length(getThetaIndex(x.di)), 
                   getThetaIndexLength(x.di))
      
      expect_equal(getLambdat.dp(x.di), rep(1L, nc))
      
      ## Testing getUpper() and getLower() in a different manner
      if(hom_test){
        expect_equal(getUpper(x.di), if(nc > 0L) Inf else numeric(0))
        expect_equal(getLower(x.di), if(nc > 0L) 0 else numeric(0))
      } else {
        expect_equal(getUpper(x.di), if(nc > 0L) rep(Inf, nc) else numeric(0))
        expect_equal(getLower(x.di), if(nc > 0L) rep(0, nc) else numeric(0))
      }
      
      ## Testing getLambdat.i in a different manner
      expect_equal(getLambdat.i(x.di), 
                   if(nc > 0L) 0:(nc-1) else integer(0))
      
      ## Testing getLambda in a different manner
      expect_equal(getLambda(x.di),
                   diag(nc) * getPar(x.di))
      
      ## Testing getVC
      vc <- getVC(x.di)
      expect_equal(vc$ccomp, numeric(0))
      expect_equal(vc$vcomp, getPar(x.di))
      
      # Testing getProfPar
      expect_equal(vc$vcomp, getProfPar(x.di))
      expect_equal(vc$vcomp*2, getProfPar(x.di, sc = 2))
      expect_equal(vc$vcomp^2, getProfPar(x.di, profscale = "varcov"))
      
      expect_equal(rep(0, length(vc$vcomp)), getProfLower(x.di))
      expect_equal(rep(Inf, length(vc$vcomp)), getProfUpper(x.di))
      
    }
  }
})

test_that("unit tests for compound symmetry covariances", {
  
  for (nc in c(0:4, 16L, 64L, 256L)){
    for(hom_test in c(TRUE, FALSE)){
      x.cs <- new("Covariance.cs", nc = nc, hom = hom_test, simulate = TRUE)
      
      ## Basic sanity tests
      expect_s4_class(x.cs, "Covariance.cs")
      expect_true(validObject(x.cs))
      
      ## par should be the same as vcomp and ccomp from getVC
      expect_equal(getPar(x.cs), c(getVC(x.cs)$vcomp, getVC(x.cs)$ccomp))
      
      ## testing for getTheta via the structure of getLambda
      ## (odd way of looking at things, but it nicely compares the relationship
      ## between these two functions.)
      lam_mat <- getLambda(x.cs)
      collected <- 1
      if(nc > 0L){
        if(hom_test){ ## for homogenous structures only
          collect <- numeric(nc*2-1)
          collected <- 1
          for(i in 1:nc){
            if(i != nc){
              collect[collected:(collected+1)] <- lam_mat[i:(i+1),i]
            } else {
              collect[collected:collected] <- lam_mat[i,i]
            }
            collected = collected + 2
          }
        } else { ## for non-homogenous structures
          grab <- nc
          collect <- numeric((nc * (nc+1)/2))
          for(i in 1:nc){
            if(i != nc){
              collect[collected:(collected + (nc-i))] <- lam_mat[i:nc,i]
            } else {
              collect[collected:collected] <- lam_mat[i,i]
            }
            collected = collected + (nc - i + 1)
          }
        }
      } else {
        collect = numeric(0)
      }
      expect_equal(getTheta(x.cs), collect)
      
      ## Testing getParLength
      if(hom_test){ ## for homogenous, should be no more than 2
        if(nc <= 2L){
          expect_equal(getParLength(x.cs), nc)
        } else {
          expect_equal(getParLength(x.cs), 2L)
        }
      } else { ## Comparing length to vcomp and ccomp 
        expect_equal(getParLength(x.cs),
                     sum(lengths(getVC(x.cs)[c("vcomp", "ccomp")])))
      }
      
      expect_equal(getParLength(x.cs), length(getPar(x.cs)))
      expect_equal(getThetaLength(x.cs), length(getTheta(x.cs)))
      
      ## Testing getThetaLength
      expect_equal(length(getThetaIndex(x.cs)), 
                   getThetaIndexLength(x.cs))
      if(hom_test){
        expect_equal(getThetaLength(x.cs), if(nc > 0L) (nc*2 - 1) else 0)
      } else {
        expect_equal(getThetaLength(x.cs), (nc * (nc+1)/2))
      }
      
      expect_equal(getLambdat.dp(x.cs), if(nc > 0L) 1:nc else integer(0))
      
      ## Testing getUpper() and getLower()
      if(nc > 1L){
        if(hom_test) {
          expect_equal(getUpper(x.cs), c(Inf, 1))
          expect_equal(getLower(x.cs), c(0.0, -1/(nc-1)))
        } else {
          expect_equal(getUpper(x.cs), c(rep(Inf, nc), 1))
          expect_equal(getLower(x.cs), c(rep(0.0, nc), -1/(nc-1)))
        }
      } else if (nc == 1L) {
        expect_equal(getUpper(x.cs), Inf)
        expect_equal(getLower(x.cs), 0)
      } else {
        expect_equal(getUpper(x.cs), numeric(0))
        expect_equal(getLower(x.cs), numeric(0))
      }
      
      ## Testing getLambdat.i in a different manner
      expect_equal(getLambdat.i(x.cs), 
                   (row(matrix(0, nc, nc)) - 1)[upper.tri(matrix(0, nc, nc), diag = TRUE)])
      
      ## Testing getProfPar
      expect_equal(getPar(x.cs), getProfPar(x.cs))
      vc.cs <- getVC(x.cs)
      expect_equal(c(vc.cs$vcomp*2, vc.cs$ccomp), getProfPar(x.cs, sc = 2))
      
      expect_equal(c(rep(0, length(vc.cs$vcomp)), rep(-1/(nc-1), length(vc.cs$ccomp))), 
                   getProfLower(x.cs))
      expect_equal(c(rep(Inf, length(vc.cs$vcomp)), rep(1, length(vc.cs$ccomp))), 
                   getProfUpper(x.cs))
    }
  }
})

test_that("unit tests for autoregressive covariances", {
  
  for (nc in c(0:4, 16L, 64L, 256L)){
    for(hom_test in c(TRUE, FALSE)){
      x.ar1 <- new("Covariance.ar1", nc = nc, hom = hom_test, simulate = TRUE)
      
      ## Basic sanity tests
      expect_s4_class(x.ar1, "Covariance.ar1")
      expect_true(validObject(x.ar1))
      
      getVC_t <- getVC(x.ar1); vcomp_t <- getVC_t$vcomp; ccomp_t <- getVC_t$ccomp
      ## par should be the same as vcomp and ccomp from getVC
      expect_equal(getPar(x.ar1), c(vcomp_t, ccomp_t))
      
      ## getTheta test by relating to getVC
      v1 <- ccomp_t^(0L:(nc - 1L))
      v2 <- ccomp_t^(0L:(nc - 2L)) * sqrt(1 - ccomp_t^2)
      if(hom_test){
        if(nc > 1L){
          theta_test <- vcomp_t * c(v1, v2)
        } else {
          theta_test <- vcomp_t
        }
      } else {
        if(nc > 1L){
          theta_test <- vcomp_t[sequence.default(from = 1L:nc, nvec = nc:1L)] *
            c(v1, v2[sequence.default(from = 1L, nvec = (nc - 1L):1L)])
        } else {
          theta_test <- vcomp_t
        }
      }
      expect_equal(getTheta(x.ar1), theta_test)
      
      ## Testing for theta and par lengths
      expect_equal(getThetaLength(x.ar1), length(getTheta(x.ar1)))
      expect_equal(getParLength(x.ar1), length(getPar(x.ar1)))
      expect_equal(getParLength(x.ar1),
                   sum(lengths(getVC(x.ar1)[c("vcomp", "ccomp")])))
      
      lam_test <- matrix(0, nc, nc)
      if(nc > 0L){
        lam_test[lower.tri(lam_test, diag = TRUE)] <-
          if(hom_test){
            theta_test[sequence.default(from = rep(c(1L, nc + 1L), 
                                                   c(1L, nc - 1L)), nvec = nc:1L)]
          } else {
            theta_test
          }
      }
      expect_equal(getLambda(x.ar1), lam_test)
      
      expect_equal(getLambdat.dp(x.ar1), if(nc > 0L) 1:nc else integer(0))
      
      ## Testing getUpper and getLower
      if(nc > 1L){
        if(hom_test) {
          expect_equal(getUpper(x.ar1), c(Inf, 1))
          expect_equal(getLower(x.ar1), c(0, -1))
        } else {
          expect_equal(getUpper(x.ar1), c(rep(Inf, nc), 1))
          expect_equal(getLower(x.ar1), c(rep(0.0, nc), -1))
        }
      } else if (nc == 1L) {
        expect_equal(getUpper(x.ar1), Inf)
        expect_equal(getLower(x.ar1), 0)
      } else {
        expect_equal(getUpper(x.ar1), numeric(0))
        expect_equal(getLower(x.ar1), numeric(0))
      }
      
      ## Testing getLambdat.i in a different manner
      expect_equal(getLambdat.i(x.ar1), 
                   (row(matrix(0, nc, nc)) - 1)[upper.tri(matrix(0, nc, nc), diag = TRUE)])
      
      ## Testing getProfPar
      expect_equal(unname(unlist(getVC(x.ar1))), getProfPar(x.ar1))
      vc.ar1 <- getVC(x.ar1)
      expect_equal(c(vc.ar1$vcomp*2, vc.ar1$ccomp), getProfPar(x.ar1, sc = 2))
      
      expect_equal(c(rep(0, length(vc.ar1$vcomp)), rep(-1, length(vc.ar1$ccomp))), 
                   getProfLower(x.ar1))
      expect_equal(c(rep(Inf, length(vc.ar1$vcomp)), rep(1, length(vc.ar1$ccomp))), 
                   getProfUpper(x.ar1))
    }
  }
})

## lme4 linear mixed models

fm1 <- lmer(Reaction ~ Days + (Days | Subject), sleepstudy, REML = FALSE)
fm1.us <- lmer(Reaction ~ Days + us(Days | Subject), sleepstudy, REML = FALSE)
fm1.cs <- lmer(Reaction ~ Days + cs(Days | Subject), sleepstudy, REML = FALSE)
fm1.diag <- lmer(Reaction ~ Days + diag(Days | Subject), 
                 sleepstudy, REML = FALSE)
sleepstudy$Daysf <- factor(sleepstudy$Days, ordered = TRUE)
fm1.ar1 <- lmer(Reaction ~ Daysf + ar1(0 + Daysf | Subject, hom = TRUE), 
                sleepstudy, REML = FALSE)
fm1.ar1A <- lmer(Reaction ~ Daysf + ar1(0 + Daysf | Subject), 
                sleepstudy, REML = FALSE)
test_that("AR1 homogeneous by default", {
  expect_equal(getME(fm1.ar1A, "par"), getME(fm1.ar1, "par"))
})

fm1.REML <- lmer(Reaction ~ Days + (Days | Subject), sleepstudy)
fm1.us.REML <- lmer(Reaction ~ Days + us(Days | Subject), sleepstudy)
fm1.cs.REML <- lmer(Reaction ~ Days + cs(Days | Subject), sleepstudy)
fm1.diag.REML <- lmer(Reaction ~ Days + diag(Days | Subject), 
                      sleepstudy)
fm1.ar1.REML <- lmer(Reaction ~ Daysf + ar1(0 + Daysf | Subject, hom = TRUE), 
                sleepstudy)

## lme4 generalized linear mixed models
gm <- glmer(use ~ age + urban + (1 + urban | district),
            data = Contraception,
            family = binomial)
# unstructured
gm.us <- glmer(use ~ age + urban + us(1 + urban | district),
               data = Contraception,
               family = binomial)
# compound symmetry
gm.cs <- glmer(use ~ age + urban + cs(1 + urban | district),
               data = Contraception,
               family = binomial)
# diagonal
gm.diag <- glmer(use ~ age + urban + diag(1 + urban | district),
                 data = Contraception,
                 family = binomial)

test_that("integration tests for coef and fixef", {
  ## Ensuring unstructured covariance results are the same as default
  expect_equal(coef(fm1), coef(fm1.us))
  expect_equal(coef(fm1.REML), coef(fm1.us.REML))
  expect_equal(fixef(fm1), fixef(fm1.us))
  expect_equal(fixef(fm1.REML), fixef(fm1.us.REML))
  expect_equal(coef(gm), coef(gm.us))
  expect_equal(fixef(gm), fixef(gm.us))
  
  ## One of the expected summaries
  opt <- options(useFancyQuotes = FALSE)
  tmpf <- function(x) capture.output(print(summary(x),digits=1))
  tfun <- function(cc) {
    w <- grep("Fixed effects:", cc)
    cc[w:length(cc)]
  }
  expected_summary <- c("Fixed effects:",                         
                        "            Estimate Std. Error t value",
                        "(Intercept)      251          7      38",
                        "Days              10          2       7",
                        "",                                       
                        "Correlation of Fixed Effects:",          
                        "     (Intr)",                            
                        "Days -0.138")  
  expect_equal(tfun(tmpf(fm1)), expected_summary)
  
  expected_sum2 <- c("Fixed effects:",                                             
                     "            Estimate Std. Error z value Pr(>|z|)    ",       
                     "(Intercept)   -0.720      0.103      -7    3e-12 ***",       
                     "age            0.009      0.005       2     0.09 .  ",       
                     "urbanY         0.742      0.169       4    1e-05 ***",       
                     "---",      
                     "Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1",
                     "",                                  
                     "Correlation of Fixed Effects:",      
                     "       (Intr) age   ",            
                     "age    -0.022       ",      
                     "urbanY -0.648  0.006")
  expect_equal(tfun(tmpf(gm.us)), expected_sum2)
  options(opt)
})

test_that("integration tests for sigma", {
  ## Ensuring unstructured covariance results are the same as default
  expect_equal(sigma(fm1), sigma(fm1.us))
  expect_equal(sigma(fm1.REML), sigma(fm1.us.REML))
  expect_equal(sigma(gm), sigma(gm.us))
  
  ## Ensuring computing sigma is consistent for lme4 
  ## against other packages
  expect_equal_nocheck(sigma(fm1), other_mod$fm1.glmmTMB_sigma)
  expect_equal_nocheck(sigma(fm1.cs), other_mod$fm1.glmmTMB.cs_sigma)
  expect_equal_nocheck(sigma(fm1.diag), other_mod$fm1.glmmTMB.diag_sigma)
  expect_equal_nocheck(sigma(fm1.ar1), other_mod$fm1.glmmTMB.ar1_sigma)
  expect_equal_nocheck(sigma(fm1), other_mod$fm1.nlme_sigma)
  expect_equal_nocheck(sigma(fm1.cs), other_mod$fm1.nlme.cs_sigma)
  expect_equal_nocheck(sigma(fm1.REML), other_mod$fm1.nlme.REML_sigma)
  expect_equal_nocheck(sigma(fm1.cs.REML), other_mod$fm1.nlme.cs.REML_sigma)
  ## Tests for glmer
  expect_true(all.equal(sigma(gm.us), other_mod$gm.glmmTMB_sigma))
  expect_true(all.equal(sigma(gm.cs), other_mod$gm.glmmTMB.cs_sigma))
  expect_true(all.equal(sigma(gm.diag), other_mod$gm.glmmTMB.diag_sigma))
})

test_that("Log likelihood tests", {
  ## Note: when it comes to more complicated models such as GLMMs
  ## we see that the mean relative difference between fitted models from 
  ## different packages will be quite large.
  ## Idea: if log likelihoods are the same, then we're okay.
  expect_equal(logLik(fm1), logLik(fm1.us))
  expect_equal(logLik(fm1.REML), logLik(fm1.us.REML))
  expect_equal(logLik(gm), logLik(gm.us))
  
  ## comparing against glmmTMB
  expect_true(all.equal.nocheck(as.numeric(logLik(fm1)), 
                                other_mod$fm1.glmmTMB_logLik))
  expect_true(all.equal.nocheck(as.numeric(logLik(fm1.cs)), 
                                other_mod$fm1.glmmTMB.cs_logLik))
  expect_true(all.equal.nocheck(as.numeric(logLik(fm1.diag)), 
                                other_mod$fm1.glmmTMB.diag_logLik))
  expect_true(all.equal.nocheck(as.numeric(logLik(fm1.ar1)), 
                                other_mod$fm1.glmmTMB.ar1_logLik))
  
  ## comparing against nlme
  expect_true(all.equal.nocheck(logLik(fm1), other_mod$fm1.nlme_logLik))
  expect_true(all.equal.nocheck(logLik(fm1.cs), other_mod$fm1.nlme.cs_logLik))
  expect_true(all.equal.nocheck(logLik(fm1.REML), 
                                other_mod$fm1.nlme.REML_logLik))
  expect_true(all.equal.nocheck(logLik(fm1.cs.REML), 
                                other_mod$fm1.nlme.cs.REML_logLik))
  
  ## glmer
  expect_equal_nocheck(as.numeric(logLik(gm.us)), other_mod$gm.glmmTMB_logLik)
  expect_equal_nocheck(as.numeric(logLik(gm.cs)), other_mod$gm.glmmTMB.cs_logLik)
  expect_equal_nocheck(as.numeric(logLik(gm.diag)), other_mod$gm.glmmTMB.diag_logLik)
})

test_that("integration tests for vcov", {
  ## Ensuring unstructured covariance results are the same as default
  expect_equal(vcov(fm1), vcov(fm1.us))
  expect_equal(vcov(fm1.REML), vcov(fm1.us.REML))
  expect_equal(vcov(gm), vcov(gm.us))
  
  ## Ensuring variance-covariance matrix are consistent between lme4 
  ## and other packages
  expect_equal_nocheck(vcov(fm1), other_mod$fm1.glmmTMB_vcov)
  
  expect_equal_nocheck(vcov(fm1.cs), other_mod$fm1.glmmTMB.cs_vcov)
  expect_equal_nocheck(vcov(fm1.diag), other_mod$fm1.glmmTMB.diag_vcov)
  expect_equal_nocheck(vcov(fm1.ar1), other_mod$fm1.glmmTMB.ar1_vcov)
  expect_equal_nocheck(vcov(fm1), other_mod$fm1.nlme_vcov)
  expect_equal_nocheck(vcov(fm1.cs), other_mod$fm1.nlme.cs_vcov)
  expect_equal_nocheck(vcov(fm1.REML), other_mod$fm1.nlme.REML_vcov)
  ## larger tolerance; likelihood matched fairly well
  expect_equal_nocheck(as.numeric(vcov(fm1.cs.REML)), 
                         as.numeric(other_mod$fm1.nlme.cs.REML_vcov), tolerance = 2e-4)
  
  ## Tests for glmer
  ## The differences are somewhat large, however, the log likelihoods are
  ## quite similar... Leaving these with a larger tolerance.
  expect_equal_nocheck(vcov(gm.us), other_mod$gm.glmmTMB_vcov, tolerance = 3e-3)
  expect_equal_nocheck(vcov(gm.cs), other_mod$gm.glmmTMB.cs_vcov, tolerance = 3e-3)
  expect_equal_nocheck(vcov(gm.diag), other_mod$gm.glmmTMB.diag_vcov, tolerance = 3e-3)
})

test_that("integration tests for VarCorr", {
  ## Ensuring unstructured covariance results are the same as default
  expect_equal(VarCorr(fm1), VarCorr(fm1.us))
  expect_equal(VarCorr(fm1.REML), VarCorr(fm1.us.REML))
  expect_equal(VarCorr(gm), VarCorr(gm.us))
  
  ## Ensuring variance components are consistent for lme4 
  ## against other packages
  x1 <- c(as.matrix(VarCorr(fm1)[[1]]))
  x2 <- c(as.matrix(VarCorr(fm1.us)[[1]]))
  
  ## Testing unstructured
  expect_equal_nocheck(x1, c(other_mod$fm1.glmmTMB_var))
  expect_equal_nocheck(x2, c(other_mod$fm1.glmmTMB.us_var))
  expect_equal_nocheck(c(as.matrix(VarCorr(fm1.REML)[[1]])), 
                         c(other_mod$fm1.nlme.REML_var))
  
  expect_equal_nocheck(x2, c(other_mod$fm1.nlme_var), tolerance = 5e-4)
  
  ## Testing cs (compound symmetry)
  x3 <- c(as.matrix(VarCorr(fm1.cs)[[1]]))
  expect_equal_nocheck(x3, c(other_mod$fm1.glmmTMB.cs_var), tolerance = 5e-4)
  expect_equal_nocheck(x3, c(other_mod$fm1.nlme.cs_var), tolerance = 5e-4)
  
  x3.REML <- c(as.matrix(VarCorr(fm1.cs.REML)[[1]]))
  z3.REML <- c(other_mod$fm1.nlme.cs.REML_var)
  expect_equal_nocheck(x3.REML, z3.REML, tolerance = 5e-4)
  
  ## Testing diag
  expect_equal_nocheck(c(as.matrix(VarCorr(fm1.diag)[[1]])), 
                         c(other_mod$fm1.glmmTMB.diag_var))
  ## Testing ar1
  expect_equal_nocheck(c(as.matrix(VarCorr(fm1.ar1)[[1]])), 
                         c(other_mod$fm1.glmmTMB.ar1_var))
  
  ## glmer
  ## Similar to before; likelihoods are quite similar, so leaving these
  ## with a higher tolerance...
  expect_equal_nocheck(c(as.matrix(VarCorr(gm.us)[[1]])), 
                         c(other_mod$gm.glmmTMB_var), tolerance = 5e-4)
  expect_equal_nocheck(c(as.matrix(VarCorr(gm.cs)[[1]])), 
                         c(other_mod$gm.glmmTMB.cs_var), tolerance = 5e-4)
  expect_equal_nocheck(c(as.matrix(VarCorr(gm.us)[[1]])), 
                         c(other_mod$gm.glmmTMB_var), tolerance = 5e-4)
})

test_that("integration tests for ranef", {
  ## Ensuring unstructured covariance results are the same as default
  expect_equal(ranef(fm1), ranef(fm1.us))
  expect_equal(ranef(fm1.REML), ranef(fm1.us.REML))
  expect_equal(ranef(gm), ranef(gm.us))
  
  ## Ensuring extracting random modes of the random effects 
  ## are consistent for lme4 against other packages
  expect_true(all.equal.nocheck(other_mod$fm1.glmmTMB_ranef$cond$Subject,
                        ranef(fm1)$Subject))
  expect_true(all.equal.nocheck(other_mod$fm1.glmmTMB.cs_ranef$cond$Subject,
                        ranef(fm1.cs)$Subject))
  expect_true(all.equal.nocheck(other_mod$fm1.glmmTMB.diag_ranef$cond$Subject,
                        ranef(fm1.diag)$Subject))
  expect_true(all.equal.nocheck(other_mod$fm1.glmmTMB.ar1_ranef$cond$Subject,
                        ranef(fm1.ar1)$Subject))

  expect_equal_nocheck(as.matrix(other_mod$fm1.nlme_ranef), 
                         as.matrix(ranef(fm1)$Subject))
  expect_equal_nocheck(as.matrix(other_mod$fm1.nlme.REML_ranef), 
                         as.matrix(ranef(fm1.REML)$Subject))
  expect_equal_nocheck(as.matrix(other_mod$fm1.nlme.cs_ranef), 
                         as.matrix(ranef(fm1.cs)$Subject))
  expect_equal_nocheck(as.matrix(other_mod$fm1.nlme.cs.REML_ranef), 
                         as.matrix(ranef(fm1.cs.REML)$Subject))
  
  ## glmer
  expect_true(all.equal.nocheck(other_mod$gm.glmmTMB_ranef$cond$district,
                         ranef(gm.us)$district))
  expect_true(all.equal.nocheck(other_mod$gm.glmmTMB.cs_ranef$cond$district,
                         ranef(gm.cs)$district))
  expect_true(all.equal.nocheck(other_mod$gm.glmmTMB.diag_ranef$cond$district,
                         ranef(gm.diag)$district))

})

test_that("integration tests for predict", {
  ## Ensuring unstructured covariance results are the same as default
  expect_equal(predict(fm1), predict(fm1.us))
  expect_equal(predict(fm1.REML), predict(fm1.us.REML))
  expect_equal(predict(gm), predict(gm.us))
  ## Ensuring extracting predictions are consistent for 
  ## lme4 against other packages
  
  expect_equal_nocheck(other_mod$fm1.glmmTMB_predict, predict(fm1))
  expect_equal_nocheck(other_mod$fm1.glmmTMB.cs_predict, predict(fm1.cs))
  expect_equal_nocheck(other_mod$fm1.glmmTMB.diag_predict, predict(fm1.diag))
  expect_equal_nocheck(other_mod$fm1.glmmTMB.ar1_predict, predict(fm1.ar1))
  expect_equal_nocheck(other_mod$fm1.nlme_predict, predict(fm1))
  expect_equal_nocheck(other_mod$fm1.nlme.REML_predict, predict(fm1.REML))
  expect_equal_nocheck(other_mod$fm1.nlme.cs_predict, predict(fm1.cs))
  expect_equal_nocheck(other_mod$fm1.nlme.cs.REML_predict, predict(fm1.cs.REML))
  
  ## glmer
  expect_equal_nocheck(other_mod$gm.glmmTMB_predict, predict(gm.us))
  expect_equal_nocheck(other_mod$gm.glmmTMB.cs_predict, predict(gm.cs))
  expect_equal_nocheck(other_mod$gm.glmmTMB.diag_predict, predict(gm.diag))
})

simfun_gamma <- function(ngrp = 50, nrep = 50, shape_gam = 2, intercept = 1, 
                         theta_val = 2, seed = NULL) {
  if (!is.null(seed)) set.seed(seed)
  dd <- expand.grid(group = 1:ngrp, rep = 1:nrep)
  dd$y <- simulate(~ 1 + (1 | group), newdata = dd, 
                   family = Gamma(link = "log"), 
                   newparams = list(
                     theta = theta_val, beta = 1, 
                     sigma = 1/sqrt(shape_gam)))[[1]]
  dd
}

simfun_pois <- function(ngrp = 50, nrep = 50, intercept = 1,
                        theta_val = 1, seed = NULL) {
  if (!is.null(seed)) set.seed(seed)
  
  dd <- expand.grid(group = 1:ngrp, rep = 1:nrep)
  
  dd$y <- simulate(~ 1 + (1 | group), newdata = dd,
                   family = poisson(link = "log"), 
                   newparams = list(theta = theta_val, beta = intercept))[[1]]
  dd
}

dd2 <- simfun_gamma(seed = 101)

dd3 <- simfun_pois(seed = 101)

glmer1 <- glmer(y ~ 1 + (1|group), family = Gamma(link = "log"), data = dd2)
lme4_v1 <- VarCorr(glmer1)$group

glmer2 <- glmer(y ~ 1 + (1|group), family = poisson(link = "log"), data = dd3)
lme4_v2 <- VarCorr(glmer2)$group

test_that("correct stdevs for glmm", {
  # high tolerance, but the numbers are relatively similar.
  expect_equal_nocheck(lme4_v1, other_mod$TMB_v1, tolerance = 1e-2)
  expect_equal_nocheck(attr(lme4_v1, 'stddev'), 
                       attr(other_mod$TMB_v1, 'stddev'), tolerance = 5e-3)
  expect_equal_nocheck(attr(lme4_v1, 'correlation'), 
                       attr(other_mod$TMB_v1, 'correlation'))
})

test_that("'xst' length matches objective function argument length", {
    m <- glmer(incidence/size ~ 1 + ar1(0 + period | herd),
               weights = size,
               data = cbpp,
               family = binomial,
               control = glmerControl(check.nobs.vs.nRE = "ignore"))
    ## Error ....: length(xst <- as.numeric(xst)) == n is not TRUE
    expect_s4_class(m, "glmerMod")
})

test_that("stop on unspecified sigma", {
    expect_error(simulate( ~ 1 + (1 | Subject),
             newdata = sleepstudy,
             newparams = list(beta = 0, theta = 1)),
             "must specify sigma")
})
test_that("stop on unspecified sigma, two-sided formula + family=gaussian", {
    expect_error(simulate(Reaction ~ Days + (Days | Subject),
             newdata = sleepstudy,
             family = gaussian,
             newparams = list(beta = c(250, 10), theta = 1)),
             "must specify sigma")
})
test_that("predict() on an already-fitted object may omit sigma from newparams", {
    ## unlike de novo simulation from a formula (above), setParams()
    ## should *not* require sigma when modifying an already-fitted
    ## object's parameters: the existing sigma is retained. This is
    ## relied upon by glmertree's internal backfitting, which calls
    ## predict(lme, newdata=data, newparams=list(beta=b)) without sigma.
    fm1 <- lmer(Reaction ~ Days + (Days | Subject), sleepstudy)
    b <- fixef(fm1)
    b[] <- 0
    expect_no_error(
        suppressWarnings(suppressMessages(
            predict(fm1, newdata = sleepstudy, newparams = list(beta = b))
        ))
    )
})
test_that("correct ordering of cov strucs", {
    set.seed(101)
    dd <- expand.grid(f1 = 1:10, f2 = 1:20, rep = 1:10)
    dd$f <- factor(sample(1:4, size = nrow(dd), replace = TRUE))
    form <- y ~ 1 + cs(f | f1, hom = TRUE) + diag(f | f2)
    dd$y <- simulate( form[-2],
                     newdata = dd,
                     newparams = list(beta = 0,
                                      theta = rep(0.5, 11),
                                      sigma = 1))[[1]]
    fit <- lmer(form, data = dd)
    vv <- VarCorr(fit)
    expect_equal(class(vv[["f1"]]), c("vcmat_homcs", "matrix", "array"))
    expect_equal(class(vv[["f2"]]), c("vcmat_diag", "matrix", "array"))
})

library("lattice")
testLevel <- if (nzchar(s <- Sys.getenv("LME4_TEST_LEVEL"))) as.numeric(s) else 1

## use old (<=3.5.2) sample() algorithm if necessary
if ("sample.kind" %in% names(formals(RNGkind))) {
    suppressWarnings(RNGkind("Mersenne-Twister", "Inversion", "Rounding"))
}

do.plots <- TRUE

L <- load(system.file("testdata/lme-tst-fits.rda",
                      package="lme4", mustWork=TRUE))

if (getRversion() > "3.0.0") {
    ## saved fits are not safe with old R versions
    gm1 <- fit_cbpp_1
    fm1 <- fit_sleepstudy_1
    fm2 <- fit_sleepstudy_2
    fm3 <- fit_penicillin_1
    fm4 <- fit_cake_1
} else {
    gm1 <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
                 data = cbpp, family = binomial)
    fm1 <- lmer(Reaction ~ Days + (1|Subject), sleepstudy)
    fm2 <- lmer(Reaction ~ Days + (Days|Subject), sleepstudy)
    fm3 <- lmer(diameter ~ (1|plate) + (1|sample), Penicillin)
    fm4 <- lmer(angle ~ temp + recipe + (1 | replicate), data=cake)
}

if (testLevel>1) {
#context("predict")
test_that("fitted values", {
    p0 <- predict(gm1)
    p0B <- predict(gm1, newdata=cbpp)
    expect_equal(p0, p0B, tolerance=2e-5) ## ? not sure why high tolerance necessary ? works OK on Linux/R 3.5.0

    ## fitted values, unconditional (level-0)
    p1 <- predict(gm1, re.form=NA)
    expect_true(length(unique(p1))==length(unique(cbpp$period)))

    ## fitted values, random-only
    p1R <- predict(gm1, random.only=TRUE)
    expect_equal(p1+p1R,p0)

    if (do.plots) matplot(cbind(p0,p1),col=1:2,type="b")

    ## neither fixed nor random -- all zero
    expect_equal(unique(predict(gm1,re.form=NA,random.only=TRUE)),0)

})

test_that("predict with newdata", {

    newdata <- with(cbpp, expand.grid(period=unique(period),herd=unique(herd)))
    ## new data, all RE
    p2 <- predict(gm1,newdata)
    ## new data, level-0
    p3 <- predict(gm1,newdata, re.form=NA)
    p3R <- predict(gm1,newdata, random.only=TRUE)
    expect_equal(p3+p3R,p2)

    if (do.plots) matplot(cbind(p2,p3),col=1:2,type="b")

})

test_that("predict on response scale", {
    p0 <- predict(gm1)
    p5 <- predict(gm1,type="response")
    expect_equal(p5, plogis(p0))
})

test_that("predict with newdata and RE", {

    newdata <- with(cbpp,expand.grid(period=unique(period),herd=unique(herd)))
    ## explicitly specify RE
    p2 <- predict(gm1,newdata)
    p4 <- predict(gm1,newdata, re.form=~(1|herd))
    expect_equal(p2, p4)

})

test_that("effects of new RE levels", {

    newdata <- with(cbpp, expand.grid(period=unique(period), herd=unique(herd)))
    newdata2 <- rbind(newdata,
                      data.frame(period=as.character(1:4), herd=rep("new",4)))
    expect_error(predict(gm1, newdata2), "new levels detected in newdata: new")

    newdata3 <- rbind(newdata,
                      data.frame(period=as.character(1),
                                 herd = paste0("new", 1:50)))

    expect_error(predict(gm1, newdata3),
                 "new levels detected in newdata: new1, new2,")


    p2 <- predict(gm1, newdata)
    p6 <- predict(gm1, newdata2, allow.new.levels=TRUE)
    expect_equal(p2, p6[1:length(p2)])  ## original values should match
    ## last 4 values should match unconditional values
    expect_true(all(tail(p6,4) ==
                        predict(gm1, newdata=data.frame(period=factor(1:4)), re.form=NA)))
})

test_that("multi-group model", {

    ## fitted values
    p0 <- predict(fm3)
    expect_equal(head(round(p0,4)),
                 c(`1` = 25.9638, `2` = 22.7663, `3` = 25.7147, `4` = 23.6799,
                   `5` = 23.7629, `6` = 20.773))
    ## fitted values, unconditional (level-0)
    p1 <- predict(fm3, re.form=NA)
    expect_equal(unique(p1),22.9722222222251)

    if (do.plots) matplot(cbind(p0,p1),col=1:2,type="b")
})

test_that("multi-group model with new data", {

    newdata <- with(Penicillin,expand.grid(plate=unique(plate),sample=unique(sample)))
    ## new data, all RE
    p2 <- predict(fm3, newdata)
    ## new data, level-0
    p3 <- predict(fm3, newdata, re.form=NA)
    ## explicitly specify RE
    p4 <-  predict(fm3, newdata, re.form= ~(1|plate)+(~1|sample))
    p4B <- predict(fm3, newdata, re.form= ~(1|sample)+(~1|plate)) ## ****
    expect_equal(p2,p4)
    expect_equal(p4,p4B)

    p5 <- predict(fm3,newdata, re.form=~(1|sample))
    p6 <- predict(fm3,newdata, re.form=~(1|plate))
    if (do.plots) matplot(cbind(p2,p3,p5,p6),type="b",lty=1,pch=16)
})

test_that("random-slopes model", {
    p0 <- predict(fm2)
    p1 <- predict(fm2, re.form=NA)
    ## linear model, so results should be identical patterns but smaller --
    ##   not including intermediate days
    newdata <- with(sleepstudy,expand.grid(Days=range(Days),Subject=unique(Subject)))
    newdata$p2 <- predict(fm2,newdata)
    newdata$p3 <- predict(fm2,newdata, re.form=NA)
    newdata$p4 <- predict(fm2,newdata, re.form=~(0+Days|Subject))
    newdata$p5 <- predict(fm2,newdata, re.form=~(1|Subject))

    ## reference values from an apparently-working run
    refval <- structure(
        list(Days = c(0, 9, 0, 9, 0, 9),
             Subject = structure(c(1L, 1L, 2L, 2L, 3L, 3L),
                                 levels = c("308", "309", "310", "330", "331", "332",
                                            "333", "334", "335", "337", "349", "350",
                                            "351", "352", "369", "370", "371", "372"), class = "factor"),
             p2 = c(253.663652396798, 430.66001930835, 211.006415533628, 227.634788908917, 212.444742696829, 257.61053840953),
             p3 = c(251.405104848485, 345.610678484848, 251.405104848485, 345.610678484848, 251.405104848485, 345.610678484848),
             p4 = c(251.405104848485, 428.401471760037, 251.405104848485, 268.033478223774, 251.405104848485, 296.570900561186),
             p5 = c(253.663652396798, 347.869226033161, 211.006415533628, 305.211989169991, 212.444742696829, 306.650316333193)),
        out.attrs =
            list(dim = c(Days = 2L, Subject = 18L),
                 dimnames = list(
                     Days = c("Days=0", "Days=9"),
                     Subject = c("Subject=308", "Subject=309", "Subject=310", "Subject=330", "Subject=331", "Subject=332",
                                 "Subject=333", "Subject=334", "Subject=335", "Subject=337", "Subject=349", "Subject=350",
                                 "Subject=351", "Subject=352", "Subject=369", "Subject=370", "Subject=371", "Subject=372")) ),
        row.names = c(NA, 6L), class = "data.frame")

    expect_equal(head(newdata), refval, tol=5e-7)
})

test_that("predict and plot random slopes", {

    tmpf <- function(data,...) {
        data$Reaction <- predict(fm2,data,...)
        if (do.plots) xyplot(Reaction~Days,group=Subject,data=data,type="l")
        return(unname(head(round(data$Reaction,3))))
    }
    expect_equal(tmpf(sleepstudy),c(253.664, 273.33, 292.996, 312.662, 332.329, 351.995))
    expect_equal(tmpf(sleepstudy, re.form=NA), c(251.405, 261.872, 272.34, 282.807, 293.274, 303.742))
    expect_equal(tmpf(sleepstudy, re.form= ~(0+Days|Subject)),
                 c(251.405, 271.071, 290.738, 310.404, 330.07, 349.736))
    expect_equal(tmpf(sleepstudy, re.form= ~(1|Subject)),
                 c(253.664, 264.131, 274.598, 285.066, 295.533, 306))

})

test_that("fewer random effect levels than original", {
    ## from 'Colonel Triq'
    summary(fm4)

    ## replicate 1 only appears in rows 1:18.
    ## rownames(cake[cake$replicate==1,])

    predict(fm4, newdata=cake[-1:-17,], re.form=~ (1 | replicate))
    predict(fm4, newdata=cake[-1:-18,], re.form=NA)
    predict(fm4, newdata=cake[-1:-18,], re.form=~ (1 | replicate))
    predict(fm4, newdata=cake[-1:-18,], re.form=~ (1 | replicate), allow.new.levels=TRUE)
    ##

    p0 <- predict(fm1,newdata=data.frame(Days=6,Subject=c("308","309")))
    p1 <- predict(fm1,newdata=data.frame(Days=rep(6,4),
                      Subject=c("308","309")))
    expect_equal(rep(unname(p0),2),unname(p1))
    p2 <- predict(fm1,newdata=data.frame(Days=6,Subject="308"))
    nd <- data.frame(Days=6,
                     Subject=factor("308",levels=levels(sleepstudy$Subject)))
    p3 <- predict(fm1,newdata=nd)
    expect_equal(p2,p3)
    expect_equal(p2,p0[1])
})

test_that("only drop columns when using new data", {
    ## Stack Overflow 34221564:
    ##  should only drop columns from model matrix when using *new* data
    ## NB: Fit depends on optimizer somewhat: "nloptwrap" is really better than "bobyqa"
    library(splines)
    sleep <- sleepstudy  #get the sleep data
    set.seed(1234567)
    sleep$age <- as.factor(sample(1:3,length(sleep),rep=TRUE))
    form1 <- Reaction ~ Days + ns(Days, df=4) +
        age + Days:age + (Days | Subject)
    m4 <- lmer(form1, sleep) # fixed-effect model matrix is rank deficient so dropping 1 column / coefficient
    expect_lte(REMLcrit(m4), 1713.171) # FIXME !? why this regression??  had 1700.6431;  "bobyqa" gave  1713.171
    expect_equal(unname(head(predict(m4, re.form=NA))),
                 c(255.203, 259.688, 265.71, 282.583, 294.784, 304.933),
                 tolerance = 0.008)
})

test_that("only look for columns that exist in re.form", {
    ## GH 457
    set.seed(101)
    n <- 200
    dd <- data.frame(x=1:n,
                     f=factor(rep(1:10,n/10)),
                     g=factor(rep(1:20,each=n/20)),
                     h=factor(rep(1:5,n/5)),
                     y=rnorm(n))
    m1 <- lmer(y~1 + f + (1|h/f) + (poly(x,2)|g), data=dd, control=lmerControl(calc.derivs=FALSE))
    expect_equal(unname(predict(m1,re.form= ~1 | h/f,       newdata=dd[1,])), 0.14786, tolerance=1e-4)
    expect_equal(unname(predict(m1,re.form= ~poly(x,2) | g, newdata=dd[1,])),
                 0.1533, tolerance=.001)
    ## *last* RE not included (off-by-one error)
    m1B <- lmer(y~1 + f + (1|g) + (1|h), data=dd, control=lmerControl(calc.derivs=FALSE, check.conv.singular = "ignore"))
    expect_equal(unname(predict(m1B,re.form=~(1|g),newdata=data.frame(f="1",g="2"))),0.1512895,tolerance=1e-5)
    set.seed(101)
    n <- 100
    xx <- c("r1", "r2", "r3", "r4", "r5")
    xxx <- c("e1", "e2", "e3")
    p <- 0.3
    School <- factor(sample(xxx, n, replace=TRUE), levels=xxx, ordered=FALSE)
    Rank <-   factor(sample(xx,  n, replace=TRUE), levels=xx,  ordered=FALSE)
    df1 <- data.frame(
        ID = as.integer(runif(n, min = 1, max = n/7)),
        xx1 = runif(n, min = 0, max = 10),
        xx2 = runif(n, min = 0, max = 10),
        xx3 = runif(n, min = 0, max = 10),
        School,
        Rank,
        yx = as.factor(rbinom(n, size = 1, prob = p))
    )
    df1 <- df1[order(df1$ID, decreasing=FALSE),]
    mm2 <- glmer(yx ~ xx1 + xx2 + xx3 + Rank +  (1 | ID) + (1 | School / Rank),
                 data = df1,
                 family = "binomial",control = glmerControl(calc.derivs =FALSE))
    n11 <-  data.frame(School= factor("e1", levels = levels(df1$School),ordered=FALSE),
                       Rank  = factor("r1", levels = levels(df1$Rank),  ordered=FALSE),
                       xx1=8.58, xx2=8.75, xx3=7.92)

    expect_equal(unname(predict(mm2, n11, type="response",re.form= ~(1 | School / Rank))),
                 0.1174628,tolerance=1e-5)

    ## bad factor levels
    mm3 <- update(mm2, . ~ . - (1|ID))
    n12 = data.frame(School="e3",Rank="r2",xx1=8.58,xx2=8.75,xx3=7.92)
    expect_equal(unname(predict(mm3, n12, type="response")),0.1832894,tolerance=1e-5)

    ## GH #452
    ## FIXME: would like to find a smaller/faster example that would test the same warning (10+ seconds)
    set.seed(101)
    n <- 300
    df2 <- data.frame(
        xx1 = runif(n, min = 0, max = 10),
        xx2 = runif(n, min = 0, max = 10),
        xx3 = runif(n, min = 0, max = 10),
        School = factor(sample(xxx, n,replace=TRUE)),
        Rank = factor(sample(xx, n, replace=TRUE)),
        yx = as.factor(rbinom(n, size = 1, prob = p))
    )
    mm4 <- suppressWarnings(glmer(yx ~ xx1 + xx2 + xx3 + Rank +  (Rank|School),
                 data = df2,
                 family = "binomial",control = glmerControl(calc.derivs =FALSE)))

    ## set tolerance to 0.1 (!) to pass win-builder on R-devel/i386 (only:
    ## tolerance = 3e-5 is OK for other combinations of (R-release, R-devel) x (i386,x64)
    expect_equal(unname(predict(mm4, n11, type="response")), 0.2675081, tolerance=0.1)
})

test_that("simulation works with non-factor", {

    set.seed(12345)
    dd <- data.frame(a=gl(10,100), b = rnorm(1000))
    test2 <- suppressMessages(simulate(~1+(b|a), newdata=dd, family=poisson,
                                       newparams= list(beta = c("(Intercept)" = 1),
                                                       theta = c(1,1,1))))
    expect_is(test2,"data.frame")
})


set.seed(666)
n <- 500
df <- data.frame(y=statmod::rinvgauss(n, mean=1, shape=2),
                 id=factor(1:20))
model_fit <- glmer(y ~ 1 + (1|id),
                   family = inverse.gaussian(link = "inverse"),
                   data = df,
                   control=glmerControl(check.conv.singular="ignore"))

test_that("simulation works for inverse gaussian", {
    expect_equal(mean(simulate(model_fit)[[1]]), 1.02704392575914,
                 tolerance=1e-5)
})

test_that("simulation complains appropriately about bad family", {
    ig <- inverse.gaussian()
    ig$family <- "junk"
    model_fit2 <- glmer(y ~ 1 + (1|id),
                        family = ig,
                        data = df,
                        control=glmerControl(check.conv.singular="ignore"))
    expect_error(simulate(model_fit2),"simulation not implemented for family")
})

test_that("prediction from large factors", {
    set.seed(101)
    N <- 50000
    X <- data.frame(y=rpois(N, 5), obs=as.factor(1:N))
    fm <- glmer(y ~ (1|obs), family="poisson", data=X,
                control=glmerControl(check.conv.singular="ignore"))
    ## FIXME: weak tests.  The main issue here is that these should
    ##  be reasonably speedy and non-memory-hogging, but those are
    ## hard to test portably ...
    expect_is(predict(fm, re.form=~(1|obs)), "numeric")
    expect_is(predict(fm, newdata=X), "numeric")
})

test_that("prediction with gamm4", {
    if (suppressWarnings(requireNamespace("gamm4"))) {
        ## loading gamm4 warngs "replacing previous import 'Matrix::update' by 'lme4::update' when loading 'gamm4'"
        ## from ?gamm4
        set.seed(0)
         ## simulate 4 term additive truth
        dat <- mgcv::gamSim(1,n=400,scale=2,verbose=FALSE)
        ## Now add 20 level random effect `fac'...
        dat$fac <- fac <- as.factor(sample(1:20,400,replace=TRUE))
        dat$y <- dat$y + model.matrix(~fac-1)%*%rnorm(20)*.5
        br <- gamm4::gamm4(y~s(x0)+x1+s(x2),data=dat,random=~(1|fac))
        expect_warning(ss <- simulate(br$mer), "modified RE names")
        expect_equal(dim(ss), c(400,1))
    }
})

test_that("prediction with spaces in variable names", {
    cbpp$`silly period` <- cbpp$period
    m <- glmer(cbind(incidence,size-incidence) ~ `silly period` + (1|herd),
               family=binomial, data=cbpp)
    expect_equal(round(head(predict(m)),3),
                 c(`1` = -0.809, `2` = -1.801,
                   `3` = -1.937, `4` = -2.388, `5` = -1.697, `6` = -2.689))
})

if (requireNamespace("statmod")) {
  test_that("simulate with rinvgauss", {
    dd <- data.frame(f=factor(rep(1:20,each=10)))
    dd$y <- simulate(~1+(1|f),
                     seed=101,
                     family=inverse.gaussian,
                     newdata=dd,
                     ## ?? gives NaN (sqrt(eta)) for low beta ?
                     newparams=list(beta=5,theta=1,sigma=1))[[1]]
    suppressMessages(m <- glmer(y~1+(1|f),
                                family=inverse.gaussian,
                                data=dd))
    set.seed(101)
    expect_equal(head(unlist(simulate(m))),
                 c(sim_11 = 0.451329390087728, sim_12 = 0.629516371309772,
                   sim_13 = 0.481236633500098,
                   sim_14 = 0.170060386109077, sim_15 = 0.258742371516342,
                   sim_16 = 0.949617440586848))
  })
}

## GH 631

test_that("sparse contrasts don't mess up predict()", {
  dd <- expand.grid(f = factor(1:101), rep1 = factor(1:2), rep2 = 1:2)
  dd$y <- suppressMessages(simulate(~1 + (rep1|f),
                   seed = 101,
                   newdata = dd,
                   newparams = list(beta = 1,
                                    theta = rep(1,3),
                                    sigma = 1),
                   family = gaussian)[[1]])
  m1 <- lmer( y ~ 1 + (1|f), data = dd)
  p1 <- predict(m1)
  p2 <- predict(m1, newdata = dd)
  expect_identical(p1, p2)
})
} ## testLevel > 1
test_that("prediction with . in formula + newdata",
{
  set.seed(101)
  mydata <- data.frame(
      groups = rep(1:3, each = 100),
      x = rnorm(300),
      dv = rnorm(300)
  )
  train_subset <- sample(1:300, 300 * .8)
  train <- mydata[train_subset,]
  test <- mydata[-train_subset,]
  mod <- lmer(dv ~ . - groups + (1 | groups), data = train)
  p1 <- predict(mod, newdata = test)
  mod2 <- lmer(dv ~ x + (1 | groups), data = train)
  p2 <- predict(mod2, newdata = test)
  expect_identical(p1, p2)
})

test_that("simulate with a factor with one level", {
    set.seed(1241)
    y <- factor(c(rep(0,1000), 1, rep(0,1000), 1), levels = c("0","1"))
    x <- rep(c("A","B"),each = 1001)
    mod <- glmer(y ~ 1 + (1|x),family = binomial,
                 control = glmerControl(check.conv.singular = "ignore"))
    s <- simulate(mod,newdata = data.frame(x = "A"), nsim = 10)
    ## very low mean, all simulated values zero
    expect_true(all(s == 0))
})


test_that("prediction standard error", {
  # note that predict.lm returns a list with 
  # fit, se.fit, df, residual.scale
  
  mod1 <- lmer(Petal.Width ~ Sepal.Length + (1 | Species), iris)
  p1 <- predict(mod1, se.fit = TRUE)
  p2 <- predict(mod1, se.fit = TRUE, newdata = iris)
  p3 <- predict(mod1, se.fit = TRUE, re.form = NA, newdata = iris)
  p4 <- predict(mod1, se.fit = TRUE, re.form = NA)
  p5 <- predict(mod1, se.fit = TRUE, re.form = ~(1 | Species))
  p6 <- predict(mod1, se.fit = TRUE, re.form = ~(1 | Species), newdata = iris)
  p7 <- predict(mod1, se.fit = TRUE, newdata = iris, random.only = TRUE)
  p8 <- predict(mod1, se.fit = TRUE, re.form = ~(1 | Species), random.only = TRUE)
  p9 <- predict(mod1, se.fit = TRUE, re.form = ~(1 | Species), newdata = iris, random.only = TRUE)

  expect_equal(unname(head(p1$se.fit)), 
               c(0.0271816400250223, 0.0272298862268211, 0.0286188379907626, 
                 0.0297645467444413, 0.0270330515271627, 0.0295876265523127))
  # re.form = NA
  expect_equal(unname(head(p3$se.fit)), 
               c(0.451147865048879, 0.451497971849052, 0.451930732595154, 0.452178035807842, 
                 0.451312573619068, 0.450778089845166))
  # random.only may need checking -- tolerance maybe too high for this?
  expect_equal(unname(p8$se.fit),
               rep(0.451569712647126, 150), tolerance = 0.001)
  expect_equal(p1, p2)  
  expect_equal(p3, p4)  
  expect_equal(p1, p5)  
  expect_equal(p1, p6)  
  expect_equal(p7, p8)  
  expect_equal(p7, p9)  
  
})

test_that("NA + re.form = NULL + simulate OK (GH #737)", {
    d <- lme4::sleepstudy
    d$Reaction[1] <- NA
    fm1 <- lmer(Reaction ~ Days + (Days | Subject), d)
    ss <- simulate(fm1, seed = 101, re.form = NULL)[[1]]
    expect_equal(c(head(ss)),
                   c(266.139101412856, 308.148180398426,
                     296.081377893883, 338.367909016478, 
                     360.294339946214, 401.91050930589))
    ss0 <- simulate(fm1, seed = 101, re.form = NA)[[1]]
    expect_equal(length(ss), length(ss0))
    ## correct dimensions with na.exclude as well ?
    fm2 <- update(fm1, na.action = na.exclude)
    ss2 <- simulate(fm2, seed = 101, re.form = NULL)[[1]]
    ss3 <- simulate(fm2, seed = 101, re.form = NA)[[1]]
    expect_equal(length(ss2), nrow(d))
    expect_equal(length(ss3), nrow(d))
})

## GH 691 parts 1 and 2
test_that("predict works with factors in left-out REs", {
    set.seed(101)
    df2 <- data.frame(yield = rnorm(100),
                      lc = factor(rep(1:2, 50)),
                      g1 = factor(rep(1:10, 10)),
                      g3 = factor(rep(1:10, each = 10)))
    m1B <- suppressWarnings(lmer(yield ~ 1 + ( 1 | g1) + (lc |g3), data  = df2,
                control = lmerControl(check.conv.singular = "ignore")))
    expect_equal(head(predict(m1B, re.form = ~(1|g1)),1),
                 c(`1` = 0.146787496519465),
                 tolerance = 1e-4)
})

test_that("predict works with dummy() in left-out REs", {
    set.seed(101)
    df3 <- data.frame(v1 = rnorm(100),
       		  v3 = factor(rep(1:10, each = 10)),
		  v4 = factor(rep(1:2, each = 50)),
		  v5 = factor(rep(1:10, 10)))
    m1C  <- lmer(v1~(1|v3) + (0+dummy(v4,"1")|v5),
                 data = df3,
                 control=lmerControl(check.nobs.vs.nlev="ignore",
                                     check.nobs.vs.nRE="ignore",
                                     check.conv.singular = "ignore"))
    expect_equal(head(predict(m1C, re.form = ~1|v3), 1),
                 c(`1` = -0.035719520719991))
})

test_that("predict se.fit on response scale", {
  p1 <- suppressWarnings(
    predict(fit_cbpp_1, type = "link", se.fit = TRUE))
  p2 <- suppressWarnings(
    predict(fit_cbpp_1, type = "response", se.fit = TRUE))
  p3 <- suppressWarnings(
    predict(fit_cbpp_1, type = "response", newdata = cbpp, se.fit = TRUE))
  expect_identical(p2$se.fit, p3$se.fit)
  expect_equal(p1$se.fit*binomial()$mu.eta(p1$fit), p2$se.fit)
})


test_that("predictions work with se.fit and subset of grouping variable levels", {
  ## Idea: when predicting where the newdata has less groups than what
  ## the full model accounted for, Cmat needs to be subsetted to match
  ## the dimensions for cbind(Z, X)
  ## Code inspired: https://github.com/lme4/lme4/issues/866#issue-3358828496
  set.seed(123)
  dat <- data.frame(
    outcome = rbinom(n = 100, size = 1, prob = 0.35),
    var_binom = as.factor(rbinom(n = 100, size = 1, prob = 0.7)),
    var_cont = rnorm(n = 100, mean = 10, sd = 7),
    grp = as.factor(sample(letters[1:4], size = 100, replace = TRUE))
  )
  m1 <- lme4::glmer(
    outcome ~ var_binom + var_cont + (1 | grp),
    data = dat,
    family = binomial(link = "logit")
  )
  ## d <- insight::get_datagrid(m1, "var_binom", include_random = TRUE)
  d <- data.frame(var_binom = factor(0:1),
                  var_cont = c(9.24717241397544, 9.24717241397544),
                  grp = factor(c("a","b"), levels = letters[1:4]))
  
  pp <- suppressWarnings(predict(m1, newdata = d, se.fit = TRUE))
  expect_equal(pp, list(fit = c(`1` = -0.4338277, `2` = -0.5993396),
                        se.fit = c(`1` = 0.4255397, `2` = 0.2779372)), 
               tol = 1e-6)
  
  d2 <- dat[sample(1:nrow(dat), size = 20),]
  d2 <- d2[!("c" == d2$grp), ]
  
  pp2 <- suppressWarnings(predict(m1, newdata = d2, se.fit = TRUE))
  expect_identical(lengths(pp2), c(fit=16L, se.fit=16L))

  expect_equal(lapply(pp2, head, 2),
               list(fit = c(`37` = -0.5109994, `29` = -0.5704263),
                    se.fit = c(`37` = 0.5151070, `29` = 0.3258730)),
               tol = 1e-7)

  set.seed(123)
  dat2 <- expand.grid(
    grp.1 = factor(c("a.1", "b.1", "c.1", "d.1")),
    grp.2 = factor(c("a.1", "b.1", "c.1", "d.1")),
    rep = 1:10)
  dat2$y <- simulate(~ 1 + (1|grp.1) + (1|grp.2),
                     family = binomial,
                     newdata = dat2,
                     newparams = list(beta = 0, theta = c(1, 1)))[[1]]
 
  m2 <- lme4::glmer(
    y ~ 1 + (1|grp.1) + (1|grp.2),
    data = dat2,
    family = binomial(link = "logit")
  )

  dsub <- expand.grid(
    grp.1 = c("a.1", "b.1"), 
    grp.2 = c("a.1", "b.1"))

  p1 <- suppressWarnings(predict(m2, newdata = dsub, se.fit = TRUE))
  p2 <- suppressWarnings(predict(m2, se.fit = TRUE))
  ss <- subset(dat2, grp.1 %in% c("a.1", "b.1") & grp.2 %in% c("a.1", "b.1"))
  w <- as.numeric(rownames(ss)[1:4])

  expect_equal(lapply(p2, function(x) x[w]), p1,
               check.attributes = FALSE)

  ## Example: we have grouping variables g1 and g2 which each have levels a, b, c, d. 
  ## This safeguards cases if we ask for levels a, b in g1 and b, c in g2
  set.seed(1)
  
  dat3 <- expand.grid(
    g1 = factor(c("a", "b", "c", "d")),
    g2 = factor(c("a", "b", "c", "d")),
    rep = 1:10
  )
  
  dat3$y <- simulate(~ 1 + (1|g1) + (1|g2),
                     family = binomial,
                     newdata = dat3,
                     newparams = list(beta = 0, theta = c(1, 1)))[[1]]
  
  m3 <- lme4::glmer(
    y ~ 1 + (1|g1) + (1|g2),
    data = dat3,
    family = binomial(link = "logit")
  )
  
  dsub2 <- expand.grid(
    g1 = c("a", "b"), 
    g2 = c("c", "d"))
  
  tp1 <- suppressWarnings(predict(m3, se.fit = TRUE))
  tp2 <- suppressWarnings(predict(m3, newdata = dsub2, se.fit = TRUE))
  
  ss2 <- subset(dat3, g1 %in% c("a", "b") & g2 %in% c("c", "d"))
  w2 <- as.numeric(rownames(ss2)[1:4])
  
  expect_equal(lapply(tp1, function(x) x[w2]), tp2,
               check.attributes = FALSE)
  
  ## Case where there are multiple grouping variables
  set.seed(1)
  sleepstudy$Subject2 <- rep(1:5, each = 36)
  
  m5 <- lmer(Reaction ~ Days + (1 + Days | Subject) +  
               (1 | Subject2), data = sleepstudy)

  d <- sleepstudy[sample(1:nrow(sleepstudy), size = 30), ]
  
  pms1 <- predict(m5, se.fit = TRUE, re.form = NULL, 
          allow.new.levels = FALSE)
  pms2 <- predict(m5, newdata = d, se.fit = TRUE, re.form = NULL, 
          allow.new.levels = FALSE)
  psw <- as.numeric(rownames(d))
  expect_equal(lapply(pms1, function(x) x[psw]), pms2, check.attributes = FALSE)
  
  ## Case where we are adding new levels
  lm1 <- lmer(Reaction ~ Days + (1 + Days | Subject), data = sleepstudy)
  d373 <- data.frame(Days = 0, Subject = "373")
  pred373 <- predict(lm1, newdata = d373, se.fit = TRUE, re.form = NULL, 
          allow.new.levels = TRUE)
  expected_373 <- fixef(lm1)["(Intercept)"] + fixef(lm1)["Days"] * d373$Days
  expect_equal(pred373$fit, expected_373, check.attributes = FALSE, 
               tol = 10e-16)
  
  ## Extending previous example: multiple new levels for different groups
  dsub3 <- expand.grid(
    g1 = c("k", "m"), 
    g2 = c("k", "m"))
  
  tp3 <- suppressWarnings(predict(m3, newdata = dsub3, se.fit = TRUE, 
                           allow.new.levels = TRUE))
  expected_tp3 <- fixef(m3)["(Intercept)"]
  
  expect_true(all(tp3$fit == expected_tp3))
  
  ## Case where user specified allow.new.levels = TRUE but 
  ## but no new levels were actually added
  tp4 <- suppressWarnings(predict(m3, se.fit = TRUE,
                                  allow.new.levels = TRUE))
  ## Ideally: predictions should remain the same as before.
  expect_equal(lapply(tp4, function(x) x[w2]), tp2,
               check.attributes = FALSE)
  
  ## Test that this also works under glmer.
  gm1 <- glmer(round(Reaction) ~ Days + (1 + Days | Subject), 
               data = sleepstudy, family = poisson)
  d2 <- data.frame(Days = 0, Subject = "373")
  gp1 <- suppressWarnings(predict(gm1, newdata = d2, se.fit = TRUE, 
                                  re.form = NULL, allow.new.levels = TRUE))
  expected_gp1 <- fixef(gm1)["(Intercept)"]
  expect_true(all(gp1$fit == expected_gp1))
})


test_that("simulation works with AR1", {
  skip_if(testLevel < 2)
  form <- Reaction ~ Days + ar1(0+factor(Days)|Subject)
  dd <- expand.grid(Subject = 1:100, Days = 1:20)
  s2 <- simulate(form[-2],
                 newdata = dd,
                 newparams = list(beta = c(0, 1),
                                  par = c(30, 0.5),
                                  sigma = 0.1),
                 seed = 101)
  dd$Reaction <- s2[[1]]
  m1 <- lmer(form, data = dd)
  expect_equal(getME(m1, "par"),
               c(`Subject.*` = 41.51375093795063, Subject.rho = 0.47173954902469345),
               tolerance = 1e-3)
})

test_that("simulation works with 'par'", {
  form <- Reaction ~ Days + ar1(0+factor(Days)|Subject)
  m1 <- lmer(form, data = sleepstudy)

  s1 <- simulate( form[-2],
                 newdata = sleepstudy,
                 newparams = list(beta = c(0, 1),
                                  theta = getME(m1, "theta"),
                                  sigma = 1),
                 seed = 101)[[1]]
  s2 <- simulate(form[-2],
                 newdata = sleepstudy,
                 newparams = list(beta = c(0, 1),
                                  par = getME(m1, "par"),
                                  sigma = 1),
                 seed = 101)[[1]]
  expect_equal(s1, s2)

})

try(detach("package:lmerTest"), silent = TRUE)

testLevel <- if (nzchar(s <- Sys.getenv("LME4_TEST_LEVEL"))) as.numeric(s) else 1

#context("summarizing/printing models")
test_that("lmer", {
  set.seed(0)
  J <- 8
  n <- 10
  N <- J * n
  beta <- c(5, 2, 4)
  u <- matrix(rnorm(J * 3), J, 3)
  
  x.1 <- rnorm(N)
  x.2 <- rnorm(N)
  g <- rep(1:J, rep(n, J))
  
  y <- 1   * (beta[1] + u[g,1]) +
       x.1 * (beta[2] + u[g,2]) +
       x.2 * (beta[3] + u[g,3]) +
       rnorm(N)

  tmpf <- function(x) capture.output(print(summary(x),digits=1))
  tfun <- function(cc) {
      w <- grep("Fixed effects:", cc)
      cc[w:length(cc)]
  }
  C1 <- lmerControl(optimizer="nloptwrap",
                    optCtrl=list(xtol_abs=1e-6, ftol_abs=1e-6))
  m1 <- lmer(y ~ x.1 + x.2 + (1 + x.1 | g), control=C1)
  m2 <- lmer(y ~ x.1 + x.2 + (1 + x.1 + x.2 | g), control=C1)
  cc1 <- tmpf(m1)
  cc2 <- tmpf(m2)
  ## FIXME: correlation of fixed effects printed inconsistently.
  ## If (1) LME4_TEST_LEVEL == 100 *and* after running all of prior
  ##   tests, something (package load? options setting?) changes
  ##   so that the fixed-effect correlations are no longer printed
  ##   out, and this test fails
  ## would like to sort this out but realistically not sure it's worth it?
  t1 <- tfun(cc1)
  vv <- vcov(m1)
  ss <- sessionInfo()
  save(m1, vv, ss, file = sprintf("test-summary_testlevel_%d.rda", testLevel))
  expect_equal(t1,
               c("Fixed effects:",
                 "            Estimate Std. Error t value", 
                 "(Intercept)      5.4        0.5      12",
                 "x.1              1.9        0.4       5", 
                 "x.2              4.0        0.1      28",
                 "",
                 "Correlation of Fixed Effects:", 
                 "    (Intr) x.1   ",
                 "x.1 -0.019       ",
                 "x.2  0.029 -0.043"
                 ))

  expect_equal(tfun(cc2),
               c("Fixed effects:",
                 "            Estimate Std. Error t value", 
                 "(Intercept)      5.4        0.4      12",
                 "x.1              2.0        0.4       5", 
                 "x.2              4.0        0.3      15",
                 "",
                 "Correlation of Fixed Effects:",
                 "    (Intr) x.1   ",
                 "x.1 -0.069       ",
                 "x.2  0.136 -0.103"
                 ))
})

## Tests with regards to auto-scaling; making sure we get expected behaviours.

set.seed(1)
sleepstudy$var1 = runif(nrow(sleepstudy), 1e6, 1e7)

form <- Reaction ~ var1 + Days + (Days | Subject)

scf1 <- lmer(form, control = lmerControl(autoscale = TRUE), sleepstudy)
scf2 <- suppressWarnings(
  update(scf1, control = lmerControl(autoscale = FALSE))
  )

test_that("lmer(): ensuring we get the internal scale for X with getME", {
  res1 <- getME(scf1, "X")[180, ]
  valtest1 <- c(1, 1.456867279040504, 1.562340900990911)
  expect_equal(res1, valtest1, check.attributes = FALSE, tolerance =1e-10)
})

test_that("lmer(): ensuring we get the internal scale for beta with getME", {
  res2 <- getME(scf1, "beta")
  valtest2 <- c(298.507891666666751, 1.621716753196637, 30.161580776828778)
  expect_equal(res2, valtest2, tolerance =1e-10)
})

test_that("lmer(): model.matrix should provide unscaled version at default", {
  res3 <- model.matrix.merMod(scf1)[180, ]
  valtest3 <- c(1, 9127734.503475949, 9)
  expect_equal(res3, valtest3,  check.attributes = FALSE, tolerance =1e-6)
  
  res4 <- model.matrix.merMod(scf1, noScale = TRUE)[180, ]
  valtest4 <- c(1, 1.45686727904050395, 1.5623409009909106)
  expect_equal(res4, valtest4, check.attributes = FALSE, tolerance =1e-6)
})

test_that("lmer(): fixef.merMod() should provide unscaled version at default", {
  res5 <- fixef.merMod(scf1)
  valtest5 <- fixef.merMod(scf2)
  expect_equal(res5, valtest5, check.attributes = FALSE, tolerance =1e-10)
  
  res6 <- fixef.merMod(scf1, noScale = TRUE)
  valtest6 <- c(298.507891666666751, 1.621716753196637, 30.161580776828778) 
  expect_equal(res6, valtest6, check.attributes = FALSE, tolerance =1e-10)
})

test_that("lmer(): vcov.merMod() should provide unscaled version at default", {
  res7 <- vcov.merMod(scf1)
  valtest7 <- vcov.merMod(scf2)
  expect_true(all.equal(res7, valtest7, check.attributes = FALSE,
                        tolerance =1e-6))
  
  res8 <- vcov.merMod(scf1, noScale = TRUE)
  valtest8 <- as(
    matrix(c(8.12278783645280e+01, -3.24525171154812e-15, 26.5884231100835215,
            -3.24525171154812e-15, 4.39230297872591e+00, 0.0344711252408276,
             2.65884231100835e+01, 3.44711252408276e-02, 19.7018459306493092), 
           nrow = 3, byrow = TRUE), "dpoMatrix")
  expect_true(all.equal(res8, valtest8, check.attributes = FALSE, 
                        tolerance =1e-10))
})

## Doing autoscaling tests with glmer instead.

set.seed(1)
cbpp$var1 = runif(nrow(cbpp), 1e3, 1e5)
form <- cbind(incidence, size - incidence) ~  var1 + period + (1 | herd)
gfit1 <- glmer(form, control = glmerControl(autoscale = TRUE),
               data = cbpp, family = binomial)
gfit2 <- suppressWarnings(
  update(gfit1, control = glmerControl(autoscale = FALSE))
  )

test_that("glmer(): back transform matches similar model", {
  gres1 <- fixef.merMod(gfit1)
  og1 <- fixef.merMod(gfit2)
  expect_equal(gres1, og1, tolerance = 1e-5)
  
  gres2 <- vcov.merMod(gfit1)
  og2 <- suppressWarnings(vcov.merMod(gfit2))
  expect_true(all.equal(gres2, og2, check.attributes = FALSE, 
                        tolerance = 0.04))
})

## examples for eval lookup
testthat::skip_on_cran()
data("cbpp", package = "lme4")

if (require(car, quietly = TRUE)) {
  test_that("infIndexPlot env lookup OK", {
    fm1 <- lmer(Reaction ~ Days + (Days | Subject), sleepstudy)
    ## silly test; the point is to see if this errors out with
    ## Error in as.list.environment(X[[i]], ...) :
    ## promise already under evaluation: recursive default argument reference or earlier problems?
    ##   Calls: infIndexPlot -> influence -> influence.merMod -> lapply -> FUN
    expect_equal(car::infIndexPlot(influence(fm1, "Subject")), NULL)
  })
}

if (require(rr2, quietly = TRUE)) {
  test_that("rr2 env lookup OK", {
    ## Error under alternate eval lookup
    ##     Error: bad 'data': object 'd' not found

    set.seed(123456)
    p1 <- 10; nsample <- 20; n <- p1 * nsample
    d <- data.frame(x1 = rnorm(n = n),
                    x2 = rnorm(n = n),
                    u1 = rep(1:p1, each = nsample),
                    u2 = rep(1:p1, times = nsample))
    d$u1 <- as.factor(d$u1); d$u2 <- as.factor(d$u2)

    ## LMM: y with random intercept
    b1 <- 1; b2 <- -1; sd1 <- 1.5
    d$y_re_intercept <- b1 * d$x1 + b2 * d$x2 +
      rep(rnorm(n = p1, sd = sd1), each = nsample) +  # random intercept u1
      rep(rnorm(n = p1, sd = sd1), times = nsample) + # random intercept u2
      rnorm(n = n)

    z.f2 <- lme4::lmer(y_re_intercept ~ x1 + x2 + (1 | u1) + (1 | u2), data = d, REML = T)
    ## NOTE, fails to produce warnings on second run of devtools::test()
    ## (possible interference from lmerTest methods being loaded ...?)
    expect_warning(R2(z.f2), "mod updated with REML = F")
})
}

## semEff::VIF() is here being applied to a previously fitted lmer
## model (shipley.growth[[3]])
## previous messing around with env evaluation had messed this up
if (suppressWarnings(require(semEff))) {
  ## suppress warning about 'cov2cor' import replacement
  test_that("semEff env lookup OK", {
    ##   Error in as.list.environment(X[[i]], ...) :
    ##     promise already under evaluation: recursive default argument reference or earlier problems?
    ##   Calls: VIF ... update.merMod -> do.call -> lapply -> FUN -> as.list.environment
  m <- shipley.growth[[3]]
  expect_equal(VIF(m),
               c(Date = 6.06283840168881,
                 DD = 6.07741017455859, lat = 1.01215136160858)
               )
  })
}

#context("data= argument and formula evaluation")

## intercept context-dependent errors ... it's too bad that
##  these errors differ between devtools::test() and
##  R CMD check, but finding the difference is too much
##  of a nightmare
## n.b. could break in other locales *if* we ever do internationalization ...
data_RE <- "(bad 'data'|variable lengths differ)"

test_that("glmerFormX", {
    set.seed(101)
    n <- 50
    x <- rbinom(n, 1, 1/2)
    y <- rnorm(n)
    z <- rnorm(n)
    r <- sample(1:5, size=n, replace=TRUE)
    d <- data.frame(x,y,z,r)

    F <- "z"
    rF <- "(1|r)"
    modStr <- (paste("x ~", "y +", F, "+", rF))
    modForm <- as.formula(modStr)

    ## WARNING: these drop/environment tests are extremely sensitive to environment
    ## they may fail/not fail, or fail differently, within a "testthat" environment vs.
    ##   when run interactively
    expect_that(m_data.3 <- glmer( modStr , data=d, family="binomial"), is_a("glmerMod"))
    expect_that(m_data.4 <- glmer( "x ~ y + z + (1|r)" , data=d, family="binomial"), is_a("glmerMod"))
    ## interactively: (interactive() is TRUE {i.e. doesn't behave as I would expect} within testing environment ...
    ## if (interactive()) {
    ## AICvec <- c(77.0516381151634, 75.0819116367084, 75.1915023640827)
    ## expect_equal(drop1(m_data.3)$AIC,AICvec)
    ## expect_equal(drop1(m_data.4)$AIC,AICvec)
    ## } else {
        ## in test environment [NOT test_
        expect_error(drop1(m_data.3),data_RE)
        expect_error(drop1(m_data.4),data_RE)
    ##}
})

test_that("glmerForm", {
    set.seed(101)
    n <- 50
    x <- rbinom(n, 1, 1/2)
    y <- rnorm(n)
    z <- rnorm(n)
    r <- sample(1:5, size=n, replace=TRUE)
    d <- data.frame(x,y,z,r)

    F <- "z"
    rF <- "(1|r)"
    modStr <- (paste("x ~", "y +", F, "+", rF))
    modForm <- as.formula(modStr)

    ## formulas have environments associated, but character vectors don't
    ## data argument not specified:
    ## should work, but documentation warns against it

    expect_that(m_nodata.0 <- glmer( x ~ y + z + (1|r) , family="binomial"), is_a("glmerMod"))
    expect_that(m_nodata.1 <- glmer( as.formula(modStr) , family="binomial"), is_a("glmerMod"))
    expect_that(m_nodata.2 <- glmer( modForm , family="binomial"), is_a("glmerMod"))
    expect_that(m_nodata.3 <- glmer( modStr , family="binomial"), is_a("glmerMod"))
    expect_that(m_nodata.4 <- glmer( "x ~ y + z + (1|r)" , family="binomial"), is_a("glmerMod"))

    ## apply drop1 to all of these ...
    m_nodata_List <- list(m_nodata.0,
                          m_nodata.1,m_nodata.2,m_nodata.3,m_nodata.4)
    d_nodata_List <- lapply(m_nodata_List,drop1)

    rm(list=c("x","y","z","r"))

    ## data argument specified
    expect_that(m_data.0 <- glmer( x ~ y + z + (1|r) , data=d, family="binomial"), is_a("glmerMod"))
    expect_that(m_data.1 <- glmer( as.formula(modStr) , data=d, family="binomial"), is_a("glmerMod"))
    expect_that(m_data.2 <- glmer( modForm , data=d, family="binomial"), is_a("glmerMod"))
    expect_that(m_data.3 <- glmer( modStr , data=d, family="binomial"), is_a("glmerMod"))
    expect_that(m_data.4 <- glmer( "x ~ y + z + (1|r)" , data=d, family="binomial"), is_a("glmerMod"))

    ff <- function() {
        set.seed(101)
        n <- 50
        x <- rbinom(n, 1, 1/2)
        y <- rnorm(n)
        z <- rnorm(n)
        r <- sample(1:5, size=n, replace=TRUE)
        d2 <- data.frame(x,y,z,r)
        glmer( x ~ y + z + (1|r), data=d2, family="binomial")
    }
    m_data.5 <- ff()

    ff2 <- function() {
        set.seed(101)
        n <- 50
        x <- rbinom(n, 1, 1/2)
        y <- rnorm(n)
        z <- rnorm(n)
        r <- sample(1:5, size=n, replace=TRUE)
        glmer( x ~ y + z + (1|r), family="binomial")
    }
    m_data.6 <- ff2()


    m_data_List <- list(m_data.0,m_data.1,m_data.2,m_data.3,m_data.4,m_data.5,m_data.6)
    badNums <- 4:5
    d_data_List <- lapply(m_data_List[-badNums],drop1)

    ## these do NOT fail if there is a variable 'd' living in the global environment --
    ## they DO fail in the testthat context
    expect_error(drop1(m_data.3),data_RE)
    expect_error(drop1(m_data.4),data_RE)

    ## expect_error(lapply(m_data_List[4],drop1))
    ## expect_error(lapply(m_data_List[5],drop1))
    ## d_data_List <- lapply(m_data_List,drop1,evalhack="parent")  ## fails on element 1
    ## d_data_List <- lapply(m_data_List,drop1,evalhack="formulaenv")  ## fails on element 4
    ## d_data_List <- lapply(m_data_List,drop1,evalhack="nulldata")  ## succeeds
    ## drop1(m_data.5,evalhack="parent") ## 'd2' not found
    ## drop1(m_data.5,evalhack="nulldata") ## 'x' not found (d2 is in environment ...)
    ## should we try to make update smarter ... ??

    ## test equivalence of (i vs i+1) for all models, all drop1() results
    for (i in 1:(length(m_nodata_List)-1)) {
        expect_equivalent(m_nodata_List[[i]],m_nodata_List[[i+1]])
        expect_equivalent(d_nodata_List[[i]],d_nodata_List[[i+1]])
    }

    expect_equivalent(m_nodata_List[[1]],m_data_List[[1]])
    expect_equivalent(d_nodata_List[[1]],d_data_List[[1]])

    for (i in 1:(length(m_data_List)-1)) {
        expect_equivalent(m_data_List[[i]],m_data_List[[i+1]])
    }
    ## allow for dropped 'bad' vals
    for (i in 1:(length(d_data_List)-1)) {
        expect_equivalent(d_data_List[[i]],d_data_List[[i+1]])
    }

})


test_that("lmerForm", {

    set.seed(101)

    x <- rnorm(10)
    y <- rnorm(10)
    z <- rnorm(10)
    r <- sample(1:3, size=10, replace=TRUE)
    d <- data.frame(x,y,z,r)

    ## example from Joehanes Roeby
    m2 <- suppressWarnings(lmer(x ~ y + z + (1|r), data=d))
    ff <- function() {
        m1 <- suppressWarnings(lmer(x ~ y + z + (1|r), data=d))
        return(anova(m1))
    }

    ff1 <- Reaction ~ Days + (Days|Subject)
    fm1 <- lmer(ff1, sleepstudy)
    fun <- function () {
        ff1 <- Reaction ~ Days + (Days|Subject)
        fm1 <- suppressWarnings(lmer(ff1, sleepstudy))
        return (anova(fm1))
    }
    anova(m2)
    ff()
    expect_equal(anova(m2),ff())
    anova(fm1)
    fun()
    expect_equal(anova(fm1),fun())

    ## test deparsing of long RE terms
    ## FIXME: delete eventually, functionality has migrated to reformulas
    varChr <- paste0("varname_",outer(letters,letters,paste0)[1:100])
    rvars <- varChr[1:9]
    form <- as.formula(paste("y ~",paste(varChr,collapse="+"),
                             "+",
                             paste0("(",paste(rvars,collapse="+"),"|f)")))
    ff <- reformulas::reOnly(form)
    environment(ff) <- .GlobalEnv
    expect_equal(ff,
     ~(us(varname_aa + varname_ba + varname_ca + varname_da + varname_ea +
       varname_fa + varname_ga + varname_ha + varname_ia | f)))
})

test_that("lapply etc.", {
    ## copied from dplyr
    failwith <- function (default = NULL, f, quiet = FALSE) {
        function(...) {
            out <- default
            try(out <- f(...), silent = quiet)
            out
        }
    }
    lmer_fw    <- failwith(NULL,function(...) lmer(...)   ,quiet=TRUE)
    expect_is(lmer_fw(Yield ~ 1|Batch, Dyestuff, REML = FALSE),
              "merMod")
    ## GH 369
    listOfFormulas <- list(
        cbind(incidence, size - incidence) ~ 1 +  (1 | herd),
        cbind(incidence, size - incidence) ~ period +  (1 | herd))
    expect_is(lapply(listOfFormulas,glmer,family=binomial,data=cbpp),"list")
})

test_that("formula and data validation work with do.call() in artificial environment", {
    ## This ensures compatibility of lmer when it's called from the
    ## C-level Rf_eval() with an environment that doesn't exist on the
    ## stack (i.e. C implementation in magrittr 2.0)
    e <- new.env()
    e$. <- mtcars
    expect_is(
        do.call(lme4::lmer, list("disp ~ (1 | cyl)", quote(.)), envir = e),
        "merMod"
    )

    fn <- function(data) {
        lme4::lmer("disp ~ (1 | cyl)", data = data)
    }
    expect_is(
        do.call(fn, list(quote(.)), envir = e),
        "merMod"
    )
})

test_that("correct environment on reOnly()", {
  ## GH 654
  f <- Reaction ~ Days + (1 | Subject)
  e <- environment(f)
  m <- lmer(f, data = sleepstudy)
  expect_identical(environment(formula(m)), e) # TRUE
  expect_identical(environment(formula(m, fixed.only = TRUE)), e) # TRUE
  expect_identical(ee <- environment(formula(m, random.only = TRUE)), e) # FALSE
})

# unit tests for weights 

test_that("weights for lmer is always 1", {
  fm1 <- lmer(Reaction ~ Days + (Days | Subject), sleepstudy)
  ## the default should be just the prior weights,
  ## tried both anyways in case it changes.
  wgt_default <- weights(fm1)
  wgt_prior <- weights(fm1, "prior")
  wgt_working <- weights(fm1, "working")
  
  expect_equal(length(wgt_default), nrow(sleepstudy))
  expect_equal(length(wgt_prior), nrow(sleepstudy))
  expect_equal(length(wgt_working), nrow(sleepstudy))
  ## checking it is indeed always a vector of 1
  expect_equal(wgt_default, rep(1, nrow(sleepstudy)))
  expect_equal(wgt_prior, rep(1, nrow(sleepstudy)))
  expect_equal(wgt_working, rep(1, nrow(sleepstudy)))
})

test_that("testing na.action", {
  sstud <- sleepstudy
  sstud$Reaction[1] <- NA
  fm1 <- lmer(Reaction ~ Days + (Days | Subject), sstud)
  fm1.exclude <- lmer(Reaction ~ Days + (Days | Subject), sstud, 
              na.action = na.exclude)
  fm1.omit <- lmer(Reaction ~ Days + (Days | Subject), sstud, 
                      na.action = na.omit)

  expect_equal(length(weights(fm1)), nrow(sleepstudy) - 1)
  expect_equal(length(weights(fm1.exclude)), nrow(sleepstudy))
  expect_equal(length(weights(fm1.omit)), nrow(sleepstudy) - 1)
  ## this is not necessarily for weights, but to ensure na.exclude and na.omit
  ## are acting as expected/
  expect_equal(length(residuals(fm1.exclude)), nrow(sleepstudy))
  expect_equal(length(residuals(fm1.omit)), nrow(sleepstudy) - 1)
  
  cbpp3 <- cbpp
  cbpp3$incidence[1] <- NA
  cbpp3$size[1] <- NA
  
  gm1 <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
                    data = cbpp3, family = binomial)
  gm1.exclude <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
                       data = cbpp3, family = binomial, na.action = na.exclude)
  gm1.omit <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
                    data = cbpp3, family = binomial, na.action = na.omit)
  expect_equal(length(weights(gm1)), nrow(cbpp) - 1)
  expect_equal(length(weights(gm1.exclude)), nrow(cbpp))
  expect_equal(length(weights(gm1.omit)), nrow(cbpp) - 1)
})

library("testthat")

test_that("checking singular fit for covariance", {
  
  set.seed(1)
  test.us <- new("Covariance.us", nc = 4L, simulate = TRUE)
  expect_equal(isSingular(test.us), FALSE)
  
  test.diag <- new("Covariance.diag", nc = 4L, simulate = TRUE)
  expect_equal(isSingular(test.diag), FALSE)
  
  test.cs <- new("Covariance.cs", nc = 4L, hom = FALSE, simulate = TRUE)
  test.homcs <- new("Covariance.cs", nc = 4L, hom = TRUE, simulate = TRUE)
  expect_equal(isSingular(test.cs), FALSE)
  expect_equal(isSingular(test.homcs), FALSE)
  
  test.ar1 <- new("Covariance.ar1", nc = 4L, hom = TRUE, simulate = TRUE)
  test.hetar1 <- new("Covariance.ar1", nc = 4L, hom = FALSE, simulate = TRUE)
  expect_equal(isSingular(test.ar1), FALSE)
  expect_equal(isSingular(test.hetar1), FALSE)
  
  test.reCovs <- c(test.us, test.diag, test.cs, test.homcs, test.ar1, test.hetar1)
  
  ## tests should fail once one of the diagional entries is 0
  sapply(test.reCovs, function(i){
    new_theta <- getTheta(i)
    new_theta[1] <- 0
    test <- setTheta(i, new_theta)
    expect_equal(isSingular(test), TRUE)
  })
  
  test.reCovs.lim <- test.reCovs[-c(1:2)]
  
  ## tests should fail once rho is equal to -1 or 1
  sapply(test.reCovs.lim, function(i){
    test <- setVC(i, vcomp = getVC(i)$vcomp, ccomp = 1)
    expect_equal(isSingular(test), TRUE)
    test2 <- setVC(i, vcomp = getVC(i)$vcomp, ccomp = -1)
    expect_equal(isSingular(test2), TRUE)
  })
})

test_that("checking singular fit for merMod", {

  set.seed(101)
  
  n_groups <- 20
  n_per_group <- 20
  n <- n_groups * n_per_group
  
  dat <- data.frame(
    group1 = rep(1:n_groups, each = n_per_group),
    group2 = rep(1:n_groups, each = n_per_group),
    x1 = rnorm(n),
    x2 = rnorm(n)
  )
  form <- y ~ 1 + (1 + x1|group1) + (1 + x2|group2)
  dat$y <- simulate(form[-2], ## one-sided formula
                    newdata = dat,
                    family = gaussian,
                    newparams = list(beta = c(-2),
                                     theta = c(2, 3, 4, 2, 3, 4),
                                     sigma = 2))[[1]]
  mod <- lmer(form, data = dat)
  ## should be no issues here, ensuring isSingular is not broken
  expect_equal(isSingular(mod), FALSE)
  
  form2 <- y2 ~ 1 + (1 + x1|group1) + (1 + x2|group2)
  dat$y2 <- simulate(form[-2], 
                    newdata = dat,
                    family = gaussian,
                    newparams = list(beta = c(-2),
                                     theta = c(0, 0, 0, 2.5, 1.5, 0.8),
                                     sigma = 2))[[1]]
  mod2 <- suppressWarnings(lmer(form2, data = dat))
  ## second one, should be an issue with group 1 as we set thetas = 0
  expect_equal(isSingular(mod2), TRUE)
  
  ## also need to consider the glmer case
  form <- y ~ 1 + (1 + x1 | group1) + (1 + x2 | group2)
  
  dat$y <- simulate(form[-2], newdata = dat,
                    family = poisson(link = "log"), 
                    newparams = list(theta = c(0, 0, 0, 1.0, 0.5, 0.3),
                                     beta = 2))[[1]]
  
  form2 <- y2 ~ 1 + (1 + x1 | group1) + (1 + x2 | group2)
  
  dat$y2 <- simulate(form2[-2], newdata = dat,
                     family = Gamma(link = "log"), 
                     newparams = list(theta = c(0, 0, 0, 1.0, 0.5, 0.3),
                                      beta = 2,
                                      sigma = 2))[[1]]
  
  
  mod_pois <- suppressWarnings(
    glmer(form, family = poisson(link = "log"), data = dat))
  
  mod_gam <- suppressWarnings(
    glmer(form2, family = Gamma(link = "log"), data = dat))
  
  expect_equal(isSingular(mod_pois), TRUE)
  expect_equal(isSingular(mod_gam), TRUE)
})

#context("storing warnings, convergence status, etc.")

test_that("storewarning", {
    gCtrl <- glmerControl(optimizer = "Nelder_Mead",
                          optCtrl = list(maxfun=3))
    expect_warning(gm1 <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
                                data=cbpp, family=binomial, control=gCtrl),
                   "failure to converge in 3")
    expect_equal(gm1@optinfo$warnings[[1]],"failure to converge in 3 evaluations")
    ## FIXME: why is conv==0 here?
})

library(lme4)
library(testthat)
library(lattice)
testLevel <- if (nzchar(s <- Sys.getenv("LME4_TEST_LEVEL"))) as.numeric(s) else 1

options(nwarnings = 5000)# instead of 50, and then use  summary(warnings())

if (testLevel>1) {
### __ was ./profile_plots.R ___
fm1 <- lmer(Reaction~ Days + (Days|Subject), sleepstudy)
pfile <- system.file("testdata","tprfm1.RData", package="lme4")
if(file.exists(pfile)) print(load(pfile)) else withAutoprint({
    system.time( tpr.fm1 <- profile(fm1, optimizer="Nelder_Mead") ) ## 5 sec (2018); >= 50 warnings !?
    save(tpr.fm1, file= "../../inst/testdata/tprfm1.RData")
})
oo <- options(warn = 2) # {warnings are errors from here on}

if(!dev.interactive(orNone=TRUE)) pdf("profile_plots.pdf")
xyplot(tpr.fm1)
splom(tpr.fm1)
densityplot(tpr.fm1, main="densityplot( profile(lmer(..)) )")

## various scale options
xyplot(tpr.fm1,scale=list(x=list(relation="same")))  ## stupid
xyplot(tpr.fm1,scale=list(y=list(relation="same")))
xyplot(tpr.fm1,scale=list(y=list(relation="same"),tck=0))

##
expect_error(xyplot(tpr.fm1,conf=50),"must be strictly between 0 and 1")

### end {profile_plots.R}

fm01ML <- lmer(Yield ~ 1|Batch, Dyestuff, REML = FALSE)

## 0.8s (on a 5600 MIPS 64bit fast(year 2009) desktop "AMD Phenom(tm) II X4 925"):
##
system.time( tpr <- profile(fm01ML) )

## test all combinations of 'which', including plots (but don't show plots)
wlist <- list(1:3,1:2,1,2:3,2,3,c(1,3))
invisible(lapply(wlist,function(w) xyplot(profile(fm01ML,which=w))))

(confint(tpr) -> CIpr)
print(xyplot(tpr))
##  comparing against lme4a reference values -- but lme4 returns sigma
## rather than log(sigma)
stopifnot(dim(CIpr) == c(3,2),
          all.equal(unname(CIpr[".sigma",]),exp(c(3.64362, 4.21446)), tolerance=1e-6),
          all.equal(unname(CIpr["(Intercept)",]),c(1486.451500,1568.548494)))

options(oo)# warnings allowed ..

## fixed-effect profiling with vector RE
data(Pastes)
fmoB <- lmer(strength ~ 1 + (cask | batch), data=Pastes,
             control = lmerControl(optimizer = "bobyqa"))
(pfmoB <- profile(fmoB, which = "beta_", alphamax=.001))
xyplot(pfmoB)# nice and easy ..

summary(
    fm <- lmer(strength ~ 1 + (cask | batch), data=Pastes,
               control = lmerControl(optimizer = "nloptwrap",
                                     calc.derivs= FALSE))
)

ls.str(environment(nloptwrap))# showing *its* defaults

pfm <- profile(fm, which = "beta_", alphamax=.001) # 197 warnings for "nloptwrap"
summary(warnings())
str(pfm) # only 3 rows, .zeta = c(0, NaN, Inf) !!!
try( xyplot(pfm) ) ## FIXME or rather the profiling or rather the "wrap on nloptr"

(testLevel <- lme4:::testLevel())
if(testLevel > 2) {

    ## 2D profiles
    fm2ML <- lmer(diameter ~ 1 + (1|plate) + (1|sample), Penicillin, REML=0)
    system.time(pr2 <- profile(fm2ML)) # 5.2 sec, 2018-05: 2.1"
    (confint(pr2) -> CIpr2)

    lme4a_CIpr2 <-
        structure(c(0.633565787613112, 1.09578224011285, -0.721864513060904,
                    21.2666273835452, 1.1821039843372, 3.55631937954106, -0.462903300019305,
                    24.6778176174587), dim = c(4L, 2L), dimnames = list(c(".sig01",
                           ".sig02", ".lsig", "(Intercept)"), c("2.5 %", "97.5 %")))
    lme4a_CIpr2[".lsig",] <- exp(lme4a_CIpr2[".lsig",])

    stopifnot(all.equal(unname(CIpr2),unname(lme4a_CIpr2),tolerance=1e-6))

    print(xyplot(pr2, absVal=0, aspect=1.3, layout=c(4,1)))
    print(splom(pr2))

    gm1 <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
                 data = cbpp, family = binomial)

    ## GLMM profiles
    system.time(pr4 <- profile(gm1))  ## ~ 10 seconds

    pr4.3 <- profile(gm1,which=3)
    xyplot(pr4,layout=c(5,1),as.table=TRUE)

    splom(pr4) ## used to fail because of NAs

    nm1 <- nlmer(circumference ~ SSlogis(age, Asym, xmid, scal) ~ Asym|Tree,
                 Orange, start = c(Asym = 200, xmid = 725, scal = 350))
    if (FALSE) {
        ## not working yet: detecting (slightly) lower deviance; not converging in 10k
        pr5 <- profile(nm1,which=1,verbose=1,maxmult=1.2)
        xyplot(.zeta~.focal|.par,type=c("l","p"),data=lme4:::as.data.frame.thpr(pr5),
               scale=list(x=list(relation="free")),
               as.table=TRUE)
    }
}  ## testLevel > 2

if (testLevel > 3) {
    fm3ML <- lmer(Reaction ~ Days + (Days|Subject), sleepstudy, REML=FALSE)
    ## ~ 4 theta-variables (+ 2 fixed), 19 seconds | 2018-05: 7.4"
    print(system.time(pr3 <- profile(fm3ML)))
    print(xyplot(pr3))
    print(splom(pr3))

    if (testLevel > 4) {
      if(requireNamespace("mlmRev")) {
        data("Contraception", package="mlmRev")
        ## fit already takes ~ 3 sec (2018-05)
        fm2 <- glmer(use ~ urban+age+livch + (urban|district), Contraception, binomial)
        print(system.time(pr5 <- profile(fm2,verbose=10))) # 2018-05: 462 sec = 7'42"
        ## -> 5 warnings notably "non-monotonic profile for .sig02" (the RE's corr.)
        print(xyplot(pr5))
      }
    }  ## testLevel > 4

}  ## testLevel > 3

library("parallel")
if (max(detectCores(), 1L, na.rm = TRUE) > 1) {

    p0 <- profile(fm1, which="theta_")
    ## http://stackoverflow.com/questions/12983137/how-do-detect-if-travis-ci-or-not
    travis <- nchar(Sys.getenv("TRAVIS")) > 0
    if(.Platform$OS.type != "windows" && !travis) {
        prof01P <- profile(fm1, which="theta_", parallel="multicore", ncpus=2)
        stopifnot(all.equal(p0,prof01P))
    }

    ## works in Solaris from an interactive console but not ???
    ##   via R CMD BATCH

    if (Sys.info()["sysname"] != "SunOS" && !travis) {
        prof01P.snow <- profile(fm1, which="theta_", parallel="snow", ncpus=2)
        stopifnot(all.equal(p0,prof01P.snow))
    }

    if (require("future.apply", quietly = TRUE)) {
        plan(sequential)
        prof01P.future.seq <- profile(fm1, which = "theta_", parallel = "future")
        stopifnot(all.equal(p0, prof01P.future.seq))
        plan(multisession)
        prof01P.future.multi <- profile(fm1, which = "theta_", parallel = "future")
        stopifnot(all.equal(p0, prof01P.future.multi))
        plan(sequential)
    }
}

## test profile/update from within functions
foo <- function() {
    gm1 <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
                 data = cbpp, family = binomial)
    ## return
    profile(gm1, which="theta_")
}
stopifnot(inherits(foo(), "thpr"))
} ## testLevel>1

if (.Platform$OS.type != "windows") {
    ## test redirection from lmer to glmer (correct options passed,
    ##   specifically glmerControl -> tolPwrss

    library("lme4")
    library("testthat")
    ## data("trees513", package = "multcomp")
    load(system.file("testdata","trees513.RData",package="lme4"))


    expect_is(mmod1 <- glmer(damage ~ species - 1 + (1 | lattice / plot),
                             data = trees513B, family = binomial()),"glmerMod")
    if(FALSE) { ## Now (2019-05) defunct; was deprecated since 2013-06:
        expect_warning(mmod2 <- lmer(damage ~ species - 1 + (1 | lattice / plot),
                                     data = trees513B, family = binomial()),
                       "calling lmer with .* is deprecated")
        mmod2@call <- mmod1@call ## hack calls to equality
        expect_equal(mmod1,mmod2)
    }
} ## skip on windows (for speed)

## Tests of a variety of GLMM families and links
## coding: family {g=Gamma, P=Poisson, G=Gaussian, B=binomial}
##         link   {l=log, i=inverse, c=cloglog, i=identity}
##         model  {1 = intercept-only, 2 = with continuous predictor}

library("lme4")

source(system.file("testdata/lme-tst-funs.R", package="lme4", mustWork=TRUE))
##-> gSim(), a general simulation function ...

str(gSim)
## function (nblk = 26, nperblk = 100, sigma = 1, beta = c(4, 3),
##           x = runif(n), shape = 2, nbinom = 10, family = Gamma())

if (.Platform$OS.type != "windows") withAutoprint({
set.seed(101)
## Gamma, inverse link (= default) :
d <- gSim()
## Gamma, log link   eta = log(mu) :
dgl <- gSim(dInitial = d, family = Gamma(link = log))
## Poisson, log link
dP <- gSim(dInitial = d, family = poisson())
##  Gaussian, log link --- need to use a non-identity link, otherwise glmer calls lmer
dG <- gSim(dInitial = d, family = gaussian(link = log), sd = 2)
## Gaussian with inverse link :     (sd small enough to avoid negative values) :
dGi <- gSim(dInitial = d, family = gaussian(link = inverse), sd = 0.01)
## binomial with cloglog link
dBc <- d
dBc$eta <- d$eta - 5 # <==> beta intercept 5 less: otherwise  y will be constant
dBc <- gSim(dInitial = dBc, ## beta = c(-1, 3),
            nbinom = 1, family = binomial(link="cloglog"))

## binomial with identity link
dBi <- d
dBc$eta <- d$eta / 10 # <==> beta slope / 10 : scale so range goes from 0.2-0.8
dBi <- gSim(dInitial = dBc, ## beta = c(4, 3/10),
            nbinom = 1, family = binomial(link="identity"))


############
## Gamma/inverse

## GLMs
gm0 <- glm(y ~ 1,       data=d, family=Gamma)
gm1 <- glm(y ~ block-1, data=d, family=Gamma)
stopifnot(all.equal(sd(coef(gm1)),1.00753942148611))

gm2  <- glmer(y ~ 1 + (1|block), d, Gamma, nAGQ=0)
gm3  <- glmer(y ~ x + (1|block), d, Gamma, nAGQ=0)
gm2B <- glmer(y ~ 1 + (1|block), d, Gamma)
gm3B <- glmer(y ~ x + (1|block), d, Gamma)

##   y ~ x + (1|block),  Gamma   is TRUE model
summary(gm3)
summary(gm3B)# should be better
## Both have "correct" beta ~= (4, 3)  -- but *too* small  (sigma_B, sigma) !!
stopifnot(exprs = {
    all.equal(fixef(gm3 ), c(`(Intercept)` = 4.07253, x = 3.080585), tol = 1e-5) # 1.21e-7
    all.equal(fixef(gm3B), c(`(Intercept)` = 4.159398, x = 3.058521),tol = 1e-5) # 1.13e-7
})
VarCorr(gm3)  # both variances / std.dev. should be ~ 1  but are too small



##
## library(hglm)
## h1 <- hglm2(y~x+(1|block), data=d, family=Gamma())
## lme4.0 fails on all of these ...

## Gamma/log
ggl1 <- glmer(y ~ 1 + (1|block), data=dgl, family=Gamma(link="log"))
ggl2 <- glmer(y ~ x + (1|block), data=dgl, family=Gamma(link="log"))# true model
(h.1.2 <- anova(ggl1, ggl2))
stopifnot(
    all.equal(unlist(h.1.2[2,]),
              c(npar = 4, AIC = 34216.014, BIC = 34239.467, logLik = -17104.007,
                "-2*log(L)" = 34208.014, Chisq = 2458.5792, Df = 1, `Pr(>Chisq)` = 0))
)
## "true" model :
summary(ggl2)
VarCorr(ggl2)

##
## library(lme4.0)
## ggl1 <- glmer(y ~ 1 + (1|block), data=dgl, family=Gamma(link="log"), verbose= 2)
## fails

## Poisson/log
gP1 <- glmer(y ~ 1 + (1|block), data=dP, family=poisson)
gP2 <- glmer(y ~ x + (1|block), data=dP, family=poisson)

## Gaussian/log
gG1 <- glmer(y ~ 1 + (1|block), data=dG, family=gaussian(link="log"))
gG2 <- glmer(y ~ x + (1|block), data=dG, family=gaussian(link="log"))

## works with lme4.0 but AIC/BIC/logLik are crazy, and scale
## parameter is not reported
## glmmML etc. doesn't allow models with scale parameters
## gG1B <- glmmadmb(y ~ 1 + (1|block), data=dG,
##                  family="gaussian",link="log",verbose=TRUE)
## what is the best guess at the estimate of the scale parameter?
## is it the same as sigma?
## gG1B$alpha

## if(Sys.info()["user"] != "maechler") { # <- seg.faults (MM)

## Gaussian/inverse
gGi1 <- glmer(y ~ 1 + (1|block), data=dGi, family=gaussian(link="inverse"))
gGi2 <- glmer(y ~ x + (1|block), data=dGi, family=gaussian(link="inverse"))


## Binomial/cloglog
gBc1 <- glmer(y ~ 1 + (1|block), data=dBc, family=binomial(link="cloglog"))
gBc2 <- glmer(y ~ x + (1|block), data=dBc, family=binomial(link="cloglog"))
## library("glmmADMB")
## glmmadmbfit <- glmmadmb(y ~ x + (1|block), data=dBc,
## family="binomial",link="cloglog")
glmmadmbfit <- list(fixef = c("(Intercept)" = -0.717146132730349, x =2.83642900561633),
                    VarCorr = structure(list(
                        block = structure(0.79992, dim = c(1L, 1L),
                                          dimnames = list("(Intercept)", "(Intercept)"))),
                        class = "VarCorr"))
stopifnot(all.equal(fixef(gBc2), glmmadmbfit$fixef, tolerance=5e-3))
## pretty loose tolerance ...
stopifnot(all.equal(unname(unlist(VarCorr(gBc2))),
                    c(glmmadmbfit$VarCorr$block), tolerance=2e-2))

gBi1 <- glmer(y ~ 1 + (1|block), data=dBi, family=binomial(link="identity"))
gBi2 <- glmer(y ~ x + (1|block), data=dBi, family=binomial(link="identity"))

## FIXME: should test more of the *results* of these efforts, not
##  just that they run without crashing ...
}) ## skip on windows (for speed)

lme4_specials <- c("us", "diag", "cs", "ar1")

## From Matrix package  isDiagonal(.) :
all0 <- function(x) !anyNA(x) && all(!x)
.isDiagonal.sq.matrix <- function(M, n = dim(M)[1L])
    all0(M[rep_len(c(FALSE, rep.int(TRUE,n)), n^2)])


### Utilities for parsing and manipulating mixed-model formulas

## abbreviated parse for long strings: deparse1() pastes w/ collapse instead
abbrDeparse <- function(x, width=60) {
    r <- deparse(x, width)
    if(length(r) > 1) paste(r[1], "...") else r
}

##' @param bars result of findbars
barnames <- function(bars) vapply(bars, function(x) deparse1(x[[3]]), "")

makeFac <- function(x,char.only=FALSE) {
    if (!is.factor(x) && (!char.only || is.character(x))) factor(x) else x
}

factorize <- function(x,frloc,char.only=FALSE) {
    ## convert grouping variables to factors as necessary
    ## TODO: variables that are *not* in the data frame are
    ##  not converted -- these could still break, e.g. if someone
    ##  tries to use the : operator
    ## TODO: some sensible tests for drop.unused.levels
    ##       (not actually used, but could come in handy)
    for (i in all.vars(RHSForm(x))) {
        if (!is.null(curf <- frloc[[i]]))
            frloc[[i]] <- makeFac(curf,char.only)
    }
    return(frloc)
}

colSort <- function(x) {
    termlev <- vapply(strsplit(x,":"),length,integer(1))
    iterms <- split(x,termlev)
    iterms <- sapply(iterms,sort,simplify=FALSE)
    ## make sure intercept term is first
    ilab <- "(Intercept)"
    if (ilab %in% iterms[[1]]) {
        iterms[[1]] <- c(ilab,setdiff(iterms[[1]],ilab))
    }
    unlist(iterms)
}

## copied from glmmTMB, replace by upstream utility package?
## test formula: does it contain a particular element?
## inForm(z~.,quote(.))
## inForm(z~y,quote(.))
## inForm(z~a+b+c,quote(c))
## inForm(z~a+b+(d+e),quote(c))
## f <- ~ a + offset(x)
## f2 <- z ~ a
## inForm(f,quote(offset))
## inForm(f2,quote(offset))
## @export
## @keywords internal
inForm <- function(form,value) {
    if (any(sapply(form,identical,value))) return(TRUE)
    if (all(sapply(form,length)==1)) return(FALSE)
    return(any(vapply(form,inForm,value,FUN.VALUE=logical(1))))
}

## was called "replaceForm" there but replaceTerm is better
## (decide on camelCase vs snake_case!)
replaceTerm <- function(term,target,repl) {
    if (identical(term,target)) return(repl)
    if (!inForm(term,target)) return(term)
    if (length(term) == 2) {
        return(substitute(OP(x),list(OP=replaceTerm(term[[1]],target,repl),
                                     x=replaceTerm(term[[2]],target,repl))))
    }
    return(substitute(OP(x,y),list(OP=replaceTerm(term[[1]],target,repl),
                                   x=replaceTerm(term[[2]],target,repl),
                                   y=replaceTerm(term[[3]],target,repl))))
}

`%i%` <- function(f1, f2, fix.order = TRUE) {
    if (!is.factor(f1) || !is.factor(f2)) stop("both inputs must be factors")
    f12 <- paste(f1, f2, sep = ":")
    ## explicitly specifying levels is faster in any case ...
    u <- which(!duplicated(f12))
    if (!fix.order) return(factor(f12, levels = f12[u]))
    ## deal with order of factor levels
    levs_rank <- length(levels(f2))*as.numeric(f1[u])+as.numeric(f2[u])
    return(factor(f12, levels = (f12[u])[order(levs_rank)]))
}

##' @param x a language object of the form  effect | groupvar
##' @param frloc model frame
##' @param drop.unused.levels (logical)
##' @return list containing grouping factor, sparse model matrix, number of levels, names
mkBlist <- function(x,frloc, drop.unused.levels=TRUE,
                    reorder.vars=FALSE) {
    frloc <- factorize(x,frloc)
    ## try to evaluate grouping factor within model frame ...
    ff0 <- replaceTerm(x[[3]], quote(`:`), quote(`%i%`))
    ff <- try(eval(substitute(makeFac(fac),
                              list(fac = ff0)),
                   frloc), silent = TRUE)
    if (inherits(ff, "try-error")) {
        stop("couldn't evaluate grouping factor ",
             deparse1(x[[3]])," within model frame:",
             "error =",
             c(ff),
             " Try adding grouping factor to data ",
             "frame explicitly if possible",call.=FALSE)
    }
    if (all(is.na(ff)))
        stop("Invalid grouping factor specification, ",
             deparse1(x[[3]]),call.=FALSE)
    ## NB: *also* silently drops <NA> levels - and mkReTrms() and hence
    ##     predict.merMod() have relied on that property  :
    if (drop.unused.levels) ff <- factor(ff, exclude=NA)
    nl <- length(levels(ff))
    ## this section implements eq. 6 of the JSS lmer paper
    ## model matrix based on LHS of random effect term (X_i)
    ##    x[[2]] is the LHS (terms) of the a|b formula
    has.sparse.contrasts <- function(x) {
      cc <- attr(x, "contrasts")
      !is.null(cc) && is(cc, "sparseMatrix")
    }
    any.sparse.contrasts <- any(vapply(frloc, has.sparse.contrasts, FUN.VALUE = TRUE))
    mMatrix <- if (!any.sparse.contrasts) model.matrix else sparse.model.matrix
    mm <- mMatrix(eval(substitute( ~ foo, list(foo = x[[2]]))), frloc)
    if (reorder.vars) {
        mm <- mm[colSort(colnames(mm)),]
    }
    ## this is J^T (see p. 9 of JSS lmer paper)
    ## construct indicator matrix for groups by observations
    ## use fac2sparse() rather than as() to allow *not* dropping
    ## unused levels where desired
    sm <- fac2sparse(ff, to = "d",
                     drop.unused.levels = drop.unused.levels)
    sm <- KhatriRao(sm, t(mm))
    dimnames(sm) <- list(
        rep(levels(ff),each=ncol(mm)),
        rownames(mm))
    list(ff = ff, sm = sm, nl = nl, cnms = colnames(mm))
}

##' Create an lmerResp, glmResp or nlsResp instance
##'
##' @title Create an lmerResp, glmResp or nlsResp instance
##' @param fr a model frame
##' @param REML logical scalar, value of REML for an lmerResp instance
##' @param family the optional glm family (glmResp only)
##' @param nlenv the nonlinear model evaluation environment (nlsResp only)
##' @param nlmod the nonlinear model function (nlsResp only)
##' @param ... where to look for response information if \code{fr} is missing.
##'   Can contain a model response, \code{y}, offset, \code{offset}, and weights,
##'   \code{weights}.
##' @return an lmerResp or glmResp or nlsResp instance
##' @family utilities
##' @export
mkRespMod <- function(fr, REML=NULL, family = NULL, nlenv = NULL, nlmod = NULL, ...)
{
    if(!missing(fr)) {
        y <- model.response(fr)
        offset <- model.offset(fr)
        weights <- model.weights(fr)
        N <- n <- nrow(fr)
        etastart_update <- model.extract(fr, "etastart")
        mustart_update <- model.extract(fr, "mustart")
    } else {
        fr <- list(...)
        y <- fr$y
        N <- n <- NROW(y)
        offset <- fr$offset
        weights <- fr$weights
        etastart_update <- fr$etastart
        mustart_update <- fr$mustart
    }
    if(length(dim(y)) == 1L)
        y <- drop(y) ## avoid problems with 1D arrays and keep names

    if(isGLMM <- !is.null(family))
        stopifnot(inherits(family, "family"))
   ## FIXME: may need to add X, or pass it somehow, if we want to use glm.fit

    ## test for non-numeric response here to avoid later
    ## confusing error messages from deeper machinery
    if (!is.null(y)) { ## 'y' may be NULL if we're doing simulation
        if(!(is.numeric(y) ||
            ((is.binom <- isGLMM && family$family == "binomial") &&
                (is.factor(y) || is.logical(y))))) {
            if (is.binom)
                stop("response must be numeric or factor")
            else {
                if (is.logical(y))
                    y <- as.integer(y)
                else stop("response must be numeric")
            }
        }
        if(!all(is.finite(y)))
            stop("NA/NaN/Inf in 'y'") # same msg as from lm.fit()
    }

    rho <- new.env()
    rho$y <- if (is.null(y)) numeric(0) else y
    if (!is.null(REML)) rho$REML <- REML
    rho$etastart <- etastart_update
    rho$mustart <- mustart_update
    rho$start <- attr(fr,"start")
    if (!is.null(nlenv)) {
        stopifnot(is.language(nlmod),
                  is.environment(nlenv),
                  is.numeric(val <- eval(nlmod, nlenv)),
                  length(val) == n,
                  ## FIXME?  Restriction, not present in ole' nlme():
                  is.matrix(gr <- attr(val, "gradient")),
                  is.numeric(gr),
                  nrow(gr) == n,
                  !is.null(pnames <- colnames(gr)))
        N <- length(gr)
        rho$mu <- as.vector(val)
        rho$sqrtXwt <- as.vector(gr)
        rho$gam <- ## FIXME more efficient  mget(pnames, envir=nlenv)
            unname(unlist(lapply(pnames,
                                 function(nm) get(nm, envir=nlenv))))
    }
    rho$offset <- if (!is.null(offset)) {
        if (length(offset) == 1L) offset <- rep.int(offset, N)
        else stopifnot(length(offset) == N)
        unname(offset)
    } else rep.int(0, N)
    rho$weights <- if (!is.null(weights)) {
        stopifnot(length(weights) == n, all(weights >= 0))
        unname(weights)
    } else rep.int(1, n)

    if(isGLMM) {
        ## need weights for initializing evaluation
        rho$nobs <- n
        ## allow trivial objects, e.g. for simulation
        if (length(y)>0) eval(family$initialize, rho)
        ## ugh. this *is* necessary;
        ##  family$initialize *ignores* mustart in env, overwrites!
        ## see ll 180-182 of src/library/stats/R/glm.R
        ## https://github.com/wch/r-source/search?utf8=%E2%9C%93&q=mukeep
        if (!is.null(mustart_update)) rho$mustart <- mustart_update
        ## family$initialize <- NULL     # remove clutter from str output
        ll <- as.list(rho)
        ans <- do.call(new, c(list(Class="glmResp", family=family),
                              ll[setdiff(names(ll), c("m", "nobs", "mustart"))]))
        if (length(y)>0)
            ans$updateMu(if (!is.null(es <- etastart_update)) es else                                       family$linkfun(rho$mustart))
        ans
    } else if (is.null(nlenv)) ## lmer
        do.call(lmerResp$new, as.list(rho))
    else ## nlmer
        do.call(nlsResp$new,
                c(list(nlenv=nlenv,
                       nlmod=substitute(~foo, list(foo=nlmod)),
                       pnames=pnames), as.list(rho)))
}

subnms <- function(form, nms) {
    ## Recursive function applied to individual terms
    sbnm <- function(term)
    {
        if (is.name(term)) {
            if (any(term == nms)) 0 else term
        } else switch(length(term),
               term, ## 1
           {   ## 2
               term[[2]] <- sbnm(term[[2]])
               term
           },
           {   ## 3
               term[[2]] <- sbnm(term[[2]])
               term[[3]] <- sbnm(term[[3]])
               term
           })
    }
    sbnm(form)
}

##' Check for a constant term (a literal 1) in an expression
##
##' In the mixed-effects part of a nonlinear model formula, a constant
##' term is not meaningful because every term must be relative to a
##' nonlinear model parameter.  This function recursively checks the
##' expressions in the formula for a a constant, calling stop() if
##' such a term is encountered.
##' @title Check for constant terms.
##' @param expr an expression
##' @return NULL.  The function is executed for its side effect.
chck1 <- function(expr) {
    if ((le <- length(expr)) == 1) {
        if (is.numeric(expr) && expr == 1)
            stop("1 is not meaningful in a nonlinear model formula")
        return()
    } else
        for (j in seq_len(le)[-1]) Recall(expr[[j]])
}

## ---> ../man/nlformula.Rd --- Manipulate a nonlinear model formula
##' @param mc matched call from the caller, with arguments 'formula','start',...
##' @return a list with components "respMod", "frame", "X", "reTrms"
nlformula <- function(mc) {
  start <- eval(mc$start, parent.frame(2L))
  if (is.numeric(start)) start <- list(nlpars = start)
  stopifnot(is.numeric(nlpars <- start$nlpars),
            lengths(nlpars) == 1L,
            length(pnames <- names(nlpars)) == length(nlpars),
            length(form <- as.formula(mc$formula)) == 3L,
            is(nlform <- eval(form[[2]]), "formula"),
            pnames %in% all.vars(nlmod <-
                as.call(nlform[[lnl <- length(nlform)]])))

  ## MM{FIXME}: fortune(106) even twice in here!
    nlform[[lnl]] <- parse(text= paste(setdiff(all.vars(form), pnames), collapse=' + '))[[1]]
    nlform <- eval(nlform)
    environment(nlform) <- environment(form)
    m <- match(c("data", "subset", "weights", "na.action", "offset"),
               names(mc), 0)
    mc <- mc[c(1, m)]
    mc$drop.unused.levels <- TRUE
    mc[[1L]] <- quote(stats::model.frame)
    mc$formula <- nlform
    fr <- eval(mc, parent.frame(2L))
    n <- nrow(fr)
    nlenv <- list2env(fr, parent=parent.frame(2L))
    lapply(pnames, function(nm) nlenv[[nm]] <- rep.int(nlpars[[nm]], n))
    respMod <- mkRespMod(fr, nlenv=nlenv, nlmod=nlmod)

    chck1(meform <- form[[3L]])
    pnameexpr <- parse(text=paste(pnames, collapse='+'))[[1]]
    formula <- as.formula(substitute(~0 + (pnameexpr) + (meform)),
                          env = environment(form))

    ## substitute  special(x | f)  with  (x | f)
    fr.form. <- noSpecials(formula, specials = lme4_specials, delete = FALSE)
    ## substitute  (x | f)  and  (x || f)  with  (x + f)
    fr.form <- sub_specials(fr.form., specials = c("|", "||"),
                            keep_args = c(2L, 2L))
    environment(fr.form.) <- environment(fr.form) <-
        environment(formula)

    fixedform <- fr.form.
    RHSForm(fixedform) <- reformulas::nobars(RHSForm(fixedform))
    frE <- do.call(rbind, lapply(seq_along(nlpars), function(i) fr)) # rbind s copies of the frame
    for (nm in pnames) # convert these variables in fr to indicators
        frE[[nm]] <- as.numeric(rep(nm == pnames, each = n))
    X <- model.matrix(fixedform, frE)
    rownames(X) <- NULL

    ## get list of calls whose first argument is a call to '|'
    ##                x | f  ->      us(x | f)
    ##     nonspecial(x | f) ->      us(x | f)
    ##        special(x | f) -> special(x | f)
    bb1 <- findbars_x(formula, specials = lme4_specials,
                      default.special = "us", target = "|",
                      expand_doublevert_method = getDoublevertDefault())
    bb0 <- lapply(bb1, function(call) {
        call <- call[[2L]]
        call[[2L]] <- substitute(0 + (foo), list(foo = call[[2L]]))
        call
    })
    reTrms <- reformulas::mkReTrms(bb0, frE, calc.lambdat = FALSE)
    reTrms <- upReTrms(reTrms, bb1) # local calc.lambdat=TRUE step
    list(respMod=respMod, frame=fr, X=X, reTrms=reTrms, pnames=pnames)
} ## {nlformula}

################################################################################
## Beginning to think about exposing tools to create devcomp lists.
## Could be useful when extending merMod objects.  Commenting them out
## however, because R CMD check is complaining:
## https://github.com/lme4/lme4/commit/8d71e439758999ea8f90eb4752487e189407ef33#commitcomment-8773017
################################################################################
##
## .dims <- function(pp, resp, nAGQ,
##                   reTrms, n, p, rcl,
##                   compDev = NULL) {
##     if(missing(rcl)) rcl <- class(resp)
##     if(missing(n)) n <- nrow(pp$V)
##     if(missing(p)) p <- ncol(pp$V)
##     c(N=nrow(pp$X), n=n, p=p, nmp=n-p,
##       nth=length(pp$theta), q=nrow(pp$Zt),
##       nAGQ=rho$nAGQ,
##       compDev=rho$compDev,
##       ## 'use scale' in the sense of whether dispersion parameter should
##       ##  be reported/used (*not* whether theta should be scaled by sigma)
##       useSc=(rcl != "glmResp" ||
##              !resp$family$family %in% c("poisson","binomial")),
##       reTrms=length(reTrms$cnms),
##       spFe=0L,
##       REML=if (rcl=="lmerResp") resp$REML else 0L,
##       GLMM=(rcl=="glmResp"),
##       NLMM=(rcl=="nlsResp"))
## }
##
## .cmp <- function(pp, resp, dims, fval,
##                  wrss, sqrLenU, pwrss,
##                  sigmaML, rcl, fac,
##                  tolPwrss = NULL,
##                  trivial.y = FALSE) {
##     if(missing(rcl)) rcl <- class(resp)
##     if(missing(fac)) fac <- as.numeric(rcl != "nlsResp")
##     if(missing(wrss)) wrss <- resp$wrss()
##     if(missing(sqrLenU)) sqrLenU <- pp$sqrL(fac)
##     if(missing(pwrss)) pwrss <- wrss + sqrLenU
##     if(missing(sigmaML)) sigmaML <- pwrss/dims[["n"]]
##     c(ldL2=pp$ldL2(), ldRX2=pp$ldRX2(), wrss=wrss,
##       ussq=sqrLenU, pwrss=pwrss,
##       drsum=if (rcl=="glmResp" && !trivial.y) resp$resDev() else NA,
##       REML=if (rcl=="lmerResp" && resp$REML != 0L && !trivial.y)
##       opt$fval else NA,
##       ## FIXME: construct 'REML deviance' here?
##       dev=if (rcl=="lmerResp" && resp$REML != 0L || trivial.y) NA else opt$fval,
##       sigmaML=sqrt(unname(if (!dims[["useSc"]] || trivial.y) NA else sigmaML)),
##       sigmaREML=sqrt(unname(if (rcl!="lmerResp" || trivial.y) NA else sigmaML*(dims[["n"]]/dims[["nmp"]]))),
##       tolPwrss=rho$tolPwrss)
## }
################################################################################

.minimalOptinfo <- function()
    list(conv = list(opt = 0L,
                     lme4 = list(messages = character(0))))

getConv <- function(x) {
    if (!is.null(x[["conv"]])) {
        x[["conv"]]
    } else x[["convergence"]]
}

getMsg <- function(x) {
    if (!is.null(x[["msg"]])) {
        x[["msg"]]
    } else if (!is.null(x[["message"]])) {
        x[["message"]]
    } else ""
}

.optinfo <- function(opt, lme4conv=NULL)
    list(optimizer = attr(opt, "optimizer"),
         control   = attr(opt, "control"),
         derivs    = attr(opt, "derivs"),
         conv      = list(opt = getConv(opt), lme4 = lme4conv),
         feval     = if (is.null(opt$feval)) NA else opt$feval,
         message   = getMsg(opt),
         warnings  = attr(opt, "warnings"),
         val       = opt$par)

##' Potentially needed in more than one place, be sure to keep consistency!
##' hack (NB families have weird names) from @aosmith16; then corrected
isNBfamily <- function(familyString)
    grepl("^Negative ?Binomial", familyString, ignore.case=TRUE)
normalizeFamilyName <- function(family) { # such as  object@resp$family
    if(isNBfamily(family$family))
        family$family <- "negative.binomial"
    family
}

##' Is it a family with no scale parameter
hasNoScale <- function(family)
    any(substr(family$family, 1L, 12L)
        == c("poisson", "binomial", "negative.bin", "Negative Bin"))



##--> ../man/mkMerMod.Rd ---Create a merMod object
##' @param rho the environment of the objective function
##' @param opt the value returned by the optimizer
##' @param reTrms reTrms list from the calling function
mkMerMod <- function(rho, opt, reTrms, fr, mc, lme4conv=NULL) {
    if(missing(mc)) mc <- match.call()
    stopifnot(is.environment(rho),
              is(pp <- rho$pp, "merPredD"),
              is(resp <- rho$resp, "lmResp"),
              is.list(opt), "par" %in% names(opt),
              c("conv", "fval") %in% substr(names(opt),1,4), ## "conv[ergence]", "fval[ues]"
              is.list(reTrms), c("flist", "cnms", "Gp", "lower") %in% names(reTrms),
              length(rcl <- class(resp)) == 1)
    n    <- nrow(pp$V)
    p    <- ncol(pp$V)
    isGLMM <- (rcl == "glmResp")
    dims <- c(N = nrow(pp$X), n=n, p=p, nmp = n-p, q = nrow(pp$Zt),
              nth = length(pp$theta),
              nAGQ= rho$nAGQ,
              compDev=rho$compDev,
              ## 'use scale' in the sense of whether dispersion parameter should
              ##  be reported/used (*not* whether theta should be scaled by sigma)
              useSc = !(isGLMM && hasNoScale(resp$family)),
              reTrms=length(reTrms$cnms),
              spFe= 0L,
              REML = if (rcl=="lmerResp") resp$REML else 0L,
              GLMM= isGLMM,
              NLMM= (rcl=="nlsResp"))
    storage.mode(dims) <- "integer"
    fac     <- as.numeric(rcl != "nlsResp")
    if (trivial.y <- (length(resp$y)==0)) {
        ## trivial model
        sqrLenU <- wrss <- pwrss <- NA
    } else {
        sqrLenU <- pp$sqrL(fac)
        wrss    <- resp$wrss()
        pwrss   <- wrss + sqrLenU
    }
    ## weights <- resp$weights
    beta    <- pp$beta(fac)
    sigmaML <- pwrss/n
    if (rcl != "lmerResp") {
        pars <- opt$par
        if (length(pars) > length(pp$theta)) beta <- pars[-(seq_along(pp$theta))]
    }
    cmp <- c(ldL2=pp$ldL2(), ldRX2=pp$ldRX2(), wrss=wrss,
             ussq=sqrLenU, pwrss=pwrss,
             drsum=if (rcl=="glmResp" && !trivial.y) resp$resDev() else NA,
             REML=if (rcl=="lmerResp" && resp$REML != 0L && !trivial.y)
                  opt$fval else NA,
             ## FIXME: construct 'REML deviance' here?
             dev=if (rcl=="lmerResp" && resp$REML != 0L || trivial.y) NA else opt$fval,
             sigmaML=sqrt(unname(if (!dims[["useSc"]] || trivial.y) NA else sigmaML)),
             sigmaREML=sqrt(unname(if (rcl!="lmerResp" || trivial.y) NA else
                                   sigmaML*(dims[["n"]]/dims[["nmp"]]))),
             tolPwrss=rho$tolPwrss)
    ## TODO:  improve this hack to get something in frame slot (maybe need weights, etc...)
    if(missing(fr)) fr <- data.frame(resp$y)
    ans <-
    new(switch(rcl, lmerResp = "lmerMod", glmResp = "glmerMod", nlsResp = "nlmerMod"),
        call=mc, frame=fr, flist=reTrms$flist, cnms=reTrms$cnms,
        Gp=reTrms$Gp, theta=pp$theta, beta=beta,
        u=if (trivial.y) rep(NA_real_,nrow(pp$Zt)) else pp$u(fac),
        lower=reTrms$lower, devcomp=list(cmp=cmp, dims=dims),
        pp=pp, resp=resp,
        optinfo = .optinfo(opt, lme4conv))
    attr(ans, "upper") <- reTrms$upper %||% rep(Inf, length(reTrms$lower))
    attr(ans, "reCovs") <- upReCovs(reTrms$reCovs %||% getReCovs(ans), rho$pp$theta)
    ans
}## {mkMerMod}

## generic argument checking
## 'type': name of calling function ("glmer", "lmer", "nlmer")
##
## NB: called from  lFormula() and glFormula()
checkArgs <- function(type,...) {
    l... <- list(...)
    if (isTRUE(l...[["sparseX"]])) warning("sparseX = TRUE has no effect at present",call.=FALSE)
    ## '...' handling up front, safe-guarding against typos ("familiy") :
    if(length(l... <- list(...))) {
        if (!is.null(l...[["family"]])) {  # call glmer if family specified
            ## we will only get here if 'family' is *not* in the arg list
            warning("calling lmer with family() is deprecated: please use glmer() instead",call.=FALSE)
            type <- "glmer"
        }
        ## Check for method argument which is no longer used
        ## (different meanings/hints depending on glmer vs lmer)
        if (!is.null(l...[["method"]])) {
            msg <- paste("Argument", sQuote("method"), "is deprecated.")
            if (type == "lmer")
                msg <- paste(msg, "Use the REML argument to specify ML or REML estimation.")
            else if (type == "glmer")
                msg <- paste(msg, "Use the nAGQ argument to specify Laplace (nAGQ=1) or adaptive",
                             "Gauss-Hermite quadrature (nAGQ>1).  PQL is no longer available.")
            warning(msg,call.=FALSE)
            l... <- l...[names(l...) != "method"]
        }
        if(length(l...)) {
            warning("extra argument(s) ",
                    paste(sQuote(names(l...)), collapse=", "),
                    " disregarded",call.=FALSE)
        }
    }
}

## check formula and data: return an environment suitable for evaluating
##  the formula.
## (1) if data is specified, return it
## (2) otherwise, if formula has an environment, use it
## (3) otherwise [e.g. if formula was passed as a string], try to use parent.frame(2)

## if #3 is true *and* the user is doing something tricky with nested functions,
## this may fail ...

## try to diagnose missing/bad data
checkFormulaData <- function(formula, data, checkLHS=TRUE,
                             checkData=TRUE, debug=FALSE) {
    wd <- tryCatch(force(data), error = identity)
    if (bad.data <- inherits(wd,"error")) {
        bad.data.msg <- wd$message
    }

    ## data not found (this *should* only happen with garbage input,
    ## OR when strings used as formulae -> drop1/update/etc.)
    ##
    if (bad.data || debug) {
        varex <- function(v, env) exists(v, envir=env, inherits=FALSE)
        allvars <- all.vars(as.formula(formula))
        allvarex <- function(env, vvec=allvars) all(vapply(vvec, varex, NA, env))
    }
    if (bad.data) { ## Choose helpful error message:
        if (allvarex(environment(formula))) {
            stop("bad 'data', but variables found in environment of formula: ",
                 "try specifying 'formula' as a formula rather ",
                 "than a string in the original model",call.=FALSE)
        } else {
            stop("bad 'data': ", bad.data.msg, call. = FALSE)
        }
    } else {
        denv <- ## The data as environment
            if (is.null(data)) {
                if (!is.null(ee <- environment(formula))) {
                    ee ## use environment of formula
            } else {
                ## e.g. no environment, e.g. because formula is a character vector
                ## parent.frame(2L) works because [g]lFormula (our calling environment)
                ## has been called within [g]lmer with env=parent.frame(1L)
                ## If you call checkFormulaData in some other bizarre way such that
                ## parent.frame(2L) is *not* OK, you deserve what you get
                ## calling checkFormulaData directly from the global
                ## environment should be OK, since trying to go up beyond the global
                ## environment keeps bringing you back to the global environment ...
                parent.frame(2L)
            }
        } else ## data specified
            list2env(data)
    }
    ##
    ## FIXME: set enclosing environment of denv to environment(formula), or parent.frame(2L) ?
    if (debug) {
        cat("Debugging parent frames in checkFormulaData:\n")
        ## find global environment -- could do this with sys.nframe() ?
        glEnv <- 1L
        while (!identical(parent.frame(glEnv),.GlobalEnv)) {
            glEnv <- glEnv+1L
        }
        ## where are vars?
        for (i in 1:glEnv) {
            OK <- allvarex(parent.frame(i))
            cat("vars exist in parent frame ", i)
            if (i == glEnv) cat(" (global)")
            cat(" ",OK, "\n")
        }
        cat("vars exist in env of formula ", allvarex(denv), "\n")
    } ## if (debug)

    stopifnot(!checkLHS || length(as.formula(formula,env=denv)) == 3)  ## check for two-sided formula
    return(denv)
}

## checkFormulaData <- function(formula,data) {
##     ee <- environment(formula)
##     if (is.null(ee)) {
##         ee <- parent.frame(2)
##     }
##     if (missing(data)) data <- ee
##     stopifnot(length(as.formula(formula,env=as.environment(data))) == 3)
##     return(data)
## }


##' Not exported; for tests (and examples) that can be slow;
##' Use   if(lme4:::testLevel() >= 1.) .....  see ../tests/README.md
testLevel <- function()
    if(nzchar(s <- Sys.getenv("LME4_TEST_LEVEL")) &&
       is.finite(s <- as.numeric(s))) s else 1

##' General conditional variance-covariance matrix
##'
##' Experimental function for estimating the variance-covariance
##' matrix of the random effects, conditional on the observed data
##' and at the (RE)ML estimate of the fixed effects and covariance
##' parameters.  Applicable for any Lambda matrix, but slower than
##' other block-by-block methods.
##' Not exported.
##'
##' TODO:
##' - Write up quick note on theory (e.g. Laplace approximation).
##' - Test.  Speed? Correctness?
##' - Do we need to think carefully about the differences
##'     between REML and ML, beyond just multiplying by a different
##'     sigma^2 estimate?
##' - is it better to do this term-by-term as in C++ code?
##'
##' @param object \code{merMod} object
##' @return Sparse covariance matrix
condVar <- function(object, scaled=TRUE) {
  Lamt <- getME(object, "Lambdat")
  L <- getME(object, "L")

  ## never do it this way! fortune("SOOOO")
  #V <- solve(L, system = "A")
  #V <- chol2inv(L)
  #s2*crossprod(Lamt, V) %*% Lamt

  LL <- solve(L, Lamt, system = "A")
  ## From ?Matrix::solve, The default, '"A"', is to solve Ax = b for x
  ##   where 'A' is sparse, positive-definite matrix that was
  ##   factored to produce 'a'.

  cc <- crossprod(Lamt, LL)
  if (scaled) cc <- sigma(object)^2*cc
  cc
}


mkMinimalData <- function(formula) {
    vars <- all.vars(formula)
    nVars <- length(vars)
    matr <- matrix(0, 2, nVars)
    data <- as.data.frame(matr)
    setNames(data, vars)
}

##' Make template for mixed model parameters
mkParsTemplate <- function(formula, data){
    if(missing(data)) data <- mkMinimalData(formula)
    mfRanef <- model.frame( reformulas::subbars(formula), data)
    mmFixef <- model.matrix( reformulas::nobars(formula) , data)
    reTrms <- reformulas::mkReTrms( reformulas::findbars(formula), mfRanef)
    cnms <- reTrms$cnms
    thetaNamesList <- mapply(mkPfun(), names(cnms), cnms)
    thetaNames <- unlist(thetaNamesList)
    betaNames <- colnames(mmFixef)
    list(beta  = setNames(numeric(length( betaNames)),  betaNames),
         theta = setNames(reTrms$theta, thetaNames),
         sigma = 1)
}

##' Make template for mixed model data
##'
##' Useful for simulating balanced designs and for
##' getting started on unbalanced simulations
##'
##' @param formula formula
##' @param data data -- not necessary
##' @param nGrps number of groups per grouping factor
##' @param rfunc function for generating covariate data
##' @param ... additional parameters for rfunc
mkDataTemplate <- function(formula, data,
                           nGrps = 2, nPerGrp = 1,
                           rfunc = NULL, ...){
    if(missing(data)) data <- mkMinimalData(formula)
    grpFacNames <- unique(barnames(reformulas::findbars(formula)))
    varNames <- all.vars(formula)
    covariateNames <- setdiff(varNames, grpFacNames)
    nGrpFac <- length(grpFacNames)
    nCov <- length(covariateNames)
    grpFac <- gl(nGrps, nPerGrp)
    grpDat <- expand.grid(replicate(nGrpFac, grpFac, simplify = FALSE))
    colnames(grpDat) <- grpFacNames
    nObs <- nrow(grpDat)
    if(is.null(rfunc)) rfunc <- function(n, ...) rep(0, n)
    params <- c(list(nObs), list(...))
    covDat <- as.data.frame(replicate(nCov, do.call(rfunc, params),
                                      simplify = FALSE))
    colnames(covDat) <- covariateNames
    cbind(grpDat, covDat)
}

##' very flexible and convenient wrt formula,
##' very unflexible wrt everything else
##'
##' starting to get a little too sugary?
quickSimulate <- function(formula, nGrps, nPerGrp, family = gaussian) {
    pr <- mkParsTemplate(formula)
    dt <- mkDataTemplate(formula, nGrps = nGrps, nPerGrp = nPerGrp, rfunc = rnorm)
    response <- deparse(formula[[2]])
    dt[[response]] <- simulate(formula, newdata = dt, newparams = pr, family = family)[[1]]
    return(dt)
}

#----------------------------------------------------------------------
# formula parsing sugar
#----------------------------------------------------------------------

##' these functions pick up where findbars leaves off, in terms of sugar
##' @param REtrm an element of the result of findbars
##' @param REtrms the result of findbars
##' @return \code{reexpr} gives a one-sided formula with the linear
##' model formula for the raw model matrix. \code{grpfact} gives an
##' expression with the name of the grouping factor associated with
##' the raw model matrix. \code{termnms} gives a character vector with
##' the names of the random effects terms.
reexpr <- function(REtrm) substitute( ~ foo, list(foo = REtrm[[2]]))
grpfact <- function(REtrm) substitute(factor(fac), list(fac = REtrm[[3]]))
termnms <- function(REtrms) vapply(REtrms, deparse1, "")

##' mmList(): list of model matrices
##' ------    called from getME() & model.matrix(*, "randomListRaw")
mmList <- function(object, ...) UseMethod("mmList")
mmList.merMod <- function(object, ...) mmList(formula(object), model.frame(object))
mmList.formula <- function(object, frame, ...) {
    bars <- reformulas::findbars(object)
    mm <- setNames(lapply(bars, function(b) model.matrix(eval(reexpr(b), frame), frame)),
                   termnms(bars))
    grp <- lapply(lapply(bars, grpfact), eval, frame)
    nl <- vapply(grp, nlevels, 1L)
    if (any(diff(nl) > 0))
        mm[order(nl, decreasing=TRUE)]
    else
        mm
}
##' examples  ---FIXME?--- put in tests // or export + 'real examples'
if(FALSE) {
    library(lme4)
    m <- lmer(Reaction ~ Days + (Days | Subject), sleepstudy)
    gm <- glmer(cbind(incidence, size-incidence) ~ period + (1|herd), cbpp, binomial)
    simForm <- y ~ x + z + (x | f) + (z | g)
    ## ::: triggers R CMD check NOTE
    ## simDat <- lme4:::quickSimulate(simForm, 10, 5)
    simDat <- simDat[simDat$f != "10", ] # unbalancedish design requiring
                                        # a flip in the order of terms
    sm <- lmer(simForm, simDat)
    ## ::: triggers R CMD check NOTE
    ## lme4:::mmList.merMod(m)
    ## lme4:::mmList.merMod(gm)
    ## smmm <- lme4:::mmList.merMod(sm)
}

nloptwrap <- local({
    ## define default control values in environment of function ...
    defaultControl <- list(algorithm="NLOPT_LN_BOBYQA",
                           xtol_abs=1e-8, ftol_abs=1e-8, maxeval=1e5)
    ##
    function(par, fn, lower, upper, control=list(),...) {
        for (n in names(defaultControl))
            if (is.null(control[[n]])) control[[n]] <- defaultControl[[n]]
        res <- nloptr(x0=par, eval_f=fn, lb=lower, ub=upper, opts=control, ...)
        with(res, list(par   = solution,
                       fval  = objective,
                       feval = iterations,
                       ## ?nloptr: "integer value with the status of the optimization (0 is success)"
                       ## most status>0 are fine (e.g. 4 "stopped because xtol_rel was reached"
                       ## but status 5 is "ran out of evaluations"
                       conv  = if (status<0 || status==5) status else 0,
                       message = message))
    }
})

nlminbwrap <- function(par, fn, lower, upper, control=list(), ...) {
    if (!is.null(control$maxfun)) {
        control$eval.max <- control$maxfun
        control$maxfun <- NULL
    }
    res <- nlminb(start = par, fn, gradient = NULL, hessian = NULL,
                  scale = 1, lower = lower, upper = upper,
                  control = control, ...)
    list(par = res$par, fval = res$objective,
         conv = res$convergence, message = res$message)
}

glmerLaplaceHandle <- function(pp, resp, nAGQ, tol, maxit, verbose) {
    .Call(glmerLaplace, pp, resp, nAGQ, tol, as.integer(maxit), verbose)
}

isFlexLambda <- function() FALSE


#' convert a list of matrices (n, pxp blocks) to a  p x p x n  array
mlist_to_array <- function(m) {
    p <- nrow(m[[1]])
    n <- length(m)
    array(unlist(lapply(m,as.matrix)),dim=c(p,p,n))
}
#' @inheritParams bdiag_to_array
bdiag_to_mlist <- function(m,n) {
    if (length(n)==1 && n<nrow(m)) {
        n <- rep(n,nrow(m)%/%n)
    }
    mm <- list()
    k <- 1
    for (i in seq_along(n)) {
        mm[[i]] <- m[k:(k+n[i]-1),k:(k+n[i]-1),drop=FALSE]
        k <- k + n[i]
    }
    return(mm)
}
##' convert a block-diagonal matrix to a pxpxn array
##' @param m a block-diagonal matrix (typically sparse)
##' @param n vector of block sizes (if length-1, will be replicated to be consistent
##' with the matrix dimensions)
##' @examples
##' mm <- Matrix::bdiag(matrix(1:4,2,2),matrix(2:5,2,2),matrix(3:6,2,2))
##' mm2 <- blkmatrix_to_matrixlist(mm,2)
##' bdiag_to_array(mm,2)
##' array_to_bdiag(bdiag_to_array(mm,2))
bdiag_to_array <- function(m,n) {
    mlist_to_array(
        bdiag_to_mlist(m,n))
}
array_to_bdiag <- function(a) {
    stopifnot(length(dim(a))==3,dim(a)[1]==dim(a)[2])
    p <- dim(a)[1]
    mlist <- split(a,slice.index(a,3))
    mlist <- lapply(mlist,matrix,nrow=p,ncol=p)
    return(.bdiag(mlist))
}


augment.RE <- function(object,rr=ranef(object)) {
    alist <- arrange.condVar(object,condVar(object))
    for (i in seq_along(rr)) {
        attr(rr[[i]],"postVar") <- alist[[i]]
    }
    class(rr) <- "ranef.mer"
    rr
}

## reorganize condVar matrix into appropriate list of arrays/lists of arrays
arrange.condVar <- function(object,cv) {
    rp <- rePos$new(object)
    trms <- rp$terms      ## mapping between grouping vars and RE terms
    n <- diff(rp$offsets) ## total number of modes per term
    cv2 <- bdiag_to_mlist(cv,n)
    cv3 <- Map(bdiag_to_array,cv2,rp$ncols)
    names(cv3) <- rp$cnms
    res <- list()
    for (i in seq_along(trms)) {
        tt <- trms[[i]]
        if (length(tt)==1) {
            ## keep single-term-per-factor condVar structures
            ## as naked arrays (not list containing a single array)
            ## for back-compatibility
            res[[i]] <- cv3[[tt]]
        } else {
            ## list of arrays
            res[[i]] <- cv3[tt]
        }
    }
    names(res) <- names(rp$flist)
    return(res)
}

## generic machinery for setting parallel options
## uses eval() (as in family()$initialize) to avoid too much list
initialize.parallel <- expression({
    if (length(parallel) > 1) parallel <- match.arg(parallel)
    if (!is.null(cl)) {
        stopifnot(inherits(cl, "cluster"))
        parallel <- "snow"
    }
    if (parallel == "multicore") {
        stopifnot(is.numeric(ncpus), length(ncpus) == 1L, is.finite(ncpus), ncpus >= 1L)
        if (.Platform$OS.type == "windows" || ncpus == 1L)
            parallel <- "no"
    } else if (parallel == "snow") {
        if (is.null(cl)) {
            stopifnot(is.numeric(ncpus), length(ncpus) == 1L, is.finite(ncpus), ncpus >= 1L)
            if (ncpus == 1L)
                parallel <- "no"
        }
    }
})

getSingTol <- function() getOption("lme4.singular.tolerance", 1e-4)

lme4_testlevel <- function() if (nzchar(s <- Sys.getenv("LME4_TEST_LEVEL"))) as.numeric(s) else 1


# stolen from car package
# the following unexported function is useful for combining results of parallel computations
combineLists <- function(..., fmatrix="list", flist="c", fvector="rbind", 
                         fdf="rbind", recurse=FALSE){
    # combine lists of the same structure elementwise
    
    # ...: a list of lists, or several lists, each of the same structure
    # fmatrix: name of function to apply to matrix elements
    # flist: name of function to apply to list elements
    # fvector: name of function to apply to data frame elements
    # recurse: process list element recursively
    
    frecurse <- function(...){
        combineLists(..., fmatrix=fmatrix, fvector=fvector, fdf=fdf, 
                     recurse=TRUE)
    }
    
    if (recurse) flist="frecurse"
    list.of.lists <- list(...)
    if (length(list.of.lists) == 1){
        list.of.lists <- list.of.lists[[1]]
        list.of.lists[c("fmatrix", "flist", "fvector", "fdf")] <- 
            c(fmatrix, flist, fvector, fdf)
        return(do.call("combineLists", list.of.lists))
    }
    if (any(!sapply(list.of.lists, is.list))) 
        stop("arguments are not all lists")
    len <- sapply(list.of.lists, length)
    if (any(len[1] != len)) stop("lists are not all of the same length")
    nms <- lapply(list.of.lists, names)
    if (any(unlist(lapply(nms, "!=", nms[[1]])))) 
        stop("lists do not all have elements of the same names")
    nms <- nms[[1]]
    result <- vector(len[1], mode="list")
    names(result) <- nms
    for(element in nms){
        element.list <- lapply(list.of.lists, "[[", element)
#        clss <- sapply(element.list, class)
        clss <- lapply(element.list, class)
#        if (any(clss[1] != clss)) stop("list elements named '", element,
        if (!all(vapply(clss, function(e) all(e == clss[[1L]]), NA)))
          stop("list elements named '", element, "' are not all of the same class")
        
        is.df <- is.data.frame(element.list[[1]])
        fn <- if (is.matrix(element.list[[1]])) fmatrix 
        else if (is.list(element.list[[1]]) && !is.df) flist 
        else if (is.vector(element.list[[1]])) fvector
        else if (is.df) fdf
        else stop("list elements named '", element, 
                  "' are not matrices, lists, vectors, or data frames")
        result[[element]] <- do.call(fn, element.list)
    }
    result
}

## copied from glmmTMB::check_dots
checkDots <- function (..., .ignore = NULL, .action = "stop") 
{
    L <- list(...)
    if (length(.ignore) > 0) {
        L <- L[!names(L) %in% .ignore]
    }
    if (length(L) > 0) {
        FUN <- get(.action)
        FUN("unknown arguments: ", paste(names(L), collapse = ","))
    }
    return(NULL)
}

## quadratic form from emulator package:
## quad.tform == x %*% M %*% t(x)
## quad.tdiag == diag(quad.tform(M, x)
## rowSums(tcrossprod(Conj(x), M) * x)
quad.tdiag <- function(M, x) {
    ## only real-valued, so drop Conj
    rowSums(tcrossprod(x, M) * x)
}

##' attempt to modularize vcov scaling; more details in the autoscale vignette
##' @param vv represents the variance-covariance matrix before modification
##' @param sc represents the scale vector
##' @param ce represents the center vector
scale_vcov <- function(vv, sc, ce) {
  other_vars <- setdiff(colnames(vv), "(Intercept)")
  ## 1. Modifying the intercept
  sig_0sq <- vv["(Intercept)", "(Intercept)"]
  sig_0isq <- vv["(Intercept)", other_vars]
  total1 <- -2 *sum((ce/sc) * sig_0isq)
  small_vv <- as.matrix(vv[other_vars, other_vars])
  total2 <- crossprod(ce / sc, small_vv %*% (ce / sc))[[1]]
  vv["(Intercept)", "(Intercept)"] <- sig_0sq + total1 + total2
  ## 2. Modifying without intercept
  updated_2 <- (sig_0isq)/sc - (small_vv %*% (ce/sc))/sc
  vv["(Intercept)", other_vars] <- updated_2
  vv[other_vars, "(Intercept)"] <- updated_2
  vv[other_vars, other_vars] <- vv[other_vars, other_vars] * outer(1/sc, 1/sc)
  vv <- as(vv, "dpoMatrix")
}

##' Used for padding NAs to Cmat accordingly in predict.merMod
##' @param mat represents the matrix that needs to be modified
##' @param mat_names represents the names of the new modified matrix
##' @param insert_after represents the placement before the zeros that need to 
##' be added
##' @param n_add represents the number rows/columns that will be padded with zeros
zero_padding <- function(mat, mat_names, insert_after, n_add = 1) {
  mat <- as.matrix(mat)
  old_dim <- nrow(mat)
  new_dim <- old_dim + n_add
  m_pad <- matrix(0, new_dim, new_dim)
  rownames(m_pad) <- mat_names
  colnames(m_pad) <- mat_names
  
  ## Top right corner
  m_pad[1:insert_after, 1:insert_after] <- mat[1:insert_after, 1:insert_after]
  
  ## Top left corner
  m_pad[1:insert_after, (insert_after + n_add + 1):new_dim] <-
    mat[1:insert_after, (insert_after + 1):old_dim]
  
  ## Bottom right corner
  m_pad[(insert_after + n_add + 1):new_dim, 1:insert_after] <-
    mat[(insert_after + 1):old_dim, 1:insert_after]
  
  ## Bottom left corner
  m_pad[(insert_after + n_add + 1):new_dim, (insert_after + n_add + 1):new_dim] <-
    mat[(insert_after + 1):old_dim, (insert_after + 1):old_dim]
  
  m_pad
}

##' if allow.new.levels = TRUE, then adds 0 padding to Cmat for prediction
##' @param Cmat represents Cmat that was computed prior to subsetting
##' @param C_factors represents the factors explicitly shown in Cmat
##' @param Z_factors represents the factors represented in the Z matrix, which
##' includes only levels of groups that need to be predicted
##' @param Cmat_names represents the names of the Cmat matrix
##' @param cnms same as cnms from object
pad_Cmat <- function(Cmat, C_factors, Z_factors, Cmat_names, cnms){
  n_padded = 0
  for (grp in intersect(names(C_factors), names(Z_factors))) {
    n_lvl <- length(levels(C_factors[[grp]]))
    added_levels <- setdiff(levels(Z_factors[[grp]]), 
                            levels(C_factors[[grp]]))
    if ((n_add <- length(added_levels)) == 0) next
    levels(C_factors[[grp]]) <- c(levels(C_factors[[grp]]), added_levels) 
    
    ## add names for clarity
    added_nms <- as.vector(sapply(added_levels, function(lv)
      paste0(grp, ".", lv, ".", cnms[[grp]])
    ))
    ## add padding
    n_padded <- n_padded + n_lvl * length(cnms[[grp]])
    n_new <- n_add * length(cnms[[grp]])
    
    Cmat_names <- c(Cmat_names[1:n_padded], 
                    added_nms, 
                    Cmat_names[(n_padded+1):length(Cmat_names)])
    ## alter Cmat
    Cmat <- zero_padding(Cmat, Cmat_names, insert_after = n_padded,
                         n_add = n_new)
    n_padded <- n_padded + n_new
  }
  list("Cmat" = Cmat, "C_factors" = C_factors, "Cmat_names" = Cmat_names)
}


getDoublevertDefault <- function() {
  getOption("lme4.doublevert.default", "split")
}

na.action.merMod <- function(object, ...) {
  na.action(model.frame(object))
}

if (lme4:::testLevel() > 1 || .Platform$OS.type != "windows") withAutoprint({
    library("lme4")
    library("testthat")

    L <- load(system.file("testdata", "lme-tst-fits.rda",
                          package="lme4", mustWork=TRUE))
    ## -> "fit_*" objects

    fm1 <- fit_sleepstudy_2
    c0  <- confint(fm1, method="Wald")
    c0B <- confint(fm1, method="Wald",parm="Days")
    expect_equal(c0["Days",],c0B["Days",])
    expect_equal(c(c0B),c(7.437592,13.496980),tolerance=1e-6)
    set.seed(101)

    for (bt in c("norm", "basic", "perc")) {
        suppressWarnings(
            confint(fm1, method="boot", boot.type=bt, nsim=10,quiet=TRUE))
    }
    for (bt in c("stud","bca","junk")) {
        expect_error(confint(fm1, method="boot", boot.type=bt, nsim=10),
                     "should be one of")
    }
    if((testLevel <- lme4:::testLevel()) > 1) {
        pr1.56 <- profile(fm1, which = 5:6)
        c1 <- confint(pr1.56, method="profile")
        expect_equal(c0[5:6,],c1,tolerance=2e-3)  ## expect Wald and profile _reasonably_ close
        print(c1,digits=3)
        ##
        c6 <- confint(pr1.56, "Days")
        expect_equal(c1[2, , drop=FALSE], c6)
        c2  <- confint(fm1,method="boot",nsim=50,parm=5:6)
        ## expect_error(confint(fm1,method="boot",nsim=50,parm="Days"),
        ##  "must be specified as an integer")
        expect_equal(c1,c2,tolerance=2e-2)
        print(c2,digits=3)
    }
    if (testLevel > 10) {
        set.seed(101)
        print(c1B <- confint(fm1, method="profile"))
        print(c2B <- confint(fm1, method="boot"))
        expect_equal(unname(c1B), unname(c2B), tolerance=2e-2)
    }
}) ## skip if windows/testLevel<1

## NOTE: Unlike the rest of the package, the functions in this file
## are licensed under the MIT License to encourage incorporation.
## 
## Copyright (c) 2020 Pavel N. Krivitsky and Benjamin Bolker
## 
## Permission is hereby granted, free of charge, to any person obtaining a copy
## of this software and associated documentation files (the "Software"), to deal
## in the Software without restriction, including without limitation the rights
## to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
## copies of the Software, and to permit persons to whom the Software is
## furnished to do so, subject to the following conditions:
## 
## The above copyright notice and this permission notice shall be included in all
## copies or substantial portions of the Software.
## 
## THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
## IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
## FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
## AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
## LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
## OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
## SOFTWARE.
##

#' A `simulate` Method for `formula` objects that dispatches based on the Left-Hand Side
#'
#' This method evaluates the left-hand side (LHS) of the given formula and
#' dispatches it to an appropriate method based on the result by
#' setting an nonce class name on the formula.
#'
#' @param object a one- or two-sided [`formula`].
#' @param nsim,seed number of realisations to simulate and the random
#'   seed to use; see [simulate()].
#' @param ... additional arguments to methods.
#' @param basis if given, overrides the LHS of the formula for the
#'   purposes of dispatching.
#' @param newdata,data if passed, the `object`'s LHS is evaluated in
#'   this environment; at most one of the two may be passed.
#'
#' The dispatching works as follows:
#'
#' 1. If `basis` is not passed, and the formula has an LHS the
#'    expression on the LHS of the formula in the `object` is
#'    evaluated in the environment `newdata` or `data` (if given), in
#'    any case enclosed by the environment of `object`. Otherwise,
#'    `basis` is used.
#' 
#' 1. The result is set as an attribute `".Basis"` on `object`. If
#'    there is no `basis` or LHS, it is not set.
#' 
#' 1. The class vector of `object` has `c("formula_lhs_\var{CLASS}",
#'    "formula_lhs")` prepended to it, where \var{CLASS} is the class
#'    of the LHS value or `basis`. If LHS or `basis` has multiple
#'    classes, they are all prepended; if there is no LHS or `basis`,
#'    `c("formula_lhs_", "formula_lhs")` is.
#' 
#' 1. [simulate()] generic is evaluated on the new `object`, with all
#'    arguments passed on, excluding `basis`; if `newdata` or `data`
#'    are missing, they too are not passed on. The evaluation takes
#'    place in the parent's environment.
#'
#' A "method" to receive a formula whose LHS evaluates to \var{CLASS}
#' can therefore be implemented by a function
#' `simulate.formula_lhs_\var{CLASS}()`. This function can expect a
#' [`formula`] object, with additional attribute `.Basis` giving the
#' evaluated LHS (so that it does not need to be evaluated again).
#'
#' @export
## See https://github.com/lme4/lme4/issues/566 for further discussion
simulate.formula <- function(object, nsim=1, seed=NULL, ..., basis, newdata, data) {
  ## utility fun for generating new class
  cfun <- function(cc) {
    c(paste0("formula_lhs_", cc), "formula_lhs", class(object))
  }

  ## grab the arguments and the call and replace the function to be called with stats::simulate
  cl <- match.call()
  cl[[1L]] <- quote(stats::simulate)
  
  if (length(object)==3 || !missing(basis)) {  ## two-sided formula or basis given
    if (missing(basis)) { # If basis is not passed, evaluate LHS.
      if (!missing(data) && !missing(newdata)) stop("At most one of ", sQuote("data"), " or ", sQuote("newdata"), " can be specified.")

      evaldata <- if (!missing(data)) data
                  else if(!missing(newdata)) newdata
                  else environment(object)

      lhs <- object[[2L]]
      .Basis <-  try(eval(lhs, envir=evaldata,
                          enclos=environment(object)), silent=TRUE)

      if (inherits(.Basis,"try-error")) {
        ## can't evaluate LHS
        stop(simpleError(paste("Error evaluating the left-hand side of the formula:", .Basis)))
      }

    } else { # Otherwise, override LHS.
      .Basis <- basis
    }

    ## Set the basis object and class.
    attr(object,".Basis") <- .Basis
    class(object) <- cfun(class(.Basis))

  } else {  ## one-sided
    class(object) <- cfun("")
  }

  ## Replace the dispatched-on argument (object) with the updated formula.
  cl[["object"]] <- object
  ## If data argument was not actually passed, remove it from the call.
  if(missing(data)) cl <- cl[names(cl)!="data"]
  ## If newdata argument was not actually passed, remove it from the call.
  if(missing(newdata)) cl <- cl[names(cl)!="newdata"]
  ## Always remove basis from the call (since it's an attribute of object now).
  cl <- cl[names(cl)!="basis"]

  # Evaluate the modified call as if in the environment from which simulate.formula() was called. (A poor man's NextMethod().)
  eval(cl, parent.frame())
}

#' @describeIn simulate.formula A function to catch the situation when there is no method implemented for the class to which the LHS evaluates.
#' 
#' @export
simulate.formula_lhs <- function(object, nsim=1, seed=NULL, ...){
  stop("No applicable method for LHS of type ", paste0(sQuote(class(attr(object, ".Basis"))), collapse=", "), ".")
}

## ----preliminaries,include=FALSE,cache=FALSE,message=FALSE,echo=FALSE----
options(width=69, show.signif.stars=FALSE, str=strOptions(strict.width="cut"))
library(knitr)
library(lme4)
library(ggplot2)
theme_set(theme_bw())
revguide <- guide_legend(reverse = TRUE)
library(grid)
library(gridExtra)
zmargin <- theme(panel.margin=unit(0,"lines"))
library(lattice)
library(DHARMa)
library(performance)
opts_chunk$set(engine='R',dev='pdf',fig.width=10,
               fig.height=6.5,prompt=TRUE,
               ## strip.white=all,
               error=FALSE, ## stop on error
               tidy=FALSE,comment=NA)
## using knitr::knit2pdf() may lead to xcolor 'incompatible color definition'
## warnings. The code below resolves that, but simultaneously breaks
## vignette compilation in other contexts ...
## https://tex.stackexchange.com/questions/148188/knitr-xcolor-incompatible-color-definition               
## knit_hooks$set(document = function(x) {sub('\\usepackage[]{xcolor}', '\\usepackage[dvipsnames]{xcolor}', x, fixed = TRUE)})


## ----m1------------------------------------------------------------
print(m1 <- glmer(cbind(incidence, size-incidence) ~ period + (1|herd),
      cbpp, binomial), corr=FALSE)

## ----density-utils, echo = FALSE-----------------------------------
zeta <- function(m, zmin=-3, zmax=3, npts=301L, rank = FALSE) {
    stopifnot (is(m, "glmerMod"),
               length(m@flist) == 1L,    # single grouping factor
              length(m@cnms[[1]]) == 1L) # single column for that grouping factor
    pp <- m
    rr <- m@resp
    u0 <- getME(pp,"u")
    sd <- 1/getME(pp,"L")@x
    ff <- as.integer(getME(pp,"flist")[[1]])
    fc <- getME(pp,"X") %*% getME(pp,"beta") # fixed-effects contribution to linear predictor
    ZL <- t(getME(pp,"Lambdat") %*% getME(pp,"Zt"))
    dc <- function(z) { # evaluate the unscaled conditional density on the deviance scale
        uu <- u0 + z * sd
        rr$updateMu(fc + ZL %*% uu)
        unname(as.vector(tapply(rr$devResid(), ff, sum))) + uu * uu
    }
    zvals <- seq(zmin, zmax, length.out = npts)
    d0 <- dc(0) # because this is the last evaluation, the model is restored to its incoming state
    # signed square root
    sqrtmat <- t(sqrt(vapply(zvals, dc, d0, USE.NAMES=FALSE) - d0)) *
      array(ifelse(zvals < 0, -1, 1), c(npts, length(u0)))
    if (rank) {
      ## order by 'badness' (variance of normalized profile)
      nvals <- exp(-0.5*sqrtmat^2)/sqrt(2*pi)/dnorm(zvals)
      colnames(sqrtmat) <- seq(ncol(sqrtmat))
      sqrtmat <- sqrtmat[,order(apply(sqrtmat, 2, var), decreasing = TRUE)]
    }
    list(zvals=zvals,
         sqrtmat= sqrtmat 
         )
}
smult <- function(x, y) { sweep(x, y, MARGIN = 2, FUN = "*") }
lagr_basis_i <- function(i, xvec, q) {
  num <- sapply(xvec, \(x) prod(x-q[-i]))
  denom <- prod(q[i]-q[-i])
  num/denom
}
lagr_basis_deriv_i <- function(i, xvec, q) {
  if (length(q)==1) return(rep(0, length(xvec)))
  f <- function(j) sapply(xvec, \(x) prod(x-q[-c(i,j)]))
  num <- rowSums(sapply(seq_along(q)[-i], f))
  denom <- prod(q[i]-q[-i])
  num/denom
}
lagr_basis <- function(xvec, q, deriv = FALSE) {
  FUN <- if (!deriv) lagr_basis_i else lagr_basis_deriv_i
  sapply(seq_along(q), \(i) FUN(i, xvec, q))
}
calc_h <- function(xvec, Q) {
  gg <- GHrule(Q)
  ell <- lagr_basis(xvec, gg[,"z"])
  ## dl_i/dx(x_i)
  bb <- lagr_basis(gg[,"z"], gg[,"z"], deriv = TRUE)
  d_ell <- if (length(bb) == 1) bb else diag(bb)
  xdiff <- outer(xvec, gg[,"z"], "-")
  h <- (1 - 2*smult(xdiff, d_ell))*ell^2
  hbar <- xdiff*ell^2
  list(h = h, hbar = hbar)
}
## FIXME: modify to work when Q = 1
ghq_fun <- function(Q, zvals, f, fbar, H = NULL) {
  q <- GHrule(Q)[,"z"]
  H <- H %||% calc_h(zvals, Q = Q)
  dz <- diff(zvals[1:2])
  Qvals <- sapply(q, \(x) which.min(abs(zvals - x)))
  if (any(Qvals == 1)) stop("need to extend zm calculation")
  q_approx <- rowSums(smult(H$h, f[Qvals])) +
    rowSums(smult(H$hbar, fbar[Qvals]))
  q_approx
}
calc_q <- function(Q, zm) {
  dz <- diff(zm$zvals[1:2])
  dmat <- exp(-0.5*zm$sqrtmat^2)/sqrt(2*pi)/dnorm(zm$zvals)
  ## finite difference (NA to match dimension)
  d_dmat <- rbind(NA,apply(dmat, 2, function(x) diff(x)/dz))
  H <- calc_h(zm$zvals, Q = Q)
  res <- sapply(1:ncol(dmat),
                \(i) ghq_fun(Q, zm$zvals, dmat[,i], d_dmat[,i], H = H)
                )
  attr(res, "nAGQ") <- Q
  res
}
plot_zeta <- function(z, q_approx = NULL, norm = FALSE,
                      ylab = if (!norm) "density" else "t(z)",
                      which = NULL, layout = c(5,3), ylim = NULL,
                      auto.key = list(lines = TRUE, points = FALSE)) {
  dmat <- exp(-0.5*z$sqrtmat^2)/sqrt(2*pi)
  Q <- attr(q_approx, "nAGQ") ## rescue attribute before subsetting
  if (!is.null(which)) {
    dmat <- dmat[, which]
    q_approx <- q_approx[, which]
  }
  ## columnwise scaling/unscaling
  ## q_approx is normalized by default
  if (norm) dmat <- dmat/dnorm(z$zvals)
  if (!norm) q_approx <- q_approx*dnorm(z$zvals)
  
  nobs   <- nrow(dmat)
  npanel <- ncol(dmat)
  idx    <- gl(npanel, nobs)
  ngrp <- 3
  
  ref_val <- if (!norm) dnorm(z$zvals) else rep(1, nobs)
  ref_val <- rep(ref_val, npanel)

  grps <- c("conditional\ndensity",
            sprintf("AGQ(%d)", Q),
            if (!norm) "N(0,1)" else "")
  groups <- factor(rep(grps, each = nobs * npanel),
                   levels = grps)
  resp <- c(as.vector(dmat), as.vector(q_approx), ref_val)
  ylim <- ylim %||% extendrange(resp)
  zvec <- rep(z$zvals, ngrp * npanel)
  xyplot(resp ~ zvec | rep(idx, ngrp),
         groups = groups,
         type = c("g", "l"), aspect = 0.6, layout = layout,
         xlab = "z", ylab = ylab,
         auto.key = auto.key,
         par.settings = list(superpose.line = list(lty = 1:3)),
         ylim = ylim)
}

## ----densities,echo=FALSE,fig.height=5-----------------------------
zm <- zeta(m1, -3.750440, 3.750440)
qq <- calc_q(3, zm)
plot_zeta(zm, q_approx = qq)

## ----GHrule5-------------------------------------------------------
GHrule(5)

## ----tfunc,echo=FALSE----------------------------------------------
plot_zeta(zm, q_approx = qq, norm = TRUE, ylim = c(0.5, 1.5))

## ----toenail, echo = FALSE-----------------------------------------
data("toenail", package = "lme4")
ttab <- with(toenail, table(table(patientID)))
## sum(as.numeric(names(ttab))*ttab/sum(ttab))

## ----toenailplot,echo=FALSE----------------------------------------
data("toenail", package = "lme4")
toemod <- glmer(outcome ~ time*treatment + (1 | patientID), 
                     data = toenail, family = binomial(link = "logit"))
zm <- zeta(toemod)
q_toe <- calc_q(3, zm)
set.seed(101); s <- sample(ncol(zm$sqrtmat), size = 9)

gridExtra::grid.arrange(
             plot_zeta(zm, q_approx = q_toe, which = s, layout = c(3,3), auto.key = FALSE),
             plot_zeta(zm, which = s, norm = TRUE, q_approx = q_toe, layout = c(3,3),
                       ylim = c(0, 5)), nrow = 1,
             ## widths hand-tweaked to equalize plot size with/without scale
             widths = c(0.5, 0.63))


## ----newcbpp2,echo=FALSE-------------------------------------------
cbpp2 <- read.csv(system.file("vignette_data", "cbpp2.csv",
            package = "lme4"))
cbpp2 <- transform(cbpp2,period=factor(period),
                   treatment=factor(treatment,
                   levels=c("Partial/null","Complete","Unknown")))

## ----cbppPlot,echo=FALSE,fig.cap="Incidence (proportion of cows becoming seropositive per observation period) vs. period. Colours show treatment category for each herd; point sizes reflect the number of seronegative cows at the start of each period. Lines connect the sets of observations from each herd.",fig.scap="Incidence"----
g0 <- ggplot(cbpp2,aes(period, incidence/size, colour=treatment, shape = treatment)) +
    ## scale_y_continuous(limits = c(0, 1))+
    labs(x="Period",y="Proportional incidence")+
    scale_colour_brewer(palette="Dark2")
## faceted plot
## g0 + 
##     geom_point(aes(size = size)) +
##     facet_wrap(~herd) +
##     zmargin
## 
g0 +  geom_point(aes(size = size),alpha=0.4) +
    geom_line(aes(group=herd),alpha=0.3)
## geom_boxplot(fill="black",alpha=0.1,width=0.5,
## aes(group=interaction(period,treatment)))
## 
## boxplot(incidence/size ~ period, data = cbpp, las = 1,
##        xlab = 'Period', ylab = 'Probability of sero-positivity')

## ----cbppModelI----------------------------------------------------
gm1 <- glmer(incidence/size ~ period + treatment + avg_size + (1 | herd),
             family = binomial,
             data = cbpp2, weights = size)

## ----cbpp_diag,eval=FALSE,echo=FALSE-------------------------------
# ## diagnostic junk about N-M vs bobyqa differences
# tmpf <- function(x) c(deviance(x), unlist(getME(x,c("theta","beta"))))
# gm1 <- glmer(incidence/size ~ period + treatment + avg_size + (1 | herd),
#              family = binomial,
#              data = cbpp2, weights = size)
# gm1B <- update(gm1,control=glmerControl(optimizer="bobyqa"))
# cbind(tmpf(gm1),tmpf(gm1B))
# dd1 <- update(gm1,devFunOnly=TRUE)
# dd1B <- update(gm1B,devFunOnly=TRUE)
# gm1@optinfo$derivs$gradient
# gm1B@optinfo$derivs$gradient
# gm1@optinfo$derivs$gradient/gm1B@optinfo$derivs$gradient
# library(numDeriv)
# grad(dd1,tmpf(gm1)[-1])
# grad(dd1B,tmpf(gm1B)[-1])

## ----cbppModelII, warning=FALSE, message=FALSE---------------------
cbpp2 <- transform(cbpp2,obs=factor(seq(nrow(cbpp2))))
bob_opt <- glmerControl(optimizer = "bobyqa")
## herd and observation-level REs
gm2 <- update(gm1,.~.+(1|obs), control = bob_opt)  
## observation-level REs only
gm3 <- update(gm1,.~.-(1|herd)+(1|obs))  

## ----diagnose, echo=FALSE, warning=FALSE, message=FALSE, results='hide'----
aa <- allFit(gm2, verbose=FALSE)
ok <- summary(aa)[["which.OK"]]
lapply(aa[ok], \(x) x@optinfo$conv$lme4$messages)

## ----summary1,echo=FALSE-------------------------------------------
ss <- summary(gm1)
cc <- capture.output(print(summary(gm1)))
reRow <- grep("^Random effects",cc)
cat(cc[1:(reRow-2)],sep="\n")

## ----summary2,echo=FALSE-------------------------------------------
feRow <- grep("^Fixed effects",cc)
cat(cc[reRow:(feRow-2)],sep="\n")

## ----summary3,echo=FALSE-------------------------------------------
corRow <- grep("^Correlation",cc)
cat(cc[feRow:(corRow-2)],sep="\n")

## ----summary4,echo=FALSE-------------------------------------------
cat(cc[corRow:length(cc)],sep="\n")

## ----glmerDiagFake,eval=FALSE--------------------------------------
# ## basic residual plot
# plot(gm1)
# ## scale-location plot
# plot(gm1,sqrt(abs(resid(.)))~fitted(.),type=c("p","smooth"))
# ## boxplot of residuals grouped by a categorical predictor
# plot(gm1,period~resid(.))
# ## Q-Q plot
# qqmath(gm1)

## ----glmerDiag,fig.cap="Graphical diagnostics (using package built-in functions)",warning=FALSE,echo=FALSE----
p1 <- plot(gm1,type=c("p","smooth"),
    main="Default (Pearson resid vs. fitted)")
p2 <- plot(gm1,sqrt(abs(resid(.)))~fitted(.),type=c("p","smooth"),
     main="Scale-location")
p3 <- plot(gm1,period~resid(.), main = "Grouped residuals")
p4 <- qqmath(gm1,main="Quantile-quantile plot",type=c("p","r"))
grid.arrange(p1,p2,p3,p4,nrow=2)

## ----glmerRanefplot,fig.cap="Graphical display of random effects. \\emph{Left}: conditional modes $\\pm 1.96 \\times \\text{conditional standard deviation}$, ordered by magnitude. \\emph{Right}: quantile-quantile plot, with linear regression line overlaid.",echo=FALSE----
rr <- ranef(gm1, condVar=TRUE)
dd <- dotplot(rr)
qq <- qqmath(rr, type=c("p","r"))
grid.arrange(dd$herd, qq$herd, ncol=2)

## ----perf_chk_mod_cbpp,fig.cap="Model diagnostics using \\code{performance::check\\_model}. The plot showcasing the normality of random effects (lower right panel) is the transpose of the right panel in Figure 6.",fig.height=10,echo=FALSE----
check_model(gm1, check = c("pp_check", "binned_residuals",
                           "qq", "reqq"))

## ----dharma_mod_cbpp,fig.cap="Model assumption checks using the DHARMa package. \\code{n.s.} denotes ``not significant''.",echo=FALSE,warning=FALSE----
## FIXME: versions of DHARMa before 0.5.0 gave a significant nonlinearity test, so the caption
## included: 'Other exploration (such as \\code{plot(simulationOutput, form = cbpp2\\$period)}) does not reveal any obvious problems such as nonlinear responses, so we proceed with the analysis despite the significant adjusted quantile test.'
## Post 0.5.0, there are no test rejections; instead we get the warning 'In getSimulations.merMod(fittedModel, nsim = n, simulateREs = simulateREs,  :
##  Model was fit with prior weights. These will be ignored in the simulation. See ?getSimulations for details.'
## (is this a false positive?)
## 
simulationOutput <- simulateResiduals(fittedModel = gm1)
plot(simulationOutput)

## ----loadbatch,echo=FALSE------------------------------------------
cbpp_batch_vars <- load(system.file("vignette_data", "cbpp_batch.rda", package = "lme4"))
contr_batch_vars <- load(system.file("vignette_data", "Contraception_batch.rda", package = "lme4"))

## ----cbppConfidence,echo=FALSE-------------------------------------
des_ord <- c("sd_(Intercept)|herd", "sd_(Intercept)|obs",
             "avg_size", "treatmentUnknown", "treatmentComplete")
combCI <- subset(cbpp_combCI,var!="(Intercept)" & !grepl("^period",var))
combCI <- transform(combCI,  var = factor(var, levels = rev(des_ord)))
combCI <- transform(combCI,
            vartype = factor(ifelse(grepl("^sd", var), "random effects", "fixed effects"))
)                    

## ----cbppcompplot,fig.cap="CBPP example: comparison of point and confidence interval estimation for different methods. Wald CIs are missing for random-effects parameters when the fitted model is singular.",fig.scap="CBPP comparison",echo=FALSE,fig.width=7,fig.height=5,warning=FALSE----
mnames <- cbpp_df_name$mnames
pd <- position_dodge(width=0.6)
ggplot(combCI, aes(var, est, colour=type, shape=model)) +
  geom_pointrange(aes(ymin=lwr, ymax=upr),
                  position=pd) +
  labs(x="", y="Estimate (log-odds of seropositivity)") +
  geom_hline(yintercept=0, lty=2) +
  coord_flip() +
  scale_colour_brewer(palette="Dark2", guide = revguide) +
  scale_shape(guide=revguide, labels = mnames) +
  facet_wrap(~ vartype, ncol = 1, scale = "free")

## ----allfit_cbpp1,message=FALSE,warning=FALSE----------------------
gm_all <- allFit(gm1)

## ----allfit_cbpp2--------------------------------------------------
ss <- summary(gm_all)
ss$sdcor

## ----covar_cbpp----------------------------------------------------
gm.ar1 <- glmer(incidence/size ~ ar1(1 + herd | period),
                family = binomial,
                data = cbpp, weights = size)
print(VarCorr(gm.ar1))

## ----loadContraception, echo=FALSE---------------------------------
library(mlmRev)
data(Contraception)

## ----graphContraception,echo=FALSE,fig.cap="Contraception example: proportion of contraceptive use by centered age, number of living children (line type), and urban/rural residence (facet). Curves fitted using generalized additive models with cubic spline smoothing. Density plots along bottom and top margins show the age distributions of women not using contraception (bottom) and using contraception (top)."----
Contraception <- transform(Contraception, 
                        facet_urban = ifelse(urban=="Y","Urban","Rural"))
gg0 <- ggplot(Contraception) +
  geom_smooth(aes(age, ifelse(use == "Y",1,0), colour = livch, fill = livch,
                  linetype = livch),
                method = "gam", alpha = 0.1,
                method.args = list(gamma = 0.6),
                formula = y ~ s(x, bs = "cs")) +
    facet_wrap(~facet_urban) +
    scale_x_continuous("Centered age") + 
  scale_y_continuous("Proportion", limits = c(0, 1), expand = c(0,0)) +
  scale_linetype("Number\nof living\nchildren") +
  scale_colour_brewer("Number\nof living\nchildren", palette="Dark2") +
  scale_fill_brewer("Number\nof living\nchildren", palette="Dark2",
                    guide = guide_legend(override.aes = list(alpha = 0.1))) +
  ## wider key to see line types:
  ## https://stackoverflow.com/a/7624378/190277
  theme(legend.key.width = unit(3, "line"))

## add marginal density ribbons
gg0 + geom_density(
        data = subset(Contraception, use == "N"),
        aes(x = age, y = after_stat(density), fill = livch),
        alpha = 0.3, colour = NA) +
  geom_ribbon(
    data = subset(Contraception, use == "Y"),
    stat = "density",
    aes(
      x = age,  
      ymin = 1 - after_stat(density) * 0.5,
      ymax = 1,
      fill = livch
    ),
    alpha = 0.3, colour = NA
  )


## ----ch,echo=FALSE-------------------------------------------------
Contraception <- transform(Contraception,
   ch = factor(livch != 0, labels = c("N","Y")),
   age_s = age/(2*sd(age)))

## ----cm1,message=FALSE---------------------------------------------
cm1 <- glmer(use ~ age_s + I(age_s^2) + urban + livch + (1|district), 
                           Contraception, binomial)
## switch from livch (ordinal) to ch (binary)
cm2 <- update(cm1, . ~ . - livch + ch)
## add age by children interaction
cm3 <- update(cm2, . ~ . + age_s:ch)
## allow urban effect to vary across districts (correlated)
cm4 <- update(cm3, . ~ . - (1|district) + (1+urban|district))
## compound symmetric/nested formulation
cm5 <- update(cm3, . ~ . - (1|district) + (1 | district/urban))
## as above but drop district effect
cm6 <- update(cm3, . ~ . - (1|district) + (1 | district:urban))

## ----anovaContraception, echo = FALSE------------------------------
## copy of bbmle::AICtab
AICtab <- function(..., mnames) {
  aic_df <- do.call(AIC, list(...))
  LLvec <- sapply(list(...), function(x) c(logLik(x)))
  rownames(aic_df) <- mnames
  aic_df <- transform(aic_df,
                        Δnegloglik = -LLvec - min(-LLvec),
                        ΔAIC = AIC - min(AIC))
  aic_df <- subset(aic_df, select = -AIC)
  aic_df <- aic_df[order(aic_df$ΔAIC),]
  aic_df
}

mod_list <- c(cm1, cm2, cm3, cm4, cm5, cm6)
op <- options(digits = 3)
mnamevec <- c("int_child    + age  + (1 | district)",
              "binary_child + age  + (1 | district)",
                "binary_child × age + (1 | district)",
                "binary_child × age + (1 + urban | district)",
                "binary_child × age + (1 | district/urban)",
                "binary_child × age + (1 | district:urban)")
aictab <- do.call(AICtab, c(mod_list, list(mnames = mnamevec)))

## ----aictab, echo  = FALSE, results = "asis"-----------------------
knitr::kable(aictab, digits = 3)

## ----contr-predplot, warning = FALSE, fig.cap="Predictions for contraception fit (\\code{(1|district/urban)} model). Heavy lines and ribbons show population-level predictions; light lines show district-level predictions.", echo = FALSE----
agevec <- seq(-15, 20)
pframe <- with(Contraception, expand.grid(age = agevec, urban = levels(urban), ch = levels(ch)))
pframe <- transform(pframe, age_s = age/(2*sd(Contraception$age)),
                    facet_urban = ifelse(urban=="Y","Urban","Rural"))
pred  <- predict(cm5, newdata = pframe, se.fit = TRUE, re.form = NA)
pframe$use <- plogis(pred$fit)
pframe$lwr <- plogis(pred$fit-1.96*pred$se.fit)
pframe$upr <- plogis(pred$fit+1.96*pred$se.fit)
pframe2 <- with(Contraception, expand.grid(age = agevec, urban = levels(urban), ch = levels(ch),
                 district = levels(district)))
pframe2 <- transform(pframe2, age_s = age/(2*sd(Contraception$age)),
            facet_urban = ifelse(urban=="Y","Urban","Rural"),
            dist_urb = paste(urban, district, sep = ":"))
c_urb <- unique(with(Contraception, paste(urban, district, sep = ":")))
pframe2 <- subset(pframe2, dist_urb %in% c_urb)
pred2  <- predict(cm5, newdata = pframe2, re.form = NULL)
pframe2$use <- plogis(pred2)
pframe2$ch <- factor(pframe2$ch,
                     levels = c("N", "Y"),
                     labels = c("no", "yes"))
pframe$ch <- factor(pframe$ch,
                     levels = c("N", "Y"),
                     labels = c("no", "yes"))

ggplot(pframe, aes(age)) + 
  geom_line(aes(y = use, colour = ch)) +
  geom_ribbon(aes(ymin = lwr, ymax = upr, fill = ch), color = NA, alpha = 0.3) +
  facet_wrap(~facet_urban) +
  scale_x_continuous("Centered age") + 
  scale_y_continuous("Proportion", limits = c(0, 1), expand = c(0,0)) +
  scale_colour_brewer("Living\nchildren", palette="Dark2",
                      guide = guide_legend(reverse = TRUE)) +
  scale_fill_brewer("Living\nchildren", palette="Dark2",
                    guide = guide_legend(reverse = TRUE)) +
  geom_line(data = pframe2, aes(group = interaction(dist_urb, ch), colour = ch, y = use), alpha = 0.2)


## ----ContraceptionBatch,echo=FALSE---------------------------------
contr_combCI <- subset(contr_combCI, var != "(Intercept)")
combCI <- transform(contr_combCI,
    vartype = factor(ifelse(grepl("^(sd|cor)", var), "random effects", "fixed effects")),
    model = factor(model, levels = rev(levels(model)))  ## flip AND flip order of guide below
    )

## ----contracompplot,fig.cap="Contraception example: comparison of point and confidence interval estimation for different methods. (Note that the Wald CI for variation in the intercept across districts, in the \\code{(1|district/urban)} model, includes negative values.)",fig.scap="Contraception comparison",echo=FALSE,fig.width=10,fig.height=10,warning=FALSE----
ggplot(combCI, aes(var, est, colour=type, shape=model)) +
  geom_pointrange(aes(ymin=lwr, ymax=upr),
                  position=position_dodge(width=0.6)) +
  labs(x="", y="Estimate (log-odds of seropositivity)") +
  geom_hline(yintercept=0, lty=2) +
  coord_flip() +
  scale_colour_brewer(palette="Dark2", guide = revguide) +
  scale_shape(guide=revguide) +
  facet_wrap(~vartype, ncol = 1, scale = "free", space = "free_y") +
  theme(legend.position = "inside",
  legend.justification = c("left", "top"),
  legend.box = "horizontal",
      legend.background = element_rect(fill = "white", colour = "grey50"),
      #legend.margin = margin(t = 5, r = 5, b = 5, l = 5, unit = "pt"),
      legend.box.margin = margin(t = 10, l = 10, unit = "pt"))

## ----cm_varcorr, echo = FALSE--------------------------------------
cc <- glmerControl(check.conv.grad = .makeCC("warning", tol = 1e-2), check.conv.hess = "ignore")
base_form <- use ~ age*ch + I(age^2) + urban
cm.diag <- glmer(update(base_form, . ~ . + diag(1 + urban|district)), Contraception, binomial, control = cc )
cm.homdiag <- glmer(update(base_form, . ~ . + diag(1 + urban|district, hom = TRUE)), Contraception, binomial, control = cc)
cm.homcs <- glmer(update(base_form, . ~ . + cs(1 + urban|district, hom = TRUE)), Contraception, binomial, control = cc)

## ----cm_cs, message = FALSE, warning = FALSE-----------------------
cm.cs <- glmer(use ~ age*ch + I(age^2) + urban + cs(1 + urban | district),
               Contraception, binomial, control = cc)

## ----cm_varcorr_view-----------------------------------------------
print(VarCorr(cm.cs))

## ----pkgs, echo=FALSE----------------------------------------------
pkglist <- c("lme4", "performance", "DHARMa", "see")
pp <- sapply(pkglist, function(x)
  sprintf("%s: %s", x, format(packageVersion(x))))
ppp <- paste(pp, collapse  = "; ")

if (lme4:::testLevel() > 1 || .Platform$OS.type!="windows") {
### lmer() convergence testing / monitoring / ...
    ##  ------------------
### The output of tests here are *not* 'diff'ed  (<==> no *.Rout.save file)
    library(lme4)

    ## convergence on boundary warnings
    load(system.file("external/test3comp.rda", package = "Matrix"))
    b3 <- lmer(Y3 ~ (1|Sample) + (1|Operator/Run), test3comp, verb = TRUE)

    if (isTRUE(try(data(Early, package = 'mlmRev')) == 'Early')) {
        Early$tos <- Early$age - 0.5        # time on study
        b1 <- lmer(cog ~ tos + trt:tos + (tos|id), Early, verb = TRUE)
    }

    cat('Time elapsed: ', proc.time(),'\n') # for ``statistical reasons''
} ## skip on windows (for speed)

## These files are not currently exported; we are still trying to figure
## out the most appropriate user interface/naming convention/etc
## These functions will become more important, and need to be
## modified/augmented, if we start allowing for varying V-C structures
## (e.g. diagonal matrices, compound symmetry ...)

## In principle we might want to extract or input information:

## 1. as variance-covariance matrices
## 2. as Cholesky factors
## 3. as 'sdcorr' matrices (std dev on diagonal, correlations off diagonal)

## and we might want the structure to be:

## 1. a concatenated vector representing the lower triangles
##    (with an attribute carrying the information about group sizes)
## 2. a list of lower-triangle vectors
## 3. a list of matrices
## 4. a block-diagonal matrix

## If we are trying to convert to and from theta vectors, we also
## have to consider whether we are returning scaled Cholesky factors/
## var-cov matrices or unscaled ones.  For the code below I have
## chosen to allow the residual variance etc. to be appended as
## the last element of a variance-covariance vector.  (This last
## part is a little less generic than the rest of it.)

##' List of matrices to concatenated vector
##'
##' Convert list of matrices to concatenated vector of lower triangles
##' with an attribute that gives the dimension of each matrix in the
##' original list.  This attribute may be used to reconstruct the
##' matrices.
##'
##' @param L list of symmetric, upper-triangular, or lower-triangular
##' square matrices
##' @return A concatenation of the elements in one triangle of each
##' matrix. An attribute \code{"clen"} gives the dimension of each
##' matrix.
mlist2vec <- function(L) {
    if(is.atomic(L)) L <- list(L)
    n <- vapply(L, nrow, 1L)
    ## allow for EITHER upper- or lower-triangular input;
    ## in either case, read off in "lower-triangular" order
    ## (column-wise)
    ff <- function(x) {
        if (all(na.omit(x[iu <- upper.tri(x)] == 0))) t(x[!iu]) else t(x)[!iu]
    }
    structure(unlist(lapply(L,ff)), clen = n)
}

## Compute dimensions of a square matrix from the size
## of the lower triangle (length as a vector)
get_clen <- function(v,n=NULL) {
    if (is.null(n)) {
        if (is.null(n <- attr(v,"clen"))) {
            ## single component
          n <- (sqrt(8*length(v)+1)-1)/2
        }
    }
    n
}

##' Concatenated vector to list of matrices
##'
##' Convert concatenated vector to list of matrices (lower triangle or
##' symmetric). These matrices could represent Cholesky factors,
##' covariance matrices, or correlation matrices (with standard
##' deviations on the diagonal).
##'
##' @param v concatenated vector
##' @param n FIXME: this has something to do with the dimension of
##' associated matrices.
##' @param symm Return symmetric matrix if \code{TRUE} or
##' lower-triangular if \code{FALSE}
##' @return List of matrices
vec2mlist <- function(v,n=NULL,symm=TRUE) {
    n <- get_clen(v,n)
    s <- split(v,rep.int(seq_along(n),n*(n+1)/2))
    m <- mapply(function(x,n0) {
        m0 <- diag(nrow=n0)
        m0[lower.tri(m0,diag=TRUE)] <- x
        if (symm) m0[upper.tri(m0)] <- t(m0)[upper.tri(m0)]
        m0
    },s,n,SIMPLIFY=FALSE)
    m
}

## Convert concatenated vector to list of ST matrices
vec2STlist <- function(v, n = NULL){
  ch <- vec2mlist(v, n, FALSE) # cholesky
  sdiag <- function(x) { ## 'safe' diag()
      if (length(x)==1) matrix(x,1,1) else diag(x)
  }
  lapply(ch, function(L) {
    ST <- L%*%sdiag(1/sdiag(L))
    diag(ST) <- diag(L)
    ST
  })
}

##' Standard deviation-correlation matrix to covariance matrix
##'
##' convert 'sdcor' format -- diagonal = std dev, off-diag=cor to and
##' from variance-covariance matrix
##'
##' @param m Standard deviation-correlation matrix
##' @return Covariance matrix
sdcor2cov  <- function(m) {
    sd <- diag(m)
    diag(m) <- 1
    m * outer(sd,sd)
}

##' Covariance matrix to standard deviation-correlation matrix
##'
##' convert cov to sdcor
##'
##' @param V Covariance matrix
##' @return Standard deviation-correlation matrix
cov2sdcor  <- function(V) {
    ## "own version" of cov2cor():  1. no warning for NA;   2. diagonal = sd(.)
    p <- (d <- dim(V))[1L]
    if (!is.numeric(V) || length(d) != 2L || p != d[2L])
        stop("'V' is not a square numeric matrix")
    sd <- sqrt(diag(V))
    Is <- 1/sd
    r <- V
    r[] <- Is * V * rep(Is, each = p)
    diag(r) <- sd
    if (any(is.na(r))) {
        warning("NA values in sdcor matrix converted to 0")
        r[is.na(r)] <- 0
    }
    r
}

## dmult <- function(m,s) {
##     diag(m) <- diag(m)*s
##     m
## }

## attempt to compute Cholesky, allow for positive semi-definite cases
##  (hackish)
safe_chol <- function(m) {
    if (any(is.na(m)) || all(m==0)) return(m)
    if (nrow(m)==1) return(sqrt(m))
    if (.isDiagonal.sq.matrix(m)) return(diag(sqrt(diag(m))))

    ## attempt regular Chol. decomp
    if (!is.null(cc <- tryCatch(chol(m), error=function(e) NULL)))
        return(cc)
    ## ... pivot if necessary ...
    cc <- suppressWarnings(chol(m,pivot=TRUE))
    oo <- order(attr(cc,"pivot"))
    cc[,oo]
    ## FIXME: pivot is here to deal with semidefinite cases,
    ## but results might be returned in a strange format: TEST
}

##' Variance-covariance to relative covariance factor
##'
##' from var-cov to scaled Cholesky:
##'
##' @param v Vector of elements from the lower triangle of a
##' variance-covariance matrix.
##' @param n FIXME: see "@param n" above
##' @param s Scale parameter
##' @return Vector of elements from the lower triangle of a relative
##' covariance factor.
Vv_to_Cv <- function(v,n=NULL,s=1) {
    if (!missing(s)) {
        v <- v[-length(v)]
    }
    r <- mlist2vec(lapply(vec2mlist(v,n,symm=TRUE),
                          function(m) t(safe_chol(m/s^2))))
    attr(r,"clen") <- get_clen(v,n)
    r
}

##' Standard-deviation-correlation to relative covariance factor
##'
##' from sd-cor to scaled Cholesky
##'
##' @param v Vector of elements from the lower triangle of a
##' standard-deviation-correlation matrix.
##' @param n FIXME: see "@param n" above
##' @param s Scale parameter
##' @return Vector of elements from the lower triangle of a relative
##' covariance factor.
Sv_to_Cv <- function(v,n=NULL,s=1) {
    if (!missing(s)) {
        v <- v[-length(v)]
    }
    r <-  mlist2vec(lapply(vec2mlist(v,n,symm=TRUE),
                           function(m) t(safe_chol(sdcor2cov(m)/s^2))))
    attr(r,"clen") <- get_clen(v,n)
    r
}

##' Relative covariance factor to variance-covariance
##'
##' from unscaled Cholesky vector to (possibly scaled)
##' variance-covariance vector
##'
##' @param v Vector of elements from the lower triangle of a
##' relative covariance factor.
##' @param n FIXME: see "@param n" above
##' @param s Scale parameter
##' @return Vector of elements from the lower triangle of a
##' variance-covariance matrix.
Cv_to_Vv <- function(v,n=NULL,s=1) {
    r <- mlist2vec(lapply(vec2mlist(v,n,symm=FALSE),
                          function(m) tcrossprod(m)*s^2))
    if (!missing(s)) r <- c(r,s^2)
    attr(r,"clen") <- get_clen(v,n)
    r
}

##' Relative covariance factor to standard-deviation-correlation
##'
##' from unscaled Chol to sd-cor vector
##'
##' @param v Vector of elements from the lower triangle of a
##' relative covariance factor.
##' @param n FIXME: see "@param n" above
##' @param s Scale parameter
##' @return Vector of elements from the lower triangle of a
##' standard-deviation-correlation matrix.
Cv_to_Sv <- function(v,n=NULL,s=1) {
    r <- mlist2vec(lapply(vec2mlist(v,n,symm=FALSE),
                          function(m) cov2sdcor(tcrossprod(m)*s^2)))
    if (!missing(s)) r <- c(r,s)
    attr(r,"clen") <- get_clen(v,n)
    r
}

## tests --> ../inst/tests/test-utils.R

## use old (<=3.5.2) sample() algorithm if necessary
if ("sample.kind" %in% names(formals(RNGkind))) {
    suppressWarnings(RNGkind("Mersenne-Twister", "Inversion", "Rounding"))
}

#context("Utilities (including *non*-exported ones)")

test_that("namedList", {
    nList <- lme4:::namedList
    a <- b <- c <- 1
    expect_identical(nList(a,b,c),  list(a = 1, b = 1, c = 1))
    expect_identical(nList(a,b,d=c),list(a = 1, b = 1, d = 1))
    expect_identical(nList(a, d=pi, c), list(a = 1, d = pi, c = 1))
})

test_that("Var-Cov factor conversions", { ## from ../../R/vcconv.R
    mlist2vec <- lme4:::mlist2vec
    Cv_to_Vv <- lme4:::Cv_to_Vv
    Cv_to_Sv <- lme4:::Cv_to_Sv
    Sv_to_Cv <- lme4:::Sv_to_Cv
    Vv_to_Cv <- lme4:::Vv_to_Cv
    ##
    set.seed(1); cvec1 <- sample(10, 6)
    v1 <- Cv_to_Vv(cvec1)
    expect_equal(unname(v1), structure(c(9, 12, 15, 65, 34, 93), clen = 3))
    expect_equal(2, as.vector(Vv_to_Cv(Cv_to_Vv(2))))
    expect_equivalent(c(v1, 1),  Cv_to_Vv(cvec1, s=3) / 3^2)
    expect_equal(as.vector(ss1 <- Sv_to_Cv(Cv_to_Sv(cvec1))), cvec1)
    expect_equal(as.vector(vv1 <- Vv_to_Cv(Cv_to_Vv(cvec1))), cvec1)
    ## for length-1 matrices, Cv_to_Sv should be equivalent
    ##   to multiplying Cv by sigma and appending sigma ....
    clist2 <- list(matrix(1),matrix(2),matrix(3))
    cvec2 <- mlist2vec(clist2)
    expect_equal(cvec2, structure(1:3, clen = rep(1,3)), tolerance=0)
    expect_true(all((cvec3 <- Cv_to_Sv(cvec2, s=2)) == c(cvec2*2,2)))
    n3 <- length(cvec3)
    expect_equivalent(Sv_to_Cv(cvec3, n=rep(1,3), s=2), cvec3[-n3]/cvec3[n3])
})

## moved to lme4

test_that("getData", {
    ## test what happens when wrong version of 'data' is found in environment of formula ...
    f <- round(Reaction) ~ 1 + (1|Subject)
    g <- function() {
        data <- sleepstudy
        m1 <- glmer(f, data = data, family = poisson)
    }
    m1 <- g()
    expect_error(getData(m1), "is not a data frame or matrix.*masked by a built-in", perl = TRUE)
    p1 <- suppressMessages(predict(m1, newparams = list(beta = 5, theta = 1), type = "response",
            re.form = ~ 1|Subject))
    expect_equal(unname(head(p1, 2)), c(444.407382298401, 444.407382298401))
})

## DON'T load lme4; test is to see if glmer.nb works when
## lme4 is not loaded

set.seed(101)
dd <- data.frame(x=runif(200), f= rep(1:20, each=10))
b <- rnorm(20)
dd <- transform(dd, y = rnbinom(200, mu  = exp(1 + 2*x + b[f]), size = 2))
g <- lme4::glmer.nb(y~x + (1|f), data = dd)
stopifnot(inherits(g, "glmerMod"))

##' Create a univariate Gauss-Hermite quadrature rule
##'
##' This version of Gauss-Hermite quadrature provides the node
##' positions and weights for a scalar integral of a function
##' multiplied by the standard normal density.
GHrule <- function (ord, asMatrix=TRUE) {
    stopifnot(length(ord) == 1,
              (ord <- as.integer(ord)) >= 0L,
              ord < 101L)
    if (ord == 0L) {
        if (asMatrix) return(matrix(0, nrow=0L, ncol=3L))
        stop ("combination of ord==0 and asMatrix==TRUE not implemented")
    }
    ## fgq_rules comes from sysdata.rda:
    ## result of
    ## library("fastGHQuad")
    ## rescale <- function(x, scale.weights=TRUE, scale.roots=TRUE) {
    ##     within(x, {
    ##         if (scale.weights) w <- w/sum(w)
    ##         if (scale.roots) x <- x*sqrt(2)
    ##     })
    ## }
    ## fgqRules <- lapply(1:100, function(n) setNames(rescale(gaussHermiteData(n)), c("z", "w")))
    ## ## However, this shows small differences !!
    ## all.equal(lme4:::fgq_rules, fgqRules, tol=0)

    fr <- as.data.frame(fgq_rules[[ord]])

    rownames(fr) <- NULL
    ## Repositioning them similar to how it's done in MixedModels.jl,
    ## by averaging the weights in forward and reverse order and the locations 
    ## in forward order and negative locations in reverse order.
    ## See: https://github.com/lme4/lme4/issues/968#issuecomment-4467149014
    fr$z <- (fr$z - rev(fr$z)) / 2
    fr$w <- (fr$w + rev(fr$w)) / 2
    fr$ldnorm <- dnorm(fr$z, log=TRUE)
    if (asMatrix) as.matrix(fr) else fr
}

if (.Platform$OS.type != "windows") {
    library(lme4)
    cat("lme4 testing level: ", testLevel <- lme4:::testLevel(), "\n")


    getNBdisp <- function(x) getME(x,"glmer.nb.theta")
    ## for now, use hidden functions [MM: this is a sign, we should *export* them]
    refitNB   <- lme4:::refitNB

    simfun <- function(sd.u=1, NBtheta=0.5,
                       nblock = 25,
                       fform = ~x,
                       beta = c(1,2),
                       nrep = 40, seed) {
        levelset <- c(LETTERS,letters)
        stopifnot(2 <= nblock, nblock <= length(levelset))
        if (!missing(seed)) set.seed(seed)
        ntot <- nblock*nrep
        d1 <- data.frame(x = runif(ntot),
                         f = factor(rep(levelset[1:nblock], each=nrep)))
        u_f <- rnorm(nblock, sd=sd.u)
        X <- model.matrix(fform, data=d1)
        transform(d1, z = rnbinom(ntot, mu = exp(X %*% beta + u_f[f]), size = NBtheta))
    }

    ##' simplified logLik() so we can compare with "glmmADMB" (and other) results
    logLik.m <- function(x) {
        L <- logLik(x)
        attributes(L) <- attributes(L)[c("class","df","nobs")]
        L
    }

    if (testLevel > 1) withAutoprint({
        set.seed(102)
        d.1 <- simfun()
        t1 <- system.time(g1 <- glmer.nb(z ~ x + (1|f), data=d.1, verbose=TRUE))
        g1
        d1 <- getNBdisp(g1)
        (g1B <- refitNB(g1, theta = d1))
        (ddev <- deviance(g1) - deviance(g1B))
        (reld <- (fixef(g1) - fixef(g1B)) / fixef(g1))
        stopifnot(abs(ddev) < 1e-6,   # was 6.18e-7, 1.045e-6, -6.367e-5, now 0
                  abs(reld) < 1e-6)# 0, then 4.63e-6,  now 0
        ## 2 Aug 2015: ddev==reld==0 on 32-bit Ubuntu 12.04

        if(FALSE) {
            ## comment out to avoid R CMD check warning :
            ## library(glmmADMB)
            t2 <- system.time(g2 <- glmmadmb(z~x+(1|f),
                                             data = d.1, family="nbinom"))
            ## matrix not pos definite in sparse choleski
            t2 # 17.1 sec elapsed
            glmmADMB_vals <- list(fixef= fixef(g2),
                                  LL   = logLik(g2),
                                  theta= g2$alpha)
        } else {
            glmmADMB_vals <-
                list(fixef = c("(Intercept)" = 0.928710, x = 2.05072),
                     LL = structure(-2944.62, class = "logLik", df = 4, nobs = 1000L),
                     theta = 0.4487)
        }


        stopifnot(exprs = {
            all.equal(   d1,         glmmADMB_vals$ theta, tolerance=0.003) #   0.0015907
            all.equal(fixef(g1B),    glmmADMB_vals$ fixef, tolerance=0.02)# was 0.009387 !
            ## Ubuntu 12.04/32-bit: 0.0094
            all.equal(logLik.m(g1B), glmmADMB_vals$ LL,    tolerance=1e-4)# 1.681e-5; Ubuntu 12.04/32-b: 1.61e-5
        })

    })## end if( testLevel > 1 )

    if(FALSE) { ## simulation study --------------------

        ## library(glmmADMB) ## avoid R CMD check warning
        simsumfun <- function(...) {
            d <- simfun(...)
            t1 <- system.time(g1 <- glmer.nb(z~x+(1|f),data=d))
            t2 <- system.time(g2 <- glmmadmb(z~x+(1|f),
                                             data=d,family="nbinom"))
            c(t.glmer=unname(t1["elapsed"]),nevals.glmer=g1$nevals,
              theta.glmer=exp(g1$minimum),
              t.glmmadmb=unname(t2["elapsed"]),theta.glmmadmb=g2$alpha)
        }

        ## library(plyr)
        ## sim50 <- raply(50,simsumfun(),.progress="text")
        save("sim50",file="nbinomsim1.RData")
        ## library(reshape)
        ## m1 <- melt(data.frame(run=seq(nrow(sim50)),sim50),id.var="run")
        ## m1 <- data.frame(m1,colsplit(m1$variable,"\\.",c("v","method")))
        ## m2 <- cast(subset(m1,v=="theta",select=c(run,value,method)),
        ##           run~method)

        library(ggplot2)
        ggplot(subset(m1,v=="theta"),aes(x=method,y=value))+
            geom_boxplot()+geom_point()+geom_hline(yintercept=0.5,colour="red")

        ggplot(subset(m1,v=="theta"),aes(x=method,y=value))+
            stat_summary(fun.data=mean_cl_normal)+
            geom_hline(yintercept=0.5,colour="red")

        ggplot(m2,aes(x=glmer-glmmadmb))+geom_histogram()
        ## glmer is slightly more biased (but maybe the MLE itself is biased???)

    }## end{simulation study}-------------------------

### epilepsy example:
    data(epil, package="MASS")
    epil2 <- transform(epil,
                       Visit  = (period-2.5)/5,
                       Base   = log(base/4),
                       Age    = log(age),
                       subject= factor(subject))

    if(FALSE) {
        ## comment out to avoid R CMD check warning :
        ## library(glmmADMB)
        t3 <- system.time(g3  <- glmmadmb(y~Base*trt+Age+Visit+(Visit|subject),
                                          data=epil2, family="nbinom")) # t3 : 8.67 sec
        glmmADMB_epil_vals <- list(fixef= fixef(g3),
                                   LL   = logLik(g3),
                                   theta= g3$alpha)
    } else {
        glmmADMB_epil_vals <-
            list(fixef =
                     c("(Intercept)"= -1.33, "Base"=0.8839167, "trtprogabide"= -0.9299658,
                       "Age"= 0.4751434, "Visit"=-0.2701603, "Base:trtprogabide"=0.3372421),
                 LL = structure(-624.551, class = "logLik", df = 9, nobs = 236L),
                 theta = 7.4702)
    }

    if (testLevel > 2) withAutoprint({
        ## "too slow" for regular testing -- 49 (MM@lynne: 33, then 26, then 14) seconds:
        (t4 <- system.time(g4 <- glmer.nb(y ~ Base*trt + Age + Visit + (Visit|subject),
                                          data = epil2, verbose=TRUE)))
        ## 1.1-7 : Warning in checkConv().. failed .. with max|grad| = 0.0089 (tol = 0.001, comp. 4)
        ## 1.1-21: 2 Warnings:  max|grad| = 0.00859, then 0.1176 (0.002, comp. 1)

        stopifnot(exprs = {
            all.equal(getNBdisp(g4),   glmmADMB_epil_vals$ theta, tolerance= 0.03) # 0.0019777
            all.equal(fixef    (g4),   glmmADMB_epil_vals$ fixef, tolerance= 0.04) # 0.003731 (0.00374 on U 12.04)
            ## FIXME: even df differ (10 vs 9) !
            ##      all.equal(logLik.m(g4), - glmmADMB_epil_vals$ LL,	tolerance= 0.0) ## was 0.0002
            all.equal(logLik.m(g4), # for now {this is not *the* truth, just our current approximation of it}:
                      structure(-624.48418, class = "logLik", df = 10, nobs = 236L),
                      ## tolerance loosened 24-03-2025, failed at 1.7e-4
                      tolerance = 5e-4)
        })
    })


    cat('Time elapsed: ', proc.time(),'\n') # for ``statistical reasons''
} ## skip on windows (for speed)

##' PCA of random-effects variance-covariance estimates
##'
##' Perform a Principal Components Analysis (PCA) of the random-effects
##' variance-covariance estimates from a fitted mixed-effects model
##' @title PCA of random-effects
##' @param x a merMod object
##' @return a \code{prcomplist} object
##' @author Douglas Bates
##' @export
rePCA <- function(x) UseMethod('rePCA')

#' @export
rePCA.merMod <- function(x) {
    chfs <- getME(x,"Tlist")  # list of lower Cholesky factors
    nms <- names(chfs)
    unms <- unique(nms)
    names(unms) <- unms
    svals <- function(m) {
        vv <- svd(m,nv=0L)
        names(vv) <- c("sdev","rotation")
        vv$center <- FALSE
        vv$scale <- FALSE
        class(vv) <- "prcomp"
        vv
    }
    structure(lapply(unms,function(m) svals(Matrix::bdiag(chfs[which(nms == m)]))),
              class="prcomplist")
}
#' @export
summary.prcomplist <- function(object,...) {
    lapply(object,summary)
}

if (lme4:::testLevel() > 1 || .Platform$OS.type != "windows") withAutoprint({

    ## testing whether lme4 and nlme play nicely.  Only known issue
    ## is lmList-masking ...
    library("lme4")
    library("nlme")
    fm1_lmer <- lmer(Reaction ~ Days   +       (Days|Subject), sleepstudy)
    fm1_lme  <- lme (Reaction ~ Days, random = ~Days|Subject,  sleepstudy)
    ## variance-covariance matrices: annoyingly different structures
    vc_lmer <- VarCorr(fm1_lmer)
    vc_lme  <- VarCorr(fm1_lme, rdig = 8)
    suppressWarnings(storage.mode(vc_lme) <- "numeric")# 2 NAs
    vc_lmerx <- c(diag(vc_lmer[[1]]), attr(vc_lmer[[1]],"correlation")[1,2])
    vc_lmex  <- c( vc_lme[1:2,1],     vc_lme[2,3])
    stopifnot(
        all.equal(vc_lmex, vc_lmerx, tolerance= 4e-4) # had 3e-5, now see 0.000296
      , ## fixed effects (much easier) :
        all.equal(fixef(fm1_lmer), fixef(fm1_lme)) # 3.6e-15
       ,
        all.equal(unname(unlist(unclass(ranef(fm1_lmer)))),
                  unname(unlist(unclass(ranef(fm1_lme)))),
                  tolerance = 2e-4) # had 2e-5, now see 8.41e-5
    )

    fm1L_lme  <- nlme::lmList(distance ~ age | Subject, Orthodont)
    fm1L_lmer <- lme4::lmList(distance ~ age | Subject, Orthodont)
    stopifnot(all.equal(fixef(fm1L_lmer),
                        fixef(fm1L_lme)))
    sm1L_e  <- summary(fm1L_lme)
    sm1L_er <- summary(fm1L_lmer)
    stopifnot(
        all.equal(coef(sm1L_e),
                  coef(sm1L_er), tol=1e-12)# even tol=0 works on some Lnx 64b
    )

    ## FIXME: test opposite order
})

library(lme4)
data("Contraception", package = "mlmRev")
library(ggplot2); theme_set(theme_bw())
library(ggdist)

Contraception <- transform(Contraception, 
                        facet_urban = ifelse(urban=="Y","Urban","Rural"))
gg0 <- ggplot(Contraception) +
    geom_smooth(aes(age, ifelse(use == "Y",1,0), colour = livch, fill = livch),
                method = "gam", alpha = 0.1,
                method.args = list(gamma = 0.6),
                formula = y ~ s(x, bs = "cs")) +
    facet_wrap(~facet_urban) +
    scale_x_continuous("Centered age") + 
    scale_y_continuous("Proportion", limits = c(0, 1), expand = c(0,0)) +
    scale_colour_brewer("Number\nof living\nchildren", palette="Dark2") +
    scale_fill_brewer("Number\nof living\nchildren", palette="Dark2")

gg0 + geom_density(
        data = subset(Contraception, use == "N"),
        aes(x = age, y = after_stat(density), fill = livch),
        alpha = 0.3, colour = NA) +
  geom_ribbon(
    data = subset(Contraception, use == "Y"),
    stat = "density",
    aes(
      x = age,  
      ymin = 1 - after_stat(density) * 0.5,
      ymax = 1,
      fill = livch
    ),
    alpha = 0.3, colour = NA
  )
ggsave("cplot.pdf")

## --> ../man/NelderMead.Rd
Nelder_Mead <- function(fn, par, lower=rep.int(-Inf, n),
                        upper=rep.int(Inf, n), control=list()) {
    n <- length(par)
    if (is.null(xst <- control[["xst"]])) xst <- rep.int(0.02,n)
    if (is.null(xt <- control[["xt"]])) xt <- xst*5e-4

    control[["xst"]] <- control[["xt"]] <- NULL

    ## mapping between simpler 'verbose' setting (0=no printing, 1=20, 2=10, 3=1)
    ##  and internal 'iprint' control (frequency of printing)
    if (is.null(verbose <- control[["verbose"]])) verbose <- 0
    control[["verbose"]] <- NULL
    if (is.null(control[["iprint"]])) {
      control[["iprint"]] <- switch(as.character(min(as.numeric(verbose),3L)),
                                    "0"=0, "1"=20,"2"=10,"3"=1)
    }
    stopifnot(is.function(fn),
              length(formals(fn)) == 1L,
              (n <- length(par <- as.numeric(par))) == length(lower <- as.numeric(lower)),
              length(upper <- as.numeric(upper)) == n,
              length(xst <- as.numeric(xst)) == n,
              all(xst != 0),
              length(xt <- as.numeric(xt)) == n)
    ## "NelderMead" reference class and constructor: --> ./AllClass.R :
    nM <- NelderMead$new(lower=lower, upper=upper, x0=par, xst=xst, xt=xt)
    cc <- do.call(function(iprint = 0L, maxfun = 10000L, FtolAbs = 1e-5,
                           FtolRel = 1e-15, XtolRel = 1e-7,
                           MinfMax= -.Machine$double.xmax, warnOnly=FALSE, ...) {
        if(...length() > 0) warning("unused control arguments ignored")
        list(iprint=iprint, maxfun=maxfun, FtolAbs=FtolAbs, FtolRel=FtolRel,
             XtolRel=XtolRel, MinfMax=MinfMax, warnOnly=warnOnly)
    }, control)
    nM$setFtolAbs(cc$FtolAbs)
    nM$setFtolRel(cc$FtolRel)
    nM$setIprint (cc$iprint)
    nM$setMaxeval(cc$maxfun)
    nM$setMinfMax(cc$MinfMax)
    it <- 0
    repeat {
        it <- it + 1
        nMres <- nM$newf(fn(nM$xeval()))
        if (nMres != 0L) break
    }

    cmsg <- "reached max evaluations"
    if (nMres == -4) {
        ## map max evals from error to warning
        cmsg <- warning(sprintf("failure to converge in %d evaluations",cc$maxfun))
        nMres <- 4
    }
                                                        ## nMres:
    msgvec <- c("nm_forced",                            ## -3
                "cannot generate a feasible simplex",   ## -2
                "initial x is not feasible",            ## -1
                "active",                               ## 0 (active)
                "objective function went below allowed minimum",           ## 1 (minf_max)
                "objective function values converged to within tolerance", ## 2 (fcvg)
                "parameter values converged to within tolerance",          ## 3 (xcvg)
                cmsg)

    if (nMres < 0) { ## i.e., in {-3, -2, -1}
        (if(cc$warnOnly) warning else stop)( msgvec[nMres+4] )
    }

    list(fval = nM$value(), par = nM$xpos(),
         convergence = pmin(0, nMres), # positive nMres is also 'convergence'
         NM.result = nMres, `message` = msgvec[nMres+4],
         control = c(cc, xst=xst, xt=xt), feval = it)
}

## drop1 may not work right with contrasts: make up an example something like this ...
## options(contrasts=c("contr.sum","contr.poly"))
## drop1(fecpoiss_lm3,test="Chisq",scope=.~.)


if (.Platform$OS.type != "windows") withAutoprint({
    library(lme4)
    oldopts <- options(contrasts=c("contr.sum","contr.poly"))
    fm1 <- lmer(Reaction~Days+(Days|Subject),data=sleepstudy)
    drop1(fm1,test="Chisq")
    ## debug(lme4:::drop1.merMod)
    drop1(fm1,test="Chisq",scope=.~.)

    fm0 <- lm(Reaction~Days+Subject,data=sleepstudy)
    drop1(fm0,test="Chisq",scope=.~.)
    options(oldopts)  ## restore original contrasts

    ff <- function() {
        lmer(Reaction~Days+(Days|Subject),data=sleepstudy)
    }
    drop1(ff())  ## OK because sleepstudy is accessible!

}) ## skip on windows (for speed)

if (.Platform$OS.type != "windows") {
    library(lme4)
#### tests of getME()

###  are names correct? --------------
    if(getRversion() < "2.15")
        paste0 <- function(...) paste(..., sep = '')
    hasInms <- function(x) grepl("(Intercept", names(x), fixed=TRUE)
    matchNms <- function(fm, PAR) {
        stopifnot(is.character(vnms <- names(fm@cnms)))
        mapply(grepl, paste0("^", vnms), names(PAR))
    }
    chkIMod <- function(fm) {## check "intercept only" model
        b1 <- getME(fm,"beta")
        f1 <- fixef(fm)
        stopifnot(hasInms(f1), f1 == b1,
                  hasInms(t1 <- getME(fm,"theta")), matchNms(fm, t1))
    }

    fm1 <- lmer(diameter ~ (1|plate) + (1|sample), Penicillin)
    chkIMod(fm1)

    fm2 <- lmer(angle ~ recipe * temperature + (1|recipe:replicate), cake)
    stopifnot(fixef(fm2) == getME(fm2,"beta"))
    getME(fm2,"theta")

    getME(fm3 <- lmer(Reaction ~ Days + (Days|Subject), sleepstudy),
          "theta")
    getME(fm4 <- lmer(Reaction ~ Days + (1|Subject) + (0+Days|Subject), sleepstudy),
          "theta")

    ## internal consistency check ensuring that all allowed 'name's work (and are not empty):
    (nmME <- eval(formals(lme4:::getME.merMod)$name))
    chkMEs <- function(fm, nms) {
        stopifnot(is.character(nms))
        str(parts <- sapply(nms, getME, object = fm, simplify=FALSE))
        isN <- sapply(parts, is.null)
        stopifnot(identical(names(isN), nms), !any(isN))
    }

    chkMEs(fm1, nmME)
    chkMEs(fm2, nmME)
    chkMEs(fm3, nmME)
    chkMEs(fm4, nmME)

    ## multiple components can now be retrieved at once
    gg <- getME(fm2,c("theta","beta"))
    gg2 <- getME(fm2,c("theta","beta","X"))

    ## list of Zt for each random-effects factor
    lapply(getME(fm2,c("Ztlist")),dim)
    ## Cholesky factors returned as a list of matrices
    getME(fm1,"ST")
    getME(fm2,"ST")

    ## distinction between number of RE terms
    ##  and number of RE grouping factors
    stopifnot(getME(fm2,"n_rtrms")==1)
    stopifnot(getME(fm2,"n_rfacs")==1)

    lapply(getME(fm4,c("Ztlist")),dim)
    stopifnot(getME(fm4,"n_rtrms")==2)
    stopifnot(getME(fm4,"n_rfacs")==1)

    stopifnot(getME(fm1,"sigma")==sigma(fm1))
} ## skip on windows (for speed)

### Adapted from Rune Haubo's ordinal code
### extended convergence checking
### http://en.wikipedia.org/wiki/Karush%E2%80%93Kuhn%E2%80%93Tucker_conditions

## global (for use in several functions)
help_str <- "\n  See ?lme4::convergence and ?lme4::troubleshooting."


##' @param derivs typically the "derivs" attribute of optimizeLmer(); with
##' "gradients" and possibly "Hessian" component
##' @param coefs estimated function value
##' @param ctrl list of lists, each with \code{action} character strings specifying
##' what should happen when a check triggers, and \code{tol} numerical tolerances,
##' as is the result of \code{\link{lmerControl}()$checkConv}.
##' @param lbound vector of lower bounds \emph{for random-effects parameters only}
##'   (length is taken to determine number of RE parameters)
##' @param debug useful if some checks are on "ignore", but would "trigger"
##' @param nobs the number of observations from the dataset
##' @param ndim the number of dimensions for the variance-covariance matrix 
##' of random effects
checkConv <- function(derivs, coefs, ctrl, lbound, ubound, debug = FALSE,
                      nobs = NULL, ndim = NULL)
{
    res <- list()
    ntheta <- length(lbound)

    if (missing(ubound))
        ubound <- rep(Inf, length(lbound))

    ## check singularity first, and unconditionally
    ## (ignore "ignore")
    ccl <- ctrl[[cstr <- "check.conv.singular"]] ; checkCtrlLevels(cstr, cc <- ccl[["action"]])
    ## similar logic to isSingular, but we don't have the fitted object to test
    is.singular <- min(coefs - lbound, ubound - coefs) < ccl$tol

    if (doCheck(cc)) {
        ## singular fit
        ## are there other circumstances where we can get a singular fit?
        if (is.singular) {
            wstr <- "boundary (singular) fit: see help('isSingular')"
            res$messages <- c(res$messages,wstr)
            switch(cc,
                   "message" = message(wstr),
                   "warning" = warning(wstr),
                   "stop" = stop(wstr),
                   stop(gettextf("unknown check level for '%s'", cstr), domain=NA))
        }
    }

    ## DON'T check remaining gradient issues
    if (is.singular) return(res)
    
    ## bail out
    if (is.null(derivs) ||
        (!is.null(nobs) && nobs > ctrl$check.conv.nobsmax) ||
        (!is.null(ndim) && ndim > ctrl$check.conv.nparmax)) {
      return(NULL)
    }
    
    if (anyNA(derivs$gradient))
        return(list(code = -5L,
                    messages = gettextf("Gradient contains NAs")))

    ## gradients:
    ## check absolute gradient (default)
    ccl <- ctrl[[cstr <- "check.conv.grad"]] ; checkCtrlLevels(cstr, cc <- ccl[["action"]])
    wstr <- NULL
    if (doCheck(cc)) {
        scgrad <- tryCatch(with(derivs,solve(chol(Hessian),gradient)),
                           error=function(e)e)
        if (inherits(scgrad, "error") ||
            ## some BLAS versions return NA rather than throwing an error?
            ##  GH #677
            any(is.na(scgrad)))
        {
            wstr <- "unable to evaluate scaled gradient"
            res$code <- -1L
        } else {
            ## find parallel *minimum* of scaled and absolute gradient
            ## the logic here is that we can sometimes get large
            ##  *scaled* gradients even when the *absolute* gradient
            ##  is small because the curvature is very flat as well ...
            mingrad <- pmin(abs(scgrad),abs(derivs$gradient))
            maxmingrad <- max(mingrad)
            if (maxmingrad > ccl$tol) {
                w <- which.max(maxmingrad)
                res$code <- -1L
                wstr <- gettextf("Model failed to converge with max|grad| = %g (tol = %g, component %d)",
                                 maxmingrad, ccl$tol,w)
                wstr <- paste0(wstr, help_str)
            }
        }
        if (!is.null(wstr)) {
            res$messages <- wstr
            switch(cc,
                   "warning" = warning(wstr),
                   "stop" = stop(wstr),
                   stop(gettextf("unknown check level for '%s'", cstr), domain=NA))
        }
        ## note: kktc package uses gmax > kkttol * (1 + abs(fval))
        ##  where kkttol defaults to 1e-3 and fval is the objective f'n value
        ## check relative gradient (only if enabled)
        if (!is.null(ccl$relTol) &&
            (max.rel.grad <- max(abs(derivs$gradient/coefs))) > ccl$relTol) {
                res$code <- -2L
                wstr <-
                    gettextf("Model failed to converge with max|relative grad| = %g (tol = %g)",
                             max.rel.grad, ccl$relTol)
                wstr <- paste0(wstr, help_str)
                res$messages <- wstr
                switch(cc,
                       "warning" = warning(wstr),
                       "stop" = stop(wstr),
                       stop(gettextf("unknown check level for '%s'", cstr), domain=NA))
        }
    }

    ccl <- ctrl[[cstr <- "check.conv.hess"]] ; checkCtrlLevels(cstr, cc <- ccl[["action"]])
    if (doCheck(cc)) {
        if (length(coefs) > ntheta) {
            ## GLMM, check for issues with beta parameters
            H.beta <- derivs$Hessian[-seq(ntheta),-seq(ntheta)]
            resHess <- checkHess(H.beta, ccl$tol, "fixed-effect")
            if (any(resHess$code!=0)) {
                res$code <- resHess$code
                res$messages <- c(res$messages,resHess$messages)
                wstr <- paste(resHess$messages,collapse=";")
                switch(cc,
                       "warning" = warning(wstr),
                       "stop" = stop(wstr),
                       stop(gettextf("unknown check level for '%s'", cstr), domain=NA))
            }
        }
        resHess <- checkHess(derivs$Hessian, ccl$tol)
        if (any(resHess$code != 0)) {
            res$code <- resHess$code
            res$messages <- c(res$messages,resHess$messages)
            wstr <- paste(resHess$messages,collapse=";")
            switch(cc,
                   "warning" = warning(wstr),
                   "stop" = stop(wstr),
                   stop(gettextf("unknown check level for '%s'", cstr), domain=NA))
        }
    }
    if (debug && length(res$messages) > 0) {
        print(res$messages)
    }
    res
}

checkHess <- function(H, tol, hesstype="") {
    ## FIXME: not sure why we decided to save messages as a list
    ## rather than as a character vector??
    res <- list(code=numeric(0),messages=list())
    evd <- tryCatch(eigen(H, symmetric=TRUE, only.values=TRUE)$values,
                    error=function(e)e)
    if (inherits(evd,"error")) {
        res$code <- -6L
        res$messages <- gettextf("Problem with Hessian check (infinite or missing values?)")
    } else {
        negative <- sum(evd < -tol)
        if(negative) {
            res$code <- -3L
            res$messages <-
                gettextf(paste("Model failed to converge:",
                               "degenerate",hesstype,"Hessian with %d negative eigenvalues"),
                         negative)
            res$messages <- paste0(res$messages, help_str)
        } else {
            zero <- sum(abs(evd) < tol)
            if(zero || inherits(tryCatch(chol(H), error=function(e)e), "error")) {
                res$code <- -4L
                res$messages <-
                    paste(hesstype,"Hessian is numerically singular: parameters are not uniquely determined")
            } else {
                res$cond.H <- max(evd) / min(evd)
                if(max(evd) * tol > 1) {
                    res$code <- c(res$code, 2L)
                    res$messages <-
                        c(res$messages,
                          paste("Model is nearly unidentifiable: ",
                                "very large eigenvalue",
                                "\n - Rescale variables?", sep=""))
                }
                if ((min(evd) / max(evd)) < tol) {
                    res$code <- c(res$code, 3L)
                    ## consider skipping warning message if we've
                    ## already hit the previous flag?
                    if(!5L %in% res$code) {
                        res$messages <-
                            c(res$messages,
                              paste("Model is nearly unidentifiable: ",
                                    "large eigenvalue ratio",
                                    "\n - Rescale variables?", sep=""))
                    }
                }
            }
        }
    }
    if (length(res$code)==0) res$code <- 0
    res
}

.onLoad <- function(libname, pkgname) {
    options(lme4.summary.cor.max = 12)
}

.onUnload <- function(libpath) {
    gc()
    if (is.loaded("lmer_Deviance", PACKAGE="lme4"))
        library.dynam.unload("lme4", libpath)
}

library(lme4)
library(merDeriv)
data("Contraception", package = "mlmRev")
source("glmer_batch_funs.R")

Contraception <- transform(Contraception,
                           ch = factor(Contraception$livch != 0, labels = c("N","Y")),
                           age_s = age/(2*sd(age)))

options(contrasts = c("contr.sum", "contr.poly"))

contr_df_name <- data.frame(
  mnames = c("ic", "bc", "bc_int", "bc_int_dist_urb_var", "bc_int_dist_urb_nest", "bc_int_dist_urb_cross"),
  mdesc =
    c("int_child    + age  + (1 | district)",
    "binary_child + age  + (1 | district)",
    "binary_child × age + (1 | district)",
    "binary_child × age + (1 + urban | district)",
    "binary_child × age + (1 | district/urban)",
    "binary_child × age + (1 | district:urban"))

mforms <- list()
mforms$ic <- use ~ age_s + I(age_s^2) + urban + livch + (1|district)
mforms$bc <- update(mforms$ic, . ~ . - livch + ch)
mforms$bc_int_urb <- update(mforms$bc, . ~ . + age_s:ch)
mforms$bc_int_dist_urb_var <- update(mforms$bc_int_urb, . ~ . - (1|district) + (1 + urban | district))
mforms$bc_int_dist_urb_nest <- update(mforms$bc_int_urb, . ~ . - (1|district) + (1 | district/urban))
mforms$bc_int_dist_urb_cross <- update(mforms$bc_int_urb, . ~ . - (1|district) + (1 | urban:district))

contr_mod_list <- lapply(mforms, glmer, data = Contraception, family = binomial)

contr_est <- do.call("rbind", Map(get_est, contr_mod_list, contr_df_name$mnames))
rownames(contr_est) <- NULL
  
contr_confint_wald <- lapply(contr_mod_list, wald_cifun)

contr_prof <- lapply(contr_mod_list, p_fun)
contr_confint_prof <- lapply(contr_prof, confint)
contr_confint_boot <- lapply(contr_mod_list, b_cifun)

## has_cor <- function(x) any(grepl("cor", x$var))
## combfun2(contr_confint_prof, "profile", df_est = contr_est, contr_df_name) |> has_cor()
## combfun1(contr_confint_prof[[4]], "hello", "hello") |> has_cor()

contr_combCI <- Map(combfun2,
                   list(contr_confint_wald, contr_confint_boot, contr_confint_prof),
                   c("Wald", "boot", "profile"),
                   MoreArgs = list(df_est = contr_est, df_name = contr_df_name)) |>
  do.call(what = "rbind")

## structured covariances: fit/store separately?
## (much faster than profile/boot/etc, if they work ...)
contr.diag <- glmer(use ~ diag(1 + age_s|district), Contraception, binomial)
contr.hetdiag <- glmer(use ~ diag(1 + age_s|district, hom = FALSE), 
                       data = Contraception, family = binomial)
contr.cs <- glmer(use ~ cs(1 + age_s|district), Contraception, binomial)
contr.hetcs <- glmer(use ~ cs(1 + age_s|district, hom = FALSE), Contraception, binomial)

## leave contr_prof out of save_list until we have better idea how to 'butcher' (reduce size/exclude environments when serializing)
## (we can get by without it

## inspired by butcher:::axe_env()

## axe_env.thpr <- function(x, verbose = FALSE, ...) {
##   a <- attributes(x)
##   for (d in c("forward", "backward")) {
##     for (i in seq_along(a[[d]])) {
##       cat(i, "\n")
##       attr(attr(a[[d]][[i]], "formula"), ".Environment") <- baseenv()
##     }
##   }
##   attributes(x) <- a
##   x
## }
## x_clean <- axe_env.thpr(x)


save(
  list = c("contr_confint_prof", "contr_confint_boot", "contr_confint_wald", "contr_est", "contr_combCI", "contr_df_name"),
  file = "Contraception_batch.rda",
  version = 2
)

## use old (<=3.5.2) sample() algorithm if necessary
if ("sample.kind" %in% names(formals(RNGkind))) {
    suppressWarnings(RNGkind("Mersenne-Twister", "Inversion", "Rounding"))
}


compFunc <- function(lmeMod, lmerMod, tol = 1e-2){
    lmeVarCorr <- nlme:::VarCorr(lmeMod)[,"StdDev"]
    lmeCoef <- summary(lmeMod)$tTable[,-c(3,5)]
    lmeOut <- c(as.numeric(lmeVarCorr), as.numeric(lmeCoef))
    keep <- !is.na(lmeOut)
    lmeOut <- lmeOut[keep]
    dn <- dimnames(lmeCoef)
    if(is.null(dn)) dn <- list("", names(lmeCoef))
    names(lmeOut) <- c(
        paste(names(lmeVarCorr), "Var"),
        as.character(do.call(outer, c(dn, list("paste")))))[keep]

    ## get nested RE variances in the same order as nlme
    ## FIXME:  not sure if this works generally
    vcLmer <- VarCorr(lmerMod)
    vcLmer <- vcLmer[length(vcLmer):1]
    ##

    lmerVarCorr <- c(sapply(vcLmer, attr, "stddev"),
                     attr(VarCorr(lmerMod), "sc"))
    ## differentiate lme4{new} and lme4.0 :
    lmerCoef <- if(is(lmerMod, "merMod"))
	summary(lmerMod)$coefficients else summary(lmerMod)@coefs
    lmerOut <- c(lmerVarCorr, as.numeric(lmerCoef))
    names(lmerOut) <- names(lmeOut)

    return(list(target = lmeOut, current = lmerOut, tolerance = tol))
}

if (.Platform$OS.type != "windows") {
set.seed(1)
nGroups <- 100
nObs <- 1000

# explanatory variable with a fixed effect
explVar1 <- rnorm(nObs)
explVar2 <- rnorm(nObs)

# random intercept among levels of a grouping factor
groupFac <- as.factor(rep(1:nGroups,each=nObs/nGroups))
randEff0 <- rep(rnorm(nGroups),each=nObs/nGroups)
randEff1 <- rep(rnorm(nGroups),each=nObs/nGroups)
randEff2 <- rep(rnorm(nGroups),each=nObs/nGroups)

# residuals with heterogeneous variance
residSD <- rpois(nObs,1) + 1
residError <- rnorm(nObs,sd=residSD)

# response variable
respVar <- randEff0 + (1+randEff1)*explVar1 + (1+randEff2)*explVar2 + residError

# rename to fit models on one line
y <- respVar
x <- explVar1
z <- explVar2
g <- groupFac
v <- residSD^2
w <- 1/v

library("nlme")
lmeMods <- list(
    ML1   = lme(y ~ x,   random = ~ 1|g,   weights = varFixed(~v), method = "ML"),
    REML1 = lme(y ~ x,   random = ~ 1|g,   weights = varFixed(~v), method = "REML"),
    ML2   = lme(y ~ x,   random = ~ x|g,   weights = varFixed(~v), method = "ML"),
    REML2 = lme(y ~ x,   random = ~ x|g,   weights = varFixed(~v), method = "REML"),
    ML1   = lme(y ~ x+z, random = ~ x+z|g, weights = varFixed(~v), method = "ML"),
    REML2 = lme(y ~ x+z, random = ~ x+z|g, weights = varFixed(~v), method = "REML"))

library("lme4")
lmerMods <- list(
    ML1 =   lmer(y ~ x +  (1|g),      weights = w, REML = FALSE),
    REML1 = lmer(y ~ x +  (1|g),      weights = w, REML = TRUE),
    ML2 =   lmer(y ~ x +  (x|g),      weights = w, REML = FALSE),
    REML2 = lmer(y ~ x +  (x|g),      weights = w, REML = TRUE),
    ML3 =   lmer(y ~ x + z + (x+z|g), weights = w, REML = FALSE),
    REML3 = lmer(y ~ x + z + (x+z|g), weights = w, REML = TRUE))

comp <- mapply(compFunc, lmeMods, lmerMods, SIMPLIFY=FALSE)
stopifnot(all(sapply(comp, do.call, what = all.equal)))
## Look at the relative differences:
sapply(mapply(compFunc, lmeMods, lmerMods, SIMPLIFY=FALSE, tol = 0),
       do.call, what = all.equal)

## add simulated weights to the sleepstudy example
n <- nrow(sleepstudy)
v <- rpois(n,1) + 1
w <- 1/v
sleepLme <- lme(Reaction ~ Days, random = ~ Days|Subject,
                sleepstudy, weights = varFixed(~v),
                method = "ML")
sleepLmer <- lmer(Reaction ~ Days + (Days|Subject),
                  sleepstudy, weights = w,
                  REML = FALSE)
sleepComp <- compFunc(sleepLme, sleepLmer)
stopifnot(do.call(all.equal, sleepComp))
## look at relative differences:
sleepComp$tolerance <- 0
do.call(all.equal, sleepComp)

if (require("mlmRev")) {
    n <- nrow(Chem97)
    v <- rpois(n,1) + 1
    w <- 1/v
    Chem97Lme <- lme(score ~ 1, random = ~ 1|lea/school, Chem97)
    Chem97Lmer <- lmer(score ~ (1|lea/school), Chem97)
    Chem97Comp <- compFunc(Chem97Lme, Chem97Lmer)
    stopifnot(do.call(all.equal, Chem97Comp))
    ## look at relative differences:
    Chem97Comp$tolerance <- 0
    do.call(all.equal, Chem97Comp)
}

set.seed(2)
n <- 40
w <- runif(n)
x <- runif(n)
g <- factor(sample(1:10,n,replace=TRUE))
Z <- model.matrix(~g-1);
y <- Z%*%rnorm(ncol(Z)) + x + rnorm(n)/w^.5
m <- lmer(y ~ x + (1|g), weights=w, REML = TRUE)

## CRAN-forbidden:
## has4.0 <- require("lme4.0"))
has4.0 <- FALSE
if(has4.0) {
    ## m.0 <- lme4.0::lmer(y ~ x + (1|g), weights=w, REML = TRUE)
    lmer0 <- get("lmer", envir=asNamespace("lme4.0"))
    m.0 <- lmer0(y ~ x + (1|g), weights=w, REML = TRUE)
    dput(fixef(m.0)) # c(-0.73065400610675, 2.02895402562926)
    dput(sigma(m.0)) # 1.73614301673377
    dput(VarCorr(m.0)$g[1,1]) # 2.35670451590395
    dput(unname(coef(summary(m.0))[,"Std. Error"]))
    ## c(0.95070076853232, 1.37650858268602)
}
fixef_lme4.0 <- c(-0.7306540061, 2.0289540256)
sigma_lme4.0 <- 1.7361430
Sigma_lme4.0 <- 2.3567045
SE_lme4.0 <- c(0.95070077, 1.37650858)
if(has4.0) try(detach("package:lme4.0"))

stopifnot(all.equal(unname(fixef(m)), fixef_lme4.0, tolerance = 1e-3))
          all.equal(unname(fixef(m)), fixef_lme4.0, tolerance = 0) #-> 1.657e-5

## but these are not at all equal :
(all.equal(sigma(m),                		sigma_lme4.0, tolerance = 10^-3)) # 0.4276
(all.equal(as.vector(VarCorr(m)$g), 		Sigma_lme4.0, tolerance = 10^-3)) # 1.038
(all.equal(as.vector(summary(m)$coefficients[,2]), SE_lme4.0, tolerance = 10^-3)) # 0.4276
## so, lme4.0 was clearly wrong here


##' make sure models that differ only in a constant
##' prior weight have identical deviance:
fm <- lmer(Reaction ~ Days + (Days|Subject), sleepstudy,REML=FALSE)
fm_wt <- lmer(Reaction ~ Days + (Days|Subject), sleepstudy, weights = rep(5, nrow(sleepstudy)),REML=FALSE)
all.equal(deviance(fm), deviance(fm_wt))
} ## skip on windows (for speed)

.simpleCap <- function(x) {
  paste0(toupper(substr(x, 1,1)), substr(x, 2, 1000000L), collapse=" ")
}

### bootMer() --- <==>  (TODO: semi-*)parametric bootstrap
### -------------------------------------------------------
## Doc: show how  this is equivalent - but faster than
##              boot(*, R = nsim, sim = "parametric", ran.gen = simulate(.,1,.), mle = x)
## --> return a "boot" object -- so we could use boot.ci() etc
## TODO: also allow "semi-parametric" model-based bootstrap:
##    resampling the (centered!) residuals (for use.u=TRUE) or for use.u=FALSE,
##    *both* the centered u's + centered residuals
##    instead of using  rnorm()
##  BUT see:
## @article{morris_blups_2002,
##      title = {The {BLUPs} are not "best" when it comes to bootstrapping},
##      volume = {56},
##      issn = {0167-7152},
##      url = {https://www.sciencedirect.com/science/article/pii/S016771520200041X},
##      doi = {10.1016/S0167-7152(02)00041-X},
##      journal = {Statistics \& Probability Letters},
##      author = {Morris, Jeffrey S},
##      year = {2002},
## }
## for an indication of why this is not necessarily a good idea!

##' Perform model-based (Semi-)parametric bootstrap for mixed models.
##'
##' The semi-parametric variant is not yet implemented, and we only
##' provide a method for \code{\link{lmer}}  and \code{\link{glmer}} results.
bootMer <- function(x, FUN, nsim = 1, seed = NULL,
                    use.u = FALSE, re.form = NA,
                    type = c("parametric","semiparametric"),
                    verbose = FALSE,
                    .progress = "none", PBargs=list(),
                    parallel = c("no", "multicore", "snow", "future"),
                    ncpus = getOption("boot.ncpus", 1L), cl = NULL)
{
    stopifnot((nsim <- as.integer(nsim[1])) > 0)
    if (.progress!="none") { ## progress bar
        pbfun <- get(paste0(.progress,"ProgressBar"))
        setpbfun <- get(paste0("set",.simpleCap(.progress),"ProgressBar"))
        pb <- do.call(pbfun,PBargs)
    }


    parallel <- match.arg(parallel)

    # (parallel, ncpus, cl) -> (parallel)
    eval(initialize.parallel)

    if (parallel != "no" && .progress != "none")
        message("progress bar disabled for parallel operations")

    FUN <- match.fun(FUN)
    type <- match.arg(type)
    if(!is.null(seed)) set.seed(seed)
    else if(!exists(".Random.seed", envir = .GlobalEnv))
        runif(1) # initialize the RNG if necessary

    mc <- match.call()
    t0 <- FUN(x)
    if (!is.numeric(t0))
        stop("bootMer currently only handles functions that return numeric vectors")

    ## FIXME: make parameters match profiling output!
    ## should getME(x, "theta") be getME(x, "par")?  Test bootMer with structured cov matrices ...
    mle <- list(beta = getME(x,"beta"), theta = getME(x,"theta"))
    if (isLMM(x)) mle <- c(mle,list(sigma = sigma(x)))
    ## FIXME: what about GLMMs with scale parameters??
    ## FIXME: remove prefix when incorporated in package

    if (type=="parametric") {
        argList <- list(x, nsim=nsim, na.action=na.exclude)
        if (!missing(re.form)) {
            argList <- c(argList,list(re.form=re.form))
        } else {
            argList <- c(argList,list(use.u=use.u))
        }
        ss <- do.call(simulate,argList)
    } else {
        if (!missing(re.form))
            stop(paste(sQuote("re.form")),
                 "cannot be used with semiparametric bootstrapping")
        if (use.u) {
            if (isGLMM(x)) warning("semiparametric bootstrapping is questionable for GLMMs")
            ss <- replicate(nsim,fitted(x)+sample(residuals(x,"response"),
                                                  replace=TRUE),
                            simplify=FALSE)
        } else {
            stop("semiparametric bootstrapping with use.u=FALSE not yet implemented")
        }
    }

    ## FIXME:: use getCall(x) ? check for existence of slot?
    ##  is control used except for merMod?
    control <- if (!is(x, "merMod")) NULL else eval.parent(x@call$control)

    # define ffun as a closure containing the referenced variables
    # in its scope to avoid explicit clusterExport statement
    # in the PSOCKcluster case
    ffun <- local({
      FUN
      refit
      x
      ss
      verbose
      parallel
      control
      length.t0 <- length(t0)
      f1 <- factory(function(i) FUN(refit(x, ss[[i]],
                                          control = control)), errval = rep(NA_real_, length.t0))
      function(i) {
          ret <- f1(i)
          if (verbose) { cat(sprintf("%5d :",i)); str(ret) }
          if (parallel == "no" && .progress!="none") { setpbfun(pb,i/nsim) }
          ret
      }})

    simvec <- seq_len(nsim)
    res <- if (parallel == "multicore") {
        parallel::mclapply(simvec, ffun, mc.cores = ncpus)
    } else if (parallel == "snow") {
        if (is.null(cl)) {
            cl <- parallel::makeCluster(ncpus)
            on.exit(parallel::stopCluster(cl))
            ## explicit export of the lme4 namespace since most FUNs will probably
            ## use some of them
            parallel::clusterExport(cl, varlist=getNamespaceExports("lme4"))
            if(RNGkind()[1L] == "L'Ecuyer-CMRG")
                parallel::clusterSetRNGStream(cl)
        }
        parallel::parLapply(cl, simvec, ffun)
    } else if(parallel == "future") {
        future.apply::future_lapply(simvec, ffun, future.seed = TRUE)
    } else lapply(simvec, ffun)

    t.star <- do.call(cbind,res)
    rownames(t.star) <- names(t0)
    msgs <- list()
    for (mtype in paste0("factory-",c("message","warning","error"))) {
        msgs[[mtype]] <- trimws(unlist(lapply(res, attr, mtype)))
        msgs[[mtype]] <- table(msgs[[mtype]])
    }
    if ((numFail <- sum(msgs[["factory-error"]])) > 0) {
        warning("some bootstrap runs failed (",numFail,"/",nsim,")")
    }
    fail.msgs <- if (numFail==0) NULL else msgs[["factory-error"]]

    ## mimic ending of boot() construction
    s <- structure(list(t0 = t0, t = t(t.star), R = nsim, data = model.frame(x),
                   seed = .Random.seed,
                   statistic = FUN, sim = "parametric", call = mc,
                   ## these two are dummies
                   ran.gen = "simulate(<lmerMod>, 1, *)", mle = mle),
                   class = c("bootMer", "boot"))
    ## leave these for back-compat
    attr(s,"bootFail") <- numFail
    attr(s,"boot.fail.msgs") <- fail.msgs
    attr(s,"boot.all.msgs") <- msgs ## store all messages (tabulated)
    attr(s,"boot_type") <- "boot"
    s
} ## {bootMer}

##' @S3method as.data.frame boot
as.data.frame.bootMer <- function(x,...) {
  as.data.frame(x$t)
}

## FIXME: collapse convergence warnings (ignore numeric values
## when tabulating) ?
print.bootWarnings <- function(x, verbose=FALSE, ...) {
    checkDots(..., .action = "warning")
    msgs <- attr(x, "boot.all.msgs")
    if (is.null(msgs) || all(lengths(msgs)==0)) {
        return(invisible(NULL))
    }
    wstr <- "\n"
    for (i in c("message","warning","error")) {
        f <- paste0("factory-",i)
        m <- sort(msgs[[f]])
        if (length(m)>0) {
            if (!verbose) {
                wstr <- c(wstr,
                          paste0(sum(m)," ",i,"(s): ",names(m)[1]))
                if (length(m)>1) {
                    wstr <- c(wstr," (and others)")
                }
                wstr <- c(wstr,"\n")
            } else {
                wstr <- paste0(i,"(s):\n")
                wstr <- c(wstr,capture.output(cat(cbind("  ",m,names(m)),sep="\n")))
                wstr <- c(wstr,"\n")
            }
        }
    }
    message(wstr)
    return(invisible(NULL))
}

print.bootMer <- function(x,...) {
    NextMethod(x,...)
    print.bootWarnings(x, verbose=FALSE)
    return(invisible(x))
}

confint.bootMer <- function(object, parm=seq(length(object$t0)), level=0.95,
                             type=c("perc","norm","basic"), ...) {
    type <- match.arg(type)
    bnms <- c(norm="normal",basic="basic",perc="percent")
    blens <- c(norm=3, basic=5, perc=5)
    bnm <- bnms[[type]]
    blen <- blens[[type]]
    safe_bootci <- function(i) {
      capture.output(val <- boot::boot.ci(object, index=i, conf=level, type=type)[[bnm]])
      if (is.null(val)) return(matrix(rep(NA_real_, blen), nrow = 1))
      val
    }
    btab0 <- t(vapply(parm, FUN=safe_bootci, FUN.VALUE=numeric(blen)))
    btab <- btab0[, (blen-1):blen, drop=FALSE]
    rownames(btab) <- names(object$t0)
    a <- (1 - level)/2
    a <- c(a, 1 - a)
    ## replicate stats::format.perc
    pct <- paste(format(100 * a, trim = TRUE,
                        scientific = FALSE, digits = 3), "%")
    colnames(btab) <- pct
    return(btab)
}

library(lme4)

options(nwarnings = 1000)

if(getRversion() < "3.2.0") {
    if(interactive()) break # gives an error
    else q() # <- undesirable when interactive !
}

## Try all "standard" (statistical) S3 methods:
.S3generics <- function(class) {
    s3m <- .S3methods(class=class)
    ii <- attr(s3m, "info")
    ii[!ii[, "isS4"], "generic"]
}

set.seed(12)
d <- data.frame(
  g = sample(c("A","B","C","D","E"), 250, replace=TRUE),
  y1 = runif(250, max=100),
  y2 = sample(c(0,1), 250, replace=TRUE)
)

fm3.1 <- lmList(y1 ~ 1 | g, data=d)
fm3.2 <- lmList(y2 ~ 1 | g, data=d, family=binomial)

data(Orthodont, package="nlme")
Orthodont <- as.data.frame(Orthodont) # no "groupedData"
fm2 <- lmList(distance ~ age | Subject, Orthodont)

s3fn <- .S3generics(class= class(fm3.1)[1]) ## works for "old and new" class

noquote(s3fn <- s3fn[s3fn != "print"])# <-- it is show() not print() that works
## [1] coef    confint fitted fixef     formula logLik  pairs  plot
## [9] predict qqnorm  ranef  residuals sigma   summary update

## In lme4 1.1-7 (July 2014), only these worked:
##  coef(), confint(), formula(), logLik(), summary(), update()

## pairs() is excluded for fm3.1 which has only intercept:
## no errors otherwise:
evs <- sapply(s3fn[s3fn != "pairs"], do.call, args = list(fm3.1))

cls <- sapply(evs, function(.) class(.)[1])
clsOk <- cls[c("confint", "fixef", "formula", "logLik",
               "ranef", "sigma", "summary", "update")]
stopifnot(identical(unname(clsOk),
                    c("lmList4.confint", "numeric", "formula", "logLik",
                      "ranef.lmList", "numeric", "summary.lmList", "lmList4")))

## --- fm2 --- non-trivial X: can use pairs(), too:
evs2 <- sapply(s3fn, do.call, args = list(fm2))

## --- fm3.2 --- no failures for this "glmList" :
ss <- function(...) suppressMessages(suppressWarnings(...))
ss(evs3.2 <- sapply(s3fn[s3fn != "pairs"], do.call,
                                  args = list(fm3.2)))

## --- fm4 ---
evs4 <- sapply(s3fn, function(fn)
    tryCatch(do.call(fn, list(fm4)), error=function(e) e))
length(warnings())
summary(warnings()) ##  4 kinds;  glm.fit: fitted probabilities numerically 0 or 1 occurred

str(sapply(evs4, class)) # more errors than above
isok4 <- !sapply(evs4, is, class2="error")

## includes a nice pairs():
evs4[isok4]
## Error msgs of those with errors, first 5, now 3, then 2 :
str(errs4 <- lapply(evs4[!isok4], conditionMessage))
## $ logLik : chr "log-likelihood not available with NULL fits"
## $ summary: chr "subscript out of bounds"
stopifnot(length(errs4) <= 2)

stopifnot(require(lme4))

(testLevel <- lme4:::testLevel())
source(system.file("testdata", "lme-tst-funs.R", package="lme4", mustWork=TRUE))# -> unn()


## "MEMSS" is just 'Suggest' -- must still work, when it's missing:
if (suppressWarnings(!require(MEMSS, quietly=TRUE)) ||
    (data(ergoStool, package="MEMSS") != "ergoStool")) {

    cat("'ergoStool' data from package 'MEMSS' is not available --> skipping test\n")
} else {

fm1   <-  lmer (effort ~ Type + (1|Subject), data = ergoStool)
##sp no longer supported since ~ 2012-3:
##sp fm1.s  <- lmer (effort ~ Type + (1|Subject), data = ergoStool, sparseX=TRUE)
## was segfaulting with sparseX (a while upto 2010-04-06)

fe1   <- fixef(fm1)
##sp fe1.s <- fixef(fm1.s)

print(s1.d <- summary(fm1))
##sp print(s1.s <- summary(fm1.s))
Tse1.d <- c(0.57601226, rep(0.51868384, 3))
stopifnot(exprs = {
    ##sp all.equal(fe1, fe1.s, tolerance= 1e-12)
    all.equal(Tse1.d, unname(se1.d <- coef(s1.d)[,"Std. Error"]),
              tolerance = 1e-6) # std.err.: no too much accuracy
    is(V.d <- vcov(fm1), "symmetricMatrix")
    ##sp all.equal(se1.d, coef(s1.s)[,"Std. Error"])#, tol = 1e-10
    ##sp all.equal(  V.d, vcov(fm1.s))#, tol = 1e-9
    all.equal(Matrix::diag(V.d), unn(se1.d)^2, tolerance= 1e-12)
})

}## if( ergoStool is available from pkg MEMSS )

### -------------------------- a "large" example -------------------------
str(InstEval)

if (FALSE) { # sparse X is not currently implemented, so forget about this:

system.time(## works with 'sparseX'; d has 1128 levels
fm7 <- lmer(y ~ d + service + studage + lectage + (1|s),
             data = InstEval, sparseX=TRUE, verbose=1L, REML=FALSE)
)
system.time(sfm7 <- summary(fm7))
fm7 # takes a while as it computes summary() again !

range(t.fm7 <- coef(sfm7)[,"t value"])## -10.94173  10.61535 for REML, -11.03438  10.70103 for ML

m.t.7 <- mean(abs(t.fm7), trim = .01)
#stopifnot(all.equal(m.t.7, 1.55326395545110, tolerance = 1.e-9)) ##REML value
stopifnot(all.equal(m.t.7, 1.56642013605506, tolerance = 1.e-6)) ## ML

hist.t <- cut(t.fm7, floor(min(t.fm7)) : ceiling(max(t.fm7)))
cbind(table(hist.t))

}# fixed effect 'd' -- with 'sparseX' only --------------------------------

if(testLevel <= 1) { cat('Time elapsed: ', proc.time(),'\n'); q("no") }

## ELSE : (testLevel > 1) :
library(lattice)
source(system.file("testdata/lme-tst-funs.R", package="lme4", mustWork=TRUE))
##--> all.equal(), isOptimized(), ...

system.time(
    fm8.N <- lmer(y ~ service * dept + studage + lectage +
                  (1|s) + (1|d), InstEval, REML=FALSE,
                  control=lmerControl("Nelder_Mead"), verbose = 1L)
    ) ## 14   sec [MM@lynne; 2022-11]
      ## 62   sec [MM@lynne; 2013-11]
      ## 59.5 sec [nb-mm3;   2013-12-31]
system.time(
    fm8.B <- lmer(y ~ service * dept + studage + lectage +
                  (1|s) + (1|d), InstEval, REML=FALSE,
                  control=lmerControl("bobyqa"), verbose = 2L)
    )
      ##  7.8 sec [MM@lynne; 2022-11]
      ## 34.1 sec [nb-mm3; 2013-12-31]

system.time(
    fm8 <- lmer(y ~ service * dept + studage + lectage +
                    (1|s) + (1|d), InstEval, REML=FALSE,
                verbose = 1L)
) ## 7.8 sec [MM@lynne; 2022-11]

stopifnot(isOptimized(fm8.N), isOptimized(fm8.B), isOptimized(fm8))
all.equal(fm8.B, fm8,   tolerance=0)# 9.78e-9 (2022-11); both versions of bobyqa
all.equal(fm8.B, fm8.N, tolerance=0)
## "Mean relative difference: 3.31 e-06"   [nb-mm3; 2013-12-31]
stopifnot(isOptimized(fm8.N), isOptimized(fm8.B), isOptimized(fm8))

str(baseOpti(fm8))
str(baseOpti(fm8.N))
str(baseOpti(fm8.B))

(sm8 <- summary(fm8.B))
str(r8 <- ranef(fm8.B))
noquote(sapply(r8, summary))
r.m8 <- cov2cor(vcov(sm8))
Matrix::image(r.m8, main="cor(<fixed eff[ lmer(*,InstEval) ]>)")

if(testLevel <= 2) { cat('Time elapsed: ', proc.time(),'\n'); q("no") }

## ELSE: testLevel > 2

## Clearly smaller X, but more RE pars
## ==> currently considerably slower than the above

system.time(
    fm9 <-
        lmer(y ~ studage + lectage +
                 (1|s) + (1|d) + (1|dept:service) + (1|dept),
             InstEval, verbose = 1L, REML=FALSE)
)
##  25.6 secs [MM@lynne; 2022-11]
## 410   secs [MM@lynne; 2013-11]

fm9
(sm9 <- summary(fm9))
rr <- ranef(fm9, condVar = TRUE) ## ~ 6 secs

noquote(sapply(rr, summary))
qqr <- qqmath(rr, strip=FALSE)
## NB: x-axis range <==> scale of RE <==> "importance" of effect
qqr$d
qqr$s
dotplot(rr,strip=FALSE)$`dept:service`

cat('Time elapsed: ', proc.time(),'\n') # for ``statistical reasons''

if (.Platform$OS.type != "windows") {

    library(lme4)

    mySumm <- function(.) {
        s <- sigma(.)
        c(beta =getME(., "beta"),
          sigma = s,
          sig01 = unname(s * getME(., "theta")))
    }

    fm1 <- lmer(Yield ~ 1|Batch, Dyestuff)
    boo01 <- bootMer(fm1, mySumm, nsim = 10)
    boo02 <- bootMer(fm1, mySumm, nsim = 10, use.u = TRUE)

    ## boo02 <- bootMer(fm1, mySumm, nsim = 500, use.u = TRUE)
    if (require(boot)) {
        boot.ci(boo02,index=2,type="perc")
    }

    fm2 <- lmer(angle ~ recipe * temperature + (1|recipe:replicate), cake)
    boo03 <- bootMer(fm2, mySumm, nsim = 10)
    boo04 <- bootMer(fm2, mySumm, nsim = 10, use.u = TRUE)

    if (lme4:::testLevel() > 1) {
        gm1 <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
                     data = cbpp, family = binomial)
        boo05 <- bootMer(gm1, mySumm, nsim = 10)
        boo06 <- bootMer(gm1, mySumm, nsim = 10, use.u = TRUE)

        cbpp$obs <- factor(seq(nrow(cbpp)))
        gm2 <- glmer(cbind(incidence, size - incidence) ~ period +
                         (1 | herd) +  (1|obs),
                     family = binomial, data = cbpp)
        boo03 <- bootMer(gm2, mySumm, nsim = 10)
        boo04 <- bootMer(gm2, mySumm, nsim = 10, use.u = TRUE)
    }
    load(system.file("testdata","culcita_dat.RData",package="lme4"))
    cmod <- glmer(predation~ttt+(1|block),family=binomial,data=culcita_dat)
    set.seed(101)
    ## FIXME: sensitive to step-halving PIRLS tests
    ## expect_warning(cc <- confint(cmod,method="boot",nsim=10,quiet=TRUE,
    ##              .progress="txt",PBargs=list(style=3)),"some bootstrap runs failed")


    library(parallel)
    if (max(detectCores(), 1L, na.rm = TRUE) > 1) {
        ## http://stackoverflow.com/questions/12983137/how-do-detect-if-travis-ci-or-not
        travis <- nchar(Sys.getenv("TRAVIS"))>0
        if(.Platform$OS.type != "windows" && !travis) {
            boo01P <- bootMer(fm1, mySumm, nsim = 10, parallel="multicore", ncpus=2)
        }

        ## works in Solaris from an interactive console but not ???
        ##   via R CMD BATCH
        if (Sys.info()["sysname"] != "SunOS")
            boo01P.snow <- bootMer(fm1, mySumm, nsim = 10, parallel="snow", ncpus=2)

        if (require("future.apply", quietly = TRUE)) {
            plan(sequential)
            boo01P.future.seq <- bootMer(fm1, mySumm, nsim = 10, seed = 101, parallel = "future")
            plan(multisession)
            boo01P.future.multi <- bootMer(fm1, mySumm, nsim = 10, seed = 101, parallel = "future")
            stopifnot(all.equal(boo01P.future.seq$t, boo01P.future.multi$t))
            plan(sequential)
        }
    }

    set.seed(101)
    dd <- data.frame(x=runif(200),
                     f=rep(1:20,each=10),
                     o=rnorm(200,mean=2))
    dd$y <- suppressMessages(simulate(~x+(1|f)+offset(o),
                                      family="poisson",
                                      newdata=dd,
                                      newparams=list(theta=1,beta=c(0,2)))[[1]])

    ## fails under flexLambda
    dd$y2 <- suppressMessages(simulate(~x+(1|f)+offset(o),
                                       family="gaussian",
                                       newdata=dd,
                                       newparams=list(theta=1,beta=c(0,2),sigma=1))[[1]])

    fm3 <- glmer(y~x+(1|f)+offset(o),
                 data=dd,family="poisson")

    fm4 <- lmer(y2~x+(1|f)+offset(o),
                data=dd)

    mySumm2 <- function(fit) return(c(fixef(fit),getME(fit,'theta')))
    ## still some issues to fix here
    bb <- bootMer(fm3,mySumm2,nsim=10)
    attr(bb,"boot.fail.msgs")
    bb2 <- bootMer(fm4,mySumm2,nsim=10)

} ## skip on windows (for speed)

## original code for reading/aggregating:

## tickdata <- read.table("Elston2001_tickdata.txt",header=TRUE,
##   colClasses=c("factor","numeric","factor","numeric","factor","factor"))

## tickdata <- transform(tickdata,cHEIGHT=scale(HEIGHT,scale=FALSE))
## for (i in names(tickdata)) {
##   if (is.factor(tickdata[[i]])) {
##     tickdata[[i]] <- factor(tickdata[[i]],levels=sort(as.numeric(levels(tickdata[[i]]))))
##   }
## }
## summary(tickdata)
## grouseticks <- tickdata

## library(reshape)
## meantick <- rename(aggregate(TICKS~BROOD,data=tickdata,FUN=mean),
##                    c(TICKS="meanTICKS"))
## vartick <- rename(aggregate(TICKS~BROOD,data=tickdata,FUN=var),
##                    c(TICKS="varTICKS"))
## uniqtick <- unique(subset(tickdata,select=-c(INDEX,TICKS)))
## grouseticks_agg <- Reduce(merge,list(meantick,vartick,uniqtick))

## save("grouseticks","grouseticks_agg",file="grouseticks.rda")
if (.Platform$OS.type != "windows") {
    library(lme4)
    data(grouseticks)
    do.plots <- FALSE
    form <- TICKS~YEAR+HEIGHT+(1|BROOD)+(1|INDEX)+(1|LOCATION)

    ## fit with lme4
    ## library(lme4)
    ## t1 <- system.time(full_mod1  <- glmer(form, family="poisson",data=grouseticks))
    ## c1 <- c(fixef(full_mod1),unlist(VarCorr(full_mod1)), logLik=logLik(full_mod1),time=t1["elapsed"])
    ## allcoefs1 <- c(unlist(full_mod1@ST),fixef(full_mod1))
    ## detach("package:lme4")

    ## lme4 summary results:
    t1 <- structure(c(1.288, 0.048, 1.36, 0, 0), class = "proc_time",
                    .Names = c("user.self",
                               "sys.self", "elapsed", "user.child", "sys.child"))

    c1 <- structure(c(11.3559080756861, 1.1804105508475, -0.978704335712111,
                      -0.0237607330254979, 0.293232458048324, 0.562551624933584,
                      0.279548178949372,
                      -424.771990224991, 1.36),
                    .Names = c("(Intercept)", "YEAR96",
                               "YEAR97", "HEIGHT", "INDEX", "BROOD", "LOCATION",
                               "logLik", "time.elapsed"
                               ))
    allcoefs1 <- structure(c(0.541509425632023, 0.750034415832756,
                             0.528723159081737,
                             11.3559080756861, 1.1804105508475,
                             -0.978704335712111, -0.0237607330254979
                             ),
                           .Names = c("", "", "", "(Intercept)",
                                      "YEAR96", "YEAR97",  "HEIGHT"))

    pars <- function(x) unlist(getME(x,c("theta","beta")))

    if (lme4:::testLevel() > 1) {
        t2 <- system.time(full_mod2  <- glmer(form, family="poisson",data=grouseticks))
        ##>> 2 x checkConv(.. "derivs") warning: 1. failed to conv.  2. nearly unidentifiable
        c2 <- c(fixef(full_mod2), unlist(VarCorr(full_mod2)),
                logLik = logLik(full_mod2), time= t2["elapsed"])
        ## refit
        ## FIXME: eventually would like to get _exactly_ identical answers on refit()
        full_mod3 <- refit(full_mod2, grouseticks$TICKS)

        print(    all.equal(pars(full_mod2), pars(full_mod3), tolerance=0))# -> 1.2e-5
        stopifnot(all.equal(pars(full_mod2), pars(full_mod3), tolerance=8e-5))
    }

    ## deviance function
    ## FIXME: does compDev do _anything_ any more?
    mm  <- glmer(form, family="poisson", data=grouseticks, devFunOnly=TRUE)
    mm2 <- glmer(form, family="poisson", data=grouseticks,
                 devFunOnly=TRUE,control=glmerControl(compDev=TRUE))
    stopifnot(all.equal(1780.5427072, mm(allcoefs1), tol = 1e-7))
} ## skip on windows (for speed)

### Class definitions for the package

##' Class "lmList4" of 'lm' Objects on Common Model
##'  --> ../man/lmList4-class.Rd
##'      ~~~~~~~~~~~~~~~~~~~~~~~
##' @keywords classes
##' @export
setClass("lmList4",
         representation(call = "call", pool = "logical",
                        groups = "ordered", # or "factor"?  nlme does use ordered()
                        origOrder = "integer" # a permutation
                        ),
         contains = "list")

## TODO?: export
setClass("lmList4.confint", contains = "array")

forceCopy <- function(x) .Call(deepcopy, x)


### FIXME
### shouldn't we have "merPred"  with two *sub* classes "merPredD" and "merPredS"
### for the dense and sparse X cases ?
##
## MM: _Or_ have "merPred" with X  of class  "mMatrix" := classUnion {matrix, Matrix}

merPredD <-
    setRefClass("merPredD", # Predictor class for mixed-effects models with dense X
                fields =
                list(Lambdat = "dgCMatrix",   # depends: theta and Lind
                     ## = t(Lambda); Lambda := lower triangular relative variance factor
                     LamtUt  = "dgCMatrix",   # depends: Lambdat and Ut
                     Lind    = "integer",     # depends: nothing
                     ## integer vector of the same length as 'x' slot in Lambdat.
                     ## Its elements should be in 1: length(theta)

                     Ptr     = "externalptr", # depends:
                     RZX     = "matrix",      # depends: lots
                     Ut      = "dgCMatrix",   # depends: Zt and weights
                     Utr     = "numeric",     # depends: lots
                     V       = "matrix",      # depends:
                     VtV     = "matrix",
                     Vtr     = "numeric",
                     X       = "matrix",    # model matrix for the fixed-effects parameters
                     Xwts    = "numeric",
                     Zt      = "dgCMatrix", # = t(Z); Z = sparse model matrix for the random effects
                     beta0   = "numeric",
                     delb    = "numeric",
                     delu    = "numeric",
                     theta   = "numeric", # numeric vector of variance component parameters
                     u0      = "numeric"),
                methods = list(
                     initialize = function(X, Zt, Lambdat, Lind, theta,
                                           n, # = sample size, usually = nrow(X)
                                           ...) {
                         if (!nargs()) return()
                         ll <- list(...)
                         X <<- as(X, "matrix")
                         Zt <<- as(Zt, "dgCMatrix")
                         Lambdat <<- as(Lambdat, "dgCMatrix")
                         Lind <<- as.integer(Lind)
                         theta <<- as.numeric(theta)
                         N <- nrow(X)
                         p <- ncol(X)
                         q <- nrow(Zt)
                         stopifnot(length(theta) > 0L,
                                   length(Lind) > 0L,
                                   all(sort(unique(Lind)) == seq_along(theta)))
                         RZX <<- if (!is.null(ll$RZX))
                             array(ll$RZX, c(q, p)) else array(0, c(q, p))
                         Utr <<- if (!is.null(ll$Utr))
                                     as.numeric(ll$Utr) else numeric(q)
                         V <<- if (!is.null(ll$V))
                             array(ll$V, c(n, p)) else array(0, c(n, p))
                         VtV <<- if (!is.null(ll$VtV))
                             array(ll$VtV, c(p, p)) else array(0, c(p, p))
                         Vtr <<- if (!is.null(ll$Vtr))
                             as.numeric(ll$Vtr) else numeric(p)
                         beta0 <<- if (!is.null(ll$beta0)) ll$beta0 else numeric(p)
                         delb <<- if (!is.null(ll$delb))
                             as.numeric(ll$delb) else numeric(p)
                         delu <<- if (!is.null(ll$delu))
                             as.numeric(ll$delu) else numeric(q)
                         u0 <<- if (!is.null(ll$u0)) ll$u0 else numeric(q)
                         Ut <<- if (n == N) Zt + 0 else
                             Zt %*% sparseMatrix(i=seq_len(N), j=as.integer(gl(n, 1, N)), x=rep.int(1,N))
                         ## The following is a kludge to overcome problems when Zt is square
                         ## by making LamtUt rectangular
                         LtUt <- Lambdat %*% Ut
                         ## if (nrow(LtUt) == ncol(LtUt))
                         ##     LtUt <- cbind2(LtUt,
                         ##                    sparseMatrix(i=integer(0),
                         ##                                 j=integer(0),
                         ##                                 x=numeric(0),
                         ##                                 dims=c(nrow(LtUt),1)))
                         LamtUt <<- LtUt
                         Xw <- ll$Xwts ## list(...)$Xwts
                         Xwts <<- if (is.null(Xw)) rep.int(1, N) else as.numeric(Xw)
                         initializePtr()
                     },
                     CcNumer      = function() {
                         'returns the numerator of the orthogonality convergence criterion'
                         .Call(merPredDCcNumer, ptr())
                     },
                     L            = function() {
                         'returns the current value of the sparse Cholesky factor'
                         .Call(merPredDL, ptr())
                     },
                     P            = function() {
                         'returns the permutation vector for the sparse Cholesky factor'
                         .Call(merPredDPvec, ptr())
                     },
                     RX           = function() {
                         'returns the dense downdated Cholesky factor for the fixed-effects parameters'
                         .Call(merPredDRX, ptr())
                     },
                     RXi          = function() {
                         'returns the inverse of the dense downdated Cholesky factor for the fixed-effects parameters'
                         .Call(merPredDRXi, ptr())
                     },
                     RXdiag       = function() {
                         'returns the diagonal of the dense downdated Cholesky factor'
                         .Call(merPredDRXdiag, ptr())
                     },
                     b            = function(fac) {
                         'random effects on original scale for step factor fac'
                         .Call(merPredDb, ptr(), as.numeric(fac))
                     },
                     beta         = function(fac) {
                         'fixed-effects coefficients for step factor fac'
                         .Call(merPredDbeta, ptr(), as.numeric(fac))
                     },
                     copy         = function(shallow = FALSE) {
                         def <- .refClassDef
                         selfEnv <- as.environment(.self)
                         vEnv    <- new.env(parent=emptyenv())

                         for (field in setdiff(names(def@fieldClasses), "Ptr")) {
                             if (shallow)
                                 assign(field, get(field, envir = selfEnv), envir = vEnv)
                             else {
                                 current <- get(field, envir = selfEnv)
                                 if (is(current, "envRefClass"))
                                     current <- current$copy(FALSE)
                                 assign(field, forceCopy(current), envir = vEnv)
                             }
                         }
                         do.call(merPredD$new, c(as.list(vEnv), n=nrow(vEnv$V), Class=def))
                     },
                     ldL2         = function() {
                         'twice the log determinant of the sparse Cholesky factor'
                         .Call(merPredDldL2, ptr())
                     },
                     ldRX2        = function() {
                         'twice the log determinant of the downdated dense Cholesky factor'
                         .Call(merPredDldRX2, ptr())
                     },
                     unsc         = function() {
                         'the unscaled variance-covariance matrix of the fixed-effects parameters'
                         .Call(merPredDunsc, ptr())
                     },
                     linPred      = function(fac) {
                         'evaluate the linear predictor for step factor fac'
                         .Call(merPredDlinPred, ptr(), as.numeric(fac))
                     },
                     installPars  = function(fac) {
                         'update u0 and beta0 to the values for step factor fac'
                         .Call(merPredDinstallPars, ptr(), as.numeric(fac))
                     },
                    initializePtr = function() {
                        Ptr <<- .Call(merPredDCreate, X, Lambdat,
                                      LamtUt, Lind, RZX, Ut, Utr, V, VtV, Vtr,
                                      Xwts, Zt, beta0, delb, delu, theta, u0)
                        .Call(merPredDsetTheta, Ptr, theta)
                        .Call(merPredDupdateXwts, Ptr, Xwts)
                        .Call(merPredDupdateDecomp, Ptr, NULL)
                     },
                     ptr          = function() {
                         'returns the external pointer, regenerating if necessary'
                         if (length(theta)) {
                             if (.Call(isNullExtPtr, Ptr)) initializePtr()
                         }
                         Ptr
                     },
                     setBeta0     = function(beta0) {
                         'install a new value of beta'
                         .Call(merPredDsetBeta0, ptr(), as.numeric(beta0))
                     },
                     setTheta     = function(theta) {
                         'install a new value of theta'
                         .Call(merPredDsetTheta, ptr(), as.numeric(theta))
                     },
                     setZt        = function(ZtNonZero) {
                         'install new values in Zt'
                         .Call(merPredDsetZt, ptr(), as.numeric(ZtNonZero))
                     },
                     solve        = function() {
                         'solve for the coefficient increments delu and delb'
                         .Call(merPredDsolve, ptr())
                     },
                     solveU       = function() {
                         'solve for the coefficient increment delu only (beta is fixed)'
                         .Call(merPredDsolveU, ptr())
                     },
                     setDelu      = function(val) {
                         'set the coefficient increment delu'
                         .Call(merPredDsetDelu , ptr(), as.numeric(val))
                     },
                     setDelb      = function(val) {
                         'set the coefficient increment delb'
                         .Call(merPredDsetDelb , ptr(), as.numeric(val))
                     },
                     sqrL         = function(fac) {
                         'squared length of u0 + fac * delu'
                         .Call(merPredDsqrL, ptr(), as.numeric(fac))
                     },
                     u            = function(fac) {
                         'orthogonal random effects for step factor fac'
                         .Call(merPredDu, ptr(), as.numeric(fac))
                     },
                     updateDecomp = function(XPenalty = NULL) {
                         'update L, RZX and RX from Ut, Vt and VtV'
                         invisible(.Call(merPredDupdateDecomp, ptr(), XPenalty))
                     },
                     updateL = function() {
                         'update LamtUt and L'
                         .Call(merPredDupdateL, ptr())
                     },
                     updateLamtUt = function() {
                         'update LamtUt and L'
                         .Call(merPredDupdateLamtUt, ptr())
                     },
                     updateRes    = function(wtres) {
                         'update Vtr and Utr using the vector of weighted residuals'
                         .Call(merPredDupdateRes, ptr(), as.numeric(wtres))
                     },
                     updateXwts   = function(wts) {
                         'update Ut and V from Zt and X using X weights'
                         .Call(merPredDupdateXwts, ptr(), wts)
                     }
                     )
                )
merPredD$lock("Lambdat", "LamtUt", "Lind", "RZX", "Ut", "Utr", "V", "VtV", "Vtr",
              "X", "Xwts", "Zt", "beta0", "delb", "delu", "theta", "u0")


## -> ../man/lmResp-class.Rd
##    ~~~~~~~~~~~~~~~~~~~~~~
lmResp <-                               # base class for response modules
    setRefClass("lmResp",
                fields =
                list(Ptr     = "externalptr",
                     mu      = "numeric",
                     offset  = "numeric",
                     sqrtXwt = "numeric",
                     sqrtrwt = "numeric",
                     weights = "numeric",
                     wtres   = "numeric",
                     y       = "numeric"),
                methods =
                list(
                    allInfo = function() {
                        'return all the information available on the object'
                        data.frame(y=y, offset=offset, weights=weights, mu=mu,
                                   rwt=sqrtrwt, wres=wtres, Xwt=sqrtXwt)
                    },
                    initialize = function(...) {
                        if (!nargs()) return()
                        ll <- list(...)
                        if (is.null(ll$y)) stop("y must be specified")
                        y <<- as.numeric(ll$y)
                        n <- length(y)
                        mu <<- if (!is.null(ll[["mu"]]))
                            as.numeric(ll[["mu"]]) else numeric(n)
                        offset <<- if (!is.null(ll$offset))
                            as.numeric(ll$offset) else numeric(n)
                        weights <<- if (!is.null(ll$weights))
                            as.numeric(ll$weights) else rep.int(1,n)
                        sqrtXwt <<- if (!is.null(ll$sqrtXwt))
                            as.numeric(ll$sqrtXwt) else sqrt(weights)
                        sqrtrwt <<- if (!is.null(ll$sqrtrwt))
                            as.numeric(ll$sqrtrwt) else sqrt(weights)
                        wtres   <<- sqrtrwt * (y - mu)
                    },
                    copy         = function(shallow = FALSE) {
                        def <- .refClassDef
                        selfEnv <- as.environment(.self)
                        vEnv    <- new.env(parent=emptyenv())
                        for (field in setdiff(names(def@fieldClasses), "Ptr")) {
                            if (shallow)
                                assign(field, get(field, envir = selfEnv), envir = vEnv)
                            else {
                                current <- get(field, envir = selfEnv)
                                if (is(current, "envRefClass"))
                                    current <- current$copy(FALSE)
                                ## deep-copy hack +0
                                assign(field, forceCopy(current), envir = vEnv)
                            }
                        }
                        do.call(new, c(as.list(vEnv), Class=def))
                    },
                    initializePtr = function() {
                        Ptr <<- .Call(lm_Create, y, weights, offset, mu, sqrtXwt,
                                      sqrtrwt, wtres)
                        .Call(lm_updateMu, Ptr, mu)
                    },
                    ptr       = function() {
                        'returns the external pointer, regenerating if necessary'
                        if (length(y)) {
                            if (.Call(isNullExtPtr, Ptr)) initializePtr()
                        }
                        Ptr
                    },
                    setOffset  = function(oo) {
                        'change the offset in the model (used in profiling)'
                        .Call(lm_setOffset, ptr(), as.numeric(oo))
                    },
                    setResp    = function(rr) {
                        'change the response in the model, usually after a deep copy'
                        .Call(lm_setResp, ptr(), as.numeric(rr))
                    },
                    setWeights = function(ww) {
                        'change the prior weights in the model'
                        .Call(lm_setWeights, ptr(), as.numeric(ww))
                    },
                    updateMu  = function(gamma) {
                        'update mu, wtres and wrss from the linear predictor'
                        .Call(lm_updateMu, ptr(), as.numeric(gamma))
                    },
                    wrss      = function() {
                        'returns the weighted residual sum of squares'
                        .Call(lm_wrss, ptr())
                    })
                )

lmResp$lock("mu", "offset", "sqrtXwt", "sqrtrwt", "weights", "wtres")#, "y")

lmerResp <-
    setRefClass("lmerResp", contains = "lmResp",
                fields = list(REML = "integer"),
                methods=
                list(initialize = function(...) {
                         REML <<- as.integer(list(...)$REML)
                         if (length(REML) != 1L) REML <<- 0L
                         callSuper(...)
                     },
                     initializePtr = function() {
                         Ptr <<- .Call(lmer_Create, y, weights, offset, mu, sqrtXwt,
                                       sqrtrwt, wtres)
                         .Call(lm_updateMu, Ptr, mu - offset)
                         .Call(lmer_setREML, Ptr, REML)
                     },
                     ptr        = function() {
                         'returns the external pointer, regenerating if necessary'
                         if (length(y))
                             if (.Call(isNullExtPtr, Ptr)) initializePtr()
                         Ptr
                     },
                     objective  = function(ldL2, ldRX2, sqrL, sigma.sq = NULL) {
                         'returns the profiled deviance or REML criterion'
                         .Call(lmer_Laplace, ptr(), ldL2, ldRX2, sqrL, sigma.sq)
                     })
                )

setOldClass("family")

##' @export
glmResp <-
    setRefClass("glmResp", contains = "lmResp",
                fields = list(eta = "numeric", family = "family", n = "numeric"),
                methods=
                list(initialize = function(...) {
                         callSuper(...)
                         ll <- list(...)
                         if (is.null(ll$family)) stop("family must be specified")
                         family <<- ll$family
                         n <<- if (!is.null(ll$n)) as.numeric(ll$n) else rep.int(1,length(y))
                         eta <<- if (!is.null(e <- ll[["etastart"]])) as.numeric(e) else numeric(length(y))
                     },
                     aic          = function() {
                         .Call(glm_aic, ptr())
                     },
                     allInfo = function() {
                         'return all the information available on the object'
                         cbind(callSuper(),
                               data.frame(eta=eta, muEta=muEta(), var=variance(),
                                          WrkWt=sqrtWrkWt(), wrkRes=wrkResids(),
                                          wrkResp=wrkResp(), devRes=devResid()))
                     },
                     devResid  = function() {
                         'returns the vector of deviance residuals'
                         .Call(glm_devResid, ptr())
                     },
                     fam       = function() {
                         'returns the name of the glm family'
                         .Call(glm_family, ptr())
                     },
                     Laplace   = function(ldL2, ldRX2, sqrL) {
                         'returns the Laplace approximation to the profiled deviance'
                         .Call(glm_Laplace, ptr(), ldL2, ldRX2, sqrL)
                     },
                     link      = function() {
                         'returns the name of the glm link'
                         .Call(glm_link, ptr())
                     },
                     muEta     = function() {
                         'returns the diagonal of the Jacobian matrix, d mu/d eta'
                         .Call(glm_muEta, ptr())
                     },
                     ptr       = function() {
                         'returns the external pointer, regenerating if necessary'
                         if (length(y)) {
                             if (.Call(isNullExtPtr, Ptr)) {
                                 Ptr <<- .Call(glm_Create, family, y, weights, offset, mu, sqrtXwt,
                                               sqrtrwt, wtres, eta, n)
                                 .Call(glm_updateMu, Ptr, eta - offset)
                             }
                         }
                         Ptr
                     },
                     resDev = function() {
                         'returns the sum of the deviance residuals'
                         .Call(glm_resDev, ptr())
                     },
                     setTheta = function(theta) {
                         'sets a new value of theta, for negative binomial distribution only'
                         .Call(glm_setTheta, ptr(), as.numeric(theta))
                     },
                     sqrtWrkWt = function() {
                         'returns the square root of the working X weights'
                         .Call(glm_sqrtWrkWt, ptr())
                     },
                     theta = function() {
                         'query the value of theta, for negative binomial distribution only'
                         .Call(glm_theta, ptr())
                     },
                     updateMu = function(gamma) {
                         'update mu, residuals, weights, etc. from the linear predictor'
                         .Call(glm_updateMu, ptr(), as.numeric(gamma))
                     },
                     updateWts = function() {
                         'update the residual and X weights from the current value of eta'
                         .Call(glm_updateWts, ptr())
                     },
                     variance = function() {
                         'returns the vector of variances'
                         .Call(glm_variance, ptr())
                     },
                     wtWrkResp = function() {
                         'returns the vector of weighted working responses'
                         .Call(glm_wtWrkResp, ptr())
                     },
                     wrkResids = function() {
                         'returns the vector of working residuals'
                         .Call(glm_wrkResids, ptr())
                     },
                     wrkResp = function() {
                         'returns the vector of working responses'
                         .Call(glm_wrkResp, ptr())
                     }
                     )
                )

glmResp$lock("family", "n", "eta")


##' @export
nlsResp <-
    setRefClass("nlsResp", contains = "lmResp",
                fields= list(gam   = "numeric",
                             nlmod = "formula",
                             nlenv = "environment",
                             pnames= "character"),
                methods=
                list(initialize = function(...) {
                         callSuper(...)
                         ll <- list(...)
                         if (is.null(ll$nlmod)) stop("nlmod must be specified")
                         nlmod <<- ll$nlmod
                         if (is.null(ll$nlenv)) stop("nlenv must be specified")
                         nlenv <<- ll$nlenv
                         if (is.null(ll$pnames)) stop("pnames must be specified")
                         pnames <<- ll$pnames
                         if (is.null(ll$gam)) stop("gam must be specified")
                         stopifnot(length(ll$gam) == length(offset))
                         gam <<- ll$gam
                     },
                     Laplace =function(ldL2, ldRX2, sqrL) {
                         'returns the profiled deviance or REML criterion'
                         .Call(nls_Laplace, ptr(), ldL2, ldRX2, sqrL)
                     },
                     ptr     = function() {
                         'returns the external pointer, regenerating if necessary'
                         if (length(y)) {
                             if (.Call(isNullExtPtr, Ptr)) {
                                 Ptr <<- .Call(nls_Create, y, weights, offset, mu, sqrtXwt,
                                               sqrtrwt, wtres, gam, nlmod[[2]], nlenv, pnames)
                                 .Call(nls_updateMu, Ptr, gam)
                             }
                         }
                         Ptr
                     },
                     updateMu=function(gamma) {
                         'update mu, residuals, gradient, etc. given the linear predictor matrix'
                         .Call(nls_updateMu, ptr(), as.numeric(gamma))
                     })
                )

nlsResp$lock("nlmod", "nlenv", "pnames")


##' Generator object for the \code{\linkS4class{glmFamily}} class
##'
##' The generator object for the \code{\linkS4class{glmFamily}} reference class.
##' Such an object is primarily used through its \code{new} method.
##'
##'
##' @param ... Named argument (see Note below)
##' @note Arguments to the \code{new} method must be named arguments.
##' @section Methods: \describe{
##'     \item{\code{new(family=family)}}{Create a new
##'        \code{\linkS4class{glmFamily}} object}
##' }
##' @seealso \code{\linkS4class{glmFamily}}
##' @keywords classes
##' @export
glmFamily <-                            # used in tests of family definitions
    setRefClass("glmFamily",
                fields=list(Ptr="externalptr", family="family"),
                methods=
                list(
                     aic = function(y, n, mu, wt, dev) {
                         'returns the value from the aic member function, which is actually the deviance'
                         nn <- length(y <- as.numeric(y))
                         stopifnot(length(n <- as.numeric(n)) == nn,
                                   length(mu <- as.numeric(mu)) == nn,
                                   length(wt <- as.numeric(wt)) == nn,
                                   all(wt >= 0),
                                   length(dev <- as.numeric(dev)) == 1L)
                         .Call(glmFamily_aic, ptr(), y, n, mu, wt, dev)
                     },
                     devResid = function(y, mu, wt) {
                         'applies the devResid function to y, mu and wt'
                         mu <- as.numeric(mu)
                         wt <- as.numeric(wt)
                         y  <- as.numeric(y)
                         stopifnot(length(mu) == length(wt),
                                   length(mu) == length(y),
                                   all(wt >= 0))
                         .Call(glmFamily_devResid, ptr(), y, mu, wt)
                     },
                     link = function(mu) {
                         'applies the (forward) link function to mu'
                         .Call(glmFamily_link, ptr(), as.numeric(mu))
                     },
                     linkInv = function(eta) {
                         'applies the inverse link function to eta'
                         .Call(glmFamily_linkInv, ptr(), as.numeric(eta))
                     },
                     muEta = function(eta) {
                         'applies the muEta function to eta'
                         .Call(glmFamily_muEta, ptr(), as.numeric(eta))
                     },
                     ptr   = function() {
                         if (length(family))
                             if (.Call(isNullExtPtr, Ptr))
                                 Ptr <<- .Call(glmFamily_Create, family)
                         Ptr
                     },
                     setTheta = function(theta) {
                         'sets a new value of theta, for negative binomial distribution only'
                         .Call(glmFamily_setTheta, ptr(), as.numeric(theta))
                     },
                     theta = function() {
                         'query the value of theta, for negative binomial distribution only'
                         .Call(glmFamily_theta, ptr())
                     },
                     variance = function(mu) {
                         'applies the variance function to mu'
                         .Call(glmFamily_variance, ptr(), as.numeric(mu))
                     })
                )
##' Class \code{"glmFamily"} - a reference class for \code{\link{family}}
##'
##' This class is a wrapper class for \code{\link{family}} objects specifying a
##' distibution family and link function for a generalized linear model
##' (\code{\link{glm}}).  The reference class contains an external pointer to a
##' C++ object representing the class.  For common families and link functions
##' the functions in the family are implemented in compiled code so they can be
##' accessed from other compiled code and for a speed boost.
##'
##'
##' @name glmFamily-class
##' @docType class
##' @note Objects from this reference class correspond to objects in a C++
##' class.  Methods are invoked on the C++ class using the external pointer in
##' the \code{Ptr} field.  When saving such an object the external pointer is
##' converted to a null pointer, which is why there is a redundant field
##' \code{ptr} that is an active-binding function returning the external
##' pointer.  If the \code{Ptr} field is a null pointer, the external pointer is
##' regenerated for the stored \code{family} field.
##' @section Extends: All reference classes extend and inherit methods from
##' \code{"\linkS4class{envRefClass}"}.
##' @seealso \code{\link{family}}, \code{\link{glmFamily}}
##' @keywords classes
##' @examples
##'
##' str(glmFamily$new(family=poisson()))
NULL

##' Generator object for the golden search optimizer class.
##'
##' The generator objects for the \code{\linkS4class{golden}} class of a scalar
##' optimizer for a parameter within an interval.  The optimizer uses reverse
##' communications.
##'
##' @param \dots additional, optional arguments.  None are used at present.
##' @note Arguments to the \code{new} methods must be named arguments.
##' \code{lower} and \code{upper} are the bounds for the scalar parameter; they must be finite.
##' @section Methods:
##' \describe{
##'      \item{\code{new(lower=lower, upper=upper)}}{Create a new
##'         \code{\linkS4class{golden}} object.}
##' }
##' @seealso \code{\linkS4class{golden}}
##' @keywords classes
##' @export
golden <-
    setRefClass("golden", # Reverse communication implementation of Golden Search
                fields =
                list(
                     Ptr     = "externalptr",
                     lowerbd = "numeric",
                     upperbd = "numeric"
                     ),
                methods =
                list(
                     initialize = function(lower, upper, ...) {
                         stopifnot(length(lower <- as.numeric(lower)) == 1L,
                                   length(upper <- as.numeric(upper)) == 1L,
                                   lower > -Inf,
                                   upper < Inf,
                                   lower < upper)
                         lowerbd <<- lower
                         upperbd <<- upper
                         Ptr <<- .Call(golden_Create, lower, upper)
                     },
                     ptr        =  function() {
                         if (length(lowerbd))
                             if (.Call(isNullExtPtr, Ptr))
                                 Ptr <<- .Call(golden_Create, lowerbd, upperbd)
                         Ptr
                     },
                     newf       = function(value) {
                         stopifnot(length(value <- as.numeric(value)) == 1L)
                         .Call(golden_newf, ptr(), value)
                     },
                     value      = function() .Call(golden_value, ptr()),
                     xeval      = function() .Call(golden_xeval, ptr()),
                     xpos       = function() .Call(golden_xpos, ptr())
                     )
            )
##' Class \code{"golden"}
##'
##' A reference class for a golden search scalar optimizer using reverse
##' communication.
##'
##'
##' @name golden-class
##' @docType class
##' @section Extends: All reference classes extend and inherit methods from
##'    \code{"\linkS4class{envRefClass}"}.
##' @keywords classes
##' @examples
##'
##' showClass("golden")
##'
NULL

## Generator object for the Nelder-Mead optimizer class  "NelderMead"
##
## A reference class for a Nelder-Mead simplex optimizer allowing box
## constraints on the parameters and using reverse communication.
NelderMead <-
    setRefClass("NelderMead", # Reverse communication implementation of Nelder-Mead simplex optimizer
                fields =
                list(
                     Ptr     = "externalptr",
                     lowerbd = "numeric",
                     upperbd = "numeric",
                     xstep   = "numeric",
                     xtol    = "numeric"
                     ),
                methods =
                list(
                     initialize = function(lower, upper, xst, x0, xt, ...) {
                         stopifnot((n <- length(lower <- as.numeric(lower))) > 0L,
                                   length(upper <- as.numeric(upper)) == n,
                                   all(lower < upper),
                                   length(xst <- as.numeric(xst)) == n,
                                   all(xst != 0),
                                   length(x0 <- as.numeric(x0)) == n,
                                   all(x0 >= lower),
                                   all(x0 <= upper),
                                   all(is.finite(x0)),
                                   length(xt <- as.numeric(xt)) == n,
                                   all(xt > 0))
                         lowerbd <<- lower
                         upperbd <<- upper
                         xstep   <<- xst
                         xtol    <<- xt
                         Ptr <<- .Call(NelderMead_Create, lowerbd, upperbd, xstep, x0, xtol)
                     },
                     ptr          = function() {
                         if (length(lowerbd))
                             if (.Call(isNullExtPtr, Ptr))
                                 Ptr <<- .Call(NelderMead_Create, lowerbd, upperbd, xstep, x0, xtol)
                         Ptr
                     },
                     newf         = function(value) {
                         stopifnot(length(value <- as.numeric(value)) == 1L)
                         .Call(NelderMead_newf, ptr(), value)
                     },
                     setForceStop = function(stp=TRUE) .Call(NelderMead_setForce_stop, ptr(), stp),
                     setFtolAbs   = function(fta)      .Call(NelderMead_setFtol_abs, ptr(), fta),
                     setFtolRel   = function(ftr)      .Call(NelderMead_setFtol_rel, ptr(), ftr),
                     setMaxeval   = function(mxev)     .Call(NelderMead_setMaxeval, ptr(), mxev),
                     setMinfMax   = function(minf)     .Call(NelderMead_setMinf_max, ptr(), minf),
                     setIprint    = function(iprint)   .Call(NelderMead_setIprint, ptr(), iprint),
                     value        = function()         .Call(NelderMead_value, ptr()),
                     xeval        = function()         .Call(NelderMead_xeval, ptr()),
                     xpos         = function()         .Call(NelderMead_xpos, ptr())
                     )
            )


##' Class "merMod" of Fitted Mixed-Effect Models
##'
##' A mixed-effects model is represented as a \code{\linkS4class{merPredD}} object
##' and a response module of a class that inherits from class
##' \code{\linkS4class{lmResp}}.  A model with a \code{\linkS4class{lmerResp}}
##' response has class \code{lmerMod}; a \code{\linkS4class{glmResp}} response
##' has class \code{glmerMod}; and a \code{\linkS4class{nlsResp}} response has
##' class \code{nlmerMod}.
##'
##' @name merMod-class
##' @aliases merMod-class lmerMod-class glmerMod-class nlmerMod-class merMod
##' show,merMod-method
##' anova.merMod coef.merMod deviance.merMod
##' fitted.merMod formula.merMod logLik.merMod
##' model.frame.merMod model.matrix.merMod print.merMod
##' show.merMod summary.merMod
##' terms.merMod update.merMod
##' vcov.merMod print.summary.merMod show.summary.merMod
##' summary.summary.merMod vcov.summary.merMod
##' @docType class
##' @section Objects from the Class: Objects are created by calls to
##' \code{\link{lmer}}, \code{\link{glmer}} or \code{\link{nlmer}}.
##' @seealso \code{\link{lmer}}, \code{\link{glmer}}, \code{\link{nlmer}},
##' \code{\linkS4class{merPredD}}, \code{\linkS4class{lmerResp}},
##' \code{\linkS4class{glmResp}}, \code{\linkS4class{nlsResp}}
##' @keywords classes
##' @examples
##'
##' showClass("merMod")
##' methods(class="merMod")
##' @export
setClass("merMod",
         representation(Gp      = "integer",
                        call    = "call",
                        frame   = "data.frame", # "model.frame" is not S4-ized yet
                        flist   = "list",
                        cnms    = "list",
                        lower   = "numeric",
                        theta   = "numeric",
                        beta    = "numeric",
                        u       = "numeric",
                        devcomp = "list",
                        pp      = "merPredD",
                        optinfo = "list"))

##' @export
setClass("lmerMod", representation(resp="lmerResp"), contains="merMod")

##' @export
setClass("glmerMod", representation(resp="glmResp"), contains="merMod")

##' @export
setClass("nlmerMod", representation(resp="nlsResp"), contains="merMod")

##' Generator object for the rePos (random-effects positions) class
##'
##' The generator object for the \code{\linkS4class{rePos}} class used
##' to determine the positions and orders of random effects associated
##' with particular random-effects terms in the model.
##' @param \dots Argument list (see Note).
##' @note Arguments to the \code{new} methods must be named arguments.
##' \code{mer}, an object of class \code{"\linkS4class{merMod}"}, is
##' the only required/expected argument.
##' @section Methods:
##' \describe{
##'      \item{\code{new(mer=mer)}}{Create a new
##'         \code{\linkS4class{rePos}} object.}
##' }
##' @seealso \code{\linkS4class{rePos}}
##' @keywords classes
##' @export
rePos <-
    setRefClass("rePos",
                fields =
                list(
                     cnms    = "list", # component names (components are terms within a RE term)
                     flist   = "list", # list of grouping factors used in the random-effects terms
                     ncols   = "integer", # number of components for each RE term
                     nctot   = "integer", # total number of components per factor
                     nlevs   = "integer", # number of levels for each unique factor
                     offsets = "integer", # points to where each term starts
                     terms   = "list" # list with one element per factor, indicating corresponding term
                     ),
                methods =
                list(
                     initialize = function(mer, ...) {
                         ##' asgn indicates unique elements of flist
                         ##'
                         stopifnot((ntrms <- length(Cnms <- mer@cnms)) > 0L,
                                   (length(Flist <- mer@flist)) > 0L,
                                   length(asgn  <- as.integer(attr(Flist, "assign"))) == ntrms)
                         cnms    <<- Cnms
                         flist   <<- Flist
                         ncols   <<- unname(lengths(cnms))
                         nctot   <<- unname(as.vector(tapply(ncols, asgn, sum)))
                         nlevs   <<- unname(vapply(flist, function(el) length(levels(el)), 0L))
                         # why not replace the sapply with ncols*nlevs[asgn] ??
                         offsets <<- c(0L, cumsum(sapply(seq_along(asgn),
                                                         function(i) ncols[i] * nlevs[asgn[i]])))
                         terms   <<- lapply(seq_along(flist), function(i) which(asgn == i))
                     }
                     )
            )
##' Class \code{"rePos"}
##'
##' A reference class for determining the positions in the random-effects vector
##' that correspond to particular random-effects terms in the model formula
##'
##' @name rePos-class
##' @docType class
##' @section Extends: All reference classes extend and inherit methods from
##'    \code{"\linkS4class{envRefClass}"}.
##' @keywords classes
##' @examples
##'
##' showClass("rePos")
##'
rePos$lock("cnms", "flist", "ncols", "nctot", "nlevs", "terms")

vcRep <-
    setRefClass("vcRep",
                fields =
                list(
                     theta     = "numeric",
                     lower     = "numeric",
                     Lambdat   = "dgCMatrix",
                     Lind      = "integer",
                     Gp        = "integer",
                     flist     = "list",
                     cnms      = "list",
                     ncols     = "integer",
                     nctot     = "integer",
                     nlevs     = "integer",
                     offsets   = "integer",
                     terms     = "list",
                     sig       = "numeric",
                     nms       = "character",
                     covar     = "list",
                     useSc     = "logical"
                     ),
                methods =
                list(
                     initialize = function(mer, ...) {
                         stopifnot((ntrms <- length(Cnms <- mer@cnms)) > 0L,
                                   (length(Flist <- mer@flist)) > 0L,
                                   length(asgn  <- as.integer(attr(Flist, "assign"))) == ntrms)
                         lower   <<- getME(mer, "lower")
                         theta   <<- getME(mer, "theta")
                         Lambdat <<- getME(mer, "Lambdat")
                         Lind    <<- getME(mer, "Lind")
                         Gp      <<- getME(mer, "Gp")
                         cnms    <<- Cnms
                         flist   <<- Flist
                         ncols   <<- unname(lengths(cnms))
                         nctot   <<- unname(as.vector(tapply(ncols, asgn, sum)))
                         nlevs   <<- unname(vapply(flist, function(el) length(levels(el)), 0L))
                         offsets <<- c(0L, cumsum(sapply(seq_along(asgn),
                                                         function(i) ncols[i] * nlevs[asgn[i]])))
                         terms   <<- lapply(seq_along(Flist), function(i) which(asgn == i))
                         sig     <<- sigma(mer)
                         nms     <<- names(Flist)[asgn]
                         covar   <<- mkVarCorr(sig, cnms, ncols, theta, nms)
                         useSc   <<- as.logical(getME(mer, "devcomp")$dims[["useSc"]])
                     },
                     asCovar     = function() {
                         ans <- lapply(covar,
                                       function(x) {
                                           attr(x, "correlation") <- attr(x, "stddev") <- NULL
                                           x
                                       })
                         attr(ans, "residVar") <- attr(covar, "sc")^2
                         ans
                     },
                     asCorr      = function() {
                         ans <- lapply(covar, function(x)
                                       list(correlation=attr(x, "correlation"),
                                            stddev=attr(x, "stddev")))
                         attr(ans, "residSD") <- attr(covar, "sc")
                         ans
                     },
                     setTheta    = function(ntheta) {
                         stopifnot(length(ntheta <- as.numeric(ntheta)) == length(lower),
                                   all(ntheta >= lower))
                         theta   <<- ntheta
                         covar   <<- mkVarCorr(sig, cnms, ncols, theta, nms)
                     },
                     setSc       = function(nSc) {
                         stopifnot(useSc,
                                   length(nSc <- as.numeric(nSc)) == 1L)
                         sig     <<- nSc
                         covar   <<- mkVarCorr(sig, cnms, ncols, theta, nms)
                     },
                     setResidVar = function(nVar) setSc(sqrt(as.numeric(nVar))),
                     setRECovar  = function(CV) {
                         if (is.matrix(CV) && length(covar) == 1L) {
                             CV <- list(CV)
                             names(CV) <- names(covar)
                         }
                         covsiz <- sapply(covar, ncol)
                         stopifnot(is.list(CV),
                                   all(names(CV) == names(covar)),
                                   all(sapply(CV, isSymmetric)),
                                   all(sapply(CV, ncol) == covsiz))
                         if (!all(lengths(cnms) == covsiz))
                             error("setRECovar currently requires distinct grouping factors")
                         theta <<- sapply(CV, function(mm)
                                      {
                                          ff <- t(chol(mm))/sig
                                          ff[upper.tri(ff, diag=TRUE)]
                                      })
                     })
                )

eps <- .Machine$double.eps
oneMeps <- 1 - eps
set.seed(1)

## sample linear predictor values for the unconstrained families
etas <- list(
    seq.int(-8, 8, by=1),
    runif(17, -8, 8),  # random sample from wide uniform dist
    rnorm(17, 0, 8),   # random sample from wide normal dist
    c(-10^30, rnorm(15, 0, 4), 10^30)
)

## sample linear predictor values for the families in which eta must be positive
etapos <- list(seq.int(1, 20, by=1),
               rexp(20),
               rgamma(20, 3),
               pmax(.Machine$double.eps, rnorm(20, 2, 1)))

## values of mu in the (0,1) interval
mubinom <- list(runif(100, 0, 1),
                rbeta(100, 1, 3),
                pmin(pmax(eps, rbeta(100, 0.1, 3)), oneMeps),
                pmin(pmax(eps, rbeta(100, 3, 0.1)), oneMeps))

test_that("inverse link and muEta functions", {
    tst.lnki <- function(fam, frm) {
        ff <- glmFamily$new(family=fam)
        ## as.numeric() needed for binomial()$linkinv breakage (also in muEta test)
        sapply(frm, function(x) expect_that(fam$linkinv(as.numeric(x)), equals(ff$linkInv(x))))
    }

    tst.muEta <- function(fam, frm) {
        ff <- glmFamily$new(family=fam)
        sapply(frm, function(x) expect_that(fam$mu.eta(as.numeric(x)), equals(ff$muEta(x))))
    }
    
    tst.lnki(binomial(),           etas) # binomial with logit link
    tst.muEta(binomial(),          etas)
    tst.lnki(binomial("probit"),   etas) # binomial with probit link
    tst.muEta(binomial("probit"),  etas)
    tst.lnki(binomial("cloglog"),  etas) # binomial with cloglog link
    tst.muEta(binomial("cloglog"), etas)
    tst.lnki(binomial("cauchit"),  etas) # binomial with cauchit link
    tst.muEta(binomial("cauchit"), etas)
    tst.lnki(poisson(),            etas) # Poisson with log link
    tst.muEta(poisson(),           etas)
    tst.lnki(gaussian(),           etas) # Gaussian with identity link
    tst.muEta(gaussian(),          etas)
    tst.lnki(Gamma(),              etapos) # gamma family
    tst.muEta(Gamma(),             etapos)
    tst.lnki(inverse.gaussian(),   etapos) # inverse Gaussian
    tst.muEta(inverse.gaussian(),  etapos)    
})

test_that("link and variance functions", {
    tst.link <- function(fam, frm) {
        ff <- glmFamily$new(family=fam)
        sapply(frm, function(x) expect_that(fam$linkfun(x), equals(ff$link(x))))
    }

    tst.variance <- function(fam, frm) {
        ff <- glmFamily$new(family=fam)
        sapply(frm, function(x) expect_that(fam$variance(x), equals(ff$variance(x))))
    }

    tst.link(    binomial(),                   mubinom)
    tst.variance(binomial(),                   mubinom)
    tst.link(    binomial("probit"),           mubinom)
    tst.link(    binomial("cauchit"),          mubinom)
    tst.link(    gaussian(),                   etas)
    tst.variance(gaussian(),                   etas)
    tst.link(    Gamma(),                      etapos)
    tst.variance(Gamma(),                      etapos)
    tst.link(    inverse.gaussian(),           etapos)
    tst.variance(inverse.gaussian(),           etapos)
    tst.variance(MASS::negative.binomial(1),   etapos)
    tst.variance(MASS::negative.binomial(0.5), etapos)    
    tst.link(    poisson(),                    etapos)
    tst.variance(poisson(),                    etapos)
})

test_that("devResid and aic", {
    tst.devres <- function(fam, frm) {
        ff <- glmFamily$new(family=fam)
        sapply(frm, function(x) {
            nn <- length(x)
            wt <- rep.int(1, nn)
            n  <- wt
            y  <- switch(fam$family,
                         binomial = rbinom(nn, 1L, x),
                         gaussian =  rnorm(nn, x),
                         poisson  =  rpois(nn, x),
                         error("Unknown family"))
            dev <- ff$devResid(y, x, wt)
            expect_that(fam$dev.resids(y, x, wt), equals(dev))
            dd  <- sum(dev)
            expect_that(fam$aic(y, n, x, wt, dd), equals(ff$aic(y, n, x, wt, dd)))
        })
    }

    tst.devres(binomial(), mubinom)
    tst.devres(gaussian(), etas)
    tst.devres(poisson(),  etapos)
})

test_that("variance", {
    tst.variance <- function(fam, frm) {
        ff <- glmFamily$new(family=fam)
        sapply(frm, function(x) expect_that(fam$variance(x), equals(ff$variance(x))))
    }
    tst.variance(MASS::negative.binomial(1.),   etapos)
    nb1       <- MASS::negative.binomial(1.)
    cppnb1    <- glmFamily$new(family=nb1)
    expect_that(cppnb1$theta(),        equals(1))
    nb2       <- MASS::negative.binomial(2.)
    cppnb1$setTheta(2)
    sapply(etapos, function(x) expect_that(cppnb1$variance(x), equals(nb2$variance(x))))
    bfam      <- glmFamily$new(family=binomial())
    if (FALSE) {
      ## segfaults on MacOS mavericks 3.1.0 ... ??
      expect_error(bfam$theta())#, "theta accessor applies only to negative binomial")
      expect_error(bfam$setTheta(2))#, "setTheta applies only to negative binomial")
    }
    })

simfun_gam <- function(ngrp = 50, nrep = 50, shape_gam = 2, intercept = 1, 
                       use_simulate = FALSE, seed = NULL) {
  if (!is.null(seed)) set.seed(seed)
  dd <- expand.grid(group = 1:ngrp, rep = 1:nrep)
  if (use_simulate) {
    dd$y <- simulate(~ 1 + (1 | group), newdata = dd, 
                     family = Gamma(link = "log"), 
                     newparams = list(
                       theta = 1, beta = 1, sigma = 1/sqrt(shape_gam)))[[1]]
    return(dd) 
  }
  b <- rnorm(ngrp)
  eta <- intercept + b  
  mu <- exp(eta)
  y <- rgamma(nrow(dd), shape = shape_gam, scale = mu/shape_gam)
  data.frame(dd, y)
}

dd1 <- simfun_gam(seed = 101)
dd2 <- simfun_gam(seed = 101, use_simulate = TRUE)

test_that("simulated Gamma data matches with simulate()", {
  expect_equal(dd1$y, dd2$y)
})

test_that("estimated Gamma shape is correct", {
  m1 <- glmer(y ~ 1 + (1|group), family = Gamma(link = "log"), data = dd2)
  shape_val <- 1/sigma(m1)^2
  expect_equal(shape_val, 2.0, tolerance = 0.05)
  expect_equal(shape_val, 1.94511502080571, tolerance = 1e-6)
})

test_that("glmer works for Gamma with small shape parameter", {
  ## shape=0.1 previously caused PIRLS divergence
  ## we are also testing for different values of Gamma to be safe
  shape_vec <- c(0.1,0.5,1,2,5)
  set.seed(123)
  for(shape_param in shape_vec){
    d <- expand.grid(block = LETTERS[1:26], rep = 1:100, KEEP.OUT.ATTRS = FALSE)
    d$x <- runif(nrow(d))
    reff_f <- rnorm(length(levels(d$block)), sd = 1)
    d$eta <- 4 + 3 * d$x + reff_f[d$block]
    d$y <- rgamma(nrow(d), scale = exp(d$eta) / shape_param, shape = shape_param)

    fit <- suppressWarnings(
      glmer(y ~ x + (1|block), data = d, family = Gamma(link = "log"))
    )
    expect_s4_class(fit, "merMod")
    ## testing for the shape parameter.
    expect_equal(1/sigma(fit)^2, shape_param, tolerance = 0.1)
    ## fixed effects should be close to true values (intercept=4, slope=3)
    expect_equal(unname(fixef(fit)), c(4, 3), tolerance = 0.5)
    print(unname(fixef(fit)))
  }
})

simfun_invgauss <- function(ngrp = 50, nrep = 500, lambda = 1, 
                            use_simulate = FALSE, seed = NULL) {
  if (!is.null(seed)) set.seed(seed)
  dd <- expand.grid(group = 1:ngrp, rep = 1:nrep)
  if (use_simulate) {
    dd$y <- simulate(~ 1 + (1 | group), newdata = dd, 
                     family = inverse.gaussian(link = "1/mu^2"), 
                     newparams = list(
                       theta = c("group.(Intercept)" = 1), 
                       beta = c("(Intercept)" = 4),
                       sigma = 1/sqrt(lambda)))[[1]]
    return(dd) 
  }
  b <- rnorm(ngrp)
  eta <- 4 + b 
  mu <- 1/sqrt(eta)
  y <- statmod::rinvgauss(nrow(dd), mean = mu, shape = lambda)
  data.frame(dd, y)
}

ddig1 <- simfun_invgauss(seed = 101, use_simulate = FALSE)
ddig2 <- simfun_invgauss(seed = 101, use_simulate = TRUE)

test_that("simulated Inverse Gaussian data matches with simulate()", {
  expect_equal(ddig1$y, ddig2$y)
})

test_that("estimated Inverse Gaussian shape is correct", {
  m1 <- glmer(y ~ 1 + (1 | group), family = inverse.gaussian(link = "1/mu^2"), 
              data = ddig2)
  shape_val <- 1/sigma(m1)^2
  expect_equal(shape_val, 1, tolerance = 0.05)
  expect_equal(shape_val, 1.032144112298057, tolerance = 1e-6)
})

data(Dyestuff, package="lme4")
n     <- nrow(Dyestuff)
ones  <- rep.int(1, n)
zeros <- rep.int(0, n)
YY    <- Dyestuff$Yield
mYY   <- mean(YY)

#context("lmerResp objects")
test_that("lmerResp", {
    mres  <- YY - mYY
    rr    <- lmerResp$new(y=YY)

    expect_that(rr$weights,                   equals(ones))
    expect_that(rr$sqrtrwt,                   equals(ones))
    expect_that(rr$sqrtXwt,                   equals(ones))
    expect_that(rr$offset,                    equals(zeros))
    expect_that(rr$mu,                        equals(zeros))
    expect_that(rr$wtres,                     equals(YY))
    expect_that(rr$wrss(),                    equals(sum(YY^2)))
    expect_that(rr$updateMu(rep.int(mYY, n)), equals(sum(mres^2)))
    expect_that(rr$REML,                      equals(0L))

    rr$REML <- 1L
    expect_that(rr$REML,                      equals(1L))
})

mlYY <- mean(log(YY))
gmeanYY <- exp(mlYY)                    # geometric mean

#context("glmResp objects")
test_that("glmResp", {
    mres  <- YY - gmeanYY
    gmean <- rep.int(gmeanYY, n)
    rr    <- glmResp$new(family=poisson(), y=YY)
    
    expect_that(rr$weights,                    equals(ones))
    expect_that(rr$sqrtrwt,                    equals(ones))
    expect_that(rr$sqrtXwt,                    equals(ones))
    expect_that(rr$offset,                     equals(zeros))
    expect_that(rr$mu,                         equals(zeros))
    expect_that(rr$wtres,                      equals(YY))
    expect_that(rr$n,                          equals(ones))

    ## wrss() causes an update of mu which becomes ones, wtres also changes
    expect_that(rr$wrss(),                     equals(sum((YY-1)^2)))
    expect_that(rr$mu,                         equals(ones))
    expect_that(rr$wtres,                      equals(YY-ones))

    expect_that(rr$updateMu(rep.int(mlYY, n)), equals(sum(mres^2)))
    expect_that(rr$mu,                         equals(gmean))
    expect_that(rr$muEta(),                    equals(gmean))
    expect_that(rr$variance(),                 equals(gmean))

    rr$updateWts()
    expect_that(1/sqrt(rr$variance()),         equals(rr$sqrtrwt))
    expect_that(as.vector(rr$sqrtXwt),         equals(rr$sqrtrwt * rr$muEta()))
})

require(lme4)
source(system.file("test-tools-1.R", package = "Matrix"))# identical3() etc

## use old (<=3.5.2) sample() algorithm if necessary
if ("sample.kind" %in% names(formals(RNGkind))) {
    suppressWarnings(RNGkind("Mersenne-Twister", "Inversion", "Rounding"))
}

## Check that quasi families throw an error
assertError(lmer(cbind(incidence, size - incidence) ~ period + (1|herd),
                 data = cbpp, family = quasibinomial))
assertError(lmer(incidence ~ period + (1|herd),
                 data = cbpp, family = quasipoisson))
assertError(lmer(incidence ~ period + (1|herd),
                 data = cbpp, family = quasi))

## check bug found by Kevin Buhr
set.seed(7)
n <- 10
X <- data.frame(y=runif(n), x=rnorm(n), z=sample(c("A","B"), n, TRUE))
fm <- lmer(log(y) ~ x | z, data=X)  ## ignore grouping factors with
## gave error inside  model.frame()
stopifnot(all.equal(c(`(Intercept)` = -0.834544), fixef(fm), tolerance=.01))

## is "Nelder_Mead" default optimizer?
(isNM   <- formals(lmerControl)$optimizer == "Nelder_Mead")
(isOldB <- formals(lmerControl)$optimizer == "bobyqa")
(isOldTol <- environment(nloptwrap)$defaultControl$xtol_abs == 1e-6)

if (.Platform$OS.type != "windows") withAutoprint({

    source(system.file("testdata", "lme-tst-funs.R", package="lme4", mustWork=TRUE))# -> uc()

    ## check working of Matrix methods on  vcov(.) etc ----------------------
    fm1 <- lmer(Reaction ~ Days + (Days|Subject), sleepstudy)
     V  <- vcov(fm)
    (V1 <- vcov(fm1))
    (C1 <- chol(V1))
    dput(dV <- as.numeric(diag(V))) # 0.17607818634.. [x86_64, F Lnx 36]
    TOL <- 0 # to show the differences below
    TOL <- 1e-5 # for the check
    stopifnot(exprs = {
        all.equal(dV, uc(if(isNM)     0.176076 else if(isOldB) 0.176068575 else
                         if(isOldTol) 0.1761714 else           0.1760782),
                  tolerance = 9*TOL) # seen 7.8e-8; Apple clang 14.0.3 had 6.3783e-5
        all.equal(sqrt(dV), as.numeric(chol(V)), tol = 1e-12)
        all.equal(diag(V1), uc(`(Intercept)` = 46.5751, Days = 2.38947), tolerance = 40*TOL)# 5e-7 (for "all" algos)
        is(C1, "dtrMatrix") # was inherits(C1, "Cholesky")
        dim(C1) == c(2,2)
        all.equal(as.numeric(C1), # 6.8245967  0. -0.2126263  1.5310962 [x86_64, F Lnx 36]
                  c(6.82377, 0, -0.212575, 1.53127), tolerance=20*TOL)# 1.2e-4  ("all" algos)
        dim(chol(crossprod(getME(fm1, "Z")))) == 36
    })
    ## printing
    signif(chol(crossprod(getME(fm, "Z"))), 5) # -> simple 4 x 4 sparse

    showProc.time() #

    ## From: Stephane Laurent
    ## To:   r-sig-mixed-models@..
    ## "crash with the latest update of lme4"
    ##
    ## .. example for which lmer() crashes with the last update of lme4 ...{R-forge},
    ## .. but not with version CRAN version (0.999999-0)
    lsDat <- data.frame(
        Operator = as.factor(rep(1:5, c(3,4,8,8,8))),
        Part = as.factor(
            c(2L, 3L, 5L,
              1L, 1L, 2L, 3L,
              1L, 1L, 2L, 2L, 3L, 3L, 4L, 5L,
              1L, 2L, 3L, 3L, 4L, 4L, 5L, 5L,
              1L, 2L, 2L, 3L, 3L, 4L, 5L, 5L)),
        y =
            c(0.34, -1.23, -2.46,
              -0.84, -1.57,-0.31, -0.18,
              -0.94, -0.81, 0.77, 0.4, -2.37, -2.78, 1.29, -0.95,
              -1.58, -2.06, -3.11,-3.2, -0.1, -0.49,-2.02, -0.75,
              1.71,  -0.85, -1.19, 0.13, 1.35, 1.92, 1.04,  1.08))

    xtabs( ~ Operator + Part, data=lsDat) # --> 4 empty cells, quite a few with only one obs.:
    ##         Part
    ## Operator 1 2 3 4 5
    ##        1 0 1 1 0 1
    ##        2 2 1 1 0 0
    ##        3 2 2 2 1 1
    ##        4 1 1 2 2 2
    ##        5 1 2 2 1 2
    lsD29 <- lsDat[1:29, ]

    ## FIXME: rank-Z test should probably not happen in this case:
    (sm3 <- summary(m3 <- lm(y ~ Part*Operator, data=lsDat)))# ok: some interactions not estimable
    stopifnot(21 == nrow(coef(sm3)))# 21 *are* estimable
    sm4  <- summary(m4 <- lm(y ~ Part*Operator, data=lsD29))
    stopifnot(20 == nrow(coef(sm4)))# 20 *are* estimable
    lf <- lFormula(y ~ (1|Part) + (1|Operator) + (1|Part:Operator), data = lsDat)
    dim(Zt <- lf$reTrms$Zt)## 31 x 31
    c(rankMatrix(Zt)) ## 21
    c(rankMatrix(Zt,method="qr")) ## 31 ||  29 (64 bit Lnx), then 21 (!)
    c(rankMatrix(t(Zt),method="qr")) ## 30, then 21 !
    nrow(lsDat)
    fm3 <- lmer(y ~ (1|Part) + (1|Operator) + (1|Part:Operator), data = lsDat,
                control=lmerControl(check.nobs.vs.rankZ="warningSmall"))

    lf29 <- lFormula(y ~ (1|Part) + (1|Operator) + (1|Part:Operator), data = lsD29)
    (fm4 <- update(fm3, data=lsD29))
    fm4. <- update(fm4, REML=FALSE,
                   control=lmerControl(optimizer="nloptwrap",
                                       optCtrl=list(ftol_abs=1e-6,
                                                    xtol_abs=1e-6)))
    ## summary(fm4.)
    stopifnot(
        all.equal(as.numeric(formatVC(VarCorr(fm4.), digits = 7)[,"Std.Dev."]),
                  c(1.040664, 0.6359187, 0.5291422, 0.4824796), tol = 1e-4)
    )
    showProc.time()

}) ## skip on windows (for speed)


cat('Time elapsed: ', proc.time(),'\n') # for ``statistical reasons''

## copied/modified from nlme

##' split, on the nm call, the rhs of a formula into a list of subformulas
splitFormula <- function(form, sep = "/")
{
    if (inherits(form, "formula") ||
        mode(form) == "call" && form[[1]] == as.name("~"))
        splitFormula(form[[length(form)]], sep = sep)
    else if (mode(form) == "call" && form[[1]] == as.name(sep))
        do.call(c, lapply(as.list(form[-1]), splitFormula, sep = sep))
    else if (mode(form) == "(")
        splitFormula(form[[2]], sep = sep)
    else if (length(form))
        list(asOneSidedFormula(form))
    ## else
    ##  NULL
}

## Recursive version of all.vars
allVarsRec <- function(object)
{
    if (is.list(object)) {
        unlist(lapply(object, allVarsRec))
    } else {
        all.vars(object)
    }
}

## simple version of getData.gnls from nlme
## but we *should* and *can* work with environment(formula(.))
getData.merMod <-  function(object) {
    mCall <- getCall(object)
    data <- eval(mCall$data, environment(formula(object)))
    if (!is.data.frame(data) && !is.matrix(data)) stop(paste(sQuote("data"),"object found is not a data frame or matrix"))
    return(data)
}

asOneFormula <-
    ## Constructs a linear formula with all the variables used in a
    ## list of formulas, except for the names in omit
    function(..., omit = c(".", "pi"))
{
    names <- unique(allVarsRec(list(...)))
    names <- names[is.na(match(names, omit))]
    if (length(names))
        as.formula(paste("~", paste(names, collapse = "+"))) # else NULL
}

getIDLabels <- function(object, form=formula(object)) {
    mf <- factorize(form,model.frame(object))
    if (length(ff <- reformulas::findbars(form))>0) {
        grps <- lapply(ff,"[[",3)
    } else {
        grps <- form[[2]]
    }
    if (identical(grps,quote(.obs))) return(seq(fitted(object)))
    fList <- lapply(grps,function(x) eval(x,mf))
    do.call(interaction,fList)
}

## TESTING
## lme4:::getIDLabels(fm1)

## Return the formula(s) for the groups associated with object.
## The result is a one-sided formula unless asList is TRUE in which case
## it is a list of formulas, one for each level.
getGroupsFormula <- function(object, asList = FALSE, sep = "+")
    UseMethod("getGroupsFormula")

getGroupsFormula.default <-
    ## Return the formula(s) for the groups associated with object.
    ## The result is a one-sided formula unless asList is TRUE in which case
    ## it is a list of formulas, one for each level.
    function(object, asList = FALSE, sep = "/")
{
    form <- formula(object)
    if (!inherits(form, "formula")){
        stop("\"Form\" argument must be a formula")
    }
    form <- form[[length(form)]]
    if (!((length(form) == 3) && (form[[1]] == as.name("|")))) {
        ## no conditioning expression
        return(NULL)
    }
    ## val <- list( asOneSidedFormula( form[[ 3 ]] ) )
    val <- splitFormula(asOneSidedFormula(form[[3]]), sep = sep)
    names(val) <- unlist(lapply(val, function(el) deparse(el[[2]])))
                                        #  if (!missing(level)) {
                                        #    if (length(level) == 1) {
                                        #      return(val[[level]])
                                        #    } else {
                                        #      val <- val[level]
                                        #    }
                                        #  }
    if (asList) as.list(val)
    else as.formula(paste("~", paste(names(val), collapse = sep)))
}

getGroupsFormula.merMod <- function(object,asList=FALSE, sep="+") {
    if (asList) {
        lapply(names(object@flist),asOneSidedFormula)
    } else {
        asOneSidedFormula(paste(names(object@flist),collapse=sep))
    }
}

getCovariateFormula <- function (object)
{
    form <- formula(object)
    if (!(inherits(form, "formula"))) {
        stop("formula(object) must return a formula")
    }
    form <- form[[length(form)]]
    if (length(form) == 3 && form[[1]] == as.name("|")) {
        form <- form[[2]]
    }
    eval(substitute(~form))
}

getResponseFormula <-
    function(object)
{
    ## Return the response formula as a one sided formula
    form <- formula(object)
    if (!(inherits(form, "formula") && (length(form) == 3))) {
        stop("\"Form\" must be a two sided formula")
    }
    as.formula(paste("~", deparse(form[[2]])))
}

##' diagnostic plots for merMod fits
##' @param x a fitted [ng]lmer model
##' @param form an optional formula specifying the desired type of plot. Any
##' variable present in the original data frame used to obtain
##' \code{x} can be referenced. In addition, \code{x} itself can be
##' referenced in the formula using the symbol \code{"."}. Conditional
##'  expressions on the right of a \code{|} operator can be used to
##'  define separate panels in a lattice display. Default is
##' \code{resid(., type = "pearson") ~ fitted(.)}, corresponding to a plot
##' of the standardized residuals versus fitted values.
##' @param abline an optional numeric value, or numeric vector of length
##'   two. If given as a single value, a horizontal line will be added to the
##'   plot at that coordinate; else, if given as a vector, its values are
##'   used as the intercept and slope for a line added to the plot. If
##'   missing, no lines are added to the plot.
##' @param id an optional numeric value, or one-sided formula. If given as
##' a value, it is used as a significance level for a two-sided outlier
##' test for the standardized, or normalized residuals. Observations with
##'   absolute standardized (normalized) residuals greater than the \eqn{1-value/2}
##' quantile of the standard normal distribution are
##' identified in the plot using \code{idLabels}. If given as a one-sided
##'   formula, its right hand side must evaluate to a  logical, integer, or
##'   character vector which is used to identify observations in the
##'   plot. If missing, no observations are identified.
##' @param idLabels an optional vector, or one-sided formula. If given as a
##'   vector, it is converted to character and used to label the
##'   observations identified according to \code{id}. If given as a
##'    vector, it is converted to character and used to label the
##'    observations identified according to \code{id}. If given as a
##'    one-sided formula, its right hand side must evaluate to a vector
##'    which is converted to character and used to label the identified
##'    observations. Default is the interaction of all the grouping variables
##'    in the data frame.  The special formula
##' @param grid an optional logical value indicating whether a grid should
##'    be added to plot. Default depends on the type of lattice plot used:
##'    if \code{xyplot} defaults to \code{TRUE}, else defaults to
##'    \code{FALSE}.
##'  @param \dots optional arguments passed to the lattice plot function.
##' @details Diagnostic plots for the linear mixed-effects fit are obtained. The
##'  \code{form} argument gives considerable flexibility in the type of
##'  plot specification. A conditioning expression (on the right side of a
##'  \code{|} operator) always implies that different panels are used for
##'  each level of the conditioning factor, according to a lattice
##'  display. If \code{form} is a one-sided formula, histograms of the
##'  variable on the right hand side of the formula, before a \code{|}
##'  operator, are displayed (the lattice function \code{histogram} is
##'  used). If \code{form} is two-sided and both its left and
##'  right hand side variables are numeric, scatter plots are displayed
##'  (the lattice function \code{xyplot} is used). Finally, if \code{form}
##'  is two-sided and its left had side variable is a factor, box-plots of
##'  the right hand side variable by the levels of the left hand side
##'  variable are displayed (the lattice function  \code{bwplot} is used).
##' @author original version in \code{nlme} package by Jose Pinheiro and Douglas Bates
##' @examples
##' data(Orthodont,package="nlme")
##' fm1 <- lmer(distance ~ age + (age|Subject), data=Orthodont)
##' ## standardized residuals versus fitted values by gender
##' plot(fm1, resid(., scaled=TRUE) ~ fitted(.) | Sex, abline = 0)
##' ## box-plots of residuals by Subject
##' plot(fm1, Subject ~ resid(., scaled=TRUE))
##' ## observed versus fitted values by Subject
##' plot(fm1, distance ~ fitted(.) | Subject, abline = c(0,1))
##' ## residuals by age, separated by Subject
##' plot(fm1, resid(., scaled=TRUE) ~ age | Sex, abline = 0)

##' if (require(ggplot2)) {
##'     ## we can create the same plots using ggplot2 and the fortify() function
##'     fm1F <- fortify(fm1)
##'     ggplot(fm1F, aes(.fitted,.resid)) + geom_point(colour="blue") +
##'            facet_grid(.~Sex) + geom_hline(yintercept=0)
##'     ## note: Subjects are ordered by mean distance
##'     ggplot(fm1F, aes(Subject,.resid)) + geom_boxplot() + coord_flip()
##'     ggplot(fm1F, aes(.fitted,distance))+ geom_point(colour="blue") +
##'         facet_wrap(~Subject) +geom_abline(intercept=0,slope=1)
##'     ggplot(fm1F, aes(age,.resid)) + geom_point(colour="blue") + facet_grid(.~Sex) +
##'         geom_hline(yintercept=0)+geom_line(aes(group=Subject),alpha=0.4)+geom_smooth(method="loess")
##'     ## (warnings about loess are due to having only 4 unique x values)
##'     detach("package:ggplot2")
##' }
##' @S3method plot merMod
##' @method plot merMod
##' @export
plot.merMod <-
    function(x, form = resid(., type = "pearson") ~ fitted(.), abline,
             id = NULL, idLabels = NULL,
             grid, ...)
        ## Diagnostic plots based on residuals and/or fitted values
{
    object <- x
    if (!inherits(form, "formula"))
        stop("\"form\" must be a formula")
    ## constructing data
    ## can I get away with using object@frame???
    allV <- all.vars(asOneFormula(form, id, idLabels))
    allV <- allV[is.na(match(allV,c("T","F","TRUE","FALSE",".obs")))]
    if (length(allV) > 0) {
        data <- getData(object)
        if (is.null(data)) {            # try to construct data
            alist <- lapply(as.list(allV), as.name)
            names(alist) <- allV
            alist <- c(list(as.name("data.frame")), alist)
            mode(alist) <- "call"
            data <- eval(alist, sys.parent(1))
        } else if (any(naV <- is.na(match(allV, names(data)))))
              stop(allV[naV], " not found in data")
    } else data <- NULL

    ## this won't do because there may well be variables we want
    ##  that were not in the model call

    ## data <- object@frame

    ## argument list
    dots <- list(...)
    args <- if (length(dots) > 0) dots else list()
    ## appending object to data, and adding observation-number variable
    if (length(data) > 0) {
        data <- cbind(data, .obs = seq(nrow(data)))
    }
    data <- as.list(c(as.list(data), . = list(object)))
    ## covariate - must always be present
    covF <- getCovariateFormula(form)
    .x <- eval(covF[[2]], data)
    if (!is.numeric(.x)) {
        stop("Covariate must be numeric")
    }
    argForm <- ~ .x
    argData <- data.frame(.x = .x, check.names = FALSE)
    if (is.null(args$xlab)) {
        if (is.null(xlab <- attr(.x, "label")))
            xlab <- deparse(covF[[2]])
        args$xlab <- xlab
    }

    ## response - need not be present
    respF <- getResponseFormula(form)
    if (!is.null(respF)) {
        .y <- eval(respF[[2]], data)
        if (is.null(args$ylab)) {
            if (is.null(ylab <- attr(.y, "label")))
                ylab <- deparse(respF[[2]])
            args$ylab <- ylab
        }
        argForm <- .y ~ .x
        argData[, ".y"] <- .y
    }

    ## groups - need not be present
    grpsF <- getGroupsFormula(form)
    if (!is.null(grpsF)) {
        ## ?? FIXME ???
        gr <- splitFormula(grpsF, sep = "*")
        for(i in seq_along(gr)) {
            auxGr <- all.vars(gr[[i]])
            for(j in auxGr)
                argData[[j]] <- eval(as.name(j), data)
        }
        argForm <-
            as.formula(paste(if (length(argForm) == 2)
                                 "~ .x |" else ".y ~ .x |",
                             deparse(grpsF[[2]])))
    }
    ## adding to args list
    args <- c(list(argForm, data = argData), args)
    if (is.null(args$strip)) {
        args$strip <- function(...) strip.default(..., style = 1)
    }
    if (is.null(args$cex)) args$cex <- par("cex")
    if (is.null(args$adj)) args$adj <- par("adj")

    if (!is.null(id)) {       ## identify points in plot
        idResType <- "pearson"  ## diff from plot.lme: 'normalized' not available
        id <- switch(mode(id),
                     numeric = {
                         if (id <= 0 || id >= 1)
                             stop(shQuote("id")," must be between 0 and 1")
                         abs(resid(object, type = idResType))/sigma(object) >
                             -qnorm(id / 2)
                     },
                     call = eval(asOneSidedFormula(id)[[2]], data),
                     stop(shQuote("id")," can only be a formula or numeric.")
                     )
        if (is.null(idLabels)) {
            idLabels <- getIDLabels(object)
        } else {
            if (inherits(idLabels,"formula")) {
                idLabels <- getIDLabels(object,idLabels)
            } else if (is.vector(idLabels)) {
                if (length(idLabels <- unlist(idLabels)) != length(id)) {
                    stop("\"idLabels\" of incorrect length")
                }
            } else stop("\"idLabels\" can only be a formula or a vector")
        }
        ## DON'T subscript by id, will be done later
        idLabels <- as.character(idLabels)
    }

    ## defining abline, if needed
    if (missing(abline)) {
        abline <- if (missing(form)) # r ~ f
                      c(0, 0) else NULL
    }

                                        #assign("id", id , where = 1)
                                        #assign("idLabels", idLabels, where = 1)
                                        #assign("abl", abline, where = 1)
    assign("abl", abline)

    ## defining the type of plot
    if (length(argForm) == 3) {
        if (is.numeric(.y)) {           # xyplot
            plotFun <- "xyplot"
            if (is.null(args$panel)) {
                args <- c(args,
                          panel = list(function(x, y, subscripts, ...)
                          {
                              x <- as.numeric(x)
                              y <- as.numeric(y)
                              dots <- list(...)
                              if (grid) panel.grid()
                              panel.xyplot(x, y, ...)
                              if (any(ids <- id[subscripts])){
                                  ltext(x[ids], y[ids], idLabels[subscripts][ids],
                                        cex = dots$cex, adj = dots$adj)
                              }
                              if (!is.null(abl)) {
                                  if (length(abl) == 2) panel.abline(a = abl, ...) else panel.abline(h = abl, ...)
                              }
                          }))
            }
        } else {                                # assume factor or character
            plotFun <- "bwplot"
            if (is.null(args$panel)) {
                args <- c(args,
                          panel = list(function(x, y, ...)
                          {
                              if (grid) panel.grid()
                              panel.bwplot(x, y, ...)
                              if (!is.null(abl)) {
                                  panel.abline(v = abl[1], ...)
                              }
                          }))
            }
        }
    } else {
        plotFun <- "histogram"
        if (is.null(args$panel)) {
            args <- c(args,
                      panel = list(function(x, ...)
                      {
                          if (grid) panel.grid()
                          panel.histogram(x, ...)
                          if (!is.null(abl)) {
                              panel.abline(v = abl[1], ...)
                          }
                      }))
        }
    }

    ## defining grid
    if (missing(grid)) {
        grid <- (plotFun == "xyplot")
    }
                                        # assign("grid", grid, where = 1)
    do.call(plotFun, as.list(args))
}

## no longer defining `fortify` S3 generic

##' @rdname fortify
##' @S3method fortify lmerMod
##' @method fortify lmerMod
##' @export
##'   as function, not as S3 method, see ../man/fortify.Rd  :
fortify.merMod <- function(model, data=getData(model), ...) {

    ## FIXME: get influence measures via influence.ME?
    ##   (expensive, induces dependency ...)
    ## FIXME: different kinds of residuals?
    ## FIXME: deal with na.omit/predict etc.
    data$.fitted <- predict(model)
    data$.resid <- resid(model)
    data$.scresid <- resid(model,type="pearson",scaled=TRUE)
    data
}


## autoplot???

##  plot method for plot.summary.mer ... coefplot-style
##  horizontal, vertical? other options???
##  scale?
plot.summary.mer <- function(object, type="fixef", ...) {
    if(any(!type %in% c("fixef","vcov")))
        stop("'type' not yet implemented: ", type)
    stop("FIXME -- not yet implemented")

}

## TO DO: allow faceting formula
## TO DO: allow qqline to be optional
## TO DO (harder): steal machinery from qq.gam for better GLMM Q-Q plots
qqmath.merMod <- function(x, data = NULL, id=NULL, idLabels=NULL, ...) {
    ## klugey attempt to detect whether user forgot to specify argument
    ## names explicitly (after addition of required 'data' argument)
    ## NOT completely tested!
    if (!is.null(data)) {
        idLabels <- id
        id <- data
        warning("qqmath.merMod takes ", sQuote("data"), "as its ",
                "first argument for S3 method compatibility: ",
                "in the future, please ",
                "specify the ", sQuote("id"), " and ",
                sQuote("idLabels"), " arguments explicitly ",
                "i.e. ",
                sQuote("qqmath(fitted_model, id = ..., [idLabels = ...], ...)"))
    }
   
    values <- residuals(x, type="pearson", scaled=TRUE)
    data <- getData(x)
    ## DRY: copied from plot.merMod, should modularize/refactor
    if (!is.null(id)) {       ## identify points in plot
        id <- switch(mode(id),
                     numeric = {
                         if (id <= 0 || id >= 1)
                             stop(shQuote("id")," must be between 0 and 1")
                         as.logical(abs(values) > -qnorm(id / 2))
                     },
                     call = eval(asOneSidedFormula(id)[[2]], data),
                     stop(shQuote("id")," can only be a formula or numeric.")
                     )
        if (is.null(idLabels)) {
            idLabels <- getIDLabels(x)
        } else {
            if (inherits(idLabels,"formula")) {
                idLabels <- getIDLabels(x,idLabels)
            } else if (is.vector(idLabels)) {
                if (length(idLabels <- unlist(idLabels)) != length(id)) {
                    stop("\"idLabels\" of incorrect length")
                }
            } else stop("\"idLabels\" can only be a formula or a vector")
        }
        idLabels <- as.character(idLabels)

    }
    ## DON'T subscript by id, will be done later
    qqpanel <- function(x, subscripts, ...) {
        dots <- list(...)
        panel.qqmathline(x, ...)
        panel.qqmath(x, ...)
        if (any(ids <- id[subscripts])) {
            xs <- x[subscripts]
            pp <- setNames(ppoints(length(xs)),
                           names(sort(xs)))
            ## want to plot qnorm(pp) vs sort(x)
            ## ... but want to pick out the elements that corresponded
            ## to ids **before** sorting
            xx <- qnorm(pp)[names(xs)[ids]]
            yy <- sort(x)[names(xs)][ids] ## quantile(values, pp)[ids]
            ltext(xx,
                  yy,
                  idLabels[ids],
                  cex = dots$cex, adj = dots$adj)
        }
    }
    qqmath(values, xlab = "Standard normal quantiles",
           ylab = "Standardized residuals",
           prepanel = prepanel.qqmathline,
           panel = qqpanel,
           ...)
}

## qqmath(~residuals(gm1)|cbpp$herd)

## this is the simpler version of the code for testing/exercising
## https://github.com/lme4/lme4/issues/35
## see also ../misc/issues/dynload.R for more complexity
pkg <- so_name <- "lme4"; doUnload <- FALSE; doTest <- TRUE
## pkg <- so_name <- "RcppEigen"; doUnload <- TRUE; doTest <- TRUE
## need to deal with the fact that DLL name != package name for lme4.0 ...
### pkg <- "lme4.0"; so_name <- "lme4"; doUnload <- TRUE
instPkgs <- as.data.frame(installed.packages(),stringsAsFactors=FALSE)
Load <- function() {
    library(pkg,character.only=TRUE)
}
Unload <- function() {
    ld <- library.dynam()
    pnames <- sapply(ld,"[[","name")
    names(ld) <- pnames
    lp <- gsub("/libs/.*$","",ld[[so_name]][["path"]])
    cat("unloading from",lp,"\n")
    library.dynam.unload(so_name, lp)
}
Detach <- function() {
    detach(paste0("package:",pkg),character.only=TRUE,unload=TRUE)
    if (doUnload) Unload()
}
tmpf <- function() {
    g <- getLoadedDLLs()
    lnames <- names(g)[is.na(instPkgs[names(g),"Priority"])]
    cat("loaded DLLs:",lnames,"\n")
    g <- g[na.omit(match(c(so_name,"nlme"),names(g)))]
    class(g) <- "DLLInfoList"
    g
}
test <- function() {
    if (doTest) {
        if (pkg %in% c("lme4","lme4.0")) {
            fm1 <- lmer(Reaction ~ Days + (Days|Subject), sleepstudy,
                        devFunOnly=TRUE)
        }
        if (pkg=="RcppEigen") {
            data(trees, package="datasets")
            mm <- cbind(1, log(trees$Girth))   # model matrix
            y  <- log(trees$Volume)            # response
            ## bare-bones direct interface
            flm <- fastLmPure(mm, y)
        }
    }
}
if (FALSE) {
  ## FIXME: disabled test for now
for (i in 1:6) {
    cat("Attempt #",i,"\n",sep="")
    cat("loading",pkg,"\n")
    Load()
    tmpf()
    test()
    cat("detaching",pkg,"\n")
    Detach()
    cat("loading nlme\n")
    library("nlme")
    tmpf()
    detach("package:nlme",unload=TRUE)
    cat("detaching nlme\n")
}
}

library(lme4)
## show important current settings {for reference, etc} -- [early, and also on Windows !]:

source(system.file("test-tools-1.R", package = "Matrix"), keep.source = FALSE)
## N.B. is.all.equal[34]() and assert.EQ() use 'tol', not 'tolerance'

str( lmerControl())
str(glmerControl())
str(nlmerControl())
ls.str(environment(nloptwrap))


## see Details under ?deviance.merMod:

fm1 <- lmer(Reaction ~ Days + (Days|Subject), sleepstudy)
fm1ML <- refitML(fm1)
REMLcrit(fm1)   ## 1743.628
deviance(fm1ML) ## 1751.939
deviance(fm1,REML=FALSE)  ## FIXME: not working yet (NA)
deviance(fm1,REML=TRUE)

## from lme4.0
oldvals <- c(REML=1743.6282722424, ML=1751.98581103058)
## leave out ML values for REML fits for now ...
stopifnot(
          is.all.equal3(REMLcrit(fm1), deviance(fm1,REML=TRUE), deviance(fm1ML,REML=TRUE),oldvals["REML"]),
          all.equal(deviance(fm1ML),deviance(fm1ML,REML=FALSE),oldvals["ML"]),
          all.equal(REMLcrit(fm1)/-2,c(logLik(fm1)),c(logLik(fm1ML,REML=TRUE)),c(logLik(fm1,REML=TRUE))),
          all.equal(deviance(fm1ML)/-2,c(logLik(fm1ML,REML=FALSE)),
                    c(logLik(fm1ML,REML=FALSE))))

## should be:
## stopifnot(
##           all.equal(deviance(fm1),deviance(fm1,REML=TRUE),deviance(fm1ML,REML=TRUE),oldvals["REML"]),
##           all.equal(deviance(fm1ML),deviance(fm1,REML=FALSE),deviance(fm1ML,REML=FALSE),oldvals["ML"]),
##           all.equal(deviance(fm1)/2,c(logLik(fm1)),c(logLik(fm1ML,REML=TRUE)),c(logLik(fm1,REML=TRUE))),
##           all.equal(deviance(fm1ML)/2,c(logLik(fm1,REML=FALSE)),c(logLik(fm1ML,REML=FALSE)),
##                     c(logLik(fm1ML,REML=FALSE))))

if (base::require("testthat", quietly = TRUE)) {
    pkg   <- "lme4"
    require(pkg, character.only=TRUE, quietly=TRUE)
    if(getRversion() < "3.5.0") { withAutoprint <- identity ; prt <- print } else { prt <- identity }
    if(Sys.getenv("USER") %in% c("maechler", "bbolker")) withAutoprint({
        ## for developers' sake:
        lP <- .libPaths() # ---- .libPaths() : ----
        prt(lP)
        ## ---- Entries in .libPaths()[1] : ----
        prt(list.files(lP[1], include.dirs=TRUE))
        prt(sessionInfo())
        prt(packageDescription("Matrix"))
        ## 'lme4' from packageDescription "file" :
        prt(attr(packageDescription("lme4"), "file"))
    })
    test_check(pkg)
    ##======== ^^^
    print(warnings()) # TODO? catch most of these by expect_warning(..)
} else {
    cat( "package 'testthat' not available, cannot run unit tests\n" )
}

#### example originally from Gabor Grothendieck

source(system.file("testdata/lme-tst-funs.R", package="lme4", mustWork=TRUE))
##--> rSim.11()
testLevel <- if (nzchar(s <- Sys.getenv("LME4_TEST_LEVEL"))) as.numeric(s) else 1

if (testLevel>1) {
set.seed(1)
d1 <- rSim.11(10000, k=4)
library(nlme)
m.lme <- lme(y ~ x, random=~ 1|fac , data=d1)
(VC.lme <- VarCorr(m.lme))
detach("package:nlme")
##
library(lme4)
fm.NM <- lmer(y ~ x + (1|fac), data=d1, control=lmerControl("Nelder_Mead"))
fm.Bq <- update(fm.NM, control=lmerControl("bobyqa"))
v.lmer <- VarCorr(fm.NM)[[1]][1,1]
stopifnot(all.equal(v.lmer,19.55,tolerance=1e-3))
## was 19.5482 with old starting values (1), 19.5493 with new start algorithm
} ## testLevel>1

## library(lme4.0)
## Emacs M-<Enter> --> setwd() correctly

## m0.0 <- glm(colonizers~Treatment*homespecies*respspecies, data=randdat, family=poisson)
## with(randdat,tapply(colonizers,list(Treatment,homespecies,respspecies),sum))
## summary(m1.0 <- glmer(form1, data=randdat, family=poisson))
## summary(m2.0 <- glmer(form2, data=randdat, family=poisson))


## detach("package:lme4.0", unload=TRUE)

load(system.file("testdata","colonizer_rand.rda",package="lme4"))
library("lme4")
packageVersion("lme4")

if (.Platform$OS.type != "windows") {
m1 <- glmer(form1,data=randdat, family=poisson)  ## PIRLS step failed
m2 <- glmer(form1,data=randdat, family=poisson, nAGQ=0)  ## OK
m3 <- glmer(form2,data=randdat, family=poisson)  ## ditto

} ## skip on windows (for speed)

### nlmer() convergence testing / monitoring / ...
##  -------------------

### The output of tests here are *not* 'diff'ed  (<==> no *.Rout.save file)
library(lme4)

## 'Theoph' Data modeling

if (lme4:::testLevel() > 1) {
    Th.start <- c(lKe=-2.5, lKa=0.5, lCl=-3)

    (nm2 <- nlmer(conc ~ SSfol(Dose, Time,lKe, lKa, lCl) ~ lKe + lKa + lCl|Subject,
                  Theoph, start = Th.start))
    (nm3 <- nlmer(conc ~ SSfol(Dose, Time,lKe, lKa, lCl) ~ (lKe|Subject) +
                                                           (lKa|Subject) + (lCl|Subject),
                  Theoph, start = Th.start))
    ## dropping   lKe  from random effects:
    (nm4 <- nlmer(conc ~ SSfol(Dose, Time,lKe, lKa, lCl) ~ lKa + lCl|Subject,
                  Theoph, start = Th.start,
                  control = nlmerControl(tolPwrss=1e-8)))
    (nm5 <- nlmer(conc ~ SSfol(Dose, Time,lKe, lKa, lCl) ~ (lKa|Subject) + (lCl|Subject),
                  Theoph, start = Th.start))
}

library("lme4")
cbpp2 <- read.csv("cbpp2.csv")
source("glmer_batch_funs.R")

cbpp2 <- transform(
  cbpp2,
  period = factor(period),
  treatment = factor(
    treatment,
    levels = c("Partial/null", "Complete", "Unknown")
  ),
  obs = factor(seq(nrow(cbpp2))),
  avg_size = avg_size / sd(avg_size, na.rm = TRUE)
)

cbpp_df_name <- data.frame(
  mnames = c("herd", "herdobs", "obs"),
  mdesc = c("herd", "herdobs", "obs"))

mforms <- list()
mforms$herd <- incidence / size ~ period + treatment + avg_size + (1 | herd)
mforms$herdobs <- update(mforms$herd, . ~ . + (1 | obs))
mforms$obs <- update(mforms$herd, . ~ . - (1|herd) + (1 | obs))

## don't use lapply because of parent.frame() eval nonsense -- check back once GH#961 is merged?
## this fails:
##   mod_list <- lapply(mforms, glmer, data = cbpp2, family = binomial, weights = size)
## when evaluating 'weights'

cc0 <- glmerControl(optimizer = "bobyqa",
                    ## avoid instability/problems with nAGQ=0 step ...
                    nAGQ0initStep = FALSE,
                    optCtrl = list(maxfun = 1e5))
cc1 <- cc0
cc1$calc.derivs <- FALSE

cbpp_mod_list <- list()
for (i in names(mforms)) {
  cbpp_mod_list[[i]] <- glmer(mforms[[i]],
                              data = cbpp2,
                              family = binomial,
                              weights = size,
                              control = cc0)
}

## turn off derivs for bootstrap CIs
## (need derivs for hacked Wald CIs on REs)
cbpp_mod_list_noderivs <- lapply(cbpp_mod_list,
                                 function(x) update(x, control = cc1))

## b_cifun(cbpp_mod_list2[[2]], nsim = 5)
cbpp_confint_boot <- lapply(cbpp_mod_list_noderivs,
                            b_cifun, nsim = 501)
cbpp_prof <- lapply(cbpp_mod_list, p_fun)

cbpp_confint_prof <- lapply(cbpp_prof, confint)
cbpp_confint_wald <- lapply(cbpp_mod_list, wald_cifun)
cbpp_est <- do.call("rbind", Map(get_est, cbpp_mod_list, cbpp_df_name$mnames))
rownames(cbpp_est) <- NULL

cbpp_combCI <- Map(combfun2,
                   list(cbpp_confint_wald, cbpp_confint_boot, cbpp_confint_prof),
                   c("Wald", "boot", "profile"),
                   MoreArgs = list(df_est = cbpp_est, df_name = cbpp_df_name)) |>
  do.call(what = "rbind")

if (FALSE) {
  
  library(ggplot2); theme_set(theme_bw())
  revguide <- guide_legend(reverse = TRUE)
  des_ord <- c("sd_(Intercept)|herd", "sd_(Intercept)|obs",
               "avg_size", "treatmentUnknown", "treatmentComplete")
  combCI <- subset(cbpp_combCI,var!="(Intercept)" & !grepl("^period",var))
  combCI <- transform(combCI,  var = factor(var, levels = rev(des_ord)))
  combCI <- transform(combCI,
                      vartype = factor(ifelse(grepl("^sd", var), "random effects", "fixed effects")))
  mnames <- cbpp_df_name$mnames
  pd <- position_dodge(width=0.6)
  ggplot(combCI, aes(var, est, colour=type, shape=model)) +
    geom_pointrange(aes(ymin=lwr, ymax=upr),
                    position=pd) +
    labs(x="", y="Estimate (log-odds of seropositivity)") +
    geom_hline(yintercept=0, lty=2) +
    coord_flip() +
    scale_colour_brewer(palette="Dark2", guide = revguide) +
    scale_shape(guide=revguide, labels = mnames) +
    facet_wrap(~ vartype, ncol = 1, scale = "free")

  ggsave("tmp.png")
}

objs <- c("cbpp_confint_prof", "cbpp_confint_boot", "cbpp_confint_wald",
          ## "cbpp_prof",
          "cbpp_est", "cbpp_combCI", "cbpp_df_name")


save(
  list = objs,
  file = "cbpp_batch.rda",
  version = 2
)
sessionInfo()

if (.Platform$OS.type != "windows") {
    ## see if we can still run lme4 functions when lme4 is not attached
    if ("package:lme4" %in% search()) detach("package:lme4")
    data(sleepstudy,package="lme4")
    data(cbpp,package="lme4")
    fm1 <- lme4::lmer(Reaction ~ Days + (Days|Subject), sleepstudy)
    gm1 <- lme4::glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
                       data = cbpp, family = binomial)
} ## skip on windows (for speed)

library(glmmTMB)

# Model 1; taken from github.com/lme4/lme4/issues/962
set.seed(101)
form <- out_bin_1 ~ ftime + trt + ar1(ftime + 0 | id_cluster) +
  (1 | id_individual)
sfun <- function(n) factor(sample(1:n, replace=TRUE, size = 1465))
dd <- data.frame(id_cluster = sfun(7), id_individual = sfun(486),
                 ftime = sfun(4), trt = sfun(2))
np <- list(beta = c(0, rep(0.1, 3), 1), theta = rep(0, 3))
dd$out_bin_1 <- simulate_new(form[-2], newdata = dd,
                             newparams = np,
                             family = binomial)[[1]]

dd$out_gauss <- simulate_new(form[-2], newdata = dd,
                             newparams = c(np, list(betadisp=0)),
                             family = gaussian)[[1]]

glmmTMB_1 <- glmmTMB(form, data = dd, family = binomial)

## store just results
glmmTMB_1_ref <- list(
  fixef_cond = unname(fixef(glmmTMB_1)$cond),
  sigma = unname(sigma(glmmTMB_1)),
  logLik = unname(c(logLik(glmmTMB_1))),
  AIC = unname(AIC(glmmTMB_1)),
  BIC = unname(BIC(glmmTMB_1)),
  nobs = nobs(glmmTMB_1),
  glmmTMB_version = packageVersion("glmmTMB")
)

glmmTMB_glmer_ar1_comparison <- list(
  glmer_ar1_dd = dd,
  glmmTMB_ar1_binomial = glmmTMB_1_ref
)

saveRDS(glmmTMB_glmer_ar1_comparison, "inst/testdata/glmmTMB_glmer_ar1_comparison.rds", version = 2)

##' @name pvalues
##' @aliases mcmcsamp
##' @title Getting p-values for fitted models
##' 
##' @description One of the most frequently asked questions about \code{lme4}
##' is "how do I calculate p-values for estimated parameters?"
##' Previous versions of \code{lme4} provided the \code{mcmcsamp}
##' function, which efficiently generated a Markov chain Monte Carlo sample
##' from the posterior distribution of the parameters, assuming
##' flat (scaled likelihood) priors. Due to difficulty in
##' constructing a version of \code{mcmcsamp} that was reliable
##' even in cases where the estimated random effect variances were near
##' zero (e.g. \url{https://stat.ethz.ch/pipermail/r-sig-mixed-models/2009q4/003115.html}), \code{mcmcsamp} has been withdrawn (or more precisely,
##' not updated to work with \code{lme4} versions >=1.0.0).
##'
##' Many users, including users of the \code{aovlmer.fnc} function
##' from the \code{languageR} package which relies on \code{mcmcsamp},
##' will be deeply disappointed by this lacuna. Users who need p-values have
##' a variety of options:
##' \itemize{
##' \item likelihood ratio tests via \code{anova} (MC,+)
##' \item profile confidence intervals via \code{\link{profile.merMod}} and \code{\link{confint.merMod}} (CI,+)
##' \item parametric bootstrap confidence intervals and model comparisons via \code{\link{bootMer}} (or \code{PBmodcomp} in the \code{pbkrtest} package) (MC/CI,*,+)
##' \item for random effects, simulation tests via the \code{RLRsim} package (MC,*)
##' \item for fixed effects, F tests via Kenward-Roger approximation using \code{KRmodcomp} from the \code{pbkrtest} package (MC)
##' \item \code{car::Anova} and \code{lmerTest::anova} provide wrappers for \code{pbkrtest}. \code{lmerTest::anova} also provides t tests via the Satterthwaite approximation (P,*)
##' }
##' In the list above, the methods marked \code{MC} provide explicit model comparisons; \code{CI} denotes confidence intervals; and \code{P} denotes parameter-level or sequential tests of all effects in a model. The starred (*) suggestions provide finite-size corrections (important when the number of groups is <50); those marked (+) support GLMMs as well as LMMs.
##' 
##' When all else fails, don't forget to keep p-values in perspective: \url{http://www.phdcomics.com/comics/archive.php?comicid=905}
##' 
if(FALSE) ## C++ code in ../src/mcmcsamp.cpp -- is also  #ifdef 0
# @S3method mcmcsamp merMod
mcmcsamp.merMod <- function(object, n=1L, verbose=FALSE, saveb=FALSE, ...) {
    n <- max(1L, as.integer(n)[1])
    dd <- getME(object, "devcomp")$dims
    ranef <- matrix(numeric(0), nrow = dd[["q"]], ncol = 0)
    if (saveb) ranef <- matrix(, nrow = dd[["q"]], ncol = n)
    sigma <- matrix(unname(sigma(object)), nrow = 1,
                    ncol = (if (dd[["useSc"]]) n else 0))
    ff <- fixef(object)
    fixef <- matrix(ff, nrow=dd[["p"]], ncol=n)
    rownames(fixef) <- names(ff)
    ## FIXME create a copy of the resp and pred modules
    ans <- new("merMCMC",
               Gp = object@Gp,
 #              ST = matrix(.Call(mer_ST_getPars, object), dd[["np"]], n),
               call = object@call,
               dims = object@dims,
               deviance = rep.int(unname(object@deviance[["ML"]]), n),
               fixef = fixef,
               nc = sapply(object@ST, nrow),
               ranef = ranef,
               sigma = sigma)
    .Call(mer_MCMCsamp, ans, object)
}

stopifnot(suppressPackageStartupMessages(require(lme4)))

## Using simple generated data -- fully balanced here, unbalanced later
set.seed(1)
dat <- within(data.frame(lagoon = factor(rep(1:4, each = 25)),
                         habitat = factor(rep(1:20, each = 5))),
          {   ## a simple  lagoon effect but no random effect
              y <- round(10*rnorm(100, m = 10*as.numeric(lagoon)))
              ## Here, *with* an RE, sigma_a = 100
              RE <- rep(round(rnorm(nlevels(habitat), sd = 100)), each = 5)
              y2 <- y + RE
          })

## FIXME:   want  lmer(* , sparseX = TRUE )  {as in lme4a}
if (FALSE) {                            # need to adapt to new structure

##' <description>
##'
##' <details>
##' @title Comparing the different versions of lmer() for same data & model
##' @param form
##' @param data
##' @param verbose
##' @return
chkLmers <- function(form, data, verbose = FALSE,
                     tol = 200e-7) # had tol = 7e-7 working ..
{
#    m   <- lmer1(form, data = data)  # ok, and more clear
#    m.  <- lmer1(form, data = data, sparseX = TRUE, verbose = verbose)
    m2  <- lmer (form, data = data, verbose = verbose) # lmem-dense
    m2. <- lmer (form, data = data, sparseX = TRUE, verbose = verbose)
    ##
    Eq <- function(x,y) all.equal(x,y, tolerance = tol)
    stopifnot(## Compare  sparse & dense of the new class results
              identical(slotNames(m2), slotNames(m2.))
              ,
              identical(slotNames(m2@fe), slotNames(m2.@fe))
              ,
              Eq(m2@resp, m2.@resp)
              ,
              Eq(m2@re, m2.@re)
              ,
              Eq(m2@fe@coef, m2.@fe@coef)
              ,
              ## and now compare with the "old" (class 'mer')
#              Eq(unname(fixef(m)), m2@fe@beta)
#              ,
#              Eq(unname(fixef(m.)), m2.@fe@beta)
#              ,
              ## to do
              ## all.equal(ranef(m)), m2@re)
              ## all.equal(ranef(m.)), m2.@re)
              TRUE)
    invisible(list(#m=m, m.=m.,
                   m2 = m2, m2. = m2.))
}

chk1 <- chkLmers(y  ~ 0+lagoon + (1|habitat), data = dat, verbose = TRUE)
chk2 <- chkLmers(y2 ~ 0+lagoon + (1|habitat), data = dat, verbose = TRUE)
chk1$m2  ## show( lmer() ) -- sigma_a == 0
chk2$m2. ## show( lmer( <sparseX>) ) --

n <- nrow(dat)
for(i in 1:20) {
    iOut <- sort(sample(n, 1+rpois(1, 3), replace=FALSE))
    cat(i,":  w/o ", paste(iOut, collapse=", ")," ")
    chkLmers(y  ~ 0+lagoon + (1|habitat), data = dat[- iOut,])
    chkLmers(y2 ~   lagoon + (1|habitat), data = dat[- iOut,])
    cat("\n")
}

## One (rare) example where the default tolerance is not sufficient:
dat. <- dat[- c(14, 34, 66, 67, 71, 88),]
try( chkLmers(y ~ 0+lagoon + (1|habitat), data = dat.) )
## Error: Eq(unname(fixef(m)), m2@fe@beta) is not TRUE
##
## but higher tolerance works:
chkLmers(y ~ 0+lagoon + (1|habitat), data = dat., tol = 2e-4, verbose=TRUE)

}
proc.time()
sessionInfo()

#### Testing  refit()
#### ----------------

library(lme4)
set.seed(101)
testLevel <- if (nzchar(s <- Sys.getenv("LME4_TEST_LEVEL"))) as.numeric(s) else 1

## for each type of model, should be able to
##  (1) refit with same data and get the same answer,
##     at least structurally (small numerical differences
##     are probably unavoidable)
##  (2) refit with simulate()d data

if (testLevel>1) {
getinfo <- function(x) {
  c(fixef(x), logLik(x), unlist(ranef(x)), unlist(VarCorr(x)))
}

dropterms <- function(x) {
    attr(x@frame,"terms") <- NULL
    x
}

if (getRversion() >= "3.0.0") {
    attach(system.file("testdata", "lme-tst-fits.rda", package="lme4"))
} else {
    ## saved fits are not safe with old R versions; just re-compute ("cheat"!):

    fit_sleepstudy_2 <- lmer(Reaction ~ Days + (Days|Subject), sleepstudy)

    cbpp$obs <- factor(seq(nrow(cbpp)))
    ## intercept-only fixed effect
    fit_cbpp_0 <- glmer(cbind(incidence, size-incidence) ~ 1 + (1|herd),
                        cbpp, family=binomial)
    ## include fixed effect of period
    fit_cbpp_1 <- update(fit_cbpp_0, . ~ . + period)
    if(FALSE) ## include observation-level RE
        fit_cbpp_2 <- update(fit_cbpp_1, . ~ . + (1|obs))
    ## specify formula by proportion/weights instead
    fit_cbpp_3 <- update(fit_cbpp_1, incidence/size ~ period + (1 | herd), weights = size)
}


## LMM
fm1 <- fit_sleepstudy_2
fm1R <- refit(fm1, sleepstudy$Reaction)
fm1S <- refit(fm1, simulate(fm1)[[1]])

stopifnot(all.equal(getinfo(fm1 ),
                    getinfo(fm1R), tolerance = 6e-3),
          all.equal(getinfo(fm1 ),
                    getinfo(fm1S), tolerance = 0.5) # <- simulate()
          )

if(FALSE) { ## show all differences
    sapply(slotNames(fm1), function(.)
        all.equal( slot(fm1,.), slot(fm1R,.), tolerance=0))
}

if (getRversion() >= "3.4.0") {
    ## differences: FALSE for resp, theta, u, devcomp, pp, optinfo?
    ## FIXME: this isn't actually tested in any way ...
    sapply(slotNames(fm1),
           function(.) isTRUE(all.equal( slot(fm1,.), slot(fm1R,.), tolerance= 1.5e-5)))
    str(fm1 @ optinfo)
    str(fm1R@ optinfo)
}

fm1ML <- refitML(fm1)
stopifnot(
    all.equal(getinfo(fm1), getinfo(fm1ML), tolerance=0.05)# 0.029998
)


## binomial GLMM (two-column)
gm1 <- fit_cbpp_1
gm1R <- refit(gm1, with(cbpp, cbind(incidence,size-incidence)))

 sim1Z <- simulate(gm1)[[1]]
 sim1Z[4,] <- c(0,0)
 (gm1. <- refit(gm1, sim1Z)) # earlier gave Error:  ... PIRLS ... failed ...

all.equal(getinfo(gm1), getinfo(gm1R), tolerance=0) # to see it --> 5.52e-4
# because glmer() uses Laplace approx. (? -- still, have *same* y !)
stopifnot(all.equal(getinfo(gm1), getinfo(gm1R), tolerance = 1e-4))

gm1S <- refit(gm1, simulate(gm1)[[1]])
          all.equal(getinfo(gm1), getinfo(gm1S), tolerance=0) # to see:
stopifnot(all.equal(getinfo(gm1), getinfo(gm1S), tolerance = 0.4))

## binomial GLMM (prob/weights)
formula(gm2 <- fit_cbpp_3)
## glmer(incidence/size ~ period + (1 | herd), cbpp, binomial, weights=size)
gm2R <- refit(gm2,  with(cbpp, incidence/size))
          all.equal(getinfo(gm2), getinfo(gm2R), tolerance= 0)
stopifnot(all.equal(getinfo(gm2), getinfo(gm2R), tolerance= 6e-4))

## FIXME: check on Windows == 2015-06: be brave
gm2S <- refit(gm2, simulate(gm2)[[1]])
          all.equal(getinfo(gm2), getinfo(gm2S), tolerance=0)# 0.17 .. upto 0.28
stopifnot(all.equal(getinfo(gm2), getinfo(gm2S), tolerance=0.40))

## from Alexandra Kuznetsova
set.seed(101)
Y <- matrix(rnorm(1000),ncol=2)
d <- data.frame(y1=Y[,1],  x=rnorm(100), f=rep(1:10,10))
fit1 <- lmer(y1 ~ x+(1|f),data=d)
fit2 <- refit(fit1, newresp = Y[,2], rename.response=TRUE)
## check, but ignore terms attribute of model frame ...
tools::assertWarning(refit(fit1, newresp = Y[,2], junk=TRUE))
if (isTRUE(all.equal(fit1,fit2))) stop("fit1 and fit2 should not be equal")
## hack number of function evaluations
u2 <- update(fit2)
fit2@optinfo$feval <- u2@optinfo$feval <-  NA

d1 <- dropterms(fit2)
d2 <- dropterms( u2 )
## They are not "all equal", but mostly :
for (i in slotNames(d1)) {
    ae <- all.equal(slot(d1,i), slot(d2,i))
    cat(sprintf("%10s: %s\n", i, if(isTRUE(ae)) "all.equal"
		else paste(ae, collapse="\n  ")))
}
          all.equal(getinfo(d1), getinfo(d2),  tolerance = 0)# -> 0.00126
stopifnot(all.equal(getinfo(d1), getinfo(d2),  tolerance = 0.005))


## Bernoulli GLMM (specified as factor)
if (requireNamespace("mlmRev")) {
    data(Contraception, package="mlmRev")
    gm3 <- glmer(use ~ urban + age + livch + (1|district),
                 Contraception, binomial)
    gm3R <- refit(gm3, Contraception$use)
    gm3S <- refit(gm3, simulate(gm3)[[1]])
    stopifnot(all.equal(getinfo(gm3 ),
                        getinfo(gm3R), tolerance = 1e-5),# 64b_Lx: 7.99e-7
              all.equal(getinfo(gm3 ),
                        getinfo(gm3S), tolerance = 0.05) # <- simulated data
              )
    cat("gm3: glmer(..):\n"        ); print(getinfo(gm3))
    cat("gm3R: refit(*, y):\n"     ); print(getinfo(gm3R))
    cat("gm3S: refit(*, sim.()):\n"); print(getinfo(gm3S))

    data(Mmmec, package="mlmRev")
    if (lme4:::testLevel() > 1) {
        gm4 <- glmer(deaths ~ uvb + (1|region), data=Mmmec,
                     family = poisson,
                     offset = log(expected))
        ## FIXME: Fails to converge (with larger maxit: "downdate .. not pos.def..")
        try( gm4R <- refit(gm4, Mmmec $ deaths) )
        try( gm4S <- refit(gm4, simulate(gm4)[[1]]) )
        if(FALSE) { ## FIXME (above)
            cat("gm4R: refit(*,y):\n" ); print( getinfo(gm4R) )
            cat("gm4S: refit(*,y):\n" ); print( getinfo(gm4S) )
            stopifnot(all.equal(getinfo(gm4),getinfo(gm4R),tolerance=6e-5))
        }
    }
}

## ----------------------------------------------------------------------
## issue: #231, http://ms.mcmaster.ca/~bolker/misc/boot_reset.html
## commits: 1a34cd0, e33d698, 53ce966, 7dbfff1, 73aa1bb, a693ba9, 8dc8cf0
## ----------------------------------------------------------------------

formGrouse <- TICKS ~ YEAR + scale(HEIGHT) + (1 | BROOD) + (1 | INDEX) + (1 | LOCATION)
gmGrouse <- glmer(formGrouse, family = "poisson", data = grouseticks)
set.seed(105)
simTICKS <- simulate(gmGrouse)[[1]]
newdata <- transform(grouseticks, TICKS = simTICKS)
gmGrouseUpdate <-  update(gmGrouse, data = newdata)
gmGrouseRefit  <-  refit(gmGrouse, newresp = simTICKS)

## compute and print tolerances
all.equal(bet.U <- fixef(gmGrouseUpdate),
          bet.R <- fixef(gmGrouseRefit), tolerance = 0)
all.equal(th.U <- getME(gmGrouseUpdate, "theta"),
          th.R <- getME(gmGrouseRefit,  "theta"), tolerance = 0)
all.equal(dev.U <- deviance(gmGrouseUpdate),
          dev.R <- deviance(gmGrouseRefit),	tolerance = 0)
stopifnot(
    all.equal(bet.U, bet.R, tolerance = 6e-5), # saw 1.0e-5
    all.equal( th.U,  th.R, tolerance = 4e-5), # saw 1.2e-5
    all.equal(dev.U, dev.R, tolerance = 2e-5)) # saw 4.6e-6
} ## testLevel>1

---
title: "Covariance Structures"
output: rmarkdown::html_vignette
bibliography: lmer.bib
vignette: >
  %\VignetteIndexEntry{Covariance Structures}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
  %\VignetteDepends{glmmTMB}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```


## Introduction

The current version of `lme4` offers four covariance classes/structures: `Covariance.us` (unstructured), `Covariance.diag` (diagonal), `Covariance.cs` (compound symmetry), and `Covariance.ar1` (autoregressive order 1). The syntax for use is somewhat similar to the `glmmTMB` package (see [covariance structures in glmmTMB](https://glmmtmb.github.io/glmmTMB/articles/covstruct.html)), although the results are slightly different.

The first half of this vignette provide a more detailed explanation of how the 
new machinery works, and the second half of the vignette will compare `lme4` 
and `glmmTMB` implementations and results.

### Background

For exact details of this structure, refer to the `lmer` [vignette](https://cran.r-project.org/package=lme4/vignettes/lmer.pdf) [@Bates_JSS]. We provide
a quick summary in this vignette.

In matrix notation, a linear mixed model can be represented as:
\[
\mathbf{y} = \mathbf{X} \boldsymbol \beta + \mathbf Z \mathbf{b} + \boldsymbol{\epsilon}
\]
Where $\mathbf{b}$ represents an unknown vector of random effects. The 
\code{Covariance} class helps define the structure of the variance-covariance 
matrix as specified as $\textrm{Var}(\mathbf{b})$. Typically, we denote the 
variance-covariance matrix as $\mathbf \Sigma$.

<!-- can't use \usepackage{bm}: https://stackoverflow.com/a/33226711/190277 -->
First, we create the relative co-factor $\mathbf \Lambda_{\mathbf \theta}$ which is a 
$q \times q$ block diagonal  matrix that depends on the variance-component 
parameter $\theta$. Let $\sigma$ be the scale parameter of the variance of a 
linear mixed model. In `lme4`, the variance-covariance matrix is constructed by:
\[
{\mathbf \Sigma}_{\mathbf{\theta}} = 
\sigma^2 {\mathbf \Lambda}_{\mathbf \theta} {\mathbf \Lambda}_{\mathbf \theta}^{\top},
\]

For generalized linear mixed models, $\mathbf \Lambda$ instead represents the *unscaled* Cholesky factor; that is,
the scaling term $\sigma^2$ is omitted from the equation above.

The major difference between the four covariance classes (`Covariance.us` (unstructured), `Covariance.diag` (diagonal), `Covariance.cs` (compound symmetry), and `Covariance.ar1` (autoregressive order 1)) is the construction of the the relative Cholesky factor $\mathbf \Lambda_{\mathbf \theta}$.

### Covariance Structures

Suppose there are $p$ number of random effect terms for a particular
grouping variable. The *unstructured covariance*, which is the default 
in `lme4`, of size $p \times p$ has the following form:
\[
\mathbf{\Sigma}
= \begin{bmatrix}
\sigma^{2}_{1} & \sigma_{12} & \dots & \sigma_{1p} \\
\sigma_{21} & \sigma^{2}_{2} & \dots & \sigma_{2p} \\
\vdots & \vdots & \ddots & \vdots \\
\sigma_{p1} & \sigma_{p2} & \dots & \sigma^{2}_{p} \\
\end{bmatrix}	
\]

The next three covariance structures can either be heterogeneous or homogeneous.
If we have a homogeneous covariance structure (`hom = TRUE`), then we assume 
$\sigma_{1} = \sigma_{2} = \dots = \sigma_{p}$.

The *diagonal covariance* has the following form:
\[
\mathbf{\Sigma}
= \begin{bmatrix}
\sigma^{2}_{1} & 0 & \dots & 0 \\
0 & \sigma^{2}_{2} & \dots & 0 \\
\vdots & \vdots & \ddots & \vdots \\
0 & 0 & \dots & \sigma^{2}_{p} \\
\end{bmatrix}	
\]
By default, we assume a heterogeneous diagonal covariance structure.

The *compound symmetric covariance* has the following form:
\[
\mathbf{\Sigma}
= \begin{bmatrix}
\sigma^{2}_{1}              & \sigma_{1}\sigma_{2}\rho  & \sigma_{1}\sigma_{3}\rho  & \dots  & \sigma_{1}\sigma_{p}\rho \\
\sigma_{2}\sigma_{1} \rho   & \sigma^{2}_{2}             & \sigma_{2}\sigma_{3}\rho  & \dots  & \sigma_{2}\sigma_{p}\rho \\
\sigma_{3}\sigma_{1} \rho   & \sigma_{3}\sigma_{2}\rho   & \sigma^{2}_{3}            & \dots  & \sigma_{3}\sigma_{p}\rho \\
\vdots                      & \vdots                     & \vdots                    & \ddots & \vdots \\
\sigma_{p}\sigma_{1} \rho   & \sigma_{p}\sigma_{2}\rho   & \sigma_{p}\sigma_{3}\rho  & \dots  & \sigma^{2}_{p} 
\end{bmatrix}	
\]
By default, we assume a heterogeneous compound symmetric covariance structure.

The *AR1 (auto-regressive order 1) covariance* has the following form:
\[
\mathbf{\Sigma}
= \begin{bmatrix}
\sigma^{2}_{1}                  & \sigma_{1}\sigma_{2} \rho      & \sigma_{1}\sigma_{3}\rho^{2}   & \dots  & \sigma_{1}\sigma_{p}\rho^{p-1} \\
\sigma_{2}\sigma_{1} \rho       & \sigma^{2}_{2}                 & \sigma_{2}\sigma_{3}\rho       & \dots  & \sigma_{2}\sigma_{p}\rho^{p-2} \\
\sigma_{3}\sigma_{1} \rho^{2}   & \sigma_{3}\sigma_{2}\rho       & \sigma^{2}_{3}                 & \dots  & \sigma_{3}\sigma_{p}\rho^{p-3} \\
\vdots                          & \vdots                         & \vdots                         & \ddots & \vdots \\
\sigma_{p}\sigma_{1} \rho^{p-1} & \sigma_{p}\sigma_{2}\rho^{p-2} & \sigma_{p}\sigma_{3}\rho^{p-3} & \dots  & \sigma^{2}_{p} 
\end{bmatrix}	
\]
Unlike the diagonal and compound symmetric structures, by default we assume a
homogeneous ar1 covariance structure.

### Construction of the Relative Co-factor

For the unstructured covariance matrix, `lme4` estimates the following parameters in `par`:
$\mathbf{\theta} = (\theta_{1}, \theta_{2}, \dots, \theta_{p(p+1)/2})$ to construct the relative
co-factor ${\mathbf \Lambda}_{\mathbf{\theta}}$ (this is the same as in previous versions):
\[
\Lambda_{\mathbf{\theta}} = \begin{bmatrix}
\theta_1 & 0 & 0 & \dots & 0 \\
\theta_2 & \theta_3 & 0 & \dots & 0 \\
\theta_4 & \theta_5 & \theta_6 & \dots & 0 \\
\vdots & \vdots & \vdots & \ddots & \vdots \\
\theta_{...} & \theta_{...} & \theta_{...} & \dots & \theta_{p(p+1)/2}
\end{bmatrix}
\]

The definition of the parameter vectors differs for the other covariance structures. In the diagonal covariance matrix 
case, $\mathbf{\theta}$ (or `par`) only contains the standard deviations. 
The relative co-factor ${\mathbf \Lambda}_{\mathbf{\theta}}$ is:
\[
{\mathbf Lambda}_{\mathbf{\theta}} = \begin{bmatrix}
\theta_1 & 0 & 0 & \dots & 0 \\
0 & \theta_2 & 0 & \dots & 0 \\
0 & 0 & \theta_3 & \dots & 0 \\
\vdots & \vdots & \vdots & \ddots & \vdots \\
0 & 0 & 0 & \dots & \theta_{p}
\end{bmatrix}
\]

For the compound symmetry covariance structure, the parameter 
vector `par` contains the $p$ standard deviations $(\sigma_1, \sigma_2, \ldots, \sigma_p)$ 
and the common correlation $\rho$. In contrast to `glmmTMB`, the correlation is estimated on its original scale (bounded between -1 and 1), rather than on an unconstrained, transformed scale.

The relative co-factor ${\mathbf \Lambda}_{\mathbf{\theta}}$ is a lower triangular $p \times p$ matrix. Consider the form:
\[
{\mathbf \Lambda}_{\mathbf{\theta}} = \begin{bmatrix}
\theta_{11} & 0 & 0 & \cdots & 0 \\
\theta_{21} & \theta_{22} & 0 & \cdots & 0 \\
\theta_{31} & \theta_{32} & \theta_{33} & \cdots & 0 \\
\vdots & \vdots & \vdots & \ddots & \vdots \\
\theta_{p1} & \theta_{p2} & \theta_{p3} & \cdots & \theta_{pp}
\end{bmatrix}
\]

Its elements $\theta_{ij}$ are constructed as follows. First, define the 
sequence $\{a_j\}$ recursively:
\[
a_1 = 0, \quad a_{j+1} = a_j + \frac{(1 - \rho \cdot a_j)^2}{1 - \rho^2 \cdot a_j} \quad \text{for } j = 1, \ldots, p-1
\]

Then the elements of ${\mathbf \Lambda}$ are given by:
\[
\theta_{ij} = \begin{cases}
 \sqrt{1 - \rho^2 a_j} & \text{if } i = j \text{ (diagonal)} \\[0.5em]
 \dfrac{\rho - \rho^2 a_j}{\sqrt{1 - \rho^2 a_j}} & \text{if } i > j \text{ (below diagonal)} \\[0.5em]
0 & \text{if } i < j \text{ (above diagonal)}
\end{cases}
\]
In the code, one can extract the values $\theta_{ij}$ via the `getTheta()` function 
of the `Covariance.cs` object, in which `theta` will be a vector in the column-wise 
elements of ${\mathbf \Lambda}$.
 
The setup is similar for the autoregressive order 1 (AR1) covariance structure. 
Again, the parameter vector contains the $p$ standard deviations 
$(\sigma_1, \sigma_2, \ldots, \sigma_p)$ and the autocorrelation parameter 
$\rho$. The relative co-factor ${\mathbf \Lambda}_{\mathbf{\theta}}$ is a lower 
triangular $p \times p$ matrix whose form is similar to the compound symmetric case.

The elements $\theta_{ij}$ are given by:
\[
\theta_{ij} = \begin{cases}
 \rho^{i-1} & \text{if } j = 1 \text{ (first column)} \\[0.5em]
 \rho^{i-j} \sqrt{1 - \rho^2} & \text{if } 1 < j \leq i \text{ (below diagonal)} \\[0.5em]
0 & \text{if } i < j \text{ (above diagonal)}
\end{cases}
\]
Again, these values can be extracted using `getTheta()` function 
of the `Covariance.ar1` object, in which `theta` will be still be a vector in 
the column-wise elements of ${\mathbf \Lambda}$.

### Extracting model components

This section illustrates how to extract
`par`, `theta`, `Lambda`, as described in the previous section, as well as
the variance covariance matrices of a model, from a `merMod` object.

We'll fit the standard `sleepstudy` example, except that we will use a
model with a compound symmetric covariance structure. Because this model has only two varying effects (intercept and slope with respect to day) per subject, and
 hence the covariance matrix is $2 \times 2$, there is no difference in overall model fit between the compound-symmetric and the unstructured covariance matrices. Howevever, the models are parameterized differently, so this example will highlight the differences between `par` and `theta`.

```{r cs-fit}
library(lme4)
fm1.cs <- lmer(Reaction ~ Days + cs(1 + Days | Subject), sleepstudy)
```

Extracting the covariance structure:

```{r getReCovs}
print(fm1.cs_cov <- getReCovs(fm1.cs))
```

The result is a list with only one element as we only have one random-effects term (`cs(1 + Days | Subject)`). To 
see the values of `par` and `theta` for this object, we can call:

```{r getInfo}
getME(fm1.cs, "par")
getME(fm1.cs, "theta")
```

The ${\mathbf \Lambda}$ matrix is large, so we'll view it instead of printing:
```{r lambda-mat}
library(Matrix)
image(getME(fm1.cs, "Lambda"))
```

To most users, the most crucial information is simply the variance-covariance matrices. Extracte these via `VarCorr.merMod()` (the list has one element per random-effect term in the model --- in this case, only one):

```{r varcorr}
vc_mat <- VarCorr(fm1.cs)
vc_mat$Subject
```

## Comparisons with glmmTMB

### Reason for Computational Differences

When fitting linear mixed models, `lme4`  parameterizes the random-effects variance–covariance matrix on an unconstrained scale, using box-constrained optimization algorithms to ensure that the variance-covariance matrix is positive semidefinite. For unstructured covariance matrices, this means that the elements of $\mathbf \theta$ that parameterize the diagonal elements of $\mathbf \Lambda$ are constrained to be $\ge 0$ (for diagonal models, all of the elements of $\mathbf \theta$ fill the diagonal of $\mathbf \Lambda$ and hence are $\ge 0$; for models such as the compound-symmetric or AR1 models that use correlation parameters, we constrain $|\rho| \le 1$. As discussed in @Bates_JSS, this constrained parameterization works well for handling model where the 
estimated covariance matrix is *singular* (i.e. $\mathbf \Sigma$ is only positive semidefinite, not positive definite). In addition, for linear mixed models `lme4` profiles the fixed-effect parameters out of the objective function [@Bates_JSS]; finally, the scale parameter $\sigma$ is not estimated directly, but is derived from the residual variance or deviance of the fitted model.

In contrast, `glmmTMB` uses direct maximum likelihood estimation via Template Model Builder (TMB), fitting to the full parameter vector $\{\mathbf \theta, \mathbf \beta, \sigma^2\}$. Covariance parameters are fitted on a transformed (unconstrained) scale: log scale for standard deviations and various scales for correlation parameters (see the `glmmTMB` [covariance structures vignette](https://cran.r-project.org/package=glmmTMB/vignettes/covstruct.html) for details). This parameterization simplifies fitting (a box-constrained algorithm isn't necessary), but is less convenient in singular fits and other cases where the maximum likelihood estimate is infinite on the unconstrained scale.

Despite these differences, we will show examples where `lme4` and `glmmTMB` provide similar estimates when they both use maximum likelihood estimation. By default, `lme4` uses the restricted maximum likelihood; hence in the following examples, we use `lmer(..., REML = FALSE)` to compare against `glmmTMB`.

### Comparison Setup

```{r comparison-setup}
if (!requireNamespace("glmmTMB", quietly = TRUE)) {
  knitr::opts_chunk$set(eval = FALSE)
} else {
  library(glmmTMB)
}

## Often want to ignore attributes and class.
## Set a fairly large tolerance for convenience.
all.equal.nocheck <- function(x, y, tolerance = 3e-5, ..., check.attributes = FALSE, check.class = FALSE) {
  require("Matrix", quietly = TRUE)
  ## working around mode-matching headaches
  if (is(x, "Matrix")) x <- matrix(x)
  if (is(y, "Matrix")) y <- matrix(y)
  all.equal(x, y, ..., tolerance = tolerance, check.attributes = check.attributes, check.class = check.class)
}

get.cor1 <- function(x) {
  v <- VarCorr(x)
  vv <- if (inherits(x, "merMod")) v$group else v$cond$group
  attr(vv, "correlation")[1,2]
}
```

### Unstructured (General Positive Definite)

This is the default setting for both `lme4` and `glmmTMB`. Below we simulate a dataset with known `beta`, `theta` and `sigma` values.

```{r unstruc-sim}
n_groups <- 20
n_per_group <- 20
n <- n_groups * n_per_group

set.seed(1)
dat.us <- data.frame(
  group = rep(1:n_groups, each = n_per_group),
  x1 = rnorm(n),
  x2 = rnorm(n)
)
## Constructing a similar dataset for the other examples
gdat.us <- dat.diag <- gdat.diag <- dat.us

form <- y ~ 1 + x1 * x2 + us(1 + x1|group)
dat.us$y <- simulate(form[-2], 
                    newdata = dat.us,
                    family = gaussian,
                    newparams = list(beta = c(-7, 5, -100, 20),
                                     theta = c(2.5, 1.4, 6.3),
                                     sigma = 2))[[1]]

form2 <- y ~ 1 + x1 + us(1 + x1|group)
gdat.us$y <- simulate(
  form2[-2],
  newdata = gdat.us,
  family = binomial,
  newparams = list(
    beta  = c(-1.5, 0.5),     
    theta = c(0.3, 0.1, 0.3)
  ))[[1]]
```


### Linear Mixed Model

```{r compare-lmm}
lme4.us <- lmer(form, data = dat.us, REML = "FALSE")
glmmTMB.us <- glmmTMB(form, dat = dat.us)

## Fixed effects
fixef(lme4.us); fixef(glmmTMB.us)$cond
all.equal.nocheck(fixef(lme4.us), fixef(glmmTMB.us)$cond)

## Sigma
sigma(lme4.us); sigma(glmmTMB.us)
all.equal.nocheck(sigma(lme4.us), sigma(glmmTMB.us))

## Log likelihoods
logLik(lme4.us); logLik(glmmTMB.us)
all.equal.nocheck(logLik(lme4.us), logLik(glmmTMB.us))
```

As expected, calculations related to the random-effects term differ slightly beyond this point.

```{r compare-cov}
## Variance-Covariance Matrix
vcov(lme4.us); vcov(glmmTMB.us)$cond
all.equal.nocheck(vcov(lme4.us), vcov(glmmTMB.us)$cond)

## Variance and Covariance Components
all.equal.nocheck(VarCorr(lme4.us)$group,
          VarCorr(glmmTMB.us)$cond$group)

## Conditional Modes of the Random Effects
all.equal.nocheck(ranef(lme4.us)$group, ranef(glmmTMB.us)$cond$group)
```

### Generalized Linear Mixed Model

```{r compare-glmm}
glme4.us <- glmer(form2, data = gdat.us, family = binomial)
gglmmTMB.us <- glmmTMB(form2, dat = gdat.us, family = binomial)

## Fixed effects
fixef(glme4.us); fixef(gglmmTMB.us)$cond
all.equal.nocheck(fixef(glme4.us), fixef(gglmmTMB.us)$cond)

## Sigma
all.equal.nocheck(sigma(glme4.us), sigma(gglmmTMB.us))

## Log likelihoods
logLik(glme4.us); logLik(gglmmTMB.us)
all.equal.nocheck(logLik(glme4.us), logLik(gglmmTMB.us))
```

As expected, calculations related to the random-effects term differ slightly beyond this point.

```{r compare-glmm-cov}
## Variance-Covariance Matrix
vcov(glme4.us); vcov(gglmmTMB.us)$cond
all.equal.nocheck(vcov(glme4.us), vcov(gglmmTMB.us)$cond)

## Variance and Covariance Components
all.equal.nocheck(VarCorr(glme4.us)$group,
          VarCorr(gglmmTMB.us)$cond$group)

## Conditional Modes of the Random Effects
all.equal.nocheck(ranef(glme4.us)$group, ranef(gglmmTMB.us)$cond$group)
```

### Diagonal

The syntax is the same for fitting a heterogeneous diagonal covariance structure for `lme4` and `glmmTMB`. It changes when we want to fit a *homogeneous* diagonal covariance structure.

To fit a homogeneous diagonal covariance structure we would write:

```{r, fit-homog-diag, eval = FALSE}
lme4.us <- lmer(Reaction ~ Days + diag(Days | Subject, hom = TRUE), sleepstudy)
glmmTMB.us <- glmmTMB(Reaction ~ Days + homdiag(Days | Subject), sleepstudy)
```

We will focus on comparisons of an estimated *heterogeneous* diagonal covariance structure.

```{r sim-het-diag}
form <- y ~ 1 + x1 * x2 + diag(1|group)
dat.diag$y <- simulate(form[-2], 
                       newdata = dat.diag,
                       family = gaussian,
                       newparams = list(beta = c(-7, 5, -100, 20),
                                        theta = c(2.5),
                                        sigma = 2))[[1]]
```


```{r fit-het-diag}
lme4.diag <- lmer(form, data = dat.diag, REML = "FALSE")
glmmTMB.diag <- glmmTMB(form, dat = dat.diag)

## Fixed effects
fixef(lme4.diag); fixef(glmmTMB.diag)$cond
all.equal.nocheck(fixef(lme4.diag), fixef(glmmTMB.diag)$cond)

## Sigma
sigma(lme4.diag); sigma(glmmTMB.diag)
all.equal.nocheck(sigma(lme4.diag), sigma(glmmTMB.diag))

## Log likelihoods
logLik(lme4.diag); logLik(glmmTMB.diag)
all.equal.nocheck(logLik(lme4.diag), logLik(glmmTMB.diag))

## Variance-Covariance Matrix
vcov(lme4.diag); vcov(glmmTMB.diag)$cond
all.equal.nocheck(vcov(lme4.diag), vcov(glmmTMB.diag)$cond)

## Variance and Covariance Components
all.equal.nocheck(VarCorr(lme4.diag)[[1]], 
          VarCorr(glmmTMB.diag)$cond$group)

## Conditional Modes of the Random Effects
all.equal.nocheck(ranef(lme4.diag)$group, ranef(glmmTMB.diag)$cond$group)
```

### Compound Symmetry

Similar to the diagonal case, the syntax is the same for fitting a heterogeneous compound symmetry covariance structure for `lme4` and `glmmTMB`: 

```{r cs-fit-2, eval = FALSE}
lme4.us <- lmer(Reaction ~ Days + cs(Days | Subject, hom = TRUE), sleepstudy)
glmmTMB.us <- glmmTMB(Reaction ~ Days + cs(Days | Subject), sleepstudy)
```

Again, it differs when we want to fit a *homogeneous* compound symmetry covariance structure, which we will use for our comparisons.

```{r simgroup}
simGroup <- function(g, n=6, phi=0.6) {
  x <- MASS::mvrnorm(mu = rep(0,n),
                     Sigma = phi^as.matrix(dist(1:n)) )  
  y <- x + rnorm(n)                              
  times <- factor(1:n)
  group <- factor(rep(g,n))
  data.frame(y, times, group)
}

set.seed(1)
dat.cs <- do.call("rbind", lapply(1:2000, simGroup))
```

<!-- this is the slow one ... cache? -->
```{r more-comp, cache=TRUE}
lme4.cs <- lmer(y ~ times + cs(0 + times | group, hom = TRUE), data = dat.cs, REML = FALSE)
glmmTMB.cs <- glmmTMB(y ~ times + homcs(0 + times | group), data = dat.cs)

## Fixed effects
fixef(lme4.cs); fixef(glmmTMB.cs)$cond
all.equal.nocheck(fixef(lme4.cs), fixef(glmmTMB.cs)$cond)

## Sigma
sigma(lme4.cs); sigma(glmmTMB.cs)
all.equal.nocheck(sigma(lme4.cs), sigma(glmmTMB.cs))

## Log likelihoods
logLik(lme4.cs); logLik(glmmTMB.cs)
all.equal.nocheck(logLik(lme4.cs), logLik(glmmTMB.cs))

## Variance-Covariance Matrix
all.equal.nocheck(vcov(lme4.cs), vcov(glmmTMB.cs)$cond)

## Variance and Covariance Components
all.equal.nocheck(VarCorr(lme4.cs)[[1]], 
          VarCorr(glmmTMB.cs)$cond$group)

## Conditional Modes of the Random Effects
all.equal.nocheck(ranef(lme4.cs)$group, ranef(glmmTMB.cs)$cond$group)

## Comparing against the predicted rho value
lme4.rho <- get.cor1(lme4.cs)
glmmTMB.rho <- get.cor1(glmmTMB.cs)
lme4.rho; glmmTMB.rho
all.equal.nocheck(lme4.rho, glmmTMB.rho)
```

### Autoregressive Order 1

For this comparison, we focus on a simulated data set with $\rho = 0.7$.

```{r ar1-sim}
set.seed(1)
dat.ar1 <- do.call("rbind", lapply(1:2000, function(g) simGroup(g, phi = 0.7)))
```

Unlike the diagonal and compound symmetry case, the syntax differs for fitting either a heterogeneous or a homogeneous AR1 model for `lme4` and `glmmTMB`.

For a *heterogeneous* AR1 covariance structure we would write the following:

```{r het-ar1-fit, eval = FALSE}
lme4.ar1 <- lmer(y ~ times + ar1(0 + times | group), data = dat.ar1, REML = FALSE)
glmmTMB.ar1 <- glmmTMB(y ~ times + hetar1(0 + times | group), data = dat.ar1)
```

We will instead focus on comparisons for a *homogeneous* AR1 covariance structure.

```{r ar1-fit, cache=TRUE}
lme4.ar1 <- lmer(y ~ times + ar1(0 + times | group, hom = TRUE), data = dat.ar1, REML = FALSE)
glmmTMB.ar1 <- glmmTMB(y ~ times + ar1(0 + times | group), data = dat.ar1)

## Fixed effects
fixef(lme4.ar1); fixef(glmmTMB.ar1)$cond
all.equal.nocheck(fixef(lme4.ar1), fixef(glmmTMB.ar1)$cond)

## Sigma
sigma(lme4.ar1); sigma(glmmTMB.ar1)
all.equal.nocheck(sigma(lme4.ar1), sigma(glmmTMB.ar1))

## Log likelihoods
logLik(lme4.ar1); logLik(glmmTMB.ar1)
all.equal.nocheck(logLik(lme4.ar1), logLik(glmmTMB.ar1))

## Variance-Covariance Matrix
all.equal.nocheck(vcov(lme4.ar1), vcov(glmmTMB.ar1)$cond)

## Variance and Covariance Components
all.equal.nocheck(VarCorr(lme4.ar1)$group, 
                  VarCorr(glmmTMB.ar1)$cond$group)

## Conditional Modes of the Random Effects
all.equal.nocheck(ranef(lme4.ar1)$group, ranef(glmmTMB.ar1)$cond$group)

## Comparing against the predicted rho value
lme4.ar1.rho <- get.cor1(lme4.ar1)
glmmTMB.ar1.rho <- get.cor1(glmmTMB.ar1)
lme4.ar1.rho; glmmTMB.ar1.rho
all.equal.nocheck(lme4.ar1.rho, glmmTMB.ar1.rho)
```

## References

\documentclass[12pt]{article}
\usepackage{Sweave,amsmath,amsfonts,bm}
\usepackage[authoryear,round]{natbib}
\bibliographystyle{plainnat}
\DeclareMathOperator \tr {tr}
\DefineVerbatimEnvironment{Sinput}{Verbatim}
{formatcom={\vspace{-1ex}},fontshape=sl,
  fontfamily=courier,fontseries=b, fontsize=\footnotesize}
\DefineVerbatimEnvironment{Soutput}{Verbatim}
{formatcom={\vspace{-1ex}},fontfamily=courier,fontseries=b,%
  fontsize=\footnotesize}
%%\VignetteIndexEntry{PLS vs GLS for LMMs}
%%\VignetteDepends{lme4}
\title{Penalized least squares versus generalized least squares
  representations of linear mixed models}
\author{Douglas Bates\\Department of Statistics\\%
  University of Wisconsin -- Madison}
\begin{document}
\SweaveOpts{engine=R,eps=FALSE,pdf=TRUE,strip.white=true,keep.source=TRUE}
\SweaveOpts{include=FALSE}
\setkeys{Gin}{width=\textwidth}
\newcommand{\code}[1]{\texttt{\small{#1}}}
\newcommand{\package}[1]{\textsf{\small{#1}}}
\newcommand{\trans}{\ensuremath{^\prime}}
<<preliminaries,echo=FALSE,results=hide>>=
options(width=65,digits=5)
#library(lme4)
@

\maketitle

\begin{abstract}
  The methods in the \code{lme4} package for \code{R} for fitting
  linear mixed models are based on sparse matrix methods, especially
  the Cholesky decomposition of sparse positive-semidefinite matrices,
  in a penalized least squares representation of the conditional model
  for the response given the random effects.  The representation is
  similar to that in Henderson's mixed-model equations.  An
  alternative representation of the calculations is as a generalized
  least squares problem.  We describe the two representations, show
  the equivalence of the two representations and explain why we feel
  that the penalized least squares approach is more versatile and more
  computationally efficient.
\end{abstract}

\section{Definition of the model}
\label{sec:Definition}

We consider linear mixed models in which the random effects are
represented by a $q$-dimensional random vector, $\bm{\mathcal{B}}$, and
the response is represented by an $n$-dimensional random vector,
$\bm{\mathcal{Y}}$.  We observe a value, $\bm y$, of the response.  The
random effects are unobserved.

For our purposes, we will assume a ``spherical'' multivariate normal
conditional distribution of $\bm{\mathcal{Y}}$, given
$\bm{\mathcal{B}}$.  That is, we assume the variance-covariance matrix
of $\bm{\mathcal{Y}}|\bm{\mathcal{B}}$ is simply $\sigma^2\bm I_n$,
where $\bm I_n$ denotes the identity matrix of order $n$.  (The term
``spherical'' refers to the fact that contours of the conditional
density are concentric spheres.)

The conditional mean,
$\mathrm{E}[\bm{\mathcal{Y}}|\bm{\mathcal{B}}=\bm b]$, is a linear
function of $\bm b$ and the $p$-dimensional fixed-effects parameter,
$\bm\beta$,
\begin{equation}
  \label{eq:condmean}
  \mathrm{E}[\bm{\mathcal{Y}}|\bm{\mathcal{B}}=\bm b]=
  \bm X\bm\beta+\bm Z\bm b ,
\end{equation}
where $\bm X$ and $\bm Z$ are known model matrices of sizes $n\times
p$ and $n\times q$, respectively. Thus
\begin{equation}
  \label{eq:yconditional}
  \bm{\mathcal{Y}}|\bm{\mathcal{B}}\sim
  \mathcal{N}\left(\bm X\bm\beta+\bm Z\bm b,\sigma^2\bm I_n\right) .
\end{equation}

The marginal distribution of the random effects
\begin{equation}
  \label{eq:remargin}
  \bm{\mathcal{B}}\sim\mathcal{N}\left(\bm 0,\sigma^2\bm\Sigma(\bm\theta)\right)
\end{equation}
is also multivariate normal, with mean $\bm 0$ and variance-covariance
matrix $\sigma^2\bm\Sigma(\bm\theta)$.  The scalar, $\sigma^2$, in
(\ref{eq:remargin}) is the same as the $\sigma^2$ in
(\ref{eq:yconditional}).  As described in the next section, the
relative variance-covariance matrix, $\bm\Sigma(\bm\theta)$, is a
$q\times q$ positive semidefinite matrix depending on a parameter
vector, $\bm\theta$.  Typically the dimension of $\bm\theta$ is much,
much smaller than $q$.


\subsection{Variance-covariance of the random effects}
\label{sec:revarcov}

The relative variance-covariance matrix, $\bm\Sigma(\bm\theta)$, must
be symmetric and positive semidefinite (i.e. $\bm x\trans\bm\Sigma\bm
x\ge0,\forall\bm x\in\mathbb{R}^q$).  Because the estimate of a
variance component can be zero, it is important to allow for a
semidefinite $\bm\Sigma$.  We do not assume that $\bm\Sigma$ is
positive definite (i.e. $\bm x\trans\bm\Sigma\bm x>0,\forall\bm
x\in\mathbb{R}^q, \bm x\ne\bm 0$) and, hence, we cannot assume that $\bm\Sigma^{-1}$
exists.

A positive semidefinite matrix such as $\bm\Sigma$ has a Cholesky
decomposition of the so-called ``LDL$\trans$'' form.  We use a
slight modification of this form,
\begin{equation}
  \label{eq:TSdef}
  \bm\Sigma(\bm\theta)=\bm T(\bm\theta)\bm S(\bm\theta)\bm
  S(\bm\theta)\bm T(\bm\theta)\trans ,
\end{equation}
where $\bm T(\bm\theta)$ is a unit lower-triangular $q\times q$ matrix
and $\bm S(\bm\theta)$ is a diagonal $q\times q$ matrix with
nonnegative diagonal elements that act as scale factors.  (They are
the relative standard deviations of certain linear combinations of the
random effects.)  Thus, $\bm T$ is a triangular matrix and $\bm S$ is
a scale matrix.

Both $\bm T$ and $\bm S$ are highly patterned.

\subsection{Orthogonal random effects}
\label{sec:orthogonal}

Let us define a $q$-dimensional random vector, $\bm{\mathcal{U}}$, of
orthogonal random effects with marginal distribution
\begin{equation}
  \label{eq:Udist}
  \bm{\mathcal{U}}\sim\mathcal{N}\left(\bm 0,\sigma^2\bm I_q\right)
\end{equation}
and, for a given value of $\bm\theta$, express $\bm{\mathcal{B}}$ as a
linear transformation of $\bm{\mathcal{U}}$,
\begin{equation}
  \label{eq:UtoB}
  \bm{\mathcal{B}}=\bm T(\bm\theta)\bm S(\bm\theta)\bm{\mathcal{U}} .
\end{equation}
Note that the transformation (\ref{eq:UtoB}) gives the desired
distribution of $\bm{\mathcal{B}}$ in that
$\mathrm{E}[\bm{\mathcal{B}}]=\bm T\bm
S\mathrm{E}[\bm{\mathcal{U}}]=\bm 0$ and
\begin{displaymath}
  \mathrm{Var}(\bm{\mathcal{B}})=\mathrm{E}[\bm{\mathcal{B}}\bm{\mathcal{B}}\trans]
  =\bm T\bm S\mathrm{E}[\bm{\mathcal{U}}\bm{\mathcal{U}}\trans]\bm
  S\bm T\trans=\sigma^2\bm T\bm S\bm S\bm T\trans=\bm\Sigma .
\end{displaymath}

The conditional distribution, $\bm{\mathcal{Y}}|\bm{\mathcal{U}}$, can
be derived from $\bm{\mathcal{Y}}|\bm{\mathcal{B}}$ as
\begin{equation}
  \label{eq:YgivenU}
  \bm{\mathcal{Y}}|\bm{\mathcal{U}}\sim\mathcal{N}\left(\bm X\bm\beta+\bm
  Z\bm T\bm S\bm u, \sigma^2\bm I\right)
\end{equation}
We will write the transpose of $\bm Z\bm T\bm S$ as $\bm A$.  Because
the matrices $\bm T$ and $\bm S$ depend on the parameter $\bm\theta$,
$\bm A$ is also a function of $\bm\theta$,
\begin{equation}
  \label{eq:Adef}
  \bm A\trans(\bm\theta)=\bm Z\bm T(\bm\theta)\bm S(\bm\theta) .
\end{equation}

In applications, the matrix $\bm Z$ is derived from indicator columns
of the levels of one or more factors in the data and is a
\emph{sparse} matrix, in the sense that most of its elements are zero.
The matrix $\bm A$ is also sparse.  In fact, the structure of $\bm T$
and $\bm S$ are such that pattern of nonzeros in $\bm A$ is that same
as that in $\bm Z\trans$.

\subsection{Sparse matrix methods}
\label{sec:sparseMatrix}

The reason for defining $\bm A$ as the transpose of a model matrix is
because $\bm A$ is stored and manipulated as a sparse matrix.  In the
compressed column-oriented storage form that we use for sparse
matrices, there are advantages to storing $\bm A$ as a matrix of $n$
columns and $q$ rows.  In particular, the CHOLMOD sparse matrix
library allows us to evaluate the sparse Cholesky factor, $\bm
L(\bm\theta)$, a sparse lower triangular matrix that satisfies
\begin{equation}
  \label{eq:SparseChol}
  \bm L(\bm\theta)\bm L(\bm\theta)\trans=
  \bm P\left(\bm A(\bm\theta)\bm A(\bm\theta)\trans+\bm I_q\right)\bm P\trans ,
\end{equation}
directly from $\bm A(\bm\theta)$.

In (\ref{eq:SparseChol}) the $q\times q$ matrix $\bm P$ is a
``fill-reducing'' permutation matrix determined from the pattern of
nonzeros in $\bm Z$.  $\bm P$ does not affect the statistical theory
(if $\bm{\mathcal{U}}\sim\mathcal{N}(\bm 0,\sigma^2\bm I)$ then $\bm
P\trans\bm{\mathcal{U}}$ also has a $\mathcal{N}(\bm 0,\sigma^2\bm I)$
distribution because $\bm P\bm P\trans=\bm P\trans\bm P=\bm I$) but,
because it affects the number of nonzeros in $\bm L$, it can have a
tremendous impact on the amount storage required for $\bm L$ and the
time required to evaluate $\bm L$ from $\bm A$.  Indeed, it is
precisely because $\bm L(\bm\theta)$ can be evaluated quickly, even
for complex models applied the large data sets, that the \code{lmer}
function is effective in fitting such models.

\section{The penalized least squares approach to linear mixed models}
\label{sec:Penalized}

Given a value of $\bm\theta$ we form $\bm A(\bm\theta)$ from which we
evaluate $\bm L(\bm\theta)$.  We can then solve for the $q\times p$
matrix, $\bm R_{\bm{ZX}}$, in the system of equations
\begin{equation}
  \label{eq:RZX}
  \bm L(\theta)\bm R_{\bm{ZX}}=\bm P\bm A(\bm\theta)\bm X
\end{equation}
and for the $p\times p$ upper triangular matrix, $\bm R_{\bm X}$, satisfying
\begin{equation}
  \label{eq:RX}
  \bm R_{\bm X}\trans\bm R_{\bm X}=
  \bm X\trans\bm X-\bm R_{\bm{ZX}}\trans\bm R_{\bm{ZX}}
\end{equation}

The conditional mode, $\tilde{\bm u}(\bm\theta)$, of the
orthogonal random effects and the conditional mle,
$\widehat{\bm\beta}(\bm\theta)$, of the fixed-effects parameters
can be determined simultaneously as the solutions to a penalized least
squares problem,
\begin{equation}
  \label{eq:PLS}
  \begin{bmatrix}
    \tilde{\bm u}(\bm\theta)\\
    \widehat{\bm\beta}(\bm\theta)
  \end{bmatrix}=
  \arg\min_{\bm u,\bm\beta}\left\|
    \begin{bmatrix}\bm y\\\bm 0\end{bmatrix} -
    \begin{bmatrix}
      \bm A\trans\bm P\trans & \bm X\\
      \bm I_q & \bm 0
    \end{bmatrix}
    \begin{bmatrix}\bm u\\\bm\beta\end{bmatrix} ,
  \right\|^2
\end{equation}
for which the solution satisfies
\begin{equation}
  \label{eq:PLSsol}
  \begin{bmatrix}
    \bm P\left(\bm A\bm A\trans+\bm I\right)\bm P\trans &
    \bm P\bm A\bm X\\
    \bm X\trans\bm A\trans\bm P\trans & \bm X\trans\bm X
  \end{bmatrix}
  \begin{bmatrix}
    \tilde{\bm u}(\bm\theta)\\
    \widehat{\bm\beta}(\bm\theta)
  \end{bmatrix}=
  \begin{bmatrix}\bm P\bm A\bm y\\\bm X\trans\bm y\end{bmatrix} .
\end{equation}
The Cholesky factor of the system matrix for the PLS problem can be
expressed using $\bm L$, $\bm R_{\bm Z\bm X}$ and $\bm R_{\bm X}$, because
\begin{equation}
  \label{eq:PLSChol}
  \begin{bmatrix}
    \bm P\left(\bm A\bm A\trans+\bm I\right)\bm P\trans & \bm P\bm
    A\bm X\\
    \bm X\trans\bm A\trans\bm P\trans & \bm X\trans\bm X
  \end{bmatrix} =
  \begin{bmatrix}
    \bm L & \bm 0\\
    \bm R_{\bm Z\bm X}\trans & \bm R_{\bm X}\trans
  \end{bmatrix}
  \begin{bmatrix}
    \bm L\trans & \bm R_{\bm Z\bm X}\\
    \bm 0 & \bm R_{\bm X}
  \end{bmatrix} .
\end{equation}

In the \code{lme4} package the \code{"mer"} class is the
representation of a mixed-effects model.  Several slots in this class
are matrices corresponding directly to the matrices in the preceding
equations. The \code{A} slot contains the sparse matrix $\bm
A(\bm\theta)$ and the \code{L} slot contains the sparse Cholesky
factor, $\bm L(\bm\theta)$.  The \code{RZX} and \code{RX} slots contain
$\bm R_{\bm Z\bm X}(\bm\theta)$ and $\bm R_{\bm X}(\bm\theta)$,
respectively, stored as dense matrices.

It is not necessary to solve for $\tilde{\bm u}(\bm\theta)$ and
$\widehat{\bm\beta}(\bm\theta)$ to evaluate the \emph{profiled}
log-likelihood, which is the log-likelihood evaluated $\bm\theta$ and
the conditional estimates of the other parameters,
$\widehat{\bm\beta}(\bm\theta)$ and $\widehat{\sigma^2}(\bm\theta)$.
All that is needed for evaluation of the profiled log-likelihood is
the (penalized) residual sum of squares, $r^2$, from the penalized
least squares problem (\ref{eq:PLS}) and the determinant $|\bm A\bm
A\trans+\bm I|=|\bm L|^2$. Because $\bm L$ is triangular, its
determinant is easily evaluated as the product
of its diagonal elements.  Furthermore, $|\bm L|^2 > 0$ because it is
equal to $|\bm A\bm A\trans + \bm I|$, which is the determinant of a
positive definite matrix.  Thus $\log(|\bm L|^2)$ is both well-defined
and easily calculated from $\bm L$.

The profiled deviance (negative twice the profiled log-likelihood), as
a function of $\bm\theta$ only ($\bm\beta$ and $\sigma^2$ at their
conditional estimates), is
\begin{equation}
  \label{eq:profiledDev}
  d(\bm\theta|\bm y)=\log(|\bm L|^2)+n\left(1+\log(r^2)+\frac{2\pi}{n}\right)
\end{equation}
The maximum likelihood estimates, $\widehat{\bm\theta}$, satisfy
\begin{equation}
  \label{eq:thetamle}
  \widehat{\bm\theta}=\arg\min_{\bm\theta}d(\bm\theta|\bm y)
\end{equation}
Once the value of $\widehat{\bm\theta}$ has been determined, the mle
of $\bm\beta$ is evaluated from (\ref{eq:PLSsol}) and the mle of
$\sigma^2$ as $\widehat{\sigma^2}(\bm\theta)=r^2/n$.

Note that nothing has been said about the form of the sparse model
matrix, $\bm Z$, other than the fact that it is sparse.  In contrast
to other methods for linear mixed models, these results apply to
models where $\bm Z$ is derived from crossed or partially crossed
grouping factors, in addition to models with multiple, nested grouping
factors.

The system (\ref{eq:PLSsol}) is similar to Henderson's ``mixed-model
equations'' (reference?).  One important difference between
(\ref{eq:PLSsol}) and Henderson's formulation is that Henderson
represented his system of equations in terms of $\bm\Sigma^{-1}$ and,
in important practical examples, $\bm\Sigma^{-1}$ does not exist at
the parameter estimates.  Also, Henderson assumed that equations like
(\ref{eq:PLSsol}) would need to be solved explicitly and, as we have
seen, only the decomposition of the system matrix is needed for
evaluation of the profiled log-likelihood.  The same is true of the
profiled the logarithm of the REML criterion, which we define later.

\section{The generalized least squares approach to linear mixed models}
\label{sec:GLS}

Another common approach to linear mixed models is to derive the
marginal variance-covariance matrix of $\bm{\mathcal{Y}}$ as a
function of $\bm\theta$ and use that to determine the conditional
estimates, $\widehat{\bm\beta}(\bm\theta)$, as the solution of a
generalized least squares (GLS) problem.  In the notation of
\S\ref{sec:Definition} the marginal mean of $\bm{\mathcal{Y}}$ is
$\mathrm{E}[\bm{\mathcal{Y}}]=\bm X\bm\beta$ and the marginal
variance-covariance matrix is
\begin{equation}
  \label{eq:marginalvarcovY}
  \mathrm{Var}(\bm{\mathcal{Y}})=\sigma^2\left(\bm I_n+\bm Z\bm T\bm
    S\bm S\bm T\trans\bm Z\trans\right)=\sigma^2\left(\bm I_n+\bm
    A\trans\bm A\right) =\sigma^2\bm V(\bm\theta) ,
\end{equation}
where $\bm V(\bm\theta)=\bm I_n+\bm A\trans\bm A$.

The conditional estimates of $\bm\beta$ are often written as
\begin{equation}
  \label{eq:condbeta}
  \widehat{\bm\beta}(\bm\theta)=\left(\bm X\trans\bm V^{-1}\bm
    X\right)^{-1}\bm X\trans\bm V^{-1}\bm y
\end{equation}
but, of course, this formula is not suitable for computation.  The
matrix $\bm V(\bm\theta)$ is a symmetric $n\times n$ positive definite
matrix and hence has a Cholesky factor.  However, this factor is
$n\times n$, not $q\times q$, and $n$ is always larger than $q$ ---
sometimes orders of magnitude larger.  Blithely writing a formula in
terms of $\bm V^{-1}$ when $\bm V$ is $n\times n$, and $n$ can be in
the millions does not a computational formula make.

\subsection{Relating the GLS approach to the Cholesky factor}
\label{sec:GLStoL}

We can use the fact that
\begin{equation}
  \label{eq:Vinv}
  \bm V^{-1}(\bm\theta)=\left(\bm I_n+\bm A\trans\bm A\right)^{-1}=
  \bm I_n-\bm A\trans\left(\bm I_q+\bm A\bm A\trans\right)^{-1}\bm A
\end{equation}
to relate the GLS problem to the PLS problem.  One way to establish
(\ref{eq:Vinv}) is simply to show that the product
\begin{multline*}
  (\bm I+\bm A\trans\bm A)\left(\bm I-\bm A\trans\left(\bm I+\bm A\bm
      A\trans\right)^{-1}\bm A\right)\\
  \begin{aligned}
    =&\bm I+\bm A\trans\bm A-\bm A\trans\left(\bm I+\bm A\bm
      A\trans\right)
    \left(\bm I+\bm A\bm A\trans\right)^{-1}\bm A\\
    =&\bm I+\bm A\trans\bm A-\bm A\trans\bm A\\
    =&\bm I .
  \end{aligned}
\end{multline*}
Incorporating the permutation matrix $\bm P$ we have
\begin{equation}
  \label{eq:PLA}
  \begin{aligned}
    \bm V^{-1}(\bm\theta)=&\bm I_n-\bm A\trans\bm P\trans\bm P\left(\bm
      I_q+\bm A\bm A\trans\right)^{-1}\bm P\trans\bm P\bm A\\
    =&\bm I_n-\bm A\trans\bm P\trans(\bm L\bm L\trans)^{-1}\bm P\bm A\\
    =&\bm I_n-\left(\bm L^{-1}\bm P\bm A\right)\trans\bm L^{-1}\bm P\bm A .
  \end{aligned}
\end{equation}
Even in this form we would not want to routinely evaluate $\bm
V^{-1}$.  However, (\ref{eq:PLA}) does allow us to simplify many
common expressions.

For example, the variance-covariance of the estimator $\widehat{\bm
  \beta}$, conditional on $\bm\theta$ and $\sigma$, can be expressed as
\begin{equation}
  \label{eq:varcovbeta}
  \begin{aligned}
  \sigma^2\left(\bm X\trans\bm V^{-1}(\bm\theta)\bm X\right)^{-1}
  =&\sigma^2\left(\bm X\trans\bm X-\left(\bm L^{-1}\bm P\bm
      A\bm X\right)\trans\left(\bm L^{-1}\bm P\bm A\bm
      X\right)\right)^{-1}\\
  =&\sigma^2\left(\bm X\trans\bm X-\bm R_{\bm Z\bm X}\trans\bm
    R_{\bm Z\bm X}\right)^{-1}\\
    =&\sigma^2\left(\bm R_{\bm X}\trans\bm R_{\bm X}\right)^{-1} .
  \end{aligned}
\end{equation}

\section{Trace of the ``hat'' matrix}
\label{sec:hatTrace}

Another calculation that is of interest to some is the
the trace of the ``hat'' matrix, which can be written as
\begin{multline}
  \label{eq:hatTrace}
  \tr\left(\begin{bmatrix}\bm A\trans&\bm X\end{bmatrix}
    \left(\begin{bmatrix}\bm A\trans&\bm X\\\bm I&\bm0\end{bmatrix}\trans
      \begin{bmatrix}\bm A\trans&\bm X\\\bm I&\bm0\end{bmatrix}\right)^{-1}
    \begin{bmatrix}\bm A\\\bm X\trans\end{bmatrix}\right)\\
  = \tr\left(\begin{bmatrix}\bm A\trans&\bm X\end{bmatrix}
    \left(\begin{bmatrix}\bm L&\bm0\\
        \bm R_{\bm{ZX}}\trans&\bm R_{\bm X}\trans\end{bmatrix}
      \begin{bmatrix}\bm L\trans&\bm R_{\bm{ZX}}\\
        \bm0&\bm R_{\bm X}\end{bmatrix}\right)^{-1}
    \begin{bmatrix}\bm A\\\bm X\trans\end{bmatrix}\right)
\end{multline}

\end{document}

---
title: "Autoscaling in lme4"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Autoscaling in lme4}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: autoscale_ref.bib  
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

Scaling predictors in a linear model (or extended linear models, such as LMMs or GLMMs) so that all of the predictors are on a similar scale (and thus, in general, the estimated parameters should have a similar scale) can improve the behaviour of various computational methods (see e.g. the "Remove eccentricity by scaling" section of @bolker2013strategies). `lmer()` and `glmer()` issue warnings suggesting that users consider re-scaling predictors when the predictor variables are on very different scales. Re-scaling predictors can improve both computation and interpretation [@schielzethSimple2010] of (extended) linear models. However, in cases where researchers want to scale predictors for computational stability, but get parameter estimates on the scale of the original (unscaled) predictors, back-transforming the predictors, covariance matrix, etc., is tedious. `lme4` now allows this scaling to be done automatically, behind the scenes.

An example of a warning message due to disparate predictor scales:

```{r setup}
library(lme4)

set.seed(1)
sleepstudy$var1 = runif(nrow(sleepstudy), 1e6, 1e7)

fit1 <- lmer(Reaction ~ var1 + Days + (Days | Subject), sleepstudy)
```

Instead of re-fitting with scaled predictors we can use a new (as of `lme4` version >= 1.1.37) argument for `(g)lmerControl`, called `autoscale`, where `scale()` is automatically applied to the continuous covariates, but when delivered to the user, the coefficient estimates and the covariances are back-transformed. This maintains the original interpretation and minimizes user intervention.

Hence:

```{r fit}
fit2 <- lmer(Reaction ~ var1 + Days + (Days | Subject), 
             control = lmerControl(autoscale = TRUE), sleepstudy)
all.equal(fixef(fit1), fixef(fit2))
all.equal(vcov(fit2), vcov(fit2))
```

```{r echo = FALSE}
## https://stackoverflow.com/a/67456510/190277
get_val <- function(x) as.numeric(gsub("^.*?(\\d+\\.?\\d+e?[+-]?\\d+).*$", "\\1", x))
get_tol <- function(x,y, dig=1) signif(get_val(all.equal(x, y, tolerance = 0)), dig)
ftol <- get_tol(fixef(fit1), fixef(fit2))
vtol <- get_tol(vcov(fit1), vcov(fit2))
```

The parameters are in fact slightly different (relative difference in fixed effects of $`r vtol`$, and in covariance matrices of $`r ftol`$); these differences will be larger for less numerically stable models (i.e. more complicated or poorly data-constrained).

## Autoscale Mechanism

The base function `scale()` is applied to the model matrix `X` as found in `lFormula()` or `glFormula()` from `R/Modular.R`. 

To back transform, we use the `scaled:center` and `scaled:scale` attributes to reverse the changes found in `fixef.merMod()`, `model.matrix.merMod()`, and `vcov.merMod()` such that the `summary()` output shows the coefficients according to the original scale.

Reverting back the changes for `fixef.merMod()`, the $\beta$ coefficients, is not too difficult. However, reverting said changes for the variance-covariance matrix is a bit more involved. The exact modification can be found from the function `scale_vcov()` (found in `R/utilities.R`), and the derivation is explained below.


Consider the following estimation of a simple linear model: 
\[
\widehat{Y} = \widehat{\beta}_{0} + \widehat{\beta}_{1} X^{*},
\]
where $X^{*}$ represents the scaled version of $X$. That is, if we let $C$ represent a vector that contains the values for centering and $S$ represent a vector that contains the values for scaling, we have:
\[
\widehat{Y} = \widehat{\beta}_{0} + \widehat{\beta}_{1} \left( \frac{X - C}{S} \right)
\]
\[
\widehat{Y} = \widehat{\beta}_{0} - \sum_{i=1}^{p} \frac{\widehat{\beta}_{i}c_{i}}{s_{i}}
+ \sum_{i=1}^{p} \frac{\widehat{\beta}_{i} x_{i}}{s_{i}}.
\]
From the above, it is clear that the new intercept can be represented as:
\[
\widehat{\beta}_{0}' = \widehat{\beta}_{0} - \sum_{i=1}^{p} \frac{\widehat{\beta}_{i}c_{i}}{s_{i}}.
\]
Similarly, the new coefficients are represented as:
\[
\widehat{\beta}_{i}' =  \frac{\widehat{\beta}_{i}}{s_{i}}.
\]
Then, the new variance-covariance matrix can be derived using the following:
\newcommand{\cov}{\textrm{Cov}}}
\[
\cov\left( \frac{\widehat{\beta}_{i}}{s_{i}}, \frac{\widehat{\beta}_{j}}{s_{j}} \right)
= \frac{1}{s_{i}s_{j}} \cov(\widehat{\beta}_{i}, \widehat{\beta_{j}}) 
= \frac{\sigma_{ij}^{2}}{s_{i}s_{j}}
\]
\begin{equation}
    \begin{split}
    \cov\left( \widehat{\beta}_{0} - \sum_{i=1}^{p} \frac{\widehat{\beta}_{i}c_{i}}{s_{i}}, \widehat{\beta}_{0} - \sum_{j=1}^{p} \frac{\widehat{\beta}_{i}c_{i}}{s_{i}} \right)
    &= \cov(\widehat{\beta}_{0}, \widehat{\beta}_{0} ) - 2 \sum_{i=1}^{p} \frac{c_{i}}{s_{i}} \cov(\widehat{\beta}_{0} , \widehat{\beta}_{i}) + \sum_{i=1}^{p} \sum_{j=1}^{p} \frac{c_{i}c_{j}}{s_{i}s_{j}} \cov(\widehat{\beta}_{i}, \widehat{\beta}_{j}) \\
    &= \sigma_{0}^{2} - 2 \sum_{i=1}^{p} \frac{c_{i}}{s_{i}} \sigma_{0i}^{2} + \sum_{i=1}^{p} \sum_{j=1}^{p} \frac{c_{i}c_{j}}{s_{i}s_{j}} \sigma_{ij}^{2}
    \end{split}
\end{equation}
\begin{equation}   
    \begin{split}
    \cov\left( \widehat{\beta}_{0} - \sum_{i=1}^{p} \frac{\widehat{\beta}_{i}c_{i}}{s_{i}}, \frac{\widehat{\beta_{j}}}{s_{j}} \right)
    &= \cov\left( \widehat{\beta}_{0}, \frac{\widehat{\beta}_{j}}{s_{j}} \right) - \sum_{i=1}^{p} \frac{c_{i}}{s_{i}s_{j}} \cov \left( \widehat{\beta}_{i}, \widehat{\beta}_{j} \right) \\
    &= \frac{\sigma_{0j}^{2}}{s_{j}} - \sum_{i=1}^{p} \frac{c_{i}}{s_{i}s_{j}} \sigma_{ij}^{2},
    \end{split}
\end{equation}
for $j = 1, 2, ..., p$.

## References

%\VignetteEngine{knitr::knitr}
%\VignetteDepends{ggplot2}
%\VignetteDepends{gamm4}
%\VignetteIndexEntry{Fitting Linear Mixed-Effects Models using lme4}
\documentclass[nojss]{jss}
\usepackage[T1]{fontenc}% for correct hyphenation and T1 encoding
\usepackage[utf8]{inputenc}%
\usepackage{lmodern}% latin modern font
\usepackage[american]{babel}  %% for texi2dvi ~ bug
\usepackage{bm,amsmath,thumbpdf,amsfonts}%,minted}
\usepackage{blkarray}
\usepackage{array}
%% huxtable-ish stuff
%% \usepackage{adjustbox}
%% \usepackage{threeparttable}
%%
\newcolumntype{P}[1]{>{\raggedright\arraybackslash}p{#1}}
\newcommand{\matindex}[1]{\mbox{\scriptsize#1}}% Matrix index
\newcommand{\github}{Github}
\DeclareMathOperator{\tr}{tr}
\DeclareMathOperator{\VEC}{vec}
\newcommand{\bmb}[1]{{\color{red} \emph{#1}}}
\newcommand{\scw}[1]{{\color{blue} \emph{#1}}}
\newcommand{\dmb}[1]{{\color{magenta} \emph{#1}}}

\shortcites{bolker_strategies_2013,sleepstudy,gelman2013bayesian}

\author{Douglas Bates\\University of Wisconsin-Madison\And
  Martin M\"achler\\ETH Zurich\And
  Benjamin M. Bolker\\McMaster University\And
  Steven C. Walker\\McMaster University
}
\Plainauthor{Douglas Bates, Martin M\"achler, Ben Bolker, Steve Walker}
\title{Fitting Linear Mixed-Effects Models Using \pkg{lme4}}
\Plaintitle{Fitting Linear Mixed-Effects Models using lme4}
\Shorttitle{Linear Mixed Models with lme4}
\Abstract{%

  Maximum likelihood or restricted maximum likelihood (REML) estimates
  of the parameters in linear mixed-effects models can be determined
  using the \code{lmer} function in the \pkg{lme4} package for
  \proglang{R}. As for most model-fitting functions in \proglang{R},
  the model is described in an \code{lmer} call by a formula, in this
  case including both fixed- and random-effects
  terms. The formula and data together determine a numerical
  representation of the model from which the profiled deviance or the
  profiled REML criterion can be evaluated as a function of some of
  the model parameters.  The appropriate criterion is optimized, using
  one of the constrained optimization functions in \proglang{R}, to
  provide the parameter estimates.  We describe the structure of the
  model, the steps in evaluating the profiled deviance or REML
  criterion, and the structure of classes or types that represents such
  a model.  Sufficient detail is included to allow specialization of
  these structures by users who wish to write functions to fit
  specialized linear mixed models, such as models incorporating
  pedigrees or smoothing splines, that are not easily expressible in
  the formula language used by \code{lmer}.}

\Keywords{%
  sparse matrix methods, linear mixed models, penalized least squares,
  Cholesky decomposition} \Address{
  Douglas Bates\\
  Department of Statistics, University of Wisconsin\\
  1300 University Ave.\\
  Madison, WI 53706, U.S.A.\\
  E-mail: \email{bates@stat.wisc.edu}\\
  \par\bigskip
  Martin M\"achler\\
  Seminar f\"ur Statistik, HG G~16\\
  ETH Zurich\\
  8092 Zurich, Switzerland\\
  E-mail: \email{maechler@stat.math.ethz.ch}\\
  % URL: \url{http://stat.ethz.ch/people/maechler}\\
  \par\bigskip
  Benjamin M. Bolker\\
  Departments of Mathematics \& Statistics and Biology \\
  McMaster University \\
  1280 Main Street W \\
  Hamilton, ON L8S 4K1, Canada \\
  E-mail: \email{bolker@mcmaster.ca}\\
  \par\bigskip
  Steven C. Walker\\
  Department of Mathematics \& Statistics \\
  McMaster University \\
  1280 Main Street W \\
  Hamilton, ON L8S 4K1, Canada \\
  E-mail: \email{scwalker@math.mcmaster.ca }
}
\newcommand{\thetavec}{{\bm\theta}}
\newcommand{\betavec}{{\bm\beta}}
\newcommand{\Var}{\operatorname{Var}}
\newcommand{\abs}{\operatorname{abs}}
\newcommand{\bLt}{\ensuremath{\bm\Lambda_{\bm\theta}}}
\newcommand{\mc}[1]{\ensuremath{\mathcal{#1}}}
\newcommand{\trans}{\ensuremath{^\top}} %  JSS wants \top
\newcommand{\yobs}{\ensuremath{\bm y_{\mathrm{obs}}}}
\newcommand*{\eq}[1]{eqn.~\ref{#1}}% or just {(\ref{#1})}
<<preliminaries,include=FALSE,cache=FALSE,message=FALSE>>=
options(width=70, show.signif.stars=FALSE,
        str=strOptions(strict.width="cut"),
        ## prefer empty continuation for reader's cut'n'paste:
        continue = "   ", #JSS: prompt = "R> ", continue = "+  ",
        useFancyQuotes = FALSE)
library("knitr")
library("lme4")
library("ggplot2")# Keeping default theme, nicer "on line":
#JSS theme_set(theme_bw())
library("grid")
zmargin <- theme(panel.spacing=unit(0,"lines"))
library("lattice")
library("minqa")
library("reformulas")
opts_chunk$set(engine='R',dev='pdf', fig.width=9, fig.height=5.5,
               prompt=TRUE, cache=TRUE, tidy=FALSE, comment=NA, error = FALSE)
knitr::render_sweave()
@
\setkeys{Gin}{width=\textwidth}
\setkeys{Gin}{height=3.5in}

\begin{document}

A version of this manuscript has been published online in the
\emph{Journal of Statistical Software}, on Oct.\ 2015, with DOI \linebreak[3]
\texttt{10.18637/jss.v067.i01},
see \url{https://www.jstatsoft.org/article/view/v067i01/}.

\section{Introduction}
\label{sec:intro}

The \pkg{lme4} package  \citep{lme4} for \proglang{R} \citep{R} provides functions to fit and
analyze linear mixed models, generalized linear mixed models and
nonlinear mixed models.  In each of these names, the term ``mixed''
or, more fully, ``mixed effects'', denotes a model that incorporates
both fixed- and random-effects terms in a linear predictor expression
from which the conditional mean of the response can be evaluated.  In
this paper we describe the formulation and representation of linear
mixed models.  The techniques used for generalized linear and
nonlinear mixed models will be described separately, in a future
paper.

At present, the main alternative to \pkg{lme4} for mixed modeling in
\proglang{R} is the \pkg{nlme} package \citep{nlme_pkg}.  The main features
distinguishing \pkg{lme4} from \pkg{nlme} are (1) more efficient
linear algebra tools, giving improved performance on large problems;
(2) simpler syntax and more efficient implementation for fitting
models with crossed random effects; (3) the implementation of profile
likelihood confidence intervals on random-effects parameters; and (4)
the ability to fit generalized linear mixed models (although in this
paper we restrict ourselves to linear mixed models).  The main
advantage of \pkg{nlme} relative to \pkg{lme4} is a user interface for
fitting models with structure in the residuals (various forms of
heteroscedasticity and autocorrelation) and in the random-effects
covariance matrices (e.g., compound symmetric models).  With some extra
effort, the computational machinery of \pkg{lme4} can be used to fit
structured models that the basic \code{lmer} function cannot handle
(see Appendix~\ref{sec:modularExamples}).


The development of general software for fitting mixed models remains
an active area of research with many open problems.  Consequently, the
\pkg{lme4} package has evolved since it was first released, and
continues to improve as we learn more about mixed models. However, we
recognize the need to maintain stability and backward compatibility of
\pkg{lme4} so that it continues to be broadly useful. In order to
maintain stability while continuing to advance mixed-model
computation, we have developed several additional frameworks that draw
on the basic ideas of \pkg{lme4} but modify its structure or
implementation in various ways. These descendants include the
\mbox{\pkg{MixedModels}} package \citep{MixedModels} in \proglang{Julia}
\citep{Julia}, the
\pkg{lme4pureR} package \citep{lme4pureR} in \proglang{R}, and the \pkg{flexLambda}
development branch of \pkg{lme4}.  The current article is largely
restricted to describing the current stable version of the \pkg{lme4}
package (1.1-7), with Appendix~\ref{sec:modularExamples} describing
hooks into the computational machinery that are designed for extension
development. The \pkg{gamm4} \citep{gamm4} and \pkg{blme} \citep{blme, blme2}
packages currently make use of these hooks.

Another goal of this article is to contrast the approach used by
\pkg{lme4} with previous formulations of mixed models.  The
expressions for the profiled log-likelihood and profiled REML
(restricted maximum likelihood) criteria derived in
Section~\ref{sec:profdev} are similar to those presented in
\citet{bates04:_linear} and, indeed, are closely related to
``Henderson's mixed-model equations''~\citep{henderson_1982}.
Nonetheless there are subtle but important changes in the formulation
of the model and in the structure of the resulting penalized least
squares (PLS) problem to be solved (Section~\ref{sec:PLSpureR}).  We
derive the current version of the PLS problem
(Section~\ref{sec:plsMath}) and contrast this result with earlier
formulations (Section~\ref{sec:previous_lmm_form}).

This article is organized into four main sections (Sections~\ref{sec:lFormula},
\ref{sec:mkLmerDevfun}, \ref{sec:optimizeLmer}, and \ref{sec:mkMerMod}),
each of which corresponds to one of the four largely separate modules
that comprise \pkg{lme4}.  Before describing the details of each
module, we describe the general form of the linear mixed model
underlying \pkg{lme4} (Section~\ref{sec:LMMs}); introduce the
\code{sleepstudy} data that will be used as an example throughout
(Section~\ref{sec:sleepstudy}); and broadly outline \pkg{lme4}'s
modular structure (Section~\ref{sec:modular}).

\subsection{Linear mixed models}
\label{sec:LMMs}

Just as a linear model is described by the distribution of a
vector-valued random response variable, $\mc{Y}$, whose observed value
is $\yobs$, a linear mixed model is described by the distribution of
two vector-valued random variables: $\mc{Y}$, the response, and
$\mc{B}$, the vector of random effects.  In a linear model the
distribution of $\mc Y$ is multivariate normal,%\begin{linenomath}
\begin{equation}
  \label{eq:linearmodel}
  \mc Y\sim\mc{N}(\bm X\bm\beta+\bm o,\sigma^2\bm W^{-1}),
\end{equation}
where $n$ is the dimension of the response vector, $\bm W$ is a
diagonal matrix of known prior weights, $\bm\beta$ is a
$p$-dimensional coefficient vector, $\bm X$ is an $n\times p$ model
matrix, and $\bm o$ is a vector of known prior offset terms. The
parameters of the model are the coefficients $\bm\beta$ and the scale
parameter $\sigma$.

In a linear mixed model it is the \emph{conditional} distribution of
$\mc Y$ given $\mc B=\bm b$ that has such a form,
\begin{equation}
  \label{eq:LMMcondY}
  ( \mc Y|\mc B=\bm b)\sim\mc{N}(\bm X\bm\beta+\bm Z\bm b+\bm o,\sigma^2\bm W^{-1}), % | <- for ESS
\end{equation}
where $\bm Z$ is the $n\times q$ model matrix for the $q$-dimensional
vector-valued random-effects variable, $\mc B$, whose value we are
fixing at $\bm b$.  The unconditional distribution of $\mc B$ is also
multivariate normal with mean zero and a parameterized $q\times q$
variance-covariance matrix, $\bm\Sigma$,
\begin{equation}
  \label{eq:LMMuncondB}
  \mc B\sim\mc N(\bm0,\bm\Sigma) .
\end{equation}
As a variance-covariance matrix, $\bm\Sigma$ must be positive
semidefinite.  It is convenient to express the model in terms of a
\emph{relative covariance factor}, $\bLt$, which is a
$q\times q$ matrix, depending on the \emph{variance-component
  parameter}, $\bm\theta$, and generating the symmetric $q\times q$
variance-covariance matrix, $\bm\Sigma$, according to%\begin{linenomath}
\begin{equation}
  \label{eq:relcovfac}
  \bm\Sigma_{\bm\theta}=\sigma^2\bLt\bLt\trans ,
\end{equation}%\end{linenomath}
where $\sigma$ is the same scale factor as in the conditional
distribution (\ref{eq:LMMcondY}).

Although Equations~\ref{eq:LMMcondY}, \ref{eq:LMMuncondB}, and
\ref{eq:relcovfac} fully describe the class of linear mixed models
that \pkg{lme4} can fit, this terse description
hides many important details.  Before moving on to these details, we
make a few observations:
\begin{itemize}
\item This formulation of linear mixed models allows for a relatively
  compact expression for the profiled log-likelihood of $\bm\theta$
  (Section~\ref{sec:profdev}, Equation~\ref{eq:profiledDeviance}).
\item The matrices associated with random effects, $\bm Z$ and $\bLt$,
  typically have a sparse structure with a sparsity pattern that
  encodes various model assumptions.  Sections~\ref{sec:LMMmatrix} and
  \ref{sec:CSCmats} provide details on these structures, and how to
  represent them efficiently.
\item The interface provided by \pkg{lme4}'s \code{lmer} function
  is slightly less general than the model described by Equations~\ref{eq:LMMcondY}, \ref{eq:LMMuncondB}, and \ref{eq:relcovfac}.  To
  take advantage of the entire range of possibilities, one may use the
  modular functions (Sections~\ref{sec:modular} and
  Appendix~\ref{sec:modularExamples}) or explore the
  experimental \pkg{flexLambda} branch of \pkg{lme4} on \github.
\end{itemize}

\subsection{Example}
\label{sec:sleepstudy}

Throughout our discussion of \pkg{lme4}, we will work with a data set
on the average reaction time per day for subjects in a sleep
deprivation study \citep{sleepstudy}. On day 0 the subjects had their
normal amount of sleep.  Starting that night they were restricted to 3
hours of sleep per night.  The response variable, \code{Reaction},
represents average reaction times in milliseconds (ms) on a series of tests given each
\code{Day} to each \code{Subject} (Figure~\ref{fig:sleepPlot}),
%
<<sleep>>=
str(sleepstudy)
@

<<sleepPlot, echo=FALSE, fig.scap="Reaction time versus days by subject", fig.cap="Average reaction time versus days of sleep deprivation by subject.  Subjects ordered (from left to right starting on the top row) by increasing slope of subject-specific linear regressions.", fig.align='center', fig.pos='tb'>>=
## BMB: seemed more pleasing to arrange by increasing slope rather than
## intercept ...
xyplot(Reaction ~ Days | Subject, sleepstudy, aspect = "xy",
       layout = c(9, 2), type = c("g", "p", "r"),
       index.cond = function(x, y) coef(lm(y ~ x))[2],
       xlab = "Days of sleep deprivation",
       ylab = "Average reaction time (ms)",
       as.table = TRUE)
@
% |

Each subject's reaction time increases approximately linearly
with the number of sleep-deprived days.
However, subjects also appear to vary in the slopes and intercepts
of these relationships, which suggests a model with random slopes and intercepts.
As we shall see, such a model may be fitted by minimizing the
REML criterion (Equation~\ref{eq:REMLdeviance}) using
<<outputExample,results="hide">>=
fm1 <- lmer(Reaction ~ Days + (Days | Subject), sleepstudy)
@
% |
The estimates of the standard deviations of the random effects for the
intercept and the slope are
\Sexpr{round(sqrt(VarCorr(fm1)$Subject[1,1]), 2)} ms % $
and
\Sexpr{round(sqrt(VarCorr(fm1)$Subject[2,2]), 2)} ms/day. % $
The fixed-effects coefficients, $\betavec$, are
\Sexpr{round(fixef(fm1)[1], 1)} ms and
\Sexpr{round(fixef(fm1)[2], 2)} ms/day
for the intercept and slope.  In this model, one interpretation of
these fixed effects is that they are the estimated population mean values of the
random intercept and slope (Section~\ref{sec:intuitiveFormulas}).

We have chosen the \code{sleepstudy} example because it is a
relatively small and simple example to illustrate the theory and
practice underlying \code{lmer}.  However, \code{lmer} is capable of
fitting more complex mixed models to larger data sets.  For example, we
direct the interested reader to \code{RShowDoc("lmerperf", package =
  "lme4")} for examples that more thoroughly exercise the
performance capabilities of \code{lmer}.

\subsection{High-level modular structure}
\label{sec:modular}

The \code{lmer} function is composed of four largely independent
modules.  In the first module, a mixed-model formula is parsed and
converted into the inputs required to specify a linear mixed model
(Section~\ref{sec:lFormula}).  The second module uses these inputs to
construct an \proglang{R} function which takes the covariance
parameters, $\bm\theta$, as arguments and returns negative twice the log
profiled likelihood or the REML criterion
(Section~\ref{sec:mkLmerDevfun}).  The third module optimizes this
objective function to produce maximum likelihood (ML) or REML
estimates of $\bm\theta$ (Section~\ref{sec:optimizeLmer}).  Finally,
the fourth module provides utilities for interpreting the optimized
model (Section~\ref{sec:mkMerMod}).

\begin{table}[tb]
  \centering
  \begin{tabular}{lllp{2.1in}}
    \hline
    Module & & \proglang{R} function & Description \\
    \hline
    Formula module & (Section~\ref{sec:lFormula}) & \code{lFormula} &
    Accepts a mixed-model formula, data, and other user inputs, and returns
    a list of objects required to fit a linear mixed model. \\
    Objective function module & (Section~\ref{sec:mkLmerDevfun}) &
    \code{mkLmerDevfun} &
    Accepts the results of \code{lFormula} and returns a function to
    calculate the deviance (or restricted deviance) as a function of the
    covariance parameters, $\bm\theta$.\\
    Optimization module & (Section~\ref{sec:optimizeLmer}) &
    \code{optimizeLmer} &
    Accepts a deviance function returned by \code{mkLmerDevfun} and
    returns the results of the optimization of that deviance function. \\
    Output module & (Section~\ref{sec:mkMerMod}) & \code{mkMerMod} &
    Accepts an optimized deviance function and packages the results
    into a useful object.  \\
    \hline
  \end{tabular}
  \caption{The high-level modular structure of \code{lmer}.}
  \label{tab:modular}
\end{table}

To illustrate this modularity, we recreate the \code{fm1} object by a
series of four modular steps; the formula module,
<<modularExampleFormula, tidy=FALSE, eval=FALSE>>=
parsedFormula <- lFormula(formula = Reaction ~ Days + (Days | Subject),
                             data = sleepstudy)
@
the objective function module,
<<modularExampleObjective, tidy=FALSE, eval=FALSE>>=
devianceFunction <- do.call(mkLmerDevfun, parsedFormula)
@
the optimization module,
<<modularExampleOptimization, tidy=FALSE, eval=FALSE>>=
optimizerOutput <- optimizeLmer(devianceFunction)
@
and the output module,
<<modularExampleOutput, tidy=FALSE, eval=FALSE>>=
mkMerMod(   rho = environment(devianceFunction),
            opt = optimizerOutput,
         reTrms = parsedFormula$reTrms,
             fr = parsedFormula$fr)
@
% |

\section{Formula module}
\label{sec:lFormula}

\subsection{Mixed-model formulas}
\label{sec:formulas}

Like most model-fitting functions in \proglang{R},
\code{lmer} takes as its first two arguments a \emph{formula}
specifying the model and the \emph{data} with which to evaluate the
formula. This second argument, \code{data}, is optional but
recommended and is usually the name of an \proglang{R} data frame.
In the \proglang{R} \code{lm} function for fitting linear models,
formulas take the form \verb+resp ~ expr+,
where \code{resp} determines the response variable and \code{expr}
is an expression that specifies the columns of the model matrix.
Formulas for the \code{lmer} function contain special random-effects
terms,
<<MMformula, eval=FALSE>>=
resp ~ FEexpr + (REexpr1 | factor1) + (REexpr2 | factor2) + ...
@
where \code{FEexpr} is an expression determining the columns of the
fixed-effects model matrix, $\bm X$, and the random-effects terms,
\code{(REexpr1 | factor1)} and \code{(REexpr2 | factor2)}, determine
both the random-effects model matrix, $\bm Z$ (Section~\ref{sec:mkZ}),
and the structure of the relative covariance factor,
$\bLt$ (Section~\ref{sec:mkLambdat}).  In principle, a
mixed-model formula may contain arbitrarily many random-effects terms,
but in practice the number of such terms is typically low.

\subsection{Understanding mixed-model formulas}
\label{sec:intuitiveFormulas}

Before describing the details of how \pkg{lme4} parses mixed-model
formulas (Section~\ref{sec:LMMmatrix}), we provide an informal
explanation and then some examples. Our discussion assumes familiarity
with the standard \proglang{R} modeling paradigm
\citep{Chambers:1993}.

Each random-effects term is of the form \code{(expr | factor)}.
The expression \code{expr} is evaluated as a linear model formula,
producing a model matrix following the same rules used in standard
\proglang{R} modeling functions (e.g., \code{lm} or \code{glm}). The
expression \code{factor} is evaluated as an \proglang{R} factor.  One
way to think about the vertical bar operator is as a special kind of
interaction between the model matrix and the grouping factor. This
interaction ensures that the columns of the model matrix have
different effects for each level of the grouping factor.  What makes
this a special kind of interaction is that these effects are modeled
as unobserved random variables, rather than unknown fixed parameters.
Much has been written about important practical and philosophical
differences between these two types of interactions
\citep[e.g., ][]{henderson_1982,gelman2005analysis}. For
example, the random-effects implementation of such interactions can be
used to obtain shrinkage estimates of regression coefficients
\citep[e.g., ][]{1977EfronAndMorris}, or account
for lack of independence in the residuals due to block structure or
repeated measurements \citep[e.g., ][]{laird_ware_1982}.

Table~\ref{tab:formulas} provides several examples of the
right-hand-sides of mixed-model formulas. The first example,
\code{(1 | g)}, % |
is the simplest possible mixed-model formula, where each level of the
grouping factor, \code{g}, has its own random intercept. The mean and
standard deviation of these intercepts are parameters to be estimated.
Our description of this model incorporates any nonzero mean of the
random effects as fixed-effects parameters.  If one wishes to specify
that a random intercept has \emph{a priori} known means, one may use
the \code{offset} function as in the second  model in
Table~\ref{tab:formulas}.  This model contains no fixed effects, or
more accurately the fixed-effects model matrix, $\bm X$, has zero
columns and $\bm\beta$ has length zero.

\begin{table}[tb]
  \centering
  \begin{tabular}{llP{1.5in}} %% see new column type for ragged right
    \hline
    Formula 			& Alternative & Meaning \\
    \hline%------------------------------------------------
    \code{(1 | g)}		& \code{1 + (1 | g)}
    & Random intercept with fixed mean. \\
    \code{0 + offset(o) + (1 | g)}    & \code{-1 + offset(o) + (1 | g)}
    & Random intercept with \emph{a priori} means. \\
    \code{(1 | g1/g2)} 		& \code{(1 | g1)+(1 | g1:g2)}   % |
    & Intercept varying among \code{g1} and \code{g2} within \code{g1}. \\
    \code{(1 | g1) + (1 | g2)} 	& \code{1 + (1 | g1) + (1 | g2)}. &
    Intercept varying among \code{g1} and \code{g2}. \\
    \code{x + (x | g)} 		& \code{1 + x + (1 + x | g)} &
    Correlated random intercept and slope. \\
    \code{x + (x || g)} 		& \code{1 + x + (1 | g) + (0 +
      x | g)}
    & Uncorrelated random intercept and slope. \\
    \hline
  \end{tabular}
  \caption{Examples of the right-hand-sides of mixed-effects model
    formulas. The names of grouping factors are denoted \code{g},
    \code{g1}, and \code{g2}, and covariates and \emph{a priori} known
    offsets as \code{x} and \code{o}.}
  \label{tab:formulas}
\end{table}

We may also construct models with multiple grouping factors. For
example, if the observations are grouped by \code{g2}, which is nested
within \code{g1}, then the third formula in Table \ref{tab:formulas}
can be used to model variation in the intercept.  A common objective
in mixed modeling is to account for such nested (or hierarchical)
structure.  However, one of the most useful aspects of \pkg{lme4} is
that it can be used to fit random effects associated with non-nested
grouping factors.  For example, suppose the data are grouped by fully
crossing two factors, \code{g1} and \code{g2}, then the fourth formula
in Table \ref{tab:formulas} may be used.  Such models are common in
item response theory, where \code{subject} and \code{item} factors are
fully crossed \citep{doran2007estimating}. In addition to varying
intercepts, we may also have varying slopes (e.g., the
\code{sleepstudy} data, Section~\ref{sec:sleepstudy}).  The fifth
example in Table~\ref{tab:formulas} gives a model where both the
intercept and slope vary among the levels of the grouping factor.

\subsubsection{Specifying uncorrelated random effects}
\label{sec:uncor}

By default, \pkg{lme4} assumes that all coefficients associated with
the same random-effects term are correlated. To specify an
uncorrelated slope and intercept (for example), one may either use
double-bar notation, \code{(x || g)}, or equivalently use multiple
random-effects terms, \code{x + (1 | g) + (0 + x | g)}, as in the
final example of Table~\ref{tab:formulas}.  For example, if one
examined the results of model \code{fm1} of the \code{sleepstudy} data
(Section~\ref{sec:sleepstudy}) using \code{summary(fm1)}, one would
see that the estimated correlation between the slope for \code{Days}
and the intercept is fairly low
(\Sexpr{round(attr(VarCorr(fm1)$Subject, "correlation")[2],3)}) % $
(See Section~\ref{sec:summary} below for more on how to extract the
random-effects covariance matrix.)  We may use double-bar notation to
fit a model that excludes a correlation parameter:
<<uncorrelatedModel,results="hide">>=
fm2 <- lmer(Reaction ~ Days + (Days || Subject), sleepstudy)
@

Although mixed models where the random slopes and intercepts are
assumed independent are commonly used to reduce the complexity of
random-slopes models, they do have one subtle drawback.  Models in
which the slopes and intercepts are allowed to have a nonzero
correlation (e.g., \code{fm1}) are invariant to additive shifts of the
continuous predictor (\code{Days} in this case).  This invariance
breaks down when the correlation is constrained to zero; any shift in
the predictor will necessarily lead to a change in the estimated
correlation, and in the likelihood and predictions of the model.  For
example, we can eliminate the correlation in \code{fm1} simply by
adding an amount equal to the ratio of the estimated
among-subject standard deviations multiplied by the estimated
correlation (i.e., $\sigma_{\text{\small slope}}/\sigma_{\text{\small
    intercept}} \cdot \rho_{\text{\small slope:intercept}}$) to the \code{Days} variable.  The use
of models such as \code{fm2} should ideally be restricted to cases where the
predictor is measured on a ratio scale (i.e., the zero point on the
scale is meaningful, not just a location defined by convenience or
convention), as is the case here.

%% <<uncorrelatedModelByShifting>>=
%% sleepstudyShift <- within(sleepstudy, {
%%     Days <- Days + (24.74*0.07)/5.92 })
%% lmer(Reaction ~ Days + (Days |  Subject), sleepstudyShift)
%% @

\subsection{Algebraic and computational account of mixed-model formulas}
\label{sec:LMMmatrix}

The fixed-effects terms of a mixed-model formula are parsed to produce
the fixed-effects model matrix, $\bm X$, in the same way that the
\proglang{R} \code{lm} function generates model matrices.  However, a
mixed-model formula incorporates $k\ge1$ random-effects terms of the
form \code{(r | f)} as well. % |
These $k$ terms are used to produce the random-effects model matrix,
$\bm Z$ (Equation~\ref{eq:LMMcondY}; Section~\ref{sec:mkZ}), and the
structure of the relative covariance factor, $\bLt$
(Equation~\ref{eq:relcovfac}; Section~\ref{sec:mkLambdat}), which are
matrices that typically have a sparse structure.  We now describe how
one might construct these matrices from the random-effects terms,
considering first a single term, \code{(r | f)}, % |
and then generalizing to multiple terms. Tables~\ref{tab:dim} and
\ref{tab:algebraic} summarize the matrices and vectors that determine
the structure of $\bm Z$ and
$\bLt$.

\begin{table}[tb]
  \centering
  \begin{tabular}{lll}
    \hline
    Symbol & Size \\
    \hline
    $n$ & Length of the response vector, $\mc{Y}$ \\
    $p$ & Number of columns of fixed-effects model matrix, $\bm X$ \\
    $q = \sum_i^k q_i$ & Number of columns of random-effects model matrix, $\bm Z$ \\
    $p_i$ &  Number of columns of the raw model matrix, $\bm X_i$ \\
    $\ell_i$ &  Number of levels of the grouping factor indices, $\bm i_i$ \\
    $q_i = p_i\ell_i$  & Number of columns of the term-wise model matrix, $\bm Z_i$ \\
    $k$ &  Number of random-effects terms \\
    $m_i = \binom{p_i+1}{2}$ & Number of covariance parameters for term
    $i$ \\
    $m = \sum_i^k m_i$ & Total number of covariance parameters \\
    \hline
  \end{tabular}
  \caption{Dimensions of linear mixed models.  The
    subscript $i = 1, \dots, k$ denotes a specific random-effects term.}
  \label{tab:dim}
\end{table}

\begin{table}[tb]
  \centering
  \begin{tabular}{lll}
    \hline
    Symbol  & Size & Description \\
    \hline
    $\bm X_i$ & $n\times p_i$ &  Raw random-effects model matrix \\
    $\bm J_i$  & $n\times \ell_i$ & Indicator matrix of grouping factor indices\\
    $\bm X_{ij}$ & $p_i\times 1$ & Column vector containing $j$th row of $\bm X_i$ \\
    $\bm J_{ij}$ & $\ell_i\times 1$ & Column vector containing $j$th row of $\bm J_i$ \\
    $\bm i_i$  & $n$ & Vector of grouping factor indices \\
    $\bm Z_i$ & $n\times q_i$ & Term-wise random-effects model matrix \\
    $\bm\theta$ & $m$ & Covariance parameters \\
    $\bm T_i$ & $p_i\times p_i$ & Lower triangular template matrix \\
    $\bm\Lambda_i$ & $q_i\times q_i$ & Term-wise relative covariance factor \\
    \hline
  \end{tabular}
  \caption{Symbols used to describe the structure of the random-effects model matrix and the relative covariance factor. The
    subscript $i = 1, \dots, k$ denotes a specific random-effects term.}
  \label{tab:algebraic}
\end{table}

The expression, \code{r}, is a linear model formula that evaluates to
an \proglang{R} model matrix, $\bm X_i$, of size $n\times p_i$, called
the \emph{raw random-effects model matrix} for term $i$. A term is
said to be a \emph{scalar} random-effects term when $p_i=1$, otherwise
it is \emph{vector-valued}.  For a \emph{simple, scalar}
random-effects term of the form \code{(1 | f)}, $\bm X_i$ is the    % |
$n\times 1$ matrix of ones, which implies a random intercept model.

The expression \code{f} evaluates to an \proglang{R} factor, called
the \emph{grouping factor}, for the term. For the $i$th term, we
represent this factor mathematically with a vector $\bm i_i$ of
\emph{factor indices}, which is an $n$-vector of values from
$1,\dots,\ell_i$.\footnote{In practice,
  fixed-effects model matrices and random-effects terms are evaluated
  with respect to a \emph{model frame}, ensuring that any expressions
  for grouping factors have been coerced to factors and any unused
  levels of these factors have been dropped.  That is, $\ell_i$, the
  number of levels in the grouping factor for the $i$th random-effects
  term, is well-defined.}
Let $\bm J_i$ be the $n\times \ell_i$ matrix of
indicator columns for $\bm i_i$.  Using the \pkg{Matrix} package
\citep{Matrix_pkg} in
\proglang{R}, we may construct the transpose of $\bm J_i$ from a
factor vector, \code{f}, by coercing \code{f} to a `\code{sparseMatrix}'
object.  For example,
<<setSeed, echo=FALSE>>=
set.seed(2)
@
<<factorToSparseMatrix>>=
(f <- gl(3, 2))
(Ji <- t(as(f, Class = "sparseMatrix")))
@
When $k>1$ we order the random-effects terms so that
$\ell_1\ge\ell_2\ge\dots\ge\ell_k$; in general, this ordering reduces
``fill-in'' (i.e., the proportion of elements that are zero in the
lower triangle of $\bLt\trans\bm Z\trans\bm W\bm Z\bLt+\bm I$ but not in
the lower triangle of its left Cholesky factor, $\bm L_{\bm\theta}$,
described below in Equation~\ref{eq:blockCholeskyDecomp}). This reduction
in fill-in provides more efficient matrix operations within the
penalized least squares algorithm (Section~\ref{sec:plsMath}).

\subsubsection{Constructing the random-effects model matrix}
\label{sec:mkZ}

The $i$th random-effects term contributes $q_i=\ell_ip_i$ columns to
the model matrix $\bm Z$.  We group these columns into a matrix, $\bm
Z_i$, which we refer to as the \emph{term-wise model matrix} for the
$i$th term.  Thus $q$, the number of columns in $\bm Z$ and the
dimension of the random variable, $\mc{B}$, is
\begin{equation}
  \label{eq:qcalc}
  q=\sum_{i=1}^k q_i = \sum_{i=1}^k \ell_i\,p_i .
\end{equation}
Creating the matrix $\bm Z_i$ from $\bm X_i$ and $\bm J_i$ is a
straightforward concept that is, nonetheless, somewhat awkward to
describe.  Consider $\bm Z_i$ as being further decomposed into
$\ell_i$ blocks of $p_i$ columns.  The rows in the first block are the
rows of $\bm X_i$ multiplied by the 0/1 values in the first column of
$\bm J_i$ and similarly for the subsequent blocks.  With these
definitions we may define the term-wise random-effects model matrix,
$\bm Z_i$, for the $i$th term as a transposed Khatri-Rao product,
\begin{equation}
  \label{eq:Zi}
  \bm Z_i = (\bm J_i\trans * \bm X_i\trans)\trans =
  \begin{bmatrix}
    \bm J_{i1}\trans \otimes \bm X_{i1}\trans \\
    \bm J_{i2}\trans \otimes \bm X_{i2}\trans \\
    \vdots \\
    \bm J_{in}\trans \otimes \bm X_{in}\trans \\
  \end{bmatrix},
\end{equation}
where $*$ and $\otimes$ are the Khatri-Rao\footnote{Note that the
  original definition of the Khatri-Rao product is more general than
  the definition used in the \pkg{Matrix} package, which is the
  definition we use here.} \citep{khatri1968solutions} and Kronecker
products, and $\bm J_{ij}\trans$ and $\bm X_{ij}\trans$ are row
vectors of the $j$th rows of $\bm J_i$ and $\bm X_i$.  These rows
correspond to the $j$th sample in the response vector, $\mc Y$, and
thus $j$ runs from $1, \dots, n$.  The \pkg{Matrix} package for
\proglang{R} contains a \code{KhatriRao} function, which can be used
to form $\bm Z_i$.  For example, if we begin with a raw model matrix,
<<rawModelMatrix>>=
(Xi <- cbind(1, rep.int(c(-1, 1), 3L)))
@
then the term-wise random-effects
model matrix is,
<<KhatriRao>>=
(Zi <- t(KhatriRao(t(Ji), t(Xi))))
@
<<sanityCheck, include=FALSE>>=
## alternative formulation of Zi (eq:Zi)
rbind(
  Ji[1,] %x% Xi[1,],
  Ji[2,] %x% Xi[2,],
  Ji[3,] %x% Xi[3,],
  Ji[4,] %x% Xi[4,],
  Ji[5,] %x% Xi[5,],
  Ji[6,] %x% Xi[6,])
@

In particular, for a simple, scalar term, $\bm Z_i$ is exactly $\bm
J_i$, the matrix of indicator columns.  For other scalar terms, $\bm
Z_i$ is formed by element-wise multiplication of the single column of
$\bm X_i$ by each of the columns of $\bm J_i$.

Because each $\bm Z_i$ is generated from indicator columns, its
cross-product, $\bm Z_i\trans\bm Z_i$ is block-diagonal consisting of
$\ell_i$ diagonal blocks each of size $p_i$.\footnote{To see this,
  note that by the properties of Kronecker products we may write the
  cross-product matrix $Z_i\trans Z_i$ as $\sum_{j=1}^n \bm J_{ij} \bm
  J_{ij}\trans \otimes \bm X_{ij} \bm X_{ij}\trans$. Because $\bm
  J_{ij}$ is a unit vector along a coordinate axis, the cross-product
  $\bm J_{ij} \bm J_{ij}\trans$ is a $p_i\times p_i$ matrix of all
  zeros except for a single $1$ along the diagonal. Therefore, the
  cross-products, $\bm X_{ij} \bm X_{ij}\trans$, will be added to one
  of the $\ell_i$ blocks of size $p_i\times p_i$ along the diagonal of
  $Z_i\trans Z_i$.} Note that this means that when $k=1$ (i.e., there
is only one random-effects term, and $\bm Z_i = \bm Z$), $\bm
Z\trans\bm Z$ will be block diagonal.  These block-diagonal properties
allow for more efficient sparse matrix computations
(Section~\ref{sec:CSCmats}).

The full random-effects model matrix, $\bm Z$, is constructed from
$k\ge 1$ blocks,
\begin{equation}
  \label{eq:Z}
  \bm Z =
  \begin{bmatrix}
    \bm Z_1 &
    \bm Z_2 &
    \hdots &
    \bm Z_k \\
  \end{bmatrix}.
\end{equation}
By transposing Equation~\ref{eq:Z} and substituting in Equation~\ref{eq:Zi}, we
may represent the structure of the transposed random-effects model
matrix as follows,
\begin{equation}
  \label{eq:Zt}
  \bm Z\trans =
  \begin{blockarray}{ccccc}
    \text{sample 1} & \text{sample 2} & \hdots & \text{sample } n & \\
    \begin{block}{[cccc]c}
    \bm J_{11} \otimes \bm X_{11} &
    \bm J_{12} \otimes \bm X_{12} &
    \hdots &
    \bm J_{1n} \otimes \bm X_{1n} &
    \text{term 1} \\
    \bm J_{21} \otimes \bm X_{21} &
    \bm J_{22} \otimes \bm X_{22} &
    \hdots &
    \bm J_{2n} \otimes \bm X_{2n} &
    \text{term 2} \\
    \vdots & \vdots & \ddots & \vdots & \vdots \\
    \end{block}
  \end{blockarray}.
\end{equation}
Note that the proportion of elements of $Z\trans$ that are structural
zeros is
\begin{equation}
  \label{eq:ZtSparsity}
  \frac{\sum_{i=1}^k p_i(\ell_i - 1)}{\sum_{i=1}^k p_i} \qquad .
\end{equation}
Therefore, the sparsity of $\bm Z\trans$ increases with the number of
grouping factor levels. As the number of levels is often large in
practice, it is essential for speed and efficiency to take account of
this sparsity, for example by using sparse matrix methods, when
fitting mixed models (Section~\ref{sec:CSCmats}).

\subsubsection{Constructing the relative covariance factor}
\label{sec:mkLambdat}

The $q\times q$ covariance factor, $\bLt$, is a block diagonal matrix
whose $i$th diagonal block, $\bm\Lambda_i$, is of size
$q_i,i=1,\dots,k$.  We refer to $\bm\Lambda_i$ as the \emph{term-wise
  relative covariance factor}.  Furthermore, $\bm\Lambda_i$ is a
homogeneous block diagonal matrix with each of the $\ell_i$
lower-triangular blocks on the diagonal being a copy of a $p_i\times
p_i$ lower-triangular \emph{template matrix}, $\bm T_i$.  The
covariance parameter vector, $\bm\theta$, of length $m_i
=\binom{p_i+1}{2}$, consists of the elements in the lower triangle of
$\bm T_i,i=1,\dots,k$.  To provide a unique representation we require
that the diagonal elements of the $\bm T_i,i=1,\dots,k$ be
non-negative.

The template, $\bm T_i$, can be constructed from the number $p_i$
alone.  In \proglang{R} code we denote $p_i$ as \code{nc}.  For
example, if we set \code{nc <- 3}\Sexpr{nc <- 3}, we could create the
template for term $i$ as,
<<nc, echo=FALSE>>=
nc <- 3
@
%% sequence() is equivalent to unlist(lapply(nvec, seq_len))
%% and (?sequence) ``mainly exists in reverence to the very early history of R''
%% scw: i like sequence, and in fact i never understood why that
%% statement is there in the help file.
<<template>>=
(rowIndices <- rep(1:nc, 1:nc))
(colIndices <- sequence(1:nc))
(template <- sparseMatrix(rowIndices, colIndices,
                          x = 1 * (rowIndices == colIndices)))
@
Note that the \code{rowIndices} and \code{colIndices} fill the entire
lower triangle, which contains the initial values of the covariance
parameter vector, $\bm\theta$,
<<thetaFromTemplate>>=
(theta <- template@x)
@
<<thetaSanityCheck, include=FALSE>>=
lFormula(Reaction ~ (Days + I(Days^2) | Subject), sleepstudy)$reTrms$theta
@
(because the \code{@x} slot of the sparse matrix \code{template} is a
numeric vector containing the nonzero elements). This template
contains three types of elements: structural zeros (denoted
by~\code{.}), off-diagonal covariance parameters (initialized at
\code{0}), and diagonal variance parameters (initialized at \code{1}).
The next step in the construction of the relative covariance factor is
to repeat the template once for each level of the grouping factor to
construct a sparse block diagonal matrix.  For example, if we set the
number of levels, $\ell_i$, to two, \code{nl <- 2}\Sexpr{nl <- 2}, we
could create the transposed term-wise relative covariance factor,
$\bm\Lambda_i\trans$, using the \code{.bdiag} function in the
\pkg{Matrix} package,
<<nl, echo=FALSE>>=
nl <- 2
@
<<relativeCovarianceBlock>>=
(Lambdati <- .bdiag(rep(list(t(template)), nl)))
@
For a model with a single random-effects term, $\bm\Lambda\trans$
would be the initial transposed relative covariance factor
$\bm\Lambda_{\bm\theta}\trans$ itself.

The transposed relative covariance factor, $\bLt\trans$, that arises
from parsing the formula and data is set at the initial value of the
covariance parameters, $\bm\theta$.  However, during model fitting, it
needs to be updated to a new $\bm\theta$ value at each iteration (see
Section~\ref{sec:plsI}).  This is achieved by constructing a vector of
indices, \code{Lind}, that identifies which elements of \code{theta}
should be placed in which elements of \code{Lambdat},
<<relativeCovarianceBlockIndices>>=
LindTemplate <- rowIndices + nc * (colIndices - 1) - choose(colIndices, 2)
(Lind <- rep(LindTemplate, nl))
@
For example, if we randomly generate a new value for \code{theta},
<<newTheta>>=
thetanew <- round(runif(length(theta)), 1)
@
we may update \code{Lambdat} as follows,
<<relativeCovarianceBlockUpdate>>=
Lambdati@x <- thetanew[Lind]
@
Section~\ref{sec:PLSpureR} describes the process of
updating the relative covariance factor in more detail.


\section{Objective function module}
\label{sec:mkLmerDevfun}

\subsection{Model reformulation for improved computational stability}
\label{sec:stability}

In our initial formulation of the linear mixed model
(Equations~\ref{eq:LMMcondY}, \ref{eq:LMMuncondB}, and \ref{eq:relcovfac}),
the covariance parameter vector, $\bm\theta$, appears only in the
marginal distribution of the random effects (Equation~\ref{eq:LMMuncondB}).
However, from the perspective of computational stability and
efficiency, it is advantageous to reformulate the model such that
$\bm\theta$ appears only in the conditional distribution for the
response vector given the random effects.  Such a reformulation allows
us to work with singular covariance matrices, which regularly arise in
practice (e.g., during intermediate steps of the nonlinear optimizer,
Section~\ref{sec:optimizeLmer}).

The reformulation is made by defining a \emph{spherical}%
\footnote{$\mathcal{N}(\bm\mu,\sigma^2\bm I)$ distributions are called
  ``spherical'' because contours of the probability density are
  spheres.}  \emph{random-effects} variable, $\mc U$, with
distribution%\begin{linenomath}
\begin{equation}
  \label{eq:distU}
  \mc U\sim\mathcal{N}(\bm 0,\sigma^2\bm I_q).
\end{equation}%\end{linenomath}
If we set,
% \begin{linenomath}
\begin{equation}
  \label{eq:BequalsLamU}
  \mc B=\bLt\mc U,
\end{equation}%\end{linenomath}
then $\mc B$ will have the desired $\mathcal{N}(\bm
0,\bm\Sigma_{\bm\theta})$ distribution (Equation~\ref{eq:LMMuncondB}).
Although it may seem more natural to define $\mc U$ in terms of $\mc
B$ we must write the relationship as in Equation~\ref{eq:BequalsLamU} to allow
for singular $\bLt$. The conditional distribution
(Equation~\ref{eq:LMMcondY}) of the response vector given the random
effects may now be reformulated as,%\begin{linenomath}
\begin{equation}
  \label{eq:distYgivenU}
  (\mc Y|\mc U=\bm u)\sim\mc{N}(\bm\mu_{\mc Y|\mc U=\bm u},\sigma^2\bm W^{-1}),
\end{equation}%\end{linenomath}
where
\begin{equation}
  \label{eq:meanYgivenU}
  \bm\mu_{\mc Y|\mc U=\bm u} = \bm X \bm\beta + \bm Z\bLt\bm u + \bm o
\end{equation} % |
is a vector of linear predictors, which can be interpreted as a
conditional mean (or mode).  Similarly, we also define $\bm\mu_{\mc
  U|\mc Y = \yobs}$ as the conditional mean (or mode) of the spherical
random effects given the observed value of the response vector. Note
also that we use the $\bm u$ symbol throughout to represent a specific
value of the random variable, $\mc U$.

\subsection{Penalized least squares}
\label{sec:plsMath}

Our computational methods for maximum likelihood fitting of the linear
mixed model involve repeated applications of the PLS method. In
particular, the PLS problem is to minimize the penalized weighted
residual sum-of-squares,
\begin{equation}
  \label{eq:pwrss}
  r^2(\bm\theta, \bm\beta, \bm u)
  = \rho^2(\bm\theta, \bm\beta, \bm u)
  + \left\| \bm u \right\|^2,
\end{equation}
over $\begin{bmatrix} \bm u \\
  \bm\beta \\ \end{bmatrix}$, where,
\begin{equation}
  \label{eq:wrss}
  \rho^2(\bm\theta, \bm\beta, \bm u) = \left\|
  \bm W^{1/2}\left[\yobs - \bm\mu_{\mc Y|\mc U=\bm u} \right]\right\|^2,
\end{equation} %|
is the weighted residual sum-of-squares. This notation makes explicit
the fact that $r^2$ and $\rho^2$ both depend on $\bm\theta$,
$\betavec$, and $\bm u$.  The reason for the word ``penalized'' is that
the term, $\left\| \bm u \right\|^2$, in Equation~\ref{eq:pwrss} penalizes models with
larger magnitude values of $\bm u$.

In the so-called ``pseudo-data'' approach we write the penalized
weighted residual sum-of-squares as the squared length of a block
matrix equation,
\begin{equation}
  \label{eq:pwrssPseudoData}
  r^2(\bm\theta,\bm\beta,\bm u) =
  \left\|
  \begin{bmatrix}
  \bm W^{1/2}(\yobs - \bm o) \\
  \bm 0
  \end{bmatrix} -
  \begin{bmatrix}
  \bm W^{1/2}\bm Z\bm\bLt & \bm W^{1/2}\bm X \\
  \bm I_q & \bm 0 \\
  \end{bmatrix}
  \begin{bmatrix}
  \bm u \\
  \bm\beta \\
  \end{bmatrix}
  \right\|^2.
\end{equation}
This pseudo-data approach shows that the PLS problem may also be
thought of as a standard least squares problem for an extended
response vector, which implies that the minimizing value,
$\begin{bmatrix}
  \bm\mu_{\mc U|\mc Y = \yobs} \\
  \widehat{\bm\beta}_{\bm\theta} \\ \end{bmatrix}$, satisfies the
normal equations,
\begin{equation}
  \label{eq:normalEquations}
  \begin{bmatrix}
    \bLt\trans \bm Z\trans \bm W (\yobs - \bm o) \\
    \bm X\trans \bm W (\yobs - \bm o) \\
  \end{bmatrix} =
  \begin{bmatrix}
    \bLt\trans \bm Z\trans \bm W \bm Z \bLt + \bm I &
    \bLt\trans \bm Z\trans \bm W \bm X \\
    \bm X\trans \bm W \bm Z \bLt &
    \bm X\trans \bm W \bm X \\
  \end{bmatrix}
  \begin{bmatrix}
    \mu_{\mc U | \mc Y = \yobs} \\ \widehat{\betavec}_{\bm\theta}
  \end{bmatrix} ,
\end{equation} % |
where $\mu_{\mc U | \mc Y = \yobs}$ is the conditional mean of $\mc U$
given that $\mc Y = \yobs$.  Note that this conditional mean depends
on $\bm\theta$, although we do not make this dependency explicit in
order to reduce notational clutter.

The cross-product matrix in Equation~\ref{eq:normalEquations} can be
Cholesky decomposed,
\begin{equation}
  \label{eq:blockCholeskyDecomp}
  \begin{bmatrix}
    \bLt\trans \bm Z\trans \bm W \bm Z \bLt + \bm I &
    \bLt\trans \bm Z\trans \bm W \bm X \\
    \bm X\trans \bm W \bm Z \bLt &
    \bm X\trans \bm W \bm X \\
  \end{bmatrix} =
  \begin{bmatrix}
    \bm L_{\bm\theta} & \bm 0 \\
    \bm R\trans_{ZX} & \bm R\trans_{X} \\
  \end{bmatrix}
  \begin{bmatrix}
    \bm L\trans_{\bm\theta} & \bm R_{ZX} \\
    \bm 0 & \bm R_{X}
  \end{bmatrix} .
\end{equation}
We may use this decomposition to rewrite the penalized weighted
residual sum-of-squares as,
\begin{equation}
  \label{eq:pwrssIdentity}
  \begin{aligned}
  r^2(\bm\theta,\bm\beta,\bm u) = r^2(\bm\theta) &
  + \left\|\bm L\trans_{\bm\theta} (\bm u - \bm\mu_{\mc U|\mc Y = \yobs})
  +    \bm R_{ZX} (\bm\beta - \widehat{\bm\beta}_{\bm\theta})\right\|^2
  + \left\|\bm R_{X} (\bm\beta - \widehat{\bm\beta}_{\bm\theta})\right\|^2,
  \end{aligned}
\end{equation} % |
where we have simplified notation by writing
$r^2(\bm\theta,\widehat{\bm\beta}_{\bm\theta},\bm\mu_{\mc U|\mc Y =
  \yobs})$ as $r^2(\bm\theta)$. This is an important expression in the
theory underlying \pkg{lme4}. It relates the penalized weighted
residual sum-of-squares, $r^2(\bm\theta,\bm\beta,\bm u)$, with its
minimum value, $r^2(\bm\theta)$.  This relationship is useful in the
next section where we integrate over the random effects, which is
required for maximum likelihood estimation.

\subsection{Probability densities}
\label{sec:densities}

The residual sums-of-squares discussed in the previous section can be
used to express various probability densities, which are required for
maximum likelihood estimation of the linear mixed model\footnote{These
  expressions only technically hold at the observed value, $\yobs$, of
  the response vector, $\mc Y$.},
\begin{align}
  \label{eq:densityYgivenU}
  f_{\mc Y | \mc U}(\yobs|\bm u) & =
  \frac{|\bm W|^{1/2}}{(2\pi\sigma^2)^{n/2}}
  \exp\left[\frac{-\rho^2(\bm\theta,\bm\beta,\bm u)}{2\sigma^2}\right] ,\\
  \label{eq:densityU}
  f_{\mc U}(\bm u) &=
  \frac{1}{(2\pi\sigma^2)^{q/2}}
  \exp\left[\frac{-\left\| \bm u \right\|^2}{2\sigma^2}\right] ,\\
  \label{eq:densityYU}
  f_{\mc Y , \mc U}(\yobs,\bm u) & =
  \frac{|\bm W|^{1/2}}{(2\pi\sigma^2)^{(n+q)/2}}
  \exp\left[\frac{-r^2(\bm\theta,\bm\beta,\bm u)}{2\sigma^2}\right] , \\
  \label{eq:densityUgivenY}
  f_{\mc U | \mc Y}(\bm u | \yobs) & =
  \frac{f_{\mc Y , \mc U}(\yobs,\bm u)}{f_{\mc Y}(\yobs)} , \\
\end{align}
where
\begin{equation}
  \label{eq:densityY}
  f_{\mc Y}(\yobs) =
  \int f_{\mc Y , \mc U}(\yobs,\bm u)d\bm u .
\end{equation}
The log-likelihood to be maximized can therefore be expressed as,
\begin{equation}
  \label{eq:likelihood}
  \mc L(\bm\theta,\bm\beta,\sigma^2 | \yobs) = \log f_{\mc Y}(\yobs).
\end{equation} % |

The integral in Equation~\ref{eq:densityY} may be more explicitly written
as,
\begin{equation}
  \label{eq:densityYexpand}
  \begin{aligned}
  f_{\mc Y}(\yobs) =
  \frac{|\bm W|^{1/2}}{(2\pi\sigma^2)^{(n+q)/2}} &
  \exp\left[\frac{
  -r^2(\bm\theta)
  - \left\|\bm R_{X} (\bm\beta - \widehat{\bm\beta}_{\bm\theta})\right\|^2
  }{2\sigma^2}\right] \\ &
    \int \exp\left[\frac{
  - \left\|\bm L\trans_{\bm\theta} (\bm u - \bm\mu_{\mc U|\mc Y = \yobs})
  +    \bm R_{ZX} (\bm\beta - \widehat{\bm\beta}_{\bm\theta})\right\|^2
  }{2\sigma^2}\right]
   d\bm u ,
  \end{aligned}
\end{equation} % |
which can be evaluated with the change of variables,
\begin{equation}
  \label{eq:changeOfVariables}
  \bm v = \bm L\trans_{\bm\theta} (\bm u - \mu_{\mc U|\mc Y = \yobs})
  +    \bm R_{ZX} (\bm\beta - \widehat{\bm\beta}_{\bm\theta}) .
\end{equation} % |
The Jacobian determinant of the transformation from $\bm u$ to $\bm
v$ is $|\bm L_{\bm\theta} |$. Therefore we are able to write the integral
as,
\begin{equation}
  \label{eq:densityYintermediate}
  \begin{aligned}
  f_{\mc Y}(\yobs) =
  \frac{|\bm W|^{1/2}}{(2\pi\sigma^2)^{(n+q)/2}} &
  \exp\left[\frac{
  -r^2(\bm\theta)
  - \left\|\bm R_{X} (\bm\beta - \widehat{\bm\beta}_{\bm\theta})\right\|^2
  }{2\sigma^2}\right] \\ &
    \int \exp\left[\frac{
  - \left\|\bm v\right\|^2
  }{2\sigma^2}\right]
   |\bm L_{\bm\theta}|^{-1} d\bm v ,
  \end{aligned}
\end{equation}
which by the properties of exponential integrands becomes,
\begin{equation}
  \label{eq:densityYfinal}
   \exp\mc L(\bm\theta,\bm\beta,\sigma^2 | \yobs) = f_{\mc Y}(\yobs) =
  \frac{|\bm W|^{1/2}|\bm L_{\bm\theta}|^{-1}}{(2\pi\sigma^2)^{n/2}}
  \exp\left[\frac{
  -r^2(\bm\theta)
  - \left\|\bm R_{X} (\bm\beta - \widehat{\bm\beta}_{\bm\theta})\right\|^2
  }{2\sigma^2}\right] .
\end{equation}

\subsection{Evaluating and profiling the deviance and the REML criterion}
\label{sec:profdev}

We are now in a position to understand why the formulation in
Equations~\ref{eq:LMMcondY} and \ref{eq:LMMuncondB} is particularly
useful. We are able to explicitly profile $\betavec$ and $\sigma$ out
of the log-likelihood (Equation~\ref{eq:likelihood}), to find a compact
expression for the profiled deviance (negative twice the profiled
log-likelihood) and the profiled REML criterion as a function of the
relative covariance parameters, $\bm\theta$, only.  Furthermore these
criteria can be evaluated quickly and accurately.

To estimate the parameters, $\bm\theta$, $\bm\beta$, and $\sigma^2$, we
minimize negative twice the log-likelihood, which can be written as,
\begin{equation}
  \label{eq:fullDeviance}
  -2\mc L(\bm\theta,\bm\beta,\sigma^2 | \yobs) =    % |
  \log\frac{|\bm L_{\bm\theta}|^2}{|\bm W|}
  + n\log(2\pi\sigma^2)
  + \frac{r^2(\bm\theta)}{\sigma^2}
  + \frac{ \left\|\bm R_{X} (\bm\beta -
    \widehat{\bm\beta}_{\bm\theta})\right\|^2}{\sigma^2} .
\end{equation}
It is very easy to profile out $\bm\beta$, because it enters into
the ML criterion only through the final term, which is zero if
$\bm\beta = \widehat{\bm\beta}_{\bm\theta}$, where
$\widehat{\bm\beta}_{\bm\theta}$ is found by solving the penalized
least-squares problem in Equation~\ref{eq:pwrssPseudoData}. Therefore we
can write a partially profiled ML criterion as,
\begin{equation}
  \label{eq:partiallyProfiledDeviance}
  -2\mc L(\bm\theta,\sigma^2 | \yobs)  =    % |
  \log\frac{|\bm L_{\bm\theta}|^2}{|\bm W|}
  + n\log(2\pi\sigma^2)
  + \frac{r^2(\bm\theta)}{\sigma^2} .
\end{equation}
This criterion is only partially profiled because it still depends on
$\sigma^2$. Differentiating this criterion with respect to $\sigma^2$
and setting the result equal to zero yields,
\begin{equation}
  \label{eq:residualVarianceEstimatingEquation}
  0 = \frac{n}{\widehat{\sigma}_{\bm\theta}^2} -
  \frac{r^2(\bm\theta)}{\widehat{\sigma}_{\bm\theta}^4},
\end{equation}
which leads to a maximum profiled likelihood estimate,
\begin{equation}
  \label{eq:residualVarianceEstimate}
  \widehat{\sigma}_{\bm\theta}^2 = \frac{r^2(\bm\theta)}{n}.
\end{equation}
This estimate can be substituted into the partially profiled criterion to yield the
fully profiled ML criterion,
\begin{equation}
  \label{eq:profiledDeviance}
  -2\mc L(\bm\theta | \yobs)  =           % |
  \log\frac{|\bm L_{\bm\theta}|^2}{|\bm W|}
  + n\left[1 + \log\left(
  \frac{2\pi r^2(\bm\theta)}{n}
  \right)\right] .
\end{equation}
This expression for the profiled deviance depends only on
$\bm\theta$. Although $q$, the number of columns in $\bm Z$ and the
size of $\bm\Sigma_{\bm\theta}$, can be very large indeed, the
dimension of $\bm\theta$ is small, frequently less than 10. The
\pkg{lme4} package uses generic nonlinear optimizers
(Section~\ref{sec:optimizeLmer}) to optimize this expression over
$\bm\theta$ to find its maximum likelihood estimate.

\subsubsection{The REML criterion}
\label{sec:reml}

The REML criterion can be obtained by integrating the marginal density
for $\mc Y$ with respect to the fixed effects \citep{laird_ware_1982},
\begin{equation}
  \label{eq:REMLintegral}
  \int f_{\mc Y}(\yobs)d\bm\beta =
  \frac{|\bm W|^{1/2}|\bm L_{\bm\theta}|^{-1}}{(2\pi\sigma^2)^{n/2}}
  \exp\left[\frac{-r^2(\bm\theta)}{2\sigma^2}\right]
  \int \exp\left[\frac{
  - \left\|\bm R_{X} (\bm\beta - \widehat{\bm\beta}_{\bm\theta})\right\|^2
  }{2\sigma^2}\right]d\bm\beta ,
\end{equation}
which can be evaluated with the change of variables,
\begin{equation}
  \label{eq:REMLchangeOfVariables}
  \bm v = \bm R_{X} (\bm\beta - \widehat{\bm\beta}_{\bm\theta}) .
\end{equation}
The Jacobian determinant of the transformation from $\bm\beta$ to
$\bm v$ is $|\bm R_X |$. Therefore we are able to write the
integral as,
\begin{equation}
  \label{eq:REMLintegralIntermediate}
  \int f_{\mc Y}(\yobs)d\bm\beta =
  \frac{|\bm W|^{1/2}|\bm L_{\bm\theta}|^{-1}}{(2\pi\sigma^2)^{n/2}}
  \exp\left[\frac{-r^2(\bm\theta)}{2\sigma^2}\right]
  \int \exp\left[\frac{
  - \left\|\bm v\right\|^2
  }{2\sigma^2}\right]|\bm R_X|^{-1} d\bm v ,
\end{equation}
which simplifies to,
\begin{equation}
  \label{eq:REMLintermediate}
  \int f_{\mc Y}(\yobs)d\bm\beta =
  \frac{|\bm W|^{1/2}|\bm L_{\bm\theta}|^{-1}|\bm R_X|^{-1}}{(2\pi\sigma^2)^{(n-p)/2}}
  \exp\left[\frac{-r^2(\bm\theta)}{2\sigma^2}\right] .
\end{equation}
Minus twice the log of this integral is the (unprofiled) REML criterion,
\begin{equation}
  \label{eq:REMLdeviance}
  -2\mc L_R(\bm\theta,\sigma^2 | \yobs) =
  \log\frac{|\bm L_{\bm\theta}|^2|\bm R_X|^2}{|\bm W|}
  + (n-p)\log(2\pi\sigma^2)
  + \frac{r^2(\bm\theta)}{\sigma^2} .
\end{equation}
Note that because $\bm\beta$ gets integrated out, the REML criterion
cannot be used to find a point estimate of $\bm\beta$. However, we
follow others in using the maximum likelihood estimate,
$\widehat{\bm\beta}_{\widehat{\bm\theta}}$, at the optimum value of
$\bm\theta = \widehat{\bm\theta}$. The REML estimate of $\sigma^2$ is,
\begin{equation}
  \label{eq:REMLresidualVarianceEstimate}
  \widehat{\sigma}_{\bm\theta}^2 = \frac{r^2(\bm\theta)}{n-p} ,
\end{equation}
which leads to the profiled REML criterion,
\begin{equation}
  \label{eq:REMLprofiled}
  -2\mc L_R(\bm\theta | \yobs) =
  \log\frac{|\bm L_{\bm\theta}|^2|\bm R_X|^2}{|\bm W|}
  + (n-p)\left[1 + \log\left(\frac{2\pi
        r^2(\bm\theta)}{n-p}\right)\right] .
\end{equation}

\subsection{Changes relative to previous formulations}
\label{sec:previous_lmm_form}

We compare the PLS problem as formulated in Section~\ref{sec:plsMath}
with earlier versions and describe why we use this version.  What have
become known as ``Henderson's mixed-model equations'' are given as
Equation 6 of \citet{henderson_1982} and would be expressed as,
\begin{equation}
  \label{eq:henderson}
  \begin{bmatrix}
    \bm X\trans\bm X/\sigma^2& \bm X\trans\bm Z/\sigma^2\\
    \bm Z\trans\bm X/\sigma^2& \bm Z\trans\bm Z/\sigma^2 + \bm\Sigma^{-1}
  \end{bmatrix}
  \begin{bmatrix}
    \widehat{\bm\beta}_\theta \\ \bm\mu_{\mc B|\mc Y=\yobs}
  \end{bmatrix}=
  \begin{bmatrix}\bm X\trans\yobs/\sigma^2\\\bm Z\trans\yobs/\sigma^2
  \end{bmatrix} ,
\end{equation}
in our notation (ignoring weights and offsets, without loss of
generality).  The matrix written as $\bm R$ in \citet{henderson_1982}
is $\sigma^2\bm I_n$ in our formulation of the model.

\citet{bates04:_linear} modified the PLS equations to
\begin{equation}
  \label{eq:batesDebRoy}
  \begin{bmatrix}
    \bm Z\trans\bm Z+\bm\Omega&\bm Z\trans\bm X\\
    \bm X\trans\bm Z&\bm X\trans\bm X
  \end{bmatrix}
  \begin{bmatrix}
     \bm\mu_{\mc B|\mc Y=\yobs}\\\widehat{\bm\beta}_\theta
  \end{bmatrix}=
  \begin{bmatrix}\bm Z\trans\yobs\\\bm X\trans\yobs\end{bmatrix} \quad .
\end{equation}
where
$\bm\Omega_\theta=\left(\bLt\trans\bLt\right)^{-1}=\sigma^2\bm\Sigma^{-1}$
is the \emph{relative precision matrix} for a given value of
$\bm\theta$.  They also showed that the profiled log-likelihood can be
expressed (on the deviance scale) as
\begin{equation}
  \label{eq:batesDebRoyprofdev}
  -2\mc L(\bm\theta)=
  \log\left(\frac{|\bm Z\trans\bm Z+\bm\Omega|}{|\bm\Omega|}\right) +
  n\left[1+\log\left(\frac{2\pi r^2_\theta}{n}\right)\right] .
\end{equation}

The primary difference between Equation~\ref{eq:henderson} and
Equation~\ref{eq:batesDebRoy} is the order of the blocks in the system matrix.
The PLS problem can be solved using a Cholesky factor of the system
matrix with the blocks in either order.  The advantage of using the
arrangement in Equation~\ref{eq:batesDebRoy} is to allow for evaluation of
the profiled log-likelihood.  To evaluate $|\bm Z\trans\bm
Z+\bm\Omega|$ from the Cholesky factor that block must be in the
upper-left corner, not the lower right.  Also, $\bm Z$ is sparse whereas $\bm
X$ is usually dense.  It is straightforward to exploit the sparsity of
$\bm Z\trans\bm Z$ in the Cholesky factorization when the block containing
this matrix is the first block to be factored.  If $\bm X\trans\bm X$ is the first block to be factored, it is much more difficult to
preserve sparsity.

The main change from the formulation in \citet{bates04:_linear} to the
current formulation is the use of a relative covariance factor,
$\bLt$, instead of a relative precision matrix, $\bm\Omega_\theta$,
and solving for the mean of $\mc U|\mc Y = \yobs$ instead of the mean
of $\mc B|\mc Y = \yobs$.  This change improves stability, because the
solution to the PLS problem in Section~\ref{sec:plsMath} is
well-defined when $\bLt$ is singular whereas the formulation in
Equation~\ref{eq:batesDebRoy} cannot be used in these cases because
$\bm\Omega_\theta$ does not exist.

It is important to allow for $\bLt$ to be singular because situations
where the parameter estimates, $\widehat{\bm\theta}$, produce a
singular $\bm\Lambda_{\widehat{\theta}}$ do occur in practice.  And
even if the parameter estimates do not correspond to a singular
$\bm\Lambda_\theta$, it may be desirable to evaluate the estimation
criterion at such values during the course of the numerical
optimization of the criterion.

\citet{bates04:_linear} also provided expressions for the gradient of
the profiled log-likelihood expressed as
Equation~\ref{eq:batesDebRoyprofdev}.  These expressions can be translated
into the current formulation.  From Equation~\ref{eq:profiledDeviance} we can see
that (again ignoring weights),
\begin{equation}
  \label{eq:grad}
  \begin{aligned}
    \nabla\left(-2\mc L(\theta)\right)
    &=\nabla\log(|\bm L_\theta|^2)+\nabla \left(n\log(r^2(\bm\theta))\right)\\
    &=\nabla\log(|\bLt\trans\bm Z\trans\bm Z\bLt+\bm I|)+
    n\left(\nabla r^2(\bm\theta)\right)/r^2(\bm\theta)\\
    &=\nabla\log(|\bLt\trans\bm Z\trans\bm Z\bLt+\bm I|)+
    \left(\nabla r^2(\bm\theta)\right)/(\widehat{\sigma^2}).
  \end{aligned}
\end{equation}
The first gradient is easy to express but difficult to evaluate for
the general model.  The individual elements of this gradient are
\begin{equation}
  \label{eq:graddet}
  \begin{aligned}
    \frac{\partial\log(|\bLt\trans\bm Z\trans\bm Z\bLt+\bm
      I|)}{\partial \theta_i} &=
    \tr\left[\frac{\partial\left(\bLt\trans\bm Z\trans\bm
          Z\bLt\right)}{\partial\theta_i}
      \left(\bLt\trans\bm Z\trans\bm Z\bLt+\bm I\right)^{-1}\right]\\
    &=\tr\left[
      \left(\bm L_\theta\bm L_\theta\trans\right)^{-1}
      \left(
        \bLt\trans\bm Z\trans\bm Z\frac{\partial\bLt}{\partial\theta_i} +
        \frac{\partial\bLt\trans}{\partial\theta_i}\bm Z\trans\bm Z\bLt
      \right)
    \right] .
  \end{aligned}
\end{equation}
The second gradient term can be expressed as a linear function of the
residual, with individual elements of the form
\begin{equation}
  \label{eq:gradr2}
  \frac{\partial r^2(\bm\theta)}{\partial\theta_i}=
  -2\bm u\trans\frac{\partial\bLt\trans}{\partial\theta_i}\bm
  Z\trans\left(\bm y-\bm Z\bLt\bm u-\bm X\widehat{\bm\beta}_{\bm\theta
   }\right),
\end{equation}
using the results of \citet{golub_pereyra_1973}. Although we do not
use these results in \pkg{lme4}, they are used for certain model types
in the \pkg{MixedModels} package for \proglang{Julia} and do provide
improved performance.


\subsection{Penalized least squares algorithm}
\label{sec:PLSpureR}

For efficiency, in \pkg{lme4} itself, PLS is implemented in compiled
\proglang{C++} code using the \pkg{Eigen} \citep{Eigen} templated \proglang{C++}
package for numerical linear algebra.  Here however, in order to
improve readability we describe a version in pure \proglang{R}.
Section~\ref{sec:CSCmats} provides greater detail on the techniques
and concepts for computational efficiency, which is important in cases
where the nonlinear optimizer (Section~\ref{sec:optimizeLmer})
requires many iterations.

The PLS algorithm takes a vector of covariance parameters,
$\bm\theta$, as inputs and returns the profiled deviance (Equation
\ref{eq:profiledDeviance}) or the REML criterion
(Equation \ref{eq:REMLprofiled}). This PLS algorithm consists of four main
steps:
\begin{enumerate}
\item Update the relative covariance factor (Section~\ref{sec:plsI}).
\item Solve the normal equations (Section~\ref{sec:plsII}).
\item Update the linear predictor and residuals (Section~\ref{sec:plsIII}).
\item Compute and return the profiled deviance (Section~\ref{sec:plsIV}).
\end{enumerate}
PLS also requires the objects described in Table~\ref{tab:inputsPLS},
which define the structure of the model.  These objects do not get
updated during the PLS iterations, and so it is useful to store
various matrix products involving them (Table~\ref{tab:constantPLS}).
Table~\ref{tab:updatePLS} lists the objects that do get updated over
the PLS iterations.  The symbols in this table correspond to a version
of \pkg{lme4} that is implemented entirely in \proglang{R} (i.e., no
compiled code as in \pkg{lme4} itself).  This implementation is called
\pkg{lme4pureR} and is currently available on \github\
(\url{https://github.com/lme4/lme4pureR/}).

\begin{table}[tb]
  \centering
  \begin{minipage}{\textwidth}
  \begin{tabular}{p{1.8in}llp{1.6in}}
    \hline
    Name/description & Pseudocode & Math & Type \\
    \hline
    Mapping from covariance parameters to relative covariance factor &
    \code{mapping} &  & function \\
    Response vector & \code{y} & $\yobs$ (Section~\ref{sec:LMMs}) & double vector \\
    Fixed-effects model matrix & \code{X} & $\bm X$ (Equation~\ref{eq:LMMcondY}) & double dense\footnote{In previous versions of \pkg{lme4} a sparse $X$ matrix, useful for models with categorical fixed-effect predictors with many levels, could be specified; this feature is not currently available.}
    matrix \\
    Transposed random-effects model matrix & \code{Zt} & $\bm Z\trans$
    (Equation~\ref{eq:LMMcondY}) &
    double sparse matrix \\
    Square-root weights matrix & \code{sqrtW} & $\bm W^{1/2}$
    (Equation~\ref{eq:LMMcondY}) & double diagonal matrix \\
    Offset & \code{offset} & $\bm o$ (Equation~\ref{eq:LMMcondY}) & double vector \\
    \hline
  \end{tabular}
  \renewcommand{\footnoterule}{%
    \kern -3pt
    \hrule width \textwidth height 0pt
    \kern 2pt
  }
  \end{minipage}
  \caption{Inputs into a linear mixed model.}
  \label{tab:inputsPLS}
\end{table}

\begin{table}[tb]
  \centering
  \begin{tabular}{ll}
    \hline
    Pseudocode & Math \\
    \hline
    \code{ZtW} & $\bm Z\trans\bm W^{1/2}$ \\
    \code{ZtWy} & $\bm Z\trans\bm W \yobs$ \\
    \code{ZtWX} & $\bm Z\trans\bm W \bm X$ \\
    \code{XtWX} & $\bm X\trans\bm W \bm X$ \\
    \code{XtWy} & $\bm X\trans\bm W \yobs$ \\
    \hline
  \end{tabular}
  \caption{Constant symbols involved in penalized least squares.}
  \label{tab:constantPLS}
\end{table}


\begin{table}[tb]
  \centering
  \begin{tabular}{p{1.7in}llp{1.5in}}
    \hline
    Name/description & Pseudocode & Math & Type \\
    \hline
    Relative covariance factor & \code{lambda} & $\bLt$
    (Equation~\ref{eq:relcovfac})
    & double sparse lower-triangular matrix  \\
    Random-effects Cholesky factor & \code{L} & $\bm L_{\bm\theta}$
    (Equation~\ref{eq:blockCholeskyDecomp})
    & double sparse triangular matrix \\
    Intermediate vector in the solution of the normal equations &
    \code{cu} & $\bm c_u$ (Equation~\ref{eq:cu}) & double vector \\
    Block in the full Cholesky factor & \code{RZX} & $\bm R_{ZX}$
    (Equation~\ref{eq:blockCholeskyDecomp})  & double dense matrix \\
    Cross-product of the fixed-effects Cholesky factor & \code{RXtRX}
    & $\bm R_X\trans\bm R_X$ (Equation~\ref{eq:RX}) & double dense matrix \\
    Fixed-effects coefficients & \code{beta} & $\bm\beta$
    (Equation~\ref{eq:LMMcondY}) & double vector \\
    Spherical conditional modes & \code{u} & $\bm u$ (Section~\ref{sec:stability}) & double vector \\
    Non-spherical conditional modes & \code{b}
    & $\bm b$ (Equation~\ref{eq:LMMcondY}) & double vector \\
    Linear predictor & \code{mu} & $\bm\mu_{\mc Y|\mc U=\bm u}$
    (Equation~\ref{eq:meanYgivenU}) & double vector \\
    Weighted residuals & \code{wtres} & $\bm W^{1/2} (\yobs - \mu)$ &
    double vector \\
    Penalized weighted residual sum-of-squares & \code{pwrss} &
    $r^2(\bm\theta)$ (Equation~\ref{eq:pwrssIdentity}) &
    double \\
    Twice the log determinant random-effects Cholesky factor  &
    \code{logDet}
    & $\log|\bm L_{\bm\theta}|^2$ & double \\
    \hline
  \end{tabular}
  \caption{Quantities updated during an evaluation of the linear mixed model
    objective function.}
  \label{tab:updatePLS}
\end{table}


\subsubsection{PLS step I: Update relative covariance factor}
\label{sec:plsI}

The first step of PLS is to update the relative covariance factor,
$\bLt$, from the current value of the covariance parameter vector,
$\bm\theta$.  The updated $\bLt$ is then used to update the random-effects
Cholesky factor, $\bm L_{\bm\theta}$
(Equation~\ref{eq:blockCholeskyDecomp}).  The mapping from the covariance
parameters to the relative covariance factor can take many forms, but
in general involves a function that takes $\bm\theta$ into the values
of the nonzero elements of $\bLt$.

If $\bLt$ is stored as an object of class `\code{dgCMatrix}' from the
\pkg{Matrix} package for \proglang{R}, then we may update $\bLt$ from
$\bm\theta$ by,
<<PLSupdateTheta, eval = FALSE>>=
Lambdat@x[] <- mapping(theta)
@
where \code{mapping} is an \proglang{R} function that both accepts and
returns a numeric vector. The nonzero elements of sparse matrix
classes in \pkg{Matrix} are stored in a slot called \code{x}.

In the current version of \pkg{lme4} (v. 1.1-7) the mapping from
$\bm\theta$ to $\bLt$ is represented as an \proglang{R} integer
vector, \code{Lind}, of indices, so that
<<PLSupdateThetaWithLind>>=
mapping <- function(theta) theta[Lind]
@
The index vector \code{Lind} is computed during the model setup and
stored in the function's environment.  Continuing the example from
Section~\ref{sec:mkLambdat}, consider a new value for \code{theta},
<<exampleNewTheta>>=
thetanew <- c(1, -0.1, 2, 0.1, -0.2, 3)
@
To put these values in the appropriate elements in \code{Lambdati},
we use \code{mapping},
<<exampleLindUpdate>>=
Lambdati@x[] <- mapping(thetanew)
Lambdati
@ %
This \code{Lind} approach can be useful for extending the capabilities
of \pkg{lme4} by using the modular approach to fitting mixed models.
For example, Appendix~\ref{sec:zeroSlopeSlope} shows how to use
\code{Lind} to fit a model where two random slopes are uncorrelated,
but both slopes are correlated with an intercept.

The mapping from the covariance parameters to the relative covariance
factor is treated differently in other implementations of the
\code{lme4} approach to linear mixed models.  At the other extreme, the
\code{flexLambda} branch of \pkg{lme4} and the \pkg{lme4pureR} package
provide the capabilities for a completely general mapping.  This added
flexibility has the advantage of allowing a much wider variety of
models (e.g., compound symmetry, auto-regression).  However, the
disadvantage of this approach is that it becomes possible to fit a
much wider variety of ill-posed models.  Finally, if one would like
such added flexibility with the current stable version of \pkg{lme4},
it is always possible to use the modular approach to wrap the
\code{Lind}-based deviance function in a general mapping function
taking a parameter to be optimized, say $\phi$, into $\bm\theta$.
However, this approach is likely to be inefficient in many cases.

The \code{update} method from the \pkg{Matrix} package efficiently
updates the random-effects Cholesky factor, $\bm L_{\bm\theta}$, from a
new value of $\bm\theta$ and the updated $\bLt$.
<<updateL, eval=FALSE>>=
L <- update(L, Lambdat %*% ZtW, mult = 1)
@
The \code{mult = 1} argument corresponds to the addition of the
identity matrix to the upper-left block on the left-hand-side of
Equation~\ref{eq:blockCholeskyDecomp}.

\subsubsection{PLS step II:  solve normal equations}
\label{sec:plsII}

With the new covariance parameters installed in $\bLt$, the next step
is to solve the normal equations (Equation~\ref{eq:normalEquations}) for the
current estimate of the fixed-effects coefficients,
$\widehat{\betavec}_{\bm\theta}$, and the conditional mode, $\mu_{\mc
  U|\mc Y = \yobs}$. We solve these equations using a sparse Cholesky
factorization (Equation~\ref{eq:blockCholeskyDecomp}).  In a complex model
fit to a large data set, the dominant calculation in the evaluation of
the profiled deviance (Equation~\ref{eq:profiledDeviance}) or REML criterion
(Equation~\ref{eq:REMLprofiled}) is this sparse Cholesky factorization
(Equation~\ref{eq:blockCholeskyDecomp}).  The factorization is performed in two
phases; a symbolic phase and a numeric phase.  The symbolic phase, in
which the fill-reducing permutation $\bm P$ is determined along with
the positions of the nonzeros in $\bm L_{\bm\theta}$, does not depend
on the value of $\bm\theta$.  It only depends on the positions of the
nonzeros in $\bm Z\bLt$.  The numeric phase uses $\bm\theta$ to
determine the numeric values of the nonzeros in $\bm
L_{\bm\theta}$. Using this factorization, the solution proceeds by the
following steps,
\begin{align}
    \bm L_{\bm\theta}\bm c_u&=\bm P\bLt\trans\bm Z\trans\bm W\bm y\label{eq:cu}\\
    \bm L_{\bm\theta}\bm R_{ZX}&=\bm P\bLt\trans\bm Z\trans\bm W\bm X\label{eq:RZX}\\
    \bm R_X\trans\bm R_X&=\bm X\trans\bm W\bm X-\bm R_{ZX}\trans\bm R_{ZX}\label{eq:RX}\\
    \left(\bm R_X\trans\bm R_X\right)\widehat{\bm\beta}_{\bm\theta}&=
      \bm X\trans\bm W\bm y-\bm R_{ZX}\bm c_u\label{eq:betahat}\\
    \bm L\trans\bm_{\bm\theta} \bm P\bm u&=\bm c_u-\bm R_{ZX}\widehat{\bm\beta}_{\bm\theta}\label{eq:tildeu}
\end{align}
which can be solved using the \pkg{Matrix} package with,
<<PLSsolve, eval = FALSE>>=
cu[] <- as.vector(solve(L, solve(L, Lambdat %*% ZtWy,
                                 system = "P"), system = "L"))
RZX[] <- as.vector(solve(L, solve(L, Lambdat %*% ZtWX,
                                  system = "P"), system = "L"))
RXtRX <- as(XtWX - crossprod(RZX), "dpoMatrix")
beta[] <- as.vector(solve(RXtRX, XtWy - crossprod(RZX, cu)))
u[] <- as.vector(solve(L, solve(L, cu - RZX %*% beta,
                                system = "Lt"), system = "Pt"))
@
Notice the nested calls to \code{solve}.  The inner calls of the first
two assignments determine and apply the permutation matrix
(\code{system = "P"}), whereas the outer calls actually solve the
equation (\code{system = "L"}). In the assignment to \code{u[]}, the
nesting is reversed in order to return to the original permutation.

\subsubsection{PLS step III: Update linear predictor and residuals}
\label{sec:plsIII}

The next step is to compute the linear predictor, $\mu_{\mc Y| \mc U}$
(Equation \ref{eq:meanYgivenU}), and the weighted residuals with new values for
$\widehat{\betavec}_\theta$ and $\mu_{\mc B | \mc Y = \yobs}$.  In
\pkg{lme4pureR} these quantities can be computed as,
<<PLSupdateResp, eval = FALSE>>=
b[] <- as.vector(crossprod(Lambdat, u))
mu[] <- as.vector(crossprod(Zt, b) + X %*% beta + offset)
wtres <- sqrtW * (y - mu)
@
where \code{b} represents the current estimate of $\mu_{\mc B | \mc Y
  = \yobs}$.

\subsubsection{PLS step IV: Compute profiled deviance}
\label{sec:plsIV}

Finally, the updated linear predictor and weighted residuals can be
used to compute the profiled deviance (or REML criterion), which in
\pkg{lme4pureR} proceeds as,
<<PLScalculateObjective, eval = FALSE>>=
pwrss <- sum(wtres^2) + sum(u^2)
logDet <- 2*determinant(L, logarithm = TRUE)$modulus
if (REML) logDet <- logDet + determinant(RXtRX,
                                         logarithm = TRUE)$modulus
attributes(logDet) <- NULL
profDev <- logDet + degFree * (1 + log(2 * pi * pwrss) - log(degFree))
@
The profiled deviance consists of three components: (1)
log-determinant(s) of Cholesky factorization (\code{logDet}), (2) the
degrees of freedom (\code{degFree}), and the penalized weighted
residual sum-of-squares (\code{pwrss}).

\subsection{Sparse matrix methods}
\label{sec:CSCmats}

In fitting linear mixed models, an instance of the PLS problem
(\ref{eq:normalEquations}) must be solved at each evaluation of the
objective function during the optimization
(Section~\ref{sec:optimizeLmer}) with respect to $\bm\theta$.  Because
this operation must be performed many times it is worthwhile
considering how to provide effective evaluation methods for objects
and calculations involving the sparse matrices associated with
random-effects terms (Sections~\ref{sec:LMMmatrix}).

The \pkg{CHOLMOD} library of \proglang{C} functions
\citep{Chen:2008:ACS:1391989.1391995}, on which the sparse matrix
capabilities of the \pkg{Matrix} package for \proglang{R} and the
sparse Cholesky factorization in \proglang{Julia} are based, allows
for separation of the symbolic and numeric phases.  Thus we perform
the symbolic phase as part of establishing the structure representing
the model (Section~\ref{sec:lFormula}).  Furthermore, because \pkg{CHOLMOD}
functions allow for updating $\bm L_{\bm\theta}$ directly from the
matrix $\bLt\trans\bm Z\trans$ without actually forming $\bLt\trans\bm
Z\trans\bm Z\bLt+\bm I$ we generate and store $\bm Z\trans$ instead of
$\bm Z$ (note that we have ignored the weights matrix, $\bm W$, for
simplicity).  We can update $\bLt\trans\bm Z\trans$ directly from
$\bm\theta$ without forming $\bLt$ and multiplying two sparse
matrices.  Although such a direct approach is used in the
\pkg{MixedModels} package for \proglang{Julia}, in \pkg{lme4} we first
update $\bLt\trans$ then form the sparse product $\bLt\trans\bm
Z\trans$.  A third alternative, which we employ in \pkg{lme4pureR}, is
to compute and save the cross-products of the model matrices, $\bm X$
and $\bm Z$, and the response, $\bm y$, before starting the
iterations. To allow for case weights, we save the products $\bm
X\trans\bm W\bm X$, $\bm X\trans\bm W\bm y$, $\bm Z\trans\bm W\bm X$,
$\bm Z\trans\bm W\bm y$ and $\bm Z\trans\bm W\bm Z$ (see
Table~\ref{tab:constantPLS}).

We wish to use structures and algorithms that allow us to take a new
value of $\bm\theta$ and evaluate the $\bm L_\theta$
(Equation~\ref{eq:blockCholeskyDecomp}) efficiently.  The key to doing so
is the special structure of $\bLt\trans\bm Z\trans\bm W^{1/2}$.  To
understand why this matrix, and not its transpose, is of interest we
describe the sparse matrix structures used in \proglang{Julia} and in
the \pkg{Matrix} package for \proglang{R}.

Dense matrices are stored in \proglang{R} and in \proglang{Julia} as a
one-dimensional array of contiguous storage locations addressed in
\emph{column-major order}.  This means that elements in the same
column are in adjacent storage locations whereas elements in the same
row can be widely separated in memory.  For this reason, algorithms
that work column-wise are preferred to those that work row-wise.

Although a sparse matrix can be stored in a \emph{triplet} format,
where the row position, column position and element value of the nonzeros
are recorded as triplets, the preferred storage forms for actual
computation with sparse matrices are compressed sparse column (CSC) or
compressed sparse row \citep[CSR,][Chapter~2]{davis06:csparse_book}. The
\pkg{CHOLMOD} (and, more generally, the \pkg{SuiteSparse} package of \proglang{C} libraries; \citealt{SuiteSparse})
uses the CSC storage format.  In this format the nonzero elements in
a column are in adjacent storage locations and access to all the
elements in a column is much easier than access to those in a row
(similar to dense matrices stored in column-major order).

The matrices $\bm Z$ and $\bm Z\bLt$ have the property that the number
of nonzeros in each row, $\sum_{i=1}^k p_i$, is constant.  For CSC
matrices we want consistency in the columns rather than the rows,
which is why we work with $\bm Z\trans$ and $\bLt\trans\bm Z\trans$
rather than their transposes.

An arbitrary $m\times n$ sparse matrix in CSC format is expressed as
two vectors of indices, the row indices and the column pointers, and a
numeric vector of the nonzero values.  The elements of the row
indices and the nonzeros are aligned and are ordered first by
increasing column number then by increasing row number within column.
The column pointers are a vector of size $n+1$ giving the location of
the start of each column in the vectors of row indices and nonzeros.

%% This para is __extra__ in comparison to JSS (2015):
Because the number of nonzeros in each column of $\bm Z\trans$, and in
each column of matrices derived from $\bm Z\trans$ (such as
$\bLt\trans\bm Z\trans\bm W^{1/2}$) is constant, the vector of
nonzeros in the CSC format can be viewed as a dense matrix, say $\bm
N$, of size $\left(\sum_{i=1}^n p_i\right)\times n$.  We do not need
to store the column pointers because the columns of $\bm Z\trans$
correspond to columns of $\bm N$.  All we need is $\bm N$, the dense
matrix of nonzeros, and the row indices, which are derived from the
grouping factor index vectors $\bm i_i,i=1,\dots,k$ and can also be
arranged as a dense matrix of size $\left(\sum_{i=1}^n
  p_i\right)\times n$.  Matrices like $\bm Z\trans$, with the property
that there are the same number of nonzeros in each column, are
sometimes called \emph{regular sparse column-oriented} (RSC) matrices.

\section{Nonlinear optimization module}
\label{sec:optimizeLmer}

The objective function module produces a function which implements the
penalized least squares algorithm for a particular mixed model. The
next step is to optimize this function over the covariance parameters,
$\bm\theta$, which is a nonlinear optimization problem.  The
\pkg{lme4} package separates the efficient computational linear
algebra required to compute profiled likelihoods and deviances for a
given value of $\bm\theta$ from the nonlinear optimization algorithms,
which use general-purpose nonlinear optimizers.

One benefit of this separation is that it allows for experimentation
with different nonlinear optimizers.  Throughout the development of
\pkg{lme4}, the default optimizers and control parameters have changed
in response to feedback from users about both efficiency and
convergence properties.  \pkg{lme4} incorporates two built-in
optimization choices, an implementation of the Nelder-Mead simplex
algorithm adapted from Steven Johnson's \pkg{NLopt} \proglang{C} library
\citep{NLopt} and a wrapper for Powell's BOBYQA algorithm, implemented
in the \pkg{minqa} package \citep{minqa_pkg} as a wrapper around
Powell's original \proglang{Fortran} code \citep{Powell_bobyqa}.  More generally,
\pkg{lme4} allows any user-specified optimizer that (1) can work with
an objective function (i.e., does not require a gradient function to
be specified), (2) allows box constraints on the parameters, and (3)
conforms to some simple rules about argument names and structure of
the output. An internal wrapper allows the use of the \pkg{optimx}
package \citep{optimx_pkg}, although the only optimizers provided via
\pkg{optimx} that satisfy the constraints above are the \code{nlminb}
and \code{L-BFGS-B} algorithms that are themselves wrappers around the
versions provided in base \proglang{R}.  Several other algorithms from Steven
Johnson's \pkg{NLopt} package are also available via the \pkg{nloptr}
wrapper package (e.g., alternate implementations of Nelder-Mead and
BOBYQA, and Powell's COBYLA algorithm).

This flexibility assists with diagnosing convergence problems -- it
is easy to switch among several algorithms to determine whether the
problem lies in a failure of the nonlinear optimization stage, as
opposed to a case of model misspecification or unidentifiability or a
problem with the underlying PLS algorithm.  To date we
have only observed PLS failures, which arise if
$\bm X\trans\bm W\bm X-\bm R_{ZX}\trans\bm R_{ZX}$ becomes singular
during an evaluation of the objective function, with badly scaled
problems (i.e., problems with continuous predictors that take a
very large or very small numerical range of values).

The requirement for optimizers that can handle box constraints stems
from our decision to parameterize the variance-covariance matrix in a
constrained space, in order to allow for singular fits. In contrast to
the approach taken in the \pkg{nlme} package \citep{nlme_pkg}, which goes to some
lengths to use an unconstrained variance-covariance parameterization
(the \emph{log-Cholesky} parameterization;
\citealt{pinheiro_unconstrained_1996}), we instead use the Cholesky
parameterization but require the elements of $\bm\theta$ corresponding
to the diagonal elements of the Cholesky factor to be non-negative.
With these constraints, the variance-covariance matrix is singular if
and only if any of the diagonal elements is exactly zero.
Singular fits are common in practical data-analysis situations,
especially with small- to medium-sized data sets and complex
variance-covariance models, so being able to fit a singular model is
an advantage: when the best fitting model lies on the boundary of a
constrained space, approaches that try to remove the constraints by
fitting parameters on a transformed scale will often give rise to
convergence warnings as the algorithm tries to find a maximum on an
asymptotically flat surface \citep{bolker_strategies_2013}.

In principle the likelihood surfaces we are trying to optimize over
are smooth, but in practice using gradient information in optimization
may or may not be worth the effort. In special cases, we have a
closed-form solution for the gradients
(Equations~\ref{eq:grad}--\ref{eq:gradr2}), but in general we will have to
approximate them by finite differences, which is expensive and has
limited accuracy.  (We have considered using automatic
differentiation approaches to compute the gradients more efficiently,
but this strategy requires a great deal of additional machinery, and
would have drawbacks in terms of memory requirements for large
problems.)  This is the primary reason for our switching to
derivative-free optimizers such as BOBYQA and Nelder-Mead in the
current version of \pkg{lme4}, although as discussed above
derivative-based optimizers based on finite differencing are available
as an alternative.

There is most likely further room for improvement in the nonlinear
optimization module; for example, some speed-up could be gained by
using parallel implementations of derivative-free optimizers that
evaluated several trial points at once \citep{klein_nelder_2013}.  In
practice users most often have optimization difficulties with poorly
scaled or centered data -- we are working to implement appropriate
diagnostic tests and warnings to detect these situations.

\section{Output module}
\label{sec:mkMerMod}

Here we describe some of the methods in \pkg{lme4} for exploring
fitted linear mixed models (Table~\ref{tab:methods}), which are
represented as objects of the \proglang{S}4 class `\code{lmerMod}'.  We
begin by describing the theory underlying these methods
(Section~\ref{sec:outputTheory}) and then continue the
\code{sleepstudy} example introduced in Section~\ref{sec:sleepstudy}
to illustrate these ideas in practice.

\subsection{Theory underlying the output module}
\label{sec:outputTheory}

\subsubsection{Covariance matrix of the fixed-effect coefficients}
\label{sec:vcov}

In the \code{lm} function, the variance-covariance matrix of the
coefficients is the inverse of the cross-product of the model matrix, times
the residual variance \citep{Chambers:1993}.  The inverse cross-product
matrix is computed by first inverting the upper triangular matrix
resulting from the QR decomposition of the model matrix, and then
taking its cross-product,
\begin{equation}
  \label{eq:varUBeta}
  \Var_{\bm\theta, \sigma}\left(
    \begin{bmatrix}
      \bm\mu_{\mc U|\mc Y=\yobs} \\ \hat\betavec
    \end{bmatrix}
  \right) =
  \sigma^2
  \begin{bmatrix}
    \bm L\trans_{\bm\theta} & \bm R_{ZX}\\
    \bm 0            & \bm R_X
  \end{bmatrix}^{-1}
  \begin{bmatrix}
    \bm L_{\bm\theta}& \bm 0\\
    \bm R_{ZX}\trans & \bm R_X\trans
  \end{bmatrix}^{-1}.
\end{equation} % |
Because of normality, the marginal covariance matrix of $\hat\betavec$ is
just the lower-right $p$-by-$p$ block of
$\Var_{\bm\theta, \sigma}\left(
    \begin{bmatrix}
      \bm\mu_{\mc U|\mc Y=\yobs} \\ \hat\betavec
    \end{bmatrix}
  \right)$.
This lower-right block is
\begin{equation}
  \label{eq:varBeta}
  \Var_{\bm\theta, \sigma}(\hat\betavec) = \sigma^2 \bm R_X^{-1} (\bm R_X\trans)^{-1},
\end{equation}
which follows from the Schur complement identity \citep[Theorem 1.2]{zhang2006schur}.  This
matrix can be extracted from a fitted \code{merMod} object as,
<<vcovByHand>>=
RX <- getME(fm1, "RX")
sigma2 <- sigma(fm1)^2
sigma2 * chol2inv(RX)
@
which could be computed with \pkg{lme4} as \code{vcov(fm1)}.

The square-root diagonal of this covariance matrix contains the
estimates of the standard errors of fixed-effects coefficients.  These
standard errors are used to construct Wald confidence intervals with
\code{confint(., method = "Wald")}.  Such confidence intervals are
approximate, and depend on the assumption that the likelihood profile
of the fixed effects is linear on the $\zeta$ scale
(Section~\ref{sec:profile}).

\subsubsection{Profiling}
\label{sec:profile}

\newcommand{\devprof}{-2 {\mc L}_p}

The theory of likelihood profiles is straightforward: the deviance (or
likelihood) profile, $\devprof()$, for a focal model parameter $P$ is
the minimum value of the deviance conditioned on a particular value of
$P$.  For each parameter of interest, our goal is to evaluate the
deviance profile for many points -- optimizing over all of the
non-focal parameters each time -- over a wide enough range and with
high enough resolution to evaluate the shape of the profile (in
particular, whether it is quadratic, which would allow use of Wald
confidence intervals and tests) and to find the values of $P$ such
that $\devprof(P)=-2 {\mc L}(\widehat{P}) + \chi^2(1-\alpha)$, which
represent the profile confidence intervals.  While profile confidence
intervals rely on the asymptotic distribution of the minimum deviance,
this is a much weaker assumption than the log-quadratic likelihood
surface required by Wald tests.

An additional challenge in profiling arises when we want to compute
profiles for quantities of interest that are not parameters of our PLS
function. We have two problems in using the deviance function defined
above to profile linear mixed models. First, a parameterization of the
random-effects variance-covariance matrix in terms of standard
deviations and correlations, or variances and covariances, is much
more familiar to users, and much more relevant as output of a
statistical model, than the parameters, $\bm\theta$, of the relative
covariance factor -- users are interested in inferences on variances
or standard deviations, not on $\bm\theta$. Second, the fixed-effect
coefficients and the residual standard deviation, both of which are
also of interest to users, are profiled out (in the sense used above)
of the deviance function (Section~\ref{sec:profdev}), so we have to
find a strategy for estimating the deviance for values of $\bm\beta$
and $\sigma^2$ other than the profiled values.

To handle the first problem we create a new version of the deviance
function that first takes a vector of standard deviations (and
correlations), and a value of the residual standard deviation,
maps them to a $\bm\theta$ vector, and then computes the PLS as
before; it uses the specified residual standard deviation rather than
the PLS estimate of the standard deviation
(Equation~\ref{eq:residualVarianceEstimate}) in the deviance calculation.
We compute a profile likelihood for the fixed-effect parameters, which
are profiled out of the deviance calculation, by adding an offset to
the linear predictor for the focal element of $\bm\beta$.  The
resulting function is not useful for general nonlinear optimization
-- one can easily wander into parameter regimes corresponding to
infeasible (non-positive semidefinite) variance-covariance matrices --
but it serves for likelihood profiling, where one focal parameter is
varied at a time and the optimization over the other parameters is
likely to start close to an optimum.

In practice, the \code{profile} method systematically varies the
parameters in a model, assessing the best possible fit that can be
obtained with one parameter fixed at a specific value and comparing
this fit to the globally optimal fit, which is the original model fit
that allowed all the parameters to vary.  The models are compared
according to the change in the deviance, which is the likelihood ratio
test statistic.  We apply a signed square root transformation to this
statistic and plot the resulting function, which we call the
\emph{profile zeta function} or $\zeta$, versus the parameter
value. The signed aspect of this transformation means that $\zeta$ is
positive where the deviation from the parameter estimate is positive
and negative otherwise, leading to a monotonically increasing function
which is zero at the global optimum. A $\zeta$ value can be compared
to the quantiles of the standard normal distribution.  For example, a
95\% profile deviance confidence interval on the parameter consists of
the values for which $-1.96 < \zeta < 1.96$.  Because the process of
profiling a fitted model can be computationally intensive -- it
involves refitting the model many times -- one should exercise
caution with complex models fit to large data sets.

The standard approach to this computational challenge is to compute
$\zeta$ at a limited number of parameter values, and to fill in the
gaps by fitting an interpolation spline.  Often one is able to invert
the spline to obtain a function from $\zeta$ to the focal parameter,
which is necessary in order to construct profile confidence intervals.
However, even if the points themselves are monotonic, it is possible
to obtain non-monotonic interpolation curves.  In such a case,
\pkg{lme4} falls back to linear interpolation, with a warning.

\newcommand{\alphamax}{\alpha_{\textrm{\small max}}}
\newcommand{\pstep}{\epsilon}

The last part of the technical specification for computing profiles is
deciding on a strategy for choosing points to sample.  In one way or
another one wants to start at the estimated value for each parameter
and work outward either until a constraint is reached (i.e., a value of
0 for a standard deviation parameter or a value of $\pm 1$ for a
correlation parameter), or until a sufficiently large deviation from
the minimum deviance is attained.  \pkg{lme4}'s profiler chooses a
cutoff $\phi$ based on the $1-\alphamax$ critical value of the
$\chi^2$ distribution with a number of degrees of freedom equal to the
\emph{total} number of parameters in the model, where $\alphamax$ is
set to 0.05 by default. The profile computation initially adjusts the
focal parameter $p_i$ by an amount $\pstep=1.01 \hat p_i$ from its ML
or REML estimate $\hat p_i$ (or by $\pstep=0.001$ if $\hat p_i$ is
zero, as in the case of a singular variance-covariance model).  The
local slope of the likelihood profile $(\zeta(\hat
p_i+\pstep)-\zeta(\hat p_i))/\pstep$ is used to pick the next point to
evaluate, extrapolating the local slope to find a new $\pstep$ that
would be expected to correspond to the desired step size $\Delta
\zeta$ (equal to $\phi/8$ by default, so that 16 points would be used
to cover the profile in both directions if the log-likelihood surface
were exactly quadratic).  Some fail-safe testing is done to ensure
that the step chosen is always positive, and less than a maximum; if a
deviance is ever detected that is lower than that of the ML deviance,
which can occur if the initial fit was wrong due to numerical
problems, the profiler issues an error and stops.
%% while scaling based on the approximate variance-covariance matrix
%% might be more accurate in some cases, it may also lead to problems
%% when the log-likelihood surface is far from quadratic

\subsubsection{Parametric bootstrapping}
\label{sec:pb}

To avoid the asymptotic assumptions of the likelihood ratio test, at
the cost of greatly increased computation time, one can estimate
confidence intervals by parametric bootstrapping -- that is, by
simulating data from the fitted model, refitting the model, and
extracting the new estimated parameters (or any other quantities of
interest).  This task is quite straightforward, since there is already
a \code{simulate} method, and a \code{refit} function which
re-estimates the (RE)ML parameters for new data, starting from the
previous (RE)ML estimates and re-using the previously computed model
structures (including the fill-reducing permutation) for efficiency.
The \code{bootMer} function is thus a fairly thin wrapper around a
\code{simulate}/\code{refit} loop, with a small amount of additional
logic for parallel computation and error-catching.  (Some of the ideas
of \code{bootMer} are adapted from \code{merBoot} \citep{merBoot}, a
more ambitious framework for bootstrapping \pkg{lme4} model fits which
unfortunately seems to be unavailable at present.)


\subsubsection{Conditional variances of random effects}
\label{sec:condVar}

It is useful to clarify that the conditional covariance concept in
\pkg{lme4} is based on a simplification of the linear mixed model.  In
particular, we simplify the model by assuming that the quantities,
$\betavec$, $\bLt$, and $\sigma$, are known (i.e., set at their
estimated values). In fact, the only way to define the conditional
covariance is at fixed parameter values.  Our approximation here is
using the estimated parameter values for the unknown ``true''
parameter values.  In this simplified case, $\mc U$ is the only
quantity in the statistical model that is both random and unknown.

Given this simplified linear mixed model, a standard result in
Bayesian linear regression modeling \citep{gelman2013bayesian}
implies that the conditional distribution of the spherical random
effects given the observed response vector is Gaussian,
\begin{equation}
  \label{eq:condGaussian}
(\mc U | \mc Y = \yobs) \sim \mathcal{N}(\mu_{\mc U | \mc Y = \yobs},
\widehat{\sigma}^2 \bm V),
\end{equation}
where
\begin{equation}
  \label{eq:condVar}
  \bm V = \left(\bm\Lambda_{\widehat{\bm\theta}}\trans\bm Z\trans \bm W
    \bm Z\bm\Lambda_{\widehat{\bm\theta}} + \bm I_q \right)^{-1} =
  \left(\bm L_{\widehat{\bm\theta}}^{-1}\right)\trans
  \left(\bm L_{\widehat{\bm\theta}}^{-1}\right)
\end{equation}
is the unscaled conditional variance and
\begin{equation}
  \label{eq:condMean}
  \mu_{\mc U | \mc Y = \yobs} = \bm V \bm\Lambda_{\widehat{\bm\theta}}\trans\bm Z\trans\bm W
    \left(\yobs-\bm o - X\widehat{\betavec} \right)
\end{equation}  % |
is the conditional mean/mode. Note that in practice the inverse in
Equation~\ref{eq:condVar} does not get computed directly, but rather an
efficient method is used that exploits the sparse structures.

The random-effects coefficient vector, $\mathbf{b}$, is often of
greater interest. The conditional covariance matrix of $\mc B$
may be expressed as
\begin{equation}
  \label{eq:condVarB}
\widehat{\sigma}^2 \bm\Lambda_{\widehat{\bm\theta}} \bm V \bm\Lambda_{\widehat{\bm\theta}}\trans.
\end{equation}

<<condVarExample, echo=FALSE, eval=FALSE>>=
s2 <- sigma(fm1)^2
Lambdat <- getME(fm1, "Lambdat")
Lambda <- getME(fm1, "Lambda")
Zt <- getME(fm1, "Zt")
Z <- getME(fm1, "Z")
y <- getME(fm1, "y")
X <- getME(fm1, "X")
beta <- getME(fm1, "beta")
L <- getME(fm1, "L")
(V <- crossprod(solve(L, system = "L")))[1:2, 1:2]
(s2 * Lambda %*% V %*% Lambdat)[1:2, 1:2]
attr(ranef(fm1, condVar = TRUE)$Subject, "postVar")[ , , 1] # should be same as Lam %*% V %*% Lamt
(U <- as.vector((V %*% Lambdat %*% Zt %*%  (y - X %*% beta))))
getME(fm1, "u")
@

\subsubsection{The hat matrix}
\label{sec:hat}

The hat matrix, $\bm H$, is a useful object in linear model
diagnostics \citep{cook1982residuals}.  In general, $\bm H$ relates
the observed and fitted response vectors, but we specifically define
it as,
\begin{equation}
  \left(\bm\mu_{\mc Y|\mc U=\bm u} - \bm o\right) =
  \bm H \left(\yobs - \bm o\right).
\end{equation}
To find $\bm H$ we note that
\begin{equation}
  \left(\bm\mu_{\mc Y|\mc U=\bm u} - \bm o\right) =
  \begin{bmatrix}
    \bm Z\bm\Lambda & \bm X \\
  \end{bmatrix}
  \begin{bmatrix}
    \mu_{\mc U | \mc Y = \yobs} \\ \widehat{\betavec}_{\bm\theta}
  \end{bmatrix}.
\end{equation}
Next we get an expression for $\begin{bmatrix}
    \mu_{\mc U | \mc Y = \yobs} \\ \widehat{\betavec}_{\bm\theta}
  \end{bmatrix}$ by solving the normal equations
  (Equation~\ref{eq:normalEquations}),
\begin{equation}
 \begin{bmatrix}
    \mu_{\mc U | \mc Y = \yobs} \\ \widehat{\betavec}_{\bm\theta}
  \end{bmatrix} =
  \begin{bmatrix}
    \bm L\trans_{\bm\theta} & \bm R_{ZX} \\
    \bm 0 & \bm R_{X}
  \end{bmatrix}^{-1}
    \begin{bmatrix}
    \bm L_{\bm\theta} & \bm 0 \\
    \bm R\trans_{ZX} & \bm R\trans_{X} \\
  \end{bmatrix}^{-1}
    \begin{bmatrix}
    \bLt\trans \bm Z\trans \\
    \bm X\trans \\
  \end{bmatrix} \bm W (\yobs - \bm o).
\end{equation} % |
By the Schur complement identity \citep{zhang2006schur},
\begin{equation}
  \begin{bmatrix}
    \bm L\trans_{\bm\theta} & \bm R_{ZX} \\
    \bm 0 & \bm R_{X}
  \end{bmatrix}^{-1} =
  \begin{bmatrix}
    \left(\bm L\trans_{\bm\theta}\right)^{-1} &
    - \left(\bm L\trans_{\bm\theta}\right)^{-1} \bm R_{ZX} \bm R_{X}^{-1}\\
    \bm 0 & \bm R_{X}^{-1}
  \end{bmatrix}.
\end{equation}
Therefore, we may write the hat matrix as
\begin{equation}
  \label{eq:hatComputational}
  \bm H = (\bm C_L\trans \bm C_L + \bm C_R\trans \bm C_R),
\end{equation}
where,
\begin{equation}
  \label{eq:CL}
  \bm L_{\bm\theta} \bm C_L = \bLt\trans \bm Z\trans\bm W^{1/2}
\end{equation}
and,
\begin{equation}
  \label{eq:CR}
  \bm R\trans_{X} \bm C_R = \bm X\trans\bm W^{1/2} - \bm R\trans_{ZX}\bm C_L.
\end{equation}

The trace of the hat matrix is often used as a measure of the
effective degrees of freedom \citep[e.g., ][]{vaida2005conditional}.
Using a relationship between the trace and vec operators, the trace of
$\bm H$ may be expressed as,
\begin{equation}
  \label{eq:traceHat}
  \tr(\bm H) = \left\| \VEC(\bm C_L) \right\|^2
  + \left\| \VEC(\bm C_R) \right\|^2.
\end{equation}

\subsection{Using the output module}
\label{sec:outputPractice}

\begin{table}
  \begin{tabular}{ll}
    \hline
    Generic (Section) & Brief description of return value \\
    \hline
    \code{anova} (\ref{sec:modComp}) &
    Decomposition of fixed-effects contributions \\
    & or model comparison. \\
    \code{as.function} &
    Function returning profiled deviance or REML criterion. \\
    \code{coef} &
    Sum of the random and fixed effects for each level. \\
    \code{confint} (\ref{sec:CI}) &
    Confidence intervals on linear mixed-model parameters. \\
    \code{deviance} (\ref{sec:summary}) &
    Minus twice maximum log-likelihood. \\
    & (Use \code{REMLcrit} for the REML criterion.) \\
    \code{df.residual} &
    Residual degrees of freedom. \\
    \code{drop1} &
    Drop allowable single terms from the model. \\
    \code{extractAIC} &
    Generalized Akaike information criterion \\
    \code{fitted} &
    Fitted values given conditional modes (Equation~\ref{eq:meanYgivenU}). \\
    \code{fixef} (\ref{sec:summary}) &
    Estimates of the fixed-effects coefficients, $\widehat{\bm\beta}$ \\
    \code{formula} (\ref{sec:uncor}) &
    Mixed-model formula of fitted model. \\
    \code{logLik} & % logLik
    Maximum log-likelihood. \\
    \code{model.frame} & % model.frame
    Data required to fit the model. \\
    \code{model.matrix} &
    Fixed-effects model matrix, $\bm X$.\\
    \code{ngrps} (\ref{sec:summary}) &
    Number of levels in each grouping factor. \\
    \code{nobs} (\ref{sec:summary}) &
    Number of observations.\\
    \code{plot} &
    Diagnostic plots for mixed-model fits. \\
    \code{predict} (\ref{sec:predict}) &
    Various types of predicted values. \\
    \code{print} &
    Basic printout of mixed-model objects. \\
    \code{profile} (\ref{sec:profile}) &
    Profiled likelihood over various model parameters. \\
    \code{ranef} (\ref{sec:summary}) &
    Conditional modes of the random effects. \\
    \code{refit} (\ref{sec:pb}) &
    A model (re)fitted to a new set of observations of the response variable. \\
    \code{refitML} (\ref{sec:modComp}) &
    A model (re)fitted by maximum likelihood.  \\
    \code{residuals}  (\ref{sec:summary}) &
    Various types of residual values. \\
    \code{sigma} (\ref{sec:summary}) &
    Residual standard deviation. \\
    \code{simulate} (\ref{sec:predict}) &
    Simulated data from a fitted mixed model. \\
    \code{summary} (\ref{sec:summary}) &
    Summary of a mixed model. \\
    \code{terms} &
    Terms representation of a mixed model. \\
    \code{update} (\ref{sec:update}) &
    An updated model using a revised formula or other arguments. \\
    \code{VarCorr} (\ref{sec:summary})  &
    Estimated random-effects variances, standard deviations, and correlations. \\
    \code{vcov} (\ref{sec:summary}) &
    Covariance matrix of the fixed-effect estimates. \\
    \code{weights} &
    Prior weights used in model fitting. \\
    \hline
  \end{tabular}
  \caption{List of currently available methods for objects of the class `\code{merMod}'.}
  \label{tab:methods}
\end{table}

The user interface for the output module largely consists of a set of
methods (Table~\ref{tab:methods}) for objects of class \code{merMod},
which is the class of objects returned by \code{lmer}.  In addition to
these methods, the \code{getME} function can be used to extract
various objects from a fitted mixed model in \pkg{lme4}.  Here we
illustrate the use of several of these methods.

\subsubsection{Updating fitted mixed models}
\label{sec:update}

To illustrate the \code{update} method for `\code{merMod}' objects we
construct a random intercept only model. This task could be done in
several ways, but we choose to first remove the random-effects term
\code{(Days | Subject)} and add a new term with a random intercept,
<<updateModel>>=
fm3 <- update(fm1, . ~ . - (Days | Subject) + (1 | Subject))
formula(fm3)
@ % |

\subsubsection{Model summary and associated accessors}
\label{sec:summary}

The \code{summary} method for `\code{merMod}' objects provides
information about the model fit.  Here we consider the output of
\code{summary(fm1)} in four steps.  The first few lines of output
indicate that the model was fitted by REML as well as the value of the
REML criterion (Equation~\ref{eq:REMLdeviance}) at convergence (which
may also be extracted using \code{REMLcrit(fm1)}).  The beginning of
the summary also reproduces the model formula and the scaled Pearson
residuals,
<<summary1,echo=FALSE>>=
ss <- summary(fm1)
cc <- capture.output(print(ss))
reRow <- grep("^Random effects", cc)
cat(cc[1:(reRow - 2)], sep = "\n")
@
This information may also be obtained using standard accessor
functions,
<<summary1reproduced, eval=FALSE>>=
formula(fm1)
REMLcrit(fm1)
quantile(residuals(fm1, "pearson", scaled = TRUE))
@
The second piece of the \code{summary} output provides information
regarding the random-effects and residual variation,
<<summary2, echo=FALSE>>=
feRow <- grep("^Fixed effects", cc)
cat(cc[reRow:(feRow - 2)], sep = "\n")
@
which can also be accessed using,
<<summary2reproduced, eval=FALSE>>=
print(vc <- VarCorr(fm1), comp = c("Variance", "Std.Dev."))
nobs(fm1)
ngrps(fm1)
sigma(fm1)
@
The print method for objects returned by \code{VarCorr} hides the internal structure of these
`\code{VarCorr.merMod}' objects. The internal structure of an object of
this class is a list of matrices, one for each random-effects term.
The standard deviations and correlation matrices for each term are
stored as attributes, \code{stddev} and \code{correlation},
respectively, of the variance-covariance matrix, and the residual
standard deviation is stored as attribute \code{sc}.  For programming
use, these objects can be summarized differently using their
\code{as.data.frame} method,
<<VarCorr>>=
as.data.frame(VarCorr(fm1))
@
which contains one row for each variance or covariance parameter. The
\code{grp} column gives the grouping factor associated with this
parameter. The \code{var1} and \code{var2} columns give the names of
the variables associated with the parameter (\code{var2} is \code{<NA>}
unless it is a covariance parameter). The \code{vcov} column gives the
variances and covariances, and the \code{sdcor} column gives these
numbers on the standard deviation and correlation scales.

The next chunk of output gives the fixed-effect estimates,
<<summary3, echo=FALSE>>=
corRow <- grep("^Correlation", cc)
cat(cc[feRow:(corRow - 2)], sep = "\n")
@
Note that there are no $p$~values (see Section~\ref{sec:pvalues}).
The fixed-effect point estimates may be obtained separately via
\code{fixef(fm1)}.
Conditional modes of the random-effects coefficients can be obtained
with \code{ranef} (see Section~\ref{sec:condVar} for information on
the theory).  Finally, we have the correlations between the fixed-effect estimates
<<summary4, echo=FALSE>>=
cat(cc[corRow:length(cc)], sep = "\n")
@
The full variance-covariance matrix of the fixed-effects estimates can
be obtained in the usual \proglang{R} way with the \code{vcov} method
(Section~\ref{sec:vcov}).  Alternatively, \code{coef(summary(fm1))} will
return the full fixed-effects parameter table as shown in the summary.

\subsubsection{Diagnostic plots}
\label{sec:diagnostics}

\pkg{lme4} provides tools for generating most of the standard
graphical diagnostic plots (with the exception of those incorporating
influence measures, e.g., Cook's distance and leverage plots), in a
way modeled on the diagnostic graphics of \pkg{nlme}
\citep{R:Pinheiro+Bates:2000}.  In particular, the familiar
\code{plot} method in base \proglang{R} for linear models (objects of
class `\code{lm}') can be used to create the standard fitted
vs. residual plots,
<<diagplot1,fig.keep="none">>=
plot(fm1, type = c("p", "smooth"))
@
scale-location plots,
<<diagplot2,fig.keep="none">>=
plot(fm1, sqrt(abs(resid(.))) ~ fitted(.),
     type = c("p", "smooth"))
@
and quantile-quantile plots (from \pkg{lattice}),
<<diagplot3,fig.keep="none">>=
qqmath(fm1, id = 0.05)
@
In contrast to \code{plot.lm}, these scale-location and Q-Q plots
are based on raw rather than standardized residuals.

In addition to these standard diagnostic plots, which
examine the validity of various assumptions (linearity,
homoscedasticity, normality) at the level of the residuals,
one can also use the \code{dotplot} and \code{qqmath} methods
for the conditional modes (i.e., `\code{ranef.mer}'
objects generated by \code{ranef(fit)})
to check for interesting patterns and normality in the conditional
modes.  \pkg{lme4} does not provide influence diagnostics, but
these (and other useful diagnostic procedures) are available
in the dependent packages \pkg{HLMdiag} and \pkg{influence.ME}
\citep{HLMdiag_pkg,influenceME_pkg}.

Finally, \emph{posterior predictive simulation}
\citep{gelman_data_2006} is a generally useful diagnostic tool,
adapted from Bayesian methods, for exploring model fit. Users pick
some summary statistic of interest.  They then compute the summary
statistic for an ensemble of simulations (Section~\ref{sec:predict}),
and see where the observed data falls within the simulated
distribution; if the observed data is extreme, we might conclude that
the model is a poor representation of reality.  For example, using the
sleep study fit and choosing the interquartile range
of the reaction times as the summary
statistic:
<<ppsim,results="hide">>=
iqrvec <- sapply(simulate(fm1, 1000), IQR)
obsval <- IQR(sleepstudy$Reaction)
post.pred.p <- mean(obsval >= c(obsval, iqrvec))
@
The (one-tailed) posterior predictive $p$~value of \Sexpr{round(post.pred.p,2)}
indicates that the model represents the data adequately, at least for
this summary statistic.  In contrast to the full Bayesian case, the
procedure described here does not allow for the uncertainty in the
estimated parameters. However, it should be a reasonable approximation
when the residual variation is large.
%% \bmb{it should be possible to put together something with
%% \code{arm::sim} to simulate beta values and \code{simulate} with
%% \code{newparams}, but (a) this would require some testing and
%% development (I don't think \code{newparams} can take a matrix) and
%% (b) it doesn't account for uncertainty in theta)}

\subsubsection{Sequential decomposition and model comparison}
\label{sec:modComp}

Objects of class `\code{merMod}' have an \code{anova} method which
returns $F$~statistics corresponding to the sequential decomposition
of the contributions of fixed-effects terms.  In order to illustrate
this sequential ANOVA decomposition, we fit a model with orthogonal
linear and quadratic \code{Days} terms,
<<anovaQuadraticModel>>=
fm4 <- lmer(Reaction ~ polyDays[ , 1] + polyDays[ , 2] +
            (polyDays[ , 1] + polyDays[ , 2] | Subject),
            within(sleepstudy, polyDays <- poly(Days, 2)))
anova(fm4)
@
<<anovaSanityCheck, include=FALSE>>=
(getME(fm4, "RX")[2, ] %*% getME(fm4, "fixef"))^2
@
The relative magnitudes of the two sums of squares indicate that the
quadratic term explains much less variation than the linear term.
Furthermore, the magnitudes of the two $F$~statistics strongly
suggest significance of the linear term, but not the quadratic term.
Notice that this is only an informal assessment of significance as
there are no $p$~values associated with these $F$~statistics, an issue
discussed in more detail in the next subsection (``Computing $p$-values'',
p.~\pageref{sec:pvalues}).

To understand how these quantities are computed, let $\bm R_i$ contain
the rows of $\bm R_X$ (Equation~\ref{eq:blockCholeskyDecomp}) associated
with the $i$th fixed-effects term.  Then the sum of squares for term
$i$ is,
\begin{equation}
  \label{eq:SS}
  \mathit{SS}_i = \widehat{\bm\beta}\trans\bm R_i\trans \bm R_i \widehat{\bm\beta}
\end{equation}
If $\mathit{DF}_i$ is the number of columns in $\bm R_i$, then the
$F$~statistic for term $i$ is,
\begin{equation}
  \label{eq:Fstat}
  F_i = \frac{\mathit{SS}_i}{\widehat{\sigma}^2 \mathit{DF}_i}
\end{equation}

For multiple arguments, the \code{anova} method returns model
comparison statistics.
<<anovaManyModels>>=
anova(fm1, fm2, fm3)
@
<<anovaRes,echo=FALSE>>=
fm3ML <- refitML(fm3)
fm2ML <- refitML(fm2)
fm1ML <- refitML(fm1)
ddiff <- deviance(fm3ML) - deviance(fm2ML)
dp <- pchisq(ddiff, 1, lower.tail = FALSE)
ddiff2 <- deviance(fm2ML) - deviance(fm1ML)
@
The output shows $\chi^2$ statistics representing the difference in
deviance between successive models, as well as $p$~values based on
likelihood ratio test comparisons.  In this case, the sequential
comparison shows that adding a linear effect of time uncorrelated with
the intercept leads to an enormous and significant drop in deviance
($\Delta\mbox{deviance} \approx \Sexpr{round(ddiff)}$, $p \approx
\Sexpr{round(dp,10)}$), while the further addition of correlation
between the slope and intercept leads to a trivial and non-significant
change in deviance ($\Delta\mbox{deviance} \approx
\Sexpr{round(ddiff2,2)}$).  For objects of class `\code{lmerMod}' the
default behavior is to refit the models with ML if fitted with
\code{REML = TRUE}, which is necessary in order to get sensible
answers when comparing models that differ in their fixed effects; this
can be controlled via the \code{refit} argument.

\subsubsection[Computing p values]{Computing $p$~values}
\label{sec:pvalues}

One of the more controversial design decisions of \pkg{lme4} has been
to omit the output of $p$~values associated with sequential ANOVA
decompositions of fixed effects.  The absence of analytical results for null
distributions of parameter estimates in complex situations
(e.g., unbalanced or partially crossed designs) is a long-standing
problem in mixed-model inference.  While the null distributions (and
the sampling distributions of non-null estimates) are asymptotically
normal, these distributions are not $t$~distributed for finite size
samples -- nor are the corresponding null distributions of
differences in scaled deviances $F$~distributed.  Thus approximate methods
for computing the approximate degrees of freedom for
$t$~distributions, or the denominator degrees of freedom for
$F$~statistics \citep{Satterthwaite_1946,kenward_small_1997}, are at
best \emph{ad hoc} solutions.

However, computing finite-size-corrected $p$~values is
sometimes necessary.  Therefore, although the package does not provide
them (except via parametric bootstrapping,
Section~\ref{sec:pb}), we have provided a help page to guide users in
finding appropriate methods:
<<pvaluesHelp, eval=FALSE>>=
help("pvalues")
@
This help page provides pointers to other packages that provide
machinery for calculating $p$~values associated with `\code{merMod}'
objects. It also suggests framing the inference problem as a
likelihood ratio test (achieved by supplying multiple `\code{merMod}'
objects to the \code{anova} method (Section~\ref{sec:modComp}), as
well as alternatives to $p$~values such as confidence intervals
(Section~\ref{sec:CI}).

Previous versions of \pkg{lme4} provided the \code{mcmcsamp} function,
which generated a Markov chain Monte Carlo sample from the posterior
distribution of the parameters (assuming flat priors). Due to
difficulty in constructing a version of \code{mcmcsamp} that was
reliable even in cases where the estimated random-effect variances
were near zero, \code{mcmcsamp} has been withdrawn.

\subsubsection{Computing confidence intervals}
\label{sec:CI}

As described above (Section~\ref{sec:vcov}), \pkg{lme4} provides confidence intervals (using
\code{confint}) via Wald approximations (for fixed-effect parameters
only), likelihood profiling, or parametric bootstrapping (the
\code{boot.type} argument selects the bootstrap confidence interval type).
<<compareCI,echo=FALSE,cache=TRUE,message=FALSE,warning=FALSE>>=
ccw <- confint(fm1, method = "Wald")
ccp <- confint(fm1, method = "profile", signames = FALSE)
ccb <- confint(fm1, method = "boot")
@
<<CIqcomp,echo=FALSE,eval=TRUE>>=
rtol <- function(x,y) {
    abs((x - y) / ((x + y) / 2))
}
bw <- apply(ccb, 1, diff)
pw <- apply(ccp, 1, diff)
mdiffpct <- round(100 * max(rtol(bw, pw)))
@
<<CIplot,echo=FALSE,eval=FALSE>>=
## obsolete
## ,fig.cap="Comparison of confidence intervals",fig.scap="CI comparison"
tf <- function(x, method) data.frame(method = method,
               par = rownames(x),
               setNames(as.data.frame(x), c("lwr", "upr")))
cc.all <- do.call(rbind, mapply(tf, list(ccw, ccp, ccb),
                               c("Wald", "profile", "boot"),
                     SIMPLIFY = FALSE))
ggplot(cc.all, aes(x = 1, ymin = lwr, ymax = upr, colour = method)) +
    geom_linerange(position = position_dodge(width = 0.5)) +
    facet_wrap( ~ par, scale = "free") +
    theme(axis.text.x = element_blank()) +
    labs(x = "")
@ %
As is typical for computationally intensive profile confidence
intervals in \proglang{R}, the intervals can be computed either directly from
fitted `\code{merMod}' objects (in which case profiling is done as an
interim step), or from a previously computed likelihood profile (of
class `\code{thpr}', for ``theta profile'').  Parametric bootstrapping
confidence intervals use a thin wrapper around the \code{bootMer}
function that passes the results to \code{boot.ci} from the \pkg{boot}
package \citep{boot_pkg,DavisonHinkley1997} for confidence interval calculation.

In the running sleep study example, the 95\% confidence intervals
estimated by all three methods are quite similar.  The largest
change is a \Sexpr{mdiffpct}\% difference in confidence
interval widths between profile and parametric bootstrap methods for
the correlation between the intercept and slope random effects
(\{\Sexpr{round(ccb[2,1],2)},\Sexpr{round(ccb[2,2],2)}\} vs.
\{\Sexpr{round(ccp[2,1],2)},\Sexpr{round(ccp[2,2],2)}\}).
%% \bmb{Mention that in our experience the profile CIs are
%% reasonably reliable for large and/or well-behaved data sets?
%% Is that too vague?  Disclaimer that a simulation study would
%% be required to know for sure?}

\subsubsection{General profile zeta and related plots}
\label{sec:profzeta}

\pkg{lme4} provides several functions (built on \pkg{lattice}
graphics, \citealt{lattice}) for plotting the profile zeta functions
(Section~\ref{sec:profile}) and other related quantities.

\begin{itemize}
\item The \emph{profile zeta} plot
  (Figure~\ref{fig:profile_zeta_plot}; \code{xyplot}) is simply a plot
  of the profile zeta function for each model parameter; linearity of
  this plot for a given parameter implies that the likelihood profile
  is quadratic (and thus that Wald approximations would be reasonably
  accurate).
\item The \emph{profile density plot}
  (Figure~\ref{fig:profile_density_plot}; \code{densityplot}) displays
  an approximation of the probability density function of the sampling
  distribution for each parameter.  These densities are derived by
  setting the cumulative distribution function (c.d.f) to be
  $\Phi(\zeta)$ where $\Phi$ is the c.d.f.{} of the standard normal
  distribution.  This is not quite the same as evaluating the
  distribution of the estimator of the parameter, which for mixed
  models can be very difficult, but it gives a reasonable
  approximation.  If the profile zeta plot is linear, then the profile
  density plot will be Gaussian.
\item The \emph{profile pairs plot}
  (Figure~\ref{fig:profile_pairs_plot}; \code{splom}) gives an
  approximation of the two-dimensional profiles of pairs of
  parameters, interpolated from the univariate profiles as described
  in \citet[Chapter~6]{bateswatts88:_nonlin}.  The profile pairs plot
  shows two-dimensional 50\%, 80\%, 90\%, 95\% and 99\% marginal
  confidence regions based on the likelihood ratio, as well as the
  \emph{profile traces}, which indicate the conditional estimates of
  each parameter for fixed values of the other parameters.  While
  panels above the diagonal show profiles with respect to the original
  parameters (with random-effects parameters on the standard
  deviation\slash{}correlation scale, as for all profile plots), the
  panels below the diagonal show plots on the $(\zeta_i,\zeta_j)$
  scale.  The below-diagonal panels allow us to see distortions from
  an elliptical shape due to nonlinearity of the traces, separately
  from the one-dimensional distortions caused by a poor choice of
  scale for the parameters.  The $\zeta$ scales provide, in some sense,
  the best possible set of single-parameter transformations for
  assessing the contours.  On the $\zeta$ scales the extent of a
  contour on the horizontal axis is exactly the same as the extent on
  the vertical axis and both are centered about zero.
\end{itemize}
<<profile_calc,echo=FALSE,cache=TRUE>>=
pf <- profile(fm1)
@
<<profile_zeta_plot,fig.cap="Profile zeta plot: \\code{xyplot(prof.obj)}",fig.scap="Profile zeta plot",echo=FALSE,fig.align='center',fig.pos='tb'>>=
xyplot(pf)
@
<<profile_density_plot,fig.cap="Profile density plot: \\code{densityplot(prof.obj)}",echo=FALSE,fig.align='center',fig.pos='tb'>>=
densityplot(pf)
@
<<profile_pairs_plot,fig.cap="Profile pairs plot: \\code{splom(prof.obj)}",echo=FALSE,fig.height=8,fig.width=8,fig.align='center',fig.pos='htb',out.height='5.5in'>>=
splom(pf)
@

For users who want to build their own graphical displays of the
profile, there is a method for  \code{as.data.frame} that converts profile
(`\code{thpr}') objects to a more convenient format.

\subsubsection{Computing fitted and predicted values; simulating}
\label{sec:predict}

Because mixed models involve random coefficients, one must always
clarify whether predictions should be based on the marginal
distribution of the response variable or on the distribution that is
conditional on the modes of the random effects
(Equation~\ref{eq:distYgivenU}).  The \code{fitted} method retrieves fitted
values that are conditional on all of the modes of the random effects;
the \code{predict} method returns the same values by default, but also
allows for predictions to be made conditional on different sets of
random effects.  For example, if the \code{re.form} argument is set to
\code{NULL} (the default), then the predictions are conditional on all
random effects in the model; if \code{re.form} is \verb+~0+ or
\code{NA}, then the predictions are made at the population level (all
random-effect values set to zero).  In models with multiple random
effects, the user can give \code{re.form} as a formula that specifies
which random effects are conditioned on.

\pkg{lme4} also provides a \code{simulate} method, which allows
similar flexibility in conditioning on random effects; in addition it
allows the user to choose (via the \code{use.u} argument) between
conditioning on the fitted conditional modes or choosing a new set of
simulated conditional modes (zero-mean Normal deviates chosen
according to the estimated random-effects variance-covariance
matrices).  Finally, the \code{simulate} method has a method for
`\code{formula}' objects, which allows for \emph{de novo} simulation in
the absence of a fitted model.  In this case, the user must specify
the random-effects ($\bm\theta$), fixed-effects ($\bm\beta$), and
residual standard deviation ($\sigma$) parameters via the
\code{newparams} argument.  The standard simulation method (based on a
`\code{merMod}' object) is the basis of parametric bootstrapping
(Section~\ref{sec:pb}) and posterior predictive simulation
(Section~\ref{sec:diagnostics}); \emph{de novo} simulation based on a
formula provides a flexible framework for power analysis.

\section{Conclusion}

Mixed modeling is an extremely useful but computationally intensive
technique.  Computational limitations are especially important because
mixed models are commonly applied to moderately large data sets
($10^4$--$10^6$ observations).  By developing an efficient, general,
and (now) well-documented platform for fitted mixed models in
\proglang{R}, we hope to provide both a practical tool for end users
interested in analyzing data and a reusable, modular framework for
downstream developers interested in extending the class of models that
can be easily and efficiently analyzed in \proglang{R}.

%% Results of some quick Google scholar searches:
%% "mixed models" +
%%  stata: 6130
%%  "PROC MIXED": 12200
%%  CRAN: 4020
%%  lme4: 5110
%%  "nlme": 2960
%%  SAS: 49500
%%  NLMIXED: 1960
%%  GLIMMIX: 4920
%%  winbugs: 2250
%%  gllamm: 1140

We have learned much about linear mixed models in the process of
developing \pkg{lme4}, both from our own attempts to develop robust
and reliable procedures and from the broad community of \pkg{lme4}
users; we have attempted to describe many of these lessons here.  In
moving forward, our main priorities are (1) to maintain the reference
implementation of \pkg{lme4} on the Comprehensive \proglang{R} Archive Network (CRAN), developing relatively few new
features; (2) to improve the flexibility, efficiency and scalability
of mixed-model analysis across multiple compatible implementations,
including both the \pkg{MixedModels} package for \proglang{Julia} and
the experimental \pkg{flexLambda} branch of \pkg{lme4}.

\section*{Acknowledgments}

Rune Haubo Christensen, Henrik Singmann, Fabian Scheipl,
Vincent Dorie, and Bin Dai contributed ideas
and code to the current version of \pkg{lme4}; the
large community of \pkg{lme4} users has exercised the software,
discovered bugs, and made many useful suggestions.
S{\o}ren H{\o}jsgaard participated in useful discussions
and Xi Xia  and Christian Zingg exercised the code
and reported problems.
We would like to thank the Banff International Research Station
for hosting a working group on \pkg{lme4}, and
an NSERC Discovery grant and NSERC postdoctoral fellowship for funding.

\clearpage

\bibliography{lmer}

\clearpage
% \bigskip \par \pagebreak[3]

\begin{appendix}

\section{Modularization examples}
\label{sec:modularExamples}

The modularized functions in \pkg{lme4} allow for finer control of the
various steps in fitting mixed models (Table~\ref{tab:modular}).  The
main advantage of these functions is the ability to extend the class
of models that \code{lmer} is capable of fitting.  In general there
are three steps to this process.
\begin{itemize}
\item Construct a mixed-model formula that is similar to the intended
  model.
\item Using the modular functions, hook into and modify the fitting
  process so that the desired model is estimated.
\item Use the estimated parameter values to make inferences.
\end{itemize}
Note that the many tools of the output module become unavailable for
sufficiently extensive departures from the class of models fitted by
\code{lmer}.  Therefore, this last inference step may be more involved
in the modular approach than with inference from models fitted by
\code{lmer}.  Here we provide two examples that illustrate this
process.

\subsection{Zero slope-slope covariance models}
\label{sec:zeroSlopeSlope}

Consider the following mixed-model formula,
<<modularSimulationFormula>>=
form <- respVar ~ 1 + (explVar1 + explVar2 | groupFac)
@ %
in which we have a fixed intercept and two random-effects terms, each
with a random slope and intercept.  The \code{lmer} function would fit
this formula to data by estimating an unstructured $3 \times 3$
covariance matrix giving the variances and covariances among the
intercept and the two slopes.  It is not possible to use the current
version of \code{lmer} to fit a model with the following two
properties: (1) the slopes are uncorrelated and (2) the intercept is
correlated with both of the slopes.  We illustrate the use of the
modular functions in \pkg{lme4} to fit such a model.

\subsubsection{Simulate from a zero slope-slope covariance model}

We simulate data from such a model using a balanced design with a
single grouping factor.  We use \pkg{lme4}'s utilities for
constructing a template from the model formula, with which to conduct
these simulations.  We simulate a balanced design with 500 levels to
the grouping factor and 20 observations per group,
<<dataTemplate>>=
set.seed(1)
dat <- mkDataTemplate(form,
  nGrps = 500,
  nPerGrp = 20,
  rfunc = rnorm)
head(dat)
@ %
The initial parameters for this model may be obtained using
\code{mkParsTemplate}.
<<parsTemplate>>=
(pars <- mkParsTemplate(form, dat))
@ %
We wish to simulate random effects from the following covariance
matrix,
<<simCorr>>=
vc <- matrix(c(1.0, 0.5, 0.5,
  0.5, 1.0, 0.0,
  0.5, 0.0, 1.0), 3, 3)
@ %
Here the intercept is correlated with each of the two slopes, but the
slopes are uncorrelated.  The \code{theta} vector that corresponds to
this covariance matrix can be computed using \pkg{lme4}'s facilities
for converting between various representations of (co-)variance
structures (see \code{?vcconv}).
<<vcTheta>>=
pars$theta[] <- Vv_to_Cv(mlist2vec(vc))
@ %
With these new parameters installed, we simulate a response variable.
<<varCorrStructure>>=
dat$respVar <- simulate(form,
  newdata = dat,
  newparams = pars,
  family = "gaussian")[[1]]
@ %
Indeed this data set appears to contain intercept-slope correlations,
but not slope-slope correlations, as we see using the \code{lmList}
function.
<<graphSims>>=
formLm <- form
formLm[[3]] <- reformulas::findbars(form)[[1]]
print(formLm)
cor(t(sapply(lmList(formLm, dat), coef)))
@ %
This \code{lmList} function computes ordinary linear models for each
level of the grouping factor.  Next we show how to fit a proper mixed
model to these data.

\subsubsection{Fitting a zero slope-slope covariance model}

The main challenge with fitting a zero slope-slope model is the need
to figure out which Cholesky decompositions correspond to
variance-covariance matrices with structural zeros between the slopes.
There are two general approaches to solving this problem.  The first
is to reorder the rows and columns of the covariance matrix so that
the intercept column comes last.  In this case, setting the second
component of the covariance parameter vector, $\bm\theta$, to zero will
achieve the desired covariance structure.  However, we consider a
second approach, which retains the ordering produced by the
\code{lFormula} function: (1) intercept, (2) first slope, and (3)
second slope. The covariance structure for the random effects in this
model is given by the following covariance matrix and its associated
Cholesky decomposition.
\begin{equation}
  \label{eq:1}
  \begin{bmatrix}
    \sigma_1^2 & \sigma_{12} & \sigma_{13} \\
    \sigma_{21} & \sigma_2^2 & 0 \\
    \sigma_{31} & 0 & \sigma_3^2 \\
  \end{bmatrix} =
  \begin{bmatrix}
    \theta_1 & 0 & 0 \\
    \theta_2 & \theta_4 & 0 \\
    \theta_3 & \theta_5 & \theta_6 \\
  \end{bmatrix}
  \begin{bmatrix}
    \theta_1 & \theta_2 & \theta_3 \\
    0 & \theta_4 & \theta_5 \\
    0 & 0 & \theta_6 \\
  \end{bmatrix}
\end{equation}
where the $\sigma$ parameters give the standard deviations and
covariances, and the $\theta$ parameters are the elements of the
Cholesky decomposition.  The key feature of this covariance matrix is
the structural zero indicating no covariance between the two
slopes.  Therefore, in order to ensure the structural zero, we must
have,
\begin{equation}
  \label{eq:2}
  \theta_5 = -\frac{\theta_2\theta_3}{\theta_4}.
\end{equation}
To represent this restriction in \proglang{R} we introduce a length
five vector, \code{phi}, which we convert into an appropriate
\code{theta} vector using the following function.
<<phiToTheta, cache = FALSE>>=
phiToTheta <- function(phi) {
    theta5 <- -(phi[2]*phi[3])/phi[4]
    c(phi[1:4], theta5, phi[5])
}
@ %
We may therefore optimize the structured model by optimizing over
\code{phi} instead of \code{theta}.  We begin to construct an
appropriate objective function for such an optimization by taking the
first two modular steps, resulting in a deviance function of
\code{theta}.
<<compute deviance function modular, cache = FALSE>>=
lf <- lFormula(formula = form, data = dat, REML = TRUE)
devf <- do.call(mkLmerDevfun, lf)
@ %
This deviance function is a function of the unconstrained \code{theta}
vector, but we require a deviance function of the constrained
\code{phi} vector.  We therefore construct a wrapper around the
deviance function as follows.
<<wrapper modular, cache = FALSE>>=
devfWrap <- function(phi) devf(phiToTheta(phi))
@ %
Finally, we optimize the deviance function,
<<opt modular, cache = FALSE>>=
opt <- bobyqa(par = lf$reTrms$theta[-5],
  fn = devfWrap,
  lower = lf$reTrms$lower[-5])
@ %
To extract and label the estimated covariance matrix of the random
effects, we may use the conversion functions (\code{?vcconv}) and the
column names of the random-effects model matrix (\code{cnms}).  We
construct this estimated matrix and compare it with the true matrix
used to simulate the data.
<<varCorr modular, cache = FALSE>>=
vcEst <- vec2mlist(Cv_to_Vv(phiToTheta(opt$par)))[[1]]
dimnames(vcEst) <- rep(lf$reTrms$cnms, 2)
round(vcEst, 2)
vc
@ %
We see that the estimated covariance matrix provides a good estimate
of the true matrix.

\subsection{Additive models}
\label{sec:additive}

It is possible to interpret additive models as a particular class of
mixed models \citep{gamm4}.  The main benefit of this approach is
that it bypasses the need to select a smoothness parameter using
cross-validation or generalized cross-validation.  However, it is
inconvenient to specify additive models using the \pkg{lme4} formula
interface.  The \pkg{gamm4} package wraps the modularized functions of
\pkg{lme4} within a more convenient interface \citep{gamm4}.  In
particular, the strategy involves the following steps,
\begin{enumerate}
\item Convert a \pkg{gamm4} formula into an \pkg{lme4} formula that
  approximates the intended model.
\item Parse this formula using \code{lFormula}.
\item Modify the resulting transposed random-effects model matrix, $\bm
  Z\trans$, so that the intended additive model results.
\item Fit the resulting model using the remaining modularized
  functions (Table~\ref{tab:modular}).
\end{enumerate}
Here we illustrate this general strategy using a simple simulated data
set,
<<simulateSplineData,message=FALSE>>=
library("gamm4")
library("splines")
set.seed(1)
n <- 100
pSimulation <- 3
pStatistical <- 8
x <- rnorm(n)
Bsimulation <- ns(x, pSimulation)
Bstatistical <- ns(x, pStatistical)
beta <- rnorm(pSimulation)
y <- as.numeric(Bsimulation %*% beta + rnorm(n, sd = 0.3))
@ %
where \code{pSimulation} is the number of columns in the simulation
model matrix and \code{pStatistical} is the number of columns in the
statistical model matrix.

We plot the resulting data along with the predictions from the
generating model,  %% BMB: stopped here
<<splineExampleDataPlot, fig.width=4, fig.height=3,fig.align="center">>=
par(mar = c(4, 4, 1, 1), las = 1, bty = "l")
plot(x, y, las = 1)
lines(x[order(x)], (Bsimulation %*% beta)[order(x)])
@ %
We now set up an approximate \pkg{lme4} model formula, and parse it
using \code{lFormula},
<<splineExampleApproximateFormula>>=
pseudoGroups <- as.factor(rep(1:pStatistical, length = n))
parsedFormula <- lFormula(y ~ x + (1 | pseudoGroups))
@ %
We now insert a spline basis into the \code{parsedFormula} object,
<<splineExampleModifyZt>>=
parsedFormula$reTrms <- within(parsedFormula$reTrms, {
    Bt <- t(as.matrix(Bstatistical))[]
    cnms$pseudoGroups <- "spline"
    Zt <- as(Bt, class(Zt))
})
@ %
Finally we continue with the remaining modular steps,
<<splineExampleRemainingModularSteps>>=
devianceFunction <- do.call(mkLmerDevfun, parsedFormula)
optimizerOutput <- optimizeLmer(devianceFunction)
mSpline <- mkMerMod(   rho = environment(devianceFunction),
                       opt = optimizerOutput,
                    reTrms = parsedFormula$reTrms,
                        fr = parsedFormula$fr)
mSpline
@ %
Computing the fitted values of this additive model requires some
custom code, and illustrates the general principle that methods for
\code{merMod} objects constructed from modular fits should only be
used if the user knows what she is doing,
<<splineExampleFittedModelPlot, fig.width=4, fig.height=3, fig.align="center">>=
xNew <- seq(min(x), max(x), length = 100)
newBstatistical <- predict(Bstatistical, xNew)
yHat <-   cbind(1, xNew) %*% getME(mSpline, "fixef") +
        newBstatistical %*% getME(mSpline, "u")
par(mar = c(4, 4, 1, 1), las = 1, bty = "l")
plot(x, y)
lines(xNew, yHat)
lines(x[order(x)], (Bsimulation %*% beta)[order(x)],lty = 2)
legend("topright", bty = "n", c("fitted", "generating"), lty = 1:2,col = 1)
@
\end{appendix}

\end{document}

%

%%% Local Variables:
%%% ess-swv-processor: knitr
%%% TeX-master: t
%%% End:

%%\VignetteIndexEntry{Computational Methods}
%%\VignetteDepends{lme4}
%%\VignetteEngine{knitr::knitr}
\documentclass[12pt]{article}
\usepackage{amsmath,amsfonts,bm}
\usepackage[authoryear,round]{natbib}
\bibliographystyle{plainnat}
\title{Computational methods for mixed models}
\author{Douglas Bates\\Department of Statistics\\%
  University of Wisconsin -- Madison}
\newcommand{\code}[1]{\texttt{\small{#1}}}
\newcommand{\package}[1]{\textsf{\small{#1}}}
\newcommand{\trans}{\ensuremath{^\prime}}
\renewcommand{\vec}{\operatorname{vec}}
\newcommand{\diag}{\operatorname{diag}}
\newcommand{\bc}[1]{\ensuremath{\bm{\mathcal{#1}}}}
<<preliminaries,include=FALSE>>=
knitr::opts_chunk$set(include=FALSE)
options(width=65,digits=5)
#library(lme4)
@

\begin{document}
\maketitle

\begin{abstract}
  The \package{lme4} package provides R functions to fit and analyze
  several different types of mixed-effects models, including linear
  mixed models, generalized linear mixed models and nonlinear mixed
  models.  In this vignette we describe the formulation of these
  models and the computational approach used to evaluate or
  approximate the log-likelihood of a model/data/parameter value
  combination.
\end{abstract}

\section{Introduction}
\label{sec:intro}

The \package{lme4} package provides \code{R} functions to fit and analyze
linear mixed models, generalized linear mixed models and nonlinear
mixed models.  These models are called \emph{mixed-effects models} or,
more simply, \emph{mixed models} because they incorporate both
\emph{fixed-effects} parameters, which apply to an entire population
or to certain well-defined and repeatable subsets of a population, and
\emph{random effects}, which apply to the particular experimental
units or observational units in the study.  Such models are also
called \emph{multilevel} models because the random effects represent
levels of variation in addition to the per-observation noise term that
is incorporated in common statistical models such as linear
regression models, generalized linear models and nonlinear regression
models.

We begin by describing common properties of these mixed models and the
general computational approach used in the \package{lme4} package. The
estimates of the parameters in a mixed model are determined as the
values that optimize an objective function --- either the likelihood
of the parameters given the observed data, for maximum likelihood
(ML) estimates, or a related objective function called the REML
criterion.  Because this objective function must be evaluated at many
different values of the model parameters during the optimization
process, we focus on the evaluation of the objective function and a
critical computation in this evaluation --- determining the solution to a
penalized, weighted least squares (PWLS) problem.

The dimension of the solution of the PWLS problem can be very large,
perhaps in the millions.  Furthermore, such problems must be solved
repeatedly during the optimization process to determine parameter
estimates.  The whole approach would be infeasible were it not for the
fact that the matrices determining the PWLS problem are sparse and we
can use sparse matrix storage formats and sparse matrix computations
\citep{davis06:csparse_book}. In particular, the whole computational
approach hinges on the extraordinarily efficient methods for
determining the Cholesky decomposition of sparse, symmetric,
positive-definite matrices embodied in the CHOLMOD library of C
functions \citep{Cholmod}.

% The three types of mixed models -- linear, generalized linear and
% nonlinear -- share common characteristics in that the model is
% specified in whole or in part by a \emph{mixed model formula} that
% describes a \emph{linear predictor} and a variance-covariance
% structure for the random effects.  In the next section we describe
% the mixed model formula and the forms of these matrices.  The
% following section presents a general formulation of the Laplace
% approximation to the log-likelihood of a mixed model.

% In subsequent sections we describe computational methods for specific
% kinds of mixed models.  In particular, we should how a profiled
% log-likelihood for linear mixed models, and for some nonlinear mixed
% models, can be evaluated exactly.

In the next section we describe the general form of the mixed models
that can be represented in the \package{lme4} package and the
computational approach embodied in the package.  In the following
section we describe a particular form of mixed model,
called a linear mixed model, and the computational details for those
models.  In the fourth section we describe computational methods for
generalized linear mixed models, nonlinear mixed models and
generalized nonlinear mixed models.


\section{Formulation of mixed models}
\label{sec:form-mixed-models}

A mixed-effects model incorporates two vector-valued random variables:
the $n$-dimensional response vector, $\bc Y$, and the $q$-dimensional
random effects vector, $\bc B$. We observe the value, $\bm y$, of $\bc Y$.
We do not observe the value of $\bc B$.

The random variable $\bc Y$ may be continuous or discrete.  That is,
the observed data, $\bm y$, may be on a continuous scale or they may
be on a discrete scale, such as binary responses or responses
representing a count. In our formulation, the random variable $\bc B$
is always continous.

We specify a mixed model by describing the unconditional distribution
of $\bc B$ and the conditional distribution $(\bc Y|\bc B=\bm b)$.

\subsection{The unconditional distribution of $\bc B$}
\label{sec:uncond-distr-B}

In our formulation, the unconditional distribution of $\bc B$ is
always a $q$-dimensional multivariate Gaussian (or ``normal'')
distribution with mean $\bm 0$ and with a parameterized covariance
matrix,
\begin{equation}
  \label{eq:2}
  \bc B\sim\mathcal{N}\left(\bm 0,\sigma^2\bm\Lambda(\bm\theta)
  \bm\Lambda\trans(\bm\theta)\right) .
\end{equation}

The scalar, $\sigma$, in (\ref{eq:2}), is called the \emph{common scale
  parameter}.  As we will see later, not all types of mixed models
incorporate this parameter. We will include $\sigma^2$ in the general
form of the unconditional distribution of $\bc B$ with the
understanding that, in some models, $\sigma\equiv 1$.

The $q\times q$ matrix $\bm\Lambda(\bm\theta)$, which is a left factor
of the covariance matrix (when $\sigma=1$) or the relative covariance
matrix (when $\sigma\ne 1$), depends on an $m$-dimensional parameter
$\bm\theta$.  Typically $m\ll q$; in the examples we show below it is
always the case that $m<5$, even when $q$ is in the thousands.  The
fact that $m$ is very small is important because, as we shall see,
determining the parameter estimates in a mixed model can be expressed
as an optimization problem with respect to $\bm\theta$ only.

The parameter $\bm\theta$ may be, and typically is, subject to
constraints. For ease of computation, we require that the constraints
be expressed as ``box'' constraints of the form
$\theta_{iL}\le\theta_i\le\theta_{iU},i=1,\dots,m$ for constants
$\theta_{iL}$ and $\theta_{iU}, i=1,\dots,m$.  We shall write the set
of such constraints as $\bm\theta_L\le\bm\theta\le\bm\theta_R$.  The matrix
$\bm\Lambda(\bm\theta)$ is required to be non-singular
(i.e.{} invertible) when $\bm\theta$ is not on the boundary.

\subsection{The conditional distribution,  $(\bc Y|\bc B=\bm b)$}
\label{sec:cond-distr-YB}

The conditional distribution, $(\bc Y|\bc B=\bm b)$, must satisfy:
\begin{enumerate}
\item The conditional mean, $\bm\mu_{\bc Y|\bc B}(\bm b) =
  \mathrm{E}[\bc Y|\bc B=\bm b]$, depends on $\bm b$ only through the
  value of the \emph{linear predictor}, $\bm Z\bm b+\bm X\bm\beta$,
  where $\bm\beta$ is the $p$-dimensional \emph{fixed-effects}
  parameter vector and the \emph{model matrices}, $\bm Z$ and $\bm X$,
  are fixed matrices of the appropriate dimension.  That is, the
  two model matrices must have the same number of rows and must have
  $q$ and $p$ columns, respectively.  The number of rows
  in $\bm Z$ and $\bm X$ is a multiple of $n$, the dimension of $\bm y$.
\item The scalar distributions, $(\mathcal{Y}_i|\bc B=\bm
  b),i=1,\dots,n$, all have the same form and are completely
  determined by the conditional mean, $\bm\mu_{\bc Y|\bc B}(\bm b)$
  and, at most, one additional parameter, $\sigma$, which is the
  common scale parameter.
\item The scalar distributions, $(\mathcal{Y}_i|\bc B=\bm
  b),i=1,\dots,n$, are independent.  That is, the components of
  $\bc Y$ are \emph{conditionally independent} given $\bc B$.
\end{enumerate}

An important special case of the conditional distribution is the
multivariate Gaussian distribution of the form
\begin{equation}
  \label{eq:1}
  (\bc Y|\bc B=\bm b)\sim\mathcal{N}(\bm Z\bm b+\bm
  X\bm\beta,\sigma^2\bm I_n)
\end{equation}
where $\bm I_n$ denotes the identity matrix of size $n$.
In this case the conditional mean, $\bm\mu_{\bc Y|\bc B}(\bm b)$, is
exactly the linear predictor, $\bm Z\bm b+\bm X\bm\beta$, a situation
we will later describe as being an ``identity link'' between the
conditional mean and the linear predictor.  Models with conditional
distribution (\ref{eq:1}) are called \emph{linear mixed models}.

\subsection{A change of variable to ``spherical'' random effects}
\label{sec:change-vari-spher}

Because the conditional distribution $(\bc Y|\bc B=\bm b)$ depends on
$\bm b$ only through the linear predictor, it is easy to express the
model in terms of a linear transformation of $\bc B$.  We define the
linear transformation from a $q$-dimensional ``spherical'' Gaussian
random variable, $\bc U$, to $\bc B$ as
\begin{equation}
  \label{eq:3}
  \bc B=\bm\Lambda(\bm\theta)\bc U,\quad
  \bc U\sim\mathcal{N}(\bm 0,\sigma^2\bm I_q).
\end{equation}
(The term ``spherical'' refers to the fact that contours of constant
probability density for $\bc U$ are spheres centered at the mean ---
in this case, $\bm0$.)

When $\bm\theta$ is not on the boundary this is an invertible
transformation.  When $\bm\theta$ is on the boundary the
transformation can fail to be invertible.  However, we will only need
to be able to express $\bc B$ in terms of $\bc U$ and that
transformation is well-defined, even when $\bm\theta$ is on the
boundary.

The linear predictor, as a function of $\bm u$, is
\begin{equation}
  \label{eq:4}
  \bm\gamma(\bm u)=\bm Z\bm\Lambda(\bm\theta)\bm u
  + \bm X\bm\beta.
\end{equation}
When we wish to emphasize the role of the model parameters,
$\bm\theta$ and $\bm\beta$, in the formulation of $\bm\gamma$, we will
write the linear predictor as $\bm\gamma(\bm u,\bm\theta,\bm\beta)$.

\subsection{The conditional density $(\bc U|\bc Y=\bm y)$}
\label{sec:cond-dens-bc}

Because we observe $\bm y$ and do not observe $\bm b$ or $\bm u$, the
conditional distribution of interest, for the purposes of statistical
inference, is $(\bc U|\bc Y=\bm y)$ (or, equivalently, $(\bc B|\bc
Y=\bm y)$).  This conditional distribution is always a continuous
distribution with conditional probability density $f_{\bc U|\bc
  Y}(\bm u|\bm y)$.

We can evaluate $f_{\bc U|\bc Y}(\bm u|\bm y)$ , up to a constant, as
the product of the unconditional density, $f_{\bc U}(\bm u)$, and the
conditional density (or the probability mass function, whichever is
appropriate), $f_{\bc Y|\bc U}(\bm y|\bm u)$.  We write this
unnormalized conditional density as
\begin{equation}
  \label{eq:5}
  h(\bm u|\bm y,\bm\theta,\bm\beta,\sigma) =
  f_{\bc Y|\bc U}(\bm y|\bm u,\bm\theta,\bm\beta,\sigma)
  f_{\bc U}(\bm u|\sigma) .
\end{equation}

We say that $h$ is the ``unnormalized'' conditional density because
all we know is that the conditional density is proportional to $h(\bm
u|\bm y,\bm\theta,\bm\beta,\sigma)$.  To obtain the conditional
density we must normalize $h$ by dividing by the value of the integral
\begin{equation}
  \label{eq:6}
  L(\bm\theta,\bm\beta,\sigma|\bm y) =
  \int_{\mathbb{R}^q}h(\bm u|\bm y,\bm\theta,\bm\beta,\sigma)\,d\bm u .
\end{equation}
We write the value of the integral (\ref{eq:6}) as
$L(\bm\theta,\bm\beta,\sigma|\bm y)$ because it is exactly the
\emph{likelihood} of the parameters $\bm\theta$, $\bm\beta$ and
$\sigma$, given the observed data $\bm y$.  The \emph{maximum
  likelihood (ML) estimates} of these parameters are the values that
maximize $L$.

\subsection{Determining the ML estimates}
\label{sec:DeterminingML}

The general problem of maximizing $L(\bm\theta,\bm\beta,\sigma|\bm y)$
with respect to $\bm\theta$, $\bm\beta$ and $\sigma$ can be formidable
because each evaluation of this function involves a potentially
high-dimensional integral and because the dimension of $\bm\beta$ can
be large.  However, this general optimization problem can be split
into manageable subproblems.  Given a value of $\bm\theta$ we can
determine the \emph{conditional mode}, $\tilde{\bm u}(\bm\theta)$, of
$\bm u$ and the \emph{conditional estimate},
$\tilde{\bm\beta}(\bm\theta)$ simultaneously using \emph{penalized,
  iteratively re-weighted least squares} (PIRLS).  The conditional
mode and the conditional estimate are defined as
\begin{equation}
  \label{eq:condmode}
  \begin{bmatrix}
    \tilde{\bm u}(\bm\theta)\\
    \tilde{\bm\beta}(\bm\theta)
  \end{bmatrix}=\arg\max_{\bm u,\bm\beta}h(\bm u|\bm
  y,\bm\theta,\bm\beta,\sigma) .
\end{equation}
(It may look as if we have missed the dependence on $\sigma$ on the
left-hand side but it turns out that the scale parameter does not
affect the location of the optimal values of quantities in the linear
predictor.)

As is common in such optimization problems, we re-express the conditional
density on the \emph{deviance scale}, which is negative twice the
logarithm of the density, where the optimization becomes
\begin{equation}
  \label{eq:condmode2}
  \begin{bmatrix}
    \tilde{\bm u}(\bm\theta)\\
    \tilde{\bm\beta}(\bm\theta)
  \end{bmatrix}=\arg\min_{\bm u,\bm\beta}-2\log\left(h(\bm u|\bm
  y,\bm\theta,\bm\beta,\sigma)\right) .
\end{equation}
It is this optimization problem that can be solved quite efficiently
using PIRLS.  In fact, for linear mixed models, which are described in
the next section, $\tilde{\bm u}(\bm\theta)$ and
$\tilde{\bm\beta}(\bm\theta)$ can be directly evaluated.

The second-order Taylor series expansion of $-2\log h$ at $\tilde{\bm
  u}(\bm\theta)$ and $\tilde{\bm\beta}(\bm\theta)$ provides the
Laplace approximation to the profiled deviance.  Optimizing this
function with respect to $\bm\theta$ provides the ML estimates of
$\bm\theta$, from which the ML estimates of $\bm\beta$ and $\sigma$
(if used) are derived.

\section{Methods for linear mixed models}
\label{sec:pwls-problem}

As indicated in the introduction, a critical step in our methods for
determining the maximum likelihood estimates of the parameters in a
mixed model is solving a penalized, weighted least squares (PWLS)
problem.  We will motivate the general form of the PWLS problem by
first considering computational methods for linear mixed models that
result in a penalized least squares (PLS) problem.

Recall from \S\ref{sec:cond-distr-YB} that, in a linear mixed model,
both the conditional distribution, $(\bc Y|\bc U=\bm u)$, and the
unconditional distribution, $\bc U$, are spherical Gaussian
distributions and that the conditional mean, $\bm\mu_{\bc Y|\bc U}(\bm
u)$, is the linear predictor, $\bm\gamma(\bm u)$.  Because all the
distributions determining the model are continuous distributions, we
consider their densities.  On the deviance scale these are
\begin{equation}
  \label{eq:7}
  \begin{aligned}
    -2\log(f_{\bc U}(\bm u))&=q\log(2\pi\sigma^2)+\frac{\|\bm u\|^2}{\sigma^2}\\
    -2\log(f_{\bc Y|\bc U}(\bm y|\bm u))&=n\log(2\pi\sigma^2)+
    \frac{\|\bm y-\bm Z\bm\Lambda(\bm\theta)\bm u-\bm X\bm\beta\|^2}{\sigma^2}\\
    -2\log(h(\bm u|\bm y,\bm\theta,\bm\beta,\sigma))
    &= (n+q)\log(2\pi\sigma^2)+
    \frac{\|\bm y-\bm\gamma(\bm u,\bm\theta,\bm\beta)\|^2+\|\bm
      u\|^2}{\sigma^2}\\
    &= (n+q)\log(2\pi\sigma^2)+
    \frac{d(\bm u|\bm y,\bm\theta,\bm\beta)}{\sigma^2}
  \end{aligned}
\end{equation}

In (\ref{eq:7}) the \emph{discrepancy} function,
\begin{equation}
  \label{eq:9}
    d(\bm u|\bm y,\bm\theta,\bm\beta) =
    \|\bm y-\bm\gamma(\bm u,\bm\theta,\bm\beta)\|^2+\|\bm u\|^2
\end{equation}
has the form of a penalized residual sum of squares in that the first
term, $\|\bm y-\bm\gamma(\bm u,\bm\theta,\bm\beta)\|^2$ is
the residual sum of squares for $\bm y$, $\bm u$, $\bm\theta$ and
$\bm\beta$ and the second term, $\|\bm u\|^2$, is a penalty on the
size of $\bm u$.  Notice that the discrepancy does not depend on the
common scale parameter, $\sigma$.

\subsection{The canonical form of the discrepancy}
\label{sec:conditional-mode-bm}

Using a so-called ``pseudo data'' representation, we can write the
discrepancy as a residual sum of squares for a regression model that
is linear in both $\bm u$ and $\bm\beta$
\begin{equation}
  \label{eq:10}
  d(\bm u|\bm y,\bm\theta,\bm\beta) =\left\|
    \begin{bmatrix} \bm y\\\bm 0 \end{bmatrix} -
    \begin{bmatrix}
      \bm Z\bm\Lambda(\bm\theta) & \bm X \\
      \bm I_q & \bm0
    \end{bmatrix}
    \begin{bmatrix}\bm u\\\bm\beta\end{bmatrix}
  \right\|^2 .
\end{equation}
The term ``pseudo data'' reflects the fact that we have added $q$
``pseudo observations'' to the observed response, $\bm y$, and to the
linear predictor, $\bm\gamma(\bm u,\bm\theta,\bm\beta)=\bm
Z\bm\Lambda(\bm\theta)\bm u+\bm X\bm\beta$, in such a way that their
contribution to the overall residual sum of squares is exactly the
penalty term in the discrepancy.

In the form (\ref{eq:10}) we can see that the discrepancy is a
quadratic form in both $\bm u$ and $\bm\beta$.  Furthermore, because
we require that $\bm X$ has full column rank, the discrepancy is a
positive-definite quadratic form in $\bm u$ and $\bm\beta$ that is
minimized at $\tilde{\bm u}(\bm\theta)$ and
$\tilde{\bm\beta}(\bm\theta)$ satisfying
\begin{equation}
  \label{eq:13}
  \begin{bmatrix}
    \bm\Lambda\trans(\bm\theta)\bm Z\trans\bm Z\bm\Lambda(\theta)
    +\bm I_q&\bm\Lambda\trans(\bm\theta)\bm Z\trans\bm X\\
    \bm X\trans\bm Z\bm\Lambda(\theta) &\bm X\trans\bm X
  \end{bmatrix}
  \begin{bmatrix}
    \tilde{\bm u}(\bm\theta)\\\tilde{\bm\beta}(\bm\theta)
  \end{bmatrix} =
  \begin{bmatrix}
    \bm\Lambda\trans(\bm\theta)\bm Z\trans\bm y\\
    \bm X\trans\bm y
  \end{bmatrix}
\end{equation}

An effective way of determining the solution to a sparse, symmetric,
positive definite system of equations such as (\ref{eq:13}) is the
sparse Cholesky decomposition \citep{davis06:csparse_book}.  If $\bm A$
is a sparse, symmetric positive definite matrix then the sparse
Cholesky factor with fill-reducing permutation $\bm P$ is the
lower-triangular matrix $\bm L$ such that
\begin{equation}
  \label{eq:14}
  \bm L\bm L\trans=\bm P\bm A\bm P\trans .
\end{equation}
(Technically, the factor $\bm L$ is only determined up to changes in
the sign of the diagonal elements.  By convention we require the
diagonal elements to be positive.)

The fill-reducing permutation represented by the permutation matrix
$\bm P$, which is determined from the pattern of nonzeros in $\bm A$
but does not depend on particular values of those nonzeros, can have a
profound impact on the number of nonzeros in $\bm L$ and hence on the
speed with which $\bm L$ can be calculated from $\bm A$.

In most applications of linear mixed models the matrix $\bm
Z\bm\Lambda(\bm\theta)$ is sparse while $\bm X$ is dense or close to
it so the permutation matrix $\bm P$ can be restricted to the form
\begin{equation}
  \label{eq:15}
  \bm P=\begin{bmatrix}\bm P_{\bm Z}&\bm0\\
    \bm0&\bm P_{\bm X}\end{bmatrix}
\end{equation}
without loss of efficiency.  In fact, in most cases we can set $\bm
P_{\bm X}=\bm I_p$ without loss of efficiency.

Let us assume that the permutation matrix is required to be of the
form (\ref{eq:15}) so that we can write the Cholesky factorization for
the positive definite system (\ref{eq:13}) as
\begin{multline}
  \label{eq:16}
  \begin{bmatrix}
    \bm L_{\bm Z}&\bm0\\\bm L_{\bm{XZ}}&\bm L_{\bm X}
  \end{bmatrix}
  \begin{bmatrix}
    \bm L_{\bm Z}&\bm0\\\bm L_{\bm{XZ}}&\bm L_{\bm X}
  \end{bmatrix}\trans =\\
  \begin{bmatrix}\bm P_{\bm Z}&\bm0\\
    \bm0&\bm P_{\bm X}\end{bmatrix}
  \begin{bmatrix}
    \bm\Lambda\trans(\bm\theta)\bm Z\trans\bm Z\bm\Lambda(\theta)
    +\bm I_q&\bm\Lambda\trans(\bm\theta)\bm Z\trans\bm X\\
    \bm X\trans\bm Z\bm\Lambda(\theta) &\bm X\trans\bm X
  \end{bmatrix}
  \begin{bmatrix}\bm P_{\bm Z}&\bm0\\
    \bm0&\bm P_{\bm X}\end{bmatrix}\trans .
\end{multline}
The discrepancy can now be written in the canonical form
\begin{equation}
  \label{eq:17}
  d(\bm u|\bm y,\bm\theta,\bm\beta) =\tilde{d}(\bm y,\bm\theta) +
  \left\|
    \begin{bmatrix}
      \bm L_{\bm Z}\trans&\bm L_{\bm{XZ}}\trans\\
      \bm 0&\bm L_{\bm X}\trans
    \end{bmatrix}
    \begin{bmatrix}
      \bm P_{\bm Z}(\bm u-\tilde{\bm u})\\
      \bm P_{\bm X}(\bm\beta-\tilde{\bm\beta})
    \end{bmatrix}
  \right\|^2
\end{equation}
where
\begin{equation}
  \label{eq:18}
  \tilde{d}(\bm y,\bm\theta)=
  d(\tilde{\bm u}(\bm\theta)|\bm y,\bm\theta,\tilde{\bm\beta}(\bm\theta))
\end{equation}
is the minimum discrepancy, given $\bm\theta$.


\subsection{The profiled likelihood for linear mixed models}
\label{sec:prof-log-likel}

Substituting (\ref{eq:17}) into (\ref{eq:7}) provides the unnormalized
conditional density $h(\bm u|\bm y,\bm\theta,\bm\beta,\sigma)$ on the
deviance scale as
\begin{multline}
  \label{eq:32}
  -2\log(h(\bm u|\bm y,\bm\theta,\bm\beta,\sigma))\\=
  (n+q)\log(2\pi\sigma^2)+\frac{\tilde{d}(\bm y,\bm\theta) +
  \left\|
    \begin{bmatrix}
      \bm L_{\bm Z}\trans&\bm L_{\bm{XZ}}\trans\\
      \bm 0&\bm L_{\bm X}\trans
    \end{bmatrix}
    \begin{bmatrix}
      \bm P_{\bm Z}(\bm u-\tilde{\bm u})\\
      \bm P_{\bm X}(\bm\beta-\tilde{\bm\beta})
    \end{bmatrix}
  \right\|^2}{\sigma^2} .
\end{multline}
As shown in Appendix \ref{sec:integr-quadr-devi}, the integral of a
quadratic form on the deviance scale, such as (\ref{eq:32}), is easily
evaluated, providing the log-likelihood,
$\ell(\bm\theta,\bm\beta,\sigma|\bm y)$, as
\begin{multline}
  \label{eq:lmmdev}
  -2\ell(\bm\theta,\bm\beta,\sigma|\bm y)\\
  =-2\log\left(L(\bm\theta,\bm\beta,\sigma|\bm y)\right)\\
  =n\log(2\pi\sigma^2)+\log(|\bm L_{\bm Z}|^2)+\frac{\tilde{d}(\bm y,\bm\theta) +
  \left\|\bm L_{\bm X}\trans\bm P_{\bm X}(\bm\beta-\tilde{\bm\beta})\right\|^2}{\sigma^2},
\end{multline}
from which we can see that the conditional estimate of $\bm\beta$,
given $\bm\theta$, is $\tilde{\bm\beta}(\bm\theta)$ and the conditional
estimate of $\sigma$, given $\bm\theta$, is
\begin{equation}
  \label{eq:condsigma}
  \tilde{\sigma^2}(\bm\theta)= \frac{\tilde{d}(\bm\theta|\bm y)}{n} .
\end{equation}
Substituting these conditional estimates into (\ref{eq:lmmdev}) produces the
\emph{profiled likelihood}, $\tilde{L}(\bm\theta|\bm y)$, as
\begin{equation}
  \label{eq:19}
  -2\tilde{\ell}(\bm\theta|\bm y))=
  \log(|\bm L_{\bm Z}(\bm\theta)|^2)+
  n\left(1+\log\left(\frac{2\pi\tilde{d}(\bm y,\bm\theta)}{n}\right)\right) .
\end{equation}
The maximum likelihood estimate of $\bm\theta$ can then be expressed as
\begin{equation}
  \label{eq:29}
  \widehat{\bm\theta}_L=\arg\min_{\bm\theta}
  \left(-2\tilde{\ell}(\bm\theta|\bm y)\right) .
\end{equation}
from which the ML estimates of $\sigma^2$ and $\bm\beta$ are
evaluated as
\begin{align}
  \label{eq:30}
  \widehat{\sigma^2_L}&=
  \frac{\tilde{d}(\widehat{\bm\theta}_L,\bm y)}{n}\\
  \widehat{\bm\beta}_L&=\tilde{\bm\beta}(\widehat{\bm\theta}_L) .
\end{align}

The important thing to note about optimizing the profiled likelihood,
(\ref{eq:19}), is that it is a $m$-dimensional optimization problem
and typically $m$ is very small.

\subsection{The REML criterion}
\label{sec:reml-criterion}

In practice the so-called REML estimates of variance components are
often preferred to the maximum likelihood estimates.  (``REML'' can be
considered to be an acronym for ``restricted'' or ``residual'' maximum
likelihood, although neither term is completely accurate because these
estimates do not maximize a likelihood.) We can motivate the use of
the REML criterion by considering a linear regression model,
\begin{equation}
  \label{eq:20}
  \bc Y\sim\mathcal{N}(\bm X\bm\beta,\sigma^2\bm I_n),
\end{equation}
in which we typically estimate $\sigma^2$ by
\begin{equation}
  \label{eq:21}
  \widehat{\sigma^2_R}=\frac{\|\bm y-\bm X\widehat{\bm\beta}\|^2}{n-p}
\end{equation}
even though the maximum likelihood estimate of $\sigma^2$ is
\begin{equation}
  \label{eq:22}
  \widehat{\sigma^2_{L}}=\frac{\|\bm y-\bm
    X\widehat{\bm\beta}\|^2}{n} .
\end{equation}

The argument for preferring $\widehat{\sigma^2_R}$ to
$\widehat{\sigma^2_{L}}$ as an estimate of $\sigma^2$ is that the
numerator in both estimates is the sum of squared residuals at
$\widehat{\bm\beta}$ and, although the residual vector $\bm y-\bm
X\bm\beta$ is an $n$-dimensional vector, the residual at
$\widehat{\bm\theta}$ satisfies $p$ linearly independent constraints,
$\bm X\trans(\bm y-\bm X\widehat{\bm\beta})=\bm 0$. That is, the residual at
$\widehat{\bm\theta}$ is the projection of the observed response
vector, $\bm y$, into an $(n-p)$-dimensional linear subspace of the
$n$-dimensional response space.  The estimate $\widehat{\sigma^2_R}$
takes into account the fact that $\sigma^2$ is estimated from
residuals that have only $n-p$ \emph{degrees of freedom}.

The REML criterion for determining parameter estimates
$\widehat{\bm\theta}_R$ and $\widehat{\sigma_R^2}$ in a linear mixed
model has the property that these estimates would
specialize to $\widehat{\sigma^2_R}$ from (\ref{eq:21}) for a linear
regression model.  Although not usually derived in this way, the REML
criterion can be expressed as
\begin{equation}
  \label{eq:23}
  c_R(\bm\theta,\bm\sigma|\bm y)=-2\log
  \int_{\mathbb{R}^p}L(\bm u|\bm
  y,\bm\theta,\bm\beta,\sigma)\,d\bm\beta
\end{equation}
on the deviance scale.  The REML estimates $\widehat{\bm\theta}_R$ and
$\widehat{\sigma_R^2}$ minimize $c_R(\bm\theta,\bm\sigma|\bm y)$.

The profiled REML criterion, a function of $\bm\theta$ only, is
\begin{equation}
  \label{eq:24}
  \tilde{c}_R(\bm\theta|\bm y)=
  \log(|\bm L_{\bm Z}(\bm\theta)|^2|\bm L_{\bm X}(\bm\theta)|^2)+(n-p)
  \left(1+\log\left(\frac{2\pi\tilde{d}(\bm\theta|\bm y)}{n-p}\right)\right)
\end{equation}
and the REML estimate of $\bm\theta$ is
\begin{equation}
  \label{eq:31}
  \widehat{\bm\theta}_R =
  \arg\min_{\bm\theta}\tilde{c}_R(\bm\theta,\bm y) .
\end{equation}
The REML estimate of $\sigma^2$ is
$\widehat{\sigma^2_R}=\tilde{d}(\widehat{\bm\theta}_R|\bm y)/(n-p)$.

It is not entirely clear how one would define a ``REML estimate'' of
$\bm\beta$ because the REML criterion, $c_R(\bm\theta,\bm\sigma|\bm
y)$, defined in (\ref{eq:23}), does not depend on $\bm\beta$.
However, it is customary (and not unreasonable) to use
$\widehat{\bm\beta}_R=\tilde{\bm\beta}(\widehat{\bm\theta}_R)$ as the REML
estimate of $\bm\beta$.

Note that the profiled REML criterion can be evaluated from a sparse
Cholesky decomposition like that in (\ref{eq:16}) but without the
requirement that the permutation can be applied to the columns of $\bm
Z\bm\Lambda(\bm\theta)$ separately from the columns of $\bm X$.  That
is, we can use a general fill-reducing permutation rather than the
specific form (\ref{eq:15}) with separate permutations represented by
$\bm P_{\bm Z}$ and $\bm P_{\bm X}$. This can be useful in cases where
both $\bm Z$ and $\bm X$ are large and sparse.

\subsection{Summary for linear mixed models}
\label{sec:lmmsummary}

A linear mixed model is characterized by the conditional distribution
\begin{equation}
  \label{eq:lmmcond}
  (\bc Y|\bc U=\bm u)\sim\mathcal{N}(\bm\gamma(\bm
  u,\bm\theta,\bm\beta),\sigma^2\bm I_n)\text{ where }
  \bm\gamma(\bm u,\bm\theta,\bm\beta)=\bm Z\bm\Lambda(\bm\theta)\bm
  u+\bm X\bm\beta
\end{equation}
and the unconditional distribution
$\bc U\sim\mathcal{N}(\bm 0,\sigma^2\bm I_q)$.  The discrepancy
function,
\begin{displaymath}
  d(\bm u|\bm y,\bm\theta,\bm\beta)=
  \left\|\bm y-\bm\gamma(\bm u,\bm\theta,\bm\beta)\right\|^2+\|\bm u\|^2,
\end{displaymath}
is minimized at the conditional mode, $\tilde{\bm u}(\bm\theta)$, and
the conditional estimate, $\tilde{\bm\beta}(\bm\theta)$, which are the
solutions to the sparse, positive-definite linear system
\begin{displaymath}
  \begin{bmatrix}
    \bm\Lambda\trans(\bm\theta)\bm Z\trans\bm Z\bm\Lambda(\theta)
    +\bm I_q&\bm\Lambda\trans(\bm\theta)\bm Z\trans\bm X\\
    \bm X\trans\bm Z\bm\Lambda(\theta) &\bm X\trans\bm X
  \end{bmatrix}
  \begin{bmatrix}
    \tilde{\bm u}(\bm\theta)\\\tilde{\bm\beta}(\bm\theta)
  \end{bmatrix} =
  \begin{bmatrix}
    \bm\Lambda\trans(\bm\theta)\bm Z\trans\bm y\\
    \bm X\trans\bm y
  \end{bmatrix} .
\end{displaymath}
In the process of solving this system we create the sparse left
Cholesky factor, $L_{\bm Z}(\bm\theta)$, which is a lower triangular
sparse matrix satisfying
\begin{displaymath}
  \bm L_{\bm Z}(\bm\theta)\bm L_{\bm Z}(\bm\theta)\trans=\bm P_{\bm
    Z}\left(\bm\Lambda\trans(\bm\theta)\bm Z\trans\bm
    Z\bm\Lambda(\theta)+\bm I_q\right)\bm P_{\bm Z}\trans
\end{displaymath}
where $\bm P_{\bm Z}$ is a permutation matrix representing a
fill-reducing permutation formed from the pattern of nonzeros in $\bm
Z\bm\Lambda(\bm\theta)$ for any $\bm\theta$ not on the boundary of the
parameter region.  (The values of the nonzeros depend on $\bm\theta$
but the pattern doesn't.)

The profiled log-likelihood, $\tilde{\ell}(\bm\theta|\bm y)$, is
\begin{displaymath}
    -2\tilde{\ell}(\bm\theta|\bm y)=
  \log(|\bm L_{\bm Z}(\bm\theta)|^2)+
  n\left(1+\log\left(\frac{2\pi\tilde{d}(\bm y,\bm\theta)}{n}\right)\right)
\end{displaymath}
where $\tilde{d}(\bm y,\bm\theta)=d(\tilde{\bm u}(\bm\theta)|\bm
y,\tilde{\bm\beta}(\bm\theta),\bm\theta)$.

\section{Generalizing the discrepancy function}
\label{sec:generalizations}

Because one of the factors influencing the choice of implementation
for linear mixed models is the extent to which the methods can also be
applied to other mixed models, we describe several other classes of
mixed models before discussing the implementation details for linear
mixed models.  At the core of our methods for determining the maximum
likelihood estimates (MLEs) of the parameters in the mixed model are
methods for minimizing the discrepancy function with respect to the
coefficients $\bm u$ and $\bm\beta$ in the linear predictor
$\bm\gamma(\bm u,\bm\theta,\bm\beta)$.

In this section we describe the general form of the discrepancy
function that we will use and a penalized iteratively reweighted least
squares (PIRLS) algorithm for determining the conditional modes
$\tilde{\bm u}(\bm\theta)$ and $\tilde{\bm\beta}(\bm\theta)$.  We then
describe several types of mixed models and the form of the discrepancy
function for each.

\subsection{A weighted residual sum of squares}
\label{sec:weighted}

As shown in \S\ref{sec:conditional-mode-bm}, the discrepancy function
for a linear mixed model has the form of a penalized residual sum of
squares from a linear model (\ref{eq:10}).  In this section we
generalize that definition to
\begin{equation}
  \label{eq:11}
  d(\bm u|\bm y,\bm\theta,\bm\beta) =\left\|\bm W^{1/2}(\bm\mu)
    \left[\bm y-\bm\mu_{\bc Y|\bc U}(\bm u,\bm\theta,\bm\beta)\right]\right\|^2+
  \|\bm 0-\bm u\|^2 .
\end{equation}
where $\bm W$ is an $n\times n$ diagonal matrix, called the
\emph{weights matrix}, with positive diagonal elements and $\bm
W^{1/2}$ is the diagonal matrix with the square roots of the weights
on the diagonal.  The $i$th weight is inversely proportional to
the conditional variances of $(\mathcal{Y}|\bc U=\bm u)$ and may
depend on the conditional mean, $\bm\mu_{\bc Y|\bc U}$.

We allow the conditional mean to be a nonlinear function of the linear
predictor, but with certain restrictions.  We require that the
mapping from $\bm u$ to $\bm\mu_{\bc Y|\bc U=\bm u}$ be expressed as
\begin{equation}
  \label{eq:uGammaEtaMu}
  \bm u\;\rightarrow\;\bm\gamma\;\rightarrow\;\bm\eta\;\rightarrow\;\bm\mu
\end{equation}
where $\bm\gamma=\bm Z\bm\Lambda(\bm\theta)\bm u+\bm X\bm\beta$ is an
$ns$-dimensional vector ($s > 0$) while $\bm\eta$ and $\bm\mu$ are
$n$-dimensional vectors.

The map $\bm\eta\rightarrow\bm\mu$ has the property that $\mu_i$
depends only on $\eta_i$, $i=1,\dots,n$.  The map
$\bm\gamma\rightarrow\bm\eta$ has a similar property in that, if we
write $\bm\gamma$ as an $n\times s$ matrix $\bm\Gamma$ such that
\begin{equation}
  \label{eq:vecGamma}
  \bm\gamma=\vec{\bm\Gamma}
\end{equation}
(i.e.{} concatenating the columns of $\bm\Gamma$ produces $\bm\gamma$)
then $\eta_i$ depends only on the $i$th row of $\bm\Gamma$,
$i=1,\dots,n$.  Thus the Jacobian matrix
$\frac{d\bm\mu}{d\bm\eta\trans}$ is an $n\times n$ diagonal matrix and
the Jacobian matrix $\frac{d\bm\eta}{d\bm\gamma\trans}$ is the
horizontal concatenation of $s$ diagonal $n\times n$ matrices.

For historical reasons, the function that maps $\eta_i$ to $\mu_i$ is
called the \emph{inverse link} function and is written
$\mu=g^{-1}(\eta)$.  The \emph{link function}, naturally, is
$\eta=g(\mu)$.  When applied component-wise to vectors $\bm\mu$ or
$\bm\eta$ we write these as $\bm\eta=\bm g(\bm\mu)$ and $\bm\mu=\bm
g^{-1}(\bm\eta)$.

Recall that the conditional distribution, $(\mathcal{Y}_i|\bc U=\bm
u)$, is required to be independent of $(\mathcal{Y}_j|\bc U=\bm u)$
for $i,j=1,\dots,n,\,i\ne j$ and that all the component conditional
distributions must be of the same form and differ only according to
the value of the conditional mean.

Depending on the family of the conditional distributions, the allowable
values of the $\mu_i$ may be in a restricted range.  For example, if
the conditional distributions are Bernoulli then
$0\le\mu_i\le1,i=1,\dots,n$. If the conditional distributions are
Poisson then $0\le\mu_i,i=1,\dots,n$.  A characteristic of the link
function, $g$, is that it must map the restricted range to an
unrestricted range.  That is, a link function for the Bernoulli
distribution must map $[0,1]$ to $[-\infty,\infty]$ and must be invertible
within the range.

The mapping from $\bm\gamma$ to $\bm\eta$ is defined by a function
$m:\mathbb{R}^s\rightarrow\mathbb{R}$, called the
\emph{nonlinear model} function, such that
$\eta_i=m(\bm\gamma_i),i=1,\dots,n$ where $\bm\gamma_i$ is the $i$th
row of $\bm\Gamma$.  The vector-valued function is $\bm\eta=\bm m(\bm\gamma)$.

Determining the conditional modes, $\tilde{\bm u}(\bm y|\bm\theta)$,
and $\tilde{\bm\beta}(\bm y|\bm\theta)$, that jointly minimize the
discrepancy,
\begin{equation}
  \label{eq:12}
  \begin{bmatrix}
    \tilde{\bm u}(\bm y|\bm\theta)\\
    \tilde{\bm\beta}(\bm y|\bm\theta)
  \end{bmatrix}
  =\arg\min_{\bm u,\bm\beta}\left[(\bm y-\bm\mu)\trans\bm W(\bm
  y-\bm\mu)+\|\bm u\|^2\right]
\end{equation}
becomes a weighted, nonlinear least squares problem except that the
weights, $\bm W$, can depend on $\bm\mu$ and, hence, on $\bm u$ and
$\bm\beta$.

In describing an algorithm for linear mixed models we called
$\tilde{\bm\beta}(\bm\theta)$ the \emph{conditional estimate}.  That
name reflects that fact that this is the maximum likelihood estimate
of $\bm\beta$ for that particular value of $\bm\theta$.  Once we have
determined the MLE, $\widehat(\bm\theta)_L$ of $\bm\theta$, we have a
``plug-in'' estimator,
$\widehat{\bm\beta}_L=\tilde{\bm\beta}(\bm\theta)$ for $\bm\beta$.

This property does not carry over exactly to other forms of mixed
models.  The values $\tilde{\bm u}(\bm\theta)$ and
$\tilde{\bm\beta}(\bm\theta)$ are conditional modes in the sense that
they are the coefficients in $\bm\gamma$ that jointly maximize the
unscaled conditional density $h(\bm u|\bm
y,\bm\theta,\bm\beta,\sigma)$.  Here we are using the adjective
``conditional'' more in the sense of conditioning on $\bc Y=\bm y$
than in the sense of conditioning on $\bm\theta$, although these
values are determined for a fixed value of $\bm\theta$.

\subsection{The PIRLS algorithm for $\tilde{\bm u}$ and $\tilde{\bm\beta}$}
\label{sec:pirls-algor-tild}

The penalized, iteratively reweighted, least squares (PIRLS) algorithm
to determine $\tilde{\bm u}(\bm\theta)$ and
$\tilde{\bm\beta}(\bm\theta)$ is a form of the Fisher scoring
algorithm.  We fix the weights matrix, $\bm W$, and use penalized,
weighted, nonlinear least squares to minimize the penalized, weighted
residual sum of squares conditional on these weights.  Then we update
the weights to those determined by the current value of $\bm\mu$ and
iterate.

To describe this algorithm in more detail we will use parenthesized
superscripts to denote the iteration number.  Thus $\bm u^{(0)}$ and
$\bm\beta^{(0)}$ are the initial values of these parameters, while
$\bm u^{(i)}$ and $\bm\beta^{(i)}$ are the values at the $i$th
iteration.  Similarly $\bm\gamma^{(i)}=\bm Z\bm\Lambda(\bm\theta)\bm
u^{(i)}+\bm X\bm\beta^{(i)}$, $\bm\eta^{(i)}=\bm m(\bm\gamma^{(i)})$ and
$\bm\mu^{(i)}=\bm g^{-1}(\bm\eta^{(i)})$.

We use a penalized version of the Gauss-Newton algorithm
\citep[ch.~2]{bateswatts88:_nonlin} for which we define the weighted Jacobian matrices
\begin{align}
  \label{eq:Jacobian}
  \bm U^{(i)}&=\bm W^{1/2}\left.\frac{d\bm\mu}{d\bm u\trans}\right|_{\bm u=\bm
    u^{(i)},\bm\beta=\bm\beta^{(i)}}=\bm W^{1/2}
  \left.\frac{d\bm\mu}{d\bm\eta\trans}\right|_{\bm\eta^{(i)}}
  \left.\frac{d\bm\eta}{d\bm\gamma\trans}\right|_{\bm\gamma^{(i)}}
  \bm Z\bm\Lambda(\bm\theta)\\
  \bm V^{(i)}&=\bm W^{1/2}\left.\frac{d\bm\mu}{d\bm\beta\trans}\right|_{\bm u=\bm
    u^{(i)},\bm\beta=\bm\beta^{(i)}}=\bm W^{1/2}
  \left.\frac{d\bm\mu}{d\bm\eta\trans}\right|_{\bm\eta^{(i)}}
  \left.\frac{d\bm\eta}{d\bm\gamma\trans}\right|_{\bm\gamma^{(i)}}
  \bm X
\end{align}
of dimension $n\times q$ and $n\times p$, respectively.
The increments at the $i$th iteration, $\bm\delta_{\bm u}^{(i)}$ and
$\bm\delta_{\bm\beta}^{(i)}$, are the solutions to
\begin{equation}
  \label{eq:PNLSinc}
  \begin{bmatrix}
    {\bm U^{(i)}}\trans\bm U^{(i)}+\bm I_q&{\bm U^{(i)}}\trans\bm V^{(i)}\\
    {\bm V^{(i)}}\trans\bm U^{(i)}&{\bm V^{(i)}}\trans\bm V^{(i)}
  \end{bmatrix}
  \begin{bmatrix}
    \bm\delta_{\bm u}^{(i)}\\
    \bm\delta_{\bm\beta}^{(i)}
  \end{bmatrix} =
  \begin{bmatrix}
    {\bm U^{(i)}}\trans\bm W^{1/2}(\bm y-\bm\mu^{(i)})-\bm u^{(i)}\\
    {\bm V^{(i)}}\trans\bm W^{1/2}(\bm y-\bm\mu^{(i)})
  \end{bmatrix}
\end{equation}
providing the updated parameter values
\begin{equation}
  \label{eq:33}
  \begin{bmatrix}\bm u^{(i+1)}\\\bm\beta^{(i+1)}\end{bmatrix}=
  \begin{bmatrix}\bm u^{(i)}\\\bm\beta^{(i)}\end{bmatrix}+\lambda
  \begin{bmatrix}\bm\delta_{\bm u}^{(i)}\\\bm\delta_{\bm\beta}^{(i)}
  \end{bmatrix}
\end{equation}
where $\lambda>0$ is a step factor chosen to ensure that
\begin{equation}
  \label{eq:34}
  (\bm y-\bm\mu^{(i+1)})\trans\bm W(\bm
  y-\bm\mu^{(i+1)})+\|\bm u^{(i+1)}\|^2  <
  (\bm y-\bm\mu^{(i)})\trans\bm W(\bm
  y-\bm\mu^{(i)})+\|\bm u^{(i)}\|^2  .
\end{equation}

In the process of solving for the increments we form the sparse, lower
triangular, Cholesky factor, $\bm L^{(i)}$, satisfying
\begin{equation}
  \label{eq:35}
  \bm L^{(i)} {\bm L^{(i)}}\trans =
  \bm P_{\bm Z}\left({\bm U^{(i)}}\trans\bm U^{(i)}+
    \bm I_n\right)\bm P_{\bm Z}\trans .
\end{equation}
After each successful iteration, determining new values of the
coefficients, $\bm u^{(i+1)}$ and $\bm\beta^{(i+1)}$, that reduce the
penalized, weighted residual sum of squares, we update the weights
matrix to $\bm W(\bm\mu^{(i+1)})$ and the weighted Jacobians,
$\bm U^{(i+1)}$ and $\bm V^{(i+1)}$, then iterate.  Convergence is
determined according to the orthogonality convergence
criterion~\citep[ch.~2]{bateswatts88:_nonlin}, suitably adjusted for
the weights matrix and the penalty.

\subsection{Weighted linear mixed models}
\label{sec:weightedLMM}

One of the simplest generalizations of linear mixed models is a
weighted linear mixed model where $s=1$, the link function, $g$, and
the nonlinear model function, $m$, are both the identity, the weights
matrix, $\bm W$, is constant and the conditional distribution family
is Gaussian.  That is, the conditional distribution can be written
\begin{equation}
  \label{eq:weightedLMM}
    (\bc Y|\bc U=\bm u)\sim\mathcal{N}(\bm\gamma(\bm
    u,\bm\theta,\bm\beta),\sigma^2\bm W^{-1})
\end{equation}
with discrepancy function
\begin{equation}
  \label{eq:wtddisc}
  d(\bm u|\bm y,\bm\theta,\bm\beta)=\left\|\bm W^{1/2}(\bm
    y-\bm Z\bm\Lambda(\bm\theta)\bm u-\bm X\bm\beta)\right\|^2+\|\bm u\|^2 .
\end{equation}
The conditional mode, $\tilde{\bm u}(\bm\theta)$, and the conditional
estimate, $\tilde{\bm\beta}(\bm\theta)$, are the solutions to
\begin{equation}
  \begin{bmatrix}
    \bm\Lambda\trans(\bm\theta)\bm Z\trans\bm W\bm Z\bm\Lambda(\theta)
    +\bm I_q&\bm\Lambda\trans(\bm\theta)\bm Z\trans\bm W\bm X\\
    \bm X\trans\bm W\bm Z\bm\Lambda(\theta) &\bm X\trans\bm W\bm X
  \end{bmatrix}
  \begin{bmatrix}
    \tilde{\bm u}(\bm\theta)\\\tilde{\bm\beta}(\bm\theta)
  \end{bmatrix} =
  \begin{bmatrix}
    \bm\Lambda\trans(\bm\theta)\bm Z\trans\bm W\bm y\\
    \bm X\trans\bm W\bm y
  \end{bmatrix} ,
\end{equation}
which can be solved directly, and the Cholesky factor, $\bm L_{\bm Z}(\bm\theta)$, satisfies
\begin{equation}
  \bm L_{\bm Z}(\bm\theta)\bm L_{\bm Z}(\bm\theta)\trans=\bm P_{\bm
    Z}\left(\bm\Lambda\trans(\bm\theta)\bm Z\trans\bm W\bm
    Z\bm\Lambda(\theta)+\bm I_q\right)\bm P_{\bm Z}\trans .
\end{equation}

The profiled log-likelihood, $\tilde{\ell}(\bm\theta|\bm y)$, is
\begin{equation}
  \label{eq:wtdprofilelik}
  -2\tilde{\ell}(\bm\theta|\bm y)=
  \log\left(\frac{|\bm L_{\bm Z}(\bm\theta)|^2}{|\bm W|}\right)+
  n\left(1+\log\left(\frac{2\pi\tilde{d}(\bm
        y,\bm\theta)}{n}\right)\right) .
\end{equation}

If the matrix $\bm W$ is fixed then we can ignore the term $|\bm W|$
in (\ref{eq:wtdprofilelik}) when determining the MLE,
$\widehat{\bm\theta}_L$.  However, in some models, we use a
parameterized weight matrix, $\bm W(\bm\phi)$, and wish to determine
the MLEs, $\widehat{\bm\phi}_L$ and $\widehat{\bm\theta}_L$
simultaneously.  In these cases we must include the term involving
$|\bm W(\bm\phi)|$ when evaluating the profiled log-likelihood.

Note that we must define the parameterization of $\bm W(\bm\phi)$
such that $\sigma^2$ and $\bm\phi$ are not a redundant
parameterization of $\sigma^2\bm W(\bm\phi)$.  For example, we could
require that the first diagonal element of $\bm W$ be unity.

\subsection{Nonlinear mixed models}
\label{sec:NLMMs}

In an unweighted, nonlinear mixed model the conditional distribution is
Gaussian, the link, $g$, is the identity and the weights matrix, $\bm
W=\bm I_n$.  That is,
\begin{equation}
  \label{eq:conddistNLMM}
  (\bc Y|\bc U=\bm u)\sim\mathcal{N}(\bm m(\bm\gamma),\sigma^2\bm I_n)
\end{equation}
with discrepancy function
\begin{equation}
  \label{eq:discNLMM}
  d(\bm u|\bm y,\bm\theta,\bm\beta)=
  \|\bm y-\bm\mu\|^2 + \|\bm u\|^2 .
\end{equation}
For a given value of $\bm\theta$ we determine the conditional modes,
$\tilde{\bm u}(\bm\theta)$ and $\tilde{\bm\beta}(\bm\theta)$, as the
solution to the penalized nonlinear least squares problem
\begin{equation}
  \label{eq:NLMMpnls}
  \begin{bmatrix}
    \tilde{\bm u}(\bm\theta)\\
    \tilde{\bm\beta}(\bm\theta)
  \end{bmatrix} = \arg\min_{\bm u,\bm\theta}d(\bm u|\bm y,\bm\theta,\bm\beta)
\end{equation}
and we write the minimum discrepancy, given $\bm y$ and $\bm\theta$,
as
\begin{equation}
  \label{eq:25}
  \tilde{d}(\bm y,\bm\theta)=d(\tilde{\bm u}(\bm\theta)|\bm
  y,\bm\theta,\tilde{\bm\beta}(\bm\theta)).
\end{equation}

Let $\tilde{\bm L}_Z(\bm\theta)$ and $\tilde{\bm L}_X(\bm\theta)$ be
the Cholesky factors at $\bm\theta$, $\tilde{\bm\beta}(\bm\theta)$ and
$\tilde{\bm u}(\bm\theta)$.  Then the \emph{Laplace approximation} to
the log-likelihood is
\begin{equation}
  \label{eq:36}
  -2\ell_P(\bm\theta,\bm\beta,\sigma|\bm y)\approx
  n\log(2\pi\sigma^2)+\log(|\tilde{\bm L}_{\bm Z}|^2)+
  \frac{\tilde{d}(\bm y,\bm\theta) +
    \left\|\tilde{\bm L}_{\bm X}\trans(\bm\beta-\tilde{\bm\beta})\right\|^2}{\sigma^2},
\end{equation}
producing the approximate profiled log-likelihood,
$\tilde{\ell}_P(\bm\theta|\bm y)$,
\begin{equation}
  \label{eq:37}
  -2\tilde{\ell}_P(\bm\theta|\bm y)\approx
  \log(|\tilde{\bm L}_{\bm Z}|^2)+n\left(1+\log(2\pi
    \tilde{d}(\bm y,\bm\theta)/n) \right).
\end{equation}

\subsubsection{Nonlinear mixed model summary}
\label{sec:nonl-mixed-model}

In a nonlinear mixed model we determine the parameter estimate,
$\widehat{\bm\theta}_P$, from the Laplace approximation to the
log-likelihood as
\begin{equation}
  \label{eq:38}
  \widehat{\bm\theta}_P =
  \arg\max_{\bm\theta}\tilde{\ell}_P(\bm\theta|\bm y)
  =\arg\min_{\bm\theta}
  \log(|\tilde{\bm L}_{\bm Z}|^2)+
  n\left(1+\log(2\pi
    \tilde{d}(\bm y,\bm\theta)/n) \right).
\end{equation}
Each evaluation of $\tilde{\ell}_P(\bm\theta|\bm y)$ requires a
solving the penalized nonlinear least squares problem
(\ref{eq:NLMMpnls}) simultaneously with respect to both sets of
coefficients, $\bm u$ and $\bm\beta$, in the linear predictor,
$\bm\gamma$.

For a weighted nonlinear mixed model with fixed weights, $\bm W$, we
replace the unweighted discrepancy function $d(\bm u|\bm
y,\bm\theta,\bm\beta)$ with the weighted discrepancy function,
%% Finish this off

\section{Details of the implementation}
\label{sec:details}

\subsection{Implementation details for linear mixed models}
\label{sec:impl-line-mixed}

The crucial step in implementing algorithms for determining ML or REML
estimates of the parameters in a linear mixed model is evaluating the
factorization (\ref{eq:16}) for any $\bm\theta$ satisfying
$\bm\theta_L\le\bm\theta\le\bm\theta_U$.  We will assume that $\bm Z$
is sparse as is $\bm Z\bm\Lambda(\bm\theta)$.

When $\bm X$ is not sparse we will use the factorization (\ref{eq:16})
setting $\bm P_{\bm X}=\bm I_p$ and storing $\bm L_{\bm X\bm Z}$ and
$\bm L_{\bm X}$ as dense matrices.  The permutation matrix $\bm P_{\bm
  Z}$ is determined from the pattern of non-zeros in $\bm
Z\bm\Lambda(\bm\theta)$ which is does not depend on $\bm\theta$, as
long as $\bm\theta$ is not on the boundary.  In fact, in most cases
the pattern of non-zeros in $\bm Z\bm\Lambda(\bm\theta)$ is the same
as the pattern of non-zeros in $\bm Z$. For many models, in particular
models with scalar random effects (described later), the matrix
$\bm\Lambda(\bm\theta)$ is diagonal.

Given a value of $\bm\theta$ we determine the Cholesky factor $\bm
L_{\bm Z}$ satisfying
\begin{equation}
  \label{eq:LZ}
  \bm L_{\bm Z}\bm L_{\bm Z}\trans=\bm P_{\bm Z}(
  \bm\Lambda\trans(\bm\theta)\bm Z\trans\bm Z\bm\Lambda(\theta)
    +\bm I_q)\bm P_{\bm Z}\trans .
\end{equation}
The CHOLMOD package allows for $\bm L_{\bm Z}$ to be calculated
directly from $\bm\Lambda\trans(\bm\theta)\bm Z\trans$ or from
$\bm\Lambda\trans(\bm\theta)\bm Z\trans\bm Z\bm\Lambda(\theta)$.  The
choice in implementation is whether to store $\bm Z\trans$ and update
it to $\bm\Lambda\trans(\bm\theta)\bm Z\trans$ or to store $\bm Z\trans\bm
Z$ and use it to form $\bm\Lambda\trans(\bm\theta)\bm Z\trans\bm
Z\bm\Lambda(\theta)$ at each evaluation.

In the \package{lme4} package we store $\bm Z\trans$ and use it to
form $\bm\Lambda\trans(\bm\theta)\bm Z\trans$ from which $\bm L_{\bm
  Z}$ is evaluated.  There are two reasons for this choice.  First,
the calculations for the more general forms of mixed models cannot be
reduced to calculations involving $\bm Z\trans\bm Z $ and by expressing
these calculations in terms of $\bm\Lambda(\bm\theta)\trans \bm Z\trans$ for
linear mixed models we can reuse the code for the more general
models.  Second, the calculation of $\bm\Lambda(\bm\theta)\trans\left(\bm
  Z\trans\bm Z\right)\bm\Lambda(\bm\theta)$ from $\bm Z\trans\bm Z$ is
complicated compared to the calculation of
$\bm\Lambda(\bm\theta)\trans\bm Z\trans$ from $\bm Z\trans$.

This choice is disadvantageous when $n\gg q$ because $\bm Z\trans$ is
much larger than $\bm Z\trans\bm Z$, even when they are stored as
sparse matrices.  Evaluation of $\bm L_{\bm Z}$ directly from $\bm
Z\trans$ requires more storage and more calculation that evaluating
$\bm L_{\bm Z}$ from $\bm Z\trans\bm Z$.

Next we evaluate $\bm L_{\bm X\bm Z}\trans$ as the solution to
\begin{equation}
  \label{eq:LXZ}
  \bm L_{\bm Z}\bm L_{\bm X\bm Z}\trans=\bm P_{\bm
    Z}\bm\Lambda\trans(\bm\theta)\bm Z\trans\bm X .
\end{equation}
Again we have the choice of calculating and storing $\bm Z\trans\bm X$
or storing $\bm X$ and using it to reevaluate $\bm Z\trans\bm X$.  In
the \package{lme4} package we store $\bm X$, because the calculations
for the more general models cannot be expressed in terms of $\bm Z\trans\bm X$.

Finally $\bm L_{\bm X}$ is evaluated as the (dense) solution to
\begin{equation}
  \label{eq:LX}
  \bm L_{\bm X}\bm L_{\bm X}\trans=
  \bm X\trans\bm X-\bm L_{\bm X\bm Z}\bm L_{\bm X\bm Z}\trans .
\end{equation}
from which $\tilde{\bm\beta}$ can be determined as the
solution to dense system
\begin{equation}
  \label{eq:tildebeta}
  \bm L_{\bm X}\bm L_{\bm X}\trans \tilde{\bm\beta}=\bm X\trans\bm y
\end{equation}
and $\tilde{\bm u}$ as the solution to the sparse system
\begin{equation}
  \label{eq:tildeu}
  \bm L_{\bm Z}\bm L_{\bm Z}\trans \tilde{u}=\bm\Lambda\trans\bm Z\trans\bm y
\end{equation}

For many models, in particular models with scalar random effects,
which are described later, the matrix $\bm\Lambda(\bm\theta)$ is
diagonal.  For such a model, if both $\bm Z$ and $\bm X$ are sparse
and we plan to use the REML criterion then we create and store
\begin{equation}
  \label{eq:8}
  \bm A=
  \begin{bmatrix}
    \bm Z\trans\bm Z & \bm Z\trans\bm X\\
    \bm X\trans\bm Z & \bm X\trans\bm X
  \end{bmatrix}\quad\text{and}\quad
  \bm c =\begin{bmatrix}\bm Z\trans\bm y\\\bm X\trans\bm y\end{bmatrix}
\end{equation}
and determine a fill-reducing permutation, $\bm P$, for $\bm A$.
Given a value of $\bm\theta$ we create the factorization
\begin{equation}
  \label{eq:26}
  \bm L(\bm\theta)\bm L(\bm\theta)\trans=\bm P\left(
    \begin{bmatrix}
      \bm\Lambda(\bm\theta) & \bm0\\\bm0&\bm I_p
    \end{bmatrix} \bm A
    \begin{bmatrix}
      \bm\Lambda(\bm\theta) & \bm0\\\bm0&\bm I_p
    \end{bmatrix}+
    \begin{bmatrix}\bm I_q&\bm0\\\bm0&\bm0\end{bmatrix}\right)
  \bm P\trans
\end{equation}
solve for $\tilde{\bm u}(\bm\theta)$ and
$\tilde{\bm\beta}(\bm\theta)$ in
\begin{equation}
  \label{eq:28}
  \bm L\bm L\trans\bm P
  \begin{bmatrix}
    \tilde{\bm u}(\bm\theta)\\\tilde{\bm\beta}(\bm\theta)
  \end{bmatrix}=
  \bm P
  \begin{bmatrix}\bm\Lambda(\bm\theta) & \bm0\\\bm0&\bm I_p
  \end{bmatrix} \bm c
\end{equation}
then evaluate $\tilde{d}(\bm y|\bm\theta)$
and the profiled REML criterion as
\begin{equation}
  \label{eq:27}
  \tilde{d}_R(\bm\theta|\bm y)=\log(|\bm L(\bm\theta)|^2)+
  (n-p)\left(1+\log\left(\frac{2\pi\tilde{d}(\bm y|\bm\theta)}
      {n-p}\right)\right) .
\end{equation}

\bibliography{lme4}
\appendix{}

\section{Notation}
\label{sec:notation}

\subsection{Random variables in the model}
\label{sec:rand-vari-model}
\begin{description}
\item[$\bc B$] Random-effects vector of dimension $q$,
  $\bc{B}\sim\mathcal{N}(\bm 0,\sigma^2\bm \Lambda(\bm\theta)\bm
  \Lambda(\bm\theta)\trans)$.
\item[$\bm U$] ``Spherical'' random-effects vector of dimension $q$,
  $\bc U\sim\mathcal{N}(\bm 0,\sigma^2\bm I_q)$, $\bc B=\bm
  \Lambda(\bm\theta)\bc U$.
\item[$\bc Y$] Response vector of dimension $n$.
\end{description}

\subsection{Parameters of the model}
\label{sec:parameters-model}
\begin{description}
\item[$\bm\beta$] Fixed-effects parameters (dimension $p$).
\item[$\bm\theta$] Parameters determining the left factor,
  $\bm\Lambda(\bm\theta)$ of the relative covariance matrix of $\bc B$
  (dimension $m$).
\item[$\sigma$] the common scale parameter - not used in some
  generalized linear mixed models and generalized nonlinear mixed
  models.
\end{description}

\subsection{Dimensions}
\label{sec:dimensions}
\begin{description}
\item[$m$] dimension of the parameter $\bm\theta$.
\item[$n$] dimension of the response vector, $\bm y$, and the random
  variable, $\bm{\mathcal{Y}}$.
\item[$p$] dimension of the fixed-effects parameter, $\bm\beta$.
\item[$q$] dimension of the random effects, $\bc B$ or $\bc U$.
\item[$s$] dimension of the parameter vector, $\bm\phi$, in the
  nonlinear model function.
\end{description}

\subsection{Matrices}
\label{sec:matrices}
\begin{description}
\item[$\bm L$] Left Cholesky factor of a positive-definite symmetric
  matrix.  $\bm L_{\bm Z}$ is $q\times q$; $\bm L_{\bm X}$ is $p\times p$.
\item[$\bm P$] Fill-reducing permutation for the random effects model
  matrix. (Size $q\times q$.)
\item[$\bm \Lambda$] Left factor of the relative covariance matrix of the
  random effects. (Size $q\times q$.)
\item[$\bm X$] Model matrix for the fixed-effects parameters,
  $\bm\beta$. (Size $(ns)\times p$.)
\item[$\bm Z$] Model matrix for the random effects. (Size $(ns)\times q$.)
\end{description}

\section{Integrating a quadratic deviance expression}
\label{sec:integr-quadr-devi}

In (\ref{eq:6}) we defined the likelihood of the parameters given the
observed data as
\begin{displaymath}
  L(\bm\theta,\bm\beta,\sigma|\bm y) =
  \int_{\mathbb{R}^q}h(\bm u|\bm y,\bm\theta,\bm\beta,\sigma)\,d\bm u .
\end{displaymath}
which is often alarmingly described as ``an intractable integral''.
In point of fact, this integral can be evaluated exactly in the case
of a linear mixed model and can be approximated quite accurately for
other forms of mixed models.

\end{document}

---
title: "lme4 performance tips"
---

<!--
%\VignetteEngine{knitr::rmarkdown}
%\VignetteIndexEntry{lme4 performance tips}
-->

```{r opts, echo = FALSE, message = FALSE}
library("knitr")
knitr::opts_chunk$set(
  )
(load(system.file("testdata", "lmerperf.rda", package="lme4")))# 'ss' 'fitlist'
```

```{r loadpkg,message=FALSE}
library("lme4")
```

## overview

In general `lme4`'s algorithms scale reasonably well with the number of observations
and the number of random effect levels. The biggest bottleneck is in the number of
*top-level parameters*, i.e. covariance parameters for
`lmer` fits or `glmer` fits with `nAGQ`=0 [`length(getME(model, "theta"))`], 
covariance and fixed-effect parameters for `glmer` fits with `nAGQ`>0. `lme4` does a derivative-free
(by default) nonlinear optimization step over the top-level parameters.

For this reason, "maximal" models involving interactions of factors with several levels each
(e.g. `(stimulus*primer | subject)`) will be slow (as well as hard to estimate): if the two factors
have `f1` and `f2` levels respectively, then the corresponding `lmer` fit will need to estimate
`(f1*f2)*(f1*f2+1)/2` top-level parameters.

`lme4` automatically constructs the random effects model matrix ($Z$) as a sparse matrix.
At present it does *not* allow an option for a sparse fixed-effects model matrix ($X$), which
is useful if the fixed-effect model includes factors with many levels. Treating such factors
as random effects instead, and using the modular framework (`?modular`) to fix the variance
of this random effect at a large value, will allow it to be modeled using a sparse matrix.
(The estimates will converge to the fixed-effect case in the limit as the variance goes to infinity.)

## setting `calc.derivs = FALSE`

After finding the best-fit model parameters (in most cases using *derivative-free*
algorithms such as Powell's BOBYQA or Nelder-Mead, `[g]lmer` does a series of finite-difference
calculations to estimate the gradient and Hessian at the MLE. These are used to
try to establish whether the model has converged reliably, and (for `glmer`) to
estimate the standard deviations of the fixed effect parameters (a less accurate
approximation is used if the Hessian estimate is not available. As currently implemented, this computation takes `2*n^2 -  n + 1` additional evaluations of the deviance,
where `n` is the total number of top-level parameters. Using  `control = [g]lmerControl(calc.derivs = FALSE)` to turn off this calculation can speed up 
the fit, e.g.

```{r noderivs, eval = FALSE}
m0 <- lmer(y ~ service * dept + (1|s) + (1|d), InstEval,
     control = lmerControl(calc.derivs = FALSE))
```

```{r calcs, echo = FALSE}
## based on loaded lmerperf file
t1 <- fitlist$basic$times[["elapsed"]]
t2 <- fitlist$noderivs$times[["elapsed"]]
pct <- round(100*(t1-t2)/t1)
e1 <- fitlist$basic$optinfo$feval
```

Benchmark results for this run with and without derivatives show an
approximately `r pct`% speedup (from `r round(t1)` to `r round(t2)` seconds on a Linux machine with
AMD Ryzen 9 2.2 GHz processors). This is a case with only 2 top-level
parameters, but the fit took only `r e1` deviance function evaluations
(see `m0@optinfo$feval`) to converge, so the effect of the additional 7 ($n^2 -n +1$)
function evaluations is noticeable.

## choice of optimizer

```{r glmeropt, echo=FALSE}
gg <- glmerControl()$optimizer
```

`lmer` uses the "`r lmerControl()$optimizer`" optimizer by default;
`glmer` uses a combination of `r gg[1]` (`nAGQ=0` stage)  and `r gg[2]`. 
These are reasonably good choices, although switching `glmer` fits to `nloptwrap` for 
both stages may be worth a try.

`allFits()` gives an easy way to check the timings of a large range of optimizers:

```{r times, as.is=TRUE, echo=FALSE}
tt <- sort(ss$times[,"elapsed"])
tt2 <- data.frame(optimizer = names(tt), elapsed = tt)
rownames(tt2) <- NULL
knitr::kable(tt2)
```

As expected, bobyqa - both the implementation in the `minqa` package [`[g]lmerControl(optimizer="bobyqa")`]
and the  one in `nloptwrap` [`optimizer="nloptwrap"` or `optimizer="nloptwrap", optCtrl = list(algorithm = "NLOPT_LN_BOBYQA"`] - are fastest.

## changing optimizer tolerances

Occasionally, the default optimizer stopping tolerances are unnecessarily strict. 
These tolerances are specific to each optimizer, and can be set via the `optCtrl` argument in `[g]lmerControl`.
To see the defaults for `nloptwrap`:

```{r default}
environment(nloptwrap)$defaultControl
```

```{r calcs2, echo = FALSE}
## based on loaded lmerperf file
t1 <- fitlist$basic$times[["elapsed"]]
t2 <- fitlist$noderivs$times[["elapsed"]]
t3 <- fitlist$nlopt_sloppy$times[["elapsed"]]
pct <- round(100*(t1-t2)/t1)
e1 <- fitlist$basic$optinfo$feval
```

In the particular case of the `InstEval` example, this doesn't help much - loosening the tolerances
to `ftol_abs=1e-4`, `xtol_abs=1e-4` only saves 2 functional evaluations and a few seconds, while loosening
the tolerances still further gives convergence warnings.

## parallelization/BLAS

There are not many options for parallelizing `lme4`. Optimized BLAS does not seem to help much.

## other packages

- `glmmTMB` may be faster than `lme4` for GLMMs with large numbers of top-level parameters, especially for negative binomial models (i.e. compared to `glmer.nb`)
- the `MixedModels.jl` package in Julia may be *much* faster for some problems. You do need to install Julia.
   - see [this short tutorial](https://github.com/ginettelafit/MixedModelswithRandJulia) or [this example](https://github.com/RePsychLing/MixedModels-lme4-bridge/blob/master/using_jellyme4.ipynb) (Jupyter notebook)
   - the [JellyMe4](https://github.com/palday/JellyMe4.jl) and [jglmm](https://github.com/mikabr/jglmm) packages provide R interfaces

---
title: "Covariance Structures comparison (lme4/glmmTMB)"
output: html_document
---

## Comparisons with glmmTMB

### Reason for Computational Differences

When fitting linear mixed models, `lme4`  parameterizes the random-effects variance–covariance matrix on an unconstrained scale, using box-constrained optimization algorithms to ensure that the variance-covariance matrix is positive semidefinite. For unstructured covariance matrices, this means that the elements of $\mathbf \theta$ that parameterize the diagonal elements of $\mathbf \Lambda$ are constrained to be $\ge 0$ (for diagonal models, all of the elements of $\mathbf \theta$ fill the diagonal of $\mathbf \Lambda$ and hence are $\ge 0$; for models such as the compound-symmetric or AR1 models that use correlation parameters, we constrain $|\rho| \le 1$. As discussed in @Bates_JSS, this constrained parameterization works well for handling model where the 
estimated covariance matrix is *singular* (i.e. $\mathbf \Sigma$ is only positive semidefinite, not positive definite). In addition, for linear mixed models `lme4` profiles the fixed-effect parameters out of the objective function [@Bates_JSS]; finally, the scale parameter $\sigma$ is not estimated directly, but is derived from the residual variance or deviance of the fitted model.

In contrast, `glmmTMB` uses direct maximum likelihood estimation via Template Model Builder (TMB), fitting to the full parameter vector $\{\mathbf \theta, \mathbf \beta, \sigma^2\}$. Covariance parameters are fitted on a transformed (unconstrained) scale: log scale for standard deviations and various scales for correlation parameters (see the `glmmTMB` [covariance structures vignette](https://cran.r-project.org/package=glmmTMB/vignettes/covstruct.html) for details). This parameterization simplifies fitting (a box-constrained algorithm isn't necessary), but is less convenient in singular fits and other cases where the maximum likelihood estimate is infinite on the unconstrained scale.

Despite these differences, we will show examples where `lme4` and `glmmTMB` provide similar estimates when they both use maximum likelihood estimation. By default, `lme4` uses the restricted maximum likelihood; hence in the following examples, we use `lmer(..., REML = FALSE)` to compare against `glmmTMB`.

### Comparison Setup

```{r comparison-setup}
if (!requireNamespace("glmmTMB", quietly = TRUE)) {
  knitr::opts_chunk$set(eval = FALSE)
} else {
  library(glmmTMB)
  library(lme4)
}

## Often want to ignore attributes and class.
## Set a fairly large tolerance for convenience.
all.equal.nocheck <- function(x, y, tolerance = 3e-5, ..., check.attributes = FALSE, check.class = FALSE) {
  require("Matrix", quietly = TRUE)
  ## working around mode-matching headaches
  if (is(x, "Matrix")) x <- matrix(x)
  if (is(y, "Matrix")) y <- matrix(y)
  all.equal(x, y, ..., tolerance = tolerance, check.attributes = check.attributes, check.class = check.class)
}

get.cor1 <- function(x) {
  v <- VarCorr(x)
  vv <- if (inherits(x, "merMod")) v$group else v$cond$group
  attr(vv, "correlation")[1,2]
}
```

### Unstructured (General Positive Definite)

This is the default setting for both `lme4` and `glmmTMB`. Below we simulate a dataset with known `beta`, `theta` and `sigma` values.

```{r unstruc-sim}
n_groups <- 20
n_per_group <- 20
n <- n_groups * n_per_group

set.seed(1)
dat.us <- data.frame(
  group = rep(1:n_groups, each = n_per_group),
  x1 = rnorm(n),
  x2 = rnorm(n)
)
## Constructing a similar dataset for the other examples
gdat.us <- dat.diag <- gdat.diag <- dat.us

form <- y ~ 1 + x1 * x2 + us(1 + x1|group)
dat.us$y <- simulate(form[-2], 
                    newdata = dat.us,
                    family = gaussian,
                    newparams = list(beta = c(-7, 5, -100, 20),
                                     theta = c(2.5, 1.4, 6.3),
                                     sigma = 2))[[1]]

form2 <- y ~ 1 + x1 + us(1 + x1|group)
gdat.us$y <- simulate(
  form2[-2],
  newdata = gdat.us,
  family = binomial,
  newparams = list(
    beta  = c(-1.5, 0.5),     
    theta = c(0.3, 0.1, 0.3)
  ))[[1]]
```


### Linear Mixed Model

```{r compare-lmm}
lme4.us <- lmer(form, data = dat.us, REML = "FALSE")
glmmTMB.us <- glmmTMB(form, dat = dat.us)

## Fixed effects
fixef(lme4.us); fixef(glmmTMB.us)$cond
all.equal.nocheck(fixef(lme4.us), fixef(glmmTMB.us)$cond)

## Sigma
sigma(lme4.us); sigma(glmmTMB.us)
all.equal.nocheck(sigma(lme4.us), sigma(glmmTMB.us))

## Log likelihoods
logLik(lme4.us); logLik(glmmTMB.us)
all.equal.nocheck(logLik(lme4.us), logLik(glmmTMB.us))
```

As expected, calculations related to the random-effects term differ slightly beyond this point.

```{r compare-cov}
## Variance-Covariance Matrix
vcov(lme4.us); vcov(glmmTMB.us)$cond
all.equal.nocheck(vcov(lme4.us), vcov(glmmTMB.us)$cond)

## Variance and Covariance Components
all.equal.nocheck(VarCorr(lme4.us)$group,
          VarCorr(glmmTMB.us)$cond$group)

## Conditional Modes of the Random Effects
all.equal.nocheck(ranef(lme4.us)$group, ranef(glmmTMB.us)$cond$group)
```

### Generalized Linear Mixed Model

```{r compare-glmm}
glme4.us <- glmer(form2, data = gdat.us, family = binomial)
gglmmTMB.us <- glmmTMB(form2, dat = gdat.us, family = binomial)

## Fixed effects
fixef(glme4.us); fixef(gglmmTMB.us)$cond
all.equal.nocheck(fixef(glme4.us), fixef(gglmmTMB.us)$cond)

## Sigma
all.equal.nocheck(sigma(glme4.us), sigma(gglmmTMB.us))

## Log likelihoods
logLik(glme4.us); logLik(gglmmTMB.us)
all.equal.nocheck(logLik(glme4.us), logLik(gglmmTMB.us))
```

As expected, calculations related to the random-effects term differ slightly beyond this point.

```{r compare-glmm-cov}
## Variance-Covariance Matrix
vcov(glme4.us); vcov(gglmmTMB.us)$cond
all.equal.nocheck(vcov(glme4.us), vcov(gglmmTMB.us)$cond)

## Variance and Covariance Components
all.equal.nocheck(VarCorr(glme4.us)$group,
          VarCorr(gglmmTMB.us)$cond$group)

## Conditional Modes of the Random Effects
all.equal.nocheck(ranef(glme4.us)$group, ranef(gglmmTMB.us)$cond$group)
```

### Diagonal

The syntax is the same for fitting a heterogeneous diagonal covariance structure for `lme4` and `glmmTMB`. It changes when we want to fit a *homogeneous* diagonal covariance structure.

To fit a homogeneous diagonal covariance structure we would write:

```{r, fit-homog-diag, eval = FALSE}
lme4.us <- lmer(Reaction ~ Days + diag(Days | Subject, hom = TRUE), sleepstudy)
glmmTMB.us <- glmmTMB(Reaction ~ Days + homdiag(Days | Subject), sleepstudy)
```

We will focus on comparisons of an estimated *heterogeneous* diagonal covariance structure.

```{r sim-het-diag}
form <- y ~ 1 + x1 * x2 + diag(1|group)
dat.diag$y <- simulate(form[-2], 
                       newdata = dat.diag,
                       family = gaussian,
                       newparams = list(beta = c(-7, 5, -100, 20),
                                        theta = c(2.5),
                                        sigma = 2))[[1]]
```


```{r fit-het-diag}
lme4.diag <- lmer(form, data = dat.diag, REML = "FALSE")
glmmTMB.diag <- glmmTMB(form, dat = dat.diag)

## Fixed effects
fixef(lme4.diag); fixef(glmmTMB.diag)$cond
all.equal.nocheck(fixef(lme4.diag), fixef(glmmTMB.diag)$cond)

## Sigma
sigma(lme4.diag); sigma(glmmTMB.diag)
all.equal.nocheck(sigma(lme4.diag), sigma(glmmTMB.diag))

## Log likelihoods
logLik(lme4.diag); logLik(glmmTMB.diag)
all.equal.nocheck(logLik(lme4.diag), logLik(glmmTMB.diag))

## Variance-Covariance Matrix
vcov(lme4.diag); vcov(glmmTMB.diag)$cond
all.equal.nocheck(vcov(lme4.diag), vcov(glmmTMB.diag)$cond)

## Variance and Covariance Components
all.equal.nocheck(VarCorr(lme4.diag)[[1]], 
          VarCorr(glmmTMB.diag)$cond$group)

## Conditional Modes of the Random Effects
all.equal.nocheck(ranef(lme4.diag)$group, ranef(glmmTMB.diag)$cond$group)
```

### Compound Symmetry

Similar to the diagonal case, the syntax is the same for fitting a heterogeneous compound symmetry covariance structure for `lme4` and `glmmTMB`: 

```{r cs-fit-2, eval = FALSE}
lme4.us <- lmer(Reaction ~ Days + cs(Days | Subject, hom = TRUE), sleepstudy)
glmmTMB.us <- glmmTMB(Reaction ~ Days + cs(Days | Subject), sleepstudy)
```

Again, it differs when we want to fit a *homogeneous* compound symmetry covariance structure, which we will use for our comparisons.

```{r simgroup}
simGroup <- function(g, n=6, phi=0.6) {
  x <- MASS::mvrnorm(mu = rep(0,n),
                     Sigma = phi^as.matrix(dist(1:n)) )  
  y <- x + rnorm(n)                              
  times <- factor(1:n)
  group <- factor(rep(g,n))
  data.frame(y, times, group)
}

set.seed(1)
dat.cs <- do.call("rbind", lapply(1:2000, simGroup))
```

<!-- this is the slow one ... cache? -->
```{r more-comp, cache=TRUE}
lme4.cs <- lmer(y ~ times + cs(0 + times | group, hom = TRUE), data = dat.cs, REML = FALSE)
glmmTMB.cs <- glmmTMB(y ~ times + homcs(0 + times | group), data = dat.cs)

## Fixed effects
fixef(lme4.cs); fixef(glmmTMB.cs)$cond
all.equal.nocheck(fixef(lme4.cs), fixef(glmmTMB.cs)$cond)

## Sigma
sigma(lme4.cs); sigma(glmmTMB.cs)
all.equal.nocheck(sigma(lme4.cs), sigma(glmmTMB.cs))

## Log likelihoods
logLik(lme4.cs); logLik(glmmTMB.cs)
all.equal.nocheck(logLik(lme4.cs), logLik(glmmTMB.cs))

## Variance-Covariance Matrix
all.equal.nocheck(vcov(lme4.cs), vcov(glmmTMB.cs)$cond)

## Variance and Covariance Components
all.equal.nocheck(VarCorr(lme4.cs)[[1]], 
          VarCorr(glmmTMB.cs)$cond$group)

## Conditional Modes of the Random Effects
all.equal.nocheck(ranef(lme4.cs)$group, ranef(glmmTMB.cs)$cond$group)

## Comparing against the predicted rho value
lme4.rho <- get.cor1(lme4.cs)
glmmTMB.rho <- get.cor1(glmmTMB.cs)
lme4.rho; glmmTMB.rho
all.equal.nocheck(lme4.rho, glmmTMB.rho)
```

### Autoregressive Order 1

For this comparison, we focus on a simulated data set with $\rho = 0.7$.

```{r ar1-sim}
set.seed(1)
dat.ar1 <- do.call("rbind", lapply(1:2000, function(g) simGroup(g, phi = 0.7)))
```

Unlike the diagonal and compound symmetry case, the syntax differs for fitting either a heterogeneous or a homogeneous AR1 model for `lme4` and `glmmTMB`.

For a *heterogeneous* AR1 covariance structure we would write the following:

```{r het-ar1-fit, eval = FALSE}
lme4.ar1 <- lmer(y ~ times + ar1(0 + times | group, hom = FALSE),
                 data = dat.ar1, REML = FALSE)
glmmTMB.ar1 <- glmmTMB(y ~ times + hetar1(0 + times | group), data = dat.ar1)
```

We will instead focus on comparisons for a *homogeneous* AR1 covariance structure.

```{r ar1-fit, cache=TRUE}
lme4.ar1 <- lmer(y ~ times + ar1(0 + times | group, hom = TRUE), data = dat.ar1, REML = FALSE)
glmmTMB.ar1 <- glmmTMB(y ~ times + ar1(0 + times | group), data = dat.ar1)

## Fixed effects
fixef(lme4.ar1); fixef(glmmTMB.ar1)$cond
all.equal.nocheck(fixef(lme4.ar1), fixef(glmmTMB.ar1)$cond)

## Sigma
sigma(lme4.ar1); sigma(glmmTMB.ar1)
all.equal.nocheck(sigma(lme4.ar1), sigma(glmmTMB.ar1))

## Log likelihoods
logLik(lme4.ar1); logLik(glmmTMB.ar1)
all.equal.nocheck(logLik(lme4.ar1), logLik(glmmTMB.ar1))

## Variance-Covariance Matrix
all.equal.nocheck(vcov(lme4.ar1), vcov(glmmTMB.ar1)$cond)

## Variance and Covariance Components
all.equal.nocheck(VarCorr(lme4.ar1)$group, 
                  VarCorr(glmmTMB.ar1)$cond$group)

## Conditional Modes of the Random Effects
all.equal.nocheck(ranef(lme4.ar1)$group, ranef(glmmTMB.ar1)$cond$group)

## Comparing against the predicted rho value
lme4.ar1.rho <- get.cor1(lme4.ar1)
glmmTMB.ar1.rho <- get.cor1(glmmTMB.ar1)
lme4.ar1.rho; glmmTMB.ar1.rho
all.equal.nocheck(lme4.ar1.rho, glmmTMB.ar1.rho)
```

%\VignetteEngine{knitr::knitr}
%\VignetteDepends{ggplot2}
%\VignetteDepends{gamm4}
%\VignetteIndexEntry{Fitting Linear Mixed-Effects Models using lme4}
\documentclass[nojss]{jss}
\usepackage[T1]{fontenc}% for correct hyphenation and T1 encoding
\usepackage[utf8]{inputenc}%
\usepackage{lmodern}% latin modern font
\usepackage[american]{babel}  %% for texi2dvi ~ bug
\usepackage{bm,amsmath,thumbpdf,amsfonts}%,minted}
\usepackage{blkarray}
\usepackage{array}
%% huxtable-ish stuff
%% \usepackage{adjustbox}
%% \usepackage{threeparttable}
%%
\newcolumntype{P}[1]{>{\raggedright\arraybackslash}p{#1}}
\newcommand{\matindex}[1]{\mbox{\scriptsize#1}}% Matrix index
\newcommand{\github}{Github}
\DeclareMathOperator{\tr}{tr}
\DeclareMathOperator{\VEC}{vec}
\newcommand{\bmb}[1]{{\color{red} \emph{#1}}}
\newcommand{\scw}[1]{{\color{blue} \emph{#1}}}
\newcommand{\dmb}[1]{{\color{magenta} \emph{#1}}}

\shortcites{bolker_strategies_2013,sleepstudy,gelman2013bayesian}

\author{Douglas Bates\\University of Wisconsin-Madison\And
  Martin M\"achler\\ETH Zurich\And
  Benjamin M. Bolker\\McMaster University\And
  Steven C. Walker\\McMaster University
}
\Plainauthor{Douglas Bates, Martin M\"achler, Ben Bolker, Steve Walker}
\title{Fitting Linear Mixed-Effects Models Using \pkg{lme4}}
\Plaintitle{Fitting Linear Mixed-Effects Models using lme4}
\Shorttitle{Linear Mixed Models with lme4}
\Abstract{%

  Maximum likelihood or restricted maximum likelihood (REML) estimates
  of the parameters in linear mixed-effects models can be determined
  using the \code{lmer} function in the \pkg{lme4} package for
  \proglang{R}. As for most model-fitting functions in \proglang{R},
  the model is described in an \code{lmer} call by a formula, in this
  case including both fixed- and random-effects
  terms. The formula and data together determine a numerical
  representation of the model from which the profiled deviance or the
  profiled REML criterion can be evaluated as a function of some of
  the model parameters.  The appropriate criterion is optimized, using
  one of the constrained optimization functions in \proglang{R}, to
  provide the parameter estimates.  We describe the structure of the
  model, the steps in evaluating the profiled deviance or REML
  criterion, and the structure of classes or types that represents such
  a model.  Sufficient detail is included to allow specialization of
  these structures by users who wish to write functions to fit
  specialized linear mixed models, such as models incorporating
  pedigrees or smoothing splines, that are not easily expressible in
  the formula language used by \code{lmer}.}

\Keywords{%
  sparse matrix methods, linear mixed models, penalized least squares,
  Cholesky decomposition} \Address{
  Douglas Bates\\
  Department of Statistics, University of Wisconsin\\
  1300 University Ave.\\
  Madison, WI 53706, U.S.A.\\
  E-mail: \email{bates@stat.wisc.edu}\\
  \par\bigskip
  Martin M\"achler\\
  Seminar f\"ur Statistik, HG G~16\\
  ETH Zurich\\
  8092 Zurich, Switzerland\\
  E-mail: \email{maechler@stat.math.ethz.ch}\\
  % URL: \url{http://stat.ethz.ch/people/maechler}\\
  \par\bigskip
  Benjamin M. Bolker\\
  Departments of Mathematics \& Statistics and Biology \\
  McMaster University \\
  1280 Main Street W \\
  Hamilton, ON L8S 4K1, Canada \\
  E-mail: \email{bolker@mcmaster.ca}\\
  \par\bigskip
  Steven C. Walker\\
  Department of Mathematics \& Statistics \\
  McMaster University \\
  1280 Main Street W \\
  Hamilton, ON L8S 4K1, Canada \\
  E-mail: \email{scwalker@math.mcmaster.ca }
}
\newcommand{\thetavec}{{\bm\theta}}
\newcommand{\betavec}{{\bm\beta}}
\newcommand{\Var}{\operatorname{Var}}
\newcommand{\abs}{\operatorname{abs}}
\newcommand{\bLt}{\ensuremath{\bm\Lambda_{\bm\theta}}}
\newcommand{\mc}[1]{\ensuremath{\mathcal{#1}}}
\newcommand{\trans}{\ensuremath{^\top}} %  JSS wants \top
\newcommand{\yobs}{\ensuremath{\bm y_{\mathrm{obs}}}}
\newcommand*{\eq}[1]{eqn.~\ref{#1}}% or just {(\ref{#1})}
<<preliminaries,include=FALSE,cache=FALSE,message=FALSE>>=
options(width=70, show.signif.stars=FALSE,
        str=strOptions(strict.width="cut"),
        ## prefer empty continuation for reader's cut'n'paste:
        continue = "   ", #JSS: prompt = "R> ", continue = "+  ",
        useFancyQuotes = FALSE)
library("knitr")
library("lme4")
library("ggplot2")# Keeping default theme, nicer "on line":
#JSS theme_set(theme_bw())
library("grid")
zmargin <- theme(panel.spacing=unit(0,"lines"))
library("lattice")
library("minqa")
library("reformulas")
opts_chunk$set(engine='R',dev='pdf', fig.width=9, fig.height=5.5,
               prompt=TRUE, cache=TRUE, tidy=FALSE, comment=NA, error = FALSE)
knitr::render_sweave()
@
\setkeys{Gin}{width=\textwidth}
\setkeys{Gin}{height=3.5in}

\begin{document}

A version of this manuscript has been published online in the
\emph{Journal of Statistical Software}, on Oct.\ 2015, with DOI \linebreak[3]
\texttt{10.18637/jss.v067.i01},
see \url{https://www.jstatsoft.org/article/view/v067i01/}.

\section{Introduction}
\label{sec:intro}

The \pkg{lme4} package  \citep{lme4} for \proglang{R} \citep{R} provides functions to fit and
analyze linear mixed models, generalized linear mixed models and
nonlinear mixed models.  In each of these names, the term ``mixed''
or, more fully, ``mixed effects'', denotes a model that incorporates
both fixed- and random-effects terms in a linear predictor expression
from which the conditional mean of the response can be evaluated.  In
this paper we describe the formulation and representation of linear
mixed models.  The techniques used for generalized linear and
nonlinear mixed models will be described separately, in a future
paper.

At present, the main alternative to \pkg{lme4} for mixed modeling in
\proglang{R} is the \pkg{nlme} package \citep{nlme_pkg}.  The main features
distinguishing \pkg{lme4} from \pkg{nlme} are (1) more efficient
linear algebra tools, giving improved performance on large problems;
(2) simpler syntax and more efficient implementation for fitting
models with crossed random effects; (3) the implementation of profile
likelihood confidence intervals on random-effects parameters; and (4)
the ability to fit generalized linear mixed models (although in this
paper we restrict ourselves to linear mixed models).  The main
advantage of \pkg{nlme} relative to \pkg{lme4} is a user interface for
fitting models with structure in the residuals (various forms of
heteroscedasticity and autocorrelation) and in the random-effects
covariance matrices (e.g., compound symmetric models).  With some extra
effort, the computational machinery of \pkg{lme4} can be used to fit
structured models that the basic \code{lmer} function cannot handle
(see Appendix~\ref{sec:modularExamples}).


The development of general software for fitting mixed models remains
an active area of research with many open problems.  Consequently, the
\pkg{lme4} package has evolved since it was first released, and
continues to improve as we learn more about mixed models. However, we
recognize the need to maintain stability and backward compatibility of
\pkg{lme4} so that it continues to be broadly useful. In order to
maintain stability while continuing to advance mixed-model
computation, we have developed several additional frameworks that draw
on the basic ideas of \pkg{lme4} but modify its structure or
implementation in various ways. These descendants include the
\mbox{\pkg{MixedModels}} package \citep{MixedModels} in \proglang{Julia}
\citep{Julia}, the
\pkg{lme4pureR} package \citep{lme4pureR} in \proglang{R}, and the \pkg{flexLambda}
development branch of \pkg{lme4}.  The current article is largely
restricted to describing the current stable version of the \pkg{lme4}
package (1.1-7), with Appendix~\ref{sec:modularExamples} describing
hooks into the computational machinery that are designed for extension
development. The \pkg{gamm4} \citep{gamm4} and \pkg{blme} \citep{blme, blme2}
packages currently make use of these hooks.

Another goal of this article is to contrast the approach used by
\pkg{lme4} with previous formulations of mixed models.  The
expressions for the profiled log-likelihood and profiled REML
(restricted maximum likelihood) criteria derived in
Section~\ref{sec:profdev} are similar to those presented in
\citet{bates04:_linear} and, indeed, are closely related to
``Henderson's mixed-model equations''~\citep{henderson_1982}.
Nonetheless there are subtle but important changes in the formulation
of the model and in the structure of the resulting penalized least
squares (PLS) problem to be solved (Section~\ref{sec:PLSpureR}).  We
derive the current version of the PLS problem
(Section~\ref{sec:plsMath}) and contrast this result with earlier
formulations (Section~\ref{sec:previous_lmm_form}).

This article is organized into four main sections (Sections~\ref{sec:lFormula},
\ref{sec:mkLmerDevfun}, \ref{sec:optimizeLmer}, and \ref{sec:mkMerMod}),
each of which corresponds to one of the four largely separate modules
that comprise \pkg{lme4}.  Before describing the details of each
module, we describe the general form of the linear mixed model
underlying \pkg{lme4} (Section~\ref{sec:LMMs}); introduce the
\code{sleepstudy} data that will be used as an example throughout
(Section~\ref{sec:sleepstudy}); and broadly outline \pkg{lme4}'s
modular structure (Section~\ref{sec:modular}).

\subsection{Linear mixed models}
\label{sec:LMMs}

Just as a linear model is described by the distribution of a
vector-valued random response variable, $\mc{Y}$, whose observed value
is $\yobs$, a linear mixed model is described by the distribution of
two vector-valued random variables: $\mc{Y}$, the response, and
$\mc{B}$, the vector of random effects.  In a linear model the
distribution of $\mc Y$ is multivariate normal,%\begin{linenomath}
\begin{equation}
  \label{eq:linearmodel}
  \mc Y\sim\mc{N}(\bm X\bm\beta+\bm o,\sigma^2\bm W^{-1}),
\end{equation}
where $n$ is the dimension of the response vector, $\bm W$ is a
diagonal matrix of known prior weights, $\bm\beta$ is a
$p$-dimensional coefficient vector, $\bm X$ is an $n\times p$ model
matrix, and $\bm o$ is a vector of known prior offset terms. The
parameters of the model are the coefficients $\bm\beta$ and the scale
parameter $\sigma$.

In a linear mixed model it is the \emph{conditional} distribution of
$\mc Y$ given $\mc B=\bm b$ that has such a form,
\begin{equation}
  \label{eq:LMMcondY}
  ( \mc Y|\mc B=\bm b)\sim\mc{N}(\bm X\bm\beta+\bm Z\bm b+\bm o,\sigma^2\bm W^{-1}), % | <- for ESS
\end{equation}
where $\bm Z$ is the $n\times q$ model matrix for the $q$-dimensional
vector-valued random-effects variable, $\mc B$, whose value we are
fixing at $\bm b$.  The unconditional distribution of $\mc B$ is also
multivariate normal with mean zero and a parameterized $q\times q$
variance-covariance matrix, $\bm\Sigma$,
\begin{equation}
  \label{eq:LMMuncondB}
  \mc B\sim\mc N(\bm0,\bm\Sigma) .
\end{equation}
As a variance-covariance matrix, $\bm\Sigma$ must be positive
semidefinite.  It is convenient to express the model in terms of a
\emph{relative covariance factor}, $\bLt$, which is a
$q\times q$ matrix, depending on the \emph{variance-component
  parameter}, $\bm\theta$, and generating the symmetric $q\times q$
variance-covariance matrix, $\bm\Sigma$, according to%\begin{linenomath}
\begin{equation}
  \label{eq:relcovfac}
  \bm\Sigma_{\bm\theta}=\sigma^2\bLt\bLt\trans ,
\end{equation}%\end{linenomath}
where $\sigma$ is the same scale factor as in the conditional
distribution (\ref{eq:LMMcondY}).

Although Equations~\ref{eq:LMMcondY}, \ref{eq:LMMuncondB}, and
\ref{eq:relcovfac} fully describe the class of linear mixed models
that \pkg{lme4} can fit, this terse description
hides many important details.  Before moving on to these details, we
make a few observations:
\begin{itemize}
\item This formulation of linear mixed models allows for a relatively
  compact expression for the profiled log-likelihood of $\bm\theta$
  (Section~\ref{sec:profdev}, Equation~\ref{eq:profiledDeviance}).
\item The matrices associated with random effects, $\bm Z$ and $\bLt$,
  typically have a sparse structure with a sparsity pattern that
  encodes various model assumptions.  Sections~\ref{sec:LMMmatrix} and
  \ref{sec:CSCmats} provide details on these structures, and how to
  represent them efficiently.
\item The interface provided by \pkg{lme4}'s \code{lmer} function
  is slightly less general than the model described by Equations~\ref{eq:LMMcondY}, \ref{eq:LMMuncondB}, and \ref{eq:relcovfac}.  To
  take advantage of the entire range of possibilities, one may use the
  modular functions (Sections~\ref{sec:modular} and
  Appendix~\ref{sec:modularExamples}) or explore the
  experimental \pkg{flexLambda} branch of \pkg{lme4} on \github.
\end{itemize}

\subsection{Example}
\label{sec:sleepstudy}

Throughout our discussion of \pkg{lme4}, we will work with a data set
on the average reaction time per day for subjects in a sleep
deprivation study \citep{sleepstudy}. On day 0 the subjects had their
normal amount of sleep.  Starting that night they were restricted to 3
hours of sleep per night.  The response variable, \code{Reaction},
represents average reaction times in milliseconds (ms) on a series of tests given each
\code{Day} to each \code{Subject} (Figure~\ref{fig:sleepPlot}),
%
<<sleep>>=
str(sleepstudy)
@

<<sleepPlot, echo=FALSE, fig.scap="Reaction time versus days by subject", fig.cap="Average reaction time versus days of sleep deprivation by subject.  Subjects ordered (from left to right starting on the top row) by increasing slope of subject-specific linear regressions.", fig.align='center', fig.pos='tb'>>=
## BMB: seemed more pleasing to arrange by increasing slope rather than
## intercept ...
xyplot(Reaction ~ Days | Subject, sleepstudy, aspect = "xy",
       layout = c(9, 2), type = c("g", "p", "r"),
       index.cond = function(x, y) coef(lm(y ~ x))[2],
       xlab = "Days of sleep deprivation",
       ylab = "Average reaction time (ms)",
       as.table = TRUE)
@
% |

Each subject's reaction time increases approximately linearly
with the number of sleep-deprived days.
However, subjects also appear to vary in the slopes and intercepts
of these relationships, which suggests a model with random slopes and intercepts.
As we shall see, such a model may be fitted by minimizing the
REML criterion (Equation~\ref{eq:REMLdeviance}) using
<<outputExample,results="hide">>=
fm1 <- lmer(Reaction ~ Days + (Days | Subject), sleepstudy)
@
% |
The estimates of the standard deviations of the random effects for the
intercept and the slope are
\Sexpr{round(sqrt(VarCorr(fm1)$Subject[1,1]), 2)} ms % $
and
\Sexpr{round(sqrt(VarCorr(fm1)$Subject[2,2]), 2)} ms/day. % $
The fixed-effects coefficients, $\betavec$, are
\Sexpr{round(fixef(fm1)[1], 1)} ms and
\Sexpr{round(fixef(fm1)[2], 2)} ms/day
for the intercept and slope.  In this model, one interpretation of
these fixed effects is that they are the estimated population mean values of the
random intercept and slope (Section~\ref{sec:intuitiveFormulas}).

We have chosen the \code{sleepstudy} example because it is a
relatively small and simple example to illustrate the theory and
practice underlying \code{lmer}.  However, \code{lmer} is capable of
fitting more complex mixed models to larger data sets.  For example, we
direct the interested reader to \code{RShowDoc("lmerperf", package =
  "lme4")} for examples that more thoroughly exercise the
performance capabilities of \code{lmer}.

\subsection{High-level modular structure}
\label{sec:modular}

The \code{lmer} function is composed of four largely independent
modules.  In the first module, a mixed-model formula is parsed and
converted into the inputs required to specify a linear mixed model
(Section~\ref{sec:lFormula}).  The second module uses these inputs to
construct an \proglang{R} function which takes the covariance
parameters, $\bm\theta$, as arguments and returns negative twice the log
profiled likelihood or the REML criterion
(Section~\ref{sec:mkLmerDevfun}).  The third module optimizes this
objective function to produce maximum likelihood (ML) or REML
estimates of $\bm\theta$ (Section~\ref{sec:optimizeLmer}).  Finally,
the fourth module provides utilities for interpreting the optimized
model (Section~\ref{sec:mkMerMod}).

\begin{table}[tb]
  \centering
  \begin{tabular}{lllp{2.1in}}
    \hline
    Module & & \proglang{R} function & Description \\
    \hline
    Formula module & (Section~\ref{sec:lFormula}) & \code{lFormula} &
    Accepts a mixed-model formula, data, and other user inputs, and returns
    a list of objects required to fit a linear mixed model. \\
    Objective function module & (Section~\ref{sec:mkLmerDevfun}) &
    \code{mkLmerDevfun} &
    Accepts the results of \code{lFormula} and returns a function to
    calculate the deviance (or restricted deviance) as a function of the
    covariance parameters, $\bm\theta$.\\
    Optimization module & (Section~\ref{sec:optimizeLmer}) &
    \code{optimizeLmer} &
    Accepts a deviance function returned by \code{mkLmerDevfun} and
    returns the results of the optimization of that deviance function. \\
    Output module & (Section~\ref{sec:mkMerMod}) & \code{mkMerMod} &
    Accepts an optimized deviance function and packages the results
    into a useful object.  \\
    \hline
  \end{tabular}
  \caption{The high-level modular structure of \code{lmer}.}
  \label{tab:modular}
\end{table}

To illustrate this modularity, we recreate the \code{fm1} object by a
series of four modular steps; the formula module,
<<modularExampleFormula, tidy=FALSE, eval=FALSE>>=
parsedFormula <- lFormula(formula = Reaction ~ Days + (Days | Subject),
                             data = sleepstudy)
@
the objective function module,
<<modularExampleObjective, tidy=FALSE, eval=FALSE>>=
devianceFunction <- do.call(mkLmerDevfun, parsedFormula)
@
the optimization module,
<<modularExampleOptimization, tidy=FALSE, eval=FALSE>>=
optimizerOutput <- optimizeLmer(devianceFunction)
@
and the output module,
<<modularExampleOutput, tidy=FALSE, eval=FALSE>>=
mkMerMod(   rho = environment(devianceFunction),
            opt = optimizerOutput,
         reTrms = parsedFormula$reTrms,
             fr = parsedFormula$fr)
@
% |

\section{Formula module}
\label{sec:lFormula}

\subsection{Mixed-model formulas}
\label{sec:formulas}

Like most model-fitting functions in \proglang{R},
\code{lmer} takes as its first two arguments a \emph{formula}
specifying the model and the \emph{data} with which to evaluate the
formula. This second argument, \code{data}, is optional but
recommended and is usually the name of an \proglang{R} data frame.
In the \proglang{R} \code{lm} function for fitting linear models,
formulas take the form \verb+resp ~ expr+,
where \code{resp} determines the response variable and \code{expr}
is an expression that specifies the columns of the model matrix.
Formulas for the \code{lmer} function contain special random-effects
terms,
<<MMformula, eval=FALSE>>=
resp ~ FEexpr + (REexpr1 | factor1) + (REexpr2 | factor2) + ...
@
where \code{FEexpr} is an expression determining the columns of the
fixed-effects model matrix, $\bm X$, and the random-effects terms,
\code{(REexpr1 | factor1)} and \code{(REexpr2 | factor2)}, determine
both the random-effects model matrix, $\bm Z$ (Section~\ref{sec:mkZ}),
and the structure of the relative covariance factor,
$\bLt$ (Section~\ref{sec:mkLambdat}).  In principle, a
mixed-model formula may contain arbitrarily many random-effects terms,
but in practice the number of such terms is typically low.

\subsection{Understanding mixed-model formulas}
\label{sec:intuitiveFormulas}

Before describing the details of how \pkg{lme4} parses mixed-model
formulas (Section~\ref{sec:LMMmatrix}), we provide an informal
explanation and then some examples. Our discussion assumes familiarity
with the standard \proglang{R} modeling paradigm
\citep{Chambers:1993}.

Each random-effects term is of the form \code{(expr | factor)}.
The expression \code{expr} is evaluated as a linear model formula,
producing a model matrix following the same rules used in standard
\proglang{R} modeling functions (e.g., \code{lm} or \code{glm}). The
expression \code{factor} is evaluated as an \proglang{R} factor.  One
way to think about the vertical bar operator is as a special kind of
interaction between the model matrix and the grouping factor. This
interaction ensures that the columns of the model matrix have
different effects for each level of the grouping factor.  What makes
this a special kind of interaction is that these effects are modeled
as unobserved random variables, rather than unknown fixed parameters.
Much has been written about important practical and philosophical
differences between these two types of interactions
\citep[e.g., ][]{henderson_1982,gelman2005analysis}. For
example, the random-effects implementation of such interactions can be
used to obtain shrinkage estimates of regression coefficients
\citep[e.g., ][]{1977EfronAndMorris}, or account
for lack of independence in the residuals due to block structure or
repeated measurements \citep[e.g., ][]{laird_ware_1982}.

Table~\ref{tab:formulas} provides several examples of the
right-hand-sides of mixed-model formulas. The first example,
\code{(1 | g)}, % |
is the simplest possible mixed-model formula, where each level of the
grouping factor, \code{g}, has its own random intercept. The mean and
standard deviation of these intercepts are parameters to be estimated.
Our description of this model incorporates any nonzero mean of the
random effects as fixed-effects parameters.  If one wishes to specify
that a random intercept has \emph{a priori} known means, one may use
the \code{offset} function as in the second  model in
Table~\ref{tab:formulas}.  This model contains no fixed effects, or
more accurately the fixed-effects model matrix, $\bm X$, has zero
columns and $\bm\beta$ has length zero.

\begin{table}[tb]
  \centering
  \begin{tabular}{llP{1.5in}} %% see new column type for ragged right
    \hline
    Formula 			& Alternative & Meaning \\
    \hline%------------------------------------------------
    \code{(1 | g)}		& \code{1 + (1 | g)}
    & Random intercept with fixed mean. \\
    \code{0 + offset(o) + (1 | g)}    & \code{-1 + offset(o) + (1 | g)}
    & Random intercept with \emph{a priori} means. \\
    \code{(1 | g1/g2)} 		& \code{(1 | g1)+(1 | g1:g2)}   % |
    & Intercept varying among \code{g1} and \code{g2} within \code{g1}. \\
    \code{(1 | g1) + (1 | g2)} 	& \code{1 + (1 | g1) + (1 | g2)}. &
    Intercept varying among \code{g1} and \code{g2}. \\
    \code{x + (x | g)} 		& \code{1 + x + (1 + x | g)} &
    Correlated random intercept and slope. \\
    \code{x + (x || g)} 		& \code{1 + x + (1 | g) + (0 +
      x | g)}
    & Uncorrelated random intercept and slope. \\
    \hline
  \end{tabular}
  \caption{Examples of the right-hand-sides of mixed-effects model
    formulas. The names of grouping factors are denoted \code{g},
    \code{g1}, and \code{g2}, and covariates and \emph{a priori} known
    offsets as \code{x} and \code{o}.}
  \label{tab:formulas}
\end{table}

We may also construct models with multiple grouping factors. For
example, if the observations are grouped by \code{g2}, which is nested
within \code{g1}, then the third formula in Table \ref{tab:formulas}
can be used to model variation in the intercept.  A common objective
in mixed modeling is to account for such nested (or hierarchical)
structure.  However, one of the most useful aspects of \pkg{lme4} is
that it can be used to fit random effects associated with non-nested
grouping factors.  For example, suppose the data are grouped by fully
crossing two factors, \code{g1} and \code{g2}, then the fourth formula
in Table \ref{tab:formulas} may be used.  Such models are common in
item response theory, where \code{subject} and \code{item} factors are
fully crossed \citep{doran2007estimating}. In addition to varying
intercepts, we may also have varying slopes (e.g., the
\code{sleepstudy} data, Section~\ref{sec:sleepstudy}).  The fifth
example in Table~\ref{tab:formulas} gives a model where both the
intercept and slope vary among the levels of the grouping factor.

\subsubsection{Specifying uncorrelated random effects}
\label{sec:uncor}

By default, \pkg{lme4} assumes that all coefficients associated with
the same random-effects term are correlated. To specify an
uncorrelated slope and intercept (for example), one may either use
double-bar notation, \code{(x || g)}, or equivalently use multiple
random-effects terms, \code{x + (1 | g) + (0 + x | g)}, as in the
final example of Table~\ref{tab:formulas}.  For example, if one
examined the results of model \code{fm1} of the \code{sleepstudy} data
(Section~\ref{sec:sleepstudy}) using \code{summary(fm1)}, one would
see that the estimated correlation between the slope for \code{Days}
and the intercept is fairly low
(\Sexpr{round(attr(VarCorr(fm1)$Subject, "correlation")[2],3)}) % $
(See Section~\ref{sec:summary} below for more on how to extract the
random-effects covariance matrix.)  We may use double-bar notation to
fit a model that excludes a correlation parameter:
<<uncorrelatedModel,results="hide">>=
fm2 <- lmer(Reaction ~ Days + (Days || Subject), sleepstudy)
@

Although mixed models where the random slopes and intercepts are
assumed independent are commonly used to reduce the complexity of
random-slopes models, they do have one subtle drawback.  Models in
which the slopes and intercepts are allowed to have a nonzero
correlation (e.g., \code{fm1}) are invariant to additive shifts of the
continuous predictor (\code{Days} in this case).  This invariance
breaks down when the correlation is constrained to zero; any shift in
the predictor will necessarily lead to a change in the estimated
correlation, and in the likelihood and predictions of the model.  For
example, we can eliminate the correlation in \code{fm1} simply by
adding an amount equal to the ratio of the estimated
among-subject standard deviations multiplied by the estimated
correlation (i.e., $\sigma_{\text{\small slope}}/\sigma_{\text{\small
    intercept}} \cdot \rho_{\text{\small slope:intercept}}$) to the \code{Days} variable.  The use
of models such as \code{fm2} should ideally be restricted to cases where the
predictor is measured on a ratio scale (i.e., the zero point on the
scale is meaningful, not just a location defined by convenience or
convention), as is the case here.

%% <<uncorrelatedModelByShifting>>=
%% sleepstudyShift <- within(sleepstudy, {
%%     Days <- Days + (24.74*0.07)/5.92 })
%% lmer(Reaction ~ Days + (Days |  Subject), sleepstudyShift)
%% @

\subsection{Algebraic and computational account of mixed-model formulas}
\label{sec:LMMmatrix}

The fixed-effects terms of a mixed-model formula are parsed to produce
the fixed-effects model matrix, $\bm X$, in the same way that the
\proglang{R} \code{lm} function generates model matrices.  However, a
mixed-model formula incorporates $k\ge1$ random-effects terms of the
form \code{(r | f)} as well. % |
These $k$ terms are used to produce the random-effects model matrix,
$\bm Z$ (Equation~\ref{eq:LMMcondY}; Section~\ref{sec:mkZ}), and the
structure of the relative covariance factor, $\bLt$
(Equation~\ref{eq:relcovfac}; Section~\ref{sec:mkLambdat}), which are
matrices that typically have a sparse structure.  We now describe how
one might construct these matrices from the random-effects terms,
considering first a single term, \code{(r | f)}, % |
and then generalizing to multiple terms. Tables~\ref{tab:dim} and
\ref{tab:algebraic} summarize the matrices and vectors that determine
the structure of $\bm Z$ and
$\bLt$.

\begin{table}[tb]
  \centering
  \begin{tabular}{lll}
    \hline
    Symbol & Size \\
    \hline
    $n$ & Length of the response vector, $\mc{Y}$ \\
    $p$ & Number of columns of fixed-effects model matrix, $\bm X$ \\
    $q = \sum_i^k q_i$ & Number of columns of random-effects model matrix, $\bm Z$ \\
    $p_i$ &  Number of columns of the raw model matrix, $\bm X_i$ \\
    $\ell_i$ &  Number of levels of the grouping factor indices, $\bm i_i$ \\
    $q_i = p_i\ell_i$  & Number of columns of the term-wise model matrix, $\bm Z_i$ \\
    $k$ &  Number of random-effects terms \\
    $m_i = \binom{p_i+1}{2}$ & Number of covariance parameters for term
    $i$ \\
    $m = \sum_i^k m_i$ & Total number of covariance parameters \\
    \hline
  \end{tabular}
  \caption{Dimensions of linear mixed models.  The
    subscript $i = 1, \dots, k$ denotes a specific random-effects term.}
  \label{tab:dim}
\end{table}

\begin{table}[tb]
  \centering
  \begin{tabular}{lll}
    \hline
    Symbol  & Size & Description \\
    \hline
    $\bm X_i$ & $n\times p_i$ &  Raw random-effects model matrix \\
    $\bm J_i$  & $n\times \ell_i$ & Indicator matrix of grouping factor indices\\
    $\bm X_{ij}$ & $p_i\times 1$ & Column vector containing $j$th row of $\bm X_i$ \\
    $\bm J_{ij}$ & $\ell_i\times 1$ & Column vector containing $j$th row of $\bm J_i$ \\
    $\bm i_i$  & $n$ & Vector of grouping factor indices \\
    $\bm Z_i$ & $n\times q_i$ & Term-wise random-effects model matrix \\
    $\bm\theta$ & $m$ & Covariance parameters \\
    $\bm T_i$ & $p_i\times p_i$ & Lower triangular template matrix \\
    $\bm\Lambda_i$ & $q_i\times q_i$ & Term-wise relative covariance factor \\
    \hline
  \end{tabular}
  \caption{Symbols used to describe the structure of the random-effects model matrix and the relative covariance factor. The
    subscript $i = 1, \dots, k$ denotes a specific random-effects term.}
  \label{tab:algebraic}
\end{table}

The expression, \code{r}, is a linear model formula that evaluates to
an \proglang{R} model matrix, $\bm X_i$, of size $n\times p_i$, called
the \emph{raw random-effects model matrix} for term $i$. A term is
said to be a \emph{scalar} random-effects term when $p_i=1$, otherwise
it is \emph{vector-valued}.  For a \emph{simple, scalar}
random-effects term of the form \code{(1 | f)}, $\bm X_i$ is the    % |
$n\times 1$ matrix of ones, which implies a random intercept model.

The expression \code{f} evaluates to an \proglang{R} factor, called
the \emph{grouping factor}, for the term. For the $i$th term, we
represent this factor mathematically with a vector $\bm i_i$ of
\emph{factor indices}, which is an $n$-vector of values from
$1,\dots,\ell_i$.\footnote{In practice,
  fixed-effects model matrices and random-effects terms are evaluated
  with respect to a \emph{model frame}, ensuring that any expressions
  for grouping factors have been coerced to factors and any unused
  levels of these factors have been dropped.  That is, $\ell_i$, the
  number of levels in the grouping factor for the $i$th random-effects
  term, is well-defined.}
Let $\bm J_i$ be the $n\times \ell_i$ matrix of
indicator columns for $\bm i_i$.  Using the \pkg{Matrix} package
\citep{Matrix_pkg} in
\proglang{R}, we may construct the transpose of $\bm J_i$ from a
factor vector, \code{f}, by coercing \code{f} to a `\code{sparseMatrix}'
object.  For example,
<<setSeed, echo=FALSE>>=
set.seed(2)
@
<<factorToSparseMatrix>>=
(f <- gl(3, 2))
(Ji <- t(as(f, Class = "sparseMatrix")))
@
When $k>1$ we order the random-effects terms so that
$\ell_1\ge\ell_2\ge\dots\ge\ell_k$; in general, this ordering reduces
``fill-in'' (i.e., the proportion of elements that are zero in the
lower triangle of $\bLt\trans\bm Z\trans\bm W\bm Z\bLt+\bm I$ but not in
the lower triangle of its left Cholesky factor, $\bm L_{\bm\theta}$,
described below in Equation~\ref{eq:blockCholeskyDecomp}). This reduction
in fill-in provides more efficient matrix operations within the
penalized least squares algorithm (Section~\ref{sec:plsMath}).

\subsubsection{Constructing the random-effects model matrix}
\label{sec:mkZ}

The $i$th random-effects term contributes $q_i=\ell_ip_i$ columns to
the model matrix $\bm Z$.  We group these columns into a matrix, $\bm
Z_i$, which we refer to as the \emph{term-wise model matrix} for the
$i$th term.  Thus $q$, the number of columns in $\bm Z$ and the
dimension of the random variable, $\mc{B}$, is
\begin{equation}
  \label{eq:qcalc}
  q=\sum_{i=1}^k q_i = \sum_{i=1}^k \ell_i\,p_i .
\end{equation}
Creating the matrix $\bm Z_i$ from $\bm X_i$ and $\bm J_i$ is a
straightforward concept that is, nonetheless, somewhat awkward to
describe.  Consider $\bm Z_i$ as being further decomposed into
$\ell_i$ blocks of $p_i$ columns.  The rows in the first block are the
rows of $\bm X_i$ multiplied by the 0/1 values in the first column of
$\bm J_i$ and similarly for the subsequent blocks.  With these
definitions we may define the term-wise random-effects model matrix,
$\bm Z_i$, for the $i$th term as a transposed Khatri-Rao product,
\begin{equation}
  \label{eq:Zi}
  \bm Z_i = (\bm J_i\trans * \bm X_i\trans)\trans =
  \begin{bmatrix}
    \bm J_{i1}\trans \otimes \bm X_{i1}\trans \\
    \bm J_{i2}\trans \otimes \bm X_{i2}\trans \\
    \vdots \\
    \bm J_{in}\trans \otimes \bm X_{in}\trans \\
  \end{bmatrix},
\end{equation}
where $*$ and $\otimes$ are the Khatri-Rao\footnote{Note that the
  original definition of the Khatri-Rao product is more general than
  the definition used in the \pkg{Matrix} package, which is the
  definition we use here.} \citep{khatri1968solutions} and Kronecker
products, and $\bm J_{ij}\trans$ and $\bm X_{ij}\trans$ are row
vectors of the $j$th rows of $\bm J_i$ and $\bm X_i$.  These rows
correspond to the $j$th sample in the response vector, $\mc Y$, and
thus $j$ runs from $1, \dots, n$.  The \pkg{Matrix} package for
\proglang{R} contains a \code{KhatriRao} function, which can be used
to form $\bm Z_i$.  For example, if we begin with a raw model matrix,
<<rawModelMatrix>>=
(Xi <- cbind(1, rep.int(c(-1, 1), 3L)))
@
then the term-wise random-effects
model matrix is,
<<KhatriRao>>=
(Zi <- t(KhatriRao(t(Ji), t(Xi))))
@
<<sanityCheck, include=FALSE>>=
## alternative formulation of Zi (eq:Zi)
rbind(
  Ji[1,] %x% Xi[1,],
  Ji[2,] %x% Xi[2,],
  Ji[3,] %x% Xi[3,],
  Ji[4,] %x% Xi[4,],
  Ji[5,] %x% Xi[5,],
  Ji[6,] %x% Xi[6,])
@

In particular, for a simple, scalar term, $\bm Z_i$ is exactly $\bm
J_i$, the matrix of indicator columns.  For other scalar terms, $\bm
Z_i$ is formed by element-wise multiplication of the single column of
$\bm X_i$ by each of the columns of $\bm J_i$.

Because each $\bm Z_i$ is generated from indicator columns, its
cross-product, $\bm Z_i\trans\bm Z_i$ is block-diagonal consisting of
$\ell_i$ diagonal blocks each of size $p_i$.\footnote{To see this,
  note that by the properties of Kronecker products we may write the
  cross-product matrix $Z_i\trans Z_i$ as $\sum_{j=1}^n \bm J_{ij} \bm
  J_{ij}\trans \otimes \bm X_{ij} \bm X_{ij}\trans$. Because $\bm
  J_{ij}$ is a unit vector along a coordinate axis, the cross-product
  $\bm J_{ij} \bm J_{ij}\trans$ is a $p_i\times p_i$ matrix of all
  zeros except for a single $1$ along the diagonal. Therefore, the
  cross-products, $\bm X_{ij} \bm X_{ij}\trans$, will be added to one
  of the $\ell_i$ blocks of size $p_i\times p_i$ along the diagonal of
  $Z_i\trans Z_i$.} Note that this means that when $k=1$ (i.e., there
is only one random-effects term, and $\bm Z_i = \bm Z$), $\bm
Z\trans\bm Z$ will be block diagonal.  These block-diagonal properties
allow for more efficient sparse matrix computations
(Section~\ref{sec:CSCmats}).

The full random-effects model matrix, $\bm Z$, is constructed from
$k\ge 1$ blocks,
\begin{equation}
  \label{eq:Z}
  \bm Z =
  \begin{bmatrix}
    \bm Z_1 &
    \bm Z_2 &
    \hdots &
    \bm Z_k \\
  \end{bmatrix}.
\end{equation}
By transposing Equation~\ref{eq:Z} and substituting in Equation~\ref{eq:Zi}, we
may represent the structure of the transposed random-effects model
matrix as follows,
\begin{equation}
  \label{eq:Zt}
  \bm Z\trans =
  \begin{blockarray}{ccccc}
    \text{sample 1} & \text{sample 2} & \hdots & \text{sample } n & \\
    \begin{block}{[cccc]c}
    \bm J_{11} \otimes \bm X_{11} &
    \bm J_{12} \otimes \bm X_{12} &
    \hdots &
    \bm J_{1n} \otimes \bm X_{1n} &
    \text{term 1} \\
    \bm J_{21} \otimes \bm X_{21} &
    \bm J_{22} \otimes \bm X_{22} &
    \hdots &
    \bm J_{2n} \otimes \bm X_{2n} &
    \text{term 2} \\
    \vdots & \vdots & \ddots & \vdots & \vdots \\
    \end{block}
  \end{blockarray}.
\end{equation}
Note that the proportion of elements of $Z\trans$ that are structural
zeros is
\begin{equation}
  \label{eq:ZtSparsity}
  \frac{\sum_{i=1}^k p_i(\ell_i - 1)}{\sum_{i=1}^k p_i} \qquad .
\end{equation}
Therefore, the sparsity of $\bm Z\trans$ increases with the number of
grouping factor levels. As the number of levels is often large in
practice, it is essential for speed and efficiency to take account of
this sparsity, for example by using sparse matrix methods, when
fitting mixed models (Section~\ref{sec:CSCmats}).

\subsubsection{Constructing the relative covariance factor}
\label{sec:mkLambdat}

The $q\times q$ covariance factor, $\bLt$, is a block diagonal matrix
whose $i$th diagonal block, $\bm\Lambda_i$, is of size
$q_i,i=1,\dots,k$.  We refer to $\bm\Lambda_i$ as the \emph{term-wise
  relative covariance factor}.  Furthermore, $\bm\Lambda_i$ is a
homogeneous block diagonal matrix with each of the $\ell_i$
lower-triangular blocks on the diagonal being a copy of a $p_i\times
p_i$ lower-triangular \emph{template matrix}, $\bm T_i$.  The
covariance parameter vector, $\bm\theta$, of length $m_i
=\binom{p_i+1}{2}$, consists of the elements in the lower triangle of
$\bm T_i,i=1,\dots,k$.  To provide a unique representation we require
that the diagonal elements of the $\bm T_i,i=1,\dots,k$ be
non-negative.

The template, $\bm T_i$, can be constructed from the number $p_i$
alone.  In \proglang{R} code we denote $p_i$ as \code{nc}.  For
example, if we set \code{nc <- 3}\Sexpr{nc <- 3}, we could create the
template for term $i$ as,
<<nc, echo=FALSE>>=
nc <- 3
@
%% sequence() is equivalent to unlist(lapply(nvec, seq_len))
%% and (?sequence) ``mainly exists in reverence to the very early history of R''
%% scw: i like sequence, and in fact i never understood why that
%% statement is there in the help file.
<<template>>=
(rowIndices <- rep(1:nc, 1:nc))
(colIndices <- sequence(1:nc))
(template <- sparseMatrix(rowIndices, colIndices,
                          x = 1 * (rowIndices == colIndices)))
@
Note that the \code{rowIndices} and \code{colIndices} fill the entire
lower triangle, which contains the initial values of the covariance
parameter vector, $\bm\theta$,
<<thetaFromTemplate>>=
(theta <- template@x)
@
<<thetaSanityCheck, include=FALSE>>=
lFormula(Reaction ~ (Days + I(Days^2) | Subject), sleepstudy)$reTrms$theta
@
(because the \code{@x} slot of the sparse matrix \code{template} is a
numeric vector containing the nonzero elements). This template
contains three types of elements: structural zeros (denoted
by~\code{.}), off-diagonal covariance parameters (initialized at
\code{0}), and diagonal variance parameters (initialized at \code{1}).
The next step in the construction of the relative covariance factor is
to repeat the template once for each level of the grouping factor to
construct a sparse block diagonal matrix.  For example, if we set the
number of levels, $\ell_i$, to two, \code{nl <- 2}\Sexpr{nl <- 2}, we
could create the transposed term-wise relative covariance factor,
$\bm\Lambda_i\trans$, using the \code{.bdiag} function in the
\pkg{Matrix} package,
<<nl, echo=FALSE>>=
nl <- 2
@
<<relativeCovarianceBlock>>=
(Lambdati <- .bdiag(rep(list(t(template)), nl)))
@
For a model with a single random-effects term, $\bm\Lambda\trans$
would be the initial transposed relative covariance factor
$\bm\Lambda_{\bm\theta}\trans$ itself.

The transposed relative covariance factor, $\bLt\trans$, that arises
from parsing the formula and data is set at the initial value of the
covariance parameters, $\bm\theta$.  However, during model fitting, it
needs to be updated to a new $\bm\theta$ value at each iteration (see
Section~\ref{sec:plsI}).  This is achieved by constructing a vector of
indices, \code{Lind}, that identifies which elements of \code{theta}
should be placed in which elements of \code{Lambdat},
<<relativeCovarianceBlockIndices>>=
LindTemplate <- rowIndices + nc * (colIndices - 1) - choose(colIndices, 2)
(Lind <- rep(LindTemplate, nl))
@
For example, if we randomly generate a new value for \code{theta},
<<newTheta>>=
thetanew <- round(runif(length(theta)), 1)
@
we may update \code{Lambdat} as follows,
<<relativeCovarianceBlockUpdate>>=
Lambdati@x <- thetanew[Lind]
@
Section~\ref{sec:PLSpureR} describes the process of
updating the relative covariance factor in more detail.


\section{Objective function module}
\label{sec:mkLmerDevfun}

\subsection{Model reformulation for improved computational stability}
\label{sec:stability}

In our initial formulation of the linear mixed model
(Equations~\ref{eq:LMMcondY}, \ref{eq:LMMuncondB}, and \ref{eq:relcovfac}),
the covariance parameter vector, $\bm\theta$, appears only in the
marginal distribution of the random effects (Equation~\ref{eq:LMMuncondB}).
However, from the perspective of computational stability and
efficiency, it is advantageous to reformulate the model such that
$\bm\theta$ appears only in the conditional distribution for the
response vector given the random effects.  Such a reformulation allows
us to work with singular covariance matrices, which regularly arise in
practice (e.g., during intermediate steps of the nonlinear optimizer,
Section~\ref{sec:optimizeLmer}).

The reformulation is made by defining a \emph{spherical}%
\footnote{$\mathcal{N}(\bm\mu,\sigma^2\bm I)$ distributions are called
  ``spherical'' because contours of the probability density are
  spheres.}  \emph{random-effects} variable, $\mc U$, with
distribution%\begin{linenomath}
\begin{equation}
  \label{eq:distU}
  \mc U\sim\mathcal{N}(\bm 0,\sigma^2\bm I_q).
\end{equation}%\end{linenomath}
If we set,
% \begin{linenomath}
\begin{equation}
  \label{eq:BequalsLamU}
  \mc B=\bLt\mc U,
\end{equation}%\end{linenomath}
then $\mc B$ will have the desired $\mathcal{N}(\bm
0,\bm\Sigma_{\bm\theta})$ distribution (Equation~\ref{eq:LMMuncondB}).
Although it may seem more natural to define $\mc U$ in terms of $\mc
B$ we must write the relationship as in Equation~\ref{eq:BequalsLamU} to allow
for singular $\bLt$. The conditional distribution
(Equation~\ref{eq:LMMcondY}) of the response vector given the random
effects may now be reformulated as,%\begin{linenomath}
\begin{equation}
  \label{eq:distYgivenU}
  (\mc Y|\mc U=\bm u)\sim\mc{N}(\bm\mu_{\mc Y|\mc U=\bm u},\sigma^2\bm W^{-1}),
\end{equation}%\end{linenomath}
where
\begin{equation}
  \label{eq:meanYgivenU}
  \bm\mu_{\mc Y|\mc U=\bm u} = \bm X \bm\beta + \bm Z\bLt\bm u + \bm o
\end{equation} % |
is a vector of linear predictors, which can be interpreted as a
conditional mean (or mode).  Similarly, we also define $\bm\mu_{\mc
  U|\mc Y = \yobs}$ as the conditional mean (or mode) of the spherical
random effects given the observed value of the response vector. Note
also that we use the $\bm u$ symbol throughout to represent a specific
value of the random variable, $\mc U$.

\subsection{Penalized least squares}
\label{sec:plsMath}

Our computational methods for maximum likelihood fitting of the linear
mixed model involve repeated applications of the PLS method. In
particular, the PLS problem is to minimize the penalized weighted
residual sum-of-squares,
\begin{equation}
  \label{eq:pwrss}
  r^2(\bm\theta, \bm\beta, \bm u)
  = \rho^2(\bm\theta, \bm\beta, \bm u)
  + \left\| \bm u \right\|^2,
\end{equation}
over $\begin{bmatrix} \bm u \\
  \bm\beta \\ \end{bmatrix}$, where,
\begin{equation}
  \label{eq:wrss}
  \rho^2(\bm\theta, \bm\beta, \bm u) = \left\|
  \bm W^{1/2}\left[\yobs - \bm\mu_{\mc Y|\mc U=\bm u} \right]\right\|^2,
\end{equation} %|
is the weighted residual sum-of-squares. This notation makes explicit
the fact that $r^2$ and $\rho^2$ both depend on $\bm\theta$,
$\betavec$, and $\bm u$.  The reason for the word ``penalized'' is that
the term, $\left\| \bm u \right\|^2$, in Equation~\ref{eq:pwrss} penalizes models with
larger magnitude values of $\bm u$.

In the so-called ``pseudo-data'' approach we write the penalized
weighted residual sum-of-squares as the squared length of a block
matrix equation,
\begin{equation}
  \label{eq:pwrssPseudoData}
  r^2(\bm\theta,\bm\beta,\bm u) =
  \left\|
  \begin{bmatrix}
  \bm W^{1/2}(\yobs - \bm o) \\
  \bm 0
  \end{bmatrix} -
  \begin{bmatrix}
  \bm W^{1/2}\bm Z\bm\bLt & \bm W^{1/2}\bm X \\
  \bm I_q & \bm 0 \\
  \end{bmatrix}
  \begin{bmatrix}
  \bm u \\
  \bm\beta \\
  \end{bmatrix}
  \right\|^2.
\end{equation}
This pseudo-data approach shows that the PLS problem may also be
thought of as a standard least squares problem for an extended
response vector, which implies that the minimizing value,
$\begin{bmatrix}
  \bm\mu_{\mc U|\mc Y = \yobs} \\
  \widehat{\bm\beta}_{\bm\theta} \\ \end{bmatrix}$, satisfies the
normal equations,
\begin{equation}
  \label{eq:normalEquations}
  \begin{bmatrix}
    \bLt\trans \bm Z\trans \bm W (\yobs - \bm o) \\
    \bm X\trans \bm W (\yobs - \bm o) \\
  \end{bmatrix} =
  \begin{bmatrix}
    \bLt\trans \bm Z\trans \bm W \bm Z \bLt + \bm I &
    \bLt\trans \bm Z\trans \bm W \bm X \\
    \bm X\trans \bm W \bm Z \bLt &
    \bm X\trans \bm W \bm X \\
  \end{bmatrix}
  \begin{bmatrix}
    \mu_{\mc U | \mc Y = \yobs} \\ \widehat{\betavec}_{\bm\theta}
  \end{bmatrix} ,
\end{equation} % |
where $\mu_{\mc U | \mc Y = \yobs}$ is the conditional mean of $\mc U$
given that $\mc Y = \yobs$.  Note that this conditional mean depends
on $\bm\theta$, although we do not make this dependency explicit in
order to reduce notational clutter.

The cross-product matrix in Equation~\ref{eq:normalEquations} can be
Cholesky decomposed,
\begin{equation}
  \label{eq:blockCholeskyDecomp}
  \begin{bmatrix}
    \bLt\trans \bm Z\trans \bm W \bm Z \bLt + \bm I &
    \bLt\trans \bm Z\trans \bm W \bm X \\
    \bm X\trans \bm W \bm Z \bLt &
    \bm X\trans \bm W \bm X \\
  \end{bmatrix} =
  \begin{bmatrix}
    \bm L_{\bm\theta} & \bm 0 \\
    \bm R\trans_{ZX} & \bm R\trans_{X} \\
  \end{bmatrix}
  \begin{bmatrix}
    \bm L\trans_{\bm\theta} & \bm R_{ZX} \\
    \bm 0 & \bm R_{X}
  \end{bmatrix} .
\end{equation}
We may use this decomposition to rewrite the penalized weighted
residual sum-of-squares as,
\begin{equation}
  \label{eq:pwrssIdentity}
  \begin{aligned}
  r^2(\bm\theta,\bm\beta,\bm u) = r^2(\bm\theta) &
  + \left\|\bm L\trans_{\bm\theta} (\bm u - \bm\mu_{\mc U|\mc Y = \yobs})
  +    \bm R_{ZX} (\bm\beta - \widehat{\bm\beta}_{\bm\theta})\right\|^2
  + \left\|\bm R_{X} (\bm\beta - \widehat{\bm\beta}_{\bm\theta})\right\|^2,
  \end{aligned}
\end{equation} % |
where we have simplified notation by writing
$r^2(\bm\theta,\widehat{\bm\beta}_{\bm\theta},\bm\mu_{\mc U|\mc Y =
  \yobs})$ as $r^2(\bm\theta)$. This is an important expression in the
theory underlying \pkg{lme4}. It relates the penalized weighted
residual sum-of-squares, $r^2(\bm\theta,\bm\beta,\bm u)$, with its
minimum value, $r^2(\bm\theta)$.  This relationship is useful in the
next section where we integrate over the random effects, which is
required for maximum likelihood estimation.

\subsection{Probability densities}
\label{sec:densities}

The residual sums-of-squares discussed in the previous section can be
used to express various probability densities, which are required for
maximum likelihood estimation of the linear mixed model\footnote{These
  expressions only technically hold at the observed value, $\yobs$, of
  the response vector, $\mc Y$.},
\begin{align}
  \label{eq:densityYgivenU}
  f_{\mc Y | \mc U}(\yobs|\bm u) & =
  \frac{|\bm W|^{1/2}}{(2\pi\sigma^2)^{n/2}}
  \exp\left[\frac{-\rho^2(\bm\theta,\bm\beta,\bm u)}{2\sigma^2}\right] ,\\
  \label{eq:densityU}
  f_{\mc U}(\bm u) &=
  \frac{1}{(2\pi\sigma^2)^{q/2}}
  \exp\left[\frac{-\left\| \bm u \right\|^2}{2\sigma^2}\right] ,\\
  \label{eq:densityYU}
  f_{\mc Y , \mc U}(\yobs,\bm u) & =
  \frac{|\bm W|^{1/2}}{(2\pi\sigma^2)^{(n+q)/2}}
  \exp\left[\frac{-r^2(\bm\theta,\bm\beta,\bm u)}{2\sigma^2}\right] , \\
  \label{eq:densityUgivenY}
  f_{\mc U | \mc Y}(\bm u | \yobs) & =
  \frac{f_{\mc Y , \mc U}(\yobs,\bm u)}{f_{\mc Y}(\yobs)} , \\
\end{align}
where
\begin{equation}
  \label{eq:densityY}
  f_{\mc Y}(\yobs) =
  \int f_{\mc Y , \mc U}(\yobs,\bm u)d\bm u .
\end{equation}
The log-likelihood to be maximized can therefore be expressed as,
\begin{equation}
  \label{eq:likelihood}
  \mc L(\bm\theta,\bm\beta,\sigma^2 | \yobs) = \log f_{\mc Y}(\yobs).
\end{equation} % |

The integral in Equation~\ref{eq:densityY} may be more explicitly written
as,
\begin{equation}
  \label{eq:densityYexpand}
  \begin{aligned}
  f_{\mc Y}(\yobs) =
  \frac{|\bm W|^{1/2}}{(2\pi\sigma^2)^{(n+q)/2}} &
  \exp\left[\frac{
  -r^2(\bm\theta)
  - \left\|\bm R_{X} (\bm\beta - \widehat{\bm\beta}_{\bm\theta})\right\|^2
  }{2\sigma^2}\right] \\ &
    \int \exp\left[\frac{
  - \left\|\bm L\trans_{\bm\theta} (\bm u - \bm\mu_{\mc U|\mc Y = \yobs})
  +    \bm R_{ZX} (\bm\beta - \widehat{\bm\beta}_{\bm\theta})\right\|^2
  }{2\sigma^2}\right]
   d\bm u ,
  \end{aligned}
\end{equation} % |
which can be evaluated with the change of variables,
\begin{equation}
  \label{eq:changeOfVariables}
  \bm v = \bm L\trans_{\bm\theta} (\bm u - \mu_{\mc U|\mc Y = \yobs})
  +    \bm R_{ZX} (\bm\beta - \widehat{\bm\beta}_{\bm\theta}) .
\end{equation} % |
The Jacobian determinant of the transformation from $\bm u$ to $\bm
v$ is $|\bm L_{\bm\theta} |$. Therefore we are able to write the integral
as,
\begin{equation}
  \label{eq:densityYintermediate}
  \begin{aligned}
  f_{\mc Y}(\yobs) =
  \frac{|\bm W|^{1/2}}{(2\pi\sigma^2)^{(n+q)/2}} &
  \exp\left[\frac{
  -r^2(\bm\theta)
  - \left\|\bm R_{X} (\bm\beta - \widehat{\bm\beta}_{\bm\theta})\right\|^2
  }{2\sigma^2}\right] \\ &
    \int \exp\left[\frac{
  - \left\|\bm v\right\|^2
  }{2\sigma^2}\right]
   |\bm L_{\bm\theta}|^{-1} d\bm v ,
  \end{aligned}
\end{equation}
which by the properties of exponential integrands becomes,
\begin{equation}
  \label{eq:densityYfinal}
   \exp\mc L(\bm\theta,\bm\beta,\sigma^2 | \yobs) = f_{\mc Y}(\yobs) =
  \frac{|\bm W|^{1/2}|\bm L_{\bm\theta}|^{-1}}{(2\pi\sigma^2)^{n/2}}
  \exp\left[\frac{
  -r^2(\bm\theta)
  - \left\|\bm R_{X} (\bm\beta - \widehat{\bm\beta}_{\bm\theta})\right\|^2
  }{2\sigma^2}\right] .
\end{equation}

\subsection{Evaluating and profiling the deviance and the REML criterion}
\label{sec:profdev}

We are now in a position to understand why the formulation in
Equations~\ref{eq:LMMcondY} and \ref{eq:LMMuncondB} is particularly
useful. We are able to explicitly profile $\betavec$ and $\sigma$ out
of the log-likelihood (Equation~\ref{eq:likelihood}), to find a compact
expression for the profiled deviance (negative twice the profiled
log-likelihood) and the profiled REML criterion as a function of the
relative covariance parameters, $\bm\theta$, only.  Furthermore these
criteria can be evaluated quickly and accurately.

To estimate the parameters, $\bm\theta$, $\bm\beta$, and $\sigma^2$, we
minimize negative twice the log-likelihood, which can be written as,
\begin{equation}
  \label{eq:fullDeviance}
  -2\mc L(\bm\theta,\bm\beta,\sigma^2 | \yobs) =    % |
  \log\frac{|\bm L_{\bm\theta}|^2}{|\bm W|}
  + n\log(2\pi\sigma^2)
  + \frac{r^2(\bm\theta)}{\sigma^2}
  + \frac{ \left\|\bm R_{X} (\bm\beta -
    \widehat{\bm\beta}_{\bm\theta})\right\|^2}{\sigma^2} .
\end{equation}
It is very easy to profile out $\bm\beta$, because it enters into
the ML criterion only through the final term, which is zero if
$\bm\beta = \widehat{\bm\beta}_{\bm\theta}$, where
$\widehat{\bm\beta}_{\bm\theta}$ is found by solving the penalized
least-squares problem in Equation~\ref{eq:pwrssPseudoData}. Therefore we
can write a partially profiled ML criterion as,
\begin{equation}
  \label{eq:partiallyProfiledDeviance}
  -2\mc L(\bm\theta,\sigma^2 | \yobs)  =    % |
  \log\frac{|\bm L_{\bm\theta}|^2}{|\bm W|}
  + n\log(2\pi\sigma^2)
  + \frac{r^2(\bm\theta)}{\sigma^2} .
\end{equation}
This criterion is only partially profiled because it still depends on
$\sigma^2$. Differentiating this criterion with respect to $\sigma^2$
and setting the result equal to zero yields,
\begin{equation}
  \label{eq:residualVarianceEstimatingEquation}
  0 = \frac{n}{\widehat{\sigma}_{\bm\theta}^2} -
  \frac{r^2(\bm\theta)}{\widehat{\sigma}_{\bm\theta}^4},
\end{equation}
which leads to a maximum profiled likelihood estimate,
\begin{equation}
  \label{eq:residualVarianceEstimate}
  \widehat{\sigma}_{\bm\theta}^2 = \frac{r^2(\bm\theta)}{n}.
\end{equation}
This estimate can be substituted into the partially profiled criterion to yield the
fully profiled ML criterion,
\begin{equation}
  \label{eq:profiledDeviance}
  -2\mc L(\bm\theta | \yobs)  =           % |
  \log\frac{|\bm L_{\bm\theta}|^2}{|\bm W|}
  + n\left[1 + \log\left(
  \frac{2\pi r^2(\bm\theta)}{n}
  \right)\right] .
\end{equation}
This expression for the profiled deviance depends only on
$\bm\theta$. Although $q$, the number of columns in $\bm Z$ and the
size of $\bm\Sigma_{\bm\theta}$, can be very large indeed, the
dimension of $\bm\theta$ is small, frequently less than 10. The
\pkg{lme4} package uses generic nonlinear optimizers
(Section~\ref{sec:optimizeLmer}) to optimize this expression over
$\bm\theta$ to find its maximum likelihood estimate.

\subsubsection{The REML criterion}
\label{sec:reml}

The REML criterion can be obtained by integrating the marginal density
for $\mc Y$ with respect to the fixed effects \citep{laird_ware_1982},
\begin{equation}
  \label{eq:REMLintegral}
  \int f_{\mc Y}(\yobs)d\bm\beta =
  \frac{|\bm W|^{1/2}|\bm L_{\bm\theta}|^{-1}}{(2\pi\sigma^2)^{n/2}}
  \exp\left[\frac{-r^2(\bm\theta)}{2\sigma^2}\right]
  \int \exp\left[\frac{
  - \left\|\bm R_{X} (\bm\beta - \widehat{\bm\beta}_{\bm\theta})\right\|^2
  }{2\sigma^2}\right]d\bm\beta ,
\end{equation}
which can be evaluated with the change of variables,
\begin{equation}
  \label{eq:REMLchangeOfVariables}
  \bm v = \bm R_{X} (\bm\beta - \widehat{\bm\beta}_{\bm\theta}) .
\end{equation}
The Jacobian determinant of the transformation from $\bm\beta$ to
$\bm v$ is $|\bm R_X |$. Therefore we are able to write the
integral as,
\begin{equation}
  \label{eq:REMLintegralIntermediate}
  \int f_{\mc Y}(\yobs)d\bm\beta =
  \frac{|\bm W|^{1/2}|\bm L_{\bm\theta}|^{-1}}{(2\pi\sigma^2)^{n/2}}
  \exp\left[\frac{-r^2(\bm\theta)}{2\sigma^2}\right]
  \int \exp\left[\frac{
  - \left\|\bm v\right\|^2
  }{2\sigma^2}\right]|\bm R_X|^{-1} d\bm v ,
\end{equation}
which simplifies to,
\begin{equation}
  \label{eq:REMLintermediate}
  \int f_{\mc Y}(\yobs)d\bm\beta =
  \frac{|\bm W|^{1/2}|\bm L_{\bm\theta}|^{-1}|\bm R_X|^{-1}}{(2\pi\sigma^2)^{(n-p)/2}}
  \exp\left[\frac{-r^2(\bm\theta)}{2\sigma^2}\right] .
\end{equation}
Minus twice the log of this integral is the (unprofiled) REML criterion,
\begin{equation}
  \label{eq:REMLdeviance}
  -2\mc L_R(\bm\theta,\sigma^2 | \yobs) =
  \log\frac{|\bm L_{\bm\theta}|^2|\bm R_X|^2}{|\bm W|}
  + (n-p)\log(2\pi\sigma^2)
  + \frac{r^2(\bm\theta)}{\sigma^2} .
\end{equation}
Note that because $\bm\beta$ gets integrated out, the REML criterion
cannot be used to find a point estimate of $\bm\beta$. However, we
follow others in using the maximum likelihood estimate,
$\widehat{\bm\beta}_{\widehat{\bm\theta}}$, at the optimum value of
$\bm\theta = \widehat{\bm\theta}$. The REML estimate of $\sigma^2$ is,
\begin{equation}
  \label{eq:REMLresidualVarianceEstimate}
  \widehat{\sigma}_{\bm\theta}^2 = \frac{r^2(\bm\theta)}{n-p} ,
\end{equation}
which leads to the profiled REML criterion,
\begin{equation}
  \label{eq:REMLprofiled}
  -2\mc L_R(\bm\theta | \yobs) =
  \log\frac{|\bm L_{\bm\theta}|^2|\bm R_X|^2}{|\bm W|}
  + (n-p)\left[1 + \log\left(\frac{2\pi
        r^2(\bm\theta)}{n-p}\right)\right] .
\end{equation}

\subsection{Changes relative to previous formulations}
\label{sec:previous_lmm_form}

We compare the PLS problem as formulated in Section~\ref{sec:plsMath}
with earlier versions and describe why we use this version.  What have
become known as ``Henderson's mixed-model equations'' are given as
Equation 6 of \citet{henderson_1982} and would be expressed as,
\begin{equation}
  \label{eq:henderson}
  \begin{bmatrix}
    \bm X\trans\bm X/\sigma^2& \bm X\trans\bm Z/\sigma^2\\
    \bm Z\trans\bm X/\sigma^2& \bm Z\trans\bm Z/\sigma^2 + \bm\Sigma^{-1}
  \end{bmatrix}
  \begin{bmatrix}
    \widehat{\bm\beta}_\theta \\ \bm\mu_{\mc B|\mc Y=\yobs}
  \end{bmatrix}=
  \begin{bmatrix}\bm X\trans\yobs/\sigma^2\\\bm Z\trans\yobs/\sigma^2
  \end{bmatrix} ,
\end{equation}
in our notation (ignoring weights and offsets, without loss of
generality).  The matrix written as $\bm R$ in \citet{henderson_1982}
is $\sigma^2\bm I_n$ in our formulation of the model.

\citet{bates04:_linear} modified the PLS equations to
\begin{equation}
  \label{eq:batesDebRoy}
  \begin{bmatrix}
    \bm Z\trans\bm Z+\bm\Omega&\bm Z\trans\bm X\\
    \bm X\trans\bm Z&\bm X\trans\bm X
  \end{bmatrix}
  \begin{bmatrix}
     \bm\mu_{\mc B|\mc Y=\yobs}\\\widehat{\bm\beta}_\theta
  \end{bmatrix}=
  \begin{bmatrix}\bm Z\trans\yobs\\\bm X\trans\yobs\end{bmatrix} \quad .
\end{equation}
where
$\bm\Omega_\theta=\left(\bLt\trans\bLt\right)^{-1}=\sigma^2\bm\Sigma^{-1}$
is the \emph{relative precision matrix} for a given value of
$\bm\theta$.  They also showed that the profiled log-likelihood can be
expressed (on the deviance scale) as
\begin{equation}
  \label{eq:batesDebRoyprofdev}
  -2\mc L(\bm\theta)=
  \log\left(\frac{|\bm Z\trans\bm Z+\bm\Omega|}{|\bm\Omega|}\right) +
  n\left[1+\log\left(\frac{2\pi r^2_\theta}{n}\right)\right] .
\end{equation}

The primary difference between Equation~\ref{eq:henderson} and
Equation~\ref{eq:batesDebRoy} is the order of the blocks in the system matrix.
The PLS problem can be solved using a Cholesky factor of the system
matrix with the blocks in either order.  The advantage of using the
arrangement in Equation~\ref{eq:batesDebRoy} is to allow for evaluation of
the profiled log-likelihood.  To evaluate $|\bm Z\trans\bm
Z+\bm\Omega|$ from the Cholesky factor that block must be in the
upper-left corner, not the lower right.  Also, $\bm Z$ is sparse whereas $\bm
X$ is usually dense.  It is straightforward to exploit the sparsity of
$\bm Z\trans\bm Z$ in the Cholesky factorization when the block containing
this matrix is the first block to be factored.  If $\bm X\trans\bm X$ is the first block to be factored, it is much more difficult to
preserve sparsity.

The main change from the formulation in \citet{bates04:_linear} to the
current formulation is the use of a relative covariance factor,
$\bLt$, instead of a relative precision matrix, $\bm\Omega_\theta$,
and solving for the mean of $\mc U|\mc Y = \yobs$ instead of the mean
of $\mc B|\mc Y = \yobs$.  This change improves stability, because the
solution to the PLS problem in Section~\ref{sec:plsMath} is
well-defined when $\bLt$ is singular whereas the formulation in
Equation~\ref{eq:batesDebRoy} cannot be used in these cases because
$\bm\Omega_\theta$ does not exist.

It is important to allow for $\bLt$ to be singular because situations
where the parameter estimates, $\widehat{\bm\theta}$, produce a
singular $\bm\Lambda_{\widehat{\theta}}$ do occur in practice.  And
even if the parameter estimates do not correspond to a singular
$\bm\Lambda_\theta$, it may be desirable to evaluate the estimation
criterion at such values during the course of the numerical
optimization of the criterion.

\citet{bates04:_linear} also provided expressions for the gradient of
the profiled log-likelihood expressed as
Equation~\ref{eq:batesDebRoyprofdev}.  These expressions can be translated
into the current formulation.  From Equation~\ref{eq:profiledDeviance} we can see
that (again ignoring weights),
\begin{equation}
  \label{eq:grad}
  \begin{aligned}
    \nabla\left(-2\mc L(\theta)\right)
    &=\nabla\log(|\bm L_\theta|^2)+\nabla \left(n\log(r^2(\bm\theta))\right)\\
    &=\nabla\log(|\bLt\trans\bm Z\trans\bm Z\bLt+\bm I|)+
    n\left(\nabla r^2(\bm\theta)\right)/r^2(\bm\theta)\\
    &=\nabla\log(|\bLt\trans\bm Z\trans\bm Z\bLt+\bm I|)+
    \left(\nabla r^2(\bm\theta)\right)/(\widehat{\sigma^2}).
  \end{aligned}
\end{equation}
The first gradient is easy to express but difficult to evaluate for
the general model.  The individual elements of this gradient are
\begin{equation}
  \label{eq:graddet}
  \begin{aligned}
    \frac{\partial\log(|\bLt\trans\bm Z\trans\bm Z\bLt+\bm
      I|)}{\partial \theta_i} &=
    \tr\left[\frac{\partial\left(\bLt\trans\bm Z\trans\bm
          Z\bLt\right)}{\partial\theta_i}
      \left(\bLt\trans\bm Z\trans\bm Z\bLt+\bm I\right)^{-1}\right]\\
    &=\tr\left[
      \left(\bm L_\theta\bm L_\theta\trans\right)^{-1}
      \left(
        \bLt\trans\bm Z\trans\bm Z\frac{\partial\bLt}{\partial\theta_i} +
        \frac{\partial\bLt\trans}{\partial\theta_i}\bm Z\trans\bm Z\bLt
      \right)
    \right] .
  \end{aligned}
\end{equation}
The second gradient term can be expressed as a linear function of the
residual, with individual elements of the form
\begin{equation}
  \label{eq:gradr2}
  \frac{\partial r^2(\bm\theta)}{\partial\theta_i}=
  -2\bm u\trans\frac{\partial\bLt\trans}{\partial\theta_i}\bm
  Z\trans\left(\bm y-\bm Z\bLt\bm u-\bm X\widehat{\bm\beta}_{\bm\theta
   }\right),
\end{equation}
using the results of \citet{golub_pereyra_1973}. Although we do not
use these results in \pkg{lme4}, they are used for certain model types
in the \pkg{MixedModels} package for \proglang{Julia} and do provide
improved performance.


\subsection{Penalized least squares algorithm}
\label{sec:PLSpureR}

For efficiency, in \pkg{lme4} itself, PLS is implemented in compiled
\proglang{C++} code using the \pkg{Eigen} \citep{Eigen} templated \proglang{C++}
package for numerical linear algebra.  Here however, in order to
improve readability we describe a version in pure \proglang{R}.
Section~\ref{sec:CSCmats} provides greater detail on the techniques
and concepts for computational efficiency, which is important in cases
where the nonlinear optimizer (Section~\ref{sec:optimizeLmer})
requires many iterations.

The PLS algorithm takes a vector of covariance parameters,
$\bm\theta$, as inputs and returns the profiled deviance (Equation
\ref{eq:profiledDeviance}) or the REML criterion
(Equation \ref{eq:REMLprofiled}). This PLS algorithm consists of four main
steps:
\begin{enumerate}
\item Update the relative covariance factor (Section~\ref{sec:plsI}).
\item Solve the normal equations (Section~\ref{sec:plsII}).
\item Update the linear predictor and residuals (Section~\ref{sec:plsIII}).
\item Compute and return the profiled deviance (Section~\ref{sec:plsIV}).
\end{enumerate}
PLS also requires the objects described in Table~\ref{tab:inputsPLS},
which define the structure of the model.  These objects do not get
updated during the PLS iterations, and so it is useful to store
various matrix products involving them (Table~\ref{tab:constantPLS}).
Table~\ref{tab:updatePLS} lists the objects that do get updated over
the PLS iterations.  The symbols in this table correspond to a version
of \pkg{lme4} that is implemented entirely in \proglang{R} (i.e., no
compiled code as in \pkg{lme4} itself).  This implementation is called
\pkg{lme4pureR} and is currently available on \github\
(\url{https://github.com/lme4/lme4pureR/}).

\begin{table}[tb]
  \centering
  \begin{minipage}{\textwidth}
  \begin{tabular}{p{1.8in}llp{1.6in}}
    \hline
    Name/description & Pseudocode & Math & Type \\
    \hline
    Mapping from covariance parameters to relative covariance factor &
    \code{mapping} &  & function \\
    Response vector & \code{y} & $\yobs$ (Section~\ref{sec:LMMs}) & double vector \\
    Fixed-effects model matrix & \code{X} & $\bm X$ (Equation~\ref{eq:LMMcondY}) & double dense\footnote{In previous versions of \pkg{lme4} a sparse $X$ matrix, useful for models with categorical fixed-effect predictors with many levels, could be specified; this feature is not currently available.}
    matrix \\
    Transposed random-effects model matrix & \code{Zt} & $\bm Z\trans$
    (Equation~\ref{eq:LMMcondY}) &
    double sparse matrix \\
    Square-root weights matrix & \code{sqrtW} & $\bm W^{1/2}$
    (Equation~\ref{eq:LMMcondY}) & double diagonal matrix \\
    Offset & \code{offset} & $\bm o$ (Equation~\ref{eq:LMMcondY}) & double vector \\
    \hline
  \end{tabular}
  \renewcommand{\footnoterule}{%
    \kern -3pt
    \hrule width \textwidth height 0pt
    \kern 2pt
  }
  \end{minipage}
  \caption{Inputs into a linear mixed model.}
  \label{tab:inputsPLS}
\end{table}

\begin{table}[tb]
  \centering
  \begin{tabular}{ll}
    \hline
    Pseudocode & Math \\
    \hline
    \code{ZtW} & $\bm Z\trans\bm W^{1/2}$ \\
    \code{ZtWy} & $\bm Z\trans\bm W \yobs$ \\
    \code{ZtWX} & $\bm Z\trans\bm W \bm X$ \\
    \code{XtWX} & $\bm X\trans\bm W \bm X$ \\
    \code{XtWy} & $\bm X\trans\bm W \yobs$ \\
    \hline
  \end{tabular}
  \caption{Constant symbols involved in penalized least squares.}
  \label{tab:constantPLS}
\end{table}


\begin{table}[tb]
  \centering
  \begin{tabular}{p{1.7in}llp{1.5in}}
    \hline
    Name/description & Pseudocode & Math & Type \\
    \hline
    Relative covariance factor & \code{lambda} & $\bLt$
    (Equation~\ref{eq:relcovfac})
    & double sparse lower-triangular matrix  \\
    Random-effects Cholesky factor & \code{L} & $\bm L_{\bm\theta}$
    (Equation~\ref{eq:blockCholeskyDecomp})
    & double sparse triangular matrix \\
    Intermediate vector in the solution of the normal equations &
    \code{cu} & $\bm c_u$ (Equation~\ref{eq:cu}) & double vector \\
    Block in the full Cholesky factor & \code{RZX} & $\bm R_{ZX}$
    (Equation~\ref{eq:blockCholeskyDecomp})  & double dense matrix \\
    Cross-product of the fixed-effects Cholesky factor & \code{RXtRX}
    & $\bm R_X\trans\bm R_X$ (Equation~\ref{eq:RX}) & double dense matrix \\
    Fixed-effects coefficients & \code{beta} & $\bm\beta$
    (Equation~\ref{eq:LMMcondY}) & double vector \\
    Spherical conditional modes & \code{u} & $\bm u$ (Section~\ref{sec:stability}) & double vector \\
    Non-spherical conditional modes & \code{b}
    & $\bm b$ (Equation~\ref{eq:LMMcondY}) & double vector \\
    Linear predictor & \code{mu} & $\bm\mu_{\mc Y|\mc U=\bm u}$
    (Equation~\ref{eq:meanYgivenU}) & double vector \\
    Weighted residuals & \code{wtres} & $\bm W^{1/2} (\yobs - \mu)$ &
    double vector \\
    Penalized weighted residual sum-of-squares & \code{pwrss} &
    $r^2(\bm\theta)$ (Equation~\ref{eq:pwrssIdentity}) &
    double \\
    Twice the log determinant random-effects Cholesky factor  &
    \code{logDet}
    & $\log|\bm L_{\bm\theta}|^2$ & double \\
    \hline
  \end{tabular}
  \caption{Quantities updated during an evaluation of the linear mixed model
    objective function.}
  \label{tab:updatePLS}
\end{table}


\subsubsection{PLS step I: Update relative covariance factor}
\label{sec:plsI}

The first step of PLS is to update the relative covariance factor,
$\bLt$, from the current value of the covariance parameter vector,
$\bm\theta$.  The updated $\bLt$ is then used to update the random-effects
Cholesky factor, $\bm L_{\bm\theta}$
(Equation~\ref{eq:blockCholeskyDecomp}).  The mapping from the covariance
parameters to the relative covariance factor can take many forms, but
in general involves a function that takes $\bm\theta$ into the values
of the nonzero elements of $\bLt$.

If $\bLt$ is stored as an object of class `\code{dgCMatrix}' from the
\pkg{Matrix} package for \proglang{R}, then we may update $\bLt$ from
$\bm\theta$ by,
<<PLSupdateTheta, eval = FALSE>>=
Lambdat@x[] <- mapping(theta)
@
where \code{mapping} is an \proglang{R} function that both accepts and
returns a numeric vector. The nonzero elements of sparse matrix
classes in \pkg{Matrix} are stored in a slot called \code{x}.

In the current version of \pkg{lme4} (v. 1.1-7) the mapping from
$\bm\theta$ to $\bLt$ is represented as an \proglang{R} integer
vector, \code{Lind}, of indices, so that
<<PLSupdateThetaWithLind>>=
mapping <- function(theta) theta[Lind]
@
The index vector \code{Lind} is computed during the model setup and
stored in the function's environment.  Continuing the example from
Section~\ref{sec:mkLambdat}, consider a new value for \code{theta},
<<exampleNewTheta>>=
thetanew <- c(1, -0.1, 2, 0.1, -0.2, 3)
@
To put these values in the appropriate elements in \code{Lambdati},
we use \code{mapping},
<<exampleLindUpdate>>=
Lambdati@x[] <- mapping(thetanew)
Lambdati
@ %
This \code{Lind} approach can be useful for extending the capabilities
of \pkg{lme4} by using the modular approach to fitting mixed models.
For example, Appendix~\ref{sec:zeroSlopeSlope} shows how to use
\code{Lind} to fit a model where two random slopes are uncorrelated,
but both slopes are correlated with an intercept.

The mapping from the covariance parameters to the relative covariance
factor is treated differently in other implementations of the
\code{lme4} approach to linear mixed models.  At the other extreme, the
\code{flexLambda} branch of \pkg{lme4} and the \pkg{lme4pureR} package
provide the capabilities for a completely general mapping.  This added
flexibility has the advantage of allowing a much wider variety of
models (e.g., compound symmetry, auto-regression).  However, the
disadvantage of this approach is that it becomes possible to fit a
much wider variety of ill-posed models.  Finally, if one would like
such added flexibility with the current stable version of \pkg{lme4},
it is always possible to use the modular approach to wrap the
\code{Lind}-based deviance function in a general mapping function
taking a parameter to be optimized, say $\phi$, into $\bm\theta$.
However, this approach is likely to be inefficient in many cases.

The \code{update} method from the \pkg{Matrix} package efficiently
updates the random-effects Cholesky factor, $\bm L_{\bm\theta}$, from a
new value of $\bm\theta$ and the updated $\bLt$.
<<updateL, eval=FALSE>>=
L <- update(L, Lambdat %*% ZtW, mult = 1)
@
The \code{mult = 1} argument corresponds to the addition of the
identity matrix to the upper-left block on the left-hand-side of
Equation~\ref{eq:blockCholeskyDecomp}.

\subsubsection{PLS step II:  solve normal equations}
\label{sec:plsII}

With the new covariance parameters installed in $\bLt$, the next step
is to solve the normal equations (Equation~\ref{eq:normalEquations}) for the
current estimate of the fixed-effects coefficients,
$\widehat{\betavec}_{\bm\theta}$, and the conditional mode, $\mu_{\mc
  U|\mc Y = \yobs}$. We solve these equations using a sparse Cholesky
factorization (Equation~\ref{eq:blockCholeskyDecomp}).  In a complex model
fit to a large data set, the dominant calculation in the evaluation of
the profiled deviance (Equation~\ref{eq:profiledDeviance}) or REML criterion
(Equation~\ref{eq:REMLprofiled}) is this sparse Cholesky factorization
(Equation~\ref{eq:blockCholeskyDecomp}).  The factorization is performed in two
phases; a symbolic phase and a numeric phase.  The symbolic phase, in
which the fill-reducing permutation $\bm P$ is determined along with
the positions of the nonzeros in $\bm L_{\bm\theta}$, does not depend
on the value of $\bm\theta$.  It only depends on the positions of the
nonzeros in $\bm Z\bLt$.  The numeric phase uses $\bm\theta$ to
determine the numeric values of the nonzeros in $\bm
L_{\bm\theta}$. Using this factorization, the solution proceeds by the
following steps,
\begin{align}
    \bm L_{\bm\theta}\bm c_u&=\bm P\bLt\trans\bm Z\trans\bm W\bm y\label{eq:cu}\\
    \bm L_{\bm\theta}\bm R_{ZX}&=\bm P\bLt\trans\bm Z\trans\bm W\bm X\label{eq:RZX}\\
    \bm R_X\trans\bm R_X&=\bm X\trans\bm W\bm X-\bm R_{ZX}\trans\bm R_{ZX}\label{eq:RX}\\
    \left(\bm R_X\trans\bm R_X\right)\widehat{\bm\beta}_{\bm\theta}&=
      \bm X\trans\bm W\bm y-\bm R_{ZX}\bm c_u\label{eq:betahat}\\
    \bm L\trans\bm_{\bm\theta} \bm P\bm u&=\bm c_u-\bm R_{ZX}\widehat{\bm\beta}_{\bm\theta}\label{eq:tildeu}
\end{align}
which can be solved using the \pkg{Matrix} package with,
<<PLSsolve, eval = FALSE>>=
cu[] <- as.vector(solve(L, solve(L, Lambdat %*% ZtWy,
                                 system = "P"), system = "L"))
RZX[] <- as.vector(solve(L, solve(L, Lambdat %*% ZtWX,
                                  system = "P"), system = "L"))
RXtRX <- as(XtWX - crossprod(RZX), "dpoMatrix")
beta[] <- as.vector(solve(RXtRX, XtWy - crossprod(RZX, cu)))
u[] <- as.vector(solve(L, solve(L, cu - RZX %*% beta,
                                system = "Lt"), system = "Pt"))
@
Notice the nested calls to \code{solve}.  The inner calls of the first
two assignments determine and apply the permutation matrix
(\code{system = "P"}), whereas the outer calls actually solve the
equation (\code{system = "L"}). In the assignment to \code{u[]}, the
nesting is reversed in order to return to the original permutation.

\subsubsection{PLS step III: Update linear predictor and residuals}
\label{sec:plsIII}

The next step is to compute the linear predictor, $\mu_{\mc Y| \mc U}$
(Equation \ref{eq:meanYgivenU}), and the weighted residuals with new values for
$\widehat{\betavec}_\theta$ and $\mu_{\mc B | \mc Y = \yobs}$.  In
\pkg{lme4pureR} these quantities can be computed as,
<<PLSupdateResp, eval = FALSE>>=
b[] <- as.vector(crossprod(Lambdat, u))
mu[] <- as.vector(crossprod(Zt, b) + X %*% beta + offset)
wtres <- sqrtW * (y - mu)
@
where \code{b} represents the current estimate of $\mu_{\mc B | \mc Y
  = \yobs}$.

\subsubsection{PLS step IV: Compute profiled deviance}
\label{sec:plsIV}

Finally, the updated linear predictor and weighted residuals can be
used to compute the profiled deviance (or REML criterion), which in
\pkg{lme4pureR} proceeds as,
<<PLScalculateObjective, eval = FALSE>>=
pwrss <- sum(wtres^2) + sum(u^2)
logDet <- 2*determinant(L, logarithm = TRUE)$modulus
if (REML) logDet <- logDet + determinant(RXtRX,
                                         logarithm = TRUE)$modulus
attributes(logDet) <- NULL
profDev <- logDet + degFree * (1 + log(2 * pi * pwrss) - log(degFree))
@
The profiled deviance consists of three components: (1)
log-determinant(s) of Cholesky factorization (\code{logDet}), (2) the
degrees of freedom (\code{degFree}), and the penalized weighted
residual sum-of-squares (\code{pwrss}).

\subsection{Sparse matrix methods}
\label{sec:CSCmats}

In fitting linear mixed models, an instance of the PLS problem
(\ref{eq:normalEquations}) must be solved at each evaluation of the
objective function during the optimization
(Section~\ref{sec:optimizeLmer}) with respect to $\bm\theta$.  Because
this operation must be performed many times it is worthwhile
considering how to provide effective evaluation methods for objects
and calculations involving the sparse matrices associated with
random-effects terms (Sections~\ref{sec:LMMmatrix}).

The \pkg{CHOLMOD} library of \proglang{C} functions
\citep{Chen:2008:ACS:1391989.1391995}, on which the sparse matrix
capabilities of the \pkg{Matrix} package for \proglang{R} and the
sparse Cholesky factorization in \proglang{Julia} are based, allows
for separation of the symbolic and numeric phases.  Thus we perform
the symbolic phase as part of establishing the structure representing
the model (Section~\ref{sec:lFormula}).  Furthermore, because \pkg{CHOLMOD}
functions allow for updating $\bm L_{\bm\theta}$ directly from the
matrix $\bLt\trans\bm Z\trans$ without actually forming $\bLt\trans\bm
Z\trans\bm Z\bLt+\bm I$ we generate and store $\bm Z\trans$ instead of
$\bm Z$ (note that we have ignored the weights matrix, $\bm W$, for
simplicity).  We can update $\bLt\trans\bm Z\trans$ directly from
$\bm\theta$ without forming $\bLt$ and multiplying two sparse
matrices.  Although such a direct approach is used in the
\pkg{MixedModels} package for \proglang{Julia}, in \pkg{lme4} we first
update $\bLt\trans$ then form the sparse product $\bLt\trans\bm
Z\trans$.  A third alternative, which we employ in \pkg{lme4pureR}, is
to compute and save the cross-products of the model matrices, $\bm X$
and $\bm Z$, and the response, $\bm y$, before starting the
iterations. To allow for case weights, we save the products $\bm
X\trans\bm W\bm X$, $\bm X\trans\bm W\bm y$, $\bm Z\trans\bm W\bm X$,
$\bm Z\trans\bm W\bm y$ and $\bm Z\trans\bm W\bm Z$ (see
Table~\ref{tab:constantPLS}).

We wish to use structures and algorithms that allow us to take a new
value of $\bm\theta$ and evaluate the $\bm L_\theta$
(Equation~\ref{eq:blockCholeskyDecomp}) efficiently.  The key to doing so
is the special structure of $\bLt\trans\bm Z\trans\bm W^{1/2}$.  To
understand why this matrix, and not its transpose, is of interest we
describe the sparse matrix structures used in \proglang{Julia} and in
the \pkg{Matrix} package for \proglang{R}.

Dense matrices are stored in \proglang{R} and in \proglang{Julia} as a
one-dimensional array of contiguous storage locations addressed in
\emph{column-major order}.  This means that elements in the same
column are in adjacent storage locations whereas elements in the same
row can be widely separated in memory.  For this reason, algorithms
that work column-wise are preferred to those that work row-wise.

Although a sparse matrix can be stored in a \emph{triplet} format,
where the row position, column position and element value of the nonzeros
are recorded as triplets, the preferred storage forms for actual
computation with sparse matrices are compressed sparse column (CSC) or
compressed sparse row \citep[CSR,][Chapter~2]{davis06:csparse_book}. The
\pkg{CHOLMOD} (and, more generally, the \pkg{SuiteSparse} package of \proglang{C} libraries; \citealt{SuiteSparse})
uses the CSC storage format.  In this format the nonzero elements in
a column are in adjacent storage locations and access to all the
elements in a column is much easier than access to those in a row
(similar to dense matrices stored in column-major order).

The matrices $\bm Z$ and $\bm Z\bLt$ have the property that the number
of nonzeros in each row, $\sum_{i=1}^k p_i$, is constant.  For CSC
matrices we want consistency in the columns rather than the rows,
which is why we work with $\bm Z\trans$ and $\bLt\trans\bm Z\trans$
rather than their transposes.

An arbitrary $m\times n$ sparse matrix in CSC format is expressed as
two vectors of indices, the row indices and the column pointers, and a
numeric vector of the nonzero values.  The elements of the row
indices and the nonzeros are aligned and are ordered first by
increasing column number then by increasing row number within column.
The column pointers are a vector of size $n+1$ giving the location of
the start of each column in the vectors of row indices and nonzeros.

%% This para is __extra__ in comparison to JSS (2015):
Because the number of nonzeros in each column of $\bm Z\trans$, and in
each column of matrices derived from $\bm Z\trans$ (such as
$\bLt\trans\bm Z\trans\bm W^{1/2}$) is constant, the vector of
nonzeros in the CSC format can be viewed as a dense matrix, say $\bm
N$, of size $\left(\sum_{i=1}^n p_i\right)\times n$.  We do not need
to store the column pointers because the columns of $\bm Z\trans$
correspond to columns of $\bm N$.  All we need is $\bm N$, the dense
matrix of nonzeros, and the row indices, which are derived from the
grouping factor index vectors $\bm i_i,i=1,\dots,k$ and can also be
arranged as a dense matrix of size $\left(\sum_{i=1}^n
  p_i\right)\times n$.  Matrices like $\bm Z\trans$, with the property
that there are the same number of nonzeros in each column, are
sometimes called \emph{regular sparse column-oriented} (RSC) matrices.

\section{Nonlinear optimization module}
\label{sec:optimizeLmer}

The objective function module produces a function which implements the
penalized least squares algorithm for a particular mixed model. The
next step is to optimize this function over the covariance parameters,
$\bm\theta$, which is a nonlinear optimization problem.  The
\pkg{lme4} package separates the efficient computational linear
algebra required to compute profiled likelihoods and deviances for a
given value of $\bm\theta$ from the nonlinear optimization algorithms,
which use general-purpose nonlinear optimizers.

One benefit of this separation is that it allows for experimentation
with different nonlinear optimizers.  Throughout the development of
\pkg{lme4}, the default optimizers and control parameters have changed
in response to feedback from users about both efficiency and
convergence properties.  \pkg{lme4} incorporates two built-in
optimization choices, an implementation of the Nelder-Mead simplex
algorithm adapted from Steven Johnson's \pkg{NLopt} \proglang{C} library
\citep{NLopt} and a wrapper for Powell's BOBYQA algorithm, implemented
in the \pkg{minqa} package \citep{minqa_pkg} as a wrapper around
Powell's original \proglang{Fortran} code \citep{Powell_bobyqa}.  More generally,
\pkg{lme4} allows any user-specified optimizer that (1) can work with
an objective function (i.e., does not require a gradient function to
be specified), (2) allows box constraints on the parameters, and (3)
conforms to some simple rules about argument names and structure of
the output. An internal wrapper allows the use of the \pkg{optimx}
package \citep{optimx_pkg}, although the only optimizers provided via
\pkg{optimx} that satisfy the constraints above are the \code{nlminb}
and \code{L-BFGS-B} algorithms that are themselves wrappers around the
versions provided in base \proglang{R}.  Several other algorithms from Steven
Johnson's \pkg{NLopt} package are also available via the \pkg{nloptr}
wrapper package (e.g., alternate implementations of Nelder-Mead and
BOBYQA, and Powell's COBYLA algorithm).

This flexibility assists with diagnosing convergence problems -- it
is easy to switch among several algorithms to determine whether the
problem lies in a failure of the nonlinear optimization stage, as
opposed to a case of model misspecification or unidentifiability or a
problem with the underlying PLS algorithm.  To date we
have only observed PLS failures, which arise if
$\bm X\trans\bm W\bm X-\bm R_{ZX}\trans\bm R_{ZX}$ becomes singular
during an evaluation of the objective function, with badly scaled
problems (i.e., problems with continuous predictors that take a
very large or very small numerical range of values).

The requirement for optimizers that can handle box constraints stems
from our decision to parameterize the variance-covariance matrix in a
constrained space, in order to allow for singular fits. In contrast to
the approach taken in the \pkg{nlme} package \citep{nlme_pkg}, which goes to some
lengths to use an unconstrained variance-covariance parameterization
(the \emph{log-Cholesky} parameterization;
\citealt{pinheiro_unconstrained_1996}), we instead use the Cholesky
parameterization but require the elements of $\bm\theta$ corresponding
to the diagonal elements of the Cholesky factor to be non-negative.
With these constraints, the variance-covariance matrix is singular if
and only if any of the diagonal elements is exactly zero.
Singular fits are common in practical data-analysis situations,
especially with small- to medium-sized data sets and complex
variance-covariance models, so being able to fit a singular model is
an advantage: when the best fitting model lies on the boundary of a
constrained space, approaches that try to remove the constraints by
fitting parameters on a transformed scale will often give rise to
convergence warnings as the algorithm tries to find a maximum on an
asymptotically flat surface \citep{bolker_strategies_2013}.

In principle the likelihood surfaces we are trying to optimize over
are smooth, but in practice using gradient information in optimization
may or may not be worth the effort. In special cases, we have a
closed-form solution for the gradients
(Equations~\ref{eq:grad}--\ref{eq:gradr2}), but in general we will have to
approximate them by finite differences, which is expensive and has
limited accuracy.  (We have considered using automatic
differentiation approaches to compute the gradients more efficiently,
but this strategy requires a great deal of additional machinery, and
would have drawbacks in terms of memory requirements for large
problems.)  This is the primary reason for our switching to
derivative-free optimizers such as BOBYQA and Nelder-Mead in the
current version of \pkg{lme4}, although as discussed above
derivative-based optimizers based on finite differencing are available
as an alternative.

There is most likely further room for improvement in the nonlinear
optimization module; for example, some speed-up could be gained by
using parallel implementations of derivative-free optimizers that
evaluated several trial points at once \citep{klein_nelder_2013}.  In
practice users most often have optimization difficulties with poorly
scaled or centered data -- we are working to implement appropriate
diagnostic tests and warnings to detect these situations.

\section{Output module}
\label{sec:mkMerMod}

Here we describe some of the methods in \pkg{lme4} for exploring
fitted linear mixed models (Table~\ref{tab:methods}), which are
represented as objects of the \proglang{S}4 class `\code{lmerMod}'.  We
begin by describing the theory underlying these methods
(Section~\ref{sec:outputTheory}) and then continue the
\code{sleepstudy} example introduced in Section~\ref{sec:sleepstudy}
to illustrate these ideas in practice.

\subsection{Theory underlying the output module}
\label{sec:outputTheory}

\subsubsection{Covariance matrix of the fixed-effect coefficients}
\label{sec:vcov}

In the \code{lm} function, the variance-covariance matrix of the
coefficients is the inverse of the cross-product of the model matrix, times
the residual variance \citep{Chambers:1993}.  The inverse cross-product
matrix is computed by first inverting the upper triangular matrix
resulting from the QR decomposition of the model matrix, and then
taking its cross-product,
\begin{equation}
  \label{eq:varUBeta}
  \Var_{\bm\theta, \sigma}\left(
    \begin{bmatrix}
      \bm\mu_{\mc U|\mc Y=\yobs} \\ \hat\betavec
    \end{bmatrix}
  \right) =
  \sigma^2
  \begin{bmatrix}
    \bm L\trans_{\bm\theta} & \bm R_{ZX}\\
    \bm 0            & \bm R_X
  \end{bmatrix}^{-1}
  \begin{bmatrix}
    \bm L_{\bm\theta}& \bm 0\\
    \bm R_{ZX}\trans & \bm R_X\trans
  \end{bmatrix}^{-1}.
\end{equation} % |
Because of normality, the marginal covariance matrix of $\hat\betavec$ is
just the lower-right $p$-by-$p$ block of
$\Var_{\bm\theta, \sigma}\left(
    \begin{bmatrix}
      \bm\mu_{\mc U|\mc Y=\yobs} \\ \hat\betavec
    \end{bmatrix}
  \right)$.
This lower-right block is
\begin{equation}
  \label{eq:varBeta}
  \Var_{\bm\theta, \sigma}(\hat\betavec) = \sigma^2 \bm R_X^{-1} (\bm R_X\trans)^{-1},
\end{equation}
which follows from the Schur complement identity \citep[Theorem 1.2]{zhang2006schur}.  This
matrix can be extracted from a fitted \code{merMod} object as,
<<vcovByHand>>=
RX <- getME(fm1, "RX")
sigma2 <- sigma(fm1)^2
sigma2 * chol2inv(RX)
@
which could be computed with \pkg{lme4} as \code{vcov(fm1)}.

The square-root diagonal of this covariance matrix contains the
estimates of the standard errors of fixed-effects coefficients.  These
standard errors are used to construct Wald confidence intervals with
\code{confint(., method = "Wald")}.  Such confidence intervals are
approximate, and depend on the assumption that the likelihood profile
of the fixed effects is linear on the $\zeta$ scale
(Section~\ref{sec:profile}).

\subsubsection{Profiling}
\label{sec:profile}

\newcommand{\devprof}{-2 {\mc L}_p}

The theory of likelihood profiles is straightforward: the deviance (or
likelihood) profile, $\devprof()$, for a focal model parameter $P$ is
the minimum value of the deviance conditioned on a particular value of
$P$.  For each parameter of interest, our goal is to evaluate the
deviance profile for many points -- optimizing over all of the
non-focal parameters each time -- over a wide enough range and with
high enough resolution to evaluate the shape of the profile (in
particular, whether it is quadratic, which would allow use of Wald
confidence intervals and tests) and to find the values of $P$ such
that $\devprof(P)=-2 {\mc L}(\widehat{P}) + \chi^2(1-\alpha)$, which
represent the profile confidence intervals.  While profile confidence
intervals rely on the asymptotic distribution of the minimum deviance,
this is a much weaker assumption than the log-quadratic likelihood
surface required by Wald tests.

An additional challenge in profiling arises when we want to compute
profiles for quantities of interest that are not parameters of our PLS
function. We have two problems in using the deviance function defined
above to profile linear mixed models. First, a parameterization of the
random-effects variance-covariance matrix in terms of standard
deviations and correlations, or variances and covariances, is much
more familiar to users, and much more relevant as output of a
statistical model, than the parameters, $\bm\theta$, of the relative
covariance factor -- users are interested in inferences on variances
or standard deviations, not on $\bm\theta$. Second, the fixed-effect
coefficients and the residual standard deviation, both of which are
also of interest to users, are profiled out (in the sense used above)
of the deviance function (Section~\ref{sec:profdev}), so we have to
find a strategy for estimating the deviance for values of $\bm\beta$
and $\sigma^2$ other than the profiled values.

To handle the first problem we create a new version of the deviance
function that first takes a vector of standard deviations (and
correlations), and a value of the residual standard deviation,
maps them to a $\bm\theta$ vector, and then computes the PLS as
before; it uses the specified residual standard deviation rather than
the PLS estimate of the standard deviation
(Equation~\ref{eq:residualVarianceEstimate}) in the deviance calculation.
We compute a profile likelihood for the fixed-effect parameters, which
are profiled out of the deviance calculation, by adding an offset to
the linear predictor for the focal element of $\bm\beta$.  The
resulting function is not useful for general nonlinear optimization
-- one can easily wander into parameter regimes corresponding to
infeasible (non-positive semidefinite) variance-covariance matrices --
but it serves for likelihood profiling, where one focal parameter is
varied at a time and the optimization over the other parameters is
likely to start close to an optimum.

In practice, the \code{profile} method systematically varies the
parameters in a model, assessing the best possible fit that can be
obtained with one parameter fixed at a specific value and comparing
this fit to the globally optimal fit, which is the original model fit
that allowed all the parameters to vary.  The models are compared
according to the change in the deviance, which is the likelihood ratio
test statistic.  We apply a signed square root transformation to this
statistic and plot the resulting function, which we call the
\emph{profile zeta function} or $\zeta$, versus the parameter
value. The signed aspect of this transformation means that $\zeta$ is
positive where the deviation from the parameter estimate is positive
and negative otherwise, leading to a monotonically increasing function
which is zero at the global optimum. A $\zeta$ value can be compared
to the quantiles of the standard normal distribution.  For example, a
95\% profile deviance confidence interval on the parameter consists of
the values for which $-1.96 < \zeta < 1.96$.  Because the process of
profiling a fitted model can be computationally intensive -- it
involves refitting the model many times -- one should exercise
caution with complex models fit to large data sets.

The standard approach to this computational challenge is to compute
$\zeta$ at a limited number of parameter values, and to fill in the
gaps by fitting an interpolation spline.  Often one is able to invert
the spline to obtain a function from $\zeta$ to the focal parameter,
which is necessary in order to construct profile confidence intervals.
However, even if the points themselves are monotonic, it is possible
to obtain non-monotonic interpolation curves.  In such a case,
\pkg{lme4} falls back to linear interpolation, with a warning.

\newcommand{\alphamax}{\alpha_{\textrm{\small max}}}
\newcommand{\pstep}{\epsilon}

The last part of the technical specification for computing profiles is
deciding on a strategy for choosing points to sample.  In one way or
another one wants to start at the estimated value for each parameter
and work outward either until a constraint is reached (i.e., a value of
0 for a standard deviation parameter or a value of $\pm 1$ for a
correlation parameter), or until a sufficiently large deviation from
the minimum deviance is attained.  \pkg{lme4}'s profiler chooses a
cutoff $\phi$ based on the $1-\alphamax$ critical value of the
$\chi^2$ distribution with a number of degrees of freedom equal to the
\emph{total} number of parameters in the model, where $\alphamax$ is
set to 0.05 by default. The profile computation initially adjusts the
focal parameter $p_i$ by an amount $\pstep=1.01 \hat p_i$ from its ML
or REML estimate $\hat p_i$ (or by $\pstep=0.001$ if $\hat p_i$ is
zero, as in the case of a singular variance-covariance model).  The
local slope of the likelihood profile $(\zeta(\hat
p_i+\pstep)-\zeta(\hat p_i))/\pstep$ is used to pick the next point to
evaluate, extrapolating the local slope to find a new $\pstep$ that
would be expected to correspond to the desired step size $\Delta
\zeta$ (equal to $\phi/8$ by default, so that 16 points would be used
to cover the profile in both directions if the log-likelihood surface
were exactly quadratic).  Some fail-safe testing is done to ensure
that the step chosen is always positive, and less than a maximum; if a
deviance is ever detected that is lower than that of the ML deviance,
which can occur if the initial fit was wrong due to numerical
problems, the profiler issues an error and stops.
%% while scaling based on the approximate variance-covariance matrix
%% might be more accurate in some cases, it may also lead to problems
%% when the log-likelihood surface is far from quadratic

\subsubsection{Parametric bootstrapping}
\label{sec:pb}

To avoid the asymptotic assumptions of the likelihood ratio test, at
the cost of greatly increased computation time, one can estimate
confidence intervals by parametric bootstrapping -- that is, by
simulating data from the fitted model, refitting the model, and
extracting the new estimated parameters (or any other quantities of
interest).  This task is quite straightforward, since there is already
a \code{simulate} method, and a \code{refit} function which
re-estimates the (RE)ML parameters for new data, starting from the
previous (RE)ML estimates and re-using the previously computed model
structures (including the fill-reducing permutation) for efficiency.
The \code{bootMer} function is thus a fairly thin wrapper around a
\code{simulate}/\code{refit} loop, with a small amount of additional
logic for parallel computation and error-catching.  (Some of the ideas
of \code{bootMer} are adapted from \code{merBoot} \citep{merBoot}, a
more ambitious framework for bootstrapping \pkg{lme4} model fits which
unfortunately seems to be unavailable at present.)


\subsubsection{Conditional variances of random effects}
\label{sec:condVar}

It is useful to clarify that the conditional covariance concept in
\pkg{lme4} is based on a simplification of the linear mixed model.  In
particular, we simplify the model by assuming that the quantities,
$\betavec$, $\bLt$, and $\sigma$, are known (i.e., set at their
estimated values). In fact, the only way to define the conditional
covariance is at fixed parameter values.  Our approximation here is
using the estimated parameter values for the unknown ``true''
parameter values.  In this simplified case, $\mc U$ is the only
quantity in the statistical model that is both random and unknown.

Given this simplified linear mixed model, a standard result in
Bayesian linear regression modeling \citep{gelman2013bayesian}
implies that the conditional distribution of the spherical random
effects given the observed response vector is Gaussian,
\begin{equation}
  \label{eq:condGaussian}
(\mc U | \mc Y = \yobs) \sim \mathcal{N}(\mu_{\mc U | \mc Y = \yobs},
\widehat{\sigma}^2 \bm V),
\end{equation}
where
\begin{equation}
  \label{eq:condVar}
  \bm V = \left(\bm\Lambda_{\widehat{\bm\theta}}\trans\bm Z\trans \bm W
    \bm Z\bm\Lambda_{\widehat{\bm\theta}} + \bm I_q \right)^{-1} =
  \left(\bm L_{\widehat{\bm\theta}}^{-1}\right)\trans
  \left(\bm L_{\widehat{\bm\theta}}^{-1}\right)
\end{equation}
is the unscaled conditional variance and
\begin{equation}
  \label{eq:condMean}
  \mu_{\mc U | \mc Y = \yobs} = \bm V \bm\Lambda_{\widehat{\bm\theta}}\trans\bm Z\trans\bm W
    \left(\yobs-\bm o - X\widehat{\betavec} \right)
\end{equation}  % |
is the conditional mean/mode. Note that in practice the inverse in
Equation~\ref{eq:condVar} does not get computed directly, but rather an
efficient method is used that exploits the sparse structures.

The random-effects coefficient vector, $\mathbf{b}$, is often of
greater interest. The conditional covariance matrix of $\mc B$
may be expressed as
\begin{equation}
  \label{eq:condVarB}
\widehat{\sigma}^2 \bm\Lambda_{\widehat{\bm\theta}} \bm V \bm\Lambda_{\widehat{\bm\theta}}\trans.
\end{equation}

<<condVarExample, echo=FALSE, eval=FALSE>>=
s2 <- sigma(fm1)^2
Lambdat <- getME(fm1, "Lambdat")
Lambda <- getME(fm1, "Lambda")
Zt <- getME(fm1, "Zt")
Z <- getME(fm1, "Z")
y <- getME(fm1, "y")
X <- getME(fm1, "X")
beta <- getME(fm1, "beta")
L <- getME(fm1, "L")
(V <- crossprod(solve(L, system = "L")))[1:2, 1:2]
(s2 * Lambda %*% V %*% Lambdat)[1:2, 1:2]
attr(ranef(fm1, condVar = TRUE)$Subject, "postVar")[ , , 1] # should be same as Lam %*% V %*% Lamt
(U <- as.vector((V %*% Lambdat %*% Zt %*%  (y - X %*% beta))))
getME(fm1, "u")
@

\subsubsection{The hat matrix}
\label{sec:hat}

The hat matrix, $\bm H$, is a useful object in linear model
diagnostics \citep{cook1982residuals}.  In general, $\bm H$ relates
the observed and fitted response vectors, but we specifically define
it as,
\begin{equation}
  \left(\bm\mu_{\mc Y|\mc U=\bm u} - \bm o\right) =
  \bm H \left(\yobs - \bm o\right).
\end{equation}
To find $\bm H$ we note that
\begin{equation}
  \left(\bm\mu_{\mc Y|\mc U=\bm u} - \bm o\right) =
  \begin{bmatrix}
    \bm Z\bm\Lambda & \bm X \\
  \end{bmatrix}
  \begin{bmatrix}
    \mu_{\mc U | \mc Y = \yobs} \\ \widehat{\betavec}_{\bm\theta}
  \end{bmatrix}.
\end{equation}
Next we get an expression for $\begin{bmatrix}
    \mu_{\mc U | \mc Y = \yobs} \\ \widehat{\betavec}_{\bm\theta}
  \end{bmatrix}$ by solving the normal equations
  (Equation~\ref{eq:normalEquations}),
\begin{equation}
 \begin{bmatrix}
    \mu_{\mc U | \mc Y = \yobs} \\ \widehat{\betavec}_{\bm\theta}
  \end{bmatrix} =
  \begin{bmatrix}
    \bm L\trans_{\bm\theta} & \bm R_{ZX} \\
    \bm 0 & \bm R_{X}
  \end{bmatrix}^{-1}
    \begin{bmatrix}
    \bm L_{\bm\theta} & \bm 0 \\
    \bm R\trans_{ZX} & \bm R\trans_{X} \\
  \end{bmatrix}^{-1}
    \begin{bmatrix}
    \bLt\trans \bm Z\trans \\
    \bm X\trans \\
  \end{bmatrix} \bm W (\yobs - \bm o).
\end{equation} % |
By the Schur complement identity \citep{zhang2006schur},
\begin{equation}
  \begin{bmatrix}
    \bm L\trans_{\bm\theta} & \bm R_{ZX} \\
    \bm 0 & \bm R_{X}
  \end{bmatrix}^{-1} =
  \begin{bmatrix}
    \left(\bm L\trans_{\bm\theta}\right)^{-1} &
    - \left(\bm L\trans_{\bm\theta}\right)^{-1} \bm R_{ZX} \bm R_{X}^{-1}\\
    \bm 0 & \bm R_{X}^{-1}
  \end{bmatrix}.
\end{equation}
Therefore, we may write the hat matrix as
\begin{equation}
  \label{eq:hatComputational}
  \bm H = (\bm C_L\trans \bm C_L + \bm C_R\trans \bm C_R),
\end{equation}
where,
\begin{equation}
  \label{eq:CL}
  \bm L_{\bm\theta} \bm C_L = \bLt\trans \bm Z\trans\bm W^{1/2}
\end{equation}
and,
\begin{equation}
  \label{eq:CR}
  \bm R\trans_{X} \bm C_R = \bm X\trans\bm W^{1/2} - \bm R\trans_{ZX}\bm C_L.
\end{equation}

The trace of the hat matrix is often used as a measure of the
effective degrees of freedom \citep[e.g., ][]{vaida2005conditional}.
Using a relationship between the trace and vec operators, the trace of
$\bm H$ may be expressed as,
\begin{equation}
  \label{eq:traceHat}
  \tr(\bm H) = \left\| \VEC(\bm C_L) \right\|^2
  + \left\| \VEC(\bm C_R) \right\|^2.
\end{equation}

\subsection{Using the output module}
\label{sec:outputPractice}

\begin{table}
  \begin{tabular}{ll}
    \hline
    Generic (Section) & Brief description of return value \\
    \hline
    \code{anova} (\ref{sec:modComp}) &
    Decomposition of fixed-effects contributions \\
    & or model comparison. \\
    \code{as.function} &
    Function returning profiled deviance or REML criterion. \\
    \code{coef} &
    Sum of the random and fixed effects for each level. \\
    \code{confint} (\ref{sec:CI}) &
    Confidence intervals on linear mixed-model parameters. \\
    \code{deviance} (\ref{sec:summary}) &
    Minus twice maximum log-likelihood. \\
    & (Use \code{REMLcrit} for the REML criterion.) \\
    \code{df.residual} &
    Residual degrees of freedom. \\
    \code{drop1} &
    Drop allowable single terms from the model. \\
    \code{extractAIC} &
    Generalized Akaike information criterion \\
    \code{fitted} &
    Fitted values given conditional modes (Equation~\ref{eq:meanYgivenU}). \\
    \code{fixef} (\ref{sec:summary}) &
    Estimates of the fixed-effects coefficients, $\widehat{\bm\beta}$ \\
    \code{formula} (\ref{sec:uncor}) &
    Mixed-model formula of fitted model. \\
    \code{logLik} & % logLik
    Maximum log-likelihood. \\
    \code{model.frame} & % model.frame
    Data required to fit the model. \\
    \code{model.matrix} &
    Fixed-effects model matrix, $\bm X$.\\
    \code{ngrps} (\ref{sec:summary}) &
    Number of levels in each grouping factor. \\
    \code{nobs} (\ref{sec:summary}) &
    Number of observations.\\
    \code{plot} &
    Diagnostic plots for mixed-model fits. \\
    \code{predict} (\ref{sec:predict}) &
    Various types of predicted values. \\
    \code{print} &
    Basic printout of mixed-model objects. \\
    \code{profile} (\ref{sec:profile}) &
    Profiled likelihood over various model parameters. \\
    \code{ranef} (\ref{sec:summary}) &
    Conditional modes of the random effects. \\
    \code{refit} (\ref{sec:pb}) &
    A model (re)fitted to a new set of observations of the response variable. \\
    \code{refitML} (\ref{sec:modComp}) &
    A model (re)fitted by maximum likelihood.  \\
    \code{residuals}  (\ref{sec:summary}) &
    Various types of residual values. \\
    \code{sigma} (\ref{sec:summary}) &
    Residual standard deviation. \\
    \code{simulate} (\ref{sec:predict}) &
    Simulated data from a fitted mixed model. \\
    \code{summary} (\ref{sec:summary}) &
    Summary of a mixed model. \\
    \code{terms} &
    Terms representation of a mixed model. \\
    \code{update} (\ref{sec:update}) &
    An updated model using a revised formula or other arguments. \\
    \code{VarCorr} (\ref{sec:summary})  &
    Estimated random-effects variances, standard deviations, and correlations. \\
    \code{vcov} (\ref{sec:summary}) &
    Covariance matrix of the fixed-effect estimates. \\
    \code{weights} &
    Prior weights used in model fitting. \\
    \hline
  \end{tabular}
  \caption{List of currently available methods for objects of the class `\code{merMod}'.}
  \label{tab:methods}
\end{table}

The user interface for the output module largely consists of a set of
methods (Table~\ref{tab:methods}) for objects of class \code{merMod},
which is the class of objects returned by \code{lmer}.  In addition to
these methods, the \code{getME} function can be used to extract
various objects from a fitted mixed model in \pkg{lme4}.  Here we
illustrate the use of several of these methods.

\subsubsection{Updating fitted mixed models}
\label{sec:update}

To illustrate the \code{update} method for `\code{merMod}' objects we
construct a random intercept only model. This task could be done in
several ways, but we choose to first remove the random-effects term
\code{(Days | Subject)} and add a new term with a random intercept,
<<updateModel>>=
fm3 <- update(fm1, . ~ . - (Days | Subject) + (1 | Subject))
formula(fm3)
@ % |

\subsubsection{Model summary and associated accessors}
\label{sec:summary}

The \code{summary} method for `\code{merMod}' objects provides
information about the model fit.  Here we consider the output of
\code{summary(fm1)} in four steps.  The first few lines of output
indicate that the model was fitted by REML as well as the value of the
REML criterion (Equation~\ref{eq:REMLdeviance}) at convergence (which
may also be extracted using \code{REMLcrit(fm1)}).  The beginning of
the summary also reproduces the model formula and the scaled Pearson
residuals,
<<summary1,echo=FALSE>>=
ss <- summary(fm1)
cc <- capture.output(print(ss))
reRow <- grep("^Random effects", cc)
cat(cc[1:(reRow - 2)], sep = "\n")
@
This information may also be obtained using standard accessor
functions,
<<summary1reproduced, eval=FALSE>>=
formula(fm1)
REMLcrit(fm1)
quantile(residuals(fm1, "pearson", scaled = TRUE))
@
The second piece of the \code{summary} output provides information
regarding the random-effects and residual variation,
<<summary2, echo=FALSE>>=
feRow <- grep("^Fixed effects", cc)
cat(cc[reRow:(feRow - 2)], sep = "\n")
@
which can also be accessed using,
<<summary2reproduced, eval=FALSE>>=
print(vc <- VarCorr(fm1), comp = c("Variance", "Std.Dev."))
nobs(fm1)
ngrps(fm1)
sigma(fm1)
@
The print method for objects returned by \code{VarCorr} hides the internal structure of these
`\code{VarCorr.merMod}' objects. The internal structure of an object of
this class is a list of matrices, one for each random-effects term.
The standard deviations and correlation matrices for each term are
stored as attributes, \code{stddev} and \code{correlation},
respectively, of the variance-covariance matrix, and the residual
standard deviation is stored as attribute \code{sc}.  For programming
use, these objects can be summarized differently using their
\code{as.data.frame} method,
<<VarCorr>>=
as.data.frame(VarCorr(fm1))
@
which contains one row for each variance or covariance parameter. The
\code{grp} column gives the grouping factor associated with this
parameter. The \code{var1} and \code{var2} columns give the names of
the variables associated with the parameter (\code{var2} is \code{<NA>}
unless it is a covariance parameter). The \code{vcov} column gives the
variances and covariances, and the \code{sdcor} column gives these
numbers on the standard deviation and correlation scales.

The next chunk of output gives the fixed-effect estimates,
<<summary3, echo=FALSE>>=
corRow <- grep("^Correlation", cc)
cat(cc[feRow:(corRow - 2)], sep = "\n")
@
Note that there are no $p$~values (see Section~\ref{sec:pvalues}).
The fixed-effect point estimates may be obtained separately via
\code{fixef(fm1)}.
Conditional modes of the random-effects coefficients can be obtained
with \code{ranef} (see Section~\ref{sec:condVar} for information on
the theory).  Finally, we have the correlations between the fixed-effect estimates
<<summary4, echo=FALSE>>=
cat(cc[corRow:length(cc)], sep = "\n")
@
The full variance-covariance matrix of the fixed-effects estimates can
be obtained in the usual \proglang{R} way with the \code{vcov} method
(Section~\ref{sec:vcov}).  Alternatively, \code{coef(summary(fm1))} will
return the full fixed-effects parameter table as shown in the summary.

\subsubsection{Diagnostic plots}
\label{sec:diagnostics}

\pkg{lme4} provides tools for generating most of the standard
graphical diagnostic plots (with the exception of those incorporating
influence measures, e.g., Cook's distance and leverage plots), in a
way modeled on the diagnostic graphics of \pkg{nlme}
\citep{R:Pinheiro+Bates:2000}.  In particular, the familiar
\code{plot} method in base \proglang{R} for linear models (objects of
class `\code{lm}') can be used to create the standard fitted
vs. residual plots,
<<diagplot1,fig.keep="none">>=
plot(fm1, type = c("p", "smooth"))
@
scale-location plots,
<<diagplot2,fig.keep="none">>=
plot(fm1, sqrt(abs(resid(.))) ~ fitted(.),
     type = c("p", "smooth"))
@
and quantile-quantile plots (from \pkg{lattice}),
<<diagplot3,fig.keep="none">>=
qqmath(fm1, id = 0.05)
@
In contrast to \code{plot.lm}, these scale-location and Q-Q plots
are based on raw rather than standardized residuals.

In addition to these standard diagnostic plots, which
examine the validity of various assumptions (linearity,
homoscedasticity, normality) at the level of the residuals,
one can also use the \code{dotplot} and \code{qqmath} methods
for the conditional modes (i.e., `\code{ranef.mer}'
objects generated by \code{ranef(fit)})
to check for interesting patterns and normality in the conditional
modes.  \pkg{lme4} does not provide influence diagnostics, but
these (and other useful diagnostic procedures) are available
in the dependent packages \pkg{HLMdiag} and \pkg{influence.ME}
\citep{HLMdiag_pkg,influenceME_pkg}.

Finally, \emph{posterior predictive simulation}
\citep{gelman_data_2006} is a generally useful diagnostic tool,
adapted from Bayesian methods, for exploring model fit. Users pick
some summary statistic of interest.  They then compute the summary
statistic for an ensemble of simulations (Section~\ref{sec:predict}),
and see where the observed data falls within the simulated
distribution; if the observed data is extreme, we might conclude that
the model is a poor representation of reality.  For example, using the
sleep study fit and choosing the interquartile range
of the reaction times as the summary
statistic:
<<ppsim,results="hide">>=
iqrvec <- sapply(simulate(fm1, 1000), IQR)
obsval <- IQR(sleepstudy$Reaction)
post.pred.p <- mean(obsval >= c(obsval, iqrvec))
@
The (one-tailed) posterior predictive $p$~value of \Sexpr{round(post.pred.p,2)}
indicates that the model represents the data adequately, at least for
this summary statistic.  In contrast to the full Bayesian case, the
procedure described here does not allow for the uncertainty in the
estimated parameters. However, it should be a reasonable approximation
when the residual variation is large.
%% \bmb{it should be possible to put together something with
%% \code{arm::sim} to simulate beta values and \code{simulate} with
%% \code{newparams}, but (a) this would require some testing and
%% development (I don't think \code{newparams} can take a matrix) and
%% (b) it doesn't account for uncertainty in theta)}

\subsubsection{Sequential decomposition and model comparison}
\label{sec:modComp}

Objects of class `\code{merMod}' have an \code{anova} method which
returns $F$~statistics corresponding to the sequential decomposition
of the contributions of fixed-effects terms.  In order to illustrate
this sequential ANOVA decomposition, we fit a model with orthogonal
linear and quadratic \code{Days} terms,
<<anovaQuadraticModel>>=
fm4 <- lmer(Reaction ~ polyDays[ , 1] + polyDays[ , 2] +
            (polyDays[ , 1] + polyDays[ , 2] | Subject),
            within(sleepstudy, polyDays <- poly(Days, 2)))
anova(fm4)
@
<<anovaSanityCheck, include=FALSE>>=
(getME(fm4, "RX")[2, ] %*% getME(fm4, "fixef"))^2
@
The relative magnitudes of the two sums of squares indicate that the
quadratic term explains much less variation than the linear term.
Furthermore, the magnitudes of the two $F$~statistics strongly
suggest significance of the linear term, but not the quadratic term.
Notice that this is only an informal assessment of significance as
there are no $p$~values associated with these $F$~statistics, an issue
discussed in more detail in the next subsection (``Computing $p$-values'',
p.~\pageref{sec:pvalues}).

To understand how these quantities are computed, let $\bm R_i$ contain
the rows of $\bm R_X$ (Equation~\ref{eq:blockCholeskyDecomp}) associated
with the $i$th fixed-effects term.  Then the sum of squares for term
$i$ is,
\begin{equation}
  \label{eq:SS}
  \mathit{SS}_i = \widehat{\bm\beta}\trans\bm R_i\trans \bm R_i \widehat{\bm\beta}
\end{equation}
If $\mathit{DF}_i$ is the number of columns in $\bm R_i$, then the
$F$~statistic for term $i$ is,
\begin{equation}
  \label{eq:Fstat}
  F_i = \frac{\mathit{SS}_i}{\widehat{\sigma}^2 \mathit{DF}_i}
\end{equation}

For multiple arguments, the \code{anova} method returns model
comparison statistics.
<<anovaManyModels>>=
anova(fm1, fm2, fm3)
@
<<anovaRes,echo=FALSE>>=
fm3ML <- refitML(fm3)
fm2ML <- refitML(fm2)
fm1ML <- refitML(fm1)
ddiff <- deviance(fm3ML) - deviance(fm2ML)
dp <- pchisq(ddiff, 1, lower.tail = FALSE)
ddiff2 <- deviance(fm2ML) - deviance(fm1ML)
@
The output shows $\chi^2$ statistics representing the difference in
deviance between successive models, as well as $p$~values based on
likelihood ratio test comparisons.  In this case, the sequential
comparison shows that adding a linear effect of time uncorrelated with
the intercept leads to an enormous and significant drop in deviance
($\Delta\mbox{deviance} \approx \Sexpr{round(ddiff)}$, $p \approx
\Sexpr{round(dp,10)}$), while the further addition of correlation
between the slope and intercept leads to a trivial and non-significant
change in deviance ($\Delta\mbox{deviance} \approx
\Sexpr{round(ddiff2,2)}$).  For objects of class `\code{lmerMod}' the
default behavior is to refit the models with ML if fitted with
\code{REML = TRUE}, which is necessary in order to get sensible
answers when comparing models that differ in their fixed effects; this
can be controlled via the \code{refit} argument.

\subsubsection[Computing p values]{Computing $p$~values}
\label{sec:pvalues}

One of the more controversial design decisions of \pkg{lme4} has been
to omit the output of $p$~values associated with sequential ANOVA
decompositions of fixed effects.  The absence of analytical results for null
distributions of parameter estimates in complex situations
(e.g., unbalanced or partially crossed designs) is a long-standing
problem in mixed-model inference.  While the null distributions (and
the sampling distributions of non-null estimates) are asymptotically
normal, these distributions are not $t$~distributed for finite size
samples -- nor are the corresponding null distributions of
differences in scaled deviances $F$~distributed.  Thus approximate methods
for computing the approximate degrees of freedom for
$t$~distributions, or the denominator degrees of freedom for
$F$~statistics \citep{Satterthwaite_1946,kenward_small_1997}, are at
best \emph{ad hoc} solutions.

However, computing finite-size-corrected $p$~values is
sometimes necessary.  Therefore, although the package does not provide
them (except via parametric bootstrapping,
Section~\ref{sec:pb}), we have provided a help page to guide users in
finding appropriate methods:
<<pvaluesHelp, eval=FALSE>>=
help("pvalues")
@
This help page provides pointers to other packages that provide
machinery for calculating $p$~values associated with `\code{merMod}'
objects. It also suggests framing the inference problem as a
likelihood ratio test (achieved by supplying multiple `\code{merMod}'
objects to the \code{anova} method (Section~\ref{sec:modComp}), as
well as alternatives to $p$~values such as confidence intervals
(Section~\ref{sec:CI}).

Previous versions of \pkg{lme4} provided the \code{mcmcsamp} function,
which generated a Markov chain Monte Carlo sample from the posterior
distribution of the parameters (assuming flat priors). Due to
difficulty in constructing a version of \code{mcmcsamp} that was
reliable even in cases where the estimated random-effect variances
were near zero, \code{mcmcsamp} has been withdrawn.

\subsubsection{Computing confidence intervals}
\label{sec:CI}

As described above (Section~\ref{sec:vcov}), \pkg{lme4} provides confidence intervals (using
\code{confint}) via Wald approximations (for fixed-effect parameters
only), likelihood profiling, or parametric bootstrapping (the
\code{boot.type} argument selects the bootstrap confidence interval type).
<<compareCI,echo=FALSE,cache=TRUE,message=FALSE,warning=FALSE>>=
ccw <- confint(fm1, method = "Wald")
## compute the profile once here and re-use it below rather than
## profiling fm1 a second time; profiling is expensive and
## makes us exceed CRAN time limits
pf <- profile(fm1)
ccp <- confint(pf)
ccb <- confint(fm1, method = "boot")
@
<<CIqcomp,echo=FALSE,eval=TRUE>>=
rtol <- function(x,y) {
    abs((x - y) / ((x + y) / 2))
}
bw <- apply(ccb, 1, diff)
pw <- apply(ccp, 1, diff)
mdiffpct <- round(100 * max(rtol(bw, pw)))
@
<<CIplot,echo=FALSE,eval=FALSE>>=
## obsolete
## ,fig.cap="Comparison of confidence intervals",fig.scap="CI comparison"
tf <- function(x, method) data.frame(method = method,
               par = rownames(x),
               setNames(as.data.frame(x), c("lwr", "upr")))
cc.all <- do.call(rbind, mapply(tf, list(ccw, ccp, ccb),
                               c("Wald", "profile", "boot"),
                     SIMPLIFY = FALSE))
ggplot(cc.all, aes(x = 1, ymin = lwr, ymax = upr, colour = method)) +
    geom_linerange(position = position_dodge(width = 0.5)) +
    facet_wrap( ~ par, scale = "free") +
    theme(axis.text.x = element_blank()) +
    labs(x = "")
@ %
As is typical for computationally intensive profile confidence
intervals in \proglang{R}, the intervals can be computed either directly from
fitted `\code{merMod}' objects (in which case profiling is done as an
interim step), or from a previously computed likelihood profile (of
class `\code{thpr}', for ``theta profile'').  Parametric bootstrapping
confidence intervals use a thin wrapper around the \code{bootMer}
function that passes the results to \code{boot.ci} from the \pkg{boot}
package \citep{boot_pkg,DavisonHinkley1997} for confidence interval calculation.

In the running sleep study example, the 95\% confidence intervals
estimated by all three methods are quite similar.  The largest
change is a \Sexpr{mdiffpct}\% difference in confidence
interval widths between profile and parametric bootstrap methods for
the correlation between the intercept and slope random effects
(\{\Sexpr{round(ccb[2,1],2)},\Sexpr{round(ccb[2,2],2)}\} vs.
\{\Sexpr{round(ccp[2,1],2)},\Sexpr{round(ccp[2,2],2)}\}).
%% \bmb{Mention that in our experience the profile CIs are
%% reasonably reliable for large and/or well-behaved data sets?
%% Is that too vague?  Disclaimer that a simulation study would
%% be required to know for sure?}

\subsubsection{General profile zeta and related plots}
\label{sec:profzeta}

\pkg{lme4} provides several functions (built on \pkg{lattice}
graphics, \citealt{lattice}) for plotting the profile zeta functions
(Section~\ref{sec:profile}) and other related quantities.

\begin{itemize}
\item The \emph{profile zeta} plot
  (Figure~\ref{fig:profile_zeta_plot}; \code{xyplot}) is simply a plot
  of the profile zeta function for each model parameter; linearity of
  this plot for a given parameter implies that the likelihood profile
  is quadratic (and thus that Wald approximations would be reasonably
  accurate).
\item The \emph{profile density plot}
  (Figure~\ref{fig:profile_density_plot}; \code{densityplot}) displays
  an approximation of the probability density function of the sampling
  distribution for each parameter.  These densities are derived by
  setting the cumulative distribution function (c.d.f) to be
  $\Phi(\zeta)$ where $\Phi$ is the c.d.f.{} of the standard normal
  distribution.  This is not quite the same as evaluating the
  distribution of the estimator of the parameter, which for mixed
  models can be very difficult, but it gives a reasonable
  approximation.  If the profile zeta plot is linear, then the profile
  density plot will be Gaussian.
\item The \emph{profile pairs plot}
  (Figure~\ref{fig:profile_pairs_plot}; \code{splom}) gives an
  approximation of the two-dimensional profiles of pairs of
  parameters, interpolated from the univariate profiles as described
  in \citet[Chapter~6]{bateswatts88:_nonlin}.  The profile pairs plot
  shows two-dimensional 50\%, 80\%, 90\%, 95\% and 99\% marginal
  confidence regions based on the likelihood ratio, as well as the
  \emph{profile traces}, which indicate the conditional estimates of
  each parameter for fixed values of the other parameters.  While
  panels above the diagonal show profiles with respect to the original
  parameters (with random-effects parameters on the standard
  deviation\slash{}correlation scale, as for all profile plots), the
  panels below the diagonal show plots on the $(\zeta_i,\zeta_j)$
  scale.  The below-diagonal panels allow us to see distortions from
  an elliptical shape due to nonlinearity of the traces, separately
  from the one-dimensional distortions caused by a poor choice of
  scale for the parameters.  The $\zeta$ scales provide, in some sense,
  the best possible set of single-parameter transformations for
  assessing the contours.  On the $\zeta$ scales the extent of a
  contour on the horizontal axis is exactly the same as the extent on
  the vertical axis and both are centered about zero.
\end{itemize}
<<profile_calc,echo=FALSE,cache=TRUE>>=
## pf was computed above, in the confidence interval
## comparison; re-used here to save time by not profiling
## fm1 twice
@
<<profile_zeta_plot,fig.cap="Profile zeta plot: \\code{xyplot(prof.obj)}",fig.scap="Profile zeta plot",echo=FALSE,fig.align='center',fig.pos='tb'>>=
xyplot(pf)
@
<<profile_density_plot,fig.cap="Profile density plot: \\code{densityplot(prof.obj)}",echo=FALSE,fig.align='center',fig.pos='tb'>>=
densityplot(pf)
@
<<profile_pairs_plot,fig.cap="Profile pairs plot: \\code{splom(prof.obj)}",echo=FALSE,fig.height=8,fig.width=8,fig.align='center',fig.pos='htb',out.height='5.5in'>>=
splom(pf)
@

For users who want to build their own graphical displays of the
profile, there is a method for  \code{as.data.frame} that converts profile
(`\code{thpr}') objects to a more convenient format.

\subsubsection{Computing fitted and predicted values; simulating}
\label{sec:predict}

Because mixed models involve random coefficients, one must always
clarify whether predictions should be based on the marginal
distribution of the response variable or on the distribution that is
conditional on the modes of the random effects
(Equation~\ref{eq:distYgivenU}).  The \code{fitted} method retrieves fitted
values that are conditional on all of the modes of the random effects;
the \code{predict} method returns the same values by default, but also
allows for predictions to be made conditional on different sets of
random effects.  For example, if the \code{re.form} argument is set to
\code{NULL} (the default), then the predictions are conditional on all
random effects in the model; if \code{re.form} is \verb+~0+ or
\code{NA}, then the predictions are made at the population level (all
random-effect values set to zero).  In models with multiple random
effects, the user can give \code{re.form} as a formula that specifies
which random effects are conditioned on.

\pkg{lme4} also provides a \code{simulate} method, which allows
similar flexibility in conditioning on random effects; in addition it
allows the user to choose (via the \code{use.u} argument) between
conditioning on the fitted conditional modes or choosing a new set of
simulated conditional modes (zero-mean Normal deviates chosen
according to the estimated random-effects variance-covariance
matrices).  Finally, the \code{simulate} method has a method for
`\code{formula}' objects, which allows for \emph{de novo} simulation in
the absence of a fitted model.  In this case, the user must specify
the random-effects ($\bm\theta$), fixed-effects ($\bm\beta$), and
residual standard deviation ($\sigma$) parameters via the
\code{newparams} argument.  The standard simulation method (based on a
`\code{merMod}' object) is the basis of parametric bootstrapping
(Section~\ref{sec:pb}) and posterior predictive simulation
(Section~\ref{sec:diagnostics}); \emph{de novo} simulation based on a
formula provides a flexible framework for power analysis.

\section{Conclusion}

Mixed modeling is an extremely useful but computationally intensive
technique.  Computational limitations are especially important because
mixed models are commonly applied to moderately large data sets
($10^4$--$10^6$ observations).  By developing an efficient, general,
and (now) well-documented platform for fitted mixed models in
\proglang{R}, we hope to provide both a practical tool for end users
interested in analyzing data and a reusable, modular framework for
downstream developers interested in extending the class of models that
can be easily and efficiently analyzed in \proglang{R}.

%% Results of some quick Google scholar searches:
%% "mixed models" +
%%  stata: 6130
%%  "PROC MIXED": 12200
%%  CRAN: 4020
%%  lme4: 5110
%%  "nlme": 2960
%%  SAS: 49500
%%  NLMIXED: 1960
%%  GLIMMIX: 4920
%%  winbugs: 2250
%%  gllamm: 1140

We have learned much about linear mixed models in the process of
developing \pkg{lme4}, both from our own attempts to develop robust
and reliable procedures and from the broad community of \pkg{lme4}
users; we have attempted to describe many of these lessons here.  In
moving forward, our main priorities are (1) to maintain the reference
implementation of \pkg{lme4} on the Comprehensive \proglang{R} Archive Network (CRAN), developing relatively few new
features; (2) to improve the flexibility, efficiency and scalability
of mixed-model analysis across multiple compatible implementations,
including both the \pkg{MixedModels} package for \proglang{Julia} and
the experimental \pkg{flexLambda} branch of \pkg{lme4}.

\section*{Acknowledgments}

Rune Haubo Christensen, Henrik Singmann, Fabian Scheipl,
Vincent Dorie, and Bin Dai contributed ideas
and code to the current version of \pkg{lme4}; the
large community of \pkg{lme4} users has exercised the software,
discovered bugs, and made many useful suggestions.
S{\o}ren H{\o}jsgaard participated in useful discussions
and Xi Xia  and Christian Zingg exercised the code
and reported problems.
We would like to thank the Banff International Research Station
for hosting a working group on \pkg{lme4}, and
an NSERC Discovery grant and NSERC postdoctoral fellowship for funding.

\clearpage

\bibliography{lmer}

\clearpage
% \bigskip \par \pagebreak[3]

\begin{appendix}

\section{Modularization examples}
\label{sec:modularExamples}

The modularized functions in \pkg{lme4} allow for finer control of the
various steps in fitting mixed models (Table~\ref{tab:modular}).  The
main advantage of these functions is the ability to extend the class
of models that \code{lmer} is capable of fitting.  In general there
are three steps to this process.
\begin{itemize}
\item Construct a mixed-model formula that is similar to the intended
  model.
\item Using the modular functions, hook into and modify the fitting
  process so that the desired model is estimated.
\item Use the estimated parameter values to make inferences.
\end{itemize}
Note that the many tools of the output module become unavailable for
sufficiently extensive departures from the class of models fitted by
\code{lmer}.  Therefore, this last inference step may be more involved
in the modular approach than with inference from models fitted by
\code{lmer}.  Here we provide two examples that illustrate this
process.

\subsection{Zero slope-slope covariance models}
\label{sec:zeroSlopeSlope}

Consider the following mixed-model formula,
<<modularSimulationFormula>>=
form <- respVar ~ 1 + (explVar1 + explVar2 | groupFac)
@ %
in which we have a fixed intercept and two random-effects terms, each
with a random slope and intercept.  The \code{lmer} function would fit
this formula to data by estimating an unstructured $3 \times 3$
covariance matrix giving the variances and covariances among the
intercept and the two slopes.  It is not possible to use the current
version of \code{lmer} to fit a model with the following two
properties: (1) the slopes are uncorrelated and (2) the intercept is
correlated with both of the slopes.  We illustrate the use of the
modular functions in \pkg{lme4} to fit such a model.

\subsubsection{Simulate from a zero slope-slope covariance model}

We simulate data from such a model using a balanced design with a
single grouping factor.  We use \pkg{lme4}'s utilities for
constructing a template from the model formula, with which to conduct
these simulations.  We simulate a balanced design with 500 levels to
the grouping factor and 20 observations per group,
<<dataTemplate>>=
set.seed(1)
dat <- mkDataTemplate(form,
  nGrps = 500,
  nPerGrp = 20,
  rfunc = rnorm)
head(dat)
@ %
The initial parameters for this model may be obtained using
\code{mkParsTemplate}.
<<parsTemplate>>=
(pars <- mkParsTemplate(form, dat))
@ %
We wish to simulate random effects from the following covariance
matrix,
<<simCorr>>=
vc <- matrix(c(1.0, 0.5, 0.5,
  0.5, 1.0, 0.0,
  0.5, 0.0, 1.0), 3, 3)
@ %
Here the intercept is correlated with each of the two slopes, but the
slopes are uncorrelated.  The \code{theta} vector that corresponds to
this covariance matrix can be computed using \pkg{lme4}'s facilities
for converting between various representations of (co-)variance
structures (see \code{?vcconv}).
<<vcTheta>>=
pars$theta[] <- Vv_to_Cv(mlist2vec(vc))
@ %
With these new parameters installed, we simulate a response variable.
<<varCorrStructure>>=
dat$respVar <- simulate(form,
  newdata = dat,
  newparams = pars,
  family = "gaussian")[[1]]
@ %
Indeed this data set appears to contain intercept-slope correlations,
but not slope-slope correlations, as we see using the \code{lmList}
function.
<<graphSims>>=
formLm <- form
formLm[[3]] <- reformulas::findbars(form)[[1]]
print(formLm)
cor(t(sapply(lmList(formLm, dat), coef)))
@ %
This \code{lmList} function computes ordinary linear models for each
level of the grouping factor.  Next we show how to fit a proper mixed
model to these data.

\subsubsection{Fitting a zero slope-slope covariance model}

The main challenge with fitting a zero slope-slope model is the need
to figure out which Cholesky decompositions correspond to
variance-covariance matrices with structural zeros between the slopes.
There are two general approaches to solving this problem.  The first
is to reorder the rows and columns of the covariance matrix so that
the intercept column comes last.  In this case, setting the second
component of the covariance parameter vector, $\bm\theta$, to zero will
achieve the desired covariance structure.  However, we consider a
second approach, which retains the ordering produced by the
\code{lFormula} function: (1) intercept, (2) first slope, and (3)
second slope. The covariance structure for the random effects in this
model is given by the following covariance matrix and its associated
Cholesky decomposition.
\begin{equation}
  \label{eq:1}
  \begin{bmatrix}
    \sigma_1^2 & \sigma_{12} & \sigma_{13} \\
    \sigma_{21} & \sigma_2^2 & 0 \\
    \sigma_{31} & 0 & \sigma_3^2 \\
  \end{bmatrix} =
  \begin{bmatrix}
    \theta_1 & 0 & 0 \\
    \theta_2 & \theta_4 & 0 \\
    \theta_3 & \theta_5 & \theta_6 \\
  \end{bmatrix}
  \begin{bmatrix}
    \theta_1 & \theta_2 & \theta_3 \\
    0 & \theta_4 & \theta_5 \\
    0 & 0 & \theta_6 \\
  \end{bmatrix}
\end{equation}
where the $\sigma$ parameters give the standard deviations and
covariances, and the $\theta$ parameters are the elements of the
Cholesky decomposition.  The key feature of this covariance matrix is
the structural zero indicating no covariance between the two
slopes.  Therefore, in order to ensure the structural zero, we must
have,
\begin{equation}
  \label{eq:2}
  \theta_5 = -\frac{\theta_2\theta_3}{\theta_4}.
\end{equation}
To represent this restriction in \proglang{R} we introduce a length
five vector, \code{phi}, which we convert into an appropriate
\code{theta} vector using the following function.
<<phiToTheta, cache = FALSE>>=
phiToTheta <- function(phi) {
    theta5 <- -(phi[2]*phi[3])/phi[4]
    c(phi[1:4], theta5, phi[5])
}
@ %
We may therefore optimize the structured model by optimizing over
\code{phi} instead of \code{theta}.  We begin to construct an
appropriate objective function for such an optimization by taking the
first two modular steps, resulting in a deviance function of
\code{theta}.
<<compute deviance function modular, cache = FALSE>>=
lf <- lFormula(formula = form, data = dat, REML = TRUE)
devf <- do.call(mkLmerDevfun, lf)
@ %
This deviance function is a function of the unconstrained \code{theta}
vector, but we require a deviance function of the constrained
\code{phi} vector.  We therefore construct a wrapper around the
deviance function as follows.
<<wrapper modular, cache = FALSE>>=
devfWrap <- function(phi) devf(phiToTheta(phi))
@ %
Finally, we optimize the deviance function,
<<opt modular, cache = FALSE>>=
opt <- bobyqa(par = lf$reTrms$theta[-5],
  fn = devfWrap,
  lower = lf$reTrms$lower[-5])
@ %
To extract and label the estimated covariance matrix of the random
effects, we may use the conversion functions (\code{?vcconv}) and the
column names of the random-effects model matrix (\code{cnms}).  We
construct this estimated matrix and compare it with the true matrix
used to simulate the data.
<<varCorr modular, cache = FALSE>>=
vcEst <- vec2mlist(Cv_to_Vv(phiToTheta(opt$par)))[[1]]
dimnames(vcEst) <- rep(lf$reTrms$cnms, 2)
round(vcEst, 2)
vc
@ %
We see that the estimated covariance matrix provides a good estimate
of the true matrix.

\subsection{Additive models}
\label{sec:additive}

It is possible to interpret additive models as a particular class of
mixed models \citep{gamm4}.  The main benefit of this approach is
that it bypasses the need to select a smoothness parameter using
cross-validation or generalized cross-validation.  However, it is
inconvenient to specify additive models using the \pkg{lme4} formula
interface.  The \pkg{gamm4} package wraps the modularized functions of
\pkg{lme4} within a more convenient interface \citep{gamm4}.  In
particular, the strategy involves the following steps,
\begin{enumerate}
\item Convert a \pkg{gamm4} formula into an \pkg{lme4} formula that
  approximates the intended model.
\item Parse this formula using \code{lFormula}.
\item Modify the resulting transposed random-effects model matrix, $\bm
  Z\trans$, so that the intended additive model results.
\item Fit the resulting model using the remaining modularized
  functions (Table~\ref{tab:modular}).
\end{enumerate}
Here we illustrate this general strategy using a simple simulated data
set,
<<simulateSplineData,message=FALSE>>=
library("gamm4")
library("splines")
set.seed(1)
n <- 100
pSimulation <- 3
pStatistical <- 8
x <- rnorm(n)
Bsimulation <- ns(x, pSimulation)
Bstatistical <- ns(x, pStatistical)
beta <- rnorm(pSimulation)
y <- as.numeric(Bsimulation %*% beta + rnorm(n, sd = 0.3))
@ %
where \code{pSimulation} is the number of columns in the simulation
model matrix and \code{pStatistical} is the number of columns in the
statistical model matrix.

We plot the resulting data along with the predictions from the
generating model,  %% BMB: stopped here
<<splineExampleDataPlot, fig.width=4, fig.height=3,fig.align="center">>=
par(mar = c(4, 4, 1, 1), las = 1, bty = "l")
plot(x, y, las = 1)
lines(x[order(x)], (Bsimulation %*% beta)[order(x)])
@ %
We now set up an approximate \pkg{lme4} model formula, and parse it
using \code{lFormula},
<<splineExampleApproximateFormula>>=
pseudoGroups <- as.factor(rep(1:pStatistical, length = n))
parsedFormula <- lFormula(y ~ x + (1 | pseudoGroups))
@ %
We now insert a spline basis into the \code{parsedFormula} object,
<<splineExampleModifyZt>>=
parsedFormula$reTrms <- within(parsedFormula$reTrms, {
    Bt <- t(as.matrix(Bstatistical))[]
    cnms$pseudoGroups <- "spline"
    Zt <- as(Bt, class(Zt))
})
@ %
Finally we continue with the remaining modular steps,
<<splineExampleRemainingModularSteps>>=
devianceFunction <- do.call(mkLmerDevfun, parsedFormula)
optimizerOutput <- optimizeLmer(devianceFunction)
mSpline <- mkMerMod(   rho = environment(devianceFunction),
                       opt = optimizerOutput,
                    reTrms = parsedFormula$reTrms,
                        fr = parsedFormula$fr)
mSpline
@ %
Computing the fitted values of this additive model requires some
custom code, and illustrates the general principle that methods for
\code{merMod} objects constructed from modular fits should only be
used if the user knows what she is doing,
<<splineExampleFittedModelPlot, fig.width=4, fig.height=3, fig.align="center">>=
xNew <- seq(min(x), max(x), length = 100)
newBstatistical <- predict(Bstatistical, xNew)
yHat <-   cbind(1, xNew) %*% getME(mSpline, "fixef") +
        newBstatistical %*% getME(mSpline, "u")
par(mar = c(4, 4, 1, 1), las = 1, bty = "l")
plot(x, y)
lines(xNew, yHat)
lines(x[order(x)], (Bsimulation %*% beta)[order(x)],lty = 2)
legend("topright", bty = "n", c("fitted", "generating"), lty = 1:2,col = 1)
@
\end{appendix}

\end{document}

%

%%% Local Variables:
%%% ess-swv-processor: knitr
%%% TeX-master: t
%%% End:

%%\VignetteIndexEntry{Computational Methods}
%%\VignetteDepends{lme4}
%%\VignetteEngine{knitr::knitr}
\documentclass[12pt]{article}
\usepackage{amsmath,amsfonts,bm}
\usepackage[authoryear,round]{natbib}
\bibliographystyle{plainnat}
\title{Computational methods for mixed models}
\author{Douglas Bates\\Department of Statistics\\%
  University of Wisconsin -- Madison}
\newcommand{\code}[1]{\texttt{\small{#1}}}
\newcommand{\package}[1]{\textsf{\small{#1}}}
\newcommand{\trans}{\ensuremath{^\prime}}
\renewcommand{\vec}{\operatorname{vec}}
\newcommand{\diag}{\operatorname{diag}}
\newcommand{\bc}[1]{\ensuremath{\bm{\mathcal{#1}}}}
<<preliminaries,include=FALSE>>=
knitr::opts_chunk$set(include=FALSE)
options(width=65,digits=5)
#library(lme4)
@

\begin{document}
\maketitle

\begin{abstract}
  The \package{lme4} package provides R functions to fit and analyze
  several different types of mixed-effects models, including linear
  mixed models, generalized linear mixed models and nonlinear mixed
  models.  In this vignette we describe the formulation of these
  models and the computational approach used to evaluate or
  approximate the log-likelihood of a model/data/parameter value
  combination.
\end{abstract}

\section{Introduction}
\label{sec:intro}

The \package{lme4} package provides \code{R} functions to fit and analyze
linear mixed models, generalized linear mixed models and nonlinear
mixed models.  These models are called \emph{mixed-effects models} or,
more simply, \emph{mixed models} because they incorporate both
\emph{fixed-effects} parameters, which apply to an entire population
or to certain well-defined and repeatable subsets of a population, and
\emph{random effects}, which apply to the particular experimental
units or observational units in the study.  Such models are also
called \emph{multilevel} models because the random effects represent
levels of variation in addition to the per-observation noise term that
is incorporated in common statistical models such as linear
regression models, generalized linear models and nonlinear regression
models.

We begin by describing common properties of these mixed models and the
general computational approach used in the \package{lme4} package. The
estimates of the parameters in a mixed model are determined as the
values that optimize an objective function --- either the likelihood
of the parameters given the observed data, for maximum likelihood
(ML) estimates, or a related objective function called the REML
criterion.  Because this objective function must be evaluated at many
different values of the model parameters during the optimization
process, we focus on the evaluation of the objective function and a
critical computation in this evalution --- determining the solution to a
penalized, weighted least squares (PWLS) problem.

The dimension of the solution of the PWLS problem can be very large,
perhaps in the millions.  Furthermore, such problems must be solved
repeatedly during the optimization process to determine parameter
estimates.  The whole approach would be infeasible were it not for the
fact that the matrices determining the PWLS problem are sparse and we
can use sparse matrix storage formats and sparse matrix computations
\citep{davis06:csparse_book}. In particular, the whole computational
approach hinges on the extraordinarily efficient methods for
determining the Cholesky decomposition of sparse, symmetric,
positive-definite matrices embodied in the CHOLMOD library of C
functions \citep{Cholmod}.

% The three types of mixed models -- linear, generalized linear and
% nonlinear -- share common characteristics in that the model is
% specified in whole or in part by a \emph{mixed model formula} that
% describes a \emph{linear predictor} and a variance-covariance
% structure for the random effects.  In the next section we describe
% the mixed model formula and the forms of these matrices.  The
% following section presents a general formulation of the Laplace
% approximation to the log-likelihood of a mixed model.

% In subsequent sections we describe computational methods for specific
% kinds of mixed models.  In particular, we should how a profiled
% log-likelihood for linear mixed models, and for some nonlinear mixed
% models, can be evaluated exactly.

In the next section we describe the general form of the mixed models
that can be represented in the \package{lme4} package and the
computational approach embodied in the package.  In the following
section we describe a particular form of mixed model,
called a linear mixed model, and the computational details for those
models.  In the fourth section we describe computational methods for
generalized linear mixed models, nonlinear mixed models and
generalized nonlinear mixed models.


\section{Formulation of mixed models}
\label{sec:form-mixed-models}

A mixed-effects model incorporates two vector-valued random variables:
the $n$-dimensional response vector, $\bc Y$, and the $q$-dimensional
random effects vector, $\bc B$. We observe the value, $\bm y$, of $\bc Y$.
We do not observe the value of $\bc B$.

The random variable $\bc Y$ may be continuous or discrete.  That is,
the observed data, $\bm y$, may be on a continuous scale or they may
be on a discrete scale, such as binary responses or responses
representing a count. In our formulation, the random variable $\bc B$
is always continous.

We specify a mixed model by describing the unconditional distribution
of $\bc B$ and the conditional distribution $(\bc Y|\bc B=\bm b)$.

\subsection{The unconditional distribution of $\bc B$}
\label{sec:uncond-distr-B}

In our formulation, the unconditional distribution of $\bc B$ is
always a $q$-dimensional multivariate Gaussian (or ``normal'')
distribution with mean $\bm 0$ and with a parameterized covariance
matrix,
\begin{equation}
  \label{eq:2}
  \bc B\sim\mathcal{N}\left(\bm 0,\sigma^2\bm\Lambda(\bm\theta)
  \bm\Lambda\trans(\bm\theta)\right) .
\end{equation}

The scalar, $\sigma$, in (\ref{eq:2}), is called the \emph{common scale
  parameter}.  As we will see later, not all types of mixed models
incorporate this parameter. We will include $\sigma^2$ in the general
form of the unconditional distribution of $\bc B$ with the
understanding that, in some models, $\sigma\equiv 1$.

The $q\times q$ matrix $\bm\Lambda(\bm\theta)$, which is a left factor
of the covariance matrix (when $\sigma=1$) or the relative covariance
matrix (when $\sigma\ne 1$), depends on an $m$-dimensional parameter
$\bm\theta$.  Typically $m\ll q$; in the examples we show below it is
always the case that $m<5$, even when $q$ is in the thousands.  The
fact that $m$ is very small is important because, as we shall see,
determining the parameter estimates in a mixed model can be expressed
as an optimization problem with respect to $\bm\theta$ only.

The parameter $\bm\theta$ may be, and typically is, subject to
constraints. For ease of computation, we require that the constraints
be expressed as ``box'' constraints of the form
$\theta_{iL}\le\theta_i\le\theta_{iU},i=1,\dots,m$ for constants
$\theta_{iL}$ and $\theta_{iU}, i=1,\dots,m$.  We shall write the set
of such constraints as $\bm\theta_L\le\bm\theta\le\bm\theta_R$.  The matrix
$\bm\Lambda(\bm\theta)$ is required to be non-singular
(i.e.{} invertible) when $\bm\theta$ is not on the boundary.

\subsection{The conditional distribution,  $(\bc Y|\bc B=\bm b)$}
\label{sec:cond-distr-YB}

The conditional distribution, $(\bc Y|\bc B=\bm b)$, must satisfy:
\begin{enumerate}
\item The conditional mean, $\bm\mu_{\bc Y|\bc B}(\bm b) =
  \mathrm{E}[\bc Y|\bc B=\bm b]$, depends on $\bm b$ only through the
  value of the \emph{linear predictor}, $\bm Z\bm b+\bm X\bm\beta$,
  where $\bm\beta$ is the $p$-dimensional \emph{fixed-effects}
  parameter vector and the \emph{model matrices}, $\bm Z$ and $\bm X$,
  are fixed matrices of the appropriate dimension.  That is, the
  two model matrices must have the same number of rows and must have
  $q$ and $p$ columns, respectively.  The number of rows
  in $\bm Z$ and $\bm X$ is a multiple of $n$, the dimension of $\bm y$.
\item The scalar distributions, $(\mathcal{Y}_i|\bc B=\bm
  b),i=1,\dots,n$, all have the same form and are completely
  determined by the conditional mean, $\bm\mu_{\bc Y|\bc B}(\bm b)$
  and, at most, one additional parameter, $\sigma$, which is the
  common scale parameter.
\item The scalar distributions, $(\mathcal{Y}_i|\bc B=\bm
  b),i=1,\dots,n$, are independent.  That is, the components of
  $\bc Y$ are \emph{conditionally independent} given $\bc B$.
\end{enumerate}

An important special case of the conditional distribution is the
multivariate Gaussian distribution of the form
\begin{equation}
  \label{eq:1}
  (\bc Y|\bc B=\bm b)\sim\mathcal{N}(\bm Z\bm b+\bm
  X\bm\beta,\sigma^2\bm I_n)
\end{equation}
where $\bm I_n$ denotes the identity matrix of size $n$.
In this case the conditional mean, $\bm\mu_{\bc Y|\bc B}(\bm b)$, is
exactly the linear predictor, $\bm Z\bm b+\bm X\bm\beta$, a situation
we will later describe as being an ``identity link'' between the
conditional mean and the linear predictor.  Models with conditional
distribution (\ref{eq:1}) are called \emph{linear mixed models}.

\subsection{A change of variable to ``spherical'' random effects}
\label{sec:change-vari-spher}

Because the conditional distribution $(\bc Y|\bc B=\bm b)$ depends on
$\bm b$ only through the linear predictor, it is easy to express the
model in terms of a linear transformation of $\bc B$.  We define the
linear transformation from a $q$-dimensional ``spherical'' Gaussian
random variable, $\bc U$, to $\bc B$ as
\begin{equation}
  \label{eq:3}
  \bc B=\bm\Lambda(\bm\theta)\bc U,\quad
  \bc U\sim\mathcal{N}(\bm 0,\sigma^2\bm I_q).
\end{equation}
(The term ``spherical'' refers to the fact that contours of constant
probability density for $\bc U$ are spheres centered at the mean ---
in this case, $\bm0$.)

When $\bm\theta$ is not on the boundary this is an invertible
transformation.  When $\bm\theta$ is on the boundary the
transformation can fail to be invertible.  However, we will only need
to be able to express $\bc B$ in terms of $\bc U$ and that
transformation is well-defined, even when $\bm\theta$ is on the
boundary.

The linear predictor, as a function of $\bm u$, is
\begin{equation}
  \label{eq:4}
  \bm\gamma(\bm u)=\bm Z\bm\Lambda(\bm\theta)\bm u
  + \bm X\bm\beta.
\end{equation}
When we wish to emphasize the role of the model parameters,
$\bm\theta$ and $\bm\beta$, in the formulation of $\bm\gamma$, we will
write the linear predictor as $\bm\gamma(\bm u,\bm\theta,\bm\beta)$.

\subsection{The conditional density $(\bc U|\bc Y=\bm y)$}
\label{sec:cond-dens-bc}

Because we observe $\bm y$ and do not observe $\bm b$ or $\bm u$, the
conditional distribution of interest, for the purposes of statistical
inference, is $(\bc U|\bc Y=\bm y)$ (or, equivalently, $(\bc B|\bc
Y=\bm y)$).  This conditional distribution is always a continuous
distribution with conditional probability density $f_{\bc U|\bc
  Y}(\bm u|\bm y)$.

We can evaluate $f_{\bc U|\bc Y}(\bm u|\bm y)$ , up to a constant, as
the product of the unconditional density, $f_{\bc U}(\bm u)$, and the
conditional density (or the probability mass function, whichever is
appropriate), $f_{\bc Y|\bc U}(\bm y|\bm u)$.  We write this
unnormalized conditional density as
\begin{equation}
  \label{eq:5}
  h(\bm u|\bm y,\bm\theta,\bm\beta,\sigma) =
  f_{\bc Y|\bc U}(\bm y|\bm u,\bm\theta,\bm\beta,\sigma)
  f_{\bc U}(\bm u|\sigma) .
\end{equation}

We say that $h$ is the ``unnormalized'' conditional density because
all we know is that the conditional density is proportional to $h(\bm
u|\bm y,\bm\theta,\bm\beta,\sigma)$.  To obtain the conditional
density we must normalize $h$ by dividing by the value of the integral
\begin{equation}
  \label{eq:6}
  L(\bm\theta,\bm\beta,\sigma|\bm y) =
  \int_{\mathbb{R}^q}h(\bm u|\bm y,\bm\theta,\bm\beta,\sigma)\,d\bm u .
\end{equation}
We write the value of the integral (\ref{eq:6}) as
$L(\bm\theta,\bm\beta,\sigma|\bm y)$ because it is exactly the
\emph{likelihood} of the parameters $\bm\theta$, $\bm\beta$ and
$\sigma$, given the observed data $\bm y$.  The \emph{maximum
  likelihood (ML) estimates} of these parameters are the values that
maximize $L$.

\subsection{Determining the ML estimates}
\label{sec:DeterminingML}

The general problem of maximizing $L(\bm\theta,\bm\beta,\sigma|\bm y)$
with respect to $\bm\theta$, $\bm\beta$ and $\sigma$ can be formidable
because each evaluation of this function involves a potentially
high-dimensional integral and because the dimension of $\bm\beta$ can
be large.  However, this general optimization problem can be split
into manageable subproblems.  Given a value of $\bm\theta$ we can
determine the \emph{conditional mode}, $\tilde{\bm u}(\bm\theta)$, of
$\bm u$ and the \emph{conditional estimate},
$\tilde{\bm\beta}(\bm\theta)$ simultaneously using \emph{penalized,
  iteratively re-weighted least squares} (PIRLS).  The conditional
mode and the conditional estimate are defined as
\begin{equation}
  \label{eq:condmode}
  \begin{bmatrix}
    \tilde{\bm u}(\bm\theta)\\
    \tilde{\bm\beta}(\bm\theta)
  \end{bmatrix}=\arg\max_{\bm u,\bm\beta}h(\bm u|\bm
  y,\bm\theta,\bm\beta,\sigma) .
\end{equation}
(It may look as if we have missed the dependence on $\sigma$ on the
left-hand side but it turns out that the scale parameter does not
affect the location of the optimal values of quantities in the linear
predictor.)

As is common in such optimization problems, we re-express the conditional
density on the \emph{deviance scale}, which is negative twice the
logarithm of the density, where the optimization becomes
\begin{equation}
  \label{eq:condmode2}
  \begin{bmatrix}
    \tilde{\bm u}(\bm\theta)\\
    \tilde{\bm\beta}(\bm\theta)
  \end{bmatrix}=\arg\min_{\bm u,\bm\beta}-2\log\left(h(\bm u|\bm
  y,\bm\theta,\bm\beta,\sigma)\right) .
\end{equation}
It is this optimization problem that can be solved quite efficiently
using PIRLS.  In fact, for linear mixed models, which are described in
the next section, $\tilde{\bm u}(\bm\theta)$ and
$\tilde{\bm\beta}(\bm\theta)$ can be directly evaluated.

The second-order Taylor series expansion of $-2\log h$ at $\tilde{\bm
  u}(\bm\theta)$ and $\tilde{\bm\beta}(\bm\theta)$ provides the
Laplace approximation to the profiled deviance.  Optimizing this
function with respect to $\bm\theta$ provides the ML estimates of
$\bm\theta$, from which the ML estimates of $\bm\beta$ and $\sigma$
(if used) are derived.

\section{Methods for linear mixed models}
\label{sec:pwls-problem}

As indicated in the introduction, a critical step in our methods for
determining the maximum likelihood estimates of the parameters in a
mixed model is solving a penalized, weighted least squares (PWLS)
problem.  We will motivate the general form of the PWLS problem by
first considering computational methods for linear mixed models that
result in a penalized least squares (PLS) problem.

Recall from \S\ref{sec:cond-distr-YB} that, in a linear mixed model,
both the conditional distribution, $(\bc Y|\bc U=\bm u)$, and the
unconditional distribution, $\bc U$, are spherical Gaussian
distributions and that the conditional mean, $\bm\mu_{\bc Y|\bc U}(\bm
u)$, is the linear predictor, $\bm\gamma(\bm u)$.  Because all the
distributions determining the model are continuous distributions, we
consider their densities.  On the deviance scale these are
\begin{equation}
  \label{eq:7}
  \begin{aligned}
    -2\log(f_{\bc U}(\bm u))&=q\log(2\pi\sigma^2)+\frac{\|\bm u\|^2}{\sigma^2}\\
    -2\log(f_{\bc Y|\bc U}(\bm y|\bm u))&=n\log(2\pi\sigma^2)+
    \frac{\|\bm y-\bm Z\bm\Lambda(\bm\theta)\bm u-\bm X\bm\beta\|^2}{\sigma^2}\\
    -2\log(h(\bm u|\bm y,\bm\theta,\bm\beta,\sigma))
    &= (n+q)\log(2\pi\sigma^2)+
    \frac{\|\bm y-\bm\gamma(\bm u,\bm\theta,\bm\beta)\|^2+\|\bm
      u\|^2}{\sigma^2}\\
    &= (n+q)\log(2\pi\sigma^2)+
    \frac{d(\bm u|\bm y,\bm\theta,\bm\beta)}{\sigma^2}
  \end{aligned}
\end{equation}

In (\ref{eq:7}) the \emph{discrepancy} function,
\begin{equation}
  \label{eq:9}
    d(\bm u|\bm y,\bm\theta,\bm\beta) =
    \|\bm y-\bm\gamma(\bm u,\bm\theta,\bm\beta)\|^2+\|\bm u\|^2
\end{equation}
has the form of a penalized residual sum of squares in that the first
term, $\|\bm y-\bm\gamma(\bm u,\bm\theta,\bm\beta)\|^2$ is
the residual sum of squares for $\bm y$, $\bm u$, $\bm\theta$ and
$\bm\beta$ and the second term, $\|\bm u\|^2$, is a penalty on the
size of $\bm u$.  Notice that the discrepancy does not depend on the
common scale parameter, $\sigma$.

\subsection{The canonical form of the discrepancy}
\label{sec:conditional-mode-bm}

Using a so-called ``pseudo data'' representation, we can write the
discrepancy as a residual sum of squares for a regression model that
is linear in both $\bm u$ and $\bm\beta$
\begin{equation}
  \label{eq:10}
  d(\bm u|\bm y,\bm\theta,\bm\beta) =\left\|
    \begin{bmatrix} \bm y\\\bm 0 \end{bmatrix} -
    \begin{bmatrix}
      \bm Z\bm\Lambda(\bm\theta) & \bm X \\
      \bm I_q & \bm0
    \end{bmatrix}
    \begin{bmatrix}\bm u\\\bm\beta\end{bmatrix}
  \right\|^2 .
\end{equation}
The term ``pseudo data'' reflects the fact that we have added $q$
``pseudo observations'' to the observed response, $\bm y$, and to the
linear predictor, $\bm\gamma(\bm u,\bm\theta,\bm\beta)=\bm
Z\bm\Lambda(\bm\theta)\bm u+\bm X\bm\beta$, in such a way that their
contribution to the overall residual sum of squares is exactly the
penalty term in the discrepancy.

In the form (\ref{eq:10}) we can see that the discrepancy is a
quadratic form in both $\bm u$ and $\bm\beta$.  Furthermore, because
we require that $\bm X$ has full column rank, the discrepancy is a
positive-definite quadratic form in $\bm u$ and $\bm\beta$ that is
minimized at $\tilde{\bm u}(\bm\theta)$ and
$\tilde{\bm\beta}(\bm\theta)$ satisfying
\begin{equation}
  \label{eq:13}
  \begin{bmatrix}
    \bm\Lambda\trans(\bm\theta)\bm Z\trans\bm Z\bm\Lambda(\theta)
    +\bm I_q&\bm\Lambda\trans(\bm\theta)\bm Z\trans\bm X\\
    \bm X\trans\bm Z\bm\Lambda(\theta) &\bm X\trans\bm X
  \end{bmatrix}
  \begin{bmatrix}
    \tilde{\bm u}(\bm\theta)\\\tilde{\bm\beta}(\bm\theta)
  \end{bmatrix} =
  \begin{bmatrix}
    \bm\Lambda\trans(\bm\theta)\bm Z\trans\bm y\\
    \bm X\trans\bm y
  \end{bmatrix}
\end{equation}

An effective way of determining the solution to a sparse, symmetric,
positive definite system of equations such as (\ref{eq:13}) is the
sparse Cholesky decomposition \citep{davis06:csparse_book}.  If $\bm A$
is a sparse, symmetric positive definite matrix then the sparse
Cholesky factor with fill-reducing permutation $\bm P$ is the
lower-triangular matrix $\bm L$ such that
\begin{equation}
  \label{eq:14}
  \bm L\bm L\trans=\bm P\bm A\bm P\trans .
\end{equation}
(Technically, the factor $\bm L$ is only determined up to changes in
the sign of the diagonal elements.  By convention we require the
diagonal elements to be positive.)

The fill-reducing permutation represented by the permutation matrix
$\bm P$, which is determined from the pattern of nonzeros in $\bm A$
but does not depend on particular values of those nonzeros, can have a
profound impact on the number of nonzeros in $\bm L$ and hence on the
speed with which $\bm L$ can be calculated from $\bm A$.

In most applications of linear mixed models the matrix $\bm
Z\bm\Lambda(\bm\theta)$ is sparse while $\bm X$ is dense or close to
it so the permutation matrix $\bm P$ can be restricted to the form
\begin{equation}
  \label{eq:15}
  \bm P=\begin{bmatrix}\bm P_{\bm Z}&\bm0\\
    \bm0&\bm P_{\bm X}\end{bmatrix}
\end{equation}
without loss of efficiency.  In fact, in most cases we can set $\bm
P_{\bm X}=\bm I_p$ without loss of efficiency.

Let us assume that the permutation matrix is required to be of the
form (\ref{eq:15}) so that we can write the Cholesky factorization for
the positive definite system (\ref{eq:13}) as
\begin{multline}
  \label{eq:16}
  \begin{bmatrix}
    \bm L_{\bm Z}&\bm0\\\bm L_{\bm{XZ}}&\bm L_{\bm X}
  \end{bmatrix}
  \begin{bmatrix}
    \bm L_{\bm Z}&\bm0\\\bm L_{\bm{XZ}}&\bm L_{\bm X}
  \end{bmatrix}\trans =\\
  \begin{bmatrix}\bm P_{\bm Z}&\bm0\\
    \bm0&\bm P_{\bm X}\end{bmatrix}
  \begin{bmatrix}
    \bm\Lambda\trans(\bm\theta)\bm Z\trans\bm Z\bm\Lambda(\theta)
    +\bm I_q&\bm\Lambda\trans(\bm\theta)\bm Z\trans\bm X\\
    \bm X\trans\bm Z\bm\Lambda(\theta) &\bm X\trans\bm X
  \end{bmatrix}
  \begin{bmatrix}\bm P_{\bm Z}&\bm0\\
    \bm0&\bm P_{\bm X}\end{bmatrix}\trans .
\end{multline}
The discrepancy can now be written in the canonical form
\begin{equation}
  \label{eq:17}
  d(\bm u|\bm y,\bm\theta,\bm\beta) =\tilde{d}(\bm y,\bm\theta) +
  \left\|
    \begin{bmatrix}
      \bm L_{\bm Z}\trans&\bm L_{\bm{XZ}}\trans\\
      \bm 0&\bm L_{\bm X}\trans
    \end{bmatrix}
    \begin{bmatrix}
      \bm P_{\bm Z}(\bm u-\tilde{\bm u})\\
      \bm P_{\bm X}(\bm\beta-\tilde{\bm\beta})
    \end{bmatrix}
  \right\|^2
\end{equation}
where
\begin{equation}
  \label{eq:18}
  \tilde{d}(\bm y,\bm\theta)=
  d(\tilde{\bm u}(\bm\theta)|\bm y,\bm\theta,\tilde{\bm\beta}(\bm\theta))
\end{equation}
is the minimum discrepancy, given $\bm\theta$.


\subsection{The profiled likelihood for linear mixed models}
\label{sec:prof-log-likel}

Substituting (\ref{eq:17}) into (\ref{eq:7}) provides the unnormalized
conditional density $h(\bm u|\bm y,\bm\theta,\bm\beta,\sigma)$ on the
deviance scale as
\begin{multline}
  \label{eq:32}
  -2\log(h(\bm u|\bm y,\bm\theta,\bm\beta,\sigma))\\=
  (n+q)\log(2\pi\sigma^2)+\frac{\tilde{d}(\bm y,\bm\theta) +
  \left\|
    \begin{bmatrix}
      \bm L_{\bm Z}\trans&\bm L_{\bm{XZ}}\trans\\
      \bm 0&\bm L_{\bm X}\trans
    \end{bmatrix}
    \begin{bmatrix}
      \bm P_{\bm Z}(\bm u-\tilde{\bm u})\\
      \bm P_{\bm X}(\bm\beta-\tilde{\bm\beta})
    \end{bmatrix}
  \right\|^2}{\sigma^2} .
\end{multline}
As shown in Appendix \ref{sec:integr-quadr-devi}, the integral of a
quadratic form on the deviance scale, such as (\ref{eq:32}), is easily
evaluated, providing the log-likelihood,
$\ell(\bm\theta,\bm\beta,\sigma|\bm y)$, as
\begin{multline}
  \label{eq:lmmdev}
  -2\ell(\bm\theta,\bm\beta,\sigma|\bm y)\\
  =-2\log\left(L(\bm\theta,\bm\beta,\sigma|\bm y)\right)\\
  =n\log(2\pi\sigma^2)+\log(|\bm L_{\bm Z}|^2)+\frac{\tilde{d}(\bm y,\bm\theta) +
  \left\|\bm L_{\bm X}\trans\bm P_{\bm X}(\bm\beta-\tilde{\bm\beta})\right\|^2}{\sigma^2},
\end{multline}
from which we can see that the conditional estimate of $\bm\beta$,
given $\bm\theta$, is $\tilde{\bm\beta}(\bm\theta)$ and the conditional
estimate of $\sigma$, given $\bm\theta$, is
\begin{equation}
  \label{eq:condsigma}
  \tilde{\sigma^2}(\bm\theta)= \frac{\tilde{d}(\bm\theta|\bm y)}{n} .
\end{equation}
Substituting these conditional estimates into (\ref{eq:lmmdev}) produces the
\emph{profiled likelihood}, $\tilde{L}(\bm\theta|\bm y)$, as
\begin{equation}
  \label{eq:19}
  -2\tilde{\ell}(\bm\theta|\bm y))=
  \log(|\bm L_{\bm Z}(\bm\theta)|^2)+
  n\left(1+\log\left(\frac{2\pi\tilde{d}(\bm y,\bm\theta)}{n}\right)\right) .
\end{equation}
The maximum likelihood estimate of $\bm\theta$ can then be expressed as
\begin{equation}
  \label{eq:29}
  \widehat{\bm\theta}_L=\arg\min_{\bm\theta}
  \left(-2\tilde{\ell}(\bm\theta|\bm y)\right) .
\end{equation}
from which the ML estimates of $\sigma^2$ and $\bm\beta$ are
evaluated as
\begin{align}
  \label{eq:30}
  \widehat{\sigma^2_L}&=
  \frac{\tilde{d}(\widehat{\bm\theta}_L,\bm y)}{n}\\
  \widehat{\bm\beta}_L&=\tilde{\bm\beta}(\widehat{\bm\theta}_L) .
\end{align}

The important thing to note about optimizing the profiled likelihood,
(\ref{eq:19}), is that it is a $m$-dimensional optimization problem
and typically $m$ is very small.

\subsection{The REML criterion}
\label{sec:reml-criterion}

In practice the so-called REML estimates of variance components are
often preferred to the maximum likelihood estimates.  (``REML'' can be
considered to be an acronym for ``restricted'' or ``residual'' maximum
likelihood, although neither term is completely accurate because these
estimates do not maximize a likelihood.) We can motivate the use of
the REML criterion by considering a linear regression model,
\begin{equation}
  \label{eq:20}
  \bc Y\sim\mathcal{N}(\bm X\bm\beta,\sigma^2\bm I_n),
\end{equation}
in which we typically estimate $\sigma^2$ by
\begin{equation}
  \label{eq:21}
  \widehat{\sigma^2_R}=\frac{\|\bm y-\bm X\widehat{\bm\beta}\|^2}{n-p}
\end{equation}
even though the maximum likelihood estimate of $\sigma^2$ is
\begin{equation}
  \label{eq:22}
  \widehat{\sigma^2_{L}}=\frac{\|\bm y-\bm
    X\widehat{\bm\beta}\|^2}{n} .
\end{equation}

The argument for preferring $\widehat{\sigma^2_R}$ to
$\widehat{\sigma^2_{L}}$ as an estimate of $\sigma^2$ is that the
numerator in both estimates is the sum of squared residuals at
$\widehat{\bm\beta}$ and, although the residual vector $\bm y-\bm
X\bm\beta$ is an $n$-dimensional vector, the residual at
$\widehat{\bm\theta}$ satisfies $p$ linearly independent constraints,
$\bm X\trans(\bm y-\bm X\widehat{\bm\beta})=\bm 0$. That is, the residual at
$\widehat{\bm\theta}$ is the projection of the observed response
vector, $\bm y$, into an $(n-p)$-dimensional linear subspace of the
$n$-dimensional response space.  The estimate $\widehat{\sigma^2_R}$
takes into account the fact that $\sigma^2$ is estimated from
residuals that have only $n-p$ \emph{degrees of freedom}.

The REML criterion for determining parameter estimates
$\widehat{\bm\theta}_R$ and $\widehat{\sigma_R^2}$ in a linear mixed
model has the property that these estimates would
specialize to $\widehat{\sigma^2_R}$ from (\ref{eq:21}) for a linear
regression model.  Although not usually derived in this way, the REML
criterion can be expressed as
\begin{equation}
  \label{eq:23}
  c_R(\bm\theta,\bm\sigma|\bm y)=-2\log
  \int_{\mathbb{R}^p}L(\bm u|\bm
  y,\bm\theta,\bm\beta,\sigma)\,d\bm\beta
\end{equation}
on the deviance scale.  The REML estimates $\widehat{\bm\theta}_R$ and
$\widehat{\sigma_R^2}$ minimize $c_R(\bm\theta,\bm\sigma|\bm y)$.

The profiled REML criterion, a function of $\bm\theta$ only, is
\begin{equation}
  \label{eq:24}
  \tilde{c}_R(\bm\theta|\bm y)=
  \log(|\bm L_{\bm Z}(\bm\theta)|^2|\bm L_{\bm X}(\bm\theta)|^2)+(n-p)
  \left(1+\log\left(\frac{2\pi\tilde{d}(\bm\theta|\bm y)}{n-p}\right)\right)
\end{equation}
and the REML estimate of $\bm\theta$ is
\begin{equation}
  \label{eq:31}
  \widehat{\bm\theta}_R =
  \arg\min_{\bm\theta}\tilde{c}_R(\bm\theta,\bm y) .
\end{equation}
The REML estimate of $\sigma^2$ is
$\widehat{\sigma^2_R}=\tilde{d}(\widehat{\bm\theta}_R|\bm y)/(n-p)$.

It is not entirely clear how one would define a ``REML estimate'' of
$\bm\beta$ because the REML criterion, $c_R(\bm\theta,\bm\sigma|\bm
y)$, defined in (\ref{eq:23}), does not depend on $\bm\beta$.
However, it is customary (and not unreasonable) to use
$\widehat{\bm\beta}_R=\tilde{\bm\beta}(\widehat{\bm\theta}_R)$ as the REML
estimate of $\bm\beta$.

Note that the profiled REML criterion can be evaluated from a sparse
Cholesky decomposition like that in (\ref{eq:16}) but without the
requirement that the permutation can be applied to the columns of $\bm
Z\bm\Lambda(\bm\theta)$ separately from the columnns of $\bm X$.  That
is, we can use a general fill-reducing permutation rather than the
specific form (\ref{eq:15}) with separate permutations represented by
$\bm P_{\bm Z}$ and $\bm P_{\bm X}$. This can be useful in cases where
both $\bm Z$ and $\bm X$ are large and sparse.

\subsection{Summary for linear mixed models}
\label{sec:lmmsummary}

A linear mixed model is characterized by the conditional distribution
\begin{equation}
  \label{eq:lmmcond}
  (\bc Y|\bc U=\bm u)\sim\mathcal{N}(\bm\gamma(\bm
  u,\bm\theta,\bm\beta),\sigma^2\bm I_n)\text{ where }
  \bm\gamma(\bm u,\bm\theta,\bm\beta)=\bm Z\bm\Lambda(\bm\theta)\bm
  u+\bm X\bm\beta
\end{equation}
and the unconditional distribution
$\bc U\sim\mathcal{N}(\bm 0,\sigma^2\bm I_q)$.  The discrepancy
function,
\begin{displaymath}
  d(\bm u|\bm y,\bm\theta,\bm\beta)=
  \left\|\bm y-\bm\gamma(\bm u,\bm\theta,\bm\beta)\right\|^2+\|\bm u\|^2,
\end{displaymath}
is minimized at the conditional mode, $\tilde{\bm u}(\bm\theta)$, and
the conditional estimate, $\tilde{\bm\beta}(\bm\theta)$, which are the
solutions to the sparse, positive-definite linear system
\begin{displaymath}
  \begin{bmatrix}
    \bm\Lambda\trans(\bm\theta)\bm Z\trans\bm Z\bm\Lambda(\theta)
    +\bm I_q&\bm\Lambda\trans(\bm\theta)\bm Z\trans\bm X\\
    \bm X\trans\bm Z\bm\Lambda(\theta) &\bm X\trans\bm X
  \end{bmatrix}
  \begin{bmatrix}
    \tilde{\bm u}(\bm\theta)\\\tilde{\bm\beta}(\bm\theta)
  \end{bmatrix} =
  \begin{bmatrix}
    \bm\Lambda\trans(\bm\theta)\bm Z\trans\bm y\\
    \bm X\trans\bm y
  \end{bmatrix} .
\end{displaymath}
In the process of solving this system we create the sparse left
Cholesky factor, $L_{\bm Z}(\bm\theta)$, which is a lower triangular
sparse matrix satisfying
\begin{displaymath}
  \bm L_{\bm Z}(\bm\theta)\bm L_{\bm Z}(\bm\theta)\trans=\bm P_{\bm
    Z}\left(\bm\Lambda\trans(\bm\theta)\bm Z\trans\bm
    Z\bm\Lambda(\theta)+\bm I_q\right)\bm P_{\bm Z}\trans
\end{displaymath}
where $\bm P_{\bm Z}$ is a permutation matrix representing a
fill-reducing permutation formed from the pattern of nonzeros in $\bm
Z\bm\Lambda(\bm\theta)$ for any $\bm\theta$ not on the boundary of the
parameter region.  (The values of the nonzeros depend on $\bm\theta$
but the pattern doesn't.)

The profiled log-likelihood, $\tilde{\ell}(\bm\theta|\bm y)$, is
\begin{displaymath}
    -2\tilde{\ell}(\bm\theta|\bm y)=
  \log(|\bm L_{\bm Z}(\bm\theta)|^2)+
  n\left(1+\log\left(\frac{2\pi\tilde{d}(\bm y,\bm\theta)}{n}\right)\right)
\end{displaymath}
where $\tilde{d}(\bm y,\bm\theta)=d(\tilde{\bm u}(\bm\theta)|\bm
y,\tilde{\bm\beta}(\bm\theta),\bm\theta)$.

\section{Generalizing the discrepancy function}
\label{sec:generalizations}

Because one of the factors influencing the choice of implementation
for linear mixed models is the extent to which the methods can also be
applied to other mixed models, we describe several other classes of
mixed models before discussing the implementation details for linear
mixed models.  At the core of our methods for determining the maximum
likelihood estimates (MLEs) of the parameters in the mixed model are
methods for minimizing the discrepancy function with respect to the
coefficients $\bm u$ and $\bm\beta$ in the linear predictor
$\bm\gamma(\bm u,\bm\theta,\bm\beta)$.

In this section we describe the general form of the discrepancy
function that we will use and a penalized iteratively reweighted least
squares (PIRLS) algorithm for determining the conditional modes
$\tilde{\bm u}(\bm\theta)$ and $\tilde{\bm\beta}(\bm\theta)$.  We then
describe several types of mixed models and the form of the discrepancy
function for each.

\subsection{A weighted residual sum of squares}
\label{sec:weighted}

As shown in \S\ref{sec:conditional-mode-bm}, the discrepancy function
for a linear mixed model has the form of a penalized residual sum of
squares from a linear model (\ref{eq:10}).  In this section we
generalize that definition to
\begin{equation}
  \label{eq:11}
  d(\bm u|\bm y,\bm\theta,\bm\beta) =\left\|\bm W^{1/2}(\bm\mu)
    \left[\bm y-\bm\mu_{\bc Y|\bc U}(\bm u,\bm\theta,\bm\beta)\right]\right\|^2+
  \|\bm 0-\bm u\|^2 .
\end{equation}
where $\bm W$ is an $n\times n$ diagonal matrix, called the
\emph{weights matrix}, with positive diagonal elements and $\bm
W^{1/2}$ is the diagonal matrix with the square roots of the weights
on the diagonal.  The $i$th weight is inversely proportional to
the conditional variances of $(\mathcal{Y}|\bc U=\bm u)$ and may
depend on the conditional mean, $\bm\mu_{\bc Y|\bc U}$.

We allow the conditional mean to be a nonlinear function of the linear
predictor, but with certain restrictions.  We require that the
mapping from $\bm u$ to $\bm\mu_{\bc Y|\bc U=\bm u}$ be expressed as
\begin{equation}
  \label{eq:uGammaEtaMu}
  \bm u\;\rightarrow\;\bm\gamma\;\rightarrow\;\bm\eta\;\rightarrow\;\bm\mu
\end{equation}
where $\bm\gamma=\bm Z\bm\Lambda(\bm\theta)\bm u+\bm X\bm\beta$ is an
$ns$-dimensional vector ($s > 0$) while $\bm\eta$ and $\bm\mu$ are
$n$-dimensional vectors.

The map $\bm\eta\rightarrow\bm\mu$ has the property that $\mu_i$
depends only on $\eta_i$, $i=1,\dots,n$.  The map
$\bm\gamma\rightarrow\bm\eta$ has a similar property in that, if we
write $\bm\gamma$ as an $n\times s$ matrix $\bm\Gamma$ such that
\begin{equation}
  \label{eq:vecGamma}
  \bm\gamma=\vec{\bm\Gamma}
\end{equation}
(i.e.{} concatenating the columns of $\bm\Gamma$ produces $\bm\gamma$)
then $\eta_i$ depends only on the $i$th row of $\bm\Gamma$,
$i=1,\dots,n$.  Thus the Jacobian matrix
$\frac{d\bm\mu}{d\bm\eta\trans}$ is an $n\times n$ diagonal matrix and
the Jacobian matrix $\frac{d\bm\eta}{d\bm\gamma\trans}$ is the
horizontal concatenation of $s$ diagonal $n\times n$ matrices.

For historical reasons, the function that maps $\eta_i$ to $\mu_i$ is
called the \emph{inverse link} function and is written
$\mu=g^{-1}(\eta)$.  The \emph{link function}, naturally, is
$\eta=g(\mu)$.  When applied component-wise to vectors $\bm\mu$ or
$\bm\eta$ we write these as $\bm\eta=\bm g(\bm\mu)$ and $\bm\mu=\bm
g^{-1}(\bm\eta)$.

Recall that the conditional distribution, $(\mathcal{Y}_i|\bc U=\bm
u)$, is required to be independent of $(\mathcal{Y}_j|\bc U=\bm u)$
for $i,j=1,\dots,n,\,i\ne j$ and that all the component conditional
distributions must be of the same form and differ only according to
the value of the conditional mean.

Depending on the family of the conditional distributions, the allowable
values of the $\mu_i$ may be in a restricted range.  For example, if
the conditional distributions are Bernoulli then
$0\le\mu_i\le1,i=1,\dots,n$. If the conditional distributions are
Poisson then $0\le\mu_i,i=1,\dots,n$.  A characteristic of the link
function, $g$, is that it must map the restricted range to an
unrestricted range.  That is, a link function for the Bernoulli
distribution must map $[0,1]$ to $[-\infty,\infty]$ and must be invertible
within the range.

The mapping from $\bm\gamma$ to $\bm\eta$ is defined by a function
$m:\mathbb{R}^s\rightarrow\mathbb{R}$, called the
\emph{nonlinear model} function, such that
$\eta_i=m(\bm\gamma_i),i=1,\dots,n$ where $\bm\gamma_i$ is the $i$th
row of $\bm\Gamma$.  The vector-valued function is $\bm\eta=\bm m(\bm\gamma)$.

Determining the conditional modes, $\tilde{\bm u}(\bm y|\bm\theta)$,
and $\tilde{\bm\beta}(\bm y|\bm\theta)$, that jointly minimize the
discrepancy,
\begin{equation}
  \label{eq:12}
  \begin{bmatrix}
    \tilde{\bm u}(\bm y|\bm\theta)\\
    \tilde{\bm\beta}(\bm y|\bm\theta)
  \end{bmatrix}
  =\arg\min_{\bm u,\bm\beta}\left[(\bm y-\bm\mu)\trans\bm W(\bm
  y-\bm\mu)+\|\bm u\|^2\right]
\end{equation}
becomes a weighted, nonlinear least squares problem except that the
weights, $\bm W$, can depend on $\bm\mu$ and, hence, on $\bm u$ and
$\bm\beta$.

In describing an algorithm for linear mixed models we called
$\tilde{\bm\beta}(\bm\theta)$ the \emph{conditional estimate}.  That
name reflects that fact that this is the maximum likelihood estimate
of $\bm\beta$ for that particular value of $\bm\theta$.  Once we have
determined the MLE, $\widehat(\bm\theta)_L$ of $\bm\theta$, we have a
``plug-in'' estimator,
$\widehat{\bm\beta}_L=\tilde{\bm\beta}(\bm\theta)$ for $\bm\beta$.

This property does not carry over exactly to other forms of mixed
models.  The values $\tilde{\bm u}(\bm\theta)$ and
$\tilde{\bm\beta}(\bm\theta)$ are conditional modes in the sense that
they are the coefficients in $\bm\gamma$ that jointly maximize the
unscaled conditional density $h(\bm u|\bm
y,\bm\theta,\bm\beta,\sigma)$.  Here we are using the adjective
``conditional'' more in the sense of conditioning on $\bc Y=\bm y$
than in the sense of conditioning on $\bm\theta$, although these
values are determined for a fixed value of $\bm\theta$.

\subsection{The PIRLS algorithm for $\tilde{\bm u}$ and $\tilde{\bm\beta}$}
\label{sec:pirls-algor-tild}

The penalized, iteratively reweighted, least squares (PIRLS) algorithm
to determine $\tilde{\bm u}(\bm\theta)$ and
$\tilde{\bm\beta}(\bm\theta)$ is a form of the Fisher scoring
algorithm.  We fix the weights matrix, $\bm W$, and use penalized,
weighted, nonlinear least squares to minimize the penalized, weighted
residual sum of squares conditional on these weights.  Then we update
the weights to those determined by the current value of $\bm\mu$ and
iterate.

To describe this algorithm in more detail we will use parenthesized
superscripts to denote the iteration number.  Thus $\bm u^{(0)}$ and
$\bm\beta^{(0)}$ are the initial values of these parameters, while
$\bm u^{(i)}$ and $\bm\beta^{(i)}$ are the values at the $i$th
iteration.  Similarly $\bm\gamma^{(i)}=\bm Z\bm\Lambda(\bm\theta)\bm
u^{(i)}+\bm X\bm\beta^{(i)}$, $\bm\eta^{(i)}=\bm m(\bm\gamma^{(i)})$ and
$\bm\mu^{(i)}=\bm g^{-1}(\bm\eta^{(i)})$.

We use a penalized version of the Gauss-Newton algorithm
\citep[ch.~2]{bateswatts88:_nonlin} for which we define the weighted Jacobian matrices
\begin{align}
  \label{eq:Jacobian}
  \bm U^{(i)}&=\bm W^{1/2}\left.\frac{d\bm\mu}{d\bm u\trans}\right|_{\bm u=\bm
    u^{(i)},\bm\beta=\bm\beta^{(i)}}=\bm W^{1/2}
  \left.\frac{d\bm\mu}{d\bm\eta\trans}\right|_{\bm\eta^{(i)}}
  \left.\frac{d\bm\eta}{d\bm\gamma\trans}\right|_{\bm\gamma^{(i)}}
  \bm Z\bm\Lambda(\bm\theta)\\
  \bm V^{(i)}&=\bm W^{1/2}\left.\frac{d\bm\mu}{d\bm\beta\trans}\right|_{\bm u=\bm
    u^{(i)},\bm\beta=\bm\beta^{(i)}}=\bm W^{1/2}
  \left.\frac{d\bm\mu}{d\bm\eta\trans}\right|_{\bm\eta^{(i)}}
  \left.\frac{d\bm\eta}{d\bm\gamma\trans}\right|_{\bm\gamma^{(i)}}
  \bm X
\end{align}
of dimension $n\times q$ and $n\times p$, respectively.
The increments at the $i$th iteration, $\bm\delta_{\bm u}^{(i)}$ and
$\bm\delta_{\bm\beta}^{(i)}$, are the solutions to
\begin{equation}
  \label{eq:PNLSinc}
  \begin{bmatrix}
    {\bm U^{(i)}}\trans\bm U^{(i)}+\bm I_q&{\bm U^{(i)}}\trans\bm V^{(i)}\\
    {\bm V^{(i)}}\trans\bm U^{(i)}&{\bm V^{(i)}}\trans\bm V^{(i)}
  \end{bmatrix}
  \begin{bmatrix}
    \bm\delta_{\bm u}^{(i)}\\
    \bm\delta_{\bm\beta}^{(i)}
  \end{bmatrix} =
  \begin{bmatrix}
    {\bm U^{(i)}}\trans\bm W^{1/2}(\bm y-\bm\mu^{(i)})-\bm u^{(i)}\\
    {\bm V^{(i)}}\trans\bm W^{1/2}(\bm y-\bm\mu^{(i)})
  \end{bmatrix}
\end{equation}
providing the updated parameter values
\begin{equation}
  \label{eq:33}
  \begin{bmatrix}\bm u^{(i+1)}\\\bm\beta^{(i+1)}\end{bmatrix}=
  \begin{bmatrix}\bm u^{(i)}\\\bm\beta^{(i)}\end{bmatrix}+\lambda
  \begin{bmatrix}\bm\delta_{\bm u}^{(i)}\\\bm\delta_{\bm\beta}^{(i)}
  \end{bmatrix}
\end{equation}
where $\lambda>0$ is a step factor chosen to ensure that
\begin{equation}
  \label{eq:34}
  (\bm y-\bm\mu^{(i+1)})\trans\bm W(\bm
  y-\bm\mu^{(i+1)})+\|\bm u^{(i+1)}\|^2  <
  (\bm y-\bm\mu^{(i)})\trans\bm W(\bm
  y-\bm\mu^{(i)})+\|\bm u^{(i)}\|^2  .
\end{equation}

In the process of solving for the increments we form the sparse, lower
triangular, Cholesky factor, $\bm L^{(i)}$, satisfying
\begin{equation}
  \label{eq:35}
  \bm L^{(i)} {\bm L^{(i)}}\trans =
  \bm P_{\bm Z}\left({\bm U^{(i)}}\trans\bm U^{(i)}+
    \bm I_n\right)\bm P_{\bm Z}\trans .
\end{equation}
After each successful iteration, determining new values of the
coefficients, $\bm u^{(i+1)}$ and $\bm\beta^{(i+1)}$, that reduce the
penalized, weighted residual sum of squqres, we update the weights
matrix to $\bm W(\bm\mu^{(i+1)})$ and the weighted Jacobians,
$\bm U^{(i+1)}$ and $\bm V^{(i+1)}$, then iterate.  Convergence is
determined according to the orthogonality convergence
criterion~\citep[ch.~2]{bateswatts88:_nonlin}, suitably adjusted for
the weights matrix and the penalty.

\subsection{Weighted linear mixed models}
\label{sec:weightedLMM}

One of the simplest generalizations of linear mixed models is a
weighted linear mixed model where $s=1$, the link function, $g$, and
the nonlinear model function, $m$, are both the identity, the weights
matrix, $\bm W$, is constant and the conditional distribution family
is Gaussian.  That is, the conditional distribution can be written
\begin{equation}
  \label{eq:weightedLMM}
    (\bc Y|\bc U=\bm u)\sim\mathcal{N}(\bm\gamma(\bm
    u,\bm\theta,\bm\beta),\sigma^2\bm W^{-1})
\end{equation}
with discrepancy function
\begin{equation}
  \label{eq:wtddisc}
  d(\bm u|\bm y,\bm\theta,\bm\beta)=\left\|\bm W^{1/2}(\bm
    y-\bm Z\bm\Lambda(\bm\theta)\bm u-\bm X\bm\beta)\right\|^2+\|\bm u\|^2 .
\end{equation}
The conditional mode, $\tilde{\bm u}(\bm\theta)$, and the conditional
estimate, $\tilde{\bm\beta}(\bm\theta)$, are the solutions to
\begin{equation}
  \begin{bmatrix}
    \bm\Lambda\trans(\bm\theta)\bm Z\trans\bm W\bm Z\bm\Lambda(\theta)
    +\bm I_q&\bm\Lambda\trans(\bm\theta)\bm Z\trans\bm W\bm X\\
    \bm X\trans\bm W\bm Z\bm\Lambda(\theta) &\bm X\trans\bm W\bm X
  \end{bmatrix}
  \begin{bmatrix}
    \tilde{\bm u}(\bm\theta)\\\tilde{\bm\beta}(\bm\theta)
  \end{bmatrix} =
  \begin{bmatrix}
    \bm\Lambda\trans(\bm\theta)\bm Z\trans\bm W\bm y\\
    \bm X\trans\bm W\bm y
  \end{bmatrix} ,
\end{equation}
which can be solved directly, and the Cholesky factor, $\bm L_{\bm Z}(\bm\theta)$, satisfies
\begin{equation}
  \bm L_{\bm Z}(\bm\theta)\bm L_{\bm Z}(\bm\theta)\trans=\bm P_{\bm
    Z}\left(\bm\Lambda\trans(\bm\theta)\bm Z\trans\bm W\bm
    Z\bm\Lambda(\theta)+\bm I_q\right)\bm P_{\bm Z}\trans .
\end{equation}

The profiled log-likelihood, $\tilde{\ell}(\bm\theta|\bm y)$, is
\begin{equation}
  \label{eq:wtdprofilelik}
  -2\tilde{\ell}(\bm\theta|\bm y)=
  \log\left(\frac{|\bm L_{\bm Z}(\bm\theta)|^2}{|\bm W|}\right)+
  n\left(1+\log\left(\frac{2\pi\tilde{d}(\bm
        y,\bm\theta)}{n}\right)\right) .
\end{equation}

If the matrix $\bm W$ is fixed then we can ignore the term $|\bm W|$
in (\ref{eq:wtdprofilelik}) when determining the MLE,
$\widehat{\bm\theta}_L$.  However, in some models, we use a
parameterized weight matrix, $\bm W(\bm\phi)$, and wish to determine
the MLEs, $\widehat{\bm\phi}_L$ and $\widehat{\bm\theta}_L$
simultaneously.  In these cases we must include the term involving
$|\bm W(\bm\phi)|$ when evaluating the profiled log-likelihood.

Note that we must define the parameterization of $\bm W(\bm\phi)$
such that $\sigma^2$ and $\bm\phi$ are not a redundant
parameterization of $\sigma^2\bm W(\bm\phi)$.  For example, we could
require that the first diagonal element of $\bm W$ be unity.

\subsection{Nonlinear mixed models}
\label{sec:NLMMs}

In an unweighted, nonlinear mixed model the conditional distribution is
Gaussian, the link, $g$, is the identity and the weights matrix, $\bm
W=\bm I_n$.  That is,
\begin{equation}
  \label{eq:conddistNLMM}
  (\bc Y|\bc U=\bm u)\sim\mathcal{N}(\bm m(\bm\gamma),\sigma^2\bm I_n)
\end{equation}
with discrepancy function
\begin{equation}
  \label{eq:discNLMM}
  d(\bm u|\bm y,\bm\theta,\bm\beta)=
  \|\bm y-\bm\mu\|^2 + \|\bm u\|^2 .
\end{equation}
For a given value of $\bm\theta$ we determine the conditional modes,
$\tilde{\bm u}(\bm\theta)$ and $\tilde{\bm\beta}(\bm\theta)$, as the
solution to the penalized nonlinear least squares problem
\begin{equation}
  \label{eq:NLMMpnls}
  \begin{bmatrix}
    \tilde{\bm u}(\bm\theta)\\
    \tilde{\bm\beta}(\bm\theta)
  \end{bmatrix} = \arg\min_{\bm u,\bm\theta}d(\bm u|\bm y,\bm\theta,\bm\beta)
\end{equation}
and we write the minimum discrepancy, given $\bm y$ and $\bm\theta$,
as
\begin{equation}
  \label{eq:25}
  \tilde{d}(\bm y,\bm\theta)=d(\tilde{\bm u}(\bm\theta)|\bm
  y,\bm\theta,\tilde{\bm\beta}(\bm\theta)).
\end{equation}

Let $\tilde{\bm L}_Z(\bm\theta)$ and $\tilde{\bm L}_X(\bm\theta)$ be
the Cholesky factors at $\bm\theta$, $\tilde{\bm\beta}(\bm\theta)$ and
$\tilde{\bm u}(\bm\theta)$.  Then the \emph{Laplace approximation} to
the log-likelihood is
\begin{equation}
  \label{eq:36}
  -2\ell_P(\bm\theta,\bm\beta,\sigma|\bm y)\approx
  n\log(2\pi\sigma^2)+\log(|\tilde{\bm L}_{\bm Z}|^2)+
  \frac{\tilde{d}(\bm y,\bm\theta) +
    \left\|\tilde{\bm L}_{\bm X}\trans(\bm\beta-\tilde{\bm\beta})\right\|^2}{\sigma^2},
\end{equation}
producing the approximate profiled log-likelihood,
$\tilde{\ell}_P(\bm\theta|\bm y)$,
\begin{equation}
  \label{eq:37}
  -2\tilde{\ell}_P(\bm\theta|\bm y)\approx
  \log(|\tilde{\bm L}_{\bm Z}|^2)+n\left(1+\log(2\pi
    \tilde{d}(\bm y,\bm\theta)/n) \right).
\end{equation}

\subsubsection{Nonlinear mixed model summary}
\label{sec:nonl-mixed-model}

In a nonlinear mixed model we determine the parameter estimate,
$\widehat{\bm\theta}_P$, from the Laplace approximation to the
log-likelihood as
\begin{equation}
  \label{eq:38}
  \widehat{\bm\theta}_P =
  \arg\max_{\bm\theta}\tilde{\ell}_P(\bm\theta|\bm y)
  =\arg\min_{\bm\theta}
  \log(|\tilde{\bm L}_{\bm Z}|^2)+
  n\left(1+\log(2\pi
    \tilde{d}(\bm y,\bm\theta)/n) \right).
\end{equation}
Each evaluation of $\tilde{\ell}_P(\bm\theta|\bm y)$ requires a
solving the penalized nonlinear least squares problem
(\ref{eq:NLMMpnls}) simultaneously with respect to both sets of
coefficients, $\bm u$ and $\bm\beta$, in the linear predictor,
$\bm\gamma$.

For a weighted nonlinear mixed model with fixed weights, $\bm W$, we
replace the unweighted discrepancy function $d(\bm u|\bm
y,\bm\theta,\bm\beta)$ with the weighted discrepancy function,
%% Finish this off

\section{Details of the implementation}
\label{sec:details}

\subsection{Implementation details for linear mixed models}
\label{sec:impl-line-mixed}

The crucial step in implementing algorithms for determining ML or REML
estimates of the parameters in a linear mixed model is evaluating the
factorization (\ref{eq:16}) for any $\bm\theta$ satisfying
$\bm\theta_L\le\bm\theta\le\bm\theta_U$.  We will assume that $\bm Z$
is sparse as is $\bm Z\bm\Lambda(\bm\theta)$.

When $\bm X$ is not sparse we will use the factorization (\ref{eq:16})
setting $\bm P_{\bm X}=\bm I_p$ and storing $\bm L_{\bm X\bm Z}$ and
$\bm L_{\bm X}$ as dense matrices.  The permutation matrix $\bm P_{\bm
  Z}$ is determined from the pattern of non-zeros in $\bm
Z\bm\Lambda(\bm\theta)$ which is does not depend on $\bm\theta$, as
long as $\bm\theta$ is not on the boundary.  In fact, in most cases
the pattern of non-zeros in $\bm Z\bm\Lambda(\bm\theta)$ is the same
as the pattern of non-zeros in $\bm Z$. For many models, in particular
models with scalar random effects (described later), the matrix
$\bm\Lambda(\bm\theta)$ is diagonal.

Given a value of $\bm\theta$ we determine the Cholesky factor $\bm
L_{\bm Z}$ satisfying
\begin{equation}
  \label{eq:LZ}
  \bm L_{\bm Z}\bm L_{\bm Z}\trans=\bm P_{\bm Z}(
  \bm\Lambda\trans(\bm\theta)\bm Z\trans\bm Z\bm\Lambda(\theta)
    +\bm I_q)\bm P_{\bm Z}\trans .
\end{equation}
The CHOLMOD package allows for $\bm L_{\bm Z}$ to be calculated
directly from $\bm\Lambda\trans(\bm\theta)\bm Z\trans$ or from
$\bm\Lambda\trans(\bm\theta)\bm Z\trans\bm Z\bm\Lambda(\theta)$.  The
choice in implementation is whether to store $\bm Z\trans$ and update
it to $\bm\Lambda\trans(\bm\theta)\bm Z$ or to store $\bm Z\trans\bm
Z$ and use it to form $\bm\Lambda\trans(\bm\theta)\bm Z\trans\bm
Z\bm\Lambda(\theta)$ at each evaluation.

In the \package{lme4} package we store $\bm Z\trans$ and use it to
form $\bm\Lambda\trans(\bm\theta)\bm Z\trans$ from which $\bm L_{\bm
  Z}$ is evaluated.  There are two reasons for this choice.  First,
the calculations for the more general forms of mixed models cannot be
reduced to calculations involving $\bm Z\trans\bm Z$ and by expressing
these calculations in terms of $\bm\Lambda(\bm\theta)\bm Z\trans$ for
linear mixed models we can reuse the code for the more general
models.  Second, the calculation of $\bm\Lambda(\bm\theta)\trans\left(\bm
  Z\trans\bm Z\right)\bm\Lambda(\bm\theta)$ from $\bm Z\trans\bm Z$ is
complicated compared to the calculation of
$\bm\Lambda(\bm\theta)\trans\bm Z\trans$ from $\bm Z\trans$.

This choice is disadvantageous when $n\gg q$ because $\bm Z\trans$ is
much larger than $\bm Z\trans\bm Z$, even when they are stored as
sparse matrices.  Evaluation of $\bm L_{\bm Z}$ directly from $\bm
Z\trans$ requires more storage and more calculation that evaluating
$\bm L_{\bm Z}$ from $\bm Z\trans\bm Z$.

Next we evaluate $\bm L_{\bm X\bm Z}\trans$ as the solution to
\begin{equation}
  \label{eq:LXZ}
  \bm L_{\bm Z}\bm L_{\bm X\bm Z}\trans=\bm P_{\bm
    Z}\bm\Lambda\trans(\bm\theta)\bm Z\trans\bm X .
\end{equation}
Again we have the choice of calculating and storing $\bm Z\trans\bm X$
or storing $\bm X$ and using it to reevaluate $\bm Z\trans\bm X$.  In
the \package{lme4} package we store $\bm X$, because the calculations
for the more general models cannot be expressed in terms of $\bm Z\trans\bm X$.

Finally $\bm L_{\bm X}$ is evaluated as the (dense) solution to
\begin{equation}
  \label{eq:LX}
  \bm L_{\bm X}\bm L_{\bm X}\trans=
  \bm X\trans\bm X-\bm L_{\bm X\bm Z}\bm L_{\bm X\bm Z} .
\end{equation}
from which $\tilde{\bm\beta}$ can be determined as the
solution to dense system
\begin{equation}
  \label{eq:tildebeta}
  \bm L_{\bm X}\bm L_{\bm X}\tilde{\bm\beta}=\bm X\trans\bm y
\end{equation}
and $\tilde{\bm u}$ as the solution to the sparse system
\begin{equation}
  \label{eq:tildeu}
  \bm L_{\bm Z}\bm L_{\bm Z}\tilde{u}=\bm\Lambda\trans\bm Z\trans\bm y
\end{equation}

For many models, in particular models with scalar random effects,
which are described later, the matrix $\bm\Lambda(\bm\theta)$ is
diagonal.  For such a model, if both $\bm Z$ and $\bm X$ are sparse
and we plan to use the REML criterion then we create and store
\begin{equation}
  \label{eq:8}
  \bm A=
  \begin{bmatrix}
    \bm Z\trans\bm Z & \bm Z\trans\bm X\\
    \bm X\trans\bm Z & \bm X\trans\bm X
  \end{bmatrix}\quad\text{and}\quad
  \bm c =\begin{bmatrix}\bm Z\trans\bm y\\\bm X\trans\bm y\end{bmatrix}
\end{equation}
and determine a fill-reducing permutation, $\bm P$, for $\bm A$.
Given a value of $\bm\theta$ we create the factorization
\begin{equation}
  \label{eq:26}
  \bm L(\bm\theta)\bm L(\bm\theta)\trans=\bm P\left(
    \begin{bmatrix}
      \bm\Lambda(\bm\theta) & \bm0\\\bm0&\bm I_p
    \end{bmatrix} \bm A
    \begin{bmatrix}
      \bm\Lambda(\bm\theta) & \bm0\\\bm0&\bm I_p
    \end{bmatrix}+
    \begin{bmatrix}\bm I_q&\bm0\\\bm0&\bm0\end{bmatrix}\right)
  \bm P\trans
\end{equation}
solve for $\tilde{\bm u}(\bm\theta)$ and
$\tilde{\bm\beta}(\bm\theta)$ in
\begin{equation}
  \label{eq:28}
  \bm L\bm L\trans\bm P
  \begin{bmatrix}
    \tilde{\bm u}(\bm\theta)\\\tilde{\bm\beta}(\bm\theta)
  \end{bmatrix}=
  \bm P
  \begin{bmatrix}\bm\Lambda(\bm\theta) & \bm0\\\bm0&\bm I_p
  \end{bmatrix} \bm c
\end{equation}
then evaluate $\tilde{d}(\bm y|\bm\theta)$
and the profiled REML criterion as
\begin{equation}
  \label{eq:27}
  \tilde{d}_R(\bm\theta|\bm y)=\log(|\bm L(\bm\theta)|^2)+
  (n-p)\left(1+\log\left(\frac{2\pi\tilde{d}(\bm y|\bm\theta)}
      {n-p}\right)\right) .
\end{equation}

\bibliography{lme4}
\appendix{}

\section{Notation}
\label{sec:notation}

\subsection{Random variables in the model}
\label{sec:rand-vari-model}
\begin{description}
\item[$\bc B$] Random-effects vector of dimension $q$,
  $\bc{B}\sim\mathcal{N}(\bm 0,\sigma^2\bm \Lambda(\bm\theta)\bm
  \Lambda(\bm\theta)\trans)$.
\item[$\bm U$] ``Spherical'' random-effects vector of dimension $q$,
  $\bc U\sim\mathcal{N}(\bm 0,\sigma^2\bm I_q)$, $\bc B=\bm
  \Lambda(\bm\theta)\bc U$.
\item[$\bc Y$] Response vector of dimension $n$.
\end{description}

\subsection{Parameters of the model}
\label{sec:parameters-model}
\begin{description}
\item[$\bm\beta$] Fixed-effects parameters (dimension $p$).
\item[$\bm\theta$] Parameters determining the left factor,
  $\bm\Lambda(\bm\theta)$ of the relative covariance matrix of $\bc B$
  (dimension $m$).
\item[$\sigma$] the common scale parameter - not used in some
  generalized linear mixed models and generalized nonlinear mixed
  models.
\end{description}

\subsection{Dimensions}
\label{sec:dimensions}
\begin{description}
\item[$m$] dimension of the parameter $\bm\theta$.
\item[$n$] dimension of the response vector, $\bm y$, and the random
  variable, $\bm{\mathcal{Y}}$.
\item[$p$] dimension of the fixed-effects parameter, $\bm\beta$.
\item[$q$] dimension of the random effects, $\bc B$ or $\bc U$.
\item[$s$] dimension of the parameter vector, $\bm\phi$, in the
  nonlinear model function.
\end{description}

\subsection{Matrices}
\label{sec:matrices}
\begin{description}
\item[$\bm L$] Left Cholesky factor of a positive-definite symmetric
  matrix.  $\bm L_{\bm Z}$ is $q\times q$; $\bm L_{\bm X}$ is $p\times p$.
\item[$\bm P$] Fill-reducing permutation for the random effects model
  matrix. (Size $q\times q$.)
\item[$\bm \Lambda$] Left factor of the relative covariance matrix of the
  random effects. (Size $q\times q$.)
\item[$\bm X$] Model matrix for the fixed-effects parameters,
  $\bm\beta$. (Size $(ns)\times p$.)
\item[$\bm Z$] Model matrix for the random effects. (Size $(ns)\times q$.)
\end{description}

\section{Integrating a quadratic deviance expression}
\label{sec:integr-quadr-devi}

In (\ref{eq:6}) we defined the likelihood of the parameters given the
observed data as
\begin{displaymath}
  L(\bm\theta,\bm\beta,\sigma|\bm y) =
  \int_{\mathbb{R}^q}h(\bm u|\bm y,\bm\theta,\bm\beta,\sigma)\,d\bm u .
\end{displaymath}
which is often alarmingly described as ``an intractable integral''.
In point of fact, this integral can be evaluated exactly in the case
of a linear mixed model and can be approximated quite accurately for
other forms of mixed models.

\end{document}

---
title: "Covariance Structures"
output: rmarkdown::html_vignette
bibliography: lmer.bib
vignette: >
  %\VignetteIndexEntry{Covariance Structures}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
  %\VignetteDepends{glmmTMB}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```


## Introduction

The current version of `lme4` offers four covariance classes/structures: `Covariance.us` (unstructured), `Covariance.diag` (diagonal), `Covariance.cs` (compound symmetry), and `Covariance.ar1` (autoregressive order 1). The syntax for use is similar to the `glmmTMB` package (see [covariance structures in glmmTMB](https://glmmtmb.github.io/glmmTMB/articles/covstruct.html)), although the results are slightly different.

This vignette provides a detailed mathematical exposition of the new machinery, along with brief notes on the appropriate syntax. In a separate document, we provide comparisons of results between `glmmTMB` and `lme4` (the computations are too slow to satisfy CRAN timing requirements; if we tried to store and re-load saved fits, they would be too big to satisfy CRAN package size requirements).

### Background

For exact details of this structure, refer to the `lmer` [vignette](https://cran.r-project.org/package=lme4/vignettes/lmer.pdf) [@Bates_JSS]. We provide
a quick summary in this vignette.

In matrix notation, a linear mixed model can be represented as:
\[
\mathbf{y} = \mathbf{X} \boldsymbol \beta + \mathbf Z \mathbf{b} + \boldsymbol{\epsilon}
\]
Where $\mathbf{b}$ represents an unknown vector of random effects, with $\mathbf b \sim \textrm{MVN}(\mathbf 0, \boldsymbol \Sigma)$.

<!-- can't use \usepackage{bm} for MathJax output: https://stackoverflow.com/a/33226711/190277 -->
We create the relative co-factor $\mathbf \Lambda_{\mathbf \theta}$ which is a 
$q \times q$ block diagonal  matrix that depends on the variance-component 
parameter vector $\boldsymbol \theta$. Let $\sigma$ be the scale parameter of the variance of a 
linear mixed model. In `lme4`, the variance-covariance matrix is constructed by:
\[
{\mathbf \Sigma}_{\mathbf{\theta}} = 
\sigma^2 {\mathbf \Lambda}_{\mathbf \theta} {\mathbf \Lambda}_{\mathbf \theta}^{\top},
\]

For generalized linear mixed models, $\mathbf \Lambda_\theta$ instead represents the *unscaled* Cholesky factor; that is,
the scaling term $\sigma^2$ is omitted from the equation above.

The major difference between the four covariance classes (`Covariance.us` (unstructured), `Covariance.diag` (diagonal), `Covariance.cs` (compound symmetry), and `Covariance.ar1` (autoregressive order 1)) is in the construction of the the relative Cholesky factor $\mathbf \Lambda_{\mathbf \theta}$.

### Covariance Structures

Suppose that a particular random-effects term has $p_i$ varying effects (i.e., the number of columns of the
model matrix constructed from the left-hand side `f` of the random-effects formula `(f|g)`.
If the grouping variable (`g`) has $\ell_i$ levels 
and $q_i = \ell_i p_i$.
In general the covariance matrix for the term, $\mathbf \Sigma_i$, is a $q \times q$ block diagonal matrix
composed of homogeneous $p_i \times p_i$ blocks; the (relative) Cholesky factor $\boldsymbol \Lambda_i$ has a similar structure. 
In what follows we will describe the construction of these
*template matrices*, for either the covariance matrix or the Cholesky factor.


The *unstructured covariance*, which is the default 
in `lme4`, of size $p \times p$ has the following form:
\[
\mathbf{\Sigma}
= \begin{bmatrix}
\sigma^{2}_{1} & \sigma_{12} & \dots & \sigma_{1p} \\
\sigma_{21} & \sigma^{2}_{2} & \dots & \sigma_{2p} \\
\vdots & \vdots & \ddots & \vdots \\
\sigma_{p1} & \sigma_{p2} & \dots & \sigma^{2}_{p} \\
\end{bmatrix}	
\]

To specify unstructured (general positive-semidefinite) covariances for a particular term, specify either `(f|g)` (as in `lme4` < 2.0)
or `us(f|g)`.

The next three covariance structures can either be heterogeneous or homogeneous.
If we have a homogeneous covariance structure (`hom = TRUE`), then we assume 
$\sigma_{1} = \sigma_{2} = \dots = \sigma_{p}$.

The *diagonal covariance* has the following form:
\[
\mathbf{\Sigma}
= \begin{bmatrix}
\sigma^{2}_{1} & 0 & \dots & 0 \\
0 & \sigma^{2}_{2} & \dots & 0 \\
\vdots & \vdots & \ddots & \vdots \\
0 & 0 & \dots & \sigma^{2}_{p} \\
\end{bmatrix}	
\]
By default, we assume a heterogeneous diagonal covariance structure.

To specify a diagonal covariance structure, use `diag(f|g)` (or `diag(f|g, hom = FALSE)`).
Unlike the unstructured covariance matrix, which is invariant to the parameterization of the terms in the varying effects model `f`,
the diagonal covariance matrix (and the others described below) depend on the contrasts assigned to factor variables and on the inclusion of an intercept term.
For example, suppose that `f` represents a factor variable with $n$ levels, and that the standard treatment contrasts are used. The template matrices will be $n \times n$.

* If the intercept is not included (i.e. the term is specified as `(0+f|g)` or `(f-1|g)`), then the variances $\sigma^2_1, \dots, \sigma^2_n$ will represent variances of observations in each level of the factor around the population mean;
* if the intercept is included (the term is specified as `(f|g)` or `(1+f|g)`), $\sigma^2_1$ will represent the among-cluster variance of observations from the first (baseline) level of `f`; $\sigma^2_2, \dots $\sigma^2_n$ will represent variances of *differences* in expected values between observations from levels 2 to $n$ and those in the baseline level.

If a different, non-default contrast is used, then the variances describe variation in the corresponding parameters. For example, with successive-difference contrasts (`MASS::contr.sdif()`) $\sigma^2_1$ would estimate among-cluster variance for observations at the first level, $\sigma^2_2$ would estimate variance in the difference between level 2 and level 1, and so forth.

Similar points apply to the compound symmetric and autoregressive covariance structures below.

The *compound symmetric covariance* has the following form:
\[
\mathbf{\Sigma}
= \begin{bmatrix}
\sigma^{2}_{1}              & \sigma_{1}\sigma_{2}\rho  & \sigma_{1}\sigma_{3}\rho  & \dots  & \sigma_{1}\sigma_{p}\rho \\
\sigma_{2}\sigma_{1} \rho   & \sigma^{2}_{2}             & \sigma_{2}\sigma_{3}\rho  & \dots  & \sigma_{2}\sigma_{p}\rho \\
\sigma_{3}\sigma_{1} \rho   & \sigma_{3}\sigma_{2}\rho   & \sigma^{2}_{3}            & \dots  & \sigma_{3}\sigma_{p}\rho \\
\vdots                      & \vdots                     & \vdots                    & \ddots & \vdots \\
\sigma_{p}\sigma_{1} \rho   & \sigma_{p}\sigma_{2}\rho   & \sigma_{p}\sigma_{3}\rho  & \dots  & \sigma^{2}_{p} 
\end{bmatrix}	
\]
By default, we assume a heterogeneous compound symmetric covariance structure.

Use `cs(f|g)` or `cs(f|g, hom = TRUE)` to specify a compound symmetric structure.

The *AR1 (auto-regressive order 1) covariance* has the following form:
\[
\mathbf{\Sigma}
= \begin{bmatrix}
\sigma^{2}_{1}                  & \sigma_{1}\sigma_{2} \rho      & \sigma_{1}\sigma_{3}\rho^{2}   & \dots  & \sigma_{1}\sigma_{p}\rho^{p-1} \\
\sigma_{2}\sigma_{1} \rho       & \sigma^{2}_{2}                 & \sigma_{2}\sigma_{3}\rho       & \dots  & \sigma_{2}\sigma_{p}\rho^{p-2} \\
\sigma_{3}\sigma_{1} \rho^{2}   & \sigma_{3}\sigma_{2}\rho       & \sigma^{2}_{3}                 & \dots  & \sigma_{3}\sigma_{p}\rho^{p-3} \\
\vdots                          & \vdots                         & \vdots                         & \ddots & \vdots \\
\sigma_{p}\sigma_{1} \rho^{p-1} & \sigma_{p}\sigma_{2}\rho^{p-2} & \sigma_{p}\sigma_{3}\rho^{p-3} & \dots  & \sigma^{2}_{p} 
\end{bmatrix}	
\]
Unlike the diagonal and compound symmetric structures, by default we assume a
homogeneous ar1 covariance structure.

Use `ar1(f|g)` or `ar1(f|g, het = TRUE)` to specify an AR1 structure.

### Construction of the Relative Co-factor

For the unstructured covariance matrix, `lme4` estimates the following parameters in `par`:
$\mathbf{\theta} = (\theta_{1}, \theta_{2}, \dots, \theta_{p(p+1)/2})$ to construct the relative
co-factor ${\mathbf \Lambda}_{\mathbf{\theta}}$ (this procedure is the same as in pre-2.0 versions of `lme4`):
\[
\Lambda_{\mathbf{\theta}} = \begin{bmatrix}
\theta_1 & 0 & 0 & \dots & 0 \\
\theta_2 & \theta_3 & 0 & \dots & 0 \\
\theta_4 & \theta_5 & \theta_6 & \dots & 0 \\
\vdots & \vdots & \vdots & \ddots & \vdots \\
\theta_{...} & \theta_{...} & \theta_{...} & \dots & \theta_{p(p+1)/2}
\end{bmatrix}
\]

The definition of the parameter vectors differs for the other covariance structures. In the diagonal covariance matrix 
case, $\mathbf{\theta}$ (or `par`) only contains the standard deviations. 
The relative co-factor ${\mathbf \Lambda}_{\mathbf{\theta}}$ is:
\[
{\mathbf \Lambda}_{\mathbf{\theta}} = \begin{bmatrix}
\theta_1 & 0 & 0 & \dots & 0 \\
0 & \theta_2 & 0 & \dots & 0 \\
0 & 0 & \theta_3 & \dots & 0 \\
\vdots & \vdots & \vdots & \ddots & \vdots \\
0 & 0 & 0 & \dots & \theta_{p}
\end{bmatrix}
\]

For the compound symmetry covariance structure, the parameter 
vector `par` contains the $p$ standard deviations $(\sigma_1, \sigma_2, \ldots, \sigma_p)$ 
and the common correlation $\rho$. In contrast to `glmmTMB`, the correlation is estimated on its original scale (bounded between -1 and 1), rather than on an unconstrained, transformed scale.

The relative co-factor ${\mathbf \Lambda}_{\mathbf{\theta}}$ is a lower triangular $p \times p$ matrix. Consider the form:
\[
{\mathbf \Lambda}_{\mathbf{\theta}} = \begin{bmatrix}
\theta_{11} & 0 & 0 & \cdots & 0 \\
\theta_{21} & \theta_{22} & 0 & \cdots & 0 \\
\theta_{31} & \theta_{32} & \theta_{33} & \cdots & 0 \\
\vdots & \vdots & \vdots & \ddots & \vdots \\
\theta_{p1} & \theta_{p2} & \theta_{p3} & \cdots & \theta_{pp}
\end{bmatrix}
\]

Its elements $\theta_{ij}$ are constructed as follows. First, define the 
sequence $\{a_j\}$ recursively:
\[
a_1 = 0, \quad a_{j+1} = a_j + \frac{(1 - \rho \cdot a_j)^2}{1 - \rho^2 \cdot a_j} \quad \text{for } j = 1, \ldots, p-1
\]

Then the elements of ${\mathbf \Lambda}$ are given by:
\[
\theta_{ij} = \begin{cases}
 \sqrt{1 - \rho^2 a_j} & \text{if } i = j \text{ (diagonal)} \\[0.5em]
 \dfrac{\rho - \rho^2 a_j}{\sqrt{1 - \rho^2 a_j}} & \text{if } i > j \text{ (below diagonal)} \\[0.5em]
0 & \text{if } i < j \text{ (above diagonal)}
\end{cases}
\]
Users can extract the values $\theta_{ij}$ via the `getTheta()` function 
of the `Covariance.cs` object, in which `theta` will be a vector in the column-wise 
elements of ${\mathbf \Lambda}$.
 
The setup is similar for the autoregressive order 1 (AR1) covariance structure. 
Again, the parameter vector contains the $p$ standard deviations 
$(\sigma_1, \sigma_2, \ldots, \sigma_p)$ and the autocorrelation parameter 
$\rho$. The relative co-factor ${\mathbf \Lambda}_{\mathbf{\theta}}$ is a lower 
triangular $p \times p$ matrix whose form is similar to the compound symmetric case.

The elements $\theta_{ij}$ are given by:
\[
\theta_{ij} = \begin{cases}
 \rho^{i-1} & \text{if } j = 1 \text{ (first column)} \\[0.5em]
 \rho^{i-j} \sqrt{1 - \rho^2} & \text{if } 1 < j \leq i \text{ (below diagonal)} \\[0.5em]
0 & \text{if } i < j \text{ (above diagonal)}
\end{cases}
\]
Again, these values can be extracted using `getTheta()` function 
of the `Covariance.ar1` object, in which `theta` will be still be a vector in 
the column-wise elements of ${\mathbf \Lambda}$.

### Extracting model components

This section illustrates how to extract
`par`, `theta`, `Lambda`, as described in the previous section, as well as
the variance covariance matrices of a model, from a `merMod` object.

We'll fit the standard `sleepstudy` example, except that we will use a
model with a compound symmetric covariance structure. Because this model has only two varying effects (intercept and slope with respect to day) per subject, and
 hence the covariance matrix is $2 \times 2$, there is no difference in overall model fit between the compound-symmetric and the unstructured covariance matrices. However, the models are parameterized differently, so this example will highlight the differences between `par` and `theta`.

```{r cs-fit}
library(lme4)
fm1.cs <- lmer(Reaction ~ Days + cs(1 + Days | Subject), sleepstudy)
```

Extracting the covariance structure:

```{r getReCovs}
print(fm1.cs_cov <- getReCovs(fm1.cs))
```

The result is a list with only one element as we only have one random-effects term (`cs(1 + Days | Subject)`). To 
see the values of `par` and `theta` for this object, we can call:

```{r getInfo}
getME(fm1.cs, "par")
getME(fm1.cs, "theta")
```

The ${\mathbf \Lambda}$ matrix is large, so we'll view it instead of printing:
```{r lambda-mat}
library(Matrix)
image(getME(fm1.cs, "Lambda"))
```

To most users, the most crucial information is simply the variance-covariance matrices. Extract these via `VarCorr.merMod()` (the list has one element per random-effect term in the model --- in this case, only one):

```{r varcorr}
vc_mat <- VarCorr(fm1.cs)
vc_mat$Subject
```

For detailed comparisons with `glmmTMB`, see `browseURL(system.file("extra_docs", "covariance_structure_comparison.html", package = "lme4"))`.

## References

---
title: "lmer Performance tips"
---

<!--
%\VignetteEngine{knitr::rmarkdown}
%\VignetteIndexEntry{lmer Performance Tips}
-->

```{r opts, echo = FALSE, message = FALSE}
library("knitr")
knitr::opts_chunk$set(
  eval = FALSE
)
```

```{r loadpkg,message=FALSE}
library("lme4")
```

# lme4 Performance Tips

- use control = `[g]lmerControl(calc.derivs = FALSE)` to turn off the time
  consuming derivative calculation that is performed after the optmization
  is finished, e.g.

```{r noderivs,eval=FALSE}
lmer(y ~ service * dept + (1|s) + (1|d), InstEval,
     control = lmerControl(calc.derivs = FALSE))
```
Note that this will disable some of the convergence tests, as well as
(for `glmer` only) making `lme4` use a less
accurate approximation to compute the standard errors of the fixed effects.

- models that only contain random effects of the form `(1|f)` use better 
  starting values for the optimization which in tests have cut run time in 
  certain examples by up to 50% relative to the previous default starting 
  values.  The `InstEval` fit shown above is one such example.

- `lmer` uses the `bobyqa` optimizer from the `minqa` package by default;
  `glmer` uses a combination of Nelder-Mead and `bobyqa`.  If you
  are specifying the `optimx` package optimizer, note that by 
  default `optimx` performs 
  certain time-consuming processing at the beginning and end which can be 
  turned off as follows (here we have specified the `"nlminb"` method but this 
  applies to any `optimx` method):

```{r nlminbfit,eval=FALSE}
library("optimx")
lmer(y ~ service * dept + (1|s) + (1|d), InstEval,
     control = lmerControl(optimizer = "optimx", calc.derivs = FALSE,
     optCtrl = list(method = "nlminb", starttests = FALSE, kkt = FALSE)))
```

- the `nloptr` package supports a variety of algorithms and importantly 
  supports additional stopping criteria which can stop the optimization
  earlier if it believes it has reached the optimum.
  For many problems using these stopping 
  criteria will result in the 
  same solution or nearly the same solution as the default optimizer but in
  less time (up to 50 percent savings have been observed); however, in
  some cases it may stop prematurely giving suboptimal results.
  (In the example below omit `print_level` if output tracing is not desired and 
  increase `maxeval` if the optimization requires more than 1000 iterations
  and you wish to allow it to proceed.)

```{r nlopt,eval=FALSE}
nlopt <- function(par, fn, lower, upper, control) {
    .nloptr <<- res <- nloptr(par, fn, lb = lower, ub = upper, 
        opts = list(algorithm = "NLOPT_LN_BOBYQA", print_level = 1,
        maxeval = 1000, xtol_abs = 1e-6, ftol_abs = 1e-6))
    list(par = res$solution,
         fval = res$objective,
         conv = if (res$status > 0) 0 else res$status,
         message = res$message
    )
}
lmer(y ~ service * dept + (1|s) + (1|d), InstEval,
    control = lmerControl(optimizer = "nloptwrap", calc.derivs = FALSE))
```

%\VignetteEngine{knitr::knitr}
%\VignetteDepends{ggplot2}
%\VignetteDepends{gridExtra}
%\VignetteDepends{performance}
%\VignetteDepends{merDeriv}
%\VignetteDepends{DHARMa}
%\VignetteIndexEntry{Fitting Generalized Linear Mixed-Effects Models using lme4}
\documentclass[nojss]{jss}
%% need no \usepackage{Sweave.sty}
\usepackage[T1]{fontenc}% for correct hyphenation and T1 encoding
\usepackage[utf8]{inputenc}
\usepackage{lmodern}% latin modern font
\usepackage[american]{babel}  %% for texi2dvi ~ bug
\usepackage{bm,amsmath,amsfonts}
\DeclareMathOperator{\tr}{tr}
\DeclareMathOperator{\VEC}{vec}
\newcommand{\bbeta}{{\bm \beta}}
\newcommand{\btheta}{{\bm \theta}}
\newcommand{\bmb}[1]{{\color{blue} #1}}

\DeclareUnicodeCharacter{0394}{$\Delta$}
%
\shortcites{lesnoff_within-herd_2004}
%% BMB: what does JSS assume?
%\bibliographystyle{chicago}
\author{
  Anna Ly\\McMaster University \And
  Rune Haubo Bojesen Christensen\\Copenhagen Research Centre \\for Biological and Precision Psychiatry\AND
  Douglas Bates\\University of Wisconsin - Madison \And
  Martin M\"achler\\ETH Zurich\AND
  Benjamin M. Bolker\\McMaster University
}
\Plainauthor{A. Ly, R. H. B. Christensen, D. M. Bates, M. M\"achler, B. M. Bolker}
\title{Fitting generalized linear mixed-effects models using \pkg{lme4}}
\Plaintitle{Fitting generalized linear mixed models using lme4}
\Shorttitle{GLMMs with lme4}
\Abstract{%
  The \pkg{lme4} \proglang{R} package can be used to fit generalized linear
  mixed models (GLMMs), which extend the class of linear mixed models
  (LMMs). The two main extensions provided by GLMMs are (1)
  allowing for the conditional distribution of the response given the
  random effects to be non-Gaussian (e.g. binomial,
  Poisson) and (2) allowing the conditional mean to be
  a nonlinear function of a linear combination of the fixed
  and random effect coefficients, via an inverse link
  function. The conditional mode of the random effects given the
  observed data, the variance-covariance matrix of the random 
  effects, and the fixed effect parameters
  are determined using penalized iteratively reweighted
  least squares. We compute an approximation of the integral
  over the distributions of the conditional modes to compute
  the maximum likelihood estimate for a given set of parameters (by default we use the
  Laplace approximation or, alternatively, the more computationally
  expensive adaptive Gauss-Hermite quadrature). The package provides
  all the standard features available for GLMs in base \proglang{R},
  including the standard set of accessor functions as
  well as the possibility of user-specified distributions 
  (within the exponential dispersion family) and link
  functions.}
%
\Keywords{%
  sparse matrix methods,
  generalized linear mixed models,
  penalized least squares,
  Cholesky decomposition}
\Address{
  Anna Ly\\
  Department of Mathematics \& Statistics \\
  McMaster University \\
  1280 Main Street W \\
  Hamilton, ON L8S 4K1, Canada \\
  Email: \email{annahuynh.ly@utoronto.ca}
  \par\bigskip
  Rune Haubo Bojesen Christensen \\
  Copenhagen Research Centre for Biological and Precision Psychiatry \\
  Mental Health Centre Copenhagen, Copenhagen University Hospital - Bispebjerg and Frederiksberg \\
  Copenhagen, Denmark \\
  Email: \email{Rune.Haubo@pm.me}
  \par\bigskip
  Douglas Bates\\
  Department of Statistics, University of Wisconsin - Madison\\
  1205 University Ave. \\
  Madison, WI 53706, U.S.A.\\
  E-mail: \email{bates@stat.wisc.edu}
  \par\bigskip
  Martin M\"achler\\
  Seminar f\"ur Statistik, HG G~16\\
  ETH Zurich\\
  8092 Zurich, Switzerland\\
  E-mail: \email{maechler@stat.math.ethz.ch}\\
  % URL: \url{http://stat.ethz.ch/people/maechler}
  \par\bigskip
  Benjamin M. Bolker\\
  Departments of Mathematics \& Statistics and Biology \\
  McMaster University \\
  1280 Main Street W \\
  Hamilton, ON L8S 4K1, Canada \\
  E-mail: \email{bolker@mcmaster.ca}
}
\newcommand{\Var}{\operatorname{Var}}
\newcommand{\abs}{\operatorname{abs}}
\newcommand{\bLt}{\ensuremath{\bm\Lambda_\theta}}
\newcommand{\mc}[1]{\ensuremath{\mathcal{#1}}}
\newcommand{\trans}{\ensuremath{^\top}}
\newcommand{\yobs}{\ensuremath{\bm y_{\mathrm{obs}}}}
\newcommand{\Lsat}{\ensuremath{L_{\mathrm{sat}}}}
\newcommand*{\eq}[1]{eqn.~\ref{#1}}% or just {(\ref{#1})}
\DeclareMathOperator*{\argmax}{arg\,max}
\DeclareMathOperator*{\argmin}{arg\,min}
\DeclareMathOperator*{\ylogy}{ylogy}
\usepackage{etoolbox}
\usepackage{lineno}
%% line numbering hack from Claude
\makeatletter
\let\@@par@orig\@@par
\renewcommand{\@@par}{%
  \@@par@orig
  \ifnum\currentgrouplevel=0
    \linenumbers
  \fi
}
\makeatother
%
<<preliminaries,include=FALSE,cache=FALSE,message=FALSE,echo=FALSE>>=
options(width=69, show.signif.stars=FALSE, str=strOptions(strict.width="cut"))
library(knitr)
library(lme4)
library(ggplot2)
theme_set(theme_bw())
revguide <- guide_legend(reverse = TRUE)
library(grid)
library(gridExtra)
zmargin <- theme(panel.margin=unit(0,"lines"))
library(lattice)
library(DHARMa)
library(performance)
opts_chunk$set(engine='R',dev='pdf',fig.width=10,
               fig.height=6.5,prompt=TRUE,
               ## strip.white=all,
               error=FALSE, ## stop on error
               tidy=FALSE,comment=NA)
## using knitr::knit2pdf() may lead to xcolor 'incompatible color definition'
## warnings. The code below resolves that, but simultaneously breaks
## vignette compilation in other contexts ...
## https://tex.stackexchange.com/questions/148188/knitr-xcolor-incompatible-color-definition               
## knit_hooks$set(document = function(x) {sub('\\usepackage[]{xcolor}', '\\usepackage[dvipsnames]{xcolor}', x, fixed = TRUE)})

@ 
\begin{document}
\linenumbers
\section{Introduction}
\label{sec:intro}

The \pkg{lme4} package for \proglang{R} can be used to fit a broad
range of mixed-effects models.  One major advantage of
\pkg{lme4} over its predecessor, \pkg{nlme}, is that it can be used to
fit generalized linear mixed
models (GLMMs), which combine the flexibility of
linear mixed models (LMMs) and generalized
linear models (GLMs). In a companion paper, we have described the
facilities in \pkg{lme4} for fitting linear mixed models (LMMs). Here
we describe the facilities for fitting GLMMs.

\section{Generalized Linear Mixed Models}
\label{sec:GLMMdef}

Generalized linear mixed models extend the class of linear models in two ways: allowing 
for both fixed-effects parameters and random effects in the linear predictor, as in
linear mixed models (LMMs), and allowing for non-Gaussian distributions of the
response, whose mean is an inverse link applied to the linear predictor,
as in generalized linear models (GLMs).

We consider several characteristics of each of these model types.

\subsection{The Linear Model}
A linear model can be written as $\mc Y\sim\mc N\left(\bm X\bm\beta,\sigma^2\bm I\right)$, where
$\mc Y$ is the $n$-dimensional random variable representing the response, $\bm X$ is an $n\times p$
model matrix and $\bm\beta$ is a $p$-dimensional coefficient vector.
The vector $\bm\eta=\bm X\bm\beta$ (the \emph{linear predictor}) is mapped to
the \emph{mean vector}, $\bm\mu=\bm g^{-1}(\bm\eta)$, in this case using an \emph{identity} link and
inverse link function.
(The \emph{link} function, $\bm g$, maps $\bm\mu$ to $\bm\eta$ and the \emph{inverse link},
$\bm g^{-1}$, maps $\bm\eta$ to $\bm\mu$.)
The maximum likelihood estimates (MLEs) of the coefficients, $\widehat{\bm\beta}$, are the values that
minimize the (weighted) sum of squared residuals
\begin{equation}
  \label{eq:LMMmles}
  \widehat{\bm\beta} = \argmin_{\bm\beta}\sum_{i=1}^n w_i (y_i - \mu_i)^2 .
\end{equation}
with weights $w_i$.
The weighted squared residuals, $d(y_i,\mu_i, w_i)= w_i (y_i - \mu_i)^2$, are the \emph{unit deviances} for
the Gaussian family.\footnote{In the \proglang{R} \emph{family} lists of functions that encapsulate the
distribution families in arguments to \code{glm} and \code{glmer}, the function that returns the unit deviances
is named \code{dev.resids}, a source of confusion in some cases because they are not the values returned
by \code{residuals(m, type="deviance")}.}

\subsection{The Generalized Linear Model}
In a \emph{generalized linear model} (GLM) the linear predictor is still $\bm\eta=\bm X\bm\beta$
but the distribution of the response, $\mc Y$, with mean $\bm\mu=\bm g^{-1}(\bm\eta)$, can be from
distribution families other than the Gaussian, specifically from the class of exponential dispersion families; the most common examples are Bernoulli, binomial, or Poisson.
Individual components of the response, ${\mc Y}_i,i=1,\dots,n$, are pairwise independent.
The distribution family determines the \emph{unit deviance} function, $d(y_i,\mu_i)$, and the MLEs
of the coefficients are those that minimize the sum of the unit deviances
\begin{equation}
  \label{eq:sumofunitdeviances}
  \widehat{\bm\beta} = \argmin_{\bm\beta}\sum_{i=1}^n d(y_i,\mu_i, w_i) .
\end{equation}

The unit deviances are the (weighted) difference between negative twice the log-likelihood at $(y_i, \mu_i)$ and
negative twice the log-likelihood at $(y_i, y_i)$ (which is often zero).
For the Poisson distribution the unit deviance function is
\begin{equation}
  \label{eq:Poissonunitdeviance}
  d(y_i,\mu_i,w_i)=
  \begin{cases}
    2 w_i \mu_i & \text{if } y_i = 0 \\
    2 w_i (y_i \log(y_i/\mu_i) - (y_i - \mu_i)) & \text{otherwise}
  \end{cases}
\end{equation}
where $w_i$ is an optional prior case weight.
For simplicity we generally neglect the prior weights and write $d(y_i, u_i)$ below.

Minimizing the sum of the squared residuals in a linear model (i.e. finding the \emph{least squares} estimates
$\widehat{\bm\beta}$ for an LM) can be performed as a direct (i.e. non-iterative) calculation.
For a generalized linear model minimizing the sum of unit deviances usually must be done through iteration.
Fortunately, this can be done using a fast, robust algorithm, \emph{iteratively reweighted least squares} \cite[IRLS:][]{McCullaghNelder1989}.

The link and inverse link functions are \emph{diagonal mappings}, in the sense that there is a scalar
function, $g$, such that the $i$th component of $\bm\eta$ is $g$
applied to the $i$th component of $\bm\mu_{\mc Y}$.  (The name
``diagonal'' reflects the fact that the Jacobian matrix,
$\frac{d\eta}{d\mu\trans}$, of such a mapping will be diagonal.)
If the distribution family is one of the exponential distribution families a
\emph{canonical link} can be derived from the form of the distribution.
The canonical link is the identity for the Gaussian family, the \emph{logit} or ``log-odds'' function,
$g(\mu)=\log(\mu/(1 - \mu))$, for the Bernoulli or binomial families, and the
natural logarithm, $g(\mu)=\log(\mu)$, for the Poisson.

\subsection{The Linear Mixed Model}
In a linear mixed model (LMM), described in \cite{bates2015fitting}, there are two vector-valued random
variables: $\mc Y$, the $n$-dimensional response, and $\mc B$, the $q$-dimensional random effects vector.
The unconditional distribution $\mc B\sim\mc N(\bm 0,\bm\Sigma_{\bm\theta})$ depends on a
\emph{covariance parameter} $\bm\theta$.
The conditional distribution, $(\mc Y| \mc B=\bm b)\sim\mc N(\bm X\bm\beta+\bm Z\bm b, \sigma^2\bm I)$,
has mean, $\bm\mu$, from the identity link applied to the linear predictor $\bm\eta=\bm X\bm\beta+\bm Z\bm b$,
where $\bm Z$ is an $n\times q$ model matrix for the random effects and $\bm b$ is a value of $\mc B$.
In practice the covariance parameter, $\bm\theta$, determines a lower triangular \emph{relative covariance factor},
$\bm\Lambda_{\bm\theta}$, with $\bm\Sigma_{\bm\theta}=\sigma^2\bm\Lambda_{\bm\theta}\bm\Lambda_{\bm\theta}\trans$
and we work with a ``spherical'' random effects variable, $\mc U\sim\mc N(\bm 0,\sigma^2\bm I)$, such that
$\mc B=\bm\Lambda_{\bm\theta}\mc U$, and for which the linear predictor is
\begin{equation}
  \label{eq:LMMlinpred}
  \bm\eta=\bm X\bm\beta+\bm Z\bm b=\bm X\bm\beta+\bm Z\bm\Lambda_{\bm\theta}\bm u .
\end{equation}
($\mc U$ is said to be ``spherical'' because contours of constant probability density
are spheres centered at the origin.)

Most generally, $\bm\Lambda_{\bm\theta}$ can be any lower triangular matrix, since
$\bm\Lambda_{\bm\theta}\bm\Lambda_{\bm\theta}\trans$ is then guaranteed to be
positive semidefinite. In \pkg{lme4} version 2.0 and later, we have also
implemented \emph{structured covariance matrices}, which introduce a more
elaborate mapping from $\bm\theta$ to $\bm\Lambda_{\bm\theta}$ that lets users
specify a range of structured forms such as diagonal, compound symmetric, or
first-order autoregressive (AR1) covariance matrices. We introduce the syntax
for these structures in the CBPP example (Section~\ref{sec:cbpp}); see the
\href{https://CRAN.R-project.org/package=lme4/vignettes/covariance_structures.html}{lme4 covariance structures vignette}
for the details of how $\bm\Lambda_{\bm\theta}$ is constructed for
each structure.

The likelihood of the parameters, $\bm\beta$, $\bm\theta$ and $\sigma^2$, given $\mc Y=\bm y$,
is the marginal density of $\mc Y$ evaluated at the observed $\bm y$.
Conceptually, we determine an expression for the integral of the joint density,
$f_{\mc Y,\mc U}(\bm y,\bm u)$, with respect to $\bm u$, then evaluate that expression at the observed $\bm y$.
In practice, it is easier to reverse the order of the evaluation at the observed $\bm y$ and the integration with
respect to $\bm u$.

The joint density is
\begin{equation}
  \label{eq:jointLMMdensity}
  f_{\mc Y,\mc U}(\bm y,\bm u)=\frac{1}{(2\pi\sigma^2)^{(n+q)/2}}
  \exp\left(\frac{\|\bm y-\bm\mu\|^2 + \|\bm u\|^2}{-2\sigma^2}\right) ,
\end{equation}
where the expression in the numerator of the integrand is the \emph{penalized sum of the unit deviances} (PSUD)
for the Gaussian family, which is the penalized sum of squared residuals (PSSR).
(The sum of the unit deviances is $\|\bm y-\bm\mu\|^2$ and the penalty is $\|\bm u\|^2$.)

To evaluate the integral we determine the \emph{conditional mode} of $\mc U$, given $\mc Y=\bm y$, which minimizes
the PSSR with respect to $\bm u$
\begin{equation}
  \label{eq:penalizedsumofsquares}
  (\tilde{\bm u} | \mc Y=\bm y,\bm\theta,\bm\beta) = \argmin_{\bm u}\left[\sum_{i=1}^n(y_i-\mu_i)^2+\|\bm u\|^2\right] .
\end{equation}
The PSSR can be minimized directly (i.e. without iteration) and the minimum doesn't depend on $\sigma^2$.
When the minimum PSSR and the determinant of the Hessian of the PSSR, which can be evaluated from a Cholesky
factor created in the process of solving eq.~\ref{eq:penalizedsumofsquares}, are available the MLE of
$\sigma^2$ can be evaluated separately, as is commonly done in linear models.

Because $\bm\beta$ and $\bm u$ both occur as coefficients in the linear predictor, the conditional estimate,
$\widehat{\bm\beta}|\bm\theta$, can be determined along with $\tilde{\bm u}$
by extending the penalized least squares problem,
(eq.~\ref{eq:penalizedsumofsquares}), to minimize with respect to both $\bm u$ and $\bm\beta$.
Thus, given a value of $\bm\theta$ and the solution to this extended PSSR problem,
the profiled log-likelihood, which is a function of $\bm\theta$ only, can be evaluated.
General nonlinear optimizers applied to the profiled log-likelihood as a function of $\bm\theta$ are
used to determine the MLEs for all the parameters.

The point of using the profiled log-likelihood instead of the original log-likelihood is that the
dimension of the general nonlinear optimization problem is much smaller in the profiled problem.

\subsection{The Generalized Linear Mixed Model}

As stated earlier, a GLMM combines the distribution family and link function, from a GLM, with random
effects in the linear predictor, as in an LMM.
The unconditional distribution of the random effects, $\mc B\sim\mc N(\bm 0, \bm\Sigma_{\bm\theta})$,
and the definitions of the spherical random effects, $\mc U$, such that $\mc B=\bm\Lambda_{\bm\theta}\mc U$ where
$\Lambda_{\bm\theta}\Lambda_{\bm\theta}\trans=\bm\Sigma$ are as in the LMM except that the scale parameter,
$\sigma$, is not present in the most common GLMMs (binomial and Poisson families).
The mean of the conditional distribution, $\mc Y|\mc U=\bm u$, is the inverse link applied to the
linear predictor $\bm\eta=\bm X\bm\beta+\bm Z\bm\Lambda_{\bm\theta}\bm u$.

The conditional mode, which minimizes the \emph{penalized sum of unit deviances} (PSUD),
\begin{equation}
  \label{eq:condmodeGLMM}
  (\tilde{\bm u}|\mc Y=\bm y,\bm\theta,\bm\beta)=\argmin_{\bm u}\left[\sum_{i=1}^n d(y_i,\mu_i) + \|\bm u\|^2\right],
\end{equation}
is determined by penalized iteratively reweighted least squares (PIRLS).

In the case of an LMM, minimizing the PSSR provided the value of the integral defining the likelihood of
the parameters, given the observed response, $\mc Y=\bm y$. 
For a GLMM, minimizing the PSUD provides a local quadratic approximation to the logarithm of the joint density,
$f_{\mc Y,\mc U}(\bm y,\bm u)$.
Approximating the integral with respect to $\bm u$ of the joint density (evaluated at the observed $\bm y$)
by the integral of this local quadratic approximation is \emph{Laplace's method}.

For simple models involving scalar random effects from a single random-effects term the multivariate integral
with respect to $\bm u$ can be factored into a product of scalar integrals, which can be evaluated by
Gauss-Hermite rules.

%\footnote{The \code{lme4} package supports fitting negative binomial GLMMs,
%which are an extension of GLMMs; the negative binomial family is within the
%exponential dispersion family if the dispersion parameter is fixed.
%The \code{glmer.nb} function uses a one-dimensional optimizer
%to estimate the dispersion parameter, fitting a negative binomial GLMM with
%a fixed dispersion parameter at each trial value. In practice this
%implementation is slower than those in other R packages (e.g., the \code{glmmTMB} package).}

To summarize:
\begin{enumerate}
\item Like a GLM, the definition of a GLMM incorporates a distribution family and a link function.
  The distribution family determines the unit deviances, $d(y_i,\mu_i),i=1,\dots,n$. Distributions in the exponential distribution family have a canonical link which is the default link in \code{glm} and in \code{glmer} for those families.
\item As in an LMM the linear predictor, $\bm\eta=\bm X\bm\beta+\bm Z\bm b$, incorporates fixed-effects      
  parameters, $\bm\beta$, and random effects, $\mc B\sim\mc N(\bm 0,\bm\Sigma_{\bm\theta})$.
\item The parameters $\bm\theta$ determine a triangular relative covariance factor, $\bm\Lambda_{\bm\theta}$,
  such that $\bm\Sigma_{\bm\theta}=\bm\Lambda_{\bm\theta}\bm\Lambda_{\bm\theta}\trans$.
  (A scale parameter, $\sigma$, is incorporated for families like the Gaussian that use one.)
\item For given values of $\bm\beta$ and $\bm\theta$, the penalized sum of unit deviances (PSUD),
  $\sum_{i=1}^n d(y_i,\mu_i)+\|\bm u\|^2$, is minimized with respect to $\bm u$ using PIRLS.
\item A local quadratic approximation to the PSUD at the conditional mode provides Laplace's approximation
  to the log-likelihood, which is minimized with respect to $\bm\beta$ and $\bm\theta$.
\item In certain simple models adaptive Gauss-Hermite quadrature (aGHQ) can be used to
  approximate the log-likelihood more accurately, at the expense of more evaluations of the PSUD for each
  candidate combination of $\bm\beta$ and $\bm\theta$.
\item If the PSUD is minimized with respect to $\bm u$ and $\bm\beta$ simultaneously then the approximate
  profiled log-likelihood becomes a function of $\bm\theta$ only, as discussed below.
  This approximation can save computation time for large problems.
\end{enumerate}

\subsection{Determining the conditional mode}
\label{sec:conditionalMode}

The iteratively reweighted least squares (IRLS) algorithm is an
efficient method of determining the maximum likelihood
estimates of the coefficients in a GLM.
As the name IRLS implies this is an iterative algorithm where the $(k+1)$st
coefficient vector, $\bm\beta^{(k+1)}$, is determined from $\bm\beta^{(k)}$, and the process
continues until convergence.
At the $k$th iteration, the linear predictor is $\bm\eta^{(k)}=\bm X\bm\beta^{(k)}$ and
the mean response vector is $\bm\mu^{(k)}=\bm g^{-1}(\bm\eta^{(k)})$.
The residual on the response scale, $\bm r^{(k)}=\bm y-\bm\mu^{(k)}$, is mapped to a \emph{working residual}
on the linear predictor scale via a linear approximation to the link function.
Because the link and inverse link are diagonal maps we can write the working residual component-wise as
\begin{equation}
  \label{eq:wrkresid}
  \tilde{r}_i^{(k)}=(y_i - \mu_i^{(k)})g^\prime(\mu_i^{(k)})\quad i=1,\dots,n ,
\end{equation}
where $g^\prime(\mu_i^{(k)})$ denotes the derivative of the link function at $\mu_i^{(k)}$.

Because the variances of the working residuals are not constant we must evaluate these variances
and use weighted least squares, with the weights inversely proportional to the variances, to determine an
increment $\bm\delta^{(k)}$ such that $\bm\beta^{(k+1)}=\bm\beta^{(k)}+\bm\delta^{(k)}$. (The \emph{working weights} based on the variances are combined with the prior weights $w_i$, if any have been specified.)
An alternative approach is to define a \emph{working response}
\begin{equation}
  \label{eq:wrkresp}
  \tilde{y}_i^{(k)}=(\eta_i^{(k)}- o_i)+\tilde{r}_i^{(k)},\quad i=1,\dots,k
\end{equation}
where $o_i$ is an optional \emph{offset} to be applied on the linear predictor scale.

IRLS is usually implemented by solving a weighted least squares problem on the working residual
for the increment, $\bm\delta^{(k)}$, at each iteration.
For the \emph{penalized iteratively reweighted least squares} (PIRLS) algorithm to determine
$\bm u^{(k+1)}$ from $\bm u^{(k)}$ it is easiest to use the working response because the penalty
is defined as $\|\bm u^{(k+1)}\|^2$ so the penalized weighted least squares calculation should be
in terms of $\bm u$.

In more detail, the PIRLS algorithm has the form
\begin{enumerate}
\item Given parameter values, $\bm\beta$ and $\bm\theta$, incorporate the fixed-effects contribution
  to the linear predictor, $\bm X\bm\beta$, as an \emph{offset}, $\bm o$, to be used in evaluating the working
  response (eq. \ref{eq:wrkresp}).
  Set the initial values of $\bm u^{(0)}=\bm 0$ so that the PIRLS iterations always start from the same value
  of $\bm u$, providing for a stable result that depends only on $\bm\beta$ and $\bm\theta$.
  Evaluate the lower triangular $\bm\Lambda_{\bm\theta}$, the linear predictor
  \begin{equation}
    \label{eq:linpredPIRLS0}
    \bm\eta^{(0)}=\bm Z\bm\Lambda_{\bm\theta}\bm u^{(0)} + \bm o,
  \end{equation}
  the conditional mean of $\mc Y|\mc U=\bm u^{(0)}$,
  \begin{equation}
    \label{eq:condmeanPIRLS0}
    \bm\mu^{(0)}=\bm g^{-1}(\bm\eta^{(0)}),
  \end{equation}
  the (diagonal) Jacobian matrix,
  \begin{equation}
    \label{eq:Jacobian}
    \bm J^{(0)}=\frac{d \bm\mu}{d \bm\eta^\top}={\bm g^{-1}}^\prime
  \end{equation}
  and the conditional variance
  \begin{equation}
    \label{eq:confvarPIRLS}
    \bm V^{(0)}=\mathrm{Var}(\mc Y|\bm\mu=\bm\mu^{(0)})
  \end{equation}
  which is also a diagonal matrix.
  
\item Establish the weights as the inverse of the combined variance contributions from $\bm\eta$:
  \begin{equation}
    \label{eq:PIRLSweights}
    \bm W^{(0)}=\left(\bm J^{(0)\top}\bm V^{(0)}\bm J^{(0)}\right)^{-1}
  \end{equation}
(we would also multiply this quantity by a diagonal matrix of prior weights, if any were specified).
  Eq. \ref{eq:PIRLSweights} is written as a product and an inverse of matrices but all the matrices involved
  are diagonal and the evaluation of eq.~\ref{eq:PIRLSweights} is performed as vector operations.

\item Evaluate $\bm u^{(1)}$ as the solution to the penalized, weighted least squares problem
  \begin{equation}
    \label{eq:PIRLSeqn}
    \left(\bm \Lambda^\top\bm Z^\top\bm W^{(0)}\bm Z\bm\Lambda + \bm I\right)\bm u^{(1)}=
    \bm\Lambda^\top\bm Z^\top \bm W^{(0)}\tilde{\bm y}^{(0)}
  \end{equation} 
  using the sparse Cholesky factor, $\bm L_{\bm\beta,\bm\theta}^{(0)}$, defined by
  \begin{equation}
    \label{eq:sparseChol}
    \bm L_{\bm\beta,\bm\theta}^{(0)}\bm L_{\bm\beta,\bm\theta}^{(0)\top}=
    \bm P\left(\bm \Lambda^\top\bm Z^\top\bm W^{(0)}\bm Z\bm\Lambda + \bm I\right)\bm P^\top ,
  \end{equation}
  where $\bm P$ is the matrix representation of a \emph{fill-reducing permutation} determined from the
  structure of $\bm Z^\top\bm Z$.
  (In simple GLMM problems $\bm P$ can be an identity matrix but for general sparse Cholesky evaluations
  determining and employing a fill-reducing permutation can be important in reducing memory usage
  and compute time.)
  
\item Evaluate PSUD, the penalized sum of unit deviances, at $\bm u^{(1)}$.
  If the PSUD at $\bm u^{(1)}$ exceeds that at $\bm u^{(0)}$ use \emph{step-halving} which, in this case,
  means replacing $\bm u^{(1)}$ by the average of $\bm u^{(0)}$ and $\bm u^{(1)}$, and trying again.
  We set a maximum, usually 10 or fewer, on the number of step-halvings that are allowed before
  declaring the algorithm to have failed.

\item If the iteration succeeds in reducing the PSUD, check for convergence by comparing the weights,
  $\bm W^{(i+1)}$, to $\bm W^{(i)}$.
  If the relative differences in the weights are small the algorithm is declared to have converged.
  Otherwise repeat the previous steps using evaluations at $\bm u^{(i)}$ to produce $\bm u^{(i+1)}$.

\end{enumerate}

\subsection{Evaluating the likelihood for GLMMs using the Laplace approximation}
\label{sec:Laplace}

Evaluating the likelihood for generalized linear mixed models
requires approximating an intractable 
integral over the random effects distribution. The \code{glmer} function offers 
several approximations, controlled by the \code{nAGQ} argument. 
The default value of \code{nAGQ=1} specifies the
\emph{Laplace approximation} \citep{madsen2011}.

A second-order Taylor series approximation to $-2\log[f_{\mc Y,\mc
  U}(\bm y,\bm u)]$ based at $\tilde{\bm u}$ provides an approximation
of the unscaled conditional density as a multiple of the density for
the multivariate Gaussian $\mathcal{N}(\tilde{\bm u},\bm L\bm
L\trans)$.  The change of variable
\begin{equation}
  \label{eq:LaplaceChg}
  \bm u = \tilde{\bm u} + \bm L\bm z
\end{equation}
provides
\begin{equation}
  \label{eq:GLMMLaplace}
  \begin{aligned}
    L(\bm\beta,\bm\theta|\bm y)&=\int_{\mathbb{R}^q}f_{\mc Y,\mc U}(\bm y,\bm u)\,d\bm u\\
    &\approx \tilde{f}\,|\bm L|\, \int_{\mathbb{R}^q}e^{-\|\bm z\|^2/2}\,(2\pi)^{-q/2}\,d\bm z\\
    &=\tilde{f}\, |\bm L|
  \end{aligned}
\end{equation}
(where $\tilde f$ is the penalized deviance evaluated at the conditional modes)
or, on the deviance scale,
\begin{equation}
  \label{eq:LaplaceDev}
  -2\ell(\bm\beta,\bm\theta|\bm y)\approx\sum_{i=1}^n d(y_i, \mu_i) +
    \|\tilde{\bm u}\|^2 + \log(|\bm L|^2)+\frac{q}{2}\log(2\pi)
\end{equation}

The Laplace approximation normally conditions on both the fixed effects $\beta$ and the variance-covariance parameters $\theta$. A further approximation, which is denoted in \code{glmer} by \code{nAGQ=0}, profiles out the fixed effects by minimizing with respect to $\bm\beta$ and $\bm u$ simultaneously in the PIRLS algorithm (i.e., replacing $\bm u$ with $(\bm \beta \; \bm u)$ and $\bm Z \bm \Lambda$ with $(\bm X | \bm Z \bm \Lambda)$ throughout, except with the penalty term still applying only to $\bm u$). This approximation is exact when (1) $\partial(\log L)/\partial \beta$ is a linear function of the conditional modes $\bm u$ and (2) when the conditional mode is equal to the conditional mean (typically, although not necessarily, implying a symmetric conditional distribution). Both assumptions hold for linear mixed models (although a Laplace approximation is not necessary there), consistent with \cite{bates2015fitting} showing that the fixed effects can be profiled out of the log-likelihood for LMMs. The Julia \code{MixedModels.jl} package offers the same approximation as the \code{fast} argument to the \code{pirls!} function (\url{https://juliastats.org/MixedModels.jl/stable/optimization/}); Template Model Builder \citep{kristensenTMB2016}, and downstream packages such as \code{glmmTMB}, provide this function via a \code{profile} argument.

By default, \code{glmer} uses a two-stage optimization procedure (described below) with \code{nAGQ=0} in the first stage; users can also specify \code{nAGQ=0} for faster, approximate model fits.

\subsubsection{Decomposing the deviance for simple models}
\label{sec:simplescalar}

A common special case of mixed models is those where scalar (typically
intercept) random effects are
associated with levels of a single grouping factor, $\bm h$.  In this
case the dimension, $q$, of the random effects is the number of levels
of $\bm h$ --- i.e.{} there is exactly one random effect associated
with each level of $\bm h$.  We will write the vector of
variance-covariance parameters, which is one-dimensional, as a scalar,
$\theta$.  The matrix $\bm\Lambda_{\bm\theta}$ is a multiple of the
identity, $\theta\bm I_q$, and $\bm Z$ is the $n\times q$ matrix of
indicators of the levels of $\bm f$.  The permutation matrix, $\bm
P$, can be set to the identity and $\bm L$ is diagonal, although not
necessarily homogeneous (i.e., a scalar multiple of the identity matrix).

Because each element of $\bm\mu$ depends on only one element of $\bm
u$ and the elements of $\mc Y$ are conditionally independent, given
$\mc U=\bm u$, the conditional densities of the $u_j,j=1,\dots,q$
given $\mc Y=\bm y$ are independent.  We partition the indices
$1,\dots,n$ as $\mathbb{I}_j,j=1,\dots,q$ according to the levels of
$\bm h$.  That is, the index $i$ is in $\mathbb{I}_j$ if $h_i=j$.
This partitioning also applies to the deviance residuals in that
the $i$th deviance residual depends only on $u_j$ when $i\in\mathbb{I}_j$.

Writing the univariate conditional densities as
\begin{equation}
  \label{eq:univariateCondDens}
  f_j(\bm y,u_j)=\exp\left(-\frac{\sum_{i\in\mathbb{I}_j}d(y_i, u_j)+u_j^2}{2}\right)(2\pi)^{-1/2}
\end{equation}
we have
\begin{equation}
  \label{eq:vectorCondDens}
  f_{\mc Y,\mc U}(\bm y,\bm u)=\prod_{j=1}^q f_j(\bm y,u_j)
\end{equation}
and
\begin{equation}
  \label{eq:ssLike}
  \begin{aligned}
    L(\bm\beta,\bm\theta|\bm y)=\prod_{j=1}^q\int_{\mathbb{R}}f_j(\bm y,u)\,du
  \end{aligned}
\end{equation}

We consider this special case both because it occurs frequently and
because, for some software, it is the only type of GLMM that can be
fit.  Also, in this particular case we can graphically assess the
quality of the Laplace approximation by comparing the actual integrand
to its approximation.

Consider the \code{cbpp} data on contagious bovine pleuropneumonia
(CBPP)
incidence according to season and herd, available in the \pkg{lme4} package
(see \ref{sec:cbpp} for more details), and the model
<<m1>>=
print(m1 <- glmer(cbind(incidence, size-incidence) ~ period + (1|herd),
      cbpp, binomial), corr=FALSE)
@
This model has been fit by minimizing the Laplace approximation to the
deviance.  We can assess the quality of this approximation by
evaluating the unscaled conditional density at $u_j(z)=\tilde{u_j} +
z/{\bm L_{j,j}}$ and comparing the ratio,
$f_j(\bm y,u)/(\tilde{f_j}\sqrt{2\pi})$, to the standard normal
density, $\phi(z)=e^{-z^2/2}/\sqrt{2\pi}$, as shown in Figure~\ref{fig:densities}.
<<density-utils, echo = FALSE>>=
zeta <- function(m, zmin=-3, zmax=3, npts=301L, rank = FALSE) {
    stopifnot (is(m, "glmerMod"),
               length(m@flist) == 1L,    # single grouping factor
              length(m@cnms[[1]]) == 1L) # single column for that grouping factor
    pp <- m
    rr <- m@resp
    u0 <- getME(pp,"u")
    sd <- 1/getME(pp,"L")@x
    ff <- as.integer(getME(pp,"flist")[[1]])
    fc <- getME(pp,"X") %*% getME(pp,"beta") # fixed-effects contribution to linear predictor
    ZL <- t(getME(pp,"Lambdat") %*% getME(pp,"Zt"))
    dc <- function(z) { # evaluate the unscaled conditional density on the deviance scale
        uu <- u0 + z * sd
        rr$updateMu(fc + ZL %*% uu)
        unname(as.vector(tapply(rr$devResid(), ff, sum))) + uu * uu
    }
    zvals <- seq(zmin, zmax, length.out = npts)
    d0 <- dc(0) # because this is the last evaluation, the model is restored to its incoming state
    # signed square root
    sqrtmat <- t(sqrt(vapply(zvals, dc, d0, USE.NAMES=FALSE) - d0)) *
      array(ifelse(zvals < 0, -1, 1), c(npts, length(u0)))
    if (rank) {
      ## order by 'badness' (variance of normalized profile)
      nvals <- exp(-0.5*sqrtmat^2)/sqrt(2*pi)/dnorm(zvals)
      colnames(sqrtmat) <- seq(ncol(sqrtmat))
      sqrtmat <- sqrtmat[,order(apply(sqrtmat, 2, var), decreasing = TRUE)]
    }
    list(zvals=zvals,
         sqrtmat= sqrtmat 
         )
}
smult <- function(x, y) { sweep(x, y, MARGIN = 2, FUN = "*") }
lagr_basis_i <- function(i, xvec, q) {
  num <- sapply(xvec, \(x) prod(x-q[-i]))
  denom <- prod(q[i]-q[-i])
  num/denom
}
lagr_basis_deriv_i <- function(i, xvec, q) {
  if (length(q)==1) return(rep(0, length(xvec)))
  f <- function(j) sapply(xvec, \(x) prod(x-q[-c(i,j)]))
  num <- rowSums(sapply(seq_along(q)[-i], f))
  denom <- prod(q[i]-q[-i])
  num/denom
}
lagr_basis <- function(xvec, q, deriv = FALSE) {
  FUN <- if (!deriv) lagr_basis_i else lagr_basis_deriv_i
  sapply(seq_along(q), \(i) FUN(i, xvec, q))
}
calc_h <- function(xvec, Q) {
  gg <- GHrule(Q)
  ell <- lagr_basis(xvec, gg[,"z"])
  ## dl_i/dx(x_i)
  bb <- lagr_basis(gg[,"z"], gg[,"z"], deriv = TRUE)
  d_ell <- if (length(bb) == 1) bb else diag(bb)
  xdiff <- outer(xvec, gg[,"z"], "-")
  h <- (1 - 2*smult(xdiff, d_ell))*ell^2
  hbar <- xdiff*ell^2
  list(h = h, hbar = hbar)
}
## FIXME: modify to work when Q = 1
ghq_fun <- function(Q, zvals, f, fbar, H = NULL) {
  q <- GHrule(Q)[,"z"]
  H <- H %||% calc_h(zvals, Q = Q)
  dz <- diff(zvals[1:2])
  Qvals <- sapply(q, \(x) which.min(abs(zvals - x)))
  if (any(Qvals == 1)) stop("need to extend zm calculation")
  q_approx <- rowSums(smult(H$h, f[Qvals])) +
    rowSums(smult(H$hbar, fbar[Qvals]))
  q_approx
}
calc_q <- function(Q, zm) {
  dz <- diff(zm$zvals[1:2])
  dmat <- exp(-0.5*zm$sqrtmat^2)/sqrt(2*pi)/dnorm(zm$zvals)
  ## finite difference (NA to match dimension)
  d_dmat <- rbind(NA,apply(dmat, 2, function(x) diff(x)/dz))
  H <- calc_h(zm$zvals, Q = Q)
  res <- sapply(1:ncol(dmat),
                \(i) ghq_fun(Q, zm$zvals, dmat[,i], d_dmat[,i], H = H)
                )
  attr(res, "nAGQ") <- Q
  res
}
plot_zeta <- function(z, q_approx = NULL, norm = FALSE,
                      ylab = if (!norm) "density" else "t(z)",
                      which = NULL, layout = c(5,3), ylim = NULL,
                      auto.key = list(lines = TRUE, points = FALSE)) {
  dmat <- exp(-0.5*z$sqrtmat^2)/sqrt(2*pi)
  Q <- attr(q_approx, "nAGQ") ## rescue attribute before subsetting
  if (!is.null(which)) {
    dmat <- dmat[, which]
    q_approx <- q_approx[, which]
  }
  ## columnwise scaling/unscaling
  ## q_approx is normalized by default
  if (norm) dmat <- dmat/dnorm(z$zvals)
  if (!norm) q_approx <- q_approx*dnorm(z$zvals)
  
  nobs   <- nrow(dmat)
  npanel <- ncol(dmat)
  idx    <- gl(npanel, nobs)
  ngrp <- 3
  
  ref_val <- if (!norm) dnorm(z$zvals) else rep(1, nobs)
  ref_val <- rep(ref_val, npanel)

  grps <- c("conditional\ndensity",
            sprintf("AGQ(%d)", Q),
            if (!norm) "N(0,1)" else "")
  groups <- factor(rep(grps, each = nobs * npanel),
                   levels = grps)
  resp <- c(as.vector(dmat), as.vector(q_approx), ref_val)
  ylim <- ylim %||% extendrange(resp)
  zvec <- rep(z$zvals, ngrp * npanel)
  xyplot(resp ~ zvec | rep(idx, ngrp),
         groups = groups,
         type = c("g", "l"), aspect = 0.6, layout = layout,
         xlab = "z", ylab = ylab,
         auto.key = auto.key,
         par.settings = list(superpose.line = list(lty = 1:3)),
         ylim = ylim)
}
@
\begin{figure}[tbp]
  \centering
<<densities,echo=FALSE,fig.height=5>>=
zm <- zeta(m1, -3.750440, 3.750440)
qq <- calc_q(3, zm)
plot_zeta(zm, q_approx = qq)
@
\caption{Comparison of univariate integrands (solid, blue line); 3-point Gauss-Hermite quadrature (dashed, yellow); and standard normal density function (green, dotted) for the CBPP model. For this model, the three approximations are nearly indistinguishable.}
  \label{fig:densities}
\end{figure}
As Figure~\ref{fig:densities} shows, the univariate integrands are very
close to the standard normal density, indicating that the Laplace
approximation to the deviance is a good approximation in this case.

\section{Adaptive Gauss-Hermite quadrature for GLMMs}
\label{sec:aGQ}
When the first integral in (\ref{eq:GLMMLaplace}) can be expressed as a product of
low-dimensional integrals, we can use Gauss-Hermite quadrature to
provide a closer approximation to the integral.  Univariate
Gauss-Hermite quadrature evaluates the integral of a function that is
multiplied by a ``kernel'' where the kernel is a multiple of
$e^{-z^2}$ or $e^{-z^2/2}$.  For statisticians the natural candidate
is the standard normal density, $\phi(z)=e^{-z^2/2}/\sqrt(2\pi)$.
A $k$th-order Gauss-Hermite formula provides knots, $z_i,i=1,...,k$,
and weights, $w_i,i=1,\dots,k$, such that
\begin{equation}
  \label{eq:GHquad}
  \int_{\mathbb{R}}t(z)\phi(z)\,dz\approx\sum_{i=1}^kw_it(z_i)
\end{equation}
The function \code{GHrule} in \pkg{lme4} (based on code in the
\pkg{SparseGrid} package) provides knots and weights relative to the
standard normal kernel for orders $k$ from 1 to 100.  For example,
<<GHrule5>>=
GHrule(5)
@
where \code{z} is the vector of knots, \code{w} is the vector
of weights, and \code{ldnorm} is the log-density of the standard
normal distribution at $z$.

The choice of the value of $k$ depends on the behavior of the function
$t(z)$.  If $t(z)$ is a polynomial of degree $2k-1$ then the
Gauss-Hermite formula for orders $k$ or greater provides an exact
answer.  The fact that we want $t(z)$ to behave like a low-order
polynomial is often neglected in the formulation of a Gauss-Hermite
approximation to a quadrature.  The quadrature knots on the $u$ scale
are chosen as
\begin{equation}
  \label{eq:quadraturepts}
  u_{i,j}(z)=\tilde{u_j} + z_i/{\bm L_{j,j}},\quad i=1,\dots,k;\;j=1,\dots,q
\end{equation}
exactly so that the function $t(z)$ should behave like a low-order
polynomial over the region of interest, which is to say the region
where quadrature knots with large weights are located.  The term
``adaptive Gauss-Hermite quadrature'' reflects the fact that the
approximating Gaussian density is scaled and shifted to provide a
second order approximation to the logarithm of the unscaled
conditional density.

Figure~\ref{fig:tfunc}
shows $t(z)$ for each of the unidimensional integrals in the
likelihood for the model \code{m1} at the parameter estimates.
While this view shows the deficiency of the Laplace approximation
(deviation from a horizontal reference line at $z=1$) clearly,
The AGQ(3) approximation fits extremely well over the range shown.
The tails of the polynomials implied by the AGQ approximation
can fluctuate widely, but these fluctuations are suppressed by
multiplying by the thin tails of the Gaussian distribution (so that
in Figure~\ref{fig:densities} the deviations of the conditional density
from the standard Normal are barely visible).

\begin{figure}[tbp]
  \centering
<<tfunc,echo=FALSE>>=
plot_zeta(zm, q_approx = qq, norm = TRUE, ylim = c(0.5, 1.5))
@
  \caption{The function $t(z)$, which is the ratio of the normalized
    unscaled conditional density to the standard normal density, for
    each of the univariate integrals in the evaluation of the deviance
    for model \code{m1} (blue, solid line).
    As in Figure~\ref{fig:densities}, the dashed yellow line shows
    the approximation for 3-point adaptive Gauss-Hermite quadrature.
    The horizontal green reference line ($z=1$)
    gives the reference value that would apply for the Laplace approximation,
    which assumes the conditional density is exactly equal to the
    standard normal. The $y$-axis limits are truncated to (0.5, 1.5).
}
  \label{fig:tfunc}
\end{figure}

<<toenail, echo = FALSE>>=
data("toenail", package = "lme4")
ttab <- with(toenail, table(table(patientID)))
## sum(as.numeric(names(ttab))*ttab/sum(ttab))
@

The CBPP data set is a relatively well-behaved data set, where Laplace approximation works well. In contrast,
a widely used data set on toenail onychomycosis \citep{debacker+1998}, which has a very low effective sample size per cluster --- an
average of about 6.5 binary observations (``moderate or severe'' vs ``none or mild'' disease) per patient --- represents
an example where Gauss-Hermite quadrature is necessary for reliable results. In this case the
conditional densities depart much more clearly from the standard normal (Figure~\ref{fig:toenailplot}; note the scale of the
density ratios goes from 0 to 10, in contrast the maximum of 4 in Figure~\ref{fig:tfunc}).
\cite{stringerAsymptoticsNumericalIntegration2022} and \cite{stringerExactGradientEvaluation2024} further explore the limitations of Laplace approximation.
%% from Alex S.: The "Stringer A, Bilodeau B, Tang Y (2022)” paper was accepted in March 2025: Bilodeau, B., Stringer, A., and Tang, Y. (2026). Asymptotics of adaptive quadrature for two-level mixed models. Bernoulli (forthcoming). It is scheduled to be published quite soon I believe.

\begin{figure}[tbp]
  \centering
<<toenailplot,echo=FALSE>>=
data("toenail", package = "lme4")
toemod <- glmer(outcome ~ time*treatment + (1 | patientID), 
                     data = toenail, family = binomial(link = "logit"))
zm <- zeta(toemod)
q_toe <- calc_q(3, zm)
set.seed(101); s <- sample(ncol(zm$sqrtmat), size = 9)

gridExtra::grid.arrange(
             plot_zeta(zm, q_approx = q_toe, which = s, layout = c(3,3), auto.key = FALSE),
             plot_zeta(zm, which = s, norm = TRUE, q_approx = q_toe, layout = c(3,3),
                       ylim = c(0, 5)), nrow = 1,
             ## widths hand-tweaked to equalize plot size with/without scale
             widths = c(0.5, 0.63))

@
\caption{Normalized unscaled conditional density (left) and ratio of density to the standard normal density (right)
  for a random sample of 9 patients from the toenail onychomycosis data set. The $y$-axis limits in the right panel ($t(z)$) are
  truncated to $(0,5)$.}
\label{fig:toenailplot}
\end{figure}

To use adaptive Gauss-Hermite quadrature for model fitting in \code{glmer} models, 
users would set the argument \code{nAGQ}, the number of quadrature points 
(i.e. $k$ in eq.~\ref{eq:GHquad}), to a value greater than 1.

Increasing the number of nodes generally improves the accuracy of the likelihood 
approximation at the expense of computation time --- although rounding errors
may accumulate when using large numbers of quadrature points.
A reasonable strategy for choosing the number of nodes is to re-fit the model with increasing values of \code{nAGQ} until the parameter estimates stabilize
\citep{tuerlinckxStatisticalInferenceGeneralized2006a}; a ``warm-start'' procedure using the parameter estimates from the previous fit as starting values (and using \code{control = glmerControl(nAGQ0initStep = FALSE)} to disable the \code{nAGQ=0} preliminary fit) will speed up this procedure.

At present, AGQ is only available 
for models with a single scalar random effect. (The \code{GLMMadaptive} package implements
AGQ for vector-valued random effect models, although it is still restricted to models with
a single random effect; \cite{rabe-heskethMaximumLikelihoodEstimation2005b} describe methods
for AGQ for models with nested random effects.)

\section{Model fitting}

  Once we can calculate the deviance by PIRLS for specified values of $\btheta$ and $\bbeta$ (or only $\btheta$ if profiling out the fixed effects via \code{nAGQ=0}), we then estimate the parameters by nonlinear optimization. This procedure largely follows the description in \cite{bates2015fitting}, using derivative-free optimizers with box constraints to prevent non-positive-(semi)definite covariance matrices. Specifically, the elements of $\btheta$ corresponding to the diagonal of $\bm\Lambda_\theta$ are currently constrained to be non-negative.
  
The only difference is that by default \code{glmer} uses a two-step fitting procedure, using \code{nAGQ=0} at the first stage to get preliminary estimates which are then used as starting points for a second optimization with Laplace approximation or Gauss-Hermite quadrature as specified by the user. Different nonlinear optimizers can be used at each stage: the current default, as specified in \code{glmerControl}, is to use Powell's BOBYQA \citep{Powell_bobyqa} followed by a box-constrained variant of the Nelder-Mead simplex algorithm. For faster, approximate fitting, the second stage can be omitted; in the rare cases where the initial \code{nAGQ=0} fit gives poor results, the first stage can be skipped via \code{glmerControl(nAGQ0initStep = FALSE)}.
  
However, because the profiled log-likelihood is an even function of the diagonal elements of $\btheta$
and $\bm\Sigma_\theta=\bm\Lambda_\theta\bm\Lambda_\theta\trans$ depends on them only through their squares, positive and negative values yield identical likelihoods. This symmetry means that constrained optimization is not strictly necessary --- an unconstrained optimizer will converge to a correct solution, approaching zero from either side in the boundary case of a singular random-effects covariance matrix. (Once a solution is found, we can map it to a unique solution where the diagonal elements are all non-negative.)
A similar approach to removing constraints could work for structured covariance matrices where correlation parameters are constrained to $(-1,1)$, e.g. by parameterizing the model in terms of a phase parameter $p$ where $\rho = \sin(p)$ (and then mapping $p$ to $(0, 2\pi)$).
Removing these constraint would permit the use of a broader class of unconstrained optimizers, though this feature has not yet been incorporated into \code{lme4}.

\section{Examples}

\subsection{CBPP}
\label{sec:cbpp}

The \code{?cbpp} help page describes the CBPP data set
\citep{lesnoff_within-herd_2004} as follows:
\begin{quote}
Contagious bovine pleuropneumonia (CBPP) is a major disease of
cattle in Africa, caused by a mycoplasma.  This dataset describes
the serological incidence of CBPP in zebu cattle during a
follow-up survey implemented in 15 commercial herds located in the
Boji district of Ethiopia.  The goal of the survey was to study
the within-herd spread of CBPP in newly infected herds. Blood
samples were quarterly collected from all animals of these herds
to determine their CBPP status.  These data were used to compute
the serological incidence of CBPP (new cases occurring during a
given time period).  Some data are missing (lost to follow-up).
\end{quote}
\cite{lesnoff_within-herd_2004} estimated
the effects of different treatments using (1) ordinary logistic regression
incorporating a variance-inflation factor, also known as
a quasi-binomial model (``logistic regression'' is sometimes
used specifically to describe analyses of Bernoulli responses, but
in this case there are multiple trials per observation [cows that
could become seropositive], and so a dispersion or scale parameter
can be estimated); (2) a GLMM implemented in \code{lme4}; and a
(3) Markov chain Monte Carlo algorithm \citep{zeger1991generalized}, 
which as they state allows for a non-parametric rather than a Normal model 
for the random effects.
The authors did not find any significant effects of treatment,
ascribing the null results to 
``a lack of power in the statistical analyses or to a quality problem for the medications used (and more generally, for health-care delivery in the Boji district).''

<<newcbpp2,echo=FALSE>>=
cbpp2 <- read.csv(system.file("vignette_data", "cbpp2.csv",
            package = "lme4"))
cbpp2 <- transform(cbpp2,period=factor(period),
                   treatment=factor(treatment,
                   levels=c("Partial/null","Complete","Unknown")))
@

(Note that Table 1 of \cite{lesnoff_within-herd_2004} contains a known
typographical error for herd 6. Consequently, results obtained using
the \code{cbpp} data set may not exactly reproduce some of the findings
reported in that paper.)

The \code{lme4} package includes two variants of the \code{cbpp}
data set. The second variant, \code{cbpp2}, contains corrected values
corresponding to Table~1 of \cite{lesnoff_within-herd_2004}. Although
the precise provenance of these data sets is unclear, the \code{cbpp}
data set matches the version held by the corresponding author of
\cite{lesnoff_within-herd_2004}.

<<cbppPlot,echo=FALSE,fig.cap="Incidence (proportion of cows becoming seropositive per observation period) vs. period. Colours show treatment category for each herd; point sizes reflect the number of seronegative cows at the start of each period. Lines connect the sets of observations from each herd.",fig.scap="Incidence">>=
g0 <- ggplot(cbpp2,aes(period, incidence/size, colour=treatment, shape = treatment)) +
    ## scale_y_continuous(limits = c(0, 1))+
    labs(x="Period",y="Proportional incidence")+
    scale_colour_brewer(palette="Dark2")
## faceted plot
## g0 + 
##     geom_point(aes(size = size)) +
##     facet_wrap(~herd) +
##     zmargin
## 
g0 +  geom_point(aes(size = size),alpha=0.4) +
    geom_line(aes(group=herd),alpha=0.3)
## geom_boxplot(fill="black",alpha=0.1,width=0.5,
## aes(group=interaction(period,treatment)))
## 
## boxplot(incidence/size ~ period, data = cbpp, las = 1,
##        xlab = 'Period', ylab = 'Probability of sero-positivity')
@

We model proportional incidence as a binomial response depending
on the additive fixed effects of period, treatment and average herd size;
to account for repeated measures we fit a model with a random effect
of herd. In practice we might also be interested in a period by treatment
interaction, but we neglect that term here.

As in \code{glm}, we can specify 
a binomial response as a proportion and use the \code{weights}
argument to specify the sample size, instead of the
more typical two-column \code{cbind(successes,failures)} format:
<<cbppModelI>>=
gm1 <- glmer(incidence/size ~ period + treatment + avg_size + (1 | herd),
             family = binomial,
             data = cbpp2, weights = size)
@

<<cbpp_diag,eval=FALSE,echo=FALSE>>=
## diagnostic junk about N-M vs bobyqa differences
tmpf <- function(x) c(deviance(x), unlist(getME(x,c("theta","beta"))))
gm1 <- glmer(incidence/size ~ period + treatment + avg_size + (1 | herd),
             family = binomial,
             data = cbpp2, weights = size)
gm1B <- update(gm1,control=glmerControl(optimizer="bobyqa"))
cbind(tmpf(gm1),tmpf(gm1B))
dd1 <- update(gm1,devFunOnly=TRUE)
dd1B <- update(gm1B,devFunOnly=TRUE)
gm1@optinfo$derivs$gradient
gm1B@optinfo$derivs$gradient
gm1@optinfo$derivs$gradient/gm1B@optinfo$derivs$gradient
library(numDeriv)
grad(dd1,tmpf(gm1)[-1])
grad(dd1B,tmpf(gm1B)[-1])
@

It is also worth considering
adding an observation-level random effect to the model \citep{harrisonComparisonObservationlevelRandom2015},
which we can do by creating a new factor based on observation
number and using \code{update()} on the previous model (we switch the optimizer to BOBYQA for both phases
for one of the updates):
<<cbppModelII, warning=FALSE, message=FALSE>>=
cbpp2 <- transform(cbpp2,obs=factor(seq(nrow(cbpp2))))
bob_opt <- glmerControl(optimizer = "bobyqa")
## herd and observation-level REs
gm2 <- update(gm1,.~.+(1|obs), control = bob_opt)  
## observation-level REs only
gm3 <- update(gm1,.~.-(1|herd)+(1|obs))  
@

<<diagnose, echo=FALSE, warning=FALSE, message=FALSE, results='hide'>>=
aa <- allFit(gm2, verbose=FALSE)
ok <- summary(aa)[["which.OK"]]
lapply(aa[ok], \(x) x@optinfo$conv$lme4$messages)
@ 
\subsubsection{Model summary}

The first part of the summary reiterates
the family and link function used,
the model formula, and gives various summary statistics
(log-likelihood etc.), as well as quantiles of the
scaled (Pearson) residuals:
<<summary1,echo=FALSE>>=
ss <- summary(gm1)
cc <- capture.output(print(summary(gm1)))
reRow <- grep("^Random effects",cc)
cat(cc[1:(reRow-2)],sep="\n")
@
These quantities are also
accessible via standard accessors
(\code{AIC()}, \code{BIC()}, \code{logLik()}).

The next chunk of \code{summary()} describes the random effects
and the number of levels associated with each grouping factor
(the latter is useful for checking that random-effects formulae
have been specified correctly):
<<summary2,echo=FALSE>>=
feRow <- grep("^Fixed effects",cc)
cat(cc[reRow:(feRow-2)],sep="\n")
@
This information is also accessible via \code{VarCorr()}, which returns
a list of variance-covariance matrices (the \code{print} method
for \code{VarCorr} objects allows control of whether the variance,
or standard deviation, or both, are printed).

Next come the estimates of the fixed effects, along with Wald
estimates of the standard error, $Z$ statistic, and $p$-value:
<<summary3,echo=FALSE>>=
corRow <- grep("^Correlation",cc)
cat(cc[feRow:(corRow-2)],sep="\n")
@
One can use \code{coef(summary())} to retrieve this information,
and optionally format it with \code{printCoefmat()}.

The last component of \code{summary()} gives the
estimated correlations among the fixed-effect parameters,
which can be useful for assessing multicollinearity
(it can also be overwhelming: it is suppressed by default
for models with more than 20 fixed-effect parameters, and
can also be suppressed by using \code{print(summary(.),correlation=FALSE)}).
<<summary4,echo=FALSE>>=
cat(cc[corRow:length(cc)],sep="\n")
@

\subsubsection{Diagnostics}

A range of graphical diagnostic tools is available
for \code{merMod} objects. The plot methods in the \code{lme4}
package are inspired by those in the \code{nlme} package, using
\code{lattice} plots to provide a reasonable blend
of convenience and flexibility. 

\code{merMod} objects are also compatible with the 
\code{performance} package and the \code{DHARMa} package, both commonly 
used for model checking.

The following code produces a standard range
of diagnostic plots (Figure~\ref{fig:glmerDiag}), similar to the ones
in base R's \code{plot.lm} method. These diagnostics will generally be
useful for models where the conditional density is approximately normal
(but heteroscedastic) --- e.g., Poisson responses with large mean or
binomial responses with large numbers of successes --- and less so otherwise,
e.g. for binary responses.

<<glmerDiagFake,eval=FALSE>>=
## basic residual plot
plot(gm1)
## scale-location plot
plot(gm1,sqrt(abs(resid(.)))~fitted(.),type=c("p","smooth"))
## boxplot of residuals grouped by a categorical predictor
plot(gm1,period~resid(.))
## Q-Q plot
qqmath(gm1)
@    

<<glmerDiag,fig.cap="Graphical diagnostics (using package built-in functions)",warning=FALSE,echo=FALSE>>=
p1 <- plot(gm1,type=c("p","smooth"),
    main="Default (Pearson resid vs. fitted)")
p2 <- plot(gm1,sqrt(abs(resid(.)))~fitted(.),type=c("p","smooth"),
     main="Scale-location")
p3 <- plot(gm1,period~resid(.), main = "Grouped residuals")
p4 <- qqmath(gm1,main="Quantile-quantile plot",type=c("p","r"))
grid.arrange(p1,p2,p3,p4,nrow=2)
@    

The \code{ranef()} accessor extracts the conditional modes; the argument
\code{condVar=TRUE} additionally extracts the variances of
the conditional modes, which are stored as an attribute
labelled \code{"postVar"}\footnote{In earlier versions of the software, these elements were called ``posterior variances'' rather than ``conditional variances'', based on the close correspondence between mixed models and hierarchical Bayesian models.} --- a three-dimensional array that
gives the variance-covariance matrix of the conditional modes
for each level of the grouping variable. The plotting methods
\code{dotplot()} and \code{qqmath()} return lists of graphical
objects showing \emph{caterpillar plots} (ordered values of
the random effects with confidence bars); in the case of
the Q-Q plot (\code{qqmath}) the $y$-axis shows corresponding 
values of the standard normal quantiles (Figure~\ref{fig:glmerRanefplot}).

<<glmerRanefplot,fig.cap="Graphical display of random effects. \\emph{Left}: conditional modes $\\pm 1.96 \\times \\text{conditional standard deviation}$, ordered by magnitude. \\emph{Right}: quantile-quantile plot, with linear regression line overlaid.",echo=FALSE>>=
rr <- ranef(gm1, condVar=TRUE)
dd <- dotplot(rr)
qq <- qqmath(rr, type=c("p","r"))
grid.arrange(dd$herd, qq$herd, ncol=2)
@

\code{performance::check\_model()} checks a variety of classical assumptions 
of GLMs. For mixed-effect models, it also assesses
the normality of the distribution of conditional modes (Figure~\ref{fig:perf_chk_mod_cbpp}).

<<perf_chk_mod_cbpp,fig.cap="Model diagnostics using \\code{performance::check\\_model}. The plot showcasing the normality of random effects (lower right panel) is the transpose of the right panel in Figure 6.",fig.height=10,echo=FALSE>>=
check_model(gm1, check = c("pp_check", "binned_residuals",
                           "qq", "reqq"))
@

The \code{DHARMa} package is also compatible with \code{merMod} objects.
\code{DHARMa} generates simulation-based residuals for
generalized linear (mixed) models and uses them for graphical
and statistical tests of model assumptions (Figure~\ref{fig:dharma_mod_cbpp}).

<<dharma_mod_cbpp,fig.cap="Model assumption checks using the DHARMa package. \\code{n.s.} denotes ``not significant''.",echo=FALSE,warning=FALSE>>=
## FIXME: versions of DHARMa before 0.5.0 gave a significant nonlinearity test, so the caption
## included: 'Other exploration (such as \\code{plot(simulationOutput, form = cbpp2\\$period)}) does not reveal any obvious problems such as nonlinear responses, so we proceed with the analysis despite the significant adjusted quantile test.'
## Post 0.5.0, there are no test rejections; instead we get the warning 'In getSimulations.merMod(fittedModel, nsim = n, simulateREs = simulateREs,  :
##  Model was fit with prior weights. These will be ignored in the simulation. See ?getSimulations for details.'
## (is this a false positive?)
## 
simulationOutput <- simulateResiduals(fittedModel = gm1)
plot(simulationOutput)
@

Having checked the diagnostics, we would now like to compare
the three models we have fitted.  Inspecting the \code{VarCorr}
components, we see that when we fit both herd- and observation-level
random effects, the among-herd variance is estimated as zero.
The appropriate procedure
at this point (e.g. whether one
drops non-significant terms, or those with small scaled magnitudes,
or those that worsen the AIC or BIC of the model)
depends on the goals of the analysis and one's
philosophy of model-building \citep{barrRandomEffectsStructure2013,matuschekBalancingTypeError2017,scandola2024}.

One might either stick with the full model, or continue
with the reduced model with observation-level random effects
only (as it has exactly the same likelihood as the full
model but uses an additional parameter, it would be chosen
according to either an information-theoretic or a
hypothesis-testing model selection framework).

Here we will start by
computing likelihood profiles and confidence intervals (CIs)
for the model incorporating both random effects;
although it has the same point estimates and maximum likelihood
as the reduced model, confidence intervals that incorporate
non-local information (i.e. profile- or parametric bootstrap-based)
will give different, more conservative results for the full model.

%% lme4 wishlist: allow [Ww]ald in confint.merMod?
<<loadbatch,echo=FALSE>>=
cbpp_batch_vars <- load(system.file("vignette_data", "cbpp_batch.rda", package = "lme4"))
contr_batch_vars <- load(system.file("vignette_data", "Contraception_batch.rda", package = "lme4"))
@

The \code{profile} method computes profile likelihoods.
The computation can be slow, since complete profiling for
a model with $p$ random- and fixed-effect parameters requires fitting $p$
profiles, each of which requires many $p-1$-dimensional optimizations.  The machinery for generating likelihood profiles for GLMMs is similar to that for LMMs (see \cite{bates2015fitting}, \S\ 5.1).

The \code{profile} method returns an
object of class \code{thpr} --- a data frame
containing the profiles, augmented with attributes containing
interpolation splines for each parameter profile and their inverses
(using \code{splines::interpSpline} and \code{splines::backSpline});
the latter are used for plotting profiles and computing confidence
intervals.  An \code{as.data.frame} method adds \code{.focal} and
\code{.par} variables to the data frame, useful for customized
plots.

Profiles can be used for univariate (\code{xyplot}) and
bivariate (\code{splom}) profile plots, and to compute
profile confidence intervals (\code{confint}). (\code{confint}
applied to a \code{glmer} fit will first fit the profile,
then use it to compute profile confidence intervals.  Given
the computational cost of profiling, it makes sense to compute
and save the profile as an intermediate step if one plans to
do anything other than computing confidence intervals.)

Two other common methods for computing confidence intervals
are parametric bootstrapping (\code{method = "boot"}) and the
classical Wald approximation (\code{method = "Wald"}).
Parametric bootstrapping is much slower, but more accurate
(and, via the \code{FUN} argument, can generate confidence intervals
for any quantity that can be derived from a fitted model).
The Wald approximation is faster and less accurate than profile
confidence intervals.
By default \code{glmer} only returns estimates for the fixed-effects
parameters, as the assumptions of the Wald approximation are often
violated badly for random-effects (co)variances and correlations.
In the examples below we use the finite-difference Hessian
(second derivative matrix of the estimated parameters) and the
delta method to compute Wald confidence intervals for random-effects
standard deviations and correlations, when possible.

Figure~\ref{fig:cbppcompplot} compares all three of these confidence intervals
across all three of the models fitted.

<<cbppConfidence,echo=FALSE>>=
des_ord <- c("sd_(Intercept)|herd", "sd_(Intercept)|obs",
             "avg_size", "treatmentUnknown", "treatmentComplete")
combCI <- subset(cbpp_combCI,var!="(Intercept)" & !grepl("^period",var))
combCI <- transform(combCI,  var = factor(var, levels = rev(des_ord)))
combCI <- transform(combCI,
            vartype = factor(ifelse(grepl("^sd", var), "random effects", "fixed effects"))
)                    
@

<<cbppcompplot,fig.cap="CBPP example: comparison of point and confidence interval estimation for different methods. Wald CIs are missing for random-effects parameters when the fitted model is singular.",fig.scap="CBPP comparison",echo=FALSE,fig.width=7,fig.height=5,warning=FALSE>>=
mnames <- cbpp_df_name$mnames
pd <- position_dodge(width=0.6)
ggplot(combCI, aes(var, est, colour=type, shape=model)) +
  geom_pointrange(aes(ymin=lwr, ymax=upr),
                  position=pd) +
  labs(x="", y="Estimate (log-odds of seropositivity)") +
  geom_hline(yintercept=0, lty=2) +
  coord_flip() +
  scale_colour_brewer(palette="Dark2", guide = revguide) +
  scale_shape(guide=revguide, labels = mnames) +
  facet_wrap(~ vartype, ncol = 1, scale = "free")
@    

If the default optimizers (BOBYQA followed by Nelder-Mead) do not
perform well, one could attempt to re-fit the model with a variety of 
different optimizers using \code{allFit()}.
The command \code{allFit(show.meth.tab=TRUE)} lists the optimizers \code{lme4}
currently supports.

To use \code{allFit()}, supply the initial fitted model as input. Useful
results are obtained from \code{summary(allFit())}; the components below
(other than \code{\$which.OK}) apply only to the optimizers that succeeded:
\begin{itemize}
  \item \code{\$which.OK} --- which optimizers worked
  \item \code{\$llik} --- log-likelihoods
  \item \code{\$fixef} --- fixed-effect estimates
  \item \code{\$sdcor} --- random-effect standard deviations and correlations
  \item \code{\$theta} --- random-effect parameters on the Cholesky scale
\end{itemize}

We will show the summary results for \code{\$sdcor}; the other components are similar.

<<allfit_cbpp1,message=FALSE,warning=FALSE>>=
gm_all <- allFit(gm1)
@

<<allfit_cbpp2>>=
ss <- summary(gm_all)
ss$sdcor
@


As of version 2.0, \code{lme4} can also specify structured variance-covariance 
matrices for (generalized) linear mixed models.
\code{lme4} now supports unstructured (general positive definite), diagonal, 
compound symmetry, and first-order autoregressive (AR1) structures.
By default, AR1 models assume a homogeneous-variance model (the variance is the same for all time steps), while the other models assume heterogeneous-variance models (variances differ for every level of the varying term); users can adjust this with the \code{hom} argument (e.g. \code{ar1(..., hom = FALSE)}).

The unstructured covariance structure is the default for mixed models.
Here we illustrate fitting an AR1 model; the next example will show a compound
symmetric model.

<<covar_cbpp>>=
gm.ar1 <- glmer(incidence/size ~ ar1(1 + herd | period),
                family = binomial,
                data = cbpp, weights = size)
print(VarCorr(gm.ar1))
@

For this particular model, a heterogeneous AR1 model (\code{ar1(..., hom = FALSE)}) results in a singular fit.

\subsection{Contraception}

\citet{huq1990bangladesh} use multilevel models to analyze data from a
fertility survey of women in Bangladesh. These data are available as
the \code{Contraception} object in the \pkg{mlmRev} package. The
response variable is binary and indicates whether or not each woman
was using contraception at the time of the survey. Covariates included
the woman's age, the number of live children she had, whether she
lived in an urban or rural setting, and the district in which she
lived.

<<loadContraception, echo=FALSE>>=
library(mlmRev)
data(Contraception)
@ 

<<graphContraception,echo=FALSE,fig.cap="Contraception example: proportion of contraceptive use by centered age, number of living children (line type), and urban/rural residence (facet). Curves fitted using generalized additive models with cubic spline smoothing. Density plots along bottom and top margins show the age distributions of women not using contraception (bottom) and using contraception (top).">>=
Contraception <- transform(Contraception, 
                        facet_urban = ifelse(urban=="Y","Urban","Rural"))
gg0 <- ggplot(Contraception) +
  geom_smooth(aes(age, ifelse(use == "Y",1,0), colour = livch, fill = livch,
                  linetype = livch),
                method = "gam", alpha = 0.1,
                method.args = list(gamma = 0.6),
                formula = y ~ s(x, bs = "cs")) +
    facet_wrap(~facet_urban) +
    scale_x_continuous("Centered age") + 
  scale_y_continuous("Proportion", limits = c(0, 1), expand = c(0,0)) +
  scale_linetype("Number\nof living\nchildren") +
  scale_colour_brewer("Number\nof living\nchildren", palette="Dark2") +
  scale_fill_brewer("Number\nof living\nchildren", palette="Dark2",
                    guide = guide_legend(override.aes = list(alpha = 0.1))) +
  ## wider key to see line types:
  ## https://stackoverflow.com/a/7624378/190277
  theme(legend.key.width = unit(3, "line"))

## add marginal density ribbons
gg0 + geom_density(
        data = subset(Contraception, use == "N"),
        aes(x = age, y = after_stat(density), fill = livch),
        alpha = 0.3, colour = NA) +
  geom_ribbon(
    data = subset(Contraception, use == "Y"),
    stat = "density",
    aes(
      x = age,  
      ymin = 1 - after_stat(density) * 0.5,
      ymax = 1,
      fill = livch
    ),
    alpha = 0.3, colour = NA
  )

@ 

<<ch,echo=FALSE>>=
Contraception <- transform(Contraception,
   ch = factor(livch != 0, labels = c("N","Y")),
   age_s = age/(2*sd(age)))
@  

Figure~\ref{fig:graphContraception} shows exploratory smooth curves of
contraceptive use as a function of centered age, stratified by number
of living children and urban/rural residence. In rural areas, women
with two living children show the highest rates of contraceptive use,
peaking near 50\% at the average age, while women in urban areas with
one living child show the highest rates near the average
age but with a more pronounced decline at older ages. In both
settings, women with no living children consistently show the lowest
rates of contraceptive use. The nonmonotonic effect of age on contraceptive use
and the apparent dependence of this age trend on child status motivate the
model specifications explored below. (These nonmonotonic trends were not
noticed in the original analysis of the data, which concluded that there
was no significant effect of age.)

We construct six \code{glmer} models with varying choices of explanatory
variables and random effects structure, comparing them via \code{anova}. The
models considered different variables.

For instance, there are two different ways to encode the number of living 
children: \code{livch} is a four-level factor distinguishing \code{0}, \code{1}, 
\code{2}, or \code{3+} children, while other models use \code{ch} instead, which is a binary indicator for whether the 
woman has any living children (in Table~\ref{tab:contraictab} and Figure~\ref{fig:contracompplot}
we label this variable as \code{binary\_child}).

Second, we vary whether child status interacts with the woman's age: two models
include \code{ch} (or \code{livch}) and \code{age} as additive terms, while the 
other four include a \code{ch} and \code{age} interaction. A quadratic effect of
age (\code{I(age\^{}2)}) was added to account for the nonlinear effect of age.

Third, we explore different random effects structures at the district level.
Three of them use a single random intercept per district \code{(1 | district)}.
One of them extends this with a random slope for urban status 
\code{(urban | district)}, allowing the urban/rural difference to vary by district. 
The next uses nested random intercepts for district and site within district (\code{1 | district/urban})
separating district-level from urban-within-district variation.
(\cite{scandola2024} refer to this formulation as a ``complex random intercepts'' model).
The remaining model uses only the \code{urban:district} grouping. 
To reduce convergence warnings and facilitate interpretability, age (already approximately centered
in the data set) was standardized by scaling by twice the standard deviation \citep{gelman2008scaling}.

<<cm1,message=FALSE>>=
cm1 <- glmer(use ~ age_s + I(age_s^2) + urban + livch + (1|district), 
                           Contraception, binomial)
## switch from livch (ordinal) to ch (binary)
cm2 <- update(cm1, . ~ . - livch + ch)
## add age by children interaction
cm3 <- update(cm2, . ~ . + age_s:ch)
## allow urban effect to vary across districts (correlated)
cm4 <- update(cm3, . ~ . - (1|district) + (1+urban|district))
## compound symmetric/nested formulation
cm5 <- update(cm3, . ~ . - (1|district) + (1 | district/urban))
## as above but drop district effect
cm6 <- update(cm3, . ~ . - (1|district) + (1 | district:urban))
@

<<anovaContraception, echo = FALSE>>=
## copy of bbmle::AICtab
AICtab <- function(..., mnames) {
  aic_df <- do.call(AIC, list(...))
  LLvec <- sapply(list(...), function(x) c(logLik(x)))
  rownames(aic_df) <- mnames
  aic_df <- transform(aic_df,
                        Δnegloglik = -LLvec - min(-LLvec),
                        ΔAIC = AIC - min(AIC))
  aic_df <- subset(aic_df, select = -AIC)
  aic_df <- aic_df[order(aic_df$ΔAIC),]
  aic_df
}

mod_list <- c(cm1, cm2, cm3, cm4, cm5, cm6)
op <- options(digits = 3)
mnamevec <- c("int_child    + age  + (1 | district)",
              "binary_child + age  + (1 | district)",
                "binary_child × age + (1 | district)",
                "binary_child × age + (1 + urban | district)",
                "binary_child × age + (1 | district/urban)",
                "binary_child × age + (1 | district:urban)")
aictab <- do.call(AICtab, c(mod_list, list(mnames = mnamevec)))
@

\begin{table}
<<aictab, echo  = FALSE, results = "asis">>=
knitr::kable(aictab, digits = 3)
@
\caption{Model comparison for Contraception fits. Note that for likelihood ratio tests, or for AIC comparisons restricted to nested models \citep{ripleySelectingAmongstLarge2004a}, the nesting sequence for the random-effects models is \code{(1+urban|district)} $>$ \code{(1|district/urban)} $>$ \{%
\code{(1|district)}, \code{(1|district:urban)}%
\}.
}
\label{tab:contraictab}
\end{table}

Table~\ref{tab:contraictab} shows that
the top three models, all of which include a child-by-age interaction and some effect of urbanization, fit approximately equally well ($\Delta$ negative log-likelihood $<0.5$). The \code{(1|district/urban)} model barely improves on the fit of \code{(1|district:urban)} (0.005 log-likelihood units), at the cost of an extra variance parameter, so it is almost 2 AIC units worse. \code{(1 + urban | district)} is a bit better ($\approx$ 0.5 log-likelihood units), but includes a covariance parameter.
Models that drop the interaction between child status and age or replace the binary child indicator with an integer count perform substantially worse ($\Delta \textrm{AIC} > 10$).

Overall, the results suggest that accounting for urban/rural variation at the district level and including the age and child interaction are both important, but the precise random effects structure for urban/rural variation is relatively
unimportant.

<<contr-predplot, warning = FALSE, fig.cap="Predictions for contraception fit (\\code{(1|district/urban)} model). Heavy lines and ribbons show population-level predictions; light lines show district-level predictions.", echo = FALSE>>=
agevec <- seq(-15, 20)
pframe <- with(Contraception, expand.grid(age = agevec, urban = levels(urban), ch = levels(ch)))
pframe <- transform(pframe, age_s = age/(2*sd(Contraception$age)),
                    facet_urban = ifelse(urban=="Y","Urban","Rural"))
pred  <- predict(cm5, newdata = pframe, se.fit = TRUE, re.form = NA)
pframe$use <- plogis(pred$fit)
pframe$lwr <- plogis(pred$fit-1.96*pred$se.fit)
pframe$upr <- plogis(pred$fit+1.96*pred$se.fit)
pframe2 <- with(Contraception, expand.grid(age = agevec, urban = levels(urban), ch = levels(ch),
                 district = levels(district)))
pframe2 <- transform(pframe2, age_s = age/(2*sd(Contraception$age)),
            facet_urban = ifelse(urban=="Y","Urban","Rural"),
            dist_urb = paste(urban, district, sep = ":"))
c_urb <- unique(with(Contraception, paste(urban, district, sep = ":")))
pframe2 <- subset(pframe2, dist_urb %in% c_urb)
pred2  <- predict(cm5, newdata = pframe2, re.form = NULL)
pframe2$use <- plogis(pred2)
pframe2$ch <- factor(pframe2$ch,
                     levels = c("N", "Y"),
                     labels = c("no", "yes"))
pframe$ch <- factor(pframe$ch,
                     levels = c("N", "Y"),
                     labels = c("no", "yes"))

ggplot(pframe, aes(age)) + 
  geom_line(aes(y = use, colour = ch)) +
  geom_ribbon(aes(ymin = lwr, ymax = upr, fill = ch), color = NA, alpha = 0.3) +
  facet_wrap(~facet_urban) +
  scale_x_continuous("Centered age") + 
  scale_y_continuous("Proportion", limits = c(0, 1), expand = c(0,0)) +
  scale_colour_brewer("Living\nchildren", palette="Dark2",
                      guide = guide_legend(reverse = TRUE)) +
  scale_fill_brewer("Living\nchildren", palette="Dark2",
                    guide = guide_legend(reverse = TRUE)) +
  geom_line(data = pframe2, aes(group = interaction(dist_urb, ch), colour = ch, y = use), alpha = 0.2)

@

As with the CBPP data set, we can also compute
profile, Wald, and parametric bootstrap confidence intervals for all of the models to
understand the effects of each variable and visualize the among-model variation.

<<ContraceptionBatch,echo=FALSE>>=
contr_combCI <- subset(contr_combCI, var != "(Intercept)")
combCI <- transform(contr_combCI,
    vartype = factor(ifelse(grepl("^(sd|cor)", var), "random effects", "fixed effects")),
    model = factor(model, levels = rev(levels(model)))  ## flip AND flip order of guide below
    )
@

<<contracompplot,fig.cap="Contraception example: comparison of point and confidence interval estimation for different methods. (Note that the Wald CI for variation in the intercept across districts, in the \\code{(1|district/urban)} model, includes negative values.)",fig.scap="Contraception comparison",echo=FALSE,fig.width=10,fig.height=10,warning=FALSE>>=
ggplot(combCI, aes(var, est, colour=type, shape=model)) +
  geom_pointrange(aes(ymin=lwr, ymax=upr),
                  position=position_dodge(width=0.6)) +
  labs(x="", y="Estimate (log-odds of seropositivity)") +
  geom_hline(yintercept=0, lty=2) +
  coord_flip() +
  scale_colour_brewer(palette="Dark2", guide = revguide) +
  scale_shape(guide=revguide) +
  facet_wrap(~vartype, ncol = 1, scale = "free", space = "free_y") +
  theme(legend.position = "inside",
  legend.justification = c("left", "top"),
  legend.box = "horizontal",
      legend.background = element_rect(fill = "white", colour = "grey50"),
      #legend.margin = margin(t = 5, r = 5, b = 5, l = 5, unit = "pt"),
      legend.box.margin = margin(t = 10, l = 10, unit = "pt"))
@

The point estimates and confidence intervals of the explanatory variables are similar across the six models, and across different methods for confidence interval construction, with a few exceptions. 

Urban residence, presence of living children, and age were all strong predictors of contraceptive use among women in Bangladesh (because we are fitting a quadratic model for the effect of age, the non-significant effect of \code{age\_s} simply means that the marginal effect of age \emph{at the mean age} is not clearly negative or positive).

Returning to the structured covariance matrices introduced in
Section~\ref{sec:cbpp}, we can replace the unstructured random effect
\code{(1+urban|district)} with diagonal (\code{diag}) or compound symmetric
(\code{cs}) structures, with either heterogeneous or homogeneous variances
(via the \code{hom} argument):

<<cm_varcorr, echo = FALSE>>=
cc <- glmerControl(check.conv.grad = .makeCC("warning", tol = 1e-2), check.conv.hess = "ignore")
base_form <- use ~ age*ch + I(age^2) + urban
cm.diag <- glmer(update(base_form, . ~ . + diag(1 + urban|district)), Contraception, binomial, control = cc )
cm.homdiag <- glmer(update(base_form, . ~ . + diag(1 + urban|district, hom = TRUE)), Contraception, binomial, control = cc)
cm.homcs <- glmer(update(base_form, . ~ . + cs(1 + urban|district, hom = TRUE)), Contraception, binomial, control = cc)
@

For example, the compound symmetric model is fitted as follows:

<<cm_cs, message = FALSE, warning = FALSE>>=
cm.cs <- glmer(use ~ age*ch + I(age^2) + urban + cs(1 + urban | district),
               Contraception, binomial, control = cc)
@

We can use \code{VarCorr} and other accessor methods as we would for models with default, unstructured covariance matrices, e.g.:

<<cm_varcorr_view>>=
print(VarCorr(cm.cs))
@

\section*{Acknowledgements}

We would like to thank Steve Walker, for early work on \code{glmer}, and Alex Stringer, for helpful comments on the manuscript.

\bibliography{glmer}

\section*{Package versions used}

<<pkgs, echo=FALSE>>=
pkglist <- c("lme4", "performance", "DHARMa", "see")
pp <- sapply(pkglist, function(x)
  sprintf("%s: %s", x, format(packageVersion(x))))
ppp <- paste(pp, collapse  = "; ")
@ 

Compiled with \Sexpr{R.version.string} and package versions \Sexpr{pp}.

\section{Appendix: derivation of PIRLS}

We seek to maximize the unscaled conditional log density for a GLMM
over the conditional modes, $\bm u$. This problem is very similar to
maximizing the log-likelihood for a GLM, which is a very
thoroughly studied problem \citep[e.g.][]{McCullaghNelder1989}.
The standard algorithm for dealing with this kind of
problem is iteratively reweighted least squares (IRLS). Here we modify
IRLS by incorporating a penalty term that accounts for variation in
the random effects; we call the resulting algorithm
penalized iteratively reweighted
least squares (PIRLS).

The unscaled conditional log-density takes the form,
\begin{equation}
  f(\bm u) = \log p(\bm y, \bm u | \bm\beta, \bm\theta) = 
  \bm\psi^\top \bm A \bm y - 
  \bm a^\top \bm \phi  + 
  \bm c -
  \frac{1}{2}\bm u^\top \bm u -
  \frac{q}{2}\log{2\pi}
  \label{eq:unsccondlogdens}
\end{equation}
where $\bm\psi$ is the $n$-by-$1$ canonical parameter of an
exponential family, $\bm\phi$ is the $n$-by-$1$ vector of cumulant
functions, $\bm c$ an $n$-by-$1$ vector of normalizing constants, and
$\bm A$ is an $n$-by-$n$ diagonal matrix of prior weights, $\bm
a$. Both $\bm a$ and $\bm c$ could depend on a dispersion parameter,
although we ignore this possibility for now.

The canonical parameter, $\bm\psi$, and vector of cumulant functions,
$\bm\phi$, depend on a linear predictor,
\begin{equation}
  \bm\eta = \bm o + \bm X \bm\beta + \bm Z \bm\Lambda_\theta \bm u
\end{equation}
where $\bm o$ is an $n$-by-$1$ vector of \emph{a priori} offsets. The
specific form of this dependency is specified by the choice of the
exponential family. The mean of this distribution, $\bm\mu$,
is the \emph{inverse link function} $g^{-1}$ applied to $\bm\eta$.

Our goal is to find the values of $\bm u$ that maximize the unscaled
conditional density, for given $\bm\theta$ and $\bm\beta$
vectors. These maximizers are the conditional modes, which we require
for the Laplace approximation and adaptive Gauss-Hermite
quadrature. To do this maximization we use a variant of the Fisher
scoring method, which is the basis of the iteratively reweighted least
squares algorithm for generalized linear models. Fisher scoring is
itself based on Newton's method, which we apply first.

\subsection{Newton's method}

To apply Newton's method, we need the gradient and the Hessian of the
unscaled conditional log-likelihood. Following standard GLM theory
\citep{McCullaghNelder1989}, we use the chain rule,
\begin{displaymath}
  \frac{d L(\bm\beta, \bm\theta | \bm y, \bm u)}{d \bm u} = 
  \frac{d L'(\bm\beta, \bm\theta | \bm y, \bm u)}{d \bm\psi}
  \frac{d \bm\psi}{d \bm\mu}
  \frac{d \bm\mu}{d \bm\eta}
  \frac{d \bm\eta}{d \bm u} - \frac{1}{2} \frac{d \, (\bm u^\top \bm u)}{d \, \bm u}
\end{displaymath}
where $L'$ represents the log-density in (\ref{eq:unsccondlogdens}) exclusive of the penalty term.
The first derivative in the first term's chain follows from basic results in GLM
theory,
\begin{displaymath}
  \frac{d L'(\bm\beta, \bm\theta | \bm y, \bm u)}{d \bm\psi} = 
  (\bm y - \bm\mu)^\top \bm A .
\end{displaymath}
Again from standard GLM theory, the next two derivatives define the
inverse diagonal variance matrix,
\begin{displaymath}
  \frac{d \bm\psi}{d \bm\mu} = \bm V^{-1}
\end{displaymath}
and the diagonal Jacobian matrix,
\begin{displaymath}
\frac{d \bm\mu}{d \bm\eta} = \bm M \quad .
\end{displaymath}
Finally, because $\bm u$ affects $\bm\eta$ only linearly,
\begin{displaymath}
  \frac{d \bm\eta}{d \bm u} = \bm Z \bm\Lambda_\theta
\end{displaymath}
Therefore we have,
\begin{equation}
  \frac{d L(\bm\beta, \bm\theta | \bm y, \bm u)}{d \bm u} = 
  (\bm y - \bm\mu)^\top \bm A
  \bm V^{-1}
  \bm M
  \bm Z \bm\Lambda_\theta -
  \bm u^\top \quad .
\label{eq:dPDEVdu}
\end{equation}
This is very similar to the gradient for GLMs with respect to fixed
effects coefficients, $\bm\beta$. The only difference induced by
including the penalty term for the random effects ($\bm u$), is the
subtraction of the $\bm u^\top$ term.

Again we apply the chain rule to take the Hessian,
\begin{equation}
  \frac{d^2 L(\bm\beta, \bm\theta | \bm y, \bm u)}{d \bm u \, d \bm u} = 
  \frac{d^2 L'(\bm\beta, \bm\theta | \bm y, \bm u)}{d \bm u \, d \bm\mu}
  \frac{d \bm\mu}{d \bm\eta}
  \frac{d \bm\eta}{d \bm u} - \bm I_q
\end{equation}
which leads to,
\begin{equation}
  \frac{d^2 L(\bm\beta, \bm\theta | \bm y, \bm u)}{d \bm u \, d \bm u} = 
  \frac{d^2 L'(\bm\beta, \bm\theta | \bm y, \bm u)}{d \bm u \, d \bm\mu}\bm
  M \bm Z \bm\Lambda_\theta 
  - \bm I_q
\end{equation}
The first derivative in this chain can be expressed as,
\begin{equation}
  \frac{d^2 L'(\bm\beta, \bm\theta | \bm y, \bm u)}{d \bm u \, d \bm\mu} =
  -\bm\Lambda_\theta^\top \bm Z^\top \bm M \bm V^{-1} \bm A  + 
  \bm\Lambda_\theta^\top \bm Z^\top \left[ \frac{d \bm M \bm V^{-1}}{d \bm\mu} \right] \bm A \bm R
\end{equation}
where $\bm R$ is a diagonal residuals matrix with $\bm y-\bm\mu$ on
the diagonal. The two terms arise from a type of product rule, where
we first differentiate the residuals, $\bm y-\bm\mu$, and then the
diagonal matrix, $\bm M \bm V^{-1}$, with respect to $\bm\mu$.

The Hessian can therefore be expressed as,
\begin{equation}
  \frac{d^2 L(\bm\beta, \bm\theta | \bm y, \bm u)}{d \bm u d \bm u} = 
  -\bm \Lambda_\theta^\top \bm Z^\top \bm M \bm A^{1/2}\bm V^{-1/2}\left( 
    \bm I_n - 
    \bm V \bm M^{-1}\left[ \frac{d \bm M \bm V^{-1}}{d \bm\mu} \right] \bm R
  \right) \bm V^{-1/2}\bm A^{1/2} \bm M \bm Z \bm\Lambda_\theta - \bm I_q
\label{eq:betaHessian}
\end{equation}
This result can be simplified by expressing it in terms of a weighted
random-effects design matrix, $\bm U = \bm A^{1/2}\bm V^{-1/2}\bm M
\bm Z \bm\Lambda_\theta$,
\begin{equation}
  \frac{d^2 L(\bm\beta, \bm\theta | \bm y, \bm u)}{d \bm u \, d \bm u} = 
  -\bm U^\top\left( 
    \bm I_n - 
    \bm V \bm M^{-1}\left[ \frac{d \bm V^{-1}\bm M}{d \bm\mu} \right] \bm R
  \right) \bm U - \bm I_q \quad .
\label{eq:betaHessiansimp}
\end{equation}

\subsection{Fisher-like scoring}

There are two ways to further simplify this expression for $\bm U^\top
\bm U$. The first is to use the canonical link function for the family
being used. Canonical links have the property that $\bm V = \bm M$,
which means that for canonical links,
\begin{equation}
  \frac{d^2 L(\bm\beta, \bm\theta | \bm y, \bm u)}{d \bm u \, d \bm u} = 
  -\bm U^\top\left( 
    \bm I_n - 
    \bm I_n \left[ \frac{d \bm I_n}{d \bm\mu} \right] \bm R
  \right) \bm U - \bm I_q = - \bm U^\top \bm U - \bm I_q
\end{equation}
The second way to simplify the Hessian is to take its expectation with
respect to the distribution of the response, conditional on the
current values of the spherical random effects coefficients, $\bm
u$. The diagonal residual matrix, $\bm R$, has expectation
0. Therefore, because the response only enters into the expression for
the Hessian via $\bm R$, we have that,
\begin{equation}
  E\left(\frac{d^2 L(\bm\beta, \bm\theta | \bm y, \bm u)}{d \bm u \, d \bm
      u} | \bm u \right) = 
  -\bm U^\top\left( 
    \bm I_n - 
    \bm U \bm M^{-1}\left[ \frac{d \bm V^{-1}\bm M}{d\mu} \right] E(\bm R)
  \right) \bm U - \bm I_q = - \bm U^\top \bm U - \bm I_q
\label{eq:simphesstwo}
\end{equation}

Like base R's \code{glm}, \code{glmer} uses Fisher scoring throughout (which is identical to Newton's method for canonical links) to enable the use of the (P)IRLS algorithm. As usual, we minimize the conditional negative log-density rather than maximizing the conditional log-density.

\end{document}

\name{Arabidopsis}
\alias{Arabidopsis}
\docType{data}
\title{
  Arabidopsis clipping/fertilization data
}
\description{
  Data on genetic variation in responses to fertilization
  and simulated herbivory in \emph{Arabidopsis}
}
\usage{data("Arabidopsis")}
\format{
  A data frame with 625 observations on the following 8 variables.
  \describe{
    \item{\code{reg}}{region: a factor with 3 levels \code{NL}
      (Netherlands), \code{SP} (Spain), \code{SW} (Sweden)}
    \item{\code{popu}}{population: a factor with the form \code{n.R}
      representing a population in region \code{R}}
    \item{\code{gen}}{genotype: a factor with 24 (numeric-valued) levels}
    \item{\code{rack}}{a nuisance factor with 2 levels, one for
      each of two greenhouse racks}
    \item{\code{nutrient}}{fertilization treatment/nutrient level (1,
      minimal nutrients or 8, added nutrients)}
    \item{\code{amd}}{simulated herbivory or "clipping" (apical meristem
      damage): \code{unclipped} (baseline) or \code{clipped}}
    \item{\code{status}}{a nuisance factor for germination method
      (\code{Normal}, \code{Petri.Plate}, or \code{Transplant})}
    \item{\code{total.fruits}}{total fruit set per plant (integer)}
  }
}
\source{
  \insertCite{banta2010comprehensive}{lme4}
}
\references{
  \insertRef{banta2010comprehensive}{lme4}
}
\examples{
data(Arabidopsis)
summary(Arabidopsis[,"total.fruits"])
table(gsub("[0-9].","",levels(Arabidopsis[,"popu"])))
library(lattice)
stripplot(log(total.fruits+1) ~ amd|nutrient, data = Arabidopsis,
          groups = gen,
          strip=strip.custom(strip.names=c(TRUE,TRUE)),
          type=c('p','a'), ## points and panel-average value --
          ## see ?panel.xyplot
          scales=list(x=list(rot=90)),
          main="Panel: nutrient, Color: genotype")
}
\keyword{datasets}

\name{GQdk}
\title{Sparse Gaussian / Gauss-Hermite Quadrature grid}
\alias{GQdk}
\alias{GQN}
\description{
  Generate the sparse multidimensional Gaussian quadrature grids.

  Currently unused.  See \code{\link{GHrule}()} for the version
  currently in use in package \pkg{lme4}.
}
\usage{
  GQdk(d = 1L, k = 1L)
  GQN
}
\arguments{
  \item{d}{integer scalar - the dimension of the function
    to be integrated with respect to the standard
    \code{d}-dimensional Gaussian density.}
  \item{k}{integer scalar - the order of the grid.  A grid
    of order \code{k} provides an exact result for a
    polynomial of total order of \code{2k - 1} or less.}
}
\value{
  \code{GQdk()} returns a matrix with \code{d + 1} columns.  The first
  column is the weights and the remaining \code{d} columns are the
  node coordinates.

  \code{GQN} is a \code{\link{list}} of lists, containing the
  non-redundant quadrature nodes and weights for integration of a scalar
  function of a \code{d}-dimensional argument with respect to the density
  function of the \code{d}-dimensional Gaussian density function.
  \cr
  The outer list is indexed by the dimension, \code{d}, in the
  range of 1 to 20.  The inner list is indexed by \code{k},
  the order of the quadrature.
}
\note{
  \code{GQN} contains only the non-redundant nodes.  To regenerate
  the whole array of nodes, all possible permutations of
  axes and all possible combinations of \eqn{\pm 1}{+/- 1}
  must be applied to the axes.  This entire array of nodes is exactly
  what \code{\link{GQdk}()} reproduces.

  The number of nodes gets very large very quickly with
  increasing \code{d} and \code{k}.  See the charts at
  \url{http://www.sparse-grids.de}.
}
\examples{
GQdk(2,5) # 53 x 3

GQN[[3]][[5]] # a 14 x 4 matrix
}

\name{VarCorr}
\title{Extract Variance and Correlation Components of Covariance}
\alias{VarCorr}
\alias{VarCorr.merMod}
\alias{as.data.frame.VarCorr.merMod}
\alias{print.VarCorr.merMod}
\description{
  This function calculates the estimated variances, standard deviations,
  and correlations between the random-effects terms in a mixed-effects
  model, of class \code{\linkS4class{merMod}} (linear, generalized or
  nonlinear).  The within-group error variance and standard deviation
  are also calculated.
}
\usage{
\method{VarCorr}{merMod}(x, sigma = 1, \dots)

%% documented because of "surprising arguments":
\method{as.data.frame}{VarCorr.merMod}(x, row.names = NULL, optional = FALSE,
              order = c("cov.last", "lower.tri"), \dots)
\method{print}{VarCorr.merMod}(x, digits = max(3L, getOption("digits") - 2L),
      comp = "Std.Dev.", formatter = format, \dots)
}
\arguments{
  \item{x}{for \code{VarCorr}: a fitted model object, usually an object inheriting from
    class \code{\linkS4class{merMod}}. For \code{as.data.frame}, a
    \code{VarCorr.merMod} object returned from \code{VarCorr}.}
  \item{sigma}{an optional numeric value used as a multiplier for the
    standard deviations.}
  \item{digits}{an optional integer value specifying the number of digits.}
  \item{order}{arrange data frame with variances/standard deviations
    first and covariances/correlations last for each random effects
    term (\code{"cov.last"}), or in the order of the lower triangle of the
    variance-covariance matrix (\code{"lower.tri"})?}
  \item{row.names, optional}{Ignored: necessary for the
    \code{as.data.frame} method.}
  \item{\dots}{Ignored for the \code{as.data.frame} method; passed to
    other \code{\link{print}()} methods for the \code{print()} method.}
  %% print():
  \item{comp}{a \code{\link{character}} vector, specifying the components
    to be printed; simply passed to \code{\link{formatVC}()}.}
  \item{formatter}{a \code{\link{function}} for formatting the numbers;
    simply passed to \code{\link{formatVC}()}.}
}
\value{
  An object of class \code{VarCorr.merMod}. The internal
  structure of the object is
  a list of matrices, one for each random effects grouping
  term.  For each grouping term, the standard deviations and
  correlation matrices for each grouping term are stored as
  attributes \code{"stddev"} and \code{"correlation"},
  respectively, of the variance-covariance matrix, and the
  residual standard deviation of LMMs is stored as attribute
  \code{"sc"}.
  
  Users may also access the attributes \code{"theta"} and 
  \code{"rho"} (if applicable) which is explained in greater 
  detail within the covariance structures vignette, and 
  \code{"profpar"}, which is a different parameterization 
  of \code{"theta"} but suitable for profiling.


The \code{as.data.frame} method produces a combined data frame with one
row for each variance or covariance parameter (and a row for the
residual error term where applicable) and the following columns:
  \describe{
    \item{grp}{grouping factor}
    \item{var1}{first variable}
    \item{var2}{second variable (\code{NA} for variance parameters)}
    \item{vcov}{variances or covariances}
    \item{sdcor}{standard deviations or correlations}
  }
}
\details{
  The \code{print} method for \code{VarCorr.merMod} objects has optional 
  arguments \code{digits} (specify digits of precision for printing) and 
  \code{comp}: the latter is a character vector with any combination of 
  \code{"Variance"} and \code{"Std.Dev."}, to specify whether variances,
  standard deviations, or both should be printed. 
}
\author{
  This is modeled after \code{\link[nlme]{VarCorr}} from
  package \CRANpkg{nlme}, by Jose Pinheiro and Douglas Bates.
}
\seealso{
  \itemize{
    \item \code{\link[lme4]{lmer}}, \code{\link[lme4]{nlmer}},
    \item The \code{"Residual Std.Dev."} output corresponds to 
      \code{\link[lme4]{sigma.merMod}}.
  }
}
\examples{
data(Orthodont, package="nlme")
fm1 <- lmer(distance ~ age + (age|Subject), data = Orthodont)
print(vc <- VarCorr(fm1))  ## default print method: standard dev and corr
## both variance and std.dev.
print(vc,comp=c("Variance","Std.Dev."), digits=2)
## variance only
print(vc, comp=c("Variance"))
## standard deviations only, but covariances rather than correlations
print(vc, corr = FALSE)
as.data.frame(vc)
as.data.frame(vc, order="lower.tri")
}
\keyword{models}

\name{isSingular}
\title{Test Fitted Model for (Near) Singularity}
\alias{isSingular}
\alias{isSingular-methods}
\alias{isSingular,merMod-method}
\alias{isSingular,Covariance.us-method}
\alias{isSingular,Covariance.diag-method}
\alias{isSingular,Covariance.cs-method}
\alias{isSingular,Covariance.ar1-method}
\alias{getSingTol}
\concept{boundary}
\description{
  Evaluates whether a fitted mixed model is (almost / near) singular, i.e.,
  the parameters are on the boundary of the feasible
  parameter space: variances of one or more linear combinations of effects
  are (close to) zero.
}
\usage{
isSingular(object, tol = getSingTol())
getSingTol()
}
\arguments{
  \item{object}{a fitted \code{merMod} object (result of \code{lmer} or \code{glmer}) 
  or a \code{Covariance} object.}
  \item{tol}{numerical tolerance for detecting singularity.}
}
\details{
  Complex mixed-effect models (i.e., those with a
  large number of variance-covariance parameters) frequently result in
  \emph{singular} fits, i.e. estimated variance-covariance matrices with
  less than full rank. Less technically, this means that some
  "dimensions" of the variance-covariance matrix have been estimated as
  exactly zero. For scalar random effects such as intercept-only models,
  or 2-dimensional random effects such as intercept+slope models,
  singularity is relatively easy to detect because it leads to
  random-effect variance estimates of (nearly) zero, or estimates of
  correlations that are (almost) exactly -1 or 1. However, for more
  complex models (variance-covariance matrices of dimension >=3)
  singularity can be hard to detect; models can often be singular
  without any of their individual variances being close to zero
  or correlations being close to +/-1.

  This function performs a simple test to determine whether any of the
  random effects covariance matrices of a fitted model are singular.
  The \code{\link{rePCA}} method provides more detail about the
  singularity pattern, showing the standard deviations
  of orthogonal variance components and the mapping from
  variance terms in the model to orthogonal components
  (i.e., eigenvector/rotation matrices).

  While singular models are statistically well defined (it is
  theoretically sensible for the true maximum likelihood estimate
  to correspond to a singular fit), there are real concerns
  that (1) singular fits correspond to overfitted models that may
  have poor power; (2) chances of numerical problems and mis-convergence
  are higher for singular models (e.g. it may be computationally
  difficult to compute profile confidence intervals for such
  models); (3) standard inferential procedures such as Wald statistics
  and likelihood ratio tests may be inappropriate.

  There is not yet consensus about how to deal with singularity, or
  more generally to choose which random-effects specification (from
  a range of choices of varying complexity) to use.  Some proposals
  include:

  \itemize{
    \item avoid fitting overly complex models in the first place,
    i.e. design experiments/restrict models \emph{a priori} such that the
    variance-covariance matrices can be estimated precisely enough
    to avoid singularity (Matuschek et al 2017)
    \item use some form of model selection to choose a model that
    balances predictive accuracy and overfitting/type I error
    (Bates et al 2015, Matuschek et al 2017)
    \item \dQuote{keep it maximal}, i.e. fit the most complex
    model consistent with the experimental design, removing only
    terms required to allow a non-singular fit (Barr et al. 2013),
    or removing further terms based on p-values or AIC
    \item use a partially Bayesian method
    that produces maximum \emph{a posteriori} (MAP)
    estimates using \emph{regularizing} priors to force
    the estimated random-effects variance-covariance matrices
    away from singularity (Chung et al 2013, \CRANpkg{blme} package)
    \item use a fully Bayesian method that both regularizes the
    model via informative priors and gives estimates
    and credible intervals for all parameters that average
    over the uncertainty in the random effects parameters
    (Gelman and Hill 2006, McElreath 2015;
    \CRANpkg{MCMCglmm}, \CRANpkg{rstanarm} and \CRANpkg{brms} packages)
  }

  \code{getSingTol()} is a utility function that just returns the
  default tolerance for testing singularity (1e-4, in the current
  version); if the user has set
  \code{options(lme4.singular.tolerance=...)},
  it will retrieve this value instead.
  
}
\value{
  a logical value
}
\seealso{\code{\link{getME}}, \code{\link{rePCA}}}
\references{
  \itemize{
    \item \insertRef{barr2013random}{lme4}
    \item \insertRef{bates2015parsimonious}{lme4}
    \item \insertRef{chung2013nondegenerate}{lme4}
    \item \insertRef{gelman2007data}{lme4}
    \item \insertRef{matuschek2017balancing}{lme4}
    \item \insertRef{mcelreath2015statistical}{lme4}
  }
}
\docType{methods}
\keyword{methods}
\keyword{utilities}

\name{confint.merMod}
\alias{confint.merMod}
\alias{confint.thpr}
\title{Compute Confidence Intervals for Parameters of a [ng]lmer Fit}
\description{
  Compute confidence intervals on the parameters of a \code{*lmer()}
  model fit (of class\code{"\linkS4class{merMod}"}).
}
\usage{
\method{confint}{merMod}(object, parm, level = 0.95,
        method = c("profile", "Wald", "boot"), zeta,
        nsim = 500,
        boot.type = c("perc", "basic", "norm"),
        FUN = NULL, quiet = FALSE,
        oldNames, signames = TRUE,
        boot.scale = c("sdcor", "vcov"),
        \dots)
\method{confint}{thpr}(object, parm, level = 0.95,
        zeta, non.mono.tol = 1e-2, \dots)
}

\arguments{
  \item{object}{a fitted [ng]lmer model or profile}
  \item{parm}{parameters for which intervals are sought. Specified by an
    integer vector of positions, \code{\link{character}} vector of
    parameter names, or (unless doing parametric bootstrapping with a
    user-specified bootstrap function) \code{"theta_"} or \code{"beta_"}
    to specify variance-covariance or fixed effects parameters only: see the
    \code{which} parameter of \code{\link[=profile.merMod]{profile}}.}
  \item{level}{confidence level \eqn{< 1}, typically above 0.90.}
  \item{method}{a \code{\link{character}} string determining the method
    for computing the confidence intervals.}
  \item{zeta}{(for \code{method = "profile"} only:) likelihood cutoff
    (if not specified, as by default, computed from \code{level}).}
  \item{nsim}{number of simulations for parametric bootstrap intervals.}
  \item{FUN}{bootstrap function; if \code{NULL}, an internal function
    that returns the fixed-effect parameters as well as the
    random-effect parameters on the standard deviation/correlation scale
    will be used. See \code{\link{bootMer}} for details.}
  \item{boot.type}{bootstrap confidence interval type, as described
    in \code{\link[boot]{boot.ci}}. (Methods \sQuote{stud} and \sQuote{bca}
    are unavailable because they require additional components to be
    calculated.)}
  \item{quiet}{(logical) suppress messages about computationally intensive profiling?}
  \item{oldNames}{(logical) deprecated; has the same purpose as \code{signames}.}
  \item{signames}{(logical) use abbreviated names of the form
    \code{.sigNN} for variance-covariance parameters rather than more
    meaningful (but longer) names of the form
    \code{sd_(Intercept)|Subject}?  See the \code{signames} argument of
    \code{\link{profile.merMod}}.}
  \item{non.mono.tol}{tolerance for detecting a non-monotonic profile
    and warning/falling back to linear interpolation}
  \item{boot.scale}{scale for bootstrap CIs (see \code{prof.scale}
  argument for \code{link{profile.merMod}})}
  \item{\dots}{additional parameters to be passed to
    \code{\link{profile.merMod}} or \code{\link{bootMer}}, respectively.}
}
\value{
  a numeric table (\code{\link{matrix}} with column and row names) of
  confidence intervals; the confidence intervals are computed on the
  standard deviation scale.
}
\note{
  The default method \code{"profile"} amounts to \preformatted{
    confint(profile(object, which=parm, ...),
            level, zeta)}
  where the \code{\link{profile}} method \code{\link{profile.merMod}}
  does almost all the computations.  Therefore it is typically
  advisable to store the \link[=profile.merMod]{profile(.)} result, say
  in \code{pp}, and then use \code{confint(pp, level=*)} e.g., for
  different levels.
}
\details{
  Depending on the \code{method} specified, \code{confint()} computes
  confidence intervals by
  \describe{
    \item{\code{"profile"}:}{computing a
      likelihood profile and finding the appropriate cutoffs
      based on the likelihood ratio test;}
    \item{\code{"Wald"}:}{approximating
      the confidence intervals (of fixed-effect parameters
      only; all variance-covariance parameters
      CIs will be returned as \code{NA})
      based on the estimated local curvature of the
      likelihood surface;}
    \item{\code{"boot"}:}{performing parametric
      bootstrapping with confidence intervals computed from the
      bootstrap distribution according to \code{boot.type} (see
      \code{\link{bootMer}}, \code{\link[boot]{boot.ci}}).}
  }
}
\examples{
if (interactive() || lme4_testlevel() >= 3) {
fm1 <- lmer(Reaction ~ Days + (Days|Subject), sleepstudy)
fm1W <- confint(fm1, method="Wald")# very fast, but not useful for "sigmas" = var-cov pars
fm1W
(fm2 <- lmer(Reaction ~ Days + (Days || Subject), sleepstudy))
(CI2 <- confint(fm2, maxpts = 8)) # method = "profile"; 8: to be much faster
\dontshow{ stopifnot(all.equal(tolerance = 5e-6, signif(unname(CI2), 7),
               array(c(15.25847, 3.964157, 22.88062, 237.5732,  7.33431,
                       37.78184, 8.768238, 28.78768, 265.2383, 13.60057),
                     dim = c(5L, 2L))))
}
if (lme4_testlevel() >= 3) {
  system.time(fm1P <- confint(fm1, method="profile", ## <- default
                              signames = FALSE))
  ## --> ~ 2.2 seconds (2022)
  set.seed(123) # (reproducibility when using bootstrap)
  system.time(fm1B <- confint(fm1, method="boot", signames=FALSE,
                              .progress="txt", PBargs= list(style=3)))
  ## --> ~ 6.2 seconds (2022) and warning, messages
} else {
    load(system.file("testdata","confint_ex.rda",package="lme4"))
}
fm1P
fm1B
} ## if interactive && testlevel>=3
} 

\docType{data}
\name{cake}
\alias{cake}
\title{Breakage Angle of Chocolate Cakes}
\format{A data frame with 270 observations on the following 5 variables.
  \describe{
    \item{\code{replicate}}{a factor with levels \code{1} to \code{15}}
    \item{\code{recipe}}{a factor with levels \code{A}, \code{B} and \code{C}}
    \item{\code{temperature}}{an ordered factor with levels \code{175}
      < \code{185} < \code{195} < \code{205} < \code{215} < \code{225}}
    \item{\code{angle}}{a numeric vector giving the angle at which the
      cake broke.}
    \item{\code{temp}}{numeric value of the baking temperature (degrees F).}
  }}
\source{
  Original data were presented in Cook (1938), and reported
  in Cochran and Cox (1957, p. 300).  Also cited in Lee,
  Nelder and Pawitan (2006).
}
\description{
  Data on the breakage angle of chocolate cakes made with
  three different recipes and baked at six different
  temperatures.  This is a split-plot design with the
  recipes being whole-units and the different temperatures
  being applied to sub-units (within replicates). The
  experimental notes suggest that the replicate numbering
  represents temporal ordering.
}
\details{
  The \code{replicate} factor is nested within the
  \code{recipe} factor, and \code{temperature} is nested
  within \code{replicate}.
}
\references{
  \itemize{
    \item \insertRef{cook1938chocolate}{lme4}
    \item \insertRef{cochran1957experimental}{lme4}
    \item \insertRef{lee2006generalized}{lme4}
  }
}
\examples{
str(cake)
## 'temp' is continuous, 'temperature' an ordered factor with 6 levels

(fm1 <- lmer(angle ~ recipe * temperature + (1|recipe:replicate), cake, REML= FALSE))
(fm2 <- lmer(angle ~ recipe + temperature + (1|recipe:replicate), cake, REML= FALSE))
(fm3 <- lmer(angle ~ recipe + temp        + (1|recipe:replicate), cake, REML= FALSE))

## and now "choose" :
anova(fm3, fm2, fm1)
}
\keyword{datasets}

\docType{class}
\name{rePos-class}
\alias{rePos-class}
\title{Class \code{"rePos"}}
\description{
  A reference class for determining the positions in the
  random-effects vector that correspond to particular
  random-effects terms in the model formula
}
\section{Extends}{
  All reference classes extend and inherit methods from
  \code{"\linkS4class{envRefClass}"}.
}
\examples{
showClass("rePos")
}
\keyword{classes}

\name{lmResp-class}
\title{Reference Classes for Response Modules, \code{"(lm|glm|nls|lmer)Resp"}}
\alias{glmResp-class}
\alias{lmerResp-class}
\alias{lmResp-class}
\alias{nlsResp-class}
\description{
  Reference classes for response modules, including linear
  models, \code{"lmResp"}, generalized linear models,
  \code{"glmResp"}, nonlinear models, \code{"nlsResp"} and
  linear mixed-effects models, \code{"lmerResp"}. Each
  reference class is associated with a C++ class of the
  same name.  As is customary, the generator object for
  each class has the same name as the class.
}
\section{Extends}{
  All reference classes extend and inherit methods from
  \code{"\linkS4class{envRefClass}"}.  Furthermore,
  \code{"glmResp"}, \code{"nlsResp"} and \code{"lmerResp"}
  all extend the \code{"lmResp"} class.
}
\note{
  Objects from these reference classes correspond to
  objects in C++ classes.  Methods are invoked on the C++
  classes using the external pointer in the \code{ptr}
  field.  When saving such an object the external pointer
  is converted to a null pointer, which is why there are
  redundant fields containing enough information as R
  objects to be able to regenerate the C++ object.  The
  convention is that a field whose name begins with an
  upper-case letter is an R object and the corresponding
  field whose name begins with the lower-case letter is a
  method.  Access to the external pointer should be through
  the method, not through the field.
}
\examples{
showClass("lmResp")
str(lmResp$new(y=1:4))
showClass("glmResp")
str(glmResp$new(family=poisson(), y=1:4))
showClass("nlsResp")
showClass("lmerResp")
str(lmerResp$new(y=1:4))
}
\seealso{
  \code{\link{lmer}}, \code{\link{glmer}},
  \code{\link{nlmer}}, \code{\linkS4class{merMod}}.
}
\keyword{classes}

\name{devcomp}
\alias{devcomp}
\title{Extract the deviance component list}
\usage{
  devcomp(x)
}
\arguments{
  \item{x}{a fitted model of class \code{\linkS4class{merMod}}}
}
\value{
  a list with components
  \item{dims}{a named integer vector of various dimensions}
  \item{cmp}{a named numeric vector of components of the deviance}
}
\description{
  Return the deviance component list
}
\details{
  A fitted model of class \code{\linkS4class{merMod}} has a
  \code{devcomp} slot as described in the value section.
}
\note{
  This function is deprecated, use \code{\link{getME}(., "devcomp")}
  %%               ----------
}

\name{mkReTrms}
\alias{mkReTrms}
\alias{mkNewReTrms}
\title{Make Random Effect Terms: Create Z, Lambda, Lind, etc.}
\description{
  From the result of \code{\link{findbars}} applied to a model formula
  and the evaluation frame \code{fr}, create the model matrix
  \code{Zt}, etc, associated with the random-effects terms.
}
\usage{
  mkReTrms(bars, fr, drop.unused.levels=TRUE,
                     reorder.terms=TRUE,
                     reorder.vars=FALSE)
  mkNewReTrms(object, newdata, re.form=NULL,
                     na.action=na.pass,
                     allow.new.levels=FALSE,
                     sparse = max(lengths(orig.random.levs)) > 100)
}
\arguments{
  \item{bars}{a list of parsed random-effects terms}
  \item{fr}{a model frame in which to evaluate these terms}
  \item{drop.unused.levels}{(logical) drop unused factor levels?}
  \item{reorder.terms}{arrange random effects terms in decreasing order
    of number of groups (factor levels)?}
  \item{reorder.vars}{arrange columns of individual random effects terms
    in alphabetical order?}
  \item{object}{a fitted \code{merMod} object}
  \item{newdata}{data frame for which to create
    new RE terms object}
  \item{re.form}{(formula, \code{NULL}, or \code{NA}) specify which random effects to condition on when predicting.  If \code{NULL},
    include all random effects; if \code{NA} or \code{~0},
    include no random effects}
  \item{na.action}{function determining what should be done
    with missing values for fixed effects in \code{newdata}}
  \item{allow.new.levels}{(logical) if new levels (or NA values) in
    \code{newdata} are allowed. If FALSE (default), such new values in
    \code{newdata} will trigger an error; if TRUE, then the prediction
    will use the unconditional (population-level) values for data with
    previously unobserved levels (or NAs)}
  \item{sparse}{generate sparse contrast matrices?}
}
\value{
  a \code{\link{list}} with components
  \item{Zt}{transpose of the sparse model matrix for the random effects}
  \item{theta}{initial values of the covariance parameters}
  \item{Lind}{an integer vector of indices determining the mapping of
    the elements of the \code{theta} vector to the \code{"x"} slot of
    \code{Lambdat}}
  \item{Gp}{a vector indexing the association of
    elements of the conditional mode vector
    with random-effect terms; if \code{nb} is the vector of numbers
    of conditional modes per term (i.e. number of groups times number
    of effects per group), \code{Gp} is \code{c(0,cumsum(nb))}
    (and conversely \code{nb} is \code{diff(Gp)})}
  \item{lower}{lower bounds on the covariance parameters}
  \item{Lambdat}{transpose of the sparse relative covariance factor}
  \item{flist}{list of grouping factors used in the
    random-effects terms}
  \item{cnms}{a list of column names
    of the random effects according to the grouping factors}
  \item{Ztlist}{list of components of the transpose of the
    random-effects model matrix, separated by random-effects term}
  \item{nl}{names of the terms (in the same order as \code{Zt},
    i.e. reflecting the \code{reorder.terms} argument)}
}
\note{\code{mkNewReTrms} is used in the context of prediction, to
  generate a new "random effects terms" object from an already fitted
  model}
\seealso{
  Other utilities: \code{\link{findbars}},
  \code{\link{mkRespMod}}, \code{\link{nlformula}},
  \code{\link{nobars}}, \code{\link{subbars}}.
  \code{\link{getME}} can retrieve these components
  from a fitted model, although their values and/or forms
  may be slightly different in the final fitted model from
  their original values as returned from \code{mkReTrms}.
}
\examples{
data("Pixel", package="nlme")
mform <- pixel ~ day + I(day^2) + (day | Dog) + (1 | Side/Dog)
(bar.f <- findbars(mform)) # list with 3 terms
mf <- model.frame(subbars(mform),data=Pixel)
rt <- mkReTrms(bar.f,mf)
names(rt)
}
%fm1 <- lmer(mform,Pixel)
%rt2 <- getME(fm1,names(rt))
%for (i in seq_along(rt))
%    cat(names(rt)[[i]],
%        isTRUE(all.equal(rt[[i]],rt2[[i]])),
%        "\n")
% ## theta and Lambda components have the same structure, but have
% ## been updated in the fitting; flist is a data frame in reTrms
% ## but converted to a list; Ztlist is quite different (has been
% ## decomposed in getME()
\keyword{utilities}

\name{bootMer}
\alias{bootMer}
\title{Model-based (Semi-)Parametric Bootstrap for Mixed Models}
\usage{
bootMer(x, FUN, nsim = 1, seed = NULL, use.u = FALSE, re.form=NA,
	type = c("parametric", "semiparametric"),
	verbose = FALSE, .progress = "none", PBargs = list(),
	parallel = c("no", "multicore", "snow"),
	ncpus = getOption("boot.ncpus", 1L), cl = NULL)
}
\arguments{
  \item{x}{a fitted \code{merMod} object: see
    \code{\link{lmer}}, \code{\link{glmer}}, etc.}
  \item{FUN}{a function taking a fitted
    \code{merMod} object as input and returning the
    \emph{statistic} of interest, which must be a (possibly named)
    numeric vector.}
  \item{nsim}{number of simulations, positive integer; the
    bootstrap \eqn{B} (or \eqn{R}).}
  \item{seed}{optional argument to \code{\link{set.seed}}.}
  \item{use.u}{logical, indicating whether the spherical
    random effects should be simulated / bootstrapped as
    well.  If \code{TRUE}, they are not changed, and all
    inference is conditional on these values. If
    \code{FALSE}, new normal deviates are drawn (see
    Details).}
  \item{re.form}{formula, \code{NA} (equivalent to \code{use.u=FALSE}),
    or \code{NULL} (equivalent to \code{use.u=TRUE}):
    alternative to \code{use.u} for
    specifying which random effects to incorporate.
    See \code{\link{simulate.merMod}} for details.}
  \item{type}{character string specifying the type of
    bootstrap, \code{"parametric"} or
    \code{"semiparametric"}; partial matching is allowed.}
  \item{verbose}{logical indicating if progress should
    print output}
  \item{.progress}{character string - type of progress bar
    to display.  Default is \code{"none"}; the function will
    look for a relevant \code{*ProgressBar} function, so
    \code{"txt"} will work in general; \code{"tk"} is
    available if the \pkg{tcltk} package is loaded; or
    \code{"win"} on Windows systems. Progress bars are
    disabled (with a message) for parallel operation.}
  \item{PBargs}{a list of additional arguments to the
    progress bar function (the package authors like
    \code{list(style=3)}).}
  \item{parallel}{The type of parallel operation to be used (if any).
    If missing, the
    default is taken from the option \code{"boot.parallel"} (and if that
    is not set, \code{"no"}).}
  \item{ncpus}{integer: number of processes to be used in parallel operation:
    typically one would choose this to be the number of available CPUs.}
  \item{cl}{An optional \pkg{parallel} or \pkg{snow} cluster for use if
    \code{parallel = "snow"}.  If not supplied, a cluster on the
    local machine is created for the duration of the \code{boot} call.}
}
\value{
  an object of S3 \code{\link{class}} \code{"boot"},
  compatible with \CRANpkg{boot} package's
  \code{\link[boot]{boot}()} result. (See Details for information on how
  to retrieve information about errors during bootstrapping.)
}
\description{
  Perform model-based (Semi-)parametric bootstrap for mixed
  models.
}
\note{
  If you are using \code{parallel="snow"}, you will need to run
  \code{clusterEvalQ(cl,library("lme4"))} before calling
  \code{bootMer} to make sure that the
  \code{lme4} package is loaded on all of the workers; you may
  additionally need to use \code{\link[parallel]{clusterExport}}
  if you are using a summary function that calls any objects
  from the environment.
}
\details{
  The semi-parametric variant is only partially implemented, and
  we only provide a method for \code{\link{lmer}} and
  \code{\link{glmer}} results.
  
  Information about warning and error messages incurred during the
  bootstrap returns can be retrieved via the attributes
  \describe{
  \item{bootFail}{number of failures (errors)}
  \item{boot.fail.msgs}{error messages}
  \item{boot.all.msgs}{messages, warnings, and error messages}
}
  e.g. \code{attr("boot.fail.msgs")} to retrieve error messages

  The working name for bootMer() was
  \dQuote{simulestimate()}, as it is an extension of \code{simulate}
  (see \link{simulate.merMod}), but we want to emphasize its potential
  for valid inference.
  \itemize{
    \item If \code{use.u} is \code{FALSE} and \code{type} is
    \code{"parametric"}, each simulation generates new values of both
    the \dQuote{\emph{spherical}} random effects \eqn{u} and the
    i.i.d. errors \eqn{\epsilon}, using \code{\link{rnorm}()}
    with parameters corresponding to the fitted model \code{x}.

    \item If \code{use.u} is \code{TRUE} and \code{type=="parametric"},
    only the i.i.d. errors (or, for GLMMs, response values drawn from
    the appropriate distributions) are resampled, with the values of
    \eqn{u} staying fixed at their estimated values.

    \item If \code{use.u} is \code{TRUE} and \code{type=="semiparametric"},
    the i.i.d. errors are sampled from the distribution of (response)
    residuals.  (For GLMMs, the resulting
    sample will no longer have the same properties as the original
    sample, and the method may not make sense; a warning is generated.)
    The semiparametric bootstrap is currently an experimental feature,
    and therefore may not be stable.

    \item The case where \code{use.u} is \code{FALSE} and
    \code{type=="semiparametric"} is not implemented; Morris (2002)
    suggests that resampling from the estimated values of \eqn{u} is not
    good practice.
  } %% itemize
} %% details
\references{
  \itemize{
    \item \insertRef{davison1997bootstrap}{lme4}
    \item \insertRef{morris2002blups}{lme4}
  }
}
\seealso{
  \itemize{
    \item \code{\link{confint.merMod}},
    for a more specific approach to bootstrap confidence
    intervals on parameters.
    \item \code{\link{refit}()}, or \code{\link[pbkrtest]{PBmodcomp}()}
    from the \CRANpkg{pbkrtest} package, for parametric bootstrap comparison
    of models.
    \item \code{\link[boot]{boot}()}, and then
    \code{\link[boot]{boot.ci}}, from the \pkg{boot} package.
    \item \code{\link{profile-methods}}, for likelihood-based inference,
    including confidence intervals.
    \item \code{\link{pvalues}},
    for more general approaches to inference and p-value computation
    in mixed models.
  }
}
\examples{
if (interactive()) {
fm01ML <- lmer(Yield ~ 1|Batch, Dyestuff, REML = FALSE)
## see ?"profile-methods"
mySumm <- function(.) { s <- sigma(.)
    c(beta =getME(., "beta"), sigma = s, sig01 = unname(s * getME(., "theta"))) }
(t0 <- mySumm(fm01ML)) # just three parameters
## alternatively:
mySumm2 <- function(.) {
    c(beta=fixef(.),sigma=sigma(.), sig01=sqrt(unlist(VarCorr(.))))
}

set.seed(101)
## 3.8s (on a 5600 MIPS 64bit fast(year 2009) desktop "AMD Phenom(tm) II X4 925"):
system.time( boo01 <- bootMer(fm01ML, mySumm, nsim = 100) )

## to "look" at it
if (requireNamespace("boot")) {
    boo01
    ## note large estimated bias for sig01
    ## (~30\% low, decreases _slightly_ for nsim = 1000)

    ## extract the bootstrapped values as a data frame ...
    head(as.data.frame(boo01))

    ## ------ Bootstrap-based confidence intervals ------------

    ## warnings about "Some ... intervals may be unstable" go away
    ##   for larger bootstrap samples, e.g. nsim=500

    ## intercept
    (bCI.1 <- boot::boot.ci(boo01, index=1, type=c("norm", "basic", "perc")))# beta

    ## Residual standard deviation - original scale:
    (bCI.2  <- boot::boot.ci(boo01, index=2, type=c("norm", "basic", "perc")))
    ## Residual SD - transform to log scale:
    (bCI.2L <- boot::boot.ci(boo01, index=2, type=c("norm", "basic", "perc"),
                       h = log, hdot = function(.) 1/., hinv = exp))

    ## Among-batch variance:
    (bCI.3 <- boot::boot.ci(boo01, index=3, type=c("norm", "basic", "perc"))) # sig01

    
    confint(boo01)
    confint(boo01,type="norm")
    confint(boo01,type="basic")

    ## Graphical examination:
    plot(boo01,index=3)

    ## Check stored values from a longer (1000-replicate) run:
    (load(system.file("testdata","boo01L.RData", package="lme4")))# "boo01L"
    plot(boo01L, index=3)
    mean(boo01L$t[,"sig01"]==0) ## note point mass at zero!
} %% if boot package available
} %% interactive
}
\keyword{htest}
\keyword{models}

\name{sigma}
\title{Extract Residual Standard Deviation 'Sigma'}
\alias{sigma}
\alias{sigma.merMod}
\description{
  Extract the estimated standard deviation of the errors, the
  \dQuote{residual standard deviation} (also misnamed the
  \dQuote{residual standard error}), from a fitted model of class
  \code{\linkS4class{merMod}}.
}
\usage{
\S3method{sigma}{merMod}(object, \dots)
}
\arguments{
  \item{object}{a fitted model.}

  \item{\dots}{additional, optional arguments, passed from or to
    methods. (None currently in our two methods.)}
}
\value{
  The value differs based on the family (see details above for an explanation.)
  \itemize{
    \item For Gaussian fitted models, this is the residual 
    standard deviation.
    \item For Gamma and Inverse Gaussian fitted models, it represents 
    the square root of the inverse of the shape parameter.
    \item For Binomial and Poisson fitted models, the value is always reported 
    as \code{1}, since their variance is determined entirely by the mean and 
    there is no separate scale parameter.
    %\item Less interpretably, the square root of the residual deviance per 
    %degree of freedom in more general models.
  }
}
\details{
  In general the dispersion parameter (which we call \code{sigma}) is the 
  square root of the constant multiplier in the 'variance function' provided 
  by the \code{family} functions.
  \itemize{
    \item For Gaussian fits, \code{gaussian()$variance = rep.int(1, length(mu))} 
    and it's known that the variance is \eqn{\sigma^{2}}. Hence, the constant
    multiplier of the variance function in this case is \eqn{\sigma^{2}}, so 
    we say \code{sigma} is \eqn{\sqrt{\sigma}}.
    \item For Gamma fits, \code{Gamma()$variance = mu^2} where 
    \code{mu = scale*shape}. The known variance is 
    \eqn{\frac{\mu^{2}}{\code{shape}}}.
    Thus, the constant multiplier here is \eqn{\frac{1}{\sqrt{\code{shape}}}}.
    \item Similarly, for Inverse Gaussian fits, we have
    \code{inverse.gaussian()$variance = mu^{3}}, with known variance
    \eqn{\frac{\mu^{3}}{\lambda}}; similarly, the constant multiplier here is 
    \eqn{\frac{1}{\sqrt{\lambda}}}. \eqn{\lambda} is referred to as the shape
    parameter.
  }
}
\seealso{
  Package \CRANpkg{lme4} provides methods for mixed-effects models of class
  \code{\linkS4class{merMod}} and lists of linear models, 
  \code{\linkS4class{lmList4}}.
}
\examples{
methods(sigma)# from R 3.3.0 on, shows methods from pkgs 'stats' *and* 'lme4'
}

\name{troubleshooting}
\alias{troubleshooting}
\title{Troubleshooting}
\description{
  This page attempts to summarize some of the common problems with
  fitting \code{[gn]lmer} models and how to troubleshoot them.

  Most of the symptoms/diagnoses/workarounds listed below are due to
  various issues in the actual mixed model fitting process. You may
  run into problems due to multicollinearity or variables that are
  incorrectly typed (e.g. a variable is accidentally coded as character
  or factor rather than numeric). These problems can often be isolated
  by trying a \code{lm} or \code{glm} fit or attempting to construct the
  design matrix via \code{model.matrix()} (in each case with the random
  effects in your model excluded). If these tests fail then the problem
  is likely not specifically an \code{lme4} issue.
  
  \itemize{
    \item \code{failure to converge in (xxxx) evaluations} The
    optimizer hit its maximum limit of function evaluations.  To
    increase this, use the \code{optControl} argument of
    \code{[g]lmerControl} -- for \code{Nelder_Mead} and \code{bobyqa}
    the relevant parameter is \code{maxfun}; for \code{optim} and
    \code{optimx}-wrapped optimizers, including
    \code{nlminbwrap}, it's \code{maxit}; for
    \code{nloptwrap}, it's \code{maxeval}.
    \item \code{Model failed to converge with max|grad| ...} The scaled
    gradient at the fitted (RE)ML estimates
    is worryingly large. Try
    \itemize{
      \item refitting the parameters starting at the current
      estimates: getting consistent results (with no warning)
      suggests a false positive
      \item switching optimizers: getting consistent
      results suggests there is not really a problem; getting a similar
      log-likelihood with different parameter estimates suggests that
      the parameters are poorly determined (possibly the result of
      a misspecified or overfitted model)
      \item compute values of the deviance in the neighbourhood of the estimated
      parameters to double-check that \code{lme4} has really found a
      local optimum.
    }
    \item \code{Hessian is numerically singular: parameters are not
      uniquely determined}
    The Hessian (inverse curvature matrix) at the maximum likelihood or REML
    estimates has a very large eigenvalue, indicating that (within numerical
    tolerances) the surface is completely flat in some direction.
    The model may be misspecified, or extremely badly scaled (see
    \code{"Model is nearly unidentifiable"}).
    \item \code{Model is nearly unidentifiable ... Rescale variables?}
    The Hessian (inverse curvature matrix) at the maximum likelihood or REML
    estimates has a large eigenvalue, indicating that the surface is
    nearly flat in some direction. Consider centering and/or scaling
    continuous predictor variables.
    \item \code{Contrasts can be applied only to factors with 2 or more levels}
    One or more of the categorical predictors in the model has fewer than two
    levels. This may be due to user error when converting these predictors to 
    factors prior to modeling, or it may result from some factor levels being
    eliminated due to \code{NA}s in other predictors. Double-check the number
    of data points in each factor level to see which one is the culprit:
    \code{lapply(na.omit(df[,vars]), table)} (where \code{df} is the 
    \code{data.frame} and \code{vars} are the column names of your predictor
    variables).
  }
}

\name{culcitalvolume}
\alias{culcitalvolume}
\docType{data}
\title{
 Coral-eating seastar Culcita novaeguineae data (volume loss version)
}
\description{
  This is the same experiment outlined in \code{\link[lme4]{culcitalogreg}} 
  except this data set only contains values where predation occurred, and 
  the original volume as well as volume lost was recorded.
}
\usage{data("culcitalvolume")}
\format{
  A data frame with 50 observations on the following 7 variables.
  \describe{
    \item{\code{ttt}}{represents the combinations of different symbionts,
      which the treatment condition is explicitly written in plain language.}
    \item{\code{volume}}{describes the size of the coral, measured in 
      cm\eqn{^3}.}
    \item{\code{predvolume}}{the amount of volume loss from the coral, measured
      in cm\eqn{^3}.}
    \item{\code{crab}}{describes whether the crab was present in the 
      experiment. \code{n}: not present, \code{y}: present.}
    \item{\code{shrimp}}{describes whether the shrimp was present in the 
      experiment. \code{n}: not present, \code{y}: present.}
    \item{\code{block}}{a numeric variable indicating the experimental block. 
      There are \code{10} blocks in total, each corresponding to a large, 
      octagonal, flow-through seawater tank approximately 0.5m deep and 2m in 
      diameter.}
    \item{\code{ttt2}}{a relabelled version of \code{ttt}.
      \itemize{
          \item \code{1}: no exosymbionts,
          \item \code{2}: pair of \emph{Alpheus lottini} only 
            (Alpheus; shrimp),
          \item \code{3}: pair of \emph{Trapezia serenei} only 
            (Trapezia; crab),
          \item \code{4}: pair of \emph{Alpheus lottini} and pair of 
            \emph{Trapezia serenei} (‘Alpheus and Trapezia’).
        }
      }
  }
}
\seealso{
  The version which contains whether or not predation occurred, 
  \code{\link[lme4]{culcitalogreg}}.
}
\source{
  \insertCite{mckeon2012multiple}{lme4}
}
\references{
  \insertRef{mckeon2012multiple}{lme4}
}
\examples{
## Modifying to create a new response variable
vdata <- transform(culcitalvolume, 
                   propeaten = predvolume/volume,
                   tvol = log(predvolume))
## One-way analysis
(cvm1 <- lmer(tvol ~ ttt2 + (1|block), data = vdata))
(cvm2 <- lmer(propeaten ~ ttt2 + (1|block), data = vdata))
## Two-way analysis 
(cvm3 <- lmer(tvol ~ crab*shrimp + (1|block), data = vdata))
(cvm4 <- lmer(propeaten ~ crab*shrimp + (1|block), data = vdata))
}
\keyword{datasets}

\name{golden-class}
\docType{class}
\title{Class \code{"golden"} and Generator for Golden Search Optimizer Class}
\alias{golden-class}
\alias{golden}
\description{
  \code{"golden"} is a reference class for a golden search scalar optimizer,
  for a parameter within an interval.

  \code{golden()} is the generator for the \code{"golden"}
  class.  The optimizer uses reverse communications.
}
\usage{
golden(...)
}
\arguments{
  \item{\dots}{(partly optional) arguments passed to
    \code{\link{new}()} must be named arguments.  \code{lower} and
    \code{upper} are the bounds for the scalar parameter; they must be
    finite.}
}
\section{Extends}{
  All reference classes extend and inherit methods from
  \code{"\linkS4class{envRefClass}"}.
}
\examples{
showClass("golden")

golden(lower= -100, upper= 1e100)
}
\keyword{classes}

\docType{data}
\name{Pastes}
\alias{Pastes}
\title{Paste strength by batch and cask}
\format{A data frame with 60 observations on the following 4 variables.
  \describe{
    \item{\code{strength}}{paste strength.}
    \item{\code{batch}}{delivery batch from which the sample was
      sample.  A factor with 10 levels: \sQuote{A} to \sQuote{J}.}
    \item{\code{cask}}{cask within the delivery batch from which the
      sample was chosen.  A factor with 3 levels: \sQuote{a} to
      \sQuote{c}.}
    \item{\code{sample}}{the sample of paste whose strength was assayed,
      two assays per sample. A factor with 30 levels: \sQuote{A:a} to
      \sQuote{J:c}.}
  }}
\references{
  \insertRef{davies1972statistical_pt2}{lme4}
}
\description{
  Strength of a chemical paste product; its quality
  depending on the delivery batch, and the cask within the
  delivery.
}
\details{
  The data are described in Davies and Goldsmith (1972) as
  coming from \dQuote{ deliveries of a chemical paste
  product contained in casks where, in addition to sampling
  and testing errors, there are variations in quality
  between deliveries \dots{} As a routine, three casks
  selected at random from each delivery were sampled and
  the samples were kept for reference. \dots{} Ten of the
  delivery batches were sampled at random and two
  analytical tests carried out on each of the 30 samples}.
}
\examples{
str(Pastes)
require(lattice)
dotplot(cask ~ strength | reorder(batch, strength), Pastes,
        strip = FALSE, strip.left = TRUE, layout = c(1, 10),
        ylab = "Cask within batch",
        xlab = "Paste strength", jitter.y = TRUE)
## Modifying the factors to enhance the plot
Pastes <- within(Pastes, batch <- reorder(batch, strength))
Pastes <- within(Pastes, sample <- reorder(reorder(sample, strength),
          as.numeric(batch)))
dotplot(sample ~ strength | batch, Pastes,
        strip = FALSE, strip.left = TRUE, layout = c(1, 10),
        scales = list(y = list(relation = "free")),
        ylab = "Sample within batch",
        xlab = "Paste strength", jitter.y = TRUE)
## Four equivalent models differing only in specification
(fm1 <- lmer(strength ~ (1|batch) + (1|sample), Pastes))
(fm2 <- lmer(strength ~ (1|batch/cask), Pastes))
(fm3 <- lmer(strength ~ (1|batch) + (1|batch:cask), Pastes))
(fm4 <- lmer(strength ~ (1|batch/sample), Pastes))
## fm4 results in redundant labels on the sample:batch interaction
head(ranef(fm4)[[1]])
## compare to fm1
head(ranef(fm1)[[1]])
## This model is different and NOT appropriate for these data
(fm5 <- lmer(strength ~ (1|batch) + (1|cask), Pastes))

L <- getME(fm1, "L")
Matrix::image(L, sub = "Structure of random effects interaction in pastes model")
}
\keyword{datasets}

\name{findbars}
\alias{findbars}
\title{Determine random-effects expressions from a formula}
\usage{
  findbars(term)
}
\arguments{
  \item{term}{a mixed-model formula}
}
\description{
  From the right hand side of a formula for a mixed-effects
  model, determine the pairs of expressions that are
  separated by the vertical bar operator.  Also expand the
  slash operator in grouping factor expressions and expand
  terms with the double vertical bar operator into separate,
  independent random effect terms.
}
\value{
  pairs of expressions that were separated by vertical bars
}
\note{
  This function is called recursively on individual terms
  in the model, which is why the argument is called
  \code{term} and not a name like \code{form}, indicating a
  formula.
}
\seealso{
  \code{\link{formula}}, \code{\link{model.frame}},
  \code{\link{model.matrix}}.

  Other utilities: \code{\link{mkRespMod}},
  \code{\link{mkReTrms}}, \code{\link{nlformula}},
  \code{\link{nobars}}, \code{\link{subbars}}
}
\examples{
findbars(f1 <- Reaction ~ Days + (Days | Subject))
## => list( Days | Subject )
## These two are equivalent:% tests in ../inst/tests/test-doubleVertNotation.R
findbars(y ~ Days + (1 | Subject) + (0 + Days | Subject))
findbars(y ~ Days + (Days || Subject))
## => list of length 2:  list ( 1 | Subject ,  0 + Days | Subject)
findbars(~ 1 + (1 | batch / cask))
## => list of length 2:  list ( 1 | cask:batch ,  1 | batch)
\dontshow{
stopifnot(identical(findbars(f1),
                    list(quote(Days | Subject))))
}
}
\keyword{models}
\keyword{utilities}

\name{vcconv}
\alias{vcconv}
\alias{mlist2vec}
\alias{vec2mlist}
\alias{vec2STlist}
\alias{sdcor2cov}
\alias{cov2sdcor}
\alias{Vv_to_Cv}
\alias{Sv_to_Cv}
\alias{Cv_to_Vv}
\alias{Cv_to_Sv}
\title{Convert between representations of (co-)variance structures}
\description{
  Convert between representations of (co-)variance structures
  (EXPERIMENTAL).  See source code for details.
}
\usage{
  mlist2vec(L)
  vec2mlist(v, n = NULL, symm = TRUE)
  vec2STlist(v, n = NULL)
  sdcor2cov(m)
  cov2sdcor(V)
  Vv_to_Cv(v, n = NULL, s = 1)
  Sv_to_Cv(v, n = NULL, s = 1)
  Cv_to_Vv(v, n = NULL, s = 1)
  Cv_to_Sv(v, n = NULL, s = 1)
}
\arguments{
  \item{L}{List of symmetric, upper-triangular, or lower-triangular
    square matrices.}
  \item{v}{Concatenated vector containing the elements of the
    lower-triangle (including the diagonal) of a symmetric or triangular
    matrix.}
  \item{n}{Number of rows (and columns) of the resulting matrix.}
  \item{symm}{Return symmetric matrix if \code{TRUE} or lower-triangular
    if \code{FALSE}.}
  \item{m}{Standard deviation-correlation matrix.}
  \item{V}{Covariance matrix.}
  \item{s}{Scale parameter.}
}
\value{
  (Co-)variance structure
}
\details{
  \describe{
    \item{\code{mlist2vec}}{Convert list of matrices to concatenated
      vector of lower triangles with an attribute that gives the
      dimension of each matrix in the original list. This attribute may
      be used to reconstruct the matrices.  Returns a concatenation of
      the elements in one triangle of each matrix. An attribute
      \code{"clen"} gives the dimension of each matrix.}
    \item{\code{vec2mlist}}{Convert concatenated vector to list of matrices (lower
      triangle or symmetric). These matrices could represent Cholesky
      factors, covariance matrices, or correlation matrices (with
      standard deviations on the diagonal).}
    \item{\code{vec2STlist}}{Convert concatenated vector to list of ST
      matrices.}
    \item{\code{sdcor2cov}}{Standard deviation-correlation matrix to
      covariance matrix convert 'sdcor' format (std dev on diagonal,
      cor on off-diag) to and from variance-covariance matrix.}
    \item{\code{cov2sdcor}}{Covariance matrix to standard
      deviation-correlation matrix (i.e. standard deviations on the
      diagonal and correlations off the diagonal).}
    \item{\code{Vv_to_Cv}}{Variance-covariance to relative covariance
      factor. Returns a vector of elements from the lower triangle of a
      relative covariance factor.}
    \item{\code{Sv_to_Cv}}{Standard-deviation-correlation to relative covariance
      factor. Returns a vector of elements from the lower triangle of a
      relative covariance factor.}
    \item{\code{Cv_to_Vv}}{Relative covariance factor to
      variance-covariance. From unscaled Cholesky vector to (possibly
      scaled) variance-covariance vector. Returns a vector of elements
      from the lower triangle of a variance-covariance matrix.}
    \item{\code{Cv_to_Sv}}{Relative covariance factor to
      standard-deviation-correlation.  From unscaled Chol to sd-cor
      vector. Returns a vector of elements from the lower triangle of a
      standard-deviation-correlation matrix.}
  }
}
\examples{
vec2mlist(1:6)
mlist2vec(vec2mlist(1:6)) # approximate inverse
}

\name{profile-methods}
\title{Profile method for merMod objects}
\docType{methods}
\alias{as.data.frame.thpr}
\alias{log.thpr}
\alias{logProf}
\alias{varianceProf}
\alias{profile-methods}
\alias{profile.merMod}
\description{
  Methods for \code{\link{profile}}() of [ng]\code{\link{lmer}} fitted
  models.

  The \code{log()} method and the more flexible \code{logProf()}
  utility transform a \code{lmer} profile into one where logarithms of standard deviations
  are used, while \code{varianceProf} converts from the
  standard-deviation to the variance scale; see Details.
}
\usage{
\method{profile}{merMod}(fitted, which = NULL, alphamax = 0.01,
        maxpts = 100, delta = NULL,
        delta.cutoff = 1/8, verbose = 0, devtol = 1e-09,
        devmatchtol = 1e-5,
        maxmult = 10, startmethod = "prev", optimizer = NULL,
        control = NULL, signames = TRUE,
        parallel = c("no", "multicore", "snow"),
        ncpus = getOption("profile.ncpus", 1L), cl = NULL,
        prof.scale = c("sdcor","varcov"),
        \dots)
\method{as.data.frame}{thpr}(x, ...)
\method{log}{thpr}(x, base = exp(1))
logProf(x, base = exp(1), ranef = TRUE,
        sigIni = if(ranef) "sig" else "sigma")
varianceProf(x, ranef = TRUE)
}
\arguments{
  \item{fitted}{a fitted model, e.g., the result of \code{\link{lmer}(..)}.}
  \item{which}{NULL value,
    integer or character vector indicating which parameters
    to profile: default (NULL) is all parameters.  For integer, i.e., indexing,
    the parameters are ordered as follows:
    \describe{
      \item{(1)}{random effects (theta) parameters; these are ordered as
	in \code{getME(.,"theta")}, i.e., as the lower triangle of a
	matrix with standard deviations on the diagonal and correlations
	off the diagonal.}
      \item{(2)}{residual standard deviation (or scale parameter for GLMMs
        where appropriate).}
      \item{(3)}{fixed effect (beta) parameters.}
    }
    Alternatively, \code{which} may be a character, containing
    \code{"beta_"} or \code{"theta_"} denoting the fixed or random
    effects parameters, respectively, or also containing parameter
    names, such as \code{".sigma"} or \code{"(Intercept)"}.
  }
  \item{alphamax}{a number in \eqn{(0,1)}, such that \code{1 - alphamax}
    is the maximum alpha value for likelihood ratio confidence
    regions; used to establish the range of values to be profiled.}
  \item{maxpts}{maximum number of points (in each direction, for each
    parameter) to evaluate in attempting to construct the profile.}
  \item{delta}{stepping scale for deciding on next point to profile.
    The code uses the local derivative of the profile at the current
    step to establish a change in the focal parameter that will lead
    to a step of \code{delta} on the square-root-deviance scale.
    If \code{NULL}, the \code{delta.cutoff} parameter will be used
    to determine the stepping scale.}
  \item{delta.cutoff}{stepping scale (see \code{delta})
    expressed as a fraction of the
    target maximum value of the profile on the square-root-deviance
    scale.  Thus a \code{delta.cutoff} setting of \code{1/n} will
    lead to a profile with approximately \code{2*n} calculated points
    for each parameter (i.e., \code{n} points in each direction,
    below and above the estimate for each parameter).}
  \item{verbose}{level of output from internal calculations.}
  \item{devtol}{tolerance for fitted deviances less than
    baseline (supposedly minimum) deviance.}
  \item{devmatchtol}{tolerance for match between original deviance
    computation and value returned from auxiliary deviance function}
  \item{maxmult}{maximum multiplier of the original step size allowed,
    defaults to 10.}
  \item{startmethod}{method for picking starting conditions for
    optimization (STUB).}
  \item{optimizer}{(character or function) optimizer to use (see
    \code{\link{lmer}} for details); default is to use the optimizer
    from the original model fit.}
  \item{control}{a \code{\link{list}} of options controlling the
    profiling (see \code{\link{lmerControl}}): default is to use the
    control settings from the original model fit.}
  \item{signames}{logical indicating if abbreviated names of the form
    \code{.sigNN} should be used; otherwise, names are more meaningful
    (but longer) of the form \code{(sd|cor)_(effects)|(group)}.  Note
    that some code for profile transformations (e.g., \code{log()} or
    \code{\link{varianceProf}}) depends on \code{signames==TRUE}.}
  \item{\dots}{potential further arguments for various methods.}
  \item{x}{an object of class \code{thpr} (i.e., output of
    \code{profile})}
  \item{base}{the base of the logarithm.  Defaults to natural
    logarithms.}
  \item{ranef}{logical indicating if the sigmas of the random effects
    should be \code{log()} transformed as well.  If false, only
    \eqn{\sigma} (standard deviation of errors) is transformed.}
  \item{sigIni}{character string specifying the initial part of the
    sigma parameters to be log transformed.}
  \item{parallel}{The type of parallel operation to be used (if any).
    If missing, the
    default is taken from the option \code{"profile.parallel"} (and if that
    is not set, \code{"no"}).}
  \item{ncpus}{integer: number of processes to be used in parallel operation:
    typically one would choose this to be the number of available CPUs.}
  \item{cl}{An optional \pkg{parallel} or \pkg{snow} cluster for use if
    \code{parallel = "snow"}.  If not supplied, a cluster on the
    local machine is created for the duration of the \code{profile}
    call.}
  \item{prof.scale}{whether to profile on the standard
    deviation-correlation scale (\code{"sdcor"}) or on
    the variance-covariance scale (\code{"varcov"})
  }

}
\value{
  \code{profile(<merMod>)} returns an object of S3 class
  \code{"thpr"}, %% = th[eta] pr[ofile], now a misnomer, as we also profile beta's
  which is \code{\link{data.frame}}-like.
  Notable methods for such a profile object
  \code{\link{confint}()}, which returns the
  confidence intervals based on the profile,
  and three plotting methods
  (which require the \pkg{lattice} package),
  \code{\link[=xyplot.thpr]{xyplot}}, \code{densityplot}, and
  \code{splom}.

  In addition, the
  \code{\link{log}()} (see above) and \code{\link{as.data.frame}()}
  methods can transform \code{"thpr"} objects in useful ways.
}
\details{
  The \code{\link{log}} method and the more flexible \code{logProf()}
  function transform the profile into one where \eqn{\log(\sigma)} is
  used instead of \eqn{\sigma}.
  By default all sigmas including the standard deviations of the random
  effects are transformed i.e., the methods return a profile with all
  of the \code{.sigNN}
  parameters replaced by \code{.lsigNN}.  If \code{ranef} is false, only
  \code{".sigma"}, the standard deviation of the errors, is transformed
  (as it should never be zero, whereas random effect standard
  deviations (\code{.sigNN}) can be reasonably be zero).
  \cr
  The forward and backward splines for the log-transformed parameters
  are recalculated.
  Note that correlation parameters are not handled sensibly at present
  (i.e., they are logged rather than taking a more applicable
  transformation such as an arc-hyperbolic tangent,
  \code{atanh(x)}=\eqn{\log((1+x)/(1-x))/2}{log((1+x)/(1-x))/2}).

  The \code{varianceProf} function works similarly, including
  non-sensibility for correlation parameters, by squaring all
  parameter values, changing the names by appending \code{sq}
  appropriately (e.g. \code{.sigNN} to \code{.sigsqNN}).
  Setting \code{prof.scale="varcov"} in the original
  \code{profile()} call is a more computationally
  intensive, but more correct, way to compute confidence
  intervals for covariance parameters.

  Methods for function \code{\link{profile}} (package
  \pkg{stats}), here for profiling (fitted) mixed effect models.

%% FIXME: ../inst/doc/profiling.txt  contains  motivation and more by
%% Doug Bates. Should add here (partly), or "link to there".
}
\seealso{
  The plotting methods \code{\link[=xyplot.thpr]{xyplot}} etc, for class
  \code{"thpr"}.

  For (more expensive) alternative confidence intervals:
  \code{\link{bootMer}}.
}
\examples{
fm01ML <- lmer(Yield ~ 1|Batch, Dyestuff, REML = FALSE)
system.time(
  tpr  <- profile(fm01ML, optimizer="Nelder_Mead", which="beta_")
)## fast; as only *one* beta parameter is profiled over -> 0.09s (2022)

## full profiling (default which means 'all') needs longer:
system.time( tpr  <- profile(fm01ML, signames=FALSE))
## ~ 0.26s (2022) + possible warning about convergence
(confint(tpr) -> CIpr)
\donttest{# too much precision (etc). but just FYI:
 trgt <- array(c(12.19854, 38.22998, 1486.451,
                 84.06305, 67.6577,  1568.548), dim = 3:2)
 stopifnot(all.equal(trgt, unname(CIpr), tol = .0001)) # had 3.1e-7
}
if (interactive()) {
 library("lattice")
 xyplot(tpr)
 xyplot(tpr, absVal=TRUE) # easier to see conf.int.s (and check symmetry)
 xyplot(tpr, conf = c(0.95, 0.99), # (instead of all five 50, 80,...)
        main = "95\% and 99\% profile() intervals")
 xyplot(logProf(tpr, ranef=FALSE),
        main = expression("lmer profile()s"~~ log(sigma)*" (only log)"))
 densityplot(tpr, main="densityplot( profile(lmer(..)) )")
 densityplot(varianceProf(tpr), main=" varianceProf( profile(lmer(..)) )")
 splom(tpr)
 splom(logProf(tpr, ranef=FALSE))
 doMore <- lme4:::testLevel() > 2 %% even more --> ../tests/profile.R
 if(doMore) { ## not typically, for time constraint reasons
   ## Batch and residual variance only
   system.time(tpr2 <- profile(fm01ML, which=1:2)) # , optimizer="Nelder_Mead" gives warning
   print( xyplot(tpr2) )
   print( xyplot(log(tpr2)) )# log(sigma) is better
   print( xyplot(logProf(tpr2, ranef=FALSE)) )

   ## GLMM example
   gm1 <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
               data = cbpp, family = binomial)
   ## running ~ 10 seconds on a modern machine {-> "verbose" while you wait}:
   print( system.time(pr4 <- profile(gm1, verbose=TRUE)) )
   print( xyplot(pr4, layout=c(5,1), as.table=TRUE) )
   print( xyplot(log(pr4), absVal=TRUE) ) # log(sigma_1)
   print( splom(pr4) )
   print( system.time( # quicker: only sig01 and one fixed effect
       pr2 <- profile(gm1, which=c("theta_", "period2"))))
   print( confint(pr2) )
   ## delta..: higher underlying resolution, only for 'sigma_1':
   print( system.time(
       pr4.hr <- profile(gm1, which="theta_", delta.cutoff=1/16)))
   print( xyplot(pr4.hr) )
 }
} # only if interactive()
}
\keyword{methods}

\docType{data}
\name{sleepstudy}
\alias{sleepstudy}
\title{Reaction times in a sleep deprivation study}
\format{A data frame with 180 observations on the following 3 variables.
  \describe{
    \item{\code{Reaction}}{Average reaction time (ms)}
    \item{\code{Days}}{Number of days of sleep deprivation}
    \item{\code{Subject}}{Subject number on which the observation was made.}
  }}
\description{
  The average reaction time per day (in milliseconds) for subjects in a sleep
  deprivation study.

  Days 0-1 were adaptation and training (T1/T2), day 2 was baseline (B);
  sleep deprivation started after day 2.

}
\details{
  These data are from the study described in Belenky et al.
  (2003), for the most sleep-deprived group (3 hours time-in-bed)
  and for the first 10 days of the study, up to the recovery period.
  The original study analyzed speed (1/(reaction time)) and treated
  day as a categorical rather than a continuous predictor.
}
\examples{
str(sleepstudy)
require(lattice)
xyplot(Reaction ~ Days | Subject, sleepstudy, type = c("g","p","r"),
       index = function(x,y) coef(lm(y ~ x))[1],
       xlab = "Days of sleep deprivation",
       ylab = "Average reaction time (ms)", aspect = "xy")
(fm1 <- lmer(Reaction ~ Days + (Days|Subject), sleepstudy, subset=Days>=2))
## independent model
(fm2 <- lmer(Reaction ~ Days + (1|Subject) + (0+Days|Subject), sleepstudy, subset=Days>=2))
}
\references{
  \insertRef{belenky2003patterns}{lme4}
}
\keyword{datasets}

\name{predict.merMod}
\alias{predict.merMod}
\title{Predictions from a model at new data values}
\description{
  The \code{\link{predict}} method for \code{\linkS4class{merMod}}
  objects, i.e. results of \code{\link{lmer}()}, \code{\link{glmer}()}, etc.
}
\usage{
\method{predict}{merMod}(object, newdata = NULL, newparams = NULL,
    re.form = NULL,
    random.only=FALSE, terms = NULL,
    type = c("link", "response"), allow.new.levels = FALSE,
    na.action = na.pass,
    se.fit = FALSE,
    \dots)
}
\arguments{
  \item{object}{a fitted model object}
  \item{newdata}{data frame for which to evaluate
    predictions.}
  \item{newparams}{new parameters to use in evaluating predictions,
    specified as in the \code{start} parameter for \code{\link{lmer}}
    or \code{\link{glmer}} -- a list with components \code{theta} and/or
    (for GLMMs) \code{beta}.}
  \item{re.form}{(formula, \code{NULL}, or \code{NA}) specify which random effects to condition on when predicting.  If \code{NULL},
    include all random effects; if \code{NA} or \code{~0},
    include no random effects.}
  \item{random.only}{(logical) ignore fixed effects, making predictions
    only using random effects?}
  \item{terms}{a \code{\link{terms}} object - unused at present.}
  \item{type}{character string - either \code{"link"}, the default, or
    \code{"response"} indicating the type of prediction object returned.}
  \item{allow.new.levels}{logical if new levels (or NA values) in
    \code{newdata} are allowed. If FALSE (default), such new values in
    \code{newdata} will trigger an error; if TRUE, then the prediction
    will use the unconditional (population-level) values for data with
    previously unobserved levels (or NAs).}
  \item{na.action}{\code{\link{function}} determining what should be done
    with missing values for fixed effects in \code{newdata}.
    The default is to predict \code{NA}: see \code{\link{na.pass}}.}
  \item{se.fit}{(Experimental) A logical value indicating whether the standard errors should be included or not. Default is FALSE.}
  \item{...}{optional additional parameters.  None are used at present.}
}
\value{
  a numeric vector of predicted values
}
\details{
  \itemize{
    \item If any random effects are included in \code{re.form}
    (i.e. it is not \code{~0} or \code{NA}),
    \code{newdata} \emph{must} contain columns
    corresponding to all of the grouping variables and
    random effects used in the original model, even if not all
    are used in prediction; however, they can be safely set to \code{NA}
    in this case.
    \item There is no option for computing standard errors of
    predictions because it is difficult to define an
    efficient method that incorporates uncertainty in the
    variance parameters; we recommend \code{\link{bootMer}}
    for this task.
  }
}
\examples{
(gm1 <- glmer(cbind(incidence, size - incidence) ~ period + (1 |herd), cbpp, binomial))
str(p0 <- predict(gm1))            # fitted values
str(p1 <- predict(gm1,re.form=NA))  # fitted values, unconditional (level-0)
newdata <- with(cbpp, expand.grid(period=unique(period), herd=unique(herd)))
str(p2 <- predict(gm1,newdata))    # new data, all RE
str(p3 <- predict(gm1,newdata,re.form=NA)) # new data, level-0
str(p4 <- predict(gm1,newdata,re.form= ~(1|herd))) # explicitly specify RE
stopifnot(identical(p2, p4))
\dontshow{

## predict() should work with variable names with spaces [as lm() does]:
dd <- expand.grid(y=1:3, "Animal ID" = 1:9)
fm <- lmer(y ~ 1 + (1 | `Animal ID`),  dd)
summary(fm)
isel <- c(7, 9, 11, 13:17, 20:22)
stopifnot(all.equal(vcov(fm)[1,1], 0.02564102564),
	  all.equal(unname(predict(fm, newdata = dd[isel,])),
		    unname( fitted(fm) [isel])))
} % dontshow
} % examples

\name{lmList}
\alias{lmList}
\alias{plot.lmList}
\title{Fit List of lm or glm Objects with a Common Model}
\description{
  Fit a list of \code{\link{lm}} or \code{\link{glm}} objects with a
  common model for different subgroups of the data.
}
\usage{
lmList(formula, data, family, subset, weights, na.action,
       offset, pool = !isGLM || .hasScale(family2char(family)),
       warn = TRUE, \dots)
}
\arguments{
  \item{formula}{a linear \code{\link{formula}} object of the form
    \code{y ~ x1+...+xn | g}.  In the formula object, \code{y}
    represents the response, \code{x1,...,xn} the covariates,
    and \code{g} the grouping factor specifying the
    partitioning of the data according to which different
    \code{lm} fits should be performed.}
  \item{family}{an optional \code{\link{family}} specification for a
    generalized linear model (\code{\link{glm}}).}
  \item{data}{an optional data frame containing the
    variables named in \code{formula}.  By default the
    variables are taken from the environment from which
    \code{lmer} is called.  See Details.}
  \item{subset}{an optional expression indicating the
    subset of the rows of \code{data} that should be used in
    the fit.  This can be a logical vector, or a numeric
    vector indicating which observation numbers are to be
    included, or a character vector of the row names to be
    included.  All observations are included by default.}
  \item{weights}{an optional vector of \sQuote{prior
      weights} to be used in the fitting process.  Should be
    \code{NULL} or a numeric vector.}
  \item{na.action}{a function that indicates what should
    happen when the data contain \code{NA}s.  The default
    action (\code{na.omit}, inherited from the \sQuote{factory
    fresh} value of \code{getOption("na.action")}) strips any
    observations with any missing values in any variables.}
  \item{offset}{this can be used to specify an \emph{a
      priori} known component to be included in the linear
    predictor during fitting.  This should be \code{NULL} or a
    numeric vector of length equal to the number of cases.
    One or more \code{\link{offset}} terms can be included in
    the formula instead or as well, and if more than one is
    specified their sum is used.  See
    \code{\link{model.offset}}.}
  \item{pool}{logical scalar indicating if the variance estimate should
    pool the residual sums of squares.  By default true if the model has
    a scale parameter (which includes all linear, \code{\link{lmer}()}, ones).}
  \item{warn}{indicating if errors in the single fits should signal a
    \dQuote{summary} \code{\link{warning}}.}
  \item{\dots}{additional, optional arguments to be passed to
    the model function or family evaluation.}
}
\value{an object of \code{\link{class}} \code{\linkS4class{lmList4}} (see
  there, notably for the \code{\link{methods}} defined).
}
\details{
  \itemize{
    \item While \code{data} is optional, the package authors
    \emph{strongly} recommend its use, especially when later applying
    methods such as \code{update} and \code{drop1} to the fitted model
    (\emph{such methods are not guaranteed to work properly if
      \code{data} is omitted}).  If \code{data} is omitted, variables will
    be taken from the environment of \code{formula} (if specified as a
    formula) or from the parent frame (if specified as a character vector).

    \item Since \pkg{lme4} version 1.1-16, if there are errors (see
    \code{\link{stop}}) in the single (\code{lm()} or \code{glm()})
    fits, they are summarized to a warning message which is returned as
    attribute \code{"warnMessage"} and signalled as \code{\link{warning}()}
    when the \code{warn} argument is true.

    In previous \pkg{lme4} versions, a general (different) warning had
    been signalled in this case.
  }
}
\examples{
fm.plm  <- lmList(Reaction ~ Days | Subject, sleepstudy)
coef(fm.plm)
fm.2  <- update(fm.plm, pool = FALSE)
## coefficients are the same, "pooled or unpooled":
stopifnot( all.equal(coef(fm.2), coef(fm.plm)) )

(ci <- confint(fm.plm)) # print and rather *see* :
plot(ci)                # how widely they vary for the individuals
}
\seealso{\code{\linkS4class{lmList4}}}
\keyword{models}

\docType{data}
\name{InstEval}
\alias{InstEval}
\title{University Lecture/Instructor Evaluations by Students at ETH}
\format{A data frame with 73421 observations on the following 7 variables.
  \describe{
    \item{\code{s}}{a factor with levels \code{1:2972} denoting
      individual students.}
    \item{\code{d}}{a factor with 1128 levels from \code{1:2160}, denoting
      individual professors or lecturers.}% ("d": \dQuote{Dozierende} in German)
    \item{\code{studage}}{an ordered factor with levels \code{2} <
      \code{4} < \code{6} < \code{8}, denoting student's \dQuote{age}
      measured in the \emph{semester} number the student has been enrolled.}
    \item{\code{lectage}}{an ordered factor with 6 levels, \code{1} <
      \code{2} < ... < \code{6}, measuring how many semesters back the
      lecture rated had taken place.}
    \item{\code{service}}{a binary factor with levels \code{0} and
      \code{1}; a lecture is a \dQuote{service}, if held for a
      different department than the lecturer's main one.}
    \item{\code{dept}}{a factor with 14 levels from \code{1:15}, using a
      random code for the department of the lecture.}

    \item{\code{y}}{a numeric vector of \emph{ratings} of lectures by
      the students, using the discrete scale \code{1:5}, with meanings
      of \sQuote{poor} to \sQuote{very good}.}
  }
  Each observation is one student's rating for a specific lecture
  (of one lecturer, during one semester in the past).}
\description{
  University lecture evaluations by students at ETH Zurich,
  anonymized for privacy protection.  This is an
  interesting \dQuote{medium} sized example of a
  \emph{partially} nested mixed effect model.
}
\details{
  The main goal of the survey is to find \dQuote{the best
  liked prof}, according to the lectures given.
  Statistical analysis of such data has been the basis for
  a (student) jury selecting the final winners.

  The present data set has been anonymized and slightly
  simplified on purpose.
}
\examples{%% lots in ../tests/vcov-etc.R
str(InstEval)

head(InstEval, 16)
xtabs(~ service + dept, InstEval)
}
\keyword{datasets}

\name{grouseticks}
\alias{grouseticks}
\alias{grouseticks_agg}
\docType{data}
\title{
  Data on red grouse ticks from Elston et al. 2001
}
\description{
  Number of ticks on the heads of red grouse chicks
  sampled in the field (\code{grouseticks})
  and an aggregated version (\code{grouseticks_agg}); see original source for more details
}
\usage{data(grouseticks)}
\format{
  \describe{
    \item{\code{INDEX}}{(factor) chick number (observation level)}
    \item{\code{TICKS}}{number of ticks sampled}
    \item{\code{BROOD}}{(factor) brood number}
    \item{\code{HEIGHT}}{height above sea level (meters)}
    \item{\code{YEAR}}{year (-1900)}
    \item{\code{LOCATION}}{(factor) geographic location code}
    \item{\code{cHEIGHT}}{centered height, derived from \code{HEIGHT}}
    \item{\code{meanTICKS}}{mean number of ticks by brood}
    \item{\code{varTICKS}}{variance of number of ticks by brood}
  }
}
\source{
  Robert Moss, via David Elston
}
\details{\code{grouseticks_agg} is just a brood-level
  aggregation of the data}
\references{
  \insertRef{elston2001analysis}{lme4}
}
\examples{
if (interactive()) {
data(grouseticks)
## Figure 1a from Elston et al
par(las=1,bty="l")
tvec <- c(0,1,2,5,20,40,80)
pvec <- c(4,1,3)
with(grouseticks_agg,plot(1+meanTICKS~HEIGHT,
                  pch=pvec[factor(YEAR)],
                  log="y",axes=FALSE,
                  xlab="Altitude (m)",
                  ylab="Brood mean ticks"))
axis(side=1)
axis(side=2,at=tvec+1,label=tvec)
box()
abline(v=405,lty=2)
## Figure 1b
with(grouseticks_agg,plot(varTICKS~meanTICKS,
                  pch=4,
                  xlab="Brood mean ticks",
                  ylab="Within-brood variance"))
curve(1*x,from=0,to=70,add=TRUE)
## Model fitting
form <- TICKS~YEAR+HEIGHT+(1|BROOD)+(1|INDEX)+(1|LOCATION)
(full_mod1  <- glmer(form, family="poisson",data=grouseticks))
}
}
\keyword{datasets}

\name{pvalues}
\alias{mcmcsamp}
\alias{pvalues}
\title{Getting p-values for fitted models}
\description{
  One of the most frequently asked questions about
  \code{lme4} is "how do I calculate p-values for estimated
  parameters?" Previous versions of \code{lme4} provided
  the \code{mcmcsamp} function, which efficiently generated
  a Markov chain Monte Carlo sample from the posterior
  distribution of the parameters, assuming flat (scaled
  likelihood) priors. Due to difficulty in constructing a
  version of \code{mcmcsamp} that was reliable even in
  cases where the estimated random effect variances were
  near zero (e.g.
  \url{https://stat.ethz.ch/pipermail/r-sig-mixed-models/2009q4/003115.html}),
  \code{mcmcsamp} has been withdrawn (or more precisely,
  not updated to work with \code{lme4} versions >=1.0.0).

  Many users, including users of the \code{aovlmer.fnc} function from
  the \code{languageR} package which relies on \code{mcmcsamp}, will be
  deeply disappointed by this lacuna. Users who need p-values have a
  variety of options. In the list below, the methods marked \code{MC}
  provide explicit model comparisons; \code{CI} denotes confidence
  intervals; and \code{P} denotes parameter-level or sequential tests of
  all effects in a model. The starred (*) suggestions provide
  finite-size corrections (important when the number of groups is <50);
  those marked (+) support GLMMs as well as LMMs.

  \itemize{
    \item likelihood ratio tests via \code{anova} or \code{drop1} (MC,+) 
    \item{profile confidence intervals via \code{\link{profile.merMod}} and
      \code{\link{confint.merMod}} (CI,+)
    }
    \item{parametric bootstrap confidence intervals and model comparisons via
      \code{\link{bootMer}} (or \code{PBmodcomp} in the
      \code{pbkrtest} package) (MC/CI,*,+)
    }
    \item{for random effects, simulation tests via the \code{RLRsim} package
      (MC,*)
    }
    \item{for fixed effects, F tests via Kenward-Roger
      approximation using \code{KRmodcomp} from the
      \code{pbkrtest} package (MC,*)
    }
    \item{\code{car::Anova} and
      \code{lmerTest::anova} provide wrappers for
      Kenward-Roger-corrected tests using \code{pbkrtest}:
      \code{lmerTest::anova} also provides t tests via the
      Satterthwaite approximation (P,*)
    }
    \item{\code{afex::mixed} is another wrapper for
      \code{pbkrtest} and \code{anova} providing
      "Type 3" tests of all effects (P,*,+)
    }
  }

  \code{arm::sim}, or \code{\link{bootMer}}, can be used
  to compute confidence intervals on predictions.

  For \code{glmer} models, the \code{summary} output provides p-values
  based on asymptotic Wald tests (P); while this is standard practice
  for generalized linear models, these tests make assumptions both about
  the shape of the log-likelihood surface and about the accuracy of
  a chi-squared approximation to differences in log-likelihoods.
  
  When all else fails, don't forget to keep p-values in
  perspective:
  \url{https://phdcomics.com/comics/archive.php?comicid=905}
}

\name{isNested}
\alias{isNested}
\title{Is f1 nested within f2?}
\usage{
  isNested(f1, f2)
}
\arguments{
  \item{f1}{factor 1}

  \item{f2}{factor 2}
}
\value{
  TRUE if factor 1 is nested within factor 2
}
\description{
  Does every level of f1 occur in conjunction with exactly
  one level of f2? The function is based on converting a
  triplet sparse matrix to a compressed column-oriented
  form in which the nesting can be quickly evaluated.
}
\examples{
with(Pastes, isNested(cask, batch))   ## => FALSE
with(Pastes, isNested(sample, batch))  ## => TRUE
}

\name{lme4-package}
\alias{lme4}
\alias{lme4-package}
\docType{package}
\title{Linear, generalized linear, and nonlinear mixed models}
\description{
  \code{lme4} provides functions for fitting and analyzing
  mixed models: linear (\code{\link{lmer}}), generalized linear
  (\code{\link{glmer}}) and nonlinear (\code{\link{nlmer}}.)
}
\section{Differences between \pkg{nlme} and \pkg{lme4}}{
  \pkg{lme4} covers approximately the same ground as the earlier
  \pkg{nlme} package.  The most important differences are:
  \itemize{
    \item \pkg{lme4} uses modern, efficient linear algebra methods
    as implemented in the \code{Eigen} package, and uses reference
    classes to avoid undue copying of large objects; it is therefore likely
    to be faster and more memory-efficient than \pkg{nlme}.
    \item \pkg{lme4} includes generalized linear mixed model (GLMM)
    capabilities, via the \code{\link{glmer}} function.
    \item \pkg{lme4} does \emph{not} currently implement \pkg{nlme}'s
    features for modeling heteroscedasticity and
    correlation of residuals.
    \item \pkg{lme4} does not currently offer the same flexibility
    as \pkg{nlme} for composing complex variance-covariance structures,
    but it does implement crossed random effects in a way that
    is both easier for the user and much faster.
    \item \pkg{lme4} offers built-in facilities for likelihood
    profiling and parametric bootstrapping.
    \item \pkg{lme4} is designed to be more modular than \pkg{nlme},
    making it easier for downstream package developers and end-users
    to re-use its components for extensions of the basic mixed model
    framework. It also allows more flexibility for specifying
    different functions for optimizing over the random-effects
    variance-covariance parameters.
    \item \pkg{lme4} is not (yet) as well-documented as \pkg{nlme}.
  }
}
\section{Differences between current (1.0.+) and previous versions of
  \pkg{lme4}}{
  \itemize{
  \item \code{[gn]lmer} now produces objects of class \code{\linkS4class{merMod}}
  rather than class \code{mer} as before
  \item the new version uses a combination of S3 and reference classes
  (see \code{\link{ReferenceClasses}}, \code{\link{merPredD-class}}, and
  \code{\link{lmResp-class}}) as well as S4 classes; partly for this reason
  it is more interoperable with \pkg{nlme}
  \item The internal structure of [gn]lmer is now more modular, allowing
  finer control of the different steps of argument checking; construction
  of design matrices and data structures; parameter estimation; and construction
  of the final \code{merMod} object (see \code{\link{modular}})
  \item profiling and parametric bootstrapping are new in
  the current version
  \item the new version of \pkg{lme4} does \emph{not} provide
  an \code{mcmcsamp} (post-hoc MCMC sampling) method, because this
  was deemed to be unreliable.  Alternatives for computing p-values
  include parametric bootstrapping (\code{\link{bootMer}}) or methods
  implemented in the \pkg{pbkrtest} package and leveraged by the
  \pkg{lmerTest} package and the \code{Anova} function in the \pkg{car} package
  (see \code{\link{pvalues}} for more details).
}
}
\section{Caveats and trouble-shooting}{
\itemize{
    \item Some users who have previously installed versions of the
    RcppEigen and minqa packages may encounter segmentation faults (!!);
    the solution is to make sure to re-install these packages before
    installing \pkg{lme4}.  (Because the problem is not with the
    explicit \emph{version} of the packages, but with running
    packages that were built with different versions of \pkg{Rcpp}
    in conjunction with each other, simply making sure you have
    the latest version, or using \code{update.packages}, will
    not necessarily solve the problem; you must actually re-install
    the packages. The problem is most likely with \pkg{minqa}.)
  }

}

\docType{class}
\name{lmList4-class}
\alias{lmList4-class}
\alias{show,lmList4-method}
\title{Class "lmList4" of 'lm' Objects on Common Model}
\description{
  Class \code{"lmList4"} is an S4 class with basically a
  \code{\link{list}} of objects of class \code{\link{lm}} with a common
  model (but different data); see \code{\link{lmList}()} which returns
  these.

  Package \pkg{nlme}'s \code{\link[nlme]{lmList}()} returns objects of
  S3 class \code{"lmList"} and provides methods for them, on which our
  methods partly build.
}
\section{Objects from the Class}{
  Objects can be created by calls of the form
  \code{new("lmList4", ...)} or, more commonly, by a call to
  \code{\link{lmList}()}.
}
\section{Methods}{
  A dozen \code{\link{methods}} are provided.  Currently, S4
  methods for \code{\link{show}}, coercion (\code{\link{as}(.,.)}) and
  others inherited via \code{"list"},  and S3 methods for
  \code{\link{coef}}, \code{\link{confint}}, \code{\link{fitted}},
  \code{\link{fixef}}, \code{\link{formula}}, \code{\link{logLik}},
  \code{\link{pairs}}, \code{\link{plot}}, \code{\link{predict}},
  \code{\link{print}}, \code{\link{qqnorm}}, \code{\link{ranef}},
  \code{\link{residuals}}, \code{\link{sigma}}, \code{\link{summary}},
  and \code{\link{update}}.

  \describe{
    \item{sigma(object)}{returns the standard deviation
      \eqn{\hat{\sigma}}{\sigma-hat} (of the errors in the linear models), assuming a
      \emph{common} variance \eqn{\sigma^2} by pooling (even when
      \code{pool = FALSE} was used in the fit).}
  }
}
\seealso{\code{\link{lmList}}}
\examples{
if(getRversion() >= "3.2.0") {
  (mm <- methods(class = "lmList4"))
  ## The S3 ("not S4") ones :
  mm[!attr(mm,"info")[,"isS4"]]
}
## For more examples:  example(lmList)  i.e., ?lmList
}
\keyword{classes}

\name{salamander}
\alias{salamander}
\docType{data}
\title{
  Mountain dusky salamander mating
}
\description{
  S. Arnold and P. Verrell conducted an experiment at the University of 
  Chicago to study breeding behaviours of mountain dusky salamanders. This
  mating data was used as an example in the "Generalized Linear Models" 
  textbook \insertCite{mccullagh1989generalized}{lme4}.
}
\usage{data("salamander")}
\format{
  A data frame with 360 observations on the following 8 variables.
  \describe{
    \item{\code{Season}}{represents the season \code{Fall} \code{Summer} of 
      1986.}
    \item{\code{Experiment}}{experiment number \code{1, 2, 3}.}
    \item{\code{TypeM}}{representing the two types of male salamanders being 
      studied. \code{R} stands for rough butt, and \code{W} stands for white 
      side.}
    \item{\code{TypeF}}{similar to the above except for female salamanders.}
    \item{\code{Cross}}{represents the cross between a female and male type. 
      \itemize{
        \item{\code{RR} = rough butt female crossed with rough butt male},
        \item{\code{RW} = rough butt female crossed with white side male},
        \item{\code{WR} = white side female crossed with rough butt male},
        \item{\code{WW} = white side female crossed with white side male}.
      }
    }
    \item{\code{Male}}{identication number of the male salamander.}
    \item{\code{Female}}{identification number of the male salamander.}
    \item{\code{Mate}}{represents whether mating has occurred. \code{1} = yes,
      \code{0} = no.}
  }
}
\details{
  Every variable in this data set is either binary or a factor.  However, 
  most of the variables in this data set are treated like a numerical variable
  (i.e., \code{Experiment, Male, Female, Mate}), so we suggest turning them
  into factor variables before use. As outlined in McCullagh and Nelder (1989): 
  This experiment, conducted by S. Arnold and P. Verrell at the University of 
  Chicago, investigated whether geographically isolated populations of mountain 
  dusky salamanders would interbreed. The goal was to see the difference in 
  mating frequencies between the \code{RW} crosses compared to the \code{WR}
  crosses. Because each salamander was involved in multiple mating trials with 
  different partners, the observations are not independent. Therefore, we use a 
  model (see the \emph{examples} section) that conditions on the specific male 
  and female salamanders in the experiment.
}
\source{
  \insertCite{mccullagh1989generalized}{lme4}
}
\references{
  \insertRef{mccullagh1989generalized}{lme4}
}
\examples{
## Making sure Male, Female, and Cross are treated as factors
salamander$Male <- factor(salamander$Male)
salamander$Female <- factor(salamander$Female)
salamander$Cross <- factor(salamander$Cross)
## Fitting the model described in 14.5.3 from McCullagh and Nelder
sal_mod <- glmer(Mate ~ (1|Female) + (1 | Male) + Cross, data = salamander,
                 family = binomial(link = "logit"))
}
\keyword{datasets}

\name{nobars}
\alias{nobars}
\title{Omit terms separated by vertical bars in a formula}
\usage{
  nobars(term)
}
\arguments{
  \item{term}{the right-hand side of a mixed-model formula}
}
\value{
  the fixed-effects part of the formula
}
\description{
  Remove the random-effects terms from a mixed-effects
  formula, thereby producing the fixed-effects formula.
}
\note{
  This function is called recursively on individual terms
  in the model, which is why the argument is called
  \code{term} and not a name like \code{form}, indicating a
  formula.
}
\examples{
nobars(Reaction ~ Days + (Days|Subject)) ## => Reaction ~ Days
}
\seealso{
  \code{\link{formula}}, \code{\link{model.frame}},
  \code{\link{model.matrix}}.

  Other utilities: \code{\link{findbars}},
  \code{\link{mkRespMod}}, \code{\link{mkReTrms}},
  \code{\link{nlformula}}, \code{\link{subbars}}
}
\keyword{models}
\keyword{utilities}

\name{influence.merMod}
\title{Influence Diagnostics for Mixed-Effects Models}
\alias{influence.merMod}
\alias{dfbeta.influence.merMod}
\alias{dfbetas.influence.merMod}
\alias{cooks.distance.influence.merMod}
\alias{cooks.distance.merMod}
\description{
  These functions compute deletion influence diagnostics for linear
  (fit by \code{\link{lmer}}) and generalized linear mixed-effects models
  (fit by \code{\link[lme4]{glmer}}).  The main functions are methods for
  the \code{\link{influence}} generic function.  Other functions are
  provided for computing \code{\link{dfbeta}}, \code{\link{dfbetas}},
  \code{\link{cooks.distance}}, and influence on variance-covariance
  components based on the objects computed by \code{influence.merMod}
}

\usage{
\method{influence}{merMod}(model, groups, data, maxfun = 1000,
          do.coef = TRUE, start = NULL,
          parallel = c("no", "multicore", "snow"),
          ncpus = getOption("influence.ncpus", 1L), cl = NULL, ncores, ...)
\method{cooks.distance}{influence.merMod}(model, ...)
\method{dfbeta}{influence.merMod}(model, which = c("fixed", "var.cov"), ...)
\method{dfbetas}{influence.merMod}(model, ...)
}

\arguments{
  \item{model}{in the case of \code{influence.merMod}, a model of class \code{"merMod"};
    in the case of \code{cooks.distance}, \code{dfbeta}, or \code{dfbetas},
    an object returned by \code{influence.merMod}}
  \item{groups}{a character vector containing the name of a grouping factor or names of grouping factors; if more than one name
    is supplied, then groups are defined by all combinations of levels of the grouping factors that appear in
    the data. If omitted, then each individual row of the data matrix is treated as a "group" to be deleted in turn.}
  \item{data}{an optional data frame with the data to which \code{model}
    was fit; \code{influence.merMod} can usually retrieve the data used to
    fit the model, unless it can't be found in the current environment, so it's usually unnecessary to supply this argument.}
  \item{maxfun}{The maximum number of function evaluations (for \code{influence.merMod})
    to perform after deleting each group; the defaults are large enough so that the iterations will typically continue to convergence.
    Setting to \code{maxfun=20} for an \code{lmer} model or \code{100} for a \code{glmer} model will typically produce a faster reasonable approximation.
    An even smaller value can be used if interest is only in influence on the fixed effects.}
  \item{which}{if \code{"fixed.effects"} (the default), return influence
    on the fixed effects; if \code{"var.cov"}, return influence on the
    variance-covariance components.}
  \item{do.coef}{if \code{FALSE}, skip potentially time-consuming
    computations, returning just a list containing hat values.}
  \item{start}{starting value for new fits (set to optimal values from
    original fit by default)}
  \item{parallel}{The type of parallel operation to be used (if any).
    If \code{"multicore"} (and \R is running on a Unix-alike), then
    \code{mclapply} is used; if \code{"snow"}, then \code{parLapply} is
    used.}
  \item{ncpus}{number of processes to be used in parallel operation.}
  \item{cl}{An optional \pkg{parallel} or \pkg{snow} cluster for use if
    \code{parallel = "snow"}.  If not supplied, a cluster on the
    local machine is created for the duration of the call.}
  \item{ncores}{deprecated: equivalent to \code{parallel = "snow"} and
    \code{ncpus = ncores}; please use \code{parallel}, \code{ncpus}, and
    \code{cl} instead.}
  \item{\dots}{ignored.}
}

\details{
\code{influence.merMod} start with the estimated variance-covariance components from \code{model} and then refit
the model omitting each group in turn, not necessarily iterating to completion. For example, \code{maxfun=20} takes up to 20 function evaluations
step away from the ML or REML solution for the full data, which usually provides decent approximations to the fully iterated estimates.

The other functions are methods for the \code{\link{dfbeta}}, \code{\link{dfbetas}}, and \code{\link{cooks.distance}} generics, to be applied to the
\code{"influence.merMod"} object produced by the \code{influence} function; the \code{dfbeta} methods can also return
influence on the variance-covariance components.
}

\value{\code{influence.merMod} returns objects of class
  \code{"influence.merMod"}, which contain the following elements:
\describe{
    \item{\code{"fixed.effects"}}{the estimated fixed effects for the model.}
    \item{\code{"fixed.effects[-groups]"}}{a matrix with columns corresponding to the fixed-effects coefficients and rows corresponding to groups, giving the
    estimated fixed effects with each group deleted in turn; \emph{groups} is formed from the name(s) of the grouping factor(s).}
    \item{\code{"var.cov.comps"}}{the estimated variance-covariance parameters for the model.}
    \item{\code{"var.cov.comps[-groups]"}}{a matrix with the estimated covariance parameters (in columns) with each group deleted in turn.}
    \item{\code{"vcov"}}{The estimated covariance matrix of the fixed-effects coefficients.}
    \item{\code{"vcov[-groups]"}}{a list each of whose elements is the estimated covariance matrix of the fixed-effects coefficients with one group deleted.}
    \item{\code{"groups"}}{a character vector giving the names of the grouping factors.}
    \item{\code{"deleted"}}{the possibly composite grouping factor, each of whose elements is deleted in turn.}
    \item{\code{"converged"}}{for \code{influence.merMod}, a logical vector indicating whether the computation converged for each group.}
    \item{\code{"function.evals"}}{for \code{influence.merMod}, a vector of the number of function evaluations performed for each group.}
}
For plotting \code{"influence.merMod"} objects, see \code{\link[car]{infIndexPlot}}.
}
\references{
  \insertRef{fox2019r}{lme4}
}

\author{
J. Fox; parallel extensions by H. Bengtsson
}

\seealso{
\code{\link[car]{infIndexPlot}}, \code{\link{influence.measures}}
}

\examples{
if (interactive()) {
  fm1 <- lmer(Reaction ~ Days + (Days | Subject), sleepstudy)
  inf_fm1 <- influence(fm1, "Subject")
  if (require("car")) {
    infIndexPlot(inf_fm1)
  }
  dfbeta(inf_fm1)
  dfbetas(inf_fm1)
  gm1 <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
               data = cbpp, family = binomial)
  inf_gm1 <- influence(gm1, "herd", maxfun=100)
  gm1.11 <- update(gm1, subset = herd != 11) # check deleting herd 11
  if (require("car")) {
    infIndexPlot(inf_gm1)
    compareCoefs(gm1, gm1.11)
  }
  if(packageVersion("car") >= "3.0.10") {
    dfbeta(inf_gm1)
    dfbetas(inf_gm1)
  }
 } %% interactive()
} %% examples
\keyword{models}

\name{vcov.merMod}
\title{Covariance matrix of estimated parameters}
\description{
  Compute the variance-covariance matrix of estimated paramers.
  Optionally also computes correlations, or the full (joint)
  covariance matrix of the fixed-effect coefficients and the conditional
  modes of the random effects.}
\alias{vcov.merMod}
\alias{vcov.summary.merMod}
\usage{
\S3method{vcov}{merMod}(object, correlation = TRUE, sigm = sigma(object),
    use.hessian = NULL, full = FALSE, noScale = NULL, \dots)
}
\arguments{
  \item{object}{an \R object of class \code{\linkS4class{merMod}}, i.e.,
    as resulting from \code{\link{lmer}()}, or \code{\link{glmer}()},
    etc.}
  \item{correlation}{(logical)
    indicates whether the correlation matrix as well as
    the variance-covariance matrix is desired
  }
  \item{sigm}{the residual standard error; by default \code{\link{sigma}(object)}.}
  \item{use.hessian}{(logical) indicates whether to use the
    finite-difference Hessian of the deviance function to compute
    standard errors of the fixed effects. See \emph{Details}.
  }
  \item{full}{return the 'full' covariance matrix, i.e. the joint
    covariance matrix of the conditional distribution of conditional
    modes (as in \code{getME(., "b")}) and fixed-effect
    parameters. (\code{correlation} and \code{use.hessian} are
    \emph{ignored} in this case.)

    Note that this option may be slow for models with large numbers of
  random-effect levels!
  }
  \item{noScale}{(logical) some users may use autoscaling. If so, the default
  returns the scaled parameters. If \code{noScale = TRUE}, then \code{vcov} 
  returns the non-scaled parameters instead.}
  \item{\dots}{extra arguments for method compatibility (ignored)}
}
\details{
  When \code{use.hessian = FALSE}, the code estimates the
  covariance matrix based on internal information about the inverse of
  the model matrix (see \code{\link{getME}(.,"RX")}). This is exact for
  linear mixed models, but approximate for GLMMs.
  The default is to use the Hessian whenever the
  fixed effect parameters are arguments to the deviance
  function (i.e. for GLMMs with \code{nAGQ>0}), and to use
  \code{\link{getME}(.,"RX")} whenever the fixed effect parameters are
  profiled out (i.e. for GLMMs with \code{nAGQ==0} or LMMs).

  \code{use.hessian=FALSE} is backward-compatible with older versions
  of \code{lme4}, but may give less accurate SE estimates when the
  estimates of the fixed-effect (see \code{\link{getME}(.,"beta")})
  and random-effect (see \code{\link{getME}(.,"theta")}) parameters
  are correlated.

  However, \code{use.hessian=TRUE} is not always more accurate: for some
  numerically unstable fits, the approximation using \code{RX} is
  actually more reliable (because the Hessian has to be computed by
  a finite difference approximation, which is also error-prone): see
  e.g. \url{https://github.com/lme4/lme4/issues/720}

}
    
\examples{
fm1 <- lmer(Reaction ~ Days + (Days | Subject), sleepstudy)
gm1 <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
             data = cbpp, family = binomial)
(v1 <- vcov(fm1))
v2 <- vcov(fm1, correlation = TRUE)
# extract the hidden 'correlation' entry in @factors
as(v2, "corMatrix")
v3 <- vcov(gm1)
v3X <- vcov(gm1, use.hessian  = FALSE)
all.equal(v3, v3X)
## full correlatiom matrix
cv <- vcov(fm1, full = TRUE)
image(cv, xlab = "", ylab = "",
      scales = list(y = list(labels = rownames(cv)),
                    at = seq(nrow(cv)),
                    x = list(labels = NULL)))
}
\value{a covariance matrix (sparse when \code{full=TRUE})}
\keyword{models}

\name{namedList}
\alias{namedList}
\title{Self-naming list function}
\description{this function takes a list of arguments and combines them
  into a list; any \emph{unnamed} arguments are automatically named
  to match their symbols.  The \code{tibble::lst()} function offers
  similarly functionality.
}
\usage{
namedList(...)
}
\arguments{
  \item{...}{comma-separated arguments}
}
\examples{
a <- 1
b <- 2
c <- 3
str(namedList(a, b, d = c))
}

\name{toenail}
\alias{toenail}
\docType{data}
\title{
  Toenail onychomycosis data from dermatophyte infections
}
\description{
  Results of a clinical trial \insertCite{de1998twelve}{lme4} to compare 
  two oral antifungal treatments: itraconazole and terbinafine to treat
  toenail onychomycosis causes by dermatophytes.
}
\usage{data("toenail")}
\format{
  A data frame with 1908 observations on the following 8 variables.
  \describe{
    \item{\code{patientID}}{a unique identifier for each patient in the trial.}
    \item{\code{outcome}}{a factor indicating the severity of the patient's 
      onycholysis (separation of the nail plate from the nail bed), with levels 
      \code{none or mild} and \code{moderate or severe}.}
    \item{\code{treatment}}{a factor indicating the treatment used with levels
      \code{itraconazole} and \code{terbinafine}.}
    \item{\code{time}}{exact timing of visit in months.}
    \item{\code{visit}}{An integer represent the sequential visit number for
      each subject.}
  }
}
\details{
  The data in \CRANpkg{lme4} was copied from the toenail data found in the 
  \CRANpkg{HSAUR3} package. From the dataset documentation from the 
  \CRANpkg{HSAUR3} package: There is a clinical trial 
  \insertCite{de1998twelve}{lme4} to compare two competing oral antifungal 
  treatments for toenail infection (dermatophyte onychomycosis). 
  A total of 378 patients were randomly allocated 
  into two treatment groups, one group receiving 250mg per day of terbinafine 
  and the other group 200mg per day of itraconazole. Patients were evaluated at 
  seven visits, intended to be at weeks 0, 4, 8, 12, 24, 36, and 48 for the 
  degree of separation of the nail plate from the nail bed (onycholysis) 
  dichotomized into \code{moderate or severe} and \code{none or mild}. But 
  patients did not always arrive exactly at the scheduled time and the exact 
  time in months that they did attend was recorded. The data is not balanced 
  since not all patients attended for all seven planned visits.
}
\source{
  \insertCite{de1998twelve}{lme4}
}
\references{
  \insertRef{de1998twelve}{lme4}
}
\examples{
toemod <- glmer(outcome ~ time*treatment + (1 | patientID), 
                data = toenail, family = binomial(link = "logit"))
}
\keyword{datasets}

\docType{data}
\name{Dyestuff}
\alias{Dyestuff}
\alias{Dyestuff2}
\title{Yield of dyestuff by batch}
\format{Data frames, each with 30 observations on the following 2 variables.
  \describe{
    \item{\code{Batch}}{a factor indicating the batch of the
      intermediate product from which the preparation was created.}
    \item{\code{Yield}}{the yield of dyestuff from the preparation
      (grams of standard color).}
  }}
\references{
  \itemize{
    \item \insertRef{davies1972statistical_pt3}{lme4}
    \item \insertRef{box1973bayesian}{lme4}
  }
}
\description{
  The \code{Dyestuff} data frame provides the yield of
  dyestuff (Naphthalene Black 12B) from 5 different
  preparations from each of 6 different batchs of an
  intermediate product (H-acid).  The \code{Dyestuff2} data
  were generated data in the same structure but with a
  large residual variance relative to the batch variance.
}
\details{
  The \code{Dyestuff} data are described in Davies and
  Goldsmith (1972) as coming from \dQuote{an investigation
  to find out how much the variation from batch to batch in
  the quality of an intermediate product (H-acid)
  contributes to the variation in the yield of the dyestuff
  (Naphthalene Black 12B) made from it.  In the experiment
  six samples of the intermediate, representing different
  batches of works manufacture, were obtained, and five
  preparations of the dyestuff were made in the laboratory
  from each sample. The equivalent yield of each
  preparation as grams of standard colour was determined by
  dye-trial.}

  The \code{Dyestuff2} data are described in Box and Tiao
  (1973) as illustrating \dQuote{ the case where
  between-batches mean square is less than the
  within-batches mean square.  These data had to be
  constructed for although examples of this sort
  undoubtably occur in practice, they seem to be rarely
  published.}
}
\examples{
\dontshow{ # useful for the lme4-authors --- development, debugging, etc:
 commandArgs()[-1]
 if(FALSE) ## R environment variables:
 local({ ne <- names(e <- Sys.getenv())
         list(R    = e[grep("^R", ne)],
              "_R" = e[grep("^_R",ne)]) })
 Sys.getenv("R_ENVIRON")
 Sys.getenv("R_PROFILE")
 cat("R_LIBS:\\n"); (RL <- strsplit(Sys.getenv("R_LIBS"), ":")[[1]])
 nRL <- normalizePath(RL)
 cat("and extra(:= not in R_LIBS) .libPaths():\\n")
 .libPaths()[is.na(match(.libPaths(), nRL))]

 structure(Sys.info()[c(4,5,1:3)], class="simple.list") #-> 'nodename' ..
 sessionInfo()
 searchpaths()
 pkgI <- function(pkgname) {
   pd <- tryCatch(packageDescription(pkgname),
                  error=function(e)e, warning=function(w)w)
   if(inherits(pd, "error") || inherits(pd, "warning"))
     cat(sprintf("packageDescription(\\"\%s\\") \%s: \%s\\n",
                 pkgname, class(pd)[2], pd$message))
   else
     cat(sprintf("\%s -- built: \%s\\n\%*s -- dir  : \%s\\n",
                 pkgname, pd$Built, nchar(pkgname), "",
                 dirname(dirname(attr(pd, "file")))))
 }
 pkgI("Matrix")
 pkgI("Rcpp")
 ## 2012-03-12{MM}: fails with --as-cran
 pkgI("RcppEigen")
 pkgI("minqa")
 pkgI("lme4")
}
require(lattice)
str(Dyestuff)
dotplot(reorder(Batch, Yield) ~ Yield, Dyestuff,
        ylab = "Batch", jitter.y = TRUE, aspect = 0.3,
        type = c("p", "a"))
dotplot(reorder(Batch, Yield) ~ Yield, Dyestuff2,
        ylab = "Batch", jitter.y = TRUE, aspect = 0.3,
        type = c("p", "a"))
(fm1 <- lmer(Yield ~ 1|Batch, Dyestuff))
(fm2 <- lmer(Yield ~ 1|Batch, Dyestuff2))
}
\keyword{datasets}

\name{mkNewReTrms}
\alias{mkNewReTrms}
\title{Make new random effect terms for prediction}
\description{
  Rebuilds the random effect terms
}
\usage{
  mkNewReTrms(object, newdata, re.form=NULL,
                     na.action=na.pass,
                     allow.new.levels=FALSE,
                     sparse = max(lengths(orig.random.levs)) > 100)
}
\arguments{
  \item{object}{a fitted \code{merMod} object}
  \item{newdata}{data frame for which to create
    new RE terms object}
  \item{re.form}{(formula, \code{NULL}, or \code{NA}) specify which random effects to condition on when predicting.  If \code{NULL},
    include all random effects; if \code{NA} or \code{~0},
    include no random effects}
  \item{na.action}{function determining what should be done
    with missing values for fixed effects in \code{newdata}}
  \item{allow.new.levels}{(logical) if new levels (or NA values) in
    \code{newdata} are allowed. If FALSE (default), such new values in
    \code{newdata} will trigger an error; if TRUE, then the prediction
    will use the unconditional (population-level) values for data with
    previously unobserved levels (or NAs)}
  \item{sparse}{generate sparse contrast matrices?}
}
\value{
  a \code{\link{list}} with components
  \item{Zt}{transpose of the sparse model matrix for the random effects}
  \item{theta}{initial values of the covariance parameters}
  \item{Lind}{an integer vector of indices determining the mapping of
    the elements of the \code{theta} vector to the \code{"x"} slot of
    \code{Lambdat}}
  \item{Gp}{a vector indexing the association of
    elements of the conditional mode vector
    with random-effect terms; if \code{nb} is the vector of numbers
    of conditional modes per term (i.e. number of groups times number
    of effects per group), \code{Gp} is \code{c(0,cumsum(nb))}
    (and conversely \code{nb} is \code{diff(Gp)})}
  \item{lower}{lower bounds on the covariance parameters}
  \item{Lambdat}{transpose of the sparse relative covariance factor}
  \item{flist}{list of grouping factors used in the
    random-effects terms}
  \item{cnms}{a list of column names
    of the random effects according to the grouping factors}
  \item{Ztlist}{list of components of the transpose of the
    random-effects model matrix, separated by random-effects term}
  \item{nl}{names of the terms (in the same order as \code{Zt},
    i.e. reflecting the \code{reorder.terms} argument)}
}
\note{\code{mkNewReTrms} is used in the context of prediction, to
  generate a new "random effects terms" object from an already fitted
  model}
\seealso{
  Other utilities: \code{\link[reformulas]{findbars}},
  \code{\link{mkRespMod}}, \code{\link{nlformula}},
  \code{\link[reformulas]{nobars}}, \code{\link[reformulas]{subbars}}.
  \code{\link{getME}} can retrieve these components
  from a fitted model, although their values and/or forms
  may be slightly different in the final fitted model from
  their original values as returned from \code{\link[reformulas]{mkReTrms}}.
}
\examples{
fm1 <- lmer(Reaction ~ Days + (Days | Subject), sleepstudy)
nd <- data.frame(Days = 5, Subject = "new")
try(mkNewReTrms(fm1, newdata = nd))
t1 <- mkNewReTrms(fm1, newdata = nd, allow.new.levels = TRUE)
}
\keyword{utilities}

\name{lmerControl}
\title{Control of Mixed Model Fitting}
\alias{glmerControl}
\alias{lmerControl}
\alias{nlmerControl}
\alias{.makeCC}
\description{
  Construct control structures for mixed model fitting.  All arguments
  have defaults, and can be grouped into
  \itemize{
    \item general control parameters, most importantly \code{optimizer},
    further \code{restart_edge}, etc;

    \item model- or data-checking specifications, in short
    \dQuote{checking options}, such as \code{check.nobs.vs.rankZ}, or
    \code{check.rankX} (currently not for \code{nlmerControl});

    \item all the parameters to be passed to the optimizer, e.g.,
    maximal number of iterations, passed via the \code{optCtrl} list
    argument.
  }
}
\usage{
lmerControl(optimizer = "nloptwrap",% was "Nelder_Mead" till Dec.2013,
				    % then "bobyqa" till Oct 2018; CRAN till 1.1-20 (2019-02-04)
    restart_edge = TRUE,
    boundary.tol = 1e-5,
    calc.derivs = NULL,
    use.last.params = FALSE,
    sparseX = FALSE,
    standardize.X = FALSE,
    autoscale = NULL,
    ## input checking options
    check.nobs.vs.rankZ = "ignore",
    check.nobs.vs.nlev = "stop",
    check.nlev.gtreq.5 = "ignore",
    check.nlev.gtr.1 = "stop",
    check.nobs.vs.nRE= "stop",
    check.rankX = c("message+drop.cols", "silent.drop.cols", "warn+drop.cols",
                    "stop.deficient", "ignore"),
    check.scaleX = c("warning","stop","silent.rescale",
                     "message+rescale","warn+rescale","ignore"),
    check.formula.LHS = "stop",
    ## convergence checking options
    check.conv.nobsmax  = 1e4,
    check.conv.nparmax  = 10,
    check.conv.grad     = .makeCC("warning", tol = 2e-3, relTol = NULL),
    check.conv.singular = .makeCC(action = "message", tol = getSingTol()),
    check.conv.hess     = .makeCC(action = "warning", tol = 1e-6),
    ## optimizer args
    optCtrl = list(),
    mod.type = "lmer"
)

glmerControl(optimizer = c("bobyqa", "Nelder_Mead"),
    restart_edge = FALSE,
    boundary.tol = 1e-5,
    calc.derivs = NULL,
    use.last.params = FALSE,
    sparseX = FALSE,
    standardize.X = FALSE,
    autoscale = NULL,
    ## input checking options
    check.nobs.vs.rankZ = "ignore",
    check.nobs.vs.nlev = "stop",
    check.nlev.gtreq.5 = "ignore",
    check.nlev.gtr.1 = "stop",
    check.nobs.vs.nRE= "stop",
    check.rankX = c("message+drop.cols", "silent.drop.cols", "warn+drop.cols",
                    "stop.deficient", "ignore"),
    check.scaleX = c("warning","stop","silent.rescale",
                     "message+rescale","warn+rescale","ignore"),
    check.formula.LHS = "stop",
    ## convergence checking options
    check.conv.nobsmax  = 1e4, 
    check.conv.nparmax  = 20,
    check.conv.grad     = .makeCC("warning", tol = 2e-3, relTol = NULL),
    check.conv.singular = .makeCC(action = "message", tol = getSingTol()),
    check.conv.hess     = .makeCC(action = "warning", tol = 1e-6),
    ## optimizer args
    optCtrl = list(),
    mod.type = "glmer",
    tolPwrss = 1e-7,
    compDev = TRUE,
    nAGQ0initStep = TRUE,
    check.response.not.const = "stop"
 )

nlmerControl(optimizer = "Nelder_Mead", tolPwrss = 1e-10,
             optCtrl = list())

.makeCC(action, tol, relTol, \dots)
}
\arguments{
  \item{optimizer}{character - name of optimizing function(s).  A
    \code{\link{character}} vector or list of functions: length 1 for
    \code{lmer} or \code{glmer}, possibly length 2 for \code{glmer}).
    Built-in optimizers are \code{"\link{Nelder_Mead}"},
    \code{"\link[minqa]{bobyqa}"} (from the \CRANpkg{minqa} package),
    \code{"\link{nlminbwrap}"} (using base \R's \code{\link{nlminb}}) and the
    default for \code{lmerControl()}, \code{"\link{nloptwrap}"}.  Any
    minimizing function that allows box constraints can be used provided
    that it
    \describe{
      \item{(1)}{takes input parameters \code{fn} (function to be
        optimized), \code{par} (starting parameter values), \code{lower}
        and \code{upper} (parameter bounds)
	and \code{control} (control parameters, passed
        through from the \code{control} argument) and}
      \item{(2)}{returns a list with (at least) elements \code{par}
        (best-fit parameters), \code{fval} (best-fit function value),
        \code{conv} (convergence code, equal to zero for
	successful convergence) and (optionally) \code{message}
        (informational message, or explanation of convergence failure).}
    }
    Special provisions are made for \code{\link[minqa]{bobyqa}},
    \code{\link{Nelder_Mead}}, and optimizers wrapped in the
    \CRANpkg{optimx} package; to use the \pkg{optimx} optimizers (including
    \code{L-BFGS-B} from base \code{\link{optim}} and
    \code{\link{nlminb}}), pass the \code{method} argument to
    \code{optim} in the \code{optCtrl} argument (you may
    need to load the \pkg{optimx} package manually using
    \code{\link{library}(optimx)}).

    For \code{glmer}, if \code{length(optimizer)==2}, the first element
    will be used for the preliminary (random effects parameters only)
    optimization, while the second will be used for the final (random
    effects plus fixed effect parameters) phase.  See
    \code{\link{modular}} for more information on these two phases.

    If \code{optimizer} is \code{NULL} (at present for \code{lmer} only),
    all of the model structures will be set up, but no optimization will
    be done (e.g. parameters will all be returned as \code{NA}).
  }
  \item{calc.derivs}{(logical) compute gradient and Hessian of nonlinear
    optimization solution? Default NULL value says these derivatives
    should be calculated if and only if
    derivative-based convergence checks
    are going to be done (i.e. when \code{nobs(model) < check.conv.nobsmax} and
    \code{ndim(model) < check.conv.nparmax} where \code{ndim} refers to the
    dimensions of the variance-covariance matrix)}
  \item{use.last.params}{(logical) should the last value of the
    parameters evaluated (\code{TRUE}), rather than the value of the
    parameters corresponding to the minimum deviance, be returned?
    This is a "backward bug-compatibility" option; use \code{TRUE}
    only when trying to match previous results.}
  \item{sparseX}{(logical) should a sparse model matrix be
    used for the fixed-effects terms?
    Currently inactive.}
  \item{restart_edge}{(logical) should the optimizer
    attempt a restart when it finds a solution at the
    boundary (i.e. zero random-effect variances or perfect
    +/-1 correlations)?  (Currently only implemented for
    \code{lmerControl}.)}
  \item{boundary.tol}{numeric - within what distance of
    a boundary should the boundary be checked for a better fit?
    (Set to zero to disable boundary checking.)}
  \item{tolPwrss}{numeric scalar - the tolerance for declaring
    convergence in the penalized iteratively weighted residual
    sum-of-squares step.}
  \item{compDev}{logical scalar - should compiled code be
    used for the deviance evaluation during the optimization
    of the parameter estimates?}
  \item{nAGQ0initStep}{Run an initial optimization phase with
    \code{nAGQ = 0}. While the initial optimization usually
    provides a good starting point for subsequent fitting
    (thus increasing overall computational speed),
    setting this option to \code{FALSE} can be useful in cases
    where the initial phase results in bad fixed-effect estimates
    (seen most often in binomial models with \code{link="cloglog"}
    and offsets).}
  \item{check.nlev.gtreq.5}{character - rules for
    checking whether all random effects have >= 5 levels.
    See \code{action}.}
  \item{check.nlev.gtr.1}{character - rules for checking
    whether all random effects have > 1 level. See \code{action}.}
  \item{check.nobs.vs.rankZ}{character - rules for
    checking whether the number of observations is greater
    than (or greater than or equal to) the rank of the random
    effects design matrix (Z), usually necessary for
    identifiable variances.  As for \code{action}, with
    the addition of \code{"warningSmall"} and \code{"stopSmall"}, which run
    the test only if the dimensions of \code{Z} are < 1e6.
    \code{nobs > rank(Z)} will be tested for LMMs and GLMMs with
    estimated scale parameters; \code{nobs >= rank(Z)} will be tested
    for GLMMs with fixed scale parameter.
    The rank test is done using the
    \code{method="qr"} option of the \code{\link[Matrix]{rankMatrix}}
    function.
  }
  \item{check.nobs.vs.nlev}{ character - rules for checking whether the
    number of observations is less than (or less than or equal to) the
    number of levels of every grouping factor, usually necessary for
    identifiable variances.  As for \code{action}.
    \code{nobs<nlevels} will be tested for LMMs and GLMMs with estimated
    scale parameters; \code{nobs<=nlevels} will be tested for GLMMs with
    fixed scale parameter.}
  \item{check.nobs.vs.nRE}{character - rules for
    checking whether the number of observations is greater
    than (or greater than or equal to) the number of random-effects
    levels for each term, usually necessary for identifiable variances.
    As for \code{check.nobs.vs.nlev}.}
  \item{check.conv.nobsmax}{threshold for the number of observations 
    above which derivative-based convergence checks are skipped.
    (Set this argument to \code{Inf} to  restore previous behaviour
    of always checking.)}
  \item{check.conv.nparmax}{threshold for the number of dimensions of the
    variance/covariance matrix above which derivative-based convergence checks 
    are skipped. (Set this argument to \code{Inf} to  restore previous behaviour
    of always checking.)}
  \item{check.conv.grad}{rules for checking the gradient of the deviance
    function for convergence.  A list as returned
    by \code{.makeCC}, or a character string with only the action.}
  \item{check.conv.singular}{rules for checking for a singular fit,
    i.e. one where some parameters are on the boundary of the feasible
    space (for example, random effects variances equal to 0 or
    correlations between random effects equal to +/- 1.0);
    as for \code{check.conv.grad} above.  The default is to use
    \code{\link{isSingular}(.., tol = *)}'s default.}
  \item{check.conv.hess}{rules for checking the Hessian of the deviance
    function for convergence.; as for \code{check.conv.grad}
    above.}
  \item{check.rankX}{character - specifying if \code{\link[Matrix]{rankMatrix}(X)}
    should be compared with \code{ncol(X)} and if columns from the design
    matrix should possibly be dropped to ensure that it has full rank.
    Sometimes needed to make the model identifiable.  The options can be
    abbreviated; the three \code{"*.drop.cols"} options all do drop
    columns, \code{"stop.deficient"} gives an error when the rank is
    smaller than the number of columns where \code{"ignore"} does no
    rank computation, and will typically lead to less easily
    understandable errors, later.}
  \item{check.scaleX}{character - check for problematic scaling of
    columns of fixed-effect model matrix, e.g. parameters measured on
    very different scales.}
  \item{check.formula.LHS}{ check whether specified formula has
    a left-hand side. Primarily for internal use within
    \code{simulate.merMod};
    \emph{use at your own risk} as it may allow the generation
    of unstable \code{merMod} objects}
  \item{check.response.not.const}{character - check that the
    response is not constant.}

  \item{optCtrl}{a \code{\link{list}} of additional arguments to be
    passed to the nonlinear optimizer (see \code{\link{Nelder_Mead}},
    \code{\link[minqa]{bobyqa}}).  In particular, both
    \code{Nelder_Mead} and \code{bobyqa} use \code{maxfun} to
    specify the maximum number of function evaluations they
    will try before giving up - in contrast to
    \code{\link{optim}} and \code{optimx}-wrapped optimizers,
    which use \code{maxit}. (Also see \code{\link{convergence}}
    for details of stopping tolerances for different optimizers.)

    \emph{Note:} All of \code{\link{lmer}()}, \code{\link{glmer}()} and
    \code{\link{nlmer}()} have an optional integer argument \code{verbose}
    which you should raise (to a positive value) in order to get diagnostic
    console output about the optimization progress.
  }
  \item{action}{character - generic choices for the severity level
    of any test, with possible values \describe{
      \item{"ignore":}{skip the test.}
      \item{"warning":}{warn if test fails.}
      \item{"message":}{print a message if test fails.}
      \item{"stop":}{throw an error if test fails.}
    }
  }
  \item{tol}{(numeric) tolerance for checking the gradient, scaled
    relative to the curvature (i.e., testing the gradient on a scale
    defined by its Wald standard deviation)}
  \item{relTol}{(numeric) tolerance for the gradient, scaled
    relative to the magnitude of the estimated coefficient}
  \item{mod.type}{model type (for internal use)}
  \item{standardize.X}{scale columns of X matrix? (not yet implemented)}
  \item{autoscale}{if \code{TRUE}, automatically applies scaling for
    the continuous covariates, and then back-transforms the estimates 
    back to the original scale to prevent numeric stability without user
    intervention. (NOTE: not compatiable with glmer yet!)}
  \item{\dots}{other elements to include in check specification}
}
\value{
  The \code{*Control} functions return a list (inheriting from class
  \code{"merControl"}) containing
  \enumerate{
    \item general control parameters, such as \code{optimizer}, \code{restart_edge};
    \item (currently not for \code{nlmerControl}:)
    \code{"checkControl"}, a \code{\link{list}} of data-checking
    specifications, e.g., \code{check.nobs.vs.rankZ};
    \item parameters to be passed to the optimizer, i.e., the \code{optCtrl}
    list, which may contain \code{maxiter}.
  }

  \code{.makeCC} returns a list containing the check specification
  (action, tolerance, and optionally relative tolerance).

}
\details{
  Note that (only!) the pre-fitting \dQuote{checking options}
  (i.e., all those starting with \code{"check."} but \emph{not}
  including the convergence checks (\code{"check.conv.*"}) or
  rank-checking (\code{"check.rank*"}) options)
  may also be set globally via \code{\link{options}}.
  In that case, \code{(g)lmerControl} will use them rather than the
  default values, but will \emph{not} override values that are passed as
  explicit arguments.

  For example, \code{options(lmerControl=list(check.nobs.vs.rankZ = "ignore"))}
  will suppress warnings that the number of observations is less than
  the rank of the random effects model matrix \code{Z}.
}
\seealso{\link{convergence} and \code{\link{allFit}()} which fits
  for a couple of optimizers;
  \code{\link{nloptwrap}} for the \code{lmerControl()} default optimizer.
}
\examples{
str(lmerControl())
str(glmerControl())
## fit with default algorithm [nloptr version of BOBYQA] ...
fm0 <- lmer(Reaction ~ Days +   ( 1 | Subject), sleepstudy)
fm1 <- lmer(Reaction ~ Days + (Days | Subject), sleepstudy)
## or with "bobyqa" (default 2013 - 2019-02) ...
fm1_bobyqa <- update(fm1, control = lmerControl(optimizer="bobyqa"))
## or with "Nelder_Mead" (the default till 2013) ...
fm1_NMead <- update(fm1, control = lmerControl(optimizer="Nelder_Mead"))
## or with the nlminb function used in older (<1.0) versions of lme4;
## this will usually replicate older results
if (require(optimx)) {
    fm1_nlminb <- update(fm1,
                         control = lmerControl(optimizer= "optimx",
                                               optCtrl  = list(method="nlminb")))
    ## The other option here is method="L-BFGS-B".
}

## Or we can wrap base::optim():
optimwrap <- function(fn,par,lower,upper,control=list(),
                      ...) {
    if (is.null(control$method)) stop("must specify method in optCtrl")
    method <- control$method
    control$method <- NULL
    ## "Brent" requires finite upper values (lower bound will always
    ##  be zero in this case)
    if (method=="Brent") upper <- pmin(1e4,upper)
    res <- optim(par=par, fn=fn, lower=lower,upper=upper,
                 control=control,method=method,...)
    with(res, list(par  = par,
                   fval = value,
                   feval= counts[1],
                   conv = convergence,
                   message = message))
}
fm0_brent <- update(fm0,
                    control = lmerControl(optimizer = "optimwrap",
                                          optCtrl = list(method="Brent")))

## You can also use functions (in addition to the lmerControl() default "NLOPT_BOBYQA")
## from the 'nloptr' package, see also  '?nloptwrap' :
if (require(nloptr)) {
    fm1_nloptr_NM <- update(fm1, control=lmerControl(optimizer="nloptwrap",
                                      optCtrl=list(algorithm="NLOPT_LN_NELDERMEAD")))
    fm1_nloptr_COBYLA <- update(fm1, control=lmerControl(optimizer="nloptwrap",
                                      optCtrl=list(algorithm="NLOPT_LN_COBYLA",
                                                   xtol_rel=1e-6,
                                                   xtol_abs=1e-10,
                                                   ftol_abs=1e-10)))
}
## other algorithm options include NLOPT_LN_SBPLX
}

\name{dummy}
\alias{dummy}
\title{Dummy variables (experimental)}
\usage{
  dummy(f, levelsToKeep)
}
\arguments{
  \item{f}{An object coercible to \code{\link{factor}}.}
  \item{levelsToKeep}{An optional character vector giving the subset of
  \code{levels(f)} to be converted to dummy variables.}
}
\value{
  A \code{\link{model.matrix}} with dummy variables as columns.
}
\description{
  Largely a wrapper for \code{\link{model.matrix}} that
  accepts a factor, \code{f}, and returns a dummy
  matrix with \code{nlevels(f)-1} columns (the first
  column is dropped by default).  Useful whenever one wishes to avoid
  the behaviour of \code{model.matrix} of always returning
  an \code{nlevels(f)}-column matrix, either by the
  addition of an intercept column, or by keeping one
  column for all levels.
}
\examples{
data(Orthodont,package="nlme")
lmer(distance ~ age + (age|Subject) +
     (0+dummy(Sex, "Female")|Subject), data = Orthodont)
}

\docType{class}
\name{NelderMead-class}
\alias{NelderMead}
\alias{NelderMead-class}
\title{Class \code{"NelderMead"} of Nelder-Mead optimizers and its Generator}
\description{
  Class \code{"NelderMead"} is a reference class for a Nelder-Mead
  simplex optimizer allowing box constraints on the parameters and using
  reverse communication.

  The \code{NelderMead()} function conveniently generates such objects.
}
\usage{
NelderMead(...)
}
\arguments{
  \item{\dots}{Argument list (see Note below).}
}
\note{
  This is the default optimizer for the second stage of
  \code{\link{glmer}} and \code{\link{nlmer}} fits.  We
  found that it was more reliable and often faster than
  more sophisticated optimizers.

  Arguments to \code{NelderMead()} and the \code{\link{new}} method must
  be named arguments:
  \describe{
    \item{lower}{numeric vector of lower bounds - elements may be \code{-Inf}.}
    \item{upper}{numeric vector of upper bounds - elements may be \code{Inf}.}
    \item{xst}{numeric vector of initial step sizes to establish the
      simplex - all elements must be non-zero.}
    \item{x0}{numeric vector of starting values for the parameters.}
    \item{xt}{numeric vector of tolerances on the parameters.}
  }
}
\section{Methods}{
  \describe{
    \item{\code{NelderMead$new(lower, upper, xst, x0, xt)}}{Create a
      new \code{\linkS4class{NelderMead}} object}
  }
}
\section{Extends}{
  All reference classes extend and inherit methods from
  \code{"\linkS4class{envRefClass}"}.
}
\references{
  Based on code in the NLopt collection.
}
\seealso{
  \code{\link{Nelder_Mead}}, the typical \dQuote{constructor}.
  Further, \code{\link{glmer}}, \code{\link{nlmer}}
}
\examples{
showClass("NelderMead")
}
\keyword{classes}

\name{ngrps}
\alias{ngrps}
\title{Number of Levels of a Factor or a "merMod" Model}
\description{
  Returns the number of levels of a \code{\link{factor}} or a set of
  factors, currently e.g., for each of the grouping factors of
  \code{\link{lmer}()}, \code{\link{glmer}()}, etc.
}
\usage{
ngrps(object, \dots)
}
\arguments{
  \item{object}{an \code{R} object, see Details.}
  \item{\dots}{currently ignored.}
}
\value{
  The number of levels (of a factor) or vector of number of levels for
  each \dQuote{grouping factor} of a
}
\details{
  Currently there are methods for objects of class
  \code{\linkS4class{merMod}}, i.e., the result of \code{\link{lmer}()} etc,
  and \code{\link{factor}} objects.
}
\examples{
ngrps(factor(seq(1,10,2)))
ngrps(lmer(Reaction ~ 1|Subject, sleepstudy))

## A named vector if there's more than one grouping factor :
ngrps(lmer(strength ~ (1|batch/cask), Pastes))
## cask:batch      batch
##         30         10

methods(ngrps) # -> "factor" and "merMod"
}

\name{nlmer}
\title{Fitting Nonlinear Mixed-Effects Models}
\alias{nlmer}
\concept{NLMM}
\description{
  Fit a nonlinear mixed-effects model (NLMM) to data, via maximum
  likelihood.
}
\usage{
nlmer(formula, data = NULL, control = nlmerControl(),
      start = NULL, verbose = 0L, nAGQ = 1L, subset, weights, na.action,
      offset, contrasts = NULL, devFunOnly = FALSE)
}
\arguments{
  \item{formula}{a three-part \dQuote{nonlinear mixed model} formula, of
    the form \code{resp ~ Nonlin(...) ~ fixed + random}, where the third
    part is similar to the RHS formula of, e.g., \code{\link{lmer}}.
    %% (see detailed documentation) FIXME!  where would that be?
    Currently, the \code{Nonlin(..)} formula part must not only return a
    numeric vector, but also must have a \code{"gradient"} attribute, a
    \code{\link{matrix}}.  The functions \code{\link{SSbiexp}},
    \code{\link{SSlogis}}, etc, see \code{\link{selfStart}}, provide
    this (and more).  Alternatively, you can use \code{\link{deriv}()}
    to automatically produce such functions or expressions.}
  \item{data}{an optional data frame containing the variables named in
    \code{formula}.  By default the variables are taken from the
    environment from which \code{lmer} is called.  While \code{data} is
    optional, the package authors \emph{strongly} recommend its use,
    especially when later applying methods such as \code{\link{update}}
    and \code{\link{drop1}} to the fitted model (\emph{such methods are
    not guaranteed to work properly if \code{data} is omitted}).  If
    \code{data} is omitted, variables will be taken from the environment
    of \code{formula} (if specified as a formula) or from the parent
    frame (if specified as a character vector).}
  \item{control}{a list (of correct class, resulting from
    \code{\link{lmerControl}()} or \code{\link{glmerControl}()}
    respectively) containing control parameters, including the nonlinear
    optimizer to be used and parameters to be passed through to the
    nonlinear optimizer, see the \code{*lmerControl} documentation for
    details.}
  \item{start}{starting estimates for the nonlinear model parameters, as
    a named numeric vector or as a list with components
    \describe{
      \item{nlpars}{required numeric vector of starting values for the
	nonlinear model parameters}
      \item{theta}{optional numeric vector of starting values for the
	covariance parameters} % FIXME: currently it is not used
    }
  }
  \item{verbose}{integer scalar.  If \code{> 0} verbose output is
    generated during the optimization of the parameter estimates.  If
    \code{> 1} verbose output is generated during the individual PIRLS steps
    (PIRLS aka PRSS, e.g. in the C++ sources).}
  \item{nAGQ}{integer scalar - the number of points per axis for
    evaluating the adaptive Gauss-Hermite approximation to the
    log-likelihood.  Defaults to 1, corresponding to the Laplace
    approximation.  Values greater than 1 produce greater accuracy in
    the evaluation of the log-likelihood at the expense of speed.  A
    value of zero uses a faster but less exact form of parameter
    estimation for GLMMs by optimizing the random effects and the
    fixed-effects coefficients in the penalized iteratively reweighted
    least squares (PIRLS) step.}

  \item{subset}{an optional expression indicating the subset of the rows
    of \code{data} that should be used in the fit.  This can be a
    logical vector, or a numeric vector indicating which observation
    numbers are to be included, or a character vector of the row names
    to be included.  All observations are included by default.}
  \item{weights}{an optional vector of \sQuote{prior weights} to be used
    in the fitting process.  Should be \code{NULL} or a numeric vector.}
  \item{na.action}{a function that indicates what should happen when the
    data contain \code{NA}s.  The default action (\code{na.omit},
    inherited from the \sQuote{factory fresh} value of
    \code{\link{getOption}("na.action")}) strips any observations with
    any missing values in any variables.}
  \item{offset}{this can be used to specify an \emph{a priori} known
    component to be included in the linear predictor during fitting.
    This should be \code{NULL} or a numeric vector of length equal to
    the number of cases.  One or more \code{\link{offset}} terms can be
    included in the formula instead or as well, and if more than one is
    specified their sum is used.  See \code{\link{model.offset}}.}
  \item{contrasts}{an optional \code{\link{list}}.  See the
    \code{contrasts.arg} of \code{\link{model.matrix.default}}.}

  \item{devFunOnly}{logical - return only the deviance evaluation
    function.  Note that because the deviance function operates on
    variables stored in its environment, it may not return
    \emph{exactly} the same values on subsequent calls (but the results
    should always be within machine tolerance).}
}
\details{
  Fit nonlinear mixed-effects models, such as those used in
  population pharmacokinetics.
}
\note{
  Adaptive Gauss-Hermite quadrature (\code{nAGQ > 1}) is not
  currently implemented for \code{nlmer}.  Several other
  methods, such as simulation or prediction with new data,
  are unimplemented or very lightly tested.

  A \code{method} argument was used in earlier versions of the \pkg{lme4}
  package.  Its functionality has been replaced by the \code{nAGQ} argument.
}
\examples{
## nonlinear mixed models --- 3-part formulas ---
## 1. basic nonlinear fit. Use stats::SSlogis for its
## implementation of the 3-parameter logistic curve.
## "SS" stands for "self-starting logistic", but the
## "self-starting" part is not currently used by nlmer ... 'start' is
## necessary
startvec <- c(Asym = 200, xmid = 725, scal = 350)
(nm1 <- nlmer(circumference ~ SSlogis(age, Asym, xmid, scal) ~ Asym|Tree,
             Orange, start = startvec))
## 2. re-run with "quick and dirty" PIRLS step
(nm1a <- update(nm1, nAGQ = 0L))

## 3. Fit the same model with a user-built function:
## a. Define formula
nform <- ~Asym/(1+exp((xmid-input)/scal))
## b. Use deriv() to construct function:
nfun <- deriv(nform,namevec=c("Asym","xmid","scal"),
              function.arg=c("input","Asym","xmid","scal"))
nm1b <- update(nm1,circumference ~ nfun(age, Asym, xmid, scal)  ~ Asym | Tree)

## 4. User-built function without using deriv():
##    derivatives could be computed more efficiently
##    by pre-computing components, but these are essentially
##    the gradients as one would derive them by hand
nfun2 <- function(input, Asym, xmid, scal) {
    value <- Asym/(1+exp((xmid-input)/scal))
    grad <- cbind(Asym=1/(1+exp((xmid-input)/scal)),
              xmid=-Asym/(1+exp((xmid-input)/scal))^2*1/scal*
                    exp((xmid-input)/scal),
              scal=-Asym/(1+exp((xmid-input)/scal))^2*
                     -(xmid-input)/scal^2*exp((xmid-input)/scal))
    attr(value,"gradient") <- grad
    value
}
stopifnot(all.equal(attr(nfun(2,1,3,4),"gradient"),
                    attr(nfun(2,1,3,4),"gradient")))
nm1c <- update(nm1,circumference ~ nfun2(age, Asym, xmid, scal)  ~ Asym | Tree)
}
\keyword{models}

\name{drop1.merMod}
\alias{drop1.merMod}
\title{Drop all possible single fixed-effect terms from a mixed effect model}
\description{
  Drop allowable single terms from the model: see \code{\link{drop1}}
  for details of how the appropriate scope for dropping terms
  is determined.
}
\usage{
\method{drop1}{merMod}(object, scope, scale = 0,
      test = c("none", "Chisq", "user"),
      k = 2, trace = FALSE, sumFun, \dots)
}
\arguments{
  \item{object}{a fitted \code{merMod} object.}
  \item{scope}{a formula giving the terms to be considered for adding or
    dropping.}
  \item{scale}{Currently ignored (included for S3 method compatibility)}
  \item{test}{should the results include a test statistic relative to the
    original model?
    The \eqn{\chi^2}{Chisq} test is a likelihood-ratio test,
    which is approximate due to finite-size effects.
  }
  \item{k}{the penalty constant in AIC}
  \item{trace}{print tracing information?}
  \item{sumFun}{a summary \code{\link{function}} to be used when
    \code{test=="user"}.  It must allow arguments \code{scale} and
    \code{k}, but these may be ignored (e.g. swallowed by \code{\dots},
    see the examples).
    The first two arguments must be \code{object}, the full model fit,
    and \code{objectDrop}, a reduced model.  If \code{objectDrop} is missing,
    \code{sumFun(*)} should return a vector with the appropriate
    length and names (the actual contents are ignored).
  }
  \item{\dots}{other arguments (ignored)}
}
\value{
  An object of class \code{anova} summarizing the differences in fit
  between the models.
}
\details{
  \code{drop1} relies on being able to find the appropriate information
  within the environment of the formula of the original model.  If the
  formula is created in an environment that does not contain the data,
  or other variables passed to the original model (for example, if
  a separate function is called to define the formula), then
  \code{drop1} will fail.  A workaround (see example below) is to
  manually specify an appropriate environment for the formula.
}
\examples{
fm1 <- lmer(Reaction ~ Days + (Days|Subject), sleepstudy)
## likelihood ratio tests
drop1(fm1,test="Chisq")
## use Kenward-Roger corrected F test, or parametric bootstrap,
## to test the significance of each dropped predictor
if (require(pbkrtest) && packageVersion("pbkrtest")>="0.3.8") {
   KRSumFun <- function(object, objectDrop, ...) {
      krnames <- c("ndf","ddf","Fstat","p.value","F.scaling")
      r <- if (missing(objectDrop)) {
          setNames(rep(NA,length(krnames)),krnames)
      } else {
         krtest <- KRmodcomp(object,objectDrop)
         unlist(krtest$stats[krnames])
      }
      attr(r,"method") <- c("Kenward-Roger via pbkrtest package")
      r
   }
   drop1(fm1, test="user", sumFun=KRSumFun)

   if(lme4:::testLevel() >= 3) { ## takes about 16 sec
     nsim <- 100
     PBSumFun <- function(object, objectDrop, ...) {
	pbnames <- c("stat","p.value")
	r <- if (missing(objectDrop)) {
	    setNames(rep(NA,length(pbnames)),pbnames)
	} else {
	   pbtest <- PBmodcomp(object,objectDrop,nsim=nsim)
	   unlist(pbtest$test[2,pbnames])
	}
	attr(r,"method") <- c("Parametric bootstrap via pbkrtest package")
	r
     }
     system.time(drop1(fm1, test="user", sumFun=PBSumFun))
   }
}
## workaround for creating a formula in a separate environment
createFormula <- function(resp, fixed, rand) {  
    f <- reformulate(c(fixed,rand),response=resp)
    ## use the parent (createModel) environment, not the
    ## environment of this function (which does not contain 'data')
    environment(f) <- parent.frame()
    f
}
createModel <- function(data) {
    mf.final <- createFormula("Reaction", "Days", "(Days|Subject)")
    lmer(mf.final, data=data)
}
drop1(createModel(data=sleepstudy))
}
\keyword{misc}

\docType{data}
\name{VerbAgg}
\alias{VerbAgg}
\title{Verbal Aggression item responses}
\format{A data frame with 7584 observations on the following 13 variables.
  \describe{
    \item{\code{Anger}}{the subject's Trait Anger score as measured on
      the State-Trait Anger Expression Inventory (STAXI)}
    \item{\code{Gender}}{the subject's gender - a factor with levels
      \code{M} and \code{F}}
    \item{\code{item}}{the item on the questionaire, as a factor}
    \item{\code{resp}}{the subject's response to the item - an ordered
      factor with levels \code{no} < \code{perhaps} < \code{yes}}
    \item{\code{id}}{the subject identifier, as a factor}
    \item{\code{btype}}{behavior type - a factor with levels
      \code{curse}, \code{scold} and \code{shout}}
    \item{\code{situ}}{situation type - a factor with levels
      \code{other} and \code{self} indicating other-to-blame and self-to-blame}
    \item{\code{mode}}{behavior mode - a factor with levels \code{want}
      and \code{do}}
    \item{\code{r2}}{dichotomous version of the response - a factor with
      levels \code{N} and \code{Y}}
  }
}
\source{
  Data originally from the UC Berkeley BEAR Center; original link is
  available at
  \url{https://web.archive.org/web/20221128003829/https://old.bear.berkeley.edu/page/materials-explanatory-item-response-models},
  but the data are no longer accessible there.
}
\description{
  These are the item responses to a questionaire on verbal
  aggression.  These data are used throughout De Boeck and
  Wilson (2004) to illustrate various forms of item
  response models.
}
\examples{
str(VerbAgg)
## Show how  r2 := h(resp) is defined:
with(VerbAgg, stopifnot( identical(r2, {
     r <- factor(resp, ordered=FALSE); levels(r) <- c("N","Y","Y"); r})))

xtabs(~ item + resp, VerbAgg)
xtabs(~ btype + resp, VerbAgg)
round(100 * ftable(prop.table(xtabs(~ situ + mode + resp, VerbAgg), 1:2), 1))
person <- unique(subset(VerbAgg, select = c(id, Gender, Anger)))
require(lattice)
densityplot(~ Anger, person, groups = Gender, auto.key = list(columns = 2),
            xlab = "Trait Anger score (STAXI)")

if(lme4:::testLevel() >= 3) { ## takes about 15 sec
    print(fmVA <- glmer(r2 ~ (Anger + Gender + btype + situ)^2 +
 		   (1|id) + (1|item), family = binomial, data =
		   VerbAgg), corr=FALSE)
} ## testLevel() >= 3
if (interactive()) {
## much faster but less accurate
    print(fmVA0 <- glmer(r2 ~ (Anger + Gender + btype + situ)^2 +
                             (1|id) + (1|item), family = binomial,
                         data = VerbAgg, nAGQ=0L), corr=FALSE)
} ## interactive()
}
\references{
  \insertRef{de2004explanatory}{lme4}
}
\keyword{datasets}

\name{glmer}
\title{Fitting Generalized Linear Mixed-Effects Models}
\alias{glmer}
\concept{ GLMM }
\description{
  Fit a generalized linear mixed-effects model (GLMM).  Both fixed
  effects and random effects are specified via the model \code{formula}.
}
\usage{
glmer(formula, data = NULL, family = gaussian
    , control = glmerControl()
    , start = NULL
    , verbose = 0L
    , nAGQ = 1L
    , subset, weights, na.action, offset, contrasts = NULL
    , mustart, etastart
    , devFunOnly = FALSE)
}
\arguments{
  \item{formula}{a two-sided linear formula object describing both the
    fixed-effects and random-effects part of the model, with the response
    on the left of a \code{~} operator and the terms, separated by
    \code{+} operators, on the right.  Random-effects terms are
    distinguished by vertical bars (\code{"|"}) separating expressions
    for design matrices from grouping factors.}

  \item{data}{an optional data frame containing the variables named in
    \code{formula}.  By default the variables are taken from the
    environment from which \code{lmer} is called.  While \code{data} is
    optional, the package authors \emph{strongly} recommend its use,
    especially when later applying methods such as \code{update} and
    \code{drop1} to the fitted model (\emph{such methods are not
    guaranteed to work properly if \code{data} is omitted}).  If
    \code{data} is omitted, variables will be taken from the environment
    of \code{formula} (if specified as a formula) or from the parent
    frame (if specified as a character vector).}

  \item{family}{a GLM family, see \code{\link[stats]{glm}} and
    \code{\link[stats]{family}}.}
  \item{control}{a list (of correct class, resulting from
    \code{\link{lmerControl}()} or \code{\link{glmerControl}()}
    respectively) containing control parameters, including the nonlinear
    optimizer to be used and parameters to be passed through to the
    nonlinear optimizer, see the \code{*lmerControl} documentation for
    details.}
  \item{start}{See the "start" argument of \code{\link{lmer}} for
    details. If two optimization steps are
    performed for a GLMM, as by default (i.e. one with \code{nAGQ=0} and
    a second with \code{nAGQ} > 0), then the first step uses starting values
    \code{par} and the second step uses starting values
    \code{c(newpar, fixef)}, where \code{newpar} is the optimum obtained
    in the first step.  See \code{\link{modular}} for details about
    finer control of optimization.}
  \item{verbose}{integer scalar.  If \code{> 0} verbose output is
    generated during the optimization of the parameter estimates.  If
    \code{> 1} verbose output is generated during the individual
    penalized iteratively reweighted least squares (PIRLS) steps.}
  \item{nAGQ}{integer scalar - the number of points per axis for
    evaluating the adaptive Gauss-Hermite approximation to the
    log-likelihood.  Defaults to 1, corresponding to the Laplace
    approximation.  Values greater than 1 produce greater accuracy in
    the evaluation of the log-likelihood at the expense of speed.  A
    value of zero uses a faster but less exact form of parameter
    estimation for GLMMs by optimizing the random effects and the
    fixed-effects coefficients in the penalized iteratively reweighted
    least squares step. (See Details.)}
  \item{subset}{an optional expression indicating the subset of the rows
    of \code{data} that should be used in the fit. This can be a logical
    vector, or a numeric vector indicating which observation numbers are
    to be included, or a character vector of the row names to be
    included.  All observations are included by default.}
  \item{weights}{an optional vector of \sQuote{prior weights} to be used
      in the fitting process.  Should be \code{NULL} or a numeric
      vector.}
  \item{na.action}{a function that indicates what should happen when the
    data contain \code{NA}s.  The default action (\code{na.omit},
    inherited from the \sQuote{factory fresh} value of
    \code{getOption("na.action")}) strips any observations with any
    missing values in any variables.}
  \item{offset}{this can be used to specify an \emph{a priori} known
    component to be included in the linear predictor during
    fitting. This should be \code{NULL} or a numeric vector of length
    equal to the number of cases.  One or more \code{\link{offset}}
    terms can be included in the formula instead or as well, and if more
    than one is specified their sum is used.  See \code{\link{model.offset}}.}
  \item{contrasts}{an optional list.  See the \code{contrasts.arg} of
    \code{\link{model.matrix.default}}.}

  \item{mustart}{optional starting values on the scale of the
    conditional mean, as in \code{\link[stats]{glm}}; see there for
    details.}
  \item{etastart}{optional starting values on the scale of the unbounded
    predictor as in \code{\link[stats]{glm}}; see there for details.}

  \item{devFunOnly}{logical - return only the deviance evaluation
    function. Note that because the deviance function operates on
    variables stored in its environment, it may not return
    \emph{exactly} the same values on subsequent calls (but the results
    should always be within machine tolerance).}
}
\value{
  An object of class \code{\link[=merMod-class]{merMod}} (more specifically,
  an object of \emph{subclass} \code{glmerMod}) for which many
  methods are available (e.g. \code{methods(class="merMod")})
}
\note{
  In earlier version of the \pkg{lme4} package, a \code{method} argument was
  used.  Its functionality has been replaced by the \code{nAGQ} argument.
}
\details{
  Fit a generalized linear mixed model, which incorporates both
  fixed-effects parameters and random effects in a linear predictor, via
  maximum likelihood.  The linear predictor is related to the
  conditional mean of the response through the inverse link function
  defined in the GLM \code{family}.

  The expression for the likelihood of a mixed-effects model is an
  integral over the random effects space.  For a linear mixed-effects
  model (LMM), as fit by \code{\link{lmer}}, this integral can be
  evaluated exactly.  For a GLMM the integral must be approximated.  The
  most reliable approximation for GLMMs
  is adaptive Gauss-Hermite quadrature,
  at present implemented only for models with
  a single scalar random effect.  The
  \code{nAGQ} argument controls the number of nodes in the quadrature
  formula.  A model with a single, scalar random-effects term could
  reasonably use up to 25 quadrature points per scalar integral (although
  the code allows for up to 100 quadrature points).

%  With vector-valued random effects the complexity of the Gauss-Hermite
%  quadrature formulas increases dramatically with the dimension.  For a
%  3-dimensional vector-valued random effect \code{nAGQ=5} requires 93
%  evaluations of the GLM deviance per evaluation of the approximate GLMM
%  deviance.  For 20-dimensional evaluations of the GLM deviance per
%  evaluation of the approximate GLMM deviance.
%  The default approximation is the Laplace approximation,
%  corresponding to \code{nAGQ=1}.

}
\seealso{
  \code{\link{lmer}} (for details on formulas and
  parameterization); \code{\link[stats]{glm}} for Generalized Linear
  Models (\emph{without} random effects).
  \code{\link{nlmer}} for nonlinear mixed-effects models.

  \code{\link{glmer.nb}} to fit negative binomial GLMMs.
}
\examples{
## generalized linear mixed model
library(lattice)
xyplot(incidence/size ~ period|herd, cbpp, type=c('g','p','l'),
       layout=c(3,5), index.cond = function(x,y)max(y))
(gm1 <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
              data = cbpp, family = binomial))
## using nAGQ=0 only gets close to the optimum
(gm1a <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
               cbpp, binomial, nAGQ = 0))
## using  nAGQ = 9  provides a better evaluation of the deviance
## Currently the internal calculations use the sum of deviance residuals,
## which is not directly comparable with the nAGQ=0 or nAGQ=1 result.
## 'verbose = 1' monitors iteratin a bit; (verbose = 2 does more):
(gm1a <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
               cbpp, binomial, verbose = 1, nAGQ = 9))

## GLMM with individual-level variability (accounting for overdispersion)
## For this data set the model is the same as one allowing for a period:herd
## interaction, which the plot indicates could be needed.
cbpp$obs <- 1:nrow(cbpp)
(gm2 <- glmer(cbind(incidence, size - incidence) ~ period +
    (1 | herd) +  (1|obs),
              family = binomial, data = cbpp))
anova(gm1,gm2)

## glmer and glm log-likelihoods are consistent
gm1Devfun <- update(gm1,devFunOnly=TRUE)
gm0 <- glm(cbind(incidence, size - incidence) ~ period,
           family = binomial, data = cbpp)
## evaluate GLMM deviance at RE variance=theta=0, beta=(GLM coeffs)
gm1Dev0 <- gm1Devfun(c(0,coef(gm0)))
## compare
stopifnot(all.equal(gm1Dev0,c(-2*logLik(gm0))))
## the toenail oncholysis data from Backer et al 1998
## these data are notoriously difficult to fit
\dontrun{
if (require("HSAUR3")) {
    gm2 <- glmer(outcome~treatment*visit+(1|patientID),
                 data=toenail,
                 family=binomial,nAGQ=20)
}
}
}
\keyword{models}

\name{glmerLaplaceHandle}
\alias{glmerLaplaceHandle}
\title{Handle for \code{glmerLaplace}}
\usage{
glmerLaplaceHandle(pp, resp, nAGQ, tol, maxit, verbose)
}
\arguments{
  \item{pp}{\code{\link{merPredD}} object}
  \item{resp}{\code{\link{lmResp}} object}
  \item{nAGQ}{see \code{\link{glmer}}}
  \item{tol}{tolerance}
  \item{maxit}{maximum number of pwrss iterations}
  \item{verbose}{display optimizer progress}
}
\description{
  Handle for calling the \code{glmerLaplace} \code{C++} function.  Not
  intended for routine use.
}
\value{
  Value of the objective function
}
\keyword{utilities}

\name{subbars}
\alias{subbars}
\title{"Sub[stitute] Bars"}
\usage{
  subbars(term)
}
\arguments{
  \item{term}{a mixed-model formula}
}
\value{
  the formula with all | operators replaced by +
}
\description{
  Substitute the '+' function for the '|' function in a mixed-model
  formula, recursively (hence the argument name \code{term}).  This
  provides a formula suitable for the current \code{\link{model.frame}}
  function.
}
\seealso{
  \code{\link{formula}}, \code{\link{model.frame}},
  \code{\link{model.matrix}}.

  Other utilities: \code{\link{findbars}}, \code{\link{nobars}},
  \code{\link{mkRespMod}}, \code{\link{mkReTrms}},
  \code{\link{nlformula}}.
}
\examples{
subbars(Reaction ~ Days + (Days|Subject)) ## => Reaction ~ Days + (Days + Subject)
}
\keyword{models}
\keyword{utilities}

\name{refit}
\alias{refit}
\alias{refit.merMod}
\title{Refit a (merMod) Model with a Different Response}
\description{
   Refit a model, possibly after modifying the response vector.  This
   makes use of the model representation and directly goes to the
   optimization.
}
\usage{
refit(object, newresp, \dots)

\method{refit}{merMod}(object, newresp = NULL, newweights = NULL,
      rename.response = FALSE,
      maxit = 100, \dots)
}
\arguments{
  \item{object}{a fitted model, usually of class
    \code{\linkS4class{lmerMod}}, to be refit with a new response.}
  \item{newresp}{an (optional) numeric vector providing the new
    response, of the same length as the original response (see
    \code{Details} for information on NA handling).  May also be a data
    frame with a single numeric column, e.g. as produced by
    \code{simulate(object)}.}
  \item{newweights}{an (optional) numeric vector of new weights}
  \item{rename.response}{when refitting the model, should the name of
    the response variable in the formula and model frame be replaced with
    the name of \code{newresp}?}
  \item{maxit}{scalar integer, currently only for GLMMs: the maximal
    number of Pwrss update iterations.}
  \item{\dots}{optional additional parameters.  For the \code{merMod}
    method, \code{control}.}
}
\value{
  an object like \code{x}, but fit to a different response vector \eqn{Y}.
}
\seealso{
  \code{\link{update.merMod}} for more flexible and extensive model
  refitting;
  \code{\link{refitML}} for refitting a REML fitted model with maximum
  likelihood (\sQuote{ML}).
}
\details{
  Refit a model, possibly after modifying the response vector.  This
  could be done using \code{\link{update}()}, but the \code{refit()}
  approach should be faster because it bypasses the creation of the
  model representation and goes directly to the optimization step.

  Setting \code{rename.response = TRUE} may be necessary if one
  wants to do further operations (such as \code{update}) on the fitted
  model.  However, the refitted model will still be slightly different
  from the equivalent model fitted via \code{update}; in particular, the
  \code{terms} component is not updated to reflect the new response
  variable, if it has a different name from the original.

  If \code{newresp} has an \code{na.action} attribute, then it is
  assumed that NA values have already been removed from the numeric
  vector; this allows the results of \code{\link{simulate}(object)} to
  be used even if the original response vector contained NA values.
  Otherwise, the length of \code{newresp} must be the same as the
  \emph{original} length of the response.
}
\examples{
## Ex. 1: using refit() to fit each column in a matrix of responses -------
set.seed(101)
Y <- matrix(rnorm(1000),ncol=10)
## combine first column of responses with predictor variables
d <- data.frame(y=Y[,1],x=rnorm(100),f=rep(1:10,10))
## (use check.conv.grad="ignore" to disable convergence checks because we
##  are using a fake example)
## fit first response
fit1 <- lmer(y ~ x+(1|f), data = d,
             control= lmerControl(check.conv.grad="ignore",
                                  check.conv.hess="ignore"))
## combine fit to first response with fits to remaining responses
res <- c(fit1, lapply(as.data.frame(Y[,-1]), refit, object=fit1))

## Ex. 2: refitting simulated data using data that contain NA values ------
sleepstudyNA <- sleepstudy
sleepstudyNA$Reaction[1:3] <- NA
fm0 <- lmer(Reaction ~ Days + (1|Subject), sleepstudyNA)
## the special case of refitting with a single simulation works ...
ss0 <- refit(fm0, simulate(fm0))
## ... but if simulating multiple responses (for efficiency),
## need to use na.action=na.exclude in order to have proper length of data
fm1 <- lmer(Reaction ~ Days + (1|Subject), sleepstudyNA, na.action=na.exclude)
ss <- simulate(fm1, 5)
res2 <- refit(fm1, ss[[5]])
}
%% unit tests --> ../tests/refit.R

\name{mkVarCorr}
\title{Make Variance and Correlation Components of Covariance}
\alias{mkVarCorr}
\usage{
mkVarCorr(sc, cnms, nc = lengths(cnms, use.names = FALSE),
          theta, nms = names(cnms), reCovs = NULL, relReCovs = TRUE)
}
\description{
  Computes from \code{reCovs} the variance and correlation components of
  random effect covariance matrices.
}
\arguments{
  \item{sc}{a numeric vector of length 1 giving the residual standard
    deviation for LMMs.  Set to \code{NULL} for GLMMs.}
  \item{cnms}{a list of character vectors giving component dimnames,
    used only to determine \code{nc} and \code{nms} if those are
    missing.}
  \item{nc}{an integer vector giving component dimensions (number of
    columns).}
  \item{theta}{a numeric vector giving covariance parameters
    (lower triangular entries of relative Cholesky factors in
    column-major order), used only to determine \code{reCovs} when
    that is missing.}
  \item{nms}{a character vector giving component names.}
  \item{reCovs}{a list of \code{\linkS4class{Covariance}} objects.}
  \item{relReCovs}{a logical indicating if the components of
    \code{reCovs} represent relative covariance matrices (in which case
    they are scaled by \code{sc^2}).}
}
\value{
  A list of matrices.
}
\seealso{\code{\link{VarCorr}}.}

\docType{class}
\name{Covariance-class}
\alias{Covariance-class}
\alias{Covariance.us-class}
\alias{Covariance.diag-class}
\alias{Covariance.cs-class}
\alias{Covariance.ar1-class}
\alias{initialize,Covariance.us-method}
\alias{initialize,Covariance.diag-method}
\alias{initialize,Covariance.cs-method}
\alias{initialize,Covariance.ar1-method}
\alias{getPar}
\alias{getPar,Covariance-method}
\alias{getPar,merMod-method}
\alias{getParLength}
\alias{getParLength,Covariance-method}
\alias{getParLength,merMod-method}
\alias{getParNames}
\alias{getParNames,Covariance.us,character,character-method}
\alias{getParNames,Covariance.diag,character,character-method}
\alias{getParNames,Covariance.cs,character,character-method}
\alias{getParNames,Covariance.ar1,character,character-method}
\alias{getParNames,merMod,missing,missing-method}
\alias{setPar}
\alias{setPar,Covariance,numeric-method}
\alias{getTheta}
\alias{getTheta,Covariance.us-method}
\alias{getTheta,Covariance.diag-method}
\alias{getTheta,Covariance.cs-method}
\alias{getTheta,Covariance.ar1-method}
\alias{getTheta,merMod-method}
\alias{getThetaLength}
\alias{getThetaLength,Covariance.us-method}
\alias{getThetaLength,Covariance.diag-method}
\alias{getThetaLength,Covariance.cs-method}
\alias{getThetaLength,Covariance.ar1-method}
\alias{getThetaLength,merMod-method}
\alias{getThetaNames}
\alias{getThetaNames,Covariance.us,character,character-method}
\alias{getThetaNames,Covariance.diag,character,character-method}
\alias{getThetaNames,Covariance.cs,character,character-method}
\alias{getThetaNames,Covariance.ar1,character,character-method}
\alias{getThetaNames,merMod,missing,missing-method}
\alias{getThetaIndex}
\alias{getThetaIndex,Covariance.us-method}
\alias{getThetaIndex,Covariance.diag-method}
\alias{getThetaIndex,Covariance.cs-method}
\alias{getThetaIndex,Covariance.ar1-method}
\alias{getThetaIndexLength}
\alias{getThetaIndexLength,Covariance.us-method}
\alias{getThetaIndexLength,Covariance.diag-method}
\alias{getThetaIndexLength,Covariance.cs-method}
\alias{getThetaIndexLength,Covariance.ar1-method}
\alias{setTheta}
\alias{setTheta,Covariance.us,numeric-method}
\alias{setTheta,Covariance.diag,numeric-method}
\alias{setTheta,Covariance.cs,numeric-method}
\alias{setTheta,Covariance.ar1,numeric-method}
\alias{getLower}
\alias{getLower,Covariance.us-method}
\alias{getLower,Covariance.diag-method}
\alias{getLower,Covariance.cs-method}
\alias{getLower,Covariance.ar1-method}
\alias{getLower,merMod-method}
\alias{getUpper}
\alias{getUpper,Covariance.us-method}
\alias{getUpper,Covariance.diag-method}
\alias{getUpper,Covariance.cs-method}
\alias{getUpper,Covariance.ar1-method}
\alias{getUpper,merMod-method}
\alias{getLambda}
\alias{getLambda,Covariance.us-method}
\alias{getLambda,Covariance.diag-method}
\alias{getLambda,Covariance.cs-method}
\alias{getLambda,Covariance.ar1-method}
\alias{getLambda,merMod-method}
\alias{getLambdat.dp}
\alias{getLambdat.dp,Covariance.us-method}
\alias{getLambdat.dp,Covariance.diag-method}
\alias{getLambdat.dp,Covariance.cs-method}
\alias{getLambdat.dp,Covariance.ar1-method}
\alias{getLambdat.i}
\alias{getLambdat.i,Covariance.us-method}
\alias{getLambdat.i,Covariance.diag-method}
\alias{getLambdat.i,Covariance.cs-method}
\alias{getLambdat.i,Covariance.ar1-method}
\alias{getVC}
\alias{getVC,Covariance.us-method}
\alias{getVC,Covariance.diag-method}
\alias{getVC,Covariance.cs-method}
\alias{getVC,Covariance.ar1-method}
\alias{getVCNames}
\alias{getVCNames,Covariance.us,character,character-method}
\alias{getVCNames,Covariance.diag,character,character-method}
\alias{getVCNames,Covariance.cs,character,character-method}
\alias{getVCNames,Covariance.ar1,character,character-method}
\alias{getVCNames,merMod,missing,missing-method}
\alias{setVC}
\alias{setVC,Covariance.us,numeric,numeric-method}
\alias{setVC,Covariance.diag,numeric,numeric-method}
\alias{setVC,Covariance.cs,numeric,numeric-method}
\alias{setVC,Covariance.ar1,numeric,numeric-method}
\alias{getProfPar}
\alias{getProfPar,Covariance.us-method}
\alias{getProfPar,Covariance.diag-method}
\alias{getProfPar,Covariance.cs-method}
\alias{getProfPar,Covariance.ar1-method}
\alias{setProfPar}
\alias{setProfPar,Covariance.us,numeric-method}
\alias{setProfPar,Covariance.diag,numeric-method}
\alias{setProfPar,Covariance.cs,numeric-method}
\alias{setProfPar,Covariance.ar1,numeric-method}
\alias{getProfPar,merMod-method}
\alias{getProfLower}
\alias{getProfLower,Covariance.us-method}
\alias{getProfLower,Covariance.diag-method}
\alias{getProfLower,Covariance.cs-method}
\alias{getProfLower,Covariance.ar1-method}
\alias{getProfLower,merMod-method}
\alias{getProfUpper}
\alias{getProfUpper,Covariance.us-method}
\alias{getProfUpper,Covariance.diag-method}
\alias{getProfUpper,Covariance.cs-method}
\alias{getProfUpper,Covariance.ar1-method}
\alias{getProfUpper,merMod-method}

\title{Virtual Class \sQuote{Covariance} of Covariance Matrices}

\description{
  An \code{S4} class that describes the within-group variance-covariance 
  structures available for the \CRANpkg{lme4} package. The syntax for use is 
  similar to the \CRANpkg{glmmTMB} package. See the \emph{Examples} section 
  on how to use.
  
  There are four main covariance classes: \code{Covariance.us} (unstructured), 
  \code{Covariance.diag} (diagonal), \code{Covariance.cs} (compound symmetry),
  and \code{Covariance.ar1} (autoregressive order 1).
}

\usage{
getPar(object)
getParLength(object)
getParNames(object, cnm, gnm, prf = NULL, old = TRUE)
setPar(object, value, pos = 0L)
getTheta(object)
getThetaLength(object)
getThetaNames(object, cnm, gnm, prf = NULL, old = TRUE)
getThetaIndex(object)
getThetaIndexLength(object)
setTheta(object, value, pos = 0L)
getLower(object)
getUpper(object)
getLambda(object)
getLambdat.dp(object)
getLambdat.i(object)
getVC(object)
getVCNames(object, cnm, gnm, prf = NULL, old = TRUE)
setVC(object, vcomp, ccomp)
getProfPar(object, profscale, sc = NULL)
setProfPar(object, profscale, sc = NULL, value, pos = 0L)
getProfLower(object, profscale, sc = NULL)
getProfUpper(object, profscale, sc = NULL)
}

\details{
  In matrix notation, a linear mixed model can be represented as:
  \deqn{
  \mathbf{y} = X \beta + Z \mathbf{b} + \boldsymbol{\epsilon}
  }
  Where \eqn{\mathbf{b}} represents an unknown vector of random effects. The
  \code{Covariance} class helps define the structure of the 
  variance-covariance matrix as specified as \eqn{Var(\mathbf{b})}. Typically, 
  we denote the variance-covariance matrix as \eqn{\Sigma}.
  
  Proper details can be found in \code{vignette("lmer")}.
  Our main focus is the restriction on \eqn{\Sigma}. Let \eqn{q} represent 
  the number of columns of the random-effects model matrix \eqn{Z}. 
  \eqn{\Sigma} can be expressed in terms of a relative cofactor 
  \eqn{\Lambda_{\theta}} which is a \eqn{q \times q} block diagonal matrix that 
  depends on the variance-component parameter \eqn{\theta}:
  \deqn{
  \mathbf{\Sigma}_{\mathbf{\theta}} = \sigma^{2} \Lambda_{\theta} 
  \Lambda_{\theta}^{T}.
  }
  Where \eqn{\sigma} is a scale parameter of the variance of a linear mixed model.
  
  If the within-group variance-covariance structure is unspecified, the default 
  is unstructured (\code{Covariance.us}), meaning we only ensure that the 
  variance-covariance matrix \eqn{\Sigma} is positive semidefinite. 
  
}
\section{Objects from the Class}{
Available standard classes:
  \describe{
    \item{\code{Covariance.us}}{Unstructured (general positive definite). This is the 
    default version.}
    \item{\code{Covariance.diag}}{Diagonal; only the diagonal entries are 
    nonzero, indicating no covariances between variables.}
    \item{\code{Covariance.cs}}{Compound symmetry.}
    \item{\code{Covariance.ar1}}{Autoregressive process of order 1.}
  }
Besides \code{Covariance.us}, the remaining classes contain a logical slot 
called \code{hom}, described in the \emph{Slots} section.
}
\section{Slots}{
  \describe{
    \item{\code{nc}}{An integer value giving the number of columns (or components).}
    \item{\code{par}}{A double vector that stores the covariance
      parameters for the model. (See \code{\link{lmer}} and
      \code{vignette("covariance_structures", package = "lme4")} for
      more details on parameterization.)
    }
    \item{\code{hom}}{A logical value which represents whether the variance-covariance 
      matrix has a \emph{homogeneous} structure - that is, whether the variances 
      are equal across groups.}
  } % describe
} % slots

\section{S4 Methods}{
\code{vcomp} contains parameters of the variance component of the 
covariance matrix (typically standard deviations). \code{ccomp} contains 
parameters of the correlation component of the covariance matrix. That is,
\itemize{
  \item \code{Covariance.us}: the entries of the lower triangle of the 
  correlation matrix in column-major order.
  \item \code{diag}: should be empty; we only care about the standard deviations.
  \item \code{cs}, \code{ar1}: \eqn{\rho}
  }
Methods that obtain names (i.e., \code{getParNames}, \code{getThetaNames}, 
and \code{getVCNames}), \code{cnm} represents the column (of the model matrix) 
name, and \code{gnm} represents the group name.

For methods that involve \code{getProf} (i.e., \code{getProfPar}, 
\code{setProfPar}, \code{getProfLower}, \code{getProfUpper}), the user may 
set the argument \code{profscale = "varcor"}, which transforms the variance 
components and correlation components into the corresponding variance-covariance 
matrix. Furthermore, the user may scale the standard deviations by a factor
of the \code{sc} argument.

\describe{
    \item{\code{getPar}:}{The vector of parameters that the optimizer cares 
    about; contains \code{c(vcomp, ccomp)}.}
    \item{\code{getParLength}:}{Obtains the length of \code{par}.}
    \item{\code{getParNames}:}{Obtains the names of \code{par} in terms of 
    \code{cnm} and \code{gnm}.}
    \item{\code{setPar}:}{Sets the value of par.}
    \item{\code{getTheta}:}{Obtains the variance-component parameter. These are 
    also the column-wise elements of \code{Lambda}, or the row-wise elements of
    \code{t(Lambda)}.}
    \item{\code{getThetaLength}:}{Obtains the length of the variance-component 
    parameter \code{theta}.}
    \item{\code{getThetaNames}:}{Obtains the names of \code{theta} in terms of 
    \code{cnm} and \code{gnm}.}
    \item{\code{getThetaIndex}:}{A mapping of the elements of \code{theta} into 
    \code{Lambda} in column-major order.}
    \item{\code{getThetaIndexLength}:}{Obtains the length of \code{getThetaIndex};
    which corresponds to the number of non-zero entries of \code{Lambda}.}
    \item{\code{setTheta}:}{Sets the value of the variance-component parameter.}
    \item{\code{getLower}:}{Retrieves the lower bound for \code{vcomp} and 
    \code{ccomp}, in that order.}
    \item{\code{getUpper}:}{Retrieves the upper bound for \code{vcomp} and 
    \code{ccomp}, in that order.}
    \item{\code{getLambda}:}{The relative covariance factor.}
    \item{\code{getLambdat.dp}:}{
    Returns a vector of length \code{nc} where each element contains the count 
    of nonzero entries in the corresponding column. Together with 
    \code{Lambdat.i}, defines the compressed sparse column representation 
    of \code{t(Lambda)}.
    }
    \item{\code{getLambdat.i}:}{
    Returns a vector of \code{0}-based row indices (\code{length <= nc*(n+1)/2}) 
    for nonzero entries in \code{t(Lambda)}, stored in column-major order. 
    Together with \code{Lambdat.dp}, defines the compressed sparse column 
    representation of \code{t(Lambda)}.
    }
    \item{\code{getVC}:}{Retrieves both \code{vcomp} and \code{ccomp}.}
    \item{\code{getVCNames}:}{Obtains the names of the values associated with
    \code{vcomp} and \code{ccomp} in terms of \code{cnm} and \code{gnm}.}
    \item{\code{setVC}:}{Sets the values for VC, which is composed of \code{vcomp}
    and \code{ccomp}.}
    \item{\code{getProfPar}:}{Converting the values from \code{vcomp}
    and \code{ccomp} (or \code{getVC}) to extract parameters from the object 
    and translates them to a different parameterization 
    suitable for profiling. If \code{profscale = "varcov"} then the 
    variance-covariance matrix is returned.}
    \item{\code{setProfPar}:}{Sets the parameters to use for profiling.}
    \item{\code{getProfLower}:}{Obtains the lower bounds of the values obtained
    by \code{getProfPar} for profiling.}
    \item{\code{getProfUpper}:}{Obtains the upper bounds of the values obtained
    by \code{getProfPar} for profiling.}
  }
}


\examples{
\dontshow{
.nms <- c("getPar", "getTheta", "getLambda", "getVC", "getReCovs")
.fns <- mget(.nms, envir = getNamespace("lme4"), mode = "function")
list2env(.fns, envir = environment())
}
## Unstructured
fm1.us <- lmer(Reaction ~ Days + us(Days | Subject), sleepstudy)
## Diagonal
fm1.diag <- lmer(Reaction ~ Days + diag(Days | Subject), sleepstudy)
fm1.diag.hom <- lmer(Reaction ~ Days + diag(Days | Subject, hom = TRUE), 
  sleepstudy)
## Compound symmetry
fm1.cs <- lmer(Reaction ~ Days + cs(Days | Subject), sleepstudy)
fm1.cs.hom <- lmer(Reaction ~ Days + cs(Days | Subject, hom = TRUE), 
  sleepstudy)
## Auto-regressive order 1
sleepstudy$Daysf <- factor(sleepstudy$Days, ordered = TRUE)
fm1.ar1 <- lmer(Reaction ~ Daysf + ar1(0 + Daysf | Subject, hom = TRUE), 
                sleepstudy, REML = FALSE)
\dontshow{
rm(list = c(.nms, ".nms", ".fns"))
}
}
\keyword{classes}
\keyword{internal}

\name{residuals.merMod}
\alias{residuals.glmResp}
\alias{residuals.lmResp}
\alias{residuals.merMod}
\title{residuals of merMod objects}
\usage{
  \method{residuals}{merMod} (object,
    type = if (isGLMM(object)) "deviance" else "response",
    scaled = FALSE, ...)

  \method{residuals}{lmResp} (object,
    type = c("working", "response", "deviance", "pearson", "partial"),
    ...)

  \method{residuals}{glmResp} (object,
    type = c("deviance", "pearson", "working", "response", "partial"),
    ...)
}
\arguments{
  \item{object}{a fitted [g]lmer (\code{merMod}) object}

  \item{type}{type of residuals}

  \item{scaled}{scale residuals by residual standard
  deviation (=scale parameter)?}

  \item{\dots}{additional arguments (ignored: for method
  compatibility)}
}
\description{
  residuals of merMod objects
}
\details{
  \itemize{ \item The default residual type varies between
  \code{lmerMod} and \code{glmerMod} objects: they try to
  mimic \code{\link{residuals.lm}} and
  \code{\link{residuals.glm}} respectively. In particular,
  the default \code{type} is \code{"response"}, i.e.
  (observed-fitted) for \code{lmerMod} objects vs.
  \code{"deviance"} for \code{glmerMod} objects.
  \code{type="partial"} is not yet implemented for either
  type. \item Note that the meaning of \code{"pearson"}
  residuals differs between \code{\link{residuals.lm}} and
  \code{\link[nlme]{residuals.lme}}.  The former returns values
  scaled by the square root of user-specified weights (if
  any), but \emph{not} by the residual standard deviation,
  while the latter returns values scaled by the estimated
  standard deviation (which will include the effects of any
  variance structure specified in the \code{weights}
  argument).  To replicate \code{lme} behaviour, use
  \code{type="pearson"}, \code{scaled=TRUE}. }
}

\name{simulate.merMod}
\title{Simulate Responses From \code{\linkS4class{merMod}} Object}
\alias{simulate.merMod}
\alias{.simulateFun}
\description{
  Simulate responses from a \code{"merMod"} fitted model object, i.e.,
  from the model represented by it.
}
\usage{
\method{simulate}{merMod}(object, nsim = 1, seed = NULL,
	 use.u = FALSE, re.form = NA,
	 newdata=NULL, newparams=NULL, family=NULL, cluster.rand=rnorm,
	 allow.new.levels = FALSE, na.action = na.pass, \dots)

.simulateFun(object, nsim = 1, seed = NULL, use.u = FALSE,
             re.form = NA,
             newdata=NULL, newparams=NULL,
             formula=NULL, family=NULL, 
             cluster.rand=rnorm,
             weights=NULL, offset=NULL,
             allow.new.levels = FALSE, na.action = na.pass,
             cond.sim = TRUE, \dots)
}
\arguments{
  \item{object}{(for \code{simulate.merMod}) a fitted model object or
    (for \code{simulate.formula}) a (one-sided) mixed model formula, as
    described for \code{\link{lmer}}.}
  \item{nsim}{positive integer scalar - the number of responses to simulate.}
  \item{seed}{an optional seed to be used in \code{\link{set.seed}}
    immediately before the simulation so as to generate a reproducible sample.}
  \item{use.u}{(logical) if \code{TRUE}, generate a simulation
    conditional on the current random-effects estimates; if \code{FALSE}
    generate new Normally distributed random-effects values. (Redundant
    with \code{re.form}, which is preferred: \code{TRUE} corresponds to
    \code{re.form = NULL} (condition on all random effects), while
    \code{FALSE} corresponds to \code{re.form = ~0} (condition on none
    of the random effects).)}
  \item{re.form}{formula for random effects to condition on.  If
    \code{NULL}, condition on all random effects; if \code{NA} or \code{~0},
    condition on no random effects.  See Details.}
  \item{newdata}{data frame for which to evaluate predictions.}
  \item{newparams}{new parameters to use in evaluating predictions,
    specified as in the \code{start} parameter for \code{\link{lmer}} or
    \code{\link{glmer}} -- a list with components \code{theta} and
    \code{beta} and (for LMMs or GLMMs that estimate a scale parameter)
    \code{sigma}}
  \item{formula}{a (one-sided) mixed model formula, as described for
    \code{\link{lmer}}.}
  \item{family}{a GLM family, as in \code{\link{glmer}}.}
  \item{cluster.rand}{Function that generates \emph{standardized} random cluster effects.  The function takes one
    argument, the number of random values to generate.}
  \item{weights}{prior weights, as in \code{\link{lmer}} or
    \code{\link{glmer}}.}
  \item{offset}{offset, as in \code{\link{glmer}}.}
  \item{allow.new.levels}{(logical) if FALSE (default), then any new
    levels (or \code{NA} values) detected in \code{newdata} will trigger an
    error; if TRUE, then the prediction will use the unconditional
    (population-level) values for data with previously unobserved levels
    (or \code{NA}s).}
  \item{na.action}{what to do with \code{NA} values in new data: see
    \code{\link{na.fail}}}
  \item{cond.sim}{(experimental) simulate the conditional
    distribution?  if \code{FALSE}, simulate only random effects; do not
    simulate from the conditional distribution, rather return the
    predicted group-level values}
  \item{\dots}{optional additional arguments (none are used in
    \code{.simulateFormula})}
}
\seealso{
  \code{\link{bootMer}} for \dQuote{simulestimate}, i.e., where each
  simulation is followed by refitting the model.
}
\details{
  \itemize{
    \item{ordinarily \code{simulate} is used to generate new
      values from an existing, fitted model (\code{merMod} object):
      however, if \code{formula}, \code{newdata}, and \code{newparams} are
      specified, \code{simulate} generates the appropriate model
      structure to simulate from. \code{formula} must be a
      \emph{one-sided} formula (i.e. with an empty left-hand side);
      in general, if \code{f} is a two-sided
      formula, \code{f[-2]} can be used to drop the LHS.}

    \item{The \code{re.form} argument allows the user to specify how the
      random effects are incorporated in the simulation.  All of the
      random effects terms included in \code{re.form} will be
      \emph{conditioned on} - that is, the conditional modes of those
      random effects will be included in the deterministic part of the
      simulation. (If new levels are used (and \code{allow.new.levels}
      is \code{TRUE}), the conditional modes for these levels will be
      set to the population mode, i.e. values of zero will be used for
      the random effects.)  Conversely, the random effect terms that are
      \emph{not} included in \code{re.form} will be \emph{simulated
      from} - that is, new values will be chosen for each group based on
      the estimated random-effects variances.

      The default behaviour (using \code{re.form=NA}) is to condition on
      none of the random effects, simulating new values for all of the
      random effects.

    }
    \item{For Gaussian fits, \code{sigma} specifies the residual
      standard deviation; for Gamma fits, it specifies the square 
      root of the dispersion parameter. The shape
      parameter can be calculated by \code{1/sigma(fitted_model)^2}.  
      For negative binomial fits, the overdispersion parameter is 
      specified via the family, e.g. 
      \code{simulate(..., family=negative.binomial(theta=1.5))}.
    }
    \item{For binomial models, \code{simulate.formula} looks for the
      binomial size first in the \code{weights} argument (if it's supplied),
      second from the left-hand side of the formula (if the formula has been
      specified in success/failure form), and defaults to 1 if neither of
      those have been supplied.
      Simulated responses will be given as proportions, unless the supplied
      formula has a matrix-valued left-hand side, in which case they will be
      given in matrix form. If a left-hand side is given, variables in that
      expression must be available in \code{newdata}.
    }
    \item{For negative binomial models, use the \code{negative.binomial}
      family (from the \CRANpkg{MASS} package)
      and specify the overdispersion parameter via the
      \code{theta} (sic) parameter of the family function, e.g.
      \code{simulate(...,family=negative.binomial(theta=1))} to simulate
      from a geometric distribution (negative binomial with
      overdispersion parameter 1).
    }
    \item{\code{cluster.rand} allows one to test effects of departures from normality for the distribution of cluster random effects, for example by using a heavy-tailed distribution or a mixture distribution.  One can also use a truncated normal to investigate true or false flagging rates; in that case the generated effects will not have a mean of 0 and standard deviation of 1; they are still considered standardized in that the simulation will multiply them by the estimated standard deviation of the cluster random effects.}
}
}
\examples{
## test whether fitted models are consistent with the
##  observed number of zeros in CBPP data set:
gm1 <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
             data = cbpp, family = binomial)
gg <- simulate(gm1,1000)
zeros <- sapply(gg,function(x) sum(x[,"incidence"]==0))
plot(table(zeros))
abline(v=sum(cbpp$incidence==0),col=2)
##
## simulate from a non-fitted model; in this case we are just
## replicating the previous model, but starting from scratch
params <- list(theta=0.5,beta=c(2,-1,-2,-3))
simdat <- with(cbpp,expand.grid(herd=levels(herd),period=factor(1:4)))
simdat$size <- 15
simdat$incidence <- sample(0:1,size=nrow(simdat),replace=TRUE)
form <- formula(gm1)[-2]  ## RHS of equation only
simulate(form,newdata=simdat,family=binomial,
    newparams=params)
## simulate from negative binomial distribution instead
simulate(form,newdata=simdat,family=negative.binomial(theta=2.5),
    newparams=params)
}

\name{mkRespMod}
\alias{mkRespMod}
\title{Create an lmerResp, glmResp or nlsResp instance}
\usage{
  mkRespMod(fr, REML = NULL, family = NULL, nlenv = NULL,
    nlmod = NULL, ...)
}
\arguments{
  \item{fr}{a model frame}

  \item{REML}{logical scalar, value of REML for an lmerResp
  instance}

  \item{family}{the optional glm family (glmResp only)}

  \item{nlenv}{the nonlinear model evaluation environment
  (nlsResp only)}

  \item{nlmod}{the nonlinear model function (nlsResp only)}

  \item{...}{where to look for response information if
  \code{fr} is missing.  Can contain a model response,
  \code{y}, offset, \code{offset}, and weights,
  \code{weights}.}
}
\value{
  an lmerResp or glmResp or nlsResp instance
}
\description{
  Create an lmerResp, glmResp or nlsResp instance
}
\seealso{
  Other utilities: \code{\link[reformulas]{findbars}},
  \code{\link[reformulas]{mkReTrms}}, \code{\link{nlformula}},
  \code{\link[reformulas]{nobars}}, \code{\link[reformulas]{subbars}}
}

\docType{data}
\name{cbpp}
\alias{cbpp}
\alias{cbpp2}
\title{Contagious bovine pleuropneumonia}

\format{\code{cbpp} is a data frame with 56 observations on the following 4 
  variables.
  \describe{
    \item{\code{herd}}{A factor identifying the herd (1 to 15).}
    \item{\code{incidence}}{The number of new serological cases for a
      given herd and time period.}
    \item{\code{size}}{A numeric vector describing herd size at the
      beginning of a given time period.}
    \item{\code{period}}{A factor with levels \code{1} to \code{4}.}
  }
  The extended version, \code{cbpp2}, has the additional variables:
  \describe{
    \item{\code{herd}}{A factor identifying the herd (1 to 15).}
    \item{\code{treatment}}{A factor referring to the control measure used
      to manage CBPP.
      \itemize{
        \item{\code{Complete} = complete isolation or antibiotic treatment,}
        \item{\code{Partial/null} = partial/null isolation and no antibiotic
          treatment,}
        \item{\code{Unknown} = strategy remained.}
      }
    }
    \item{\code{avg_size}}{The average number of animals housed in a della 
      (a temporary paddock used for holding cattle on the farm).}
  }}
  
\description{
  Contagious bovine pleuropneumonia (CBPP) is a major
  disease of cattle in Africa, caused by a mycoplasma.
  This dataset describes the serological incidence of CBPP
  in zebu cattle during a follow-up survey implemented in
  15 commercial herds located in the Boji district of
  Ethiopia.  The goal of the survey was to study the
  within-herd spread of CBPP in newly infected herds. Blood
  samples were quarterly collected from all animals of
  these herds to determine their CBPP status.  These data
  were used to compute the serological incidence of CBPP
  (new cases occurring during a given time period).  Some
  data are missing (lost to follow-up).
  
  There are two datasets, although their precise origins are unclear.
  The dataset used in the classical \code{lme4} examples, \code{cbpp},
  matches the file maintained by the package authors. The extended
  dataset, \code{cbpp2}, corresponds to Table 1 of
  \insertCite{lesnoff2004within}{lme4}, with the exception of herd 6,
  which is a known typographical error in the original paper.
}
\details{
  Serological status was determined using a competitive
  enzyme-linked immuno-sorbent assay (cELISA).
}
\source{
  \insertCite{lesnoff2004within}{lme4}
}
\references{
  \insertRef{lesnoff2004within}{lme4}
}
\examples{
## response as a matrix
(m1 <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
             family = binomial, data = cbpp))
## response as a vector of probabilities and usage of argument "weights"
m1p <- glmer(incidence / size ~ period + (1 | herd), weights = size,
             family = binomial, data = cbpp)
## Confirm that these are equivalent:
stopifnot(all.equal(fixef(m1), fixef(m1p), tolerance = 1e-5),
          all.equal(ranef(m1), ranef(m1p), tolerance = 1e-5))
%% more extensive variations of the above --> ../tests/glmer-1.R

## GLMM with individual-level variability (accounting for overdispersion)
cbpp$obs <- 1:nrow(cbpp)
(m2 <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd) +  (1|obs),
              family = binomial, data = cbpp))
              
## Fitting the model for cbpp2
gm1 <- glmer(incidence/size ~ period + treatment + avg_size + (1 | herd),
             family = binomial,
             data = cbpp2, weights = size,
             control = glmerControl(optimizer="bobyqa"))
## Adding an observation-level random effect
cbpp2 <- transform(cbpp2,obs=factor(seq(nrow(cbpp2))))
## Herd and observation-level REs (below causes singular fit issues)
gm2 <- update(gm1,.~.+(1|obs)) 
## observation-level REs only (no singular fit issue)
gm3 <- update(gm1,.~.-(1|herd)+(1|obs)) 
}
\keyword{datasets}

\name{deviance}
\alias{deviance}
\alias{deviance.merMod}

\title{Deviance of Fitted Mixed-Effect Models}

\description{
  Calculates the deviance of a model of class merMod. Some important details
  regarding the calculation of the deviance is found below.
}

\usage{
\S3method{deviance}{merMod}(object, REML = NULL, \dots)
}

\arguments{
  \item{object}{an \R object of class \code{\linkS4class{merMod}}, i.e.,
    as resulting from \code{\link{lmer}()}, or \code{\link{glmer}()},
    etc.}
   \item{REML}{Logical. If \code{TRUE}, return the restricted log-likelihood
    rather than the log-likelihood.  If \code{NULL} (the default),
    set \code{REML} to \code{isREML(object)} (see \code{\link{isREML}}).}
   \item{\dots}{Optional additional arguments.}
}


\section{Deviance and log-likelihood of GLMMs}{

  One must be careful when defining the deviance of a GLM.  For example,
  should the deviance be defined as minus twice the log-likelihood or
  does it involve subtracting the deviance for a saturated model?  To
  distinguish these two possibilities we refer to absolute deviance
  (minus twice the log-likelihood) and relative deviance (relative to a
  saturated model, e.g. Section 2.3.1 in McCullagh and Nelder 1989).

  With GLMMs however, there is an additional complication involving the
  distinction between the likelihood and the conditional likelihood.
  The latter is the likelihood obtained by conditioning on the estimates
  of the conditional modes of the spherical random effects coefficients,
  whereas the likelihood itself (i.e. the unconditional likelihood)
  involves integrating out these coefficients.  The following table
  summarizes how to extract the various types of deviance for a
  \code{glmerMod} object:

  \tabular{lrr}{
             \tab conditional              \tab unconditional \cr
    relative \tab \code{deviance(object)}  \tab NA in \code{lme4} \cr
    absolute \tab \code{object@resp$aic()} \tab \code{-2*logLik(object)}
  }

  This table requires two caveats:

  \itemize{
    \item If the link function involves a scale parameter
    (e.g. \code{Gamma}) then \code{object@resp$aic() - 2 * getME(object,
    "devcomp")$dims["useSc"]} is required for the absolute-conditional
    case.
    \item If adaptive Gauss-Hermite quadrature is used, then
    \code{logLik(object)} is currently only proportional to the
    absolute-unconditional log-likelihood.
  }
  
  Note that \code{summary()} reports the unconditional absolute deviance,
  \code{-2*logLik(object)}.

  For more information about this topic see the \code{misc/logLikGLMM}
  directory in the package source.
}

\name{glmer.nb}
\title{Fitting Negative Binomial GLMMs}
\alias{glmer.nb}
\alias{negative.binomial}% re-exported, needed e.g. in update()
\concept{GLMM}
\description{
  Fits a generalized linear mixed-effects model (GLMM) for the negative
  binomial family, building on \code{\link{glmer}}, and initializing via
  \code{\link[MASS]{theta.ml}} from \pkg{MASS}.
}
\usage{
glmer.nb(\dots, interval = log(th) + c(-3, 3),
         tol = 5e-5, verbose = FALSE, nb.control = NULL,
         initCtrl = list(limit = 20, eps = 2*tol, trace = verbose,
                         theta = NULL))
}
\arguments{
  \item{\dots}{arguments as for \code{glmer(.)} such as \code{formula},
    \code{data}, \code{control}, etc, but \emph{not} \code{family}!}
  \item{interval}{interval in which to start the optimization.  The
    default is symmetric on log scale around the initially estimated theta.}
  \item{tol}{tolerance for the optimization via \code{\link{optimize}}.}
  \item{verbose}{\code{\link{logical}} indicating how much
    progress information should be printed during the optimization.  Use
    \code{verbose = 2} (or larger) to enable \code{verbose=TRUE} in the
    \code{\link{glmer}()} calls.}
  \item{nb.control}{optional \code{\link{list}}, like the output of \code{\link{glmerControl}()},
    used in \code{\link{refit}(*, control = control.nb)} during the
    optimization (\code{control}, if included in \code{\dots},
    will be used in the initial-stage \code{glmer(...,family=poisson)}
    fit, and passed on to the later optimization stages as well)}
    \item{initCtrl}{(\emph{\bold{experimental}, do not rely on this}:) a
    \code{\link{list}} with named components as in the default, passed to
    \code{\link[MASS]{theta.ml}} (package \CRANpkg{MASS}) for the initial
    value of the negative binomial parameter \code{theta}.
    May also include a \code{theta} component, in which case the
    initial estimation step is skipped}
}
\value{
  An object of class \code{glmerMod}, for which many
  methods are available (e.g. \code{methods(class="glmerMod")}), see
  \code{\link{glmer}}.
}
\note{For historical reasons, the shape parameter of the negative
  binomial and the random effects parameters in our (G)LMM models are
  both called \code{theta} (\eqn{\theta}), but are unrelated here.

  The negative binomial \eqn{\theta} can be extracted from a fit
  \code{g <- glmer.nb()} by \code{\link{getME}(g, "glmer.nb.theta")}.

  Parts of \code{glmer.nb()} are still experimental and methods are
  still missing or suboptimal.  In particular, there is no inference
  available for the dispersion parameter \eqn{\theta}, yet.

  To fit a negative binomial model with \emph{known} overdispersion
  parameter (e.g. as part of a model comparison exercise, use
  \code{glmer} with the \code{\link[MASS]{negative.binomial}} family from the
  \code{MASS} package, e.g.
  \code{glmer(...,family=MASS::negative.binomial(theta=1.75))}.

}
\seealso{
  \code{\link{glmer}}; from package \CRANpkg{MASS},
  \code{\link[MASS]{negative.binomial}} (which we re-export currently) and
  \code{\link[MASS]{theta.ml}}, the latter for initialization of
  optimization.

  The \sQuote{Details} of \code{\link{pnbinom}} for the definition of
  the negative binomial distribution.
}
\examples{
set.seed(101)
dd <- expand.grid(f1 = factor(1:3),
                  f2 = LETTERS[1:2], g=1:9, rep=1:15,
          KEEP.OUT.ATTRS=FALSE)
summary(mu <- 5*(-4 + with(dd, as.integer(f1) + 4*as.numeric(f2))))
dd$y <- rnbinom(nrow(dd), mu = mu, size = 0.5)
str(dd)
require("MASS")## and use its glm.nb() - as indeed we have zero random effect:
\dontrun{
m.glm <- glm.nb(y ~ f1*f2, data=dd, trace=TRUE)
summary(m.glm)
m.nb <- glmer.nb(y ~ f1*f2 + (1|g), data=dd, verbose=TRUE)
m.nb
## The neg.binomial theta parameter:
getME(m.nb, "glmer.nb.theta")
LL <- logLik(m.nb)
## mixed model has 1 additional parameter (RE variance)
stopifnot(attr(LL,"df")==attr(logLik(m.glm),"df")+1)
plot(m.nb, resid(.) ~ g)# works, as long as data 'dd' is found
}
}
\keyword{models}

\name{plot.merMod}
\title{Diagnostic Plots for 'merMod' Fits}
\alias{plot.merMod}
\alias{qqmath.merMod}
\alias{diagnostics}
\alias{diagnostic.plots}
\alias{diagnosticPlots}
\alias{diagnostic_plots}
\alias{qqplot}
\usage{
\method{plot}{merMod}(x,
     form = resid(., type = "pearson") ~ fitted(.), abline,
     id = NULL, idLabels = NULL, grid, \dots)
\method{qqmath}{merMod}(x, data = NULL, id = NULL, idLabels = NULL, \dots)
}
\arguments{
  \item{x}{a fitted [ng]lmer model.}
  \item{form}{an optional formula specifying the desired
  type of plot. Any variable present in the original data
  frame used to obtain \code{x} can be referenced. In
  addition, \code{x} itself can be referenced in the
  formula using the symbol \code{"."}. Conditional
  expressions on the right of a \code{|} operator can be
  used to define separate panels in a lattice display.
  Default is \code{resid(., type = "pearson") ~ fitted(.)},
  corresponding to a plot of the standardized residuals
  versus fitted values.}

  \item{abline}{an optional numeric value, or numeric
  vector of length two. If given as a single value, a
  horizontal line will be added to the plot at that
  coordinate; else, if given as a vector, its values are
  used as the intercept and slope for a line added to the
  plot. If missing, no lines are added to the plot.}

  \item{id}{an optional numeric value, or one-sided
  formula. If given as a value, it is used as a
  significance level for a two-sided outlier test for the
  standardized, or normalized residuals. Observations with
  absolute standardized (normalized) residuals greater than
  the \eqn{1-value/2} quantile of the standard normal
  distribution are identified in the plot using
  \code{idLabels}. If given as a one-sided formula, its
  right hand side must evaluate to a logical, integer, or
  character vector which is used to identify observations
  in the plot. If missing, no observations are identified.}

  \item{idLabels}{an optional vector, or one-sided formula.
  If given as a vector, it is converted to character and
  used to label the observations identified according to
  \code{id}. If given as a vector, it is converted to
  character and used to label the observations identified
  according to \code{id}. If given as a one-sided formula,
  its right hand side must evaluate to a vector which is
  converted to character and used to label the identified
  observations. Default is the interaction of all the
  grouping variables in the data frame.  The special
  formula \code{idLabels=~.obs} will label the observations
  according to observation number.}

  \item{data}{ignored: required for S3 method compatibility.}
  \item{grid}{an optional logical value indicating whether
  a grid should be added to plot. Default depends on the
  type of lattice plot used: if \code{xyplot} defaults to
  \code{TRUE}, else defaults to \code{FALSE}.}

  \item{\dots}{optional arguments passed to the lattice
  plot function.}
}
\description{
  diagnostic plots for merMod fits.  Some of these plots may be supplemented or 
  superseded by functions from other packages.  See the \emph{See Also} section 
  below for recommended alternatives.
}
\details{
  Diagnostic plots for the linear mixed-effects fit are
  obtained. The \code{form} argument gives considerable
  flexibility in the type of plot specification. A
  conditioning expression (on the right side of a \code{|}
  operator) always implies that different panels are used
  for each level of the conditioning factor, according to a
  lattice display. If \code{form} is a one-sided formula,
  histograms of the variable on the right hand side of the
  formula, before a \code{|} operator, are displayed (the
  lattice function \code{histogram} is used). If
  \code{form} is two-sided and both its left and right hand
  side variables are numeric, scatter plots are displayed
  (the lattice function \code{xyplot} is used). Finally, if
  \code{form} is two-sided and its left had side variable
  is a factor, box-plots of the right hand side variable by
  the levels of the left hand side variable are displayed
  (the lattice function \code{bwplot} is used).

  \code{qqmath} produces a Q-Q plot of the residuals
  (see \code{\link{qqmath.ranef.mer}} for Q-Q plots of the
  conditional mode values).
}
\seealso{
  \itemize{
    \item \code{\link[performance]{check_model}} in the \CRANpkg{performance} package.
    \item \CRANpkg{DHARMa} package, which evaluates model diagnostics through 
    simulating residuals (see: \code{\link[DHARMa]{simulateResiduals}}).
    \item \code{\link[lattice]{qqmath}} in the \CRANpkg{lattice} package.
    \item \code{\link[car]{influencePlot}} in the \CRANpkg{car} package.
  }
}
\author{
  original version in \CRANpkg{nlme} package by Jose Pinheiro
  and Douglas Bates.
}
\examples{
data(Orthodont,package="nlme")
fm1 <- lmer(distance ~ age + (age|Subject), data=Orthodont)
## standardized residuals versus fitted values by gender
plot(fm1, resid(., scaled=TRUE) ~ fitted(.) | Sex, abline = 0)
## box-plots of residuals by Subject
plot(fm1, Subject ~ resid(., scaled=TRUE))
## observed versus fitted values by Subject
plot(fm1, distance ~ fitted(.) | Subject, abline = c(0,1))
## residuals by age, separated by Subject
plot(fm1, resid(., scaled=TRUE) ~ age | Sex, abline = 0)
## scale-location plot, with red smoothed line
scale_loc_plot <- function(m, line.col = "red", line.lty = 1,
                           line.lwd = 2) {
  plot(fm1, sqrt(abs(resid(.))) ~ fitted(.),
       type = c("p", "smooth"),
       par.settings = list(plot.line =
                             list(alpha=1, col = line.col,
                                  lty = line.lty, lwd = line.lwd)))
}
scale_loc_plot(fm1)
## Q-Q plot
lattice::qqmath(fm1, id=0.05)
ggp.there <- "package:ggplot2" \%in\% search()
if (ggp.there || require("ggplot2")) {
    ## we can create the same plots using ggplot2 and the fortify() function
    fm1F <- fortify.merMod(fm1)
    ggplot(fm1F, aes(.fitted, .resid)) + geom_point(colour="blue") +
           facet_grid(. ~ Sex) + geom_hline(yintercept=0)
    ## note: Subjects are ordered by mean distance
    ggplot(fm1F, aes(Subject,.resid)) + geom_boxplot() + coord_flip()
    ggplot(fm1F, aes(.fitted,distance)) + geom_point(colour="blue") +
        facet_wrap(~Subject) +geom_abline(intercept=0,slope=1)
    ggplot(fm1F, aes(age,.resid)) + geom_point(colour="blue") + facet_grid(.~Sex) +
        geom_hline(yintercept=0)+ geom_line(aes(group=Subject),alpha=0.4) +
        geom_smooth(method="loess")
    ## (warnings about loess are due to having only 4 unique x values)
    if(!ggp.there) detach("package:ggplot2")
}
}

\name{reformulas_imports}
\title{(deprecated) formula-processing functions}
\description{
  Formula-processing functions that have moved to \code{reformulas}
}
\alias{subbars}
\alias{nobars}
\alias{findbars}
\alias{mkReTrms}
\alias{isNested}
\alias{expandDoubleVerts}
\alias{reOnly}
\usage{
subbars(...)

nobars(...)

findbars(...)

mkReTrms(...)

isNested(...)

expandDoubleVerts(...)

reOnly(...)
}

\arguments{
  \item{...}{arguments to be passed through to functions in the
    \code{reformulas} package}
}
\details{
  All of these functions are now located in the \code{reformulas}
  package. The versions here are 'shims' that throw a warning and
  call the \code{reformulas} version. (\code{reOnly} will not be
  fully fixed in \code{reformulas} until version 0.4.5 is on CRAN.)
}

\name{lmResp}
\alias{glmResp}
\alias{lmerResp}
\alias{lmResp}
\alias{nlsResp}
\title{Generator objects for the response classes}
\usage{
  lmResp(...)
}
\arguments{
  \item{...}{List of arguments (see Note).}
}
\description{
  The generator objects for the
  \code{\linkS4class{lmResp}},
  \code{\linkS4class{lmerResp}},
  \code{\linkS4class{glmResp}} and
  \code{\linkS4class{nlsResp}} reference classes. Such
  objects are primarily used through their \code{new}
  methods.
}
\note{
  Arguments to the \code{new} methods must be named
  arguments.
  \describe{
    \item{y}{ the numeric response vector}
    \item{family}{ a \code{\link{family}} object}
    \item{nlmod}{ the nonlinear model function}
    \item{nlenv}{ an environment holding data objects for evaluation of
      \code{nlmod}} 
    \item{pnames}{ a character vector of parameter names}
    \item{gam}{ a numeric vector - the initial linear predictor}
  }
} % note about named arguments
\section{Methods}{
  \describe{ \item{\code{new(y=y)}:}{Create a new
      \code{\linkS4class{lmResp}} or
      \code{\linkS4class{lmerResp}} object.}
    \item{\code{new(family=family, y=y)}:}{Create a new
      \code{\linkS4class{glmResp}} object.}
    \item{\code{new(y=y, nlmod=nlmod, nlenv=nlenv,
	pnames=pnames, gam=gam)}:}{Create a new
      \code{\linkS4class{nlsResp}} object.} }
} % Methods
\seealso{
  \code{\linkS4class{lmResp}},
  \code{\linkS4class{lmerResp}},
  \code{\linkS4class{glmResp}},
  \code{\linkS4class{nlsResp}}
}
\keyword{classes}

\name{mkMerMod}
\title{Create a 'merMod' Object}
\alias{mkMerMod}
\description{
  Create an object of (a subclass of) class \code{\linkS4class{merMod}}
  from the environment of the objective function and the value returned
  by the optimizer.
}
\usage{
  mkMerMod(rho, opt, reTrms, fr, mc, lme4conv = NULL)
}
\arguments{
  \item{rho}{the environment of the objective function}
  \item{opt}{the optimization result returned by the optimizer
    (a \code{\link{list}}: see \code{\link{lmerControl}} for required elements)}
  \item{reTrms}{random effects structure from the calling function
       (see \code{\link[reformulas]{mkReTrms}} for required elements)}
  \item{fr}{model frame (see \code{\link{model.frame}})}
  \item{mc}{matched call from the calling function}
  \item{lme4conv}{lme4-specific convergence information
    (results of \code{checkConv})}
}
\value{
  an object from a class that inherits from \code{\linkS4class{merMod}}.
}
%% FIXME: TODO
%% \examples{
%% ## 1) An "empty"   merMod object :
%% ## 2) A  "lmer()-like" merMod object, using our "modular" approach instead of lmer()
%% }

\name{isNewMerMod}
\alias{isNewMerMod}
\alias{forceNewMerMod}
\alias{anyStructured}
\title{Test if a \sQuote{merMod} Object Has Current Representation}
\description{
  \code{isNewMerMod} tests if a \code{\linkS4class{merMod}} object has
  the latest representation (in other words, if the object could have
  been generated using the current version of \pkg{lme4}).  The
  representation of a \code{merMod} object was last changed in
  \pkg{lme4} version 2.0-0.

  \code{forceNewMerMod} updates a \code{\linkS4class{merMod}} object so
  that it has the latest representation.  It is useful in code that
  might handle objects generated by old versions of \pkg{lme4}.

  \code{anyStructured} tests if a \code{\linkS4class{merMod}} object
  specifies random effects with structured covariance matrices.
}
\usage{
isNewMerMod(object)
forceNewMerMod(object, reference = object)
anyStructured(object)
}
\arguments{
  \item{object, reference}{\code{\linkS4class{merMod}} objects.}
}
\value{
  \code{isNewMerMod} and \code{anyStructured} return \code{TRUE} or
  \code{FALSE}.

  \code{forceNewMerMod} returns the argument if it is \dQuote{current}
  and a modified copy of the argument otherwise.
}
\seealso{Virtual class \linkS4class{Covariance}.}
\examples{
fm1 <- lmer(Reaction ~ Days +   us(Days | Subject), sleepstudy)
fm2 <- lmer(Reaction ~ Days + diag(Days | Subject), sleepstudy)
stopifnot(isNewMerMod(fm1), identical(fm1, forceNewMerMod(fm1)),
          !anyStructured(fm1), anyStructured(fm2))
\dontshow{
.fm1 <- fm1
attr(.fm1, "upper") <- attr(.fm1, "reCovs") <- NULL
stopifnot(!isNewMerMod(.fm1), identical(fm1, forceNewMerMod(.fm1)))
rm(.fm1)
}
}

\name{lme4_testlevel}
\alias{lme4_testlevel}
\title{Detect testing level for lme4 examples and tests}
\description{
  Reads the environment variable \code{LME4_TEST_LEVEL} to determine
  which tests and examples to run
}
\usage{
lme4_testlevel()
}
\value{
  a numeric value: 1 for standard/'light' testing, larger values for
  more testing. Defaults to 1 if the environment variable is not set.
}

\name{checkConv}
\alias{checkConv}
\title{
  Extended Convergence Checking
}
\description{
  Primarily internal code for checking optimization convergence,
  see \link{convergence} for a more detailed discussion.
}
\usage{
checkConv(derivs, coefs, ctrl, lbound, ubound,
          debug = FALSE, nobs = NULL, ndim = NULL)
}

\arguments{
  \item{derivs}{typically the "derivs" attribute of \code{optimizeLmer()}; with
    "gradients" and possibly "Hessian" component}
  \item{coefs}{ current coefficient estimates}
  \item{ctrl}{list of lists, each with \code{action} character strings specifying
    what should happen when a check triggers, and \code{tol} numerical tolerances,
    as is the result of \code{\link{lmerControl}()$checkConv}.}
  \item{lbound, ubound}{vectors of lower and upper bounds \emph{for random-effects parameters only}
    (length is taken to determine number of RE parameters)}
  \item{debug}{enable debugging output, useful if some checks are on 
    "ignore", but would "trigger"}
  \item{nobs}{number of observations (needed to determine whether to do
  convergence checks for large data)}
  \item{ndim}{number of dimensions for the variance-covariance matrix 
  of random effects (needed to determine whether to do the
  gradient/hessian checks for large nonlinear optimization problems)}
}
\value{
  A result list containing
  \item{code }{The return code for the check}
  \item{messages }{A character vector of warnings and messages generated by the check}
}

\seealso{
  \code{\link{convergence}}
}

\name{simulate.formula}
\alias{simulate.formula}
\title{A \code{simulate} Method for \code{formula} objects that dispatches based on the Left-Hand Side}
\description{
This method evaluates the left-hand side (LHS) of the given formula and
dispatches it to an appropriate method based on the result by
setting an nonce class name on the formula.
}
\usage{
\method{simulate}{formula}(object, nsim = 1 , seed = NULL, \dots,
basis, newdata, data)
}
\arguments{
\item{object}{a one- or two-sided \code{\link{formula}}.}
\item{nsim,seed}{number of realisations to simulate and the random
  seed to use; see \code{\link{simulate}}}
\item{...}{additional arguments to methods}
\item{basis}{if given, overrides the LHS of the formula for the
  purposes of dispatching}
\item{newdata,data}{if passed, the \code{object}'s LHS is evaluated in
  this environment; at most one of the two may be passed.}
}
\details{
The dispatching works as follows:

\enumerate{
\item If \code{basis} is not passed, and the formula has an LHS the
   expression on the LHS of the formula in the \code{object} is
   evaluated in the environment \code{newdata} or \code{data} (if given), in
   any case enclosed by the environment of \code{object}. Otherwise,
   \code{basis} is used.
\item The result is set as an attribute \code{".Basis"} on \code{object}. If
   there is no \code{basis} or LHS, it is not set.
\item The class vector of \code{object} has \code{c("formula_lhs_\var{CLASS}",
   "formula_lhs")} prepended to it, where \var{CLASS} is the class
   of the LHS value or \code{basis}. If LHS or \code{basis} has multiple
   classes, they are all prepended; if there is no LHS or \code{basis},
   \code{c("formula_lhs_", "formula_lhs")} is.

\item \code{\link{simulate}} generic is evaluated on the new \code{object}, with all
   arguments passed on, excluding \code{basis}; if \code{newdata} or \code{data}
   are missing, they too are not passed on. The evaluation takes
   place in the parent's environment.
}
A "method" to receive a formula whose LHS evaluates to \var{CLASS}
can therefore be implemented by a function
\code{simulate.formula_lhs_\var{CLASS}()}. This function can expect a
\code{\link{formula}} object, with additional attribute \code{.Basis} giving the
evaluated LHS (so that it does not need to be evaluated again).
}

\name{nlformula}
\alias{nlformula}
\title{Manipulate a Nonlinear Model Formula}
\description{
  Check and manipulate the formula for a nonlinear model, such as
  specified in \code{\link{nlmer}}.
}
\usage{
  nlformula(mc)
}
\arguments{
  \item{mc}{matched call from the calling function, typically
    \code{\link{nlmer}()}.  Should have arguments named
    \describe{
      \item{formula:}{a formula of the form \code{resp ~ nlmod ~ meform}
	where \code{resp} is an expression for the response,
	\code{nlmod} is the nonlinear model expression and
	\code{meform} is the mixed-effects model formula.
	\code{resp} can be omitted when, e.g., optimizing a design.}
      \item{data:}{a data frame in which to evaluate the model function}
      \item{start:}{either a numeric vector containing initial estimates
	for the nonlinear model parameters or a list with components
	\describe{
	  \item{\code{nlpars}:}{the initial estimates of the nonlinear
	    model parameters}
	  \item{\code{theta}:}{the initial estimates of the variance
	    component parameters} 
	}
      }
    }
  }
}
\value{
  a list with components
  \item{"respMod"}{a response module of class \code{"\linkS4class{nlsResp}"}} 
  \item{"frame"}{the model frame, including a terms attribute}
  \item{"X"}{the fixed-effects model matrix}
  \item{"reTrms"}{the random-effects terms object}
}
\details{
  The model formula for a nonlinear mixed-effects model is
  of the form \code{resp ~ nlmod ~ mixed} where
  \code{resp} is an expression (usually just a name) for
  the response, \code{nlmod} is the call to the nonlinear
  model function, and \code{mixed} is the mixed-effects
  formula defining the linear predictor for the parameter
  matrix.  If the formula is to be used for optimizing
  designs, the \code{resp} part can be omitted.
}
\seealso{
  Other utilities: \code{\link[reformulas]{findbars}},
  \code{\link{mkRespMod}}, \code{\link[reformulas]{mkReTrms}},
  \code{\link[reformulas]{nobars}}, \code{\link[reformulas]{subbars}}
}

\name{lmer}
\title{Fit Linear Mixed-Effects Models}
\alias{lmer}
\concept{ LMM }
\description{
  Fit a linear mixed-effects model (LMM) to data, via restricted maximum
  likelihood (REML) or maximum
  likelihood.
}
\usage{
lmer(formula, data = NULL, REML = TRUE, control = lmerControl(),
     start = NULL, verbose = 0L, subset, weights, na.action,
     offset, contrasts = NULL, devFunOnly = FALSE)
}
\arguments{
  \item{formula}{a two-sided linear formula object describing both the
    fixed-effects and random-effects part of the model, with the
    response on the left of a \code{~} operator and the terms, separated
    by \code{+} operators, on the right.  Random-effects terms are
    distinguished by vertical bars (\code{|}) separating expressions
    for design matrices from grouping factors. By default, non-scalar
    random effects (where the design matrix has more than one column,
    e.g. \code{(1+x|f)}) are fitted with \emph{unstructured} (general
    positive semidefinite) covariance matrices.
    \itemize{
      \item Two vertical bars
      (\code{||}) can be used to specify multiple uncorrelated random
      effects for the same grouping variable. %----------
      With default settings, the \code{||}-syntax \emph{works
	only for design matrices containing numeric (continuous) predictors};
      to fit models with independent categorical effects, use
      \code{diag(f|g)} or set \code{options(lme4.doublevert.default = "diag_special")} (see
      \code{\link{getDoublevertDefault}}).
      \item Tags preceding a random effect term specify covariance
      structure:
      \itemize{
	\item \code{us} (default: \code{us(f|g)} is equivalent to \code{(f|g)}):
	unstructured, positive semi-definite
	\item \code{diag}: diagonal (all correlations set to zero).
	Specify \code{diag(f|g, hom = TRUE)}
	to fit a homogeneous diagonal covariance matrix
	\item \code{cs}: compound symmetric (all pairwise correlations
	set identical). Specify \code{cs(f|g, hom = TRUE)} for homogeneous
	variances.
	\item \code{ar1}: autoregressive order 1. Note that AR1 models
	are homogeneous by default; specify \code{ar1(f|g, hom = FALSE)}
	for heterogeneous variances.
      } % itemize (cov struct types)
    } % itemize (cov struct info)
  } % formula

  \item{data}{an optional data frame containing the variables named in
    \code{formula}.  By default the variables are taken from the
    environment from which \code{lmer} is called. While \code{data} is
    optional, the package authors \emph{strongly} recommend its use,
    especially when later applying methods such as \code{update} and
    \code{drop1} to the fitted model (\emph{such methods are not
    guaranteed to work properly if \code{data} is omitted}). If
    \code{data} is omitted, variables will be taken from the environment
    of \code{formula} (if specified as a formula) or from the parent
    frame (if specified as a character vector).}

  \item{REML}{logical scalar - Should the estimates be chosen to
    optimize the REML criterion (as opposed to the log-likelihood)?}

  \item{control}{a list (of correct class, resulting from
    \code{\link{lmerControl}()} or \code{\link{glmerControl}()}
    respectively) containing control parameters, including the nonlinear
    optimizer to be used and parameters to be passed through to the
    nonlinear optimizer, see the \code{*lmerControl} documentation for
    details.}

  \item{start}{a numeric vector or a named \link{list} with one optional
    component named \code{par} or \code{theta}, giving starting values
    for covariance parameters.  Numeric \code{start} is equivalent to
    \code{list(par = start)}.  Parameters corresponding to unstructured
    covariance matrices are on the scale of the Cholesky factor of the
    relative covariance matrix.  By default, all relative covariance
    matrices are identity matrices.}

  \item{verbose}{integer scalar.  If \code{> 0} verbose output is
    generated during the optimization of the parameter estimates.  If
    \code{> 1} verbose output is generated during the individual
    penalized iteratively reweighted least squares (PIRLS) steps.}

  \item{subset}{an optional expression indicating the subset of the rows
    of \code{data} that should be used in the fit. This can be a logical
    vector, or a numeric vector indicating which observation numbers are
    to be included, or a character vector of the row names to be
    included.  All observations are included by default.}

  \item{weights}{an optional vector of \sQuote{prior weights} to be used
    in the fitting process.  Should be \code{NULL} or a numeric vector.
    Prior \code{weights} are \emph{not} normalized or standardized in
    any way.  In particular, the diagonal of the residual covariance
    matrix is the squared residual standard deviation parameter
    \code{\link{sigma}} times the vector of inverse \code{weights}.
    Therefore, if the \code{weights} have relatively large magnitudes,
    then in order to compensate, the \code{\link{sigma}} parameter will
    also need to have a relatively large magnitude.}

  \item{na.action}{a function that indicates what should happen when the
    data contain \code{NA}s.  The default action (\code{na.omit},
    inherited from the 'factory fresh' value of
    \code{getOption("na.action")}) strips any observations with any
    missing values in any variables.}

  \item{offset}{this can be used to specify an \emph{a priori} known
    component to be included in the linear predictor during
    fitting. This should be \code{NULL} or a numeric vector of length
    equal to the number of cases.  One or more \code{\link{offset}}
    terms can be included in the formula instead or as well, and if more
    than one is specified their sum is used.  See
    \code{\link{model.offset}}.}

  \item{contrasts}{an optional list. See the \code{contrasts.arg} of
    \code{model.matrix.default}.}

  \item{devFunOnly}{logical - return only the deviance evaluation
    function. Note that because the deviance function operates on
    variables stored in its environment, it may not return
    \emph{exactly} the same values on subsequent calls (but the results
    should always be within machine tolerance).}
}
\value{
  An object of class \code{\link[=merMod-class]{merMod}} (more specifically,
  an object of \emph{subclass} \code{lmerMod}), for which many methods
  are available (e.g. \code{methods(class="merMod")})
}
\note{
  In earlier version of the \pkg{lme4} package, a \code{method} argument was
  used.  Its functionality has been replaced by the \code{REML} argument.

  Also, \code{lmer(.)} allowed a \code{family} argument (to effectively
  switch to \code{glmer(.)}).  This has been deprecated in summer 2013,
  and been disabled in spring 2019.
}
\details{
  \itemize{
    \item If the \code{formula} argument is specified as a character
      vector, the function will attempt to coerce it to a formula.
      However, this is not recommended (users who want to construct
      formulas by pasting together components are advised to use
      \code{\link{as.formula}} or \code{\link{reformulate}}); model fits
      will work but subsequent methods such as \code{\link{drop1}},
      \code{\link{update}}, etc. may fail.

    \item When handling perfectly collinear predictor variables
      (i.e. fixed-effect design matrices of less than full rank),
      \code{[gn]lmer} is not as sophisticated
      as modeling frameworks such as
      \code{\link{lm}} and \code{\link{glm}}. While it does
      automatically drop collinear variables (with a message
      rather than a warning), it does not automatically fill
      in \code{NA} values for the dropped coefficients;
      these can be added via \code{fixef(fitted.model, add.dropped=TRUE)}.
      This information can also be retrieved via
      \code{attr(getME(fitted.model, "X"), "col.dropped")}.
    
    \item the deviance function returned when \code{devFunOnly} is
      \code{TRUE} takes a single numeric vector argument which defines
      the scaled variance-covariance matrices of the random effects.
      \itemize{
	\item In the case of unstructured covariances, this vector
	is directly mapped to the \code{theta} vector, which represents
	the unique non-zero values in the Cholesky factor of the
	(scaled) covariance matrix. For models with only simple
	(intercept-only) random effects, \code{par} (and thus
	\code{theta}) is a vector of the
	standard deviations of the random effects.  For more complex or
	multiple random effects, running \code{getME(.,"par")} or
	(equivalently) \code{getME(., "theta")} to retrieve the
	\code{theta} 
	vector for a fitted model and examining
	the names of the vector is probably the easiest way to determine
	the correspondence between the elements of the \code{theta} vector
	and elements of the lower triangles of the Cholesky factors of the
	random effects.
	\item For structured covariances, the \code{getTheta} method
	translates the parameter vector to the \code{theta}
	(Cholesky-factor element) scale for internal use. The parameter
	vector is usually composed of a set of standard-deviation values
	(one if \code{hom = TRUE} or many if \code{hom = FALSE}),
	followed by one or more parameters that determine the correlation matrix.
      } % itemize (covariance parameterizations)	
    } % itemize (details)
} % details
\seealso{
  \code{\link[stats]{lm}} for linear models;
  \code{\link{glmer}} for generalized linear; and
  \code{\link{nlmer}} for nonlinear mixed models.
  
  \code{\link{plot.merMod}} for plot diagnostics.
}
\examples{
## linear mixed models - reference values from older code
(fm1 <- lmer(Reaction ~ Days + (Days | Subject), sleepstudy))
summary(fm1) # (with its own print method; see class?merMod % ./merMod-class.Rd
plot(fm1) # plotting the model diagnostics; see ?plot.merMod

str(terms(fm1))
stopifnot(identical(terms(fm1, fixed.only=FALSE),
                    terms(model.frame(fm1))))
attr(terms(fm1, FALSE), "dataClasses") # fixed.only=FALSE needed for dataCl.

## Maximum Likelihood (ML), and "monitor" iterations via 'verbose':
fm1_ML <- update(fm1, REML=FALSE, verbose = 1)
(fm2 <- lmer(Reaction ~ Days + (Days || Subject), sleepstudy))
anova(fm1, fm2)
sm2 <- summary(fm2)
print(fm2, digits=7, ranef.comp="Var") # the print.merMod()         method
print(sm2, digits=3, corr=FALSE)       # the print.summary.merMod() method

## Fit sex-specific variances by constructing numeric dummy variables
## for sex and sex:age; in this case the estimated variance differences
## between groups in both intercept and slope are zero ...
data(Orthodont,package="nlme")
Orthodont$nsex <- as.numeric(Orthodont$Sex=="Male")
Orthodont$nsexage <- with(Orthodont, nsex*age)
lmer(distance ~ age + (age|Subject) + (0+nsex|Subject) +
     (0 + nsexage|Subject), data=Orthodont)
}
\keyword{models}

\name{refitML}
\title{Refit a Model by Maximum Likelihood Criterion}
\alias{refitML}
\alias{refitML.merMod}
\description{
  Refit a (\code{merMod}) model using the maximum likelihood criterion.
}
\usage{
refitML(x, \dots)
\method{refitML}{merMod} (x, optimizer = "bobyqa", \dots)
}
\arguments{
  \item{x}{a fitted model, usually of class
    \code{"\linkS4class{lmerMod}"}, to be refit according to the maximum
    likelihood criterion.}
  \item{\dots}{optional additional parameters.  None are used
    at present.}
  \item{optimizer}{a string indicating the optimizer to be used.}
}
\value{
  an object like \code{x} but fit by maximum likelihood
}
\details{
  This function is primarily used to get a maximum likelihood fit of a
  linear mixed-effects model for an \code{\link{anova}} comparison.
}
\seealso{ \code{\link{refit}} and \code{\link{update.merMod}} for more
  extensive refitting.
}

\name{mkSimulateTemplate}
\title{Make templates suitable for guiding mixed model simulations}
\alias{mkParsTemplate}
\alias{mkDataTemplate}
\description{
  Make data and parameter templates suitable for guiding mixed model
  simulations, by specifying a model formula and other information
  (EXPERIMENTAL).  Most useful for simulating balanced designs and for 
  getting started on unbalanced simulations.
}
\usage{
mkParsTemplate(formula, data)
mkDataTemplate(formula, data, nGrps = 2, nPerGrp = 1, rfunc = NULL, ...)
}
\arguments{
  \item{formula}{A mixed model formula (see \code{\link{lmer}}).}
  \item{data}{A data frame containing the names in \code{formula}.}
  \item{nGrps}{Number of levels of a grouping factor.}
  \item{nPerGrp}{Number of observations per level.}
  \item{rfunc}{Function for generating covariate data (e.g. \code{\link{rnorm}}.}
  \item{...}{Additional parameters for \code{rfunc}.}
}
\seealso{
  These functions are designed to be used with \code{\link{simulate.merMod}}.
}

\name{rePCA}
\alias{rePCA}
\title{PCA of random-effects covariance matrix}
\usage{
rePCA(x)
}
\arguments{
\item{x}{a \code{merMod} object}
}
\value{
a \code{prcomplist} object
}
\description{
PCA of random-effects variance-covariance estimates
}
\details{
  Perform a Principal Components Analysis (PCA) of the random-effects
  variance-covariance estimates from a fitted mixed-effects model.
  This allows the user to detect and diagnose overfitting problems
  in the random effects model (see Bates et al. 2015 for details).
}
\examples{
  fm1 <- lmer(Reaction~Days+(Days|Subject), sleepstudy)
  rePCA(fm1)
}
\author{
  Douglas Bates
}
\seealso{\code{\link{isSingular}}}
\references{
  \itemize{
    \item \insertRef{bates2015parsimonious}{lme4}
  }
}

\docType{class}
\name{glmFamily-class}
\alias{glmFamily-class}
\title{Class \code{"glmFamily"} - a reference class for \code{\link{family}}}
\description{
  This class is a wrapper class for \code{\link{family}}
  objects specifying a distibution family and link function
  for a generalized linear model (\code{\link{glm}}).  The
  reference class contains an external pointer to a C++
  object representing the class.  For common families and
  link functions the functions in the family are
  implemented in compiled code so they can be accessed from
  other compiled code and for a speed boost.
}
\note{
  Objects from this reference class correspond to objects
  in a C++ class.  Methods are invoked on the C++ class
  using the external pointer in the \code{Ptr} field.  When
  saving such an object the external pointer is converted
  to a null pointer, which is why there is a redundant
  field \code{ptr} that is an active-binding function
  returning the external pointer.  If the \code{Ptr} field
  is a null pointer, the external pointer is regenerated
  for the stored \code{family} field.
}
\section{Extends}{
  All reference classes extend and inherit methods from
  \code{"\linkS4class{envRefClass}"}.
}
\examples{
str(glmFamily$new(family=poisson()))
}
\seealso{
  \code{\link{family}}, \code{\link{glmFamily}}
}
\keyword{classes}

\name{modular}
\title{Modular Functions for Mixed Model Fits}
\alias{glFormula}
\alias{lFormula}
\alias{mkGlmerDevfun}
\alias{mkLmerDevfun}
\alias{modular}
\alias{optimizeGlmer}
\alias{optimizeLmer}
\alias{updateGlmerDevfun}
\usage{
lFormula(formula, data = NULL, REML = TRUE,
    subset, weights, na.action, offset, contrasts = NULL,
    control = lmerControl(), ...)

mkLmerDevfun(fr, X, reTrms, REML = TRUE, start = NULL,
    verbose = 0, control = lmerControl(), ...)

optimizeLmer(devfun,
             optimizer    = formals(lmerControl)$optimizer,
             restart_edge = formals(lmerControl)$restart_edge,
             boundary.tol = formals(lmerControl)$boundary.tol,
             start = NULL, verbose = 0L,
             control = list(), ...)

glFormula(formula, data = NULL, family = gaussian,
    subset, weights, na.action, offset, contrasts = NULL,
    start, mustart, etastart, control = glmerControl(), ...)

mkGlmerDevfun(fr, X, reTrms, family,
              nAGQ = if (control$nAGQ0initStep) 0L else 1L,
              verbose = 0L, maxit = 100L, control = glmerControl(), ...)

optimizeGlmer(devfun,
    optimizer = if (nAGQ > 0L) "Nelder_Mead" else "bobyqa",
    restart_edge = FALSE,
    boundary.tol = formals(glmerControl)$boundary.tol,
    verbose = 0L, control = list(),
    nAGQ = if (missing(stage) || stage == 1L) 0L else 1L, stage,
    start = NULL, ...)

updateGlmerDevfun(devfun, reTrms, nAGQ = 1L)
}
\arguments{
  \item{formula}{a two-sided linear formula object
    describing both the fixed-effects and random-effects parts
    of the model, with the response on the left of a \code{~}
    operator and the terms, separated by \code{+} operators,
    on the right.  Random-effects terms are distinguished by
    vertical bars (\code{"|"}) separating expressions for
    design matrices from grouping factors.}

  \item{data}{an optional data frame containing the
    variables named in \code{formula}.  By default the
    variables are taken from the environment from which
    \code{lmer} is called. While \code{data} is optional, the
    package authors \emph{strongly} recommend its use,
    especially when later applying methods such as
    \code{update} and \code{drop1} to the fitted model
    (\emph{such methods are not guaranteed to work properly
      if \code{data} is omitted}). If \code{data} is omitted,
    variables will be taken from the environment of
    \code{formula} (if specified as a formula) or from the
    parent frame (if specified as a character vector).}

  \item{REML}{(logical) indicating to fit \bold{re}stricted maximum
    likelihood model.}

  \item{subset}{an optional expression indicating the
    subset of the rows of \code{data} that should be used in
    the fit. This can be a logical vector, or a numeric
    vector indicating which observation numbers are to be
    included, or a character vector of the row names to be
    included.  All observations are included by default.}

  \item{weights}{an optional vector of \sQuote{prior weights} to be used
    in the fitting process.  Should be \code{NULL} or a numeric vector.}

  \item{na.action}{a function that indicates what should
    happen when the data contain \code{NA}s.  The default
    action (\code{na.omit}, inherited from the 'factory
    fresh' value of \code{getOption("na.action")}) strips any
    observations with any missing values in any variables.}

  \item{offset}{this can be used to specify an \emph{a priori} known
    component to be included in the linear predictor during
    fitting.  This should be \code{NULL} or a numeric vector of length
    equal to the number of cases.  One or more \code{\link{offset}}
    terms can be included in the formula instead or as well, and if
    more than one is specified their sum is used.  See
    \code{\link{model.offset}}.}

  \item{contrasts}{an optional \code{\link{list}}.  See the
    \code{contrasts.arg} of \code{\link{model.matrix.default}}.}

  \item{control}{a list giving
    \describe{
      \item{for \code{[g]lFormula}:}{all
	options for running the model, see \code{\link{lmerControl}};}
      \item{for \code{mkLmerDevfun,mkGlmerDevfun}:}{options
	for the inner optimization step;}
      \item{for \code{optimizeLmer} and \code{optimizeGlmer}:}{control
	parameters for nonlinear optimizer (typically inherited from the
	\dots argument to \code{\link{lmerControl}}).}
	% FIXME: reference optCtrl
    }
  }
  \item{fr}{A model frame containing the variables needed to create an
    \code{\link{lmerResp}} or \code{\link{glmResp}} instance.}
  \item{X}{fixed-effects design matrix}
  \item{reTrms}{information on random effects structure (see
    \code{\link[reformulas]{mkReTrms}}).}
  \item{start}{starting values (see \code{\link{lmer}};
    for \code{glFormula}, should be just a numeric vector of
    fixed-effect coefficients)}
  \item{verbose}{print output?}
  \item{maxit}{maximal number of Pwrss update iterations.}
  \item{devfun}{a deviance function, as generated by \code{\link{mkLmerDevfun}}}
  \item{nAGQ}{number of Gauss-Hermite quadrature points}
  \item{stage}{optimization stage (1: nAGQ=0, optimize over theta only;
    2: nAGQ possibly >0, optimize over theta and beta)}
  \item{optimizer}{character - name of optimizing
    function(s).  A character vector or list of functions:
    length 1 for \code{lmer} or \code{glmer}, possibly length
    2 for \code{glmer}.  The built-in optimizers are
    \code{"\link{Nelder_Mead}"} and \code{"\link[minqa]{bobyqa}"}
    (from the \CRANpkg{minqa} package).  Any minimizing function
    that allows box constraints can be used provided that it
    \enumerate{
      \item{takes input parameters \code{fn} (function to be
	optimized), \code{par} (starting parameter values),
	\code{lower} (lower bounds) and \code{control} (control
	parameters, passed through from the \code{control}
	argument) and}
      \item{returns a list with (at least) elements
	\code{par} (best-fit parameters), \code{fval} (best-fit
	function value), \code{conv} (convergence code) and
	(optionally) \code{message} (informational message, or
	explanation of convergence failure)}.
    }
    Special provisions are made for \code{\link[minqa]{bobyqa}},
    \code{\link{Nelder_Mead}}, and optimizers wrapped in the
    \CRANpkg{optimx} package; to use \pkg{optimx} optimizers
    (including \code{L-BFGS-B} from base \code{\link{optim}}
    and \code{\link{nlminb}}), pass the \code{method}
    argument to \code{optim} in the \code{control} argument.

    For \code{glmer}, if \code{length(optimizer)==2}, the
    first element will be used for the preliminary (random
    effects parameters only) optimization, while the second
    will be used for the final (random effects plus fixed
    effect parameters) phase. See \code{\link{modular}} for
    more information on these two phases.
  }
  \item{restart_edge}{see \code{\link{lmerControl}}}
  \item{boundary.tol}{see \code{\link{lmerControl}}}
  \item{family}{a GLM family; see \code{\link[stats]{glm}}
    and \code{\link[stats]{family}}.}
  \item{mustart}{optional starting values on the scale of
    the conditional mean; see \code{\link[stats]{glm}} for details.}
  \item{etastart}{optional starting values on the scale of
    the unbounded predictor; see \code{\link[stats]{glm}} for details.}
  \item{\dots}{other potential arguments; for \code{optimizeLmer} and
    \code{optimizeGlmer}, these are passed to internal function
    \code{optwrap}, which has relevant parameters \code{calc.derivs}
    and \code{use.last.params} (see \code{\link{lmerControl}}).}
}
\value{
  \code{lFormula} and \code{glFormula} return a list containing
  components:

  \describe{
    \item{fr}{model frame}
     \item{X}{fixed-effect design matrix}
     \item{reTrms}{list containing information on random effects structure:
     result of \code{\link[reformulas]{mkReTrms}}}
     \item{REML}{(lFormula only): logical indicating if restricted maximum
     likelihood was used (Copy of argument.)}
  }

  \code{mkLmerDevfun} and \code{mkGlmerDevfun} return a function to
      calculate deviance (or restricted deviance) as a function of the
      theta (random-effect) parameters.  \code{updateGlmerDevfun}
      returns a function to calculate the deviance as a function of a
      concatenation of theta and beta (fixed-effect) parameters. These
      deviance functions have an environment containing objects required
      for their evaluation. CAUTION: The \code{\link{environment}} of
      functions returned by \code{mk(Gl|L)merDevfun} contains reference
      class objects (see \code{\link{ReferenceClasses}},
      \code{\link{merPredD-class}}, \code{\link{lmResp-class}}), which
      behave in ways that may surprise many users. For example, if the
      output of \code{mk(Gl|L)merDevfun} is naively copied, then
      modifications to the original will also appear in the copy (and
      vice versa). To avoid this behavior one must make a deep copy (see
      \code{\link{ReferenceClasses}} for details).

  \code{optimizeLmer} and \code{optimizeGlmer} return the results of an
  optimization.
}
\description{
  Modular functions for mixed model fits
}
\details{
  These functions make up the internal components of an [gn]lmer fit.
  \itemize{
    \item \code{[g]lFormula} takes the arguments that would normally be
    passed to \code{[g]lmer}, checking for errors and processing the
    formula and data input to create a list of objects required to fit a
    mixed model.
    \item \code{mk(Gl|L)merDevfun} takes the output of the previous
    step (minus the \code{formula} component) and creates a
    deviance function
    \item \code{optimize(Gl|L)mer} takes a
    deviance function and optimizes over \code{theta} (or
    over \code{theta} and \code{beta}, if \code{stage} is set
    to 2 for \code{optimizeGlmer}
    \item \code{updateGlmerDevfun} takes the first stage of a GLMM
    optimization (with \code{nAGQ=0}, optimizing over \code{theta} only)
    and produces a second-stage deviance function
    \item \code{\link{mkMerMod}} takes the \emph{environment} of a
    deviance function, the results of an optimization, a list of
    random-effect terms, a model frame, and a model all and produces a
    \code{[g]lmerMod} object.
  }
}
\examples{
### Fitting a linear mixed model in 4 modularized steps

## 1.  Parse the data and formula:
lmod <- lFormula(Reaction ~ Days + (Days|Subject), sleepstudy)
names(lmod)
## 2.  Create the deviance function to be optimized:
(devfun <- do.call(mkLmerDevfun, lmod))
ls(environment(devfun)) # the environment of 'devfun' contains objects
                        # required for its evaluation
## 3.  Optimize the deviance function:
opt <- optimizeLmer(devfun)
opt[1:3]
## 4.  Package up the results:
mkMerMod(environment(devfun), opt, lmod$reTrms, fr = lmod$fr)


### Same model in one line
lmer(Reaction ~ Days + (Days|Subject), sleepstudy)


### Fitting a generalized linear mixed model in six modularized steps

## 1.  Parse the data and formula:
glmod <- glFormula(cbind(incidence, size - incidence) ~ period + (1 | herd),
                   data = cbpp, family = binomial)
    #.... see what've got :
str(glmod, max=1, give.attr=FALSE)
## 2.  Create the deviance function for optimizing over theta:
(devfun <- do.call(mkGlmerDevfun, glmod))
ls(environment(devfun)) # the environment of devfun contains lots of info
## 3.  Optimize over theta using a rough approximation (i.e. nAGQ = 0):
(opt <- optimizeGlmer(devfun))
## 4.  Update the deviance function for optimizing over theta and beta:
(devfun <- updateGlmerDevfun(devfun, glmod$reTrms))
## 5.  Optimize over theta and beta:
opt <- optimizeGlmer(devfun, stage=2)
str(opt, max=1) # seeing what we'got
## 6.  Package up the results:
(fMod <- mkMerMod(environment(devfun), opt, glmod$reTrms, fr = glmod$fr))

### Same model in one line
fM <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
            data = cbpp, family = binomial)
all.equal(fMod, fM, check.attributes=FALSE, tolerance = 1e-12)
        # ----  --  even tolerance = 0  may work
}
\keyword{models}

\name{glmFamily}
\alias{glmFamily}
\title{Generator object for the \code{\linkS4class{glmFamily}} class}
\usage{
  glmFamily(...)
}
\arguments{
  \item{...}{Named argument (see Note below)}
}
\description{
  The generator object for the
  \code{\linkS4class{glmFamily}} reference class. Such an
  object is primarily used through its \code{new} method.
}
\note{
  Arguments to the \code{new} method must be named
  arguments.
}
\section{Methods}{
  \describe{ \item{\code{new(family=family)}}{Create a new
  \code{\linkS4class{glmFamily}} object} }
}
\seealso{
  \code{\linkS4class{glmFamily}}
}
\keyword{classes}

\name{summary}
\alias{summary}
\alias{summary.merMod}
\alias{show.summary.merMod}
\alias{print.summary.merMod}
\title{
  Summary for a [ng]lmer Fit
}
\description{
  \code{summary.merMod} Computes and returns a list of summary statistics of a 
  fitted mixed-effects model of class \code{\linkS4class{merMod}}, as produced 
  by [ng]lmer fits. The summary includes estimates for the fixed and random 
  effects. \code{print.summary} controls the output for the summary method.
}
\usage{
\S3method{summary}{merMod}(object,
       correlation = (p <= getOption("lme4.summary.cor.max")),
       use.hessian = NULL, \dots)
\S3method{print}{summary.merMod}(x, digits = max(3, getOption("digits") - 3),
      correlation = NULL, symbolic.cor = FALSE,
      signif.stars = getOption("show.signif.stars"),
      ranef.comp = c("Variance", "Std.Dev."),
      ranef.corr = any(ranef.comp == "Std.Dev."), show.resids = TRUE, \dots)
}
\arguments{
  \item{object}{a fitted [ng]lmer model.}
  \item{correlation}{(logical) indicates whether the correlation matrix
    should be computed and stored along with the covariance.
    For \code{print.summary.merMod}, indicates whether the correlation
    matrix of the fixed-effects parameters should be printed.  In the
    latter case, when \code{NULL} (the default), the correlation matrix
    is printed when it has been computed by \code{summary(.)}, and when
    \eqn{p <= 12}, and the cutoff \code{12} may be modified by
    \code{options(lme4.summary.cor.max = <n>)}}
  \item{use.hessian}{(logical) indicates whether to use the
    finite-difference Hessian of the deviance function to compute
    standard errors of the fixed effects; see \code{\link{vcov.merMod}}
    for details}
  \item{x}{an \R object of class \code{summary.merMod}.}
  \item{digits}{number of significant digits for printing.}
  \item{symbolic.cor}{should a symbolic encoding of the fixed-effects 
    correlation matrix be printed?  If so, the \code{\link{symnum}} function is 
    used.}
  \item{signif.stars}{(logical) should significance stars be used?}
  \item{ranef.comp}{character vector of length one or two, indicating
    if random-effects parameters should be reported on the variance and/or
    standard deviation scale.}
  \item{ranef.corr}{(logical) print correlations (rather than covariances) of
    random effects?}
  \item{show.resids}{should the quantiles of the scaled residuals be printed?}
  \item{\dots}{potentially further arguments passed from other methods.}
}
\seealso{
  \itemize{
    \item\code{\link[base]{summary}} for the original function in base \code{R},
    \item\code{\link[lme4]{lmer}}, and \code{\link[lme4]{glmer}} for model
      fitting,
    \item\code{\link[lme4]{sigma.merMod}} documents the result obtained in
      \code{"Residual Std.Dev"}; the name "Residual Std.Dev" is mostly 
      applicable for \code{\link[lme4]{lmer}} fits, but the meaning differs for
      \code{\link[lme4]{glmer}} fits.
  }
}
\value{
  An S3 object of class \code{"summary.merMod"}, which is a list containing 
  fixed and random effect estimates along with other standard model summary 
  information.
}
\examples{
fm1 <- lmer(Reaction ~ Days + (Days | Subject), sleepstudy)
smry <- summary(fm1)
# Obtaining the variance-covariance matrix of the fixed effects
smry$vcov
# Obtaining the correlation matrix of fixed effects
smry$vcov@factors$correlation
}

\docType{class}
\name{merMod-class}
\title{Class "merMod" of Fitted Mixed-Effect Models}
\alias{anova.merMod}
\alias{as.function.merMod}
\alias{coef.merMod}
\alias{df.residual.merMod}
\alias{extractAIC.merMod}
\alias{family.merMod}
\alias{fitted.merMod}
\alias{formula.merMod}
\alias{glmerMod-class}
\alias{lmerMod-class}
\alias{logLik.merMod}
\alias{merMod}
\alias{merMod-class}
\alias{model.frame.merMod}
\alias{model.matrix.merMod}
\alias{ngrps.merMod}
\alias{nobs.merMod}
\alias{nobs}
\alias{nlmerMod-class}
\alias{print.merMod}
\alias{show,merMod-method}
\alias{show.merMod}
\alias{terms.merMod}
\alias{update.merMod}
\alias{weights.merMod}
\alias{REMLcrit}
\description{
  A mixed-effects model is represented as a
  \code{\linkS4class{merPredD}} object and a response
  module of a class that inherits from class
  \code{\linkS4class{lmResp}}.  A model with a
  \code{\linkS4class{lmerResp}} response has class \code{lmerMod}; a
  \code{\linkS4class{glmResp}} response has class \code{glmerMod}; and a
  \code{\linkS4class{nlsResp}} response has class \code{nlmerMod}.
}
\usage{
\S3method{anova}{merMod}(object, \dots, refit = TRUE, model.names=NULL)
\S3method{as.function}{merMod}(x, \dots)
\S3method{coef}{merMod}(object, \dots)
REMLcrit(object)
\S3method{extractAIC}{merMod}(fit, scale = 0, k = 2, \dots)
\S3method{family}{merMod}(object, \dots)
\S3method{formula}{merMod}(x, fixed.only = FALSE, random.only = FALSE, \dots)
\S3method{fitted}{merMod}(object, \dots)
\S3method{logLik}{merMod}(object, REML = NULL, \dots)
\S3method{nobs}{merMod}(object, \dots)
\S3method{ngrps}{merMod}(object, \dots)
\S3method{terms}{merMod}(x, fixed.only = TRUE, random.only = FALSE, \dots)
\S3method{model.frame}{merMod}(formula, fixed.only = FALSE, \dots)
\S3method{model.matrix}{merMod}(object, 
  type = c("fixed", "random", "randomListRaw"), noScale = NULL, \dots)
\S3method{print}{merMod}(x, digits = max(3, getOption("digits") - 3),
      correlation = NULL, symbolic.cor = FALSE,
      signif.stars = getOption("show.signif.stars"), 
      ranef.comp = "Std.Dev.",
      ranef.corr = any(ranef.comp == "Std.Dev."), \dots)
\S3method{update}{merMod}(object, formula., \dots, evaluate = TRUE)
\S3method{weights}{merMod}(object, type = c("prior", "working"), \dots)
}
\arguments{
  \item{object}{an \R object of class \code{\linkS4class{merMod}}, i.e.,
    as resulting from \code{\link{lmer}()}, or \code{\link{glmer}()},
    etc.}
  \item{x}{an \R object of class \code{merMod} or \code{summary.merMod}.}
  \item{fit}{an \R object of class \code{\linkS4class{merMod}}.}
  \item{formula}{in the case of \code{model.frame}, a
    \code{\linkS4class{merMod}} object.}
  \item{refit}{logical indicating if objects of class \code{lmerMod} should be
    refitted with ML before comparing models. The default is
    \code{TRUE} to prevent the common mistake of inappropriately
    comparing REML-fitted models with different fixed effects,
    whose likelihoods are not directly comparable.}
  \item{model.names}{character vectors of model names to be used in the
    anova table.}
  \item{scale}{Not currently used (see \code{\link{extractAIC}}).}
  \item{k}{see \code{\link{extractAIC}}.}
  \item{REML}{Logical. If \code{TRUE}, return the restricted log-likelihood
    rather than the log-likelihood.  If \code{NULL} (the default),
    set \code{REML} to \code{isREML(object)} (see \code{\link{isREML}}).}
  \item{fixed.only}{logical indicating if only the fixed effects
    components (terms or formula elements) are sought.  If false, all
    components, including random ones, are returned.}
  \item{random.only}{complement of \code{fixed.only}; indicates
    whether random components only are sought. (Trying to specify
    \code{fixed.only} and \code{random.only} at the same time
    will produce an error.)}
  \item{correlation}{(logical)
    for \code{print.summary.merMod}, indicates whether the correlation
    matrix of the fixed-effects parameters should be printed.  In the
    latter case, when \code{NULL} (the default), the correlation matrix
    is printed when it has been computed by \code{summary(.)}, and when
    \eqn{p <= 12}, and the cutoff \code{12} may be modified by
    \code{options(lme4.summary.cor.max = <n>)}
  }
\item{digits}{number of significant digits for printing}
  \item{symbolic.cor}{should a symbolic encoding of the fixed-effects correlation
    matrix be printed?  If so, the \code{\link{symnum}} function is used.}
  \item{signif.stars}{(logical) should significance stars be used?}
  \item{ranef.comp}{character vector of length one or two, indicating
    if random-effects parameters should be reported on the variance and/or
    standard deviation scale.}
  \item{formula.}{see \code{\link{update.formula}}.}
  \item{evaluate}{see \code{\link{update}}.}
  \item{type}{For \describe{
      \item{\code{weights()}, }{type of weights to be returned; either \code{"prior"} for
	the initially supplied weights or \code{"working"} for the weights
	at the final iteration of the penalized iteratively reweighted least
	squares algorithm (PIRLS).}
      \item{\code{model.matrix()}, }{type of model matrix to
	return: one of \code{"fixed"} giving the fixed effects model matrix,
	\code{"random"} giving the random effects model matrix, or
	\code{"randomListRaw"} giving a list of the raw random effects model
	matrices associated with each random effects term.}
  }}
  \item{ranef.corr}{(logical) print correlations (rather than covariances) of
    random effects?}
  \item{noScale}{(logical) some users may use autoscaling. If so, the default
    returns the scaled parameters. If \code{noScale = TRUE}, then \code{fixef} 
    returns the non-scaled parameters instead.}
  \item{\dots}{potentially further arguments passed from other methods.}
} %% arguments
\section{Objects from the Class}{
  Objects of class \code{merMod} are created by calls to
  \code{\link{lmer}}, \code{\link{glmer}} or \code{\link{nlmer}}.
}
\section{S3 methods}{
The following S3 methods with arguments given above exist (this list is currently not complete):
\describe{
  %% TODO:  document differences between update and update.merMod
  \item{\code{anova}:}{returns the sequential decomposition of the contributions of
    fixed-effects terms or, for multiple arguments, model comparison statistics.
    For objects of class \code{lmerMod} the default behavior is to refit the models
    with ML if fitted with \code{REML = TRUE}, this can be controlled via the
    \code{refit} argument. See also \code{\link{anova}}.}
  \item{\code{as.function}:}{returns the deviance function, the same as
    \code{\link{lmer}(*, devFunOnly=TRUE)}, and \code{\link{mkLmerDevfun}()}
    or \code{\link{mkGlmerDevfun}()}, respectively.}
  \item{\code{coef}:}{Computes the sum of the random and fixed effects
    coefficients for each explanatory variable for each level of each
    grouping factor.}
  \item{\code{extractAIC}:}{Computes the (generalized) Akaike An
    Information Criterion. If \code{isREML(fit)}, then \code{fit} is
    refitted using maximum likelihood.}
  \item{\code{family}:}{\code{\link{family}} of fitted
    GLMM. (\emph{Warning:} this accessor may not work properly with
    customized families/link functions.)}
  \item{\code{fitted}:}{Fitted values, given the conditional modes of
    the random effects.  For more flexible access to fitted values, use
    \code{\link{predict.merMod}}.}
  \item{\code{logLik}:}{Log-likelihood at the fitted value of the
    parameters.  Note that for GLMMs, the returned value is only
    proportional to the log probability density (or distribution) of the
    response variable.  See \code{\link{logLik}}. }
  \item{\code{model.frame}:}{returns the \code{frame} slot of \code{\linkS4class{merMod}}.}
  \item{\code{model.matrix}:}{returns the fixed effects model matrix.}
  \item{\code{nobs}, \code{ngrps}:}{Number of observations and vector of
    the numbers of levels in each grouping factor.  See \code{\link{ngrps}}.}
   \item{\code{update}:}{See \code{\link{update}}.}
 }
}
\section{Deviance and log-likelihood of GLMMs}{

  One must be careful when defining the deviance of a GLM.  For example,
  should the deviance be defined as minus twice the log-likelihood or
  does it involve subtracting the deviance for a saturated model?  To
  distinguish these two possibilities we refer to absolute deviance
  (minus twice the log-likelihood) and relative deviance (relative to a
  saturated model, e.g. Section 2.3.1 in McCullagh and Nelder 1989).

  With GLMMs however, there is an additional complication involving the
  distinction between the likelihood and the conditional likelihood.
  The latter is the likelihood obtained by conditioning on the estimates
  of the conditional modes of the spherical random effects coefficients,
  whereas the likelihood itself (i.e. the unconditional likelihood)
  involves integrating out these coefficients.  The following table
  summarizes how to extract the various types of deviance for a
  \code{glmerMod} object:

  \tabular{lrr}{
             \tab conditional              \tab unconditional \cr
    relative \tab \code{deviance(object)}  \tab NA in \code{lme4} \cr
    absolute \tab \code{object@resp$aic()} \tab \code{-2*logLik(object)}
  }

  This table requires two caveats:

  \itemize{
    \item If the link function involves a scale parameter
    (e.g. \code{Gamma}) then \code{object@resp$aic() - 2 * getME(object,
    "devcomp")$dims["useSc"]} is required for the absolute-conditional
    case.
    \item If adaptive Gauss-Hermite quadrature is used, then
    \code{logLik(object)} is currently only proportional to the
    absolute-unconditional log-likelihood.
  }

  For more information about this topic see the \code{misc/logLikGLMM}% ../misc/logLikGLMM/
  directory in the package source.

  %% We define the conditional likelihood as the density of the response
  %% variable, \eqn{y}, conditional on the spherical random effects,
  %% \eqn{u}:

  %% \deqn{f_{\theta,\beta}(y|u)}{f(y|u)}

  %% which depends on the fixed effect and covariance parameters,
  %% \eqn{\beta} and \eqn{\theta}.  We define the likelihood itself as:

  %% \deqn{f_{\theta,\beta}(y) = \int f_{\theta,\beta}(y|u) f(u)}{f(y) = integral(f(y|u)f(u))}

  %% where \eqn{f(u)} is an independent multivariate normal.  It is not
  %% typically possible to evaluate this integral for GLMMs.  In
  %% \code{lme4} we approximate it using the Laplace approximation and
  %% adaptive Gauss-Hermite quadrature.

  %% For canonical links, the Laplace approximation to the log-likelihood
  %% is:

  %% \deqn{log f_{\theta,\beta}(y | u)-0.5|u|^2 - log|L_{\theta}|}
  %% {log f(y|u) - 0.5|u|^2 - log|L|}

  %% where \eqn{|u|^2} is the squared length of the vector of spherical
  %% random effects, \eqn{u}, and \eqn{|L_{\theta}|}{|L|} is the
  %% determinant of the Cholesky factor for the cross-product matrix for
  %% the random effects (\code{\link{getME}(., "L")}).  Note that in the
  %% non-canonical link case, the above equation is technically an
  %% approximation to the Laplace approximation.

  %% For models with a single scalar random effect one may also obtain an
  %% approximate log-likelihood using adaptive Gauss-Hermite quadrature as:

  %% \deqn{-\log|\bm L| - \frac{q}{2}\log(2\pi) + \sum_j^q \log \left[\sum_k^{N_{AGQ}} w_k f(y, u_{jk}) e^{-\frac{1}{2}u_{jk}^2 - l_k}\right]}

  %% where \eqn{N_{AGQ}} is the number of quadrature nodes, \eqn{q} is the
  %% number of levels of the random effect, the \eqn{w_k} and \eqn{l_k} are
  %% weights and log of the normal density associated with the \eqn{k}th
  %% quadrature node (\code{\link{GHrule}}), and \eqn{u_{jk}} is the value
  %% of the spherical random effect coefficient associated with the
  %% \eqn{j}th level of the grouping factor and \eqn{k}th quadrature node.

}
\section{Slots}{
  \describe{
    \item{\code{resp}:}{A reference class object for an \pkg{lme4}
      response module (\code{\link{lmResp-class}}).}
    \item{\code{Gp}:}{See \code{\link{getME}}.}
    \item{\code{call}:}{The matched call.}
    \item{\code{frame}:}{The model frame containing all of the variables
      required to parse the model formula.}
    \item{\code{flist}:}{See \code{\link{getME}}.}
    \item{\code{cnms}:}{See \code{\link{getME}}.}
    \item{\code{lower}:}{See \code{\link{getME}}.}
    \item{\code{theta}:}{Covariance parameter vector.}
    \item{\code{beta}:}{Fixed effects coefficients.}
    \item{\code{u}:}{Conditional model of spherical random effects
      coefficients.}
    \item{\code{devcomp}:}{See \code{\link{getME}}.}
    \item{\code{pp}:}{A reference class object for an \pkg{lme4}
      predictor module (\code{\link{merPredD-class}}).}
    \item{\code{optinfo}:}{List containing information about the
      nonlinear optimization.}
  }
}
\seealso{
  \code{\link{lmer}}, \code{\link{glmer}},
  \code{\link{nlmer}}, \code{\linkS4class{merPredD}},
  \code{\linkS4class{lmerResp}},
  \code{\linkS4class{glmResp}},
  \code{\linkS4class{nlsResp}}

  Other methods for \code{merMod} objects documented elsewhere include:
  \code{\link{fortify.merMod}}, \code{\link{drop1.merMod}},
  \code{\link{isLMM.merMod}}, \code{\link{isGLMM.merMod}},
  \code{\link{isNLMM.merMod}}, \code{\link{isREML.merMod}},
  \code{\link{plot.merMod}}, \code{\link{predict.merMod}},
  \code{\link{profile.merMod}}, \code{\link{ranef.merMod}},
  \code{\link{refit.merMod}}, \code{\link{refitML.merMod}},
  \code{\link{residuals.merMod}}, \code{\link{sigma.merMod}},
  \code{\link{simulate.merMod}}, \code{\link{summary.merMod}}.
}
\examples{
showClass("merMod")
methods(class="merMod")## over 30  (S3) methods available

m1 <- lmer(Reaction ~ Days + (Days | Subject), sleepstudy)
print(m1, ranef.corr = TRUE)   ## print correlations of REs
print(m1, ranef.corr = FALSE)  ## print covariances of REs

}
\keyword{classes}

\docType{methods}
\name{fixef}
\alias{fixed.effects}
\alias{fixef}
\alias{fixef.merMod}
\title{Extract fixed-effects estimates}
\usage{
  \method{fixef}{merMod} (object, add.dropped=FALSE, noScale = NULL, \dots)
}
\arguments{
  \item{object}{any fitted model object from which fixed
  effects estimates can be extracted.}
\item{add.dropped}{for models with rank-deficient design
  matrix, reconstitute the full-length parameter vector by
  adding \code{NA} values in appropriate locations?}
\item{noScale}{(logical) some users may use autoscaling. If so, the default
  returns the scaled parameters. If \code{noScale = TRUE}, then \code{fixef} 
  returns the non-scaled parameters instead.}
\item{\dots}{optional additional arguments. Currently
  none are used in any methods.}
}
\value{
  a named, numeric vector of fixed-effects estimates.
}
\description{
  Extract the fixed-effects estimates
}
\details{
  Extract the estimates of the fixed-effects parameters
  from a fitted model.
}
\examples{
fixef(lmer(Reaction ~ Days + (1|Subject) + (0+Days|Subject), sleepstudy))
fm2 <- lmer(Reaction ~ Days + Days2 + (1|Subject),
            data=transform(sleepstudy,Days2=Days))
fixef(fm2,add.dropped=TRUE)
## first two parameters are the same ...
stopifnot(all.equal(fixef(fm2,add.dropped=TRUE)[1:2],
                    fixef(fm2)))
}
\keyword{models}

\name{culcitalogreg}
\alias{culcitalogreg}
\docType{data}
\title{
 Coral-eating seastar Culcita novaeguineae data (binary predation version)
}
\description{
  This data shows the results from a block design experiment to record the 
  interactions of ‘guard’ crustaceans, shrimp (\emph{Alpheus lottini}) and crab 
  (\emph{Trapezia serenei}), that defend their coral host 
  (\emph{Pocillopora cf. meandrina}) from seastar predators 
  (\emph{Culcita novaeguineae}) \insertCite{mckeon2012multiple}{lme4}. 
  The coral and their seastar predator were 
  placed in large, octagonal, flow-through seawater tanks measuring 
  approximately 0.5m deep and 2m in diameter. There were 10 tanks in total, 
  each serving as an experimental block. Four treatment groups were tested: 
  (1) no exosymbionts, (2) shrimp only, (3) crab only, and (4) both crab and 
  shrimp present. Each treatment was replicated twice within each block.
}
\usage{data("culcitalogreg")}
\format{
  A data frame with 80 observations on the following 9 variables.
  \describe{
    \item{\code{block}}{a numeric variable indicating the experimental block. 
    There are \code{10} blocks in total, each corresponding to a large, 
    octagonal, flow-through seawater tank approximately 0.5m deep and 2m in 
    diameter.}
    \item{\code{ttt}}{a number which represents the combinations of different 
      symbionts.
      \itemize{
          \item \code{1}: no exosymbionts,
          \item \code{2}: pair of \emph{Alpheus lottini} only 
            (Alpheus; shrimp),
          \item \code{3}: pair of \emph{Trapezia serenei} only 
            (Trapezia; crab),
          \item \code{4}: pair of \emph{Alpheus lottini} and pair of 
            \emph{Trapezia serenei} (‘Alpheus and Trapezia’).
        }
      }
    \item{\code{predation}}{(binary) indicates whether the seastar predator 
      consumed the coral. \code{0} = did not eat; \code{1} = ate.}
    \item{\code{ttt.1}}{a relabelled version of \code{ttt} that explicitly 
      indicates the corresponding treatment condition in plain language.}
    \item{\code{crab}}{describes whether the crab was present in the 
      experiment. \code{C-}: not present, \code{C+}: present.}
    \item{\code{shrimp}}{describes whether the shrimp was present in the
      experiment. \code{S-}: not present, \code{S+}: present.}
    \item{\code{ttt2}}{a relabelled version of \code{ttt} that uses letters 
      (a, b, c, d) instead of numeric values (1, 2, 3, 4), respectively, to 
      represent the treatment groups.}
    \item{\code{crab2}}{a relabelling of \code{crab} where \code{n} corresponds 
      to \code{C-} (no crab), and \code{y} corresponds to \code{C+} (crab 
      present).}
    \item{\code{shrimp2}}{a relabelling of \code{shrimp} where \code{n} 
      corresponds to \code{S-} (no shrimp), and \code{y} corresponds to 
      \code{S+} (shrimp present).}
  }
}
\details{
  Full details of the experiment \insertCite{mckeon2012multiple}{lme4}, is 
  quoted below:
  "Twenty replicates in total were conducted for each exosymbiont treatment in 
  a temporally blocked design with two replicates in each of ten temporal 
  blocks. Trials were conducted in a large octagonal flow-through seawater tank
  approximately 0.5 m deep and 2 m across. The tank was divided into eight equal 
  sections using plastic screening. Each section was provisioned with a seastar 
  refugium constructed from concrete blocks. We placed Pocillopora colonies 
  into the tank from the field during mid- to late afternoon. To minimize 
  variation driven by search time, a single Culcita was placed directly on top 
  of the coral colony. The following morning, we measured the coral size 
  (length, width, height) and the feeding scars left by the Culcita (length, 
  width, depth)."
}
\seealso{
  The version which only contains predation counts but includes the amount of 
  volume of the coral and the amount of volume lost, 
  \code{\link[lme4]{culcitalvolume}}.
}
\source{
  \insertCite{mckeon2012multiple}{lme4}
}
\references{
  \insertRef{mckeon2012multiple}{lme4}
}
\examples{
cul_mod <- glmer(predation ~ ttt2 + (1|block), data=culcitalogreg,
                     family = binomial(link = "logit"))
}
\keyword{datasets}

\name{plots.thpr}
\title{Mixed-Effects Profile Plots (Regular / Density / Pairs)}
\alias{xyplot.thpr}
\alias{densityplot.thpr}
\alias{splom.thpr}
\description{
  Xyplot, Densityplot, and Pairs plot methods for a mixed-effects model
  profile.

  \code{xyplot()} draws \dQuote{zeta diagrams}, also visualizing
  confidence intervals and their asymmetry.

  \code{densityplot()} draws the profile densities.

  \code{splom()} draws profile pairs plots.  Contours are for the
  marginal two-dimensional regions (i.e. using df = 2).
}
\usage{
\method{xyplot}{thpr}(x, data = NULL,
       levels = sqrt(qchisq(pmax.int(0, pmin.int(1, conf)), df = 1)),
       conf = c(50, 80, 90, 95, 99)/100,
       absVal = FALSE, scales=NULL,
       which = 1:nptot, \dots)

\method{densityplot}{thpr}(x, data, npts = 201, upper = 0.999, \dots)

\method{splom}{thpr}(x, data,
      levels = sqrt(qchisq(pmax.int(0, pmin.int(1, conf)), 2)),
      conf = c(50, 80, 90, 95, 99)/100,  which = 1:nptot,
      draw.lower = TRUE, draw.upper = TRUE, \dots)
}
\arguments{
  \item{x}{a mixed-effects profile, i.e., of class \code{"thpr"},
    typically resulting from \code{\link{profile}(fm)} where \code{fm}
    is a fitted model from \code{\link{lmer}} (or its generalizations).}
  \item{data}{unused - only for compatibility with generic.}
  \item{npts}{the number of points to use for the \code{densityplot()}.}
  \item{upper}{a number in \eqn{(0,1)} to specify upper (and lower)
      boundaries as \code{+/- qnorm(upper)}.}
  \item{levels}{the contour levels to be shown; usually
    derived from \code{conf}.}
  \item{conf}{numeric vector of confidence levels to be
    shown as contours.}
  \item{absVal}{logical indicating if \code{\link{abs}(.)}olute values
      should be plotted, often preferred for confidence interval
      visualization.}
    \item{scales}{plotting options to be passed to \code{\link[lattice]{xyplot}}}
    \item{which}{integer or character vector indicating which parameters
    to profile: default is all parameters (see
    \code{\link{profile-methods}}
    for details).}
  \item{draw.lower}{(logical) draw lower-triangle (zeta scale) panels?}
  \item{draw.upper}{(logical) draw upper-triangle (standard dev/cor scale) panels?}
  \item{\dots}{further arguments passed to \code{\link[lattice]{xyplot}},
    \code{\link[lattice]{densityplot}}, or \code{\link[lattice]{splom}} from package
    \CRANpkg{lattice}, respectively.}
}
\seealso{ \code{\link[=profile-methods]{profile}}, notably for an
  example.
}
\value{
  \describe{
    \item{xyplot:}{a density plot, a \code{"trellis"} object (\pkg{lattice} package)
      which when \code{\link{print}()}ed produces plots on the current
      graphic device.}
    \item{densityplot:}{a density plot, a \code{"trellis"} object, see above.}
    \item{splom:}{a pairs plot, aka \bold{s}catter\bold{plo}t
      \bold{m}atrix, a \code{"trellis"} object, see above.}
  }
}
\examples{
## see   example("profile.merMod")
}

\name{getReCovs}
\alias{getReCovs}
\title{Extract Fitted Covariance Structures}
\usage{getReCovs(object)}
\arguments{
  \item{object}{an \R object of class \code{\linkS4class{merMod}}, i.e.,
    as resulting from \code{\link{lmer}()}, or \code{\link{glmer}()},
    etc.}
}
\value{a list of covariance objects}
\description{Returns a list with one covariance object for each
  random effect in the model. If the object is an old-style (pre-version
  2.0.0) fitted model, it automatically converts all terms to
  \code{Covariance.us} objects (except for terms including double-bar
  \sQuote{||} notation, which are converged to \code{Covariance.diag}
  objects).
}
\examples{
  m1 <- lmer(Reaction ~ Days + (Days || Subject), sleepstudy)
  sapply(getReCovs(m1), "class")
}
  

\name{hatvalues.merMod}
\alias{hatvalues.merMod}
\title{Diagonal elements of the hat matrix}
\usage{
  \method{hatvalues}{merMod} (model, fullHatMatrix = FALSE, ...)
}
\arguments{
  \item{model}{An object of class \code{\link{merMod}}.}
  \item{fullHatMatrix}{Return full hat matrix (not just diagonal values)?}
  \item{...}{Not currently used}
}
\value{
  The diagonal elements of the hat matrix.
}
\description{
  Returns the values on the diagonal of the hat matrix, which is the
  matrix that transforms the response vector (minus any offset) into the
  fitted values (minus any offset).  Note that this method should only
  be used for linear mixed models.  It is not clear if the hat matrix
  concept even makes sense for generalized linear mixed models.
}
\examples{
m <- lmer(Reaction ~ Days + (Days | Subject), sleepstudy)
hatvalues(m)
}

\name{isREML}
\alias{isGLMM}
\alias{isLMM}
\alias{isNLMM}
\alias{isREML}
\alias{isGLMM.merMod}
\alias{isLMM.merMod}
\alias{isNLMM.merMod}
\alias{isREML.merMod}
\title{Check characteristics of models}
\usage{
  isREML(x, ...)

  isLMM(x, ...)

  isNLMM(x, ...)

  isGLMM(x, ...)
}
\arguments{
  \item{x}{a fitted model.}

  \item{...}{additional, optional arguments.  (None are
  used in the merMod methods)}
}
\value{
  a logical value
}
\description{
  Check characteristics of models: whether a model fit
  corresponds to a linear (LMM), generalized linear (GLMM),
  or nonlinear (NLMM) mixed model, and whether a linear
  mixed model has been fitted by REML or not
  (\code{isREML(x)} is always \code{FALSE} for GLMMs and
  NLMMs).
}
\details{
  These are generic functions.  At present the only methods
  are for mixed-effects models of class
  \code{\linkS4class{merMod}}.
}
\examples{
fm1 <- lmer(Reaction ~ Days + (Days|Subject), sleepstudy)
gm1 <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
              data = cbpp, family = binomial)
nm1 <- nlmer(circumference ~ SSlogis(age, Asym, xmid, scal) ~ Asym|Tree,
             Orange, start = c(Asym = 200, xmid = 725, scal = 350))

isLMM(fm1)
isGLMM(gm1)
## check all :
is.MM <- function(x) c(LMM = isLMM(x), GLMM= isGLMM(x), NLMM= isNLMM(x))
stopifnot(cbind(is.MM(fm1), is.MM(gm1), is.MM(nm1))
	  == diag(rep(TRUE,3)))
}
\seealso{
  getME
}

\name{expandDoubleVerts}
\alias{expandDoubleVerts}
\alias{||}
\title{Expand terms with \code{'||'} notation into separate \code{'|'} terms}
\usage{
  expandDoubleVerts(term)
}
\arguments{
  \item{term}{a mixed-model formula}
}
\value{
 the modified term
}
\description{
  From the right hand side of a formula for a mixed-effects model,
  expand terms with the double vertical bar operator
  into separate, independent random effect terms.
}
\note{
  Because \code{||} works at the level of formula parsing, it
  has no way of knowing whether a variable is a factor. It
  just takes the terms within a random-effects term and literally splits them
  into the intercept and separate no-intercept terms,
  e.g. \code{(1+x+y|f)} would be split into \code{(1|f) + (0+x|f) + (0+y|f)}.
  However, \code{||} will fail to break up factors into separate terms;
  the \code{\link{dummy}} function can be useful in this case, although
  it is not as convenient as \code{||}.
}
\examples{
  m <- ~ x + (x || g)
  expandDoubleVerts(m)
  set.seed(101)
  dd <- expand.grid(f=factor(letters[1:3]),g=factor(1:200),rep=1:3)
  dd$y <- simulate(~f + (1|g) + (0+dummy(f,"b")|g) + (0+dummy(f,"c")|g),
          newdata=dd,
          newparams=list(beta=rep(0,3),
                         theta=c(1,2,1),
                         sigma=1),
          family=gaussian)[[1]]
  m1 <- lmer(y~f+(f|g),data=dd)
  VarCorr(m1)
  m2 <- lmer(y~f+(1|g) + (0+dummy(f,"b")|g) + (0+dummy(f,"c")|g),
               data=dd)
  VarCorr(m2)
}
\seealso{
  \code{\link{formula}}, \code{\link{model.frame}},
  \code{\link{model.matrix}}, \code{\link{dummy}}.

  Other utilities: \code{\link{mkRespMod}},
  \code{\link{mkReTrms}}, \code{\link{nlformula}},
  \code{\link{nobars}}, \code{\link{subbars}}
}
\keyword{models}
\keyword{utilities}

\name{fortify}
\alias{fortify}
\alias{fortify.merMod}
\alias{getData}
\alias{getData.merMod}
\title{add information to data based on a fitted model}
\usage{
  fortify.merMod(model, data = getData(model),
    ...)
\method{getData}{merMod}(object)
}
\arguments{
  \item{model}{fitted model}
  \item{object}{fitted model}
  \item{data}{original data set, if needed}
  \item{\dots}{additional arguments}
}
\description{
  \code{fortify} adds information to data based on a fitted model;
  \code{getData} retrieves data as specified in the \code{data}
  argument
}
\details{
  \itemize{
    \item{\code{fortify} is defined in the
    \CRANpkg{ggplot2} package, q.v. for more details.
    \code{fortify} is \emph{not} defined here,
    and \code{fortify.merMod} is defined as a function
    rather than an S3 method, to avoid (1) inducing
    a dependency on \pkg{ggplot2} or (2) masking
    methods from \pkg{ggplot2}.
    This feature is both experimental and semi-deprecated,
    as the help page for \code{fortify} itself says:
    \dQuote{Rather than using this function, I now recommend using the
      \code{broom} package, which implements a much wider range of
      methods. \code{fortify} may be deprecated in the future.}
    The \code{broom.mixed} package is recommended for mixed models
    in general.
  }
  \item{
    \code{getData} is a bare-bones implementation; it relies on
    a \code{data} argument having been specified and the data being
    available in the environment of the formula. Unlike the functions
    in the \code{nlme} package, it does not do anything special with
    \code{na.action} or \code{subset}.
  }
  }
}
\examples{
  fm1 <- lmer(Reaction~Days+(1|Subject),sleepstudy)
  names(fortify.merMod(fm1))
}

\docType{data}
\name{Penicillin}
\alias{Penicillin}
\title{Variation in penicillin testing}
\format{A data frame with 144 observations on the following 3 variables.
  \describe{
    \item{\code{diameter}}{diameter (mm) of the zone of inhibition of
      the growth of the organism.}
    \item{\code{plate}}{assay plate.  A factor with levels \sQuote{a} to
      \sQuote{x}.}
    \item{\code{sample}}{penicillin sample.  A factor with levels
      \sQuote{A} to \sQuote{F}.}
  }}
\references{
  \insertRef{davies1972statistical}{lme4}
}
\description{
  Six samples of penicillin were tested using the \emph{B.
  subtilis} plate method on each of 24 plates.  The
  response is the diameter (mm) of the zone of inhibition
  of growth of the organism.
}
\details{
  The data are described in Davies and Goldsmith (1972) as
  coming from an investigation to \dQuote{assess the
  variability between samples of penicillin by the \emph{B.
  subtilis} method.  In this test method a bulk-inoculated
  nutrient agar medium is poured into a Petri dish of
  approximately 90 mm. diameter, known as a plate.  When
  the medium has set, six small hollow cylinders or pots
  (about 4 mm. in diameter) are cemented onto the surface
  at equally spaced intervals.  A few drops of the
  penicillin solutions to be compared are placed in the
  respective cylinders, and the whole plate is placed in an
  incubator for a given time.  Penicillin diffuses from the
  pots into the agar, and this produces a clear circular
  zone of inhibition of growth of the organisms, which can
  be readily measured.  The diameter of the zone is related
  in a known way to the concentration of penicillin in the
  solution.}
}
\examples{
str(Penicillin)
require(lattice)
dotplot(reorder(plate, diameter) ~ diameter, Penicillin, groups = sample,
        ylab = "Plate", xlab = "Diameter of growth inhibition zone (mm)",
        type = c("p", "a"), auto.key = list(columns = 3, lines = TRUE,
        title = "Penicillin sample"))
(fm1 <- lmer(diameter ~ (1|plate) + (1|sample), Penicillin))

L <- getME(fm1, "L")
Matrix::image(L, main = "L",
              sub = "Penicillin: Structure of random effects interaction")
}
\keyword{datasets}

\name{GHrule}
\alias{GHrule}
\title{Univariate Gauss-Hermite quadrature rule}
\description{
  Create a univariate Gauss-Hermite quadrature rule.
}
\usage{
  GHrule(ord, asMatrix = TRUE)
}
\arguments{
  \item{ord}{scalar integer between 1 and 100 - the order,
    or number of nodes and weights, in the rule.  When the
    function being multiplied by the standard normal density
    is a polynomial of order \eqn{2k-1} the rule of order \eqn{k}
    integrates the product exactly.}

  \item{asMatrix}{logical scalar - should the result be
    returned as a matrix.  If \code{FALSE} a data frame is
    returned.  Defaults to \code{TRUE}.}
}
\value{
  a matrix (or data frame, is \code{asMatrix} is false) with \code{ord}
  rows and three columns which are \code{z} the node positions, \code{w}
  the weights and \code{ldnorm}, the logarithm of the normal density
  evaluated at the nodes.
}
\details{
  This version of Gauss-Hermite quadrature provides the node positions
  and weights for a scalar integral of a function multiplied by the
  standard normal density.

  Originally based on package \CRANpkg{SparseGrid}'s hidden \code{GQN()},
  then on \CRANpkg{fastGHQuad}'s \code{gaussHermiteData(.)}.% ==> in ../R/GHrule.R
}
\references{
  \insertRef{liu1994note}{lme4}
}
\seealso{a different interface is available via \code{\link{GQdk}()}.
}
\examples{
(r5  <- GHrule( 5, asMatrix=FALSE))
(r12 <- GHrule(12, asMatrix=FALSE))

## second, fourth, sixth, eighth and tenth central moments of the
## standard Gaussian N(0,1) density:
ps <- seq(2, 10, by = 2)
cbind(p = ps, "E[X^p]" = with(r5,  sapply(ps, function(p) sum(w * z^p)))) # p=10 is wrong for 5-rule
p <- 1:15
GQ12 <- with(r12, sapply(p, function(p) sum(w * z^p)))
cbind(p = p, "E[X^p]" = zapsmall(GQ12))
## standard R numerical integration can do it too:
intL <- lapply(p, function(p) integrate(function(x) x^p * dnorm(x),
                                        -Inf, Inf, rel.tol=1e-11))
integR <- sapply(intL, `[[`, "value")
cbind(p, "E[X^p]" = integR)# no zapsmall() needed here
all.equal(GQ12, integR, tol=0)# => shows small difference
stopifnot(all.equal(GQ12, integR, tol = 1e-10))
(xactMom <- cumprod(seq(1,13, by=2)))
stopifnot(all.equal(xactMom, GQ12[2*(1:7)], tol=1e-14))
## mean relative errors :
mean(abs(GQ12  [2*(1:7)] / xactMom - 1)) # 3.17e-16
mean(abs(integR[2*(1:7)] / xactMom - 1)) # 9.52e-17 {even better}
}

\name{schizophrenia}
\alias{schizophrenia}
\docType{data}
\title{
  National Institute of Mental Health Schizophrenia Collaborative Study
}
\description{
  In the NIMH Schizophrenia Collaborative Study, patients were randomized to 
  receive one of four medications, either placebo or one of three different 
  anti-psychotic drugs. The protocol indicated subjects were to then be 
  evaluated at weeks 0, 1, 3, 6 to assess severity of illness; additionally 
  some measurements were made at weeks 2, 4, and 5.
}
\usage{data(schizophrenia)}
\format{
  A data frame with 1603 observations on the following 9 variables.
  \describe{
    \item{\code{id}}{a numeric vector indicating the unique patient 
      identifier.}
    \item{\code{imps79}}{a numeric vector from item 79 on the Inpatient 
      Multidimensional Psychiatric Scale indicating severity of illness.  The 
      scores were applied to be interpreted as follows:
      \itemize{
        \item{\code{1} = normal, not ill at all},
        \item{\code{2} = borderline mentally ill},
        \item{\code{3} = mildly ill},
        \item{\code{4} = moderately ill},
        \item{\code{5} = markedly ill},
        \item{\code{6} = severely ill},
        \item{\code{7} = among the most extremely ill}.
      }
    }
    \item{\code{imps79b}}{a binary version of imps79.}
    \item{\code{imps79o}}{an ordinally scaled version of imps79.}
    \item{\code{int}}{a numeric vector of ones; used in stand-alone package to 
      indicate intercept.}
    \item{\code{TxDrug}}{a numeric vector indicating treatment with drug 
      \code{1} or placebo \code{0}.}
    \item{\code{Week}}{a numeric vector indicating time, in weeks.}
    \item{\code{SqrtWeek}}{the square root of the Week variable.}
    \item{\code{TxSWeek}}{a variable representing the \code{TxDrug * Week} 
      interaction.}
  }
}

\source{
  This data and most of this documentation was taken from the \CRANpkg{mixor} 
  package, which is no longer from CRAN. The references below mostly reference 
  the deprecated \CRANpkg{mixor} program.
}
\references{
  \itemize{
    \item \insertRef{hedeker1996mixor}{lme4}
    \item \insertRef{hedeker2006longitudinal}{lme4}
  }
}
\examples{
schmod <- glmer(imps79 ~ TxDrug * Week + (1 | id),
                data = schizophrenia, family = Gamma(link = "log"))
}
\keyword{datasets}

\name{factorize}
\alias{factorize}
\title{Attempt to convert grouping variables to factors}
\usage{
factorize(x,frloc,char.only=FALSE)
}
\arguments{
  \item{x}{a formula}
  \item{frloc}{a data frame}
  \item{char.only}{(logical) convert only \code{character} variables to factors?}
}
\description{
  If variables within a data frame are not factors, try to
  convert them.  Not intended for end-user use; this is
  a utility function that needs to be exported, for technical reasons.
}
\value{
  a copy of the data frame with factors converted
}
\keyword{models}
\keyword{utilities}

\name{nloptwrap}
\alias{nloptwrap}
\alias{nlminbwrap}
\title{Wrappers for additional optimizers}
\description{
  Wrappers to allow use of alternative optimizers, from the \command{NLopt}
  library (via \CRANpkg{nloptr}) or elsewhere, for the nonlinear optimization stage.
}
\usage{
nloptwrap (par, fn, lower, upper, control = list(), ...)
nlminbwrap(par, fn, lower, upper, control = list(), ...)
}
\arguments{
  \item{par}{starting parameter vector}
  \item{fn}{objective function}
  \item{lower, upper}{numeric vector of lower and upper bounds.}
  \item{control}{\code{\link{list}} of control parameters, corresponding to
    \code{optCtrl = *}, e.g., in \code{\link{lmerControl}()}.  For
    \code{nloptwrap}, see \code{defaultControl} in \sQuote{Details}.}
  \item{\dots}{additional arguments to be passed to objective function}
}
\value{
  \item{par}{estimated parameters}
  \item{fval}{objective function value at minimum}
  \item{feval}{number of function evaluations}
  \item{conv}{convergence code (0 if no error)}
  \item{message}{convergence message}
}
\details{
Using alternative optimizers is an important trouble-shooting
tool for mixed models.  These wrappers provide convenient access to
the optimizers provided by Steven Johnson's \code{NLopt} library
(via the \CRANpkg{nloptr} R package), and to the \code{\link{nlminb}}
optimizer from base R.  \code{nlminb} is also available via the
\CRANpkg{optimx} package; this wrapper provides access to \code{nlminb()}
without the need to install/link the package, and without the additional
post-fitting checks that are implemented by \CRANpkg{optimx} (see examples
below).

  One important difference between the \CRANpkg{nloptr}-provided
  implementation of BOBYQA and the \CRANpkg{minqa}-provided version
  accessible via \code{optimizer="bobyqa"} is that it provides simpler
  access to optimization tolerances.  \code{\link[minqa]{bobyqa}} provides
  only the \code{rhoend} parameter (\dQuote{[t]he smallest value of the
  trust region radius that is allowed}), while \pkg{nloptr} provides a more
  standard set of tolerances for relative or absolute change in the
  objective function or the parameter values (\code{ftol_rel},
  \code{ftol_abs}, \code{xtol_rel}, \code{xtol_abs}).

  The default (empty) \code{control} list corresponds to the following settings:
  \describe{
    \item{\code{nlminbwrap}:}{\code{control} exactly corresponds to
      \code{\link{nlminb}()}'s defaults, see there.}
    \item{\code{nloptwrap}:}{\code{environment(nloptwrap)$defaultControl}
      contains the defaults, notably \code{algorithm = "NLOPT_LN_BOBYQA"}.

      \code{nloptr::nloptr.print.options()} shows and explains the many
      possible algorithm and options.}
  }
}

\examples{
library(lme4)
ls.str(environment(nloptwrap)) # 'defaultControl' algorithm "NLOPT_LN_BOBYQA"
## Note that  'optimizer =  "nloptwrap"' is now the default for lmer() :
fm1 <- lmer(Reaction ~ Days + (Days|Subject), sleepstudy)
## tighten tolerances
fm1B <- update(fm1, control= lmerControl(optCtrl = list(xtol_abs=1e-8, ftol_abs=1e-8)))
## run for longer (no effect in this case)
fm1C <- update(fm1,control = lmerControl(optCtrl = list(maxeval=10000)))

  logLik(fm1B) - logLik(fm1)  ## small difference in log likelihood
c(logLik(fm1C) - logLik(fm1)) ## no difference in LL
## Nelder-Mead
fm1_nloptr_NM <- update(fm1, control=
                  lmerControl(optimizer = "nloptwrap",
                              optCtrl = list(algorithm = "NLOPT_LN_NELDERMEAD")))
## other nlOpt algorithm options include NLOPT_LN_COBYLA, NLOPT_LN_SBPLX, see
if(interactive())% (the package *is* installed w/ 'lme4')
  nloptr::nloptr.print.options()

fm1_nlminb <- update(fm1, control=lmerControl(optimizer = "nlminbwrap"))
if (require(optimx)) { ## the 'optimx'-based nlminb :
  fm1_nlminb2 <- update(fm1, control=
                lmerControl(optimizer = "optimx",
                            optCtrl = list(method="nlminb", kkt=FALSE)))
  cat("Likelihood difference (typically zero):  ",
      c(logLik(fm1_nlminb) - logLik(fm1_nlminb2)), "\n")
}


}
\author{Gabor Grothendieck (\code{nlminbwrap})}% Ben Bolker for nloptwrap

\name{rePos}
\alias{rePos}
\title{Generator object for the rePos (random-effects positions) class}
\usage{
  rePos(...)
}
\arguments{
  \item{\dots}{Argument list (see Note).}
}
\description{
  The generator object for the \code{\linkS4class{rePos}}
  class used to determine the positions and orders of
  random effects associated with particular random-effects
  terms in the model.
}
\note{
  Arguments to the \code{new} methods must be named
  arguments. \code{mer}, an object of class
  \code{"\linkS4class{merMod}"}, is the only
  required/expected argument.
}
\section{Methods}{
  \describe{ \item{\code{new(mer=mer)}}{Create a new
  \code{\linkS4class{rePos}} object.} }
}
\seealso{
  \code{\linkS4class{rePos}}
}
\keyword{classes}

\name{merPredD}
\alias{merPredD}
\title{Generator object for the \code{\linkS4class{merPredD}} class}
\usage{
merPredD(...)
}
\arguments{
  \item{...}{List of arguments (see Note).}
}
\description{
  The generator object for the
  \code{\linkS4class{merPredD}} reference class. Such an
  object is primarily used through its \code{new} method.
}
\note{
  \code{merPredD(...)} is a short form of \code{new("merPredD", ...)}
  to create a new \code{\linkS4class{merPredD}} object and the
  \code{...} must be named arguments, \code{(X, Zt, Lambdat, Lind,
    theta,n)}:
  \describe{
    \item{\code{X}:}{dense model matrix for the fixed-effects parameters, to be
      stored in the \code{X} field.}
    \item{\code{Zt}:}{transpose of the sparse model matrix for the random
      effects.  It is stored in the \code{Zt} field.}
    \item{\code{Lambdat}:}{transpose of the sparse lower triangular relative
      variance factor (stored in the \code{Lambdat} field).}
    \item{\code{Lind}:}{integer vector of the same length as the \code{x} slot in the
      \code{Lambdat} field.  Its elements should be in the range 1 to
      the length of the \code{theta} field.}
    \item{\code{theta}:}{numeric vector of variance component parameters (stored
      in the \code{theta} field).}
    \item{\code{n}:}{sample size, usually \code{nrow(X)}.}
  }
}
\seealso{
  The class definition, \code{\linkS4class{merPredD}}, also for examples.
}
\keyword{classes}

\name{merPredD-class}
\alias{merPredD-class}
\title{Class \code{"merPredD"} - a Dense Predictor Reference Class}
\description{
  A reference class (see mother class definition
  \code{"\linkS4class{envRefClass}"} for a mixed-effects model predictor
  module with a dense model matrix for the fixed-effects parameters.
  The reference class is associated with a C++ class of the same name.
  As is customary, the generator object, \code{\link{merPredD}}, for the
  class has the same name as the class.
}
\note{
  Objects from this reference class correspond to objects in a C++
  class.  Methods are invoked on the C++ class object using the external
  pointer in the \code{Ptr} field.  When saving such an object the
  external pointer is converted to a null pointer, which is why there
  are redundant fields containing enough information as \R objects to be
  able to regenerate the C++ object.  The convention is that a field
  whose name begins with an upper-case letter is an \R object and the
  corresponding field, whose name begins with the lower-case letter is a
  method.  References to the external pointer should be through the
  method, not directly through the \code{Ptr} field.
}
\examples{
showClass("merPredD")
pp <- slot(lmer(Yield ~ 1|Batch, Dyestuff), "pp")
stopifnot(is(pp, "merPredD"))
str(pp) # an overview of all fields and methods' names.
}
\seealso{
  \code{\link{lmer}}, \code{\link{glmer}},
  \code{\link{nlmer}}, \code{\link{merPredD}},
  \code{\linkS4class{merMod}}.
}
\keyword{classes}

\name{prt-utilities}
\title{Print and Summary Method Utilities for Mixed Effects}
\alias{.prt.methTit}
\alias{.prt.VC}
\alias{.prt.aictab}
\alias{.prt.call}
\alias{.prt.family}
\alias{.prt.grps}
\alias{.prt.methTit}
\alias{.prt.resids}
\alias{.prt.warn}
\alias{formatVC}
\alias{llikAIC}
\alias{methTitle}
\description{
  The \code{\link{print}}, \code{\link{summary}} methods (including the
  \code{print} for the \code{summary()} result) in \pkg{lme4} are
  modular, using about ten small utility functions.  Other packages,
  building on \pkg{lme4} can use the same utilities for ease of
  programming and consistency of output.

  Notably see the Examples.

  \code{llikAIC()} extracts the  log likelihood, AIC, and related
  statics from a Fitted LMM.

  \code{formatVC()} \dQuote{format()}s the \code{\link{VarCorr}} matrix of the
  random effects -- for \code{\link{print}()}ing and
  \code{\link{show}()}ing; it is also the \dQuote{workhorse} of
  \code{.prt.VC()}, and returns a \code{\link{character}} matrix.

  \code{.prt.*()} all use \code{\link{cat}} and \code{\link{print}} to
  produce output.
}
\usage{
llikAIC(object, devianceFUN = devCrit, chkREML = TRUE,
        devcomp = object@devcomp)

methTitle(dims)

.prt.methTit(mtit, class)
.prt.family (famL)
.prt.resids (resids, digits, title = "Scaled residuals:", \dots)
.prt.call   (call, long = TRUE)
.prt.aictab (aictab, digits = 1)
.prt.grps   (ngrps, nobs)
.prt.warn   (optinfo, summary = FALSE, \dots)

.prt.VC (varcor, digits, comp = "Std.Dev.", formatter = format, \dots)
formatVC(varcor, digits = max(3, getOption("digits") - 2),
         corr_digits = max(2, digits-2),
         maxdim = 10,
         comp = "Std.Dev.", formatter = format,
         useScale = attr(varcor, "useSc"),
         \dots)
}
\arguments{
  %% llikAIC() :
  \item{object}{a LMM model fit}
  \item{devianceFUN}{the function to be used for computing the deviance;
    should not be changed for \pkg{lme4} created objects.}
  \item{chkREML}{optional logical indicating if \code{object} maybe a REML
    fit.}% use TRUE for \pkg{lme4} fits
  \item{devcomp}{for \pkg{lme4} always the equivalent of
    \code{object@devcomp}; here a \code{\link{list}}}%... FIXME

  %% methTitle():
  \item{dims}{for \pkg{lme4} always the equivalent of
    \code{object@devcomp$dims}, a named vector or list with components
    \code{"GLMM"}, \code{"NLMM"}, \code{"REML"}, and \code{"nAGQ"} of
    which the first two are \code{\link{logical}} scalars, and the latter
    two typically are \code{FALSE} or \code{\link{numeric}}.}

  %% .prt.methTit
  \item{mtit}{the result of \code{methTitle(object)}}
  \item{class}{typically \code{\link{class}(object)}.}

  %% .prt.family (famL)
  \item{famL}{a \code{\link{list}} with components \code{family} and
    \code{link}, each a \code{\link{character}} string; note that standard
    \R \code{\link{family}} objects can be used directly, as well.}

  %% .prt.resids (resids, digits, title = "Scaled residuals:", \dots)
  \item{resids}{numeric vector of model \code{\link{residuals}}.}
  \item{digits}{non-negative integer of (significant) digits to print minimally.}
  \item{title}{\code{\link{character}} string.}
  \item{\dots}{optional arguments passed on, e.g., to \code{\link{residuals}()}.}

  %% .prt.call   (call, long = TRUE)
  \item{call}{the \code{\link{call}} of the model fit; e.g., available
    via (generic) function \code{\link{getCall}()}.}
  \item{long}{logical indicating if the output may be long, e.g.,
    printing the \code{control} part of the call if there is one.}

  %% .prt.aictab (aictab, digits = 1)
  \item{aictab}{typically the \code{AICtab} component of the result of
    \code{llikAIC()}.}

  %% .prt.VC     (varcor, digits, comp, formatter = format, \dots)
  \item{varcor}{typically the result of \code{\link{VarCorr}()}.}
  \item{comp}{optional \code{\link{character}} vector of length 1 or 2,
    containing \code{"Std.Dev."} and/or \code{"Variance"}, indicating the
    columns to use.}
  \item{formatter}{a \code{\link{function}} used for formatting the numbers.}
  \item{corr_digits}{the number of significant digits for correlations.}
  \item{maxdim}{maximum dimensions (numbers of standard deviations/variances 
  and number of rows of correlation matrices) to report per random effects term.}
  %% .prt.grps   (ngrps, nobs)
  \item{ngrps}{integer (vector), typically the result of
    \code{\link{ngrps}(object)}.}
  \item{nobs}{integer; the number of observations, e.g., the result
    of \code{\link{nobs}}.}

  %% .prt.warn   (optinfo, summary = FALSE, \dots)
  \item{optinfo}{typically \code{object @ optinfo}, the optimization
    infos, including warnings if there were.}
  \item{summary}{logical}

  %% formatVC {additional to .prt.VC() etc above}
  \item{useScale}{(logical) whether the parent model estimates a scale parameter.}
}
\value{
  \code{llikAIC()} returns a \code{\link{list}} with components

  \item{logLik}{which is \code{\link{logLik}(object)}, and}
  \item{AICtab}{ a \dQuote{table} of \code{\link{AIC}}, \code{\link{BIC}},
    \code{\link{logLik}}, deviance and \code{\link{df.residual}()} values.}
}
\examples{
## Create a few "lme4 standard" models ------------------------------
fm1 <- lmer(Reaction ~ Days + (Days | Subject), sleepstudy)
fmM <- update(fm1, REML=FALSE) # -> Maximum Likelihood
fmQ <- update(fm1, . ~ Days + (Days | Subject))

gm1 <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
             data = cbpp, family = binomial)
gmA <- update(gm1, nAGQ = 5)


(lA1 <- llikAIC(fm1))
(lAM <- llikAIC(fmM))
(lAg <- llikAIC(gmA))

(m1 <- methTitle(fm1 @ devcomp $ dims))
(mM <- methTitle(fmM @ devcomp $ dims))
(mG <- methTitle(gm1 @ devcomp $ dims))
(mA <- methTitle(gmA @ devcomp $ dims))

.prt.methTit(m1, class(fm1))
.prt.methTit(mA, class(gmA))

.prt.family(gaussian())
.prt.family(binomial())
.prt.family( poisson())

.prt.resids(residuals(fm1), digits = 4)
.prt.resids(residuals(fmM), digits = 2)

.prt.call(getCall(fm1))
.prt.call(getCall(gm1))

.prt.aictab ( lA1 $ AICtab ) # REML
.prt.aictab ( lAM $ AICtab ) # ML --> AIC, BIC, ...

V1 <- VarCorr(fm1)
m <- formatVC(V1)

stopifnot(is.matrix(m), is.character(m), ncol(m) == 5)
print(m, quote = FALSE) ## prints all but the first line of .prt.VC() below:
.prt.VC( V1, digits = 4)
## Random effects:
##  Groups   Name        Std.Dev. Corr
##  Subject  (Intercept) 24.740
##           Days         5.922   0.066
##  Residual             25.592
p1 <- capture.output(V1)
p2 <- capture.output( print(m, quote=FALSE) )
pX <- capture.output( .prt.VC(V1, digits = max(3, getOption("digits")-2)) )
stopifnot(identical(p1, p2),
          identical(p1, pX[-1])) # [-1] : dropping 1st line

(Vq <- VarCorr(fmQ)) # default print()
print(Vq, comp = c("Std.Dev.", "Variance"))
print(Vq, comp = c("Std.Dev.", "Variance"), corr=FALSE)
print(Vq, comp = "Variance")

.prt.grps(ngrps = ngrps(fm1),
          nobs  = nobs (fm1))
## --> Number of obs: 180, groups:  Subject, 18

.prt.warn(fm1 @ optinfo) # nothing .. had no warnings
.prt.warn(fmQ @ optinfo) # (ditto)
}
\keyword{utilities}

\name{ranef}
\alias{ranef}
\alias{ranef.merMod}
\alias{dotplot.ranef.mer}
\alias{qqmath.ranef.mer}
\alias{as.data.frame.ranef.mer}
\title{Extract the modes of the random effects}
\usage{
\method{ranef}{merMod} (object, condVar = TRUE,
      drop = FALSE, whichel = names(ans), postVar = FALSE, ...)
\method{dotplot}{ranef.mer} (x, data, main = TRUE, transf = I, level = 0.95, ...)
\method{qqmath}{ranef.mer} (x, data, main = TRUE, level = 0.95, ...)
\method{as.data.frame}{ranef.mer} (x, ...)
}
\arguments{
  \item{object}{an object of a class of fitted models with
    random effects, typically a
    \code{\linkS4class{merMod}} object.}
  \item{condVar}{a logical argument indicating if
    the conditional variance-covariance matrices of the random effects
    should be added as an attribute.}
  \item{drop}{should components of the return value that would be data frames
    with a single column, usually a column called
    \sQuote{\code{(Intercept)}}, be returned as named
    vectors instead?}
  \item{whichel}{character vector of names of
    grouping factors for which the random effects should be
    returned.}
  \item{postVar}{a (deprecated) synonym for \code{condVar}}
  \item{x}{a random-effects object (of class \code{ranef.mer})
    produced by \code{ranef}}
  \item{main}{include a main title, indicating the grouping factor,
    on each sub-plot?}
  \item{transf}{transformation for random effects: for example,
    \code{exp} for plotting parameters from a (generalized)
    logistic regression on the odds rather than log-odds scale}
  \item{data}{This argument is required by the \code{dotplot}
    and \code{qqmath} generic methods, but is not actually used.}
  \item{level}{confidence level for confidence intervals}
  \item{\dots}{some methods for these generic functions
    require additional arguments.}
}
\value{
  \itemize{
    \item{
      From \code{ranef}:
      An object of class \code{ranef.mer} composed of
      a list of data frames, one for each grouping factor for
      the random effects.  The number of rows in the data frame
      is the number of levels of the grouping factor.  The
      number of columns is the dimension of the random effect
      associated with each level of the factor.

      If \code{condVar} is \code{TRUE} each of the data frames
      has an attribute called \code{"postVar"}.
      \itemize{
	\item If there is
	a single random-effects term for a given grouping factor,
	this attribute is a
	three-dimensional array with symmetric faces; each face
	contains the variance-covariance matrix for a particular
	level of the grouping factor.
	\item If there is more than one random-effects term for
	a given grouping factor (e.g. \code{(1|f) + (0+x|f)}),
	this attribute is a list of arrays as described above,
	one for each term.
      }
      (The name
      of this attribute is a historical artifact,
      and may be changed to \code{condVar} at some point in the future.)

      When \code{drop} is \code{TRUE} any components that would
      be data frames of a single column are converted to named
      numeric vectors.
    }
    \item{
      From \code{as.data.frame}:

      This function converts the random effects to a "long format"
      data frame with columns
      \describe{
	\item{grpvar}{grouping variable}
	\item{term}{random-effects term, e.g. \dQuote{(Intercept)} or
	  \dQuote{Days}}
	\item{grp}{level of the grouping variable (e.g., which Subject)}
	\item{condval}{value of the conditional mean}
	\item{condsd}{conditional standard deviation}
      }
    } % as.data.frame value
  } % itemize
} % value
\description{
  A generic function to extract the conditional modes of
  the random effects from a fitted model object.  For
  linear mixed models the conditional modes of the random
  effects are also the conditional means.
}
\details{
  If grouping factor i has k levels and j random effects
  per level the ith component of the list returned by
  \code{ranef} is a data frame with k rows and j columns.
  If \code{condVar} is \code{TRUE} the \code{"postVar"}
  attribute is an array of dimension j by j by k (or a list
  of such arrays).  The kth
  face of this array is a positive definite symmetric j by
  j matrix.  If there is only one grouping factor in the
  model the variance-covariance matrix for the entire
  random effects vector, conditional on the estimates of
  the model parameters and on the data, will be block
  diagonal; this j by j matrix is the kth diagonal
  block.  With multiple grouping factors the faces of the
  \code{"postVar"} attributes are still the diagonal blocks
  of this conditional variance-covariance matrix but the
  matrix itself is no longer block diagonal.
}
\note{
  To produce a (list of) \dQuote{caterpillar plots} of the random
  effects apply \code{\link[lattice:xyplot]{dotplot}} to
  the result of a call to \code{ranef} with \code{condVar =
    TRUE}; \code{\link[lattice:xyplot]{qqmath}} will generate
  a list of Q-Q plots.
}
\examples{
library(lattice) ## for dotplot, qqmath
fm1 <- lmer(Reaction ~ Days + (Days|Subject), sleepstudy)
fm2 <- lmer(Reaction ~ Days + (1|Subject) + (0+Days|Subject), sleepstudy)
fm3 <- lmer(diameter ~ (1|plate) + (1|sample), Penicillin)
ranef(fm1)
str(rr1 <- ranef(fm1))
dotplot(rr1)  ## default
qqmath(rr1)
## specify free scales in order to make Day effects more visible
dotplot(rr1,scales = list(x = list(relation = 'free')))[["Subject"]]
## plot options: ... can specify appearance of vertical lines with
## lty.v, col.line.v, lwd.v, etc..
dotplot(rr1, lty = 3, lty.v = 2, col.line.v = "purple",
        col = "red", col.line.h = "gray")
ranef(fm2)
op <- options(digits = 4)
ranef(fm3, drop = TRUE)
options(op)
## as.data.frame() provides RE's and conditional standard deviations:
str(dd <- as.data.frame(rr1))
if (require(ggplot2)) {
    ggplot(dd, aes(y=grp,x=condval)) +
        geom_point() + facet_wrap(~term,scales="free_x") +
        geom_errorbarh(aes(xmin=condval -2*condsd,
                           xmax=condval +2*condsd), height=0)
}
}
\keyword{methods}
\keyword{models}

\name{devfun2}
\alias{devfun2}
\title{Deviance Function in Terms of Standard Deviations/Correlations}
\description{
  The deviance is profiled with respect to the fixed-effects
  parameters but not with respect to sigma; that is, the
  function takes parameters for the variance-covariance parameters
  and for the residual standard deviation.  The random-effects
  variance-covariance parameters are on the standard deviation/correlation
  scale, not the theta (Cholesky factor) scale.
}
\usage{
devfun2(fm, useSc = if(isLMM(fm)) TRUE else NA,
        scale = c("sdcor", "varcov"), ...)
}
\arguments{
  \item{fm}{a fitted model inheriting from  class \code{"\linkS4class{merMod}"}.}
  \item{useSc}{(\code{\link{logical}}) indicating whether a scale parameter
    has been in the model or should be used.}% FIXME, see also ../R/profile.R
  \item{scale}{a character string indicating the scale of the argument
    of the deviance function.}
  \item{\dots}{
    arguments passed to the internal \code{profnames} function
    (\code{signames=TRUE} to use old-style \code{.sigxx names},
    \code{FALSE} uses (sd_cor|xx);
    also \code{prefix=c("sd","cor")})
  }
}
\value{
  Returns a function that takes a vector of standard deviations and
  correlations and returns the deviance (or REML criterion).  The
  function has additional attributes
  \describe{
    \item{optimum}{a named vector giving the parameter values
      at the optimum}
    \item{basedev}{the deviance at the optimum, (i.e., \emph{not} the
      REML criterion).}
    \item{thopt}{the optimal variance-covariance parameters on the
      \dQuote{theta} (Cholesky factor) scale}
    \item{stderr}{standard errors of fixed effect parameters}
  }
}
\note{
 Even if the original model was fitted using \code{REML=TRUE} as by default
 with \code{\link{lmer}()}, this returns the deviance, i.e., the objective
 function for maximum (log) likelihood (ML).

 For the REML objective function, use \code{\link{getME}(fm, "devfun")}
 instead.
}
\examples{
m1 <- lmer(Reaction~Days+(Days|Subject),sleepstudy)
dd <- devfun2(m1, useSc=TRUE)
pp <- attr(dd, "optimum")
## extract variance-covariance and residual std dev parameters
sigpars <- pp[grepl("^\\\\.sig", names(pp))]
all.equal(unname(dd(sigpars)),deviance(refitML(m1)))
}
\keyword{utilities}

\name{getDoublevertDefault}
\alias{getDoublevertDefault}
\title{Retrieve default double-vertical-bar-expansion method}
\description{
  Retrieve information about whether random effect terms specified with
  double bars (\code{||}) should be split semantically into separate,
  intercept-free terms, or encoded with a \code{diag()} special
}
\usage{
getDoublevertDefault()
}
\value{
	a (1-element) character vector suitable for \code{reformulas::findbars_x}
	specifying whether to expand random-effects terms containing double bars (\code{||})
	as a set of independent random effects terms ("split": i.e., \code{(1+a+b||f)} becomes
	\code{(1|f) + (0+a|f) + (0+b|f)}) or whether to specify that the covariance matrix is
	diagonal ("diag_special": i.e., \code{diag(1+a+b|f)}).
}
\details{
    To set the global behaviour retrieved by \code{getDoublevertDefault()}
    to "diag_special", run \code{options(lme4.doublevert.default = "diag_special")}

    The default value of "split" is backward compatible with the behaviour of \code{lme4} <
    2.0; double bars do \emph{not} work as typically expected with factors
    as varying terms. 
    That is, if \code{g} is a factor
    then \code{(1 + g||f)} will expand (with method "split") to \code{(1|f) + (0 + g|f)}; the
    second term will \emph{not} produce a covariance matrix where the contrasts generated by
    the factor are independent.

    "diag_special" is probably the generally preferred option; by
    transforming \code{(1+a+b||f)}
    into \code{diag(1+a+b|f)}, it ensures that variation across groups
    in the contrasts coded by the factor will be independent. This matches
    the results of specifying a double-bar term in \code{afex::mixed()},
    or \code{glmmTMB}.

    Note that if \code{f} is a factor, even with "diag_special" set,
    models containing \code{(0+f||g)} vs. \code{(f||g)} will give
    different results. \code{(0+f||g)} creates a set of dummy variables
    that are indicator variables for each level of the factor, so the
    random effect term will describe variation in the intercept for each
    level of the factor. With default treatment contrasts set,
    \code{(f||g)} (equivalent to \code{(1+f||g)}
    because R automatically adds the intercept term) will estimate
    variation first in the intercept in the baseline level of the
    factor, then in the differences between observations in the baseline and successive
    levels of the factor (see \code{\link{contr.treatment}}).
    
  }
  

  

\name{allFit}
\alias{allFit}
\title{Refit a fitted model with all available optimizers to check convergence}
\usage{
allFit(object, meth.tab = NULL, data = NULL,
       verbose = TRUE,
       show.meth.tab = FALSE,
       maxfun = 1e5,
       parallel = c("no", "multicore", "snow", "future"),
       ncpus = getOption("allFit.ncpus", 1L), cl = NULL,
       catch.errs = TRUE,
       start_from_mle = TRUE)
}
\arguments{
  \item{object}{a fitted model.}
  \item{meth.tab}{a matrix (or data.frame) with columns.
    \describe{
      \item{method}{the name of a specific optimization method to pass
	to the optimizer (leave blank for built-in optimizers).}
      \item{optimizer}{the \code{optimizer} function to use.}
    }
  }
  \item{data}{data to be included with result (for later debugging etc.)}
  \item{verbose}{(logical) report progress in detail?}
  \item{show.meth.tab}{(logical) return table of methods?}
  \item{maxfun}{passed as part of \code{optCtrl} to set the maximum
    number of function evaluations: this is \emph{automatically}
    converted to the correct specification (e.g. \code{maxfun},
    \code{maxfeval}, \code{maxit}, etc.) for each optimizer.}
  \item{parallel}{the type of parallel operation to be used, if any.
    If \code{"no"}, sequential processing is used. If \code{"multicore"}
    and \R is running on a Unix-alike operating system, \code{mclapply}
    is used; if \code{"snow"}, \code{parLapply} is used; if \code{"future"},
    \code{future.apply::future_lapply} is used.}
  \item{ncpus}{number of processes to be used in parallel operation when
    \code{parallel} is either \code{"multicore"} or \code{"snow"}.
    If \code{ncpus == 1}, \code{parallel = "multicore"} and
    \code{parallel = "snow"} are ignored and the same as
    \code{parallel = "no"}. The default value can be controlled by an
    \R option.}
  \item{cl}{An optional \pkg{parallel} or \pkg{snow} cluster; if specified,
    arguments \code{parallel} and \code{ncpus} are ignored.
    If not supplied and \code{parallel = "snow"}, a temporary cluster with
    \code{ncpus} nodes is created for the duration of the call.}
  \item{catch.errs}{(logical) wrap model fits in \code{tryCatch} clause
    to skip over errors? (\code{catch.errs=FALSE} is probably only
    useful for debugging).}
  \item{start_from_mle}{(logical) if \code{TRUE}, initialize the 
    refitting process using starting values from the model’s MLE.}
}
\value{
  an object of type \code{allFit}, which is a list of fitted \code{merMod} objects (unless \code{show.meth.tab} is
  specified, in which case a data frame of methods is returned). The
  \code{summary} method for this class
  extracts tables with a variety of useful information
  about the different fits (see examples).
}
\description{
   Attempt to re-fit a [g]lmer model with a range of optimizers.
The default is to use all known optimizers for R that satisfy the
requirements (i.e. they do not require functions and allow
box constraints: see \sQuote{optimizer} in \code{\link{lmerControl}}).
These optimizers fall in four categories; (i) built-in
(minqa::bobyqa, lme4::Nelder_Mead, nlminbwrap), (ii) wrapped via optimx
(most of optimx's optimizers that allow box constraints require
an explicit gradient function to be specified; the two provided
here are the base R functions that can be accessed via optimx),
(iii) wrapped via nloptr (see examples for the list of options),
(iv) \sQuote{dfoptim::nmkb} (via the (unexported) \code{nmkbw} wrapper:
this appears as \sQuote{nmkbw} in \code{meth.tab})
}
\details{
\itemize{
\item Needs packages \code{optimx}, and \code{dfoptim} to use all optimizers.
\item If you are using \code{parallel="snow"} (e.g. when running in
parallel on Windows), you will need to set up a cluster yourself and run
\code{clusterEvalQ(cl,library("lme4"))} before calling
\code{allFit} to make sure that the
\code{lme4} package is loaded on all of the workers.
\item Control arguments in \code{control$optCtrl} that are unused by a particular optimizer will be \emph{silently} ignored (in particular, the \code{maxfun} specification is only respected by \code{bobyqa}, \code{Nelder_Mead}, and \code{nmkbw}).
\item Because \code{allFit} works by calling \code{update}, it may be fragile if the original model call contains references to variables, especially if they were originally defined in other environments or no longer exist when \code{allFit} is called.
}
}
\seealso{\code{slice},\code{slice2D} from the \CRANpkg{bbmle} package}
\examples{
if (interactive()) {
library(lme4)
  gm1 <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
             data = cbpp, family = binomial)
  ## show available methods
  allFit(show.meth.tab=TRUE) 
  gm_all <- allFit(gm1)
  ss <- summary(gm_all)
  ss$which.OK            ## logical vector: which optimizers worked?
  ## the other components only contain values for the optimizers that worked
  ss$llik                ## vector of log-likelihoods
  ss$fixef               ## table of fixed effects
  ss$sdcor               ## table of random effect SDs and correlations
  ss$theta               ## table of random effects parameters, Cholesky scale
} %% interactive()
\dontrun{
  ## Parallel examples for Windows
  nc <- max(detectCores() - 1L, 1L, na.rm = TRUE)
  optCls <- makeCluster(nc)
  clusterEvalQ(optCls,library("lme4"))
  ### not necessary here because using a built-in
  ## data set, but in general you should clusterExport() your data
  clusterExport(optCls, "cbpp")
  system.time(af1 <- allFit(m0, parallel = 'snow', 
                          ncpus = nc, cl=optCls))
  stopCluster(optCls)
} %% dontrun
} %% examples
\keyword{models}

\name{getME}
\title{Extract or Get Generalized Components from a Fitted Mixed Effects Model}
\alias{getL}
\alias{getL,merMod-method}
\alias{getME}
\alias{getME.merMod}
\usage{
getME(object, name, ...)

\S3method{getME}{merMod}(object,
      name = c("X", "Z", "Zt", "Ztlist", "mmList", "y", "mu", "u", "b",
               "Gp", "Tp", "L", "Lambda", "Lambdat", "Lind", "Tlist",
               "A", "RX", "RZX", "sigma", "flist",
               "fixef", "beta", "theta", "ST", "par", "REML", "is_REML",
               "n_rtrms", "n_rfacs", "N", "n", "p", "q", "npar",
               "p_i", "l_i", "q_i", "k", "m_i", "m",
               "cnms", "devcomp", "offset", "lower", "devfun", "devarg",
               "glmer.nb.theta"),
      \dots)
}
\arguments{
  \item{object}{a fitted mixed-effects model of class
    \code{"\linkS4class{merMod}"}, i.e., typically the result of
    \code{\link{lmer}()}, \code{\link{glmer}()} or \code{\link{nlmer}()}.}

  \item{name}{a character vector specifying the name(s) of
    the \dQuote{component}.  If \code{length(name) > 1} or if \code{name
    = "ALL"}, a named \code{\link{list}} of components will be returned.  Possible values are:\cr
    \describe{
      \item{\code{"X"}}{fixed-effects model matrix}
      \item{\code{"Z"}}{random-effects model matrix}
      \item{\code{"Zt"}}{transpose
	of random-effects model matrix.  Note that the structure
	of \code{Zt} has changed since \code{lme4.0}; to get a
	backward-compatible structure, use
	\code{do.call(Matrix::rBind,getME(.,"Ztlist"))}}
      \item{\code{"Ztlist"}}{list of components of the transpose of the
	random-effects model matrix, separated by individual
	variance component}
      \item{\code{"mmList"}}{list of raw model matrices associated with random
	effects terms}
      \item{\code{"y"}}{response vector}
      \item{\code{"mu"}}{conditional mean of the response}
      \item{\code{"u"}}{conditional mode of the \dQuote{spherical}
	random effects variable}
      \item{\code{"b"}}{conditional mode of the
	random effects variable}
      \item{\code{"Gp"}}{groups pointer vector.
	A pointer to the beginning of each group of random
	effects corresponding to the random-effects terms,
	beginning with 0 and including a final element giving the
	total number of random effects}
      \item{\code{"Tp"}}{theta pointer vector.  A pointer to the beginning of the theta
	sub-vectors corresponding to the random-effects terms,
	beginning with 0 and including a final element giving the
	number of thetas.}
      \item{\code{"L"}}{sparse Cholesky factor of the penalized random-effects model.}
      \item{\code{"Lambda"}}{relative covariance factor \eqn{\Lambda}{Lambda} of the random effects.}
      \item{\code{"Lambdat"}}{transpose \eqn{\Lambda'}{Lambda'} of \eqn{\Lambda}{Lambda} above.}
      \item{\code{"Lind"}}{index vector for inserting elements of
	\eqn{\theta}{theta} into the nonzeros of \eqn{\Lambda}{Lambda}.}
      \item{\code{"Tlist"}}{vector of template matrices from which the blocks of
	\eqn{\Lambda}{Lambda} are generated.}
      \item{\code{"A"}}{Scaled sparse model matrix (class
	\code{"\link[Matrix:dgCMatrix-class]{dgCMatrix}"}) for
	the unit, orthogonal random effects, \eqn{U}, equal to
	\code{getME(.,"Zt") \%*\% getME(.,"Lambdat")}}
      \item{\code{"RX"}}{Cholesky factor for the fixed-effects parameters}
      \item{\code{"RZX"}}{cross-term in the full Cholesky factor}
      \item{\code{"sigma"}}{residual standard error; note that \code{sigma(object)} is preferred.}
      \item{\code{"flist"}}{a list of the grouping variables (factors)
	involved in the random effect terms}
      \item{\code{"fixef"}}{fixed-effects parameter estimates}
      \item{\code{"beta"}}{fixed-effects parameter estimates (identical
	to the result of \code{\link{fixef}}, but without names)}
      \item{\code{"theta"}}{lower triangular entries of Cholesky
	factors of relative random effect covariance matrices, stored in
	column-major order.}
      \item{\code{"par"}}{relative random effect covariance parameter
	estimates.}
      \item{\code{"ST"}}{A list of S and T factors in the TSST' Cholesky
      factorization of the relative variance matrices of the random
      effects associated with each random-effects term.  The unit lower
      triangular matrix, \eqn{T}, and the diagonal matrix, \eqn{S}, for
      each term are stored as a single matrix with diagonal elements
      from \eqn{S} and off-diagonal elements from \eqn{T}.}
      \item{\code{"n_rtrms"}}{number of random-effects terms}
      \item{\code{"n_rfacs"}}{number of distinct random-effects grouping factors}
      \item{\code{"N"}}{number of rows of \code{X}}
      \item{\code{"n"}}{length of the response vector, \code{y}}
      \item{\code{"p"}}{number of columns of the fixed effects model matrix, \code{X}}
      \item{\code{"q"}}{number of columns of the random effects model matrix, \code{Z}}
      \item{\code{"npar"}}{number of optimization parameters for
        random effects covariance matrices (see \code{npar} under
        the \code{name} argument above)}
      \item{\code{"p_i"}}{numbers of columns of the raw model matrices, \code{mmList}}
      \item{\code{"l_i"}}{numbers of levels of the grouping factors}
      \item{\code{"q_i"}}{numbers of columns of the term-wise model matrices, \code{ZtList}}
      \item{\code{"k"}}{number of random effects terms}
      \item{\code{"m_i"}}{numbers of covariance parameters in each term}
      \item{\code{"m"}}{total number of covariance parameters, i.e., the
	same as \code{dims@nth} below.}
      \item{\code{"cnms"}}{the \dQuote{component names}, a \code{\link{list}}.}
      \item{\code{"REML"}}{\code{0} indicates the model was fitted by maximum
	likelihood, any other positive integer indicates fitting by
	restricted maximum likelihood}
      \item{\code{"is_REML"}}{same as the result of \code{\link{isREML}(.)}}
      \item{\code{"devcomp"}}{a list consisting of a named numeric vector,
        \code{cmp}, and a named integer vector, \code{dims}, describing
        the fitted model.  The elements of \code{cmp} are:\cr
	\describe{
          \item{ldL2}{twice the log determinant of \code{L}}
          \item{ldRX2}{twice the log determinant of \code{RX}}
          \item{wrss}{weighted residual sum of squares}
          \item{ussq}{squared length of \code{u}}
          \item{pwrss}{penalized weighted residual sum of squares,
            \dQuote{wrss + ussq}}
	  \item{drsum}{sum of residual deviance (GLMMs only)}
	  \item{REML}{REML criterion at optimum (LMMs fit
            by REML only)}
	  \item{dev}{deviance criterion at optimum
            (models fit by ML only)}
	  \item{sigmaML}{ML estimate of residual standard deviation}
	  \item{sigmaREML}{REML estimate of residual standard deviation}
	  \item{tolPwrss}{tolerance for declaring convergence in the
	  penalized iteratively weighted residual sum-of-squares (GLMMs only)}
	} The elements of \code{dims} are:
	\describe{
	  \item{N}{number of rows of \code{X}}
          \item{n}{length of \code{y}}
	  \item{p}{number of columns of \code{X}}
	  \item{nmp}{\code{n-p}}
	  \item{nth}{length of \code{theta}}
	  \item{q}{number of columns of \code{Z}}
	  \item{nAGQ}{see \code{\link{glmer}}}
          \item{compDev}{see \code{\link{glmerControl}}}
          \item{useSc}{\code{TRUE} if model has a scale parameter}
	  \item{reTrms}{number of random effects terms}
          \item{REML}{\code{0} indicates the model was fitted by maximum
	    likelihood, any other positive integer indicates fitting by
	    restricted maximum likelihood}
          \item{GLMM}{\code{TRUE} if a GLMM}
	  \item{NLMM}{\code{TRUE} if an NLMM}
    \item{npar}{number of optimization parameters for random effects
      covariance matrices;
      equals \code{nth} for models with only unstructured
      covariances, less than \code{nth} for models including any
      structured covariance models (e.g., \code{ar1})}
	}
}
      \item{\code{"offset"}}{model offset}
      \item{\code{"lower"}}{lower bounds on random-effects model
	parameters (i.e, "theta" parameters). In order to constrain
	random effects covariance matrices to be semi-positive-definite,
	this vector is equal to 0 for elements of
	the \code{theta} vector corresponding to diagonal elements of
	the Cholesky factor, \code{-Inf}
	otherwise. (\code{getME(.,"lower")==0} can be used as a test to
	identify diagonal elements, as in \code{isSingular}.)
      }
      \item{\code{"devfun"}}{deviance function.}
      \item{\code{"devarg"}}{deviance function argument, the
	concatenation of \code{getME(object, "par")} and
	(if \code{object} is a \code{\linkS4class{glmerMod}} object)
	\code{getME(object, "beta")}.}
      \item{\code{"glmer.nb.theta"}}{negative binomial \eqn{\theta} parameter,
	only for \code{\link{glmer.nb}}.}

      %% -- keep at the very end:
      \item{\code{"ALL"}}{get all of the above as a \code{\link{list}}.}
    }
  }
  \item{\dots}{currently unused in \pkg{lme4}, potentially further
    arguments in methods.}
}
\value{
  Unspecified, as very much depending on the \code{\link{name}}.
}
\description{
  Extract (or \dQuote{get}) \dQuote{components} -- in a
  generalized sense -- from a fitted mixed-effects model,
  i.e., (in this version of the package) from an object of
  class \code{"\linkS4class{merMod}"}.
}
\details{
  The goal is to provide \dQuote{everything a user may
  want} from a fitted \code{"merMod"} object \emph{as far}
  as it is not available by methods, such as
  \code{\link{fixef}}, \code{\link{ranef}},
  \code{\link{vcov}}, etc.
}
\seealso{
  \code{\link{getCall}()}.  More standard methods for \code{"merMod"}
  objects, such as \code{\link{ranef}}, \code{\link{fixef}},
  \code{\link{vcov}}, etc.: see \code{methods(class="merMod")}
}
\examples{
## shows many methods you should consider *before* using getME():
methods(class = "merMod")

(fm1 <- lmer(Reaction ~ Days + (Days|Subject), sleepstudy))
Z <- getME(fm1, "Z")
stopifnot(is(Z, "CsparseMatrix"),
          c(180,36) == dim(Z),
	  all.equal(fixef(fm1), b1 <- getME(fm1, "beta"),
		    check.attributes=FALSE, tolerance = 0))

## A way to get *all* getME()s :
## internal consistency check ensuring that all work:
parts <- getME(fm1, "ALL")
str(parts, max=2)
stopifnot(identical(Z,  parts $ Z),
          identical(b1, parts $ beta))
}
\keyword{utilities}

\name{gopherdat2}
\alias{gopherdat2}
\docType{data}
\title{
  Gopher tortoises shell remains
}
\description{
  Data of fresh Gopher tortoise shell remains \insertCite{ozgul2009upper}{lme4}.
}
\usage{data("gopherdat2")}
\format{
  A data frame with 30 observations on the following 7 variables.
  \describe{
    \item{\code{Site}}{a factor representing the sampling site.  There are 
      \code{10} levels: 
        \itemize{
          \item \code{BS} (Big Shoals State Park).
          \item \code{CB} (Camp Blanding Wildlife Management Area).
          \item \code{Cent} (privately-owned property in central Florida).
          \item \code{CF} (Cecil Field/Branan Field Wildlife and Environmental 
            Area).
          \item \code{FC} (Fort Cooper State Park).
          \item \code{FE} (Flying Eagle Wildlife Management Area).
          \item \code{GH} (Gold Head Branch State Park).
          \item \code{Old} (Perry Oldenburg Wildlife and Environmental Area).
          \item \code{Ord} (Ordway-Swisher Biological Station). 
          \item \code{TE} (Tenoroc Fish Management Area).
        }
      }
    \item{\code{year}}{a numeric vector of the sampling year.}  
    \item{\code{shells}}{a numeric vector of the number of shells found.}
    \item{\code{Area}}{a numeric vector representing site area 
      (units unknown).}
    \item{\code{density}}{a numeric vector representing population density.}
    \item{\code{prev}}{a numeric vector representing the seroprevalence 
      (frequency of antibodies to disease) of \emph{M. agassizi}. }
  }
}
\details{
  Details of the study is described in \insertCite{ozgul2009upper}{lme4} as
  follows:
  "The fieldwork was conducted between 2003 and 2006, during late spring/summer 
  (May–September). Systematic surveys were conducted to locate tortoise burrows 
  and shell remains from deceased tortoises at study sites and consisted of a 
  line of four to eight observers spaced 10 m apart walking parallel transects 
  across the study area."
}
\source{
  \insertCite{ozgul2009upper}{lme4}
}
\references{
  \insertRef{ozgul2009upper}{lme4}
}
\examples{
## Simple model gives a singular fit:
gopher_glmer <- glmer(shells ~ factor(year) + prev + offset(log(Area))
                   + (1|Site), data = gopherdat2, family = "poisson")
## The site-level variance for this model is indeed zero:
VarCorr(gopher_glmer)
## So a Poisson GLM gives the same answer here:
gopher_glm <- glm(shells ~ factor(year) + prev + offset(log(Area)),
                  data = gopherdat2, family = "poisson")
all.equal(fixef(gopher_glmer), coef(gopher_glm))
}
\keyword{datasets}

\name{convergence}
\alias{convergence}
\title{Assessing Convergence for Fitted Models}

\description{
  \code{[g]lmer} fits may produce convergence warnings;
  these do \strong{not} necessarily mean the fit is incorrect (see
  \dQuote{Theoretical details} below). The following steps are recommended
  assessing and resolving convergence warnings
  (also see examples below):

  \itemize{
    \item double-check the model specification and the data
    \item adjust stopping (convergence) tolerances for the nonlinear optimizer,
    using the \code{optCtrl} argument to \code{[g]lmerControl}
    (see \dQuote{Convergence controls} below)
    \item center and scale continuous predictor variables (e.g. with \code{\link{scale}})
    \item double-check the Hessian calculation with the more expensive
    Richardson extrapolation method (see examples)
    \item restart the fit from the reported optimum, or from a point
    perturbed slightly away from the reported optimum
    \item use \code{\link{allFit}} to try the fit with all available optimizers (e.g. several different implementations
    of BOBYQA and Nelder-Mead, L-BFGS-B from \code{optim}, \code{nlminb},
    \dots).  While this will of course be slow for large fits, we consider
    it the gold standard; if all optimizers converge to values that
    are practically equivalent, then we would consider the convergence
    warnings to be false positives.
  } % end itemize
}  % end description
\details{
  \subsection{Convergence controls}{
    \itemize{
      \item the controls for the \code{nloptwrap} optimizer (the default
  for \code{lmer}) are
      \describe{
	\item{ftol_abs}{(default 1e-6) stop on small change in deviance}
	\item{ftol_rel}{(default 0) stop on small relative change in deviance}
	\item{xtol_abs}{(default 1e-6) stop on small change of parameter values}
	\item{xtol_rel}{(default 0) stop on small relative change of
	  parameter values}
	\item{maxeval}{(default 1000) maximum number of function evaluations}
      }
      Changing \code{ftol_abs} and \code{xtol_abs} to stricter values
      (e.g. 1e-8) is a good first step for resolving convergence
      problems, at the cost of slowing down model fits.
      \item the controls for \code{minqa::bobyqa} (default for
      \code{glmer} first-stage optimization) are
      \describe{
	\item{rhobeg}{(default 2e-3) initial radius of the trust region}
	\item{rhoend}{(default 2e-7) final radius of the trust region}
	\item{maxfun}{(default 10000) maximum number of function evaluations}
      }
      \code{rhoend}, which describes the scale of parameter uncertainty
      on convergence, is approximately analogous to \code{xtol_abs}.
      \item the controls for \code{Nelder_Mead} (default for
      \code{glmer} second-stage optimization) are
      \describe{
        \item{FtolAbs}{(default 1e-5) stop on small change in deviance}
	\item{FtolRel}{(default 1e-15) stop on small relative change in deviance}
        \item{XtolRel}{(default 1e-7) stop on small change of parameter
	  values}
	\item{maxfun}{(default 10000) maximum number of function evaluations}
     } % Nelder_Mead controls
   } % list of optimizers
 } % convergence controls
 
\subsection{Theoretical issues}{\pkg{lme4} uses general-purpose nonlinear optimizers
  (e.g. Nelder-Mead or Powell's BOBYQA method) to estimate the
  variance-covariance matrices of the random effects.  Assessing
  the convergence of such algorithms reliably is difficult.  For
  example, evaluating the
  \href{https://en.wikipedia.org/wiki/Karush\%E2\%80\%93Kuhn\%E2\%80\%93Tucker_conditions}{Karush-Kuhn-Tucker conditions} (convergence criteria which
  reduce in simple cases to showing that
  the gradient is zero and the Hessian is positive definite) is
  challenging because of the difficulty of evaluating the gradient and
  Hessian.

  We (the \code{lme4} authors and maintainers) are still in the process
  of finding the best strategies for testing convergence.  Some of the
  relevant issues are
  \itemize{
    \item the gradient and Hessian are the basic ingredients of
    KKT-style testing, but (at least for now) \code{lme4} estimates
    them by finite-difference approximations which are sometimes
    unreliable.

    \item The Hessian computation in particular represents
    a difficult tradeoff between computational expense and
    accuracy.  At present the Hessian computations used
    for convergence checking (and for estimating standard errors
    of fixed-effect parameters for GLMMs) follow the \CRANpkg{ordinal} package
    in using a naive but computationally cheap centered finite difference
    computation (with a fixed step size of \eqn{10^{-4}}{1e-4}).  A more
    reliable but more expensive approach is to use
    \href{https://en.wikipedia.org/wiki/Richardson_extrapolation}{Richardson extrapolation},
    as implemented in the \CRANpkg{numDeriv} package.

    \item it is important to scale the estimated gradient at
    the estimate appropriately; two reasonable approaches are
    \enumerate{
      \item scale gradients by the inverse Cholesky factor of the
      Hessian, equivalent to scaling gradients by the
      estimated Wald standard error
      of the estimated parameters.  \code{lme4} uses this
      approach; it requires the Hessian to be estimated (although the Hessian is
      required \href{https://github.com/lme4/lme4/issues/47}{for
	reliable estimation of the fixed-effect standard errors for GLMMs}
      in any case).
      \item use unscaled gradients on the random-effects parameters,
      since these are essentially already unitless (for LMMs they are scaled
      relative to the residual variance; for GLMMs they are scaled
      relative to the sampling variance of the conditional distribution);
      for GLMMs, scale fixed-effect gradients by the standard deviations
      of the corresponding input variable
    }
    \item Exploratory analyses suggest that (1) the naive estimation
    of the Hessian may fail for large data sets (number of observations
    greater than approximately
    \eqn{10^{5}}{1e5}); (2) the magnitude of the scaled
    gradient increases with sample size, so that warnings will occur
    even for apparently well-behaved fits with large data sets.
  } % itemize
} % theoretical issues
} % details
\examples{
if (interactive()) {
fm1 <- lmer(Reaction ~ Days + (Days | Subject), sleepstudy)

## 1. decrease stopping tolerances
strict_tol <- lmerControl(optCtrl=list(xtol_abs=1e-8, ftol_abs=1e-8))
if (all(fm1@optinfo$optimizer=="nloptwrap")) {
    fm1.tol <- update(fm1, control=strict_tol)
}

## 2. center and scale predictors:
ss.CS <- transform(sleepstudy, Days=scale(Days))
fm1.CS <- update(fm1, data=ss.CS)

## 3. recompute gradient and Hessian with Richardson extrapolation
devfun <- update(fm1, devFunOnly=TRUE)
if (isLMM(fm1)) {
    pars <- getME(fm1,"theta")
} else {
    ## GLMM: requires both random and fixed parameters
    pars <- getME(fm1, c("theta","fixef"))
}
if (require("numDeriv")) {
    cat("hess:\n"); print(hess <- hessian(devfun, unlist(pars)))
    cat("grad:\n"); print(grad <- grad(devfun, unlist(pars)))
    cat("scaled gradient:\n")
    print(scgrad <- solve(chol(hess), grad))
}
## compare with internal calculations:
fm1@optinfo$derivs

## compute reciprocal condition number of Hessian
H <- fm1@optinfo$derivs$Hessian
Matrix::rcond(H)

## 4. restart the fit from the original value (or
## a slightly perturbed value):
fm1.restart <- update(fm1, start=pars)
set.seed(101)
pars_x <- runif(length(pars),pars/1.01,pars*1.01)
fm1.restart2 <- update(fm1, start=pars_x,
                       control=strict_tol)

## 5. try all available optimizers

  fm1.all <- allFit(fm1)
  ss <- summary(fm1.all)
  ss$ fixef               ## fixed effects
  ss$ llik                ## log-likelihoods
  ss$ sdcor               ## SDs and correlations
  ss$ theta               ## Cholesky factors
  ss$ which.OK            ## which fits worked

} %% interactive()
} % examples

\seealso{\code{\link{lmerControl}}, \code{\link{isSingular}}}

\name{NelderMead}
\alias{Nelder_Mead}
\title{Nelder-Mead Optimization of Parameters, Possibly (Box) Constrained}
\description{
  Nelder-Mead optimization of parameters, allowing
  optimization subject to box constraints (contrary to the default,
  \code{method = "Nelder-Mead"}, in \R's \code{\link{optim}()}), and
  using reverse communications.
}
\usage{
Nelder_Mead(fn, par, lower = rep.int(-Inf, n), upper = rep.int(Inf, n),
            control = list())
}
\arguments{
  \item{fn}{a \code{\link{function}} of a single numeric vector argument
    returning a numeric scalar.}

  \item{par}{numeric vector of starting values for the parameters.}

  \item{lower}{numeric vector of lower bounds (elements may be \code{-Inf}).}
  \item{upper}{numeric vector of upper bounds (elements may be \code{Inf}).}

  \item{control}{a named list of control settings. Possible settings are
    \describe{
      \item{iprint}{numeric scalar - frequency of printing evaluation information.
	Defaults to 0 indicating no printing.}
      \item{maxfun}{numeric scalar - maximum number of function
	evaluations allowed (default:10000).}
      \item{FtolAbs}{numeric scalar - absolute tolerance on
	change in function values (default: 1e-5)}
      \item{FtolRel}{numeric scalar - relative tolerance on
	change in function values (default:1e-15)}
      \item{XtolRel}{numeric scalar - relative tolerance on
	change in parameter values (default: 1e-7)}
      \item{MinfMax}{numeric scalar - maximum value of the
	minimum (default: .Machine$double.xmin)}
      \item{xst}{numeric vector of initial step sizes to
	establish the simplex - all elements must be non-zero
	(default: rep(0.02,length(par)))}
      \item{xt}{numeric vector of tolerances on the parameters (default: xst*5e-4)}
      \item{verbose}{numeric value: 0=no printing,
	1=print every 20 evaluations, 2=print every 10
	evalutions, 3=print every evaluation.  Sets
	\sQuote{iprint}, if specified, but does not override it.}
      \item{warnOnly}{a logical indicating if non-convergence (codes
	-1,-2,-3) should not \code{\link{stop}(.)}, but rather only call
	\code{\link{warning}} and return a result which might inspected.
	Defaults to \code{FALSE}, i.e., stop on non-convergence.}
    }
  }
}
\value{
  a \code{\link{list}} with components
  \item{fval}{numeric scalar - the minimum function value achieved}
  \item{par}{numeric vector - the value of \code{x} providing the minimum}
  \item{convergence}{integer valued scalar, if not \code{0}, an error code:
    \describe{
      \item{-4}{\code{nm_evals}: maximum evaluations reached}
      \item{-3}{\code{nm_forced}: ?}
      \item{-2}{\code{nm_nofeasible}: cannot generate a feasible simplex}
      \item{-1}{\code{nm_x0notfeasible}: initial x is not feasible (?)}
      \item{0}{successful convergence}
    }
  }
  \item{message}{a string specifying the kind of convergence.}
  \item{control}{the \code{\link{list}} of control settings after
    substituting for defaults.}
  \item{feval}{the number of function evaluations.}
}
\seealso{
  The \code{\linkS4class{NelderMead}} class definition and generator
  function.
}
\examples{
fr <- function(x) {   ## Rosenbrock Banana function
    x1 <- x[1]
    x2 <- x[2]
    100 * (x2 - x1 * x1)^2 + (1 - x1)^2
}
p0 <- c(-1.2, 1)

oo  <- optim(p0, fr) ## also uses Nelder-Mead by default
o.  <- Nelder_Mead(fr, p0)
o.1 <- Nelder_Mead(fr, p0, control=list(verbose=1))# -> some iteration output
stopifnot(identical(o.[1:4], o.1[1:4]),
          all.equal(o.$par, oo$par, tolerance=1e-3))# diff: 0.0003865

%%## but this shows that something "does not work"
o.2 <- Nelder_Mead(fr, p0, control=list(verbose=3, XtolRel=1e-15, FtolAbs= 1e-14))
all.equal(o.2[-5],o.1[-5], tolerance=1e-15)# TRUE, unexpectedly
}
\keyword{classes}
